(* ::Package:: *)

(*:Date: Sun May 17 16:47:16 MET DST 1998 *)
(*:Version 1.1: May 24, 2012 *)

(* Title: CURVES.m *)

(*:Author: Alfred Gray *)

(* Copyright 1994-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Versions: 2.2, 3.0 *)


(* Package Version 1.1. 
Adapted to Mathematica Version 7.0  
by Rolf Sulanke, starting May 24, 2012 *)

(* Changes in V. 1.1: No additional packages loaded. The Print command atthe beginning is 
commented out now, instead use `?CURVES* in the notebooks. In veriflat::usage the word "same" added. 
The command Protect["`*"]; added before the END of the packages protecting all the symbols defined 
in the context of the package.*)

(*:Summary: 
This package consists of definitions of parametrically defined
plane curves, implicitly defined plane curves, polar parametrizations
and space curves.
*)

(*:Keywords:
agnesi, agnesiimplicit, airyspiral, alysoid, alysoidnds, alysoidprime, archimedesspiral, archimedesspiralpolar, astroid, astroidimplicit, ast3d, baseballseam, bellcurve, besselcurve, besseln, bernstein, beziercurve, bicorn, bicylinder, biquadratic, bow, bowpolar, bowtie, bulletnose, bulletnoseimplicit, cardioid, cardioidimplicit, cardioidpolar, cardioidunitspeed, cartesianimplicit, cartesianoval, cassini, cassiniimplicit, cassinipolar, catenary, catenaryunitspeed, cayleysextic, circle, circleimplicit, circleinvolute, circleinvoluteunitspeed, circleunitspeed, cissoid, cissoidimplicit, clelia, clothoid, clothoidnds, clothoidprime, cnccoprofile, cnchyprofile, cochleoid, cochleoidpolar, conchoid, conchoidimplicit, conicalhelix, coshspiral, cosn, cothspiral, cpcprofile, crosscurve, crosscurveimplicit, cubicalellipse, cubicparabola, cubicparabolaimplicit, curves3d, cycloid, cycloidunitspeed, delaunay, delaunaykappa2, delaunaynds, deltoid, deltoidbis, deltoidimplicit, deltoidinvolute, deltoidunitspeed, devilimplicit, diamond, eight, eightknot, elasticainflect, elasticanoninflect, ellipse, ellipseimplicit, ellipseinc, epicycloid, epicycloidunitspeed, epispiral, epispiralpolar, epitrochoid, epitrochoid3d, fermatspiral, fermatspiralpolar, folium, foliumimplicit, foliumpolar, genparabola, genhelix, hankelspiral, helix, helixunitspeed, hippopede, hippopedeimplicit, horopter, hyperbola, hyperbolabis, hyperbolaimplicit, hyperbolicspiral, hyperbolicspiralpolar, hypocycloid, hypocycloidinvolute, hypocycloidunitspeed, hypotrochoid, icurvs2d, kampyle, kampylepolar, keplerimplicit, keplerorbit, keplerorbitpolar, kochsnowflake, lehrkappa2, lehrnds, lemniscate, lemniscatebis, lemniscateimplicit, lemniscatepolar, lemniscateunitspeed, limacon, limaconimplicit, limaconpolar, line, line3d, lissajous, lissajous3d, lituus, lituuspolar, logistic, logspiral, logspiralpolar, logspiralunitspeed, nephroid, nephroidimplicit, nephroidpolar, nephroidunitspeed, newtonphillipsimplicit, ngon, nielsenspiral, oneparametersubgroup, pacman, parabola, parabolaimplicit, parabolicspiral, perseusimplicit, piriform, piriformimplicit, predatorpreyimplicit, pseudocatenary, pseudocatenaryprime, pseudospheregeodesic, pseudosphericalloxodrome, rectangle, regularpolygon, reuleauxpolygon, rose, rosepolar, sc, scarab, scimplicit, seiffertspiral, semicubic, semicubicimplicit, serpentine, serpentineimplicit, sinhspiral, sinn, sinoval, sphericalcardioid, sphericalellipse, sphericalhelix, sphericalloxodrome, sphericalloxodromeunitspeed, sphericalnephroid, sphericalspiral, spring, strophoid, strophoidimplicit, tanhspiral, teardrop, teeth, tennisballseam, tooth, torusknot, tractrix, tractrixminus, tractrixplus, tractrixunitspeed, triangle, trident, tridentimplicit, trisectrix, tschirnhausen, twicubic, twistedn, veryflat, viviani, watt, wattimplcit, wigglyellipse, zcprofile
*)

BeginPackage["CURVES`"]

(* ,"CSPROGS`","PLTPROGS`","Graphics`Graphics`","Graphics`ImplicitPlot`" has been i the BeginPackage List; deleted by R. 
Sulanke, since obsolete now. *)


  (*  curves:= Print[Select[Names["CURVES`*"],#!="curves"&]]  not necessary*)

(* Parametrically Defined Plane Curves *)

    agnesi::usage="t->agnesi[a][t] is the parametrized curve whose implicit equation is x^2*y==4*a^2*(2*a - y)."

    airyspiral::usage="t->airyspiral[a][t] is a parametrized curve formed using Airy functions."

    alysoid::usage="s->alysoid[a,b,c][s] is a unit-speed curve which is a generalization of a catenary. Its curvature is -c/(a*b + s^2). s->alysoid[a,a,a][s] is the same as s->catenaryunitspeed[a][s]."

    alysoidkappa2::usage="s->alysoidkappa2[a,b,c][s] is the signed curvature of an alysoid. It is used in the definition of s->alysoidnds[smin,smax][a,b,c][s]."

    alysoidnds::usage="s->alysoidnds[smin,smax][a,b,c][s] is an approximation over the interval smin<s<smax found by NDSolve to s->alysoid[a,b,c][s]." 

    alysoidprime::usage="t->alysoidprime[a,b,c][t] is the velocity of  t->alysoid[a,b,c][t]. It is used in the definitions of s->alysoid[a,b,c][s] and s->alysoidnds[smin,smax][a,b,c][s]."  

    archimedesspiral::usage="t->archimedesspiral[n,a][t] is the spiral of Archimedes of radius a and degree n."

    astroid::usage="t->astroid[a][t] is the parametrized curve whose implicit equation is x^(2/3) + y^(2/3)==a^(2/3). t->astroid[a,b][t] is the parametrized curve whose implicit equation is (x/a)^(2/3) + (y/b)^(2/3)==1. t->astroid[n,a,b][t] is the parametrized curve whose implicit equation is (x/a)^n + (y/b)^n==1."

    bellcurve::usage="t->bellcurve[mean,variance][t] is the bell curve of statistics."

    bicorn::usage="t->bicorn[a][t] is a curve with two horns."

    bow::usage="t->bow[a][t] is a parametrized curve whose shape is a bow. The polar equation is r==a*(1 - Tan[theta]^2)."

    bowtie::usage="t->bowtie[a,b][t] is a parametrized curve whose shape is a bowtie."

    bulletnose::usage="t->bulletnose[a,b][t] is the parametric form of the curve whose implicit equation is a^2/x^2 - b^2/y^2==1."

    cardioid::usage="t->cardioid[a][t] is a cardioid or heart-shaped curve that is traced by a point on the circumference of a circle of radius 2*a rolling around a fixed circle of the same radius. It has the polar equation r==2*a*(1 + Cos[theta]). The implicit equation is (x^2 + y^2 - 2*a*x)^2 - 4*a^2*(x^2 + y^2)==0. A cardioid is a special case of an epicycloid."

    cardioidunitspeed::usage="s->cardioidunitspeed[a][s] is a unit-speed parametrization of t->cardioid[a][t]."

    cartesianoval::usage="u->cartesianoval[a,b][u,v] and v->cartesianoval[a,b][u,v] are orthogonal Cartesian ovals."

    cassini::usage="t->cassini[a,b,pm1,pm2][t] is a parametrization of an oval of Cassini. The parameters pm1 and pm2 are plus or minus 1. The implicit equation is (x^2 + y^2 + a^2)^2 - b^4 - 4*a^2*x^2==0."

    catenary::usage="t->catenary[a][t] is the curve formed by a perfectly flexible inextensible chain of uniform density hanging from two supports. It is given by t->{a*Cosh[t/a],t}."

    catenaryunitspeed::usage="s->catenaryunitspeed[a][s] is a unit-speed parametrization of t->catenary[a][t]."

    cayleysextic::usage="t->cayleysextic[a][t] is Cayley's sextic curve."

    circle::usage="t->circle[a][t] is a circle centered at the origin of radius a. The nonparametric form is x^2 + y^2==a^2."

    circleinvolute::usage="t->circleinvolute[n,a][t] is the nth involute starting at {a,0} of the circle t->{a*Cos[t],a*Sin[t]}. t->circleinvolute[a][t] is the same as t->circleinvolute[1,a][t]."  

    circleinvoluteunitspeed::usage="s->circleinvoluteunitspeed[n,a][s] is a unit-speed parametrization of t->circleinvolute[n,a][t]. s->circleinvoluteunitspeed[a][s] is the same as s->circleinvoluteunitspeed[1,a][s]."  

    circleunitspeed::usage="s->circleunitspeed[a][s] is a unit-speed parametrization of t->circle[a][t]."

    cissoid::usage="t->cissoid[a][t] is an ivy-shaped curve whose implicit equation is x^3 + x*y^2 - 2*a*y^2==0."

    clothoid::usage="s->clothoid[n,a][s] is the nth clothoid. The curvature is proportional to the nth power of the arc length function. It is a unit-speed curve. s->clothoid[1,a][s] is the ordinary clothoid, also known as Euler's spiral. s->clothoid[a][s] is the same as s->clothoid[1,a][s]."  

    clothoidnds::usage="s->clothoidnds[smin,smax][n,a][s] is an approximation over the interval smin<s<smax found by NDSolve to s->clothoid[n,a][s]."  

    clothoidprime::usage="t->clothoidprime[n,a][t] is the velocity of the nth clothoid. It is used in the definitions of s->clothoid[n,a][s] and s->clothoidnds[smin,smax][n,a][s]."

    cnccoprofile::usage="s->cnccoprofile[pm,a,b][s] is a profile curve for a surface of revolution of constant negative curvature -1/a^2 of conic type. It is a unit-speed curve. The parameter pm is plus or minus 1."

    cnchyprofile::usage="s->cnchyprofile[pm,a,b][s] is a profile curve for a surface of revolution of constant negative curvature -1/a^2 of hyperboloid type. It is a unit-speed curve. The parameter pm is plus or minus 1."

    cochleoid::usage="t->cochleoid[n,a][t] is the parametrized curve whose polar equation is r==a*(Sin[theta]/theta)^n."

    conchoid::usage="t->conchoid[a,b][t] is the parametrized curve whose implicit equation is (x^2 + y^2)*(x - b)^2==a^2*x^2."

    coshspiral::usage="t->coshspiral[n,a][t]  is a hyperbolic cosine spiral or Poinsot cosh spiral of radius a."

    cothspiral::usage="t->cothspiral[a][t] is a hyperbolic cotangent spiral of radius a."

    cpcprofile::usage="s->cpcprofile[pm,a,b][s] is a profile curve for a surface of revolution of constant positive curvature 1/a^2. It is a unit-speed curve. The parameter pm is plus or minus 1. There are 3 cases: football type (a>b), sphere (a=b), barrel type (a<b)." 

    crosscurve::usage="t->crosscurve[a,b][t] is the parametric form of the curve whose implicit equation is a^2/x^2 + b^2/y^2==1."

    cubicparabola::usage="t->cubicparabola[a,b,c,d][t] is the general cubic parabola whose implicit equation is y==a*x^3 + b*x^2 + c*x + d."

    cycloid::usage="t->cycloid[a,b][t] is the locus of points traced out by a point on a circle of radius b when a concentric circle of radius a rolls without slipping along a straight line. The cycloid is called prolate if b > a and curtate if a > b. t->cycloid[a][t] is the same as t->cycloid[a,a][t]."  

    cycloidunitspeed::usage="s->cycloidunitspeed[a,a][s] is a unit-speed parametrization of t->cycloid[a,a][t]."

    delaunay::usage="See delaunaynds."

    delaunaykappa2::usage="s->delaunaykappa2[a,e][s] is the signed curvature of a Delaunay curve. It is used in the definition of s->delaunaynds[smin,smax][a,e][s]."

    delaunaynds::usage="s->delaunaynds[smin,smax][a,e][s] is an approximation over the interval smin<s<smax found by NDSolve to a unit-speed parametrization of a Delaunay curve. A Delaunay curve is the path of a focus of a conic with long axis a and eccentricity e as the conic rolls on a straight line. If 0<e<1, the conic is an ellipse; if e>1, the conic is a hyperbola."

    deltoid::usage="t->deltoid[a][t] is a parametrized curve which resembles a delta. A deltoid is a special case of an hypocycloid."

    deltoidbis::usage="t->deltoidbis[a][t] is a parametrized curve which resembles a delta. A deltoid is a special case of an hypocycloid."

    deltoidinvolute::usage="t->deltoidinvolute[a][t] is the involute of t->deltoid[a][t] starting at t=0."

    deltoidunitspeed::usage="s->deltoidunitspeed[a][s] is a unit-speed parametrization of t->deltoid[a][t]."

    diamond::usage="t->diamond[a][t] is a parametrized curve whose shape is a diamond. t->diamond[n,a,b][t] is a generalization of t->diamond[a][t] that is similar to t->astroid[n,a,b][t]."

    eight::usage="t->eight[t] is a parametrized curve which resembles a figure eight."

    elasticainflect::usage="s->elasticainflect[a,k][s] is an inflectional elastica."

    elasticanoninflect::usage="s->elasticanoninflect[a,k][s] is a noninflectional elastica."

    ellipse::usage="t->ellipse[a,b][t] is an ellipse centered at the origin. The nonparametric form is x^2/a^2 + y^2/b^2==1."

    ellipseinc::usage="t->ellipseinc[a,b,c,psi][t] is the locus of points at a distance c along a line meeting an t->ellipse[a,b] at the point ellipse[a,b][t] with an angle psi."

    epicycloid::usage="t->epicycloid[a,b][t] is a parametrized curve traced out by a point P on the circumference of a circle (of radius b) rolling outside another circle (of radius a). Special cases are: cardioid (b=a), nephroid (b=a/2)."

    epicycloidunitspeed::usage="s->epicycloidunitspeed[a,b][s] is a unit-speed parametrization of t->epicycloid[a,b][t]."

    epispiral::usage="t->epispiral[n,a][t] is an epispiral. The curve has n branches if n is odd and 2*n branches if n is even. Each branch has two asymptotes radiating out from the center. The polar equation is r==a/Sin[n*theta]."

    epitrochoid::usage="t->epitrochoid[a,b,h][t] is a parametrized curve traced out by a point P attached to a circle (of radius b) rolling outside another circle (of radius a). Let h be the distance from P to the center of the moving circle. Special cases are: circle (a=0), limacon (a=b), epicycloid (h=b), cardioid (h=b=a), nephroid (h=b=a/2). There are a/b loops. For an epicycloid the loops degenerate to points."

    fermatspiral::usage="t->fermatspiral[a][t] is a spiral of Fermat of radius a, where t is positive. t->fermatspiral[n,a][t] is a spiral of Fermat of radius a and degree n, where t may be positive or negative. Special cases are: fermatspiral (n=1/4), lituus (n=3/4), archimedesspiral (n=(m - 1)/(2*m))."

    folium::usage="t->folium[t] is the folium of Descartes. The nonparametric form is x^3 + y^3 - 3*x*y==0. t->folium[n,a,b,c][t] is a generalized folium of Descartes. The nonparametric form is a*x^(2*n + 1) + b*y^(2*n + 1) - c*x^n*y^n==0. "

    genparabola::usage="t->genparabola[n][t] is the curve t->{t,t^n}."

    hankelspiral::usage="t->hankelspiral[n,a][t] is a Hankel spiral of order n."

    hippopede::usage="t->hippopede[a,b][t] is the parametric form of a curve whose shape resembles a horse fetter."

    hyperbola::usage="t->hyperbola[a,b][t] is the standard parametrization of a hyperbola centered at the origin using Cosh and Sinh. The nonparametric form is x^2/a^2 - y^2/b^2==1."

    hyperbolabis::usage="t->hyperbolabis[a,b][t] is a parametrization of a hyperbola centered at the origin using Sec and Tan. The nonparametric form is x^2/a^2 - y^2/b^2==1."

    hyperbolicspiral::usage="t->hyperbolicspiral[a][t] is a hyperbolic spiral of radius a. It is a special case of a spiral of Archimedes."

    hypocycloid::usage="t->hypocycloid[a,b][t] is a parametrized curve traced out by a point P on the circumference of a circle (of radius b) rolling inside another circle (of radius a). Special cases are: deltoid (b=a/3), astroid (b=a/4)."

    hypocycloidinvolute::usage="t->hypocycloidinvolute[a,b][t] is the involute of t->hypocycloid[a,b][t] starting at t=0."

    hypocycloidunitspeed::usage="s->hypocycloidunitspeed[a,b][s] is a unit-speed parametrization of t->hypocycloid[a,b][t]."

    hypotrochoid::usage="t->hypotrochoid[a,b,h][t] is a parametrized curve traced out by a point P on one circle (of radius b) rolling inside another circle (of radius a). Let h be the distance from P to the center of the moving circle. Special cases are: ellipse (a=2*b), hypocycloid  (h=b), deltoid (h=b=a/3), astroid (h=b=a/4). There are a/b loops when a/b is an integer larger than 2 and h>b. For a hypocycloid the loops degenerate to points."

    kampyle::usage="t->kampyle[a][t] is the kampyle or crooked staff of Eudoxus. The polar equation is r==a*Sec[theta]^2."

    keplerorbit::usage="t->keplerorbit[a,e][t] is a Kepler orbit, where e is the eccentricity. The orbit is part of an ellipse (0<=e<1), parabola (e=1) or hyperbola (e>1)."
 
    lehr::usage="See lehrnds."

    lehrkappa2::usage="s->lehrkappa2[a,b,c][s] is the signed curvature of a Lehr curve. It is used in the definition of s->lehrnds[smin,smax][a,b,c][s]."
    
    lehrnds::usage="t->lehrnds[smin,smax][a,b,c][s] is a Lehr curve. Its signed curvature is a + b*Cos[c*s]"

    lemniscate::usage="t->lemniscate[a][t] is the parametrized curve whose implicit equation is (x^2 + y^2)^2==a^2*(x^2 - y^2)."

    lemniscatebis::usage="t->lemniscatebis[a][theta] is a parametrization of
a lemniscate in which theta measures the angle subtended by a point on the curve."

    lemniscateunitspeed::usage="s->lemniscateunitspeed[a][s] is a unit-speed parametrization of t->lemniscate[a][t]."

    limacon::usage="t->limacon[a,b][t] is the parametrized curve whose implicit equation is (x^2 + y^2 - 2*a*x)^2==b^2*(x^2 + y^2). t->limacon[a,2*a][t] coincides with t->cardioid[a][t]."

    line::usage="t->line[a1,b1,a2,b2][t] is a parametrized line in R^2 with line[a1,b1,a2,b2][0]={a2,b2} and line[a1,b1,a2,b2]'[0]={a1,b1}."

    lissajous::usage="t->lissajous[n,d,a,b][t] is a Lissajous or Bowditch curve."

    lituus::usage="t->lituus[a][t] is the lituus spiral of radius a. It is a trumpet-shaped curve that is the locus of points P such that the square of distance of P from the origin is inversely proportional to the angle theta that p makes with the horizontal axis. The polar equation is r==a*(theta)^(-1/2)."

    logistic::usage="t->logistic[k,a,b][t] is the logistic curve used to model growth in biological populations for which saturation occurs."

    logspiral::usage="t->logspiral[a,b][t] is the log spiral of radius a and slant b. The polar equation is r==a*E^(b*theta)."

    logspiralunitspeed::usage="s->logspiralunitspeed[a,b][s] is a unit-speed parametrization of t->logspiral[a,b][t]."

    nephroid::usage="t->nephroid[a][t] is a nephroid or kidney-shaped curve that is traced by a point on the circumference of a circle of radius a rolling around a fixed circle of radius 2*a. It has the polar equation r==2*a*(Sin[theta/2]^(2/3) + Cos[theta/2]^(2/3))^(3/2). The implicit equation is (x^2 + y^2 - 4*a^2)^3 - 108*a^4*y^2==0. A nephroid is a special case of an epicycloid."

    nephroidunitspeed::usage="s->nephroidunitspeed[a][s] is a unit-speed parametrization of t->nephroid[a][t]."

    ngon::usage="t->ngon[n,a][t] is a parametrization of a regular polygon with n sides, where a is the length of the apothem."

    nielsenspiral::usage="t->nielsenspiral[a][t] is Nielsen's spiral of radius a."

    parabola::usage="t->parabola[a][t] is the vertical parabola with vertex at the origin, focus at {0,a} and directrix t->{t,-a}. The nonparametric form is x^2==4*a*y."

    parabolicspiral::usage="t->parabolicspiral[n,a,b][t] is a parabolic spiral of radius a, offset b and degree n, where t may be positive or negative. Special cases are: fermatspiral (b=0), lituus (n=3/4,b=0), archimedesspiral (n=(m - 1)/(2*m),b=0)."

    piriform::usage="t->piriform[a,b][t] is a pear-shaped curve whose implicit equation is  a^4*y^2 - b^2*x^3*(2*a - x)==0."

    pseudocatenary::usage="s->pseudocatenary[a,b][s] is a unit-speed curve which is a generalization of a catenary. Its curvature is -a/(a*b + s^2). s->pseudocatenary[a,a][s] is the same as s->catenaryunitspeed[a][s]."

    pseudocatenarykappa2::usage="s->pseudocatenarykappa2[a,b][s] is the signed curvature of a pseudocatenary. It is used in the definition of s->pseudocatenarynds[smin,smax][a,b,c][s]."

    pseudocatenarynds::usage="s->pseudocatenarynds[a,b][s] is an approximation over the interval smin<s<smax found by NDSolve to s->pseudocatenary[a,b][s]."

    pseudocatenaryprime::usage="t->pseudocatenaryprime[a,b][t] is the velocity of t->pseudocatenary[a,b][t]."

    rectangle::usage="t->rectangle[n,a,b][t] is a generalized rectangle. t->rectangle[1,a,b][t] is an ellipse. t->rectangle[2,a,b][t] is a parametrization of an ordinary rectangle. t->rectangle[3,a,b][t] is an astroid rotated by -Pi/4."

    reuleauxpolygon::usage="t->reuleauxpolygon[n,a][t] is a parametrization of a Reuleaux constant width curve with 2*n +1 sides, where a is the distance between any vertex and the center."

    rose::usage="t->rose[n,a][t] is a rose of radius a with n petals if n is odd and 2*n petals if n is even. The polar equation is r==a*Cos[n*theta]."

    sc::usage="t->sc[t] is the semicubical parabola t->{t^2,t^3}. The nonparametric form is y^2==x^3."

    scarab::usage="t->scarab[a,b][t] is a parametrization of a scarab."

    scunitspeed::usage="s->scunitspeed[s] is a unit-speed parametrization of t->sc[t]."

    semicubic::usage="z->semicubic[a,b,c,d][z] is a parametrization of the semicubic y^2 = a*x^3 + b*x^2 + c*x + d using the Weierstrass P function and its derivative."

    serpentine::usage="t->serpentine[a,b][t] is a serpentine curve."

    sinhspiral::usage="t->sinhspiral[n,a][t] is a hyperbolic sine spiral or Poinsot sinh spiral of radius a."

    sinoval::usage="t->sinoval[n,a][t] is the oval t->{a*Cos[t],a*Sin[Sin[...Sin[t]]]}."

    spring::usage="t->spring[a,b,v][t] is a plane curve which resembles a spring. It is the projection of t->helix[a,b][t] onto a plane whose normal makes an angle v with the z-axis."

    strophoid::usage="t->strophoid[a][t] is the parametrized curve whose implicit equation is y^2*(a - x)==x^3. t->strophoid[m,a][t] is the parametrized curve whose implicit equation is y^2*(a - x)==x^2*(a*m + x). The case m=0 is the cissoid t->cissoid[a/2][t] and the case m=1 is the strophoid t->strophoid[a][t]."

    talbot::usage="t->talbot[a,b,f][t] is Talbot's curve."

    tanhspiral::usage="t->tanhspiral[a][t] is the hyperbolic tangent spiral of radius a."

    teardrop::usage="t->teardrop[a,b,c,d,n][t] is a curve resembling a teardrop that has high order contact with the horizontal axis."

    tractrix::usage="t->tractrix[a][t] is a tractrix. It represents the path of a particle P pulled by an inextensible string whose end moves along a line. The evolute of a catenary is a tractrix. A tractrix is the profile curve of a surface of revolution of constant Gaussian curvature -1/a^2. t->tractrix[a,b][t] is a generalization of a tractrix in the same way that an ellipse is a generalization of a circle. In the terminology of Bianchi, t->tractrix[a,b][t] is a shortened tractrix for b<a or extended tractrix for b>a."

    tractrixminus::usage="s->tractrixminus[a][s] is a parametrization of the bottom half of t->tractrix[a][t]."

    tractrixplus::usage="s->tractrixplus[a][s] is a parametrization of the top half of t->tractrix[a][t]."

    tractrixunitspeed::usage="s->tractrixunitspeed[a][s] is a unit-speed parametrization of t->tractrix[a][t]."

    triangle::usage="t->triangle[t] is a parametrization of an equilateral
triangle."

    trident::usage="t->trident[a,b,c,d][t] is the parametric form of the curve whose implicit equation is x*y==a*x^3 + b*x^2 + c*x + d."

    trisectrix::usage="t->trisectrix[a][t] is the trisectrix of Maclaurin."

    tschirnhausen::usage="t->tschirnhausen[n,a][t] is a parametrization of Tschirnhausen's cubic."

    watt::usage="t->watt[a,b,c][t] is a parametrization of Watt's curve."

    zcprofile::usage="t->zcprofile[pm,a,c][t] is the profile curve of a generalized helicoid of zero Gaussian curvature. The parameter pm is +1 or -1."

(* Implicitly Defined Plane Curves *)

    agnesiimplicit::usage="agnesiimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve t->{2*a*Tan[t],2*a*Cos[t]^2}."

     astroidimplicit::usage="astroidimplicit[n,a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{a*Cos[t]^n,b*Sin[t]^n}."

    bulletnoseimplicit::usage="bulletnoseimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{a*Cos[t],b*Cot[t]}."

    cardioidimplicit::usage="cardioidimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve t->{a*(1 + Cos[t])*Cos[t],a*(1 + Cos[t])*Sin[t]}."

    cartesianimplicit::usage="cartesianimplicit[a,c,m][x,y]==0 is an oval of Descartes: ((1 - m^2)*(x^2 + y^2) + 2*m^2*c*x + a^2 - m^2*c^2)^2 - 4*a^2*(x^2 + y^2)==0."

    cassiniimplicit::usage="cassiniimplicit[a,b][x,y]==0 is an oval of Cassini: (x^2 + y^2 + a^2)^2 - b^4 - 4*a^2*x^2==0."

    circleimplicit::usage="circleimplicit[a][x,y]==0 is the nonparametric form of the parametrized circle t->{a*Cos[t],a*Sin[t]}."

    cissoidimplicit::usage="cissoidimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve t->{2*a*t^2/(1 + t^2),2*a*t^3/(1 + t^2)}."

    conchoidimplicit::usage="conchoidimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{b + a*Cos[t],b*Tan[t] + a*Sin[t]}."

    crosscurveimplicit::usage="crosscurveimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{a*Sec[t],b*Csc[t]}."

    cubicparabolaimplicit::usage="cubicparabolaimplicit[a,b,c,d][x,y]==0 is the nonparametric form of the parametrized curve t->{t,a*t^3 + b*t^2 + c*t + d}."

    deltoidimplicit::usage="deltoidimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve t->{2*a*Cos[t]*(1 + Cos[t]) - a,2*a*Sin[t]*(1 - Cos[t])}."

    devilimplicit::usage="devilimplicit[a,b][x,y]==0 is the devil's curve: y^2*(y^2 - b^2) - x^2*(x^2 - a^2)==0."

    duererimplicit::usage="duererimplicit[a,b][x,y]==0 is Duerer's conchoid.
Let q + r =b. The locus of points {x,y} whose distance from {q,0} is a and
lie on the line through {q,0} and {r,0} constitute Duerer's conchoid."

    ellipseimplicit::usage="ellipseimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{a*Cos[t],b*Sin[t]}."

    foliumimplicit::usage="foliumimplicit[x,y]==0 is the nonparametric form of the parametrized curve t->{3*t/(1 +t^3),3*t^2/(1 + t^3)}. foliumimplicit[n,a,b,c][x,y]==0 is the nonparametric form of the parametrized curve t->{c*t^n/(a + b*t^(2*n + 1)),c*t^(n + 1)/(a + b*t^(2*n + 1))}."

     hippopedeimplicit::usage="hippopedeimplicit[a,b][x,y]==0 is the nonparametric form of a curve whose shape resembles a horse fetter."

    hyperbolaimplicit::usage="hyperbolaimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{a*Cosh[t],b*Sinh[t]}."

    keplerimplicit::usage="keplerimplicit[a,b][x,y]==0 is the folium of Kepler: (x*(x - b) + y^2) - 4*a*(x - b)*y^2==0."

    lemniscateimplicit::usage="lemniscateimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve t->{a*Cos[t]/(1 + Sin[t]^2),a*Sin[t]*Cos[t]/(1 + Sin[t]^2)}. lemniscateimplicit[b*Sqrt[2]][x,y] coincides with cassiniimplicit[b,b][x,y]."

    limaconimplicit::usage="limaconimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{(2*a*Cos[t]+b)*Cos[t](2*a*Cos[t]+b)*,Sin[t]}."

    nephroidimplicit::usage="nephroidimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve
t->{a*(3*Cos[t] - Cos[3*t]),a*(3*Sin[t] - Sin[3*t])}."

    newtonphillipsimplicit::usage="newtonphillipsimplicit[m,n,a,b][x,y]==0 is the nonparametric curve studied by Newton and Phillips."

    parabolaimplicit::usage="parabolaimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve t->{2*a*t,a*t^2}."

    perseusimplicit::usage="perseusimplicit[a,b,c][x,y]==0 is the nonparametric form of a spiric of Perseus. perseusimplicit[a,b^2/(2*a),b^2/(2*a)][x,y]==0 coincides with cassiniimplicit[a,b][x,y]==0. A section of a torus by a plane parallel to the axis of revolution is a spiric of Perseus."

    piriformimplicit::usage="piriformimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{(1 + Sin[t])*a,(1 + Sin[t])*b*Cos[t]}."

    predatorpreyimplicit::usage="predatorpreyimplicit[a,b,p,q,c][x,y]==0 is the implicit solution of the predator-prey system x'==x*(a - b*y),y'==y*(-p + q*x) centered at {p/q,a/b} with pseudoradius c."

    scimplicit::usage="scimplicit[x,y]==0 is the nonparametric form of the semicubical parabola t->{t^2,t^3}."

    semicubicimplicit::usage="semicubicimplicit[a,b,c,d][x,y]==0 is the nonparametric form of the curve y^2 = a*x^3 + b*x^2 + c*x + d."

    serpentineimplicit::usage="serpentineimplicit[a,b][x,y]==0 is the nonparametric form of the parametrized curve t->{t,a*t/(1 + b*t^2)}."

    strophoidimplicit::usage="strophoidimplicit[a][x,y]==0 is the nonparametric form of the parametrized curve t->{a*(t^2 - 1)/(t^2 + 1),a*t*(t^2 - 1)/(t^2 + 1)}. strophoidimplicit[m,a][x,y]==0 is the nonparametric form of the parametrized curve t->{a*(t^2 - m)/(t^2 + 1),a*t*(t^2 - m)/(t^2 + 1)}."

    tridentimplicit::usage="tridentimplicit[a,b,c,d][x,y]==0 is the nonparametric form of the curve x*y==a*x^3 + b*x^2 + c*x + d."

    wattimplicit::usage="wattimplicit[a,b,c][x,y]==0 is the nonparametric form of Watt's curve."

(* Polar Defined Plane Curves *)

    archimedesspiralpolar::usage="theta->archimedesspiralpolar[n,a][theta] is the polar form of a spiral of Archimedes of radius a and degree n."

    bowpolar::usage="theta->bowpolar[a][theta] is the polar form of a parametrized curve whose shape is a bow."

    cardioidpolar::usage="theta->cardioidpolar[a][theta] is the polar form of a heart-shaped curve that is traced by a point on the circumference of a circle of radius 2*a rolling around a fixed circle of the same radius."

    cassinipolar::usage="theta->cassinipolar[a,b,pm1,pm2][theta] is the polar form of an oval of Cassini. The parameters pm1 and pm2 are plus or minus 1. The implicit equation is (x^2 + y^2 + a^2)^2 - b^4 - 4*a^2*x^2==0."

    cochleoidpolar::usage="theta->cochleoidpolar[n,a][theta] is the polar form of a cochleoid of degree n."

    epispiralpolar::usage="theta->epispiralpolar[n,a][theta] is the polar form of a curve that has n branches if n is odd and 2*n branches if n is even. Each branch has two asymptotes radiating out from the center."

    fermatspiralpolar::usage="theta->fermatspiralpolar[a][theta] is the polar form of the spiral of Fermat of radius a, which may be positive or negative."

    foliumpolar::usage="theta->foliumpolar[theta] is the polar form of the folium of Descartes. The nonparametric form is x^3 + y^3 - 3*x*y==0."

    hyperbolicspiralpolar::usage="theta->hyperbolicspiralpolar[a][theta] is the polar form of the hyperbolic spiral of radius a. It is a special case of a spiral of Archimedes."

    kampylepolar::usage="theta->kampylepolar[a][theta] is the polar form of the kampyle or crooked staff of Eudoxus."

    keplerorbitpolar::usage="theta->keplerorbitpolar[a,e][theta] is the polar form of a Kepler orbit, where e is the eccentricity. The orbit is part of an ellipse (0<=e<1), parabola (e=1) or hyperbola (e>1)."

    lemniscatepolar::usage="theta->lemniscatepolar[a][theta] is the polar form of the curve whose implicit equation is (x^2 + y^2)^2==a^2*(x^2 - y^2). The parameter a is positive or negative."

    limaconpolar::usage="theta->limaconpolar[a,b][theta] is the polar form of the curve whose implicit equation is (x^2 + y^2 - 2*a*x)^2==b^2*(x^2 + y^2)."

    lituuspolar::usage="theta->lituuspolar[a][theta] is the polar form of the lituus spiral of radius a, which may be positive or negative."

    logspiralpolar::usage="theta->logspiralpolar[a,b][theta] is the polar form of the log spiral of radius a and slant b."

    nephroidpolar::usage="theta->nephroidpolar[a][theta] is the polar form of a kidney-shaped curve that is traced by a point on the circumference of a circle of radius a rolling around a fixed circle of radius 2*a."

    pacman::usage="theta->pacman[n][theta] is the polar form of a pacman-shaped curve of degree n."

    rosepolar::usage="theta->rosepolar[n,a][theta] is the polar form of a rose of radius a with n petals if n is odd and 2*n petals if n is even."

(* Parametrically Defined Space Curves *)

    ast3d::usage="t->ast3d[n,a,b][t] is a space curve analog of the plane curve astroid."

    baseballseam::usage="t->baseballseam[a,c][t] is an approximation to the seam on a baseball of radius a."

    besselcurve::usage="t->besselcurve[a,b,c][t] is a space curve whose components are Bessel functions."

    bicylinder::usage="t->bicylinder[a,b,pm][t] is a parametrization of the intersection of the circular cylinders x^2 + y^2 = a^2 and y^2 + z^2 = b^2. One component is obtained by taking pm = +1, the other by taking pm = -1."

    biquadratic::usage="t->biquadratic[a,k][t] is a parametrization of the
intersection of the circular cylinder x^2 + y^2 = a^2 and the elliptic cylinder
x^2 + k^2*z^2 = a^2."

    clelia::usage="t->clelia[a][m,n][t] is a clelia of type (m,n) on a sphere of radius a. The number of petals is n."

    conicalhelix::usage="t->conicalhelix[a,b,c,d][t] is a parametrization of a helix on a cone over t->ellipse[a,b][t]. The slant of the cone is determined by c and the amount of twisting is determined by d."

    cubicalellipse::usage="t->cubicalellipse[a,b,c,d,e,f,pm][t] is a cubical ellipse for pm = 1 and a cubical hyperbola for pm = -1."

    eightknot::usage="t->eightknot[t] is a parametrization of an eight knot."

    epitrochoid3d::usage="t->epitrochoid3d[a,b,h,omega][t] is the parametrized curve traced out by a point P attached to a circle of radius b rolling along a fixed circle of radius a. The rolling circle lies in a plane whose inclination with the plane of the fixed circle is omega, and h is the distance of P to the center of the rolling circle. The curve t->epitrochoid3d[a,b,h,0][t] is a (planar) epitrochoid and the curve epitrochoid3d[a,b,h,Pi][t] is a (planar) hypotrochoid."

    genhelix::usage="t->genhelix[a,b][f][t] is a generalized elliptical helix with slant function f."

    helix::usage="t->helix[a,b][t] is a circular helix of radius a and slant b. t->helix[a,b,c][t] is an elliptical helix of slant c."

    helixunitspeed::usage="s->helixunitspeed[a,b][s] is a unit-speed parametrization of a circular helix of radius a and slant b."

    horopter::usage="t->horopter[a,b][t] is a horopter. It is the intersection of the hyperbolic paraboloid y==b*x*z and the cylinder over the curve y^2==x*(2*a - x)."

    line3d::usage="t->line3d[a1,b1,c1,a2,b2,c2][t] is a parametrized line in R^3 with line3d[a1,b1,c1,a2,b2,c2][0]={a2,b2,c2} and line3d[a1,b1,c1,a2,b2,c2]'[0]={a1,b1,c1}."

    lissajous3d::usage="t->lissajous3d[n,d,a,b,c][t] is a space curve analog of Lissajous curve."

    oneparametersubgroup::usage="t->oneparametersubgroup[mat,vec][t] is the
curve in R^n formed by the one parameter subgroup t->exp(t*mat) acting on a vector v in R^n, where mat is an n by n matrix."

    pseudospheregeodesic::usage="t->pseudospheregeodesic[a,c,u0,pm][t] is a geodesic of slant c on a pseudosphere of constant negative curvature -1/a^2. The parameter pm is plus or minus 1."

    pseudosphericalloxodrome::usage="t->pseudosphericalloxodrome[a,b][t] is a loxodrome on a pseudosphere of constant negative curvature -1/a^2."

    seiffertspiral::usage="t->seiffertspiral[a,k][t] is Seiffert's spiral of slant k on an sphere of radius a."

    sphericalcardioid::usage="t->sphericalcardioid[a][t] is a spherical curve on a sphere of radius 3|a|. It is a curve of constant slope that projects to t->cardioid[-a][t + Pi]."

    sphericalellipse::usage="t->sphericalellipse[a,b,c,pm][t]
is a parametrization of the intersection of the sphere x^2 + y^2 + z^2= a^2 and the elliptical cylinder x^2/b^2 + y^2/c^2 = 1. One component is obtained by taking pm = +1, the other by taking pm = -1."

    sphericalhelix::usage="t->sphericalhelix[a,b][t] is a spherical helix on a sphere of radius |a + 2*b|. It is a curve of constant slope that projects to t->epicycloid[a,b][t]."

    sphericalloxodrome::usage="t->sphericalloxodrome[a,b,c][t] is a loxodrome on a sphere of radius a. It is the image under the stereographic injection {u,v}->stereographicsphere[a][u,v] of the logarithmic spiral t->logspiral[b,c][t]. A sphericalloxodrome meets each meridian of the sphere at the same angle."

    sphericalloxodromeunitspeed::usage="s->sphericalloxodromeunitspeed[a,b,c][s] is a unit-speed parametrization of a loxodrome on a sphere of radius a."

    sphericalnephroid::usage="t->sphericalnephroid[a][t] is a spherical helix on a sphere of radius 4|a|. It is a curve of constant slope that projects to t->nephroid[a][t]."

    sphericalspiral::usage="t->sphericalspiral[a][m,n][t] is a spherical spiral of type (m,n) on a sphere of radius a."

    tennisballseam::usage="t->tennisballseam[n,a,b,c][t] is a curve on the unit sphere in R^3.  It is the normal to t->wigglyellipse[n,a,b,c][t]. For those values of n, a, b and c for which the curve has no self-intersections, the curve (parametrized on 0<=t<=2*Pi) is the boundary of subsets of the unit sphere of equal area."

    torusknot::usage="t->torusknot[a,b,c][p,q][t] is a torus knot of type (p,q) on an elliptical torus of type (a,b,c)."

    twicubic::usage="t->twicubic[t] is the twisted cubic t->{t,t^2,t^3}."

    veryflat::usage="t->veryflat[1][t] is the curve t->{t,0,5*E^(-1/t^2)}.
t->veryflat[2][t] is the curve t->If[t<0,{t,5*E^(-1/t^2),0},
If[t==0,{0,0,0},{t,0,5*E^(-1/t^2)}]]. These curves both have the same curvature and torsion zero, but are not congruent by a Euclidean motion."

    viviani::usage="t->viviani[a][t] is a Viviani curve on a sphere of radius 2*a. It is the intersection of the cylinder (x - a)^2 + y^2==a^2 and the sphere x^2 + y^2 + z^2==4*a^2."

    wigglyellipse::usage="t->wigglyellipse[n,a,b,c][t] is an ellipse perturbed vertically by a sin curve. The normal to t->wigglyellipse[n,a,b,c][t] divides the unit sphere into two regions of equal area."

(* Parametrically Defined Curves in R^n*)

    besseln::usage="t->besseln[n,a][t] is the curve t->a*{BesselJ[0,t],...,BesselJ[n - 1,t]}."

    bernstein::usage="t->bernstein[p,q[t]] is the (p,q)-th Bernstein polynomial."

    beziercurve::usage="t->beziercurve[pts][t] is the Bezier curve in R^n determined by pts, where pts is a list of points in R^n."

    cosn::usage="t->cosn[n,a][t] is the curve t->a*{Cos[t],Cos[2*t],...,Cos[n*t]}."

    beziercurve::usage="t->beziercurve[pts][t] is the Bezier curve in R^n determined by pts, where pts is a list of points in R^n."

    lagrangecurve::usage="t->hermitecurve[p0,m0,m1,p1][t] is the Hermite curve in R^n determined by p0 and p1 with directions m0 and m1."

    sinn::usage="t->sinn[n,a][t] is the curve t->a*{Sin[t],Sin[2*t],...,Sin[n*t]}."

    twistedn::usage="t->twistedn[n,a][t] is the curve t->a*{t,t^2,...,t^n}."

(* Approximations *)

   regularpolygon::usage="regularpolygon[n,a] is a regular plolygon with n vertices circumscribed by a circle of radius a."

   teeth::usage="teeth[m,t][shortlines] is the mth iterate of tooth of height t."

   tooth::usage="tooth[t][Line[{{a,b},{c,d}}]] constructs from the line Line[{{a,b},{c,d}}] a broken line with a tooth of height t in the middle."

   kochsnowflake::usage="kochsnowflake[m] is a Koch snow flake constructed as the mth iterate of an equilateral triangle using a tooth of height (3 + Sqrt[3])/6."


Begin["`Private`"]

(* Parametrically Defined Plane Curves *)

    agnesi[a_][t_]:= {2*a*Tan[t],2*a*Cos[t]^2}

    agnesi[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[agnesi[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    airyspiral[a_][t_]:={a*AiryAi[t],a*AiryBi[t]}

    airyspiral[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[airyspiral[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    alysoid[a_,b_,c_][s_]:=
        Integrate[alysoidprime[a,b,c][ss],{ss,0,s}] + {a,0}

    alysoid[a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[alysoidnds[tmin,tmax][a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    alysoidkappa2[a_,b_,c_]:= -c/(a*b + s^2)

    alysoidnds[smin_,smax_][a_,b_,c_][s_]:=
         Module[{x,y,tmp,ss},
             tmp=NDSolve[{x'[ss]==alysoidprime[a,b,c][ss][[1]],
               y'[ss]==alysoidprime[a,b,c][ss][[2]],
               x[0]==a,y[0]==0},{x,y},{ss,smin,smax},
               MaxSteps->2000];
               First[{x[s],y[s]} /. tmp]]

    alysoidprime[a_,b_,c_][t_]:=
        {Sin[(c/Sqrt[a*b])*ArcTan[t/Sqrt[a*b]]],
         Cos[(c/Sqrt[a*b])*ArcTan[t/Sqrt[a*b]]]}

    archimedesspiral[n_,a_][t_]:=
        {a*t^(1/n)*Cos[t],a*t^(1/n)*Sin[t]}

    archimedesspiral[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[archimedesspiral[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    astroid[a_][t_]:= {a*Cos[t]^3,a*Sin[t]^3}              

    astroid[a_,b_][t_]:= {a*Cos[t]^3,b*Sin[t]^3}

    astroid[n_,a_,b_][t_]:= {a*Cos[t]^n,b*Sin[t]^n}

    astroid[n_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[astroid[n,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    bellcurve[mean_,variance_][t_]:= {t,
        (1/Sqrt[2*Pi*variance^2])*E^(-(t - mean)^2/(2*variance^2))}

    bellcurve[mean_,variance_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[bellcurve[mean,variance][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    bicorn[a_][t_]:= {a*Sin[t],a*Cos[t]^2(2 + Cos[t])/(3 + Sin[t]^2)}

    bicorn[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[bicorn[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    bow[a_][t_]:= {a*(1 - Tan[t]^2)Cos[t],a*(1 - Tan[t]^2)Sin[t]}

    bow[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[bow[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    bowtie[a_,b_][t_]:= {a*(1 + Cos[t]^2)*Sin[t],
                          (b + Sin[t]^2)*Cos[t]}
  
    bowtie[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[bowtie[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    bulletnose[a_,b_][t_]:= {a*Cos[t],b*Cot[t]}
    
    bulletnose[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[bulletnose[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cardioid[a_][t_]:= {2*a*Cos[t]*(1 + Cos[t]),
                        2*a*Sin[t]*(1 + Cos[t])} 
    
    cardioid[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cardioid[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cardioidunitspeed[a_][s_]:=
        {2*a*Cos[2*ArcSin[s/(8*a)]]*(1 + Cos[2*ArcSin[s/(8*a)]]),
         2*a*(1 + Cos[2*ArcSin[s/(8*a)]])*Sin[2*ArcSin[s/(8*a)]]}

   cartesianoval[a_,b_][u_,v_]:=
      {a*Re[WeierstrassP[u + I*v,b,0]],
       a*Im[WeierstrassP[u + I*v,b,0]]}

    cassini[a_,b_,pm1_,pm2_][t_]:= 
      {pm1*Sqrt[a^2*Cos[2*t] +
           pm2*Sqrt[b^4 - a^4*Sin[2*t]^2]]*Cos[t],
       pm1*Sqrt[a^2*Cos[2*t] +
           pm2*Sqrt[b^4 - a^4*Sin[2*t]^2]]*Sin[t]}

    cassini[a_,b_,pm1_,pm2_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cassini[a,b,pm1,pm2][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    catenary[a_][t_]:= {a*Cosh[t/a],t} 

    catenary[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[catenary[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    catenaryunitspeed[a_][s_]:=
        {a*Sqrt[1 + s^2/a^2],a*ArcSinh[s/a]}

    cayleysextic[a_][t_]:=
          {4*a*Cos[t/2]^4*(-1 + 2*Cos[t]),
           4*a*Cos[t/2]^3*Sin[3*t/2]}

    cayleysextic[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cayleysextic[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    circle[a_][t_]:= {a*Cos[t],a*Sin[t]}

    circle[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[circle[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    circleunitspeed[a_][s_]:= {a*Cos[s/a],a*Sin[s/a]}

    circleinvolute[n_,a_][t_]:= Module[{tmp1,tmp2},
         tmp1=E^(I*tt)Normal[Series[E^(-I*tt),{tt,0,n}]];
         tmp2=a*ComplexExpand[{Re[tmp1],Im[tmp1]}];
         tmp2 /. tt->t]

    circleinvolute[a_][t_]:= circleinvolute[1,a][t]

    circleinvolute[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[circleinvolute[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    circleinvoluteunitspeed[n_,a_][s_]:=
         circleinvolute[n,a][((n + 1)!*s/a)^(1/(n +1))]

    circleinvoluteunitspeed[a_][s_]:=
         circleinvoluteunitspeed[1,a][s]

    cissoid[a_][t_]:= {2*a*t^2/(1 + t^2),2*a*t^3/(1 + t^2)}

    cissoid[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cissoid[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    clothoid[n_,a_][s_]:=
         Integrate[clothoidprime[n,a][ss],{ss,0,s}]

    clothoid[a_][s_]:=
         {a*Sqrt[Pi]*FresnelS[s/Sqrt[Pi]],
          a*Sqrt[Pi]*FresnelC[s/Sqrt[Pi]]}

    clothoid[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[clothoidnds[tmin,tmax][n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    clothoidnds[smin_,smax_][n_,a_][s_]:=
         Module[{x,y,tmp,ss},
             tmp=NDSolve[{x'[ss]==clothoidprime[n,a][ss][[1]],
               y'[ss]==clothoidprime[n,a][ss][[2]],
               x[0]==0,y[0]==0},{x,y},{ss,smin,smax},
               MaxSteps->2000];
               First[{x[s],y[s]} /. tmp]]

    clothoidprime[n_,a_][t_]:=
         {a*Sin[t^(n + 1)/(n + 1)],a*Cos[t^(n + 1)/(n + 1)]}
       
    cnccoprofile[pm_,a_,b_][s_]:=
        {pm*b*Sinh[s/a],-I*Sqrt[a^2 - b^2]*EllipticE[I*s/a,
                       -b^2/(a^2 - b^2)]}

    cnccoprofile[pm_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cnccoprofile[pm,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cnchyprofile[pm_,a_,b_][s_]:=
        {pm*b*Cosh[s/a],-I*a*EllipticE[I*s/a,-b^2/a^2]}

    cnchyprofile[pm_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cnchyprofile[pm,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cochleoid[n_,a_][t_]:=
       {a*(Sin[t]/t)^n*Cos[t],a*(Sin[t]/t)^n*Sin[t]}

    cochleoid[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cochleoid[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    conchoid[a_,b_][t_]:= {b + a*Cos[t],b*Tan[t] + a*Sin[t]}

    conchoid[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[conchoid[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    coshspiral[n_,a_][t_]:=
            {a*Cos[t]/Cosh[n*t],a*Sin[t]/Cosh[n*t]}

    coshspiral[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[coshspiral[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cothspiral[a_][t_]:= {-Sinh[2*t]/(Cos[2*a*t] - Cosh[2*t]),
                            Sin[2*a*t]/(Cos[2*a*t] - Cosh[2*t])}

    cothspiral[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cothspiral[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cpcprofile[pm_,a_,b_][s_]:=
       {pm*b*Cos[s/a],a*EllipticE[s/a,b^2/a^2]}

    cpcprofile[pm_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cpcprofile[pm,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    crosscurve[a_,b_][t_]:= {a*Sec[t],b*Csc[t]}
    
    crosscurve[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[crosscurve[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cubicparabola[a_,b_,c_,d_][t_]:= {t,a*t^3 + b*t^2 + c*t + d}

    cubicparabola[a_,b_,c_,d_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cubicparabola[a,b,c,d][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cycloid[a_,b_][t_]:= {a*t - b*Sin[t],a - b*Cos[t]}

    cycloid[a_][t_]:= {a*t - a*Sin[t],a - a*Cos[t]}

    cycloid[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[cycloid[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    cycloidunitspeed[a_,a_][s_]:=
         {4*a*ArcSin[Sqrt[s/(8*a)]] - (4*a - s)*Sqrt[(s/a)*(8 - s/a)]/8,
          s - s^2/(8*a)}

    delaunay[a_,e_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[delaunaynds[tmin,tmax][a,e][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    delaunaynds[smin_,smax_][a_,e_][s_]:=
      intrinsic[delaunaykappa2[a,e],0,{0,0,0},
      MaxSteps->2000,{smin,smax}][s]

    delaunaykappa2[a_,e_][t_]:=
         e*(e - Cos[t/a])/(a*(1 - 2*e*Cos[t/a] + e^2))

    deltoid[a_][t_]:= {2*a*Cos[t]*(1 + Cos[t]) - a,
                       2*a*Sin[t]*(1 - Cos[t])}

    deltoid[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[deltoid[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    deltoidbis[a_][t_]:=
        {a*(2*Cos[t] - Sin[2*t]),a*(2*Sin[t] - Cos[2*t])}

    deltoidbis[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[deltoidbis[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    deltoidinvolute[a_][t_]:=
        {(a/3)*(8*Cos[t/2] + 2*Cos[t] - Cos[2*t]),
         (a/3)*(-8*Sin[t/2] + 2*Sin[t] + Sin[2*t])}

    deltoidinvolute[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[deltoidinvolute[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    deltoidunitspeed[a_][s_]:=
        {a*(2*Cos[2*ArcCot[3*s/Sqrt[64*a^2 - 9*s^2]]/3] + 
         Cos[4*ArcCot[3*s/Sqrt[64*a^2 - 9*s^2]]/3]),
         -8*a*Cos[ArcCot[3*s/Sqrt[64*a^2 - 9*s^2]]/3]*
         Sin[ArcCot[3*s/Sqrt[64*a^2 - 9*s^2]]/3]^3}

    diamond[a_][t_]:=
        {a*Cos[t]*Sqrt[Cos[t]^2],a*Sin[t]*Sqrt[Sin[t]^2]}

    diamond[n_,a_,b_][t_]:=
        {a*Sqrt[Cos[t]^2]^(n - 1)*Cos[t],
         b*Sqrt[Sin[t]^2]^(n - 1)*Sin[t]}

    diamond[n_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[diamond[n,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    eight[t_]:= {Sin[t],Sin[t]*Cos[t]}

    eight[tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[eight[t]],{t,tmin,tmax},opt,
            Axes->None,PlotPoints->150]

    elasticainflect[a_,k_][s_]:= 
       {(2/a)*EllipticE[JacobiAmplitude[a*s,k^2],k^2] - s,
          -(2*k/a)*JacobiCN[a*s,k^2]}

    elasticainflect[a_,k_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[elasticainflect[a,k][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    elasticanoninflect[a_,k_][s_]:= 
        {(2/(k*a))*EllipticE[JacobiAmplitude[a*s/k,k^2],k^2] +
              (1 - 2/k^2) s,
          -(2/(k*a))*JacobiDN[a*s/k,k^2]}

    elasticanoninflect[a_,k_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[elasticanoninflect[a,k][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    ellipse[a_,b_][t_]:= {a*Cos[t],b*Sin[t]}

    ellipse[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[ellipse[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    ellipseinc[a_,b_,c_,psi_][t_]:=
        Module[{k},k=1 - b^2/a^2;
           {a*JacobiSN[t,k] + (c/a)(a*Cos[psi]*JacobiCN[t,k] +
                     b*Sin[psi]*JacobiSN[t,k])/JacobiDN[t,k],
            b*JacobiCN[t,k] + (c/a)(a*Sin[psi]*JacobiCN[t,k] -
                   b*Cos[psi]*JacobiSN[t,k])/JacobiDN[t,k]}]

    ellipseinc[a_,b_,c_,psi_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[ellipseinc[a,b,c,psi][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    epicycloid[a_,b_][t_]:=
            {(a + b)*Cos[t] - b*Cos[((a + b)/b)*t],
             (a + b)*Sin[t] - b*Sin[((a + b)/b)*t]}

    epicycloid[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[epicycloid[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    epicycloidunitspeed[a_,b_][s_]:=
    {(a + b)*Cos[4*b*ArcSin[Sqrt[a*s/(b*(8*a + 8*b))]]/a] - 
      b*Cos[4*(a + b)*ArcSin[Sqrt[a*s/(b*(8*a + 8*b))]]/a],
     (a + b)*Sin[4*b*ArcSin[Sqrt[a*s/(b*(8*a + 8*b))]]/a] -
      b*Sin[4*(a + b)*ArcSin[Sqrt[a*s/(b*(8*a + 8*b))]]/a]}

    epispiral[n_,a_][t_]:= {a*Cos[t]/Sin[n*t],a*Sin[t]/Sin[n*t]}

    epispiral[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[epispiral[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    epitrochoid[a_,b_,h_][t_]:=
            {(a + b)*Cos[t] - h*Cos[((a + b)/b)*t],
             (a + b)*Sin[t] - h*Sin[((a + b)/b)*t]}

    epitrochoid[a_,b_,h_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[epitrochoid[a,b,h][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    fermatspiral[a_][t_]:= {a*Sqrt[t]*Cos[t],a*Sqrt[t]*Sin[t]}

    fermatspiral[n_,a_][t_]:= {a*t/(t^2)^n*Cos[Sqrt[t^2]],
                               a*t/(t^2)^n*Sin[Sqrt[t^2]]}

    fermatspiral[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[fermatspiral[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    folium[t_]:= {3*t/(1 + t^3),3*t^2/(1 + t^3)}

    folium[n_,a_,b_,c_][t_]:= {c*t^n/(a + b*t^(2*n + 1)),
                           c*t^(n + 1)/(a + b*t^(2*n + 1))}
(*
    folium[tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[folium[t]],{t,tmin,tmax},opt,
            Axes->None,AspectRatio->Automatic]
*)
    genparabola[n_][t_]:= {t,t^n}

    genparabola[n_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[genparabola[n][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hankelspiral[n_,a_][t_]:= {a*BesselJ[n,t],a*BesselY[n,t]}

    hankelspiral[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hankelspiral[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hippopede[a_,b_][t_]:=
              {2*Cos[t]*Sqrt[a*b - b^2*Sin[t]^2],
               2*Sin[t]*Sqrt[a*b - b^2*Sin[t]^2]}

    hippopede[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hippopede[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hyperbola[a_,b_][t_]:= {a*Cosh[t],b*Sinh[t]}

    hyperbola[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hyperbola[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hyperbolabis[a_,b_][t_]:= {a*Sec[t],b*Tan[t]}

    hyperbolabis[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hyperbolabis[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hyperbolicspiral[a_][t_]:= {a*Cos[t]/t,a*Sin[t]/t}

    hyperbolicspiral[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hyperbolicspiral[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hypocycloid[a_,b_][t_]:=
            {(a - b)*Cos[t] + b*Cos[((a - b)/b)*t],
             (a - b)*Sin[t] - b*Sin[((a - b)/b)*t]}
    
    hypocycloid[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hypocycloid[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hypocycloidinvolute[a_,b_][t_]:=
        {(a^2*Cos[t] - 3*a*b*Cos[t] + 2*b^2*Cos[t] - 
          a*b*Cos[t - a*t/b] + 2*b^2*Cos[t - a*t/b] + 
          4*a*b*Cos[t - a*t/(2*b)] - 
          4*b^2*Cos[t - a*t/(2*b)])/a,
         (a^2*Sin[t] - 3*a*b*Sin[t] + 2*b^2*Sin[t] - 
          a*b*Sin[t - a*t/b] + 2*b^2*Sin[t - a*t/b] + 
          4*a*b*Sin[t - a*t/(2*b)] - 
          4*b^2*Sin[t - a*t/(2*b)])/a}

    hypocycloidinvolute[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hypocycloidinvolute[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    hypocycloidunitspeed[a_,b_][s_]:=
    {(a - b)*Cos[4*b*ArcSin[Sqrt[a*s/((8*a - 8*b)*b)]]/a] +
      b*Cos[4*(a - b)*ArcSin[Sqrt[a*s/((8*a - 8*b)*b)]]/a],
     (a - b)*Sin[4*b*ArcSin[Sqrt[a*s/((8*a - 8*b)*b)]]/a]
      -b*Sin[4*(a - b)*ArcSin[Sqrt[a*s/((8*a - 8*b)*b)]]/a]}

    hypotrochoid[a_,b_,h_][t_]:=
            {(a - b)*Cos[t] + h*Cos[((a - b)/b)*t],
             (a - b)*Sin[t] - h*Sin[((a - b)/b)*t]}

    hypotrochoid[a_,b_,h_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[hypotrochoid[a,b,h][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    kampyle[a_][t_]:= {a*Sec[t],a*Sin[t]*Sec[t]^2}

    kampyle[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[kampyle[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    keplerorbit[a_,e_][t_]:= {a*Cos[t]/(1 - e*Cos[t]),
                              a*Sin[t]/(1 - e*Cos[t])}

    keplerorbit[a_,e_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[keplerorbit[a,e][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    lehrkappa2[a_,b_,c_][t_]:=a + b*Cos[c*t]
    
    lehr[a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[lehrnds[tmin,tmax][a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    lehrnds[smin_,smax_][a_,b_,c_][s_]:=
         intrinsic[lehrkappa2[a,b,c],0,{0,0,0},
         MaxSteps->2000,{smin,smax}][s]

    lemniscate[a_][t_]:=
         {a*Cos[t]/(1 + Sin[t]^2),
          a*Sin[t]*Cos[t]/(1 + Sin[t]^2)}

    lemniscate[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[lemniscate[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    lemniscatebis[a_][theta__]:=
         {a*Sqrt[Cos[2*theta]]*Cos[theta],
          a*Sqrt[Cos[2*theta]]*Sin[theta]}

    lemniscatebis[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[lemniscatebis[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    lemniscateunitspeed[a_][s_]:=
         {a*JacobiCN[s/a,-1]/JacobiDN[s/a,-1]^2,
          a*JacobiCN[s/a,-1]JacobiSN[s/a,-1]/JacobiDN[s/a,-1]^2}

    limacon[a_,b_][t_]:= {(2*a*Cos[t] + b)*Cos[t],
                          (2*a*Cos[t] + b)*Sin[t]}

    limacon[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[limacon[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    line[a1_,b1_,a2_,b2_][t_]:= {a1*t + a2,b1*t + b2}

    lissajous[n_,d_,a_,b_][t_]:= {a*Sin[n*t + d],b*Sin[t]}

    lissajous[n_,d_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[lissajous[n,d,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    lituus[a_][t_]:= {a*t/(t^2)^(3/4)*Cos[Sqrt[t^2]],
                      a*t/(t^2)^(3/4)*Sin[Sqrt[t^2]]}

    lituus[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[lituus[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    logistic[k_,a_,b_][t_]:= {t,k/(1 + E^(a + b*t))}

    logistic[k_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[logistic[k,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    logspiral[b_][t_]:= {E^(b*t)*Cos[t],E^(b*t)*Sin[t]}

    logspiral[a_,b_][t_]:= {a*E^(b*t)*Cos[t],a*E^(b*t)*Sin[t]}

    logspiral[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[logspiral[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    logspiralunitspeed[a_,b_][s_]:= 
         {b*s*Cos[Log[b*s/(a*Sqrt[1 + b^2])]/b]/Sqrt[1 + b^2],
          b*s*Sin[Log[b*s/(a*Sqrt[1 + b^2])]/b]/Sqrt[1 + b^2]}

    nephroid[a_][t_]:= {a*(3*Cos[t] - Cos[3*t]),
                        a*(3*Sin[t] - Sin[3*t])}

    nephroid[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[nephroid[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    nephroidunitspeed[a_][s_]:=
      {s/2 - a*Cos[3*ArcCos[s/(6*a)]],
       a*(3*Sqrt[1 - s^2/(36*a^2)] - Sin[3*ArcCos[s/(6*a)]])}

    ngon[n_,a_][t_]:=
        Which@@Flatten[Table[{(k - 1)/n<=t<=k/n,
        a*{Cos[2*Pi*t],Sin[2*Pi*t]}/Cos[2*Pi*t - (2*k - 1)*Pi/n]},
        {k,1,n}],1]

    ngon[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[ngon[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    nielsenspiral[a_][t_]:= {a*CosIntegral[t],a*SinIntegral[t]}

    nielsenspiral[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[nielsenspiral[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    parabola[a_][t_]:= {2*a*t,a*t^2}

    parabola[a_,b_][t_]:= {2*b*t,a*t^2}

    parabola[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[parabola[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    parabolicspiral[n_,a_,b_][t_]:=
        {(a*t/(t^2)^n + b)*Cos[Sqrt[t^2]],
        (a*t/(t^2)^n + b)*Sin[Sqrt[t^2]]}

    parabolicspiral[n_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[parabolicspiral[n,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    piriform[a_,b_][t_]:=
           {a*(1 + Sin[t]),b*Cos[t]*(1 + Sin[t])}

    piriform[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[piriform[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    pseudocatenary[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[pseudocatenarynds[tmin,tmax][a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    pseudocatenary[a_,b_][s_]:= alysoid[a,b,a][s]

    pseudocatenarykappa2[a_,b_]:= -a/(a*b + s^2)

    pseudocatenarynds[smin_,smax_][a_,b_][s_]:=
         alysoidnds[smin,smax][a,b,a][s]

    pseudocatenaryprime[a_,b_][t_]:= alysoidprime[a,b,a][t]

    rectangle[n_,a_,b_][t_]:=
            {a*(Cos[t]*Sqrt[Cos[t]^2]^(n - 1) +
                Sin[t]*Sqrt[Sin[t]^2]^(n - 1)),
             b*(-Cos[t]*Sqrt[Cos[t]^2]^(n - 1) +
                Sin[t]*Sqrt[Sin[t]^2]^(n - 1))}

    rectangle[n_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[rectangle[n,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    reuleauxpolygon[n_,a_][t_]:= Which@@Flatten[
        Table[{k/(2*n + 1)<=t<=(k + 1)/(2*n + 1),
            a*{Re[E^(2*Pi*I*k/(2*n + 1)) +
               2*I*Sin[Pi*n/(2*n + 1)]*
               E^(Pi*I*((k + n)/(2*n + 1) + t))],
               Im[E^(2*Pi*I*k/(2*n + 1)) +
               2*I*Sin[Pi*n/(2*n + 1)]*
               E^(Pi*I*((k + n)/(2*n + 1) + t))]}},
              {k,0,2*n}],1]

    reuleauxpolygon[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[reuleauxpolygon[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    rose[n_,a_][t_]:= {a*Cos[n*t]*Cos[t],a*Cos[n*t]*Sin[t]}

    sc[t_]:= {t^2,t^3}

    sc[tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[sc[t]],{t,tmin,tmax},opt,
            Axes->None,AspectRatio->Automatic]

    scunitspeed[s_]:= {(s + 8/27)^(2/3) - 4/9,
                      ((s + 8/27)^(2/3) - 4/9)^(3/2)}

    scarab[a_,b_][t_]:= {(a*Cos[2*t] - b*Cos[t])*Cos[t],
                         (a*Cos[2*t] - b*Cos[t])*Sin[t]}

    scarab[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[scarab[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    semicubic[a_,b_,c_,d_][z_]:=
    {WeierstrassP[z,{b^2/12 - a*c/4,-b^3/216 + a*b*c/48 - a^2*d/16}],
     WeierstrassPPrime[z,{b^2/12 - a*c/4,-b^3/216 + a*b*c/48 - a^2*d/16}]}

    semicubic[a_,b_,c_,d_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[semicubic[a,b,c,d][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    serpentine[a_,b_][t_]:= {t,a*t/(1 + b*t^2)}

    serpentine[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[serpentine[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    sinhspiral[n_,a_][t_]:=
            {a*Cos[t]/Sinh[n*t],a*Sin[t]/Sinh[n*t]}

    sinhspiral[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[sinhspiral[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    sinoval[n_,a_][t_]:= {a*Cos[t],a*Nest[Sin,t,n]}

    sinoval[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[sinoval[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    spring[a_,b_,v_][t_]:=
        {a*Cos[t],a*Cos[v]*Sin[t] + b*t*Sin[v]}

    spring[a_,b_,v_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[spring[a,b,v][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    strophoid[a_][t_]:= {a*(t^2 - 1)/(t^2 + 1),
                         t*a*(t^2 - 1)/(t^2 + 1)}

    strophoid[m_,a_][t_]:= {a*(t^2 - m)/(t^2 + 1),
                            t*a*(t^2 - m)/(t^2 + 1)}

    strophoid[m_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[strophoid[m,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    talbot[a_,b_,f_][t_]:= {(a^2 + f^2*Sin[t]^2)*Cos[t]/a,
                            (a^2 - 2*f^2 + f^2*Sin[t]^2)*Sin[t]/b}

    talbot[a_,b_,f_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[talbot[a,b,f][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    tanhspiral[a_][t_]:= {Sinh[2*t]/(Cos[2*a*t] + Cosh[2*t]),
                          Sin[2*a*t]/(Cos[2*a*t] + Cosh[2*t])}

    tanhspiral[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[tanhspiral[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    teardrop[a_,b_,c_,d_,n_][t_]:=
          {a*Sin[t],d*(b*Sin[t] + c*Sin[2*t])^n}

    teardrop[a_,b_,c_,d_,n_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[teardrop[a,b,c,d,n][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    tractrix[a_][t_]:= {a*Sin[t],a*(Cos[t] + Log[Tan[t/2]])}

    tractrix[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[tractrix[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    tractrix[a_,b_][t_]:= {a*Sin[t],b*(Cos[t] + Log[Tan[t/2]])}

    tractrixminus[a_][t_]:= {t,Sqrt[a^2 - t^2] - a*Log[2/(a*t) +
         2*Sqrt[a^2 - t^2]/(a^2*t)] + a*Log[2/t^2]}

    tractrixminus[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[tractrixminus[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    tractrixplus[a_][t_]:= {t,-Sqrt[a^2 - t^2] + a*Log[2/(a*t) +
         2*Sqrt[a^2 - t^2]/(a^2*t)] - a*Log[2/t^2]}

    tractrixplus[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[tractrixplus[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    tractrixunitspeed[a_][s_]:= {a*E^(-Sqrt[s^2]/a),
        a*ArcTanh[Sqrt[1 - E^(-2*Sqrt[s^2]/a)]] -
        a*Sqrt[1 - E^(-2*Sqrt[s^2]/a)]}

    triangle[t_]:= If[t<1,t*{1,0},
         If[t<2,{1,0} + (t - 1)*{-1/2,Sqrt[3]/2},
         {1/2,Sqrt[3]/2} + (t - 2)*{-1/2,-Sqrt[3]/2}]]

    triangle[tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[triangle[t]],{t,tmin,tmax},opt,
            Axes->None,AspectRatio->Automatic]

    trident[a_,b_,c_,d_][t_]:= {t,a*t^2 + b*t + c + d/t}

    trident[a_,b_,c_,d_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[trident[a,b,c,d][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    trisectrix[a_][t_]:=
        {a*(1 - 4*Cos[t]^2),a*(1 - 4*Cos[t]^2)*Tan[t]}

    trisectrix[a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[trisectrix[a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    tschirnhausen[n_,a_][t_]:= {a*Cos[t]/Cos[t/3]^n,
                                a*Sin[t]/Cos[t/3]^n}

    tschirnhausen[n_,a_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[tschirnhausen[n,a][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    watt[a_,b_,c_][t_]:=
        {Sqrt[b^2 - (a*JacobiCN[t,a^2/c^2] +
              c*JacobiDN[t,a^2/c^2])^2]*Cos[t],
         Sqrt[b^2 - (a*JacobiCN[t,a^2/c^2] +
              c*JacobiDN[t,a^2/c^2])^2]*Sin[t]}

    watt[a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[watt[a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

    zcprofile[pm_,a_,c_][t_]:=
       {t,pm*Sqrt[a^2*t^2 - c^2] + c*ArcSin[c/(a*t)]}

    zcprofile[pm_,a_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot[Evaluate[zcprofile[pm,a,c][t]],
            {t,tmin,tmax},opt,Axes->None,AspectRatio->Automatic]

(* Implicitly Defined Plane Curves *)

    agnesiimplicit[a_][x_,y_]:= x^2*y - 4*a^2*(2*a - y)

    astroidimplicit[n_,a_,b_][x_,y_]:=
              (x/a)^(2/n) + (y/b)^(2/n) - 1

    bulletnoseimplicit[a_,b_][x_,y_]:= (a/x)^2 - (b/y)^2 - 1

    cardioidimplicit[a_][x_,y_]:=
            (x^2 + y^2 - 2*a*x)^2 - 4*a^2*(x^2 + y^2)

    cartesianimplicit[a_,c_,m_][x_,y_]:=
            ((1 - m^2)*(x^2 + y^2) + 2*m^2*c*x + a^2 - m^2*c^2)^2 -
              4*a^2*(x^2 + y^2)

    cassiniimplicit[a_,b_][x_,y_]:=
             (x^2 + y^2 + a^2)^2 - b^4 - 4*a^2*x^2

    circleimplicit[a_][x_,y_]:= x^2 + y^2 - a^2

    cissoidimplicit[a_][x_,y_]:= x^3 + x*y^2 - 2*a*y^2

    conchoidimplicit[a_,b_][x_,y_]:=
                   (x^2 + y^2)*(x - b)^2 - a^2*x^2

    crosscurveimplicit[a_,b_][x_,y_]:= (a/x)^2 + (b/y)^2 - 1

    cubicparabolaimplicit[a_,b_,c_,d_][x_,y_]:=
          a*x^3 + b*x^2 + c*x + d - y

    deltoidimplicit[a_][x_,y_]:= (x^2 + y^2)^2 -
         8*a*x*(x^2 - 3*y^2) + 18*a^2*(x^2 + y^2) - 27*a^4

    devilimplicit[a_,b_][x_,y_]:= y^2*(y^2 - b^2) - x^2*(x^2 - a^2)

    duererimplicit[a_,b_][x_,y_]:=
             2*y^2*(x^2 + y^2) + 2*b(x + y)*(a^2 - y^2) +
             (b^2 - 3*a^2)*y^2 - a^2*x^2 +a^2*(a^2 - b^2)

    ellipseimplicit[a_,b_][x_,y_]:= x^2/a^2 + y^2/b^2 - 1

    foliumimplicit[x_,y_]:= x^3 + y^3 - 3*x*y

    foliumimplicit[n_,a_,b_,c_][x_,y_]:=
             a*x^(2*n + 1) + b*y^(2*n + 1) - c*x^n*y^n

    hippopedeimplicit[a_,b_][x_,y_]:=
         (x^2 + y^2)^2 + 4*b*(b - a)*(x^2 + y^2) - 4*b^2*x^2

    hyperbolaimplicit[a_,b_][x_,y_]:= x^2/a^2 - y^2/b^2 - 1

    keplerimplicit[a_,b_][x_,y_]:= ((x - b)^2 + y^2)*
               (x*(x - b) + y^2) - 4*a*(x - b)*y^2

    lemniscateimplicit[a_][x_,y_]:=
               (x^2 + y^2)^2 - a^2*(x^2 - y^2)

    lemniscateimplicit[xp_,yp_][x_,y_]:=
	Product[(x - xp[[j]])^2+(y - yp[[j]])^2,{j,Length[xp]}]

    limaconimplicit[a_,b_][x_,y_]:=
         (x^2 + y^2 - 2*a*x)^2 - b^2*(x^2 + y^2)

    nephroidimplicit[a_][x_,y_]:=
           (x^2 + y^2 - 4*a^2)^3 - 108*a^4*y^2

    newtonphillipsimplicit[m_,n_,a_,b_][x_,y_]:=
        Sin[y]*Sin[m*y] - a*Sin[x]*Sin[n*x] - b

    parabolaimplicit[a_][x_,y_]:= x^2 - 4*a*y

    perseusimplicit[a_,b_,c_][x_,y_]:=
            (x^2 + y^2 + a^2 - b^2 +c^2)^2 - 4*a^2*(c^2 + x^2)

    piriformimplicit[a_,b_][x_,y_]:= a^4*y^2 - b^2*x^3*(2*a - x)

    predatorpreyimplicit[a_,b_,p_,q_,c_][x_,y_]:=
             y^a*x^p*E^(-b*y - q*x) -
             c*(a/b)^a*(p/q)^p*E^(-a - p)

    scimplicit[x_,y_]:= y^2 - x^3

    semicubicimplicit[a_,b_,c_,d_][x_,y_]:=
                      a*x^3 + b*x^2 + c*x + d - y^2

    serpentineimplicit[a_,b_][x_,y_]:= y - a*x + b*x^2*y

    strophoidimplicit[a_][x_,y_]:= y^2*(a - x) - x^2*(a + x)

    strophoidimplicit[m_,a_][x_,y_]:= y^2*(a - x) - x^2*(a*m + x)

    tridentimplicit[a_,b_,c_,d_][x_,y_]:=
                      a*x^3 + b*x^2 + c*x + d - x*y

    wattimplicit[a_,b_,c_,d_][x_,y_]:= (x^2 + y^2)^3 -
        2*(a^2 + b^2 - c^2)^2*(x^2 + y^2)^2 +
        ((a^2 + b^2 - c^2)^4 + 4*a^2y^2)*(x^2 + y^2) -
        4*a^2*b^2*y^2

(* Polar Defined Plane Curves *)

    archimedesspiralpolar[n_,a_][theta_]:= a*theta^(1/n)

    bowpolar[a_][theta_]:= a*(1 - Tan[theta]^2)

    cardioidpolar[a_][theta_]:= 2*a*(1 + Cos[theta])

    cassinipolar[a_,b_,pm1_,pm2_][theta_]:=
           pm1*Sqrt[a^2*Cos[2*theta] +
           pm2*Sqrt[b^4 - a^4*Sin[2*theta]^2]]
       
    cochleoidpolar[n_,a_][theta_]:= a*(Sin[theta]/theta)^n

    epispiralpolar[n_,a_][theta_]:= a/Sin[n*theta]

    fermatspiralpolar[a_][theta_]:= a*Sqrt[theta]

    foliumpolar[theta_]:=
         3*Cos[theta]*Sin[theta]/(Cos[theta]^3 + Sin[theta]^3)

    foliumpolar[n_,a_,b_,c_][theta_]:=
         c*Cos[theta]^n*Sin[theta]^n/
         (a*Cos[theta]^(2*n + 1) + b*Sin[theta]^(2*n + 1))

    hyperbolicspiralpolar[a_][theta_]:= a/theta

    kampylepolar[a_][theta_]:= a*Sec[theta]^2

    keplerorbitpolar[a_,e_][theta_]:= a/(1 - e*Cos[theta])

    lemniscatepolar[a_][theta_]:= a*Sqrt[Cos[2*theta]]

    limaconpolar[a_,b_][theta_]:= 2*a*Cos[theta] + b

    limaconpolar[n_,a_,b_][theta_]:= 2*a*Cos[n*theta] + b

    lituuspolar[a_][theta_]:= a/Sqrt[theta]

    logspiralpolar[a_,b_][theta_]:= a*E^(b*theta)

    nephroidpolar[a_][theta_]:=
        2*a*(Sin[theta/2]^(2/3) + Cos[theta/2]^(2/3))^(3/2)
 
    pacman[n_][theta_]:= 1 + Cos[theta]^n

    rosepolar[n_,a_][theta_]:= a*Cos[n*theta]

(* Parametrically Defined Space Curves *)

    ast3d[n_,a_,b_][t_]:= {a*Cos[t]^n,b*Sin[t]^n,Cos[2*t]}

    ast3d[n_,a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[ast3d[n,a,b][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    baseballseam[a_,c_][t_]:=
        {a*Sin[Pi/2 - (Pi/2 - c)*Cos[t]]*Cos[t/2 + c*Sin[2*t]],
         a*Sin[Pi/2 - (Pi/2 - c)*Cos[t]]*Sin[t/2 + c*Sin[2*t]],
         a*Cos[Pi/2 - (Pi/2 - c)*Cos[t]]}

    baseballseam[a_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[baseballseam[a,c][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    besselcurve[a_,b_,c_][t_]:=
         {BesselJ[a,t],BesselJ[b,t],BesselJ[c,t]}

    besselcurve[a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[besselcurve[a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    bicylinder[a_,b_,pm_][t_]:= {a*Cos[t],a*Sin[t],
                                 pm*Sqrt[b^2 - a^2*Sin[t]^2]}

    bicylinder[a_,b_,pm_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[bicylinder[a,b,pm][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    biquadratic[a_,k_][t_]:=
           {a*JacobiSN[t,k],a*JacobiCN[t,k],a*JacobiDN[t,k]}

    biquadratic[a_,k_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[biquadratic[a,k][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    clelia[a_][m_,n_][t_]:=
           {a*m*Cos[t]*Sin[n*t],a*m*Sin[t]*Sin[n*t],
            a*Sqrt[1 - m^2*Sin[n*t]^2]}

    clelia[a_][m_,n_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[clelia[a][m,n][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    conicalhelix[a_,b_,c_,d_][t_]:= {a*t*Cos[d*t],b*t*Sin[d*t],c*t}

    conicalhelix[a_,b_,c_,d_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[conicalhelix[a,b,c,d][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->500]

    cubicalellipse[a_,b_,c_,d_,e_,f_,pm_][t_]:=
           {a*t/(1 + pm*t^2),
           (b*t + c*t^2)/(1 + pm*t^2),
           (d*t + e*t^2 + f*t^3)/(1 + pm*t^2)}

    cubicalellipse[a_,b_,c_,d_,e_,f_,pm_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[cubicalellipse[a,b,c,d,e,f,pm][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    eightknot[t_]:=
        {10*(Cos[t] + Cos[3*t]) + Cos[2*t] + Cos[4*t],
         6*Sin[t] + 10*Sin[3*t],
         4*Sin[3*t]*Sin[5*t/2] + 4*Sin[4*t] - 2*Sin[6*t]}

    eightknot[tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[eightknot[t]],{t,tmin,tmax},opt,
            Axes->None,PlotPoints->150]

    epitrochoid3d[a_,b_,h_,omega_][t_]:=
        {Cos[t]*(a + b*Cos[omega]) - 
         h*(Cos[t]*Cos[a*t/b]*Cos[omega] - Sin[t]*Sin[a*t/b]),
         Sin[t]*(a + b*Cos[omega]) - 
         h*(Sin[t]*Cos[a*t/b]*Cos[omega] + Cos[t]*Sin[a*t/b]),
        (b - h*Cos[a*t/b])*Sin[omega]}

    epitrochoid3d[a_,b_,h_,omega_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[epitrochoid3d[a,b,h,omega][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    genhelix[a_,b_][f_][t_]:= {a*Cos[t],b*Sin[t],f[t]}

    helix[a_,c_][t_]:= {a*Cos[t],a*Sin[t],c*t}

    helix[a_,b_,c_][t_]:= {a*Cos[t],b*Sin[t],c*t}

    helix[a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[helix[a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    helixunitspeed[a_,b_][s_]:=
        {a*Cos[s/Sqrt[a^2 + b^2]],a*Sin[s/Sqrt[a^2 + b^2]],
         b*s/Sqrt[a^2 + b^2]}

    horopter[a_,b_][t_]:=
        {2*a/(1 + b^2*t^2),2*a*b*t/(1 + b^2 t^2),t}

    horopter[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[horopter[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    line3d[a1_,b1_,c1_,a2_,b2_,c2_][t_]:=
         {a1*t + a2,b1*t + b2,c1*t + c2}

    lissajous3d[n_,d_,a_,b_,c_][t_]:=
         {a*Sin[n*t + d],b*Sin[t],c*Cos[t]} 

    lissajous3d[n_,d_,a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[lissajous3d[n,d,a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    oneparametersubgroup[mat_,vec_][t_]:=
        ComplexExpand[Normal[MatrixExp[Rationalize[t*mat]]].vec]

    pseudospheregeodesic[a_,c_,u0_,pm_][t_]:=
        {a*Sin[t]*Cos[u0 + pm*Sqrt[a^2 - 2*c^2 - a^2*Cos[2*t]]/
         (Sqrt[2]*c*Sin[t])],
         a*Sin[t]*Sin[u0 + pm*Sqrt[a^2 - 2*c^2 - a^2*Cos[2*t]]/
         (Sqrt[2]*c*Sin[t])],a*(Cos[t] + Log[Tan[t/2]])}

    pseudospheregeodesic[a_,c_,u0_,pm_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[pseudospheregeodesic[a,c,u0,pm][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    pseudosphericalloxodrome[a_,b_][t_]:=
         {a*Cos[b*Cosh[t]]/Cosh[t],
          a*Sin[b*Cosh[t]]/Cosh[t],a*(t - Tanh[t])}

    pseudosphericalloxodrome[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[pseudosphericalloxodrome[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->500]

    seiffertspiral[a_,k_][t_]:=
        {a*Cos[t]*JacobiSN[t,k],a*Sin[t]*JacobiSN[t,k],
         a*JacobiCN[t,k]}

    seiffertspiral[a_,k_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[seiffertspiral[a,k][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    sphericalcardioid[a_][t_]:=
        {2*a*Cos[t] - a*Cos[2*t],2*a*Sin[t] - a*Sin[2*t],
         Sqrt[8]*a*Cos[t/2]}

    sphericalcardioid[a_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[sphericalcardioid[a][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    sphericalellipse[a_,b_,c_,pm_][t_]:=
        {b*JacobiCN[t,(c^2 - b^2)/(a^2 - b^2)],
         c*JacobiSN[t,(c^2 - b^2)/(a^2 - b^2)],
         pm*Sqrt[a^2 - b^2]*JacobiDN[t,(c^2 - b^2)/(a^2 - b^2)]}

    sphericalellipse[a,b,c,pm][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[sphericalellipse[a,b,c,pm][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    sphericalhelix[a_,b_][t_]:=
        {(a + b)*Cos[t] - b*Cos[(a + b)*t/b],
         (a + b)*Sin[t] - b*Sin[(a + b)*t/b],
          2*Sqrt[a*b + b^2]*Cos[a*t/(2*b)]}

    sphericalhelix[a_,b_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[sphericalhelix[a,b][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    sphericalloxodrome[a_,b_,c_][t_]:=
        {2*a*b*E^(c*t)*Cos[t]/(1 + b^2*E^(2*c*t)),
         2*a*b*E^(c*t)*Sin[t]/(1 + b^2*E^(2*c*t)),
         a*(-1 + b^2*E^(2*c*t))/(1 + b^2*E^(2*c*t))}

    sphericalloxodrome[a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[sphericalloxodrome[a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    sphericalloxodromeunitspeed[a_,b_,c_][s_]:=
        {2*a*Cos[Log[Tan[c*s/(2*a*Sqrt[1 + c^2])]]/c]*
          Tan[c*s/(2*a*Sqrt[1 + c^2])]/
          (1 + b^2*Tan[c*s/(2*a*Sqrt[1 + c^2])]^2), 
        2*a*Sin[Log[Tan[c*s/(2*a*Sqrt[1 + c^2])]]/c]*
          Tan[c*s/(2*a*Sqrt[1 + c^2])]/
          (1 + b^2*Tan[c*s/(2*a*Sqrt[1 + c^2])]^2), 
          -a*Cos[c*s/(a*Sqrt[1 + c^2])]}

    sphericalnephroid[a_][t_]:=
        {3*a*Cos[t] - a*Cos[3*t],3*a*Sin[t] - a*Sin[3*t],
         Sqrt[12]*a*Cos[t]}

    sphericalnephroid[a_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[sphericalnephroid[a][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    sphericalspiral[a_][m_,n_][t_]:=
          {a*Cos[m*t]*Cos[n*t],a*Sin[m*t]*Cos[n*t],a*Sin[n*t]}

    sphericalspiral[a_][m_,n_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[sphericalspiral[a][m,n][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    tennisballseam[n_,a_,b_,c_][t_]:=
          {-a*(Cos[t]*(b^2 + c^2*n^2*Cos[n*t]^2) + 
           c^2*n^3*Sin[t]*Sin[2*n*t]/2)/
           (Sqrt[b^2*Cos[t]^2 + c^2*n^2*Cos[n*t]^2 + 
            a^2*Sin[t]^2]*Sqrt[a^2*b^2*Cos[t]^4 + 
            b^2*c^2*n^2*Cos[n*t]^2*Sin[t]^2 + 
            a^2*b^2*Sin[t]^4 + a^2*c^2*n^4*Sin[t]^2*Sin[n*t]^2 + 
            Cos[t]^2*(a^2*c^2*n^2*Cos[n*t]^2 + 
            b^2*(2*a^2*Sin[t]^2 + c^2*n^4*Sin[n*t]^2)) + 
            a^2*c^2*n^3*Sin[2*t]*Sin[2*n*t]/2 - 
            b^2*c^2*n^3*Sin[2*t]*Sin[2*n*t]/2]),
            b*(-2*(a^2 + c^2*n^2*Cos[n*t]^2)*Sin[t] + 
            c^2*n^3*Cos[t]*Sin[2*n*t])/
            (2*Sqrt[b^2*Cos[t]^2 + c^2*n^2*Cos[n*t]^2 + 
            a^2*Sin[t]^2]*Sqrt[a^2*b^2*Cos[t]^4 + 
            b^2*c^2*n^2*Cos[n*t]^2*Sin[t]^2 + 
            a^2*b^2*Sin[t]^4 + a^2*c^2*n^4*Sin[t]^2*Sin[n*t]^2 + 
            Cos[t]^2*(a^2*c^2*n^2*Cos[n*t]^2 + 
            b^2*(2*a^2*Sin[t]^2 + c^2*n^4*Sin[n*t]^2)) + 
            a^2*c^2*n^3*Sin[2*t]*Sin[2*n*t]/2 - 
            b^2*c^2*n^3*Sin[2*t]*Sin[2*n*t]/2]),
            -c*n*((a^2 - b^2)*Cos[t]*Cos[n*t]*Sin[t] + 
            b^2*n*Cos[t]^2*Sin[n*t] + a^2*n*Sin[t]^2*Sin[n*t])/
            (Sqrt[b^2*Cos[t]^2 + c^2*n^2*Cos[n*t]^2 + 
            a^2*Sin[t]^2]*Sqrt[a^2*b^2*Cos[t]^4 + 
            b^2*c^2*n^2*Cos[n*t]^2*Sin[t]^2 + 
            a^2*b^2*Sin[t]^4 + a^2*c^2*n^4*Sin[t]^2*Sin[n*t]^2 + 
            Cos[t]^2*(a^2*c^2*n^2*Cos[n*t]^2 + 
            b^2*(2*a^2*Sin[t]^2 + c^2*n^4*Sin[n*t]^2)) + 
            a^2*c^2*n^3*Sin[2*t]*Sin[2*n*t]/2 - 
            b^2*c^2*n^3*Sin[2*t]*Sin[2*n*t]/2])}

    tennisballseam[n_,a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[tennisballseam[n,a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    torusknot[a_,b_,c_][p_,q_][t_]:=
         {(a + b*Cos[q*t])*Cos[p*t],
          (a + b*Cos[q*t])*Sin[p*t],c*Sin[q*t]}

    torusknot[a_,b_,c_][p_,q_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[torusknot[a,b,c][p,q][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

(*
     tprime[a_,b_][v_]:=1/Sqrt[-Cos[v]*(a/b + Cos[v])]


    torusasym[c_,d_,a_,b_][s_]:=
         Module[{uu,tmp1,tmp2,ss},
             tmp1=NDSolve[{uu'[ss]==tprime[a,b][ss],
               uu[c]==d},uu,{ss,0.5001Pi,1.4999Pi},
               MaxSteps->2000];
               tmp2=First[uu[s] /. tmp1];
		{(a + b*Cos[s])*Cos[tmp2],(a + b*Cos[s])*Sin[tmp2],
		b*Sin[s]}]
*)

    twicubic[t_]:= {t,t^2,t^3}

    twicubic[tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[twicubic[t]],{t,tmin,tmax},opt,
            Axes->None,PlotPoints->150]

    veryflat[1][t_]:= {t,0,5*E^(-1/t^2)}

    veryflat[1][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[veryflat[1][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    veryflat[2][t_]:= If[t<0,{t,5*E^(-1/t^2),0},
                     If[t==0,{0,0,0},{t,0,5*E^(-1/t^2)}]]

    veryflat[2][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[veryflat[2][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    viviani[a_][t_]:= {a*(1 + Cos[t]),a*Sin[t],2*a*Sin[t/2]}

    viviani[a_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[viviani[a][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

    wigglyellipse[n_,a_,b_,c_][t_]:= {a*Cos[t],b*Sin[t],c*Sin[n*t]}

    wigglyellipse[n_,a_,b_,c_][tmin_,tmax_,opt___]:=
        ParametricPlot3D[Evaluate[wigglyellipse[n,a,b,c][t]],
            {t,tmin,tmax},opt,Axes->None,PlotPoints->150]

(* Parametrically Defined Curves in R^n*)

    besseln[n_,a_][t_]:= Table[a*BesselJ[k,t],{k,0,n - 1}]

    bernstein[p_,q_][t_]:=Binomial[p,q]*t^q*(1 - t)^(p - q)
    
    beziercurve[pts_][t_]:= Module[{n=Length[pts]},
         Sum[bernstein[n - 1,k][t]*pts[[k + 1]],
         {k,0,n - 1}]]

         cosn[n_,a_][t_]:= Table[a*Sin[k*t],{k,1,n}]

    hermitecurve[p0_,m0_,m1_,p1_][t_]:=
        (bernstein[3,0][t] + bernstein[3,1][t])*p0 +
        bernstein[3,1][t]*m0 - bernstein[3,2][t]*m1 +
        (bernstein[3,2][t] + bernstein[3,3][t])*p1

    ell[ts_,i_,t_]:=Product[(ts[[j]]-t)/(ts[[j]]-ts[[i]]),{j,1,i-1}]*
        Product[(ts[[j]]-t)/(ts[[j]]-ts[[i]]),{j,i+1,Length[ts]}]
		
    lagrangecurve[pts_,ts_][t_]:= Module[{n=Length[pts]},
         Sum[ell[ts,k+1,t]*pts[[k+1]],{k,0,n - 1}]]

    sinn[n_,a_][t_]:= Table[a*Sin[k*t],{k,1,n}]

    twistedn[n_,a_][t_]:= Table[a*t^k,{k,1,n}]

(* Approximations *)

    regularpolygon[n_,a_]:=
        approxcurve[0,2*Pi,n][circle[a]]

    teeth[m_,t_][shortlines_]:=
	    Nest[Flatten[(tooth[t]/@#)]&,shortlines,m]

(*
    tooth[t_][Line[{{a_,b_},{c_,d_}}]]:=
        {Line[{{a,b},(2/3)*{a,b} + (1/3)*{c,d}}],
         Line[{(2/3)*{a,b} + (1/3)*{c,d},
         (t/2)*{a - b + c + d,a + b - c + d} +
         ((1 - t)/2)*{a + b + c - d,-a + b + c + d}}],
         Line[{(t/2)*{a - b + c + d,a + b - c + d} +
         ((1 - t)/2)*{a + b + c - d,-a + b + c + d},
         (1/3)*{a,b} + (2/3)*{c,d}}],
        Line[{(1/3)*{a,b} + (2/3)*{c,d},{c,d}}]}
*)

tooth[t_][Line[{{a_,b_},{c_,d_}}]]:=
        {Line[{{a,b},(2/3)*{a,b} + (1/3)*{c,d}}],
         Line[{(2/3)*{a,b} + (1/3)*{c,d},
         {(a + c)/2 + t*(b - d),(b + d)/2 +t*(c - a)}}],
         Line[{{(a + c)/2 + t*(b - d),(b + d)/2 +t*(c - a)},
         (1/3)*{a,b} + (2/3)*{c,d}}],
        Line[{(1/3)*{a,b} + (2/3)*{c,d},{c,d}}]}

   kochsnowflake[m_]:=
    teeth[m,(3 + Sqrt[3])/6][linetolines[regularpolygon[3,1]]]

   kochsnowflake[m_,t_]:=
    teeth[m,t][linetolines[regularpolygon[3,1]]]
    
End[]

Protect["`*"];

EndPackage[]

