(* ::Package:: *)

(* ::Subsection:: *)
(*(* euvecv2.m *)*)


(* ::Text:: *)
(*(* Created by Rolf Sulanke *)*)
(*(* Version v2 2016.06.30. *)*)
(**)
(*(* Version 1: February 24, 1999. Revised May 25, 1999. Revised October 4, 2000.*)
(*Revised January 30, 2010,revised November 24,2011, February 2013. *)*)
(**)
(*(*Warning. The package can not substitute the package euvec.m in the item *)
(*Spheres, MathSource, Enhancements/Geometry 0210-092. *)*)
(**)
(*(* This package contains definitions and procedures for n-dimensional*)
(*Euclidean geometry in terms of vector algebra. Applications, explanations,*)
(*and certain more special concepts are included in the notebook Euklid.nb *)*)
(**)
(*(* Keywords: vector objects, null vector, basis vectors, random vector,Circles in 3 dimensions,*)
(*normalization, cross product, stereographic projection, inverse stereographic*)
(*projection, hyperplanes, spheres, spheres through four points, hyperspheres,*)
(*orthoframes in the 4-dimensional Euclidean vector space,*)
(*Erhard Schmidt's orthogonalization, orthogonal complement. *)*)
(**)
(*(*Most of the concepts depend on the dimension dim, which must be defined in*)
(*the  Global Context! *)*)
(**)
(*(* Main changes: orthoerg, orthoframe, and ortho1 are replaced by the*)
(* more general orthocomplement,*)
(*orthobase[v] is replaced by the help of orthocomplement[{v}].*)
(*November 2011: Basic functions for circles in the 3-space transferred from the package mcirc.m into this package.*)
(*Definition of unit simplified and corrected, December 2, 2015.*)*)
(**)
(*(*Change February 2013: instead of vectorcalc.m now the enlarged package tensalgv3.m *)
(*will be loaded together with euvec.v2m. *)*)


(* ::Subsection:: *)
(*(* Copyright*)


(* ::Text:: *)
(*This notebook and the accompanying packages are  public. Authors who intend to publish a changed or completed version of them should do it under their own names with the condition that they cite the original notebook with the Internet address or other source where they got it. I am not responsible for errors or damages originated by the use of the procedures contained in my notebooks or packages; everybody who applies them should test carefully whether they are appropriate for his purposes. *)*)
(**)


BeginPackage["euvecv2`",{"tensalgv3`"}]


(*The usages *)

cross::usage = "cross[v] gives the vector product of a list v of 
n - 1 vectors of the Euclidean space of dimension n ."

sterproj::usage ="sterproj is the stereographical projection of the unit 
n-sphere from the north pole stb[n+1] onto the Euclidean (equatorial) n-space. 
WARNING: The result is correct only for arguments of norm 1 !";

invsterproj::usage="invsterproj is the inverse of the general Euclidean 
stereographic projection sterproj.";

spherpt3D::usage = "If a is a vector of dimension 4, then spherpt3D[a] is 
the graphics3D primitive Point of the stereographical projection sterproj of 
the normed vector a.";

hyperplane::usage ="hyperplane[plist] returns the normal vector n and the
distance p from the origin of  the hyperplane  of the N-dimensional Euclidean
space spanned by the N points in general position listed in plist; plist must
be a square  matrix of order N."

unit::usage = "unit[s_List] or unit[s1,...,sn], s ={s1,...,sn}, generates a parameter 
representation of the unit n-sphere with centre o in the Euclidean (n+1)-space. 
The parameter ranges  0 <s1<=2Pi, -Pi/2 <= si <= Pi/2,i = 2,...,n, give the whole
n-sphere."

sphericalreflection::usage = "sphericalreflection[x, a, r] gives the point in
 inversion with x \
with 
respect to the hypersphere with center a and radius r."

esorthonorm::usage = "esorthonorm[b_List, opts] is the Erhard Schmidt
orthogonalization for the list b of vectors with respect to a positive
semi-definite scalar product which can be specified as an option; the
default is Dot. esorthonorm may be replaced by the built-in function 
Orthogonalize now."

norm::usage = "norm[v] is the norm of the vector v with respect to a positive
semi-definite scalar product which can be specified as an option for innerprod; the
default is Dot."

innerprod::usage = "innerprod is an option for esorthonorm, norm, and orthonorm;the
default is Dot (esorthonorm), resp. pssp (orthonorm)."

normed::usage = "normed is an option for esorthonorm and orthonorm which
 specifies whether
the vectors produced by esorthonorm or orthonorm are normed, or only
 orthogonal;the default
is True."

print::usage = "print is an option for esorthonorm and orthonorm which specifies
 whether
the (unnormed) vectors produced by esorthonorm or orthonorm are printed during
 the
orthogonalization procedure;the default is False."

neglect::usage = "neglect is an option for esorthonorm and orthonorm which
 specifies whether
vectors with components or scalar square less than 10^neglect
produced by esorthonorm  or orthonorm during the orthogonalization procedure are
 set to
zero;the default is neglect -> -10.For neglect -> 0 no neglecting takes place."

orthocomplement::usage = "In a finite-dimensional vector space provided with a 
positive definite scalar product orthocomplement[vList, opts] yields an orthonormal
basis of the orthogonal complement of the span of vList; opts are the same as to
esorthonorm." ;

(* Only for the case dim = 4 :*)

subsphere::usage = "subsphere[n,p][v,w] is the parameter  representation of the
2-sphere in the Euclidean 4-space, in which  the hyperplane n.x=p intersects
the unit 3-sphere. n is a normal vector of the hyperplane,  
vp=p/Sqrt[n.n] is the distance  of the hyperplane from the origin, 
which must satisfy -1<vp<1; 
v (0 <= v < 2Pi), w  (-Pi/2 <= w <= Pi/2), are the parameters for the
2-sphere.  Its radius is Sqrt[1-vp^2], and its center is n*p/n.n."

tg4frame::usage = "tg4frame[s,t,u] is an orthonormal moving frame in the
Euclidean 4-space such that the fourth vector is the radius vector (= normal
vector) of the unit 3-sphere with geographic coordinates s,t,u."

subspheremf::usage ="subspheremf[s,t,u,p][v,w] is the parameter  representation
 of
the 2-sphere in the Euclidean 4-space, in which  the hyperplane n.x=p
intersects the unit 3-sphere. 
s,t,u are geographical coordinates on the unit 3-sphere, 
n=unit[{s,t,u}] is the normed normal vector of the hyperplane, 
p, -1<p<1, is the distance of the  hyperplane from the origin, 
v  (0 <= v < 2Pi), w  (-Pi/2 <= w <= Pi/2), are the parameters for the
2-sphere.  Its radius is Sqrt[1-p^2], and its center is n*p."

(*For Graphics3D, indepentently of the value of dim: *)
 
sphere::usage = "sphere[m,r][u,v] is the parameter representation of 
the sphere with center m and radius r in the Euclidean 3-space with geographical
 
coordinates u, 0<=u<=2 Pi, v, -Pi/2 <= v <= Pi/2."

sphereplot3D::usage = "sphereplot3D[m,r, u0,u1,v0,v1,opt] is the plot of the 
part of the sphere with center m and radius r, which is defined by u0<=u<=u1, 
v0<=v<=v1; the default is the whole sphere given by the default values u0=0, 
u1=2 Pi, v0 =-Pi/2, v1=Pi/2. The options opt are transferred to
ParametricPlot3D."

sph3D::usage = "sph3D[u,v,w] is the standard parameter representation of the unit 
3-sphere with center at o in the 4-dimensional Euclidean vector space with 
parameters u, 0<= u <= 2Pi, v,w, -Pi/2 <=v,w<=Pi/2.";

center3pts::usage = "center3pts[p1,p2,p3] returns the center of the circle 
through the three points p1,p2,p3, or the infinite vector,
if the points belong to a line.";

posvec3pts::usage= "posvec3pts[p1,p2,p3] is the position vector of the circle 
through the 3 points p1,p2,p3 in the Euclidean 3-space: it is a unit vector orthogonal to the plane of 
the circle uniquely defined up to sign. If the points belong to a 
line posvec3pts returnes the null vector null[3].";

radius3pts::usage = "radius3pts[p1,p2,p3] returns the radius of the circle 
through the three points p1,p2,p3, or Infinity, if the points belong 
to a line.";

circle3pts::usage = "circle3pts[p1,p2,p3][t] is a parameter representation of \
the circle (or line) through the three points p1,p2,p3.";

plotcircle3pts::usage = "plotcircle3pts[p1,p2,p3] plots the circle 
(or line) through the three points p1,p2,p3.";

(* WARNING: Always, if there exist a line through the three points, e.g. if 
the points coincide, center3pts and radius3pts are infinite!*)

circle3D::usage = "circle3D[x,y,z,u,v,r][t] is the parameter
representation of the circle in the Euclidean 3-space with  radius r and 
center {x,y,z} in the plane orthogonal to the position vector 
unit[{u,v]]={Cos[u] Cos[v], Sin[u] Cos[v], Sin[v]}.";
  
gencircle::usage = "gencircle[p_List, v_List][t] is the parameter
representation of the circle in the Euclidean 3-space with radius r = norm[v] \
and center p in the plane orthogonal to the vector v.";
  
plotcircle3D::usage = "plotcircle3D[x,y,z,u,v,r,opts] plots  the circle in 
the Euclidean 3-space with radius r and center {x,y,z} in the  plane 
orthogonal to the vector unit[u,v]={Cos[u] Cos[v], Sin[u] Cos[v], Sin[v]}, \
with options opts of ParametricPlot3D.";
  
plotgencircle::usage = "plotgencircle[p_List,v_List, opts] plots  the circle 
in the Euclidean 3-space with radius norm[v]  and center p in the 
plane orthogonal to  the vector v with options opts of ParametricPlot3D.";

tube::usage = "tube[x,a,b,r,opt] plots the tube of the curve x (Enter only 
the name, not the parameter!)  with tube radius r for the parameter s, 
a<s<b, with options opt of ParametricPlot3D.";

  
unitvec::usage = "unitvec[u,v][t] is for fixed u,v and variable t, 
0 < t < 2 Pi, a parameter representation of the  unit circle with center o in \
the plane orthogonal to the radius vector 
unit{u,v}]={Cos[u] Cos[v], Sin[u] Cos[v], Sin[v]} 
of the unit sphere in the Euclidean 3-space.";	 
  


Begin["`Private`"]
  
(* The dimension must be set within the Global Context after loading the package. *)
(*
dimm := Global`dim;  
Definition of the stereographic projection of the n-sphere onto the \
\
n-dimensional Euclidean space: *)

sterproj[v_List] :=Module[{m=Length[v],l, w},
		l=null[m];
		w=v[[m]];
		If[smoothing[w - 1]== 0,For[i=1,i<m,i++,l[[i]]=Infinity],
		l=Simplify[ stb[m,m] + (v- stb[m,m])/(1- w)],
		l=Simplify[ stb[m,m] + (v- stb[m,m])/(1- w)]];
		Take[l,m-1]]
		


(*The inverse stereographic projection*)

invsterproj[y_List]:= Module[{n=Length[y]+1,x=Append[y,0], u=y.y},
If[u == Infinity, Return[stb[n,n]]];
		x*2/(1+u) +stb[n,n]*(u-1)/(u+1)]


spherpt3D[a_] := Point[sterproj[dotnorme[a]]]

(* A private version of the Euclidean cross product for arbitrary dimension:*)

cross[b_List] := Module[{d = Length[b] + 1,v},
		v = null[d];
			For[i=1,i<d,i++,If[Length[b[[i]] ]!= d, 
			Print[b[[i]], " is not a correct argument!"];Return[],
			v[[i]] = Det[Join[b, {stb[i, d]}]]]];
			v[[d]] = Det[Join[b, {stb[d, d]}]];
			Return[v]]

hyperplane[pt_List] := Module[{n,q,p,dd=Length[pt], fin},
		If[dd == 0, Goto[fin]];
		For[i=1,i<= dd,i++, If[Length[pt[[i]]] != dd, 
		Print["Length of ", pt[[i]], " is unequal to ",dd ];Goto[fin]]];
		q = Table[pt[[i]] - pt[[dd]], {i,dd - 1}];
			n = cross[q];
	If[n === null[dd], 
	Print["The points ", pt, " are not in general position!"];Goto[fin]];
n = n/Sqrt[n.n];
p = n.pt[[1]];		
Return[{n, p}];
	Label[fin];
 Print["Invalid arguments!"];	
Return[]]

(* Inversions at hyperspheres *)

sphericalreflection[{Infinity}, a_List,r_:1] := a;
sphericalreflection[x_List, a_List, r_:1] := If[Not[x==a],
a + ((x - a) * r^2)/((x-a).(x-a)),{Infinity},
a + ((x - a) * r^2)/((x-a).(x-a))]

(*A fast Erhard Schmidt orthonormalization procedure; may be applied also for
functional spaces, the inner product of which must be positive semi-definite. *)


 Options[esorthonorm] = {innerprod ->Dot,normed -> True, print ->False, 
neglect-> -10}

esorthonorm[m_List, opts___] := 
  Module[
		{ma = m, mb = {}, k = Length[m], rd = Length[m], j = 0, i, ne, prod, pr, 
      bn, sm, aus, h, f},
		{prod, nor, pr, bn} = 
			{innerprod, normed, print, neglect} /. {opts} /. Options[esorthonorm];
    If[Head[m[[1]] ] == List, rd = Length[m[[1]]]];
	f[h_,bn_:-10]:=If[ bn == 0, h, Chop[h,10^bn]];
	sm[aus_] := MapAll[f, aus];
		Label[ne];
    If[Length[mb] >= Min[rd, k], Return[mb]];
		j = j + 1;
    If[j > k, Return[mb]];
			ma[[j]] = sm[ma[[j]] -
	Sum[mb[[i]]*prod[ma[[j]], mb[[i]]]/prod[mb[[i]], mb[[i]]], 
	{i, 1, Length[mb]}]]; 		
If[pr == True, Print[ma[[j]]]];
If[prod[ma[[j]], ma[[j]]] == 0, Goto[ne],
        If[nor == True, 
        AppendTo[mb, sm[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]]]],
				AppendTo[mb, ma[[j]]]],
				If[nor == True,
				AppendTo[mb, sm[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]]]], 			
AppendTo[mb, ma[[j]]]]];		
		Goto[ne]];

Options[norm] ={innerprod -> Dot};

norm[v_, opts___] := Module[{prod},
{prod} = {innerprod}/.{opts}/.Options[norm];
Sqrt[prod[v,v]]];

orthocomplement[v_List,opts___]:=Module[{ve= v,d, veo,rk,bas, vm,vef,veff,i},
If[Not[MatrixQ[ve]],Print["Incorrect argument!"];Return[]];
d= Length[ve[[1]]];
veo=esorthonorm[ve,opts];
rk = Length[veo];
bas = Table[stb[i,d],{i,d}];
vm  = Join[veo,bas];
vef = esorthonorm[vm, opts];
veff =Table[vef[[i]],{i,rk+1,d}];
Return[veff]] 


(*  Options[esorthonorm] = {innerprod ->Dot,normed -> True, print ->False, 
neglect-> -10}

esorthonorm[m_List, opts___] := 
  Module[
		{ma = m, mb = {}, k = Length[m], rd = Length[m], j = 0, i, ne, prod, pr, 
      bn, sm, aus, h, f},
		{prod, nor, pr, bn} = 
			{innerprod, normed, print, neglect} /. {opts} /. Options[esorthonorm];
    If[Head[m[[1]] ] == List, rd = Length[m[[1]]]];
	f[h_,bn_:-10]:=If[ bn == 0, h, Chop[h,10^bn]];
	sm[aus_] := MapAll[f, aus];
		Label[ne];
    If[Length[mb] >= Min[rd, k], Return[mb]];
		j = j + 1;
    If[j > k, Return[mb]];
			ma[[j]] = sm[Simplify[ma[[j]] -
	Sum[mb[[i]]*prod[ma[[j]], mb[[i]]]/prod[mb[[i]], mb[[i]]], 
	{i, 1, Length[mb]}],TimeConstraint\[Rule] Infinity]]; 		
If[pr == True, Print[ma[[j]]]];
If[prod[ma[[j]], ma[[j]]] == 0, Goto[ne],
        If[nor == True, 
        AppendTo[mb, sm[Simplify[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]],TimeConstraint \[Rule] Infinity]]],
				AppendTo[mb, Simplify[ma[[j]],TimeConstraint \[Rule] Infinity]]],
				If[nor == True,
				AppendTo[mb, sm[Simplify[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]],TimeConstraint \[Rule] Infinity]]], 			
AppendTo[mb, Simplify[ma[[j]],TimeConstraint \[Rule] Infinity]]]];		
		Goto[ne]];

Options[norm] ={innerprod -> Dot};

norm[v_, opts___] := Module[{prod},
{prod} = {innerprod}/.{opts}/.Options[norm];
Sqrt[prod[v,v]]];

orthocomplement[v_List,opts___]:=Module[{ve= v,d, veo,rk,bas, vm,vef,veff,i},
If[Not[MatrixQ[ve]],Print["Incorrect argument!"];Return[]];
d= Length[ve[[1]]];
veo=esorthonorm[ve,opts];
rk = Length[veo];
bas = Table[stb[i,d],{i,d}];
vm  = Join[veo,bas];
vef = esorthonorm[vm, opts];
veff =Table[vef[[i]],{i,rk+1,d}];
Return[veff]] *)


(* The following definitions work in case dim = 4 only! *)

subsphere[n_List, p_][v_,w_] := Module[{fr,pa,nv,pv,s,err},
	s=Simplify[Sqrt[n.n]];
	If[Length[n]!= 4 || s == 0, Goto[err]];	
	nv = n/s;
	pv = p/s;
	If[Abs[pv]> 1, Goto[err]];
	fr = orthocomplement[{nv}];
	pa := nv*pv +
				((fr[[1]]*Cos[v] + fr[[2]]*Sin[v])*Cos[w] +
							fr[[3]]*Sin[w])*Sqrt[1 - pv^2];
		Return[pa];
		Label[err];
		Print[ n, " or ", p,":  incorrect arguments!"]; Return[]]

tg4frame[s_,t_,u_] :={{Sin[s],-Cos[s],0,0},
			{-Cos[s]*Sin[t],-Sin[s]*Sin[t], Cos[t], 0},
		{-Cos[s] Cos[t] Sin[u],-Cos[t] Sin[s] Sin[u],-Sin[t] Sin[u],Cos[u]},{Cos[s] \
\
Cos[t] Cos[u],Cos[t] Cos[u] Sin[s],Cos[u] Sin[t],Sin[u]}}

subspheremf[s_,t_,u_, p_][v_,w_] := Module[{fr,pa},
	If[Abs[p]>1, Print["Parameter p = ",p," is too large!"];Return[]];
		fr = tg4frame[s,t,u];
			pa := fr[[4]] *p +
				((fr[[1]]*Cos[v] + fr[[2]]*Sin[v])*Cos[w] +
							fr[[3]]*Sin[w])*Sqrt[1 - p^2];
		Return[pa]]


(*Recursive Definition of the parameter representation of the unit n-sphere*)

(*This definition is independent of a globally defined dimension dim. *)

unit[s__] := Module[{un,i, di=Length[{s}],tt,ss},
If[(di==1)&&Head[s]===List, ss=s,ss={s}];di=Length[ss];
un[1,tt_]:={Cos[tt[[1]]], Sin[tt[[1]]]};
If[di==1,Return[un[1,ss]]];
For[i=2,i<di+1,i++,
un[i,ss]=AppendTo[un[i-1,ss],0]*Cos[ss[[i]]]+stb[1+i,i+1]*Sin[ss[[i]]]];
un[di,ss]];


(* Old, incorrect!
unit[n_][s_List] := Module[{un,i},
		un[1]={Cos[s[[1]]], Sin[s[[1]]]};
			For[i=2, i<= n, i++,
		un[i]=AppendTo[un[i-1],0]*Cos[s[[i]]]+stb[1+i,i+1]*Sin[s[[i]]]];
				Return[un[n]]]/;Length[s]>=n
				
unit[s_List] := unit[Length[s]][s];

unit[u_,v_] := unit[{u,v}]*)

(*Spheres in the Euclidean 3-space *)

sphere[m_List, r_][u_, v_] := {m[[1]] + r*Cos[u]*Cos[v], 
    m[[2]] + r*Cos[v]*Sin[u], m[[3]] + r*Sin[v]} /; Length[m] == 3
    
sphereplot3D[m_List, r_, u0_:0, u1_:2 Pi,v0_:-Pi/2,v1_:Pi/2, opt___] := 
  ParametricPlot3D[sphere[m,r][u,v],{u,u0,u1},{v,v0,v1},opt]

sph3D[u_,v_,w_] := unit[{u,v,w}];

(* Circle through 3 points *)

posvec3pts[p1_List,p2_List,p3_List] :=
	Module[{ a},
		a = Cross[p2-p1, p3-p1];
		If[a !=null[3],Return[Chop[a/Sqrt[a.a]]], Return[null[3]],
      Return[Chop[a/Sqrt[a.a]]]]];

center3pts[p1_List,p2_List,p3_List]:= 
  Module[{nn=Length[p1], q1= p1 - p3, q2 =p2 - p3,dn,dn1,dn2},
    dn = Det[{{q1.q1, q1.q2},{q1.q2,q2.q2}}];
    If[Chop[dn] == 0, Return[Table[Infinity,{nn}]]];
    dn1 = Det[{{q1.q1/2, q1.q2},{q2.q2/2,q2.q2}}];
    dn2 = Det[{{q1.q1, q1.q1/2},{q1.q2,q2.q2/2}}];
    Return[p3+(dn1*q1 + dn2*q2)/dn]];

radius3pts[p1_List,p2_List,p3_List] :=
	Module[{a},
		a = center3pts[p1,p2,p3];
		If[a.a == Infinity, Return[Infinity]];
		Return[ Sqrt[(p1-a).(p1 - a)]]];
		
(* Now we  give a definition of a 3D-circle through three points. The circle 
may eventually be a line (circle of infinite radius): *)

circle3pts[p1_List,p2_List,p3_List][t_] :=
	Module[{q2 , q3 , a, m,r,nn =Length[p1]},
If[p1.p1 == Infinity,Return[p2 + t*(p3-p2)]];
If[p2.p2 == Infinity,Return[p1 + t*(p3-p1)]];
If[p3.p3 == Infinity,Return[p1 + t*(p2-p1)]];
m = center3pts[p1,p2,p3];
If[m.m ==Infinity,If[Chop[p2 -p1]==null[nn], Return[p1 + t*(p3 - p1)], Return[p1 + t*(p2 - p1)]]];
r = Sqrt[(p1-m).(p1 - m)];	
	q2 = p2-p1; q3 = p3-p1;
a = Orthogonalize[{q2,q3}];
		q2  = Normalize[a[[1]]];
		q3 =  Normalize[a[[2]]];
		m +(q2 *Cos[t] + q3*Sin[t])*r];

(* Here is the corresponding plot command: *)

plotcircle3pts[p1_List,p2_List,p3_List,opt___]:= 
  ParametricPlot3D[Evaluate[circle3pts[p1,p2,p3][t]],{t, -Pi, Pi},opt];

(*Parametrisation of the circle space. The general circle *)
  
  unitvec[u_,v_][t_] := {-Cos[t] Sin[u] - Sin[t] Cos[u]Sin[v],
	Cos[t]  Cos[u] - Sin[t] Sin[u] Sin[v], Sin[t]Cos[v]};
  
  
circle3D[x_,y_,z_,u_,v_,r_][t_]:={x, y,z} + unitvec[u,v][t]*r;
  
  
gencircle[p_List,v_List][t_]:=Module[{a = v.v,b1, b2},
			If[Length[p]!=3||Length[v]!=3, Print["Incorrect arguments!!!"];Return[]];
	  	If[a== 0, Return[p]];
			If[v[[1]] == 0 && v[[2]] ==0, Return[p + {Cos[t], Sin[t], 0}*Sqrt[a]]];
			b1={-v[[2]], v[[1]], 0}/Sqrt[a - v[[3]]^2];
			b2 = Cross[v, b1];
			Return[p+b1*Sqrt[a]*Cos[t] +b2 Sin[t]]]
  
  
(*The corresponding Plot commands *)
  
  
plotcircle3D[x_,y_,z_,u_,v_,r_, opts___]:= 
ParametricPlot3D[Evaluate[circle3D[x,y,z,u,v,r][t]], {t, 0, 2 Pi}, opts];
  
  
plotgencircle[p_List,v_List, opts___]:= ParametricPlot3D[
Evaluate[gencircle[p,v][t]], {t, 0, 2 Pi}, opts];
  
 

(*Tubes*)
  
  
tube[x_, a_,b_,r_, opts___] := Module[{t,s, w,v, u},
			v[s_] := D[x[u],u]/.u -> s;
			w[s_]:=If[v[s].v[s]=== 0.,Return[v[s]], Return[ v[s]/Sqrt[v[s].v[s]]]];
			ParametricPlot3D[gencircle[x[s],r*w[s]][t], {s,a,b},{t,0,2 Pi},opts]];

End[]

Protect["`*"];

EndPackage[]


