(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[   2192603,      55297]
NotebookOptionsPosition[   1836594,      48222]
NotebookOutlinePosition[   1842179,      48394]
CellTagsIndexPosition[   1841389,      48363]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Pseudo - Riemannian Geometry
and
Tensor Analysis\
\>", "Title",
 CellChangeTimes->{{3.6229252843619504`*^9, 3.622925326825225*^9}, {
   3.6998806493858843`*^9, 3.699880669185174*^9}, {3.703487566678461*^9, 
   3.703487570822508*^9}, 3.7034876223097973`*^9},
 TextAlignment->Center,ExpressionUUID->"f245cfa7-6f2d-4053-bfb9-544cdb93418d"],

Cell["by", "Text",
 CellChangeTimes->{{3.665928568743827*^9, 3.665928569959427*^9}},
 TextAlignment->Center,ExpressionUUID->"e1d4ab09-06cd-4640-ad33-9103460785ed"],

Cell[TextData[StyleBox["Rolf Sulanke",
 FontSize->18]], "Text",
 CellChangeTimes->{{3.66592858694363*^9, 3.665928598191532*^9}, 
   3.665928759702148*^9},
 TextAlignment->Center,ExpressionUUID->"31f51928-44c4-4750-bbfa-bea043be87b9"],

Cell[TextData[{
 "Started February 1, 2015.\nVersion 1 finished May 20, 2016. \nVersion 2 \
started March 30, 2017\nVersion 2 finished July 10, 2017\n",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 9.0.1,, 10.4.1, 11.1.1\n"
}], "Text",
 CellChangeTimes->{{3.665929319045529*^9, 3.665929335645252*^9}, {
   3.6659297651800756`*^9, 3.665929841139741*^9}, {3.67048146761204*^9, 
   3.670481478723239*^9}, {3.670566754862083*^9, 3.6705667678057327`*^9}, {
   3.6705668032538977`*^9, 3.670566825933617*^9}, {3.672205080266404*^9, 
   3.672205082610276*^9}, 3.672831536729742*^9, {3.699880702976687*^9, 
   3.699880764080439*^9}, {3.700367004199651*^9, 3.700367018990803*^9}, 
   3.7018449600218897`*^9, {3.702806415506468*^9, 3.70280645843377*^9}, {
   3.7061004143859043`*^9, 3.706100533538941*^9}, {3.70610177901606*^9, 
   3.7061017937690268`*^9}, {3.708479353169241*^9, 3.70847936280059*^9}, {
   3.708575681122425*^9, 
   3.708575692050158*^9}},ExpressionUUID->"e906996d-cb76-4e39-98f7-\
24de483b8a52"],

Cell[CellGroupData[{

Cell["Preface", "Section",
 CellChangeTimes->{{3.670411603564979*^9, 
  3.670411609372921*^9}},ExpressionUUID->"7b87f206-691e-4a74-9ada-\
aa5d6daf4aee"],

Cell[TextData[{
 "In this notebook I develop and explain ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " tools for applications to Riemannian geometry and relativity theory. \
Together with the notebooks concerned with Euclidean geometry of curves and \
surfaces  in [",
 ButtonBox["EDG",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "] it may be used as an interactive introductory textbook of differential \
geometry. Clearly, these notebooks don\[CloseCurlyQuote]t cover the full \
content of an introductory course for this field. Therein the applications of \
algebra and calculus to differential geometry, the algorithmic methods are \
emphasized, and the rich graphical tools of Mathematica are applied  to \
illustrate the concepts in dimensions 2 and 3.  In order to learn \
differential geometry, the notebooks will give best profits when using them \
as an accompanying tool to an introductory course or to studying one of the \
numerous textbooks. Despite of this seemingly restrictive recommendation I \
tried to compose a coherent textbook containing the fundamental concepts of \
differential geometry understandable for  users with a basic knowledge  of \
calculus, linear and tensor algebra. The tools of tensor algebra as applied \
in this notebook can be found and downloaded ",
 Cell[BoxData[
  FormBox[
   ButtonBox["here",
    BaseStyle->"Hyperlink",
    ButtonData->{
      URL[
      "http://www-irm.mathematik.hu-berlin.de/~sulanke/geo_alg.html#tens"], 
      None},
    ButtonNote->
     "http://www-irm.mathematik.hu-berlin.de/~sulanke/geo_alg.html#tens"], 
   TraditionalForm]],ExpressionUUID->"0c35b284-2620-4a0f-8881-0d4a91d80120"],
 ", they are included in the packed file RG.zip. Furthermore I hope that the \
fine structured presentation of the content, the  tables of keywords and \
usages, and the open access to the program sources written as Mathematica \
packages will be useful for researchers and engineers who need to apply \
differential geometric methods in their work. This open access may also be \
useful for those striving to learn programming their own Mathematica \
functions or to test and, if possible, to improve the proposed programs."
}], "Text",
 CellChangeTimes->{{3.665929993739565*^9, 3.6659300710513983`*^9}, {
   3.6659301423151608`*^9, 3.665930145486039*^9}, 3.66598118118519*^9, {
   3.6659812221845303`*^9, 3.66598122941651*^9}, 3.665986953806396*^9, 
   3.665986990413457*^9, 3.670224214908765*^9, {3.670224809591052*^9, 
   3.67022491911003*^9}, {3.6702249953570766`*^9, 3.670225043948221*^9}, {
   3.6702250891529408`*^9, 3.6702251062283897`*^9}, 3.67032998823279*^9, {
   3.670330213325346*^9, 3.670330292381012*^9}, {3.6703303930692377`*^9, 
   3.6703304082684603`*^9}, {3.670330911704134*^9, 3.670330938479475*^9}, {
   3.670330994278347*^9, 3.670331070404955*^9}, {3.670331117021308*^9, 
   3.670331210612981*^9}, {3.670332121610248*^9, 3.670332384003188*^9}, {
   3.67033243645051*^9, 3.670332451554903*^9}, {3.670332508739933*^9, 
   3.6703325737382603`*^9}, 3.670333052525236*^9, {3.670333298405849*^9, 
   3.6703333445496407`*^9}, {3.6703334473736477`*^9, 
   3.6703335790034857`*^9}, {3.670333615380703*^9, 3.670333677372388*^9}, 
   3.670333779987893*^9, 3.670333896951133*^9, {3.670333995025611*^9, 
   3.670334169359264*^9}, 3.670334211681067*^9, {3.6703342614085197`*^9, 
   3.6703342921231117`*^9}, {3.670334346647807*^9, 3.670334348975939*^9}, {
   3.670334379295783*^9, 3.6703343816784773`*^9}, {3.670334419815073*^9, 
   3.670334479830607*^9}, {3.6703345255051517`*^9, 3.670334559814384*^9}, 
   3.670334619552124*^9, 3.6703346749735527`*^9, {3.670335002536025*^9, 
   3.67033518080341*^9}, {3.670335261985916*^9, 3.670335276585126*^9}, {
   3.670335550590892*^9, 3.670335552495284*^9}, {3.670391387450437*^9, 
   3.6703914039142733`*^9}, {3.670395423827512*^9, 3.670395456962393*^9}, {
   3.670395490490397*^9, 3.67039552200708*^9}, {3.67039555910045*^9, 
   3.670395560083434*^9}, {3.670395612049554*^9, 3.670395612053957*^9}, {
   3.670395750943211*^9, 3.670395750961844*^9}, 3.6703963456885443`*^9, 
   3.670481626488039*^9, {3.670562455214007*^9, 3.67056248121834*^9}, 
   3.670563667056739*^9, {3.671681841853077*^9, 3.671681845834322*^9}, {
   3.6716818759811983`*^9, 3.6716818779800367`*^9}, {3.671682068587874*^9, 
   3.671682142789447*^9}, {3.672123402987399*^9, 3.672123503250449*^9}, {
   3.672123568072937*^9, 3.672123576529632*^9}, {3.672123636891109*^9, 
   3.672123677095133*^9}, 3.672123768202544*^9, {3.672123883540991*^9, 
   3.672123922176161*^9}, {3.6721239985166893`*^9, 3.6721241598371983`*^9}, {
   3.672124374675055*^9, 3.672124377456171*^9}, {3.672199291076782*^9, 
   3.6721992947078342`*^9}, {3.672199369210458*^9, 3.672199417243135*^9}, 
   3.675916504898954*^9, {3.706156323624566*^9, 3.706156342762278*^9}, {
   3.706156956803687*^9, 3.70615708162745*^9}, {3.706160969820554*^9, 
   3.70616097220536*^9}, {3.7084051915861387`*^9, 
   3.7084051970150623`*^9}},ExpressionUUID->"092495f2-d158-4942-85fc-\
9faf682756c7"],

Cell["\<\
The contents of the notebook can be described shortly as follows. Section 1, \
based on concepts and functions contained in the notebook euklsfv3.nb of \
Euclidean surface geometry, treats the Riemannian spaces of constant \
curvature. They serve as standard examples in the following sections. \
\>", "Text",
 CellChangeTimes->{{3.6704187005677767`*^9, 3.670418715747196*^9}, {
   3.670418912837492*^9, 3.670418962893216*^9}, {3.6704189932688932`*^9, 
   3.67041900021279*^9}, {3.6704190440607767`*^9, 3.670419326818344*^9}, {
   3.670419498177203*^9, 3.670419597360024*^9}, {3.67041966360844*^9, 
   3.670419670358713*^9}, {3.670419715766534*^9, 3.670419795654499*^9}, {
   3.670419842390107*^9, 3.670420069668866*^9}, {3.6704201851881323`*^9, 
   3.67042023828316*^9}, {3.670420524810054*^9, 3.670420581161446*^9}, {
   3.67042065357577*^9, 3.670420724622703*^9}, {3.670420789111969*^9, 
   3.6704208022144938`*^9}, {3.670420845175065*^9, 3.670420879101924*^9}, 
   3.670420956444744*^9, {3.6704210163578377`*^9, 3.670421021061899*^9}, {
   3.670421139644806*^9, 3.6704211751087923`*^9}, {3.670421223475767*^9, 
   3.670421533139949*^9}, {3.670421606310923*^9, 3.670421619291157*^9}, {
   3.670421723186758*^9, 3.670422150592064*^9}, {3.670422282480775*^9, 
   3.670422312547407*^9}, {3.670422349681596*^9, 3.6704223882499723`*^9}, {
   3.670422429111699*^9, 3.670422477218891*^9}, 3.670481890918062*^9, {
   3.6704827518816957`*^9, 3.67048275448796*^9}, {3.670482797496139*^9, 
   3.6704830430993967`*^9}, {3.670483179490932*^9, 3.670483495129854*^9}, {
   3.670483542336948*^9, 3.670483646113809*^9}, {3.670483728293908*^9, 
   3.670483732815035*^9}, {3.670483765238934*^9, 3.670483819417521*^9}, {
   3.670483853280786*^9, 3.670483981838615*^9}, {3.6704840524224997`*^9, 
   3.670484094651664*^9}, {3.6705627176566563`*^9, 3.670562719468795*^9}, {
   3.670562810010645*^9, 3.6705628399356337`*^9}, 3.6705629148068037`*^9, {
   3.67056368088538*^9, 3.670563691238216*^9}, 3.67056373931173*^9, {
   3.670563781986814*^9, 3.6705638575758667`*^9}, 3.670563950572999*^9, 
   3.670648807537462*^9, {3.670648842948238*^9, 3.670648871696719*^9}, {
   3.670648981762845*^9, 3.6706490503789454`*^9}, {3.670649195447864*^9, 
   3.670649250538859*^9}, {3.671682474843362*^9, 3.671682475755872*^9}, {
   3.672124689010509*^9, 3.672124699389187*^9}, {3.672124885645996*^9, 
   3.672124893191663*^9}, {3.6721249587332478`*^9, 3.672125012324892*^9}, {
   3.672125104062958*^9, 3.672125108204659*^9}, {3.6721252054678707`*^9, 
   3.6721252180056133`*^9}, {3.672125301731147*^9, 3.6721253032035437`*^9}, {
   3.6721253732821836`*^9, 3.672125375394496*^9}, {3.672125414370039*^9, 
   3.672125416208016*^9}, 3.67219971113881*^9, {3.672199793956353*^9, 
   3.672199818871739*^9}, 3.672199872703514*^9, 3.6722019201391687`*^9, {
   3.7061572395507812`*^9, 
   3.706157240694487*^9}},ExpressionUUID->"ff8a0b50-daad-4a8f-b2f1-\
a95d8b6cf590"],

Cell["\<\
Section 2 starts with the definition of differentiable manifolds and its \
tangent, cotangent, and tensor bundles.  I introduce the calculus of \
differential forms and the corresponding Mathematica functions and operations \
which form  the exterior algebra including the exterior differential \
operator, a calculus widely used in geometry, physics and topology. This \
exterior calculus exists on each differentiable manifold  independently of \
any additional structure. Differential forms are antisymmetric covariant \
tensor fields, and only for such fields the exterior differential is defined. \
The only differential operator for vector fields existing in such generality \
is the commutator of two fields, treated in Subsection 2.4.9.  \
\>", "Text",
 CellChangeTimes->{{3.6704187005677767`*^9, 3.670418715747196*^9}, {
   3.670418912837492*^9, 3.670418962893216*^9}, {3.6704189932688932`*^9, 
   3.67041900021279*^9}, {3.6704190440607767`*^9, 3.670419326818344*^9}, {
   3.670419498177203*^9, 3.670419597360024*^9}, {3.67041966360844*^9, 
   3.670419670358713*^9}, {3.670419715766534*^9, 3.670419795654499*^9}, {
   3.670419842390107*^9, 3.670420069668866*^9}, {3.6704201851881323`*^9, 
   3.67042023828316*^9}, {3.670420524810054*^9, 3.670420581161446*^9}, {
   3.67042065357577*^9, 3.670420724622703*^9}, {3.670420789111969*^9, 
   3.6704208022144938`*^9}, {3.670420845175065*^9, 3.670420879101924*^9}, 
   3.670420956444744*^9, {3.6704210163578377`*^9, 3.670421021061899*^9}, {
   3.670421139644806*^9, 3.6704211751087923`*^9}, {3.670421223475767*^9, 
   3.670421533139949*^9}, {3.670421606310923*^9, 3.670421619291157*^9}, {
   3.670421723186758*^9, 3.670422150592064*^9}, {3.670422282480775*^9, 
   3.670422312547407*^9}, {3.670422349681596*^9, 3.6704223882499723`*^9}, {
   3.670422429111699*^9, 3.670422477218891*^9}, 3.670481890918062*^9, {
   3.6704827518816957`*^9, 3.67048275448796*^9}, {3.670482797496139*^9, 
   3.6704830430993967`*^9}, {3.670483179490932*^9, 3.670483495129854*^9}, {
   3.670483542336948*^9, 3.670483646113809*^9}, {3.670483728293908*^9, 
   3.670483732815035*^9}, {3.670483765238934*^9, 3.670483819417521*^9}, {
   3.670483853280786*^9, 3.670483981838615*^9}, {3.6704840524224997`*^9, 
   3.670484094651664*^9}, {3.6705627176566563`*^9, 3.670562719468795*^9}, {
   3.670562810010645*^9, 3.6705628399356337`*^9}, 3.6705629148068037`*^9, {
   3.67056368088538*^9, 3.670563691238216*^9}, 3.67056373931173*^9, {
   3.670563781986814*^9, 3.6705638575758667`*^9}, 3.670563950572999*^9, 
   3.670648807537462*^9, {3.670648842948238*^9, 3.670648871696719*^9}, {
   3.670648981762845*^9, 3.6706490503789454`*^9}, {3.670649195447864*^9, 
   3.670649250538859*^9}, {3.671682474843362*^9, 3.671682475755872*^9}, {
   3.672124689010509*^9, 3.672124699389187*^9}, {3.672124885645996*^9, 
   3.672124893191663*^9}, {3.6721249587332478`*^9, 3.672125012324892*^9}, {
   3.672125104062958*^9, 3.672125108204659*^9}, {3.6721252054678707`*^9, 
   3.6721252180056133`*^9}, {3.672125301731147*^9, 3.6721253032035437`*^9}, {
   3.6721253732821836`*^9, 3.672125375394496*^9}, {3.672125414370039*^9, 
   3.672125416208016*^9}, 3.67219971113881*^9, {3.672199793956353*^9, 
   3.672199818871739*^9}, 3.672199872703514*^9, 3.6722019201391687`*^9, {
   3.708405421326622*^9, 
   3.708405427488051*^9}},ExpressionUUID->"0d9221c7-1d1e-4201-8992-\
d8a9e21733ef"],

Cell["\<\
In Section 3 is shown that the partial derivatives of the coordinates of a \
vector field can not be considered as the coordinates of a tensor. To get a \
gradient tensor or a differential of arbitrary tensor fields an additional \
structure must be given on the manifold, an affine or, more precisely, a \
linear connection. The theory of linear connections is developed in this \
Section, gradients and  absolute differentials of tensor fields, torsion and \
curvature tensors of the linear connection are defined. The geometric \
background of this structure is the question whether there exist absolutely \
parallel tensor fields, whose gradient is a null tensor. Geodesics are \
defined as autoparallel curves. \
\>", "Text",
 CellChangeTimes->{{3.6704187005677767`*^9, 3.670418715747196*^9}, {
   3.670418912837492*^9, 3.670418962893216*^9}, {3.6704189932688932`*^9, 
   3.67041900021279*^9}, {3.6704190440607767`*^9, 3.670419326818344*^9}, {
   3.670419498177203*^9, 3.670419597360024*^9}, {3.67041966360844*^9, 
   3.670419670358713*^9}, {3.670419715766534*^9, 3.670419795654499*^9}, {
   3.670419842390107*^9, 3.670420069668866*^9}, {3.6704201851881323`*^9, 
   3.67042023828316*^9}, {3.670420524810054*^9, 3.670420581161446*^9}, {
   3.67042065357577*^9, 3.670420724622703*^9}, {3.670420789111969*^9, 
   3.6704208022144938`*^9}, {3.670420845175065*^9, 3.670420879101924*^9}, 
   3.670420956444744*^9, {3.6704210163578377`*^9, 3.670421021061899*^9}, {
   3.670421139644806*^9, 3.6704211751087923`*^9}, {3.670421223475767*^9, 
   3.670421533139949*^9}, {3.670421606310923*^9, 3.670421619291157*^9}, {
   3.670421723186758*^9, 3.670422150592064*^9}, {3.670422282480775*^9, 
   3.670422312547407*^9}, {3.670422349681596*^9, 3.6704223882499723`*^9}, {
   3.670422429111699*^9, 3.670422477218891*^9}, 3.670481890918062*^9, {
   3.6704827518816957`*^9, 3.67048275448796*^9}, {3.670482797496139*^9, 
   3.6704830430993967`*^9}, {3.670483179490932*^9, 3.670483495129854*^9}, {
   3.670483542336948*^9, 3.670483646113809*^9}, {3.670483728293908*^9, 
   3.670483732815035*^9}, {3.670483765238934*^9, 3.670483819417521*^9}, {
   3.670483853280786*^9, 3.670483981838615*^9}, {3.6704840524224997`*^9, 
   3.670484094651664*^9}, {3.6705627176566563`*^9, 3.670562719468795*^9}, {
   3.670562810010645*^9, 3.6705628399356337`*^9}, 3.6705629148068037`*^9, {
   3.67056368088538*^9, 3.670563691238216*^9}, 3.67056373931173*^9, {
   3.670563781986814*^9, 3.6705638575758667`*^9}, 3.670563950572999*^9, 
   3.670648807537462*^9, {3.670648842948238*^9, 3.670648871696719*^9}, {
   3.670648981762845*^9, 3.6706490503789454`*^9}, {3.670649195447864*^9, 
   3.670649250538859*^9}, {3.671682474843362*^9, 3.671682475755872*^9}, {
   3.672124689010509*^9, 3.672124699389187*^9}, {3.672124885645996*^9, 
   3.672124893191663*^9}, {3.6721249587332478`*^9, 3.672125012324892*^9}, {
   3.672125104062958*^9, 3.672125108204659*^9}, {3.6721252054678707`*^9, 
   3.6721252180056133`*^9}, {3.672125301731147*^9, 3.6721253032035437`*^9}, {
   3.6721253732821836`*^9, 3.672125375394496*^9}, {3.672125414370039*^9, 
   3.672125416208016*^9}, 3.67219971113881*^9, {3.672199793956353*^9, 
   3.672199818871739*^9}, 3.672199872703514*^9, {3.672199916565855*^9, 
   3.6721999168936462`*^9}, {3.672199970077284*^9, 
   3.672199976345995*^9}},ExpressionUUID->"2162a546-d69c-47fa-b1d9-\
6ef8657075bf"],

Cell["\<\
Section 4 is devoted to pseudo-Riemannian manifolds. The theorem of \
Levi-Civita is proved: On each such manifold there exists a uniquely defined \
linear connection with torsion tensor zero for which the absolute \
differential of the metric tensor is zero. The special properties of this \
Levi-Civita connection are deduced. Lowering and raising of indices yield \
canonical isomorphisms between tensor spaces of distinct covariance types of \
the same degree. Using the duality of vector and tensor spaces canonically \
defined scalar products in these spaces are introduced. The special \
properties of the Riemannian curvature tensor, the symmetries of its \
covariant form, the Bianchi identity are considered. The sectional curvature \
is introduced and the Gaussian curvature of surfaces is identified as its \
special case.\
\>", "Text",
 CellChangeTimes->{{3.6704187005677767`*^9, 3.670418715747196*^9}, {
   3.670418912837492*^9, 3.670418962893216*^9}, {3.6704189932688932`*^9, 
   3.67041900021279*^9}, {3.6704190440607767`*^9, 3.670419326818344*^9}, {
   3.670419498177203*^9, 3.670419597360024*^9}, {3.67041966360844*^9, 
   3.670419670358713*^9}, {3.670419715766534*^9, 3.670419795654499*^9}, {
   3.670419842390107*^9, 3.670420069668866*^9}, {3.6704201851881323`*^9, 
   3.67042023828316*^9}, {3.670420524810054*^9, 3.670420581161446*^9}, {
   3.67042065357577*^9, 3.670420724622703*^9}, {3.670420789111969*^9, 
   3.6704208022144938`*^9}, {3.670420845175065*^9, 3.670420879101924*^9}, 
   3.670420956444744*^9, {3.6704210163578377`*^9, 3.670421021061899*^9}, {
   3.670421139644806*^9, 3.6704211751087923`*^9}, {3.670421223475767*^9, 
   3.670421533139949*^9}, {3.670421606310923*^9, 3.670421619291157*^9}, {
   3.670421723186758*^9, 3.670422150592064*^9}, {3.670422282480775*^9, 
   3.670422312547407*^9}, {3.670422349681596*^9, 3.6704223882499723`*^9}, {
   3.670422429111699*^9, 3.670422477218891*^9}, 3.670481890918062*^9, {
   3.6704827518816957`*^9, 3.67048275448796*^9}, {3.670482797496139*^9, 
   3.6704830430993967`*^9}, {3.670483179490932*^9, 3.670483495129854*^9}, {
   3.670483542336948*^9, 3.670483646113809*^9}, {3.670483728293908*^9, 
   3.670483732815035*^9}, {3.670483765238934*^9, 3.670483819417521*^9}, {
   3.670483853280786*^9, 3.670483981838615*^9}, {3.6704840524224997`*^9, 
   3.670484094651664*^9}, {3.6705627176566563`*^9, 3.670562719468795*^9}, {
   3.670562810010645*^9, 3.6705628399356337`*^9}, 3.6705629148068037`*^9, {
   3.67056368088538*^9, 3.670563691238216*^9}, 3.67056373931173*^9, {
   3.670563781986814*^9, 3.6705638575758667`*^9}, 3.670563950572999*^9, 
   3.670648807537462*^9, {3.670648842948238*^9, 3.670648871696719*^9}, {
   3.670648981762845*^9, 3.6706490503789454`*^9}, {3.670649195447864*^9, 
   3.670649250538859*^9}, {3.671682474843362*^9, 3.671682475755872*^9}, {
   3.672124689010509*^9, 3.672124699389187*^9}, {3.672124885645996*^9, 
   3.672124893191663*^9}, {3.6721249587332478`*^9, 3.672125012324892*^9}, {
   3.672125104062958*^9, 3.672125108204659*^9}, {3.6721252054678707`*^9, 
   3.6721252180056133`*^9}, {3.672125301731147*^9, 3.6721253032035437`*^9}, {
   3.6721253732821836`*^9, 3.672125375394496*^9}, {3.672125414370039*^9, 
   3.672125416208016*^9}, 3.67219971113881*^9, {3.672199793956353*^9, 
   3.672199818871739*^9}, 3.672199872703514*^9, {3.672199916565855*^9, 
   3.6721999168936462`*^9}, 3.672199970077284*^9, 3.6722000836347113`*^9, 
   3.675916992178858*^9, 
   3.706073264671692*^9},ExpressionUUID->"f1db59fd-78da-4e26-84f5-\
100598a34761"],

Cell["\<\
The second version of the notebook is - in difference to the first version - \
now separated from the elementary Euclidean differential geometry; it can be \
read and applied independently of these elementary part. All necessary \
packages and notebooks are collected in the packed file RG.zip.  I included \
also the notebook vectensalgv3.nb containing an implementation of tensor \
algebra into Mathematica as it is used in the current notebook. Since many \
definitions in the new versions are changed or are new, the Mathematica \
packages of different versions should not be mixed; use exactly those \
contained in RG.zip. \
\>", "Text",
 CellChangeTimes->{{3.706158125817275*^9, 3.70615826544083*^9}, {
  3.706158335169188*^9, 3.706158394232314*^9}, {3.706158431080501*^9, 
  3.706158535439516*^9}, {3.706158585607563*^9, 3.7061586010075417`*^9}, {
  3.706158695127016*^9, 3.706158780070784*^9}, {3.706158820045836*^9, 
  3.70615883863692*^9}, {3.706158937308599*^9, 3.7061591038683443`*^9}, {
  3.706159136388205*^9, 3.706159185388463*^9}, {3.7061592503057632`*^9, 
  3.706159258507937*^9}, {3.70615935854493*^9, 3.706159360971457*^9}, {
  3.706161013932708*^9, 3.706161024694909*^9}, {3.7084056089198313`*^9, 
  3.708405609605205*^9}, {3.708405668748898*^9, 
  3.708405670516699*^9}},ExpressionUUID->"1a0e824a-c136-42b9-94ec-\
3384ef6c372d"],

Cell["\<\
I conclude this preface expressing my sincere thanks to the Humboldt \
University in Berlin for the continuous support giving me access to Wolfram's \
Mathematica and the possibility to continue my work also in my  retired \
position.\
\>", "Text",
 CellChangeTimes->{{3.6704187005677767`*^9, 3.670418715747196*^9}, {
   3.670418912837492*^9, 3.670418962893216*^9}, {3.6704189932688932`*^9, 
   3.67041900021279*^9}, {3.6704190440607767`*^9, 3.670419326818344*^9}, {
   3.670419498177203*^9, 3.670419597360024*^9}, {3.67041966360844*^9, 
   3.670419670358713*^9}, {3.670419715766534*^9, 3.670419795654499*^9}, {
   3.670419842390107*^9, 3.670420069668866*^9}, {3.6704201851881323`*^9, 
   3.67042023828316*^9}, {3.670420524810054*^9, 3.670420581161446*^9}, {
   3.67042065357577*^9, 3.670420724622703*^9}, {3.670420789111969*^9, 
   3.6704208022144938`*^9}, {3.670420845175065*^9, 3.670420879101924*^9}, 
   3.670420956444744*^9, {3.6704210163578377`*^9, 3.670421021061899*^9}, {
   3.670421139644806*^9, 3.6704211751087923`*^9}, {3.670421223475767*^9, 
   3.670421533139949*^9}, {3.670421606310923*^9, 3.670421619291157*^9}, {
   3.670421723186758*^9, 3.670422150592064*^9}, {3.670422282480775*^9, 
   3.670422312547407*^9}, {3.670422349681596*^9, 3.6704223882499723`*^9}, {
   3.670422429111699*^9, 3.670422477218891*^9}, 3.670481890918062*^9, {
   3.6704827518816957`*^9, 3.67048275448796*^9}, {3.670482797496139*^9, 
   3.6704830430993967`*^9}, {3.670483179490932*^9, 3.670483495129854*^9}, {
   3.670483542336948*^9, 3.670483646113809*^9}, {3.670483728293908*^9, 
   3.670483732815035*^9}, {3.670483765238934*^9, 3.670483819417521*^9}, {
   3.670483853280786*^9, 3.670483981838615*^9}, {3.6704840524224997`*^9, 
   3.670484094651664*^9}, {3.6705627176566563`*^9, 3.670562719468795*^9}, {
   3.670562810010645*^9, 3.6705628399356337`*^9}, 3.6705629148068037`*^9, {
   3.67056368088538*^9, 3.670563691238216*^9}, 3.67056373931173*^9, {
   3.670563781986814*^9, 3.6705638575758667`*^9}, 3.670563950572999*^9, 
   3.670648807537462*^9, {3.670648842948238*^9, 3.670648871696719*^9}, {
   3.670648981762845*^9, 3.6706490503789454`*^9}, {3.670649195447864*^9, 
   3.670649250538859*^9}, {3.671682474843362*^9, 3.671682475755872*^9}, {
   3.672124689010509*^9, 3.672124699389187*^9}, {3.672124885645996*^9, 
   3.672124893191663*^9}, {3.6721249587332478`*^9, 3.672125012324892*^9}, {
   3.672125104062958*^9, 3.672125108204659*^9}, {3.6721252054678707`*^9, 
   3.6721252180056133`*^9}, {3.672125301731147*^9, 3.6721253032035437`*^9}, {
   3.6721253732821836`*^9, 3.672125375394496*^9}, {3.672125414370039*^9, 
   3.672125416208016*^9}, 3.67219971113881*^9, {3.672199793956353*^9, 
   3.672199818871739*^9}, 3.672199872703514*^9, {3.672199916565855*^9, 
   3.6721999168936462`*^9}, 3.672199970077284*^9, {3.6722000836347113`*^9, 
   3.67220009147127*^9}},ExpressionUUID->"b3f80b45-87f1-446d-a368-\
4f594eec3dc5"],

Cell[CellGroupData[{

Cell["Keywords", "Subsection",
 CellChangeTimes->{{3.672201017982773*^9, 3.672201023750716*^9}, {
  3.672201229260995*^9, 
  3.672201233604622*^9}},ExpressionUUID->"78c89644-4717-437a-98bb-\
478565d2b179"],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {"\<\"absolute differential\"\>", "\<\"index raising\"\>"},
     {"\<\"absolute parallel\"\>", "\<\"inner geometry \"\>"},
     {"\<\"affine connection\"\>", "\<\"inverse metric\"\>"},
     {"\<\"algebra Lie\"\>", "\<\"inverse metric tensor\"\>"},
     {"\<\"algebra of functions\"\>", "\<\"isotropic\"\>"},
     {"\<\"alternated absolute differential\"\>", "\<\"Jacobi\"\>"},
     {"\<\"antisymmetric tensors\"\>", "\<\"Jacobian\"\>"},
     {"\<\"arclength\"\>", "\<\"Kronecker symbol\"\>"},
     {"\<\"atlas\"\>", "\<\"Levi-Civita connection\"\>"},
     {"\<\"auto-parallel curves\"\>", "\<\"linear connection\"\>"},
     {"\<\"Bianchi's identity\"\>", "\<\"local coordinates\"\>"},
     {"\<\"bivector\"\>", "\<\"Lorentzian space\"\>"},
     {"\<\"category\"\>", "\<\"manifolds\"\>"},
     {"\<\"chart\"\>", "\<\"metric\"\>"},
     {"\<\"Christoffel symbols\"\>", "\<\"metric tensor\"\>"},
     {"\<\"class Ck\"\>", "\<\"multi indices\"\>"},
     {"\<\"commutator\"\>", "\<\"nabla\"\>"},
     {"\<\"complete atlas\"\>", "\<\"n-sphere\"\>"},
     {"\<\"commutator\"\>", "\<\"orientable manifold\"\>"},
     {"\<\"conformal\"\>", "\<\"orientation\"\>"},
     {"\<\"connection forms\"\>", "\<\"oriented atlas\"\>"},
     {"\<\"contraction\"\>", "\<\"parallel displacement\"\>"},
     {"\<\"contravariant\"\>", "\<\"p-form\"\>"},
     {"\<\"coordinate transformation\"\>", "\<\"product rule\"\>"},
     {"\<\"covariant curvature tensor\"\>", "\<\"pseudo-Euclidean space\"\>"},
     {"\<\"covector fields\"\>", "\<\"pseudo-Riemannian space\"\>"},
     {"\<\"curvature tensor\"\>", "\<\"regular\"\>"},
     {"\<\"curve\"\>", "\<\"Ricci tensor\"\>"},
     {"\<\"diffeomorphism\"\>", "\<\"Riemannian space\"\>"},
     {"\<\"differential\"\>", "\<\"Riemannian spaces of constant \
curvature\"\>"},
     {"\<\"differential form\"\>", "\<\"sectional curvature\"\>"},
     {"\<\"dimension\"\>", "\<\"singular\"\>"},
     {"\<\"directional derivative\"\>", "\<\"smooth\"\>"},
     {"\<\"duality\"\>", "\<\"spacelike\"\>"},
     {"\<\"equi\"\>", "\<\"sphere\"\>"},
     {"\<\"equivalent tensors\"\>", "\<\"sphere of imaginary radius\"\>"},
     {"\<\"Euclidean metric\"\>", "\<\"stereographic projection\"\>"},
     {"\<\"exterior algebra\"\>", "\<\"strict coordinates\"\>"},
     {"\<\"exterior differential\"\>", "\<\"submanifold\"\>"},
     {"\<\"exterior product\"\>", "\<\"sum convention\"\>"},
     {"\<\"fiber\"\>", "\<\"surface\"\>"},
     {"\<\"fiber type\"\>", "\<\"symmetric tensors\"\>"},
     {"\<\"flat linear connection\"\>", "\<\"tangent basis \"\>"},
     {"\<\"\[CapitalGamma]-symbols\"\>", "\<\"tangent bundle\"\>"},
     {"\<\"Gaussian curvature\"\>", "\<\"tangent space\"\>"},
     {"\<\"Gradient\"\>", "\<\"tangent vector\"\>"},
     {"\<\"gradient tensor\"\>", "\<\"tensor fields\"\>"},
     {"\<\"Hausdorff space\"\>", "\<\"tensor metric\"\>"},
     {"\<\"homeomorphism\"\>", "\<\"tensor product\"\>"},
     {"\<\"hyperbolic space\"\>", "\<\"timelike\"\>"},
     {"\<\"hyperboloid\"\>", "\<\"torsion tensor\"\>"},
     {"\<\"hypersurface\"\>", "\<\"traditional notation\"\>"},
     {"\<\"immersiom\"\>", "\<\"vector fields\"\>"},
     {"\<\"index\"\>", "\<\"wedge\"\>"},
     {"\<\"index lowering\"\>", "\<\"wedge product\"\>"}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{{3.706164950236555*^9, 3.7061649701083307`*^9}, 
   3.706165027890562*^9},ExpressionUUID->"d189f472-d1e8-42f4-a379-\
e96d4931b127"],

Cell[CellGroupData[{

Cell["keylist", "Subsubsection",
 CellChangeTimes->{{3.706161765874268*^9, 
  3.7061617707137747`*^9}},ExpressionUUID->"3a867856-9761-40de-b5a9-\
9e92f03e9daf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"keylist", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "\"\<absolute differential\>\"", ",", "\"\<absolute parallel\>\"", ",", 
     "\"\<affine connection\>\"", ",", "\"\<algebra Lie\>\"", ",", 
     "\"\<algebra of functions\>\"", ",", 
     "\"\<alternated absolute differential\>\"", ",", 
     "\"\<antisymmetric tensors\>\"", ",", "\"\<arclength\>\"", ",", 
     "\"\<atlas\>\"", ",", "\"\<auto-parallel curves\>\"", ",", 
     "\"\<Bianchi's identity\>\"", ",", "\"\<bivector\>\"", ",", 
     "\"\<category\>\"", ",", "\"\<chart\>\"", ",", 
     "\"\<Christoffel symbols\>\"", ",", "\"\<class Ck\>\"", ",", 
     "\"\<commutator\>\"", ",", "\"\<complete atlas\>\"", ",", 
     "\"\<commutator\>\"", ",", "\"\<conformal\>\"", ",", 
     "\"\<connection forms\>\"", ",", "\"\<contraction\>\"", ",", " ", 
     "\"\<contravariant\>\"", ",", " ", "\"\<coordinate transformation\>\"", 
     ",", "\"\<covariant curvature tensor\>\"", ",", 
     "\"\<covector fields\>\"", ",", "\"\<curvature tensor\>\"", ",", 
     "\"\<curve\>\"", ",", "\"\<diffeomorphism\>\"", ",", 
     "\"\<differential\>\"", ",", "\"\<differential form\>\"", ",", 
     "\"\<dimension\>\"", ",", "\"\<directional derivative\>\"", ",", 
     "\"\<duality\>\"", ",", "\"\<equi\>\"", ",", 
     "\"\<equivalent tensors\>\"", ",", "\"\<Euclidean metric\>\"", ",", 
     "\"\<exterior algebra\>\"", ",", "\"\<exterior differential\>\"", ",", 
     "\"\<exterior product\>\"", ",", "\"\<fiber\>\"", ",", 
     "\"\<fiber type\>\"", ",", "\"\<flat linear connection\>\"", ",", 
     "\"\<\[CapitalGamma]-symbols\>\"", ",", "\"\<Gaussian curvature\>\"", 
     ",", "\"\<Gradient\>\"", ",", "\"\<gradient tensor\>\"", ",", 
     "\"\<Hausdorff space\>\"", ",", "\"\<homeomorphism\>\"", ",", 
     "\"\<hyperbolic space\>\"", ",", "\"\<hyperboloid\>\"", ",", 
     "\"\<hypersurface\>\"", ",", "\"\<immersiom\>\"", ",", "\"\<index\>\"", 
     ",", "\"\<index lowering\>\"", ",", "\"\<index raising\>\"", ",", 
     "\"\<inner geometry \>\"", ",", "\"\<inverse metric\>\"", ",", 
     "\"\<inverse metric tensor\>\"", ",", "\"\<isotropic\>\"", ",", " ", 
     "\"\<Jacobi\>\"", ",", "\"\<Jacobian\>\"", ",", 
     "\"\<Kronecker symbol\>\"", ",", "\"\<Levi-Civita connection\>\"", ",", 
     "\"\<linear connection\>\"", ",", "\"\<local coordinates\>\"", ",", 
     "\"\<Lorentzian space\>\"", ",", "\"\<manifolds\>\"", ",", 
     "\"\<metric\>\"", ",", "\"\<metric tensor\>\"", ",", 
     "\"\<multi indices\>\"", ",", "\"\<nabla\>\"", ",", "\"\<n-sphere\>\"", 
     ",", "\"\<orientable manifold\>\"", ",", "\"\<orientation\>\"", ",", 
     "\"\<oriented atlas\>\"", ",", "\"\<parallel displacement\>\"", ",", 
     "\"\<p-form\>\"", ",", "\"\<product rule\>\"", ",", 
     "\"\<pseudo-Euclidean space\>\"", ",", "\"\<pseudo-Riemannian space\>\"",
      ",", "\"\<regular\>\"", ",", "\"\<Ricci tensor\>\"", ",", 
     "\"\<Riemannian space\>\"", ",", 
     "\"\<Riemannian spaces of constant curvature\>\"", ",", 
     "\"\<sectional curvature\>\"", ",", "\"\<singular\>\"", ",", 
     "\"\<smooth\>\"", ",", "\"\<spacelike\>\"", ",", "\"\<sphere\>\"", ",", 
     "\"\<sphere of imaginary radius\>\"", ",", 
     "\"\<stereographic projection\>\"", ",", "\"\<strict coordinates\>\"", 
     ",", "\"\<submanifold\>\"", ",", "\"\<sum convention\>\"", ",", 
     "\"\<surface\>\"", ",", "\"\<symmetric tensors\>\"", ",", 
     "\"\<tangent basis \>\"", ",", "\"\<tangent bundle\>\"", ",", 
     "\"\<tangent space\>\"", ",", "\"\<tangent vector\>\"", ",", 
     "\"\<tensor fields\>\"", ",", "\"\<tensor metric\>\"", ",", 
     "\"\<tensor product\>\"", ",", "\"\<timelike\>\"", ",", 
     "\"\<torsion tensor\>\"", ",", "\"\<traditional notation\>\"", ",", 
     "\"\<vector fields\>\"", ",", "\"\<wedge\>\"", ",", 
     "\"\<wedge product\>\""}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.669705487058323*^9, 3.669705539759247*^9}, 
   3.6697055740711*^9, 3.669705640973783*^9, {3.669705683925351*^9, 
   3.66970570228459*^9}, 3.669720498576982*^9, 3.6700773674217443`*^9, {
   3.6701232835078917`*^9, 3.670123346468383*^9}, {3.670570439709383*^9, 
   3.670570485855321*^9}, 3.7060732647066193`*^9, {3.706159471640396*^9, 
   3.706159499988471*^9}, {3.706159613036303*^9, 3.706159628579515*^9}, {
   3.706159813780374*^9, 3.7061598246824408`*^9}, {3.706159883367529*^9, 
   3.7061599079063883`*^9}, {3.706159961408043*^9, 3.706159973385577*^9}, {
   3.706160078769188*^9, 
   3.7061601033288717`*^9}},ExpressionUUID->"1479cf29-903f-43d7-8e93-\
08f5a499067f"],

Cell[BoxData[
 RowBox[{"keylist", "//", "TableForm"}]], "Input",
 CellChangeTimes->{{3.665983044459744*^9, 3.665983055714059*^9}, {
   3.665991473308419*^9, 3.665991480172097*^9}, 3.669705962617461*^9, {
   3.6701231914139833`*^9, 3.670123200948642*^9}, {3.670123415620709*^9, 
   3.670123424266881*^9}, {3.672833878417891*^9, 
   3.672833883481596*^9}},ExpressionUUID->"f32f98eb-b8f9-47b2-9799-\
4694e9d3f9ed"],

Cell[BoxData[
 RowBox[{"Multicolumn", "[", 
  RowBox[{"keylist", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.7061649244585953`*^9, 
  3.706164943587668*^9}},ExpressionUUID->"7e9e469d-a1f4-449d-9150-\
73893e5032dd"]
}, Closed]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Copyright", "Section",
 CellChangeTimes->{{3.541686221104368*^9, 3.541686224511538*^9}, {
   3.541686639857975*^9, 3.541686645161036*^9}, 3.541836408310912*^9, 
   3.541836718110009*^9, 3.705734046757715*^9, 
   3.7060997931120653`*^9},ExpressionUUID->"9c233649-9d70-46cd-9739-\
be724a14842a"],

Cell["\<\
This notebook and the accompanying packages are  public. Authors who intend \
to publish a changed or completed version of them should do it under their \
own names with the condition that they cite the original notebook with the \
Internet address or other source where they got it. I am not responsible for \
errors or damages originated by the use of the procedures contained in my \
notebooks or packages; everybody who applies them should test carefully \
whether they are appropriate for his purposes.\
\>", "Text",
 CellChangeTimes->{{3.470471120664312*^9, 3.470471146997974*^9}, {
   3.498225321006962*^9, 3.498225323459318*^9}, 3.541836408310948*^9, 
   3.541836718110042*^9, 
   3.5994668492244267`*^9},ExpressionUUID->"ee24a9fa-e3df-454e-a893-\
eeec2a0e794f"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section",
 CellChangeTimes->{{3.670564116004434*^9, 
  3.670564122204315*^9}},ExpressionUUID->"640d0647-09af-4bda-b542-\
9ac0559f2824"],

Cell["\<\
1. Before starting to work with the notebook first time read the Section \
Copyright and adapt  the Subsection \[OpenCurlyDoubleQuote]Prepare \
Initialization\[CloseCurlyDoubleQuote] to your computer! \
\>", "Text",
 CellChangeTimes->{{3.665929993739565*^9, 3.6659300710513983`*^9}, {
   3.6659301423151608`*^9, 3.665930145486039*^9}, 3.66598118118519*^9, {
   3.6659812221845303`*^9, 3.66598122941651*^9}, 3.665986953806396*^9, 
   3.665986990413457*^9, 3.670224214908765*^9, {3.670224809591052*^9, 
   3.67022491911003*^9}, {3.6702249953570766`*^9, 3.670225043948221*^9}, {
   3.6702250891529408`*^9, 3.6702251062283897`*^9}, 3.67032998823279*^9, {
   3.670330213325346*^9, 3.670330292381012*^9}, {3.6703303930692377`*^9, 
   3.6703304082684603`*^9}, {3.670330911704134*^9, 3.670330938479475*^9}, {
   3.670330994278347*^9, 3.670331070404955*^9}, {3.670331117021308*^9, 
   3.670331210612981*^9}, {3.670332121610248*^9, 3.670332384003188*^9}, {
   3.67033243645051*^9, 3.670332451554903*^9}, {3.670332508739933*^9, 
   3.6703325737382603`*^9}, 3.670333052525236*^9, {3.670333298405849*^9, 
   3.6703333445496407`*^9}, {3.6703334473736477`*^9, 
   3.6703335790034857`*^9}, {3.670333615380703*^9, 3.670333677372388*^9}, 
   3.670333779987893*^9, 3.670333896951133*^9, {3.670333995025611*^9, 
   3.670334169359264*^9}, 3.670334211681067*^9, {3.6703342614085197`*^9, 
   3.6703342921231117`*^9}, {3.670334346647807*^9, 3.670334348975939*^9}, {
   3.670334379295783*^9, 3.6703343816784773`*^9}, {3.670334419815073*^9, 
   3.670334479830607*^9}, {3.6703345255051517`*^9, 3.670334559814384*^9}, 
   3.670334619552124*^9, 3.6703346749735527`*^9, {3.670335002536025*^9, 
   3.67033518080341*^9}, {3.670335261985916*^9, 3.670335276585126*^9}, {
   3.670335550590892*^9, 3.670335552495284*^9}, {3.670391387450437*^9, 
   3.6703914039142733`*^9}, {3.670395423827512*^9, 3.670395456962393*^9}, {
   3.670395490490397*^9, 3.67039552200708*^9}, {3.67039555910045*^9, 
   3.670395560083434*^9}, {3.670395612049554*^9, 3.670395612053957*^9}, {
   3.670395750943211*^9, 3.670395750961844*^9}, {3.6703963456885443`*^9, 
   3.670396383195747*^9}, {3.670396425579256*^9, 3.670396450227216*^9}, {
   3.670396681225173*^9, 3.670396698960898*^9}, {3.670401899888967*^9, 
   3.670401902345056*^9}, {3.6704108121799507`*^9, 3.6704108458757687`*^9}, {
   3.670412048577314*^9, 3.6704120623370323`*^9}, 3.67056424911559*^9, {
   3.67056431863511*^9, 3.67056432557092*^9}, {3.670564367451038*^9, 
   3.670564401346396*^9}},ExpressionUUID->"28434613-4ecc-47a5-bc48-\
13cd99cb42a0"],

Cell["\<\
2. To get information about a concept listed in the keylist (see Subsection \
Keywords)  mark the item and use Edit/Find of the menu to find all places in \
which the marked text appears.\
\>", "Text",
 CellChangeTimes->{{3.6703967158244534`*^9, 3.670396720184587*^9}, {
   3.670397120742556*^9, 3.670397197758564*^9}, {3.6704016585632143`*^9, 
   3.670401803449758*^9}, {3.6704018393936167`*^9, 3.670401888545534*^9}, {
   3.670402060590785*^9, 3.670402075023139*^9}, 3.6704021132472963`*^9, 
   3.670410872635791*^9},ExpressionUUID->"8e157162-6c03-4082-8f18-\
ee903aa16911"],

Cell["\<\
3. Tables of all new Mathematica functions introduced in the packages and in \
the Global Context are contained in Section \[OpenCurlyDoubleQuote]Imported \
Functions and their Usages\[CloseCurlyDoubleQuote].\
\>", "Text",
 CellChangeTimes->{{3.670410961346451*^9, 3.670411063961438*^9}, {
  3.670411229368278*^9, 3.670411246632094*^9}, {3.6704113150235357`*^9, 
  3.670411417174666*^9}, {3.701497568987829*^9, 
  3.701497571899308*^9}},ExpressionUUID->"d94f51c1-2e28-4a81-b237-\
469bf98ad2f7"],

Cell[CellGroupData[{

Cell["Prepare Initialization", "Subsection",
 CellChangeTimes->{{3.572085090826617*^9, 3.572085097846629*^9}, {
   3.614675039233622*^9, 3.614675041977779*^9}, 3.61468526962821*^9, {
   3.6704107887877817`*^9, 
   3.670410792859742*^9}},ExpressionUUID->"e9d3cdde-0e3a-44c2-b0ea-\
2c47039ded1c"],

Cell["\<\
Before starting the first session adapt this Section to your computer and \
operating system. Insert the address of your working directory, e.g. for \
Windows:\
\>", "Text",
 CellChangeTimes->{{3.599463466951742*^9, 3.599463520087782*^9}, {
   3.599463875888132*^9, 3.5994638948642178`*^9}, 3.59946686361625*^9, {
   3.624770686442224*^9, 3.6247707098734655`*^9}, {3.665930779189818*^9, 
   3.665930828277932*^9}, {3.6701236681134377`*^9, 
   3.67012366979361*^9}},ExpressionUUID->"68fe155c-3183-41e8-8030-\
73fe8a4a50b7"],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", 
  "\"\<H:\\\\mymath\\\\diffgeo\\\\DiffGeoCourse\>\"", "]"}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.571294164375502*^9, 3.5712942516264553`*^9}, 
   3.5919452705433164`*^9, 3.5920226711016054`*^9, 3.6089604686226373`*^9, 
   3.6089669513028173`*^9, 3.609849899827975*^9, 3.6114596276456995`*^9, 
   3.6225992444605618`*^9, {3.622883761555369*^9, 3.622883763286972*^9}, 
   3.623755959921055*^9, 3.6277079613261795`*^9, 3.627712256148449*^9, {
   3.6510374579360237`*^9, 3.6510374612485294`*^9}, 3.6663265362672873`*^9, {
   3.673846667005191*^9, 
   3.673846677173512*^9}},ExpressionUUID->"55b3da63-534d-49b6-80aa-\
11113072b5a9"],

Cell["For Linux: ", "Text",
 CellChangeTimes->{{3.6247705161991253`*^9, 3.624770522922737*^9}, {
  3.669982216292211*^9, 
  3.6699822381318417`*^9}},ExpressionUUID->"69f184f2-515b-4538-af1c-\
acf2da5f7769"],

Cell[BoxData[
 RowBox[{"Directory", "[", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.703735741862505*^9, 
  3.703735751433772*^9}},ExpressionUUID->"512b4d3d-d104-42e7-9d6f-\
8e0cfc6a4e41"],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", 
  "\"\</home/rolf/mymath/diffgeo/Riemann.orig/Riemann\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.571294164375502*^9, 3.5712942516264553`*^9}, 
   3.5919452705433164`*^9, 3.5920226711016054`*^9, 3.6089604686226373`*^9, 
   3.6089669513028173`*^9, 3.609849899827975*^9, 3.6114596276456995`*^9, 
   3.6225992444605618`*^9, {3.622883761555369*^9, 3.622883763286972*^9}, 
   3.623755959921055*^9, {3.6247705367911615`*^9, 3.624770550487985*^9}, {
   3.624770639532942*^9, 3.6247706546493683`*^9}, {3.651037500783783*^9, 
   3.6510375039556646`*^9}, 3.6738467056773977`*^9, {3.6998837265186996`*^9, 
   3.699883730239897*^9}, {3.703735810083372*^9, 3.7037358242897663`*^9}, {
   3.708406160908627*^9, 
   3.708406169380884*^9}},ExpressionUUID->"73b46175-db8e-432d-bdb0-\
3b16f2ad23df"],

Cell["\<\
Regard  eventually deviating notations depending on your operating system! \
Ensure that the corresponding cell is an initialization cell, and the other \
cell doesn\[CloseCurlyQuote]t have this property (Menu Cell/ Cell Properties).\
\>", "Text",
 CellChangeTimes->{{3.599463651304287*^9, 3.599463772472217*^9}, 
   3.599466052630864*^9, {3.608967233202941*^9, 3.6089672358820944`*^9}, 
   3.6247707634127593`*^9, {3.6247709006692357`*^9, 3.6247710039882174`*^9}, {
   3.6247711621314993`*^9, 3.624771186016141*^9}, 3.6268556704224873`*^9, {
   3.6271881066198015`*^9, 
   3.627188135901053*^9}},ExpressionUUID->"6d86f52a-8ee5-4b72-af5f-\
c36d06953102"],

Cell[BoxData["$Path"], "Input",ExpressionUUID->"c5fa054e-1c0e-4d80-b7e4-2f7455babc89"],

Cell[TextData[{
 " Copy all the needed ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " packages into your working directory, or a directory of $Path. \n\
Obligatory are the packages\nriemannv2.m, tensalgv3.m, euvecv2.m, and \
neuvecv2.m and the subnotebook index.nb. Alfred Gray\[CloseCurlyQuote]s \
catalog SURFS.m, and his catalog CURVES.m are very useful and may be imported \
if needed.. "
}], "Text",
 CellChangeTimes->{{3.5994639548406477`*^9, 3.599463992919826*^9}, {
   3.5994640376963873`*^9, 3.599464151447893*^9}, {3.5994642220319304`*^9, 
   3.5994642628482647`*^9}, {3.599464422128375*^9, 3.599464461632635*^9}, 
   3.5994644995758047`*^9, {3.599465959441534*^9, 3.5994660385980616`*^9}, 
   3.5994664734869356`*^9, {3.59948869731579*^9, 3.599488700560596*^9}, {
   3.6089672770904512`*^9, 3.608967282066736*^9}, {3.6089686143229365`*^9, 
   3.6089686872831097`*^9}, {3.6147521287995605`*^9, 3.614752179214444*^9}, {
   3.627188375438616*^9, 3.627188376561818*^9}, {3.665930904037223*^9, 
   3.665931084819688*^9}, {3.665931230969969*^9, 3.6659312349147*^9}, 
   3.700025192036145*^9, {3.7014976681629467`*^9, 3.701497677059264*^9}, {
   3.706100069342566*^9, 3.7061000761028137`*^9}, {3.706161224695128*^9, 
   3.706161294414578*^9}},ExpressionUUID->"1ba463ec-8c37-406d-b954-\
1ad88a18cddf"],

Cell["\<\
All these packages together with this notebook are contained in the file \
RG.zipwhich can be downloaded from my homepage.\
\>", "Text",
 CellChangeTimes->{{3.5994661710726385`*^9, 3.5994662775667295`*^9}, {
   3.6190050343161287`*^9, 3.6190050353701887`*^9}, {3.6271884497495513`*^9, 
   3.6271884511691537`*^9}, {3.665931304562168*^9, 3.6659313138660603`*^9}, {
   3.70610013604722*^9, 3.706100139174999*^9}, {3.706161062951438*^9, 
   3.706161066807439*^9}, {3.706161108735423*^9, 3.7061611094153137`*^9}, 
   3.708406281347135*^9},ExpressionUUID->"67c14754-a3c2-47be-8941-\
b2e449fc7cd1"],

Cell["Now save the notebook with your changes. ", "Text",
 CellChangeTimes->{{3.5994665178194714`*^9, 3.5994666645828657`*^9}, 
   3.599466711174531*^9},ExpressionUUID->"d28249fa-a763-4eaa-800c-\
10c51d58f5b4"]
}, Closed]],

Cell[CellGroupData[{

Cell["Start Initialization", "Subsection",
 CellChangeTimes->{{3.586059422846844*^9, 3.586059429305256*^9}, {
  3.5994666801267548`*^9, 
  3.5994666865821238`*^9}},ExpressionUUID->"2002252d-de38-4d9a-af4d-\
654011d91217"],

Cell[TextData[{
 "Start the session with the menu entry\n",
 StyleBox["Evaluation/Evaluate Initialization Cells",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{3.5994667527829103`*^9, 3.5994668982962337`*^9, 
  3.6174540239292994`*^9},ExpressionUUID->"0c045186-26b2-455b-94e6-\
ad823389187f"],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<riemannv2`\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5866138547444267`*^9, 3.5866138692368517`*^9}, {
   3.586613950076194*^9, 3.5866139579230075`*^9}, {3.5866140674976*^9, 
   3.5866140721308084`*^9}, {3.599488731869851*^9, 3.5994887396542645`*^9}, {
   3.6146876112871456`*^9, 3.6146876600459347`*^9}, 3.620195830390541*^9, 
   3.62019876191989*^9, {3.622600738722787*^9, 3.6226007397367887`*^9}, 
   3.6510375195493884`*^9, {3.665931347193974*^9, 3.665931351673382*^9}, {
   3.701498627046966*^9, 3.7014986287627163`*^9}},
 EmphasizeSyntaxErrors->
  True,ExpressionUUID->"4e55b106-846f-4a11-8a4b-bd0eed2553d1"],

Cell["\<\
With this command the packages eudiffgeov3.m,, euvecv2.m, neuvecv2.m, and \
tensalgv3.m are loaded automatically. \
\>", "Text",
 CellChangeTimes->{{3.6147522693505993`*^9, 3.614752329286028*^9}, 
   3.6147549420204678`*^9, {3.665931364001266*^9, 3.665931375601225*^9}, {
   3.665931409826481*^9, 3.6659314300009813`*^9}, 3.665931580791669*^9, {
   3.666159093137746*^9, 3.666159116481143*^9}, 
   3.7000259119255657`*^9},ExpressionUUID->"cd8194fa-1ada-4875-965b-\
e529088ac7d9"],

Cell["\<\
We occasionally apply some definitions of  Alfred Gray`s catalogs of surfaces \
 or curves:\
\>", "Text",
 CellChangeTimes->{{3.585969287625098*^9, 3.585969330509574*^9}, {
   3.599467063031656*^9, 3.599467071758155*^9}, 3.608968714234651*^9, {
   3.614749854875474*^9, 3.614749855920676*^9}, {3.614751974502735*^9, 
   3.614752017430191*^9}, {3.665932029571863*^9, 3.665932074459149*^9}, 
   3.6659870618681593`*^9},ExpressionUUID->"1b749b2d-d4be-4d2f-ab47-\
124ba6f57e3d"],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<SURFS`\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.585969335797983*^9, 
  3.5859693845168686`*^9}},ExpressionUUID->"7c458d99-4217-4ad1-9770-\
204c102e3cc8"],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<CURVES`\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.585969335797983*^9, 3.5859693845168686`*^9}, {
  3.599467094937481*^9, 
  3.599467126408281*^9}},ExpressionUUID->"a2222035-8284-4504-9441-\
b752c79e7c3d"],

Cell["\<\
Finally we set the dimension dim of the manifold and the index ind of the \
pseudo-Euclidean metric. These values may be changed during the session.\
\>", "Text",
 CellChangeTimes->{{3.5859759313146715`*^9, 3.585975988666952*^9}, {
  3.599464337184517*^9, 3.5994643851282587`*^9}, {3.6659321746421556`*^9, 
  3.6659323475526667`*^9}},ExpressionUUID->"8edff557-e991-4985-897f-\
c730339b724e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"ind", " ", "=", " ", "1"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5712056261758223`*^9, 3.571205633008634*^9}, {
   3.571290521047902*^9, 3.5712905431843405`*^9}, 3.572246357078577*^9, {
   3.5859758815878277`*^9, 3.585975898307784*^9}, 3.5994643960898857`*^9, {
   3.665932356320733*^9, 
   3.665932368808509*^9}},ExpressionUUID->"b8ef2a5a-230b-4106-9342-\
7f12370b8b84"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Imported Functions and their Usages", "Section",
 CellChangeTimes->{{3.660371762836636*^9, 3.660371768668347*^9}, {
  3.665986090467059*^9, 
  3.6659861281796*^9}},ExpressionUUID->"5514a4bb-f51d-4089-be08-0b8030a9340b"],

Cell["\<\
This Subsection contains tables of the symbols defined in the imported \
packages and in the Global context.\
\>", "Text",
 CellChangeTimes->{{3.627189064555636*^9, 3.6271891325329556`*^9}, {
   3.6271893877476053`*^9, 3.6271895372584705`*^9}, {3.6271896925739117`*^9, 
   3.627189757351226*^9}, {3.665932407648643*^9, 3.66593250700745*^9}, 
   3.66593261863035*^9, {3.665934491545096*^9, 3.665934626888296*^9}, {
   3.670564673136174*^9, 3.670564674192185*^9}, 3.670564713600008*^9, 
   3.672832002481967*^9, {3.672833702002837*^9, 3.672833703314961*^9}, 
   3.672834191901526*^9},ExpressionUUID->"6c7490d8-d8d0-4e7c-850b-\
57881e9037b0"],

Cell[CellGroupData[{

Cell["Hints", "Subsection",
 CellChangeTimes->{{3.672832010845109*^9, 
  3.672832013901222*^9}},ExpressionUUID->"3a45bfd3-bacd-4184-819a-\
ad8a5e6d7476"],

Cell["\<\
Use ?symbol to get information about the contents of  symbol,  e.g.\
\>", "Text",
 CellChangeTimes->{{3.627189064555636*^9, 3.6271891325329556`*^9}, {
   3.6271893877476053`*^9, 3.6271895372584705`*^9}, {3.6271896925739117`*^9, 
   3.627189757351226*^9}, {3.665932407648643*^9, 3.66593250700745*^9}, 
   3.66593261863035*^9, {3.665934491545096*^9, 3.665934626888296*^9}, {
   3.670564673136174*^9, 3.670564674192185*^9}, 3.670564713600008*^9, 
   3.672832002481967*^9},ExpressionUUID->"a9e00951-3d5a-4751-b399-\
23afaac4aa08"],

Cell[BoxData[
 RowBox[{"?", "vec"}]], "Input",
 CellChangeTimes->{{3.6271897619064336`*^9, 
  3.6271897862776766`*^9}},ExpressionUUID->"9f7bb773-ef8e-4a8a-8ffd-\
478c936396ea"],

Cell[BoxData[
 RowBox[{"?", "d"}]], "Input",
 CellChangeTimes->{{3.670565803490346*^9, 
  3.6705658045423813`*^9}},ExpressionUUID->"4cc5fe02-72a1-44f9-a945-\
c66779054e8e"],

Cell[TextData[{
 "This may be applied also to all built-in symbols of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " too, e.g. "
}], "Text",
 CellChangeTimes->{{3.627189871044626*^9, 3.6271898728230295`*^9}, {
   3.627189904807086*^9, 3.6271900373493195`*^9}, 3.6271926848071775`*^9, {
   3.66598619594663*^9, 3.6659861973144093`*^9}, {3.670564753879403*^9, 
   3.67056475534312*^9}},ExpressionUUID->"d008c924-d960-4663-90d5-\
95f081303a10"],

Cell[BoxData[
 RowBox[{"?", "D"}]], "Input",
 CellChangeTimes->{{3.62718999802925*^9, 
  3.62718999802925*^9}},ExpressionUUID->"80d87ada-d210-4527-a3f8-\
3777e239215e"],

Cell[TextData[{
 "Click ",
 Cell[BoxData[
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/D"]], "PrintUsage",
  CellChangeTimes->{3.6271899996516533`*^9},ExpressionUUID->
  "4a2ea118-b3ba-48ff-800e-949e2fd8450c"],
 " to read the corresponding help page. "
}], "Text",
 CellChangeTimes->{{3.627190069972977*^9, 3.6271900917838154`*^9}, {
   3.62719012261347*^9, 3.6271901802635717`*^9}, 
   3.6705657394208183`*^9},ExpressionUUID->"b1004939-087a-4602-9516-\
03be1f72f39f"],

Cell[BoxData[
 RowBox[{"?", "Context"}]], "Input",
 CellChangeTimes->{{3.66593315671798*^9, 
  3.665933156719692*^9}},ExpressionUUID->"890c9506-4798-4842-b1f2-\
5bcc89b7c1cf"],

Cell["\<\
Each Mathematica symbol has a uniquely defined context, e.g. \
\>", "Text",
 CellChangeTimes->{{3.6271907081877027`*^9, 3.6271909014054437`*^9}, 
   3.6271925396807213`*^9, 3.627192628438278*^9, {3.627192938764891*^9, 
   3.6271929510421124`*^9}, {3.6271930081724496`*^9, 3.62719300846885*^9}, {
   3.665932803572648*^9, 3.665932910619907*^9}, 3.67012489491586*^9, {
   3.6705648630546017`*^9, 3.670564948677863*^9}, {3.670565004941559*^9, 
   3.670565077572958*^9}, {3.670565529214388*^9, 3.6705655352406483`*^9}, {
   3.670565773374626*^9, 
   3.670565778982583*^9}},ExpressionUUID->"80093576-2503-4816-bb10-\
e3f7962790b1"],

Cell[BoxData[
 RowBox[{"Context", "[", "d", "]"}]], "Input",
 CellChangeTimes->{{3.665932917150882*^9, 
  3.66593291715226*^9}},ExpressionUUID->"cc12674d-a261-47dd-8d20-\
f13755e10d56"],

Cell[BoxData[
 RowBox[{"Context", "[", "D", "]"}]], "Input",
 CellChangeTimes->{{3.665932917150882*^9, 3.66593291715226*^9}, {
  3.665934065011921*^9, 
  3.665934073986747*^9}},ExpressionUUID->"fe16f2e6-2e46-43f0-a0a3-\
b5591c975454"],

Cell[BoxData[
 RowBox[{"Context", "[", "dim", "]"}]], "Input",
 CellChangeTimes->{{3.665932917150882*^9, 3.66593291715226*^9}, {
  3.665934065011921*^9, 
  3.665934099954069*^9}},ExpressionUUID->"c92854ba-754c-4d25-8597-\
b16e9db609ac"],

Cell["\<\
The context names are System, Global, or the name of the  package in which \
the symbol is introduced.\
\>", "Text",
 CellChangeTimes->{{3.6659330114028673`*^9, 3.665933149712717*^9}, 
   3.6659333945799713`*^9, {3.6659341716972847`*^9, 3.6659342036730223`*^9}, {
   3.670125031849616*^9, 
   3.670125062337022*^9}},ExpressionUUID->"2ea5f129-4f27-4685-9ed6-\
fda2c120f84d"],

Cell["The actual default context is", "Text",
 CellChangeTimes->{{3.665933522984679*^9, 3.6659335634817553`*^9}, 
   3.6659870673479433`*^9},ExpressionUUID->"f4dab96b-3814-448e-acc5-\
687f68b81328"],

Cell[BoxData[
 RowBox[{"Context", "[", "]"}]], "Input",
 CellChangeTimes->{{3.6705660199074993`*^9, 
  3.670566026140766*^9}},ExpressionUUID->"546c0b25-86c0-4f95-912f-\
39c51530f560"],

Cell["To see the explicit definition of a symbol use ??, e.g.", "Text",
 CellChangeTimes->{{3.627190069972977*^9, 3.6271900917838154`*^9}, {
   3.62719012261347*^9, 3.6271901802635717`*^9}, 3.6705657394208183`*^9, {
   3.6705662164347467`*^9, 
   3.670566247962484*^9}},ExpressionUUID->"537a641c-41fc-44ef-bf00-\
92841a61031d"],

Cell[BoxData[
 RowBox[{"??", "d"}]], "Input",
 CellChangeTimes->{{3.6271901901403894`*^9, 3.627190199687606*^9}, {
   3.627190256138506*^9, 3.6271902584941096`*^9}, 
   3.627190577991473*^9},ExpressionUUID->"d4aaf5d2-e51f-4846-88fc-\
0a7b23b6736d"],

Cell["\<\
I recommend to look into the package and see the definition without the \
prefixed context. For important or often used new functions I copied its \
Mathematica definition as not evaluatable cells into the notebook. \
\>", "Text",
 CellChangeTimes->{{3.6271907081877027`*^9, 3.6271909014054437`*^9}, 
   3.6271925396807213`*^9, 3.627192628438278*^9, {3.627192938764891*^9, 
   3.6271929510421124`*^9}, {3.6271930081724496`*^9, 3.62719300846885*^9}, {
   3.665932803572648*^9, 3.665932910619907*^9}, 3.67012489491586*^9, {
   3.6705648630546017`*^9, 3.670564948677863*^9}, {3.670565004941559*^9, 
   3.670565077572958*^9}, 
   3.670565529214388*^9},ExpressionUUID->"33ae1aad-a17a-4aa3-bf8f-\
7ae8c6d290e0"],

Cell["\<\
Many Mathematica built-in functions are \[OpenCurlyDoubleQuote]ReadProtected\
\[CloseCurlyDoubleQuote]; its defining Mathematica program is hidden:\
\>", "Text",
 CellChangeTimes->{{3.67056527171491*^9, 
  3.670565355138303*^9}},ExpressionUUID->"59b9f869-4840-40e3-b6fc-\
50907cf066f4"],

Cell[BoxData[
 RowBox[{"??", "D"}]], "Input",
 CellChangeTimes->{{3.670565129053185*^9, 
  3.670565129972315*^9}},ExpressionUUID->"0c6c773c-14aa-49e3-a82d-\
c3805ad9b5d4"],

Cell[BoxData[
 RowBox[{"?", "ReadProtected"}]], "Input",
 CellChangeTimes->{{3.670565363263135*^9, 
  3.670565369370036*^9}},ExpressionUUID->"46478d2e-dbde-4f34-86ad-\
149162ce26e4"],

Cell[BoxData[
 RowBox[{"?", "Contexts"}]], "Input",
 CellChangeTimes->{{3.66593315671798*^9, 3.665933156719692*^9}, {
  3.665933415628833*^9, 
  3.665933421195603*^9}},ExpressionUUID->"be503de2-51fc-4f73-a4e1-\
60c528badd9b"],

Cell[TextData[{
 "The list of all contexts is very large since ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " comes with a lot of built-in or additional loadable packages:"
}], "Text",
 CellChangeTimes->{{3.6659338257570477`*^9, 
  3.6659338955483513`*^9}},ExpressionUUID->"6bd8cef0-8a44-40b9-9dbd-\
726a4a051303"],

Cell[BoxData[
 RowBox[{"Contexts", "[", "]"}]], "Input",
 CellChangeTimes->{{3.665933291429811*^9, 
  3.665933298365471*^9}},ExpressionUUID->"51798630-7a72-45ab-b99f-\
4b221506e836"]
}, Closed]],

Cell[CellGroupData[{

Cell["Symbol Tables. Usages.", "Subsection",
 CellChangeTimes->{{3.672832090780476*^9, 
  3.672832115188204*^9}},ExpressionUUID->"9c1f3292-e616-46e9-9469-\
334cef4b062c"],

Cell[CellGroupData[{

Cell["Riemannian Geometry and Linear Connections", "Subsubsection",
 CellChangeTimes->{{3.701498061559266*^9, 3.70149808699891*^9}, 
   3.702032471059389*^9, {3.7030421862643003`*^9, 
   3.703042187040419*^9}},ExpressionUUID->"a05a83fa-1765-4bc5-916d-\
aeba5441df04"],

Cell[TextData[{
 "The list of all symbol with the same context can be generated by a command \
like this, here the package riemannv2.m about\n",
 StyleBox["Riemannian geometry and linear connections:",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.665934269848153*^9, 3.665934386480563*^9}, {
   3.672832701754607*^9, 3.672832716586173*^9}, {3.672832788898028*^9, 
   3.6728328391693*^9}, 3.6728330090165033`*^9, 3.672833729394853*^9, {
   3.7014980255359507`*^9, 
   3.701498026791954*^9}},ExpressionUUID->"8c67220c-bd8a-488a-acf1-\
b99e0cc847fa"],

Cell[BoxData[
 RowBox[{"?", "riemannv2`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.665929497843286*^9, 3.66592950483538*^9}, {
  3.701497984728421*^9, 
  3.701497987904351*^9}},ExpressionUUID->"60b3295e-abdc-46e0-868d-\
f1814927fa93"],

Cell[BoxData[
 RowBox[{"lor", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.704263076130748*^9, 
  3.704263119384859*^9}},ExpressionUUID->"ac3b1292-5917-44a1-b4f8-\
3aa54f4f62ef"],

Cell[BoxData[
 RowBox[{"?", "dd"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.703042221107059*^9, 
  3.703042223223713*^9}},ExpressionUUID->"cc877281-3ca5-408d-90b4-\
5adeb85085eb"],

Cell["\<\
 Click the symbol to read its \[OpenCurlyDoubleQuote]usage\
\[CloseCurlyDoubleQuote], a short description of the content and the syntax \
of the function called with the symbol,  or use ?Symbol instead.\
\>", "Text",
 CellChangeTimes->{
  3.665934691705264*^9, {3.665934725312172*^9, 3.665934745864292*^9}, 
   3.66598631640864*^9, {3.670566558032034*^9, 
   3.670566570687921*^9}},ExpressionUUID->"562a137b-285f-417d-9c32-\
9d2c79c458c8"]
}, Closed]],

Cell[CellGroupData[{

Cell["Gaussian Curvature", "Subsubsection",
 CellChangeTimes->{{3.701498061559266*^9, 3.70149808699891*^9}, 
   3.702032471059389*^9, {3.7030421862643003`*^9, 3.703042187040419*^9}, {
   3.7060804379891367`*^9, 
   3.706080454701277*^9}},ExpressionUUID->"a1daffb9-3624-4369-9338-\
e5fe5108ba9a"],

Cell[TextData[{
 "We introduce and adapt here some functions defined in the package \
eudiffgeov3.m to be applied in this notebook. ",
 StyleBox["Don\[CloseCurlyQuote]t load this package together with \
riemannv2.m; some functions or modules of eudiffgeov3.m are not compatible \
with  those of riemannv2.m!",
  FontColor->RGBColor[1, 0, 0]]
}], "Text",
 CellChangeTimes->{{3.706080464108864*^9, 3.7060807262517977`*^9}, {
  3.70609950655986*^9, 3.70609958072018*^9}, {3.706099627783946*^9, 
  3.7060996443121243`*^9}},ExpressionUUID->"afee2d48-505f-4a8b-8af7-\
b22512757069"],

Cell[BoxData[
 RowBox[{"?", "tv"}]], "Input",
 CellChangeTimes->{{3.7060849515114927`*^9, 
  3.706084952939389*^9}},ExpressionUUID->"f2348077-d15c-4b18-aca6-\
49bb727c097c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"amf", "[", "3", "]"}], "[", "f_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"dotnorme", "[", 
     RowBox[{"Cross", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"tv", "[", "f", "]"}], "[", "1", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"tv", "[", "f", "]"}], "[", "2", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}], "/;", "\n", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", 
        RowBox[{"f", "[", 
         RowBox[{"u", ",", "v"}], "]"}], "]"}], " ", "==", " ", "List"}], "&&", 
      RowBox[{
       RowBox[{"Length", "[", 
        RowBox[{"f", "[", 
         RowBox[{"u", ",", "v"}], "]"}], "]"}], "==", " ", "3"}]}], ")"}]}]}],
   ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.706079995081018*^9, {3.706084992782263*^9, 
   3.706085022001693*^9}},ExpressionUUID->"1453cfc6-8bbc-4b10-999f-\
de20555f399d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amf", "::", "usage"}], " ", "=", " ", 
   "\"\<amf[i][f][u,v]is the i-th vector, i = 1,2,3, of the orthonormal \
moving \nframe adapted to the parameter representation of the surface f[u,v] \
in the \nEuclidean 3-space.\>\""}], ";"}]], "Input",
 InitializationCell->
  True,ExpressionUUID->"a3cbe7de-b1f2-4ee9-aa00-5429224d1d1b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff_", "]"}], "[", 
     RowBox[{"u_", ",", "v_"}], "]"}], "[", 
    RowBox[{"x_List", ",", "y_List"}], "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "ff", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", "x"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", 
        RowBox[{
         RowBox[{"amf", "[", "3", "]"}], "[", "ff", "]"}], "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ".", "y"}], ")"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.706080256578292*^9},ExpressionUUID->"25307d71-9366-40b4-91fb-\
655b87303bce"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fund2", "::", "usage"}], "=", " ", 
   "\"\<fund2[f][u,v][xx,yy] is the value of the second fundamental form of \n\
the surface f[u,v] in the Eucliddean 3-space for the tangent vectors xx,yy.\>\
\""}], ";"}]], "Input",
 InitializationCell->
  True,ExpressionUUID->"840491e8-3592-45bd-951e-7fddf27a9151"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"1", ",", "2"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fd2", "[", 
      RowBox[{"2", ",", "1"}], "]"}], "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"fund2", "[", "ff", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.706080256578292*^9, 
  3.706080266767049*^9}},ExpressionUUID->"937a557a-614d-433d-be2b-\
22ed607f0dbb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"fd2", "::", "usage"}], " ", "=", " ", 
    "\"\<{fd2[i,j][f][u,v], i,j = 1, 2} is the matrix of the coordinates  \n\
with respect to the local standard basis {du,dv} of the second fundamental \
form \nof the surface f[u,v] in the Euclidean 3-space.\>\""}], ";"}], 
  "\n"}]], "Input",
 InitializationCell->
  True,ExpressionUUID->"52db5bf5-c8c9-44dd-a6d4-b5d178e2bbaf"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"gauss", "[", "ff_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"fd2", "[", 
          RowBox[{"i", ",", "j"}], "]"}], "[", "ff", "]"}], "[", 
        RowBox[{"u", ",", "v"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}], "/", "\n", 
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"rgt", "[", "ff", "]"}], "[", 
         RowBox[{"u", ",", "v"}], "]"}], "[", 
        RowBox[{"i", ",", "j"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.706079675124263*^9, 3.706080796939583*^9, {3.706088511678919*^9, 
   3.706088565672379*^9}},ExpressionUUID->"2eafcd5a-25ab-4a67-a8a1-\
ab7505abcb71"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gauss", "::", "usage"}], " ", "=", " ", 
   "\"\<gauss[f][u,v] is the Gaussian curvature of the surface f[u,v]\nin the \
Euclidean 3-space.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{3.706079675124263*^9, 
  3.706080796939583*^9},ExpressionUUID->"d691ca25-517b-4a94-97cc-\
d8826734d97f"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gauss", "[", 
    RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7060885985260983`*^9, 
  3.706088673737146*^9}},ExpressionUUID->"805598d0-c135-4527-8a76-\
e5a4fd6db37d"]
}, Closed]],

Cell[CellGroupData[{

Cell["Euclidean Linear Algebra", "Subsubsection",
 CellChangeTimes->{{3.701498216862012*^9, 
  3.701498229661664*^9}},ExpressionUUID->"277c8a40-5505-4565-8a9d-\
d185439778bd"],

Cell["Package euvecv2.m", "Text",
 CellChangeTimes->{{3.672833198190639*^9, 3.672833241110346*^9}, 
   3.672834307094248*^9, {3.7014982473098717`*^9, 
   3.701498249597754*^9}},ExpressionUUID->"af2255e3-3d37-4749-96f2-\
02cbce3f5d44"],

Cell[BoxData[
 RowBox[{"?", "euvecv2`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6320391312608085`*^9, 3.6320391323996105`*^9}, {
  3.634799295086132*^9, 
  3.634799324136383*^9}},ExpressionUUID->"8091782b-8ec7-402d-b948-\
cdc0e2ea3ea8"],

Cell[BoxData[
 RowBox[{"Attributes", "[", "innerprod", "]"}]], "Input",
 CellChangeTimes->{{3.673787929150423*^9, 
  3.673787956838335*^9}},ExpressionUUID->"e6c8bad7-9828-49e5-b0fe-\
242e63804845"]
}, Closed]],

Cell[CellGroupData[{

Cell["Tensor Algebra", "Subsubsection",
 CellChangeTimes->{{3.7014982853570337`*^9, 
  3.701498293317316*^9}},ExpressionUUID->"5f67b2d7-c1d1-44f7-9ded-\
e903a5b063e3"],

Cell["Package tensalgv3.m", "Text",
 CellChangeTimes->{{3.672833290173728*^9, 3.672833340093347*^9}, 
   3.67283421693338*^9, 3.6728343164622393`*^9, 3.699884392994048*^9, {
   3.701498251958838*^9, 
   3.701498275013556*^9}},ExpressionUUID->"c9aa5cc1-d3fb-479e-bf6d-\
91859599400d"],

Cell[BoxData[
 RowBox[{"?", "tensalgv3`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6324584219750133`*^9, 3.6324584257814198`*^9}, {
   3.6347982023613963`*^9, 3.6347982033773985`*^9}, {3.65192303816436*^9, 
   3.651923052279142*^9}, 
   3.69988439952015*^9},ExpressionUUID->"6c787980-f460-4acc-b4f4-\
b9eec8570621"],

Cell[BoxData[
 RowBox[{"projpt", "[", 
  RowBox[{"{", 
   RowBox[{"5.", ",", "a", ",", "b", ",", "c"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6738470503769073`*^9, 
  3.673847185825201*^9}},ExpressionUUID->"33e434f2-4b10-4073-b502-\
de5b26497592"],

Cell[BoxData[
 RowBox[{"Attributes", "[", "projpt", "]"}]], "Input",
 CellChangeTimes->{{3.673847211989397*^9, 
  3.673847238072318*^9}},ExpressionUUID->"038a3dc3-0660-4766-a782-\
81c3a4bbd438"]
}, Closed]],

Cell[CellGroupData[{

Cell["Pseudo-Euclidean Linear Algebra", "Subsubsection",
 CellChangeTimes->{{3.7014983202049513`*^9, 
  3.701498340452976*^9}},ExpressionUUID->"4717da45-b0b8-4ed8-9afc-\
d46336401675"],

Cell["Package neuvecv2.m", "Text",
 CellChangeTimes->{{3.672833390164997*^9, 3.672833438396696*^9}, 
   3.672834328182325*^9, {3.70149835312497*^9, 
   3.7014983538367443`*^9}},ExpressionUUID->"216ab05c-7349-4945-8dc3-\
27f925e663f1"],

Cell[BoxData[
 RowBox[{"?", "neuvecv2`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6324584219750133`*^9, 3.6324584257814198`*^9}, {
  3.6347982023613963`*^9, 3.6347982033773985`*^9}, {3.65192303816436*^9, 
  3.651923052279142*^9}, {3.6659347863849697`*^9, 
  3.6659348077681303`*^9}},ExpressionUUID->"2e10e346-7fbc-4a61-a393-\
7fbf8be17574"]
}, Closed]],

Cell[CellGroupData[{

Cell["Alfred Gray\[CloseCurlyQuote]s  Catalogue of Curves and Surfaces", \
"Subsubsection",
 CellChangeTimes->{{3.701498393852515*^9, 
  3.701498432036117*^9}},ExpressionUUID->"d419db38-a77a-4402-92db-\
1691da4d2c6f"],

Cell[TextData[{
 "The two next cells evaluate to ",
 StyleBox["Alfred Gray\[CloseCurlyQuote]s catalogs of curves and surfaces:",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.672832275754974*^9, 3.672832387596777*^9}, 
   3.672834222373241*^9},ExpressionUUID->"f7a5bbcd-8546-4aaa-af65-\
f6a24861be9b"],

Cell["Package CURVES.m:", "Text",
 CellChangeTimes->{{3.6728335047321053`*^9, 3.672833533012076*^9}, 
   3.6728335812436857`*^9},ExpressionUUID->"68460d51-8e98-4775-9673-\
3c87de96ed52"],

Cell[BoxData[
 RowBox[{"?", "CURVES`*"}]], "Input",
 CellChangeTimes->{{3.665982439563437*^9, 
  3.6659824426034107`*^9}},ExpressionUUID->"fbfa0e09-6a10-4f36-9710-\
978c1eb6a626"],

Cell["Package SURFS.m:", "Text",
 CellChangeTimes->{{3.6728335399537983`*^9, 
  3.672833592651785*^9}},ExpressionUUID->"2e2a45e5-5536-4692-a93a-\
499928fcecea"],

Cell[BoxData[
 RowBox[{"?", "SURFS`*"}]], "Input",
 CellChangeTimes->{{3.6324584219750133`*^9, 3.6324584257814198`*^9}, {
  3.6347982023613963`*^9, 3.6347982033773985`*^9}, {3.651760621529701*^9, 
  3.651760630434094*^9}},ExpressionUUID->"c6463c20-aecf-441a-a90d-\
4befb76dfa9e"]
}, Closed]],

Cell[CellGroupData[{

Cell["The Global Context", "Subsubsection",
 CellChangeTimes->{{3.701498461083753*^9, 
  3.7014984722276573`*^9}},ExpressionUUID->"04a037da-fcbb-4cc4-bf12-\
b18fe067cda0"],

Cell[TextData[{
 "Evaluate the next cell to see the symbols actually used in the ",
 StyleBox["Global Context:",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6201992773654275`*^9, 3.620199340423375*^9}, {
   3.672833608491651*^9, 3.672833611355482*^9}, 3.672834345445994*^9, {
   3.6759212704853153`*^9, 
   3.6759212762768993`*^9}},ExpressionUUID->"b9668115-04c1-44f7-a2bb-\
a14b4d234e19"],

Cell[BoxData[
 RowBox[{"?", "Global`*"}]], "Input",
 CellChangeTimes->{{3.6201992089418144`*^9, 
  3.6201992164298277`*^9}},ExpressionUUID->"fca3fac9-89eb-4f19-b4d4-\
48a41b967751"]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1. Riemannian Spaces of Constant Curvature", "Section",
 CellChangeTimes->{{3.630503957402012*^9, 3.6305039594300156`*^9}, {
   3.630756824164742*^9, 3.6307568393123684`*^9}, 3.6308230597151585`*^9, {
   3.63142953607233*^9, 3.6314295406743383`*^9}, {3.6377283855593348`*^9, 
   3.6377284117517805`*^9}, 3.653370985758643*^9, 3.654350418876013*^9, 
   3.657522188021221*^9, 3.657526709214322*^9, {3.6659348859193897`*^9, 
   3.665934890175433*^9}, 3.6659911645264673`*^9, 
   3.701920398043889*^9},ExpressionUUID->"dd3f0dc2-9417-4d27-8eba-\
46ce977e86ae"],

Cell["\<\
  In this Section we introduce  n-dimensional Riemannian metrics of constant \
curvature. The manifolds considered here are open, connected sets of an \
n-dimensional Euclidean space provided with orthonormal Cartesian \
coordinates. More general manifold concept will be treated in Subsection 2.1.\
\>", "Text",
 CellChangeTimes->{{3.702606827189809*^9, 3.7026069675970783`*^9}, {
  3.702607012972896*^9, 3.702607230458996*^9}, {3.702612765766058*^9, 
  3.702612767709509*^9}},ExpressionUUID->"79bb2371-ee3a-439c-8dea-\
d1d1960146b7"],

Cell[CellGroupData[{

Cell["1.1.  Pseudo-Riemannian Metrics  ", "Subsection",
 CellChangeTimes->{{3.6314296796345844`*^9, 3.631429764904735*^9}, {
  3.660455601903885*^9, 3.6604556072478313`*^9}, {3.661496531596258*^9, 
  3.661496550876772*^9}, {3.662272390214491*^9, 3.6622723989258423`*^9}, {
  3.662711656297037*^9, 3.6627116838722878`*^9}, {3.6627225598472977`*^9, 
  3.662722561886978*^9}, {3.6627253363106117`*^9, 3.662725340078616*^9}, {
  3.702613004930952*^9, 3.702613014594643*^9}},
 FontColor->RGBColor[
  0.7803921568627451, 0.4235294117647059, 
   0.1607843137254902],ExpressionUUID->"441f0c9b-50c9-4b41-99af-07daf17bcea3"],

Cell[CellGroupData[{

Cell["1.1.1.  A General pseudo-Riemannian Metric  ", "Subsubsection",
 CellChangeTimes->{{3.702612965979891*^9, 
  3.702612983402677*^9}},ExpressionUUID->"84d1382d-5539-4dc9-a1e1-\
8c01905d929c"],

Cell[TextData[{
 "A  ",
 StyleBox["pseudo-Riemannian space of index k",
  FontColor->RGBColor[0, 0, 1]],
 " is a differentiable manifold on which a smooth twofold covariant symmetric \
non-degenerate tensor  field  RG[x,gg]  of index k is given.  If this field \
is positive definite at each point, i.e. k=0,  one speaks about a ",
 StyleBox["Riemannian space (or manifold)",
  FontColor->RGBColor[0, 0, 1]],
 ". The field itself is called a ",
 StyleBox["pseudo-Riemannian ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["respectively",
  FontColor->GrayLevel[0]],
 StyleBox[" Riemannian metric.",
  FontColor->RGBColor[0, 0, 1]],
 " \nIn this section we consider introductory examples of Riemannian spaces \
on an elementary level. They will serve as standard examples in  the \
forthcoming sections. "
}], "Text",
 CellChangeTimes->{{3.6230444750187387`*^9, 3.623044567434301*^9}, {
   3.6230450135638847`*^9, 3.6230451749305677`*^9}, {3.623045224821754*^9, 
   3.6230452269787817`*^9}, {3.62304532981092*^9, 3.6230453661589837`*^9}, {
   3.6308220756186304`*^9, 3.630822091561858*^9}, 3.630822945912959*^9, {
   3.6308230431011295`*^9, 3.6308230451447334`*^9}, {3.6308231371068945`*^9, 
   3.6308231701321526`*^9}, 3.6308232175094357`*^9, {3.6308232477422895`*^9, 
   3.6308232638259172`*^9}, {3.6308232939495707`*^9, 
   3.6308233230904217`*^9}, {3.6308233668484983`*^9, 
   3.6308233995305557`*^9}, {3.6308234513694468`*^9, 3.63082345306985*^9}, {
   3.6308234887159123`*^9, 3.630823532349189*^9}, {3.63082356691885*^9, 
   3.6308236105209265`*^9}, 3.630823682827053*^9, {3.630823868185399*^9, 
   3.630823899635054*^9}, {3.6309953805796504`*^9, 3.6309953812660513`*^9}, 
   3.63142957016439*^9, {3.6377284844011083`*^9, 3.637728516271964*^9}, {
   3.637728546520417*^9, 3.637728602196915*^9}, {3.6377288026197968`*^9, 
   3.6377288529892893`*^9}, {3.637728900301573*^9, 3.637728936831238*^9}, {
   3.66031441350058*^9, 3.6603144159714746`*^9}, {3.66278946075266*^9, 
   3.662789478831665*^9}, {3.662789521768197*^9, 3.662789523023389*^9}, 
   3.665987110956296*^9, {3.700027518593512*^9, 3.7000275590812197`*^9}, {
   3.700043855596665*^9, 3.7000439519563417`*^9}},
 CellTags->"defRsp",ExpressionUUID->"95da0e7c-02d8-4123-8770-dd98e0202c16"],

Cell[TextData[{
 StyleBox["A  symmetric,twofold covariant tensor field",
  FontColor->RGBColor[0, 0, 1]],
 " is usually defined by its symmetric, coordinate matrix, as defined in \
riemannv2.m:"
}], "Text",
 CellChangeTimes->{{3.6308250576756973`*^9, 3.630825104916581*^9}, {
   3.630825144510947*^9, 3.6308251487853546`*^9}, {3.63082518053141*^9, 
   3.6308252032470503`*^9}, {3.630921050189472*^9, 3.630921057927086*^9}, 
   3.662711130261458*^9, {3.662711190013135*^9, 3.6627113138115597`*^9}, 
   3.6627197187202063`*^9, 3.66598726647731*^9, 3.7000433580582447`*^9, {
   3.700043578973946*^9, 3.7000436642374487`*^9}, 3.700044014873167*^9, {
   3.7000440626906233`*^9, 3.7000440768667803`*^9}, {3.700045284145282*^9, 
   3.7000453074970818`*^9}, {3.702525534499955*^9, 
   3.702525538467964*^9}},ExpressionUUID->"378e3208-251f-4cf0-a705-\
c4a69ada3d94"],

Cell[BoxData[
 RowBox[{"?", "RG"}]], "Input",
 CellChangeTimes->{{3.700042744644565*^9, 
  3.7000427467028017`*^9}},ExpressionUUID->"c0a4c9a7-3f4f-446e-a561-\
91805953babb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RG", "[", 
     RowBox[{"gg_:", "g"}], "]"}], "[", "x__", "]"}], ":=", 
   RowBox[{"Normal", "[", 
    RowBox[{"SymmetrizedArray", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"i_", ",", "j_"}], "}"}], ":>", 
       RowBox[{
        RowBox[{"gg", "[", "x", "]"}], "[", 
        RowBox[{"i", ",", "j"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}], ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", "\n", 
      RowBox[{"{", 
       RowBox[{"Symmetric", "[", 
        RowBox[{"{", 
         RowBox[{"1", ",", "2"}], "}"}], "]"}], "}"}]}], "]"}], "]"}]}], 
  ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.7026074176654654`*^9, 3.702607437032297*^9}, {
  3.702607468041061*^9, 3.702607516952574*^9}, {3.702608320233798*^9, 
  3.702608326618082*^9}},ExpressionUUID->"8c37ab0a-2e8c-40bf-baae-\
8b16a141c639"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RG", "[", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.700044139844413*^9, 3.700044176553866*^9}, {
  3.702609356040367*^9, 3.7026093568869743`*^9}, {3.702609420160049*^9, 
  3.7026094565756893`*^9}},ExpressionUUID->"db96acfd-88b6-4584-95b4-\
09d97e2ee34d"],

Cell["\<\
Unfortunately, g[x,y][2,1] is not defined by this definition, thus the \
relation\
\>", "Text",
 CellChangeTimes->{{3.7000442109130373`*^9, 3.700044304040676*^9}, {
  3.700044640341851*^9, 
  3.700044654317378*^9}},ExpressionUUID->"195f635b-b38e-42bc-a75c-\
cfbc7dbaa661"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "[", 
   RowBox[{"2", ",", "1"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "[", 
   RowBox[{"1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.700044331637598*^9, 3.700044374472094*^9}, {
  3.700044566379822*^9, 
  3.7000446178623457`*^9}},ExpressionUUID->"d29b7fa4-1dd1-4202-99e4-\
474c2d26b8f1"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.700044661459234*^9, 
  3.700044671981593*^9}},ExpressionUUID->"3de5355a-158a-491b-9b5c-\
97af2cc30a9c"],

Cell["\<\
isn\[CloseCurlyQuote]t confirmed. The complete list of the coordinates of the \
field is obtained by\
\>", "Text",
 CellChangeTimes->{{3.7000449445075006`*^9, 3.700044957946906*^9}, {
  3.700045008362928*^9, 
  3.7000450745787077`*^9}},ExpressionUUID->"ec389161-d5b0-4f52-a245-\
b525966f7344"],

Cell[BoxData[
 RowBox[{"?", "rg"}]], "Input",
 CellChangeTimes->{{3.700035888602375*^9, 
  3.7000358906325073`*^9}},ExpressionUUID->"be1550a9-6866-4f5c-a22f-\
189d7d5b26e9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"rg", "[", 
       RowBox[{"gg_:", "g"}], "]"}], "[", "x__", "]"}], "[", 
     RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", 
    RowBox[{
     RowBox[{"Evaluate", "[", 
      RowBox[{
       RowBox[{"RG", "[", "gg", "]"}], "[", "x", "]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"i", ",", "j"}], "]"}], "]"}]}], ";"}], "\n"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.702608516832005*^9, 
  3.702608560695422*^9}},ExpressionUUID->"9dcaa8ce-da6d-4f21-b53b-\
60f8e6de9ee0"],

Cell["Now we get", "Text",
 CellChangeTimes->{{3.700045145321824*^9, 
  3.700045148545738*^9}},ExpressionUUID->"1e8a26dd-5a76-46fd-b00f-\
5dd9d5697851"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rg", "[", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], "[", 
   RowBox[{"2", ",", "1"}], "]"}], "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"rg", "[", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], "[", 
   RowBox[{"1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.700035914259836*^9, 3.7000360076199102`*^9}, {
  3.7000426599795427`*^9, 3.70004267371152*^9}, {3.700042822519594*^9, 
  3.7000428267260017`*^9}, {3.7000432049361687`*^9, 3.700043216747568*^9}, {
  3.700043256223607*^9, 3.700043269571178*^9}, {3.7000452295055428`*^9, 
  3.700045234633346*^9}, {3.702525808698263*^9, 
  3.702525814841902*^9}},ExpressionUUID->"270aff66-2802-4124-b233-\
a59f4d1d775c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rg", "[", "]"}], "[", 
   RowBox[{"x", ",", "y"}], "]"}], "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.700035914259836*^9, 3.7000360076199102`*^9}, {
  3.7000426599795427`*^9, 3.70004267371152*^9}, {3.700042822519594*^9, 
  3.7000428267260017`*^9}, {3.7000432049361687`*^9, 3.700043216747568*^9}, {
  3.700043256223607*^9, 3.700043269571178*^9}, {3.7000452295055428`*^9, 
  3.700045234633346*^9}, {3.702525808698263*^9, 3.702525814841902*^9}, {
  3.7026098004042587`*^9, 
  3.702609808563653*^9}},ExpressionUUID->"e1db6e81-0b90-442d-b260-\
4c395d38ba97"],

Cell[TextData[{
 StyleBox["Warning. ",
  FontColor->RGBColor[1, 0, 0]],
 " Using RG or  rg don\[CloseCurlyQuote]t forget to enter the identifier \
gg-slot of the metric! Omitting the identifier gg-slot gives undefined \
expressions:"
}], "Text",
 CellChangeTimes->{{3.70247572724876*^9, 3.702475782727293*^9}, {
  3.702524610986663*^9, 3.702524676354027*^9}, {3.702526074953434*^9, 
  3.7025260895282183`*^9}, {3.70260983957145*^9, 3.702609843403125*^9}, {
  3.7026099836749363`*^9, 
  3.7026099851142683`*^9}},ExpressionUUID->"ab79f887-68a2-4cde-9e5e-\
1a99dded2433"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rg", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"1", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.702475788599333*^9, 
  3.702475829415023*^9}},ExpressionUUID->"50d91f22-6611-4b13-857b-\
664ce00654bb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rg", "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "[", 
   RowBox[{"1", ",", "2"}], "]"}], "-", 
  RowBox[{
   RowBox[{"rg", "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "[", 
   RowBox[{"2", ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.702524532811853*^9, 
  3.7025245634752483`*^9}},ExpressionUUID->"a75f9fa0-d908-4ac8-b6b8-\
8dbb3b53e961"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rg", "[", 
   RowBox[{"x", ",", "y"}], "]"}], "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{
  3.702608443521324*^9},ExpressionUUID->"8317e8b8-1b91-433f-ab13-\
085fa6398e6f"],

Cell[BoxData[
 RowBox[{"RG", "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.7026100600201397`*^9, 
  3.7026100692018623`*^9}},ExpressionUUID->"5f179aaf-f11a-4ec9-9329-\
1f443ae904a4"],

Cell["\<\
Entering an undefined symbol, e.g. h, one gets  a formal, non specvified \
metric:\
\>", "Text",
 CellChangeTimes->{{3.702475192412055*^9, 3.702475386938756*^9}, 
   3.7024757037123833`*^9, {3.7024768459587717`*^9, 3.702476853030506*^9}, {
   3.702525890706523*^9, 3.702525891857267*^9}, 3.7025259220732393`*^9, {
   3.702525984433002*^9, 3.7025259961364107`*^9}, {3.70252604001672*^9, 
   3.702526043560254*^9}},ExpressionUUID->"7f35f241-a59d-44d3-8822-\
f2aaf880f78b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"rg", "[", "h", "]"}], "[", 
      RowBox[{"x", ",", "y"}], "]"}], "[", 
     RowBox[{"i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.702475396567774*^9, 3.7024754891460733`*^9}, {
   3.70252438358891*^9, 3.7025244188446703`*^9}, 3.70252593763358*^9, {
   3.702610116921859*^9, 
   3.7026101617693853`*^9}},ExpressionUUID->"dc1b6662-213c-436f-a9c1-\
f15f3f03df84"],

Cell["If a well defined metric is chosen, e.g.", "Text",
 CellChangeTimes->{{3.702475547608859*^9, 
  3.7024755980728617`*^9}},ExpressionUUID->"4bdcac04-548c-43b1-9e54-\
01ad16330424"],

Cell[BoxData[
 RowBox[{"?", "eucl"}]], "Input",
 CellChangeTimes->{{3.7024756088981953`*^9, 3.702475611112383*^9}, 
   3.702610191656672*^9},ExpressionUUID->"a6890570-58d5-45e1-b072-\
461a85f4473a"],

Cell["this metric is applied:", "Text",
 CellChangeTimes->{{3.702476355239173*^9, 
  3.702476397553974*^9}},ExpressionUUID->"823829ee-4bdc-4379-8aec-\
586e40fedd81"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"rg", "[", "eucl", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.702524471020405*^9, 3.7025244725161448`*^9}, 
   3.702610219393095*^9},ExpressionUUID->"4674f1d1-aeb5-4981-b47b-\
514d3a4d0dff"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{"EUCL", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.702610253507201*^9, 
  3.702610287111918*^9}},ExpressionUUID->"9d382ddd-57e0-4125-8324-\
c5d509031c1d"],

Cell[BoxData[
 RowBox[{"?", "EUCL"}]], "Input",
 CellChangeTimes->{{3.702610306729911*^9, 
  3.702610310904147*^9}},ExpressionUUID->"8d52235b-333d-4b3b-ac1a-\
9e35817ec12e"],

Cell[BoxData[
 RowBox[{"?", "var"}]], "Input",
 CellChangeTimes->{{3.7026103217448263`*^9, 
  3.70261032398412*^9}},ExpressionUUID->"eba3e470-9d6d-4b31-84b0-\
ef84813de998"],

Cell[TextData[{
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 StyleBox["1",
  FontColor->RGBColor[0, 0, 1]],
 ". A symbolic, non-specified field is useful for proving general identities \
for pseudo-Riemannian spaces by symbolic calculations. RG returns it as the \
default field:"
}], "Text",
 CellChangeTimes->{{3.6308250576756973`*^9, 3.630825104916581*^9}, {
   3.630825144510947*^9, 3.6308251487853546`*^9}, {3.63082518053141*^9, 
   3.6308252032470503`*^9}, {3.630921050189472*^9, 3.630921057927086*^9}, 
   3.662711130261458*^9, {3.662711190013135*^9, 3.6627113138115597`*^9}, 
   3.6627197187202063`*^9, 3.66598726647731*^9, 3.7000433580582447`*^9, {
   3.700043578973946*^9, 3.7000436642374487`*^9}, 3.7000440961873302`*^9, {
   3.700045562199321*^9, 3.7000456425183496`*^9}, {3.700045672646379*^9, 
   3.700045718998156*^9}, {3.702612508911418*^9, 
   3.702612516151424*^9}},ExpressionUUID->"be9ca9b7-360d-4907-8551-\
09d52236d629"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RG", "[", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], " ", "]"}], "\[Equal]", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"rg", "[", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.700042852221053*^9, 3.7000429894769993`*^9}, {
   3.700043040557005*^9, 3.700043107620969*^9}, 3.700043231791933*^9, 
   3.700043289703989*^9, {3.702610409615411*^9, 
   3.702610484064609*^9}},ExpressionUUID->"9bcd6162-47c5-4862-afda-\
18429ecd543a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rg", "[", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "[", 
   RowBox[{"2", ",", "1"}], "]"}], "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"rg", "[", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "[", 
   RowBox[{"1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.700028105956921*^9, 3.700028125955697*^9}, {
   3.700028358541925*^9, 3.700028396785692*^9}, 3.700029214209708*^9, 
   3.700029290701*^9, {3.7000293518883963`*^9, 3.700029398202299*^9}, {
   3.700029787067758*^9, 3.700029791607058*^9}, {3.700029896522086*^9, 
   3.70002990923812*^9}},ExpressionUUID->"2077a8e3-17d2-46d8-9f21-\
a4a9f97a76a3"],

Cell["It is symmetric by definition:", "Text",
 CellChangeTimes->{{3.6308254016682*^9, 
  3.6308254155094247`*^9}},ExpressionUUID->"41f47e45-fdc9-40f2-84ae-\
d57a8f63b134"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", 
  RowBox[{
   RowBox[{"RG", "[", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.662381034259103*^9, 3.662381045364979*^9}, {
  3.700046171794359*^9, 3.700046205377877*^9}, {3.702610606182536*^9, 
  3.702610640085682*^9}},ExpressionUUID->"ce215825-9caa-4422-9bd0-\
bda7dd06d039"],

Cell["Differential operators act on the tensor as expected:", "Text",
 CellChangeTimes->{{3.630825347046504*^9, 
  3.630825376581356*^9}},ExpressionUUID->"25d8df9c-75e1-4b8e-8b07-\
1104ab64807e"],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"RG", "[", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.630763360373422*^9, 3.63076341048071*^9}, {
   3.6623811967655354`*^9, 3.662381204836235*^9}, {3.662719917782711*^9, 
   3.6627199439342737`*^9}, {3.662719977998567*^9, 3.6627199946542387`*^9}, {
   3.665987422284048*^9, 3.665987423228156*^9}, 3.665990489108589*^9, {
   3.702610676046013*^9, 
   3.7026106994453907`*^9}},ExpressionUUID->"27691efd-61b0-4c76-b738-\
456b53eafd65"],

Cell[BoxData[
 RowBox[{"?", "d"}]], "Input",
 CellChangeTimes->{{3.702611019625392*^9, 
  3.702611022634884*^9}},ExpressionUUID->"9f4b534b-b74f-433c-9b85-\
1755c535c6eb"],

Cell[BoxData[
 RowBox[{"Attributes", "[", "d", "]"}]], "Input",
 CellChangeTimes->{{3.702612622216588*^9, 
  3.7026126321181793`*^9}},ExpressionUUID->"f1ebc9a0-2a97-47d2-8966-\
c027be40445d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"RG", "[", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.630763360373422*^9, 3.63076341048071*^9}, {
   3.6623811967655354`*^9, 3.662381204836235*^9}, {3.662719917782711*^9, 
   3.6627199439342737`*^9}, {3.662719977998567*^9, 3.6627199946542387`*^9}, {
   3.665987422284048*^9, 3.665987423228156*^9}, 3.665990489108589*^9, {
   3.702610676046013*^9, 3.7026106994453907`*^9}, {3.70261074332668*^9, 
   3.702610788732839*^9}, {3.70261086356548*^9, 
   3.702610941723762*^9}},ExpressionUUID->"ce1bbfcb-b8be-467e-874e-\
d294d648adf3"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.700046335133506*^9, 
  3.7000463351359453`*^9}},ExpressionUUID->"fc95dcbe-224b-44a8-abb9-\
ba0ab0260fc9"]
}, Closed]],

Cell[CellGroupData[{

Cell["1.1.2. Inner Geometry of an Immersion", "Subsubsection",
 CellChangeTimes->{{3.662722586576187*^9, 3.6627226311025667`*^9}, {
   3.6627227137017097`*^9, 3.662722715389777*^9}, 
   3.702612820604147*^9},ExpressionUUID->"7ad4e423-5768-4761-8c2f-\
949c277a82d9"],

Cell[TextData[{
 "We remember now the ",
 StyleBox["inner geometry of an immersion smf, here a surface",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6308239607275624`*^9, 3.6308240073956456`*^9}, {
  3.662711889006709*^9, 3.662711892902277*^9}, {3.702701024990735*^9, 
  3.7027010332066593`*^9}},ExpressionUUID->"f5b02f9a-e2b1-41f1-9413-\
3fdd9fc36485"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"smf", "[", "u__", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"imm", "[", "f", "]"}], "[", "u", "]"}]}], ";", 
  RowBox[{"smf", "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.702700896937633*^9, 3.70270099879113*^9}, {
  3.7034918670130377`*^9, 3.703491871284143*^9}, {3.703682866587641*^9, 
  3.7036828915701427`*^9}},ExpressionUUID->"cafe88d7-6388-45d6-b402-\
b22d1310c3eb"],

Cell[BoxData[
 RowBox[{"?", "imm"}]], "Input",
 CellChangeTimes->{{3.6305051572461195`*^9, 3.6305051758257523`*^9}, {
   3.6305052800183353`*^9, 3.6305053376136365`*^9}, {3.6307569354085374`*^9, 
   3.630757161062934*^9}, {3.660314611834421*^9, 3.6603146347945423`*^9}, 
   3.660314791681642*^9, {3.660316345788452*^9, 3.660316427675856*^9}, {
   3.660316514091043*^9, 3.660316541362328*^9}, {3.660368628647705*^9, 
   3.6603686308949547`*^9}, {3.7027010536325207`*^9, 
   3.7027010549745502`*^9}},ExpressionUUID->"aa3b89f4-829a-42e0-b08d-\
ba137e078a6d"],

Cell["Now a hypersurface in the Euclidean 4-space:", "Text",
 CellChangeTimes->{{3.630824056369132*^9, 3.630824084856782*^9}, {
  3.7027011427977*^9, 
  3.702701144797402*^9}},ExpressionUUID->"d1ff09c1-5f79-44bf-a52c-\
70858a671e41"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "4"}], ";", " ", 
  RowBox[{"smf", "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7026481335612297`*^9, 3.7026481557853937`*^9}, {
  3.702650495574603*^9, 3.702650578189501*^9}, {3.70270120361377*^9, 
  3.7027012254683323`*^9}, {3.7034918938855658`*^9, 3.703491895531836*^9}, {
  3.703682954795163*^9, 
  3.703682979329212*^9}},ExpressionUUID->"d0156e66-b19c-492c-9f8a-\
2c3dccba5cb9"],

Cell[TextData[{
 "The differential of the immersion smf[var[x]]  at a point u has the matrix, \
 often named the ",
 StyleBox["Jacobian matrix",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.5947126263848686`*^9, 3.594712689952505*^9}, {
   3.594713832024828*^9, 3.594713852521*^9}, 3.5947142700248795`*^9, {
   3.6305051294312706`*^9, 3.6305051384480867`*^9}, {3.6614934122472267`*^9, 
   3.661493447526939*^9}},ExpressionUUID->"e206bc09-db05-4491-b4c5-\
3bc4a8046f30"],

Cell[BoxData[
 RowBox[{"?", "jac"}]], "Input",
 CellChangeTimes->{{3.660393118850107*^9, 
  3.66039312118512*^9}},ExpressionUUID->"0d314787-2fe0-46d5-bcc7-\
a62d8c4ccf02"],

Cell[BoxData[
 RowBox[{
  RowBox[{"jac", "[", "smf", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.702647672842111*^9, 3.702647701844449*^9}, 
   3.702647755190543*^9, {3.702649711085935*^9, 
   3.7026497609239197`*^9}},ExpressionUUID->"8e4292c8-3ad9-470e-affb-\
ad5cd9a534f0"],

Cell[BoxData[
 RowBox[{"MatrixRank", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.702649772616602*^9, 
  3.7026497848517036`*^9}},ExpressionUUID->"be4dfcbc-60b5-41c4-ace5-\
6e16d8c2403e"],

Cell[TextData[{
 "In particular, ",
 StyleBox["curves  surfaces, hypersurfaces (m=n-1), and submanifolds",
  FontColor->RGBColor[0, 0, 1]],
 " are supposed to be regular immersions; singular points have to be \
considered explicitly in any concrete case."
}], "Text",
 CellChangeTimes->{{3.660372080946056*^9, 3.660372086834036*^9}, {
   3.6603721208192*^9, 3.660372261720704*^9}, 3.665987487499354*^9, 
   3.7026135422151947`*^9},ExpressionUUID->"146421fd-67fa-40f2-bab7-\
c190959ce6fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"imm", "[", "fx", "]"}], "[", "t", "]"}]}], ";", 
  RowBox[{"\[Gamma]", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.702649839301723*^9, 3.702649865180142*^9}, {
   3.7027013476462927`*^9, 3.702701429920866*^9}, {3.702701526923913*^9, 
   3.7027015281613197`*^9}, 3.702701565715432*^9, {3.7034923976631947`*^9, 
   3.703492401591342*^9}},ExpressionUUID->"385d2460-2f7e-4153-8fb2-\
923d63aecf2b"],

Cell["The basis of the tangent spaces of the immersion f:", "Text",
 CellChangeTimes->{{3.6307568730084276`*^9, 3.63075690253928*^9}, {
   3.660455899109201*^9, 3.660455900205171*^9}, 
   3.6604583882883053`*^9},ExpressionUUID->"bbf0f0e4-af41-480a-8062-\
eb87173f5a2b"],

Cell[BoxData[
 RowBox[{"?", "tv"}]], "Input",
 CellChangeTimes->{{3.660457729697226*^9, 3.660457734213678*^9}, 
   3.660799775784038*^9},ExpressionUUID->"67b1f8a2-090e-4e29-bb1d-\
1cd6ab5dab79"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "smf", "]"}], "[", "1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6230366867896442`*^9, 3.6230367194405017`*^9}, {
   3.623036828375493*^9, 3.6230368288902936`*^9}, 3.6307573173284082`*^9, 
   3.660457780870804*^9, 3.660458233194749*^9, 3.6607998058077297`*^9, {
   3.702628034308531*^9, 3.702628050481855*^9}, {3.703683051714658*^9, 
   3.703683057896832*^9}},ExpressionUUID->"02054863-a9c7-49a9-a4cd-\
74e56a914b08"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"jac", "[", "smf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6627241463818483`*^9, 
  3.6627242142927017`*^9}},ExpressionUUID->"a6017ee3-2dba-44ef-815e-\
05d94296a29a"],

Cell[TextData[{
 "A ",
 StyleBox["tangent vector",
  FontColor->RGBColor[0, 0, 1]],
 " is a linear combination of the tangent basis vectors with the coordinates \
{s1,s2,...} \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "m"], TraditionalForm]],ExpressionUUID->
  "294ef6e2-3051-48ee-9890-80f27e83fe0a"],
 ". A ",
 StyleBox["tangent vector field",
  FontColor->RGBColor[0, 0, 1]],
 " is obtained if the coordinates are given as functions of the parameters :"
}], "Text",
 CellChangeTimes->{{3.594878538608076*^9, 3.5948785573125095`*^9}, {
   3.594886028196831*^9, 3.5948861049957657`*^9}, {3.5955967368876705`*^9, 
   3.5955967454471607`*^9}, {3.5959112371428165`*^9, 3.595911291181311*^9}, {
   3.5959113324745836`*^9, 3.595911408431117*^9}, {3.59591146137761*^9, 
   3.595911536491742*^9}, 
   3.5959116685147743`*^9},ExpressionUUID->"77397915-ef5d-42c0-9b7e-\
079246c1ad8b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.662724313921517*^9, 
  3.662724324075588*^9}},ExpressionUUID->"7759ac12-dafe-4fde-88d7-\
f769b74ab5fb"],

Cell[BoxData[
 RowBox[{"?", "vv"}]], "Input",
 CellChangeTimes->{{3.6307574580718555`*^9, 
  3.6307574596162577`*^9}},ExpressionUUID->"4b5ce838-e0ff-4021-99d5-\
1570b26da0fd"],

Cell[BoxData[
 RowBox[{"Context", "[", "vv", "]"}]], "Input",
 CellChangeTimes->{{3.660393269156455*^9, 
  3.660393276863934*^9}},ExpressionUUID->"0a77dad5-1cbe-490d-9d12-\
2f510c58e9ff"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5955963089271927`*^9, 3.595596389983829*^9}, {
   3.59559642496883*^9, 3.5955964744026575`*^9}, {3.59559657706553*^9, 
   3.595596688818921*^9}, {3.595596899914995*^9, 3.595596910010573*^9}, {
   3.5955969406903276`*^9, 3.5955969705550356`*^9}, {3.5955970131314707`*^9, 
   3.5955970157876225`*^9}, 3.6148618615003433`*^9, 3.6148635019056244`*^9, {
   3.630504867850011*^9, 3.6305049015772705`*^9}, {3.630757479022692*^9, 
   3.630757479989894*^9}, 
   3.635835093237258*^9},ExpressionUUID->"6b386ecf-4ac1-442b-bcd1-\
f4b846e700fd"],

Cell[TextData[{
 "Remark that this field is defined as a tangent vector field of the \
parameter manifold. Its image under the differential of the immersion is a \
vector function on  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "m"], TraditionalForm]],ExpressionUUID->
  "6d30cb96-1a7a-4aa8-81e6-9a52de179d3e"],
 " with values in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "n"], TraditionalForm]],ExpressionUUID->
  "6acbe03e-fb3b-4868-b2fa-67d4a3f73359"],
 ", e. g."
}], "Text",
 CellChangeTimes->{{3.5959117538781238`*^9, 3.5959119023591843`*^9}, {
   3.595911954619276*^9, 3.595911975195712*^9}, {3.5959121801956725`*^9, 
   3.595912259677812*^9}, 
   3.5960892898931456`*^9},ExpressionUUID->"bd66aff0-f532-4260-81b9-\
a8e102ba34af"],

Cell[BoxData[
 RowBox[{"?", "imm"}]], "Input",
 CellChangeTimes->{{3.703487323287805*^9, 3.703487348703669*^9}, 
   3.70348751152894*^9},ExpressionUUID->"6fd40046-d8ea-422e-964c-\
685c37b00be8"],

Cell[BoxData[
 RowBox[{"smf", "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.702563735357525*^9, 3.702563740446459*^9}, {
  3.7036762015068283`*^9, 
  3.703676214211041*^9}},ExpressionUUID->"6901e04f-c38f-490c-b2c4-\
3dea2f1e5ad4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "smf", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ".", 
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "\[Equal]", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tv", "[", "smf", "]"}], "[", "i", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "*", 
     RowBox[{
      RowBox[{"\[Xi]", "[", "i", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5959119820753245`*^9, 3.5959119820753245`*^9}, {
   3.595912052790249*^9, 3.5959121489800177`*^9}, {3.617779223413473*^9, 
   3.617779229123083*^9}, 3.63050957571988*^9, {3.6307576067089167`*^9, 
   3.630757748310365*^9}, 3.6607998797668324`*^9, {3.666882212934876*^9, 
   3.6668822377799177`*^9}, {3.7027106060602303`*^9, 3.7027106396992207`*^9}, 
   3.702710711322616*^9, 3.702710927825532*^9, {3.703676140850754*^9, 
   3.7036761462433653`*^9}, {3.703683170409889*^9, 
   3.7036831921286917`*^9}},ExpressionUUID->"32a1e55d-4ddd-43aa-a07e-\
8bc94ab0cb55"],

Cell[BoxData[
 RowBox[{"?", "tvf"}]], "Input",
 CellChangeTimes->{{3.630816825073808*^9, 
  3.6308168282874136`*^9}},ExpressionUUID->"23745606-4aeb-41ca-a7e4-\
85f88f1d9412"],

Cell[BoxData[
 RowBox[{"Context", "[", "tvf", "]"}]], "Input",
 CellChangeTimes->{{3.6604587976122932`*^9, 
  3.660458811197146*^9}},ExpressionUUID->"6ae4f03b-ad59-4202-b628-\
bd0de546e24f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tvf", "[", "smf", "]"}], "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.594879354302309*^9, 3.594879363319125*^9}, {
   3.5948795443418427`*^9, 3.5948795935287294`*^9}, 3.5955708665119724`*^9, 
   3.6177792687627525`*^9, {3.630757768528001*^9, 3.630757774674412*^9}, 
   3.701920454521132*^9, {3.702711053625379*^9, 
   3.702711067758841*^9}},ExpressionUUID->"f1e54b7d-bb59-494e-9ec1-\
cbe523eb2d04"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.7036763864332123`*^9, 
  3.703676395352195*^9}},ExpressionUUID->"04c379ef-5376-43bf-a7d9-\
84e39c3950c8"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "smf", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ".", 
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.660458649958427*^9, 3.66045866832789*^9}, {
   3.701920487339168*^9, 3.70192048926237*^9}, {3.701920526662264*^9, 
   3.701920556029625*^9}, 
   3.7019206265780277`*^9},ExpressionUUID->"c403d3b7-280b-46e5-a7d6-\
ca3d74f0d6e6"],

Cell[TextData[{
 "The ",
 StyleBox["Riemann  (or \[OpenCurlyDoubleQuote]inner\[CloseCurlyDoubleQuote]) \
metric",
  FontColor->RGBColor[0, 0, 1]],
 " induced by an immersion has the components"
}], "Text",
 CellChangeTimes->{{3.6230426337493*^9, 3.623042659582945*^9}, {
   3.6308206409497104`*^9, 3.630820647782522*^9}, {3.660458652566166*^9, 
   3.660458653311119*^9}, 3.66045873906921*^9, {3.66149447339819*^9, 
   3.661494501253891*^9}},ExpressionUUID->"4522dbf9-373a-4292-9fcf-\
62bf4dd9f953"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{"smf", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.703681773315209*^9, 
  3.7036818016510983`*^9}},ExpressionUUID->"8aea0237-6785-4209-8e25-\
5ce14f0eeaeb"],

Cell[BoxData[
 RowBox[{"?", "rgt"}]], "Input",
 CellChangeTimes->{{3.630820655176935*^9, 3.6308206597633433`*^9}, {
  3.662736006429047*^9, 
  3.662736011108021*^9}},ExpressionUUID->"5b28fd98-745a-44d8-b7a9-\
639ffbb6b6f0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgt", "[", "f_", "]"}], "[", "x__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "f", "]"}], "[", "i", "]"}], "[", "x", "]"}], ".", 
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "f", "]"}], "[", "j", "]"}], "[", "x", 
    "]"}]}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.662723750220274*^9, 3.6627239032316008`*^9}, {
   3.66489244359061*^9, 3.664892450854856*^9}, 3.6650349940817213`*^9, {
   3.665987986054358*^9, 3.665987995774414*^9}, {3.703683305725967*^9, 
   3.703683374708537*^9}},ExpressionUUID->"bc67166f-e3e0-4cf0-8a6b-\
7fcc4820e1a0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rgt", "[", "smf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"1", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.7036834009726353`*^9, 3.7036834058763447`*^9}, 
   3.7036835604446173`*^9, 
   3.7036836681964073`*^9},ExpressionUUID->"68c04a67-0968-4ac9-924a-\
b0e7286d700e"],

Cell[TextData[{
 "They are the dot-products of the ",
 StyleBox["tangent basis vectors ",
  FontColor->RGBColor[0, 0, 1]],
 "tv[i],e.g."
}], "Text",
 CellChangeTimes->{{3.6308246770836267`*^9, 3.630824711532488*^9}, {
  3.662723178017277*^9, 3.6627231812656393`*^9}, {3.665988100244851*^9, 
  3.665988120540641*^9}},ExpressionUUID->"a831f033-5d6e-4a01-9220-\
cef37e537211"],

Cell[BoxData[
 RowBox[{"?", "tv"}]], "Input",
 CellChangeTimes->{{3.665988127970791*^9, 
  3.665988129260922*^9}},ExpressionUUID->"f2875e0b-a841-4889-84a0-\
450cf43c083f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tv", "[", "smf", "]"}], "[", "2", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.665988147284556*^9, 3.665988169068123*^9}, {
  3.702711128333661*^9, 3.702711134917444*^9}, {3.703683992496155*^9, 
  3.703683999880615*^9}},ExpressionUUID->"9dfd27ce-21cd-426a-991f-\
b2aef4cf25db"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgt", "[", "smf", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", "[", 
   RowBox[{"1", ",", "3"}], "]"}], "==", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "smf", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ".", 
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "smf", "]"}], "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.630824404140745*^9, 3.630824491277899*^9}, {
  3.630824564622628*^9, 3.6308246465971727`*^9}, {3.660799946853548*^9, 
  3.660799951275361*^9}, {3.662723135746262*^9, 3.6627231410983677`*^9}, {
  3.6627231779733343`*^9, 3.662723354688654*^9}, {3.662723487896606*^9, 
  3.662723651511215*^9}, {3.66273606762294*^9, 3.662736101239558*^9}, {
  3.662736131742271*^9, 3.6627361361895027`*^9}, {3.6648924688863487`*^9, 
  3.664892477006996*^9}, {3.70271115159013*^9, 3.702711182014229*^9}, {
  3.7036840373671827`*^9, 
  3.703684062631255*^9}},ExpressionUUID->"bcb88286-88be-4d51-99fd-\
0dcae68e3f0c"],

Cell["Their matrix is", "Text",
 CellChangeTimes->{{3.662731975187942*^9, 
  3.662731985699815*^9}},ExpressionUUID->"38cf99cc-d25e-4a45-b4b4-\
c40cc029581e"],

Cell[BoxData[
 RowBox[{"?", "RG"}]], "Input",
 CellChangeTimes->{{3.70271122445443*^9, 
  3.702711225940691*^9}},ExpressionUUID->"0f2c9937-7c05-4e43-84df-\
6b64ba012046"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RG", "[", 
   RowBox[{"rgt", "[", "smf", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.662723664806488*^9, 3.662723688165183*^9}, 
   3.662723739334103*^9, {3.662723929303399*^9, 3.662723934319206*^9}, {
   3.662736261607462*^9, 3.6627362637033787`*^9}, {3.702711278181141*^9, 
   3.702711334556693*^9}},ExpressionUUID->"803c6313-5df1-4911-90b7-\
296952e64b99"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{"RGT", "[", "smf", "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6627239915868464`*^9, 3.66272402460618*^9}, {
  3.662736297174048*^9, 
  3.662736307221085*^9}},ExpressionUUID->"5fe97dde-3db7-4aa8-b4ec-\
2f4c16ccd70e"],

Cell["\<\
where RGT is defined  independently of RG in the package riemannv2.m.\
\>", "Text",
 CellChangeTimes->{{3.662724634917611*^9, 3.6627246886348867`*^9}, {
  3.662732068931945*^9, 3.662732076908033*^9}, {3.662736325548092*^9, 
  3.662736363034615*^9}, {3.702722899739026*^9, 
  3.7027229301207323`*^9}},ExpressionUUID->"a6184300-a4f4-42d6-8d1f-\
664b644f5012"],

Cell[BoxData[
 RowBox[{"?", "RGT"}]], "Input",
 CellChangeTimes->{{3.662736374576033*^9, 
  3.662736379465961*^9}},ExpressionUUID->"5256a8d4-84cd-4b84-9d2e-\
f1d8d99b1e73"],

Cell[TextData[{
 "The Euclidean differential geometry of immersions, in particular curves and \
surfaces, are treated in ",
 ButtonBox["A. Gray",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray"],
 " with many  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " programming. See also the notebooks in [",
 ButtonBox["EDG",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "]."
}], "Text",
 CellChangeTimes->{{3.6659885146248283`*^9, 3.6659887242071753`*^9}, {
   3.6659894474032907`*^9, 3.665989447405712*^9}, {3.665989490853484*^9, 
   3.6659894908557663`*^9}, 
   3.702723436020916*^9},ExpressionUUID->"f169b392-4076-46ce-aa05-\
fa6ce17c1390"]
}, Closed]],

Cell[CellGroupData[{

Cell["1.1.3. The Inverse Metric Matrix", "Subsubsection",
 CellChangeTimes->{{3.662724946986801*^9, 3.662724990065234*^9}, 
   3.662725423179758*^9, {3.662736404128048*^9, 3.662736406383914*^9}, 
   3.662802426806786*^9, 3.7026128264683247`*^9, {3.7027235011229973`*^9, 
   3.7027235040672626`*^9}, {3.702723584522563*^9, 3.70272358531453*^9}, 
   3.702725334553776*^9},ExpressionUUID->"ce2ba340-decb-4d39-9e9d-\
0b5560dbb4d7"],

Cell["\<\
By definition, the metric tensor of a pseudo--Riemannian space is symmetric; \
it is supposed to be non-degenerate too:\
\>", "Text",
 CellChangeTimes->{{3.623043670498921*^9, 3.623043745472653*^9}, {
  3.6230437784979105`*^9, 3.6230438393848176`*^9}, {3.6308211373549824`*^9, 
  3.6308211567302165`*^9}, {3.6308211877430706`*^9, 3.630821242031166*^9}, {
  3.630821283589639*^9, 3.630821324586511*^9}, {3.6308262282256603`*^9, 
  3.6308262328920684`*^9}},ExpressionUUID->"f6de9f42-8517-4134-bce8-\
dac8a1dae453"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Det", "[", 
   SubscriptBox[
    RowBox[{"{", 
     RowBox[{
      RowBox[{"g", "[", 
       RowBox[{"x", ",", "y", ",", "..."}], "]"}], "[", 
      RowBox[{"i", ",", "j"}], "]"}], "}"}], 
    RowBox[{"(", 
     RowBox[{"n", ",", "n"}], ")"}]], "]"}], "\[NotEqual]", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.622958278637192*^9, 3.622958388258584*^9}, 
   3.6229584262290506`*^9, {3.6229584893935614`*^9, 3.6229585660208964`*^9}, {
   3.6229585960821495`*^9, 3.6229587444696093`*^9}, {3.6229588030009127`*^9, 
   3.6229588872722607`*^9}, 3.622958944617961*^9, {3.622959280408551*^9, 
   3.6229592920461717`*^9}, {3.622959323137026*^9, 3.622959326600232*^9}, {
   3.62295950157014*^9, 3.62295966118962*^9}, 3.6308213461613493`*^9, {
   3.630821429871096*^9, 3.6308214453775234`*^9}, {3.630821478075181*^9, 
   3.630821529274471*^9}, {3.630821560006525*^9, 3.630821571051344*^9}, {
   3.630821623982237*^9, 3.630821655416292*^9}, 3.6308217107651896`*^9, {
   3.630821751512461*^9, 3.6308217761917048`*^9}, 3.6308218410410185`*^9, {
   3.630821880462288*^9, 3.630821928557172*^9}, {3.630821989475279*^9, 
   3.6308220403625684`*^9}, 3.6308264977677355`*^9, 
   3.6623816696158323`*^9},ExpressionUUID->"e8e0de0e-20b6-4342-94c0-\
d8d5da6919a3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{"Det", "[", 
   RowBox[{
    RowBox[{"RG", "[", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.630825948197564*^9, 3.6308259903080387`*^9}, {
   3.6627322262836857`*^9, 3.662732248996043*^9}, {3.6648927462743998`*^9, 
   3.664892751514172*^9}, {3.665989691345845*^9, 3.665989740705776*^9}, 
   3.6659906408520613`*^9, {3.7027236240344152`*^9, 
   3.702723647210383*^9}},ExpressionUUID->"46a35793-9a35-4854-a53c-\
ff95bd70434a"],

Cell[TextData[{
 "Clearly, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " may not confirm that the determinant isn\[CloseCurlyQuote]t zero:"
}], "Text",
 CellChangeTimes->{{3.6308265393146086`*^9, 
  3.630826642800591*^9}},ExpressionUUID->"e9a73472-edae-42f0-9de6-\
b1ba7820e48c"],

Cell[BoxData[
 RowBox[{"TrueQ", "[", 
  RowBox[{"%", "\[NotEqual]", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.630826023680497*^9, 3.630826044025735*^9}, 
   3.63082608631281*^9},ExpressionUUID->"108ff74f-0c6a-49a7-9671-\
8d99c5d124d7"],

Cell[BoxData[
 RowBox[{"TrueQ", "[", 
  RowBox[{"%%", "\[Equal]", " ", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.630826023680497*^9, 3.630826044025735*^9}, 
   3.63082608631281*^9, {3.630826271350136*^9, 
   3.6308263141158113`*^9}},ExpressionUUID->"43ca271b-6ded-4583-83d7-\
57447b518d91"],

Cell["In spite of", "Text",
 CellChangeTimes->{{3.702723746416685*^9, 
  3.702723753256733*^9}},ExpressionUUID->"fad61af5-c418-47d0-bd6a-\
45bbb40cb0ad"],

Cell[BoxData[
 RowBox[{"MatrixRank", "[", 
  RowBox[{
   RowBox[{"RG", "[", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.702723762265695*^9, 
  3.702723809616588*^9}},ExpressionUUID->"fcdff5f6-4e85-46c1-adbc-\
e229f2f6d817"],

Cell[TextData[{
 "By default, calculating inverse general matrices, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " assumes that this condition is satisfied. We use the built-in function"
}], "Text",
 CellChangeTimes->{{3.63092127653027*^9, 3.6309214057296968`*^9}, {
   3.6309354022614803`*^9, 3.630935419608711*^9}, 3.662790734002557*^9, {
   3.662790774794335*^9, 
   3.6627908194096527`*^9}},ExpressionUUID->"a0643915-b27e-4e11-81e6-\
b8aacdacd9e2"],

Cell[BoxData[
 RowBox[{"?", "Inverse"}]], "Input",
 CellChangeTimes->{{3.630827199054372*^9, 3.6308272030011787`*^9}, {
  3.6627906878027153`*^9, 
  3.6627906951309147`*^9}},ExpressionUUID->"02c8881b-29e2-4b42-b131-\
3f70f08c4652"],

Cell["The inverse matrix field is", "Text",
 CellChangeTimes->{{3.662794126477804*^9, 
  3.662794144349546*^9}},ExpressionUUID->"fee4fcda-171e-4101-9cd0-\
3e3dec357cee"],

Cell[BoxData[
 RowBox[{"?", "INV"}]], "Input",
 CellChangeTimes->{{3.662733636200851*^9, 3.6627336403479137`*^9}, 
   3.662790862401332*^9, 3.662790954209506*^9, 
   3.662794204070492*^9},ExpressionUUID->"3657b87b-9609-49ac-97df-\
a5b195ecbe5a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"INV", "[", "xx_", "]"}], "[", "u__", "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"m", "=", " ", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "u", "}"}], "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"MatrixQ", "[", 
        RowBox[{"xx", "[", "u", "]"}], "]"}], ",", " ", 
       RowBox[{"Return", "[", 
        RowBox[{"Inverse", "[", 
         RowBox[{"xx", "[", "u", "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"MatrixQ", "[", 
        RowBox[{"xx", "[", 
         RowBox[{"{", "u", "}"}], "]"}], "]"}], ",", " ", 
       RowBox[{"Return", "[", 
        RowBox[{"Inverse", "[", 
         RowBox[{"xx", "[", 
          RowBox[{"{", "u", "}"}], "]"}], "]"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", " ", 
     RowBox[{"Print", "[", "\"\<Wrong argument!\>\"", "]"}], ";", 
     RowBox[{"Return", "[", "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6627969437144957`*^9, 3.662797085143388*^9}, {
   3.662797134743637*^9, 3.662797147031002*^9}, {3.662797186847034*^9, 
   3.662797191878537*^9}, 3.6627972598944283`*^9, {3.662797385964918*^9, 
   3.662797475715966*^9}, {3.6627975700043793`*^9, 3.662797661962076*^9}, {
   3.662797766065325*^9, 3.6627977671701*^9}, {3.662797799394493*^9, 
   3.6627978190972013`*^9}, {3.662797878792225*^9, 3.662797912952322*^9}, {
   3.662798161942498*^9, 3.662798195717787*^9}, 3.662802248769289*^9, 
   3.66280359144662*^9, 
   3.663484085437138*^9},ExpressionUUID->"27e81bff-73b8-4883-a0d7-\
b97cfea30adc"],

Cell[BoxData[
 RowBox[{"?", "inv"}]], "Input",
 CellChangeTimes->{{3.6627973156343117`*^9, 3.6627973723803864`*^9}, {
  3.66280193806787*^9, 
  3.6628019412912083`*^9}},ExpressionUUID->"c099d38d-b16b-419d-a88c-\
cb61d487f72f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"inv", "[", "xx_", "]"}], "[", "u__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"INV", "[", "xx", "]"}], "[", "u", "]"}], "\[LeftDoubleBracket]", 
   RowBox[{"i", ",", "j"}], "\[RightDoubleBracket]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663483961308814*^9, 3.663483968004847*^9}, {
  3.664892812385599*^9, 
  3.664892818634704*^9}},ExpressionUUID->"570e6dc9-daa1-4efc-87db-\
20959866ed0a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"INV", "[", 
   RowBox[{"RG", "[", "]"}], "]"}], "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.662795361161602*^9, 3.6627953894330807`*^9}, {
  3.6627958140854053`*^9, 3.662795829060547*^9}, {3.6627958919247913`*^9, 
  3.662795896819894*^9}, {3.66279592992419*^9, 3.662795954219022*^9}, {
  3.6627971115117893`*^9, 3.662797113903213*^9}, {3.6628017851729183`*^9, 
  3.662801786628729*^9}, {3.665989943080216*^9, 3.665989944464182*^9}, {
  3.665990679603364*^9, 3.665990686387149*^9}, {3.7027241798063517`*^9, 
  3.702724181148677*^9}},ExpressionUUID->"e43c35ba-1ae7-41dc-8601-\
0a9213c78574"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"inv", "[", 
    RowBox[{"RG", "[", "]"}], "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.662802025760325*^9, 3.6628020999299603`*^9}, {
   3.663484123763261*^9, 3.663484132067708*^9}, {3.664892872586927*^9, 
   3.664892883513769*^9}, 3.702724314299243*^9, {3.702724543689899*^9, 
   3.7027245869609327`*^9}},ExpressionUUID->"3e9de83a-dc4c-4ca7-8c78-\
c97d8fb81d6a"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{"%%", "[", 
   RowBox[{"[", 
    RowBox[{"2", ",", "1"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.70272461523597*^9, 
  3.702724627512156*^9}},ExpressionUUID->"adfb14fb-50cb-4af6-bfcb-\
aafaf7da2a78"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", 
  RowBox[{
   RowBox[{"INV", "[", 
    RowBox[{"RG", "[", "]"}], "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663484403452083*^9, 3.66348441229021*^9}, {
   3.6634844617769136`*^9, 3.6634844671914186`*^9}, 3.702724770764492*^9, {
   3.702724808960339*^9, 
   3.702724816134856*^9}},ExpressionUUID->"4f613020-0357-456b-93ca-\
fb5da2602645"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "4"}], ";", 
  RowBox[{
   RowBox[{"INV", "[", 
    RowBox[{"RG", "[", "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.630827227497222*^9, 3.630827283868121*^9}, {
   3.6309365508290977`*^9, 3.6309365558367066`*^9}, {3.6310144485713835`*^9, 
   3.631014450224986*^9}, 3.6310146610657578`*^9, {3.631014787533581*^9, 
   3.63101479285319*^9}, 3.6310222374676237`*^9, 3.63102277460797*^9, {
   3.6310796898920603`*^9, 3.631079690890462*^9}, 3.6606490902325706`*^9, {
   3.660810172417576*^9, 3.660810173424844*^9}, {3.660810206593014*^9, 
   3.66081023935983*^9}, {3.660810328968485*^9, 3.660810329263278*^9}, {
   3.662794239541382*^9, 3.6627942807654753`*^9}, {3.662794328773402*^9, 
   3.662794330020705*^9}, {3.662794429204718*^9, 3.6627944349320707`*^9}, {
   3.662796614653281*^9, 3.662796619084957*^9}, {3.662797215782696*^9, 
   3.6627972223663883`*^9}, {3.662801832733282*^9, 3.6628018392598658`*^9}, 
   3.6634844995916243`*^9, {3.664892992479993*^9, 3.664892998247711*^9}, 
   3.7027248609475527`*^9, {3.7027251422530947`*^9, 
   3.702725143139555*^9}},ExpressionUUID->"5596b70b-52c6-46e5-aa9e-\
374165faf5b3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RG", "[", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.660648405290381*^9, 3.6606484396856823`*^9}, {
   3.660648562645915*^9, 3.6606485662846203`*^9}, 3.660649111392948*^9, {
   3.6627324226031723`*^9, 3.66273244101195*^9}, {3.66280187861204*^9, 
   3.662801882915564*^9}, 3.663484508038726*^9, {3.702725165083256*^9, 
   3.702725177075427*^9}},ExpressionUUID->"ad9d171c-4218-4d62-9e1e-\
0797fb2f45d0"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%%", ".", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.660810271481678*^9, 
  3.6608103075195513`*^9}},ExpressionUUID->"af117bd5-32c0-4b3a-bdd5-\
ffca210e4536"],

Cell["OK.", "Text",
 CellChangeTimes->{{3.6310234734802017`*^9, 3.631023532031104*^9}, {
   3.631023614986451*^9, 3.6310236556335225`*^9}, 3.631023704448381*^9, {
   3.6606493414941063`*^9, 3.660649352294189*^9}, {3.660718559472521*^9, 
   3.660718580648272*^9}, {3.660718612008214*^9, 3.660718673231522*^9}, 
   3.66238061987713*^9, {3.66238179569448*^9, 
   3.662381797998248*^9}},ExpressionUUID->"b693736d-a499-4ce7-bc55-\
ef21c7f6f1a4"]
}, Closed]],

Cell[CellGroupData[{

Cell["1.1.4. The Euclidean Metric", "Subsubsection",
 CellChangeTimes->{{3.662722586576187*^9, 3.6627226311025667`*^9}, {
   3.662722688838019*^9, 3.662722708709766*^9}, 3.662725007288598*^9, 
   3.66272545202726*^9, 
   3.702612831700321*^9},ExpressionUUID->"cdb18208-8a2a-4b7e-aef2-\
cfb13a396b07"],

Cell[TextData[{
 StyleBox["The Euclidean space",
  FontColor->RGBColor[0, 0, 1]],
 " is a special case. In orthonormal Cartesian coordinates its metric tensor  \
is"
}], "Text",
 CellChangeTimes->{{3.6230423310861635`*^9, 3.623042360929016*^9}, {
  3.6308254452782774`*^9, 3.630825516968404*^9}, {3.660463273006692*^9, 
  3.660463295526745*^9}, {3.662731616548667*^9, 
  3.662731616750328*^9}},ExpressionUUID->"8b700843-b43b-454e-a793-\
7a45aaef8d50"],

Cell[BoxData[
 RowBox[{"?", "EUCL"}]], "Input",
 CellChangeTimes->{{3.6604635726395884`*^9, 3.6604635754604397`*^9}, 
   3.7027254265529547`*^9},ExpressionUUID->"a7435dfe-a71c-44aa-92ed-\
d33540671757"],

Cell[BoxData[
 RowBox[{"?", "eucl"}]], "Input",
 CellChangeTimes->{{3.6627219648595047`*^9, 3.6627219682362556`*^9}, 
   3.702725482864271*^9},ExpressionUUID->"f7d19208-e8e9-4f94-b9bf-\
5c2db154d06d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"eucl", "[", "u__", "]"}], " ", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
  RowBox[{"KroneckerDelta", "[", 
   RowBox[{"i", ",", "j"}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6229266469151435`*^9, 3.6229267397197065`*^9}, {
  3.6648930616632347`*^9, 3.664893067559772*^9}, {3.702725497960224*^9, 
  3.702725499008564*^9}},ExpressionUUID->"3670dd9b-9e21-43a6-a546-\
4835d47e5048"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EUCL", "[", "x__", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"RG", "[", "eucl", "]"}], "[", "x", "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.622926857343913*^9, 3.6229269349540496`*^9}, {
   3.6229269828773336`*^9, 3.6229270283514137`*^9}, {3.6307666749268436`*^9, 
   3.6307667092469044`*^9}, {3.6627203542751102`*^9, 3.662720439457878*^9}, {
   3.6627204892336082`*^9, 3.662720512169374*^9}, {3.662720665904872*^9, 
   3.662720677991761*^9}, {3.6627207983271713`*^9, 3.662720813159121*^9}, 
   3.662721206515339*^9, {3.662721497024788*^9, 3.662721512944412*^9}, {
   3.6659910022965927`*^9, 3.66599101266437*^9}, 3.702725436422512*^9, {
   3.7027255396798553`*^9, 3.702725564135434*^9}, {3.7027255967362967`*^9, 
   3.702725624695191*^9}},ExpressionUUID->"02fc6bb2-f4b3-4c69-82cc-\
307e0380e289"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EUCL", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "==", " ", 
  RowBox[{"Table", "[", " ", 
   RowBox[{
    RowBox[{"KroneckerDelta", "[", 
     RowBox[{"i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.622926857343913*^9, 3.6229269349540496`*^9}, {
   3.6229269828773336`*^9, 3.6229270283514137`*^9}, {3.6307666749268436`*^9, 
   3.6307667092469044`*^9}, {3.660463160265767*^9, 3.6604631903679743`*^9}, 
   3.660463316486843*^9, {3.662721630104032*^9, 3.662721646983173*^9}, {
   3.662721680559663*^9, 3.662721733710672*^9}, 
   3.7027254421814127`*^9},ExpressionUUID->"99d118ec-8411-4966-9822-\
98e4b5156f97"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EUCL", "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6229270384602313`*^9, 3.6229270615170717`*^9}, 
   3.7027254429015636`*^9},ExpressionUUID->"406e17ad-8824-4428-9763-\
d3cbda822c41"],

Cell[BoxData[
 RowBox[{"EUCL", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.6627221773879557`*^9, 
  3.7027254447008963`*^9},ExpressionUUID->"5fb42b0f-fbf2-43e3-8211-\
18a34bfce625"],

Cell["\<\
EUCL is also obtained as the inner metric of the identity map:\
\>", "Text",
 CellChangeTimes->{{3.631257758971185*^9, 3.631257810439676*^9}, 
   3.702725445745851*^9},ExpressionUUID->"9f3485e5-1478-429e-9dfe-\
951d9ec54bff"],

Cell[BoxData[
 RowBox[{"?", "identity"}]], "Input",
 CellChangeTimes->{{3.63125750675014*^9, 
  3.631257520138964*^9}},ExpressionUUID->"37b4c92e-d740-4bbc-b102-\
d4fecf9c3293"],

Cell[BoxData[
 RowBox[{
  RowBox[{"EUCL", "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"RGT", "[", "identity", "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6312575492994156`*^9, 3.631257580889471*^9}, 
   3.662380618408697*^9, 3.662381352889509*^9, 
   3.7027254495415163`*^9},ExpressionUUID->"79c89278-01a6-4826-8a21-\
99910c8eb9d8"],

Cell["", "Item",
 CellChangeTimes->{
  3.7084069263046494`*^9},ExpressionUUID->"1eff9810-5f5c-4b70-a93a-\
df618f0fc71e"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["1.2.  Riemannian Spaces of Constant Positive Curvature",
  FontColor->RGBColor[
   0.7803921568627451, 0.4196078431372549, 0.1607843137254902]],
 " "
}], "Subsection",
 CellChangeTimes->{{3.631429845815478*^9, 3.631429867505516*^9}, {
   3.6314299150040007`*^9, 3.6314299185608063`*^9}, {3.6314300543674464`*^9, 
   3.6314300791462903`*^9}, {3.6314594169713783`*^9, 3.6314594187965813`*^9}, 
   3.6315184366017904`*^9, {3.660835698584939*^9, 3.660835708200655*^9}, 
   3.660903125960135*^9, {3.6614389782109823`*^9, 3.6614389833142023`*^9}, {
   3.661493120913967*^9, 3.6614931306899357`*^9}, 3.661496492596365*^9, {
   3.665991141734613*^9, 3.665991144030686*^9}, 3.666102122142708*^9, 
   3.702725342122693*^9, 3.702781545436769*^9},
 FontColor->RGBColor[
  0.6, 0.4, 0.2],ExpressionUUID->"01a311f9-759b-4a7f-b4c1-d49abfcb635e"],

Cell[BoxData[
 RowBox[{"?", "spher"}]], "Input",
 CellChangeTimes->{{3.661496734717622*^9, 
  3.661496738769987*^9}},ExpressionUUID->"79def7db-7997-44c9-a61b-\
d532b701243c"],

Cell["It is defined by", "Text",
 CellChangeTimes->{{3.661497678017153*^9, 3.661497693256983*^9}, {
  3.702725836500907*^9, 
  3.702725840364702*^9}},ExpressionUUID->"3d01d160-4626-4649-abe7-\
3ed263ce6c98"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"spher", "[", "r_", "]"}], "[", "s__", "]"}], ":=", 
  RowBox[{"r", "*", 
   RowBox[{"unit", "[", "s", "]"}]}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6311986977784986`*^9, 3.631198842000752*^9}, {
   3.6311988844484262`*^9, 3.6311989731189823`*^9}, {3.6311993825573015`*^9, 
   3.631199430464986*^9}, {3.631199469855055*^9, 3.631199508589923*^9}, {
   3.6312518651667967`*^9, 3.6312518861060333`*^9}, {3.6312549088355618`*^9, 
   3.6312549219747853`*^9}, {3.63151113872659*^9, 3.6315111866810746`*^9}, {
   3.63151215182357*^9, 3.6315121659727945`*^9}, 3.6315122251280985`*^9, {
   3.6582045618142023`*^9, 
   3.65820458494978*^9}},ExpressionUUID->"b999c22f-0a4e-40ca-9554-\
cc6f8f2d47fb"],

Cell[BoxData[
 RowBox[{"?", "unit"}]], "Input",
 CellChangeTimes->{{3.6659946711374197`*^9, 
  3.665994674112852*^9}},ExpressionUUID->"f4bb5388-dc87-46b9-94e1-\
6e959059863d"],

Cell[BoxData[
 RowBox[{"Context", "[", "unit", "]"}]], "Input",
 CellChangeTimes->{{3.6659947049557333`*^9, 
  3.6659947194404707`*^9}},ExpressionUUID->"cf909c2e-1a6a-45f0-9ce4-\
dc6305049b6a"],

Cell[TextData[{
 StyleBox["Example 1. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["Spheres, n= 2",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.631175984160801*^9, 3.6311760545793247`*^9}, {
   3.6312613809189653`*^9, 3.6312613842271714`*^9}, {3.6314299395896435`*^9, 
   3.6314299493456607`*^9}, 3.6314300446018295`*^9, 3.660746161156151*^9, {
   3.6614977228088207`*^9, 3.661497760184639*^9}, {3.6649496174780207`*^9, 
   3.664949619926691*^9}, 3.700128341121084*^9, 
   3.700128845028618*^9},ExpressionUUID->"19f454b7-57a9-451a-ad15-\
b29531279e0a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.661497779652981*^9, 
  3.661497807424518*^9}},ExpressionUUID->"d64fedfa-5c0f-49ed-b226-\
afa7ffa26702"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"RGT", "[", 
    RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.622927287639469*^9, 3.6229273133639145`*^9}, {
   3.6229273813800335`*^9, 3.622927399866066*^9}, 3.6315122565153537`*^9, {
   3.6315138705097885`*^9, 3.6315139005866413`*^9}, 
   3.661496758258459*^9},ExpressionUUID->"be1b28f0-366c-4e64-8ed1-\
bcfdf9a0d5fc"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGsphere", "[", "r_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SuperscriptBox["r", "2"], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}], ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", 
      SuperscriptBox["r", "2"]}], "}"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.631176152344696*^9, 3.6311762012039824`*^9}, {
  3.631197563388906*^9, 
  3.6311975704401183`*^9}},ExpressionUUID->"02a876b5-26d1-4fd1-99d0-\
1cefeff5c6ce"],

Cell[BoxData[
 RowBox[{"?", "INV"}]], "Input",
 CellChangeTimes->{{3.664893454391873*^9, 
  3.664893457355276*^9}},ExpressionUUID->"aa72aad6-0815-4339-9359-\
c7a1522e6043"],

Cell[BoxData[
 RowBox[{
  RowBox[{"INV", "[", 
   RowBox[{"RGsphere", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6311762079275937`*^9, 3.631176226366826*^9}, {
  3.631176296145749*^9, 
  3.6311763028381605`*^9}},ExpressionUUID->"706c4a2e-bec0-4e8c-86c4-\
664f67cfd1d1"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ".", 
   RowBox[{
    RowBox[{"RGsphere", "[", "r", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6311976785351086`*^9, 
  3.631197726817193*^9}},ExpressionUUID->"d696339e-16aa-4f9d-a088-\
12a349f3b02a"],

Cell[BoxData[
 RowBox[{"?", "rgt"}]], "Input",
 CellChangeTimes->{{3.662806817293948*^9, 3.662806818668047*^9}, 
   3.6628070197156897`*^9, 
   3.702726010821005*^9},ExpressionUUID->"fca73f55-1cc0-488f-bda0-\
f541ec48ad4d"],

Cell[BoxData[
 RowBox[{"?", "RGT"}]], "Input",
 CellChangeTimes->{{3.664896218795576*^9, 
  3.6648962207074738`*^9}},ExpressionUUID->"8e6351bf-285d-4e95-85ac-\
4ff654d2b231"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"gsph", "[", "r_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"rgt", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"i", ",", "j"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.631198154257944*^9, 3.631198312379822*^9}, 
   3.6315122565153537`*^9, 3.6315139810827827`*^9, 3.661497906424035*^9, {
   3.6628067821567907`*^9, 3.6628067828200083`*^9}, 3.6628068491885023`*^9, {
   3.6648933176282043`*^9, 3.6648933362361507`*^9}, 3.664893746816319*^9, {
   3.664948807011825*^9, 3.664948815540078*^9}, {3.702726039803815*^9, 
   3.702726067084903*^9}, {3.702731160960322*^9, 
   3.702731168503661*^9}},ExpressionUUID->"fad5713d-9dfd-42df-9150-\
a0fb9a2c73f7"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"rgt", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"1", ",", "1"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.631198144117926*^9, 3.631198144819927*^9}, {
   3.631251752035596*^9, 3.631251758109408*^9}, 3.6315122565153537`*^9, 
   3.6315139915972013`*^9, 3.661497961983163*^9, {3.6614980006635303`*^9, 
   3.66149800976689*^9}, 3.662806798188717*^9, 3.662806879836306*^9, {
   3.664893686669293*^9, 3.664893690041888*^9}, 3.6648938228576117`*^9, {
   3.6648938574491787`*^9, 3.664893879639924*^9}, {3.702726091628722*^9, 
   3.702726116724122*^9}},ExpressionUUID->"2d5bec5d-457c-4d17-88b9-\
41973afdd130"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"gsph", "[", "r", "]"}], "[", 
    RowBox[{"2", ",", "2"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.664893924536333*^9, 3.664893985887274*^9}, {
  3.664948846667739*^9, 
  3.664948853020574*^9}},ExpressionUUID->"780dcf04-f410-47ee-8cbe-\
b1e30d6aa6f6"],

Cell[BoxData[
 RowBox[{"?", "dVol"}]], "Input",
 CellChangeTimes->{{3.664894610435976*^9, 3.664894612049397*^9}, {
  3.702730673188849*^9, 
  3.7027306746994963`*^9}},ExpressionUUID->"33ecf0d5-aeee-4486-a3db-\
a8d4926bfb86"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dVol", "[", 
     RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.622960836152484*^9, 3.6229608430008955`*^9}, {
  3.6229608760729537`*^9, 3.6229609012669983`*^9}, {3.622960943839473*^9, 
  3.6229609822467403`*^9}, {3.622961776896536*^9, 3.6229617806093426`*^9}, {
  3.622974686081276*^9, 3.62297469413089*^9}, {3.622975029391079*^9, 
  3.622975037362693*^9}, {3.6311979149223237`*^9, 3.631197915858325*^9}, {
  3.6311980194893074`*^9, 3.631198052904566*^9}, {3.6311983875563536`*^9, 
  3.631198389880758*^9}, {3.6312543472857723`*^9, 3.6312543642772026`*^9}, {
  3.664894311037779*^9, 
  3.66489432385995*^9}},ExpressionUUID->"5799f2e3-a6dd-4984-a461-\
4c3a8521e5ca"],

Cell[BoxData[
 RowBox[{"?", "dV"}]], "Input",
 CellChangeTimes->{{3.702780033398532*^9, 3.702780052020226*^9}, 
   3.702780102147643*^9},ExpressionUUID->"ed785b61-4611-4e8c-b517-\
8510f14a6a70"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dV", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.702780033398532*^9, 3.702780052020226*^9}, 
   3.702780102147643*^9},ExpressionUUID->"e5f3358b-cf41-4cc2-a180-\
d5cc7ad4deca"],

Cell[TextData[{
 "OK. Now we consider  the \n\n",
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". n-spheres in arbitrary dimension n:"
}], "Text",
 CellChangeTimes->{{3.6312548199718056`*^9, 3.6312548921279325`*^9}, {
   3.660746064661182*^9, 3.660746089532773*^9}, 3.6607462332676277`*^9, {
   3.664949604028614*^9, 
   3.664949605492688*^9}},ExpressionUUID->"ba343d45-e598-47b4-b620-\
4a8d27cfb17b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6311990324926863`*^9, 3.6311990642855425`*^9}, {
   3.63119934196603*^9, 3.6311993529016495`*^9}, 3.6312545641335545`*^9, 
   3.6312546028256226`*^9, 3.631511205073507*^9, {3.6315112362891617`*^9, 
   3.631511254026393*^9}, 3.6315122563905535`*^9, {3.658204671149723*^9, 
   3.658204692701001*^9}},ExpressionUUID->"0c5ea8e9-683d-40ac-b9c5-\
fae6fdb23ad3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}]], "Input",
 CellChangeTimes->{{3.6311990324926863`*^9, 3.6311990642855425`*^9}, {
   3.63119934196603*^9, 3.6311993529016495`*^9}, 3.6312545641335545`*^9, 
   3.6312546028256226`*^9, 3.631511205073507*^9, {3.6315112362891617`*^9, 
   3.631511254026393*^9}, 3.631511861179459*^9, 3.631511898900326*^9, 
   3.6315122564061537`*^9},ExpressionUUID->"25b6dbf2-b768-4ebc-931b-\
60c1b43401dc"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"RGT", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}], ",", 
     RowBox[{"w", ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6311996962114525`*^9, 3.6311997151186857`*^9}, {
   3.6311997451799383`*^9, 3.6311997515759497`*^9}, {3.6312546773403544`*^9, 
   3.6312546969515886`*^9}, 3.6315113272061214`*^9, 
   3.631512256421754*^9},ExpressionUUID->"ca544bf8-76be-4114-ba20-\
fc104977c5a6"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"INV", "[", 
     RowBox[{"RGT", "[", 
      RowBox[{"spher", "[", "r", "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}], ",", 
     RowBox[{"w", ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6311998328988924`*^9, 3.63119991117983*^9}, 
   3.6315113272061214`*^9, 
   3.631512256421754*^9},ExpressionUUID->"8dffc159-8d45-400d-9fee-\
fd3501125097"],

Cell[BoxData[
 RowBox[{"%", ".", "%%"}]], "Input",
 CellChangeTimes->{{3.6312547338320537`*^9, 
  3.6312547380908613`*^9}},ExpressionUUID->"e510a47a-7662-4673-bf7d-\
ef5d2ca51d2c"],

Cell["\<\
The Riemannian metric of the inner geometry of the n-sphere is\
\>", "Text",
 CellChangeTimes->{{3.6315151529960413`*^9, 
  3.631515178002885*^9}},ExpressionUUID->"5029235b-58c6-4fd3-9c34-\
35e04aa4c436"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGpos", "[", "r_", "]"}], "[", "s__", "]"}], " ", ":=", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"RGT", "[", 
      RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "s", "]"}], ",", " ", 
    RowBox[{
     RowBox[{"{", "s", "}"}], "\[Element]", "Reals"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.631254981040489*^9, 3.631255019406957*^9}, {
   3.6312550518746147`*^9, 3.6312551740816298`*^9}, {3.6312553221594906`*^9, 
   3.6312553494283385`*^9}, {3.631255724200599*^9, 3.63125577549749*^9}, 
   3.6312558092443495`*^9, {3.6315110409456186`*^9, 3.6315110409456186`*^9}, 
   3.6315113938494387`*^9, 3.631511491006409*^9, 3.631511589348982*^9, 
   3.6315122564373536`*^9, 3.6315151280671973`*^9, {3.662807218808858*^9, 
   3.662807231304575*^9}, 3.664949664204461*^9, {3.666003599497604*^9, 
   3.666003607089921*^9}, {3.666100962128537*^9, 3.666100973192459*^9}, {
   3.700129214930127*^9, 
   3.70012923921763*^9}},ExpressionUUID->"6e46d93e-76bd-44ae-ab26-\
57486277e304"],

Cell[BoxData[
 RowBox[{"?", "RGpos"}]], "Input",
 CellChangeTimes->{{3.6377590011163387`*^9, 3.637759008370352*^9}, {
   3.63775909842071*^9, 3.637759121508751*^9}, {3.6377591774462504`*^9, 
   3.637759324545909*^9}, {3.660746467834179*^9, 3.660746472193928*^9}, {
   3.660746524049155*^9, 3.6607465421453533`*^9}, {3.660746719456024*^9, 
   3.660746763415327*^9}, 3.6615097779270353`*^9, 
   3.661509809622416*^9},ExpressionUUID->"e28295eb-4fc0-4c2c-b919-\
044be0087def"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGpos", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.66610103422266*^9},ExpressionUUID->"a1a89896-09f7-41b1-8385-f14b7793ee20"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGpos", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.631255380086393*^9, 3.6312554082016425`*^9}, {
   3.6312558389976015`*^9, 3.6312558406980047`*^9}, 3.631511040976819*^9, 
   3.6315115712529497`*^9},ExpressionUUID->"171a7027-6043-450f-88de-\
6d145fd8a4cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGpos", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{
  3.6315116836667476`*^9},ExpressionUUID->"142a328c-2682-4052-8b45-\
7c93c7e9a36d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGpos", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{
  3.6315116836667476`*^9, {3.6315126429280324`*^9, 
   3.631512651757648*^9}},ExpressionUUID->"49ae794e-22e2-4d00-b79b-\
7edfdb479973"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"INV", "[", 
    RowBox[{"RGpos", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631255429000479*^9, 3.63125552576085*^9}, {
   3.6312558526632257`*^9, 3.631255855459631*^9}, 3.631511040976819*^9, 
   3.6315115763853593`*^9},ExpressionUUID->"e78d32a2-20c6-4ea7-9b0f-\
1872b9e9e710"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"INV", "[", 
    RowBox[{"RGpos", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631255429000479*^9, 3.63125552576085*^9}, {
   3.6312558526632257`*^9, 3.631255855459631*^9}, 3.631511040976819*^9, 
   3.6315115763853593`*^9, {3.63151260751597*^9, 
   3.631512619715192*^9}},ExpressionUUID->"3430de38-afad-4a04-aade-\
9e8cdc97b5a9"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ".", 
   RowBox[{
    RowBox[{"RGpos", "[", "r", "]"}], "[", 
    RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6608872328873568`*^9, 
  3.660887316609786*^9}},ExpressionUUID->"d4502f79-951e-4d70-8a87-\
70a0bbaed379"],

Cell["\<\
\[FilledSquare]
From the shape of the matrices RGpos[r][s1,s2,...,sn], n = 1,...,5, we deduce \
the general definition of the n-dimensional Riemannian metric of positive \
constant curvature being independent of the immersion of the n-sphere:\
\>", "Text",
 CellChangeTimes->{{3.6315154129352565`*^9, 3.6315156217432265`*^9}, {
  3.700129666894167*^9, 
  3.700129679191833*^9}},ExpressionUUID->"640fe3ae-d063-4a89-8b6f-\
6f9dcd06dac3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGposk", "[", "r", "]"}], "[", "s__", "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "s", "}"}], "]"}]}], ",", "i", ",", "j", ",", "f", ",", 
      "diag"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "i_", "]"}], "[", "t__", "]"}], " ", ":=", 
      RowBox[{
       RowBox[{"r", "^", "2"}], "*", " ", 
       RowBox[{"Product", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Cos", "[", 
           RowBox[{
            RowBox[{"{", "t", "}"}], "[", 
            RowBox[{"[", "j", "]"}], "]"}], "]"}], "^", "2"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", 
           RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"diag", " ", "[", "t__", "]"}], ":=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"f", "[", "i", "]"}], "[", "t", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}]}], ";", 
     RowBox[{"DiagonalMatrix", "[", 
      RowBox[{"diag", "[", "s", "]"}], "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.631512836851973*^9, 3.6315129322929406`*^9}, {
  3.631512979061823*^9, 3.631513110866454*^9}, {3.6315131412241077`*^9, 
  3.6315133407796583`*^9}, {3.631513418576995*^9, 3.6315134221962013`*^9}, {
  3.631513504751546*^9, 
  3.631513530226391*^9}},ExpressionUUID->"58cef54b-7b77-4d86-bbfe-\
7f5eecdf8976"],

Cell[BoxData[
 RowBox[{"?", "RGposk"}]], "Input",
 CellChangeTimes->{{3.6312604742911687`*^9, 3.6312604833431845`*^9}, {
   3.6312605215652523`*^9, 3.631260789969125*^9}, {3.6312609123925405`*^9, 
   3.6312609162613473`*^9}, {3.631511040961219*^9, 3.6315110578404484`*^9}, {
   3.6315115149212513`*^9, 3.631511545856106*^9}, {3.6315152158173513`*^9, 
   3.63151523764179*^9}, {3.660887532535921*^9, 3.660887541215858*^9}, 
   3.666004126381751*^9, 3.666005059739861*^9, {3.6660672538787117`*^9, 
   3.666067257222973*^9}},ExpressionUUID->"d0b621ed-022e-4f36-b4b7-\
20eea4b3de01"],

Cell["Its components are", "Text",
 CellChangeTimes->{{3.631429049128605*^9, 3.6314290553218155`*^9}, {
   3.631458012399311*^9, 3.631458015332116*^9}, 
   3.700129463455138*^9},ExpressionUUID->"047b97cb-4f30-4caf-99e1-\
6b42e2e58403"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgposk", "[", "r_", "]"}], "[", 
     RowBox[{"i_", ",", "j_"}], "]"}], "[", "s__", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"RGposk", "[", "r", "]"}], "[", "s", "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"i", ",", "j"}], "]"}], "]"}]}], "\[LineSeparator]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6313687848306084`*^9, 3.631368925414857*^9}, {
   3.631369001749791*^9, 3.6313690727329164`*^9}, 3.631425771561412*^9, 
   3.631425880153202*^9, {3.6314574851339846`*^9, 3.6314574974268064`*^9}, {
   3.6315172138892345`*^9, 3.631517226480457*^9}, {3.662807340055869*^9, 
   3.662807348623396*^9}, 3.664949741308488*^9},
 CellTags->"rgpos",ExpressionUUID->"538ba085-94cb-4c7c-abf1-67e28fe5526b"],

Cell[BoxData[
 RowBox[{"?", "rgposk"}]], "Input",
 CellChangeTimes->{{3.631457925039157*^9, 3.631457931637969*^9}, {
  3.631458037733755*^9, 3.631458201066042*^9}, {3.631517238712878*^9, 
  3.631517266328927*^9}, {3.660887735806385*^9, 
  3.660887752470068*^9}},ExpressionUUID->"e2f75c0c-d089-4c14-9634-\
4baa0f93ba1e"],

Cell[BoxData[
 RowBox[{"Context", "[", "RGposk", "]"}]], "Input",
 CellChangeTimes->{{3.666067294435055*^9, 
  3.6660673119339647`*^9}},ExpressionUUID->"b8e909d0-67a1-4369-9bef-\
f2830686c8b0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGposk", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631513375614519*^9, 3.631513398156559*^9}, {
   3.6315135556232357`*^9, 3.6315135723152647`*^9}, 
   3.66610143723575*^9},ExpressionUUID->"3a127bfd-304b-424f-9c83-\
ee009862f710"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "\[Equal]", 
   RowBox[{
    RowBox[{"RGpos", "[", "r", "]"}], "[", 
    RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6315135906140966`*^9, 3.6315136454481936`*^9}, 
   3.666101478715372*^9},ExpressionUUID->"15ffae13-b35e-4969-a60a-\
12dadbd19275"],

Cell["\<\
The singularities of the parameter representation spher[r][s1,...,sn]  of the \
n-sphere (not of the n-sphere, of course) are the zeros of the determinant \
Det[RGposk[r][s1,s2,...].\
\>", "Text",
 CellChangeTimes->{{3.660892638129854*^9, 3.660892800520492*^9}, {
   3.660892832640153*^9, 3.660892902527891*^9}, {3.660893327924013*^9, 
   3.660893411395424*^9}, 3.664949765390922*^9, {3.700129810732679*^9, 
   3.700129832733285*^9}},ExpressionUUID->"e47159ae-b164-42bf-b70e-\
f17cc1534d96"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{
      RowBox[{"RGposk", "[", "r", "]"}], "[", 
      RowBox[{
       RowBox[{"vec", "[", 
        RowBox[{"s", ",", "i"}], "]"}], "/.", 
       RowBox[{"List", " ", "\[Rule]", "Sequence"}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "5"}], "}"}]}], "]"}], "//", "TableForm"}]], "Input",
 CellChangeTimes->{{3.66089231558295*^9, 
  3.660892455932033*^9}},ExpressionUUID->"e167f2f6-4cb9-4d1f-9f26-\
49d90d391fd4"],

Cell["For example, if s2 = \[PlusMinus] \[Pi]/2:", "Text",
 CellChangeTimes->{{3.6608934631067247`*^9, 
  3.6608935440099087`*^9}},ExpressionUUID->"ca7c967d-3e1f-4e53-8b2e-\
10fffadc57f4"],

Cell[BoxData[
 RowBox[{"Det", "[", 
  RowBox[{
   RowBox[{"RGposk", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", 
    RowBox[{"Pi", "/", "2"}], ",", "s3", ",", "s4", ",", "s5"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.660893085758749*^9, 
  3.660893108534243*^9}},ExpressionUUID->"57369869-d626-46e1-be37-\
eb2f37888356"],

Cell["the whole subsphere ", "Text",
 CellChangeTimes->{{3.6608936237249804`*^9, 
  3.660893676680599*^9}},ExpressionUUID->"7ec08ec5-4b2e-4a95-8192-\
19be04368f37"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"s1", ",", 
   RowBox[{"Pi", "/", "2"}], ",", "s3", ",", "s4", ",", "s5"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6608932248910027`*^9, 
  3.660893257077379*^9}},ExpressionUUID->"e186045b-02bf-4eae-9be2-\
c7159c78825e"],

Cell["consists of singular points.", "Text",
 CellChangeTimes->{{3.660893702673493*^9, 
  3.6608937358654413`*^9}},ExpressionUUID->"757b68a9-0250-4521-ab6c-\
39cb0ba53415"],

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Det", "[", 
     RowBox[{
      RowBox[{"RGposk", "[", "r", "]"}], "[", 
      RowBox[{
       RowBox[{"vec", "[", 
        RowBox[{"s", ",", "5"}], "]"}], "/.", 
       RowBox[{"List", "\[Rule]", "Sequence"}]}], "]"}], "]"}], "\[Equal]", 
    "0"}], ",", 
   RowBox[{"vec", "[", 
    RowBox[{"s", ",", "5"}], "]"}], ",", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.660889408008333*^9, 3.66088942009277*^9}, {
   3.660889475847412*^9, 3.66088955089384*^9}, 3.660889600141138*^9, {
   3.660890433822813*^9, 3.660890465581674*^9}, {3.660890526500784*^9, 
   3.66089053254097*^9}, 3.660891115897476*^9, {3.660891182967925*^9, 
   3.660891196670559*^9}, {3.660891334846384*^9, 
   3.660891345069766*^9}},ExpressionUUID->"735a2445-eb41-4759-89ae-\
b3b6673b06d8"],

Cell["\<\
A parameter representation of the pointed n-sphere (the n-spher outside one \
point, the \[OpenCurlyDoubleQuote]North pole\[CloseCurlyDoubleQuote]), is \
given by the inverse stereographic projection\
\>", "Text",
 CellChangeTimes->{
  3.660900408707889*^9, {3.660900440768724*^9, 3.6609005725024147`*^9}, 
   3.666101675377636*^9, 
   3.7027808487293253`*^9},ExpressionUUID->"ae2eb508-22e6-47ff-83b9-\
49564b41146c"],

Cell[BoxData[
 RowBox[{"?", "invsterproj"}]], "Input",
 CellChangeTimes->{
  3.631517724511935*^9, {3.660894922518408*^9, 3.66089497887875*^9}, {
   3.660895069309619*^9, 
   3.6608950910692596`*^9}},ExpressionUUID->"003a3a4a-feb9-4d09-998a-\
f02b9bb685eb"],

Cell[BoxData[
 RowBox[{"?", "sph"}]], "Input",
 CellChangeTimes->{{3.6609013291728973`*^9, 
  3.660901331280061*^9}},ExpressionUUID->"739d1c10-f5a3-4dfe-ad50-\
7e33ed816c3c"],

Cell[BoxData[
 RowBox[{"Context", "[", "sph", "]"}]], "Input",
 CellChangeTimes->{{3.666101746495762*^9, 
  3.666101757137445*^9}},ExpressionUUID->"605ba5de-9491-44e9-829f-\
8bdcf18eed04"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sph", "[", "r_", "]"}], "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{"r", "*", 
   RowBox[{"invsterproj", "[", 
    RowBox[{"{", "x", "}"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.660895121196206*^9, 3.6608951753723383`*^9}, {
  3.660895238692503*^9, 3.6608952467877617`*^9}, {3.660895281315287*^9, 
  3.660895333490958*^9}, {3.66089537217869*^9, 3.660895376810398*^9}, {
  3.6615109869595757`*^9, 3.66151099559977*^9}, {3.664949810595433*^9, 
  3.664949828549158*^9}, {3.6649541990229607`*^9, 
  3.664954208214629*^9}},ExpressionUUID->"122a60f1-012a-4764-a4f9-\
3c48f6037e03"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sph", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.660895339804571*^9, 
  3.6608953576667213`*^9}},ExpressionUUID->"4d4d8e3b-eb4b-4a36-9407-\
82dc59782678"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ".", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.666101834385894*^9, 
  3.6661018481126623`*^9}},ExpressionUUID->"b2d68867-08e3-4b4a-adf3-\
fb9f1736650e"],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sph", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "30"}], ",", "30"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "30"}], ",", "30"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}], ",", 
   RowBox[{"PlotPoints", "\[Rule]", "200"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.660895404593508*^9, 3.660895486361759*^9}, {
   3.660895629280941*^9, 3.660895637471694*^9}, {3.660895865046629*^9, 
   3.66089587781408*^9}, {3.660895916910223*^9, 3.660895927253372*^9}, 
   3.6608959883817663`*^9},ExpressionUUID->"ef02f9d5-a6c5-49d9-9425-\
a0fb5832acd1"],

Cell[TextData[{
 StyleBox["Remark",
  FontColor->RGBColor[0, 0, 1]],
 ". Without the options the unit sphere would appear very distorted by \
numerical reasons, try it. "
}], "Text",
 CellChangeTimes->{{3.660896051548037*^9, 3.660896133291047*^9}, 
   3.660903006120965*^9},ExpressionUUID->"78214f63-ff6e-41ab-8d8a-\
002a310f3ab8"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sph", "[", "r", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ".", 
   RowBox[{
    RowBox[{"sph", "[", "r", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6608955276520567`*^9, 
  3.66089559301637*^9}},ExpressionUUID->"14a83bc8-fe6d-47e2-a479-\
dc482ac8e88e"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Together", "[", 
   RowBox[{
    RowBox[{"RGT", "[", 
     RowBox[{"sph", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.660896173659894*^9, 3.660896175322733*^9}, {
  3.660896212556209*^9, 
  3.660896309730152*^9}},ExpressionUUID->"9e14719d-aaf2-4717-b7a2-\
c4ae414ff45e"],

Cell[TextData[{
 "The stereographic map and its inverse are ",
 StyleBox["conformal: ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["that means, they preserve angles. \nI",
  FontColor->GrayLevel[0]],
 "ts Riemannian metric is defined by"
}], "Text",
 CellChangeTimes->{{3.660896696254084*^9, 3.660896707478022*^9}, {
   3.6608968526609983`*^9, 3.660896929476244*^9}, {3.666102453435514*^9, 
   3.666102480371296*^9}, {3.666102517474781*^9, 3.66610257207423*^9}, 
   3.6661026180660152`*^9},ExpressionUUID->"5468ad8f-8651-40ab-bd0a-\
bbaa002eef4a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGcnfpos", "[", "r_", "]"}], "[", "x__", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"2", "*", "r"}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{"1", "+", " ", 
         RowBox[{
          RowBox[{"{", "x", "}"}], ".", 
          RowBox[{"{", "x", "}"}]}]}], ")"}]}], ")"}], "^", "2"}], ")"}], "*", 
   RowBox[{"EUCL", "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.660896941329919*^9, 3.660897077099*^9}, {
   3.660897112898778*^9, 3.660897159714901*^9}, {3.660897274929454*^9, 
   3.6608972789456*^9}, {3.660897330808964*^9, 3.660897349064599*^9}, 
   3.661511174720913*^9, 3.661512658877324*^9, {3.664954269166235*^9, 
   3.664954279637805*^9}, 
   3.7027254541252337`*^9},ExpressionUUID->"f0d84094-63ea-46ac-b2a7-\
7d91efdfea5b"],

Cell[TextData[{
 "The conformity of the map x \[Element]",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["E", "n"], " "}], TraditionalForm]],ExpressionUUID->
  "4c49e9b7-fd06-4e55-b67a-246fe5e64eae"],
 "\[RightTeeArrow]sph[r][x] \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["S", "n"], TraditionalForm]],ExpressionUUID->
  "10ab776f-f7a0-4e91-bb65-27622ee34e92"],
 " is evident."
}], "Text",
 CellChangeTimes->{{3.6661028786106653`*^9, 3.66610293267673*^9}, {
  3.666102994325816*^9, 
  3.6661031690333*^9}},ExpressionUUID->"f6c1c216-45b0-415e-97e8-7e002887a59c"],

Cell[BoxData[
 RowBox[{"?", "rgcnfpos"}]], "Input",
 CellChangeTimes->{{3.6313687848306084`*^9, 3.631368925414857*^9}, {
   3.631369001749791*^9, 3.6313690727329164`*^9}, 3.631425771561412*^9, 
   3.631425880153202*^9, {3.6314574851339846`*^9, 3.6314574974268064`*^9}, {
   3.6315172138892345`*^9, 3.631517226480457*^9}, {3.660901507366622*^9, 
   3.660901571325904*^9}, {3.661511276689864*^9, 3.6615112926821404`*^9}, {
   3.6615118680054293`*^9, 3.6615119012458487`*^9}},
 CellTags->"rgpos",ExpressionUUID->"df75de80-ddb4-4456-a9ac-fdbae91eba86"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGcnfpos", "[", "r", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6608973558789186`*^9, 
  3.660897390896431*^9}},ExpressionUUID->"2cc6bc13-68fa-4d2b-bce9-\
47f14bc5b0f5"],

Cell["Obviously, all points of this Riemannian space are regular:", "Text",
 CellChangeTimes->{{3.6609028933538218`*^9, 
  3.6609029496974993`*^9}},ExpressionUUID->"5f7b1ee8-ba06-4583-8a40-\
e2547ad17773"],

Cell[BoxData[
 RowBox[{"Det", "[", 
  RowBox[{
   RowBox[{"RGcnfpos", "[", "r", "]"}], "[", 
   RowBox[{"x1", ",", "x2", ",", "x3", ",", "x4"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.660902593193179*^9, 3.660902685004321*^9}, 
   3.660902740587763*^9, {3.660902837627763*^9, 
   3.660902842674938*^9}},ExpressionUUID->"8f8e8981-482b-46f6-901d-\
5ca88efc1303"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1.3.  Riemannian Spaces of Constant Negative \
Curvature ",
 FontColor->RGBColor[
  0.7803921568627451, 0.4235294117647059, 0.1607843137254902]]], "Subsection",
 CellChangeTimes->{{3.6705670180859737`*^9, 3.6705670467170343`*^9}, {
   3.700130233485709*^9, 3.700130235813326*^9}, 
   3.7001993710322247`*^9},ExpressionUUID->"e1f1d7ac-4fd3-496e-b154-\
d2377bcfea2f"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1.3.1. Spheres of imaginary Radius \
(n-Pseudo-Spheres).",
 FontColor->RGBColor[
  0.7803921568627451, 0.4196078431372549, 
   0.1607843137254902]]], "Subsubsection",
 CellChangeTimes->{{3.631429845815478*^9, 3.631429867505516*^9}, {
   3.6314299150040007`*^9, 3.6314299185608063`*^9}, {3.6314300543674464`*^9, 
   3.6314300791462903`*^9}, {3.6314301360097914`*^9, 
   3.6314301448706064`*^9}, {3.6349994459477444`*^9, 3.6349994573397646`*^9}, 
   3.660903132224381*^9, {3.6612376173451557`*^9, 3.661237618288887*^9}, {
   3.6612409097459717`*^9, 3.66124100167321*^9}, 3.702781560573512*^9, 
   3.7027870461216373`*^9},ExpressionUUID->"978b4fea-2cc3-4702-85c0-\
717f04191d88"],

Cell[TextData[{
 "We consider the pseudo-Euclidean model of the n-dimensional hyperbolic \
space, see e.g.  [",
 ButtonBox["OSIII",
  BaseStyle->"Hyperlink",
  ButtonData->"OSIII"],
 "], Section 2.6. To this aim some tools of the ",
 StyleBox["pseudo-Euclidean geometry",
  FontColor->RGBColor[0, 0, 1]],
 " are needed:"
}], "Text",
 CellChangeTimes->{{3.6312863043442974`*^9, 3.631286343283966*^9}, {
   3.6312863813208327`*^9, 3.6312864303109193`*^9}, {3.63128657259717*^9, 
   3.631286627983268*^9}, {3.6312866671589365`*^9, 3.6312866671589365`*^9}, {
   3.631286727896844*^9, 3.6312868278340206`*^9}, 3.6314190681828375`*^9, 
   3.6314302802560453`*^9, {3.6701266422508373`*^9, 3.6701266427061863`*^9}, {
   3.6701268304733047`*^9, 3.6701268311928387`*^9}, {3.670126970024008*^9, 
   3.670127073028738*^9}},ExpressionUUID->"c15f0098-2c3d-4531-8ece-\
8a5b2679b6da"],

Cell[BoxData[
 RowBox[{"?", "neuvecv2`*"}]], "Input",
 CellChangeTimes->{{3.631338393812256*^9, 3.631338412922289*^9}, {
  3.63776115628654*^9, 
  3.637761157863143*^9}},ExpressionUUID->"653f5140-44c7-4577-9b67-\
8e67950d1c71"],

Cell[TextData[{
 "First we consider the 2-dimensional ",
 StyleBox["sphere of imaginary radius",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6314261690033092`*^9, 
  3.631426238532632*^9}},ExpressionUUID->"27b51963-d5e0-4f6a-b9ae-\
7f2776b0007a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{"vec", "[", "x", "]"}], ",", 
    RowBox[{"vec", "[", "x", "]"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{"-", 
   RowBox[{"r", "^", "2"}]}]}]], "Input",
 CellChangeTimes->{{3.631338440081937*^9, 3.6313384646675797`*^9}, {
  3.6314262715890894`*^9, 
  3.631426278203501*^9}},ExpressionUUID->"01e0011c-aa2a-4d23-abfc-\
c7c58e3f74f3"],

Cell[BoxData[
 RowBox[{"ContourPlot3D", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{
     RowBox[{"pssp", "[", 
      RowBox[{
       RowBox[{"vec", "[", "x", "]"}], ",", 
       RowBox[{"vec", "[", "x", "]"}]}], "]"}], "\[Equal]", " ", 
     RowBox[{"-", "1"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "[", "3", "]"}], ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6314263427720146`*^9, 3.6314263820528836`*^9}, {
   3.631426562670001*^9, 3.6314265993300657`*^9}, {3.6314266596241713`*^9, 
   3.631426737577508*^9}, 3.702781837860363*^9, 3.702781880995481*^9, {
   3.702782221136787*^9, 
   3.702782257704645*^9}},ExpressionUUID->"ae15a7ca-9627-42c2-be07-\
e99b7cbe1918"],

Cell["\<\
By symmetry reason it suffices to consider one sheet of the two-sheeted \
hyperboloid, say the positive (right) sheet. We give parameter \
representations for all dimensions n. We set\
\>", "Text",
 CellChangeTimes->{{3.6314270988905287`*^9, 3.6314271415540047`*^9}, {
   3.6314271760103006`*^9, 3.6314272983773165`*^9}, {3.6314273312691746`*^9, 
   3.6314273442639976`*^9}, {3.631430406390669*^9, 3.6314304260059032`*^9}, 
   3.637759952697815*^9, {3.661237797991201*^9, 3.661237804023423*^9}, {
   3.661238222859433*^9, 
   3.661238224092305*^9}},ExpressionUUID->"bbf64666-1533-4bcd-9ce2-\
85e34935c486"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hypsp", "[", "r_", "]"}], "[", "s__", "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "s", "}"}], "]"}]}], ",", "t"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"n", "\[Equal]", "1"}], ",", " ", 
       RowBox[{"Return", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"r", " ", 
           RowBox[{"Cosh", "[", "s", "]"}]}], ",", 
          RowBox[{"r", " ", 
           RowBox[{"Sinh", "[", "s", "]"}]}]}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"t", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", "s", "}"}], "[", 
         RowBox[{"[", "i", "]"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "2", ",", "n"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"r", "*", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Cosh", "[", 
          RowBox[{
           RowBox[{"{", "s", "}"}], "[", 
           RowBox[{"[", "1", "]"}], "]"}], "]"}], "}"}], ",", 
        RowBox[{"Evaluate", "[", 
         RowBox[{
          RowBox[{"Sinh", "[", 
           RowBox[{
            RowBox[{"{", "s", "}"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}], "]"}], "*", 
          RowBox[{"unit", "[", "t", "]"}]}], "]"}]}], "]"}]}]}]}], 
   "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6313418041810455`*^9, 3.6313419815065565`*^9}, {
   3.6313420979763613`*^9, 3.631342101564368*^9}, {3.6313421514844556`*^9, 
   3.631342160906872*^9}, {3.631342252666233*^9, 3.631342476230226*^9}, {
   3.631342531501123*^9, 3.631342563527979*^9}, {3.631342823627236*^9, 
   3.6313428743585253`*^9}, {3.6313429794247093`*^9, 
   3.6313430038075523`*^9}, {3.6313430514656363`*^9, 3.6313432810512395`*^9}, 
   3.6313434214982862`*^9, 3.6313434637899604`*^9, {3.631343501807227*^9, 
   3.6313435122748456`*^9}, {3.6313436977747717`*^9, 
   3.6313437115651956`*^9}, {3.6313440517081933`*^9, 3.631344119942713*^9}, {
   3.6313442028880587`*^9, 3.631344217458484*^9}, {3.631344251497744*^9, 
   3.6313443097170463`*^9}, {3.6313443405739*^9, 3.63134435723473*^9}, 
   3.6313446285816064`*^9, {3.631344876419242*^9, 3.631344884749656*^9}, 
   3.631344920411319*^9, {3.6313603526751766`*^9, 3.6313603548591805`*^9}, {
   3.6313603853612337`*^9, 3.6313604229145*^9}, {3.6313604677257795`*^9, 
   3.6313604741841908`*^9}, {3.631360548709324*^9, 3.6313606235974555`*^9}, {
   3.6313607114136133`*^9, 3.631360722068432*^9}, {3.6313607678927126`*^9, 
   3.6313607781419306`*^9}, {3.631361004147132*^9, 3.6313610225435643`*^9}, {
   3.631420330100254*^9, 3.631420338711469*^9}, {3.6314204099567947`*^9, 
   3.6314204164620056`*^9}, 3.660747464201707*^9, {3.660979581948326*^9, 
   3.660979591820073*^9}, {3.660980054736224*^9, 3.660980086399876*^9}, 
   3.661237996639988*^9},ExpressionUUID->"44235e6e-2ee7-455e-b368-\
0a7ec30510e5"],

Cell[BoxData[
 RowBox[{"?", "hypsp"}]], "Input",
 CellChangeTimes->{{3.63145875213701*^9, 3.6314588358779573`*^9}, {
   3.6314589067256813`*^9, 3.6314589600933757`*^9}, {3.631458995302637*^9, 
   3.6314590911648054`*^9}, 3.631459129150872*^9, {3.661266219602331*^9, 
   3.6612662263779707`*^9}},ExpressionUUID->"5d3c57cf-16a6-490a-bb1f-\
8c7550a0b78f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hypsp", "[", "r", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.661237705866515*^9, 3.661237730536274*^9}, 
   3.661238035039316*^9, {3.661238319987391*^9, 3.661238323050275*^9}, {
   3.702784069736805*^9, 
   3.702784073064762*^9}},ExpressionUUID->"5b441746-a001-48b9-a3fd-\
666bfae576a7"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{"%", ",", "%"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.66123833046618*^9, 
  3.661238364578416*^9}},ExpressionUUID->"758c8b06-bd15-493d-a35c-\
c2c58974862a"],

Cell[TextData[{
 "The radius vector is ",
 StyleBox["timelike",
  FontColor->RGBColor[0, 0, 1]],
 ", and the tangent vectors are ",
 StyleBox["spacelike",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.6314305183896666`*^9, 
  3.6314305841647835`*^9}},ExpressionUUID->"e7313dbe-8cf0-48bb-b9dc-\
32fa4dd96a48"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypsp", "[", "r", "]"}], "]"}], "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypsp", "[", "r", "]"}], "]"}], "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6612385086735773`*^9, 3.6612385186850863`*^9}, {
  3.661238590751791*^9, 3.661238606192148*^9}, {3.7027841193318768`*^9, 
  3.7027841570251427`*^9}, {3.703684147454886*^9, 
  3.703684197534827*^9}},ExpressionUUID->"23f6d40a-df40-41a4-b341-\
3c5d20f12065"],

Cell[TextData[{
 "and ",
 StyleBox["pseudo - orthogonal",
  FontColor->RGBColor[0, 0, 1]],
 " to the radius vector:"
}], "Text",
 CellChangeTimes->{{3.666162080597527*^9, 3.666162118564332*^9}, 
   3.666162198723897*^9},ExpressionUUID->"3651f5c8-931f-4f27-8181-\
0e1fa9ecf6c6"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hypsp", "[", "r", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypsp", "[", "r", "]"}], "]"}], "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6612385086735773`*^9, 3.6612385186850863`*^9}, {
  3.661238590751791*^9, 3.661238624367569*^9}, {3.66616203888363*^9, 
  3.66616204917139*^9}, {3.702784180600964*^9, 3.7027841869925003`*^9}, {
  3.703684228038506*^9, 
  3.703684234126152*^9}},ExpressionUUID->"0c1f8ec5-3f8d-4e63-bdcf-\
4115fcbd5a4d"],

Cell["\<\
Mind:  (u, v)\[RightArrow] (0 , v)  is a singular point of this parameter \
representation:\
\>", "Text",
 CellChangeTimes->{{3.661238841629221*^9, 3.6612388769807243`*^9}, {
   3.661238913573875*^9, 3.661238978044232*^9}, 3.6612392644491167`*^9, {
   3.661239309928797*^9, 3.6612393294407988`*^9}, {3.661239368704365*^9, 
   3.66123937719228*^9}, {3.661239573182459*^9, 3.661239592446467*^9}, {
   3.664954546065744*^9, 3.664954560077887*^9}, {3.666162244764612*^9, 
   3.6661622679808817`*^9}, {3.6661624418879642`*^9, 
   3.6661624492148247`*^9}, {3.7001981649369297`*^9, 3.700198212973588*^9}, {
   3.702784227543614*^9, 
   3.702784228743434*^9}},ExpressionUUID->"ccc3c961-e0d8-4843-9711-\
57eb7ce04ab5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"hypsp", "[", "r", "]"}], "[", 
  RowBox[{"0", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.664954575157889*^9, 
  3.6649545961606503`*^9}},ExpressionUUID->"cafe0ba5-f35b-4f74-99e0-\
4e6e9a3d01e4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"hypsp", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",ExpressionUUID->"9d02c2df-0a40-\
41ec-b892-baef8302ed9f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"hypsp", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "/.", " ", 
  RowBox[{"u", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.666162317009905*^9, 3.6661623536702414`*^9}, {
  3.666162384375803*^9, 3.6661623878467407`*^9}, {3.700198248668149*^9, 
  3.700198299991997*^9}},ExpressionUUID->"e6135231-5cce-4adb-b244-\
9e03cc586a51"],

Cell["\<\
The hyperbolic line is represented by  by the positive branch of the hyperbola\
\>", "Text",
 CellChangeTimes->{{3.661238841629221*^9, 3.6612388769807243`*^9}, {
   3.661238913573875*^9, 3.661238978044232*^9}, 3.6612392644491167`*^9, {
   3.661239309928797*^9, 3.6612393294407988`*^9}, {3.661239368704365*^9, 
   3.66123937719228*^9}, {3.661239573182459*^9, 3.661239592446467*^9}, {
   3.664954546065744*^9, 3.664954560077887*^9}, {3.666162430335082*^9, 
   3.666162434566736*^9}},ExpressionUUID->"d136f692-e4bc-43c0-861e-\
ef70a1362711"],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"hypsp", "[", "1", "]"}], "[", "s", "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"s", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6314203505050898`*^9, 3.6314203662923174`*^9}, {
  3.661239603798628*^9, 
  3.661239641486169*^9}},ExpressionUUID->"b2481e1f-96c3-4801-91ea-\
478e8c975658"],

Cell["\<\
The pseudo-Euclidean model of the hyperbolic plane with r = 1 is \
\>", "Text",
 CellChangeTimes->{{3.631427660970956*^9, 3.631427718699058*^9}, {
   3.631427783556372*^9, 3.631427800078801*^9}, {3.661239523335063*^9, 
   3.661239531302619*^9}, 3.661239568294818*^9, 
   3.6612396715814943`*^9},ExpressionUUID->"afb566c6-26df-46e0-80c3-\
6ccef413be3a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"hypsp", "[", "1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6313432951380644`*^9, 3.631343310379291*^9}, {
   3.6313433587861757`*^9, 3.6313433631385837`*^9}, {3.6313435279216733`*^9, 
   3.631343528093273*^9}, {3.631344904702091*^9, 3.6313449062776937`*^9}, {
   3.6313611074821143`*^9, 3.631361120445737*^9}, 3.631427621762087*^9, 
   3.660979755250746*^9, 
   3.661525117087001*^9},ExpressionUUID->"933b153b-a6dd-4bc4-beb3-\
6f6b9f559376"],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"hypsp", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.631343544426502*^9, 3.631343632207856*^9}, 
   3.631343773746905*^9, {3.6313602737912345`*^9, 3.6313602810804477`*^9}, {
   3.6313605114016576`*^9, 3.6313605216392756`*^9}, {3.700198422125186*^9, 
   3.700198462308683*^9}},ExpressionUUID->"5d4cc8cd-3607-4a42-aad5-\
1fb899255467"],

Cell["\<\
The tangent spaces with the metric induced by the pseudo-Euclidean scalar \
product are Euclidean, and the metric induced by pssp on the hyperbolic \
immersions hypsp[r][s,...] is positive definite. Thus it defines a Riemannian \
geometry. We define the metric tensor for general n as follows:.\
\>", "Text",
 CellChangeTimes->{{3.6314283093880997`*^9, 3.631428485973211*^9}, {
   3.631428522296075*^9, 3.6314285893061934`*^9}, {3.6314307063681993`*^9, 
   3.6314307253594327`*^9}, 3.6314591911141815`*^9, {3.661239722684952*^9, 
   3.661239741965219*^9}, {3.661239772285323*^9, 3.66123981162053*^9}, 
   3.661239842060083*^9, {3.700198615678355*^9, 3.700198617950242*^9}, {
   3.7001986923735943`*^9, 
   3.700198694245446*^9}},ExpressionUUID->"9f4f0194-a1d8-4593-9649-\
706ea9352240"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypsp", "[", "r_", "]"}], "[", "s__", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "s", "}"}], "]"}]}], ",", "i", ",", "j", ",", "rm"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"rm", ":=", 
      RowBox[{"FullSimplify", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"pssp", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"tv", "[", 
               RowBox[{"hypsp", "[", "r", "]"}], "]"}], "[", "i", "]"}], "[", 
             "s", "]"}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"tv", "[", 
               RowBox[{"hypsp", "[", "r", "]"}], "]"}], "[", "j", "]"}], "[", 
             "s", "]"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "n"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "n"}], "}"}]}], "]"}], ",", 
        RowBox[{"Element", "[", 
         RowBox[{
          RowBox[{"{", "s", "}"}], ",", "Reals"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", "rm"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.631362453686095*^9, 3.6313624640007133`*^9}, {
   3.631362554692074*^9, 3.6313626275342054`*^9}, 3.631362665399472*^9, {
   3.6313627096285505`*^9, 3.6313627428586087`*^9}, {3.631362773625863*^9, 
   3.6313629939136515`*^9}, 3.631363024511306*^9, {3.6313676817330627`*^9, 
   3.631367759324999*^9}, {3.6313677972974663`*^9, 3.6313679623163576`*^9}, {
   3.6313679945334144`*^9, 3.631368051042714*^9}, {3.6313680903577833`*^9, 
   3.631368254167274*^9}, 3.6313682931683426`*^9, {3.6313683539344497`*^9, 
   3.631368377352091*^9}, {3.631421171643732*^9, 3.631421185824157*^9}, 
   3.6314212668038993`*^9, 3.6314257715146112`*^9, {3.6612399309320297`*^9, 
   3.661239934035489*^9}, {3.6612668286858664`*^9, 3.661266864477579*^9}, {
   3.70368431796661*^9, 
   3.703684355750371*^9}},ExpressionUUID->"5e8f257b-e0d6-404e-b7e0-\
aff5f4389aba"],

Cell[BoxData[
 RowBox[{"?", "RGhypsp"}]], "Input",
 CellChangeTimes->{{3.661266913705296*^9, 3.661266954309321*^9}, {
  3.661267037795965*^9, 
  3.661267041299679*^9}},ExpressionUUID->"e504eb73-470c-459f-af76-\
1125a4c859a5"],

Cell[BoxData[
 RowBox[{"Context", "[", "RGhypsp", "]"}]], "Input",
 CellChangeTimes->{{3.661266913705296*^9, 3.661266954309321*^9}, {
  3.661267037795965*^9, 3.661267041299679*^9}, {3.700198804349823*^9, 
  3.7001988302610273`*^9}},ExpressionUUID->"d0c14470-8d8c-4a99-9960-\
ca12570927da"],

Cell["Its coordinates are", "Text",
 CellChangeTimes->{{3.631429049128605*^9, 3.6314290553218155`*^9}, {
  3.631458012399311*^9, 3.631458015332116*^9}, {3.6614109827784977`*^9, 
  3.661410987578287*^9}},ExpressionUUID->"e90df04f-45cc-4962-9815-\
d2aa4e2bd7fa"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rghypsp", "[", "r_", "]"}], "[", "s__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"RGhypsp", "[", "r", "]"}], "[", "s", "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"i", ",", "j"}], "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6313687848306084`*^9, 3.631368925414857*^9}, {
   3.631369001749791*^9, 3.6313690727329164`*^9}, 3.631425771561412*^9, 
   3.631425880153202*^9, {3.6314574851339846`*^9, 3.6314574974268064`*^9}, {
   3.6314592774915333`*^9, 3.6314592901743555`*^9}, 3.661240099063397*^9, 
   3.6612402296690407`*^9, {3.7027844372465887`*^9, 
   3.702784446567239*^9}},ExpressionUUID->"5d1a645c-882f-4dc3-831d-\
dd5323ace927"],

Cell[BoxData[
 RowBox[{"?", "rghypsp"}]], "Input",
 CellChangeTimes->{{3.661266960644992*^9, 
  3.6612670204032383`*^9}},ExpressionUUID->"17849d3e-cb1f-4c71-8e61-\
a96773e17f90"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypsp", "[", "r", "]"}], "[", "s", "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631421044753109*^9, 3.63142107924477*^9}, {
   3.631421380606099*^9, 3.631421382930503*^9}, 3.6314257715302114`*^9, 
   3.661240100950862*^9},ExpressionUUID->"5a3e5029-9cbf-4aa7-a2f6-\
181b81648c98"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypsp", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631421044753109*^9, 3.63142107924477*^9}, {
   3.631421380606099*^9, 3.631421382930503*^9}, {3.631421456344232*^9, 
   3.6314214701658564`*^9}, 3.6314257715302114`*^9, 
   3.6612401019921627`*^9},ExpressionUUID->"f99be15a-358c-48b5-b305-\
6dc4babef287"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypsp", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631368264806493*^9, 3.6313682835743256`*^9}, {
   3.6313685047599154`*^9, 3.6313685116571283`*^9}, 3.6313685604560137`*^9, 
   3.6314214960775023`*^9, {3.631421535389571*^9, 3.6314215414891815`*^9}, 
   3.6314257715458117`*^9, 
   3.661240103181991*^9},ExpressionUUID->"aa9069be-eb3f-424c-9ddd-\
b550c7f2e0af"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypsp", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631368264806493*^9, 3.6313682835743256`*^9}, {
   3.6313685047599154`*^9, 3.6313685116571283`*^9}, 3.6313685604560137`*^9, 
   3.6314214960775023`*^9, {3.631421535389571*^9, 3.6314215414891815`*^9}, 
   3.6314257715458117`*^9, {3.6609802980775223`*^9, 3.6609803108048763`*^9}, 
   3.661240104478239*^9},ExpressionUUID->"9e7b7c75-c3ed-4b38-add9-\
c5cd8d0fe869"],

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "/.", 
   RowBox[{"s3", "\[Rule]", 
    RowBox[{"Pi", "/", "2"}]}]}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.7027859843383713`*^9, 3.702785995905199*^9}, {
  3.7027860374340563`*^9, 
  3.702786048752804*^9}},ExpressionUUID->"16a5725b-0597-4d8a-8cdc-\
0bdf2984deb5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rghypsp", "[", "r", "]"}], "[", 
    RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}], "[", 
   RowBox[{"2", ",", "2"}], "]"}], "/.", 
  RowBox[{"s3", "\[Rule]", 
   RowBox[{"Pi", "/", "2"}]}]}]], "Input",
 CellChangeTimes->{{3.7027858084988956`*^9, 3.702785811834783*^9}, {
  3.702785844860054*^9, 
  3.7027859330176277`*^9}},ExpressionUUID->"40a3f313-2a16-4433-aebe-\
6ba209a03fa2"],

Cell["\<\
This formulas show that the parameter representations have a lot of \
singularities: besides of s1= 0 consider the points with si = \[Pi]/2 for \
i>2, n>2.\
\>", "Text",
 CellChangeTimes->{{3.631428802084368*^9, 3.6314289416798143`*^9}, {
   3.661240389855194*^9, 3.66124047201423*^9}, {3.661240536581463*^9, 
   3.661240608429428*^9}, {3.661240714956036*^9, 3.661240720500054*^9}, 
   3.661240756492819*^9, 3.666104907388955*^9, 
   3.702785431803961*^9},ExpressionUUID->"c3201a21-40da-47a7-b814-\
708bf9f4164d"]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3.2. Conformal Model of the hyperbolic n-spaces", "Subsubsection",
 CellChangeTimes->{{3.631429845815478*^9, 3.631429867505516*^9}, {
   3.6314299150040007`*^9, 3.6314299185608063`*^9}, {3.6314300543674464`*^9, 
   3.6314300791462903`*^9}, {3.6314301360097914`*^9, 
   3.6314301448706064`*^9}, {3.6349994459477444`*^9, 3.6349994573397646`*^9}, 
   3.660903132224381*^9, {3.6612376173451557`*^9, 3.661237618288887*^9}, {
   3.6612409097459717`*^9, 3.66124100167321*^9}, {3.6612410938162107`*^9, 
   3.661241159799244*^9}, 3.661268518558423*^9, {3.6614127597853117`*^9, 
   3.6614127736252604`*^9}, {3.6661059189881983`*^9, 3.666105922867775*^9}, 
   3.666164441381628*^9, 3.700199383625279*^9, 3.700200540430951*^9, 
   3.702787082074286*^9, 
   3.702793180470861*^9},ExpressionUUID->"3b370f27-9736-43f5-ac66-\
de5d79c402ec"],

Cell["\<\
Since the parameter representations of the last subsection in higher \
dimensions n have many singularities we will give now another presentation \
having in addition the advantage to be conformal. We set (in riemannv2.m)\
\>", "Text",
 CellChangeTimes->{{3.6314270988905287`*^9, 3.6314271415540047`*^9}, {
   3.6314271760103006`*^9, 3.6314272983773165`*^9}, {3.6314273312691746`*^9, 
   3.6314273442639976`*^9}, {3.631430406390669*^9, 3.6314304260059032`*^9}, 
   3.637759952697815*^9, {3.661237797991201*^9, 3.661237804023423*^9}, {
   3.661238222859433*^9, 3.661238224092305*^9}, {3.661241865288987*^9, 
   3.661242046551393*^9}, {3.661242149758396*^9, 3.661242174733974*^9}, 
   3.6661631205662813`*^9, {3.7001999964988127`*^9, 3.700200007786612*^9}, {
   3.7027912043274727`*^9, 
   3.702791206591342*^9}},ExpressionUUID->"88c81b18-cf5f-4008-bc16-\
fd134fcd66f1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hypcnf", "[", "r_", "]"}], "[", "s__", "]"}], ":=", 
  RowBox[{"r", "*", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          RowBox[{"{", "s", "}"}], ".", 
          RowBox[{"{", "s", "}"}]}]}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          RowBox[{"{", "s", "}"}], ".", 
          RowBox[{"{", "s", "}"}]}]}], ")"}]}], "}"}], ",", 
     RowBox[{"2", "*", 
      RowBox[{
       RowBox[{"{", "s", "}"}], "/", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          RowBox[{"{", "s", "}"}], ".", 
          RowBox[{"{", "s", "}"}]}]}], ")"}]}]}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6313418041810455`*^9, 3.6313419815065565`*^9}, {
   3.6313420979763613`*^9, 3.631342101564368*^9}, {3.6313421514844556`*^9, 
   3.631342160906872*^9}, {3.631342252666233*^9, 3.631342476230226*^9}, {
   3.631342531501123*^9, 3.631342563527979*^9}, {3.631342823627236*^9, 
   3.6313428743585253`*^9}, {3.6313429794247093`*^9, 
   3.6313430038075523`*^9}, {3.6313430514656363`*^9, 3.6313432810512395`*^9}, 
   3.6313434214982862`*^9, 3.6313434637899604`*^9, {3.631343501807227*^9, 
   3.6313435122748456`*^9}, {3.6313436977747717`*^9, 
   3.6313437115651956`*^9}, {3.6313440517081933`*^9, 3.631344119942713*^9}, {
   3.6313442028880587`*^9, 3.631344217458484*^9}, {3.631344251497744*^9, 
   3.6313443097170463`*^9}, {3.6313443405739*^9, 3.63134435723473*^9}, 
   3.6313446285816064`*^9, {3.631344876419242*^9, 3.631344884749656*^9}, 
   3.631344920411319*^9, {3.6313603526751766`*^9, 3.6313603548591805`*^9}, {
   3.6313603853612337`*^9, 3.6313604229145*^9}, {3.6313604677257795`*^9, 
   3.6313604741841908`*^9}, {3.631360548709324*^9, 3.6313606235974555`*^9}, {
   3.6313607114136133`*^9, 3.631360722068432*^9}, {3.6313607678927126`*^9, 
   3.6313607781419306`*^9}, {3.631361004147132*^9, 3.6313610225435643`*^9}, {
   3.631420330100254*^9, 3.631420338711469*^9}, {3.6314204099567947`*^9, 
   3.6314204164620056`*^9}, 3.660747464201707*^9, {3.660979581948326*^9, 
   3.660979591820073*^9}, {3.660980054736224*^9, 3.660980086399876*^9}, 
   3.661237996639988*^9, 3.661242244740244*^9, {3.661242467428146*^9, 
   3.661242514332158*^9}, {3.661242551763921*^9, 3.661242594987163*^9}, {
   3.661242626810676*^9, 3.661242638579006*^9}, {3.661243182125792*^9, 
   3.661243203005245*^9}, {3.661243302348866*^9, 3.661243319924284*^9}, {
   3.66124341225193*^9, 3.661243419675802*^9}, {3.661251178422718*^9, 
   3.661251198181988*^9}, {3.6612670853961782`*^9, 3.661267099547879*^9}, {
   3.661267132890765*^9, 3.661267238498267*^9}, {3.661267690942237*^9, 
   3.661267933595752*^9}, 3.661268028187755*^9, {3.661268065418989*^9, 
   3.661268078291058*^9}},ExpressionUUID->"c538fb76-5bfb-4468-b536-\
cd48f808ec76"],

Cell[BoxData[
 RowBox[{"?", "hypcnf"}]], "Input",
 CellChangeTimes->{{3.63145875213701*^9, 3.6314588358779573`*^9}, {
   3.6314589067256813`*^9, 3.6314589600933757`*^9}, {3.631458995302637*^9, 
   3.6314590911648054`*^9}, 3.631459129150872*^9, 3.661242246273761*^9, {
   3.661242654361999*^9, 3.6612426820660143`*^9}, {3.661267545710703*^9, 
   3.661267560047278*^9}},ExpressionUUID->"467e01ad-79e3-4e7e-9b2c-\
69eafbd3a791"],

Cell[TextData[{
 StyleBox["Hint.",
  FontColor->RGBColor[1, 0, 0]],
 " Always have in mind: The argument {s}  must belong to the open disk, or \
n-ball!"
}], "Text",
 CellChangeTimes->{{3.661267246912877*^9, 3.6612672621774883`*^9}, {
  3.6612681184037867`*^9, 3.661268209736821*^9}, {3.661268351823779*^9, 
  3.661268404783227*^9}},ExpressionUUID->"29468eb8-ef2b-42c6-81d1-\
cc3cfec31b2e"],

Cell[BoxData[
 RowBox[{"Context", "[", "hypcnf", "]"}]], "Input",
 CellChangeTimes->{{3.7002000478206463`*^9, 
  3.700200064257947*^9}},ExpressionUUID->"c7c00d2f-a68a-4f5b-9091-\
e8e5eca1abe0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"hypcnf", "[", "r", "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.66124328819565*^9, 
  3.6612432927803164`*^9}},ExpressionUUID->"775b6d18-0dbf-41f9-81ba-\
69740f75f03e"],

Cell["\<\

The hyperbolic line is represented by  by the positive branch of the \
hyperbola, here r=1:\
\>", "Text",
 CellChangeTimes->{{3.661238841629221*^9, 3.6612388769807243`*^9}, {
   3.661238913573875*^9, 3.661238978044232*^9}, 3.6612392644491167`*^9, {
   3.661239309928797*^9, 3.6612393294407988`*^9}, {3.661239368704365*^9, 
   3.66123937719228*^9}, {3.661239573182459*^9, 3.661239592446467*^9}, 
   3.661242777896359*^9, {3.6612428784558067`*^9, 
   3.6612428890075293`*^9}},ExpressionUUID->"6b92b38e-bed4-4bb7-a69b-\
2437d3ffeb23"],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"hypcnf", "[", "1", "]"}], "[", "s", "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"s", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6314203505050898`*^9, 3.6314203662923174`*^9}, {
   3.661239603798628*^9, 3.661239641486169*^9}, 3.661242254640903*^9, {
   3.6612512456701193`*^9, 3.6612512555809793`*^9}, {3.6612515522111464`*^9, 
   3.661251555858294*^9}},ExpressionUUID->"5d7fed49-27e2-486a-92ea-\
0c5a6a81728d"],

Cell["\<\
The pseudo-Euclidean model of the hyperbolic plane with r = 1 is \
\>", "Text",
 CellChangeTimes->{{3.631427660970956*^9, 3.631427718699058*^9}, {
   3.631427783556372*^9, 3.631427800078801*^9}, {3.661239523335063*^9, 
   3.661239531302619*^9}, 3.661239568294818*^9, 
   3.6612396715814943`*^9},ExpressionUUID->"7c9fd821-c5f1-4e35-a9e6-\
e83681c5f962"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hypcnf", "[", "1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.661237705866515*^9, 3.661237730536274*^9}, 
   3.661238035039316*^9, {3.661238319987391*^9, 3.661238323050275*^9}, 
   3.661242248689312*^9, 
   3.6612500176888933`*^9},ExpressionUUID->"b898c814-8354-42b0-b453-\
c3cdd42c890b"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{"%", ",", "%"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.66123833046618*^9, 
  3.661238364578416*^9}},ExpressionUUID->"f8ff48c0-a2fe-414c-92d9-\
f1ce0751a046"],

Cell[TextData[{
 "The radius vector is ",
 StyleBox["timelike",
  FontColor->RGBColor[0, 0, 1]],
 ", and the tangent vectors are ",
 StyleBox["spacelike ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["and pseudo-orthogonal to the radius vector,",
  FontColor->GrayLevel[0]],
 " if the argument belongs to the unit disk: "
}], "Text",
 CellChangeTimes->{{3.6314305183896666`*^9, 3.6314305841647835`*^9}, {
  3.661251737856311*^9, 3.661251760888029*^9}, {3.666163388753913*^9, 
  3.66616342100945*^9}},ExpressionUUID->"e29ed14a-ab98-4af7-bb34-\
430bb4e357cd"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6612385086735773`*^9, 3.6612385186850863`*^9}, {
  3.661238590751791*^9, 3.661238606192148*^9}, {3.661242250738613*^9, 
  3.661242251586301*^9}, {3.703684426748185*^9, 
  3.703684453100236*^9}},ExpressionUUID->"7d85b310-1ee8-4d0a-b604-\
5c7f35ad1049"],

Cell[BoxData[
 RowBox[{"pssp", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", 
      RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", 
      RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.702791391470511*^9, 3.702791422733677*^9}, {
  3.703684480492392*^9, 
  3.703684504188786*^9}},ExpressionUUID->"81de89f4-0e41-44b0-94ca-\
2df5c1364523"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Together", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.702791511988165*^9, 
  3.702791538844688*^9}},ExpressionUUID->"b799f208-a64c-4433-8d96-\
5e195410b4cd"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6612385086735773`*^9, 3.6612385186850863`*^9}, {
  3.661238590751791*^9, 3.661238624367569*^9}, {3.661242252737715*^9, 
  3.661242253682425*^9}, {3.702791573157434*^9, 3.702791602813366*^9}, {
  3.702791633038155*^9, 3.702791641116891*^9}, {3.703684528732032*^9, 
  3.703684556340682*^9}},ExpressionUUID->"7da509b0-2620-4f97-8cb9-\
4a4fb8bc522a"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hypcnf", "[", "r", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6612385086735773`*^9, 3.6612385186850863`*^9}, {
  3.661238590751791*^9, 3.661238624367569*^9}, {3.661242252737715*^9, 
  3.661242253682425*^9}, {3.666163474953164*^9, 3.666163511080113*^9}, {
  3.702791681123596*^9, 3.702791703179647*^9}, {3.703684587651516*^9, 
  3.703684595988276*^9}},ExpressionUUID->"f093ab04-50d7-4387-b2b5-\
e97abd965d59"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"hypcnf", "[", "r", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", 
       RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6612385086735773`*^9, 3.6612385186850863`*^9}, {
  3.661238590751791*^9, 3.661238624367569*^9}, {3.661242252737715*^9, 
  3.661242253682425*^9}, {3.666163474953164*^9, 3.666163511080113*^9}, {
  3.702791681123596*^9, 3.702791720195634*^9}, {3.703684615019413*^9, 
  3.7036846217160273`*^9}},ExpressionUUID->"2ef60533-a995-42e4-8ebe-\
60bdb5bc17ef"],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"hypcnf", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"s", "*", 
      RowBox[{"Cos", "[", "t", "]"}]}], ",", 
     RowBox[{"s", "*", 
      RowBox[{"Sin", "[", "t", "]"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", ".8"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.631343544426502*^9, 3.631343632207856*^9}, 
   3.631343773746905*^9, {3.6313602737912345`*^9, 3.6313602810804477`*^9}, {
   3.6313605114016576`*^9, 3.6313605216392756`*^9}, 3.661242256210453*^9, {
   3.661251921974842*^9, 3.661251995166622*^9}, {3.661252072486273*^9, 
   3.66125214078909*^9}, {3.661252248740902*^9, 
   3.6612523550920773`*^9}},ExpressionUUID->"24989352-766a-4403-8d4c-\
5e87cb002992"],

Cell["\<\
Remember that the argument {u,v} belongs to the unit disk. The tangent spaces \
with the metric induced by the pseudo-Euclidean scalar product are Euclidean, \
and the metric induced by pssp on the hyperbolic immersions hypcnf[r][s,...] \
is positive definite and thus defines a Riemannian geometry. We define the \
metric tensor for general n as follows:.\
\>", "Text",
 CellChangeTimes->{{3.6314283093880997`*^9, 3.631428485973211*^9}, {
   3.631428522296075*^9, 3.6314285893061934`*^9}, {3.6314307063681993`*^9, 
   3.6314307253594327`*^9}, 3.6314591911141815`*^9, {3.661239722684952*^9, 
   3.661239741965219*^9}, {3.661239772285323*^9, 3.66123981162053*^9}, 
   3.661239842060083*^9, 3.6612422583095503`*^9, {3.6612523956098347`*^9, 
   3.661252463562009*^9}, {3.6612676063981543`*^9, 3.661267613319067*^9}, 
   3.66126849153519*^9},ExpressionUUID->"98c8b6bf-614e-4f1c-a0e7-\
3b02fe4e3756"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r_", "]"}], "[", "s__", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "s", "}"}], "]"}]}], ",", "i", ",", "j", ",", "rm"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"rm", ":=", 
      RowBox[{"FullSimplify", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"pssp", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"tv", "[", 
               RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "i", "]"}], "[",
              "s", "]"}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"tv", "[", 
               RowBox[{"hypcnf", "[", "r", "]"}], "]"}], "[", "j", "]"}], "[",
              "s", "]"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "n"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "n"}], "}"}]}], "]"}], ",", 
        RowBox[{"Element", "[", 
         RowBox[{
          RowBox[{"{", "s", "}"}], ",", "Reals"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", "rm"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.631362453686095*^9, 3.6313624640007133`*^9}, {
   3.631362554692074*^9, 3.6313626275342054`*^9}, 3.631362665399472*^9, {
   3.6313627096285505`*^9, 3.6313627428586087`*^9}, {3.631362773625863*^9, 
   3.6313629939136515`*^9}, 3.631363024511306*^9, {3.6313676817330627`*^9, 
   3.631367759324999*^9}, {3.6313677972974663`*^9, 3.6313679623163576`*^9}, {
   3.6313679945334144`*^9, 3.631368051042714*^9}, {3.6313680903577833`*^9, 
   3.631368254167274*^9}, 3.6313682931683426`*^9, {3.6313683539344497`*^9, 
   3.631368377352091*^9}, {3.631421171643732*^9, 3.631421185824157*^9}, 
   3.6314212668038993`*^9, 3.6314257715146112`*^9, {3.6612399309320297`*^9, 
   3.661239934035489*^9}, {3.661242259025854*^9, 3.661242265129019*^9}, {
   3.702791901867044*^9, 3.7027919454831333`*^9}, {3.7036846708593483`*^9, 
   3.7036846949880342`*^9}},ExpressionUUID->"1e7fcd78-8a67-49f2-87b4-\
807d69c312a9"],

Cell[BoxData[
 RowBox[{"?", "RGhypcnf"}]], "Input",
 CellChangeTimes->{{3.661316722164904*^9, 
  3.661316723524701*^9}},ExpressionUUID->"2a1e5ca9-503a-4a20-851e-\
dfb5e726d15f"],

Cell["Its coordinates are", "Text",
 CellChangeTimes->{{3.631429049128605*^9, 3.6314290553218155`*^9}, {
  3.631458012399311*^9, 3.631458015332116*^9}, {3.661316800762948*^9, 
  3.661316805778715*^9}},ExpressionUUID->"d0307092-8324-4fef-bbb1-\
f3a9b2563174"],

Cell[BoxData[
 RowBox[{"?", "rghypcnf"}]], "Input",
 CellChangeTimes->{{3.661316764115053*^9, 
  3.661316773826991*^9}},ExpressionUUID->"df11a945-78a1-4653-865e-\
86c450f73c3e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rghypcnf", "[", "r_", "]"}], "[", "s__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"RGhypcnf", "[", "r", "]"}], "[", "s", "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"i", ",", "j"}], "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6313687848306084`*^9, 3.631368925414857*^9}, {
   3.631369001749791*^9, 3.6313690727329164`*^9}, 3.631425771561412*^9, 
   3.631425880153202*^9, {3.6314574851339846`*^9, 3.6314574974268064`*^9}, {
   3.6314592774915333`*^9, 3.6314592901743555`*^9}, 3.661240099063397*^9, 
   3.6612402296690407`*^9, {3.6612422660806713`*^9, 3.661242266960465*^9}, {
   3.70279203937724*^9, 
   3.702792045721809*^9}},ExpressionUUID->"d537a068-f01a-422e-878c-\
c4f88f35373d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r", "]"}], "[", "s", "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631421044753109*^9, 3.63142107924477*^9}, {
   3.631421380606099*^9, 3.631421382930503*^9}, 3.6314257715302114`*^9, 
   3.661240100950862*^9, 
   3.661242267666169*^9},ExpressionUUID->"cc242382-eb48-4219-b5a0-\
9b07073fc4f6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631421044753109*^9, 3.63142107924477*^9}, {
   3.631421380606099*^9, 3.631421382930503*^9}, {3.631421456344232*^9, 
   3.6314214701658564`*^9}, 3.6314257715302114`*^9, 3.6612401019921627`*^9, 
   3.6612422684823*^9},ExpressionUUID->"d439d34a-f036-4940-9020-ac18efb6f46a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631368264806493*^9, 3.6313682835743256`*^9}, {
   3.6313685047599154`*^9, 3.6313685116571283`*^9}, 3.6313685604560137`*^9, 
   3.6314214960775023`*^9, {3.631421535389571*^9, 3.6314215414891815`*^9}, 
   3.6314257715458117`*^9, 3.661240103181991*^9, 
   3.661242269321967*^9},ExpressionUUID->"e417010c-ce44-4b0b-ac79-\
10feed6ef13d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631368264806493*^9, 3.6313682835743256`*^9}, {
   3.6313685047599154`*^9, 3.6313685116571283`*^9}, 3.6313685604560137`*^9, 
   3.6314214960775023`*^9, {3.631421535389571*^9, 3.6314215414891815`*^9}, 
   3.6314257715458117`*^9, {3.6609802980775223`*^9, 3.6609803108048763`*^9}, 
   3.661240104478239*^9, 
   3.661242270082062*^9},ExpressionUUID->"58a2c136-08a6-4352-8059-\
499db85bec67"],

Cell[TextData[{
 "This formulas show that the parameter representations \
don\[CloseCurlyQuote]t have singularities, and that the map {si} \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["D", "n"], TraditionalForm]],ExpressionUUID->
  "78e0ddaa-522c-482f-868d-b85fd7280217"],
 " \[RightTeeArrow] hypcnf[r][s1,...,sn] \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "n"], TraditionalForm]],ExpressionUUID->
  "b139aca2-c106-4cd2-aa5f-8e815e0a0057"],
 " is conformal; here ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["D", "n"], TraditionalForm]],ExpressionUUID->
  "3bcc8603-2e5d-4b6b-a4f3-20b02919bc60"],
 " denotes the n-dimensional open unit ball and  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "n"], TraditionalForm]],ExpressionUUID->
  "093cc947-6d00-48c5-b08d-2fdd68a0fbef"],
 " the n-dimensional hyperbolic space modelled by the immersion hypcnf[r]."
}], "Text",
 CellChangeTimes->{{3.631428802084368*^9, 3.6314289416798143`*^9}, {
   3.661240389855194*^9, 3.66124047201423*^9}, {3.661240536581463*^9, 
   3.661240608429428*^9}, {3.661240714956036*^9, 3.661240720500054*^9}, 
   3.661240756492819*^9, {3.661252647496025*^9, 3.661252671343796*^9}, {
   3.661252729119853*^9, 3.661253017788731*^9}, {3.661253070508129*^9, 
   3.6612531375239687`*^9}, {3.700200338514708*^9, 3.700200371617041*^9}, 
   3.7027921969245977`*^9},ExpressionUUID->"f76f3999-b47d-493f-9c79-\
c7f8be256a2e"],

Cell["\<\
Now we consider the tensor RGhypcnf[r][s...] for increasing dimension and \
derive from the results an abstract definition of  the n-dimensional \
Riemannian metric of constant negative curvature conformal to the \
n-dimensional Euclidean space.\
\>", "Text",
 CellChangeTimes->{{3.6314323419908886`*^9, 3.6314324016095943`*^9}, {
   3.6314324351400537`*^9, 3.6314324420664654`*^9}, {3.6314324783325987`*^9, 
   3.631432635837877*^9}, 3.661240109441928*^9, 3.661242284415133*^9, {
   3.661411489266836*^9, 3.661411556105932*^9}, 3.661411949116658*^9, 
   3.672200470859146*^9, 
   3.702792516668393*^9},ExpressionUUID->"99d1edcc-ea58-4c1b-8c5e-\
9473f9fe9ac4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631368264806493*^9, 3.6313682835743256`*^9}, {
   3.6313685047599154`*^9, 3.6313685116571283`*^9}, 3.6313685604560137`*^9, 
   3.6314214960775023`*^9, {3.631421535389571*^9, 3.6314215414891815`*^9}, {
   3.63142157453004*^9, 3.6314215871192617`*^9}, 3.6314257715458117`*^9, 
   3.66124011091772*^9, 
   3.661261532027769*^9},ExpressionUUID->"8407a6d2-dc7d-4fc8-a071-\
140cd0b38eef"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.631368264806493*^9, 3.6313682835743256`*^9}, {
   3.6313685047599154`*^9, 3.6313685116571283`*^9}, 3.6313685604560137`*^9, 
   3.6314214960775023`*^9, {3.631421535389571*^9, 3.6314215414891815`*^9}, {
   3.63142157453004*^9, 3.6314215871192617`*^9}, {3.631421622936925*^9, 
   3.6314216344029446`*^9}, 3.6314257715458117`*^9, 3.6612401122686863`*^9, 
   3.661261532844254*^9},ExpressionUUID->"6915cbde-4325-47fb-898b-\
bef906922c66"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGneg", "[", "r_", "]"}], "[", "s__", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "*", 
      RowBox[{"r", "/", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{
          RowBox[{"{", "s", "}"}], ".", 
          RowBox[{"{", "s", "}"}]}]}], ")"}]}]}], ")"}], "^", "2"}], "*", 
   RowBox[{"IdentityMatrix", "[", 
    RowBox[{"Length", "[", 
     RowBox[{"{", "s", "}"}], "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6314337207247944`*^9, 3.6314338143097596`*^9}, 
   3.631433912666333*^9, 3.631434453733489*^9, {3.661261683418488*^9, 
   3.6612618966087313`*^9}, 
   3.6612621351300173`*^9},ExpressionUUID->"ef180ff5-a362-4df6-8076-\
3d1cb286d556"],

Cell[BoxData[
 RowBox[{"?", "RGneg"}]], "Input",
 CellChangeTimes->{{3.6314575901845694`*^9, 3.6314576713983116`*^9}, {
   3.6314577191187954`*^9, 3.6314577721432886`*^9}, {3.631457804326145*^9, 
   3.6314578829658833`*^9}, {3.631457945365993*^9, 3.6314579888432693`*^9}, 
   3.661262052303228*^9, 3.661262432811761*^9, {3.661262499259576*^9, 
   3.661262578282*^9}, {3.661262616945889*^9, 3.6612626211855993`*^9}, {
   3.6614116104567966`*^9, 
   3.6614116168330917`*^9}},ExpressionUUID->"610548f7-30e3-4aa6-a268-\
002574233537"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGneg", "[", "r", "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.631433823968177*^9, 3.6314338367601995`*^9}, 
   3.6314344587430983`*^9, 
   3.6612621351655083`*^9},ExpressionUUID->"846e93bd-e2be-4886-980b-\
83951ca4ab94"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGneg", "[", "r", "]"}], "[", 
  RowBox[{"s1", ",", "s2"}], "]"}]], "Input",
 CellChangeTimes->{{3.631433823968177*^9, 3.6314338367601995`*^9}, {
   3.6314338698372574`*^9, 3.631433883396682*^9}, 3.6314344619567037`*^9, 
   3.66126213520077*^9},ExpressionUUID->"78aa0d09-2da9-4b02-bca9-\
a5f1a817eb2b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGneg", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}], 
  "\[Equal]", " ", 
  RowBox[{
   RowBox[{"RGhypcnf", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.631433823968177*^9, 3.6314338367601995`*^9}, {
   3.6314338698372574`*^9, 3.631433883396682*^9}, 3.631433954734008*^9, {
   3.631433992258074*^9, 3.63143403515055*^9}, 3.631434466340311*^9, 
   3.6612401150938377`*^9, 3.661261534916212*^9, 3.6612621352287617`*^9, 
   3.6614119984841633`*^9, 
   3.661412029356503*^9},ExpressionUUID->"5f8d9a17-4b00-444e-b5f1-\
cfd39e005b6f"],

Cell["Its coordinates are", "Text",
 CellChangeTimes->{{3.631429049128605*^9, 3.6314290553218155`*^9}, {
  3.631458012399311*^9, 3.631458015332116*^9}, {3.6614121018423147`*^9, 
  3.66141210548208*^9}},ExpressionUUID->"cfba0477-6399-42e1-b777-\
9f1409e33aff"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rgneg", "[", "r_", "]"}], "[", "s__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"RGneg", "[", "r", "]"}], "[", "s", "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"i", ",", "j"}], "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6313687848306084`*^9, 3.631368925414857*^9}, {
   3.631369001749791*^9, 3.6313690727329164`*^9}, 3.631425771561412*^9, 
   3.631425880153202*^9, {3.6314574851339846`*^9, 3.6314574974268064`*^9}, 
   3.6612626849161177`*^9, 3.661262858201511*^9, {3.7027925946443033`*^9, 
   3.702792600876602*^9}},ExpressionUUID->"14f9a960-d255-4ee7-b12f-\
67a5c5cfa241"],

Cell[BoxData[
 RowBox[{"?", "rgneg"}]], "Input",
 CellChangeTimes->{{3.631457925039157*^9, 3.631457931637969*^9}, {
   3.631458037733755*^9, 3.631458201066042*^9}, 3.661262694771881*^9, 
   3.661262819752235*^9, 3.661262860042487*^9, {3.661411786110408*^9, 
   3.661411806742889*^9}},ExpressionUUID->"45546446-9c86-4d6c-98dd-\
11a510b7474e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rgneg", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}], "[", 
  RowBox[{"2", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.702793083903285*^9, 
  3.7027931300148478`*^9}},ExpressionUUID->"398d5116-45a7-43f0-8695-\
7c0b0daa3d6e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGneg", "[", "r", "]"}], "[", 
  RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}]], "Input",
 CellChangeTimes->{{3.6314349044720855`*^9, 3.6314349126017*^9}, 
   3.661262861255821*^9},ExpressionUUID->"7807709c-52c8-4f32-bb54-\
2a7370db2fb3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"INV", "[", 
   RowBox[{"RGneg", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4"}], "]"}]], "Input",
 CellChangeTimes->{{3.6313686376167493`*^9, 3.6313686543731785`*^9}, {
   3.6314217441335373`*^9, 3.631421759078364*^9}, 3.631425771561412*^9, {
   3.631434116548893*^9, 3.631434136208928*^9}, 
   3.661262864424156*^9},ExpressionUUID->"6f472502-a66c-4b30-9a8b-\
a8baf3488f07"],

Cell[BoxData[
 RowBox[{
  RowBox[{"INV", "[", 
   RowBox[{"INV", "[", 
    RowBox[{"RGneg", "[", "r", "]"}], "]"}], "]"}], "[", 
  RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], "]"}]], "Input",
 CellChangeTimes->{{3.6314218484977207`*^9, 3.6314218944710016`*^9}, 
   3.631425771561412*^9, 3.6314341470061474`*^9, 
   3.661262866327305*^9},ExpressionUUID->"303d0de5-30ea-45a8-bafc-\
a833aad07ffa"],

Cell[BoxData[
 RowBox[{"%", "==", 
  RowBox[{
   RowBox[{"RGneg", "[", "r", "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6314219268098583`*^9, 3.6314219593359156`*^9}, 
   3.6314257715770116`*^9, 3.6315178118364887`*^9, 
   3.6612628735842333`*^9},ExpressionUUID->"25993e05-5f13-41dc-95a8-\
f2d96c6ec2ef"]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2. Tensor Fields", "Section",
 CellChangeTimes->{{3.631859555473789*^9, 3.6318595915722523`*^9}, {
   3.644932830043013*^9, 3.6449328370182257`*^9}, {3.6568355830460663`*^9, 
   3.6568355937834177`*^9}, {3.6661644635250483`*^9, 3.6661644663726*^9}, 
   3.666190685918912*^9, 3.7018687114949093`*^9, {3.702526138807631*^9, 
   3.702526141143256*^9}},ExpressionUUID->"3f95adbb-87f1-4de1-88f4-\
6b20aa097185"],

Cell["\<\
In this and the following Sections we often use the functions and modules of \
tensor algebra contained in the package tensalgv3.m. To learn this matter I \
recommend first to study the notebook vectensalgv3.nb. \
\>", "Text",
 CellChangeTimes->{{3.7003646601445208`*^9, 3.7003646818641453`*^9}, {
   3.700364728992001*^9, 3.700364812343175*^9}, {3.700365145828484*^9, 
   3.700365177140039*^9}, {3.7003653905630007`*^9, 3.7003654615863743`*^9}, {
   3.700365515442049*^9, 3.700365590368998*^9}, {3.700365638248865*^9, 
   3.700365641312745*^9}, 3.703771108770348*^9, {3.7040300291075687`*^9, 
   3.70403003616681*^9}, {3.704030144303688*^9, 
   3.704030154357888*^9}},ExpressionUUID->"65e460b7-cff1-476c-a89b-\
7372f165b5fc"],

Cell[CellGroupData[{

Cell["2.1. Manifolds", "Subsection",
 CellChangeTimes->{{3.6318596052378764`*^9, 3.631859621945506*^9}, 
   3.700200551479865*^9, 
   3.7005454752038813`*^9},ExpressionUUID->"6693537b-4750-493f-a5b2-\
eb56b165335a"],

Cell[CellGroupData[{

Cell["2.1.1. The Definition", "Subsubsection",
 CellChangeTimes->{{3.6367866947201924`*^9, 3.6367867204758377`*^9}, 
   3.702793192591632*^9, 
   3.702794884506928*^9},ExpressionUUID->"531d3913-c29f-4f16-9ea9-\
42b70fd62574"],

Cell[TextData[{
 "The basic concept of modern differential geometry is that of a \
differentiable manifold. Many textbooks about differential geometry \
containing Riemannian geometry or higher geometric structures start with a \
chapter titled or devoted to \[OpenCurlyDoubleQuote]Differentiable Manifolds\
\[CloseCurlyDoubleQuote], see e.g. ",
 ButtonBox["[K-N1]",
  BaseStyle->"Hyperlink",
  ButtonData->"K-N1"],
 ", [",
 ButtonBox["H78",
  BaseStyle->"Hyperlink",
  ButtonData->"H78"],
 "],  [",
 ButtonBox["S-W",
  BaseStyle->"Hyperlink",
  ButtonData->"S-W"],
 "]. The study of one of these chapters  would be a useful background for the \
following. Also in most modern presentations of elementary Euclidean \
differential geometry one finds chapters about differentiable manifolds, see \
e.g. [",
 ButtonBox["B-L",
  BaseStyle->"Hyperlink",
  ButtonData->"B-L"],
 "], Chapter 7, \[Section] 97, or [",
 ButtonBox["G94",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray3"],
 "], \n Chapter 25. To give the following a rigid base we describe now the \
definition of this concept and fix the notation used in the notebook."
}], "Text",
 CellChangeTimes->{{3.6318641219640913`*^9, 3.6318642682261505`*^9}, {
   3.6318643101162243`*^9, 3.6318644052465925`*^9}, {3.6318645048969684`*^9, 
   3.631864543700237*^9}, {3.6318646488060875`*^9, 3.631864720897814*^9}, {
   3.631864773848307*^9, 3.631864980824272*^9}, {3.631865016835136*^9, 
   3.6318650846720552`*^9}, {3.631865186258835*^9, 3.6318652121900806`*^9}, {
   3.631865360234741*^9, 3.6318654065728235`*^9}, {3.6318654972053833`*^9, 
   3.631865531685444*^9}, {3.6318656154393916`*^9, 3.631865669953888*^9}, {
   3.631865810336935*^9, 3.6318658367517815`*^9}, {3.631865876818652*^9, 
   3.6318659093643103`*^9}, {3.6318659483059783`*^9, 3.631866017766301*^9}, {
   3.63186668092947*^9, 3.6318666920366898`*^9}, {3.631866734412365*^9, 
   3.631866734412365*^9}, {3.631866766318421*^9, 3.631866820175716*^9}, {
   3.6318668559973793`*^9, 3.6318668559973793`*^9}, {3.631866908639872*^9, 
   3.631866908639872*^9}, {3.631886379946087*^9, 3.631886434881784*^9}, {
   3.6319742073059235`*^9, 3.6319742094899273`*^9}, 3.632037475097948*^9, 
   3.632037635852275*^9, {3.6449137914804673`*^9, 3.64491379290007*^9}, {
   3.6492288835738688`*^9, 3.649228883589492*^9}, 3.670567577825404*^9, {
   3.700221328684739*^9, 3.700221328701326*^9}, {3.700221420708119*^9, 
   3.7002214591890373`*^9}, {3.700221536887991*^9, 
   3.7002215368910027`*^9}},ExpressionUUID->"033432b0-6e36-467d-8360-\
4bf94fae822b"],

Cell[TextData[{
 "Let M denote a topological space always assumed to be ",
 StyleBox["Hausdorff",
  FontColor->RGBColor[0, 0, 1]],
 ", i.e. that for any  two points x, y \[Element] M, x\[NotEqual]y,  there \
exist separating  open neighborhoods U, V with x\[Element]U, y\[Element]V, \
and U\[Intersection]V = \[EmptySet]. Let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "c0b1c4b3-c434-49ec-91e0-99040eacf4fc"],
 " be an open subset of M and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],ExpressionUUID->
  "8e066704-f04f-4e36-98b6-eb39fc66c2c7"],
 " the n-dimensional real point space with its standard topology. A map"
}], "Text",
 CellChangeTimes->{{3.631866126167292*^9, 3.631866190652206*^9}, {
   3.631867022516473*^9, 3.631867154429705*^9}, {3.6318671892807107`*^9, 
   3.631867260200435*^9}, {3.6318672968020997`*^9, 3.631867363265217*^9}, 
   3.6318673938764706`*^9, {3.6318864661794395`*^9, 3.631886475196255*^9}, {
   3.631886917605635*^9, 3.6318869552097015`*^9}, 3.631886999205779*^9, {
   3.6318870896325383`*^9, 3.631887217444964*^9}, {3.631887305285119*^9, 
   3.6318873192939434`*^9}, {3.63188736823923*^9, 3.631887398256683*^9}, {
   3.6318874520163774`*^9, 3.6318874660136023`*^9}, {3.6318875093544784`*^9, 
   3.6318875484717474`*^9}, {3.6318875788802013`*^9, 
   3.6318877615891232`*^9}, {3.631887862782902*^9, 3.6318879576264687`*^9}, {
   3.6318879982725406`*^9, 3.6318880215477815`*^9}, 3.6318881064873314`*^9, 
   3.631888172592648*^9, {3.631888319524858*^9, 3.631888327326872*^9}, {
   3.6318884957607684`*^9, 3.631888567530895*^9}, {3.6318886107801714`*^9, 
   3.6318886263801985`*^9}, {3.6318886583350554`*^9, 3.631888867089423*^9}, {
   3.631888943164158*^9, 3.631889015010284*^9}, {3.6318890990596323`*^9, 
   3.63188912048047*^9}},ExpressionUUID->"b30b9351-0ec5-4748-ac0f-\
7211251d5753"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\[Phi]\[Iota]",
    TraditionalForm], ":", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["U", "\[Iota]"],
      TraditionalForm], "\[LongRightArrow]", 
     RowBox[{
      FormBox["\[Phi]\[Iota]",
       TraditionalForm], "(", 
      FormBox[
       SubscriptBox["U", "\[Iota]"],
       TraditionalForm], ")"}]}], "\[Subset]", 
    FormBox[
     SuperscriptBox["R", "n"],
     TraditionalForm]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}, {3.6378160097442374`*^9, 3.6378160122714415`*^9}, 
   3.637816043756297*^9, {3.666327458877562*^9, 3.666327458877562*^9}},
 TextAlignment->Center,ExpressionUUID->"8bd26fb1-08d3-4a09-b733-5af4bc302f84"],

Cell[TextData[{
 "is said to be a ",
 StyleBox["chart ",
  FontColor->RGBColor[0, 0, 1]],
 "with domain ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "87dbd4e3-732b-421c-9084-eb9315a0dbf7"],
 ", if it is bijective and ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "efa60bcd-42dd-46d9-b4fc-2c48084773a8"],
 ", as well as its inverse ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Phi]\[Iota]", 
    RowBox[{"-", "1"}]], TraditionalForm]],ExpressionUUID->
  "282f89f0-04f5-40dc-ac8d-dfce984bfa24"],
 ", are continuous; shortly ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "ed1be1d0-4c43-4b8d-89d1-117bdda190ad"],
 " is a ",
 StyleBox["homeomorphism",
  FontColor->RGBColor[0, 0, 1]],
 " of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "bf1ed146-1929-4fc5-bb88-a644493f4301"],
 " onto  ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "3e72bf38-6feb-4e65-854a-96678e124070"],
 "(",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "3e7c992a-2385-4105-9b08-de5bc40d8663"],
 "). The space M is named an ",
 StyleBox["n-dimensional (topological) manifold",
  FontColor->RGBColor[0, 0, 1]],
 " and denoted by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "52197b74-20ec-4ce4-b86d-c27c7d3a44e8"],
 ", if every point x\[Element]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "45dbe9cf-ab5f-437a-9166-84f108d787d0"],
 " has an open neighborhood ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "74173afb-8fed-410c-a7aa-ba3b5b4d836e"],
 ", x\[Element]",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "672defdb-aeff-418a-acf9-317065b134cc"],
 ", serving as the domain of a chart ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "b8d7cd98-3352-4b11-b7ec-2ac1eac36367"],
 " with values in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],ExpressionUUID->
  "49e772ed-9494-4156-98fd-28274db0b823"],
 ". The non-negative integer n is called the ",
 StyleBox["dimension",
  FontColor->RGBColor[0, 0, 1]],
 " of the manifold. By formal reason one admits the case n=0 what means that \
M is a trivial topological space with the discrete topology: every subset is \
open. For n>0 a chart defines",
 StyleBox[" local coordinates",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.631866126167292*^9, 3.631866190652206*^9}, {
   3.631867022516473*^9, 3.631867154429705*^9}, {3.6318671892807107`*^9, 
   3.631867260200435*^9}, {3.6318672968020997`*^9, 3.631867363265217*^9}, 
   3.6318673938764706`*^9, {3.6318864661794395`*^9, 3.631886475196255*^9}, {
   3.631886917605635*^9, 3.6318869552097015`*^9}, 3.631886999205779*^9, {
   3.6318870896325383`*^9, 3.631887217444964*^9}, {3.631887305285119*^9, 
   3.6318873192939434`*^9}, {3.63188736823923*^9, 3.631887398256683*^9}, {
   3.6318874520163774`*^9, 3.6318874660136023`*^9}, {3.6318875093544784`*^9, 
   3.6318875484717474`*^9}, {3.6318875788802013`*^9, 
   3.6318877615891232`*^9}, {3.631887862782902*^9, 3.6318879576264687`*^9}, {
   3.6318879982725406`*^9, 3.6318880215477815`*^9}, 3.6318881064873314`*^9, 
   3.631888172592648*^9, {3.631888319524858*^9, 3.631888327326872*^9}, {
   3.6318884957607684`*^9, 3.631888567530895*^9}, {3.6318886107801714`*^9, 
   3.6318886263801985`*^9}, {3.6318886583350554`*^9, 3.631888867089423*^9}, {
   3.631888943164158*^9, 3.631889015010284*^9}, {3.6318890990596323`*^9, 
   3.63188912048047*^9}, 3.6318893036165934`*^9, 3.631889442004837*^9, {
   3.631889490760923*^9, 3.631889494364529*^9}, {3.631889679445525*^9, 
   3.6318897386868296`*^9}, {3.6318898067129498`*^9, 3.631889911957136*^9}, {
   3.631890009383708*^9, 3.631890012768914*^9}, 3.6318901035478745`*^9, {
   3.6319742953992786`*^9, 3.6319743146341124`*^9}, {3.631974588788994*^9, 
   3.631974686850766*^9}, 3.632037701368791*^9, {3.6449140504877224`*^9, 
   3.6449140663841505`*^9}, 3.64491461870312*^9, {3.666327458877562*^9, 
   3.6663274588931875`*^9}},ExpressionUUID->"3530bbbc-cf69-4bc9-8074-\
08aeb4437b20"],

Cell[BoxData[
 RowBox[{
  RowBox[{"x", "\[Element]", 
   RowBox[{
    FormBox[
     SubscriptBox["U", "\[Iota]"],
     TraditionalForm], " ", "\[RightTeeArrow]", " ", 
    RowBox[{
     FormBox["\[Phi]\[Iota]",
      TraditionalForm], 
     RowBox[{"(", 
      FormBox["x",
       TraditionalForm], ")"}]}]}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["x", "i"], 
     RowBox[{"(", "x", ")"}]}], "}"}], "\[Element]", 
   RowBox[{
    FormBox["\[Phi]\[Iota]",
     TraditionalForm], 
    RowBox[{"(", 
     FormBox[
      SubscriptBox["U", "\[Iota]"],
      TraditionalForm], ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6319747265216355`*^9, 3.631974735975252*^9}, {
  3.6319747928841524`*^9, 3.631974813881789*^9}, {3.6319748452222443`*^9, 
  3.6319748609002714`*^9}, {3.6319748993387394`*^9, 3.631975027274564*^9}, {
  3.6663274588931875`*^9, 3.6663274588931875`*^9}},
 TextAlignment->Center,ExpressionUUID->"cc30da6b-9027-4d99-8d01-6779d804a0d9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]\[Iota]", "[", "x_", "]"}], ":=", " ", 
   RowBox[{"vec", "[", "x", "]"}]}], ";", 
  RowBox[{"\[Phi]\[Iota]", "[", "y", "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6666064052711253`*^9, 3.6666064550080996`*^9}, 
   3.666606704958542*^9},ExpressionUUID->"9bca7013-910f-4873-ad23-\
36894c7cdad2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]\[Iota]", "::", "usage"}], " ", "=", 
   "\"\<\[Phi]\[Iota][x]={[x[1],...,x[n]},n = dim, the dimension of the \
manifold, is a symbolic general chart map.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6666065278087215`*^9, 3.666606681394826*^9}, {
   3.6666067167403965`*^9, 3.6666067385696573`*^9}, 3.6666067902442036`*^9, 
   3.6666068206989093`*^9},ExpressionUUID->"f8eb65fc-d012-4be3-b50a-\
02426df9bc37"],

Cell[TextData[{
 "Here the ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    RowBox[{"x", "[", "i", "]"}], ",", " ", 
    RowBox[{"i", " ", "=", " ", "1"}], ",", "...", ",", "n", ","}],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6319747265216355`*^9, 3.631974735975252*^9}, {
   3.6319747928841524`*^9, 3.631974813881789*^9}, {3.6319748452222443`*^9, 
   3.6319748609002714`*^9}, {3.6319748993387394`*^9, 3.631975027274564*^9}},
  ExpressionUUID->"a1d799c1-a258-41a2-9173-a8c00ad259a5"],
 " are the coordinates of  ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"x", "\[Element]", 
    FormBox[
     SubscriptBox["U", "\[Iota]"],
     TraditionalForm]}],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6319747265216355`*^9, 3.631974735975252*^9}, {
   3.6319747928841524`*^9, 3.631974813881789*^9}, {3.6319748452222443`*^9, 
   3.6319748609002714`*^9}, {3.6319748993387394`*^9, 3.631975027274564*^9}},
  ExpressionUUID->"69d6ea8c-f11b-4a77-be1b-385bd13fa63d"],
 " in the chart ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,ExpressionUUID->
  "4c913ce1-a673-4ff3-890a-79f034f4c662"],
 ". The notation ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["x", "i"], 
   RowBox[{"(", "x", ")"}]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6319747265216355`*^9, 3.631974735975252*^9}, {
   3.6319747928841524`*^9, 3.631974813881789*^9}, {3.6319748452222443`*^9, 
   3.6319748609002714`*^9}, {3.6319748993387394`*^9, 3.631975027274564*^9}, {
   3.6663274588931875`*^9, 3.6663274588931875`*^9}},
  TextAlignment->Center,ExpressionUUID->
  "8a12b665-8ea7-40d9-b3f1-106c7cefcb8c"],
 "\[DotEqual]",
 " x[i] is traditional; the upper index i is the number of the coordinate, \
not a power.  We may use it only in not evaluatable, purely describing \
formulas.  In evaluations ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " interprets  the upper indices always  as powers, what is nonsense in our \
case:"
}], "Text",
 CellChangeTimes->{{3.6319752962658367`*^9, 3.631975326093089*^9}, {
   3.631975358073145*^9, 3.6319754517981095`*^9}, {3.6319754820153627`*^9, 
   3.631975509409011*^9}, {3.631975543619871*^9, 3.6319755959891624`*^9}, 
   3.631975709947363*^9, {3.6319757613338532`*^9, 3.6319757751398773`*^9}, {
   3.6319758787396593`*^9, 3.631975923028137*^9}, {3.6319759725426245`*^9, 
   3.631976160975355*^9}, {3.6319762182430553`*^9, 3.631976221160261*^9}, {
   3.666327458908813*^9, 3.666327458908813*^9}, {3.666606887905493*^9, 
   3.6666069953485594`*^9}},ExpressionUUID->"ae6ff1a4-7af8-4489-8a6c-\
0d62cfa7ef40"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["x", "3"], "/.", " ", 
  RowBox[{"x", "\[Rule]", "2"}]}]], "Input",
 CellChangeTimes->{{3.6319756731936984`*^9, 3.631975688216525*^9}, {
   3.700221908415461*^9, 3.700221914199326*^9}, 3.7002219461245527`*^9},
 TextAlignment->Center,ExpressionUUID->"d8142666-f6b0-4883-9c0e-5d782cb17ed6"],

Cell["The correct syntax for defining the coordinate functions is", "Text",
 CellChangeTimes->{{3.6319762732019525`*^9, 
  3.6319763024364033`*^9}},ExpressionUUID->"14e347bd-8327-4e2f-99d1-\
5b0e039613d6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "i"], 
    RowBox[{"(", "x", ")"}]}], " ", "\[Rule]", " ", 
   RowBox[{
    RowBox[{"x", "[", "i", "]"}], "[", "x", "]"}]}], ",", " ", 
  RowBox[{"i", " ", "=", " ", "1"}], ",", "...", ",", "n", ",", 
  RowBox[{"x", "\[Element]", 
   RowBox[{
    FormBox[
     SubscriptBox["U", "\[Iota]"],
     TraditionalForm], "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6319763405940704`*^9, 3.631976360920906*^9}, {
  3.631976455067072*^9, 3.631976472351902*^9}, {3.6319773228497963`*^9, 
  3.631977357840657*^9}},
 TextAlignment->Center,ExpressionUUID->"d5a9f928-e79b-4624-8765-556be2b9f73a"],

Cell[TextData[{
 "A family of charts ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     RowBox[{"{", "\[Phi]\[Iota]", "}"}], 
     RowBox[{"\[Iota]", "\[Element]", 
      StyleBox["\[ScriptCapitalJ]",
       FontSlant->"Plain"]}]], " "}], TraditionalForm]],ExpressionUUID->
  "0136f361-c89b-4f3f-a6bc-a95c1ec7079a"],
 " is named an ",
 StyleBox["atlas of the manifold",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "1f680ab7-4c66-4c76-bc19-1ccd1353d4e2"],
 ", if the union of their domains is the whole manifold:"
}], "Text",
 CellChangeTimes->{{3.6319768100612955`*^9, 3.6319768312773323`*^9}, {
   3.631976878904216*^9, 3.6319768956118455`*^9}, {3.631976935719516*^9, 
   3.631976935719516*^9}, {3.6319769758427863`*^9, 3.6319769935956173`*^9}, 
   3.631977036714093*^9, {3.6319770977726*^9, 3.631977195990373*^9}, {
   3.631977229514832*^9, 3.631977268826901*^9}, 3.6320378264994397`*^9, {
   3.666327458908813*^9, 
   3.666327458908813*^9}},ExpressionUUID->"97fcf730-36c7-491e-82b4-\
b75fbbeb56e3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   UnderscriptBox["\[Union]", 
    RowBox[{"\[Iota]", "\[Element]", "\[ScriptCapitalJ]"}]], " ", 
   SubscriptBox["U", "\[Iota]"]}], " ", "=", " ", 
  RowBox[{
   SuperscriptBox["M", "n"], "."}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6319773967315254`*^9, 3.631977417963163*^9}, {
  3.631977477024867*^9, 3.6319775796886473`*^9}},
 TextAlignment->Center,ExpressionUUID->"b8e229bc-887e-495b-944c-ad30b86d8886"],

Cell[TextData[{
 "If the intersection of two domains is not empty:  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["U", "\[Kappa]"], " "}], TraditionalForm]],ExpressionUUID->
  "77b2ad2b-2b66-43ac-8a16-56146b95761f"],
 "\[Intersection]",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["U", "\[Iota]"]}], TraditionalForm]],ExpressionUUID->
  "747264c5-a598-4a15-be90-df9392b10f3e"],
 " \[NotEqual] \[EmptySet], then each point of this intersection has two \
systems of coordinates:"
}], "Text",
 CellChangeTimes->{{3.6319779839165573`*^9, 3.631978022199024*^9}, 
   3.6319780696699076`*^9, {3.6319781077183743`*^9, 3.631978167372879*^9}, {
   3.6319781988537345`*^9, 3.6319782137361603`*^9}, {3.631978244327814*^9, 
   3.631978251488227*^9}, {3.631978330174765*^9, 3.631978371733238*^9}, {
   3.6319784056632977`*^9, 3.631978442260962*^9}, {3.632029979039479*^9, 
   3.6320299822218847`*^9}},ExpressionUUID->"00d7858e-2db1-455a-b96d-\
1ea46a14101b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    FormBox["\[Phi]\[Iota]",
     TraditionalForm], 
    RowBox[{"(", 
     FormBox["x",
      TraditionalForm], ")"}]}], " ", "=", 
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["x", "i"], 
     RowBox[{"(", "x", ")"}]}], "}"}]}], ",", " ", 
  RowBox[{
   RowBox[{
    FormBox[
     SubscriptBox["\[Phi]", "\[Kappa]"],
     TraditionalForm], 
    RowBox[{"(", 
     FormBox["x",
      TraditionalForm], ")"}]}], " ", "=", 
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["y", "k"], 
     RowBox[{"(", "x", ")"}]}], "}"}]}], ",", 
  RowBox[{"x", " ", "\[Element]", " ", 
   RowBox[{
    FormBox[
     RowBox[{
      SubscriptBox["U", "\[Kappa]"], " ", "\[Intersection]", " ", 
      SubscriptBox["U", "\[Iota]"]}],
     TraditionalForm], ".", " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.631978559989169*^9, 3.631978652060531*^9}, 
   3.632029688598169*^9, {3.632029836642429*^9, 3.632029846189646*^9}, {
   3.632029880104105*^9, 3.6320299236125813`*^9}, 3.632030457273919*^9, {
   3.6320309569427967`*^9, 3.6320309653356113`*^9}, {3.632031072851*^9, 
   3.632031108809063*^9}, {3.6320311457655277`*^9, 3.63203127786656*^9}, 
   3.6329142658306437`*^9, 3.640146043505746*^9, 3.666327458924437*^9},
 TextAlignment->Center,ExpressionUUID->"de61ed76-c2e3-4ec3-99fd-789a7e2f05c1"],

Cell[TextData[{
 "Since the charts are bijective maps, the corresponding ",
 StyleBox["coordinate transformation",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", 
    RowBox[{"\[Kappa]", ",", "\[Iota]"}]], TraditionalForm]],ExpressionUUID->
  "48de8d4c-f680-4595-80b4-24fa2f39ba6d"],
 " is defined :"
}], "Text",
 CellChangeTimes->{{3.632030137426557*^9, 3.6320302701827903`*^9}, 
   3.632030301616845*^9, 3.632031420747211*^9, {3.6329147686893673`*^9, 
   3.632914785389391*^9}, 
   3.7002225727474003`*^9},ExpressionUUID->"22211072-c44b-4da7-80fa-\
35c1779f4c1a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Phi]", 
    RowBox[{"\[Kappa]", ",", "\[Iota]"}]], ":=", " ", 
   RowBox[{
    FormBox[
     SubscriptBox["\[Phi]", "\[Kappa]"],
     TraditionalForm], "\[SmallCircle]", 
    SuperscriptBox["\[Phi]\[Iota]", 
     RowBox[{"-", "1"}]]}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.632030366466159*^9, 3.6320304088202333`*^9}, {
   3.6320304761031523`*^9, 3.6320304901743765`*^9}, {3.63203052082843*^9, 
   3.632030588438949*^9}, {3.632030620902606*^9, 3.6320306380782366`*^9}, {
   3.6320306835991163`*^9, 3.632030700915147*^9}, {3.632030774859277*^9, 
   3.632030816183749*^9}, 3.6320308971634912`*^9, 3.632031336319863*^9, {
   3.632031563659062*^9, 3.6320315968559203`*^9}, {3.632031629241577*^9, 
   3.632031635887189*^9}, 3.6329142802496643`*^9, 3.63291432593373*^9, 
   3.666327458924437*^9},
 TextAlignment->Center,ExpressionUUID->"8dfb344e-3c37-4338-91bf-20f0e3b1377e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Phi]", 
    RowBox[{"\[Kappa]", ",", "\[Iota]"}]], ":", 
   RowBox[{
    RowBox[{"{", 
     SuperscriptBox["x", "i"], "}"}], " ", "\[Element]", " ", 
    RowBox[{
     RowBox[{
      FormBox["\[Phi]\[Iota]",
       TraditionalForm], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["U", "\[Kappa]"], " ", "\[Intersection]", " ", 
        SubscriptBox["U", "\[Iota]"]}], ")"}]}], " ", "\[RightTeeArrow]", " ", 
     RowBox[{"{", 
      SuperscriptBox["y", "j"], "}"}]}]}]}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[Phi]", "\[Kappa]"],
      TraditionalForm], "\[SmallCircle]", 
     SuperscriptBox["\[Phi]\[Iota]", 
      RowBox[{"-", "1"}]]}], 
    RowBox[{"(", 
     RowBox[{"{", 
      SuperscriptBox["x", "i"], "}"}], ")"}]}], "\[Element]", " ", 
   RowBox[{
    FormBox[
     SubscriptBox["\[Phi]", "\[Kappa]"],
     TraditionalForm], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["U", "\[Kappa]"], " ", "\[Intersection]", " ", 
       SubscriptBox["U", "\[Iota]"]}], ")"}], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.632030366466159*^9, 3.6320304088202333`*^9}, {
   3.6320304761031523`*^9, 3.6320304901743765`*^9}, {3.63203052082843*^9, 
   3.632030588438949*^9}, {3.632030620902606*^9, 3.6320306380782366`*^9}, {
   3.6320306835991163`*^9, 3.632030700915147*^9}, {3.632030774859277*^9, 
   3.632030816183749*^9}, 3.6320308971634912`*^9, 3.632031336319863*^9, {
   3.632031563659062*^9, 3.6320315756710835`*^9}, 3.6320329324366665`*^9, 
   3.632914347613761*^9, 3.6401460586689725`*^9, {3.6663274589400654`*^9, 
   3.6663274589400654`*^9}},
 TextAlignment->Center,ExpressionUUID->"4a871fae-563e-4209-b8db-54cdc4c4821b"],

Cell[TextData[{
 "The maps are homeomorphisms between open sets in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,ExpressionUUID->
  "88727fe2-2361-4005-9b7f-7007f7a5b028"],
 ".  The ",
 Cell[BoxData[
  StyleBox[
   SubscriptBox["\[Phi]", 
    RowBox[{"\[Kappa]", ",", "\[Iota]"}]],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.632030366466159*^9, 3.6320304088202333`*^9}, {
    3.6320304761031523`*^9, 3.6320304901743765`*^9}, {3.63203052082843*^9, 
    3.632030588438949*^9}, {3.632030620902606*^9, 3.6320306380782366`*^9}, {
    3.6320306835991163`*^9, 3.632030700915147*^9}, {3.632030774859277*^9, 
    3.632030816183749*^9}, 3.6320308971634912`*^9, 3.632031336319863*^9, {
    3.632031563659062*^9, 3.6320315756710835`*^9}, 3.6320329324366665`*^9, 
    3.632914347613761*^9},
  TextAlignment->Center,ExpressionUUID->
  "783bfd05-2f0b-445c-ad0b-0eb8149ee834"],
 " being maps between n-tuples are described by a family of real functions \
with n real variables acting on the components of the n-tuples:"
}], "Text",
 CellChangeTimes->{{3.632031453663269*^9, 3.632031459326079*^9}, {
  3.6320316675552444`*^9, 3.632031791341462*^9}, {3.636438597181491*^9, 
  3.6364386315327516`*^9}, {3.636438674666827*^9, 3.63643872728572*^9}, {
  3.6364387767534065`*^9, 
  3.6364388193414817`*^9}},ExpressionUUID->"503b7910-2fae-486b-af5f-\
8e9efa1fa61f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Phi]", 
     RowBox[{"\[Kappa]", ",", "\[Iota]"}]], ":", 
    RowBox[{"{", " ", 
     RowBox[{"y", "[", "j", "]"}], " ", "}"}]}], "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{"\[Kappa]", ",", "\[Iota]"}]], "[", "j", "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", "...", ",", 
      RowBox[{"x", "[", "n", "]"}]}], "]"}], "}"}]}], ",", " ", 
  RowBox[{"j", " ", "=", " ", "1"}], ",", 
  RowBox[{"..", " ", "n"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6366382844056425`*^9, 3.6366382971664653`*^9}, 
   3.636638578026159*^9, 3.636638614967024*^9, {3.666537377941116*^9, 
   3.6665374360534925`*^9}, 
   3.666537468242653*^9},ExpressionUUID->"fadb6786-29a7-47e1-984d-\
bd430d350815"],

Cell[TextData[{
 "For a correct ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "  definition it  we change the traditional notation. The reason is that \
expressions of the form"
}], "Text",
 CellChangeTimes->{{3.636612514186352*^9, 3.636612583544073*^9}, 
   3.6366384151462727`*^9, {3.6449265400463085`*^9, 3.644926596915409*^9}, {
   3.644926654619194*^9, 3.6449267454964795`*^9}, 3.666607245595911*^9, {
   3.700222745090693*^9, 
   3.700222751506525*^9}},ExpressionUUID->"fff60825-2031-45f5-ab42-\
56c6500567e9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Phi]", 
     RowBox[{"\[Kappa]", ",", "\[Iota]"}]], "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"\[Kappa]", ",", "\[Iota]"}]], "[", "i", "]"}], "[", "x", 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}], 
  "\[LineSeparator]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6366116778220825`*^9, 3.6366117011909237`*^9}, {
   3.6366117506274104`*^9, 3.636611782779067*^9}, {3.636612605072111*^9, 
   3.636612623807744*^9}, {3.6366379543714633`*^9, 3.636638045366423*^9}, {
   3.636638199463494*^9, 3.63663823705956*^9}, 3.636638756237672*^9, {
   3.6366388608202553`*^9, 3.6366388728478765`*^9}, {3.6366389864316764`*^9, 
   3.636639001719703*^9}, 
   3.6628075771899223`*^9},ExpressionUUID->"386496b9-9b8c-4241-a9af-\
ad9baa9e8a05"],

Cell[TextData[{
 " don't define symbols in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".  Thus they don\[CloseCurlyQuote]t act as expected for symbols in often \
used commands:"
}], "Text",
 CellChangeTimes->{{3.6449268332742343`*^9, 3.6449268889743323`*^9}, {
   3.6449269246243954`*^9, 3.6449270125113506`*^9}, 3.6449271732018356`*^9, {
   3.644927236842348*^9, 3.6449272467035656`*^9}, 
   3.6449285571500764`*^9},ExpressionUUID->"cd7124b0-fa0e-4197-952a-\
574eb61e4657"],

Cell[TextData[StyleBox["Example",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6449273656601753`*^9, 
  3.6449273687665815`*^9}},ExpressionUUID->"4a9a1ff6-b2ef-48e5-996e-\
dbad7959887b"],

Cell["The corresponding Output  is entered as text:", "Text",
 CellChangeTimes->{{3.6449282733087764`*^9, 
  3.644928322063862*^9}},ExpressionUUID->"f8120f9c-e8c7-4c30-9666-\
a09f2d958f35"],

Cell[BoxData[
 RowBox[{"??", 
  SubscriptBox["\[Phi]", 
   RowBox[{"\[Kappa]", ",", "\[Iota]"}]]}]], "Input",
 CellChangeTimes->{{3.6449270211069655`*^9, 3.644927047761615*^9}, 
   3.7035745130374823`*^9, {3.7035745691557503`*^9, 3.7035745718037863`*^9}, 
   3.703574646350387*^9},ExpressionUUID->"61da7909-b736-4a77-9e17-\
45f13697adb0"],

Cell[TextData[{
 Cell[BoxData[
  RowBox[{
   StyleBox[
    RowBox[{"Information", "::", "ssym"}], "MessageName"], ":", 
   " ", "\<\"\\\"\[NoBreak]\\!\\(\\n\\*SubscriptBox[\\(\[Phi]\\), \
\\(\[Kappa], \[Iota]\\)]\\)\[NoBreak] is not a symbol or a valid string \
pattern.\"\>"}]], "MSG",
  CellChangeTimes->{3.6449282361123104`*^9},ExpressionUUID->
  "2f9b0026-7df9-4bb8-b569-61727a3def71"],
 "\n",
 Cell[BoxData[
  RowBox[{"Information", "[", 
   RowBox[{
    SubscriptBox["\[Phi]", 
     RowBox[{"\[Kappa]", ",", "\[Iota]"}]], ",", 
    RowBox[{"LongForm", "\[Rule]", "False"}]}], "]"}]],
  CellChangeTimes->{3.644927049103217*^9},ExpressionUUID->
  "ab7c6af7-cc29-4449-b3e7-038ceee0d000"]
}], "Text",
 CellChangeTimes->{{3.644928104199078*^9, 3.644928124123313*^9}, 
   3.6449282562091465`*^9, {3.6449283527199163`*^9, 
   3.6449283767313585`*^9}},ExpressionUUID->"afc10a8b-fa35-4666-9d6d-\
04f0ea2bdce7"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  SubscriptBox["f", 
   RowBox[{"\[Kappa]", ",", "\[Iota]"}]], "]"}]], "Input",
 CellChangeTimes->{{3.644927080244872*^9, 
  3.6449270994993057`*^9}},ExpressionUUID->"103c3f7b-d419-4f42-b7f3-\
c93f92bd79a3"],

Cell[TextData[Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Clear", "::", "ssym"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\!\(\*SubscriptBox[\(f\), \(\[Kappa], \[Iota]\)]\)\
\[NoBreak] is not a symbol or a string. \"\>"}]], "MSG",
 CellChangeTimes->{
  3.6449282361747108`*^9},ExpressionUUID->"186c7315-ea2d-4c2f-9264-\
8799a4672cf4"]], "Text",
 CellChangeTimes->{
  3.6449284073582125`*^9},ExpressionUUID->"777a7fa8-fa57-4cdf-b813-\
15cc0a5a5be0"],

Cell[BoxData[
 RowBox[{"Remove", "[", 
  SubscriptBox["f", 
   RowBox[{"\[Kappa]", ",", "\[Iota]"}]], "]"}]], "Input",
 CellChangeTimes->{{3.644927080244872*^9, 
  3.6449271379417734`*^9}},ExpressionUUID->"29815633-2fac-4efe-b0b3-\
fbb132a873e7"],

Cell[TextData[Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"emove", "::", "ssym"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\!\(\*SubscriptBox[\(f\), \(\[Kappa], \[Iota]\)]\)\
\[NoBreak] is not a symbol.\"\>"}]], "MSG",
 CellChangeTimes->{
  3.644928236221511*^9},ExpressionUUID->"31e466d2-a76b-4c54-8674-\
df1f52e527d7"]], "Text",
 CellChangeTimes->{
  3.644928453741094*^9},ExpressionUUID->"6fe9c71a-2919-429a-b8f3-\
fa96fa829613"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.644927399128235*^9, 
  3.644927399128235*^9}},ExpressionUUID->"a0f23a35-cc24-4d5f-9b64-\
b512ec1fa7d3"],

Cell[TextData[{
 "Here is a better usable ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " object:"
}], "Text",
 CellChangeTimes->{{3.6449273656601753`*^9, 3.6449273687665815`*^9}, {
  3.6449286439878297`*^9, 
  3.64492870647394*^9}},ExpressionUUID->"d74e4c7e-807d-4a7d-855e-\
601d03612932"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
   RowBox[{"{", "x__", "}"}], "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f\[Kappa]\[Iota]", "[", "i", "]"}], "[", "x", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6366116778220825`*^9, 3.6366117011909237`*^9}, {
   3.6366117506274104`*^9, 3.636611782779067*^9}, {3.636612605072111*^9, 
   3.636612623807744*^9}, {3.6366379543714633`*^9, 3.636638045366423*^9}, {
   3.636638199463494*^9, 3.63663823705956*^9}, 3.636638756237672*^9, {
   3.6366388608202553`*^9, 3.6366388728478765`*^9}, {3.6366389864316764`*^9, 
   3.636639001719703*^9}, 3.6449288368469725`*^9, 3.644931605216854*^9, {
   3.666537579576537*^9, 
   3.666537587311308*^9}},ExpressionUUID->"a6e2abbd-90fa-4fbe-8c9c-\
75082344aab1"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]\[Kappa]\[Iota]", "::", "usage"}], " ", "=", 
    "\"\<\[Phi]\[Kappa]\[Iota][{x[1],...,x[n]}]= \[Phi]\[Kappa]\[SmallCircle]\
\!\(\*SuperscriptBox[\(\[Phi]\[Iota]\), \((\(-1\))\)]\)[{x[1],...,x[n]}] is \
the coordinate transformation of the charts \[Phi]\[Iota] \[RightArrow] \
\[Phi]\[Kappa] in the intersection of the chart domains \
\!\(\*SubscriptBox[\(U\), \(\[Kappa]\)]\) \[Intersection] \!\(\*SubscriptBox[\
\(U\), \(\[Iota]\)]\)\[NotEqual]\[EmptySet].\>\""}], ";"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.640060424437602*^9, 3.640060456043257*^9}, {
   3.640060564135847*^9, 3.640060651620801*^9}, {3.6400606855820603`*^9, 
   3.6400607199801207`*^9}, {3.640060768511806*^9, 3.6400607736598153`*^9}, {
   3.640060823595503*^9, 3.6400608578219633`*^9}, {3.6400609223748765`*^9, 
   3.6400609964126062`*^9}, 3.6400610436806893`*^9, {3.640061078718351*^9, 
   3.640061107313201*^9}, {3.6400611376708546`*^9, 3.6400611565468874`*^9}, 
   3.6400612761522975`*^9, 3.6400613075863533`*^9, {3.644928860063813*^9, 
   3.6449288833274546`*^9}, {3.6665376358294554`*^9, 3.666537684831997*^9}, {
   3.666537787431062*^9, 3.666537834824125*^9}, {3.6665378706697435`*^9, 
   3.666537906984112*^9}},ExpressionUUID->"20e13410-a2d5-4953-9fe3-\
cb7f57e11431"],

Cell[BoxData[
 RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
  RowBox[{"vec", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6665379499394827`*^9, 
  3.666537997051298*^9}},ExpressionUUID->"fbb9f95c-7a10-4ded-a3a2-\
ee11849853f9"],

Cell[BoxData[
 RowBox[{"?", "\[Phi]\[Kappa]\[Iota]"}]], "Input",
 CellChangeTimes->{{3.6400613301127925`*^9, 3.64006134582202*^9}, 
   3.6400614143997407`*^9, 
   3.6449289922890463`*^9},ExpressionUUID->"6df6a618-4cbc-4006-8b57-\
5dd229a256cf"],

Cell["The definitions of \[Phi]\[Kappa]\[Iota], \[Phi]\[Iota] imply", "Text",
 CellChangeTimes->{{3.666608443751696*^9, 
  3.6666084940511613`*^9}},ExpressionUUID->"c25d13b2-fa19-4e16-8f05-\
6129678946f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Phi]\[Kappa]", "[", "x", "]"}], " ", "\[Equal]", "  ", 
  RowBox[{
   RowBox[{"\[Phi]\[Kappa]\[Iota]", 
    RowBox[{"@", "*"}], "\[Phi]\[Iota]"}], "@", "x"}]}]], "Input",
 CellChangeTimes->{{3.666607391962864*^9, 3.666607456638089*^9}, {
   3.666607622943575*^9, 3.6666076891188545`*^9}, {3.666608315495038*^9, 
   3.666608364966363*^9}, {3.666608515052264*^9, 3.666608519396242*^9}, 
   3.7002236074098263`*^9, 3.700223648032619*^9},
 EmphasizeSyntaxErrors->
  True,ExpressionUUID->"47ddf962-186b-42a4-a411-0694d73351be"],

Cell["The coordinates of x in the chart \[Phi]\[Kappa] are", "Text",
 CellChangeTimes->{{3.6666086293863115`*^9, 
  3.6666086859517417`*^9}},ExpressionUUID->"f88383d2-8feb-4f70-8b83-\
ef5833128680"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"y", "[", 
    RowBox[{"{", "x__", "}"}], "]"}], "[", "j_", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
    RowBox[{"{", "x", "}"}], "]"}], "[", 
   RowBox[{"[", "j", "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6366118565359964`*^9, 3.6366118639304094`*^9}, {
   3.636611897361268*^9, 3.6366119371413383`*^9}, 3.636612163138935*^9, 
   3.6366122402810707`*^9, {3.636612286488352*^9, 3.636612305473585*^9}, {
   3.6366381070801315`*^9, 3.6366381133201427`*^9}, 3.636639020845337*^9, 
   3.6449294500126534`*^9, {3.666538475576058*^9, 3.666538481404482*^9}, {
   3.6666077988589334`*^9, 3.666607821703868*^9}, {3.6666081320011764`*^9, 
   3.6666081419391937`*^9}},ExpressionUUID->"c09b0872-d7c9-4646-b6eb-\
eb53fffd68d6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"y", "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}], "[", "2", "]"}]], "Input",
 CellChangeTimes->{
  3.636612004720657*^9, {3.63661203506271*^9, 3.636612082112393*^9}, {
   3.6366381344581795`*^9, 3.636638139106988*^9}, {3.666607857549472*^9, 
   3.666607883394564*^9}, {3.666608165424781*^9, 
   3.666608190301055*^9}},ExpressionUUID->"912ba9e2-fce0-48bc-abfc-\
e4f8e15b0b80"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Phi]\[Kappa]", "[", "x", "]"}], "\[Equal]", " ", 
  RowBox[{"vec", "[", 
   RowBox[{"y", "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.666607924177915*^9, 3.6666079284906387`*^9}, {
  3.666608089873982*^9, 3.6666080951086264`*^9}, {3.6666082106927357`*^9, 
  3.666608255476307*^9}},ExpressionUUID->"4db165cd-9c7a-4dd3-850a-\
f977c2c1fc48"],

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"vec", "[", "x", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6665385716279054`*^9, 3.6665386628513975`*^9}, {
  3.666538703744138*^9, 
  3.666538705009822*^9}},ExpressionUUID->"8eadc1db-6ae4-44b5-99e0-\
af651e826143"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
  RowBox[{"vec", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6665385716279054`*^9, 3.6665386628513975`*^9}, {
  3.666538703744138*^9, 3.666538705009822*^9}, {3.6665387860609035`*^9, 
  3.666538812077856*^9}, {3.701921212135972*^9, 
  3.7019212344121437`*^9}},ExpressionUUID->"3aa829e6-1781-4171-9633-\
3be19f4169f8"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.6665388252035375`*^9, 
  3.6665388297350426`*^9}},ExpressionUUID->"c6c005e4-afd2-4c05-917e-\
0a8330278fd9"],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6665388703308835`*^9, 
  3.6665388958165836`*^9}},ExpressionUUID->"8c11a6f6-8dad-4e05-b554-\
0b8e48ef8677"],

Cell[TextData[{
 "For such functions the concepts of differentiability are defined. The \
functions ",
 Cell[BoxData["f\[Kappa]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.632030366466159*^9, 3.6320304088202333`*^9}, {
    3.6320304761031523`*^9, 3.6320304901743765`*^9}, {3.63203052082843*^9, 
    3.632030588438949*^9}, {3.632030620902606*^9, 3.6320306380782366`*^9}, {
    3.6320306835991163`*^9, 3.632030700915147*^9}, {3.632030774859277*^9, 
    3.632030816183749*^9}, 3.6320308971634912`*^9, 3.632031336319863*^9, {
    3.632031563659062*^9, 3.6320315968559203`*^9}, {3.632031629241577*^9, 
    3.632031635887189*^9}, {3.6320318351307387`*^9, 3.632031956420952*^9}},
  TextAlignment->Center,ExpressionUUID->
  "8efdc50a-976f-48eb-8469-55f0aa6c8c45"],
 "[j] are said to be of ",
 StyleBox["class ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "4264750c-18a9-49fe-b884-2bb66fefcd43"],
 ", if all their partial derivatives up to order k with respect to the \
coordinates exist and are continuous. Here k may have the values k = 0,1,...,\
\[Infinity], where k=0 simply means the continuity of the functions and k =\
\[Infinity], that all derivatives exist and are continuous. Besides of these \
the class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Omega]"], TraditionalForm]],ExpressionUUID->
  "54d858c4-28db-4479-a5bc-d57fefd98ede"],
 " is considered which consists of all real analytic functions. A ",
 StyleBox["differentiable n-dimensional manifold of class ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "f6dd1121-035d-4629-87d4-3951715fe46e"],
 " is by definition an n-dimensional manifold on which an atlas is \
distinguished whose coordinate transformations are all of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "105b5e7d-7b1f-4668-8958-b1273bd329df"],
 ". Two such atlases  of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "904c8eba-1263-479d-8512-a284944d2384"],
 " are said to be ",
 StyleBox["equivalent",
  FontColor->RGBColor[0, 0, 1]],
 " if their union is an atlas of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "a029de52-09e3-4dca-97f5-96ee2cb9b12e"],
 " again; the union of all atlases of an equivalence class is  the",
 StyleBox[" complete atlas",
  FontColor->RGBColor[0, 0, 1]],
 " of the differentiable manifold."
}], "Text",
 CellChangeTimes->{{3.6320321284424543`*^9, 3.6320321756013365`*^9}, {
   3.6320322135874033`*^9, 3.632032338356423*^9}, {3.632032369104077*^9, 
   3.632032733754717*^9}, {3.6320327776531944`*^9, 3.6320328621585426`*^9}, {
   3.632032900909011*^9, 3.632032900909011*^9}, {3.6320329490818954`*^9, 
   3.6320329490818954`*^9}, {3.632032981529952*^9, 3.632033146952643*^9}, {
   3.6320378354704556`*^9, 3.6320378877833476`*^9}, {3.6322930613615036`*^9, 
   3.6322931625652823`*^9}, {3.632293193506137*^9, 3.6322932802894907`*^9}, {
   3.6329149500546265`*^9, 3.6329149718386574`*^9}, 3.636606853412009*^9, {
   3.6366996147740183`*^9, 3.6366997064709797`*^9}, {3.6366997477330523`*^9, 
   3.6366997509154577`*^9}, {3.6366998491522255`*^9, 
   3.6366998612266464`*^9}, {3.6449317525419135`*^9, 
   3.6449317525419135`*^9}, {3.6666118300098953`*^9, 3.6666118348539033`*^9}, 
   3.700229215105521*^9},ExpressionUUID->"e4ace768-f644-4408-b7c7-\
b6a74b99f163"],

Cell[TextData[{
 "The importance of the concept of  differentiable manifolds consists in the \
possibility, that for them  the strong tools of real analysis nay be applied \
and generalized, as often done in elementary local considerations in surface \
theory or classical physics, simply considering a situation in  a given \
coordinate system. The introduction of an atlas is necessary for solving \
global problems concerning the geometric objects in the whole. Geometric \
properties must be invariant under coordinate transformations. To take this \
into account the consideration of these transformations is unavoidable.  \
Often the adaption of a coordinate system to a given situation such that the \
describing functions or equations are as simple as possible is a deciding \
step in solving the problem. Joining a new chart to a given atlas is allowed \
only if the new enlarged atlas belongs to the same class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "4719b83d-1f10-4bac-9e77-9c7cc9787aef"],
 " as  the given. "
}], "Text",
 CellChangeTimes->{{3.632033730639104*^9, 3.632033860793534*^9}, {
   3.632033925574848*^9, 3.6320340750843124`*^9}, {3.632034108193571*^9, 
   3.632034113858381*^9}, {3.6320341758712587`*^9, 3.632034210271319*^9}, {
   3.632034306256688*^9, 3.63203438651723*^9}, {3.6320346477699842`*^9, 
   3.632034665478015*^9}, {3.632035492389676*^9, 3.6320355002540903`*^9}, {
   3.6320356536927614`*^9, 3.6320356953352346`*^9}, {3.6320358046782274`*^9, 
   3.6320360321374297`*^9}, {3.6320360628432837`*^9, 3.632036186323902*^9}, {
   3.6366999888230724`*^9, 3.636699998089489*^9}, 
   3.6367000471519756`*^9},ExpressionUUID->"bc3ec288-a9bb-4572-b0c8-\
e860fe75e19b"],

Cell[TextData[{
 "Sometimes, for finer analytic considerations, it is unavoidable to fix the \
class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "72d7e848-58ef-4728-878e-6741f8b5d93a"],
 " of differentiability. Moreover, for some existence proofs additional \
assumptions about the topology of the manifold are necessary, look into the \
cited monographs.  For our purposes such finer considerations are not \
necessary. Always if we consider a differentiable function we assume that all \
the derivatives we use exist and are continuous as functions of the \
coordinates. At any rate this is true in the class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Infinity]"], TraditionalForm]],ExpressionUUID->
  "b1cb89da-ee14-4acf-be00-685720982049"],
 ".  In the following we use the word ",
 StyleBox["\[OpenCurlyDoubleQuote]smooth\[CloseCurlyDoubleQuote]",
  FontColor->RGBColor[0, 0, 1]],
 " in the sense \[OpenCurlyDoubleQuote]of class  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Infinity]"], TraditionalForm]],ExpressionUUID->
  "f0d77beb-c898-41ca-8b79-5adaddc86690"],
 "\[CloseCurlyDoubleQuote]. "
}], "Text",
 CellChangeTimes->{{3.6320362743552575`*^9, 3.6320362818764706`*^9}, {
   3.6320363383553953`*^9, 3.6320366102248755`*^9}, 3.6320367995808096`*^9, 
   3.63203710561275*^9, {3.6320372077943306`*^9, 3.6320372261751633`*^9}, {
   3.6320372574280186`*^9, 3.6320372807560596`*^9}, 3.6320373391871085`*^9, {
   3.632037894023359*^9, 3.6320379277546177`*^9}, {3.636607107614455*^9, 
   3.6366071100168595`*^9}, {3.636607151200932*^9, 3.6366072590283213`*^9}, {
   3.6367001569745693`*^9, 3.636700168300189*^9}, 3.639920158970461*^9, 
   3.6449147243153057`*^9, 3.700229501059389*^9, {3.700229576906723*^9, 
   3.700229613930443*^9}, {3.7002297907767887`*^9, 3.7002297950333633`*^9}, {
   3.700229854224886*^9, 3.700229933688561*^9}, {3.700229983568112*^9, 
   3.700230026113471*^9}, {3.700230058111466*^9, 3.7002300836715097`*^9}, {
   3.700230158679043*^9, 3.700230212079021*^9}, {3.700230263582974*^9, 
   3.700230265222157*^9}},ExpressionUUID->"b95d796d-0552-472c-a6e5-\
5debfb5cc219"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "2.1.2. The ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,ExpressionUUID->
  "d3b0adbf-4178-4d01-ac06-822fb9f083ea"],
 " and the n-Spheres ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["S", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "c6f19394-1d97-41d7-b22c-05de99478dee"]
}], "Subsubsection",
 CellChangeTimes->{{3.636786782220746*^9, 3.636786814575203*^9}, {
   3.6367868534816713`*^9, 3.6367868886129327`*^9}, {3.666612341176992*^9, 
   3.666612343442734*^9}, 3.666621090424732*^9, {3.7002302853241863`*^9, 
   3.7002302905636883`*^9}, 
   3.70028958429333*^9},ExpressionUUID->"4f23d230-3d20-441d-816f-\
972c90b11eaf"],

Cell[TextData[{
 StyleBox["Example 1. The real spaces ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "80434784-eb9f-473b-979e-ee29065a9bcd"],
 " themselves are differentiable manifolds of arbitrary class; the smallest \
atlas consists of one chart, namely the identity map.  A larger atlas of \
class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Omega]"], TraditionalForm]],ExpressionUUID->
  "759ea877-20cd-4285-b562-c1068b2fe068"],
 " consists of all Cartesian coordinates. Joining all charts whose \
transformations with the Cartesian coordinates are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "42b457fd-c58b-45b3-909b-891fe8c19f3e"],
 " yields a complete atlas of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "15880b94-58c9-49e1-9f4a-0666af6e6c28"],
 " for the ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,ExpressionUUID->
  "23f75e1e-d2c4-47d9-8c61-66396a1d6564"],
 ". \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.632038022378785*^9, 3.6320380411964183`*^9}, {
  3.6320380725730734`*^9, 3.632038330408529*^9}, {3.632038456200017*^9, 
  3.632038603808877*^9}, {3.6320386411477427`*^9, 3.632038746560529*^9}, {
  3.6320388745661554`*^9, 3.6320388799013643`*^9}, {3.632140987678393*^9, 
  3.6321409946984053`*^9}, {3.6321410517477055`*^9, 3.6321410542125096`*^9}, {
  3.632202592015008*^9, 3.632202596868617*^9}, {3.6322028187589116`*^9, 
  3.632202823220519*^9}, {3.700279676140215*^9, 
  3.700279679372272*^9}},ExpressionUUID->"edc38707-abba-472f-ab39-\
599df1ab0d7b"],

Cell[TextData[{
 StyleBox["Example 2. The n-spheres ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["S", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "7f1e551b-b658-424e-a606-b8da87856e00"],
 " (see Subsection 1.2)  have an analytic atlas consisting of two charts. One \
chart is the stereographic projection from the North pole N = stb[n+1] of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["S", "n"], TraditionalForm]],ExpressionUUID->
  "478c198f-6d4e-4e96-8b80-07a3fb6f321e"],
 " onto the whole space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,ExpressionUUID->
  "afd57004-1480-466c-b7eb-27b94824d2a8"],
 ". The domain of this chart is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["S", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "d1e1d711-41b7-465e-b834-250f1e3f2ced"],
 " \\{N}:"
}], "Text",
 CellChangeTimes->{{3.632038950067689*^9, 3.6320390102323947`*^9}, {
  3.63203904151545*^9, 3.6320390685230975`*^9}, {3.6320391798180943`*^9, 
  3.632039185714905*^9}, {3.6321411653159046`*^9, 3.632141269555288*^9}, {
  3.632141323141382*^9, 3.632141427396365*^9}, {3.632141459860022*^9, 
  3.6321414817000604`*^9}, {3.63214151545852*^9, 3.63214156673581*^9}, {
  3.6377294094817147`*^9, 3.6377294279697475`*^9}, {3.666189180292927*^9, 
  3.6661891808601923`*^9}, {3.6661892347555723`*^9, 3.666189237747621*^9}, {
  3.666189295795836*^9, 3.666189301228312*^9}, {3.70027978390687*^9, 
  3.7002797849548683`*^9}},ExpressionUUID->"30d016f5-c011-4189-947d-\
8a438697fde3"],

Cell[BoxData[
 RowBox[{"?", "sterproj"}]], "Input",
 CellChangeTimes->{{3.63203919460892*^9, 
  3.6320392003653307`*^9}},ExpressionUUID->"582a96fa-2573-4bed-823c-\
676635aac83a"],

Cell["The inverse map is", "Text",
 CellChangeTimes->{{3.6321416059230785`*^9, 
  3.632141612974291*^9}},ExpressionUUID->"b64f2663-054e-44d8-bcef-\
99d3bf23d495"],

Cell[BoxData[
 RowBox[{"?", "invsterproj"}]], "Input",
 CellChangeTimes->{{3.6320436850229993`*^9, 
  3.6320436923082123`*^9}},ExpressionUUID->"b1679efc-9dda-4b65-a87f-\
0ddb9283cd78"],

Cell[TextData[{
 "We consider the case n=2, higher dimensions may be treated in the same way. \
We start with the inverse map ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["f1", 
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "7efca7f5-211f-4b7b-af15-0d0844506b7d"],
 " of the first chart being defined on the whole plane ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "2"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631889130606888*^9, 3.631889130606888*^9}, {
   3.6318891635425463`*^9, 3.6318892755013437`*^9}, {3.631889398677561*^9, 
   3.631889413686787*^9}},
  TextAlignment->Center,ExpressionUUID->
  "156f290e-5e45-4fbc-88d5-d570c9eb1593"],
 ":"
}], "Text",
 CellChangeTimes->{{3.632141710256062*^9, 3.6321417240932865`*^9}, {
   3.632141813465843*^9, 3.6321418389562883`*^9}, {3.632141916426024*^9, 
   3.6321419434452715`*^9}, {3.632141973990125*^9, 3.632142023489012*^9}, {
   3.6321421101783648`*^9, 3.632142110412365*^9}, 3.6322028514585695`*^9, {
   3.666613467105177*^9, 
   3.6666134962941856`*^9}},ExpressionUUID->"4f6994a6-4bc7-44bb-8eb8-\
18c45ec88210"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.632201297627926*^9, 
  3.6322013019803333`*^9}},ExpressionUUID->"63b56bcc-3745-4e60-a57e-\
11284f059d8d"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"invsterproj", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.63203956830518*^9, 3.6320395827060056`*^9}, {
   3.6320396631537476`*^9, 3.6320396729223647`*^9}, 3.6320397146428385`*^9, {
   3.632039843149666*^9, 3.6320398486096754`*^9}, {3.632040065987259*^9, 
   3.632040067828062*^9}, {3.632042993080978*^9, 3.6320429995745897`*^9}, {
   3.632120820001009*^9, 
   3.6321208402498446`*^9}},ExpressionUUID->"0a8a6444-2b4f-4a43-80a9-\
125b78d8e026"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ".", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.632120877986311*^9, 
  3.632120895442742*^9}},ExpressionUUID->"978df08a-67b4-4ebb-a9ac-\
52d59ee269cb"],

Cell[TextData[{
 "Obviously, the North pole {0,0,1} of the sphere does not belong to the \
range of this map. we define ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["f1", 
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "45f3bdbc-b8f4-4256-b47f-da7e2c500538"],
 "\[DotEqual]invf1 by"
}], "Text",
 CellChangeTimes->{{3.6666137599015884`*^9, 3.666613813310604*^9}, {
   3.666613845030997*^9, 3.666613922425629*^9}, 
   3.7002831610572166`*^9},ExpressionUUID->"59523128-ef4b-4ab4-8860-\
525f1233e72d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"invf1", "[", 
   RowBox[{"{", 
    RowBox[{"x_", ",", "y_"}], "}"}], "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    FractionBox[
     RowBox[{"2", " ", "x"}], 
     RowBox[{"1", "+", 
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["y", "2"]}]], ",", 
    FractionBox[
     RowBox[{"2", " ", "y"}], 
     RowBox[{"1", "+", 
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["y", "2"]}]], ",", 
    RowBox[{"1", "-", 
     FractionBox["2", 
      RowBox[{"1", "+", 
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["y", "2"]}]]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6320393232687473`*^9, 3.632039468403204*^9}, {
   3.63203950592527*^9, 3.6320395197956944`*^9}, {3.6320398116628103`*^9, 
   3.6320398194784236`*^9}, {3.6320400022647467`*^9, 
   3.6320400381508102`*^9}, {3.632040075144475*^9, 3.6320401036653256`*^9}, {
   3.632061380505129*^9, 3.6320614084019785`*^9}, {3.6320642513329926`*^9, 
   3.6320642623622117`*^9}, {3.632120055724467*^9, 3.632120074007699*^9}, {
   3.632120303842903*^9, 3.632120325807741*^9}, {3.6321203655878115`*^9, 
   3.6321203702054195`*^9}, 3.6321205366733117`*^9, {3.6321210536582193`*^9, 
   3.632121069804248*^9}, 3.632121102205505*^9, 3.6321211351683626`*^9, {
   3.6321241074259834`*^9, 3.6321241284704204`*^9}, 
   3.6321420689942923`*^9},ExpressionUUID->"49299524-6786-4ebd-b617-\
e0eb6cc470cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", "=", "15"}], ";", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{"invf1", "[", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"-", "c"}], ",", "c"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", "c"}], ",", "c"}], "}"}], ",", 
    RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}], ",", " ", 
    RowBox[{"PlotRange", "\[Rule]", " ", "All"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6321212051656857`*^9, 3.6321213570475526`*^9}, {
   3.6321214156256557`*^9, 3.6321214341116877`*^9}, {3.6321241395152397`*^9, 
   3.632124147081253*^9}, {3.6321251089321423`*^9, 3.6321251116621475`*^9}, {
   3.6321252104259205`*^9, 3.632125228397152*^9}, {3.6321421838260937`*^9, 
   3.632142229892975*^9}, {3.666613987850882*^9, 3.6666140456663885`*^9}, 
   3.70028324971518*^9},ExpressionUUID->"532e14a6-6440-440d-b952-\
aaf3bdcd941a"],

Cell["\<\
Plot this figure for increasing values of c; for c -> Infinity one gets the \
stated domain of the chart f1, the complement of\
\>", "Text",
 CellChangeTimes->{{3.632142736152725*^9, 3.632142817631069*^9}, {
  3.6666141022005625`*^9, 
  3.6666141118104353`*^9}},ExpressionUUID->"cd2df1ac-b4d3-4a6f-9590-\
415bec0f538d"],

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"invf1", "[", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}], "]"}], ",", 
   RowBox[{"x", "\[Rule]", " ", "Infinity"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.632121467745347*^9, 3.632121526900651*^9}, {
  3.6321241580948725`*^9, 
  3.6321241660352864`*^9}},ExpressionUUID->"dce0a7f4-9ba5-49e9-9210-\
9956a53c4c99"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "f1", "]"}], ";", 
  RowBox[{
   RowBox[{"f1", "[", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "y_", ",", "z_"}], "}"}], "]"}], " ", ":=", " ", 
   RowBox[{"sterproj", "[", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.632121701667758*^9, 3.632121796125924*^9}, {
   3.6321218603200364`*^9, 3.632121869617653*^9}, {3.6321219439361835`*^9, 
   3.6321219747618375`*^9}, {3.6321220169443116`*^9, 
   3.6321221142104826`*^9}, {3.632122180494999*^9, 3.6321222014458356`*^9}, {
   3.6321226038953424`*^9, 3.632122610977755*^9}, 3.6666141636412396`*^9, {
   3.6666199412246027`*^9, 
   3.666619943896617*^9}},ExpressionUUID->"997a0895-d0a8-434b-ba1a-\
369719798c91"],

Cell["\<\
This map is  valid for points of the unit sphere only:
x^2+x^2+z^2 =1.
The half sphere below the equator plane is mapped onto the disk bordered by \
the equator:\
\>", "Text",
 CellChangeTimes->{{3.6321431819997125`*^9, 3.6321431957921367`*^9}, {
   3.63214335272797*^9, 3.6321434024316573`*^9}, {3.632200206962002*^9, 
   3.632200227446838*^9}, 3.6322028557641764`*^9, {3.666620260616119*^9, 
   3.666620344542357*^9}, {3.7002834394152803`*^9, 3.7002834439111147`*^9}, {
   3.7002835330141706`*^9, 
   3.700283534198121*^9}},ExpressionUUID->"a99cdda6-08cf-4d80-9c13-\
fc69e8903cde"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "y", ",", " ", 
    RowBox[{"-", 
     RowBox[{"Sqrt", "[", 
      RowBox[{"1", "-", 
       RowBox[{"x", "^", "2"}], "-", 
       RowBox[{"y", "^", "2"}]}], "]"}]}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321221432421336`*^9, 3.632122151572548*^9}, {
   3.632122213879058*^9, 3.632122287401987*^9}, {3.6321226221005745`*^9, 
   3.6321226294637876`*^9}, 3.6321441758200207`*^9, 
   3.6321442513331537`*^9},ExpressionUUID->"1360eb1f-427c-45b1-bff8-\
338d78052b05"],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{"f1", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"r", "*", 
       RowBox[{"Cos", "[", "t", "]"}]}], ",", 
      RowBox[{"r", "*", 
       RowBox[{"Sin", "[", "t", "]"}]}], ",", 
      RowBox[{"-", 
       RowBox[{"Sqrt", "[", 
        RowBox[{"1", "-", 
         RowBox[{"r", "^", "2"}]}], "]"}]}]}], "}"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.63214364221448*^9, 3.632143868181679*^9}, {
  3.632144064123424*^9, 3.6321441478295717`*^9}, {3.6321442134668875`*^9, 
  3.6321442234353046`*^9}, {3.6321442687155848`*^9, 
  3.6321442997636395`*^9}},ExpressionUUID->"ff869e2f-d477-4f37-9ba8-\
2a1e425a6355"],

Cell["The points of the equator are mapped onto itself:", "Text",
 CellChangeTimes->{{3.6321434143052783`*^9, 
  3.632143442605728*^9}},ExpressionUUID->"609588fb-5b32-411b-8210-\
5561b50bd19b"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "y", ",", "0"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321221432421336`*^9, 3.632122151572548*^9}, {
  3.632122213879058*^9, 3.6321223157004366`*^9}, {3.632122642848611*^9, 
  3.6321226489638214`*^9}},ExpressionUUID->"c3daf031-1654-4132-b456-\
cd317a0a4675"],

Cell["\<\
The half sphere above the equator is mapped outside the disk bordered by the \
equator:\
\>", "Text",
 CellChangeTimes->{{3.6321434815141964`*^9, 3.63214357997797*^9}, 
   3.6322028579013805`*^9},ExpressionUUID->"9b3e9824-027f-4077-b668-\
0296df0be610"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "y", ",", " ", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"1", "-", 
      RowBox[{"x", "^", "2"}], "-", 
      RowBox[{"y", "^", "2"}]}], "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321221432421336`*^9, 3.632122151572548*^9}, {
  3.632122213879058*^9, 3.6321222747659645`*^9}, {3.6321227119411325`*^9, 
  3.6321227473063946`*^9}},ExpressionUUID->"4dc1b5d4-1840-4783-8a11-\
781b319ca0e5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", "=", ".1"}], ";", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{"f1", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"r", "*", 
        RowBox[{"Cos", "[", "t", "]"}]}], ",", 
       RowBox[{"r", "*", 
        RowBox[{"Sin", "[", "t", "]"}]}], ",", 
       RowBox[{"Sqrt", "[", 
        RowBox[{"1", "-", 
         RowBox[{"r", "^", "2"}]}], "]"}]}], "}"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "c", ",", "1"}], "}"}], ",", 
    RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.63214364221448*^9, 3.632143868181679*^9}, {
   3.632144064123424*^9, 3.6321441478295717`*^9}, {3.6321442134668875`*^9, 
   3.6321442234353046`*^9}, {3.6321442687155848`*^9, 
   3.6321442997636395`*^9}, {3.6321443655101557`*^9, 
   3.6321444037020226`*^9}, {3.6321445321354494`*^9, 
   3.6321445440578704`*^9}, {3.632144579380333*^9, 3.6321446495728564`*^9}, {
   3.6321446795933094`*^9, 3.6321447357302084`*^9}, {3.6321447680720654`*^9, 
   3.6321448181199536`*^9}, 3.6321448555826197`*^9, {3.632145063258586*^9, 
   3.6321450653957896`*^9}, {3.632145095788643*^9, 3.632145115448678*^9}, 
   3.6322004174479733`*^9, {3.632200464256056*^9, 
   3.6322004860688944`*^9}},ExpressionUUID->"6eb70c81-04e4-41f6-b4af-\
8ca4e7a5b7b7"],

Cell["At the North pole itself f1 is not defined:", "Text",
 CellChangeTimes->{{3.632144879559862*^9, 3.6321449309699526`*^9}, {
  3.666620436406492*^9, 
  3.666620448110219*^9}},ExpressionUUID->"7b758477-2377-44c8-9621-\
27c20854e910"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0", ",", "1"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.632144936913563*^9, 
  3.6321449519091897`*^9}},ExpressionUUID->"2103cb9c-405c-434c-bea0-\
3e2dd26ce52f"],

Cell["The South pole S= -{0,0,1} is mapped to the origin:", "Text",
 CellChangeTimes->{{3.6322006582331977`*^9, 3.6322007039804783`*^9}, {
  3.6322007577577734`*^9, 3.632200785951023*^9}, {3.632200828890299*^9, 
  3.6322008492483344`*^9}, {3.6661896432081633`*^9, 
  3.666189664631888*^9}},ExpressionUUID->"56056e78-f89c-474e-af71-\
3b661ef5b234"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0", ",", 
    RowBox[{"-", "1"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.632144936913563*^9, 3.6321449519091897`*^9}, {
  3.6322006441775727`*^9, 
  3.632200649715583*^9}},ExpressionUUID->"b2712fc8-30d8-4b48-94f3-\
5f0232816419"],

Cell["f1 and invf1 are inverse to each other :", "Text",
 CellChangeTimes->{{3.6321452870065804`*^9, 3.632145310644622*^9}, 
   3.6321453822255483`*^9},ExpressionUUID->"466883f6-6d81-446e-80e3-\
508adc171e70"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"invf1", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.632122354014104*^9, 3.6321223688497295`*^9}, {
  3.6321242067045574`*^9, 
  3.632124213256569*^9}},ExpressionUUID->"75821b7e-0f4b-486a-8d11-\
932d1c140a53"],

Cell[BoxData[
 RowBox[{"invf1", "[", 
  RowBox[{"f1", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321451532983446`*^9, 
  3.632145185922002*^9}},ExpressionUUID->"348ee08e-3b73-497b-b83c-\
974d02c2989a"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "^", "2"}], "+", 
     RowBox[{"y", "^", "2"}], "+", 
     RowBox[{"z", "^", "2"}]}], "\[Equal]", "1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6321452150824537`*^9, 
  3.63214524716011*^9}},ExpressionUUID->"e67935ec-8635-4a02-8a8b-\
0d779f9e41c8"],

Cell["\<\
To get a second chart f2 defined also at the North pole we reflect the sphere \
at the equator plane and the apply the stereographic projection. We again \
start with the inverse map\
\>", "Text",
 CellChangeTimes->{{3.6322009337656837`*^9, 3.632201098507374*^9}, 
   3.6322028623025885`*^9},ExpressionUUID->"3029407b-4f8f-440a-9111-\
34ea64ff951c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "invf2", "]"}], ";", 
  RowBox[{
   RowBox[{"invf2", "[", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "y_"}], "}"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     FractionBox[
      RowBox[{"2", " ", "x"}], 
      RowBox[{"1", "+", 
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["y", "2"]}]], ",", 
     FractionBox[
      RowBox[{"2", " ", "y"}], 
      RowBox[{"1", "+", 
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["y", "2"]}]], ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      FractionBox["2", 
       RowBox[{"1", "+", 
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["y", "2"]}]]}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.6320393232687473`*^9, 3.632039468403204*^9}, {
   3.63203950592527*^9, 3.6320395197956944`*^9}, {3.6320398116628103`*^9, 
   3.6320398194784236`*^9}, {3.6320400022647467`*^9, 
   3.6320400381508102`*^9}, {3.632040075144475*^9, 3.6320401036653256`*^9}, {
   3.632061380505129*^9, 3.6320614084019785`*^9}, {3.6320642513329926`*^9, 
   3.6320642623622117`*^9}, {3.632120055724467*^9, 3.632120074007699*^9}, {
   3.632120303842903*^9, 3.632120325807741*^9}, {3.6321203655878115`*^9, 
   3.6321203702054195`*^9}, 3.6321205366733117`*^9, {3.6321210536582193`*^9, 
   3.632121069804248*^9}, 3.632121102205505*^9, 3.6321211351683626`*^9, {
   3.6321241074259834`*^9, 3.6321241284704204`*^9}, {3.632124296763516*^9, 
   3.6321243249371653`*^9}},ExpressionUUID->"7c655ab4-a1a4-4424-ae8a-\
d336346c9e0a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", "=", "3"}], ";", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{"invf2", "[", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"-", "c"}], ",", "c"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", 
      RowBox[{"-", "c"}], ",", "c"}], "}"}], ",", 
    RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}], ",", " ", 
    RowBox[{"PlotRange", "\[Rule]", " ", "All"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6321212051656857`*^9, 3.6321213570475526`*^9}, {
   3.6321214156256557`*^9, 3.6321214341116877`*^9}, {3.6321241395152397`*^9, 
   3.632124147081253*^9}, 3.6321243363719854`*^9, {3.63212436730684*^9, 
   3.6321243942792873`*^9}},ExpressionUUID->"35b9c5be-8310-46da-8923-\
26ee8aedc32e"],

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"invf2", "[", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}], "]"}], ",", 
   RowBox[{"x", "\[Rule]", " ", "Infinity"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.632121467745347*^9, 3.632121526900651*^9}, {
   3.6321241580948725`*^9, 3.6321241660352864`*^9}, 
   3.6321244158385253`*^9},ExpressionUUID->"b2404933-c726-46fd-8676-\
0267ef46112c"],

Cell["The chart f2 is defined by", "Text",
 CellChangeTimes->{{3.6322011944177437`*^9, 
  3.6322012056965632`*^9}},ExpressionUUID->"39a0baee-222b-4ec3-93a2-\
78777e04f8c1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "f2", "]"}], ";", 
  RowBox[{
   RowBox[{"f2", "[", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "y_", ",", "z_"}], "}"}], "]"}], " ", ":=", " ", 
   RowBox[{"sterproj", "[", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", 
      RowBox[{"-", "z"}]}], "}"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.632121701667758*^9, 3.632121796125924*^9}, {
  3.6321218603200364`*^9, 3.632121869617653*^9}, {3.6321219439361835`*^9, 
  3.6321219747618375`*^9}, {3.6321220169443116`*^9, 3.6321221142104826`*^9}, {
  3.632122180494999*^9, 3.6321222014458356`*^9}, {3.6321226038953424`*^9, 
  3.632122610977755*^9}, {3.632124433060955*^9, 3.632124455181794*^9}, {
  3.6321456893334894`*^9, 
  3.6321456976483045`*^9}},ExpressionUUID->"fb2db198-1598-4812-83de-\
75e25384673b"],

Cell[BoxData[
 RowBox[{"f2", "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0", ",", "1"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.632201238776622*^9, 3.6322012713846793`*^9}, {
  3.6322014093361225`*^9, 
  3.6322014161885347`*^9}},ExpressionUUID->"2ed3b6b7-5058-4eb9-9d93-\
30886a1fdc9d"],

Cell["\<\
Now the half sphere below the equator plane is mapped outside of the equator:\
\>", "Text",
 CellChangeTimes->{{3.632201642970935*^9, 3.6322017022268395`*^9}, 
   3.632202900867856*^9},ExpressionUUID->"0be5db9f-31ad-4c81-a0d3-\
c4010a60d334"],

Cell[BoxData[
 RowBox[{"f2", "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "y", ",", " ", 
    RowBox[{"-", 
     RowBox[{"Sqrt", "[", 
      RowBox[{"1", "-", 
       RowBox[{"x", "^", "2"}], "-", 
       RowBox[{"y", "^", "2"}]}], "]"}]}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321221432421336`*^9, 3.632122151572548*^9}, {
   3.632122213879058*^9, 3.632122287401987*^9}, {3.6321226221005745`*^9, 
   3.6321226294637876`*^9}, 
   3.6321244685978174`*^9},ExpressionUUID->"fc44f6bd-8e0b-4052-a65d-\
58842f185d20"],

Cell[BoxData[
 RowBox[{
  RowBox[{"c", "=", " ", ".3"}], ";", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{"f2", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"r", "*", 
        RowBox[{"Cos", "[", "t", "]"}]}], ",", 
       RowBox[{"r", "*", 
        RowBox[{"Sin", "[", "t", "]"}]}], ",", 
       RowBox[{"-", 
        RowBox[{"Sqrt", "[", 
         RowBox[{"1", "-", 
          RowBox[{"r", "^", "2"}]}], "]"}]}]}], "}"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "c", ",", "1"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.63214364221448*^9, 3.632143868181679*^9}, {
   3.632144064123424*^9, 3.6321441478295717`*^9}, {3.6321442134668875`*^9, 
   3.6321442234353046`*^9}, {3.6321442687155848`*^9, 3.6321442997636395`*^9}, 
   3.6322015513399734`*^9, {3.632201584462832*^9, 
   3.632201616166088*^9}},ExpressionUUID->"a55fdda3-d4a4-4e0d-9f3e-\
22867fa511f0"],

Cell[BoxData[
 RowBox[{"f2", "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "y", ",", "0"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321221432421336`*^9, 3.632122151572548*^9}, {
   3.632122213879058*^9, 3.6321223157004366`*^9}, {3.632122642848611*^9, 
   3.6321226489638214`*^9}, 
   3.6321244861166487`*^9},ExpressionUUID->"1069c15a-4881-4082-91cf-\
4d53f1a65d34"],

Cell["\<\
The half sphere above the equator plane is mapped onto the disk bordered by \
the equator:\
\>", "Text",
 CellChangeTimes->{{3.6322017386081038`*^9, 3.632201800267412*^9}, 
   3.6322029038942614`*^9},ExpressionUUID->"3d5cab4d-1135-435d-9e47-\
a902c94e0fd7"],

Cell[BoxData[
 RowBox[{"f2", "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "y", ",", " ", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"1", "-", 
      RowBox[{"x", "^", "2"}], "-", 
      RowBox[{"y", "^", "2"}]}], "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321221432421336`*^9, 3.632122151572548*^9}, {
   3.632122213879058*^9, 3.6321222747659645`*^9}, {3.6321227119411325`*^9, 
   3.6321227473063946`*^9}, 
   3.6321244987838707`*^9},ExpressionUUID->"f47825e6-19b8-4540-b119-\
bae40562ccac"],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{
   RowBox[{"f2", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"r", "*", 
       RowBox[{"Cos", "[", "t", "]"}]}], ",", 
      RowBox[{"r", "*", 
       RowBox[{"Sin", "[", "t", "]"}]}], ",", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"1", "-", 
        RowBox[{"r", "^", "2"}]}], "]"}]}], "}"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.63214364221448*^9, 3.632143868181679*^9}, {
  3.632144064123424*^9, 3.6321441478295717`*^9}, {3.6321442134668875`*^9, 
  3.6321442234353046`*^9}, {3.6321442687155848`*^9, 3.6321442997636395`*^9}, {
  3.632201844825091*^9, 
  3.6322018558739104`*^9}},ExpressionUUID->"6ebe4028-73e5-4b18-ba8d-\
8f2e8f5f8c64"],

Cell[BoxData[
 RowBox[{"f2", "[", 
  RowBox[{"invf2", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.632122354014104*^9, 3.6321223688497295`*^9}, {
  3.6321242067045574`*^9, 3.632124213256569*^9}, {3.632124520639509*^9, 
  3.6321245299527254`*^9}},ExpressionUUID->"891f5ee4-1679-4ce7-817d-\
cc0fde78de52"],

Cell["\<\
The coordinate transformations are both the reflection at the unit circle:\
\>", "Text",
 CellChangeTimes->{{3.632202123758383*^9, 3.6322021645554543`*^9}, 
   3.632202916657084*^9},ExpressionUUID->"ca131b83-7593-4474-a5b6-\
8d96cf9e9f72"],

Cell[BoxData[
 RowBox[{"f2", "[", 
  RowBox[{"invf1", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.632122354014104*^9, 3.6321223688497295`*^9}, {
  3.6321242067045574`*^9, 3.632124213256569*^9}, {3.632124520639509*^9, 
  3.6321245299527254`*^9}, {3.632125975217264*^9, 
  3.6321259840156794`*^9}},ExpressionUUID->"c4b82d5b-d61f-4067-b254-\
3fc06eb49cb0"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"invf2", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6321260057153177`*^9, 3.632126021393345*^9}, {
  3.666620718046077*^9, 
  3.6666207270309153`*^9}},ExpressionUUID->"5ee56e23-c0bb-44ca-9a5a-\
b52f42b8a5e7"],

Cell["\<\
Both open sets, the domains of the coordinate transformations, coincide with \
the complement of the origin. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.6322252358134737`*^9, 
  3.632225310163204*^9}},ExpressionUUID->"68b5157a-7c54-4da4-ae94-\
6be42124729a"],

Cell[TextData[{
 "An atlas of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "bff21691-88ff-45ea-bea1-138c9f7f3c69"],
 " , k>0, is ",
 StyleBox["oriented",
  FontColor->RGBColor[0, 0, 1]],
 ", if all its coordinate transformations have positive Jacobian determinant; \
a differentiable manifold is said to be ",
 StyleBox["orientable",
  FontColor->RGBColor[0, 0, 1]],
 " if  there exists an oriented subatlas of its complete atlas.  Two oriented \
at atlases are said to be ",
 StyleBox["equivalent",
  FontColor->RGBColor[0, 0, 1]],
 " if their union is an oriented atlas again; each equivalence class of \
oriented atlases is an ",
 StyleBox["orientation",
  FontColor->RGBColor[0, 0, 1]],
 " of the orientable manifold. To fix an orientation it suffices to \
distinguish an oriented atlas of the orientable manifold. A connected \
orientable manifold has exactly two orientations; the transition between them \
is achieved by inverting the orientation of all charts of an oriented atlas \
e.g. by permuting two of the coordinate functions of the chart."
}], "Text",
 CellChangeTimes->{{3.6322924362440853`*^9, 3.6322924407056932`*^9}, {
   3.6322924855443935`*^9, 3.632292580287162*^9}, {3.632292871775569*^9, 
   3.6322929498501067`*^9}, {3.6322934424847775`*^9, 3.632293540939951*^9}, {
   3.632293637412922*^9, 3.6322936982338295`*^9}, {3.6322937455451126`*^9, 
   3.632293746871115*^9}, {3.6322938121848307`*^9, 3.632294073361692*^9}, {
   3.632294149927027*^9, 3.632294233683975*^9}, 3.632294339738363*^9, {
   3.6322943958128614`*^9, 3.632294396577263*^9}, 3.666620872272812*^9, 
   3.6666209031337833`*^9, {3.700289238528552*^9, 3.700289249577261*^9}, {
   3.700289310562294*^9, 
   3.7002893270101147`*^9}},ExpressionUUID->"025f729f-99d9-44fd-8b65-\
bcb1950343a9"],

Cell["\<\
The atlas constructed in the last example is  not oriented. The coordinate \
transformation is\
\>", "Text",
 CellChangeTimes->{{3.632294454916766*^9, 3.6322945354269085`*^9}, {
  3.6661900703646927`*^9, 
  3.666190081492836*^9}},ExpressionUUID->"56244ab9-9f95-4d1e-82d2-\
d00413a3c8a9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f2$1", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    FractionBox["x", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["y", "2"]}]], ",", 
    FractionBox["y", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["y", "2"]}]]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6322254144805875`*^9, 3.6322254850083113`*^9}, 
   3.632291555154538*^9, 3.6322915913154016`*^9, {3.632291635510279*^9, 
   3.6322916688631377`*^9}, 
   3.632291703635599*^9},ExpressionUUID->"38f404b7-10aa-4502-93ef-\
05efb9809935"],

Cell["It has the Jacobian matrix", "Text",
 CellChangeTimes->{{3.632294556769746*^9, 
  3.632294568894967*^9}},ExpressionUUID->"60dfba30-2d59-4458-8630-\
c0fdad9d55b7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"jac", "[", "f2$1", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.6322254921843243`*^9, 3.6322255176591687`*^9}, 
   3.632291724773636*^9},ExpressionUUID->"9d601cc9-40ed-4df1-9a26-\
92dee37d3bd8"],

Cell["whose determinant is negative :", "Text",
 CellChangeTimes->{{3.6322945820477905`*^9, 
  3.632294610305441*^9}},ExpressionUUID->"bb4c511c-5ba8-42d6-a30f-\
ecd9f06486a9"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Det", "[", "%", "]"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["y", "2"]}], ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.632225538188805*^9, 
  3.6322256112437334`*^9}},ExpressionUUID->"409f561c-516f-4542-864b-\
3e656da8fec7"],

Cell["\<\
The sphere is orientable. We obtain an oriented atlas replacing f2 by the \
chart \
\>", "Text",
 CellChangeTimes->{{3.632294682482168*^9, 
  3.6322947479976835`*^9}},ExpressionUUID->"0c400c2f-b3e9-4bf8-a629-\
3a3d420ef43b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f3", "[", 
   RowBox[{"{", 
    RowBox[{"x_", ",", "y_", ",", "z_"}], " ", "}"}], "]"}], " ", ":=", " ", 
  RowBox[{"f2", "[", 
   RowBox[{"{", 
    RowBox[{"y", ",", "x", ",", "z"}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6322257007098904`*^9, 3.632225704063896*^9}, {
  3.6322257765572233`*^9, 3.632225823107705*^9}, {3.6322266161350985`*^9, 
  3.632226658208372*^9}},ExpressionUUID->"a22dd5fd-aa32-437d-abbb-\
1d6a0f7f1ddd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"invf3", "[", 
   RowBox[{"{", 
    RowBox[{"x_", ",", "y_"}], "}"}], "]"}], " ", ":=", " ", 
  RowBox[{"invf2", "[", 
   RowBox[{"{", 
    RowBox[{"y", ",", "x"}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.632225953913935*^9, 3.6322260190284495`*^9}, {
  3.632226671078395*^9, 
  3.6322266917172313`*^9}},ExpressionUUID->"7cbcdef1-3e07-448b-b531-\
72ddcdd5e72d"],

Cell[BoxData[
 RowBox[{"f3", "[", 
  RowBox[{"invf3", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6322262763353014`*^9, 
  3.6322262796581073`*^9}},ExpressionUUID->"a377952b-f661-4804-8327-\
e5c728ccab44"],

Cell["The new coordinate transformation for the atlas {f1, f3} is", "Text",
 CellChangeTimes->{{3.6322951500013933`*^9, 
  3.6322951996164813`*^9}},ExpressionUUID->"8efacf69-871b-4ea5-9c88-\
d4bea2d7ec59"],

Cell[BoxData[
 RowBox[{"f3", "[", 
  RowBox[{"invf1", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.632226247412851*^9, 3.6322262582548695`*^9}, 
   3.6322267134636693`*^9},ExpressionUUID->"a9dad553-d839-405c-848b-\
93112a9b7c99"],

Cell[BoxData[
 RowBox[{"f1", "[", 
  RowBox[{"invf3", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.632295316781088*^9, 
  3.63229534614434*^9}},ExpressionUUID->"24ab5b4e-a6eb-40a5-8892-\
edd13df92b73"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f3$1", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    FractionBox["y", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["y", "2"]}]], ",", 
    FractionBox["x", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["y", "2"]}]]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6322267853485956`*^9, 3.632226786736998*^9}, {
   3.63222691305042*^9, 3.6322269554824944`*^9}, 
   3.6322951345377665`*^9},ExpressionUUID->"966dffeb-7553-4ac7-a052-\
004f5cb96ced"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Det", "[", 
    RowBox[{
     RowBox[{"jac", "[", "f3$1", "]"}], "[", 
     RowBox[{"x", ",", "y"}], "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["y", "2"]}], ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.632226729094897*^9, 3.6322267396405153`*^9}, {
   3.63222697588733*^9, 3.6322270841515207`*^9}, 
   3.63229522160292*^9},ExpressionUUID->"1844a4d8-d6fa-4324-8709-\
13eff44cee35"],

Cell["The atlas {f3, f1} is oriented. \[FilledSquare]", "Text",
 CellChangeTimes->{{3.632295398836633*^9, 3.632295454582532*^9}, {
  3.6666210607825685`*^9, 
  3.66662107342383*^9}},ExpressionUUID->"e7a21303-7e03-4432-ad87-\
07530109472e"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.1.3. The Category of Differentiable Manifolds", "Subsubsection",
 CellChangeTimes->{{3.6367869235725946`*^9, 3.636786980216294*^9}, 
   3.700289590205969*^9, 
   3.7003659647095613`*^9},ExpressionUUID->"38955002-8be9-4962-a642-\
97f4e7e107d5"],

Cell[TextData[{
 "Let f : ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "7fa863f2-d597-46d3-9795-c2c727e1db7f"],
 " \[LongRightArrow] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]],ExpressionUUID->
  "2fa82e8c-851a-49fa-8432-05c2c9204f3e"],
 " be a map between differentiable manifolds. Then its coordinate ",
 "representative"
}], "Text",
 CellChangeTimes->{{3.6322030546311274`*^9, 3.6322030824655766`*^9}, {
  3.6322031187552404`*^9, 3.632203121407245*^9}, {3.6322052224904127`*^9, 
  3.6322053904447083`*^9}, {3.6325775454818707`*^9, 
  3.6325775561798897`*^9}},ExpressionUUID->"a24031f4-ee12-41c4-8530-\
9c609349adae"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", 
   RowBox[{"\[Kappa]", ",", "\[Iota]"}]], ":=", 
  RowBox[{
   SubscriptBox["\[Upsilon]", "\[Kappa]"], "\[SmallCircle]", "f", 
   "\[SmallCircle]", 
   RowBox[{
    SubsuperscriptBox["\[Phi]", "\[Iota]", 
     RowBox[{"-", "1"}]], ":", 
    RowBox[{"\[Phi]\[Iota]", 
     RowBox[{
      RowBox[{"(", 
       SubscriptBox["U", "\[Iota]"], ")"}], "\[LongRightArrow]", 
      SubscriptBox["\[Upsilon]", "\[Kappa]"]}], 
     RowBox[{"(", 
      SubscriptBox["V", "\[Kappa]"], ")"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6322053940171146`*^9, 3.6322055147929277`*^9}, {
   3.632205559134206*^9, 3.6322056519487696`*^9}, {3.6322056928736415`*^9, 
   3.6322057688275757`*^9}, {3.6322058005142317`*^9, 3.632205925813052*^9}, {
   3.632205968840928*^9, 3.6322060256476283`*^9}, 3.6322065657210307`*^9, 
   3.632206599655091*^9, 3.666327458971315*^9},
 TextAlignment->Center,ExpressionUUID->"27b3477b-63fb-4466-95cf-bd46b5c94b36"],

Cell[TextData[{
 "are maps between open sets of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],ExpressionUUID->
  "805d03f3-84e8-486c-b98e-8335197af968"],
 " into open sets of  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "m"], TraditionalForm]],ExpressionUUID->
  "2e7f3299-b7ba-43de-a872-413daabcc45e"],
 ".  The map",
 StyleBox[" f is said to be of class ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "9bc1317b-652b-40c2-9bfb-cad867b50cad"],
 ", if all its coordinate representatives ",
 Cell[BoxData[
  StyleBox[
   SubscriptBox["f", 
    RowBox[{"\[Kappa]", ",", "\[Iota]"}]],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6322053940171146`*^9, 3.6322055147929277`*^9}, {
    3.632205559134206*^9, 3.6322056519487696`*^9}, {3.6322056928736415`*^9, 
    3.6322057688275757`*^9}, {3.6322058005142317`*^9, 3.632205925813052*^9}, {
    3.632205968840928*^9, 3.6322060256476283`*^9}, 3.6322065657210307`*^9},
  TextAlignment->Center,ExpressionUUID->
  "fb99209b-3061-4d53-82e0-6e61b8e92cb9"],
 " for all charts ",
 Cell[BoxData["\[Phi]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6322053940171146`*^9, 3.6322055147929277`*^9}, {
    3.632205559134206*^9, 3.6322056519487696`*^9}, {3.6322056928736415`*^9, 
    3.6322057688275757`*^9}, {3.6322058005142317`*^9, 3.632205925813052*^9}, {
    3.632205968840928*^9, 3.6322060256476283`*^9}, 3.6322065657210307`*^9, 
    3.632206599655091*^9},
  TextAlignment->Center,ExpressionUUID->
  "cba8462d-ca6e-4243-b7ea-054d8e438bfd"],
 " of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "9f742612-eb74-4e59-84ae-74aaf096f64c"],
 " and ",
 Cell[BoxData[
  SubscriptBox["\[Upsilon]", "\[Kappa]"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6322053940171146`*^9, 3.6322055147929277`*^9}, {
    3.632205559134206*^9, 3.6322056519487696`*^9}, {3.6322056928736415`*^9, 
    3.6322057688275757`*^9}, {3.6322058005142317`*^9, 3.632205925813052*^9}, {
    3.632205968840928*^9, 3.6322060256476283`*^9}, 3.6322065657210307`*^9, 
    3.632206599655091*^9},
  TextAlignment->Center,ExpressionUUID->
  "1801f8fc-fc47-4422-a312-9026e80c398e"],
 " of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", "m"], TraditionalForm]],ExpressionUUID->
  "846da19c-5e0e-4201-ab7b-3e374070a593"],
 " are k-fold continuously differentiable. This property of f is correctly \
defined if the manifolds themselves are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "5e961454-1045-477c-b376-0774188dfdd0"],
 " at least. \nSince the coordinate transformations",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  SubscriptBox["\[Upsilon]", 
   RowBox[{"\[Lambda]", ",", "\[Kappa]"}]]],
  Evaluatable->False,
  CellChangeTimes->{{3.6322073317825775`*^9, 3.632207387215475*^9}, {
   3.632207419027931*^9, 3.632207465974414*^9}, {3.632207516620103*^9, 
   3.6322075911962347`*^9}},
  TextAlignment->Center,
  FontWeight->"Bold",ExpressionUUID->"1e10814f-ed97-4c6d-89c8-a7887e8301e6"],
 StyleBox[", ",
  FontWeight->"Bold"],
 Cell[BoxData[
  SubscriptBox["\[Phi]", 
   RowBox[{"\[Iota]", ",", "\[Mu]"}]]],
  Evaluatable->False,
  CellChangeTimes->{{3.6322073317825775`*^9, 3.632207387215475*^9}, {
   3.632207419027931*^9, 3.632207465974414*^9}, {3.632207516620103*^9, 
   3.6322075911962347`*^9}},
  TextAlignment->Center,
  FontWeight->"Bold",ExpressionUUID->"e5112483-71a4-4d4f-969b-c1e5bf45ffd6"],
 " of both manifolds are of  class  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "8693d5d2-b991-43a6-a346-2d89de40338f"],
 ", also the transformed coordinate representatives of  f"
}], "Text",
 CellChangeTimes->{{3.632206072982112*^9, 3.632206129804412*^9}, {
   3.632206170086683*^9, 3.632206204566744*^9}, {3.6322062360359993`*^9, 
   3.632206237564802*^9}, {3.6322062876946907`*^9, 3.6322063628158226`*^9}, {
   3.6322064343295994`*^9, 3.6322064557816377`*^9}, {3.6322064953609076`*^9, 
   3.6322065042217226`*^9}, 3.632206583444662*^9, {3.632206621542929*^9, 
   3.6322067671871862`*^9}, {3.632206815178871*^9, 3.632206815459671*^9}, {
   3.6322068550585413`*^9, 3.632206917747452*^9}, 3.632206988542972*^9, {
   3.6322071323364253`*^9, 3.632207218959778*^9}, {3.6322072542354403`*^9, 
   3.6322072788874836`*^9}, 3.632207610559869*^9, {3.6322078076766167`*^9, 
   3.632207829674655*^9}, {3.6325775783631287`*^9, 3.6325775960847597`*^9}, {
   3.6329151526819167`*^9, 3.63291516225193*^9}, 3.63591882082285*^9, 
   3.666190766462776*^9, {3.6663274589869423`*^9, 3.6663274589869423`*^9}, 
   3.700291613514975*^9},ExpressionUUID->"1df99378-6c88-4db7-949a-\
71a0af520d73"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", 
   RowBox[{"\[Lambda]", ",", "\[Mu]"}]], " ", "=", " ", 
  RowBox[{
   SubscriptBox["\[Upsilon]", 
    RowBox[{"\[Lambda]", ",", "\[Kappa]"}]], "\[SmallCircle]", 
   SubscriptBox["f", 
    RowBox[{"\[Kappa]", ",", "\[Iota]"}]], "\[SmallCircle]", 
   SubscriptBox["\[Phi]", 
    RowBox[{"\[Iota]", ",", "\[Mu]"}]]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6322073317825775`*^9, 3.632207387215475*^9}, {
  3.632207419027931*^9, 3.632207465974414*^9}, {3.632207516620103*^9, 
  3.6322075911962347`*^9}, {3.632915110074856*^9, 3.632915119323869*^9}},
 TextAlignment->Center,ExpressionUUID->"4bd4f1b6-a90f-4c0c-bced-5595116a56ef"],

Cell[TextData[{
 "are k - times continuously differentiable. Thus the defined property of f \
is invariant under coordinate transformations. Since the identity maps and \
the composition of maps of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "beaf2432-8ed0-4a1d-bc05-a074ec4653e1"],
 " are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "59f0a17f-a2cc-42c4-8a75-23193bf11045"],
 " too, the manifolds of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "b737badc-3503-43d2-9b08-863480a4ba4e"],
 " with the maps of class",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["C", "k"]}], TraditionalForm]],ExpressionUUID->
  "9fed724d-6900-4bad-9b4f-f95f6f3266f9"],
 " form a ",
 ButtonBox["category.",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/Category.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/Category.html"]
}], "Text",
 CellChangeTimes->{{3.632207765332142*^9, 3.632207772289754*^9}, {
  3.6322078487086887`*^9, 3.6322079379684467`*^9}, {3.6322079810461226`*^9, 
  3.632208114709558*^9}, {3.632298671306418*^9, 3.6322986972564106`*^9}, {
  3.63237715774813*^9, 3.63237715774813*^9}, {3.700291680593987*^9, 
  3.700291719902152*^9}},ExpressionUUID->"6e947f95-e0c8-4832-a67d-\
0c2a6d30f5b0"],

Cell[TextData[{
 StyleBox["General assumption and notations",
  FontColor->RGBColor[0, 0, 1]],
 ".\nIn the following, if not stated explicitly otherwise, we assume:\nThere \
is given a smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "3c61079e-c4c6-41c3-b846-f17fe3e38d58"],
 " of dimension n = dim with an atlas ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"{", "\[Phi]\[Iota]", "}"}], 
    RowBox[{"\[Iota]", "\[Element]", "\[ScriptCapitalJ]"}]], 
   TraditionalForm]],ExpressionUUID->"fd55ae76-973c-46fa-bb71-d6d913907d6a"],
 ". The small Greek indices \[Iota] \[Element] \[ScriptCapitalJ] distinguish \
the charts. They shouldn\[CloseCurlyQuote]t be confused with the small Latin \
indices being related to the coordinates x[i] of a fixed chart. Any chart ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "44c6244f-258f-42a1-8e92-471e961bb718"],
 " defines a coordinate system on its open domain ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "973b2d32-a0a5-4e71-b67a-7542b591d6e4"],
 " \[Subset] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "fff09a21-5923-4783-a859-48f377a1f9ad"],
 ". The point  x \[Element]",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["U", "\[Iota]"]}], TraditionalForm]],ExpressionUUID->
  "38239ce0-34ac-4045-8ab3-5f1857f7d548"],
 " has the coordinates "
}], "Text",
 CellChangeTimes->{{3.63991980165423*^9, 3.639919902265807*^9}, {
   3.6399199455598836`*^9, 3.639920003116385*^9}, {3.639920038361847*^9, 
   3.6399201182941895`*^9}, {3.639920192727321*^9, 3.6399202970803056`*^9}, {
   3.63992033365277*^9, 3.6399204012792892`*^9}, {3.6399205111078835`*^9, 
   3.6399205385835323`*^9}, {3.6399206037372465`*^9, 
   3.6399206037372465`*^9}, {3.6399207192246504`*^9, 3.639920756263116*^9}, {
   3.6399208042270017`*^9, 3.639920840843266*^9}, {3.639920927604244*^9, 
   3.6399210100407896`*^9}, {3.639921218420618*^9, 3.639921280690729*^9}, {
   3.6399213437998405`*^9, 3.63992141720397*^9}, {3.6399214861660914`*^9, 
   3.639921556280615*^9}, {3.6399216187725315`*^9, 3.639921659386403*^9}, {
   3.639922222110799*^9, 3.639922259210664*^9}, {3.639922293690725*^9, 
   3.639922308543951*^9}, {3.6399223426828113`*^9, 3.63992241587054*^9}, {
   3.639922685201619*^9, 3.6399227696057673`*^9}, {3.6399763977817125`*^9, 
   3.6399765216615295`*^9}, {3.6399765551235886`*^9, 3.63997661293729*^9}, 
   3.639976722823883*^9, 3.6492226235826654`*^9, {3.666327382795497*^9, 
   3.666327382795497*^9}, {3.6663274589869423`*^9, 
   3.6663274589869423`*^9}},ExpressionUUID->"7801a86f-5bbc-44a2-a0ac-\
c89802ca0090"],

Cell[BoxData[
 RowBox[{"\[Phi]\[Iota]", "[", "x", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6399216636784105`*^9, 3.6399216788416376`*^9}, {
   3.6399217447275534`*^9, 3.639921783312422*^9}, 3.639921818177884*^9, {
   3.6399218688049726`*^9, 3.639921872112179*^9}, {3.639921951713519*^9, 
   3.6399219573295293`*^9}, {3.639922075400938*^9, 3.639922145191663*^9}, {
   3.6663271497209506`*^9, 3.6663271511272697`*^9}, 3.6663273049477153`*^9, {
   3.6666130697408442`*^9, 
   3.6666130919451184`*^9}},ExpressionUUID->"16e17323-4efb-4273-a86c-\
ca271eed84d8"],

Cell[BoxData[
 RowBox[{"?", "\[Phi]\[Iota]"}]], "Input",
 CellChangeTimes->{{3.6666131003049264`*^9, 
  3.666613126587538*^9}},ExpressionUUID->"3ec82258-0aef-4843-b2f4-\
3fa4133bb13d"],

Cell["In the present setting we have the dimension", "Text",
 CellChangeTimes->{{3.639922496728284*^9, 
  3.639922569614013*^9}},ExpressionUUID->"ee66fd10-1cfe-4705-8998-\
a05883d976ba"],

Cell[BoxData["dim"], "Input",ExpressionUUID->"256e393e-71c7-42f2-b979-a4aa613994aa"],

Cell[TextData[{
 "We consider the important special case m=1, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["X", 
    RowBox[{"1", " "}]], TraditionalForm]],ExpressionUUID->
  "a5dab622-c3d4-4c1c-bbf1-38e94541b97b"],
 "= R the field of the real numbers with the identity map as the single chart \
of the atlas.  Let  \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "030543f3-3676-4561-a598-fcdff433623e"],
 "  be the set of smooth real  functions on  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "7cf3ddda-edef-4ae2-953d-ee35c102921f"],
 ". Their coordinate representatives are given by "
}], "Text",
 CellChangeTimes->{{3.632542221082301*^9, 3.632542419218249*^9}, {
   3.632542490057973*^9, 3.6325426485854516`*^9}, 3.632542958729596*^9, 
   3.6325776004527674`*^9, {3.6330589594240017`*^9, 3.633058981934841*^9}, 
   3.636436127837554*^9, {3.6364361671808233`*^9, 3.636436179442445*^9}, {
   3.6364389782277603`*^9, 
   3.636438982845369*^9}},ExpressionUUID->"8005adbe-ffd4-4676-b87f-\
610411ff7a9a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f\[Iota]", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["x", "1"], ",", "...", ",", 
      SuperscriptBox["x", "n"]}], ")"}]}], " ", ":=", " ", 
   RowBox[{"f", 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[Phi]", "\[Iota]", 
       RowBox[{"-", "1"}]], 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "1"], ",", "...", ",", 
        SuperscriptBox["x", "n"]}], ")"}]}], " ", ")"}]}]}], ",", "   ", 
  RowBox[{"f", "\[Element]", 
   RowBox[{"\[ScriptCapitalF]", 
    FormBox[
     SuperscriptBox["M", "n"],
     TraditionalForm]}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.632542675526699*^9, 3.6325427203611774`*^9}, {
   3.632542769080063*^9, 3.6325429251427374`*^9}, {3.6325429886816487`*^9, 
   3.6325430310045233`*^9}, 3.6330590135716968`*^9, 3.644923865924056*^9},
 TextAlignment->Center,ExpressionUUID->"0748a434-dcbe-406d-8a93-2a770dae632b"],

Cell[TextData[{
 "The function f \[Element] \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "20b4e859-d24d-49a5-9b96-003f290c003f"],
 " doesn\[CloseCurlyQuote]t depend on the coordinates. Conversely,  one often \
defines f for \nx \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "5c53547b-fd7a-4dd8-835b-62924c51382d"],
 "  by its coordinate representatives ",
 Cell[BoxData["f\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.632542675526699*^9, 3.6325427203611774`*^9}, {
    3.632542769080063*^9, 3.6325429251427374`*^9}, {3.6325429886816487`*^9, 
    3.6325430310045233`*^9}, 3.6330590135716968`*^9},
  TextAlignment->Center,ExpressionUUID->
  "b913b6d8-f0f2-4877-82ae-4b65f830c69b"],
 " as shown in the"
}], "Text",
 CellChangeTimes->{{3.639977103604952*^9, 3.6399771407642174`*^9}, {
   3.639977171184271*^9, 3.6399772195599556`*^9}, {3.6399772798384614`*^9, 
   3.6399773299457493`*^9}, {3.639977365669812*^9, 3.639977444527951*^9}, {
   3.644922470753194*^9, 3.644922485261219*^9}, {3.6449238561272383`*^9, 
   3.6449238561272383`*^9}, {3.6449239068081284`*^9, 
   3.6449239105677347`*^9}, {3.6449240432629414`*^9, 3.644924185831993*^9}, {
   3.644925558612178*^9, 3.644925574649006*^9}, 
   3.644925620515087*^9},ExpressionUUID->"8444435c-7667-40b3-9816-\
085f53140f92"],

Cell[TextData[{
 " ",
 StyleBox["Example",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.639977103604952*^9, 3.6399771407642174`*^9}, {
  3.639977171184271*^9, 3.6399772195599556`*^9}, {3.6399772798384614`*^9, 
  3.6399773299457493`*^9}, {3.639977365669812*^9, 3.639977444527951*^9}, {
  3.644922470753194*^9, 3.644922485261219*^9}, {3.6449238561272383`*^9, 
  3.6449238561272383`*^9}, {3.6449239068081284`*^9, 3.6449239105677347`*^9}, {
  3.6449240432629414`*^9, 3.644924185831993*^9}, {3.644925558612178*^9, 
  3.644925574649006*^9}, {3.644925620515087*^9, 
  3.644925628705101*^9}},ExpressionUUID->"67e7135c-5da0-4a84-82c2-\
517a9696ffbb"],

Cell[BoxData[
 RowBox[{"?", "var"}]], "Input",
 CellChangeTimes->{{3.700292230895769*^9, 
  3.700292233087472*^9}},ExpressionUUID->"7af43380-8bcb-4f79-88b2-\
42ea0eda47e3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f\[Iota]", "[", 
   RowBox[{"x1_", ",", "x2_", ",", "x3_"}], "]"}], " ", ":=", 
  RowBox[{"x3", "*", 
   RowBox[{"Cos", "[", 
    RowBox[{"x1", "*", "x2"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6449225196330795`*^9, 3.6449226154000487`*^9}, {
   3.644922717093028*^9, 3.6449227225374374`*^9}, {3.64492279767517*^9, 
   3.644922806602386*^9}, 3.6449228485382595`*^9, 3.644923191600665*^9, {
   3.644923231603135*^9, 3.6449232708412046`*^9}, {3.6449233032630615`*^9, 
   3.6449233357307186`*^9}, {3.644923631117642*^9, 
   3.644923634175247*^9}},ExpressionUUID->"a592e2f5-1f11-4356-a2a9-\
584c14a90c8f"],

Cell[BoxData[
 RowBox[{"f\[Iota]", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.644923368805777*^9, {3.644923680027728*^9, 
   3.644923683506534*^9}},ExpressionUUID->"533155ac-b1da-4314-9468-\
b9395fcff90b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], ":=", 
  RowBox[{"f\[Iota]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6449231377423697`*^9, 3.644923153704198*^9}, {
   3.644923405191041*^9, 3.644923462724145*^9}, 
   3.6449237097809806`*^9},ExpressionUUID->"05584663-723a-4920-b2d5-\
03f68f2d31c4"],

Cell[BoxData[
 RowBox[{"f", "[", "x", "]"}]], "Input",
 CellChangeTimes->{{3.6449227535542927`*^9, 3.6449227623059072`*^9}, {
  3.6449234845349836`*^9, 
  3.6449234924129972`*^9}},ExpressionUUID->"db3d7b06-c603-49a8-92e2-\
7b1fd66de3c5"],

Cell[TextData[{
 "valid for x \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "75bad070-288e-4e78-b59f-7652bddaf9a9"],
 ", the domain of the  chart ",
 Cell[BoxData["\[Phi]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6322053940171146`*^9, 3.6322055147929277`*^9}, {
    3.632205559134206*^9, 3.6322056519487696`*^9}, {3.6322056928736415`*^9, 
    3.6322057688275757`*^9}, {3.6322058005142317`*^9, 3.632205925813052*^9}, {
    3.632205968840928*^9, 3.6322060256476283`*^9}, 3.6322065657210307`*^9, 
    3.632206599655091*^9},
  TextAlignment->Center,ExpressionUUID->
  "6f101453-3db0-4bd1-8e7a-bd67cc84934d"],
 ". "
}], "Text",
 CellChangeTimes->{{3.632543116570674*^9, 3.6325431216718826`*^9}, {
   3.6325431526379366`*^9, 3.6325431827147894`*^9}, {3.632543312257417*^9, 
   3.6325433594631*^9}, {3.632543470706896*^9, 3.632543480659713*^9}, {
   3.632543688093277*^9, 3.632543772879426*^9}, 3.6325440567843246`*^9, 
   3.632813638210785*^9, 3.639924035445595*^9, {3.6399244581963415`*^9, 
   3.63992447460957*^9}, 3.639924507311228*^9, {3.639977568360968*^9, 
   3.6399775707945724`*^9}, {3.6399776289202747`*^9, 3.6399777639073114`*^9}, 
   3.6449259123938017`*^9, {3.666327434860709*^9, 
   3.666327434860709*^9}},ExpressionUUID->"aed93678-4f36-4ecf-8bda-\
dca47835d953"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"f", ",", "f\[Iota]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6449259253574247`*^9, 3.6449259507114697`*^9}, {
  3.666621959563483*^9, 
  3.6666219622511005`*^9}},ExpressionUUID->"83bad27b-25a2-4d0f-ba1e-\
bf8b8ebd4bbf"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.632543116570674*^9, 3.6325431216718826`*^9}, {
   3.6325431526379366`*^9, 3.6325431827147894`*^9}, {3.632543312257417*^9, 
   3.6325433594631*^9}, {3.632543470706896*^9, 3.632543480659713*^9}, {
   3.632543688093277*^9, 3.632543772879426*^9}, 3.6325440567843246`*^9, 
   3.632813638210785*^9, 3.639924035445595*^9, {3.6399244581963415`*^9, 
   3.63992447460957*^9}, 3.639924507311228*^9, {3.639977568360968*^9, 
   3.6399775707945724`*^9}, {3.6399776289202747`*^9, 3.6399777639073114`*^9}, 
   3.6449259123938017`*^9, {3.644925969589503*^9, 3.644925993024744*^9}, {
   3.6449302361540394`*^9, 3.644930239866846*^9}, {3.70031058691295*^9, 
   3.7003106025928698`*^9}, 
   3.70036305118114*^9},ExpressionUUID->"79709955-c57d-4f60-bd6d-\
190a19a21324"],

Cell[TextData[{
 "A coordinate  transformation  in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.632542675526699*^9, 3.6325427203611774`*^9}, {
   3.632542769080063*^9, 3.6325429251427374`*^9}, {3.6325429886816487`*^9, 
   3.6325430310045233`*^9}},
  TextAlignment->Center,ExpressionUUID->
  "c1ab5f8d-823e-4a7a-a741-41fce57ea6c7"],
 " for  x \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "da0a5844-77d2-4429-a1ec-41a2ab080581"],
 "\[Intersection]",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Kappa]"], TraditionalForm]],ExpressionUUID->
  "1e2c8906-a749-4e06-bbc1-c40cda9ebbc0"],
 " yields the transformation of the coordinate representatives of the \
functions"
}], "Text",
 CellChangeTimes->{{3.632543116570674*^9, 3.6325431216718826`*^9}, {
   3.6325431526379366`*^9, 3.6325431827147894`*^9}, {3.632543312257417*^9, 
   3.6325433594631*^9}, {3.632543470706896*^9, 3.632543480659713*^9}, {
   3.632543688093277*^9, 3.632543772879426*^9}, 3.6325440567843246`*^9, 
   3.632813638210785*^9, {3.639978283575424*^9, 3.639978286149429*^9}, {
   3.639978319861088*^9, 
   3.6399783899832115`*^9}},ExpressionUUID->"1e332206-5ce5-453b-b503-\
ee111374bb1a"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["f", "\[Kappa]"], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["f", "\[Iota]"], "\[SmallCircle]", 
     SubscriptBox["\[Phi]", 
      RowBox[{"\[Iota]", ",", "\[Kappa]"}]]}], "  ", "for", " ", "x"}], 
   "\[Element]", 
   RowBox[{
    FormBox[
     SubscriptBox["U", "\[Iota]"],
     TraditionalForm], "\[Intersection]", 
    RowBox[{
     FormBox[
      SubscriptBox["U", "\[Kappa]"],
      TraditionalForm], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6325438231583147`*^9, 3.6325438989588475`*^9}, 
   3.6329867040228395`*^9, {3.639978571052729*^9, 3.639978579461144*^9}},
 TextAlignment->Center,ExpressionUUID->"258305ee-72e6-4d9c-acdc-2cc9ac0e951b"],

Cell[TextData[{
 "Thus the coordinate representatives  of the function f are related to each \
other by this rule. We consider the coordinate transformations ",
 Cell[BoxData[
  StyleBox[
   SubscriptBox["\[Phi]", 
    RowBox[{"\[Iota]", ",", "\[Kappa]"}]],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6325438231583147`*^9, 3.6325438989588475`*^9}, 
    3.6329867040228395`*^9, {3.639978571052729*^9, 3.639978579461144*^9}},
  TextAlignment->Center,ExpressionUUID->
  "b601e5e4-2dbb-4759-9e1c-9ca2639db8e7"],
 " detailed in the next Subsection."
}], "Text",
 CellChangeTimes->{{3.63997842912368*^9, 3.639978496843399*^9}, {
  3.6399786201772156`*^9, 
  3.639978729783008*^9}},ExpressionUUID->"39b13252-8189-4883-b231-\
404e153f06c4"],

Cell[TextData[{
 "The algebraic operations in the field R and the derivation rules for real \
functions admit the definition of the following operations in \
\[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.632542675526699*^9, 3.6325427203611774`*^9}, {
   3.632542769080063*^9, 3.6325429251427374`*^9}, {3.6325429886816487`*^9, 
   3.6325430310045233`*^9}},
  TextAlignment->Center,ExpressionUUID->
  "31564256-e3f7-40c2-937f-24257fe9df01"],
 ":\n1. The addition of functions:"
}], "Text",
 CellChangeTimes->{{3.632544115175227*^9, 3.6325443718580785`*^9}, {
   3.633058682039914*^9, 3.633058682039914*^9}, 3.633242166892674*^9, {
   3.633242222693972*^9, 3.6332422631760435`*^9}, {3.6399787777998924`*^9, 
   3.6399788029003363`*^9}},ExpressionUUID->"9c98862a-8107-400b-b02b-\
c5fd4a2bbe3b"],

Cell[BoxData[
 RowBox[{"f", ",", " ", 
  RowBox[{"h", " ", "\[Element]", " ", 
   RowBox[{
    RowBox[{"\[ScriptCapitalF]", 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}], "\[RightTeeArrow]", " ", 
    RowBox[{"plus", "[", 
     RowBox[{"f", ",", "h"}], "]"}]}], " ", "\[Element]", " ", 
   RowBox[{"\[ScriptCapitalF]", 
    FormBox[
     SuperscriptBox["M", "n"],
     TraditionalForm]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6325443770216875`*^9, 3.6325444449754066`*^9}, {
  3.633059082516218*^9, 
  3.633059090909033*^9}},ExpressionUUID->"532e2e5a-fe72-4132-9215-\
4b42a930cbf2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"plus", "[", 
    RowBox[{"f", ",", "h"}], "]"}], "[", "x", "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], " ", "+", " ", 
   RowBox[{"h", "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6325444569874277`*^9, 
  3.6325444935226917`*^9}},ExpressionUUID->"cd810c5b-2da3-455c-b58d-\
ce66f2a9bc8a"],

Cell["2. The multiplication with a real number:", "Text",
 CellChangeTimes->{{3.632544635451741*^9, 
  3.6325446901454372`*^9}},ExpressionUUID->"feea3c8d-5470-462b-b842-\
412eebe13ddd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "\[Element]", " ", "R"}], ",", " ", 
  RowBox[{
   RowBox[{"f", "\[Element]", 
    RowBox[{
     RowBox[{"\[ScriptCapitalF]", 
      FormBox[
       SuperscriptBox["M", "n"],
       TraditionalForm]}], "\[RightTeeArrow]", "af"}]}], ":=", " ", 
   RowBox[{
    RowBox[{"a", "*", "f"}], " ", "\[Element]", " ", 
    RowBox[{"\[ScriptCapitalF]", 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6325447033586607`*^9, 3.6325447259007*^9}, {
  3.632544762841565*^9, 3.6325447722327814`*^9}, {3.6325448367544947`*^9, 
  3.632544893866195*^9}, {3.632545047963266*^9, 3.632545068149701*^9}, {
  3.6325452002975335`*^9, 3.632545223291974*^9}, {3.633059115947077*^9, 
  3.6330591230450892`*^9}},ExpressionUUID->"330a1525-543d-466d-ab85-\
7c2170aedfee"],

Cell[BoxData[
 RowBox[{
  RowBox[{"af", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"a", "*", 
   RowBox[{"f", "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.632544931665062*^9, 3.6325449921619678`*^9}, 
   3.6325450859649324`*^9, 3.6325451287246075`*^9, {3.632545234227593*^9, 
   3.6325452417780066`*^9}},ExpressionUUID->"d442cab8-e3ab-4d5b-b167-\
4b7b85b04c94"],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], "=", 
  RowBox[{"3", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"x", "^", "2"}], "+", 
     RowBox[{"y", "^", "2"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.66624809884464*^9, 3.666248149990699*^9}, 
   3.700363671134981*^9, {3.700363706352659*^9, 3.700363774256308*^9}, {
   3.700363811945999*^9, 
   3.700363830752182*^9}},ExpressionUUID->"8b051998-2f1e-4936-a6ff-\
3d26c9f58546"],

Cell[BoxData[
 RowBox[{"h", "[", 
  RowBox[{"3", ",", "4"}], "]"}]], "Input",
 CellChangeTimes->{{3.7003637820909443`*^9, 3.7003637879598494`*^9}, {
  3.700363903364435*^9, 
  3.7003639197586946`*^9}},ExpressionUUID->"50967f8a-ab9c-4437-9672-\
87a9a845a529"],

Cell[BoxData[
 RowBox[{"Clear", "[", "h", "]"}]], "Input",
 CellChangeTimes->{{3.7003639379562073`*^9, 
  3.7003639466309547`*^9}},ExpressionUUID->"ded2c081-0310-4b4f-a6c9-\
03542bbbb6f2"],

Cell["3. The multiplication of two functions:", "Text",
 CellChangeTimes->{{3.6325453499953966`*^9, 
  3.632545390446267*^9}},ExpressionUUID->"8f7e593e-e0e0-4c76-8a22-\
c7659f8255f2"],

Cell[BoxData[
 RowBox[{"f", ",", " ", 
  RowBox[{
   RowBox[{"h", " ", "\[Element]", 
    RowBox[{
     RowBox[{"\[ScriptCapitalF]", 
      FormBox[
       SuperscriptBox["M", "n"],
       TraditionalForm]}], "\[RightTeeArrow]", "fh"}]}], ":=", " ", 
   RowBox[{
    RowBox[{"f", "*", "h"}], "\[Element]", " ", 
    RowBox[{"\[ScriptCapitalF]", 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6325454418171577`*^9, 3.6325455117988806`*^9}, {
  3.6330591341523085`*^9, 3.6330591407823205`*^9}, {3.666248236716654*^9, 
  3.6662482620369577`*^9}},ExpressionUUID->"6343515e-d599-42f3-bdc9-\
df7e6b55d382"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"f", ",", "h"}], "]"}], ";", 
  RowBox[{
   RowBox[{"fh", "[", "x_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], " ", "*", 
    RowBox[{"h", "[", "x", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6325455233429008`*^9, 3.6325455553697567`*^9}, 
   3.666248268189067*^9, {3.700363979231011*^9, 
   3.700363980462801*^9}},ExpressionUUID->"1e4809e5-0110-47ce-bd83-\
2eacddc2aba9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], " ", "*", 
   RowBox[{"h", "[", "x", "]"}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"h", "[", "x", "]"}], "*", 
   RowBox[{"f", "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.637818715245376*^9, 3.637818727664998*^9}, {
  3.6378187749546814`*^9, 3.637818826830773*^9}, {3.6399789775582433`*^9, 
  3.6399790396151524`*^9}},ExpressionUUID->"a82f43de-27f2-44f2-bd1f-\
36af4f40b6df"],

Cell[TextData[{
 "One easily proves the following\n",
 StyleBox["Proposition 1.",
  FontColor->RGBColor[0, 0, 1]],
 " The set ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"\[ScriptCapitalF]", 
    FormBox[
     SuperscriptBox["M", "n"],
     TraditionalForm]}],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6325443770216875`*^9, 3.6325444449754066`*^9}},
  ExpressionUUID->"41efe5e0-bbeb-43c5-999d-969848182b47"],
 ", n > 0, with the operations 1., 2. is an infinite dimensional real vector \
space. With the operation 3. added it becomes a commutative and associative \
real algebra with unity element one being the constant function"
}], "Text",
 CellChangeTimes->{{3.632545660732342*^9, 3.6325457030240164`*^9}, {
   3.632545734239671*^9, 3.6325457918661723`*^9}, {3.632545828479437*^9, 
   3.632545997022133*^9}, {3.632546043681815*^9, 3.632546055007435*^9}, 
   3.6325462036132956`*^9, {3.632546255202586*^9, 3.632546289226246*^9}, 
   3.6325463270719123`*^9, 3.632577262074864*^9, {3.632664914095626*^9, 
   3.63266491665403*^9}, {3.633059161222756*^9, 
   3.633059161222756*^9}},ExpressionUUID->"98dd7a73-7008-4380-856a-\
759710c79159"],

Cell[BoxData[
 RowBox[{"?", "one"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6325459991749363`*^9, 3.6325460187997713`*^9}, 
   3.6338455845462036`*^9, 3.6338456539743257`*^9, {3.639917636185045*^9, 
   3.639917657900283*^9}, {3.666248471027935*^9, 
   3.666248481715926*^9}},ExpressionUUID->"ab37dac7-adda-401f-9286-\
57018235960e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"one", "[", "x", "]"}], "*", 
   RowBox[{"f", "[", "x", "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{"f", "[", "x", "]"}]}]], "Input",
 CellChangeTimes->{{3.6662485069337597`*^9, 
  3.666248551873811*^9}},ExpressionUUID->"196d31e7-6660-403a-aaea-\
870f756a639d"],

Cell["\<\
\[FilledSquare]
Remark. Obviously, an analogue proposition is valid for  the complex valued \
smooth functions on a differentiable manifold. In both cases the unity is the \
function one.\
\>", "Text",
 CellChangeTimes->{{3.632546224922933*^9, 3.632546224922933*^9}, {
  3.6452377166765165`*^9, 3.645237837030728*^9}, {3.6452378994308376`*^9, 
  3.6452379454977183`*^9}, {3.666622155667391*^9, 3.6666221613239183`*^9}, {
  3.7003640655324078`*^9, 
  3.7003641163647537`*^9}},ExpressionUUID->"da411071-95c7-4a3f-97ca-\
9e7c7f14861e"],

Cell["We remember the general null function", "Text",
 CellChangeTimes->{{3.6399792230558743`*^9, 
  3.639979245192313*^9}},ExpressionUUID->"f88621b4-a914-4f5c-801c-\
7f34010cbe77"],

Cell[BoxData[
 RowBox[{"?", "zero"}]], "Input",
 CellChangeTimes->{{3.63997926026194*^9, 
  3.639979269840357*^9}},ExpressionUUID->"facaf512-e10c-44c9-a7fa-\
1a7e26506fc8"],

Cell[BoxData[
 RowBox[{"zero", "[", "x", "]"}]], "Input",
 CellChangeTimes->{{3.6399177120479784`*^9, 3.639917715417584*^9}, 
   3.639979315033636*^9},ExpressionUUID->"4d2cef79-52b6-4cfd-bf9d-\
052c5a827cb4"],

Cell[BoxData[
 RowBox[{"zero", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.639979404258193*^9, 
  3.639979404258193*^9}},ExpressionUUID->"afdc1536-2a01-4a76-8875-\
53f537f05bfb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"one", "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], " ", "*", 
   RowBox[{"zero", "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}]}], "\[Equal]", " ", 
  RowBox[{"zero", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.632546101869917*^9, 3.6325461402927847`*^9}, {
  3.637818865946042*^9, 3.6378189033277082`*^9}, {3.700364166133519*^9, 
  3.7003641890867863`*^9}},ExpressionUUID->"b8dbb4e3-20f1-4d9b-a78e-\
fbdc775b0e86"],

Cell["\<\
Functions may be considered as tensor fields of type (0,0) defined by\
\>", "Text",
 CellChangeTimes->{{3.6399805790962563`*^9, 
  3.6399806311535482`*^9}},ExpressionUUID->"d8923b52-0651-40c2-9c39-\
d9cb208471f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "f", "]"}], ";", 
  RowBox[{
   RowBox[{"tens0", "[", "f_", "]"}], " ", ":=", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "0", ",", "0", ",", "dim"}],
     "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6398976528294744`*^9, 3.639897661066289*^9}, {
  3.639897875941066*^9, 3.639897939495578*^9}, {3.6399796019573402`*^9, 
  3.6399796104125547`*^9}},ExpressionUUID->"32b99913-3275-4d4c-a74c-\
70e5b4d59c14"],

Cell[BoxData[
 RowBox[{"tens0", "[", "h", "]"}]], "Input",
 CellChangeTimes->{{3.639897947935192*^9, 
  3.6398979593544126`*^9}},ExpressionUUID->"d9c018f0-433a-4588-bbe3-\
064a74fad491"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6398979708984327`*^9, 
  3.6398979883548636`*^9}},ExpressionUUID->"1a0d6f57-2c36-4f84-bc4d-\
afd8c7d17d16"],

Cell["\<\
The function is obtained as the single coordinate of the tencor:\
\>", "Text",
 CellChangeTimes->{{3.63998070628328*^9, 
  3.639980742210143*^9}},ExpressionUUID->"1b4e2f63-fa66-4817-8eef-\
43d61195e957"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tens0", "[", "h", "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6449304150363555`*^9, 
  3.644930442525604*^9}},ExpressionUUID->"c9ad1601-17fd-41c5-8b02-\
32ca5deb4b8d"],

Cell[BoxData[
 RowBox[{"tens0", "[", "one", "]"}]], "Input",
 CellChangeTimes->{{3.6398980386181517`*^9, 
  3.6398980468081665`*^9}},ExpressionUUID->"183de157-e0b1-4ce6-87cf-\
07d3e850d1cf"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6666222322651005`*^9, 
  3.6666222940964155`*^9}},ExpressionUUID->"6a47be53-5b01-41d3-97a7-\
c1a76483bb9a"],

Cell["The function  zero generates the null tensor[0,0]:", "Text",
 CellChangeTimes->{{3.639980999626195*^9, 3.6399810547410917`*^9}, 
   3.700364306311022*^9},ExpressionUUID->"476e3ed9-ca37-4efa-bd7c-\
604bd0a89dc7"],

Cell[BoxData[
 RowBox[{"tens0", "[", "zero", "]"}]], "Input",
 CellChangeTimes->{{3.6398981036078663`*^9, 
  3.6398981240595016`*^9}},ExpressionUUID->"dc80b096-395d-4f56-bcd6-\
fc63aeffcb8d"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6399799192306976`*^9, 
  3.639979925689109*^9}},ExpressionUUID->"8128a42d-6db5-4707-b7ac-\
082749faac15"],

Cell[BoxData[
 RowBox[{"nulltensor", "[", 
  RowBox[{"0", ",", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.639898136321123*^9, 3.639898149939947*^9}, 
   3.639918355595908*^9},ExpressionUUID->"a6eb8279-3bbf-48ef-a6df-\
c62a0585c2c5"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6399799544555593`*^9, 
  3.6399799600247693`*^9}},ExpressionUUID->"f3a2cbe0-2031-4c43-9d00-\
5961d2753f98"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%%", ",", 
   RowBox[{"tens0", "[", "zero", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.63991836791993*^9, 3.639918394083576*^9}, {
  3.6399184750009184`*^9, 3.639918490008145*^9}, {3.6399799870128164`*^9, 
  3.6399800188836727`*^9}},ExpressionUUID->"068f5bff-f38b-41d0-87a2-\
879b80686189"],

Cell["\<\
The tensor definitions differ formally, but essentially they define the same \
tensor, since the coordinate functions agree:\
\>", "Text",
 CellChangeTimes->{{3.639980390257925*^9, 3.6399804782888794`*^9}, 
   3.6662518463915167`*^9, {3.702795817200502*^9, 
   3.702795818232383*^9}},ExpressionUUID->"81ec28e4-1502-4284-b686-\
cb66cbe1a90e"],

Cell[BoxData[
 RowBox[{"?", "tensorToArray"}]], "Input",
 CellChangeTimes->{{3.6399187888266697`*^9, 
  3.639918789528671*^9}},ExpressionUUID->"f4c5ca7f-c739-47e8-bcb7-\
26c63ad7ff27"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{"nulltensor", "[", 
   RowBox[{"0", ",", "0"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6399185196793966`*^9, 3.6399186383020053`*^9}, 
   3.639918738469781*^9, 3.6399187780626507`*^9, {3.6399188353771515`*^9, 
   3.6399188599939947`*^9}, {3.639980124932659*^9, 
   3.6399801863499665`*^9}},ExpressionUUID->"6c7f5d77-ab6c-4549-b0ae-\
558c6f170253"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{"tens0", "[", "zero", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6399185196793966`*^9, 3.6399186383020053`*^9}, 
   3.639918738469781*^9, 3.6399187780626507`*^9, {3.6399188353771515`*^9, 
   3.6399188599939947`*^9}, {3.639980124932659*^9, 
   3.6399802477048745`*^9}},ExpressionUUID->"a2efd4a7-b20b-4c21-b43c-\
f5cb6366f1a0"],

Cell[BoxData[
 RowBox[{"?", "equi"}]], "Input",
 CellChangeTimes->{{3.6576105025519133`*^9, 
  3.657610506706593*^9}},ExpressionUUID->"185fd295-f7e7-4407-9bf9-\
669d0025f9d9"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2.2 Tangent Bundle", "Subsubsection",
 CellChangeTimes->{{3.636444205894038*^9, 3.636444208234042*^9}, {
   3.6367865389695187`*^9, 3.6367865565351496`*^9}, {3.6399810915103564`*^9, 
   3.63998109342916*^9}, 3.6401461081210594`*^9, 3.6449337514124355`*^9, 
   3.6450087073197975`*^9, 3.66625226829585*^9, 3.6683477377942543`*^9, {
   3.7003659841667356`*^9, 3.7003659928450108`*^9}, {3.700366079516666*^9, 
   3.70036608423659*^9}, 3.700461495254325*^9, {3.701846047741761*^9, 
   3.7018460496054363`*^9}, 3.701865264927787*^9, {3.702795850784165*^9, 
   3.702795853080077*^9}, 
   3.702806656770932*^9},ExpressionUUID->"c4140200-0697-47d1-a1f3-\
334c5e74e322"],

Cell[TextData[{
 "Now we will introduce the tangent spaces of a differentiable manifold at \
the points x ",
 "\[Element]",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6325454418171577`*^9, 3.6325455117988806`*^9}},
  ExpressionUUID->"b8881bee-3f59-4074-b56c-778c16a02623"],
 ". To do this we consider differentiable curves \[Gamma]: <a,b> \
\[RightArrow] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6325454418171577`*^9, 3.6325455117988806`*^9}},
  ExpressionUUID->"3a55d163-6b29-4ff3-9e4d-ad505f8f6db8"],
 " where <a,b> \[Subset] R denotes an open interval containing 0 and \
\[Gamma][0] = x is an arbitrary, for this consideration fixed point of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6325454418171577`*^9, 3.6325455117988806`*^9}},
  ExpressionUUID->"373b56e7-0aa2-4237-b143-eb63a6d892bb"],
 ".  The curve \[Gamma]  being a map between manifolds of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "c6cfb385-f019-4970-9ea5-852d1812d8cc"],
 " is  of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "5fd40c2f-8264-4899-bd56-8277d039b600"],
 " if all its coordinate representatives  "
}], "Text",
 CellChangeTimes->{{3.632640085256051*^9, 3.63264014174975*^9}, {
   3.632640189335834*^9, 3.6326402444587317`*^9}, {3.6326402836032004`*^9, 
   3.6326404436306825`*^9}, {3.632640511344802*^9, 3.632640797840107*^9}, 
   3.63264090162649*^9, {3.6326409342189474`*^9, 3.6326409435965643`*^9}, {
   3.6326410303895173`*^9, 3.6326410452915435`*^9}, {3.632641124561283*^9, 
   3.632641179042579*^9}, {3.6326412099658337`*^9, 3.6326412364118805`*^9}, {
   3.632666296877063*^9, 3.632666353090962*^9}, {3.632666421349082*^9, 
   3.6326664512115345`*^9}, {3.6326665009620223`*^9, 3.632666518048053*^9}, {
   3.632666559813326*^9, 3.6326665971803923`*^9}, {3.632666631144652*^9, 
   3.632666691491558*^9}, {3.6326667327888308`*^9, 3.6326667369540386`*^9}, 
   3.632669297232051*^9, {3.633059181736792*^9, 3.633059181736792*^9}, {
   3.633059218684457*^9, 3.6330592494945107`*^9}, 3.636607917942279*^9, {
   3.6366081685647187`*^9, 3.636608328168599*^9}, {3.636608382831095*^9, 
   3.636608565569816*^9}, 3.636608870659552*^9, 3.6376758902061977`*^9, {
   3.640059827720954*^9, 
   3.6400598663154216`*^9}},ExpressionUUID->"7236a6ce-c270-4af5-b699-\
10605ae42b81"],

Cell[BoxData[
 RowBox[{"\[Gamma]\[Iota]", ":=", " ", 
  RowBox[{"Composition", "[", 
   RowBox[{"\[Phi]\[Iota]", ",", "\[Gamma]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6366087165624814`*^9, 3.6366087598525577`*^9}, {
   3.636608824327471*^9, 3.636609016972209*^9}, {3.6663274364232955`*^9, 
   3.6663274387515182`*^9}, 3.6663276337928677`*^9, {3.6664169998465843`*^9, 
   3.666417005912156*^9}, 3.666850037599478*^9, 3.7003686545809603`*^9, {
   3.700368886705243*^9, 
   3.700368914006901*^9}},ExpressionUUID->"58013f82-2f08-4538-a399-\
173862d05b23"],

Cell[TextData[{
 "for ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t", "]"}], "\[Element]", 
   SubscriptBox["U", "\[Iota]"], " "}]], "Input",ExpressionUUID->
  "fc7771bc-a7a4-4890-b308-4f96fbe52ac0"],
 "have this property. Let the curve \[Gamma] in the manifold be defined by \
the symbolic function fx: <a,b> -> ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "14b6f338-1651-40f0-8504-4048f310e058"]
}], "Text",
 CellChangeTimes->{{3.6662776975691223`*^9, 3.6662777416403418`*^9}, 
   3.666418789880125*^9, {3.668338208806439*^9, 3.668338218189796*^9}, {
   3.700368665448875*^9, 3.7003686731187153`*^9}, {3.700368706410288*^9, 
   3.700368758034799*^9}, {3.700368790869413*^9, 3.700368790869631*^9}, {
   3.700369402322497*^9, 3.700369412122135*^9}, {3.700369498177301*^9, 
   3.700369529403164*^9}, {3.700369771167848*^9, 
   3.7003697954937143`*^9}},ExpressionUUID->"774b057c-97ca-43e3-8574-\
5a634635185a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.701938253238357*^9, 
  3.7019382586117907`*^9}},ExpressionUUID->"7f002176-85f4-465f-9091-\
5606b6f32373"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"fx", "[", "t", "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.700369431122154*^9, 
  3.7003694673142977`*^9}},ExpressionUUID->"b4706a9f-7d65-4187-83ad-\
e0038445d7f9"],

Cell[BoxData[
 RowBox[{"\[Gamma]\[Iota]", "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.700368855375566*^9, 
  3.70036885808744*^9}},ExpressionUUID->"0ad48493-0dea-4659-806a-\
55bdf341391d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Phi]\[Iota]", "[", 
   RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}], "\[Equal]", "%"}]], "Input",
 CellChangeTimes->{{3.66627729516794*^9, 3.666277326383222*^9}, {
   3.666278687786067*^9, 3.666278690817622*^9}, 
   3.666327442767351*^9},ExpressionUUID->"0a66bf0c-10f5-40e9-8e60-\
cef192f02da6"],

Cell["\[Gamma]\[Iota][t] may be defined also using @:", "Text",
 CellChangeTimes->{{3.6662781588162746`*^9, 3.666278160544189*^9}, {
  3.700369992430916*^9, 
  3.700370072076708*^9}},ExpressionUUID->"ab1aa335-79b7-45f9-ae68-\
a94b8cf27053"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Phi]\[Iota]", "@", 
   RowBox[{"\[Gamma]", "@", "t"}]}], " ", "\[Equal]", " ", 
  RowBox[{"\[Gamma]\[Iota]", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.666277376919978*^9, 3.6662774450146103`*^9}, {
   3.666277506758409*^9, 3.6662775248689957`*^9}, 3.666277643676012*^9, {
   3.666278261631721*^9, 3.666278285206484*^9}, 3.666327445142497*^9, {
   3.700370112573532*^9, 3.7003701311578074`*^9}, 
   3.708407292076655*^9},ExpressionUUID->"4c0746e3-838c-42b0-93a2-\
85b29710cf0e"],

Cell[TextData[{
 "The composition with the parameter transformation \nh: t",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"->", 
    RowBox[{"h", "[", "t", "]"}]}]}]], "Input",ExpressionUUID->
  "ca0c116b-425e-4e95-93a8-d473ada68173"],
 ",  then is"
}], "Text",
 CellChangeTimes->{{3.666278736049033*^9, 3.666278767200912*^9}, {
  3.666278819416089*^9, 3.666278827392295*^9}, {3.666327445970642*^9, 
  3.666327445970642*^9}, {3.666850236281646*^9, 
  3.6668502692364764`*^9}},ExpressionUUID->"7fc5c4df-90c6-419d-848b-\
27ca18292410"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Phi]\[Iota]", "@", 
   RowBox[{"\[Gamma]", "@", 
    RowBox[{"h", "@", "t"}]}]}], "\[Equal]", " ", 
  RowBox[{"\[Phi]\[Iota]", "[", 
   RowBox[{"\[Gamma]", "[", 
    RowBox[{"h", "[", "t", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.700370558033293*^9, 3.7003706419131193`*^9}, {
  3.708407304526864*^9, 
  3.7084073143247623`*^9}},ExpressionUUID->"38a29dcc-1ded-4eb7-bd3c-\
7eeeb1a1f563"],

Cell[TextData[{
 "This is the image \[Phi]\[Iota][\[Gamma]] of the part of the curve lying in \
the chart domain ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "9dde1ec7-cdb7-494d-b709-60b7d4ea3826"],
 ",  a curve in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]],
  ExpressionUUID->"254858f0-7c49-41c0-99c6-8449095da574"],
 ". The curve \[Gamma] and the function fx are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "801f266b-40de-41f7-8076-609d817ba70b"],
 " if all its coordinate representations are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "5bc44b87-b04d-4dfc-9b41-996d95504485"],
 ", and this is the case iff all its coordinate functions fx[t][i] have this \
property. In the following we assume that all manifolds and maps we consider \
belong to a sufficiently high class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "0a9b21d3-3c4d-4a19-ba04-28f5b1146d13"],
 " or are smooth (k=\[Infinity]); all derivatives we write down should exist \
and are continuous, if not explicitly concrete exceptions appear."
}], "Text",
 CellChangeTimes->{{3.6663297570747085`*^9, 3.6663298043583856`*^9}, {
   3.6663298430166416`*^9, 3.666329929927393*^9}, {3.6663299657105103`*^9, 
   3.6663299846021156`*^9}, {3.666414650055575*^9, 3.666414664611616*^9}, {
   3.6664189821486797`*^9, 3.666419127468095*^9}, {3.666419161454793*^9, 
   3.6664192853175077`*^9}, {3.6664195055638494`*^9, 3.666419788401569*^9}, {
   3.6664199472755203`*^9, 3.666419950900689*^9}, 3.666850533890784*^9, {
   3.6683386583391733`*^9, 
   3.6683386877561083`*^9}},ExpressionUUID->"026ca68c-155b-4588-88f6-\
77eb5faf9a74"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"vec", "[", 
   RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}]}]], "Input",ExpressionUUID->\
"65449225-69ee-4540-bf86-a616e8fdea46"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"\[Gamma]\[Iota]", "[", "t", "]"}], "\[Equal]", " ", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.636609076080713*^9, 3.636609088108334*^9}, 
   3.6366092031429358`*^9, {3.6366391009670773`*^9, 3.636639175223208*^9}, {
   3.6367835045533886`*^9, 3.636783515067807*^9}, {3.6367835516578717`*^9, 
   3.6367836260076027`*^9}, 3.6449332336977234`*^9, {3.666279447596574*^9, 
   3.6662794529471607`*^9}, {3.666279505404317*^9, 3.666279578324007*^9}, 
   3.666279626691553*^9, {3.66641469732442*^9, 3.666414709677582*^9}, 
   3.7003690585896683`*^9},ExpressionUUID->"5adb4e6f-9f17-410b-a421-\
c6d520e79230"],

Cell[TextData[{
 "The tangent vector of this curve laying in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]],
  ExpressionUUID->"064dbba2-1781-4a23-8265-7e7d1056704f"],
 " is"
}], "Text",
 CellChangeTimes->{{3.6664147990328083`*^9, 3.6664148167397065`*^9}, {
  3.668338785080318*^9, 
  3.66833881883995*^9}},ExpressionUUID->"4adef16e-c099-4200-a5e4-\
b8c94838af94"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "\[Gamma]\[Iota]", "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.666253323151504*^9, 3.6662533348725033`*^9}, 
   3.66627966323999*^9, 3.666330035401617*^9, {3.701939218061391*^9, 
   3.701939249290983*^9}},ExpressionUUID->"7120ef26-0c03-46d1-bb64-\
c5d8fbd557f7"],

Cell[TextData[{
 "We consider the curves through a point x\[Element]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "7751aff9-0b45-454a-aff0-336c14da3c02"],
 " and assume"
}], "Text",
 CellChangeTimes->{{3.666279686306045*^9, 3.666279687569994*^9}, {
  3.666414888325155*^9, 
  3.6664149855140295`*^9}},ExpressionUUID->"37a51847-5f07-4862-a4b4-\
54d661e4f5f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "0", "]"}], " ", "=", " ", "x"}], ";"}]], "Input",
 CellChangeTimes->{{3.6664150723137293`*^9, 3.6664151276827307`*^9}, {
   3.6664181413888865`*^9, 3.6664181420287447`*^9}, 3.666418218969941*^9, 
   3.7003691304793587`*^9},ExpressionUUID->"a0d680b9-962f-4f80-836a-\
2cceaf75c8b9"],

Cell["This implies", "Text",
 CellChangeTimes->{{3.666415237293648*^9, 
  3.6664152500678062`*^9}},ExpressionUUID->"08791b6a-115a-4fc8-900e-\
7452942d31c6"],

Cell[BoxData[
 RowBox[{"\[Gamma]\[Iota]", "[", "0", "]"}]], "Input",ExpressionUUID->"922f0e15-d242-41fa-a9d8-f73b17c7ab9a"],

Cell[TextData[{
 " Thus the tangent vector of the curve ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"\[Gamma]\[Iota]", "[", "t", "]"}],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.632669845328819*^9, 3.632669871262065*^9}, {
    3.6326699415989885`*^9, 3.6326700156673203`*^9}, {3.6326701742404003`*^9, 
    3.632670218404478*^9}, {3.6326703423218975`*^9, 3.6326703496403103`*^9}, {
    3.632670396885194*^9, 3.6326704691433225`*^9}, {3.632670516090805*^9, 
    3.632670521333414*^9}, {3.6326705692490993`*^9, 3.6326705801399183`*^9}, {
    3.632670674108285*^9, 3.632670750311819*^9}, 3.632731041451792*^9, 
    3.636608127068646*^9, {3.6366092340777903`*^9, 3.6366092564638295`*^9}},
  ExpressionUUID->"bb4d81dd-a4de-415f-9acd-7e9af4fdc801"],
 " at the point ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"\[Gamma]\[Iota]", "[", "0", "]"}],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.632669845328819*^9, 3.632669871262065*^9}, {
    3.6326699415989885`*^9, 3.6326700156673203`*^9}, {3.6326701742404003`*^9, 
    3.632670218404478*^9}, {3.6326703423218975`*^9, 3.6326703496403103`*^9}, {
    3.632670396885194*^9, 3.6326704691433225`*^9}, {3.632670516090805*^9, 
    3.632670521333414*^9}, {3.6326705692490993`*^9, 3.6326705801399183`*^9}, {
    3.632670674108285*^9, 3.632670750311819*^9}, 3.632731041451792*^9, 
    3.636608127068646*^9, {3.6366092340777903`*^9, 3.6366092564638295`*^9}},
  ExpressionUUID->"51ad3e2a-89d6-4f7f-820c-54e0d0302b28"],
 " is a vector in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]],
  ExpressionUUID->"1bdd68c0-edff-48ec-a620-4d24d9483324"],
 ":"
}], "Text",
 CellChangeTimes->{{3.6366095540811524`*^9, 3.636609642611308*^9}, {
   3.636610383394209*^9, 3.636610384860612*^9}, {3.6366106391722584`*^9, 
   3.6366106472842727`*^9}, {3.636610787559719*^9, 3.6366107944081306`*^9}, 
   3.6366114365052586`*^9, 3.636785650345158*^9, {3.636785810120639*^9, 
   3.636785832007477*^9}, {3.636785870898345*^9, 3.6367859876801505`*^9}, {
   3.636786042311446*^9, 3.6367860468198547`*^9}, {3.636786097785144*^9, 
   3.6367861121683693`*^9}, 3.637639978464441*^9, {3.6449332381281314`*^9, 
   3.644933239610134*^9}, 3.6664200582873745`*^9, {3.668340904663007*^9, 
   3.668340904663046*^9}},ExpressionUUID->"16846dd9-bb2e-4524-b516-\
fc0f716b9c68"],

Cell[BoxData[
 RowBox[{"\[Xi]\[Iota]", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "\[Gamma]\[Iota]", "]"}], "[", "t", "]"}], "/.", 
   RowBox[{"t", "\[Rule]", "0"}]}]}]], "Input",
 CellChangeTimes->{{3.636786176565282*^9, 3.636786244082201*^9}, 
   3.6449332455849447`*^9, 3.644993999572448*^9, {3.701939267480261*^9, 
   3.701939290196554*^9}},ExpressionUUID->"9cd6aeb7-d75c-46a3-b2d1-\
adb801859967"],

Cell[TextData[{
 "This vector depends on the chosen chart ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  ExpressionUUID->"b690f404-c49f-4c0f-8ffd-94c2bfa752ca"],
 " with x\[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "74ae4798-88f4-4f87-ae7e-6c4506eb8a14"],
 " ; it will be the coordinate representative of the tangent vector of the \
curve \[Gamma][t] at \[Gamma][0} = x we are going to define. To this aim we \
consider two charts with ",
 StyleBox["  ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{"x", " ", "\[Element]", " ", 
   FormBox[
    RowBox[{
     SubscriptBox["U", "\[Kappa]"], " ", "\[Intersection]", " ", 
     SubscriptBox["U", "\[Iota]"]}],
    TraditionalForm]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.631978559989169*^9, 3.631978652060531*^9}, 
    3.632029688598169*^9, {3.632029836642429*^9, 3.632029846189646*^9}, {
    3.632029880104105*^9, 3.6320299236125813`*^9}, 3.632030457273919*^9, {
    3.6320309569427967`*^9, 3.6320309653356113`*^9}, {3.632031072851*^9, 
    3.632031108809063*^9}, {3.6320311457655277`*^9, 3.63203127786656*^9}, 
    3.6329142658306437`*^9},
  TextAlignment->Center,
  FontWeight->"Bold",ExpressionUUID->"3b1aed15-b686-49c7-94e0-bdd8f1390108"],
 ":"
}], "Text",
 CellChangeTimes->{{3.636787358797759*^9, 3.636787545576887*^9}, {
   3.6367875809421487`*^9, 3.636787666367899*^9}, {3.6367876995647573`*^9, 
   3.636787705133967*^9}, 3.6449940650925627`*^9, {3.6663274576431212`*^9, 
   3.6663274576431212`*^9}},ExpressionUUID->"643bcf8d-2426-48ee-897d-\
f219f8e128da"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]\[Iota]", "[", "x", "]"}], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", "...", ",", 
      RowBox[{"x", "[", "n", "]"}]}], "}"}]}], ";", "   ", 
   RowBox[{
    RowBox[{"\[Phi]\[Kappa]", "[", "x", "]"}], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"y", "[", "1", "]"}], ",", "...", ",", 
      RowBox[{"y", "[", "n", "]"}]}], "}"}]}], ";"}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}, 
   3.6663274576743813`*^9, {3.6663599433328905`*^9, 
   3.666359949661339*^9}},ExpressionUUID->"909bb338-2a42-42d0-871c-\
d9ade6473f92"],

Cell[TextData[{
 "Considering the coordinate transformation",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Phi]\[Kappa]\[Iota]"],
  CellChangeTimes->{{3.636609853180478*^9, 3.6366098849109335`*^9}, {
    3.6366099787606983`*^9, 3.6366099939395247`*^9}, {3.6366100550292325`*^9, 
    3.636610072516863*^9}, {3.636610120252947*^9, 3.636610127335359*^9}, 
    3.6366103216493006`*^9},
  FontWeight->"Bold",ExpressionUUID->"e0aad6ba-7799-4690-91f7-8e680c8c6fd2"],
 StyleBox[":",
  FontWeight->"Bold"]
}], "Text",
 CellChangeTimes->{{3.6366095540811524`*^9, 3.636609642611308*^9}, {
   3.636610383394209*^9, 3.636610384860612*^9}, {3.6366106391722584`*^9, 
   3.6366106472842727`*^9}, {3.636610787559719*^9, 3.6366107944081306`*^9}, 
   3.6366114365052586`*^9, 3.636785650345158*^9, {3.6367876913747425`*^9, 
   3.6367876940891476`*^9}, {3.644929456268264*^9, 
   3.644929456268264*^9}},ExpressionUUID->"903a08dd-9b33-44a2-a042-\
5e2dd19c4ed1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]\[Kappa]", "[", "t_", "]"}], ":=", 
  RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
   RowBox[{"\[Gamma]\[Iota]", "[", "t", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6366134308127613`*^9, 3.636613463448019*^9}, {
   3.6366135364093475`*^9, 3.6366135548017797`*^9}, 3.6449294579228673`*^9, 
   3.644933247082547*^9, 3.6449334691981387`*^9, 
   3.668341650236486*^9},ExpressionUUID->"68deeaef-3537-43b7-b75b-\
8745c4661f2c"],

Cell[BoxData[
 RowBox[{"\[Gamma]\[Kappa]", "[", "t", "]"}]], "Input",
 CellChangeTimes->{3.636611207746457*^9, 
  3.6449334632857285`*^9},ExpressionUUID->"f095a16e-13b1-44ad-9224-\
bd31dfb669e2"],

Cell[BoxData[
 RowBox[{"\[Gamma]\[Kappa]", "[", "0", "]"}]], "Input",ExpressionUUID->"378986a7-7b84-47d1-9344-af5568779b2b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]\[Kappa]", "[", "0", "]"}], "==", 
  RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.636611207746457*^9, {3.6366399548969774`*^9, 3.6366400060026674`*^9}, {
   3.636640038637924*^9, 3.6366400418203297`*^9}, 3.644929466520482*^9, 
   3.644933455920515*^9, {3.668341687308648*^9, 
   3.66834169282766*^9}},ExpressionUUID->"a296bb4b-10bb-4f9c-bc2b-\
b187f9eb35a8"],

Cell[TextData[{
 StyleBox["The differential of ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData["\[Phi]\[Kappa]\[Iota]"],
  CellChangeTimes->{{3.636609853180478*^9, 3.6366098849109335`*^9}, {
    3.6366099787606983`*^9, 3.6366099939395247`*^9}, {3.6366100550292325`*^9, 
    3.636610072516863*^9}, {3.636610120252947*^9, 3.636610127335359*^9}, 
    3.6366103216493006`*^9},
  FontWeight->"Bold",
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "60f44545-efba-48ab-aafe-76529c57f03c"],
 StyleBox[" is the transposed Jacobian matrix",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.632640085256051*^9, 3.63264014174975*^9}, {
   3.632640189335834*^9, 3.6326402444587317`*^9}, {3.6326402836032004`*^9, 
   3.6326404436306825`*^9}, {3.632640511344802*^9, 3.632640797840107*^9}, 
   3.63264090162649*^9, {3.6326409342189474`*^9, 3.6326409435965643`*^9}, {
   3.6326410303895173`*^9, 3.6326410452915435`*^9}, {3.632641124561283*^9, 
   3.632641179042579*^9}, {3.6326412099658337`*^9, 3.6326412364118805`*^9}, {
   3.632666296877063*^9, 3.632666353090962*^9}, {3.632666421349082*^9, 
   3.6326664512115345`*^9}, {3.6326665009620223`*^9, 3.632666518048053*^9}, {
   3.632666559813326*^9, 3.6326665971803923`*^9}, {3.632666631144652*^9, 
   3.632666691491558*^9}, {3.6326667327888308`*^9, 3.6326667369540386`*^9}, 
   3.632669297232051*^9, {3.633059181736792*^9, 3.633059181736792*^9}, {
   3.633059218684457*^9, 3.6330592494945107`*^9}, 3.636607917942279*^9, {
   3.6366138768019447`*^9, 3.63661388552236*^9}, {3.6366139197020206`*^9, 
   3.636613934334846*^9}, {3.640061911871414*^9, 3.640061916317422*^9}, 
   3.640062519735238*^9, {3.6449294688292866`*^9, 3.6449294688292866`*^9}, {
   3.6664203411572313`*^9, 
   3.666420344876178*^9}},ExpressionUUID->"3fde278b-d77a-4c87-8397-\
e6d340dba51c"],

Cell[BoxData[
 RowBox[{"?", "jac"}]], "Input",
 CellChangeTimes->{{3.640061618572899*^9, 
  3.640061621209304*^9}},ExpressionUUID->"a5d2ae90-54c0-4c62-8883-\
1542e4482677"],

Cell[BoxData[
 RowBox[{"Context", "[", "jac", "]"}]], "Input",
 CellChangeTimes->{{3.7018494232029257`*^9, 
  3.701849438844521*^9}},ExpressionUUID->"e9fb0e48-82b0-4923-90c1-\
63e3be6f673c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "4"}], ";"}]], "Input",
 CellChangeTimes->{{3.701865535187663*^9, 3.7018655405452414`*^9}, 
   3.701865637148285*^9},ExpressionUUID->"3df6b0ee-a975-4116-8eb8-\
87b9ad059818"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6366136867000113`*^9, 3.636613728274084*^9}, {
   3.6366150655864325`*^9, 3.636615072653245*^9}, 3.64492947095089*^9, {
   3.666851250995226*^9, 3.6668512611363764`*^9}, {3.6668515078152666`*^9, 
   3.666851569459092*^9}, {3.7018656591165657`*^9, 3.701865665561936*^9}, {
   3.7018688513796186`*^9, 
   3.701868875494059*^9}},ExpressionUUID->"8093857e-2cd4-4228-8fbc-\
ba02bf03108c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{"jac", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6400618500017056`*^9, 3.640061850828507*^9}, {
   3.640061927908242*^9, 3.6400619394366627`*^9}, 3.6449294753188972`*^9, {
   3.666360129201888*^9, 3.6663601347490683`*^9}, {3.666420292501645*^9, 
   3.666420299711958*^9}, {3.6668513993565335`*^9, 3.6668514044661703`*^9}, {
   3.6668514592033777`*^9, 3.6668514884392548`*^9}, {3.668341165897307*^9, 
   3.6683412051360826`*^9}, {3.6683413997593317`*^9, 
   3.66834140059067*^9}},ExpressionUUID->"8829b0d4-e8cc-4c9f-9462-\
fe4c2188ee3b"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.6664203153221397`*^9, 
  3.666420319634878*^9}},ExpressionUUID->"c36d935c-08d4-4638-90c8-\
d8043dc4b85c"],

Cell[BoxData[
 RowBox[{"\[Gamma]\[Kappa]", "[", "t", "]"}]], "Input",ExpressionUUID->"2f492352-a39e-4b91-a012-a044eb7403f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "\[Gamma]\[Kappa]", "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.701922339868174*^9, 
  3.701922356109984*^9}},ExpressionUUID->"294f8699-9eff-41ae-8267-\
b2e48259a1b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "\[Gamma]\[Kappa]", "]"}], "[", "t", "]"}], "/.", 
    RowBox[{"t", "\[Rule]", "0"}]}], "]"}], "/.", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fx", "[", "0", "]"}], "[", "i", "]"}], "->", 
     RowBox[{"x", "[", "i", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "4"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.636613972586113*^9, 3.6366140216949997`*^9}, {
   3.6366140548918576`*^9, 3.636614060351867*^9}, {3.636614155761635*^9, 
   3.6366141820944815`*^9}, {3.6366142242145553`*^9, 3.63661423850418*^9}, 
   3.6366143013410907`*^9, {3.6366143489835744`*^9, 3.636614370074811*^9}, {
   3.6366144192928977`*^9, 3.636614435080126*^9}, {3.6366400659847727`*^9, 
   3.636640080617598*^9}, 3.644933448385702*^9, {3.7004530184716177`*^9, 
   3.7004530451219797`*^9}, {3.700453080858696*^9, 3.700453089353776*^9}, {
   3.7004531217792807`*^9, 3.700453167170685*^9}, 3.701865869976234*^9, {
   3.70192237544657*^9, 3.701922446137298*^9}, {3.701922516962532*^9, 
   3.701922521525179*^9}},ExpressionUUID->"ef39f231-a95e-42c9-95ae-\
77b4ba4f26af"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], ".", 
   RowBox[{
    RowBox[{"d", "[", "\[Gamma]\[Iota]", "]"}], "[", "t", "]"}]}], "/.", 
  RowBox[{"t", "\[Rule]", "0"}]}]], "Input",
 CellChangeTimes->{
  3.6366144640493765`*^9, {3.6366146800473557`*^9, 3.636614694336981*^9}, {
   3.6366147766271257`*^9, 3.6366147920711527`*^9}, 3.636614902269746*^9, {
   3.6366149415662155`*^9, 3.636614945029421*^9}, {3.636615185207443*^9, 
   3.636615197859065*^9}, {3.636615369755767*^9, 3.636615524617239*^9}, 
   3.636639391455188*^9, {3.636639428786053*^9, 3.636639442810478*^9}, {
   3.6366394797825427`*^9, 3.63663948386975*^9}, {3.6366401065448437`*^9, 
   3.636640112192054*^9}, 3.6449294765980997`*^9, 3.64493324848755*^9, {
   3.6683422773357153`*^9, 3.6683423152868557`*^9}, 3.701865984374917*^9, {
   3.701866016505619*^9, 3.701866051673184*^9}, {3.701921951594187*^9, 
   3.701921977329837*^9}, {3.701922535577599*^9, 
   3.701922550932702*^9}},ExpressionUUID->"c1b40804-6f27-4d18-aae5-\
b2111a9bbab3"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "\[Equal]", " ", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.636614814644392*^9, 3.636614836827631*^9}, {
   3.668342493861979*^9, 3.6683424989970083`*^9}, {3.700452487007256*^9, 
   3.7004525984537497`*^9}, {3.7004527417235327`*^9, 3.700452889947733*^9}, 
   3.700453062832301*^9},ExpressionUUID->"bb30eb67-848c-4c3a-b8b6-\
2344717da1c1"],

Cell[TextData[{
 "The inverse of the transformation  ",
 Cell[BoxData["\[Phi]\[Kappa]\[Iota]"],
  CellChangeTimes->{{3.633668027920967*^9, 3.6336680719130445`*^9}, {
   3.633754797263999*^9, 3.6337548418956776`*^9}},
  FontWeight->"Bold",ExpressionUUID->"eb0fc17b-2712-4a3d-93c9-244939a356b8"],
 " is"
}], "Text",
 CellChangeTimes->{{3.6337720412475605`*^9, 3.633772070080411*^9}, {
  3.6367879376211753`*^9, 3.6367879828612547`*^9}, {3.644929477861702*^9, 
  3.644929477861702*^9}},ExpressionUUID->"4c059bd6-bf2b-4775-934e-\
d2851828f2b3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Phi]\[Iota]\[Kappa]", "[", 
   RowBox[{"{", "y__", "}"}], "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f\[Iota]\[Kappa]", "[", "i", "]"}], "[", "y", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "y", "}"}], "]"}]}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.633330419208383*^9, 3.6333304385524173`*^9}, {
   3.633331299338729*^9, 3.6333313160619583`*^9}, {3.633333623918318*^9, 
   3.6333336383951435`*^9}, {3.6333410257472467`*^9, 3.6333410337968607`*^9}, 
   3.633419845505786*^9, 3.6334199398547516`*^9, {3.6336679422144165`*^9, 
   3.633667943712019*^9}, {3.636788025184129*^9, 3.6367880537165794`*^9}, 
   3.644929936694311*^9, 3.64493213661479*^9, {3.6683425678611526`*^9, 
   3.668342574316375*^9}},ExpressionUUID->"2d96c593-cefe-4fe9-9a01-\
8e2f2b757345"],

Cell[BoxData[
 RowBox[{"\[Phi]\[Iota]\[Kappa]", "[", 
  RowBox[{"vec", "[", "y", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.633330419208383*^9, 3.6333304385524173`*^9}, {
   3.633331299338729*^9, 3.6333313160619583`*^9}, {3.633333623918318*^9, 
   3.6333336383951435`*^9}, {3.6333410473728848`*^9, 
   3.6333410533008947`*^9}, {3.633341105880988*^9, 3.633341153933072*^9}, 
   3.64492994195152*^9, {3.6683426012524443`*^9, 
   3.668342620211945*^9}},ExpressionUUID->"a3189ff5-7fa5-4e3a-9946-\
618ce57df771"],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  RowBox[{" ", 
   StyleBox[
    RowBox[{"\[Phi]\[Iota]\[Kappa]", ",", "\[Phi]\[Kappa]\[Iota]"}],
    FontWeight->"Bold"]}]],
  CellChangeTimes->{{3.633668027920967*^9, 3.6336680719130445`*^9}, {
   3.633754797263999*^9, 3.6337548418956776`*^9}},ExpressionUUID->
  "f2b34a3c-db2d-4928-9704-a58f0ed14357"],
 " are inverse to each other, the composition"
}], "Text",
 CellChangeTimes->{{3.633756300148697*^9, 3.633756386397649*^9}, 
   3.6337718084471493`*^9, {3.6337721063856754`*^9, 3.633772111970485*^9}, {
   3.644929487440119*^9, 3.644929487440119*^9}, {3.644929944119924*^9, 
   3.644929944119924*^9}},ExpressionUUID->"a70bb0f8-4293-4921-b6c9-\
0ce0c73d3038"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.701866131463635*^9, 
  3.7018661370713587`*^9}},ExpressionUUID->"0f18a37a-d1a0-4898-8145-\
990408458d7a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ftr", "[", "x__", "]"}], ":=", " ", 
  RowBox[{"\[Phi]\[Iota]\[Kappa]", "[", 
   RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
    RowBox[{"{", "x", "}"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.633668027920967*^9, 3.6336680719130445`*^9}, {
   3.633754797263999*^9, 3.6337548418956776`*^9}, {3.6337602792033124`*^9, 
   3.633760284195321*^9}, 3.6337717317298136`*^9, 3.6449294903437243`*^9, 
   3.644929946335128*^9, {3.668342689595934*^9, 3.668342696267959*^9}, 
   3.66834272780304*^9},ExpressionUUID->"2557bdda-5790-4d22-8eec-\
a6c22e1700a8"],

Cell[BoxData[
 RowBox[{"ftr", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.633754963919092*^9, 
  3.6337549679594994`*^9}},ExpressionUUID->"31c0d5bf-0fa7-4f54-b308-\
fc93a3c614b9"],

Cell[" equals the identity map", "Text",
 CellChangeTimes->{{3.6337718237547765`*^9, 3.633771871635661*^9}, {
  3.6340201252954235`*^9, 
  3.634020132549436*^9}},ExpressionUUID->"0b055a4a-7508-4aa6-b563-\
0c901bce4e66"],

Cell[BoxData[
 RowBox[{"identity", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6337721405849357`*^9, 3.6337721455457444`*^9}, 
   3.6337721803846054`*^9},ExpressionUUID->"af0305b6-b907-42b9-b490-\
801d1b0caa28"],

Cell["Now the differential", "Text",
 CellChangeTimes->{{3.633772312628439*^9, 
  3.6337723208672533`*^9}},ExpressionUUID->"451561ac-ffc4-4255-8384-\
ce619208e0c3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "ftr", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.633758613700777*^9, 3.6337586261183987`*^9}, 
   3.63376125937644*^9},ExpressionUUID->"99de920d-aa47-446a-9758-\
a6613c6ee602"],

Cell["\<\
decomposes into the product of two differentials, expressed by the \
corresponding matrices:\
\>", "Text",
 CellChangeTimes->{{3.6337724575862947`*^9, 3.633772515704397*^9}, {
   3.6337726465256276`*^9, 3.6337726597584515`*^9}, 
   3.633772706702934*^9},ExpressionUUID->"b0350ee5-e307-442c-bd73-\
0981f2e76001"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "ftr", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "==", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "\[Phi]\[Iota]\[Kappa]", "]"}], "[", 
       RowBox[{"vec", "[", "y", "]"}], "]"}], "/.", " ", 
      RowBox[{
       RowBox[{"y", "[", "i_", "]"}], " ", "\[Rule]", 
       RowBox[{
        RowBox[{"f\[Kappa]\[Iota]", "[", "i", "]"}], "[", 
        RowBox[{
         RowBox[{"x", "[", "1", "]"}], ",", 
         RowBox[{"x", "[", "2", "]"}], ",", 
         RowBox[{"x", "[", "3", "]"}]}], "]"}]}]}], ")"}], ".", 
    RowBox[{
     RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
     RowBox[{"vec", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.633759928822895*^9, 3.6337599390877132`*^9}, {
   3.633760033011879*^9, 3.6337601492420835`*^9}, {3.6337601797431374`*^9, 
   3.6337602272512207`*^9}, {3.6337603081433635`*^9, 
   3.6337603120121703`*^9}, {3.6337603450394287`*^9, 
   3.6337603815006924`*^9}, {3.63376048774788*^9, 3.6337604928178887`*^9}, 
   3.6337605868336544`*^9, {3.6337609137374315`*^9, 3.633761087708338*^9}, 
   3.6337611238510017`*^9, {3.6337724356194563`*^9, 3.633772435791056*^9}, 
   3.633773625484954*^9, {3.6367882309796906`*^9, 3.6367882475313196`*^9}, 
   3.644929491264126*^9, 3.6449299477391305`*^9, 3.64493196621889*^9, {
   3.668342797067518*^9, 3.668342816899351*^9}, {3.668342860987049*^9, 
   3.668342875994553*^9}, {3.701866231998307*^9, 3.701866381389634*^9}, 
   3.7019236264845343`*^9, 3.701923660596322*^9, {3.701923726844123*^9, 
   3.70192378201058*^9}},ExpressionUUID->"74b19941-18ec-45bc-827e-\
56778059163b"],

Cell["Since, as explained above, ftr == identity, we have", "Text",
 CellChangeTimes->{{3.6337727160337505`*^9, 3.633772769719445*^9}, 
   3.63393532704147*^9, {3.668342943241417*^9, 
   3.668342945601194*^9}},ExpressionUUID->"a4a89818-cbfd-4628-99e6-\
6df4f6998c42"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "ftr", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"d", "[", "identity", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.633772775337455*^9, 3.633772890557658*^9}, {
   3.63377293702054*^9, 3.6337729950918427`*^9}, {3.6337730293369026`*^9, 
   3.6337730598545566`*^9}, 3.6337731580053296`*^9, {3.6337732014730062`*^9, 
   3.6337732034074097`*^9}, 
   3.633773233659863*^9},ExpressionUUID->"7ea7dac1-31e0-4a05-9da5-\
49d8198932a2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "identity", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], " ", "\[Equal]", " ", 
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"1", "0", "0"},
      {"0", "1", "0"},
      {"0", "0", "1"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.7]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
   Function[BoxForm`e$, 
    MatrixForm[BoxForm`e$]]]}]], "Input",
 CellChangeTimes->{{3.633773281510147*^9, 3.633773309125196*^9}, {
  3.7004534631749*^9, 
  3.700453511366238*^9}},ExpressionUUID->"b91e1ee3-27b9-4464-ad11-\
2c1d63082e02"],

Cell[TextData[{
 StyleBox["Lemma 1.",
  FontColor->RGBColor[0, 0, 1]],
 " The differentials of the coordinate transformations ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.633759928822895*^9, 3.6337599390877132`*^9}, {
    3.633760033011879*^9, 3.6337601492420835`*^9}, {3.6337601797431374`*^9, 
    3.6337602272512207`*^9}, {3.6337603081433635`*^9, 
    3.6337603120121703`*^9}, {3.6337603450394287`*^9, 
    3.6337603815006924`*^9}, {3.63376048774788*^9, 3.6337604928178887`*^9}, 
    3.6337605868336544`*^9, {3.6337609137374315`*^9, 3.633761087708338*^9}, 
    3.6337611238510017`*^9, {3.6337724356194563`*^9, 3.633772435791056*^9}},
  ExpressionUUID->"6fc84529-6921-43b6-a6cf-1bae7f35f101"],
 ", ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"d", "[", "\[Phi]\[Iota]\[Kappa]", "]"}], "[", 
   RowBox[{"vec", "[", "y", "]"}], "]"}]],
  CellChangeTimes->{{3.633759928822895*^9, 3.6337599390877132`*^9}, {
    3.633760033011879*^9, 3.6337601492420835`*^9}, {3.6337601797431374`*^9, 
    3.6337602272512207`*^9}, {3.6337603081433635`*^9, 
    3.6337603120121703`*^9}, {3.6337603450394287`*^9, 
    3.6337603815006924`*^9}, {3.63376048774788*^9, 3.6337604928178887`*^9}, 
    3.6337605868336544`*^9, {3.6337609137374315`*^9, 3.633761087708338*^9}, 
    3.6337611238510017`*^9, {3.6337724356194563`*^9, 3.633772435791056*^9}, 
    3.633773625484954*^9},
  FontWeight->"Bold",ExpressionUUID->"79d11c4f-26d0-404c-9d4c-a8851a654425"],
 " with ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]\[Iota]", "[", "x", "]"}], " ", "=", " ", 
    RowBox[{"vec", "[", "x", "]"}]}], ",", "   ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "\[Kappa]"], "[", "x", "]"}], " ", "=", " ", 
    RowBox[{"vec", "[", "y", "]"}]}]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"4e6ee94f-9f2f-4297-9d2b-66e94d3f3f04"],
 " \nand the corresponding matrices are inverse to each other.  For three \
charts ",
 Cell[BoxData["\[Phi]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"cc5fb14e-3bc9-4d2d-bc32-1e7f553126e9"],
 ", ",
 Cell[BoxData[
  SubscriptBox["\[Phi]", "\[Kappa]"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"02495849-f2fb-462f-8c70-6c520580ddbf"],
 ", ",
 Cell[BoxData[
  SubscriptBox["\[Phi]", "\[Lambda]"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"e1fdf994-c3fe-4446-8ed4-7b4e074a13f3"],
 ", and \nx \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Lambda]"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631978559989169*^9, 3.631978652060531*^9}, 
    3.632029688598169*^9, {3.632029836642429*^9, 3.632029846189646*^9}, {
    3.632029880104105*^9, 3.6320299236125813`*^9}, 3.632030457273919*^9, {
    3.6320309569427967`*^9, 3.6320309653356113`*^9}, {3.632031072851*^9, 
    3.632031108809063*^9}, {3.6320311457655277`*^9, 3.63203127786656*^9}, 
    3.6329142658306437`*^9},
  TextAlignment->Center,
  FontWeight->"Bold",ExpressionUUID->"28766791-cb50-4cf1-bf41-c6974ac44063"],
 "\[Intersection]",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["U", "\[Kappa]"], "\[Intersection]", 
    RowBox[{
     SubscriptBox["U", "\[Iota]"], 
     StyleBox["  ",
      FontWeight->"Plain"], 
     StyleBox["the",
      FontWeight->"Plain"], 
     StyleBox[" ",
      FontWeight->"Plain"], 
     StyleBox["composition",
      FontWeight->"Plain"], 
     StyleBox[" ",
      FontWeight->"Plain"], 
     StyleBox["rules",
      FontWeight->"Plain"]}]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.631978559989169*^9, 3.631978652060531*^9}, 
    3.632029688598169*^9, {3.632029836642429*^9, 3.632029846189646*^9}, {
    3.632029880104105*^9, 3.6320299236125813`*^9}, 3.632030457273919*^9, {
    3.6320309569427967`*^9, 3.6320309653356113`*^9}, {3.632031072851*^9, 
    3.632031108809063*^9}, {3.6320311457655277`*^9, 3.63203127786656*^9}, 
    3.6329142658306437`*^9},
  TextAlignment->Center,
  FontWeight->"Bold",ExpressionUUID->"a3344d20-8a21-4f97-a0e7-e49cb8be6079"],
 "  ",
 Cell[BoxData["\[Phi]\[Lambda]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"810f282e-828b-437e-8014-b47e73d7998b"],
 " == ",
 Cell[BoxData["\[Phi]\[Lambda]\[Kappa]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"de1cef1f-731b-4ce2-a059-c2d0103a2205"],
 "\[SmallCircle]",
 Cell[BoxData["\[Phi]\[Kappa]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"a76ac2ba-8542-4faa-b609-0ddfedbcf012"],
 " and for the differentials"
}], "Text",
 CellChangeTimes->{{3.6337734152973833`*^9, 3.6337734841629047`*^9}, {
   3.6337735233375735`*^9, 3.633773564841647*^9}, {3.633773677675046*^9, 
   3.6337737049936943`*^9}, 3.633773764512799*^9, {3.633773868797383*^9, 
   3.633773987447592*^9}, {3.6338449521054926`*^9, 3.633844959203505*^9}, 
   3.6338450168144064`*^9, {3.6341072983105917`*^9, 3.634107299496194*^9}, 
   3.634109003643187*^9, {3.6367883265766582`*^9, 3.6367883265766582`*^9}, {
   3.636799356616341*^9, 3.6367993651183558`*^9}, {3.636799398471215*^9, 
   3.63679942992087*^9}, {3.6367994807145596`*^9, 3.63679954946388*^9}, {
   3.6367996291176195`*^9, 3.6367997115013647`*^9}, {3.636799752950637*^9, 
   3.6367998043371277`*^9}, {3.636799846753602*^9, 3.6367998555364175`*^9}, {
   3.6367998897628775`*^9, 3.636799899060494*^9}, {3.636799972521023*^9, 
   3.6367999811946383`*^9}, {3.6368000247031145`*^9, 3.636800035061533*^9}, {
   3.636800074436002*^9, 3.636800150907336*^9}, {3.6368001840105944`*^9, 
   3.6368002322926793`*^9}, {3.6368002630247335`*^9, 
   3.6368002833047686`*^9}, {3.6368004274022217`*^9, 
   3.6368004306782274`*^9}, {3.636800570360873*^9, 3.636800573075278*^9}, {
   3.6449294945245314`*^9, 3.6449294964433346`*^9}, {3.644929948753132*^9, 
   3.644929948753132*^9}, {3.6449946864790535`*^9, 3.644994737678344*^9}, {
   3.66632745772125*^9, 3.66632745772125*^9}, {3.668342972916367*^9, 
   3.6683430632421217`*^9}},ExpressionUUID->"e08a3368-282f-40c2-9f67-\
a80a1697191e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", " ", "\[Phi]\[Lambda]\[Iota]", "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"d", "[", "\[Phi]\[Lambda]\[Kappa]", "]"}], "\[SmallCircle]", 
   RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6368003166420274`*^9, 3.636800413455797*^9}, 
   3.6449294972877364`*^9, {3.644994768129597*^9, 3.644994780750019*^9}, {
   3.644994821668891*^9, 
   3.6449948457085333`*^9}},ExpressionUUID->"60a719e9-04d0-40ed-8332-\
c9dc193be3dd"],

Cell["are satisfied. \[FilledSquare] ", "Text",
 CellChangeTimes->{{3.6368004524090657`*^9, 
  3.6368005016115522`*^9}},ExpressionUUID->"d2676eb9-1ed0-4b0f-982a-\
3ae6b567a790"],

Cell[TextData[{
 StyleBox["Remark",
  FontColor->RGBColor[0, 0, 1]],
 ". The built-in Mathematica object InverseFunction is not designed to work \
with functions of several variables, try"
}], "Text",
 CellChangeTimes->{{3.6664208696665688`*^9, 
  3.6664209711649623`*^9}},ExpressionUUID->"4041a681-2400-4bf6-9bfd-\
5699366f1301"],

Cell[BoxData[
 RowBox[{"?", "InverseFunction"}]], "Input",
 CellChangeTimes->{{3.666420977269591*^9, 
  3.666420987707642*^9}},ExpressionUUID->"ae3ccd1f-13a7-45be-a22e-\
07073e69fab9"],

Cell[TextData[{
 StyleBox["Notation.",
  FontColor->RGBColor[0, 0, 1]],
 " We introduce  the traditional  notations ",
 Cell[BoxData[
  SubsuperscriptBox["A", "i", "j"]],
  Evaluatable->False,
  CellChangeTimes->{{3.633263302173919*^9, 3.6332633842828636`*^9}},
  FontWeight->"Bold",ExpressionUUID->"7eff295e-6356-4750-a3e3-6e5bfe66dae4"],
 ", ",
 Cell[BoxData[
  SubsuperscriptBox["B", "i", "j"]],
  Evaluatable->False,
  CellChangeTimes->{{3.633263302173919*^9, 3.6332633842828636`*^9}},
  FontWeight->"Bold",ExpressionUUID->"a69cbd57-892c-426a-8d18-1ff8b4b6c7c7"],
 " and define the corresponding ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expressions. Here and in the following lhs \[DotEqual] rhs means that lhs \
is the traditional notation of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expression rhs.\n ",
 Cell[BoxData[
  SubsuperscriptBox["A", 
   RowBox[{"i", "  "}], "j"]],
  Evaluatable->False,
  CellChangeTimes->{{3.633263302173919*^9, 3.6332633842828636`*^9}},
  FontWeight->"Bold",ExpressionUUID->"1dfa9c75-473e-4a38-adae-c46b0bcfae46"],
 "= ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      SuperscriptBox["y", "j"]}], 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["x", "i"]}]], " "}], TraditionalForm]],ExpressionUUID->
  "1cc51eff-b72c-4958-82a3-047e30cb4e84"],
 StyleBox["\[DotEqual] ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{
   RowBox[{"A\[Kappa]\[Iota]", "[", 
    RowBox[{"i", ",", "j"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", "...", ",", 
    RowBox[{"x", "[", "n", "]"}]}], "]"}]],
  CellChangeTimes->{{3.6331875903732033`*^9, 3.6331876454609003`*^9}, 
    3.633188196498871*^9, 3.63326148156371*^9, {3.633262878720773*^9, 
    3.633262881965579*^9}},
  FontWeight->"Bold",ExpressionUUID->"ba6f2bf1-c3c3-4c95-93aa-abb19a5458dc"],
 " with"
}], "Text",
 CellChangeTimes->{{3.6332636291077156`*^9, 3.6332636754729967`*^9}, {
  3.6340255860283165`*^9, 3.6340256285589914`*^9}, {3.6340256894834995`*^9, 
  3.6340256920575037`*^9}, {3.634025815247321*^9, 3.634025943276147*^9}, {
  3.6341088038224363`*^9, 3.6341089482006893`*^9}, {3.6341093814758506`*^9, 
  3.6341093863430595`*^9}, {3.63436097176575*^9, 3.6343609733569527`*^9}, {
  3.634361021841838*^9, 3.6343612216937885`*^9}, {3.634361255109047*^9, 
  3.634361303999533*^9}, {3.636800154214542*^9, 3.636800177474183*^9}, {
  3.6368007545659966`*^9, 3.6368007753296328`*^9}, {3.637673942147152*^9, 
  3.6376739934616423`*^9}, {3.6449950417072773`*^9, 
  3.6449950417072773`*^9}},ExpressionUUID->"30b21e58-fdfb-4ac1-b511-\
91047a91f5f8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A\[Kappa]\[Iota]", "[", 
   RowBox[{"i_", ",", "j_"}], " ", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
     RowBox[{"{", "##", "}"}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"j", ",", "i"}], "]"}], "]"}], "&"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.63318724209979*^9, 3.6331872571860166`*^9}, {
   3.6331873076727057`*^9, 3.6331873845888414`*^9}, 3.633187431236923*^9, {
   3.6331874671979866`*^9, 3.633187546967927*^9}, 3.633187580808387*^9, {
   3.633187727835046*^9, 3.633187744467675*^9}, {3.6331878764685073`*^9, 
   3.633187908217563*^9}, {3.6331879483750343`*^9, 3.633187949872637*^9}, 
   3.633188182237446*^9, {3.6331882227537174`*^9, 3.633188246485359*^9}, {
   3.633261392261153*^9, 3.633261433277625*^9}, {3.633262798622232*^9, 
   3.6332628012742367`*^9}, {3.633262849295121*^9, 3.6332628675199537`*^9}, {
   3.6332629287882614`*^9, 3.633262959170315*^9}, 3.633937609153878*^9, {
   3.6340265587572308`*^9, 3.6340265892748847`*^9}, 3.644929498223738*^9, {
   3.644994998339202*^9, 3.644995005218814*^9}, {3.668343292726872*^9, 
   3.668343296286893*^9}, 3.701845703949748*^9, {3.701845739089662*^9, 
   3.701845850904187*^9}, {3.701925485602112*^9, 3.701925498438979*^9}, {
   3.701925535486704*^9, 3.70192556130245*^9}, 
   3.701928815520472*^9},ExpressionUUID->"8a0260be-16cf-49bf-b066-\
ce13e0aa3ec6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A\[Kappa]\[Iota]", "[", 
   RowBox[{"1", ",", "2"}], "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7018685694654417`*^9, 
  3.701868596746663*^9}},ExpressionUUID->"179b0095-aba3-4aa9-ad07-\
308e8bcf3f8f"],

Cell[TextData[{
 " and  ",
 Cell[BoxData[
  SubsuperscriptBox["B", "i", "j"]],
  Evaluatable->False,
  CellChangeTimes->{{3.633263302173919*^9, 3.6332633842828636`*^9}},
  FontWeight->"Bold",ExpressionUUID->"dc1efd4a-5f5a-4423-b451-8b1032f9e68c"],
 " =  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      SuperscriptBox["x", "j"]}], 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["y", "i"]}]], " "}], TraditionalForm]],ExpressionUUID->
  "e071dc09-a822-45b5-9f4f-c1c9965dbcf1"],
 " ",
 StyleBox["\[DotEqual] ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{
   RowBox[{"A\[Iota]\[Kappa]", "[", 
    RowBox[{"i", ",", "j"}], "]"}], "[", 
   RowBox[{
    RowBox[{"y", "[", "1", "]"}], ",", "...", ",", 
    RowBox[{"y", "[", "n", "]"}]}], " ", "]"}]],
  CellChangeTimes->{{3.6331875903732033`*^9, 3.6331876454609003`*^9}, 
    3.633188196498871*^9, 3.63326148156371*^9, {3.633262878720773*^9, 
    3.633262881965579*^9}},
  FontWeight->"Bold",ExpressionUUID->"ac9fb7f0-c36c-47c4-8304-737ab16f8bdf"],
 "with"
}], "Text",
 CellChangeTimes->{{3.634025963260782*^9, 3.6340259652731853`*^9}, {
  3.6340260006570473`*^9, 3.634026080241788*^9}, {3.6343613576012273`*^9, 
  3.634361404229709*^9}, {3.636800817574507*^9, 3.6368008186509094`*^9}, {
  3.6449950768541393`*^9, 3.6449950844201527`*^9}, {3.645278652059719*^9, 
  3.6452786562873263`*^9}},ExpressionUUID->"e44b5ae7-72b0-49e7-ad89-\
2bb2cbdef1a7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A\[Iota]\[Kappa]", "[", 
   RowBox[{"i_", ",", "j_"}], " ", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "\[Phi]\[Iota]\[Kappa]", "]"}], "[", 
     RowBox[{"{", "##", "}"}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"j", ",", "i"}], "]"}], "]"}], "&"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.63318724209979*^9, 3.6331872571860166`*^9}, {
   3.6331873076727057`*^9, 3.6331873845888414`*^9}, 3.633187431236923*^9, {
   3.6331874671979866`*^9, 3.633187546967927*^9}, 3.633187580808387*^9, {
   3.633187727835046*^9, 3.633187744467675*^9}, {3.6331878764685073`*^9, 
   3.633187908217563*^9}, {3.6331879483750343`*^9, 3.633187949872637*^9}, 
   3.633188182237446*^9, {3.6331882227537174`*^9, 3.633188246485359*^9}, {
   3.633261392261153*^9, 3.633261433277625*^9}, {3.633262798622232*^9, 
   3.6332628012742367`*^9}, {3.633262849295121*^9, 3.6332628675199537`*^9}, {
   3.6332629287882614`*^9, 3.633262959170315*^9}, {3.6339326026058846`*^9, 
   3.633932654460376*^9}, {3.634026606875716*^9, 3.6340266175305347`*^9}, {
   3.6401520004698086`*^9, 3.640152020547044*^9}, 3.6401520890779643`*^9, 
   3.6401609733567686`*^9, 3.644929950174735*^9, 3.644995209797573*^9, {
   3.6684326988761168`*^9, 3.668432706011655*^9}, {3.7018451316051083`*^9, 
   3.701845274192781*^9}, {3.7018455475273857`*^9, 3.701845565363554*^9}, {
   3.701925632737053*^9, 3.7019256619256144`*^9}, 
   3.701926852304085*^9},ExpressionUUID->"9b927eea-9275-4dd0-bbd5-\
3d4d22692ec1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A\[Iota]\[Kappa]", "[", 
   RowBox[{"1", ",", "2"}], "]"}], "[", 
  RowBox[{
   RowBox[{"y", "[", "1", "]"}], ",", 
   RowBox[{"y", "[", "2", "]"}], ",", 
   RowBox[{"y", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6331875903732033`*^9, 3.6331876454609003`*^9}, 
   3.633188196498871*^9, 3.63326148156371*^9, {3.633262878720773*^9, 
   3.633262881965579*^9}, 3.644995266740073*^9, 3.70184532726926*^9, {
   3.7018455088999977`*^9, 3.701845513667248*^9}, {3.7019256863810787`*^9, 
   3.7019256930152063`*^9}},ExpressionUUID->"387fa57b-23e5-422f-ab2b-\
76f54990eebf"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"f\[Iota]\[Kappa]", "[", "2", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "y", "]"}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.633188034945586*^9, 3.633188105713311*^9}, {
   3.6338729839209785`*^9, 3.6338729945289974`*^9}, 3.636800884451825*^9, 
   3.6449319704776974`*^9, {3.66834343932621*^9, 3.668343445253585*^9}, {
   3.668343489942136*^9, 3.668343490964765*^9}, 3.701845612131572*^9, {
   3.701925730707457*^9, 3.70192573360421*^9}, {3.70192884936867*^9, 
   3.7019288921081553`*^9}, 
   3.701928932323468*^9},ExpressionUUID->"f7014c7d-c29c-415c-86cf-\
0031e722e168"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A\[Iota]\[Kappa]", "[", 
   RowBox[{"2", ",", "1"}], "]"}], "[", 
  RowBox[{
   RowBox[{"y", "[", "1", "]"}], ",", 
   RowBox[{"y", "[", "2", "]"}], ",", 
   RowBox[{"y", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6331875903732033`*^9, 3.6331876454609003`*^9}, 
   3.633188196498871*^9, 3.63326148156371*^9, {3.633262878720773*^9, 
   3.633262881965579*^9}, {3.6339327356741185`*^9, 3.633932742709731*^9}, {
   3.633932783737803*^9, 3.633932815593059*^9}, 3.63411107589882*^9, 
   3.6449951797363205`*^9, {3.701845883543882*^9, 
   3.7018458882497663`*^9}},ExpressionUUID->"675f8a4b-6622-44b0-9425-\
101d1bfa58dd"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f\[Iota]\[Kappa]", "[", "1", "]"}], "[", 
     RowBox[{
      RowBox[{"y", "[", "1", "]"}], ",", 
      RowBox[{"y", "[", "2", "]"}], ",", 
      RowBox[{"y", "[", "3", "]"}]}], "]"}], ",", 
    RowBox[{"y", "[", "2", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.633188034945586*^9, 3.633188105713311*^9}, {
   3.6338729839209785`*^9, 3.6338729945289974`*^9}, {3.633932836060295*^9, 
   3.6339328610515385`*^9}, 3.6368859809079704`*^9, 3.636886103118986*^9, 
   3.6368861573360815`*^9, 
   3.6449321422152*^9},ExpressionUUID->"b58f61fb-daf3-4ec8-9a04-15c9a6470710"],

Cell["Remember", "Text",
 CellChangeTimes->{{3.634106451743905*^9, 
  3.6341064550511103`*^9}},ExpressionUUID->"2361787f-86ae-4a4c-954d-\
1f1166a5cfc0"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"jac", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.701929082672271*^9, 
  3.7019291011860323`*^9}},ExpressionUUID->"e15dce52-af7c-44dc-a31a-\
ed1da83cfff9"],

Cell[BoxData[
 RowBox[{
  SubscriptBox[
   RowBox[{"{", 
    SubsuperscriptBox["A", "i", "j"], "}"}], 
   RowBox[{"i", ",", "j"}]], "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{"jac", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6340267400485506`*^9, {3.634026776446415*^9, 3.634026812361678*^9}, {
   3.6340268487137423`*^9, 3.6340268869992094`*^9}, {3.634026999925208*^9, 
   3.634027019598843*^9}, {3.6340272794433007`*^9, 3.6340272877445154`*^9}, {
   3.634106471087939*^9, 3.634106499620389*^9}, 3.6449294994561405`*^9, {
   3.70192912482379*^9, 
   3.701929130941392*^9}},ExpressionUUID->"ccc05aa2-73cc-4a7a-899a-\
068b60cc5845"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"jac", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}], "==", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6340267400485506`*^9, {3.634026776446415*^9, 3.634026812361678*^9}, {
   3.6340268487137423`*^9, 3.6340268869992094`*^9}, {3.634026999925208*^9, 
   3.634027019598843*^9}, 3.634027089446166*^9, {3.6341065317564454`*^9, 
   3.6341065335192485`*^9}, {3.6449295006105423`*^9, 3.644929501624544*^9}, {
   3.66834355520581*^9, 
   3.6683436136839437`*^9}},ExpressionUUID->"fa38e938-962e-44e5-a259-\
31d0b8b3ab5d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"jac", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
   RowBox[{"vec", "[", "x", "]"}], "]"}], "\[Equal]", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"A\[Kappa]\[Iota]", "[", 
      RowBox[{"i", ",", "j"}], "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6340271407460566`*^9, 3.634027150886074*^9}, {
   3.634027191481346*^9, 3.6340272428416367`*^9}, 3.634106565312104*^9, 
   3.644929502794546*^9, 3.644995266740073*^9, {3.668343658547573*^9, 
   3.668343663171554*^9}},ExpressionUUID->"bb4ccb57-f842-45e3-964b-\
ea19d1b6413d"],

Cell["Finally we use the built-in", "Text",
 CellChangeTimes->{{3.634109493203247*^9, 
  3.634109508132473*^9}},ExpressionUUID->"83854caa-1319-4640-b1d3-\
a6270b0e4ba6"],

Cell[BoxData[
 RowBox[{"?", "KroneckerDelta"}]], "Input",
 CellChangeTimes->{{3.6341095140448837`*^9, 
  3.6341095219540977`*^9}},ExpressionUUID->"1f30cfc3-9f71-4339-a9d9-\
0284620cf0f5"],

Cell[BoxData[
 RowBox[{"?", "\[Delta]"}]], "Input",
 CellChangeTimes->{{3.634109634305495*^9, 
  3.634109636177498*^9}},ExpressionUUID->"227d3ee0-cb57-4b0d-b597-\
72872e492152"],

Cell["For shortness we set", "Text",
 CellChangeTimes->{{3.634109672073161*^9, 
  3.634109681791978*^9}},ExpressionUUID->"54673cd7-2c0c-49d7-8ea5-\
3ddd1e665080"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Delta]", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", " ", 
  RowBox[{"KroneckerDelta", "[", 
   RowBox[{"i", ",", "j"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.634109634305495*^9, 3.634109636177498*^9}, {
   3.6341096925871973`*^9, 3.6341097410876827`*^9}, 3.634109880974111*^9, {
   3.666360664213985*^9, 
   3.6663606866995087`*^9}},ExpressionUUID->"fce61884-9f64-476d-9db3-\
0942dc5e3370"],

Cell[BoxData[
 RowBox[{"\[Delta]", "[", 
  RowBox[{"2", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.666360777657338*^9, 
  3.666360800408538*^9}},ExpressionUUID->"c9d7161e-cd30-4395-a830-\
7075f86656e0"],

Cell[TextData[{
 "in traditional notation ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Delta]", "i", "j"], TraditionalForm]],ExpressionUUID->
  "d046a750-5122-49d8-9465-e95e60c56814"],
 " \[DotEqual] \[Delta][i,j]. With this Notations Lemma 1 implies the \
following formulas:"
}], "Text",
 CellChangeTimes->{{3.6341099743028765`*^9, 3.6341101656616144`*^9}, {
   3.6368727053801813`*^9, 3.6368727146153975`*^9}, 3.63687276150908*^9, {
   3.6368728034263535`*^9, 3.6368728056883574`*^9}, 
   3.6376759291810665`*^9},ExpressionUUID->"b9a19f14-8b97-4712-8141-\
69475667b797"],

Cell[TextData[{
 StyleBox["Corollary 1.",
  FontColor->RGBColor[0, 0, 1]],
 " The differentials of the transformations \[Phi]\[Kappa]",
 "\[Iota]",
 ", ",
 Cell[BoxData["\[Phi]\[Iota]\[Kappa]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"50fcb169-97c0-4818-97ac-c7ee1f648093"],
 " of the coordinates ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]\[Iota]", "[", "x", "]"}], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", "...", ",", 
      RowBox[{"x", "[", "n", "]"}]}], "}"}]}], ",", "   ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "\[Kappa]"], "[", "x", "]"}], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"y", "[", "1", "]"}], ",", "...", ",", 
      RowBox[{"y", "[", "n", "]"}]}], "}"}]}]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6330733981808405`*^9, 3.633073487780966*^9}},
  FontWeight->"Bold",ExpressionUUID->"8e1ba382-a738-4872-802c-99cb07f4fc4e"],
 "\nsatisfy the identities"
}], "Text",
 CellChangeTimes->{{3.6341102867972283`*^9, 3.6341102967988462`*^9}, {
   3.6341103797853923`*^9, 3.6341103974310236`*^9}, {3.6341104371380935`*^9, 
   3.6341104853325787`*^9}, 3.634110518003036*^9, {3.6341105785391426`*^9, 
   3.634110580769947*^9}, {3.6341362358444057`*^9, 3.634136291151503*^9}, {
   3.634136324898363*^9, 3.634136324898363*^9}, {3.6449299757139797`*^9, 
   3.6449300786227612`*^9}, {3.6663274577368765`*^9, 
   3.6663274577368765`*^9}},ExpressionUUID->"1c9261d6-0bf7-4a7f-8bc2-\
2162833b9b5a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"A\[Kappa]\[Iota]", "[", 
       RowBox[{"i", ",", "j"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", "...", ",", 
       RowBox[{"x", "[", "n", "]"}]}], "]"}], "*", 
     RowBox[{
      RowBox[{"A\[Iota]\[Kappa]", "[", 
       RowBox[{"j", ",", "k"}], "]"}], "[", 
      RowBox[{
       RowBox[{"y", "[", "1", "]"}], ",", "...", ",", 
       RowBox[{"y", "[", "n", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "n"}], "}"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{"\[Delta]", "[", 
   RowBox[{"i", ",", "k"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6341106905477405`*^9, {3.6341107264318037`*^9, 3.6341107513158474`*^9}, {
   3.63411081517676*^9, 3.634110922499349*^9}, 3.6342859982715797`*^9, 
   3.64499517975192*^9, 
   3.6449952667556734`*^9},ExpressionUUID->"e6a8da2e-0ee9-4ecc-8192-\
2f2278baeb62"],

Cell["or, in traditional notation:", "Text",
 CellChangeTimes->{{3.634112350965068*^9, 
  3.6341123675976973`*^9}},ExpressionUUID->"4f78100f-8411-415c-a1e8-\
2ba10a84f6d9"],

Cell[BoxData[
 RowBox[{"    ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubsuperscriptBox["A", "j", 
      RowBox[{"k", " "}]], 
     SubsuperscriptBox["B", "i", "j"]}], "=", " ", 
    SubsuperscriptBox["\[Delta]", "i", "k"]}], ";", "  ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["B", "j", 
      RowBox[{"k", " "}]], " ", 
     SubsuperscriptBox["A", "i", "j"]}], " ", "=", " ", 
    RowBox[{
     SubsuperscriptBox["\[Delta]", "i", "k"], ".", 
     StyleBox[
      RowBox[{" ", 
       StyleBox[" ",
        FontColor->GrayLevel[0]]}]], 
     StyleBox["\[FilledSquare]",
      FontColor->GrayLevel[0]]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6341105496594915`*^9, 3.6341105736095343`*^9}, {
  3.634112436665019*^9, 3.6341125989233046`*^9}, {3.6343605196925554`*^9, 
  3.6343606278163457`*^9}, {3.701929248462349*^9, 
  3.701929258505067*^9}},ExpressionUUID->"4b1b41e3-07f1-4efe-8778-\
daf29e85f419"],

Cell[TextData[{
 ".Traditionally one uses the",
 StyleBox[" sum convention",
  FontColor->RGBColor[0, 0, 1]],
 ": if in an expression the same coordinate index i appears twice, once as a \
subscript and once as a superscript, then the expression means the sum over \
the index range i = 1, ..., dim. "
}], "Text",
 CellChangeTimes->{{3.6376736055933585`*^9, 3.637673614065173*^9}, 
   3.6376736604334545`*^9},ExpressionUUID->"d09d4479-8c1f-404a-9bad-\
d8cc64459fa8"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.634020675043195*^9, 
  3.6340206951692305`*^9}},ExpressionUUID->"25213464-1076-4bba-ba34-\
69067b4e544b"],

Cell[TextData[{
 "To define the tangent space ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "a8caaa26-3bf1-466e-8bec-99318bd9310f"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "82ac4ed7-93a1-494b-a9e3-fe98a679ff0b"],
 "  of the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "e0cf3d13-4756-48ee-bfca-e6792c5afb83"],
 " at the point x we consider the set of pairs"
}], "Text",
 CellChangeTimes->{{3.636874272387334*^9, 3.636874276708542*^9}, {
   3.636874337876249*^9, 3.6368743863455343`*^9}, {3.6368744303064113`*^9, 
   3.636874437217223*^9}, {3.6368748026634655`*^9, 3.6368748062202716`*^9}, 
   3.636883496809066*^9, 3.6368845237349987`*^9, 3.666424277439842*^9, {
   3.6664243236107407`*^9, 
   3.666424341208145*^9}},ExpressionUUID->"eafae50c-24c5-4c3f-846c-\
19b6a7f33e98"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["V", "x"], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Iota]", ",", "\[Xi]"}], "}"}], "|", " ", 
    RowBox[{
     SubscriptBox["x\[Epsilon]U", "\[Iota]"], "&&", " ", 
     RowBox[{"\[Xi]", "\[Element]", 
      SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}]}]}], "}"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6368745476654177`*^9, 3.6368747985450583`*^9}, {
  3.6369496173887215`*^9, 
  3.6369496261287365`*^9}},ExpressionUUID->"2cc3c8ac-a5ca-48dc-9bb3-\
f48a397bb413"],

Cell["We introduce the equivalence relation", "Text",
 CellChangeTimes->{{3.6368835739128027`*^9, 
  3.6368836097324653`*^9}},ExpressionUUID->"3c177fbe-18b5-4bd0-9541-\
912c6dca9092"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Iota]", ",", "\[Xi]"}], "}"}], "\[TildeTilde]", " ", 
   RowBox[{"{", 
    RowBox[{"\[Kappa]", ",", "\[Eta]"}], "}"}]}], " ", ":=", " ", 
  RowBox[{"(", " ", 
   RowBox[{"\[Eta]", " ", "\[Equal]", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", "x", "]"}], ".", 
     "\[Xi]"}]}], ")"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.636883661262757*^9, 3.636883775357758*^9}, {
   3.636883812634224*^9, 3.636883820044237*^9}, {3.636883890299161*^9, 
   3.6368839134692016`*^9}, {3.6369459136629806`*^9, 
   3.6369459816125016`*^9}, {3.636946068003854*^9, 3.636946091437095*^9}, {
   3.6369483182062273`*^9, 3.6369483289254465`*^9}, 
   3.6449295051969504`*^9},ExpressionUUID->"caabc340-2b3b-4904-aa2b-\
695dbd55bc01"],

Cell[TextData[{
 "Here and sometimes in the following we write by abuse of notation x instead \
of correctly ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "47134b21-cfc7-4d35-b2a0-cf3830e11318"],
 "[x].\nFrom Lemma 1 and ",
 Cell[BoxData[
  RowBox[{"d", "[", 
   SubscriptBox["\[Phi]", 
    RowBox[{"\[Iota]", ",", "\[Iota]"}]], "]"}]],
  CellChangeTimes->{{3.636883661262757*^9, 3.636883775357758*^9}, {
   3.636883812634224*^9, 3.636883820044237*^9}, {3.636883890299161*^9, 
   3.6368839134692016`*^9}},
  FontWeight->"Bold",ExpressionUUID->"a25bcd50-c64f-4092-b9c9-a7fd44c90bcc"],
 " = identity it follows  immediately that \[OpenCurlyDoubleQuote]\
\[TildeTilde]\[CloseCurlyDoubleQuote] is indeed an equivalence relation. The \
tangent vectors",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Xi]\[Kappa]"],
  CellChangeTimes->{{3.6368737065587397`*^9, 3.6368737631400394`*^9}, {
   3.636873864727418*^9, 3.636873885693855*^9}},
  FontWeight->"Bold",ExpressionUUID->"cc93d9dc-8a1f-4c72-acfd-8a42941f9ca4"],
 StyleBox[", ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Xi]\[Iota]"],
  CellChangeTimes->{{3.6368737065587397`*^9, 3.6368737631400394`*^9}, {
   3.636873864727418*^9, 3.636873885693855*^9}},
  FontWeight->"Bold",ExpressionUUID->"b5b3fb3a-77e5-46f2-adad-54749024d849"],
 " of the coordinate representatives ",
 Cell[BoxData["\[Gamma]\[Kappa]"],
  CellChangeTimes->{{3.6368737065587397`*^9, 3.6368737631400394`*^9}, {
   3.636873864727418*^9, 3.636873885693855*^9}},
  FontWeight->"Bold",ExpressionUUID->"bc981b36-8873-41a6-8dbc-ae4b2ae4f489"],
 StyleBox[", ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Gamma]\[Iota]"],
  CellChangeTimes->{{3.6368737065587397`*^9, 3.6368737631400394`*^9}, {
   3.636873864727418*^9, 3.636873885693855*^9}},
  FontWeight->"Bold",ExpressionUUID->"afd048c6-dc59-483f-9303-75c95d9dc302"],
 " of the curve ",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{"\[Gamma]", "[", "t", "]"}]],
  CellChangeTimes->{{3.6368737065587397`*^9, 3.6368737631400394`*^9}, {
   3.636873864727418*^9, 3.636873885693855*^9}},
  FontWeight->"Bold",ExpressionUUID->"72185766-b013-45d4-9ccf-b98e4472471f"],
 "for t=0 are equivalent:"
}], "Text",
 CellChangeTimes->{{3.6368841358735933`*^9, 3.636884158716034*^9}, {
   3.6368841961275034`*^9, 3.636884296872881*^9}, {3.6368845418934307`*^9, 
   3.636884566059873*^9}, 3.6368846001167336`*^9, {3.636884655688031*^9, 
   3.6368847575974107`*^9}, {3.6368847877094636`*^9, 
   3.6368848697892084`*^9}, {3.644933252590357*^9, 3.644933252590357*^9}, {
   3.6449334380740843`*^9, 3.6449334380740843`*^9}, {3.6449939996348476`*^9, 
   3.6449939996348476`*^9}, {3.6449957187352667`*^9, 
   3.6449957187352667`*^9}, {3.6449958814903526`*^9, 3.644995966838103*^9}, {
   3.644995997772957*^9, 3.6449961095315533`*^9}, {3.6663274577525034`*^9, 
   3.6663274577525034`*^9}, {3.6664244635162325`*^9, 
   3.6664244922806654`*^9}, {3.668345404028748*^9, 3.668345418380546*^9}, {
   3.668345878143918*^9, 
   3.668345880775165*^9}},ExpressionUUID->"ec210915-76dc-4469-a34f-\
7ddcd7df3f19"],

Cell[BoxData["\[Xi]\[Iota]"], "Input",
 CellChangeTimes->{{3.6368737065587397`*^9, 3.6368737226423683`*^9}, 
   3.644993999650448*^9},ExpressionUUID->"1c1642a4-8a8c-4c11-85c8-\
b7bcc3ac403c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Xi]\[Kappa]", " ", ":=", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", "\[Gamma]\[Kappa]", "]"}], "[", "t", "]"}], "/.", 
     RowBox[{"t", "\[Rule]", "  ", "0"}]}], ")"}]}], ";", "\[Xi]\[Kappa]", 
  " "}]], "Input",
 CellChangeTimes->{{3.6368737065587397`*^9, 3.6368737631400394`*^9}, {
   3.636873864727418*^9, 3.636873885693855*^9}, 3.636885554764018*^9, 
   3.6449334295856686`*^9, 3.644995718750867*^9, {3.701939474485241*^9, 
   3.701939515017428*^9}, {3.701939741898878*^9, 3.701939898958693*^9}, 
   3.7019399395460052`*^9, {3.701940084856284*^9, 
   3.7019401028696823`*^9}},ExpressionUUID->"4adf81f9-054b-4d86-80a4-\
b70665d0d95a"],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"fx", "[", "0", "]"}], "[", "i", "]"}], "->", 
    RowBox[{"x", "[", "i", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}]}], "]"}]], "Input",ExpressionUUID->\
"ae56db06-9e54-46ff-bb60-c760f89c4228"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Xi]\[Kappa]", " ", "/.", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"fx", "[", "0", "]"}], "[", "1", "]"}], "\[Rule]", 
        RowBox[{"x", "[", "1", "]"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"fx", "[", "0", "]"}], "[", "2", "]"}], "\[Rule]", 
        RowBox[{"x", "[", "2", "]"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"fx", "[", "0", "]"}], "[", "3", "]"}], "\[Rule]", 
        RowBox[{"x", "[", "3", "]"}]}]}], "}"}]}], ")"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
     RowBox[{"vec", "[", "x", "]"}], "]"}], ".", "\[Xi]\[Iota]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6368739106538982`*^9, 3.6368739142419047`*^9}, {
   3.636873985003629*^9, 3.6368740089964714`*^9}, 3.6368855454946012`*^9, 
   3.644929506600953*^9, 3.644993999650448*^9, 3.644995718766467*^9, {
   3.668346039711485*^9, 3.6683460478463783`*^9}, {3.701939610340734*^9, 
   3.7019396337685547`*^9}, 3.701939958939508*^9, 3.701940006946146*^9, 
   3.701940414138983*^9, {3.701940525542383*^9, 
   3.7019405340576363`*^9}},ExpressionUUID->"d56af982-6ac6-445f-ab19-\
8efeef4d748b"],

Cell[TextData[{
 "and all these tangent vectors for all charts defined in a neighborhood of x \
form a complete equivalence class which we consider as ",
 StyleBox["the tangent vector of the curve",
  FontColor->RGBColor[0, 0, 1]],
 " \[Gamma][t] at t= 0  in the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "f8eb7f77-c703-48d8-a410-72ee3be7d66a"],
 ". Thus we define the ",
 StyleBox["tangent space",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "27334f20-5da2-4edb-9db2-1f660f0d9996"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "0f495587-4785-47ff-a13c-18eab024e0f1"],
 " of the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "e68d8f07-5a07-4507-8ab8-22595fb1503d"],
 " at the point x as the set of the equivalence classes:"
}], "Text",
 CellChangeTimes->{{3.6368849175176926`*^9, 3.6368852855225415`*^9}, {
  3.636886476427245*^9, 
  3.6368865061492977`*^9}},ExpressionUUID->"4f700bde-ee18-4f15-a25d-\
aaabdae567a8"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["T", "x"], 
    SuperscriptBox["M", "n"]}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["V", "x"], "/"}], "\[TildeTilde]"}]}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.636885309550584*^9, 
  3.636885360395074*^9}},ExpressionUUID->"6529259f-3ccb-4c66-bd24-\
3695926610ed"],

Cell[TextData[{
 "Let us write ",
 Cell[BoxData[
  StyleBox["\[Xi]",
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
    3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
    3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
    3.6369492905353436`*^9},ExpressionUUID->
  "2540d1e2-e44a-4520-91db-04e26b1b9dba"],
 "\[Iota]",
 " = {\[Iota], \[Xi]} etc.  for the elements in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", "x"], TraditionalForm]],ExpressionUUID->
  "4a48dc83-3cba-45ee-8b15-c67acb7b183d"],
 ". We set"
}], "Text",
 CellChangeTimes->{{3.6369495527226076`*^9, 3.636949558635018*^9}, {
  3.636949590853075*^9, 3.6369495994486895`*^9}, {3.636949656991393*^9, 
  3.636949777543805*^9}, {3.640064481267907*^9, 3.640064481267907*^9}, {
  3.6449961749892683`*^9, 
  3.6449962002457128`*^9}},ExpressionUUID->"e3f64e1f-d301-4874-95bd-\
c3b93358e74a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Xi]\[Iota]", " ", "=", " ", 
   RowBox[{"vec", "[", "\[Xi]", "]"}]}], ";", "  ", 
  RowBox[{"\[Eta]\[Iota]", "=", " ", 
   RowBox[{"vec", "[", "\[Eta]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
   3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
   3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
   3.6369492905353436`*^9, {3.636951209895336*^9, 3.6369512295835705`*^9}, 
   3.644993999650448*^9, {3.6449962254709573`*^9, 
   3.6449962507118015`*^9}},ExpressionUUID->"7ac34348-fcdb-4bfb-9429-\
9e57faf897d1"],

Cell[TextData[{
 "we consider the canonical map q:  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", "x"], TraditionalForm]],ExpressionUUID->
  "f6a12245-883d-45cc-aab3-7de407131855"],
 " \[LongRightArrow] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "a1c74033-4b6a-4ecd-99fe-f65514627cd6"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "5cd3baa5-c7e2-4a99-a16e-55f8253cb17d"],
 "   and define"
}], "Text",
 CellChangeTimes->{{3.6369498401767173`*^9, 3.636949868494767*^9}, {
  3.6369500991237745`*^9, 3.6369501533846703`*^9}, {3.636950191031536*^9, 
  3.6369502000055523`*^9}, {3.63764050755497*^9, 3.6376405078669705`*^9}, {
  3.666424674787487*^9, 3.6664246763008995`*^9}, {3.666424708235627*^9, 
  3.6664247085857787`*^9}},ExpressionUUID->"1df8c3d4-1d35-4ab4-90ec-\
3352cab76665"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"\[Xi]\[Iota]", "\[Element]", " ", 
    RowBox[{
     SubscriptBox["V", "x"], " ", "\[RightTeeArrow]", " ", 
     RowBox[{"X", "[", " ", "\[Xi]\[Iota]", "]"}]}]}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"q", "[", "\[Xi]\[Iota]", "]"}], "\[Element]", " ", 
    FormBox[
     RowBox[{
      SubscriptBox["T", "x"], 
      SuperscriptBox["M", "n"]}],
     TraditionalForm]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6369499036768293`*^9, 3.6369499105876412`*^9}, {
   3.6369499426808977`*^9, 3.6369499727451506`*^9}, {3.636950026729246*^9, 
   3.636950094412566*^9}, {3.6369502267595997`*^9, 3.636950251612443*^9}, 
   3.636950392821292*^9, {3.6369504260065503`*^9, 3.636950426677352*^9}, {
   3.644993999650448*^9, 
   3.644993999650448*^9}},ExpressionUUID->"82302d86-7ce8-40b9-9dd3-\
f53125102fed"],

Cell[TextData[{
 "Now we fix a chart ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", "\[Iota]"], TraditionalForm]],
  ExpressionUUID->"547ee02f-fb9f-4305-b8cf-72b3a3aaefc6"],
 " and consider the subset ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", 
    RowBox[{"x", ",", "\[Iota]"}]], TraditionalForm]],ExpressionUUID->
  "7e714871-2723-4104-9f28-144ceb3be977"],
 " := {",
 Cell[BoxData[
  StyleBox["\[Xi]\[Iota]",
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
    3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
    3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
    3.6369492905353436`*^9},ExpressionUUID->
  "f87b76c3-65ec-45fb-87ac-14b6ca95044a"],
 " \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", "x"], TraditionalForm]],ExpressionUUID->
  "d8d01557-8e13-46b1-9c2c-761c3dc56da7"],
 "}. Then the restriction"
}], "Text",
 CellChangeTimes->{{3.636950600183257*^9, 3.63695065829936*^9}, {
  3.6369506896594152`*^9, 3.6369507365414977`*^9}, {3.636950776543968*^9, 
  3.636950975137718*^9}, {3.6449963687416086`*^9, 
  3.6449963687416086`*^9}},ExpressionUUID->"b5be5825-915f-4342-a039-\
c7e8784c52fc"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["q", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "08988e95-b459-4816-bc6b-d9dafe619a24"],
 " := q|",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", 
    RowBox[{"x", ",", "\[Iota]"}]], TraditionalForm]],ExpressionUUID->
  "82c25444-83a9-4881-b026-4be7e18259e3"],
 "  is injective: "
}], "Text",
 CellChangeTimes->{{3.6369510124549837`*^9, 3.636951193261707*^9}, {
   3.6369512459187994`*^9, 3.6369512624704285`*^9}, {3.636951312396517*^9, 
   3.636951346002976*^9}, {3.636951392227857*^9, 3.636951473687601*^9}, {
   3.636953845813635*^9, 3.636953846328436*^9}, {3.6369539321803875`*^9, 
   3.636954005941317*^9}, {3.636954042929982*^9, 3.636954209360075*^9}, {
   3.636954242931334*^9, 3.6369542655991735`*^9}, {3.636954317941266*^9, 
   3.636954349983722*^9}, {3.6369543944312005`*^9, 3.6369544187984433`*^9}, {
   3.636954450031698*^9, 3.6369544758351436`*^9}, {3.6369545157108135`*^9, 
   3.6369545485976715`*^9}, {3.636954585336736*^9, 3.6369545876923404`*^9}, {
   3.6369546199863973`*^9, 3.636954768342058*^9}, {3.637640673742062*^9, 
   3.6376406819476767`*^9}, 3.6376759714067416`*^9, {3.6449301144766245`*^9, 
   3.6449301144766245`*^9}, {3.6449939996816473`*^9, 
   3.6449939996816473`*^9}, {3.644996458940967*^9, 3.6449964794238033`*^9}, {
   3.644996520046274*^9, 3.6449965200618744`*^9}, {3.645005984697015*^9, 
   3.645005990359825*^9}, {3.645006032401899*^9, 3.6450060397807117`*^9}, {
   3.6450063850093184`*^9, 3.6450064099693623`*^9}, {3.645006456504244*^9, 
   3.645006456504244*^9}, {3.6450065889952765`*^9, 3.6450065889952765`*^9}, 
   3.7019296683297777`*^9},ExpressionUUID->"c4dd7142-9831-466f-93de-\
4e7e309d0d45"],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  RowBox[{"\[Xi]\[Iota]", " "}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6369499036768293`*^9, 3.6369499105876412`*^9}, {
    3.6369499426808977`*^9, 3.6369499727451506`*^9}, {3.636950026729246*^9, 
    3.636950094412566*^9}, {3.6369502267595997`*^9, 3.636950251612443*^9}, 
    3.636950392821292*^9, {3.6369504260065503`*^9, 3.636950426677352*^9}},
  FontWeight->"Bold",ExpressionUUID->"4238bea2-e6c0-466c-9b73-e9dee8e770b4"],
 StyleBox["\[TildeTilde] ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Eta]"],
  CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
    3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
    3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
    3.6369492905353436`*^9, {3.636951209895336*^9, 3.6369512295835705`*^9}},
  FontWeight->"Bold",ExpressionUUID->"e129ac18-9b97-41c2-a502-966961caf3f7"],
 "\[Iota]",
 StyleBox[",",
  FontWeight->"Bold"],
 " then",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{"\[Xi]\[Iota]", " "}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6369499036768293`*^9, 3.6369499105876412`*^9}, {
    3.6369499426808977`*^9, 3.6369499727451506`*^9}, {3.636950026729246*^9, 
    3.636950094412566*^9}, {3.6369502267595997`*^9, 3.636950251612443*^9}, 
    3.636950392821292*^9, {3.6369504260065503`*^9, 3.636950426677352*^9}},
  FontWeight->"Bold",ExpressionUUID->"e21016ca-1257-4b81-8428-df7b4a737dbe"],
 StyleBox["=  ",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", 
    RowBox[{"\[Iota]", ",", "\[Iota]"}]], TraditionalForm]],
  FontWeight->"Bold",ExpressionUUID->"1d823b55-3725-4187-b4e3-859093a4b4ba"],
 Cell[BoxData[
  RowBox[{"[", "\[Eta]\[Iota]", "]"}]],
  CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
    3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
    3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
    3.6369492905353436`*^9, {3.636951209895336*^9, 3.6369512295835705`*^9}},
  FontWeight->"Bold",ExpressionUUID->"8785f621-ce23-42c9-acc2-3b6021e9b344"],
 StyleBox[" = ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Eta]\[Iota]"],
  CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
    3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
    3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
    3.6369492905353436`*^9, {3.636951209895336*^9, 3.6369512295835705`*^9}},
  FontWeight->"Bold",ExpressionUUID->"8a7feab9-1f9f-4d9a-89b6-b25fe6b584c5"],
 StyleBox[", ",
  FontWeight->"Bold"],
 "since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", 
    RowBox[{"\[Iota]", ",", "\[Iota]"}]], TraditionalForm]],ExpressionUUID->
  "25b16032-d986-41a0-8201-6c2eedcfe982"],
 " = identity. \nIt is surjective too:  If ",
 Cell[BoxData[
  StyleBox["\[Eta]",
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
    3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
    3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
    3.6369492905353436`*^9, {3.636951209895336*^9, 3.6369512295835705`*^9}},
  ExpressionUUID->"32d15760-d90c-4cb9-859f-cb37afef1bb0"],
 "\[Kappa]",
 " represents an element X \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "46293fe2-58ae-4494-97aa-e20aa7cdc0ab"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "ea5d7920-4de6-4595-b207-0f13b835b338"],
 ", then also  ",
 Cell[BoxData["\[Xi]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6369499036768293`*^9, 3.6369499105876412`*^9}, {
    3.6369499426808977`*^9, 3.6369499727451506`*^9}, {3.636950026729246*^9, 
    3.636950094412566*^9}, {3.6369502267595997`*^9, 3.636950251612443*^9}, 
    3.636950392821292*^9, {3.6369504260065503`*^9, 3.636950426677352*^9}},
  FontWeight->"Bold",ExpressionUUID->"504f70e6-2347-405e-a1a1-72c9d3e12dd8"],
 StyleBox[" = ",
  FontWeight->"Bold"],
 Cell[BoxData[
  StyleBox[
   RowBox[{"d", "[", "\[Phi]\[Iota]\[Kappa]", "]"}],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.6368739106538982`*^9, 3.6368739142419047`*^9}, {
    3.636873985003629*^9, 3.6368740089964714`*^9}, 3.6368855454946012`*^9},
  ExpressionUUID->"59448989-f273-4a8a-be4e-c99d93f8a916"],
 "[x]",
 StyleBox[".",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{" ", 
   StyleBox[
    RowBox[{
     StyleBox["\[Eta]",
      FontWeight->"Bold"], "\[Kappa]"}]]}]],
  CellChangeTimes->{{3.6369488101048946`*^9, 3.636948959461958*^9}, {
    3.636949019920065*^9, 3.6369490378600965`*^9}, {3.636949071183755*^9, 
    3.636949132910266*^9}, {3.636949180151149*^9, 3.6369492136016083`*^9}, 
    3.6369492905353436`*^9, {3.636951209895336*^9, 3.6369512295835705`*^9}},
  FontWeight->"Bold",ExpressionUUID->"b2b4f1b8-ab8b-4626-a1e3-32e251e31433"],
 " represents X.  Thus we may identify ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", 
    RowBox[{"x", ",", "\[Iota]"}]], TraditionalForm]],ExpressionUUID->
  "33b0262b-7b1c-482a-8215-a396141fde20"],
 " with",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  SuperscriptBox["\[DoubleStruckCapitalR]", "n"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6368745476654177`*^9, 3.6368747985450583`*^9}, {
   3.6369496173887215`*^9, 3.6369496261287365`*^9}},
  FontWeight->"Bold",ExpressionUUID->"9c2a1acf-dcdc-4649-9391-38e2f31af427"],
 "; applying the vector operations in ",
 Cell[BoxData[
  StyleBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "n"],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6368745476654177`*^9, 3.6368747985450583`*^9}, {
   3.6369496173887215`*^9, 3.6369496261287365`*^9}},ExpressionUUID->
  "b34599af-6f07-447e-b985-f421ef450465"],
 " we transfer the vector structure of the space ",
 Cell[BoxData[
  SuperscriptBox["\[DoubleStruckCapitalR]", "n"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6368745476654177`*^9, 3.6368747985450583`*^9}, {
   3.6369496173887215`*^9, 3.6369496261287365`*^9}},
  FontWeight->"Bold",ExpressionUUID->"79f1aec9-dccf-41ee-94ba-b123704689e3"],
 " to the tangent space  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "4b26b82d-e21a-4e1c-bd18-a31bd84a9a5a"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "448278d6-5610-4d65-9b94-fd0dd63848a7"],
 "  defining"
}], "Text",
 CellChangeTimes->{{3.6369510124549837`*^9, 3.636951193261707*^9}, {
   3.6369512459187994`*^9, 3.6369512624704285`*^9}, {3.636951312396517*^9, 
   3.636951346002976*^9}, {3.636951392227857*^9, 3.636951473687601*^9}, {
   3.636953845813635*^9, 3.636953846328436*^9}, {3.6369539321803875`*^9, 
   3.636954005941317*^9}, {3.636954042929982*^9, 3.636954209360075*^9}, {
   3.636954242931334*^9, 3.6369542655991735`*^9}, {3.636954317941266*^9, 
   3.636954349983722*^9}, {3.6369543944312005`*^9, 3.6369544187984433`*^9}, {
   3.636954450031698*^9, 3.6369544758351436`*^9}, {3.6369545157108135`*^9, 
   3.6369545485976715`*^9}, {3.636954585336736*^9, 3.6369545876923404`*^9}, {
   3.6369546199863973`*^9, 3.636954768342058*^9}, {3.637640673742062*^9, 
   3.6376406819476767`*^9}, 3.6376759714067416`*^9, {3.6449301144766245`*^9, 
   3.6449301144766245`*^9}, {3.6449939996816473`*^9, 
   3.6449939996816473`*^9}, {3.644996458940967*^9, 3.6449964794238033`*^9}, {
   3.644996520046274*^9, 3.6449965200618744`*^9}, {3.645005984697015*^9, 
   3.645005990359825*^9}, {3.645006032401899*^9, 3.6450060397807117`*^9}, {
   3.6450063850093184`*^9, 3.6450064099693623`*^9}, {3.645006456504244*^9, 
   3.645006456504244*^9}, {3.6450065889952765`*^9, 3.6450065889952765`*^9}, 
   3.701929683174674*^9},ExpressionUUID->"7ef21eb7-e8f4-4590-a9ca-\
ea6027a29dc2"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"X", "[", " ", "\[Xi]\[Iota]", "]"}], " ", "+", "  ", 
     RowBox[{"X", "[", " ", "\[Eta]\[Iota]", "]"}]}], " ", ":=", "  ", 
    RowBox[{
     RowBox[{"X", "[", " ", 
      RowBox[{"\[Xi]\[Iota]", "+", " ", "\[Eta]\[Iota]"}], "]"}], " ", "for", 
     "  ", "\[Xi]\[Iota]"}]}], ",", 
   RowBox[{
    RowBox[{"\[Eta]\[Iota]", " ", "\[Element]", " ", 
     SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}], ";"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6369548113991337`*^9, 3.6369548944148803`*^9}, {
  3.636954990359049*^9, 3.636955087099819*^9}, {3.636955290736977*^9, 
  3.6369553109546127`*^9}, {3.6449939996816473`*^9, 3.6449939996816473`*^9}, {
  3.6450065890108767`*^9, 
  3.6450065890264764`*^9}},ExpressionUUID->"16163534-ffa3-440d-b6b6-\
a6dbbfefeab7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a", "*", 
    RowBox[{"X", "[", " ", "\[Xi]\[Iota]", "]"}]}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"X", "[", " ", 
      RowBox[{"a", "*", "\[Xi]\[Iota]"}], "]"}], " ", "for", " ", 
     "\[Xi]\[Iota]"}], "\[Element]", " ", 
    SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}]}], ",", " ", 
  RowBox[{"a", "\[Element]", " ", 
   RowBox[{"\[DoubleStruckCapitalR]", "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6369549066141014`*^9, 3.636954980749432*^9}, {
  3.6369550968810363`*^9, 3.636955158316944*^9}, {3.636955321314031*^9, 
  3.6369553243092365`*^9}, {3.6449939996816473`*^9, 
  3.6449939996972475`*^9}},ExpressionUUID->"3f56989a-08a6-43d8-916e-\
506dddf7bb5c"],

Cell[TextData[{
 "Since the representatives of the elements for distinct charts \[Iota], \
\[Kappa] are related by linear isomorphisms ",
 Cell[BoxData[
  RowBox[{"d", "[", "\[Phi]\[Iota]\[Kappa]", "]"}]],
  CellChangeTimes->{{3.6368739106538982`*^9, 3.6368739142419047`*^9}, {
    3.636873985003629*^9, 3.6368740089964714`*^9}, 3.6368855454946012`*^9},
  FontWeight->"Bold",ExpressionUUID->"7ef9ea1c-929e-48fa-b574-98e237c2e043"],
 StyleBox["[x]",
  FontWeight->"Bold"],
 ", these definitions are independent  of  the chosen chart.  We summarize"
}], "Text",
 CellChangeTimes->{{3.6369554218496084`*^9, 3.636955539420816*^9}, {
   3.6369555946489134`*^9, 3.6369556226529627`*^9}, {3.636955658226418*^9, 
   3.6369557618166003`*^9}, {3.6369558983750415`*^9, 
   3.6369560944133873`*^9}, {3.6369561353382597`*^9, 
   3.6369561712555227`*^9}, {3.6369562038323803`*^9, 
   3.6369562311988287`*^9}, {3.636956264493287*^9, 3.6369564531634197`*^9}, {
   3.6369566478445635`*^9, 3.6369566748990107`*^9}, {3.6369567223124943`*^9, 
   3.6369568017538347`*^9}, {3.636956834003091*^9, 3.636956841134304*^9}, 
   3.636956896957202*^9, 3.636956939036476*^9, {3.6449301168946285`*^9, 
   3.6449301168946285`*^9}},ExpressionUUID->"6abbfa2d-d5c9-4185-9e15-\
b6597d497bbb"],

Cell[TextData[{
 StyleBox["Proposition 1.",
  FontColor->RGBColor[0, 0, 1]],
 "  For each x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6369499036768293`*^9, 3.6369499105876412`*^9}, {
    3.6369499426808977`*^9, 3.6369499727451506`*^9}, {3.636950026729246*^9, 
    3.636950094412566*^9}, {3.6369502267595997`*^9, 3.636950251612443*^9}, 
    3.636950392821292*^9, {3.6369504260065503`*^9, 3.636950426677352*^9}},
  ExpressionUUID->"12032ab6-2c08-4d06-ab29-40cba2c1b68b"],
 " the tangent space ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "ef7f2383-37e8-4192-874e-b376da9ca283"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "f667f795-c2b2-4ce4-83d2-f49c4e186d8d"],
 "  with the just defined operations is a real n-dimensional vector space.  \
Any tangent vector X \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "fef8fe09-8305-48bf-b28c-863229052bcf"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "04fe7e45-33c8-4418-815c-62252e972b84"],
 " has in each chart ",
 Cell[BoxData["\[Phi]\[Iota]"],
  CellChangeTimes->{{3.6368739106538982`*^9, 3.6368739142419047`*^9}, {
    3.636873985003629*^9, 3.6368740089964714`*^9}, 3.6368855454946012`*^9},
  FontWeight->"Bold",ExpressionUUID->"4f7bc04e-155e-443a-9c47-649982b493a0"],
 " with x \[Element]",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "c997f424-bfba-4d09-a7d8-c6a0ce81d998"],
 " a uniquely defined representative ",
 Cell[BoxData["\[Xi]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6369549066141014`*^9, 3.636954980749432*^9}, {
   3.6369550968810363`*^9, 3.636955158316944*^9}, {3.636955321314031*^9, 
   3.6369553243092365`*^9}},
  FontWeight->"Bold",ExpressionUUID->"70f91268-993f-4deb-abef-7e2533c9dc74"],
 Cell[BoxData[
  RowBox[{"\[Element]", " ", 
   SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6369548113991337`*^9, 3.6369548944148803`*^9}, {
   3.636954990359049*^9, 3.636955087099819*^9}, {3.636955290736977*^9, 
   3.6369553109546127`*^9}},
  FontWeight->"Bold",ExpressionUUID->"1d289f72-d83a-45f2-b4f0-9e361a78f1be"],
 " whose components ",
 Cell[BoxData["\[Xi]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6369549066141014`*^9, 3.636954980749432*^9}, {
   3.6369550968810363`*^9, 3.636955158316944*^9}, {3.636955321314031*^9, 
   3.6369553243092365`*^9}},
  FontWeight->"Bold",ExpressionUUID->"87e887d2-e7d6-4762-b82f-646deff491e8"],
 "[j] are named the ",
 StyleBox["vector coordinates of",
  FontColor->RGBColor[0, 0, 1]],
 " X in the chart  ",
 Cell[BoxData["\[Phi]\[Iota]"],
  CellChangeTimes->{{3.6368739106538982`*^9, 3.6368739142419047`*^9}, {
    3.636873985003629*^9, 3.6368740089964714`*^9}, 3.6368855454946012`*^9},
  FontWeight->"Bold",ExpressionUUID->"71bc37b3-1183-4543-84a5-46d12529c0ef"],
 ". The union T",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "f7b1a332-0b54-47f9-b234-09f07d3bb66b"],
 ":= ",
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Union]", 
    RowBox[{"x", "\[Element]", 
     SuperscriptBox["M", "n"]}]], TraditionalForm]],ExpressionUUID->
  "915c6f59-86d2-40d1-846a-0bc0a1921702"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "874a7c7e-1d76-425c-ac88-35450de55f5c"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "4debfe2b-18e7-43e4-9ec6-0401b67fd721"],
 " is named the",
 StyleBox[" tangent bundle ",
  FontColor->RGBColor[0, 0, 1]],
 "of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "fa5ead46-d5b9-4c91-b11e-0fc18f143cb6"],
 " with the ",
 StyleBox["projection",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.6369554218496084`*^9, 3.636955539420816*^9}, {
   3.6369555946489134`*^9, 3.6369556226529627`*^9}, {3.636955658226418*^9, 
   3.6369557618166003`*^9}, {3.6369558983750415`*^9, 
   3.6369560944133873`*^9}, {3.6369561353382597`*^9, 
   3.6369561712555227`*^9}, {3.6369562038323803`*^9, 
   3.6369562311988287`*^9}, {3.636956264493287*^9, 3.6369564531634197`*^9}, {
   3.6369566478445635`*^9, 3.6369566748990107`*^9}, {3.6369567223124943`*^9, 
   3.6369568017538347`*^9}, {3.636956834003091*^9, 3.636956841134304*^9}, 
   3.636956896957202*^9, {3.636956939036476*^9, 3.6369569690393295`*^9}, {
   3.6369570043793917`*^9, 3.636957120937598*^9}, {3.636957153966856*^9, 
   3.636957182766507*^9}, {3.6369572292055902`*^9, 3.6369572766648736`*^9}, 
   3.6376409201600947`*^9, {3.6449939996972475`*^9, 3.6449939997128477`*^9}, {
   3.6663274577681274`*^9, 
   3.6663274577681274`*^9}},ExpressionUUID->"3d745f63-1935-43e7-8ebb-\
c511a865c08f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"p", ":", 
    RowBox[{"X", "\[Element]", 
     RowBox[{
      RowBox[{"T", 
       FormBox[
        SuperscriptBox["M", 
         RowBox[{"n", " "}]],
        TraditionalForm]}], "\[RightTeeArrow]", " ", 
      RowBox[{"p", "[", "X", "]"}]}]}]}], " ", ":=", " ", 
   RowBox[{"x", "\[Element]", 
    FormBox[
     SuperscriptBox["M", 
      RowBox[{"n", " "}]],
     TraditionalForm]}]}], ",", " ", 
  RowBox[{
   RowBox[{"if", "  ", "X"}], "\[Element]", 
   RowBox[{
    FormBox[
     RowBox[{
      SubscriptBox["T", "x"], 
      SuperscriptBox["M", "n"]}],
     TraditionalForm], "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.636957287011692*^9, 3.6369575000062675`*^9}, {
   3.636957532645525*^9, 3.63695753545453*^9}, {3.636957615773472*^9, 
   3.6369576224814835`*^9}, 3.6369577599783273`*^9, 3.637558836878151*^9, {
   3.6376690796392403`*^9, 
   3.637669083539247*^9}},ExpressionUUID->"56e99d69-6a23-4613-b290-\
1aabd53ebc73"],

Cell[TextData[{
 "The",
 StyleBox[" fibers",
  FontColor->RGBColor[0, 0, 1]],
 " of the bundle are the tangent spaces  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "fde01d31-2553-45b8-8bd0-22427afe709c"],
 "[x] = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "fde99b26-ff7f-43b1-9fdf-9ee9a66244c3"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "3fc367f8-1008-4b63-8c80-015589ffe0fd"],
 ", and the ",
 StyleBox["fiber type",
  FontColor->RGBColor[0, 0, 1]],
 " is ",
 Cell[BoxData[
  SuperscriptBox["\[DoubleStruckCapitalR]", "n"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6369548113991337`*^9, 3.6369548944148803`*^9}, {
   3.636954990359049*^9, 3.636955087099819*^9}, {3.636955290736977*^9, 
   3.6369553109546127`*^9}},
  FontWeight->"Bold",ExpressionUUID->"d71dfdd6-e3b9-4321-83b0-9672921bff76"],
 ". If ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "d729089e-80cf-41c6-a44b-e8ef899c8895"],
 " is a differentiable manifold of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "023d6122-16c4-4b11-852e-d10723568531"],
 " then T",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "8de9929f-7050-4aa3-bd85-027567d5680e"],
 " is a 2n-dimensional manifold  of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "646142ea-2e19-43fe-9b4d-7851860ede20"],
 " whose atlas is uniquely defined by the atlas of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "949b3d43-6528-44f9-b94b-eafcac178f8b"],
 " in a canonical way (see the proof).\n",
 StyleBox["Proof. ",
  FontColor->RGBColor[0, 0, 1]],
 "Only the last statement of Proposition 1  remains to show. Aiming this we \
first define local vector fields on the domain ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "c61e9373-3580-4593-8a29-84de356a07f2"],
 " of  a chart ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", "\[Iota]"], TraditionalForm]],
  ExpressionUUID->"34828eff-625b-4d93-87ec-ec6c260ac9e1"],
 ". The",
 StyleBox[" local standard basis in the chart ",
  FontColor->RGBColor[0, 0, 1]],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", "\[Iota]"], TraditionalForm]],
  ExpressionUUID->"aca9e70f-d8bf-43c3-82ff-827a2a3c4f8c"],
 " consists of the local vector fields corresponding to the standard basis \
vectors of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "n"], TraditionalForm]],ExpressionUUID->
  "a82bed9c-a9b1-4528-8743-02019c485e8f"],
 ":"
}], "Text",
 CellChangeTimes->{{3.6369577765465565`*^9, 3.636957953830669*^9}, {
   3.6369580972406445`*^9, 3.636958097505845*^9}, {3.6369581297063017`*^9, 
   3.636958129987102*^9}, {3.6369582642347393`*^9, 3.636958377692139*^9}, {
   3.6369584236422205`*^9, 3.6369584888583355`*^9}, {3.636958519453989*^9, 
   3.636958662881042*^9}, {3.6369587495949945`*^9, 3.6369587825774527`*^9}, {
   3.6375645248930697`*^9, 3.637564528418676*^9}, {3.6375645692907476`*^9, 
   3.6375645920511875`*^9}, {3.6375646595584393`*^9, 
   3.6375648571413903`*^9}, {3.6375649024402704`*^9, 3.6375649130034895`*^9}, 
   3.6375649643023796`*^9, 3.6376759859323673`*^9, {3.64006470922931*^9, 
   3.6400647141277184`*^9}, {3.6400647563961935`*^9, 3.6400647639642067`*^9}, 
   3.700458352929599*^9},ExpressionUUID->"2b86b916-b995-4db3-a5a3-\
e824b0e36898"],

Cell[BoxData[
 RowBox[{"?", "tx\[Iota]"}]], "Input",
 CellChangeTimes->{{3.6664263903545227`*^9, 
  3.6664264081757746`*^9}},ExpressionUUID->"55e715e0-7050-4a7c-aa15-\
5165972f86ef"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tx\[Iota]", "[", "i_", "]"}], "[", "x__", "]"}], " ", ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"Range", "[", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}], "]"}], ",", "i"}], "]"}], ",", " ", 
    RowBox[{"stb", "[", 
     RowBox[{"i", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Index \>\"", ",", "i", ",", "\"\< invalid!\>\""}], "]"}], 
     ";", 
     RowBox[{"Return", "[", "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.633070866967064*^9, 3.6330709280295563`*^9}, {
   3.633071046021305*^9, 3.6330710629562736`*^9}, 3.633071213934909*^9, {
   3.6330712545012293`*^9, 3.6330712936524687`*^9}, {3.637559598703489*^9, 
   3.6375596367675557`*^9}, {3.6375597385577345`*^9, 
   3.6375597461081476`*^9}, {3.6375597878838205`*^9, 
   3.6375597913314266`*^9}, {3.637560271001069*^9, 3.637560347659604*^9}, {
   3.637560387548874*^9, 3.6375604732554245`*^9}, 3.6452543116060424`*^9, {
   3.645255349691066*^9, 3.6452553781923156`*^9}, {3.645255439032422*^9, 
   3.6452554645384674`*^9}, {3.6452663046860933`*^9, 
   3.6452663181801176`*^9}, {3.666425038957491*^9, 3.6664251047316422`*^9}, {
   3.6664252013030863`*^9, 
   3.666425230870733*^9}},ExpressionUUID->"bccf41a0-ce24-4cf0-b505-\
42102a025f55"],

Cell[BoxData[
 RowBox[{"Context", "[", "tx\[Iota]", "]"}]], "Input",
 CellChangeTimes->{{3.668346502396817*^9, 
  3.668346523747037*^9}},ExpressionUUID->"3999c239-3f0c-41f3-a8d1-\
95a3fabf674c"],

Cell[TextData[{
 "Here is the definition of a",
 StyleBox[" local vector field",
  FontColor->RGBColor[0, 0, 1]],
 " with coordinates v "
}], "Text",
 CellChangeTimes->{{3.6375863323959227`*^9, 3.637586350803955*^9}, {
  3.637644457703023*^9, 3.6376444869842744`*^9}, {3.668400342435293*^9, 
  3.668400347603305*^9}},ExpressionUUID->"9bd2b47a-9d87-44a3-806f-\
0e77ca35eb2e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"X\[Iota]", "[", "v_", "]"}], "[", "x__", "]"}], ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", "v", "]"}], " ", "\[Equal]", " ", 
     RowBox[{"Length", "[", 
      RowBox[{"{", "x", "}"}], "]"}]}], ",", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"v", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "*", 
       RowBox[{
        RowBox[{"tx\[Iota]", "[", "i", "]"}], "[", "x", "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Print", "[", "\"\<Arguments invalid!\>\"", "]"}], ";", 
     RowBox[{"Return", "[", "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6375606340449066`*^9, 3.637560642999323*^9}, {
   3.6375606872254004`*^9, 3.6375607419658966`*^9}, {3.6375607864103746`*^9, 
   3.6375609099781914`*^9}, {3.6375609668246913`*^9, 3.6375611754126577`*^9}, 
   3.6375612180631323`*^9, {3.637561248436386*^9, 3.6375612699020233`*^9}, {
   3.645254034845956*^9, 3.6452540538935895`*^9}, {3.6452541776642075`*^9, 
   3.64525425310594*^9}, 3.645254311621642*^9, {3.666447513188171*^9, 
   3.6664475198447604`*^9}, 3.666447913146371*^9, 3.6664479476481667`*^9, 
   3.6684002773010263`*^9},ExpressionUUID->"59233c31-38d9-41a7-85e3-\
738d7ff32298"],

Cell[BoxData[
 RowBox[{"?", "X\[Iota]"}]], "Input",
 CellChangeTimes->{{3.6683994011216927`*^9, 
  3.668399421547192*^9}},ExpressionUUID->"70779eab-a454-4869-b9e1-\
1b8253cdec55"],

Cell[BoxData[
 RowBox[{"Context", "[", "X\[Iota]", "]"}]], "Input",
 CellChangeTimes->{{3.668346863516397*^9, 
  3.6683468987606087`*^9}},ExpressionUUID->"d4a628af-e6cf-4ea0-bc65-\
c7831b3d3cc0"],

Cell[TextData[StyleBox["Examples of local vector fields",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6400650419804983`*^9, 3.64006505440012*^9}, {
  3.6400688117430134`*^9, 
  3.6400688231214333`*^9}},ExpressionUUID->"5706883f-b437-49ab-9d23-\
4928f753b0db"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"X\[Iota]", "[", 
    RowBox[{"stb", "[", "2", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"tx\[Iota]", "[", "2", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6375642475869827`*^9, 3.6375643018126783`*^9}, {
   3.637564342325949*^9, 3.6375643508747644`*^9}, {3.637586478474579*^9, 
   3.637586482000185*^9}, 3.6452543116372423`*^9, 
   3.645254546651655*^9},ExpressionUUID->"18b6b422-ee0f-4776-9e45-\
9ee2816f000e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"X\[Iota]", "[", 
    RowBox[{"stb", "[", "2", "]"}], "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.70193883714386*^9, 
  3.701938884414256*^9}},ExpressionUUID->"be12dd12-6e67-4e9a-8740-\
549d750bfbbf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "Z", "]"}], ";", 
  RowBox[{
   RowBox[{"Z", "[", 
    RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"X\[Iota]", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "x", "]"}], ",", 
       RowBox[{"Sin", "[", "x", "]"}], ",", "z"}], "}"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6375613365453405`*^9, 3.637561363299388*^9}, {
   3.6375614211286893`*^9, 3.637561460393958*^9}, {3.637561940906002*^9, 
   3.6375619864736824`*^9}, {3.6375867847655168`*^9, 
   3.6375868677420626`*^9}, {3.63758712252171*^9, 3.6375871385117383`*^9}, {
   3.637587343464898*^9, 3.637587421199835*^9}, {3.637587577434109*^9, 
   3.637587581926917*^9}, {3.6375876534062424`*^9, 3.6375877405947957`*^9}, 
   3.645254546651655*^9},ExpressionUUID->"93daec75-ba8c-4fe0-9474-\
6be684a5f53e"],

Cell[BoxData[
 RowBox[{"Z", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.63758719086543*^9},ExpressionUUID->"f4598c45-9349-4abf-95fe-b5b1950c615f"],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"vec", "[", "x", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6375620200761414`*^9, 3.637562067937025*^9}, {
  3.668347068592556*^9, 3.6683470732392683`*^9}, {3.668347112255419*^9, 
  3.668347119867999*^9}},ExpressionUUID->"ce9dd74e-7ac3-4b56-90df-\
26fff1e742a3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "Z", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.701929848161126*^9, 
  3.7019298877868156`*^9}},ExpressionUUID->"1e73f315-2bca-408e-bcf9-\
3c0a240c1857"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Transpose", "[", "%", "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6375867041601753`*^9, 3.637586713083391*^9}, {
  3.6376454957668457`*^9, 
  3.6376455225676928`*^9}},ExpressionUUID->"a478193a-783b-4783-8133-\
b031d7ac0930"],

Cell["\<\
The  differential of the field can be calculated using the definition of the \
field \
\>", "Text",
 CellChangeTimes->{{3.6375881020474305`*^9, 3.6375881369134917`*^9}, 
   3.6376759944967823`*^9, {3.6683972763420153`*^9, 3.668397318261352*^9}, {
   3.668400487170063*^9, 3.668400509849792*^9}, {3.701929400567136*^9, 
   3.701929402999132*^9}},ExpressionUUID->"5c53f304-06ad-46a5-a9a2-\
cd9b34382abf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"X\[Iota]", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Cos", "[", 
      RowBox[{"x", "[", "1", "]"}], "]"}], ",", 
     RowBox[{"Sin", "[", 
      RowBox[{"x", "[", "1", "]"}], "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "}"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.701930405822781*^9},ExpressionUUID->"208dacc3-7c24-436c-a738-\
33aae448bc67"],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"X\[Iota]", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", 
        RowBox[{"x", "[", "1", "]"}], "]"}], ",", 
       RowBox[{"Sin", "[", 
        RowBox[{"x", "[", "1", "]"}], "]"}], ",", 
       RowBox[{"x", "[", "3", "]"}]}], "}"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"vec", "[", "x", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6375878634138117`*^9, 3.63758795933838*^9}, 
   3.6452545466672554`*^9, {3.645271421478219*^9, 3.6452714262674274`*^9}, 
   3.645271472849109*^9, 3.645271535280419*^9, {3.668397101536149*^9, 
   3.6683971132878923`*^9}, {3.668397161545314*^9, 3.668397173871932*^9}, {
   3.70193002439075*^9, 3.7019300308819313`*^9}, {3.701930113593465*^9, 
   3.701930115131597*^9}, 3.701930154448947*^9, {3.701930237557048*^9, 
   3.701930240344261*^9}, {3.701930278412711*^9, 3.701930280736206*^9}, {
   3.7019303226481047`*^9, 3.7019303578232603`*^9}, {3.701930559018652*^9, 
   3.701930607749175*^9}},ExpressionUUID->"8987f044-680d-455a-84dc-\
142683a21d5d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"X\[Iota]", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Cos", "[", 
       RowBox[{"x", "[", "1", "]"}], "]"}], ",", 
      RowBox[{"Sin", "[", 
       RowBox[{"x", "[", "1", "]"}], "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "}"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6375878634138117`*^9, 3.63758795933838*^9}, 
   3.6452545466672554`*^9, {3.645271421478219*^9, 3.6452714262674274`*^9}, 
   3.645271472849109*^9, 3.645271535280419*^9, {3.668397101536149*^9, 
   3.6683971132878923`*^9}, {3.668397161545314*^9, 3.668397173871932*^9}, {
   3.70193002439075*^9, 3.7019300308819313`*^9}, {3.701930113593465*^9, 
   3.701930115131597*^9}, 3.701930154448947*^9, {3.701930237557048*^9, 
   3.701930240344261*^9}, {3.701930278412711*^9, 3.701930280736206*^9}, {
   3.7019303226481047`*^9, 3.7019303578232603`*^9}, {3.701930559018652*^9, 
   3.701930607749175*^9}, {3.701930783525302*^9, 
   3.701930840450842*^9}},ExpressionUUID->"8688feae-8d20-4979-adbe-\
79f2eb555543"],

Cell["\[FilledSmallSquare]", "Text",
 CellChangeTimes->{{3.640068778867755*^9, 
  3.640068778867755*^9}},ExpressionUUID->"e01f48df-8988-4c6e-883b-\
ab87dd30426c"],

Cell[TextData[{
 "A smooth curve whose tangent vector at the point x with t -> 0 is  given by \
its coordinate representative \[Xi] \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "n"], TraditionalForm]],
  ExpressionUUID->"9f5ea3dc-ec69-46d6-a925-5731d6225fec"],
 " in the chart ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "76126817-ef51-4287-ac8f-74f8834ea7ee"],
 "  is obtained by"
}], "Text",
 CellChangeTimes->{{3.6400691415971956`*^9, 3.640069219721534*^9}, {
  3.6400692503483877`*^9, 3.640069322372115*^9}, {3.640069356586975*^9, 
  3.6400694978686256`*^9}, {3.640069534939291*^9, 3.64006955694133*^9}, {
  3.64006961967704*^9, 3.640069640802478*^9}, {3.666327457799379*^9, 
  3.666327457799379*^9}},ExpressionUUID->"ea97bcb5-d9dc-46b6-b7c4-\
b9ce08f5170c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Gamma]", "\[Iota]"], "[", "\[Xi]_", "]"}], "[", "t_", 
   "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"vec", "[", "x", "]"}], " ", "+", " ", 
   RowBox[{"t", "*", 
    RowBox[{"vec", "[", "\[Xi]", "]"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6400695926557927`*^9, 3.640069599566605*^9}, {
   3.6400697272728305`*^9, 3.64006977822752*^9}, 3.6449332541503596`*^9, {
   3.700459356308567*^9, 3.7004594426756144`*^9}, {3.7019408710192204`*^9, 
   3.70194088362307*^9}},ExpressionUUID->"1200bd37-447c-4b1c-ba4e-\
4da6182a8905"],

Cell[TextData[{
 Cell[BoxData[
  FormBox["tx\[Iota]", TraditionalForm]],ExpressionUUID->
  "028c6f7c-4c3e-487a-acec-c5a35be24594"],
 "[i]  is the tangent vector of the i-th coordinate line of the chart ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "84b3005b-7bab-4b47-9e54-8711aff4c3c4"],
 ":"
}], "Text",
 CellChangeTimes->{{3.633071979622704*^9, 3.633072122202859*^9}, {
   3.6330721571068554`*^9, 3.6330721590989695`*^9}, 3.6330723850908957`*^9, {
   3.6452543116372423`*^9, 3.6452543116372423`*^9}, {3.666327457799379*^9, 
   3.666327457799379*^9}},ExpressionUUID->"43ba257e-c56a-4bb7-9467-\
3566c2512534"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Gamma]", "\[Iota]"], "[", "\[Xi]", "]"}], "[", "t", 
  "]"}]], "Input",
 CellChangeTimes->{{3.640069853492053*^9, 3.6400698556000566`*^9}, 
   3.644933255616762*^9, 3.700459571653984*^9, {3.7019408957484083`*^9, 
   3.7019408965585423`*^9}},ExpressionUUID->"4cb635ea-cea8-45e0-9f75-\
06e29cabf66f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Gamma]", "\[Iota]"], "[", "\[Xi]", "]"}], "[", "t", "]"}],
   "/.", 
  RowBox[{
   RowBox[{"\[Xi]", "[", "i_", "]"}], "\[Rule]", " ", 
   RowBox[{"KroneckerDelta", "[", 
    RowBox[{"i", ",", "2"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6330714535486145`*^9, 3.633071459372947*^9}, {
   3.633071492669852*^9, 3.6330715145251017`*^9}, {3.6330715798698397`*^9, 
   3.63307162708554*^9}, {3.6330717251571493`*^9, 3.6330717746609807`*^9}, {
   3.6330718118611083`*^9, 3.6330718269329705`*^9}, 3.644933257067565*^9, 
   3.700459573015091*^9, {3.701940909883011*^9, 
   3.701940910830727*^9}},ExpressionUUID->"d3efaec7-ea16-40b5-9200-\
e93e5a92971e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{
      SubscriptBox["\[Gamma]", "\[Iota]"], "[", "\[Xi]", "]"}], "]"}], "[", 
    "t", "]"}], "/.", 
   RowBox[{
    RowBox[{"\[Xi]", "[", "i_", "]"}], "\[Rule]", " ", 
    RowBox[{"KroneckerDelta", "[", 
     RowBox[{"i", ",", "3"}], "]"}]}]}], "/.", 
  RowBox[{"t", "\[Rule]", " ", "0"}]}]], "Input",
 CellChangeTimes->{{3.633072192547882*^9, 3.6330722764056787`*^9}, {
   3.637565489870307*^9, 3.6375655044251328`*^9}, 3.637565549954013*^9, 
   3.6375656431071777`*^9, 3.6449332585359674`*^9, 3.6683980278956347`*^9, 
   3.700459574882731*^9, {3.701940748872273*^9, 3.701940765511901*^9}, {
   3.7019409328151913`*^9, 
   3.7019409814701433`*^9}},ExpressionUUID->"84979253-76ec-45d6-87f0-\
57784aa72eac"],

Cell[BoxData[
 RowBox[{"%", " ", "==", 
  RowBox[{
   FormBox[
    RowBox[{"tx\[Iota]", "[", "3", "]"}],
    TraditionalForm], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6375656722968297`*^9, 3.637565727362327*^9}, 
   3.6402402276515656`*^9, 3.6452543116372423`*^9, 
   3.6683980497036037`*^9},ExpressionUUID->"ff02c909-febe-45ab-88fd-\
d8a57eed0b79"],

Cell[TextData[{
 "Do not confuse ",
 Cell[BoxData[
  SubscriptBox["\[Gamma]", "\[Iota]"]], "Input",
  CellChangeTimes->{{3.633072192547882*^9, 3.6330722764056787`*^9}, {
    3.637565489870307*^9, 3.6375655044251328`*^9}, 3.637565549954013*^9, 
    3.6375656431071777`*^9, 3.6449332585359674`*^9, 3.6683980278956347`*^9, 
    3.700459574882731*^9},ExpressionUUID->
  "012cd828-6cc1-44b7-bbda-e88386fd6acb"],
 " and   \[Gamma]\[Iota]:"
}], "Text",
 CellChangeTimes->{{3.700459785387718*^9, 
  3.700459858851017*^9}},ExpressionUUID->"61c68199-c2fc-4fc2-92f0-\
a4c39a4886a3"],

Cell[BoxData[
 RowBox[{"\[Gamma]\[Iota]", "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.7004596501970253`*^9, 
  3.7004597013238087`*^9}},ExpressionUUID->"0ac55096-9cc1-44e9-925b-\
f00c399793eb"],

Cell[TextData[{
 "Now let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "9eba9ccc-e845-4830-a5fd-dddbf469e0be"],
 " \[Subset] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "36cb4f4b-b5d2-46b5-a06f-21259e93913f"],
 " be the domain of the chart ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "79801953-e14b-4720-b0ff-80da75cde4ef"],
 ". Let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "ad1b7584-f2de-4d18-aa0d-1807e7b8d741"],
 " := ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]], "["}], TraditionalForm]],ExpressionUUID->
  "4e71b2e5-f163-4ee0-97aa-65663a809a61"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "2ecd5f4d-fe45-4133-9a3f-41ac8a7e473a"],
 "] be the preimage of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "15964432-7e6b-48e4-94a4-b798f9e826b4"],
 " under the projection p. We define the bijective map"
}], "Text",
 CellChangeTimes->{{3.6375899728107166`*^9, 3.6375899749947205`*^9}, {
  3.637664977591406*^9, 3.6376650121610665`*^9}, {3.6376654328158054`*^9, 
  3.637665516899953*^9}, {3.637665546961206*^9, 3.6376655593476276`*^9}, {
  3.6376656658301935`*^9, 3.637665909472034*^9}, {3.6376659551769156`*^9, 
  3.6376659822197638`*^9}, {3.6376660391874647`*^9, 3.6376660404198666`*^9}, {
  3.666327457799379*^9, 
  3.666327457799379*^9}},ExpressionUUID->"092cc3ee-d793-4a15-ae61-\
a1b6ffb96443"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalPsi]\[Iota]", ":", 
   RowBox[{"X", " ", "\[Element]", " ", 
    RowBox[{
     SubscriptBox["V", "\[Iota]"], " ", "\[RightTeeArrow]", " ", 
     RowBox[{"\[CapitalPsi]\[Iota]", "[", "X", "]"}]}]}]}], " ", ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]\[Iota]", "[", 
      RowBox[{"p", "[", "X", "]"}], "]"}], ",", " ", "\[Xi]\[Iota]"}], "}"}], 
   " ", "\[Element]", " ", 
   RowBox[{
    RowBox[{"\[Phi]\[Iota]", "[", 
     SubscriptBox["U", "\[Iota]"], "]"}], "\[Times]", 
    SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}], "\[Subset]", " ", 
   SuperscriptBox["\[DoubleStruckCapitalR]", 
    RowBox[{"2", "n"}]]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}, 
   3.6379349487481613`*^9, 3.644993999728448*^9, {3.6663274578150043`*^9, 
   3.6663274578150043`*^9}, {3.668401234755887*^9, 
   3.668401289336196*^9}},ExpressionUUID->"81baf7f1-4e66-4f68-ab9a-\
d57b19d6c713"],

Cell[TextData[{
 "Obviously, this map is surjective. Let ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    StyleBox["\[CapitalPsi]",
     FontWeight->"Bold"], "\[Iota]"}]]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  ExpressionUUID->"af6a8833-ef08-4746-9b9a-9c3950e227d5"],
 "[Y] =",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox[
    RowBox[{
     RowBox[{"\[Phi]\[Iota]", "[", 
      RowBox[{"p", "[", "Y", "]"}], "]"}], ",", "  ", "\[Eta]\[Iota]"}],
    FontWeight->"Bold"], " ", "}"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  ExpressionUUID->"6df7af8e-8e12-40af-82bd-081661bf0ce9"],
 " = ",
 Cell[BoxData[
  RowBox[{" ", 
   StyleBox[
    RowBox[{"\[CapitalPsi]\[Iota]", "[", "X", "]"}],
    FontWeight->"Bold"], 
   StyleBox[" ",
    FontWeight->"Bold"]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  ExpressionUUID->"9e594058-3dee-435d-94fa-032ff09883ec"],
 ". Then p[Y] = p[X] := x since ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "e3a58eea-f9a2-4204-ab04-7d1663ed6ded"],
 " is a homeomorphism, and since",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Eta]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  FontWeight->"Bold",ExpressionUUID->"45a30b69-e55a-4438-a50c-79e365f0875b"],
 StyleBox[" = ",
  FontWeight->"Bold"],
 Cell[BoxData["\[Xi]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  FontWeight->"Bold",ExpressionUUID->"54a9c0fd-2bdb-4b48-9e74-d60a751c332c"],
 " implies  X = Y: In each chart the map\n",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox["\[Xi]\[Iota]", TraditionalForm]],
  FontWeight->"Bold",ExpressionUUID->"254c0a3f-b533-4133-8bd8-5f6d05759e2e"],
 Cell[BoxData[
  RowBox[{"\[Element]", " ", 
   SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  FontWeight->"Bold",ExpressionUUID->"4525b3a2-9f33-4686-bb8e-a479afc36345"],
 StyleBox[" ",
  FontWeight->"Bold"],
 "\[RightTeeArrow] X= q[",
 Cell[BoxData[
  FormBox["\[Xi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "64a2edb3-212a-45af-9735-f1244e2cb616"],
 "] \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "cd8cfabe-6527-43d7-9fe2-b4dc17fa9049"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "e4c04b78-ecbf-4f6f-8f0c-5f816bd55c32"],
 " is a linear isomorphism. We transfer the topology of the open subset  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]\[Iota]", "[", 
     SubscriptBox["U", "\[Iota]"], "]"}], "\[Times]", 
    SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}], "\[Subset]", " ", 
   SuperscriptBox["\[DoubleStruckCapitalR]", 
    RowBox[{"2", "n"}]]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  FontWeight->"Bold",ExpressionUUID->"fd45be53-ce34-424d-8323-b236a5a22b78"],
 " onto ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "eed91193-df67-49b9-9bb4-61ea34a019cd"],
 ". Let \[ScriptCapitalJ] be the set of all charts of the atlas of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "b269569a-2374-4680-9a73-9275452fc54d"],
 ". Then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"{", 
     FormBox[
      SubscriptBox["V", "\[Iota]"],
      TraditionalForm], "}"}], 
    RowBox[{"\[Iota]", "\[Element]", "\[ScriptCapitalJ]"}]], 
   TraditionalForm]],ExpressionUUID->"0a56bf8f-819e-4ebe-96c6-de1f0adf332c"],
 " is a covering of ",
 Cell[BoxData[
  RowBox[{"T", 
   FormBox[
    SuperscriptBox["M", 
     RowBox[{"n", " "}]],
    TraditionalForm]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.636957287011692*^9, 3.6369575000062675`*^9}, {
    3.636957532645525*^9, 3.63695753545453*^9}, {3.636957615773472*^9, 
    3.6369576224814835`*^9}, 3.6369577599783273`*^9, 3.637558836878151*^9, {
    3.6376690796392403`*^9, 3.637669083539247*^9}},
  FontWeight->"Bold",ExpressionUUID->"c512b6e4-3738-47b3-b78f-4d1ef85696d2"],
 " by sets homeomorphic to open subsets of ",
 Cell[BoxData[
  StyleBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", 
    RowBox[{"2", "n"}]],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  ExpressionUUID->"61667c93-b2bc-47cc-8614-cde7f7ec6f78"],
 " on which charts ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    StyleBox["\[CapitalPsi]",
     FontWeight->"Bold"], "\[Iota]"}]]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  ExpressionUUID->"96756880-d3d1-4ac9-995d-fbbf4e7ecc43"],
 " are defined. The coordinate transformations of this atlas for X \[Element] \
",
 Cell[BoxData[
  SubscriptBox["V", "\[Kappa]"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  FontWeight->"Bold",ExpressionUUID->"92141fcd-875c-43eb-b32e-06c65a465c3e"],
 StyleBox["\[Intersection]",
  FontWeight->"Bold"],
 Cell[BoxData[
  SubscriptBox["V", "\[Iota]"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  FontWeight->"Bold",ExpressionUUID->"84fa86b3-1c01-4155-95e1-a20d42972f61"],
 " are\n  ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    StyleBox["\[CapitalPsi]",
     FontWeight->"Bold"], "\[Kappa]\[Iota]"}]]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  FontWeight->"Bold",ExpressionUUID->"8d77a832-0b5a-4ba5-811e-9f7064978684"],
 " [{x, \[Xi]\[Iota]}]= {",
 Cell[BoxData[
  FormBox["\[CapitalPhi]\[Kappa]\[Iota]", TraditionalForm]],ExpressionUUID->
  "67870c5d-00c7-4ba2-ba49-286c8954bdc6"],
 "[x], d[",
 Cell[BoxData[
  FormBox["\[CapitalPhi]\[Kappa]\[Iota]", TraditionalForm]],ExpressionUUID->
  "40c17f46-ab44-4501-8e91-551251cf060a"],
 "][x][",
 Cell[BoxData[
  StyleBox[
   RowBox[{".", "\[Xi]\[Iota]"}],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}},
  ExpressionUUID->"939d02de-0f38-48e9-8888-69a4fd09a739"],
 "]}  with x = {x[1],x[2],x[3]};\n  they are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "b1c419fe-f931-4e90-ac67-fd8522e0b218"],
 " if",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["M", "n"]}], TraditionalForm]],ExpressionUUID->
  "f851c19a-ca37-4388-b36e-b9d93ac4d6a9"],
 " is of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "7370df35-81c0-4e6d-a99c-4105972713cb"],
 ". In particular, these maps are homeomorphisms such that the topologies \
defined by the charts on the intersections are compatible. Clearly, the \
projection\n  p:",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"T", 
    FormBox[
     SuperscriptBox["M", 
      RowBox[{"n", " "}]],
     TraditionalForm]}]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.636957287011692*^9, 3.6369575000062675`*^9}, {
    3.636957532645525*^9, 3.63695753545453*^9}, {3.636957615773472*^9, 
    3.6369576224814835`*^9}, 3.6369577599783273`*^9, 3.637558836878151*^9, {
    3.6376690796392403`*^9, 3.637669083539247*^9}},
  FontWeight->"Bold",ExpressionUUID->"c66fb8d6-a635-4b17-a63a-9d4da27b4166"],
 " \[RightArrow] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.636957287011692*^9, 3.6369575000062675`*^9}, {
    3.636957532645525*^9, 3.63695753545453*^9}, {3.636957615773472*^9, 
    3.6369576224814835`*^9}, 3.6369577599783273`*^9, 3.637558836878151*^9, {
    3.6376690796392403`*^9, 3.637669083539247*^9}},
  FontWeight->"Bold",ExpressionUUID->"5fb4cdb1-2318-4f98-9b07-21a8523af672"],
 " is of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "37655842-75a9-46e9-81c1-6497e638c812"],
 ". \[FilledRectangle]"
}], "Text",
 CellChangeTimes->{{3.6376671833950877`*^9, 3.637667202867922*^9}, {
   3.637667251409608*^9, 3.6376672620976267`*^9}, {3.6376672941928835`*^9, 
   3.637667519676882*^9}, {3.6376675817613916`*^9, 3.6376676104246426`*^9}, {
   3.637667660845134*^9, 3.6376678110515985`*^9}, {3.637667843897657*^9, 
   3.6376680088651495`*^9}, {3.637668104979319*^9, 3.637668119818945*^9}, {
   3.6376682018432903`*^9, 3.6376682665184045`*^9}, {3.6376683242087064`*^9, 
   3.637668356162563*^9}, {3.6376684024548445`*^9, 3.637668460373147*^9}, {
   3.6376684937008057`*^9, 3.6376685105674353`*^9}, {3.6376686228432336`*^9, 
   3.6376687429824457`*^9}, {3.63766877919611*^9, 3.637669026623347*^9}, {
   3.6376691127016983`*^9, 3.637669173354006*^9}, {3.63766922100449*^9, 
   3.6376695487248697`*^9}, {3.637669585997137*^9, 3.637669611850383*^9}, {
   3.6376696427902374`*^9, 3.6376696814487133`*^9}, {3.6376697168969755`*^9, 
   3.63766977041407*^9}, {3.6376698069573345`*^9, 3.6376698862164745`*^9}, {
   3.6376699388131676`*^9, 3.6376699408743715`*^9}, {3.637669971921426*^9, 
   3.6376701459455338`*^9}, {3.6376703951465745`*^9, 
   3.6376706195099707`*^9}, {3.637670685111992*^9, 3.637670722992859*^9}, {
   3.6376707791589575`*^9, 3.637670937632437*^9}, 3.637670979196911*^9, 
   3.637675514464934*^9, {3.6376760262780385`*^9, 3.637676031146247*^9}, 
   3.6376761096938553`*^9, {3.640070529976448*^9, 3.640070529976448*^9}, {
   3.644993999759648*^9, 3.644993999759648*^9}, {3.645006589042076*^9, 
   3.645006589042076*^9}, {3.645007554886573*^9, 3.645007616974682*^9}, {
   3.6450076761143856`*^9, 3.6450076761143856`*^9}, {3.6450077124624496`*^9, 
   3.6450077124624496`*^9}, {3.6450077447857065`*^9, 
   3.6450077463769093`*^9}, {3.6663274578462563`*^9, 
   3.6663274578462563`*^9}, {3.668398498928265*^9, 3.6683985282761602`*^9}, {
   3.668398623731298*^9, 3.66839863566743*^9}, {3.668398674755007*^9, 
   3.6683986878828993`*^9}, {3.668398730378633*^9, 3.668398759436385*^9}, {
   3.668398796009884*^9, 3.6683988084577312`*^9}, {3.668398847968897*^9, 
   3.668398887641254*^9}, {3.6684010641460752`*^9, 3.668401073320405*^9}, {
   3.668401304396001*^9, 3.668401311018827*^9}, {3.668401346281945*^9, 
   3.668401346281994*^9}, 
   3.668401376585232*^9},ExpressionUUID->"ae87acca-be61-4a97-8a41-\
6ece8fd2487d"],

Cell[TextData[{
 "Often in theoretic considerations, one assumes k = \[Infinity], then also  ",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"T", 
    FormBox[
     SuperscriptBox["M", 
      RowBox[{"n", " "}]],
     TraditionalForm]}]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.636957287011692*^9, 3.6369575000062675`*^9}, {
    3.636957532645525*^9, 3.63695753545453*^9}, {3.636957615773472*^9, 
    3.6369576224814835`*^9}, 3.6369577599783273`*^9, 3.637558836878151*^9, {
    3.6376690796392403`*^9, 3.637669083539247*^9}},
  FontWeight->"Bold",ExpressionUUID->"404e3eac-48ad-4871-93a2-bd703c8e27a0"],
 " is of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Infinity]"], TraditionalForm]],ExpressionUUID->
  "783ddaa6-d4f9-4fe4-98f7-c5d704626c02"],
 ". In the following we assume this if speaking about ",
 StyleBox["smooth manifolds, smooth maps",
  FontColor->RGBColor[0, 0, 1]],
 " etc."
}], "Text",
 CellChangeTimes->{{3.6376671833950877`*^9, 3.637667202867922*^9}, {
   3.637667251409608*^9, 3.6376672620976267`*^9}, {3.6376672941928835`*^9, 
   3.637667519676882*^9}, {3.6376675817613916`*^9, 3.6376676104246426`*^9}, {
   3.637667660845134*^9, 3.6376678110515985`*^9}, {3.637667843897657*^9, 
   3.6376680088651495`*^9}, {3.637668104979319*^9, 3.637668119818945*^9}, {
   3.6376682018432903`*^9, 3.6376682665184045`*^9}, {3.6376683242087064`*^9, 
   3.637668356162563*^9}, {3.6376684024548445`*^9, 3.637668460373147*^9}, {
   3.6376684937008057`*^9, 3.6376685105674353`*^9}, {3.6376686228432336`*^9, 
   3.6376687429824457`*^9}, {3.63766877919611*^9, 3.637669026623347*^9}, {
   3.6376691127016983`*^9, 3.637669173354006*^9}, {3.63766922100449*^9, 
   3.6376695487248697`*^9}, {3.637669585997137*^9, 3.637669611850383*^9}, {
   3.6376696427902374`*^9, 3.6376696814487133`*^9}, {3.6376697168969755`*^9, 
   3.63766977041407*^9}, {3.6376698069573345`*^9, 3.6376698862164745`*^9}, {
   3.6376699388131676`*^9, 3.6376699408743715`*^9}, {3.637669971921426*^9, 
   3.6376701459455338`*^9}, {3.6376703951465745`*^9, 
   3.6376706195099707`*^9}, {3.637670685111992*^9, 3.637670722992859*^9}, {
   3.6376707791589575`*^9, 3.637670937632437*^9}, 3.637670979196911*^9, {
   3.637675514464934*^9, 3.637675556823409*^9}, 
   3.637676139227708*^9},ExpressionUUID->"f9618acb-b706-4810-95f5-\
bc4b51f5f1bd"],

Cell[TextData[{
 StyleBox["Notation",
  FontColor->RGBColor[0, 0, 1]],
 ".  As before, the small Greek indices \[Iota], \[Kappa],... distinguish the \
charts, and the small  Latin indices i, j,... = 1,..., dim, denote the \
coordinate indices. The ",
 StyleBox["basis vectors in the chart",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  FormBox["\[Phi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "3b2ed8ef-61a7-4950-94ec-4556f6c5da34"],
 " are traditional denoted by"
}], "Text",
 CellChangeTimes->{{3.633244802496011*^9, 3.633244838801275*^9}, {
  3.6332448716453333`*^9, 3.6332449281858325`*^9}, {3.633244966908101*^9, 
  3.633244967144101*^9}, {3.63324513071379*^9, 3.63324520444652*^9}, {
  3.633246042577997*^9, 3.6332461483425837`*^9}, {3.633246924255962*^9, 
  3.6332469318395753`*^9}, {3.633246971436445*^9, 3.6332470653148108`*^9}, {
  3.6332471383519297`*^9, 3.633247273099367*^9}, {3.6332473237908564`*^9, 
  3.6332473816125584`*^9}, {3.633413907783994*^9, 3.633413948718466*^9}, {
  3.633763013029731*^9, 3.633763022264947*^9}, {3.633763483720559*^9, 
  3.6337635163754168`*^9}, {3.6376761452805185`*^9, 3.6376761482289233`*^9}, {
  3.666327457861882*^9, 
  3.666327457861882*^9}},ExpressionUUID->"ff87f2d7-3659-4f03-86e9-\
8aed90c59fe4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "i"], 
   SubscriptBox["x", "\[Iota]"]}], " ", "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{"tx\[Iota]", "[", "i", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.633246273249404*^9, 3.63324629958425*^9}, {
   3.6332463527707434`*^9, 3.6332463827101965`*^9}, {3.6332464174690576`*^9, 
   3.633246431682683*^9}, 3.6332464678631463`*^9, 3.63324683255319*^9, 
   3.6337632357245226`*^9, 3.637674045139934*^9, {3.637674111419252*^9, 
   3.6376741246948752`*^9}, {3.640070612138193*^9, 3.6400706157125998`*^9}, 
   3.6452543116372423`*^9, 3.6452545466672554`*^9, {3.6452719635479712`*^9, 
   3.6452719835004063`*^9}},ExpressionUUID->"3c0f4236-3ea7-4989-99ec-\
f9dceabaf5bc"],

Cell["\<\
The standard  basis representation of a tangent vector X can traditionally be \
written as\
\>", "Text",
 CellChangeTimes->{{3.6332474962555604`*^9, 3.6332478138965206`*^9}, {
   3.6332480527649417`*^9, 3.633248053451343*^9}, {3.6332725432285223`*^9, 
   3.6332726145870476`*^9}, {3.6332726455863028`*^9, 
   3.6332726629355326`*^9}, {3.633763104311491*^9, 3.6337631083206987`*^9}, {
   3.6337635473434715`*^9, 3.633763625289209*^9}, 3.633765789975021*^9, 
   3.6376735615796804`*^9, 3.6376738399877725`*^9, {3.637674149489319*^9, 
   3.6376741558385305`*^9}, {3.637675631222341*^9, 
   3.63767563627675*^9}},ExpressionUUID->"13685f11-a516-4039-9ac7-\
b86529bf1d05"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"X", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Xi]", "i"], " ", 
      RowBox[{
       SubscriptBox["\[PartialD]", "i"], 
       SubscriptBox["x", "\[Iota]"]}], "   ", "with", " ", 
      SuperscriptBox["\[Xi]", "i"]}], " ", "\[DotEqual]", " ", 
     RowBox[{
      RowBox[{"\[Xi]\[Iota]", "[", "i", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], 
  "       "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SuperscriptBox["\[Xi]", "i"], " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], 
    SubscriptBox["x", "\[Iota]"]}]}], "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{"X\[Iota]", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"\[Xi]", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.63324783079335*^9, 3.63324798364682*^9}, 
   3.6332483083897924`*^9, {3.6337632472841434`*^9, 3.63376326254097*^9}, {
   3.6337632981890326`*^9, 3.633763335945099*^9}, {3.6337633665231533`*^9, 
   3.6337634053556213`*^9}, 3.633765858636742*^9, {3.6338496217008944`*^9, 
   3.6338496496873436`*^9}, {3.6345504851087065`*^9, 
   3.6345505303955865`*^9}, {3.6376729574502096`*^9, 
   3.6376729901840672`*^9}, {3.6376730301875377`*^9, 
   3.6376730520159764`*^9}, {3.637674085471406*^9, 3.6376740910874157`*^9}, 
   3.640240229710769*^9, 3.6449939997752476`*^9, 3.645007920005214*^9, {
   3.6450080952091217`*^9, 3.6450081053647394`*^9}, {3.6450081879356847`*^9, 
   3.64500820805972*^9}, 3.6452543116528425`*^9, {3.6452545466672554`*^9, 
   3.6452545466672554`*^9}, {3.645272088535391*^9, 3.645272088535391*^9}, 
   3.6452721275354595`*^9, {3.645272326139408*^9, 3.6452723411934347`*^9}, {
   3.6684006971372023`*^9, 3.6684007224248037`*^9}, {3.668400755584412*^9, 
   3.668400810687871*^9}, 3.668400845801033*^9, 
   3.668400896816572*^9},ExpressionUUID->"fb64c834-ac42-4a41-a4f5-\
aa006ac3c266"],

Cell[TextData[{
 "with ",
 Cell[BoxData[
  FormBox["\[Xi]", TraditionalForm]],ExpressionUUID->
  "56bac109-a848-408f-8cd1-d38f4ce41478"],
 "[i][var[x]] the ",
 StyleBox["vector coordinates of",
  FontColor->RGBColor[0, 0, 1]],
 " X",
 "\[Iota]",
 " in the chart ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "\[CapitalPsi]\[Iota]"}], TraditionalForm]],ExpressionUUID->
  "f42c1002-76fb-4205-93ef-228dac7cef25"],
 "."
}], "Text",
 CellChangeTimes->{{3.633072989338456*^9, 3.6330730443076005`*^9}, {
   3.63307309832269*^9, 3.6330732429019594`*^9}, {3.6330732813311577`*^9, 
   3.6330733093077574`*^9}, 3.633248320388214*^9, {3.633248353495472*^9, 
   3.633248398273551*^9}, 3.6332484727112827`*^9, 3.633420828947113*^9, {
   3.6338502131471334`*^9, 3.633850255548008*^9}, {3.645007868665524*^9, 
   3.6450078898191614`*^9}, 3.6450081202939663`*^9, {3.6452721589695144`*^9, 
   3.6452721589695144`*^9}, {3.666327457861882*^9, 3.666327457861882*^9}, {
   3.6684009725537577`*^9, 3.668400972553804*^9}, {3.6684011590223007`*^9, 
   3.668401159653652*^9}},ExpressionUUID->"abe4f94b-b5b4-43a7-8e5e-\
3a05387a6acc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sum", "[", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Xi]\[Iota]", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", "i", "]"}], "*", 
     RowBox[{
      RowBox[{"tx\[Iota]", "[", "i", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"X\[Iota]", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"\[Xi]\[Iota]", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6338496620425653`*^9, 3.63384969860903*^9}, 
   3.633875554537094*^9, 3.637673082956831*^9, 3.640240231286372*^9, 
   3.645007926463626*^9, 3.645008127532379*^9, {3.6450082299121585`*^9, 
   3.6450082327045636`*^9}, 3.6452543116528425`*^9, {3.6452722511188765`*^9, 
   3.6452722546288824`*^9}, {3.668401493785885*^9, 3.668401509345779*^9}, 
   3.66840154184297*^9, {3.6684015853625717`*^9, 3.66840159525054*^9}, 
   3.668401639216885*^9, 3.6684016922004128`*^9, 
   3.668401733120726*^9},ExpressionUUID->"7b27c4f2-dd76-4f71-b9e8-\
7a7268a29a7b"],

Cell[TextData[{
 "For another chart ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "\[Kappa]"], TraditionalForm]],ExpressionUUID->
  "89053f7f-0714-4a21-9cde-b184bceb5cb1"],
 " we have the basis representation of the vector X and the coordinate \
transformation"
}], "Text",
 CellChangeTimes->{{3.637675296630949*^9, 3.6376753507680445`*^9}, {
  3.6376756652822013`*^9, 3.6376756843786345`*^9}, {3.6376757272857103`*^9, 
  3.6376757312949176`*^9}},ExpressionUUID->"13fe156e-0814-42bc-948c-\
2e3d4d196c4d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"X", " ", "=", " ", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Eta]", "i"], " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", "i"], 
      SubscriptBox["x", "\[Kappa]"]}], "   ", "with", "  ", 
     SuperscriptBox["\[Eta]", "i"]}], " ", "\[DotEqual]", " ", 
    RowBox[{"\[Xi]\[Kappa]", "[", "i", "]"}]}]}], ",", " ", 
  RowBox[{
   SuperscriptBox["\[Eta]", "i"], " ", "=", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], " ", 
    SubsuperscriptBox["A", "j", "i"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6376742776201453`*^9, 3.6376743630608964`*^9}, {
   3.6376744634494734`*^9, 3.637674492717125*^9}, {3.63767452357998*^9, 
   3.6376745334889975`*^9}, {3.637674811299488*^9, 3.6376748175706987`*^9}, 
   3.637675280810521*^9, 
   3.644995718766467*^9},ExpressionUUID->"4865def5-8f83-43b5-ba35-\
470a3f295bd6"],

Cell[TextData[{
 "The corresponding standard basis vectors transform ",
 StyleBox["contragredient",
  FontColor->RGBColor[0, 0, 1]],
 " to the vector coordinates:"
}], "Text",
 CellChangeTimes->{{3.6376753627850657`*^9, 3.6376754176603622`*^9}, 
   3.6376757679941826`*^9},ExpressionUUID->"18aebd85-dd0e-4aa8-beef-\
b56e31a6e500"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "i"], 
   SubscriptBox["x", "\[Kappa]"]}], " ", "=", " ", 
  RowBox[{
   SubsuperscriptBox["B", "i", "k"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "k"], 
    SubscriptBox["x", "\[Iota]"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.637674613792339*^9, 3.6376747229541316`*^9}, 
   3.63767493684731*^9, 3.6452545466672554`*^9, {3.6452724352303996`*^9, 
   3.6452724545276337`*^9}},ExpressionUUID->"d7a76882-4434-43b1-88b7-\
0cf7bbe09346"],

Cell["Indeed, we have", "Text",
 CellChangeTimes->{{3.637675438331399*^9, 
  3.6376754586848354`*^9}},ExpressionUUID->"098b3cee-2cd4-4beb-aab8-\
180a6c4df35e"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"X", " ", "=", "  ", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Eta]", "i"], " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", "i"], 
      SubscriptBox["x", "\[Kappa]"]}]}], "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Xi]", "j"], " ", 
      SubsuperscriptBox["A", "j", "i"], 
      SubsuperscriptBox["B", "i", "k"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "k"], 
       SubscriptBox["x", "\[Iota]"]}]}], "  ", "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Xi]", "j"], 
       SubsuperscriptBox["\[Delta]", "j", "k"], 
       RowBox[{
        SubscriptBox["\[PartialD]", "k"], 
        SubscriptBox["x", "\[Iota]"]}]}], " ", "=", " ", 
      RowBox[{
       SuperscriptBox["\[Xi]", "j"], 
       RowBox[{
        SubscriptBox["\[PartialD]", "j"], 
        SubscriptBox["x", "\[Iota]"]}], "     ", 
       "\[FilledSquare]"}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.637674862020177*^9, 3.637674882649414*^9}, {
  3.6376749389221134`*^9, 3.637674939577315*^9}, {3.63767497072097*^9, 
  3.6376749988176193`*^9}, {3.6376750435661016`*^9, 3.637675194988369*^9}, {
  3.637675802305643*^9, 3.6376758336188984`*^9}, {3.645254546682855*^9, 
  3.645254546682855*^9}, {3.6452724994869127`*^9, 3.645272510001331*^9}, {
  3.6452725479561977`*^9, 
  3.6452725594222174`*^9}},ExpressionUUID->"b3955108-8ffe-442b-8d12-\
5edcca06bde8"],

Cell["\<\
In the following we consider vector fields also in tensor terms as fields of \
tensors of type (0,1), e.g.\
\>", "Text",
 CellChangeTimes->{{3.640071203395037*^9, 3.6400712863543835`*^9}, {
  3.6450083870824347`*^9, 
  3.6450083899216394`*^9}},ExpressionUUID->"0108450b-6517-43b3-9b35-\
48d11611ea95"],

Cell[BoxData[
 RowBox[{
  RowBox[{"X\[Iota]", "[", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Xi]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6400709343281612`*^9, 3.6400710073628902`*^9}, {
   3.640071057694579*^9, 3.6400711023322573`*^9}, {3.6450084395453267`*^9, 
   3.645008445894538*^9}, 3.645254546682855*^9, 3.645272601292691*^9, 
   3.6452726791524277`*^9},ExpressionUUID->"8aff232b-45f6-4970-aee5-\
e1d45501e4f4"],

Cell[BoxData[
 RowBox[{"vecTotensor", "[", 
  RowBox[{"vec", "[", 
   RowBox[{"\[Xi]", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6400712999888077`*^9, 3.64007132395745*^9}, {
  3.6450084968130274`*^9, 3.6450085144878583`*^9}, {3.6450086062472196`*^9, 
  3.645008609164425*^9}},ExpressionUUID->"27a184b9-9450-4d34-b829-\
8fec6a6c3706"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6400713313986635`*^9, 3.6400713378414745`*^9}, {
  3.668401874095759*^9, 
  3.668401920902011*^9}},ExpressionUUID->"cbb6227c-e230-4bc4-9466-\
87e0347f339f"],

Cell["\<\
In particular, we introduce the basis vectors in tensor terms adapted to the \
traditional notation:\
\>", "Text",
 CellChangeTimes->{{3.6400710195933113`*^9, 3.6400710505653667`*^9}, {
   3.640071370748933*^9, 3.6400714374802504`*^9}, 
   3.700461310711527*^9},ExpressionUUID->"a0b96707-d961-4d64-ab6c-\
01a9fa669f8f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unprotect", "[", "D", "]"}], ";", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "i_"], 
    SubscriptBox["x", "\[Iota]"]}], ":=", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"tx\[Iota]", "[", "i", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}], ";", 
  RowBox[{"Protect", "[", "D", "]"}], ";"}]], "Input",
 CellChangeTimes->{
  3.6398956327647257`*^9, {3.6398956846504173`*^9, 3.6398956934644327`*^9}, {
   3.6398957318093*^9, 3.6398957522297363`*^9}, {3.6398967836879473`*^9, 
   3.6398968133124*^9}, 3.639896890126934*^9, {3.6398969288150024`*^9, 
   3.6398969294234037`*^9}, {3.6398969697026744`*^9, 3.639896996378721*^9}, {
   3.6398971214597406`*^9, 3.6398971405541744`*^9}, 3.640240233033575*^9, 
   3.6452543116528425`*^9, {3.6452727960902333`*^9, 
   3.6452728217210784`*^9}},ExpressionUUID->"c03d566b-9419-4ace-aa76-\
dbbcf56956b2"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[PartialD]", "1"], 
  SubscriptBox["x", "\[Iota]"]}]], "Input",
 CellChangeTimes->{{3.639897065486842*^9, 3.6398970723352547`*^9}, 
   3.645254546682855*^9, {3.645272854543536*^9, 
   3.645272859192344*^9}},ExpressionUUID->"899b0383-9597-4d2f-8a6a-\
c0b0c359f9bb"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6398970859696784`*^9, 3.6398970915700884`*^9}, 
   3.668402005158326*^9},ExpressionUUID->"b7eb1b6e-09a3-49cf-a102-\
bbef73bd8fe2"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[PartialD]", "4"], 
  SubscriptBox["x", "\[Iota]"]}]], "Input",
 CellChangeTimes->{{3.639897065486842*^9, 3.6398970723352547`*^9}, {
   3.639897178399841*^9, 3.6398972105670977`*^9}, 3.645254546682855*^9, 
   3.6452729041516232`*^9},ExpressionUUID->"94ccdc5d-3984-4476-8182-\
ea5505df7454"],

Cell["\[FilledSmallSquare]", "Text",
 CellChangeTimes->{{3.640071488216941*^9, 
  3.640071488216941*^9}},ExpressionUUID->"f78a0619-23c8-4f3b-9ded-\
fdfb761c05bc"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2.3. Cotangent Bundle", "Subsubsection",
 CellChangeTimes->{{3.6337630447913866`*^9, 3.633763065416623*^9}, {
   3.636873447910286*^9, 3.6368734624339113`*^9}, {3.637667706468614*^9, 
   3.6376677084030175`*^9}, {3.6376731727645893`*^9, 3.6376731790398006`*^9}, 
   3.63767620274142*^9, 3.6381564779600434`*^9, {3.6400715023847656`*^9, 
   3.640071504256769*^9}, 3.6402420688004*^9, {3.6403166203031273`*^9, 
   3.6403166286803417`*^9}, 3.6450087141214094`*^9, 3.6450819727055054`*^9, {
   3.6684020367807903`*^9, 3.668402039108667*^9}, 3.668434290278523*^9, 
   3.700461502150755*^9, 3.7005454114689503`*^9, {3.7019413904986477`*^9, 
   3.70194139437044*^9}, 3.702020216970649*^9, 3.702806664785091*^9, 
   3.702807460826186*^9},ExpressionUUID->"4194d1d4-5865-4f7c-bb1e-\
0031d8756a6d"],

Cell[TextData[{
 "For any point x of the differentiable manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "a635ef86-8b95-41b0-8dff-af676ca45b25"],
 " we consider the ",
 StyleBox["dual vector space",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", "*"], TraditionalForm]],ExpressionUUID->
  "c010aedb-4d73-4497-85b7-f8d070a88f1b"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "295e9ae2-8483-4e0d-9719-bfcd32a1211f"],
 "consisting of the ",
 StyleBox["covectors",
  FontColor->RGBColor[0, 0, 1]],
 " being the linear functions on the tangent vector space ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "701752fb-081e-488d-b06a-f165adb3dbe6"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "b8a84c70-b57b-4b76-860f-a533aff453a7"],
 ". The ",
 StyleBox["cotangent bundle",
  FontColor->RGBColor[0, 0, 1]],
 " of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "a4bc5eef-bd56-4261-a048-0f53d1ce0e60"],
 " is the union ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["T", "*"], TraditionalForm]],ExpressionUUID->
  "88240f68-a0e0-4e7e-bbd6-7421bbb7a9e6"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["M", 
     RowBox[{"n", " "}]], " ", ":=", 
    UnderscriptBox["\[Union]", 
     RowBox[{"x", "\[Element]", 
      SuperscriptBox["M", 
       RowBox[{"n", " "}]]}]]}], TraditionalForm]],ExpressionUUID->
  "377e69f7-f6aa-41f9-b95b-fcba53f4a3b7"],
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", "*"], TraditionalForm]],ExpressionUUID->
  "affd503c-1807-411e-a34c-4e0644d40b3e"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "0a28f4b3-c27a-471e-be45-65914a9db111"],
 "with the projection                                              "
}], "Text",
 CellChangeTimes->{{3.6345415554246097`*^9, 3.6345416547499843`*^9}, {
  3.6345442762524085`*^9, 3.634544309620867*^9}, {3.634544446569908*^9, 
  3.6345445120356236`*^9}, {3.634970495290568*^9, 3.6349705507138658`*^9}, {
  3.6379031617611523`*^9, 3.6379032314620743`*^9}, {3.6379033056558046`*^9, 
  3.6379033985695677`*^9}, {3.6379040093252654`*^9, 3.637904014163274*^9}, {
  3.637904052794942*^9, 3.637904098132622*^9}, {3.637904151711117*^9, 
  3.637904357547081*^9}, {3.6379044067409678`*^9, 
  3.6379044247599993`*^9}},ExpressionUUID->"79054dae-2910-462a-9337-\
017aef3c4e63"],

Cell[TextData[Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"p", ":", "  ", 
      RowBox[{"\[Omega]", "\[Element]", " ", 
       FormBox[
        RowBox[{
         RowBox[{
          SuperscriptBox["T", "*"], 
          SuperscriptBox["M", 
           RowBox[{"n", " "}]]}], "\[RightTeeArrow]", " ", 
         RowBox[{"p", "[", "\[Omega]"}]}],
        TraditionalForm]}]}], "]"}], "=", "x"}], ",", " ", 
   RowBox[{
    RowBox[{"if", "  ", "\[Omega]"}], "\[Element]", " ", 
    RowBox[{
     FormBox[
      RowBox[{
       SubsuperscriptBox["T", "x", "*"], 
       SuperscriptBox["M", 
        RowBox[{"n", " "}]]}],
      TraditionalForm], "."}]}]}],
  FontColor->GrayLevel[0]]],
 Evaluatable->False,
 CellChangeTimes->{{3.637904432859414*^9, 3.6379044937271214`*^9}, {
  3.637904546169814*^9, 3.637904660070016*^9}, {3.6379047113573065`*^9, 
  3.6379047162305155`*^9}, {3.6379047480167713`*^9, 3.6379047581431894`*^9}},
 FontWeight->
  "Bold",ExpressionUUID->"05271ca5-5c6f-4627-937b-86072c81c7ff"]], "Text",
 CellChangeTimes->{
  3.6379048431949396`*^9},ExpressionUUID->"6785a327-3bfd-4fa6-88c6-\
d0ecbb415af7"],

Cell[TextData[{
 StyleBox["Proposition 1",
  FontColor->RGBColor[0, 0, 1]],
 ". For any differentiable manifold of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "87b68bdf-d091-4726-917a-ea9bc14e17fa"],
 " the cotangent bundle ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"p", ":", 
    RowBox[{
     FormBox[
      RowBox[{
       RowBox[{
        SuperscriptBox["T", "*"], 
        SuperscriptBox["M", 
         RowBox[{"n", " "}]]}], "\[RightArrow]", " "}],
      TraditionalForm], 
     FormBox[
      SuperscriptBox["M", 
       RowBox[{"n", " "}]],
      TraditionalForm]}]}],
   FontColor->GrayLevel[0]]],
  Evaluatable->False,
  CellChangeTimes->{{3.637904432859414*^9, 3.6379044937271214`*^9}, {
   3.637904546169814*^9, 3.637904660070016*^9}, {3.6379047113573065`*^9, 
   3.6379047162305155`*^9}, {3.6379047480167713`*^9, 3.6379047581431894`*^9}},
  FontWeight->"Bold",ExpressionUUID->"44547f0c-cf3b-41fe-99d9-0fb31abfead2"],
 "is a 2n-dimensional manifold of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "28ace3ed-5491-4345-b908-c880d0dfe7d7"],
 " with an atlas canonically defined by the atlas of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.637904432859414*^9, 3.6379044937271214`*^9}, {
   3.637904546169814*^9, 3.637904660070016*^9}, {3.6379047113573065`*^9, 
   3.6379047162305155`*^9}, {3.6379047480167713`*^9, 3.6379047581431894`*^9}},
  FontWeight->"Bold",ExpressionUUID->"ace563bc-93f6-40ff-924a-fc95eb1e2ce2"],
 " (see the proof)."
}], "Text",
 CellChangeTimes->{{3.637904953350134*^9, 3.637905056162716*^9}, {
   3.6379051261634398`*^9, 3.6379052036315765`*^9}, {3.6379052533432646`*^9, 
   3.637905524049346*^9}, {3.6379326970067906`*^9, 3.6379327143268213`*^9}, 
   3.638156518568915*^9},ExpressionUUID->"62867d42-d138-4033-a3c3-\
909e2ca3f813"],

Cell[TextData[{
 StyleBox["Proof.",
  FontColor->RGBColor[0, 0, 1]],
 " The construction of the atlas is analog to that of the tangent bundle. The \
coordinates of the covector \[Omega] with respect to the dual of the standard \
 basis in the chart ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", "\[Iota]"], TraditionalForm]],
  ExpressionUUID->"ca5ba905-397b-497c-97ac-7b679a3aa530"],
 " of",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.637904432859414*^9, 3.6379044937271214`*^9}, {
   3.637904546169814*^9, 3.637904660070016*^9}, {3.6379047113573065`*^9, 
   3.6379047162305155`*^9}, {3.6379047480167713`*^9, 3.6379047581431894`*^9}},
  FontWeight->"Bold",ExpressionUUID->"718cb0b1-10fa-4733-aeb8-f135afde17b3"],
 "are given by"
}], "Text",
 CellChangeTimes->{{3.6379112566656513`*^9, 3.63791135787403*^9}, {
  3.6379327387096643`*^9, 3.637932816439001*^9}, {3.6379328566286716`*^9, 
  3.6379329000037484`*^9}, {3.6379329337350082`*^9, 3.6379329392418175`*^9}, {
  3.6379331144803267`*^9, 3.6379332026842823`*^9}, {3.640146217929652*^9, 
  3.640146219801656*^9}, {3.668429145465233*^9, 
  3.668429162296123*^9}},ExpressionUUID->"56ff3fef-8530-43a6-af4f-\
ae8112c2ef52"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]\[Iota]", "[", "i_", "]"}], "[", "x__", "]"}], " ", ":=", 
  " ", 
  RowBox[{"\[Omega]", "[", 
   RowBox[{
    RowBox[{"tx\[Iota]", "[", "i", "]"}], "[", "x", "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.637932966608266*^9, {3.637933021731163*^9, 3.637933084463874*^9}, {
   3.63793321848711*^9, 3.6379333754105873`*^9}, {3.637933769127282*^9, 
   3.6379337863995123`*^9}, 3.6452543116684427`*^9, 
   3.6684295813006563`*^9},ExpressionUUID->"dc3617a3-7126-4dd9-906c-\
7c80a38d813e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Omega]\[Iota]", "[", "2", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.63793348473398*^9, 3.63793351833184*^9, {3.637933560580714*^9, 
   3.6379335763397427`*^9}, 3.640240234702778*^9, 
   3.6684295823257017`*^9},ExpressionUUID->"c7d6e2c7-c730-43e9-8cbc-\
18b82fc4d21c"],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "73608a9e-f368-49d7-a986-5d65ad22934d"],
 " be the domain of ",
 Cell[BoxData[
  FormBox["\[CapitalPhi]\[Iota]", TraditionalForm]],ExpressionUUID->
  "aa0461ee-d67a-44ed-9d00-4b41f101826b"],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["V", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "c3f54048-6a38-449a-b7b8-d1e0a235b21c"],
 " := ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "199dd202-d179-4903-9838-3d4e8eed1cb7"],
 "[",
 Cell[BoxData[
  FormBox[
   SubscriptBox["U", "\[Iota]"], TraditionalForm]],ExpressionUUID->
  "e41470e4-0ff2-46d0-8be4-5ec7df4d8dcc"],
 "]\[Subset]",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.637904432859414*^9, 3.6379044937271214`*^9}, {
   3.637904546169814*^9, 3.637904660070016*^9}, {3.6379047113573065`*^9, 
   3.6379047162305155`*^9}, {3.6379047480167713`*^9, 3.6379047581431894`*^9}},
  FontWeight->"Bold",ExpressionUUID->"49f3a76c-cdcd-4a48-9ae3-1983bef69123"],
 ". Then the  corresponding chart of the atlas of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.637904432859414*^9, 3.6379044937271214`*^9}, {
   3.637904546169814*^9, 3.637904660070016*^9}, {3.6379047113573065`*^9, 
   3.6379047162305155`*^9}, {3.6379047480167713`*^9, 3.6379047581431894`*^9}},
  FontWeight->"Bold",ExpressionUUID->"127167f8-f1f5-4c30-bd2b-eb9558d4eb9a"],
 " is defined by"
}], "Text",
 CellChangeTimes->{{3.6379341256435103`*^9, 3.6379343583403215`*^9}, {
  3.637934447395278*^9, 3.637934537210636*^9}, {3.66842931645788*^9, 
  3.66842933095182*^9}},ExpressionUUID->"bb49681c-ff63-4961-b897-\
2f844917bf87"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalPsi]\[Iota]", ":", 
   RowBox[{"\[Omega]", " ", "\[Element]", " ", 
    RowBox[{
     SubscriptBox["V", "\[Iota]"], " ", "\[RightTeeArrow]", " ", 
     RowBox[{"\[CapitalPsi]\[Iota]", "[", "\[Omega]", "]"}]}]}]}], " ", ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]\[Iota]", "[", 
      RowBox[{"p", "[", "\[Omega]", "]"}], "]"}], ",", " ", 
     "\[Omega]\[Iota]"}], "}"}], " ", "\[Element]", " ", 
   RowBox[{
    RowBox[{"\[Phi]\[Iota]", "[", 
     SubscriptBox["U", "\[Iota]"], "]"}], "\[Times]", 
    SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}], "\[Subset]", " ", 
   SuperscriptBox["\[DoubleStruckCapitalR]", 
    RowBox[{"2", "n"}]]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}, {
   3.637934676657682*^9, 3.6379346873281007`*^9}, {3.63793472108956*^9, 
   3.6379347291089745`*^9}, {3.6379347873030767`*^9, 
   3.6379347984902964`*^9}, {3.666327457877509*^9, 3.666327457877509*^9}, {
   3.66842945012584*^9, 3.668429450870658*^9}, 
   3.668429584372225*^9},ExpressionUUID->"0b10b6aa-3615-4d11-b640-\
d76881e55b31"],

Cell[TextData[{
 "Here ",
 Cell[BoxData["\[Omega]\[Iota]"],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}, {
   3.637934676657682*^9, 3.6379346873281007`*^9}, {3.63793472108956*^9, 
   3.6379347291089745`*^9}, {3.6379347873030767`*^9, 3.6379347984902964`*^9}},
  FontWeight->"Bold",ExpressionUUID->"2d1bdfcd-9f2b-4830-af02-e1ec10da3835"],
 " = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"\[Omega]\[Iota]", "[", "j", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "}"}], "j"], TraditionalForm]],
  ExpressionUUID->"1235b4df-5df5-4771-bd68-50d92cb821a2"],
 " \[Element] ",
 Cell[BoxData[
  StyleBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "n"],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}, {
   3.637934676657682*^9, 3.6379346873281007`*^9}, {3.63793472108956*^9, 
   3.6379347291089745`*^9}, {3.6379347873030767`*^9, 3.6379347984902964`*^9}},
  ExpressionUUID->"dacd76ca-2ade-4134-91ca-52ff2896c3e2"],
 " denotes the n-tuple of coordinates of ",
 "\[Omega]. "
}], "Text",
 CellChangeTimes->{{3.637935171788154*^9, 3.6379351731141567`*^9}, {
   3.637935205914415*^9, 3.637935356966881*^9}, {3.637935431557612*^9, 
   3.6379355203161683`*^9}, 3.6379356491355953`*^9, {3.640146409575989*^9, 
   3.6401464333660307`*^9}, {3.668429585939725*^9, 
   3.668429591868806*^9}},ExpressionUUID->"bab097ad-7325-4dae-bd08-\
4b158ff86d5c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalPsi]\[Iota]", ":", 
   RowBox[{"\[Omega]", " ", "\[Element]", " ", 
    RowBox[{
     SubscriptBox["V", "\[Iota]"], " ", "\[RightTeeArrow]", " ", 
     RowBox[{"\[CapitalPsi]\[Iota]", "[", "\[Omega]", "]"}]}]}]}], " ", ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]\[Iota]", "[", 
      RowBox[{"p", "[", "\[Omega]", "]"}], "]"}], ",", " ", 
     "\[Omega]\[Iota]"}], "}"}], " ", "\[Element]", " ", 
   RowBox[{
    RowBox[{"\[Phi]\[Iota]", "[", 
     SubscriptBox["U", "\[Iota]"], "]"}], "\[Times]", 
    SuperscriptBox["\[DoubleStruckCapitalR]", "n"]}], "\[Subset]", " ", 
   SuperscriptBox["\[DoubleStruckCapitalR]", 
    RowBox[{"2", "n"}]]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6376660582370977`*^9, 3.6376662410304213`*^9}, {
   3.6376662981199217`*^9, 3.6376664339989624`*^9}, {3.6376664678122225`*^9, 
   3.637666480325444*^9}, {3.637666524020522*^9, 3.637666570925205*^9}, 
   3.6379350469115343`*^9, {3.6379827700547967`*^9, 3.637982795451641*^9}, {
   3.666327457893136*^9, 3.666327457893136*^9}, {3.668429451821188*^9, 
   3.668429452725491*^9}, 
   3.668429603452072*^9},ExpressionUUID->"c8e8f107-cc80-4287-acb6-\
0f3d5d38c2b4"],

Cell[TextData[{
 "The coordinate transformations in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.637904432859414*^9, 3.6379044937271214`*^9}, {
   3.637904546169814*^9, 3.637904660070016*^9}, {3.6379047113573065`*^9, 
   3.6379047162305155`*^9}, {3.6379047480167713`*^9, 3.6379047581431894`*^9}},
  FontWeight->"Bold",ExpressionUUID->"1298151c-6ea4-4745-b6f6-c332fd33a06f"],
 " are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "e14af46c-d13f-4f7e-a850-e7ec096e8ed9"],
 ":"
}], "Text",
 CellChangeTimes->{{3.637983243078828*^9, 
  3.637983321858966*^9}},ExpressionUUID->"d4e9cc6b-dc07-4e66-bb9f-\
ddf4c6f7d6c4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Omega]", "\[Element]", " ", 
    RowBox[{
     SubscriptBox["V", "\[Kappa]"], 
     StyleBox["\[Intersection]",
      FontWeight->"Bold"], 
     SubscriptBox["V", "\[Iota]"]}]}], " ", ";"}], 
  "    "}], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{
   StyleBox[
    RowBox[{
     StyleBox["\[CapitalPsi]",
      FontWeight->"Bold"], "\[Kappa]\[Iota]"}]], 
   StyleBox[":",
    FontWeight->"Bold"], 
   RowBox[{
    RowBox[{
     StyleBox["{",
      FontWeight->"Bold"], 
     RowBox[{
      RowBox[{"var", "[", "x", "]"}], ",", "\[Omega]\[Iota]"}], "}"}], 
    "\[RightTeeArrow]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       FormBox["\[Phi]\[Kappa]\[Iota]",
        TraditionalForm], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], ",", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"d", "[", 
         FormBox["\[Phi]\[Iota]\[Kappa]",
          TraditionalForm], "]"}], "[", 
        RowBox[{"\[CapitalPhi]\[Kappa]\[Iota]", "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], ".", 
       "\[Omega]\[Iota]"}]}], "}"}], " "}]}]}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6378228354459114`*^9, 3.637822864979763*^9}, 
   3.637982843390526*^9, {3.637982880814991*^9, 3.6379828892546062`*^9}, 
   3.6379829346818857`*^9, {3.6379830163636293`*^9, 3.6379830307312546`*^9}, 
   3.6379833483010125`*^9, 3.6401621573520484`*^9, {3.6401622774878597`*^9, 
   3.640162289718281*^9}, {3.640162347266782*^9, 3.640162538070717*^9}, 
   3.6401627915867624`*^9, {3.640163195050271*^9, 3.6401632157983074`*^9}, 
   3.6402365308662724`*^9, 3.6449295092841578`*^9, 3.6449301190786324`*^9, {
   3.668429605532385*^9, 3.668429606924128*^9}, 3.668429796980365*^9, {
   3.6684298345786963`*^9, 
   3.668429845130328*^9}},ExpressionUUID->"a223765a-cd01-4075-9a29-\
e7361555ed61"],

Cell["\<\
In the next example we calculate the transformation rule for the coordinates \
of the covector d[h]. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.637983388424283*^9, 3.6379834226663427`*^9}, {
   3.6401619318380527`*^9, 3.6401620024749765`*^9}, 
   3.66842990905796*^9},ExpressionUUID->"c691f1db-12bc-47c0-8d7a-\
202a9ec25e89"],

Cell[TextData[{
 StyleBox["Example.",
  FontColor->RGBColor[0, 0, 1]],
 " In differential geometry the differentials of functions are important \
examples of covectors. The value of d[h] for the tangent vector X \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "eef3ccfc-5355-456b-9805-c43beb9f7e4e"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "0ab831df-8838-4379-b7b0-25a4ea5046cf"],
 " is the",
 StyleBox[" derivative ",
  FontColor->RGBColor[0, 0, 1]],
 "of h in the ",
 StyleBox["direction ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["X",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6345415554246097`*^9, 3.6345416547499843`*^9}, {
   3.6345442762524085`*^9, 3.634544309620867*^9}, {3.634544446569908*^9, 
   3.6345445120356236`*^9}, {3.634970495290568*^9, 3.6349705507138658`*^9}, {
   3.6379031617611523`*^9, 3.6379032314620743`*^9}, {3.6379033056558046`*^9, 
   3.6379033844983435`*^9}, {3.63793282696902*^9, 3.637932835190234*^9}, 
   3.637932865757688*^9, 3.637983167059894*^9, {3.637983467893223*^9, 
   3.637983473400032*^9}, {3.637990844836482*^9, 3.6379909139758034`*^9}, 
   3.6379909465018606`*^9, {3.640147376325287*^9, 3.640147391769314*^9}, {
   3.640147739509525*^9, 3.640147758947159*^9}, {3.640147839708501*^9, 
   3.640148002057986*^9}, 
   3.64014838020265*^9},ExpressionUUID->"daeac705-5863-4d7b-86e1-\
cf8c411dfa02"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "h", "]"}], "[", "X", "]"}], " ", "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     SubscriptBox["h", "\[Iota]"], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ".", 
   RowBox[{"vec", "[", "\[Xi]\[Iota]", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6401480438816595`*^9, 3.6401481027249627`*^9}, 
   3.6401481589630613`*^9, {3.6401482315343885`*^9, 3.6401482547160296`*^9}, 
   3.6449939997752476`*^9},ExpressionUUID->"bbb5776a-e5ae-4ca7-ab56-\
f5c28a33af17"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"vec", "[", "\[Xi]\[Iota]", "]"}],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6379909845347276`*^9, 3.6379910311164093`*^9}, {
   3.637991084952104*^9, 3.6379911219241686`*^9}, {3.637991156025829*^9, 
   3.6379911747614613`*^9}},ExpressionUUID->
  "146ef799-b660-4ca8-94e3-e77dd4cc13f2"],
 " denotes the coordinate vector of X in the chart ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", "\[Iota]"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6378228354459114`*^9, 3.637822864979763*^9}, 
    3.637982843390526*^9, {3.637982880814991*^9, 3.6379828892546062`*^9}, 
    3.6379829346818857`*^9, {3.6379830163636293`*^9, 3.6379830307312546`*^9}, 
    3.6379833483010125`*^9},ExpressionUUID->
  "ea8f8546-4149-4d95-9128-bec4900e46bb"],
 ":"
}], "Text",
 CellChangeTimes->{{3.637991231217961*^9, 3.6379912955836735`*^9}, {
  3.637991336159345*^9, 3.637991368030201*^9}, {3.64014827771047*^9, 
  3.64014827771047*^9}, {3.644993999790848*^9, 
  3.644993999790848*^9}},ExpressionUUID->"b380eab5-7486-4cc3-a7f8-\
a25c96ac9387"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     SubscriptBox["h", "\[Iota]"], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ".", 
   RowBox[{"vec", "[", "\[Xi]\[Iota]", "]"}]}], "\[Equal]", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        SubscriptBox["h", "\[Iota]"], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
       RowBox[{"x", "[", "i", "]"}]}], "]"}], "*", 
     RowBox[{"\[Xi]\[Iota]", "[", "i", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.644993999790848*^9, 
  3.7005439354997673`*^9},ExpressionUUID->"4b1d8871-494f-444d-892f-\
8f4bd004ef6f"],

Cell[TextData[{
 "Clearly, the coordinates of d[h] in the chart ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", "\[Iota]"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6378228354459114`*^9, 3.637822864979763*^9}, 
    3.637982843390526*^9, {3.637982880814991*^9, 3.6379828892546062`*^9}, 
    3.6379829346818857`*^9, {3.6379830163636293`*^9, 3.6379830307312546`*^9}, 
    3.6379833483010125`*^9},ExpressionUUID->
  "65e48b35-7e3c-4720-b835-311f5edbc910"],
 " are its partial derivatives with respect to the coordinates x[i]."
}], "Text",
 CellChangeTimes->{{3.637991987478489*^9, 3.637992012516533*^9}, {
  3.6379920514854016`*^9, 
  3.6379920820302553`*^9}},ExpressionUUID->"e8f38c75-c6a5-4a16-9a04-\
6b941d725b5f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    SubscriptBox["h", "\[Iota]"], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ".", 
  RowBox[{
   RowBox[{"tx\[Iota]", "[", "3", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6379835264401255`*^9, 3.637983532446136*^9}, {
   3.6379835634745903`*^9, 3.637983656513154*^9}, {3.637983736198094*^9, 
   3.6379837645433435`*^9}, {3.640236650674483*^9, 3.640236673403723*^9}, 
   3.6452543116840425`*^9},ExpressionUUID->"f06a9e62-8fbd-4d4f-81be-\
ef419129c459"],

Cell["\<\
To obtain the transformation rule we consider  the representative \
\>", "Text",
 CellChangeTimes->{{3.6379922130704856`*^9, 
  3.637992245861743*^9}},ExpressionUUID->"4f9f30da-8796-4280-8f1d-\
c737c2e2afb7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "y", "]"}], ";", 
  RowBox[{
   RowBox[{
    SubscriptBox["h", "\[Kappa]"], "[", "y__", "]"}], " ", ":=", " ", 
   RowBox[{
    SubscriptBox["h", "\[Iota]"], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"f\[Iota]\[Kappa]", "[", "1", "]"}], "[", "y", "]"}], ",", 
     RowBox[{
      RowBox[{"f\[Iota]\[Kappa]", "[", "2", "]"}], "[", "y", "]"}], ",", 
     RowBox[{
      RowBox[{"f\[Iota]\[Kappa]", "[", "3", "]"}], "[", "y", "]"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.637992347137121*^9, 3.637992373766368*^9}, {
   3.6379924425624886`*^9, 3.6379924982857866`*^9}, {3.6379928458387966`*^9, 
   3.637992909190508*^9}, {3.637992943884969*^9, 3.6379930037578745`*^9}, {
   3.637993108480858*^9, 3.6379931428009186`*^9}, 3.637993331922451*^9, {
   3.6449321438844028`*^9, 
   3.6449321464896073`*^9}},ExpressionUUID->"9187e75b-cbf4-498a-9d1d-\
df0dce6c636f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "y", "]"}], ";", 
  RowBox[{
   SubscriptBox["h", "\[Kappa]"], "[", 
   RowBox[{"var", "[", "y", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.637992685954116*^9, 3.6379932408470907`*^9, {3.6379933030624*^9, 
   3.6379933103008127`*^9}, {3.640236729049021*^9, 
   3.6402367346026306`*^9}},ExpressionUUID->"6b7b67ec-7b60-47f8-83a3-\
1db8e5ee0333"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["h", "\[Kappa]"], "[", 
       RowBox[{"var", "[", "y", "]"}], "]"}], ",", 
      RowBox[{"y", "[", "1", "]"}]}], "]"}], "-", " ", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"A\[Iota]\[Kappa]", "[", 
         RowBox[{"1", ",", "j"}], "]"}], "[", 
        RowBox[{"var", "[", "y", "]"}], "]"}], "*", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "\[Iota]"], "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
         RowBox[{"x", "[", "j", "]"}]}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "/.", 
   RowBox[{
    RowBox[{"x", "[", "l_", "]"}], "->", 
    RowBox[{
     RowBox[{"f\[Iota]\[Kappa]", "[", "l", "]"}], "[", 
     RowBox[{"var", "[", "y", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.637993554878042*^9, 3.63799359907292*^9}, {
   3.6379957696317644`*^9, 3.6379957862145934`*^9}, {3.640148642907111*^9, 
   3.64014865919354*^9}, {3.6401487454928913`*^9, 3.640148753370906*^9}, {
   3.6401495019694204`*^9, 3.6401495049178257`*^9}, {3.64015098779403*^9, 
   3.6401510143920765`*^9}, {3.6401526223493013`*^9, 
   3.6401526263273077`*^9}, {3.640152663923374*^9, 3.640152691582223*^9}, {
   3.640236801230347*^9, 3.6402368567196445`*^9}, 3.64493214789361*^9, 
   3.64499517976752*^9},ExpressionUUID->"440b080d-9ee3-4640-b610-\
195085b4f10f"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      SubscriptBox["h", "\[Kappa]"], "]"}], "[", 
     RowBox[{"var", "[", "y", "]"}], "]"}], "-", "  ", 
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{
       RowBox[{"d", "[", 
        FormBox["\[Phi]\[Iota]\[Kappa]",
         TraditionalForm], "]"}], "[", 
       RowBox[{"vec", "[", "y", "]"}], "]"}], "]"}], ".", 
     RowBox[{
      RowBox[{"d", "[", 
       SubscriptBox["h", "\[Iota]"], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "/.", 
   RowBox[{
    RowBox[{"x", "[", "l_", "]"}], "->", 
    RowBox[{
     RowBox[{"f\[Iota]\[Kappa]", "[", "l", "]"}], "[", 
     RowBox[{"var", "[", "y", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.637993554878042*^9, 3.63799359907292*^9}, {
   3.6379957696317644`*^9, 3.6379957862145934`*^9}, {3.640148642907111*^9, 
   3.64014865919354*^9}, {3.6401487454928913`*^9, 3.640148753370906*^9}, {
   3.6401495019694204`*^9, 3.6401495049178257`*^9}, {3.64015098779403*^9, 
   3.6401510143920765`*^9}, {3.6401526223493013`*^9, 
   3.6401526263273077`*^9}, {3.640152663923374*^9, 3.640152691582223*^9}, 
   3.640162713368225*^9, {3.6401627547082977`*^9, 3.6401627565959005`*^9}, {
   3.640162841054449*^9, 3.640162852114869*^9}, {3.6401629110985727`*^9, 
   3.640162913251376*^9}, 3.640162955839451*^9, {3.640162994106318*^9, 
   3.6401630155563555`*^9}, {3.6401630467408104`*^9, 3.640163065320443*^9}, 
   3.6401632607419863`*^9, {3.640163416024659*^9, 3.6401634348694925`*^9}, 
   3.64016351889124*^9, {3.6401635725241337`*^9, 3.6401636895243397`*^9}, {
   3.640164255274933*^9, 3.6401642929645996`*^9}, {3.640236896608915*^9, 
   3.640236903691327*^9}, 3.6449301207010355`*^9, 3.644932149001212*^9, {
   3.6684314042790833`*^9, 
   3.6684314049253063`*^9}},ExpressionUUID->"55846baf-98c9-4fc1-a056-\
d3dcfd50fc15"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       SubscriptBox["h", "\[Kappa]"], "[", 
       RowBox[{"var", "[", "y", "]"}], "]"}], ",", 
      RowBox[{"y", "[", "2", "]"}]}], "]"}], "-", " ", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"A\[Iota]\[Kappa]", "[", 
         RowBox[{"2", ",", "j"}], "]"}], "[", 
        RowBox[{"var", "[", "y", "]"}], "]"}], "*", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "\[Iota]"], "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
         RowBox[{"x", "[", "j", "]"}]}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "/.", 
   RowBox[{
    RowBox[{"x", "[", "l_", "]"}], "->", 
    RowBox[{
     RowBox[{"f\[Iota]\[Kappa]", "[", "l", "]"}], "[", 
     RowBox[{"var", "[", "y", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.637993554878042*^9, 3.63799359907292*^9}, {
   3.6379957696317644`*^9, 3.6379957862145934`*^9}, {3.640148642907111*^9, 
   3.64014865919354*^9}, {3.6401487454928913`*^9, 3.640148753370906*^9}, {
   3.6401492953782578`*^9, 3.640149322194704*^9}, {3.640161401780321*^9, 
   3.640161421935557*^9}, {3.640161454040413*^9, 3.640161456364817*^9}, {
   3.6401614902168765`*^9, 3.640161500918495*^9}, 3.6401615344897547`*^9, {
   3.6401617212376823`*^9, 3.6401617271968927`*^9}, {3.6402369221773596`*^9, 
   3.6402369617702293`*^9}, 3.644932155991024*^9, 3.64499517978312*^9, {
   3.6450809038662186`*^9, 3.645080913772236*^9}, {3.6684319777124863`*^9, 
   3.668431979430791*^9}, {3.668432023822063*^9, 
   3.6684320241444607`*^9}},ExpressionUUID->"208a69e7-6c21-4c12-bcee-\
742355c73d99"],

Cell["\<\
In traditional notation the calculated transformation rule can be written as\
\>", "Text",
 CellChangeTimes->{{3.640237751303216*^9, 
  3.640237801441704*^9}},ExpressionUUID->"5c2b170a-b5cb-4749-893b-\
d45d75e239a1"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"\[PartialD]", 
    SubscriptBox["h", "\[Kappa]"]}], 
   RowBox[{"\[PartialD]", 
    SuperscriptBox["y", "i"]}]], " ", "=", " ", 
  RowBox[{
   SubsuperscriptBox["B", "i", "j"], 
   FractionBox[
    RowBox[{"\[PartialD]", 
     SubscriptBox["h", "\[Iota]"]}], 
    RowBox[{"\[PartialD]", 
     SuperscriptBox["x", "j"]}]]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6402379716380033`*^9, 3.640238064520566*^9}, {
  3.6402380977954245`*^9, 
  3.640238141615902*^9}},ExpressionUUID->"672c527f-f763-48d7-8fb6-\
70c32c031700"],

Cell["\<\
This is a special case of the transformation rule for the covector \
coordinates obtained by inserting the transformation formula for the adapted \
tangent bases \
\>", "Text",
 CellChangeTimes->{{3.6402383366162443`*^9, 3.6402384460192366`*^9}, 
   3.6402386543732023`*^9, 
   3.6402404234787097`*^9},ExpressionUUID->"2cb584c1-c998-4307-b74d-\
c47145f1e2d9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "i"], 
   SubscriptBox["x", "\[Kappa]"]}], " ", "=", " ", 
  RowBox[{
   SubsuperscriptBox["B", "i", "j"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    SubscriptBox["x", "\[Iota]"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.640238512818554*^9, 3.640238639350376*^9}, 
   3.6402398320660706`*^9, 3.645254546698455*^9, {3.645434990323738*^9, 
   3.645435006220166*^9}},ExpressionUUID->"3ac3f793-fab2-4ec6-902a-\
814c03e2cc8f"],

Cell[TextData[{
 "into the form ",
 Cell[BoxData[
  StyleBox["\[Omega]",
   FontWeight->"Bold"]],
  InitializationCell->True,
  CellChangeTimes->{
   3.637932966608266*^9, {3.637933021731163*^9, 3.637933084463874*^9}, {
    3.63793321848711*^9, 3.6379333754105873`*^9}, {3.637933769127282*^9, 
    3.6379337863995123`*^9}},ExpressionUUID->
  "08dfdd37-de24-4e5f-aec0-316fb32af803"]
}], "Text",
 CellChangeTimes->{{3.6402386814080496`*^9, 3.6402386885996623`*^9}, {
  3.6402387393153515`*^9, 
  3.6402387501885705`*^9}},ExpressionUUID->"8cdabd4e-8f4e-4bbf-9118-\
a1c4ca8d741b"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Omega]", 
   RowBox[{"\[Kappa]", ",", "i"}]], "=", " ", 
  RowBox[{
   RowBox[{"\[Omega]", "[", 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], 
     SubscriptBox["x", "\[Kappa]"]}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"\[Omega]", "[", 
     RowBox[{
      SubsuperscriptBox["B", "i", "j"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "j"], 
       SubscriptBox["x", "\[Iota]"]}]}], "]"}], "=", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["B", "i", "j"], 
      RowBox[{"\[Omega]", "[", 
       RowBox[{
        SubscriptBox["\[PartialD]", "j"], 
        SubscriptBox["x", "\[Iota]"]}], "]"}]}], "=", 
     RowBox[{
      SubsuperscriptBox["B", "i", "j"], 
      SubscriptBox["\[Omega]", 
       RowBox[{"\[Iota]", ",", "j"}]], " ", 
      "\[FilledSquare]"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.640238794711049*^9, 3.640238843835535*^9}, {
  3.640239745173918*^9, 3.64023974625032*^9}, {3.6402397762803726`*^9, 
  3.640239814890441*^9}, {3.640239853781309*^9, 3.6402399920911517`*^9}, {
  3.645254546714055*^9, 3.645254546714055*^9}, {3.70054442432672*^9, 
  3.700544506752095*^9}},ExpressionUUID->"72d30928-c31d-4194-bd82-\
1f7ac5ded7e8"],

Cell[TextData[{
 "The standard cobasis d",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"i", " "}]], TraditionalForm]],ExpressionUUID->
  "2a43ab28-ed06-4562-a173-0c1c7186cd55"],
 "\[DotEqual]  d[x[i]][var[x]] in the chart ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalPhi]", "\[Iota]"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6378228354459114`*^9, 3.637822864979763*^9}, 
    3.637982843390526*^9, {3.637982880814991*^9, 3.6379828892546062`*^9}, 
    3.6379829346818857`*^9, {3.6379830163636293`*^9, 3.6379830307312546`*^9}, 
    3.6379833483010125`*^9},ExpressionUUID->
  "3566c707-e175-49dc-b01f-f3f691a231db"],
 " is given by the differential of the coordinate functions d",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"i", " "}]], TraditionalForm]],ExpressionUUID->
  "7d9ac363-37d3-4d18-9be2-6f189ad9f498"],
 "\[DotEqual]  d[x[i]][var[x]]; it is the dual basis to  the basis {",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[PartialD]", "j"], 
   SubscriptBox["x", "\[Iota]"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.640238794711049*^9, 3.640238843835535*^9}, {
   3.640239745173918*^9, 3.64023974625032*^9}, {3.6402397762803726`*^9, 
   3.640239814890441*^9}, {3.640239853781309*^9, 3.6402399920911517`*^9}, {
   3.645254546714055*^9, 3.645254546714055*^9}, {3.70054442432672*^9, 
   3.700544506752095*^9}},ExpressionUUID->
  "6ef6e815-6b1f-4d88-8995-8ecf219a65e8"],
 "}:"
}], "Text",
 CellChangeTimes->{{3.637997022572138*^9, 3.6379971051282845`*^9}, {
  3.640240483585615*^9, 3.640240547873328*^9}, {3.6402405782621813`*^9, 
  3.640240580914186*^9}, {3.702806805855501*^9, 
  3.7028069055982637`*^9}},ExpressionUUID->"36773d0a-de20-4cad-b4a7-\
775b3c9e070f"],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"x", "[", "i", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.637997119561711*^9, 3.6379972218660917`*^9}, 
   3.640240106298952*^9},ExpressionUUID->"cadb1adf-a54b-4949-a406-\
81f8b8a6a23a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{"x", "[", "i", "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ".", 
     RowBox[{
      RowBox[{"tx\[Iota]", "[", "j", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.637997119561711*^9, 3.637997370965356*^9}, {
   3.640240109356558*^9, 3.640240111275361*^9}, 
   3.6452543117152424`*^9},ExpressionUUID->"54abcfda-5d65-4dfd-aa6b-\
046455d6795d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"y", "[", "i_", "]"}], "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"f\[Kappa]\[Iota]", "[", "i", "]"}], "[", "x", "]"}]}]], "Input",
 CellChangeTimes->{{3.638096156811421*^9, 3.638096179852661*^9}, {
   3.638096257379198*^9, 3.638096298770071*^9}, 
   3.6449319762653074`*^9},ExpressionUUID->"67461b42-52fb-4d7d-a0bd-\
b9a1d7f18da5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"y", "[", "2", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.638096310251691*^9, 3.638096314592499*^9}, 
   3.6380963468261557`*^9, 
   3.6402401135061655`*^9},ExpressionUUID->"38165874-be2c-4d7f-9c4b-\
9b798b7384c1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"y", "[", "2", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.638096385097023*^9, 3.6380964180376816`*^9}, 
   3.6402401219301796`*^9},ExpressionUUID->"a06f9ed5-d5b9-43b6-8911-\
615c39a2525c"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"A\[Kappa]\[Iota]", "[", 
       RowBox[{"j", ",", "2"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{"x", "[", "j", "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6380965858671775`*^9, 3.638096637136868*^9}, {
   3.638096678730541*^9, 3.6380967531194725`*^9}, {3.638096787677533*^9, 
   3.6380968302424088`*^9}, {3.640240131180996*^9, 3.6402401357518044`*^9}, 
   3.644995266771273*^9},ExpressionUUID->"95aaad1e-5ac6-4be3-999c-\
1d66b63f476e"],

Cell[TextData[{
 "This shows the stated transformation rule \nd",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["x", "\[Kappa]"], 
    RowBox[{"i", " "}]], TraditionalForm]],ExpressionUUID->
  "06c964e1-fa4f-40b0-9aa9-129b78e9357a"],
 "= ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["A", "j", "i"], TraditionalForm]],ExpressionUUID->
  "43707296-fecf-42c8-a35e-c9b12236dd67"],
 " d",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["x", "\[Iota]"], "j"], TraditionalForm]],ExpressionUUID->
  "c9b014f4-7140-40cd-93ae-22177896307d"],
 " . \[FilledSquare]\n",
 StyleBox["Notation.",
  FontColor->RGBColor[0, 0, 1]],
 " The traditional notation for the coordinate representation and the \
transformations is"
}], "Text",
 CellChangeTimes->{{3.638097997641466*^9, 3.6380980168637*^9}, {
   3.6380981521943383`*^9, 3.6380983103498173`*^9}, 3.6402406561843185`*^9, {
   3.640240716400424*^9, 3.6402409253939915`*^9}, {3.6452545467608557`*^9, 
   3.6452545467608557`*^9}, {3.702807263229608*^9, 3.702807263229735*^9}, {
   3.702807318334305*^9, 3.702807318334443*^9}, {3.702807357095832*^9, 
   3.702807383186039*^9}, {3.702807415450097*^9, 
   3.702807422517178*^9}},ExpressionUUID->"95556943-49be-4987-9e17-\
7d3af3d4c918"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"\[Omega]", " ", "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Omega]\[Iota]", "i"], 
     SuperscriptBox["dx", "i"]}], "\[DotEqual]", " ", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["\[Omega]\[Iota]", "i"], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
       RowBox[{
        RowBox[{"d", "[", 
         RowBox[{"x", "[", "i", "]"}], "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "n"}], "}"}]}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.638098395410167*^9, 3.63809862408477*^9}, {
   3.6380986616536365`*^9, 3.6380987458217845`*^9}, 3.638098836719545*^9, {
   3.6380989484569416`*^9, 3.6380989518733478`*^9}, {3.6380998217886806`*^9, 
   3.638099825131087*^9}, 3.6402407935269594`*^9, {3.640241011818143*^9, 
   3.640241018073754*^9}, 3.6402410549522185`*^9, {3.64024113540156*^9, 
   3.6402411368367624`*^9}, {3.668433120408209*^9, 3.668433140828586*^9}, 
   3.668433172584515*^9},ExpressionUUID->"ec406c26-201c-43ab-80fb-\
5533373ba07b"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  SubscriptBox["\[Omega]\[Kappa]", "i"]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.638098395410167*^9, 3.63809862408477*^9}, {
    3.6380986616536365`*^9, 3.6380987458217845`*^9}, 3.638098836719545*^9, {
    3.6380989484569416`*^9, 3.6380989518733478`*^9}, {3.6380998217886806`*^9, 
    3.638099825131087*^9}, 3.6402407935269594`*^9, {3.640241011818143*^9, 
    3.640241018073754*^9}, 3.6402410549522185`*^9, {3.64024113540156*^9, 
    3.6402411368367624`*^9}, {3.668433120408209*^9, 3.668433140828586*^9}},
  ExpressionUUID->"68ebd835-40ca-42fb-8cbc-efd38062f71b"],
 ",  ",
 Cell[BoxData[
  SubscriptBox["\[Omega]\[Iota]", "i"]],
  Evaluatable->False,
  CellChangeTimes->{{3.638098395410167*^9, 3.63809862408477*^9}, {
    3.6380986616536365`*^9, 3.6380987458217845`*^9}, 3.638098836719545*^9, {
    3.6380989484569416`*^9, 3.6380989518733478`*^9}, {3.6380998217886806`*^9, 
    3.638099825131087*^9}, 3.6402407935269594`*^9, {3.640241011818143*^9, 
    3.640241018073754*^9}, 3.6402410549522185`*^9, {3.64024113540156*^9, 
    3.6402411368367624`*^9}},ExpressionUUID->
  "1eb2b684-98cf-4653-b133-368360f60843"],
 " are the coordinates of the covector \[Omega] in the charts corresponding \
to \[Kappa], \[Iota].. The proved transformation rules are in traditional form"
}], "Text",
 CellChangeTimes->{{3.6380987676502233`*^9, 3.638098812129902*^9}, {
   3.638098848856366*^9, 3.638098905708866*^9}, {3.63809896451437*^9, 
   3.63809896451437*^9}, {3.6380998379387093`*^9, 3.6380998411211147`*^9}, {
   3.6450816247970915`*^9, 3.6450816442401266`*^9}, {3.6450816877204027`*^9, 
   3.645081688250804*^9}, 3.6450817259640703`*^9, {3.645081781847369*^9, 
   3.6450818127082233`*^9}, {3.668433202586484*^9, 
   3.668433328397087*^9}},ExpressionUUID->"12630cc3-9fc5-4685-a31d-\
2a3b44d981d0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Omega]\[Kappa]", "j"], "=", " ", 
   RowBox[{
    SubsuperscriptBox["B", "j", "i"], 
    SubscriptBox["\[Omega]\[Iota]", "i"]}]}], ",", " ", 
  RowBox[{
   RowBox[{"d", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["x", "\[Kappa]"], 
      RowBox[{"j", " "}]],
     TraditionalForm]}], "=", 
   RowBox[{
    FormBox[
     SubsuperscriptBox["A", "l", "j"],
     TraditionalForm], "d", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["x", "\[Iota]"], "l"],
     TraditionalForm]}]}], ",", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Omega]\[Kappa]", "j"], " ", "d", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["x", "\[Kappa]"], "j"],
     TraditionalForm]}], " ", "\[Equal]", " ", 
   RowBox[{
    SubscriptBox["\[Omega]\[Iota]", "i"], 
    SubsuperscriptBox["B", "j", "i"], "  ", 
    SubsuperscriptBox["A", "l", "j"], "d", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["x", "\[Iota]"], "l"],
     TraditionalForm]}], "\[Equal]", "  ", 
   RowBox[{
    SubscriptBox["\[Omega]\[Iota]", "i"], 
    SubsuperscriptBox["\[Delta]", "l", "i"], "d", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["x", "\[Iota]"], "l"],
     TraditionalForm]}], " ", "\[Equal]", 
   RowBox[{
    SubscriptBox["\[Omega]\[Iota]", "i"], "d", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["x", "\[Iota]"], "i"],
     TraditionalForm]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6380990076992464`*^9, 3.638099062539343*^9}, {
   3.638099108048623*^9, 3.6380992558240833`*^9}, {3.638099300256962*^9, 
   3.638099302097765*^9}, {3.638099334425022*^9, 3.6380993989839354`*^9}, {
   3.638099448490823*^9, 3.6380995288535643`*^9}, 3.640241213638098*^9, 
   3.640241258971777*^9, 3.64024128914223*^9, {3.6402413343043094`*^9, 
   3.6402414038024316`*^9}, 3.6402414389804935`*^9, {3.6402414892593813`*^9, 
   3.640241612156398*^9}, {3.6452545467608557`*^9, 3.6452545467764554`*^9}, 
   3.6684333690234528`*^9, {3.6684334088799877`*^9, 3.6684334232072973`*^9}, {
   3.668433464509598*^9, 3.6684334847056017`*^9}, {3.668433515381336*^9, 
   3.668433524748069*^9}, {3.6684335963658123`*^9, 3.668433678989091*^9}, {
   3.66843373273193*^9, 3.668433760460781*^9}, {3.668433823195335*^9, 
   3.668433864962549*^9}, {3.700544974457178*^9, 3.7005449905283318`*^9}, {
   3.700545032440667*^9, 3.700545065848225*^9}, {3.7005451242480907`*^9, 
   3.700545185182706*^9}},ExpressionUUID->"ea29f98e-51e4-4e49-832f-\
3157025009ad"],

Cell[" \[FilledSquare]", "Text",
 CellChangeTimes->{{3.6380996077977037`*^9, 3.6380997654373817`*^9}, 
   3.640241627709625*^9},ExpressionUUID->"9de29ff2-f230-4031-8eda-\
f488da907c1b"],

Cell[TextData[StyleBox["Notation (Summary)",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6345494593889055`*^9, 3.634549494442167*^9}},
 CellTags->
  "transfcoords",ExpressionUUID->"9ebfee58-3354-49b5-a02b-11a7150a83ac"],

Cell[TextData[{
 " ",
 Cell[BoxData[
  SubsuperscriptBox["A", 
   RowBox[{"i", "  "}], "j"]],
  Evaluatable->False,
  CellChangeTimes->{{3.633263302173919*^9, 3.6332633842828636`*^9}},
  FontWeight->"Bold",ExpressionUUID->"a80ea238-4909-472d-a6f5-9907aed31b35"],
 "= ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      SuperscriptBox["y", "j"]}], 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["x", "i"]}]], " "}], TraditionalForm]],ExpressionUUID->
  "5c2e6130-3b02-4e3f-bcc9-a5de5aec0611"],
 StyleBox["\[DotEqual] ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{
   RowBox[{"A\[Kappa]\[Iota]", "[", 
    RowBox[{"i", ",", "j"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", "...", ",", 
    RowBox[{"x", "[", "n", "]"}]}], "]"}]],
  CellChangeTimes->{{3.6331875903732033`*^9, 3.6331876454609003`*^9}, 
    3.633188196498871*^9, 3.63326148156371*^9, {3.633262878720773*^9, 
    3.633262881965579*^9}},
  FontWeight->"Bold",ExpressionUUID->"2410db79-924a-470a-a6e2-04a59d018d70"],
 ";\n ",
 Cell[BoxData[
  SubsuperscriptBox["B", "i", "j"]],
  Evaluatable->False,
  CellChangeTimes->{{3.633263302173919*^9, 3.6332633842828636`*^9}},
  FontWeight->"Bold",ExpressionUUID->"07faeb89-f6f5-4cb9-8314-2c33b6a02fa3"],
 " =  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      SuperscriptBox["x", "j"]}], 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["y", "i"]}]], " "}], TraditionalForm]],ExpressionUUID->
  "6811b2a9-1772-444d-bbaf-b4a83e533194"],
 " ",
 StyleBox["\[DotEqual] ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{
   RowBox[{"A\[Iota]\[Kappa]", "[", 
    RowBox[{"i", ",", "j"}], "]"}], "[", 
   RowBox[{
    RowBox[{"y", "[", "1", "]"}], ",", "...", ",", 
    RowBox[{"y", "[", "n", "]"}]}], " ", "]"}]],
  CellChangeTimes->{{3.6331875903732033`*^9, 3.6331876454609003`*^9}, 
    3.633188196498871*^9, 3.63326148156371*^9, {3.633262878720773*^9, 
    3.633262881965579*^9}},
  FontWeight->"Bold",ExpressionUUID->"d784a2c7-c736-4274-830a-f5cc7a9a57a3"],
 ";"
}], "Text",
 CellChangeTimes->{{3.645278591219612*^9, 3.6452786010476294`*^9}, {
  3.645278690310986*^9, 
  3.645278694086193*^9}},ExpressionUUID->"745ee643-2389-41f2-8cc1-\
d73789b814d6"],

Cell[TextData[StyleBox["Traditional:",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.668434019816759*^9, 
  3.668434026240452*^9}},ExpressionUUID->"82eaeabf-716c-45ee-a4b9-\
289e6dc95cf0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Vectors", ":", "  ", 
    RowBox[{
     SubsuperscriptBox["\[Xi]", "\[Iota]", "i"], 
     RowBox[{
      SubscriptBox["\[PartialD]", "i"], 
      SubscriptBox["x", "\[Iota]"]}]}]}], " ", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubsuperscriptBox["\[Xi]", "\[Kappa]", "i"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "i"], 
       SubscriptBox["x", "\[Kappa]"]}]}], " ", "\[DotEqual]", " ", "X"}], 
    "\[Element]", 
    FormBox[
     RowBox[{
      SubscriptBox["T", "x"], 
      SuperscriptBox["M", 
       RowBox[{"n", " "}]]}],
     TraditionalForm]}]}], ",", "  ", 
  RowBox[{
   SubsuperscriptBox["\[Xi]", "\[Kappa]", "i"], " ", "=", " ", 
   RowBox[{
    SubsuperscriptBox["A", "j", "i"], " ", 
    SubsuperscriptBox["\[Xi]", "\[Iota]", "j"]}]}], ",", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    SubscriptBox["x", "\[Kappa]"]}], " ", "=", " ", 
   RowBox[{
    SubsuperscriptBox[
     StyleBox["B",
      FontWeight->"Plain"], "j", "i"], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], 
     SubscriptBox["x", "\[Iota]"]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6345495243318195`*^9, 3.634549596934347*^9}, {
  3.6345496310828066`*^9, 3.6345499633789907`*^9}, {3.6345503905257406`*^9, 
  3.6345504252670016`*^9}, {3.634550585229683*^9, 3.6345507063794956`*^9}, {
  3.6349706105519714`*^9, 3.6349706507748423`*^9}, {3.6452545467764554`*^9, 
  3.645254546792055*^9}, {3.645272088550991*^9, 3.645272088566591*^9}},
 TextAlignment->Center,ExpressionUUID->"7746d7af-4ad7-4ebb-bc89-12a4ad3d8dc1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Covectors", ":", "  ", 
    RowBox[{
     SubscriptBox["\[Omega]", "\[Iota]i"], 
     SubsuperscriptBox["dx", "\[Iota]", "i"]}]}], "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["\[Omega]", "\[Kappa]i"], 
      SubsuperscriptBox["dx", "\[Kappa]", "i"]}], " ", "\[DotEqual]", " ", 
     "\[Omega]"}], " ", "\[Element]", " ", 
    FormBox[
     RowBox[{
      SubsuperscriptBox["T", "x", "*"], 
      SuperscriptBox["M", 
       RowBox[{"n", " "}]]}],
     TraditionalForm]}]}], ",", "  ", 
  RowBox[{
   SubscriptBox["\[Omega]", "\[Kappa]j"], " ", "=", " ", 
   RowBox[{
    SubsuperscriptBox[
     StyleBox["B",
      FontWeight->"Plain"], "j", "i"], "  ", 
    SubscriptBox["\[Omega]", "\[Iota]i"]}]}], ",", " ", 
  RowBox[{
   SubsuperscriptBox["dx", "\[Kappa]", "i"], " ", "=", " ", 
   RowBox[{
    SubsuperscriptBox["A", "j", "i"], " ", 
    SubsuperscriptBox["dx", "\[Iota]", "j"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6345495243318195`*^9, 3.634549596934347*^9}, {
   3.6345496310828066`*^9, 3.6345499633789907`*^9}, {3.6345503905257406`*^9, 
   3.6345504252670016`*^9}, {3.634550585229683*^9, 3.6345507063794956`*^9}, {
   3.6345510258212566`*^9, 3.63455102789606*^9}, {3.6345510893757677`*^9, 
   3.634551266404879*^9}, {3.6345513001633387`*^9, 3.6345513105529566`*^9}, {
   3.634551349038224*^9, 3.6345513730622663`*^9}, {3.634551405915924*^9, 
   3.6345514475211973`*^9}, {3.634551489360471*^9, 3.634551541994963*^9}, {
   3.6349706606184597`*^9, 3.6349706761514874`*^9}, {3.6402417295778036`*^9, 
   3.6402417427442265`*^9}, 3.6501911288387012`*^9},
 TextAlignment->Center,ExpressionUUID->"b37673f2-4137-4e4e-adae-9c700f3448ee"],

Cell[BoxData[
 RowBox[{"    ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubsuperscriptBox["A", "j", 
      RowBox[{"k", " "}]], 
     SubsuperscriptBox["B", "i", "j"]}], "=", " ", 
    SubsuperscriptBox["\[Delta]", "i", "k"]}], ",", "  ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["B", "j", 
      RowBox[{"k", " "}]], " ", 
     SubsuperscriptBox["A", "i", "j"]}], " ", "=", " ", 
    RowBox[{
     SubsuperscriptBox["\[Delta]", "i", "k"], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6341105496594915`*^9, 3.6341105736095343`*^9}, {
   3.634112436665019*^9, 3.6341125989233046`*^9}, {3.6343605196925554`*^9, 
   3.6343606278163457`*^9}, 3.634551867598735*^9},
 TextAlignment->Center,ExpressionUUID->"520b3dd6-b1fc-44bb-a470-99216ef8cda2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox["dx", "\[Iota]", "j"], ".", 
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], 
    SubscriptBox["x", "\[Iota]"]}]}], " ", "=", " ", 
  SubsuperscriptBox["\[Delta]", "i", "j"]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{3.668434126744368*^9, 3.668434159065393*^9},
 TextAlignment->Center,ExpressionUUID->"0dd55052-7043-44b8-b396-fedb4dd42277"],

Cell[TextData[{
 StyleBox["Exercise",
  FontColor->RGBColor[0, 0, 1]],
 ". Translate the traditional formulas contained in this summary into \
Mathematica expressions and verify the formulas using symbolic calculations \
with Mathematica. \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6345519750205235`*^9, 3.6345521051871524`*^9}, {
  3.638156620265335*^9, 
  3.638156640406971*^9}},ExpressionUUID->"c48a65b3-6b66-456e-a023-\
4cf9da9a53db"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{
  3.6346238198140507`*^9},ExpressionUUID->"99ded239-bc1a-4fed-a812-\
29aae7c7068b"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.3. Vector and Tensor Fields on a Manifold", "Subsection",
 CellChangeTimes->{{3.6346238480425*^9, 3.634623854376111*^9}, {
   3.6346239166046205`*^9, 3.634623945339871*^9}, {3.6358314641820974`*^9, 
   3.635831482126129*^9}, 3.6381705955687523`*^9, 3.638190205226172*^9, 
   3.6402419854026527`*^9, 3.6403279076641035`*^9, {3.65363306611515*^9, 
   3.653633097116003*^9}, 3.666252176780931*^9, 3.6662522156918488`*^9, {
   3.668434258798979*^9, 3.6684342631506643`*^9}, 3.668493782207129*^9, 
   3.700545421892852*^9, 
   3.700807148522641*^9},ExpressionUUID->"093cf404-3037-4811-bba6-\
b8fee14d6727"],

Cell[CellGroupData[{

Cell["2.3.1. Definition of Tensor Fields", "Subsubsection",
 CellChangeTimes->{{3.6349940033747053`*^9, 3.634994040699971*^9}, 
   3.6450819941283436`*^9, 3.6450888830034647`*^9, 3.700545434997488*^9, 
   3.700573315201543*^9, 3.7028074717864656`*^9, 
   3.702878210150734*^9},ExpressionUUID->"3156ffd5-d6ab-40fd-8ad5-\
173a643bf2de"],

Cell["\<\
In the package tensalgv3.m and the notebook vectensalgv3.nb I implemented a \
tensor concept into Mathematica allowing to consider and apply covariant, \
contravariant, and mixed type tensors:\
\>", "Text",
 CellChangeTimes->{{3.6346299040849047`*^9, 3.6346301465873303`*^9}, {
   3.634630182685794*^9, 3.634630246177905*^9}, {3.63463027722196*^9, 
   3.63463027722196*^9}, 3.6347958986409163`*^9, {3.634795928805972*^9, 
   3.6347959486356077`*^9}, {3.6347960390653667`*^9, 3.634796051063788*^9}, {
   3.6347961266695213`*^9, 3.634796128978325*^9}, 3.635831308854224*^9, 
   3.668487247429328*^9, {3.700546887640394*^9, 3.7005469028717737`*^9}, {
   3.700546990847039*^9, 3.7005470327827187`*^9}, {3.700547108854205*^9, 
   3.700547238605464*^9}, 3.700547281796934*^9, {3.702807646057025*^9, 
   3.702807679880476*^9}},ExpressionUUID->"20c9ba73-9e3d-43c2-b542-\
8bcddb750185"],

Cell[BoxData[
 RowBox[{"?", "tensor"}]], "Input",
 CellChangeTimes->{{3.63479625608115*^9, 
  3.6347962620111604`*^9}},ExpressionUUID->"a092e73d-5412-442f-b207-\
8d39f212e761"],

Cell[TextData[{
 "At any point x of the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "5a58d254-faf3-47bf-94bc-60bd2a47d6c7"],
 " there exists  the complete tensor algebra based on the  tangent vector \
space ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["T", "x"]}], TraditionalForm]],ExpressionUUID->
  "7310c1a2-bdb3-4bef-a59e-a8b1fe6da4ac"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "9f121280-e740-4956-ab12-70f1901ff1ff"],
 ". \n",
 StyleBox["Definitions and",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 StyleBox["Notations.",
  FontColor->RGBColor[0, 0, 1]],
 " "
}], "Text",
 CellChangeTimes->{{3.6349697420030317`*^9, 3.6349699716984386`*^9}, {
   3.634970024187931*^9, 3.6349702459731245`*^9}, {3.6349702949516106`*^9, 
   3.6349703803157625`*^9}, {3.634970706811542*^9, 3.6349708110533257`*^9}, {
   3.6349708707500315`*^9, 3.6349709107077017`*^9}, {3.634971245187493*^9, 
   3.634971327504238*^9}, {3.634971364776704*^9, 3.6349713981199627`*^9}, {
   3.6349714421044407`*^9, 3.6349714957493353`*^9}, 3.6349715440686207`*^9, {
   3.634971578675482*^9, 3.6349716395587893`*^9}, {3.634971705137705*^9, 
   3.6349718055262823`*^9}, {3.6349718644439864`*^9, 3.634971882920419*^9}, {
   3.6349719152496767`*^9, 3.634971938277317*^9}, {3.6349720454403067`*^9, 
   3.6349720872699804`*^9}, 3.6349721276176515`*^9, {3.6349721733367324`*^9, 
   3.6349721735083323`*^9}, {3.634991375343164*^9, 3.634991388684188*^9}, {
   3.6349914255490522`*^9, 3.6349914620229163`*^9}, 3.6349915554116807`*^9, {
   3.6349937438928456`*^9, 3.6349937438928456`*^9}, {3.634994228646903*^9, 
   3.634994236527917*^9}, {3.6359188209944506`*^9, 3.635918821103651*^9}, {
   3.63818786473625*^9, 3.638187867793855*^9}, 3.638190284246311*^9, {
   3.640316124331456*^9, 3.6403161488858995`*^9}, {3.6403162932797527`*^9, 
   3.6403163328726225`*^9}, {3.6403163738226943`*^9, 
   3.6403164215743785`*^9}, {3.640316469950063*^9, 3.640316566311433*^9}, {
   3.640316647805976*^9, 3.640316666697609*^9}, 3.6403320619514*^9, {
   3.6450821350553913`*^9, 3.6450821394077992`*^9}, 3.6450823636467667`*^9, {
   3.668487406134207*^9, 3.668487407117858*^9}, 3.668487440431488*^9, 
   3.700547518001775*^9},ExpressionUUID->"117e7905-2c79-4b02-9749-\
e7ad3e535bc2"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"k", ",", " ", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "e52b9791-ee31-48e3-8b28-cc13036b3f1c"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "ce1b8035-1402-4f9b-876c-3560f8f09603"],
 " =(",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[CircleTimes]", "k"], TraditionalForm]],ExpressionUUID->
  "4c1f5b79-9c2d-424f-aea0-7fcea2380979"],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubsuperscriptBox["T", "x", "*"]}], TraditionalForm]],ExpressionUUID->
  "382d8545-17f3-419f-8844-3f167308bde7"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "3b66acc1-0205-41d9-a0ff-927e9a844637"],
 Cell[BoxData[
  FormBox[
   RowBox[{")", "\[CircleTimes]"}], TraditionalForm]],ExpressionUUID->
  "c2ee4d5a-0c0c-4fd0-8f01-a29a77de5b55"],
 "(",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[CircleTimes]", "h"], TraditionalForm]],ExpressionUUID->
  "f3424d52-00c8-4e2c-b500-53d0e6fc9454"],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["T", "x"], 
     SuperscriptBox["M", 
      RowBox[{"n", " "}]]}]}], TraditionalForm]],ExpressionUUID->
  "87057981-560c-4787-aadd-64c54698f121"],
 ") "
}], "Text",
 CellChangeTimes->{{3.6349697420030317`*^9, 3.6349699716984386`*^9}, {
   3.634970024187931*^9, 3.6349702459731245`*^9}, {3.6349702949516106`*^9, 
   3.6349703803157625`*^9}, {3.634970706811542*^9, 3.6349708110533257`*^9}, {
   3.6349708707500315`*^9, 3.6349709107077017`*^9}, {3.634971245187493*^9, 
   3.634971327504238*^9}, {3.634971364776704*^9, 3.6349713981199627`*^9}, {
   3.6349714421044407`*^9, 3.6349714957493353`*^9}, 3.6349715440686207`*^9, {
   3.634971578675482*^9, 3.6349716395587893`*^9}, {3.634971705137705*^9, 
   3.6349718055262823`*^9}, {3.6349718644439864`*^9, 3.634971882920419*^9}, {
   3.6349719152496767`*^9, 3.634971938277317*^9}, {3.6349720454403067`*^9, 
   3.6349720872699804`*^9}, 3.6349721276176515`*^9, {3.6349721733367324`*^9, 
   3.6349721735083323`*^9}, {3.634991375343164*^9, 3.634991388684188*^9}, {
   3.6349914255490522`*^9, 3.6349914620229163`*^9}, 3.6349915554116807`*^9, {
   3.6349937438928456`*^9, 3.6349937438928456`*^9}, {3.634994228646903*^9, 
   3.634994236527917*^9}, {3.6359188209944506`*^9, 3.635918821103651*^9}, {
   3.63818786473625*^9, 3.638187867793855*^9}, 3.638190284246311*^9, {
   3.640316124331456*^9, 3.6403161488858995`*^9}, {3.6403162932797527`*^9, 
   3.6403163328726225`*^9}, {3.6403163738226943`*^9, 
   3.6403164215743785`*^9}, {3.640316469950063*^9, 3.640316566311433*^9}, {
   3.640316647805976*^9, 3.640316666697609*^9}, 3.6403320619514*^9, {
   3.6450821350553913`*^9, 3.6450821394077992`*^9}, 3.6450823636467667`*^9, {
   3.668487406134207*^9, 3.668487407117858*^9}, 3.668487440431488*^9, 
   3.70054753155665*^9, {3.7028078584029827`*^9, 3.7028078682641983`*^9}},
 TextAlignment->Center,ExpressionUUID->"2bc223df-c635-4321-bbe5-b963f58cdb71"],

Cell[TextData[{
 " is the space of ",
 StyleBox["k-fold covariant, h-fold contravariant tensors",
  FontColor->RGBColor[0, 0, 1]],
 " over the vector space ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "84c802f8-c4f2-4d50-a5d0-40e36c6914ac"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "c0c1c2b0-5fcb-4b3c-be31-49f319e87835"],
 ". Their union  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["T", 
    RowBox[{"(", 
     RowBox[{"k", ",", " ", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "6ba2222c-65da-4b9d-8814-13b3e32e8d5a"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "7434a79e-3cb5-4788-a197-7e9d5a686213"],
 " := ",
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Union]", 
    RowBox[{"x", "\[Element]", 
     SuperscriptBox["M", 
      RowBox[{"n", " "}]]}]], TraditionalForm]],ExpressionUUID->
  "8ad370dd-6ebf-4a90-aaa7-a4173c017545"],
 " ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"k", ",", " ", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "749619e5-de80-478a-8c2d-512dc88f1fee"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "9321bf9f-a767-468b-91c5-9ab8e30cbe6b"],
 "  is called the",
 StyleBox[" bundle of tensors of type {k,h} ",
  FontColor->RGBColor[0, 0, 1]],
 "over the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "b3a394d9-86ca-45ac-ba46-0bc18600495e"],
 "; it is itself a smooth manifold whose coordinates  are the point \
coordinates and the tensor coordinates defined in the charts of the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "1b4b8cd6-00d1-4c10-b725-8e8b0ff07242"],
 " with respect to the tensor basis whose elements are tensor products of the \
standard basis vectors and covectors of the dual basis. The construction of \
an atlas of the tensor bundle is quite analogous to the construction in case \
of the cotangent bundle, see Subsection 2.2.3.  For details see e.g. [",
 ButtonBox["K-N1",
  BaseStyle->"Hyperlink",
  ButtonData->"K-N1"],
 "] or [",
 ButtonBox["S-W",
  BaseStyle->"Hyperlink",
  ButtonData->"S-W"],
 "]. The ",
 StyleBox["projection",
  FontColor->RGBColor[0, 0, 1]],
 " p of the tensor bundle is the smooth map"
}], "Text",
 CellChangeTimes->{{3.6349697420030317`*^9, 3.6349699716984386`*^9}, {
   3.634970024187931*^9, 3.6349702459731245`*^9}, {3.6349702949516106`*^9, 
   3.6349703803157625`*^9}, {3.634970706811542*^9, 3.6349708110533257`*^9}, {
   3.6349708707500315`*^9, 3.6349709107077017`*^9}, {3.634971245187493*^9, 
   3.634971327504238*^9}, {3.634971364776704*^9, 3.6349713981199627`*^9}, {
   3.6349714421044407`*^9, 3.6349714957493353`*^9}, 3.6349715440686207`*^9, {
   3.634971578675482*^9, 3.6349716395587893`*^9}, {3.634971705137705*^9, 
   3.6349718055262823`*^9}, {3.6349718644439864`*^9, 3.634971882920419*^9}, {
   3.6349719152496767`*^9, 3.634971938277317*^9}, {3.6349720454403067`*^9, 
   3.6349720872699804`*^9}, 3.6349721276176515`*^9, {3.6349721733367324`*^9, 
   3.6349721735083323`*^9}, {3.634991375343164*^9, 3.634991388684188*^9}, {
   3.6349914255490522`*^9, 3.6349914620229163`*^9}, 3.6349915554116807`*^9, {
   3.6349937438928456`*^9, 3.6349937438928456`*^9}, {3.634994228646903*^9, 
   3.634994236527917*^9}, {3.6359188209944506`*^9, 3.635918821103651*^9}, {
   3.63818786473625*^9, 3.638187867793855*^9}, 3.638190284246311*^9, {
   3.640316124331456*^9, 3.6403161488858995`*^9}, {3.6403162932797527`*^9, 
   3.6403163328726225`*^9}, {3.6403163738226943`*^9, 
   3.6403164215743785`*^9}, {3.640316469950063*^9, 3.640316566311433*^9}, {
   3.640316647805976*^9, 3.640316666697609*^9}, 3.6403320619514*^9, {
   3.6450821350553913`*^9, 3.6450821394077992`*^9}, 3.6450823636467667`*^9, {
   3.668487406134207*^9, 3.668487407117858*^9}, 3.668487440431488*^9, 
   3.7005480023390703`*^9, {3.7005480594655533`*^9, 3.700548060143671*^9}, {
   3.702808295918611*^9, 
   3.702808312911263*^9}},ExpressionUUID->"ea87918c-5a25-48de-abe5-\
a364cd3ea116"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"p", ":", 
    RowBox[{"ts", "\[Element]", 
     RowBox[{
      FormBox[
       RowBox[{
        SuperscriptBox["T", 
         RowBox[{"(", 
          RowBox[{"k", ",", "h"}], ")"}]], 
        SuperscriptBox["M", "n"]}],
       TraditionalForm], "\[RightTeeArrow]", 
      RowBox[{"p", "[", "ts", "]"}]}]}]}], " ", "=", 
   RowBox[{"x", "\[Element]", 
    SuperscriptBox["M", "n"]}]}], ",", " ", 
  RowBox[{
   RowBox[{"if", " ", "ts"}], " ", "\[Element]", "  ", 
   RowBox[{
    FormBox[
     SubsuperscriptBox["T", "x", 
      RowBox[{"(", 
       RowBox[{"k", ",", " ", "h"}], ")"}]],
     TraditionalForm], 
    RowBox[{
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}, {3.634993376370197*^9, 
   3.6349933837646093`*^9}, 
   3.6349941130950994`*^9},ExpressionUUID->"8247b670-51b3-4fd1-b930-\
de45227351c7"],

Cell[TextData[{
 "The preimages  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "fd559685-c149-4451-bcb3-cdcf57c08b9b"],
 "[x] = ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"k", ",", " ", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "05da0e95-cb57-4d1f-9d3b-b883828f7674"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "dd7f54a9-58eb-497d-9824-d9eedcd21c12"],
 " are the",
 StyleBox[" fibers",
  FontColor->RGBColor[0, 0, 1]],
 " of the bundle, They have, by the construction of the tensor spaces, the \
structure of a real vector space of dimension ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["n", 
    RowBox[{"k", "+", "h"}]], TraditionalForm]],ExpressionUUID->
  "fff86587-a95a-4fa3-8e13-8d266b0e8948"],
 ".  A",
 StyleBox[" tensor field of type ",
  FontColor->RGBColor[0, 0, 1]],
 "{k,h] is a smooth function",
 StyleBox[" ts:",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{"x", "\[Element]", 
   SuperscriptBox["M", "n"]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"f8dd4d5c-bef5-427c-8345-e8ca3247ea50"],
 StyleBox["\[RightTeeArrow] ts(x) ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{"\[Element]", 
   RowBox[{
    FormBox[
     SuperscriptBox["T", 
      RowBox[{"(", 
       RowBox[{"k", ",", " ", "h"}], ")"}]],
     TraditionalForm], 
    FormBox[
     SuperscriptBox["M", "n"],
     TraditionalForm]}]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"b9689ef8-6369-4c45-958e-e39245998ee1"],
 " such that the composition p\[SmallCircle]ts = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["identity", 
    SuperscriptBox["M", "n"]], TraditionalForm]],ExpressionUUID->
  "8a813ce8-76fc-43ab-9815-f6133d2f6e78"],
 " is the identity map of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "4596122e-f748-4901-921b-4da2fbe9634d"],
 ", that means ",
 StyleBox["ts[x] ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{"\[Element]", "  ", 
   RowBox[{
    FormBox[
     SubsuperscriptBox["T", "x", 
      RowBox[{"(", 
       RowBox[{"k", ",", " ", "h"}], ")"}]],
     TraditionalForm], 
    FormBox[
     SuperscriptBox["M", "n"],
     TraditionalForm]}]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"80a4421e-aa1a-4ff6-ba6e-50f39c0cea1e"],
 " belongs to the fiber over x for all ",
 Cell[BoxData[
  StyleBox[
   RowBox[{"x", "\[Element]", 
    SuperscriptBox["M", "n"]}],
   FontWeight->"Bold"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},ExpressionUUID->
  "053ba4e4-65f2-404f-9e2f-ce87f4eefee3"],
 ". Such a map is sometimes called a ",
 StyleBox["section",
  FontColor->RGBColor[0, 0, 1]],
 " of the bundle. The set of all smooth tensor fields of type (k,h) on the \
smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"2feff5c2-71dc-49d7-9e58-a3690bd90db2"],
 " will be denoted by ",
 Cell[BoxData[
  RowBox[{
   FormBox[
    SuperscriptBox["\[ScriptCapitalJ]", 
     RowBox[{"(", 
      RowBox[{"k", ",", " ", "h"}], ")"}]],
    TraditionalForm], 
   FormBox[
    SuperscriptBox["M", "n"],
    TraditionalForm]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"350f55c6-5e16-4eee-8aa4-0ceb694c3544"],
 "\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6349920538150597`*^9, 3.6349921302817945`*^9}, {
   3.634992171267067*^9, 3.6349923204927306`*^9}, 3.63499238794565*^9, {
   3.634992497339443*^9, 3.6349925475483317`*^9}, {3.634992582669994*^9, 
   3.6349927905199614`*^9}, {3.6349928721759057`*^9, 3.63499289164674*^9}, {
   3.634992924334798*^9, 3.634993045570012*^9}, {3.634993093645697*^9, 
   3.6349931183445406`*^9}, {3.6349931987610826`*^9, 3.634993258817589*^9}, 
   3.6349932959350543`*^9, {3.634993409916256*^9, 3.63499341230306*^9}, 
   3.634993471765765*^9, 3.634993634254452*^9, {3.634994178266414*^9, 
   3.6349941977528486`*^9}, {3.6349943140614543`*^9, 
   3.6349944035738125`*^9}, {3.6349945176456137`*^9, 3.634994578419721*^9}, {
   3.6519081174250717`*^9, 3.651908298247011*^9}, 
   3.702808350220788*^9},ExpressionUUID->"9ee7503a-60ff-4d1f-bd48-\
0024848c7feb"],

Cell[TextData[{
 "\nClearly, with the definitions formulated in the preceding paragraph, all \
the algebraic operations and properties of tensor algebra over a field may be \
applied pointwise for tensor fields on a manifold. In addition to these \
algebraic operations  some important analytic operations will be considered \
in this section. First we will treat vector fields and its action on \
functions as  differential operators and the exterior calculus of \
differential forms. Both structures exist on arbitrary smooth manifolds \
without supposing any additional structure; they are properties of the \
manifolds depending on the differentiable structure only. Later we introduce \
the absolute derivatives of smooth tensor fields what requires the presence \
of an additional structure, e.g. a pseudo-Riemannian metric. We considered \
already the elementary 2-dimensional case of  this analysis on manifolds in \
the notebook [",
 ButtonBox["EDG",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "] about surfaces;  some knowledge of surface theory would be useful to \
understand the mentioned generalizations. "
}], "Text",
 CellChangeTimes->{{3.6349920538150597`*^9, 3.6349921302817945`*^9}, {
   3.634992171267067*^9, 3.6349923204927306`*^9}, 3.63499238794565*^9, {
   3.634992497339443*^9, 3.6349925475483317`*^9}, {3.634992582669994*^9, 
   3.6349927905199614`*^9}, {3.6349928721759057`*^9, 3.63499289164674*^9}, {
   3.634992924334798*^9, 3.634993045570012*^9}, {3.634993093645697*^9, 
   3.6349931183445406`*^9}, {3.6349931987610826`*^9, 3.634993258817589*^9}, 
   3.6349932959350543`*^9, {3.634993409916256*^9, 3.63499341230306*^9}, 
   3.634993471765765*^9, 3.634993634254452*^9, {3.634994178266414*^9, 
   3.6349941977528486`*^9}, {3.6349943140614543`*^9, 
   3.6349944035738125`*^9}, {3.6349945176456137`*^9, 3.634994578419721*^9}, 
   3.6358296797844768`*^9, {3.6358298286184006`*^9, 3.6358298668210707`*^9}, {
   3.6358298975495253`*^9, 3.6358301949334507`*^9}, {3.635830264725374*^9, 
   3.6358303518947277`*^9}, {3.6358303925045996`*^9, 
   3.6358306516260595`*^9}, {3.6358307523206367`*^9, 
   3.6358307749592767`*^9}, {3.6358308345095816`*^9, 3.6358309183807297`*^9}, 
   3.6358309650660124`*^9, {3.635830995895667*^9, 3.635831085091024*^9}, {
   3.635831212159254*^9, 3.6358312806317744`*^9}, 3.6358313255618534`*^9, {
   3.635831382006753*^9, 3.6358314033807907`*^9}, {3.6361210235159245`*^9, 
   3.636121102888864*^9}, 3.638187491054991*^9, 3.6381880629953985`*^9, {
   3.6381881265247107`*^9, 3.638188132281121*^9}, {3.6381881745641956`*^9, 
   3.638188185577815*^9}, {3.640316949229705*^9, 3.640316949229705*^9}, {
   3.6450825404034786`*^9, 3.6450825428526826`*^9}, 
   3.7034203458881617`*^9},ExpressionUUID->"b22c783b-e63d-4cfe-8d41-\
e169a6089adc"],

Cell["\<\
A simple method to define a tensor field - exactly, the local coordinate \
representation of a tensor field - is directly using the function\
\>", "Text",
 CellChangeTimes->{{3.703420465526215*^9, 3.703420597277308*^9}, {
   3.703420673220792*^9, 3.703420707204481*^9}, 
   3.70356903135861*^9},ExpressionUUID->"a485dbde-48c7-4469-ad91-\
43233d83f70c"],

Cell[BoxData[
 RowBox[{"?", "tensor"}]], "Input",
 CellChangeTimes->{{3.703420712241856*^9, 
  3.703420715540328*^9}},ExpressionUUID->"02664ef6-cb42-4718-96f2-\
4032bb95f8e9"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". A tensor field of type (1,1) on a manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "2"], TraditionalForm]],ExpressionUUID->
  "a5458ebd-3966-4b36-b647-efb2d5e70134"],
 ":"
}], "Text",
 CellChangeTimes->{{3.7034208101952*^9, 3.703420935565111*^9}, {
  3.7034215273498793`*^9, 3.7034215801332417`*^9}, {3.7034221314650297`*^9, 
  3.703422170771351*^9}},ExpressionUUID->"b13da655-c767-4b2a-bf85-\
b66ba29d2728"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{"Clear", "[", "ta", "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "1"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.703420941390489*^9, 3.703420980554194*^9}, {
  3.703421087473165*^9, 
  3.7034211951607924`*^9}},ExpressionUUID->"6d0101d5-afd6-42e0-b9fd-\
0643a8d3ad05"],

Cell[BoxData[
 RowBox[{"var", "[", "x", "]"}]], "Input",
 CellChangeTimes->{{3.7034222001936073`*^9, 
  3.703422205632491*^9}},ExpressionUUID->"b8d2050b-b885-45fd-afaf-\
f5bc332c73b6"],

Cell["\<\
[1], x[2] = var[x] are the point coordinates in a given chart of the manifold.\
\>", "Text",
 CellChangeTimes->{{3.7034222843994226`*^9, 
  3.7034223640950537`*^9}},ExpressionUUID->"01ec1685-7afa-47d7-80cf-\
53871aa09dd3"],

Cell[BoxData[
 RowBox[{"ta", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703421213845643*^9, 
  3.7034212675359917`*^9}},ExpressionUUID->"cf7f0198-479c-40c2-aa70-\
d2302b71e07e"],

Cell[TextData[{
 "The tensor coordinates of ta with respect to the tensor basis corresponding \
to the standard  basis of the tangent spaces ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["T", "x"], 
    SuperscriptBox["M", 
     RowBox[{"2", " "}]]}], TraditionalForm]],ExpressionUUID->
  "11642ba2-5ca6-4ea7-af01-576d49d38c4b"],
 "  are  obtained  by"
}], "Text",
 CellChangeTimes->{{3.703421973713889*^9, 3.703422028801531*^9}, {
  3.703422406718576*^9, 
  3.703422633612874*^9}},ExpressionUUID->"3f91d9fa-dab5-4695-aaa9-\
5b880711b521"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ta", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.70342132413195*^9, 
  3.703421374367072*^9}},ExpressionUUID->"2936ddc1-732b-47c2-abed-\
8969ede99132"],

Cell["Here is the coordinate Array of the tensor:", "Text",
 CellChangeTimes->{{3.7034220717771673`*^9, 
  3.7034220950650187`*^9}},ExpressionUUID->"dd2def27-90e5-4462-8fbd-\
b983a9b848fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorToArray", "[", 
   RowBox[{"ta", "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.7034213949996967`*^9, 
  3.703421429222403*^9}},ExpressionUUID->"2a71a65a-93ae-406a-8630-\
e1a823af5b13"],

Cell["in traditional form:", "Text",
 CellChangeTimes->{{3.7034228888505783`*^9, 
  3.703422909978396*^9}},ExpressionUUID->"c1a32448-64ca-40bb-9af4-\
bac7399a4f56"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tradf", "[", "%", "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.703421635645421*^9, 3.703421646252842*^9}, {
  3.703421697885104*^9, 
  3.7034217061964006`*^9}},ExpressionUUID->"068d3355-09e3-4b88-9db7-\
7cf3b66928e7"],

Cell["\<\
Remember, that the first index family -traditionally the subscript indices - \
are covariant, and the second family -traditional the superscript indices - \
are contravariant under a basis transformation. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.703422770595415*^9, 3.703422857986915*^9}, {
   3.703422927042284*^9, 3.7034229499540577`*^9}, {3.7034229892739*^9, 
   3.7034232286751423`*^9}, 3.703425721966537*^9, {3.703425787603199*^9, 
   3.7034259835695057`*^9}, {3.703426017937954*^9, 3.703426096017167*^9}, {
   3.7034261545285263`*^9, 3.703426171327795*^9}, {3.7034262061603603`*^9, 
   3.703426221615903*^9}, 
   3.703475347518732*^9},ExpressionUUID->"a791571a-1afd-4a05-95a9-\
26a73c8cdef7"],

Cell[TextData[{
 "By  reasons connected with the derivation of composed functions, see the \
notebook fieldproblem.nb, we will prefer another definition of tensor fields \
following the \n",
 StyleBox["Rule: ",
  FontColor->RGBColor[0, 0, 1]],
 "The tensor index tuples ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["i", "p"], ",", " ", 
   SubscriptBox["j", "q"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.703569605398942*^9, 3.703569691881323*^9}, {
   3.703569739368348*^9, 3.703569745808646*^9}},
  TextAlignment->Center,ExpressionUUID->
  "9e6fff3a-1c01-4f5b-9b9d-b6f20006c1a5"],
 " will be positioned between the tensor name and the field coordinates; the \
coordinates of the tensor field ta[x,...] of type p, q have the shape"
}], "Text",
 CellChangeTimes->{{3.703422770595415*^9, 3.703422857986915*^9}, {
   3.703422927042284*^9, 3.7034229499540577`*^9}, {3.7034229892739*^9, 
   3.7034232286751423`*^9}, 3.703425721966537*^9, {3.703425787603199*^9, 
   3.7034259835695057`*^9}, {3.703426017937954*^9, 3.703426096017167*^9}, {
   3.7034261545285263`*^9, 3.703426171327795*^9}, {3.7034262061603603`*^9, 
   3.703426221615903*^9}, 3.703475355897715*^9, {3.703475424913113*^9, 
   3.703475444752689*^9}, 3.703478540103149*^9, 3.703485692797518*^9, {
   3.703569130237502*^9, 3.703569326900013*^9}, {3.703569361283348*^9, 
   3.70356937381161*^9}, {3.7035694153554287`*^9, 3.703569593018072*^9}, {
   3.703569909236865*^9, 3.703569928231948*^9}, 
   3.7035699595035353`*^9},ExpressionUUID->"4830ae9f-ce28-42ff-ab20-\
f608094b5216"],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", 
   RowBox[{
    SubscriptBox["i", "p"], ",", " ", 
    SubscriptBox["j", "q"]}], "]"}], "[", 
  RowBox[{"x", ",", "..."}], "]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.703569605398942*^9, 3.703569691881323*^9}, {
  3.703569739368348*^9, 3.703569745808646*^9}},
 TextAlignment->Center,ExpressionUUID->"b28b1ec0-27f1-48a9-9d37-1d2435482987"],

Cell[BoxData[
 RowBox[{"?", "tensorfield"}]], "Input",
 CellChangeTimes->{{3.7034805375956783`*^9, 
  3.7034805440955477`*^9}},ExpressionUUID->"798155c1-5804-4828-8616-\
86ddf74d1863"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tensorfield", "[", 
     RowBox[{"a_", ",", "p_", ",", "q_"}], "]"}], "[", "x__", "]"}], ":=", 
   " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"bb", ",", "b", ",", "ii", ",", "jj", ",", 
       RowBox[{"n", "=", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}]}], "}"}], ",", "\n", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"p", "\[Equal]", "0"}], ")"}], "&&", 
         RowBox[{"(", 
          RowBox[{"q", "\[Equal]", "0"}], ")"}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", 
           RowBox[{"{", "}"}]}], "]"}], "=", " ", 
         RowBox[{"a", "[", "x", "]"}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"ii_", ",", "jj_"}], "]"}], ":=", " ", 
         RowBox[{
          RowBox[{"a", "[", 
           RowBox[{"ii", ",", "jj"}], "]"}], "[", "x", "]"}]}]}], "]"}], ";", 
      "\n", 
      RowBox[{"bb", "=", 
       RowBox[{"tensor", "[", 
        RowBox[{"b", ",", "p", ",", "q", ",", "n"}], "]"}]}], ";", "\n", 
      "bb"}]}], "]"}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.7033992511985817`*^9, 3.703399293772421*^9}, {
   3.703399331179821*^9, 3.70339945381026*^9}, {3.703399486906472*^9, 
   3.70339949538643*^9}, {3.7033995254021482`*^9, 3.703399538210168*^9}, {
   3.7033995789858503`*^9, 3.70339964490539*^9}, {3.703399686665222*^9, 
   3.7033998645996857`*^9}, {3.703399917063393*^9, 3.703399917383213*^9}, {
   3.7033999541030617`*^9, 3.70339995987122*^9}, {3.703475675822983*^9, 
   3.703475693518515*^9}, {3.703475911104986*^9, 3.703475937804191*^9}, {
   3.7034760686352997`*^9, 3.703476133794878*^9}, {3.703476306066512*^9, 
   3.703476322873362*^9}, 3.703476445321447*^9, 3.703476481208331*^9, 
   3.703476667951139*^9, {3.703476698719757*^9, 3.703476724870783*^9}, {
   3.703476856949171*^9, 3.703476859901597*^9}, {3.703477206355646*^9, 
   3.703477243594893*^9}, {3.7034772891941967`*^9, 3.703477322177795*^9}, {
   3.703477356401969*^9, 3.7034773592661753`*^9}, {3.7034774941856213`*^9, 
   3.703477733310614*^9}, {3.7034783721936274`*^9, 3.703478431288262*^9}, {
   3.703478463545046*^9, 3.7034785195151253`*^9}, {3.7034786132951117`*^9, 
   3.7034786209667587`*^9}, {3.703478883053894*^9, 3.703478896501135*^9}, {
   3.703478929532927*^9, 3.703478935337461*^9}, {3.703479008157324*^9, 
   3.703479046260138*^9}, 3.703479094092539*^9, {3.703479916094305*^9, 
   3.703479917605328*^9}, {3.7034799557014303`*^9, 3.703480034116424*^9}, 
   3.7034857417346354`*^9, 
   3.703655855061452*^9},ExpressionUUID->"a12ae26a-8ca2-4826-b310-\
6b879740c6c4"],

Cell["\<\
The only difference to the simpler definition mentioned above is that now the \
tensor field  obeys the Rule:\
\>", "Text",
 CellChangeTimes->{{3.7034860093706093`*^9, 3.703486207425241*^9}, {
   3.703486326504794*^9, 3.703486353392275*^9}, 3.703570046606469*^9, {
   3.70357081672427*^9, 3.7035708432529182`*^9}, {3.7035708745344143`*^9, 
   3.7035708776063766`*^9}, {3.703570962465872*^9, 3.703570971322163*^9}, {
   3.703571051149025*^9, 
   3.703571076917976*^9}},ExpressionUUID->"d62b8885-19df-4fe5-8ad7-\
89d736251e6b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ta", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], "=", 
  RowBox[{
   RowBox[{"tensorfield", "[", 
    RowBox[{"a", ",", "2", ",", "1"}], "]"}], "[", 
   RowBox[{"x", ",", "y"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.703492901620639*^9, 3.703492963330832*^9}, {
  3.703571162681388*^9, 
  3.7035711843854103`*^9}},ExpressionUUID->"06ee9352-f67d-4ca5-a91e-\
f73e8de0e55f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ta", "[", 
   RowBox[{"x", ",", "y"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703492982171701*^9, 3.7034930170256453`*^9}, {
  3.7035711433848877`*^9, 
  3.703571144999606*^9}},ExpressionUUID->"793175d5-60db-4ae9-bda7-\
9db4a76bdc5b"],

Cell["\<\
Warning. The function  tradf does not act on coordinates of tensor fields:\
\>", "Text",
 CellChangeTimes->{{3.7034932717354393`*^9, 
  3.7034933625030823`*^9}},ExpressionUUID->"db9035dd-9a54-48b9-8ba7-\
33682e14804d"],

Cell[BoxData[
 RowBox[{"tradf", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.703493100949431*^9, 
  3.7034931089530077`*^9}},ExpressionUUID->"78b5502f-4876-4331-b336-\
68118394cc7d"],

Cell[BoxData[
 RowBox[{"tradf", "[", 
  RowBox[{
   RowBox[{"a", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "2", "}"}], ",", 
    RowBox[{"{", "1", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.703493100949431*^9, 3.7034931089530077`*^9}, {
  3.703493162986548*^9, 
  3.703493211794014*^9}},ExpressionUUID->"93adcf7a-b7eb-4897-8559-\
5f2d9a055e5f"],

Cell[TextData[{
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". Vector fields. A vector field is simply defined by"
}], "Text",
 CellChangeTimes->{{3.703486393759267*^9, 3.703486434105089*^9}, {
   3.703487050650219*^9, 3.703487099793914*^9}, 
   3.703487199201222*^9},ExpressionUUID->"3a84f648-b44b-4c0d-97cb-\
3b2edbc0db52"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{"Clear", "[", "vx", "]"}], ";", 
  RowBox[{
   RowBox[{"vx", "[", "x__", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"imm", "[", "fx", "]"}], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.703487144307516*^9, 3.7034871454897013`*^9}, {
   3.703487888644294*^9, 3.703488004906851*^9}, 3.703500392225851*^9, 
   3.703500433679678*^9, {3.7035004817432327`*^9, 3.703500495110033*^9}, 
   3.703500800333117*^9, 
   3.703571274748836*^9},ExpressionUUID->"c37d2151-c628-47bb-98a7-\
6fe810728944"],

Cell[BoxData[
 RowBox[{"vx", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.703492514107649*^9, 
  3.70349252391807*^9}},ExpressionUUID->"ba25a44d-c01f-4169-bb7e-\
4153d441b77e"],

Cell[BoxData[
 RowBox[{"?", "imm"}]], "Input",
 CellChangeTimes->{{3.70349139078023*^9, 
  3.703491393327183*^9}},ExpressionUUID->"d21edf42-4774-4273-a3bd-\
0c69af5a4ba7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tvx", "[", 
   RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vx", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7034880714613333`*^9, 
  3.703488156545429*^9}},ExpressionUUID->"9da5338f-f6ed-4592-8b3e-\
f79eb6d72ad9"],

Cell[BoxData[
 RowBox[{"tvx", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7034881621385393`*^9, 
  3.70348817482504*^9}},ExpressionUUID->"0d5f033e-31da-4af9-818d-\
55b5223b7821"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.7034881803497467`*^9, 
  3.703488187577015*^9}},ExpressionUUID->"75b06aa6-e643-465b-88c2-\
f6637d7de6a7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vX", "[", "x__", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"tensorfield", "[", 
    RowBox[{"fx", ",", "0", ",", "1"}], "]"}], "[", "x", "]"}]}]], "Input",
 CellChangeTimes->{{3.7035026886504507`*^9, 
  3.703502748371641*^9}},ExpressionUUID->"3ff02fb1-25b9-44eb-bb65-\
56c5f4be610c"],

Cell[BoxData[
 RowBox[{"vX", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.703502757997756*^9, 
  3.70350277667519*^9}},ExpressionUUID->"f93ddfe3-5b83-4407-b316-\
fce664ccba14"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.703502794626129*^9, 
  3.703502804210723*^9}},ExpressionUUID->"983523cf-95ef-44d3-b050-\
8092d72125bd"],

Cell["formally, the coordinates differ :", "Text",
 CellChangeTimes->{{3.703571416455316*^9, 
  3.7035714504980307`*^9}},ExpressionUUID->"a7dd1077-29ab-445d-b527-\
f356ffc285b8"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"tvx", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "/.", 
    RowBox[{
     RowBox[{
      RowBox[{"fx", "[", 
       RowBox[{
        RowBox[{"{", "}"}], ",", 
        RowBox[{"{", "i_", "}"}]}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}], "->", 
     RowBox[{
      RowBox[{"fx", "[", "i", "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}]}], "]"}]}]}], ",", 
   RowBox[{"vX", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703488218996134*^9, 3.7034882195688267`*^9}, {
  3.70349083090079*^9, 3.703490899692092*^9}, {3.7035028733478537`*^9, 
  3.703502876906827*^9}, {3.703502912588101*^9, 3.703502981394085*^9}, {
  3.703503026482812*^9, 3.703503036425568*^9}, {3.7035030703534822`*^9, 
  3.7035031063532543`*^9}, {3.703503169098907*^9, 
  3.703503183042348*^9}},ExpressionUUID->"9aa0fbe0-09d1-4d4c-8604-\
3516f9e2385c"],

Cell["\<\
A general method to define a tensor by its coordinates is to create first the \
Array of coordinates in the wanted shape and then to use the function\
\>", "Text",
 CellChangeTimes->{{3.7035731910079937`*^9, 
  3.7035733211133213`*^9}},ExpressionUUID->"1b71103b-3e4a-4cb4-9706-\
986648852a74"],

Cell[BoxData[
 RowBox[{"?", "ArrayTotensor"}]], "Input",
 CellChangeTimes->{{3.703573331358808*^9, 
  3.703573359466775*^9}},ExpressionUUID->"4da19528-a478-47da-ba1c-\
8e156d8dd791"],

Cell["in our example:", "Text",
 CellChangeTimes->{{3.703573475689826*^9, 
  3.703573498978149*^9}},ExpressionUUID->"2d6572a5-d17f-4b7c-8de6-\
8089fd5032f7"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Clear", "[", 
    RowBox[{"ar", ",", "vvX"}], "]"}], ";", 
   RowBox[{
    RowBox[{"ar", "[", "x__", "]"}], " ", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"fx", "[", "i", "]"}], "[", "x", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}], ";", 
   RowBox[{
    RowBox[{"vvX", "[", "x__", "]"}], ":=", 
    RowBox[{"ArrayTotensor", "[", 
     RowBox[{"ar", "[", "x", "]"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.703571519773881*^9, 3.703571743237933*^9}, {
   3.703571839362258*^9, 3.7035718527855473`*^9}, 3.703572008896103*^9, {
   3.7035721204893417`*^9, 3.703572195251219*^9}, {3.7035722252833233`*^9, 
   3.7035722467163973`*^9}, {3.703572286477294*^9, 3.703572340335436*^9}, {
   3.703572425522235*^9, 3.703572443504656*^9}, {3.703572522954048*^9, 
   3.7035726202910757`*^9}, {3.70357271622053*^9, 
   3.703572732068421*^9}},ExpressionUUID->"703c0e55-0490-460a-b642-\
39b0dc585e81"],

Cell[BoxData[
 RowBox[{"vvX", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7035717545473633`*^9, 
  3.703571772231023*^9}},ExpressionUUID->"c688fa23-46f6-4569-bc22-\
bbad35475653"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7035717805825653`*^9, 
  3.703571793703864*^9}},ExpressionUUID->"9426b89d-a9cf-4990-9ae5-\
2960219b5b3a"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{"tvx", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"vvX", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703572938422969*^9, 
  3.703572960246608*^9}},ExpressionUUID->"3b57150f-1c30-4ce1-ac10-\
7dc0a8947917"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{"Clear", "[", "vY", "]"}], ";", 
  RowBox[{
   RowBox[{"vY", "[", "x__", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"tensorfield", "[", 
     RowBox[{"fy", ",", "0", ",", "1"}], "]"}], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.703399997281568*^9, 3.70340018267673*^9}, {
   3.703402509283259*^9, 3.70340253698598*^9}, {3.703476344705514*^9, 
   3.703476355536202*^9}, {3.703478124012302*^9, 3.703478177874161*^9}, 
   3.703480074180729*^9, {3.703485797814022*^9, 
   3.70348580418087*^9}},ExpressionUUID->"25a18c20-bafb-4d20-a061-\
c5f65ecae157"],

Cell[BoxData[
 RowBox[{"vY", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7034001968581133`*^9, 3.703400207020358*^9}, 
   3.703402554618863*^9, {3.703478189099764*^9, 
   3.703478194426134*^9}},ExpressionUUID->"61a80af1-8d3a-4e19-a36d-\
0692620a6d27"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.7034787338368053`*^9, 
  3.7034787412135897`*^9}},ExpressionUUID->"2c6e43ef-0aeb-4796-b7a3-\
b387fae38ede"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vY", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703475567951734*^9, 3.70347559330271*^9}, {
  3.703478213010379*^9, 
  3.7034782193694553`*^9}},ExpressionUUID->"1b95c7ca-31a3-43c1-913e-\
33fe38ea716b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vY", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "4", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703475567951734*^9, 3.70347559330271*^9}, {
  3.703478213010379*^9, 3.7034782193694553`*^9}, {3.703480602368204*^9, 
  3.703480609271648*^9}},ExpressionUUID->"26da4945-766d-49f2-af78-\
402e753061f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
  RowBox[{"vY", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",ExpressionUUID->\
"69c4070c-5158-4ea3-87cb-86fa7eaf1a56"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7035717805825653`*^9, 
  3.703571793703864*^9}},ExpressionUUID->"7ef659b0-927b-408d-a165-\
dc6d3e5065aa"],

Cell[BoxData[
 RowBox[{"3.", "*", 
  RowBox[{"vX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.7034809426768007`*^9},ExpressionUUID->"4a18cfd4-8b20-444f-9794-\
e22f5f03e4aa"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7035717805825653`*^9, 
  3.703571793703864*^9}},ExpressionUUID->"eec46947-8754-43a7-8f9b-\
abfac0e72dcb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
  RowBox[{"vX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.7034809426768007`*^9, {3.7035742002343473`*^9, 
   3.70357422395396*^9}},ExpressionUUID->"cf2b7c79-dd86-41df-9f39-\
ca823932094b"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7035717805825653`*^9, 
  3.703571793703864*^9}},ExpressionUUID->"8212bc13-ed24-4005-8e7e-\
5acc2070e945"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.7035748303230963`*^9, 
  3.70357483032535*^9}},ExpressionUUID->"7b753feb-d8f7-4c10-a173-\
d5a01f732977"],

Cell[TextData[StyleBox["Algebraic Operations",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6349697420030317`*^9, 3.6349699716984386`*^9}, {
   3.634970024187931*^9, 3.6349702459731245`*^9}, {3.6349702949516106`*^9, 
   3.6349703803157625`*^9}, {3.634970706811542*^9, 3.6349708110533257`*^9}, {
   3.6349708707500315`*^9, 3.6349709107077017`*^9}, {3.634971245187493*^9, 
   3.634971327504238*^9}, {3.634971364776704*^9, 3.6349713981199627`*^9}, {
   3.6349714421044407`*^9, 3.6349714957493353`*^9}, 3.6349715440686207`*^9, {
   3.634971578675482*^9, 3.6349716395587893`*^9}, {3.634971705137705*^9, 
   3.6349718055262823`*^9}, {3.6349718644439864`*^9, 3.634971882920419*^9}, {
   3.6349719152496767`*^9, 3.634971938277317*^9}, {3.6349720454403067`*^9, 
   3.6349720872699804`*^9}, 3.6349721276176515`*^9, 3.634972195786172*^9, {
   3.636121121187696*^9, 3.636121122388898*^9}, 3.6361211532730646`*^9, {
   3.636270628245629*^9, 3.636270652897672*^9}, {3.703420945794499*^9, 
   3.703420952962428*^9}, 
   3.7034227064280443`*^9},ExpressionUUID->"1bcb556a-4272-400f-84f3-\
be3c60f34bde"],

Cell[TextData[{
 "Example 2 illustrates\n",
 StyleBox["Proposition 1. ",
  FontColor->RGBColor[0, 0, 1]],
 "The set  ",
 Cell[BoxData[
  RowBox[{
   FormBox[
    SuperscriptBox["\[ScriptCapitalJ]", 
     RowBox[{"(", 
      RowBox[{"k", ",", " ", "h"}], ")"}]],
    TraditionalForm], 
   FormBox[
    SuperscriptBox["M", "n"],
    TraditionalForm]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"e359f46f-1ed2-4507-a73d-4b7fbda9fadf"],
 " of all smooth tensor fields of type {k, h} on the smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "1538e98e-7791-484b-9071-b64d7f88851d"],
 ", n>0, is an infinite dimensional vector space over the real  field \
\[DoubleStruckCapitalR], and a Module over the real algebra of smooth \
functions on ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["M", 
     RowBox[{"n", " "}]], " ", "with", " ", "real", "  ", "values", " "}], 
   TraditionalForm]],ExpressionUUID->"26c9c024-1b55-48af-9fbc-1d5e7778e2f8"],
 ". The vector and Module operations are defined as the pointwise algebraic \
tensor operations. In the same way tensor products and contractions  of \
tensor fields on a given manifold are declared as the pointwise operations \
for the tensors of the fields.   \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.70357545574275*^9, 3.703575545727446*^9}, 
   3.70357561595496*^9, {3.703575695742687*^9, 3.703575928550207*^9}, {
   3.7035759588142643`*^9, 3.703575961574135*^9}, {3.7035760096520643`*^9, 
   3.7035760113973827`*^9}},ExpressionUUID->"0e21e038-a23e-4e93-b159-\
c2aa018ef44d"],

Cell[TextData[{
 StyleBox["Example 3",
  FontColor->RGBColor[0, 0, 1]],
 ". Standard basis fields of a chart \[Phi]\[Iota]."
}], "Text",
 CellChangeTimes->{{3.703593888802691*^9, 
  3.703594029962685*^9}},ExpressionUUID->"e3f22b8e-c27f-46d2-bd65-\
39d53bf8df9f"],

Cell[BoxData[
 RowBox[{"?", "tx\[Iota]"}]], "Input",
 CellChangeTimes->{{3.6684908409826803`*^9, 
  3.668490862879627*^9}},ExpressionUUID->"a31d4010-8701-4614-947e-\
e76d7e99f71a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{
   RowBox[{"tx\[Iota]", "[", "2", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6684908409826803`*^9, 3.668490862879627*^9}, {
  3.703587524004006*^9, 
  3.703587577298687*^9}},ExpressionUUID->"a852b5ba-3a3c-49c8-a428-\
1b9d8bc83c80"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"b", "[", "i_", "]"}], "[", "x__", "]"}], ":=", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{
    RowBox[{"tx\[Iota]", "[", "i", "]"}], "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.703587720453114*^9, 3.7035877347471457`*^9}, {
  3.703587825536766*^9, 
  3.703587956184265*^9}},ExpressionUUID->"f3c15fd9-95f9-469c-be5d-\
9f613e84bd32"],

Cell[BoxData[
 RowBox[{"type", "[", 
  RowBox[{
   RowBox[{"b", "[", "3", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7035879718050117`*^9, 
  3.703588004221305*^9}},ExpressionUUID->"b4953cb5-c9ed-4884-ac8d-\
3fc5738c9c30"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"b", "[", "3", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.70358802737562*^9, 3.703588057797113*^9}, 
   3.703588122031959*^9},ExpressionUUID->"c57e9627-81cf-409b-9303-\
1083c065c6f2"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{
  3.7035940897323713`*^9},ExpressionUUID->"37506cdd-e4a9-48a0-bb54-\
59a5ea1ebf37"],

Cell[TextData[{
 StyleBox["Example 4",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[".  Contraction. ",
  FontColor->GrayLevel[0]],
 " Using the general method explained in Example 2 we define a tensor field \
of type (1,2):"
}], "Text",
 CellChangeTimes->{{3.634995546090033*^9, 3.6349956029299335`*^9}, {
   3.634995727180953*^9, 3.6349957778266425`*^9}, {3.6349958336277423`*^9, 
   3.6349958854268336`*^9}, {3.6349959454483423`*^9, 3.6349959475231457`*^9}, 
   3.634996091159999*^9, {3.634996147046298*^9, 3.634996226462438*^9}, {
   3.634996361060875*^9, 3.63499640359455*^9}, {3.634996497894117*^9, 
   3.6349965500862093`*^9}, {3.634996580524863*^9, 3.6349966317935534`*^9}, 
   3.635831422822425*^9, 3.636270738329423*^9, 3.6381875195250406`*^9, {
   3.703594156866819*^9, 3.70359429689042*^9}, {3.703595403766767*^9, 
   3.703595412174842*^9}, {3.70359545095945*^9, 
   3.703595464222548*^9}},ExpressionUUID->"22c9a429-a827-42d9-9c2c-\
af1b106c4908"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ar3", "[", "x__", "]"}], ":=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{
        RowBox[{"{", "i", "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "k"}], "}"}]}], "]"}], "[", "x", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"ar3", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.7035881733935328`*^9, 3.703588184474456*^9}, {
  3.703588263868827*^9, 3.703588382796569*^9}, {3.7035889756842327`*^9, 
  3.70358903673982*^9}, {3.7035890671556263`*^9, 3.703589129107843*^9}, {
  3.703589173411729*^9, 3.703589308691627*^9}, {3.7035893440275784`*^9, 
  3.7035893712429743`*^9}, {3.703589406691474*^9, 3.7035894912829523`*^9}, {
  3.703589563133646*^9, 3.703589623306663*^9}, {3.7035897507242613`*^9, 
  3.703589782714303*^9}},ExpressionUUID->"15f8bd5e-22dc-4315-b205-\
ed4bbfdfb1d5"],

Cell["The same tensor is obtained using tensorfield:", "Text",
 CellChangeTimes->{{3.703594404945833*^9, 
  3.703594429642062*^9}},ExpressionUUID->"e9590be3-aab7-476a-8bbc-\
3c450ab70182"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ta", "[", "x__", "]"}], ":=", 
  RowBox[{
   RowBox[{"tensorfield", "[", 
    RowBox[{"a", ",", "1", ",", "2"}], "]"}], "[", "x", "]"}]}]], "Input",
 CellChangeTimes->{{3.7035881733935328`*^9, 3.703588184474456*^9}, {
   3.703588263868827*^9, 3.703588382796569*^9}, {3.7035889756842327`*^9, 
   3.70358903673982*^9}, {3.7035890671556263`*^9, 3.703589129107843*^9}, {
   3.703589173411729*^9, 3.703589308691627*^9}, 
   3.7035893440275784`*^9},ExpressionUUID->"b27f6966-5700-4111-9134-\
ef65c3e6d441"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7035883988929853`*^9, 3.703588409428479*^9}, {
  3.703589676659709*^9, 3.703589701370088*^9}, {3.703589794690414*^9, 
  3.703589821409912*^9}},ExpressionUUID->"d6d9db29-d9fc-4843-9b1f-\
e8e4ddf5abc1"],

Cell[BoxData[
 RowBox[{"type", "[", 
  RowBox[{"ta", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.700570205494666*^9, 3.7005702293683653`*^9}, 
   3.7035884304531307`*^9},ExpressionUUID->"a96e6eb7-df1f-4eb3-806b-\
6416d6fe7a0d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ta", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703588457345142*^9, 3.703588470223714*^9}, {
  3.703588505796527*^9, 3.703588522228455*^9}, {3.703588567862296*^9, 
  3.703588589204358*^9}},ExpressionUUID->"09633979-3c87-436e-a290-\
2ea64df9b0fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Transpose", "[", 
   RowBox[{"coords", "[", 
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.634995078631005*^9, 3.6349950885390224`*^9}, {
   3.6349951213674803`*^9, 3.634995139808713*^9}, 3.640240274654448*^9, 
   3.700551228507822*^9, 3.700570510257449*^9, 3.7035945069943666`*^9, {
   3.703594539515272*^9, 3.7035945464979057`*^9}, {3.7035946100191803`*^9, 
   3.703594734553846*^9}},ExpressionUUID->"58037ba5-a6e0-4228-953f-\
5cfd23f1e7d6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorToArray", "[", 
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellChangeTimes->{{3.703594790776308*^9, 
  3.703594887412819*^9}},ExpressionUUID->"4ac80fcf-4761-43c8-912a-\
d4e05fcaa845"],

Cell["The contraction of tensors is done pointwise:", "Text",
 CellChangeTimes->{{3.635920355794163*^9, 3.635920377028801*^9}, 
   3.6381875299478593`*^9},ExpressionUUID->"4ea11926-820a-478c-ba05-\
7c0ddbc11c40"],

Cell[BoxData[
 RowBox[{"?", "contraction"}]], "Input",
 CellChangeTimes->{{3.635919515839879*^9, 
  3.635919517743082*^9}},ExpressionUUID->"301fa0e8-5abe-472c-955f-\
93a14dd5b61e"],

Cell[BoxData[
 RowBox[{"Context", "[", "contraction", "]"}]], "Input",
 CellChangeTimes->{{3.635919515839879*^9, 3.635919517743082*^9}, {
  3.7005498376684637`*^9, 
  3.700549855131826*^9}},ExpressionUUID->"98bdebf2-d8f5-4d5c-87e5-\
564cd8a156ea"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tc", "]"}], ";", 
  RowBox[{
   RowBox[{"tc", "[", "x__", "]"}], ":=", 
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{"ta", "[", "x", "]"}], ",", "1", ",", "1"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.634995078631005*^9, 3.6349950885390224`*^9}, {
   3.6349951213674803`*^9, 3.634995139808713*^9}, {3.63591945948158*^9, 
   3.635919494259041*^9}, {3.635919556390351*^9, 3.63591957865759*^9}, {
   3.6359196241084704`*^9, 3.635919693618593*^9}, 3.6359197873127584`*^9, {
   3.6359198195660157`*^9, 3.6359198584960847`*^9}, 3.6359199584302607`*^9, {
   3.635920033012393*^9, 3.6359200623464446`*^9}, 3.7005512562023087`*^9, 
   3.700551316690159*^9, {3.7005705530017433`*^9, 3.7005705793902607`*^9}, {
   3.703594967081422*^9, 
   3.703595001664322*^9}},ExpressionUUID->"544c5d1f-7604-428c-97a5-\
2c88c16bfb2c"],

Cell[BoxData[
 RowBox[{"type", "[", 
  RowBox[{"tc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.635919711078024*^9, 3.635919744983884*^9}, 
   3.6359199692118797`*^9, 3.6359200942077007`*^9, 3.6402402953868847`*^9, 
   3.7005512967256804`*^9, 
   3.7005705948972692`*^9},ExpressionUUID->"5a1e4630-4404-4b49-8cf9-\
590056dbaf54"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635919711078024*^9, 3.635919744983884*^9}, {
   3.635919888922138*^9, 3.635919911824979*^9}, {3.635919982754704*^9, 
   3.635919999263533*^9}, 3.6359201166133404`*^9, 3.6402403034208984`*^9, 
   3.700551355355246*^9, 
   3.700570622835648*^9},ExpressionUUID->"275b53fc-b6cf-48ae-bcbd-\
90bdfd8e96b6"],

Cell["\<\
Often it is convenient to omit the coordinate variables in the notation of \
the tensor fields, but they can mot be omitted at the coordinates.\
\>", "Text",
 CellChangeTimes->{{3.635920428141491*^9, 3.6359205030994244`*^9}, {
   3.635920639206501*^9, 3.6359206635225644`*^9}, 3.638187593943172*^9, {
   3.6536341352238083`*^9, 3.653634144031769*^9}, {3.7028113861282997`*^9, 
   3.702811388352128*^9}},ExpressionUUID->"fb2043b1-8df0-48fe-bbe8-\
942019cd9f07"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc"}], "]"}], ";", 
  RowBox[{"ta", "=", " ", 
   RowBox[{
    RowBox[{"tensorfield", "[", 
     RowBox[{"a", ",", "1", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.635920674903791*^9, 3.635920720022502*^9}, {
   3.6359207601854167`*^9, 3.6359207928529*^9}, 3.6402403113145123`*^9, 
   3.701578767722405*^9, {3.702811496152376*^9, 3.7028115018309402`*^9}, {
   3.703595174247707*^9, 
   3.703595212920786*^9}},ExpressionUUID->"5009f4c0-10c6-40b0-a4a4-\
15d766ee92df"],

Cell[BoxData[
 RowBox[{"type", "[", "ta", "]"}]], "Input",
 CellChangeTimes->{{3.635920812994152*^9, 
  3.6359208437190332`*^9}},ExpressionUUID->"b59fcdd8-1461-45b6-b02a-\
de12c85b763a"],

Cell[BoxData[
 RowBox[{"ta", "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635920864887882*^9, 
  3.6359208868855376`*^9}},ExpressionUUID->"d2491b7c-51b3-4586-8ac4-\
770cd9e9c3d7"],

Cell[BoxData[
 RowBox[{"tc", "=", 
  RowBox[{"contraction", "[", 
   RowBox[{"ta", ",", "1", ",", "2"}], "]"}]}]], "Input",ExpressionUUID->\
"7af8418d-7477-460d-bc84-e4e564027457"],

Cell[BoxData[
 RowBox[{"tc", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6359212173039937`*^9, 3.635921251689855*^9}, 
   3.700570713917625*^9, {3.703595582631185*^9, 3.703595585395194*^9}, 
   3.70359568971944*^9},ExpressionUUID->"7be178b9-1d06-4c5d-b6eb-\
dd614d8434ca"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Transpose", "[", 
   RowBox[{"coords", "[", "tc", "]"}], "]"}], "//", "TableForm"}]], "Input",
 CellChangeTimes->{{3.703595514319654*^9, 3.7035955481520443`*^9}, {
  3.703595698966004*^9, 3.703595717237948*^9}, {3.703595762142353*^9, 
  3.703595777445315*^9}},ExpressionUUID->"594ce1e2-1af2-4bef-b926-\
0b07b66d43de"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.635922245357214*^9, 
  3.635922245357214*^9}},ExpressionUUID->"7e9107a0-d0de-4807-b7de-\
8e3b4dbd2afc"],

Cell[TextData[{
 StyleBox["Example 5. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["Tensor product",
  FontColor->GrayLevel[0]],
 StyleBox[".",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.703596498047927*^9, 3.703596548498081*^9}, {
  3.7035965810180893`*^9, 
  3.703596606946061*^9}},ExpressionUUID->"13577411-c5f2-41b6-a1ae-\
ec065d4cf8e1"],

Cell[BoxData[
 RowBox[{"?", "tensorproduct"}]], "Input",
 CellChangeTimes->{{3.6359225545247602`*^9, 3.6359225712363896`*^9}, {
  3.6359226247358837`*^9, 3.6359226299004927`*^9}, {3.6359304182049294`*^9, 
  3.6359304289251485`*^9}},ExpressionUUID->"60011c43-5450-45c2-9a01-\
8824b449c730"],

Cell["\<\
The product of two factors can also be calculated using the infix operator\
\>", "Text",
 CellChangeTimes->{{3.6362779593331776`*^9, 
  3.6362780018744526`*^9}},ExpressionUUID->"dd4d82a4-efb0-4729-8f8e-\
c8e30d31333e"],

Cell[BoxData[
 RowBox[{"tb", "=", " ", 
  RowBox[{
   RowBox[{"tensorfield", "[", 
    RowBox[{"b", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6359223620986204`*^9, 3.6359223672934294`*^9}, 
   3.6402403424521675`*^9, {3.70359594111691*^9, 
   3.703595975478532*^9}},ExpressionUUID->"57bc9335-c016-44a9-92b1-\
dee0e65a6633"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tc", "]"}], ";", 
  RowBox[{"tc", " ", "=", " ", 
   RowBox[{"ta", "\[CircleTimes]", "tb"}]}]}]], "Input",
 CellChangeTimes->{{3.635923511488651*^9, 3.6359235292590823`*^9}, {
  3.6359303807618637`*^9, 3.6359303875644755`*^9}, {3.700572050072526*^9, 
  3.70057205914736*^9}},ExpressionUUID->"171289e8-e7c2-4387-a43a-\
a1d82b123ca1"],

Cell[BoxData[
 RowBox[{"tc", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635922636158104*^9, 
  3.6359226937468057`*^9}},ExpressionUUID->"e5ba6ea3-e4e0-4636-9cbc-\
bb7a59c8dab4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"ta", "\[CircleTimes]", "tb"}], ")"}], "\[CircleTimes]", 
  "tc"}]], "Input",
 CellChangeTimes->{{3.6359304946520643`*^9, 3.6359305039672804`*^9}, {
  3.635930555357771*^9, 3.63593058314542*^9}, {3.6359307625425367`*^9, 
  3.635930767846546*^9}, {3.6361204071196413`*^9, 3.636120410988448*^9}, {
  3.6361205498598924`*^9, 
  3.6361205585491076`*^9}},ExpressionUUID->"90df1b1e-f263-4eab-b244-\
cf7ae9ec031f"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6359305178825054`*^9, 3.6359305279289227`*^9}, 
   3.635930722306066*^9, 
   3.635930786426179*^9},ExpressionUUID->"1247936d-a29e-4635-9397-\
17ff9b1c91b3"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"ta", "\[CircleTimes]", "tb"}], ")"}], "\[CircleTimes]", "tc"}], 
   ",", 
   RowBox[{"tc", "\[CircleTimes]", "tc"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635930609760067*^9, 3.6359306630501614`*^9}, 
   3.703596093245521*^9},ExpressionUUID->"372a70eb-c593-4b9b-844d-\
3410aaf79db4"],

Cell["\<\
The program needed more than 1 hour to confirm this identity.\
\>", "Text",
 CellChangeTimes->{{3.702877347730657*^9, 
  3.702877437890127*^9}},ExpressionUUID->"9240e47d-4ba4-479b-8277-\
841bcf2522ec"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"ta", "\[CircleTimes]", "tb"}], ")"}], "\[CircleTimes]", "tc"}], 
   ")"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "2", ",", "3", ",", "3", ",", "2", ",", "1", ",", "2", ",", 
     "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.700572232754115*^9, 3.700572299441621*^9}, {
  3.700572337803557*^9, 
  3.700572342729197*^9}},ExpressionUUID->"a7dd1885-c556-488c-839b-\
4f32e6eaefb3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"tc", "\[CircleTimes]", "tc"}], ")"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "2", ",", "3", ",", "3", ",", "2", ",", "1", ",", "2", ",", 
     "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.700572232754115*^9, 3.700572299441621*^9}, {
  3.700572337803557*^9, 
  3.7005724210805264`*^9}},ExpressionUUID->"fea4cc2c-8459-43d4-864b-\
0501a3f0b294"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.7005724285664997`*^9, 
  3.700572430544273*^9}},ExpressionUUID->"437320f5-5f07-4dd6-b2eb-\
433eaa314409"],

Cell["\[GraySquare]", "Text",
 CellChangeTimes->{{3.703596254673617*^9, 
  3.7035962546758633`*^9}},ExpressionUUID->"2a1645c0-9d04-41ad-b457-\
0ad36044dc4b"],

Cell["The tensor product has the", "Text",
 CellChangeTimes->{{3.703596996297236*^9, 3.703597016153501*^9}, 
   3.7035971252170067`*^9},ExpressionUUID->"4b5c28da-aa01-4226-8a90-\
cc4fc0bce4bb"],

Cell[BoxData[
 RowBox[{"Attributes", "[", "tensorproduct", "]"}]], "Input",
 CellChangeTimes->{{3.703596916319993*^9, 
  3.70359694995404*^9}},ExpressionUUID->"fcb5dd19-501d-4941-8ed6-\
99f8972d1d7b"],

Cell["\<\
Thus the tensor product applied to tensor fields is associative too.\
\>", "Text",
 CellChangeTimes->{{3.7035970493038063`*^9, 
  3.703597111851756*^9}},ExpressionUUID->"552fbba7-7979-4583-a1c3-\
f59a2e6dd474"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ta", ",", "tb", ",", "tc"}], "]"}]], "Input",
 CellChangeTimes->{{3.7005732643555737`*^9, 3.700573291401701*^9}, 
   3.703597163121254*^9},ExpressionUUID->"b4f01a6d-64f6-4847-a1df-\
0a6a489e69b0"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3.2. Functions as Tensor Fields of Type {0, 0}", "Subsubsection",
 CellChangeTimes->{{3.634896600491453*^9, 3.634896648742338*^9}, 
   3.634996752964367*^9, {3.6349993842044353`*^9, 3.6349993892900443`*^9}, 
   3.634999514022665*^9, 3.6381902390968313`*^9, 3.6389433787586246`*^9, 
   3.645088893863084*^9, 3.6452516261993256`*^9, 3.6684938136879473`*^9, 
   3.668495829182453*^9, 3.700573321898506*^9, 3.7006359233066807`*^9, 
   3.702878223956667*^9, 3.702883728494733*^9, {3.703597275096395*^9, 
   3.703597276840066*^9}, 
   3.7037360110333157`*^9},ExpressionUUID->"30ea83d1-6b9b-46ca-bace-\
e28b6c7e1865"],

Cell["\<\
We consider smooth functions on the manifold with real or complex numbers as \
values; functions whose values are Lists, e.g. vectors, \
aren\[CloseCurlyQuote]t allowed to be tensor coordinates: \
\>", "Text",
 CellChangeTimes->{{3.6349973806912746`*^9, 3.6349975243573284`*^9}, 
   3.638190575459631*^9, 
   3.6389421897713366`*^9},ExpressionUUID->"3f27dc33-1e6b-407b-9215-\
2cb4b162218e"],

Cell[BoxData[
 RowBox[{"?", "tensor"}]], "Input",
 CellChangeTimes->{{3.638269949279179*^9, 
  3.638269953990388*^9}},ExpressionUUID->"8ba4d4be-7415-4ab0-83eb-\
583d39c78446"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". Incorrect coordinates."
}], "Text",
 CellChangeTimes->{{3.7006333026641407`*^9, 3.700633313687888*^9}, {
  3.7006333809594793`*^9, 
  3.700633397575549*^9}},ExpressionUUID->"bfe078c0-2d7f-4c04-9f7d-\
14f749b0771b"],

Cell[BoxData[
 RowBox[{"?", "unit"}]], "Input",
 CellChangeTimes->{{3.6347965237602224`*^9, 3.634796527551029*^9}, 
   3.634796570505905*^9, {3.634796607910775*^9, 
   3.63479661076758*^9}},ExpressionUUID->"1c6bf720-9038-4dfb-98d0-\
700a4bb3155a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"hsph", "[", 
   RowBox[{"r_", ",", 
    RowBox[{"n_:", "dim"}]}], "]"}], ":=", " ", 
  RowBox[{"tensor", "[", 
   RowBox[{
    RowBox[{"r", "*", 
     RowBox[{"unit", "[", 
      RowBox[{"vec", "[", 
       RowBox[{"x", ",", "n"}], "]"}], "]"}]}], ",", "0", ",", "0", ",", 
    "n"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6347964665615215`*^9, 3.6347964855653553`*^9}, {
   3.6347966592750654`*^9, 3.634796746177219*^9}, {3.6347967864312897`*^9, 
   3.634796885675065*^9}, 3.634797014635293*^9, 
   3.6684951665640287`*^9},ExpressionUUID->"9f1fde78-9d33-4891-97d4-\
d75231edf67c"],

Cell[BoxData[
 RowBox[{"hsph", "[", "r", "]"}]], "Input",
 CellChangeTimes->{
  3.638270393131159*^9},ExpressionUUID->"bfb46763-6664-492a-a3e8-\
400e21072512"],

Cell["The evaluation yields the error message", "Text",
 CellChangeTimes->{{3.634997566095402*^9, 3.6349975702450094`*^9}, {
  3.6389422377414207`*^9, 
  3.6389422520466456`*^9}},ExpressionUUID->"fe2f7234-0018-4c50-9f30-\
01566df9b3c9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"hsph", "[", "r", "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6382714801958685`*^9, 
  3.63827151537393*^9}},ExpressionUUID->"5247c221-a1ec-4bb3-89f2-\
74a0be8ca142"],

Cell[BoxData[
 RowBox[{"coords", "[", 
  RowBox[{"hsph", "[", "r", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6347968985646877`*^9, 
  3.6347969387739587`*^9}},ExpressionUUID->"c85b4acc-0a96-427c-9906-\
804c57594e7a"],

Cell["The tensor coordinates can\[CloseCurlyQuote]t be Lists!", "Text",
 CellChangeTimes->{{3.634877440584118*^9, 
  3.6348774774937825`*^9}},ExpressionUUID->"d2685cbf-7ad7-4670-a837-\
262f29bae5a4"],

Cell[BoxData[
 RowBox[{"Clear", "[", "hsph", "]"}]], "Input",
 CellChangeTimes->{{3.638538259415346*^9, 
  3.6385382891801987`*^9}},ExpressionUUID->"da380492-ba0a-435b-9678-\
1292a44e3da2"],

Cell["\<\
\[FilledSquare]
Tensors may be defined with unspecified coordinates:\
\>", "Text",
 CellChangeTimes->{{3.6348972960810747`*^9, 3.6348973055970917`*^9}, {
  3.6348974324797473`*^9, 3.634897450294978*^9}, {3.700633472319556*^9, 
  3.700633479905422*^9}},ExpressionUUID->"596e031e-79ac-4ac0-8ab1-\
1751e9080222"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a", ",", "ff", ",", "tensf"}], "]"}], ";", 
  RowBox[{"tensf", ":=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{"ff", ",", "0", ",", "0"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6347964665615215`*^9, 3.6347964855653553`*^9}, {
   3.6347966592750654`*^9, 3.634796746177219*^9}, {3.6347967864312897`*^9, 
   3.634796885675065*^9}, 3.634797014635293*^9, {3.6348775121570435`*^9, 
   3.634877656535297*^9}, 3.634877791381934*^9, {3.634877878523687*^9, 
   3.634877878710887*^9}, 3.634878117687707*^9, {3.6348782755911846`*^9, 
   3.63487830739964*^9}, {3.634896697211623*^9, 3.63489671287405*^9}, {
   3.6349655676467786`*^9, 3.6349655781299973`*^9}, {3.6382724879576387`*^9, 
   3.638272512870882*^9}, {3.63827258558261*^9, 3.638272627780684*^9}, 
   3.638272763859723*^9},ExpressionUUID->"4de4c56c-6b3b-4fa5-a13a-\
c9e217248659"],

Cell[BoxData[
 RowBox[{"type", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.70063351742955*^9, 
  3.700633526934476*^9}},ExpressionUUID->"e1c5b9ac-938d-4ae3-bf68-\
321a8294e5b2"],

Cell["The third part 3 is the default dimension on the manifold.", "Text",
 CellChangeTimes->{{3.700633546670046*^9, 
  3.7006335934539948`*^9}},ExpressionUUID->"0c5addfe-836f-428e-a39a-\
bdfaa0e56f23"],

Cell[BoxData[
 RowBox[{"tensf", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6348780717144265`*^9, 
  3.634878097704072*^9}},ExpressionUUID->"f058e315-83aa-475a-b9f5-\
a143a6d7ecd5"],

Cell[BoxData[
 RowBox[{"coords", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.634877665942114*^9, 
  3.634877675894931*^9}},ExpressionUUID->"8717f4c5-11fd-4af2-85d0-\
1e12fee86a91"],

Cell["The single coordinate ff can be a function:", "Text",
 CellChangeTimes->{{3.6349977790193377`*^9, 3.634997813920599*^9}, {
  3.638272790270569*^9, 
  3.6382728065569983`*^9}},ExpressionUUID->"8462682c-a212-48d3-bae9-\
8c425cd95838"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a", ",", "ff", ",", "tensf"}], "]"}], ";", 
  RowBox[{"tensf", ":=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"ff", "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}], ",", 
       RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", "0", ",", "0"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6347964665615215`*^9, 3.6347964855653553`*^9}, {
   3.6347966592750654`*^9, 3.634796746177219*^9}, {3.6347967864312897`*^9, 
   3.634796885675065*^9}, 3.634797014635293*^9, {3.6348775121570435`*^9, 
   3.634877656535297*^9}, 3.634877791381934*^9, {3.634877878523687*^9, 
   3.634877878710887*^9}, 3.634878117687707*^9, {3.6348782755911846`*^9, 
   3.63487830739964*^9}, {3.634896697211623*^9, 3.63489671287405*^9}, {
   3.6349655676467786`*^9, 3.6349655781299973`*^9}, {3.6382724879576387`*^9, 
   3.638272512870882*^9}, {3.63827258558261*^9, 3.638272627780684*^9}, 
   3.638272763859723*^9, 3.638272881156329*^9, {3.7006314169117517`*^9, 
   3.700631422761969*^9}},ExpressionUUID->"ee2e791e-35ba-4a77-9462-\
7bec5b87b412"],

Cell[BoxData[
 RowBox[{"type", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.6348032177260623`*^9, 3.63480322778808*^9}, {
  3.634803273624961*^9, 3.634803308354622*^9}, {3.6348946257758117`*^9, 
  3.634894628833417*^9}},ExpressionUUID->"fe7e5504-0aaf-4f64-85aa-\
fc7401e5d830"],

Cell["The only coordinate is the function itself: ", "Text",
 CellChangeTimes->{{3.700633695085013*^9, 
  3.700633727780642*^9}},ExpressionUUID->"ef9da763-626c-4cf7-b675-\
f032f8186fa2"],

Cell[BoxData[
 RowBox[{"tensf", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6348780717144265`*^9, 3.634878097704072*^9}, {
  3.700631441167186*^9, 
  3.700631451683139*^9}},ExpressionUUID->"d3d077d1-64e9-498e-8278-\
8db962a811c6"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.668495395873567*^9, 
  3.6684954058259907`*^9}},ExpressionUUID->"f36f7046-eeaa-4252-a00f-\
a53d5e277ca5"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{"nulltensor", "[", 
   RowBox[{"0", ",", "0"}], "]"}], "]"}]], "Input",ExpressionUUID->"f7bce746-\
9b6f-468e-b307-8ad8cc2cc1da"],

Cell["\<\
Mathematica has a built - in function TensorRank which may be applied to \
Arrays:\
\>", "Text",
 CellChangeTimes->{{3.7006338660119658`*^9, 3.7006339017585917`*^9}, {
  3.700633949234509*^9, 
  3.700634009578277*^9}},ExpressionUUID->"a3d21d76-94a2-47df-8b72-\
5422e9c5eb6a"],

Cell[BoxData[
 RowBox[{"?", "TensorRank"}]], "Input",
 CellChangeTimes->{{3.668495438315495*^9, 
  3.668495445977446*^9}},ExpressionUUID->"f47eaad9-29d3-458c-a294-\
0e9fa58ebec1"],

Cell[TextData[{
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". Null tensor of type {0,0}"
}], "Text",
 CellChangeTimes->{{3.700634125041081*^9, 3.700634186048765*^9}, 
   3.7036532339601517`*^9},ExpressionUUID->"b2ef2566-243d-4a1f-bea9-\
f36d5fddf120"],

Cell[BoxData[
 RowBox[{"tn0", "=", 
  RowBox[{"nulltensor", "[", 
   RowBox[{"0", ",", "0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7006341992224483`*^9, 
  3.70063422898453*^9}},ExpressionUUID->"9a1ea5e6-74fa-4431-84ef-\
8586a7b85f8c"],

Cell["The function here is  zero", "Text",
 CellChangeTimes->{{3.7006342575518703`*^9, 
  3.7006343356397142`*^9}},ExpressionUUID->"3c08ad06-c3f6-4905-bac7-\
a2387c7eaca5"],

Cell[BoxData[
 RowBox[{"?", "zero"}]], "Input",
 CellChangeTimes->{{3.700634337926447*^9, 
  3.700634337928936*^9}},ExpressionUUID->"5eab7dda-76a8-42a5-a7ee-\
24366e2477a3"],

Cell[BoxData[
 RowBox[{"?", "nulltensor"}]], "Input",
 CellChangeTimes->{{3.700634351187286*^9, 
  3.700634356511649*^9}},ExpressionUUID->"475bc898-03c5-4b14-ae15-\
d8652874048b"],

Cell[BoxData[
 RowBox[{"type", "[", "tn0", "]"}]], "Input",
 CellChangeTimes->{{3.700634374180593*^9, 
  3.700634394423161*^9}},ExpressionUUID->"ec0fff87-28cf-472c-a6e5-\
003e56d0e8fe"],

Cell[BoxData[
 RowBox[{"coords", "[", "tn0", "]"}]], "Input",
 CellChangeTimes->{{3.638538989561429*^9, 3.638538993149435*^9}, {
  3.700634411770506*^9, 
  3.700634414335018*^9}},ExpressionUUID->"b1995bab-81d6-432b-8485-\
faf817142528"],

Cell[BoxData[
 RowBox[{"TensorRank", "[", 
  RowBox[{"tensorToArray", "[", "tn0", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6385385826011143`*^9, 3.638538655593642*^9}, {
  3.700634433820581*^9, 
  3.700634456558557*^9}},ExpressionUUID->"568fd2e4-77b4-488a-99ac-\
1a237f8199b0"],

Cell["Only the MatrixRank of the coordinates matrix is zero:", "Text",
 CellChangeTimes->{{3.6385390607599545`*^9, 3.6385391269820704`*^9}, 
   3.6385395247359695`*^9},ExpressionUUID->"c8cf9ab6-00c4-40ed-a720-\
2ac92992330c"],

Cell[BoxData[
 RowBox[{"MatrixRank", "[", "%%", "]"}]], "Input",
 CellChangeTimes->{{3.6385387197957554`*^9, 3.6385387776350565`*^9}, 
   3.6385390051926565`*^9, 3.638539040089918*^9, 
   3.7006344728184633`*^9},ExpressionUUID->"ed099c49-4fe4-4a5e-b843-\
221f567c7d30"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.700634520323393*^9, 
  3.700634520325268*^9}},ExpressionUUID->"7bb33fa7-b4e9-4dbe-a7a3-\
0411a8b28330"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.6347970549849644`*^9, 3.6347970752358*^9}, {
   3.6347971373983097`*^9, 3.634797172631172*^9}, 
   3.6348946570070667`*^9},ExpressionUUID->"e954fa52-09d8-4743-9beb-\
3a2d11c32ecf"],

Cell[BoxData[
 RowBox[{"TensorRank", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6382730449098167`*^9, 
  3.6382730662818537`*^9}},ExpressionUUID->"8a96c9de-1213-4fa6-a477-\
bf45a463ee29"],

Cell["\<\
On the other hand, any rectangular Array may be transformed into a tensor in \
several ways:\
\>", "Text",
 CellChangeTimes->{{3.6348996006464868`*^9, 
  3.634899657358587*^9}},ExpressionUUID->"e62402db-7be2-4370-a4f5-\
0c0c21d13b4a"],

Cell[BoxData[
 RowBox[{"?", "ArrayTotensor"}]], "Input",
 CellChangeTimes->{{3.6348980591169686`*^9, 
  3.634898060645771*^9}},ExpressionUUID->"112baf00-a63a-4d9d-839f-\
e7af4f7bf196"],

Cell["\<\
The dimension of this tensor is read from the dimension d of the underlying \
vector space; in our case of a fuction d is set to 1.  The default value of \
the covariance is zero:\
\>", "Text",
 CellChangeTimes->{{3.700634833162921*^9, 3.700634862819068*^9}, {
  3.700634893274626*^9, 3.7006349859938173`*^9}, {3.7006350415456667`*^9, 
  3.700635128512961*^9}},ExpressionUUID->"c63c3d7a-09fd-4192-b7c1-\
065b8f807cdb"],

Cell[BoxData[
 RowBox[{"tensff", " ", "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{"{", 
    RowBox[{"ff", "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6347972627207313`*^9, 3.6347972842527695`*^9}, {
   3.634797421861615*^9, 3.634797425496422*^9}, 3.634802734471009*^9, {
   3.6348030285871277`*^9, 3.6348030313639326`*^9}, {3.6348947006403427`*^9, 
   3.634894705850752*^9}, {3.6348977699772587`*^9, 3.634897778386673*^9}, {
   3.634966084335286*^9, 3.634966088422493*^9}, 
   3.638273118573146*^9},ExpressionUUID->"d86dcb7b-4e68-49fa-872d-\
461827b0d582"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.700635146721088*^9, 
  3.70063515852059*^9}},ExpressionUUID->"3a3f7003-52b3-449d-9fc7-\
904614e2e388"],

Cell[BoxData[
 RowBox[{"coords", "[", "%%", "]"}]], "Input",
 CellChangeTimes->{{3.6348030984206524`*^9, 3.6348031064254665`*^9}, 
   3.700635189292905*^9},ExpressionUUID->"3cba6f9e-0f5f-4bc4-90b7-\
4b9141d9b312"],

Cell[BoxData[
 RowBox[{"type", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.634966174394244*^9, 
  3.6349661954854813`*^9}},ExpressionUUID->"5c80a848-0a2e-4006-bd4f-\
602db235db5f"],

Cell["\<\
The type of tensf yields  as default value the dimension d = dim =3. This isn\
\[CloseCurlyQuote]t reproduced of course: The Array contains one element \
only, thus d is set equal 1. Indeed, since the value of the function is a \
scalar, the dimension of the vector space is inessential.\
\>", "Text",
 CellChangeTimes->{{3.634965856980487*^9, 3.634965975977496*^9}, {
   3.6349660263967843`*^9, 3.6349660651316524`*^9}, 3.634966164800227*^9, {
   3.6349979224699907`*^9, 3.634998018709961*^9}, {3.634998054346424*^9, 
   3.6349980599040337`*^9}, {3.634998145949113*^9, 
   3.634998179039771*^9}},ExpressionUUID->"5a7a946e-f919-4cfe-8977-\
c967dbbb55b9"],

Cell[BoxData[
 RowBox[{"?", "equi"}]], "Input",
 CellChangeTimes->{{3.634966210196307*^9, 
  3.634966212239911*^9}},ExpressionUUID->"406a219d-90ec-4d08-b92d-\
f2046763d080"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"tensf", ",", "tensff"}], "]"}]], "Input",
 CellChangeTimes->{{3.6349662433931656`*^9, 
  3.634966258369192*^9}},ExpressionUUID->"61fb1dd0-be63-4134-bf2e-\
b2782e4833c2"],

Cell["equi returned False since the types of the tensors differ.", "Text",
 CellChangeTimes->{{3.700635357313113*^9, 
  3.7006354130302343`*^9}},ExpressionUUID->"9653728d-3b47-4679-a49f-\
f730ac76b1bf"],

Cell[BoxData[
 RowBox[{"TensorRank", "[", 
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{"ff", "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], "}"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6385398808221946`*^9, 
  3.6385399127710505`*^9}},ExpressionUUID->"d6e9e28b-e2e1-4b67-b259-\
541f95660446"],

Cell["\<\
The symbol ff can be specified after or before the definition of the tensor:\
\>", "Text",
 CellChangeTimes->{{3.6349666960903606`*^9, 3.6349667435144444`*^9}, 
   3.634966901308721*^9, {3.6349983721008444`*^9, 
   3.6349983732084465`*^9}},ExpressionUUID->"87457212-effe-4e30-9e71-\
9c53b847b3e3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ff", "]"}], ";", 
  RowBox[{
   RowBox[{"ff", "[", "x__", "]"}], ":=", " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"Norm", "[", 
     RowBox[{"{", "x", "}"}], "]"}], "]"}]}], ";", 
  RowBox[{"tensf", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.634802789796706*^9, 3.6348028178183556`*^9}, {
  3.6348038718100157`*^9, 3.634803904014076*^9}, {3.6348039929100327`*^9, 
  3.634804049666933*^9}, {3.634896821450241*^9, 3.63489691746841*^9}, {
  3.638943035901222*^9, 3.638943066056075*^9}, {3.6389431575502357`*^9, 
  3.6389431610446424`*^9}},ExpressionUUID->"75a78d40-52c8-43fe-87ab-\
ece11d2aca9d"],

Cell[BoxData[
 RowBox[{"coords", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.6348029595782056`*^9, 3.634802966896619*^9}, 
   3.634896967076497*^9, 
   3.634897004828563*^9},ExpressionUUID->"a80da443-6007-45a1-b965-\
88443bc7424b"],

Cell[BoxData[
 RowBox[{"tensff", "=", 
  RowBox[{"ArrayTotensor", "[", "%", "]"}]}]], "Input",
 CellChangeTimes->{{3.6348040756897783`*^9, 3.634804088437001*^9}, {
  3.700635613593176*^9, 
  3.7006356176690493`*^9}},ExpressionUUID->"2bb34560-24cd-44d3-88ab-\
1f15afc63832"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.634804115630849*^9, 
  3.634804126441668*^9}},ExpressionUUID->"2cb77d44-d05e-4b8e-894f-\
b9a66bcab571"],

Cell[BoxData[
 RowBox[{"type", "[", "tensf", "]"}]], "Input",
 CellChangeTimes->{{3.6348041618567305`*^9, 3.634804171203147*^9}, 
   3.634897040537026*^9},ExpressionUUID->"d9fd8f9e-a860-4bb4-acfd-\
fe66a815130b"],

Cell[BoxData[
 RowBox[{"type", "[", "tensff", "]"}]], "Input",
 CellChangeTimes->{{3.6348041618567305`*^9, 3.634804171203147*^9}, 
   3.634897040537026*^9, {3.700635674702298*^9, 
   3.7006356927741213`*^9}},ExpressionUUID->"0654f70d-9db9-4790-b20c-\
73c446c91351"],

Cell[BoxData[
 RowBox[{"tensff", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.700635709164898*^9, 3.700635737267782*^9}, 
   3.7006357917762136`*^9},ExpressionUUID->"51d72c4b-b809-4cae-89cf-\
1082cdb4d60b"],

Cell[BoxData[
 RowBox[{"coords", "[", "tensff", "]"}]], "Input",
 CellChangeTimes->{{3.6349668793750825`*^9, 3.634966885817894*^9}, {
  3.7006358299836817`*^9, 
  3.7006358322829514`*^9}},ExpressionUUID->"4ce908a8-ab29-4869-8dd8-\
5531f8e48c39"],

Cell[BoxData[
 RowBox[{"?", "tensorfield"}]], "Input",
 CellChangeTimes->{{3.7036536573015747`*^9, 
  3.7036536652410793`*^9}},ExpressionUUID->"feaa0509-9920-4b3e-9334-\
1d2de0a23fb5"],

Cell[BoxData[
 RowBox[{"ff", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7036538790154877`*^9, 
  3.703653887962551*^9}},ExpressionUUID->"fe590d4e-3bab-44a7-ba98-\
e516ebdd799e"],

Cell["\<\
The simplest way to define the tensor field corresponding to a scalar \
function is\
\>", "Text",
 CellChangeTimes->{{3.703654967800749*^9, 
  3.703655014176791*^9}},ExpressionUUID->"9a21e201-2fe0-4ea2-8bdd-\
5d0bb2b8c4cb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorfield", "[", 
   RowBox[{"ff", ",", "0", ",", "0"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.703653720491585*^9, 
  3.703653755883616*^9}},ExpressionUUID->"37bc547c-680f-4db6-8272-\
391e008845f0"],

Cell["The only coordinate is the function:", "Text",
 CellChangeTimes->{{3.7036550705200357`*^9, 
  3.703655097352107*^9}},ExpressionUUID->"0f7c00c5-2998-4d04-86d9-\
dc7b7ac3432a"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703653776364952*^9, 
  3.70365380240343*^9}},ExpressionUUID->"47a4e1f6-c798-4616-bc63-\
8bfd5fe6d338"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ff", ",", " ", "tensf", ",", "tensff"}], "]"}]], "Input",
 CellChangeTimes->{{3.6349669363619823`*^9, 3.634966973068847*^9}, {
  3.638943336872551*^9, 3.6389433405853577`*^9}, {3.700635890134635*^9, 
  3.700635906418503*^9}},ExpressionUUID->"f754692e-4e6c-4166-af21-\
1b7c0f635b91"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3.3. Vector Fields", "Subsubsection",
 CellChangeTimes->{{3.635140125815361*^9, 3.635140155678814*^9}, {
   3.638943483247608*^9, 3.6389434859932127`*^9}, 3.6393158311341515`*^9, 
   3.6452516346701403`*^9, 3.645275707648147*^9, 3.6684958363264008`*^9, 
   3.668582114252289*^9, 3.700635929147588*^9, 3.700801677701576*^9, 
   3.703736017857955*^9, 
   3.703741942139901*^9},ExpressionUUID->"cca10944-0ff7-4d52-af43-\
fd3f72025de0"],

Cell[TextData[{
 "A vector field X is a smooth section of the tangent bundle ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["TM", "n"], TraditionalForm]],ExpressionUUID->
  "9c2a5e78-c8dc-4a5f-9550-a461061a257c"],
 " := ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[Union]", 
     RowBox[{"x", "\[Element]", 
      SuperscriptBox["M", "n"]}]]}], TraditionalForm]],ExpressionUUID->
  "a316466d-1c6e-4b35-8c7d-72681270d6d4"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "e6226876-0c40-4395-a286-1621bc856a9c"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "802b6c48-1c32-4d35-9f9a-f749a5775c13"],
 " which can be identified canonically with the tensor bundle  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["T", 
     RowBox[{"(", 
      RowBox[{"0", ",", "1"}], ")"}]], 
    SuperscriptBox["M", "n"]}], TraditionalForm]],ExpressionUUID->
  "27107f98-4738-43e8-ad39-9753546289ae"],
 " of 1-fold contravariant tensors. Its coordinate functions can be given by"
}], "Text",
 CellChangeTimes->{{3.63514024316197*^9, 3.635140290783254*^9}, {
   3.635140322784911*^9, 3.6351405690343475`*^9}, {3.635140600975604*^9, 
   3.635140649125289*^9}, {3.635140965841649*^9, 3.6351409885144897`*^9}, 
   3.6359188212284513`*^9, {3.6389482271998234`*^9, 3.6389482482598605`*^9}, 
   3.6392807293698945`*^9, {3.703661015705823*^9, 
   3.7036610188573647`*^9}},ExpressionUUID->"af53f264-8cb6-4ec9-81b5-\
4dc372e56d54"],

Cell[BoxData[
 RowBox[{"?", "vv"}]], "Input",
 CellChangeTimes->{{3.7036756153184443`*^9, 
  3.703675616758512*^9}},ExpressionUUID->"15a20442-5a70-4daa-a8fa-\
3ae3cf37f514"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "a", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.70367565030145*^9, 
  3.703675671478058*^9}},ExpressionUUID->"9f054607-1f8e-4240-ba86-\
f93203ab898b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"fx", ",", "vx"}], "]"}], ";", 
  RowBox[{
   RowBox[{"vx", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{
    RowBox[{"vv", "[", "fx", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.635140808955372*^9, 3.6351408641270695`*^9}, 
   3.6392908590234723`*^9, {3.703661058058653*^9, 3.703661064232641*^9}, {
   3.703737605509315*^9, 3.703737651804837*^9}, {3.703737703199418*^9, 
   3.703737759531909*^9}},ExpressionUUID->"8d5ecbd8-7453-46d7-8982-\
9c8f29b93cd2"],

Cell["The corresponding tensor field is", "Text",
 CellChangeTimes->{{3.635142419002819*^9, 3.6351424414572587`*^9}, {
  3.6351424753621187`*^9, 3.6351424769533215`*^9}, {3.6354094362895727`*^9, 
  3.635409449346795*^9}},ExpressionUUID->"200c5a41-db1c-4d7a-833f-\
4cdc80012192"],

Cell[BoxData[
 RowBox[{"vX", " ", "=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vx", "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6351410773396487`*^9, 3.635141135678152*^9}, 
   3.6351412024738703`*^9, {3.6351416932033367`*^9, 3.63514173436121*^9}, 
   3.6389438439826417`*^9, {3.6392908870763216`*^9, 3.639290910307763*^9}, 
   3.703737876197*^9},ExpressionUUID->"5eb6f2bc-41ec-43ea-a97b-15a4503b5523"],

Cell[BoxData[
 RowBox[{"vX", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.700800625211532*^9, 
  3.700800641165289*^9}},ExpressionUUID->"13571538-53ad-4671-a4a8-\
b66ae391e66b"],

Cell["all coordinates of the tensor can be seen by", "Text",
 CellChangeTimes->{{3.7037379483758583`*^9, 
  3.703738010649337*^9}},ExpressionUUID->"3225d699-1c40-4de4-8384-\
0c678353402e"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "vX", "]"}]], "Input",
 CellChangeTimes->{{3.6351412165450945`*^9, 3.635141228953117*^9}, 
   3.638943847679848*^9, 3.639290930353798*^9, {3.668580638008459*^9, 
   3.668580646384028*^9}},ExpressionUUID->"d6aabe8c-2e5a-4df9-bc2e-\
7dfdcbc90363"],

Cell["or", "Text",
 CellChangeTimes->{{3.703738029713112*^9, 
  3.703738030112952*^9}},ExpressionUUID->"d93e486c-d845-4889-9abc-\
da7f1ebdc3ed"],

Cell[BoxData[
 RowBox[{"coords", "[", "vX", "]"}]], "Input",
 CellChangeTimes->{{3.703738036747223*^9, 
  3.703738054089216*^9}},ExpressionUUID->"4415a86d-3d14-41e9-b6c3-\
441422cf43d0"],

Cell["The same tensor can be defined by", "Text",
 CellChangeTimes->{{3.703738109040461*^9, 
  3.703738174584819*^9}},ExpressionUUID->"07fd3d0f-0055-4a8e-a666-\
90ac1857c9a0"],

Cell[BoxData[
 RowBox[{"tfX", ":=", 
  RowBox[{
   RowBox[{"tensorfield", "[", 
    RowBox[{"fx", ",", "0", ",", "1"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.703738179243479*^9, 3.703738256687883*^9}, {
   3.703738630709961*^9, 3.703738662972727*^9}, 
   3.703738745269305*^9},ExpressionUUID->"a0d2a35e-e2cc-4524-af97-\
d0ff0ecb3803"],

Cell["The coordinates differ formally:", "Text",
 CellChangeTimes->{{3.703738368269128*^9, 
  3.703738410302402*^9}},ExpressionUUID->"184c5d99-9f31-4898-8c76-\
7a138883bda9"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"tfX", ",", "vX"}], "]"}]], "Input",
 CellChangeTimes->{{3.70373826590341*^9, 
  3.703738302281519*^9}},ExpressionUUID->"ba6c5ca4-db96-4a40-b59e-\
7e197f656aac"],

Cell["The same tensor is obtained by", "Text",
 CellChangeTimes->{{3.70373978722747*^9, 
  3.703739802211165*^9}},ExpressionUUID->"807e8db8-d458-4fe0-b00d-\
15aa83e95c62"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tfX", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{
      RowBox[{"{", "}"}], ",", 
      RowBox[{"{", "i_", "}"}]}], "]"}], "[", "x__", "]"}], ":=", 
   RowBox[{
    RowBox[{"fx", "[", "i", "]"}], "[", "x", "]"}]}], ";", 
  RowBox[{"tfX", " ", ":=", 
   RowBox[{
    RowBox[{"tensorfield", "[", 
     RowBox[{"f", ",", "0", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.7037384361902447`*^9, 3.703738456670104*^9}, {
   3.703738506709825*^9, 3.703738557221641*^9}, {3.703738760300885*^9, 
   3.703738777285418*^9}, {3.7037388201316557`*^9, 3.7037388443231*^9}, {
   3.7037389092270107`*^9, 3.703739070513373*^9}, 3.703739158817721*^9, 
   3.703739364647852*^9},ExpressionUUID->"5c014661-36a1-4f45-84ec-\
b23e12e9aeb4"],

Cell[BoxData[
 RowBox[{"coords", "[", "tfX", "]"}]], "Input",
 CellChangeTimes->{{3.7037385771088448`*^9, 3.703738616493093*^9}, 
   3.703739175809834*^9},ExpressionUUID->"20e82aae-3628-4e19-a4fd-\
2bed51d311b7"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"vX", ",", "tfX"}], "]"}]], "Input",
 CellChangeTimes->{{3.703739093620627*^9, 3.7037391170729027`*^9}, {
  3.703739191296482*^9, 
  3.703739196960112*^9}},ExpressionUUID->"d652a3f8-8b09-4e64-8265-\
ddea12f070a2"],

Cell["Another vector  field :", "Text",
 CellChangeTimes->{{3.6351424898877444`*^9, 
  3.635142521920601*^9}},ExpressionUUID->"a17d9ddd-c21b-42a4-8f36-\
acfe31df0c0e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "Y", "]"}], ";", 
  RowBox[{"vY", " ", ":=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"vv", "[", "fy", "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}]}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6351421819526005`*^9, 3.6351422117234535`*^9}, {
   3.6351422461119137`*^9, 3.635142278238371*^9}, {3.6392808622041283`*^9, 
   3.6392808747309504`*^9}, {3.639280930657048*^9, 3.6392809357270575`*^9}, 
   3.639290952618037*^9, {3.639291036692585*^9, 3.639291037675387*^9}, {
   3.703675519232325*^9, 3.703675526016181*^9}, {3.703739971882359*^9, 
   3.703740012833479*^9}},ExpressionUUID->"2f29c827-38dc-4dcb-bc1b-\
065216a736e2"],

Cell[BoxData[
 RowBox[{"coords", "[", "vY", "]"}]], "Input",
 CellChangeTimes->{{3.6351423082432237`*^9, 3.6351423158872375`*^9}, {
   3.6392808898317766`*^9, 3.639280890596178*^9}, 
   3.6392910482210054`*^9},ExpressionUUID->"2dc44828-feeb-4c4d-8b5e-\
c3132b752abc"],

Cell["Vector fields may be added", "Text",
 CellChangeTimes->{{3.6351425529072556`*^9, 
  3.6351425656096783`*^9}},ExpressionUUID->"619212b3-4044-4b6b-8b1a-\
3d26593acf3f"],

Cell[BoxData[
 RowBox[{"vX", "+", "vY"}]], "Input",
 CellChangeTimes->{{3.6351423306800632`*^9, 3.6351423342076693`*^9}, 
   3.6389438626714745`*^9, {3.6392809035286007`*^9, 3.6392809042462015`*^9}, {
   3.639291058255823*^9, 
   3.639291063622233*^9}},ExpressionUUID->"447eef19-4f4f-4209-a363-\
fde47665bc06"],

Cell["The coordinate functions are added pointwise", "Text",
 CellChangeTimes->{{3.6351427941654854`*^9, 
  3.6351428143071213`*^9}},ExpressionUUID->"57543813-4634-49a5-8e21-\
7d9cd6099674"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6351423547997055`*^9, 
  3.635142362228319*^9}},ExpressionUUID->"6d2d987e-93be-42ee-913e-\
0d440041f67b"],

Cell["\<\
Vector fields may be multiplied by real numbers and scalar functions \
\>", "Text",
 CellChangeTimes->{{3.6351428432355723`*^9, 3.635142874661028*^9}, 
   3.635155739742696*^9, {3.6354842372372065`*^9, 3.6354842589212446`*^9}, 
   3.635773962105775*^9},ExpressionUUID->"98976727-3fbe-49a7-a096-\
05d4df5a09f3"],

Cell[BoxData[
 RowBox[{"3", "*", "vX", " "}]], "Input",
 CellChangeTimes->{{3.6354843191061497`*^9, 3.6354844119887133`*^9}, 
   3.6354844997232676`*^9, 3.6389438673046827`*^9, 
   3.639291191627859*^9},ExpressionUUID->"f365051b-bbd5-4a8b-96fe-\
9761fd85fa69"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6354845134512916`*^9, 
  3.635484534183728*^9}},ExpressionUUID->"e00c5af2-e9eb-4f68-bd66-\
77e30f554182"],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], "*", "vX"}]], "Input",
 CellChangeTimes->{{3.6351419957740717`*^9, 3.6351420303225327`*^9}, 
   3.6351430065032606`*^9, 3.6389438700658875`*^9, {3.6392912038134804`*^9, 
   3.6392912299343276`*^9}},ExpressionUUID->"5ce94aff-d033-482c-9b85-\
3f0fc419c131"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.635142041886153*^9, 
  3.635142060108985*^9}},ExpressionUUID->"ae9b9014-77ca-4786-bdec-\
95410a75e306"],

Cell[TextData[{
 StyleBox["Proposition 1.",
  FontColor->RGBColor[0, 0, 1]],
 " The set \[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "9ac788ae-736f-4735-afd7-72418cbc9f6f"],
 ", n>0, of all smooth vector fields on the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "3e046106-0225-4c22-b9b3-5cc3b0b7d5ea"],
 " is an infinite dimensional vector space over the field of real numbers and \
a Module over the algebra \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "952bcdf1-4eb3-4ad6-811d-8bea04f6756f"],
 " of real functions on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "2067932f-8f94-4bad-bb4a-4255a45c7053"],
 ". \[FilledSquare]\nThis is a special case of Proposition 1 in Subsection \
2.3.1."
}], "Text",
 CellChangeTimes->{{3.6354097062324467`*^9, 3.6354097244532785`*^9}, {
   3.6354098039042177`*^9, 3.6354098788623495`*^9}, {3.635483692406249*^9, 
   3.6354837845400114`*^9}, {3.6354838818373823`*^9, 3.63548395489231*^9}, {
   3.63548401148921*^9, 3.635484032861247*^9}, 3.6354840949025564`*^9, {
   3.635484792722982*^9, 3.635484792722982*^9}, {3.6355798295995975`*^9, 
   3.6355798295995975`*^9}, 3.635579894199311*^9, 3.635666021755267*^9, 
   3.635773971373192*^9, {3.700800771340706*^9, 3.700800774940613*^9}, {
   3.703740263456105*^9, 3.703740317583354*^9}, {3.703740357943509*^9, 
   3.703740362231472*^9}},ExpressionUUID->"5ce97ab5-91db-4595-bb0b-\
11d0c53e2dc7"],

Cell[TextData[{
 StyleBox["Remark.",
  FontColor->RGBColor[0, 0, 1]],
 " The product with a function can be obtained in tensor terms too. Consider \
the function as the tensor"
}], "Text",
 CellChangeTimes->{{3.639281829359027*^9, 3.6392819097771683`*^9}, {
  3.6392820097579412`*^9, 
  3.6392820322027807`*^9}},ExpressionUUID->"b7b7b290-f809-4fa7-9d17-\
ad4449d65cab"],

Cell[BoxData[
 RowBox[{"tf", "=", 
  RowBox[{"tensor", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["a", "\[Iota]"], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}]}], "]"}], ",", "0", ",", "0"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6392814266599193`*^9, 3.639281442883948*^9}, {
   3.6392814785612106`*^9, 3.6392815179512796`*^9}, {3.639282046508006*^9, 
   3.639282050813614*^9}, 
   3.703740510455714*^9},ExpressionUUID->"d7945e21-c0be-4946-83ed-\
7af69c14ecb6"],

Cell[BoxData[
 RowBox[{"coords", "[", "tf", "]"}]], "Input",
 CellChangeTimes->{{3.639281529105299*^9, 
  3.639281551662939*^9}},ExpressionUUID->"c4f415b4-f936-47c0-97a7-\
dcd71e64db1a"],

Cell[BoxData[
 RowBox[{"vZ", " ", "=", 
  RowBox[{"tf", "\[CircleTimes]", "vX"}]}]], "Input",
 CellChangeTimes->{{3.639281575796181*^9, 3.6392816077606373`*^9}, {
  3.6392912638995876`*^9, 
  3.6392912683455954`*^9}},ExpressionUUID->"e8e4245e-353f-4727-84bd-\
3fa6613d6f35"],

Cell[BoxData[
 RowBox[{"coords", "[", "vZ", "]"}]], "Input",
 CellChangeTimes->{{3.6392816208022604`*^9, 3.6392816523299155`*^9}, 
   3.63929127694421*^9},ExpressionUUID->"e19118a3-c07b-40c4-8297-\
76090a6af51b"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{"tf", "\[CircleTimes]", "vX"}], ",", 
   RowBox[{
    RowBox[{
     SubscriptBox["a", "\[Iota]"], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "*", "vX"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7008013571277246`*^9, 3.70080141995982*^9}, {
  3.700801454448584*^9, 
  3.700801466935707*^9}},ExpressionUUID->"35ea65a7-d6ed-4ef6-9529-\
f2a7a52e3bf1"],

Cell[TextData[{
 StyleBox["The coordinates of a tensor product  tf\[CircleTimes]vX with a \
function tf are the products of all coordinates  of vX with the function",
  FontSlant->"Italic"],
 ". \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.635155802694807*^9, 3.6351559017161813`*^9}, 
   3.639282119364936*^9, {3.6392821580666084`*^9, 3.6392821728730345`*^9}, {
   3.6392822887461514`*^9, 3.639282364100685*^9}, {3.668581197292561*^9, 
   3.668581220563623*^9}, {3.700801029810485*^9, 3.7008011101535273`*^9}, {
   3.7008011952411957`*^9, 3.7008012014331083`*^9}, {3.7008012373157063`*^9, 
   3.700801271632925*^9}, 
   3.700801314760355*^9},ExpressionUUID->"26f0526e-0286-4442-b392-\
9bce877a1f2c"],

Cell["\<\
The standard basis vector fields are defined as tensor fields only on the \
chart domains:\
\>", "Text",
 CellChangeTimes->{{3.635155802694807*^9, 3.6351559017161813`*^9}, 
   3.639282119364936*^9, {3.6392821580666084`*^9, 3.6392821728730345`*^9}, {
   3.6392822887461514`*^9, 3.6392823770019073`*^9}, {3.668581299906896*^9, 
   3.668581344970704*^9}},ExpressionUUID->"083c501e-9f79-49dd-8d83-\
65258cbaf13c"],

Cell[BoxData[
 RowBox[{"?", "tx\[Iota]"}]], "Input",
 CellChangeTimes->{{3.703740904457247*^9, 3.703740909546277*^9}, 
   3.703740943813677*^9},ExpressionUUID->"f6ed04b5-299d-4909-bb31-\
df1706073f07"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tx\[Iota]", "[", "2", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.703740904457247*^9, 3.703740909546277*^9}, 
   3.703740943813677*^9, {3.703740983915717*^9, 
   3.703741012497861*^9}},ExpressionUUID->"755d4fed-feaa-49c7-a17b-\
5f5eb9860138"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"bx\[Iota]", "[", "i_", "]"}], "[", "x__", "]"}], " ", ":=", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"stb", "[", 
    RowBox[{"i", ",", 
     RowBox[{"Length", "[", 
      RowBox[{"{", "x", "}"}], "]"}]}], "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6351413703757668`*^9, 3.635141454540315*^9}, {
   3.635141488848776*^9, 3.6351415025164*^9}, {3.6363807426773987`*^9, 
   3.6363807484494085`*^9}, 3.636380806902711*^9, 3.636380866572816*^9, 
   3.636380976568609*^9, 3.639282463760462*^9, 3.63928253044298*^9, {
   3.668581480701687*^9, 3.668581490873637*^9}, {3.6685815595867577`*^9, 
   3.668581578017177*^9}, 3.668581632057291*^9, {3.668581673009116*^9, 
   3.6685816851203814`*^9}, 3.7037412397683353`*^9, {3.703741558742543*^9, 
   3.70374157123002*^9}},ExpressionUUID->"96564f62-e20f-4c64-a7e0-\
8290266535a7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"bx\[Iota]", "::", "usage"}], " ", "=", " ", 
   "\"\<bx\[Iota][i][x1,...,xn]is the i-th local basis vector field defined \
as a tensor field of type (0,1) on a chart domain.\>\""}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.668582202132333*^9, 3.668582270931476*^9}, {
   3.668582343562697*^9, 3.668582453449376*^9}, 3.668582486001759*^9, 
   3.703741294844637*^9, 
   3.703741375118176*^9},ExpressionUUID->"93ec5fb0-713e-4989-962c-\
31d4ab258cd3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bx\[Iota]", "[", "2", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635141514952622*^9, 3.6351415544724913`*^9}, {
   3.6363810022930546`*^9, 3.636381017830682*^9}, {3.668581714233378*^9, 
   3.668581736175811*^9}, 
   3.703741445442938*^9},ExpressionUUID->"bd885846-6f0f-448d-973d-\
ed409009d603"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6351415709189205`*^9, 
  3.63514158757215*^9}},ExpressionUUID->"8b60cd5e-986d-47bc-a5cb-\
874ad54c977e"],

Cell[TextData[{
 "Here is the local basis decomposition of the vector field vX \[Element] \
\[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "5034490d-3da1-4e50-a731-3430c0900474"]
}], "Text",
 CellChangeTimes->{{3.6351559918747396`*^9, 3.6351560735312834`*^9}, {
   3.635579839568015*^9, 3.6355798462448263`*^9}, 3.638943875603897*^9, 
   3.6393166722720284`*^9},ExpressionUUID->"29d23457-1fc2-4585-9f05-\
0895f05b222f"],

Cell[BoxData[
 RowBox[{"vXb", " ", "=", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"fx", "[", "i", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
     RowBox[{
      RowBox[{"bx\[Iota]", "[", "i", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.63514129861924*^9, 3.6351413290140934`*^9}, {
   3.6351417948575163`*^9, 3.635141862355236*^9}, {3.6363810946920166`*^9, 
   3.636381109449643*^9}, 3.638943887943519*^9, {3.639282652700795*^9, 
   3.639282658410405*^9}, 3.639291345983332*^9, {3.639291417469658*^9, 
   3.63929141863966*^9}, {3.6452738611509037`*^9, 3.645273877608933*^9}, {
   3.668581857655375*^9, 3.668581880398982*^9}, 3.703741448402656*^9, 
   3.703741632997982*^9, {3.70374169854935*^9, 
   3.703741720324829*^9}},ExpressionUUID->"a3d93c5f-2bbe-4b5c-97df-\
6a24d8c40327"],

Cell["\<\
The tensor definitions differ, but the tensors coincide since the \
corresponding coordinates are the same:\
\>", "Text",
 CellChangeTimes->{{3.6351562016965094`*^9, 
  3.635156284974456*^9}},ExpressionUUID->"56c0b95b-a738-4656-b7f8-\
5d340b376c77"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"vX", ",", "vXb"}], "]"}]], "Input",
 CellChangeTimes->{{3.635141890347685*^9, 3.635141900956704*^9}, {
  3.6389439078335543`*^9, 3.638943908941156*^9}, {3.6392913715401773`*^9, 
  3.63929137876299*^9}},ExpressionUUID->"79c211b6-abca-4979-95a9-\
eda40dc175b7"],

Cell["\<\
In most applications it is simpler to work directly with the vector fields \
instead of their tensor form as 1-fold contravariant tensors which we  will \
use for tensor operations only. The transition from vectors to tensors can be \
done by the function vecTotensor as shown in the examples above. The inverse \
transition can be made using the function tensorToArray:\
\>", "Text",
 CellChangeTimes->{{3.63548493947244*^9, 3.635484942140044*^9}, {
   3.6354850733830748`*^9, 3.6354852239469395`*^9}, {3.635485285208247*^9, 
   3.6354853099030905`*^9}, {3.6354853553147707`*^9, 
   3.6354853738944025`*^9}, {3.635485547132707*^9, 3.6354856396564693`*^9}, {
   3.635485677112136*^9, 3.635485698000572*^9}, {3.6354857533026695`*^9, 
   3.6354857990263495`*^9}, 3.6357742499072046`*^9, 3.635918821290851*^9, 
   3.6393167310373316`*^9, 
   3.6685820590392447`*^9},ExpressionUUID->"764513ab-4286-49c1-a3e0-\
195814617e27"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vx", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}]}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{"tensorToArray", "[", "vX", "]"}]}]], "Input",
 CellChangeTimes->{{3.6354858064207625`*^9, 3.635485874452482*^9}, 
   3.6389439138083644`*^9, {3.6392914603279333`*^9, 3.639291470267151*^9}, {
   3.668582020405397*^9, 3.668582034781063*^9}, 
   3.703741883475851*^9},ExpressionUUID->"47d8779b-6b72-4851-8f50-\
f92e0634eefe"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3.4. Directional Derivatives", "Subsubsection",
 CellChangeTimes->{{3.645275625373603*^9, 3.645275626668405*^9}, {
   3.6452756570416584`*^9, 3.6452756689912796`*^9}, {3.645275950743374*^9, 
   3.6452759613513927`*^9}, 3.645363128659916*^9, 3.668582120956892*^9, 
   3.668595906569112*^9, 3.7008016843261833`*^9, 3.700806200242238*^9, 
   3.7028837426787777`*^9, 3.702893094005081*^9, 3.703741949284358*^9, 
   3.7037510308842793`*^9, 
   3.703751068595948*^9},ExpressionUUID->"5c24a4a7-efb4-44db-898c-\
7c67905ceadf"],

Cell["We continue considering the local vector field", "Text",
 CellChangeTimes->{{3.645275729066985*^9, 3.6452757701886573`*^9}, {
  3.645275801965913*^9, 
  3.645275874677641*^9}},ExpressionUUID->"b132afd9-fca9-4865-8817-\
8ff9fcec7ee7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.703746046475569*^9, 
  3.703746052995738*^9}},ExpressionUUID->"7c4cd890-3187-4383-aeac-\
6650d659e293"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"vx", "[", "x__", " ", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"vv", "[", "fx", "]"}], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.64527588009085*^9, 3.64527589147887*^9}, {
  3.703745826006172*^9, 
  3.70374585337386*^9}},ExpressionUUID->"409e115b-c4e7-4dea-b179-\
bc25ab07d971"],

Cell[BoxData[
 RowBox[{"vx", "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6452758978904815`*^9, 
  3.6452759147541113`*^9}},ExpressionUUID->"fb04d67d-c1b1-4452-bc8b-\
42c9c8821172"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "h", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.6354888353500824`*^9},ExpressionUUID->"7e12e89b-bbec-4dfb-a4ff-\
a427f2bb22eb"],

Cell[BoxData[
 RowBox[{"%", ".", 
  RowBox[{"vx", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.635489638470693*^9, 3.6354896462239065`*^9}, {
  3.6452742389211674`*^9, 
  3.6452742654568143`*^9}},ExpressionUUID->"4b78b8ff-cfa1-45f2-bf92-\
51c355d18df4"],

Cell[TextData[{
 "This is the ",
 StyleBox["directional derivative",
  FontColor->GrayLevel[0]],
 " of the function h \[Element] \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "075441de-3e39-4231-8876-36e05fd1ecf4"],
 " with respect to the vector field vx  \[Element] \[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "0c9c5046-8044-408c-a8aa-dc4037c7f9d2"],
 " ."
}], "Text",
 CellChangeTimes->{{3.6355797139877944`*^9, 3.6355797665286865`*^9}, {
   3.6355799224665604`*^9, 3.635579951872612*^9}, {3.6355800908628273`*^9, 
   3.6355801225222836`*^9}, 3.638943925024784*^9, {3.6392862950846243`*^9, 
   3.639286331327488*^9}, 3.6393167923298397`*^9, {3.645274182729869*^9, 
   3.6452741914034843`*^9}, {3.6452743140821*^9, 3.645274328933326*^9}, {
   3.6452743804602165`*^9, 
   3.645274413438674*^9}},ExpressionUUID->"7646310c-b6f2-4e2b-9ed5-\
a1dc56e1a43d"],

Cell[TextData[{
 "We define the ",
 StyleBox["directional derivative",
  FontColor->RGBColor[0, 0, 1]],
 "  of the function  f \[Element] \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "0ebd0071-45cf-4333-8564-1a455a835c5a"],
 " at the point x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6325454418171577`*^9, 3.6325455117988806`*^9}},
  ExpressionUUID->"96f3f107-4519-4e2d-83e6-752208e0f793"],
 " with respect to the vector X using the chart representations of f and X; \
we omit the chart index \[Iota]:"
}], "Text",
 CellChangeTimes->{
  3.63661382858226*^9, {3.6381687493588963`*^9, 3.6381687754597425`*^9}, {
   3.638168850923075*^9, 3.6381689060741725`*^9}, {3.6381689442192397`*^9, 
   3.6381690209075775`*^9}, {3.6381690542624364`*^9, 3.638169078039279*^9}, 
   3.638169134344778*^9, {3.6381691703516417`*^9, 3.638169184315666*^9}, {
   3.6381692174999247`*^9, 
   3.638169232600751*^9}},ExpressionUUID->"3155f4d4-d39a-4081-9a63-\
d38f360bf048"],

Cell[BoxData[
 RowBox[{"?", "der"}]], "Input",
 CellChangeTimes->{{3.6664493239694242`*^9, 
  3.666449325672645*^9}},ExpressionUUID->"c4be2812-b797-4237-9503-\
c70b4c2d5f3d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"der", "[", 
    RowBox[{"v_", ",", "ff_"}], "]"}], "[", "x__", "]"}], ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Head", "[", "v", "]"}], "===", "List"}], ")"}], "&&", 
     RowBox[{
      RowBox[{"Length", "[", "v", "]"}], "\[Equal]", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}]}]}], ",", 
    RowBox[{"Return", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"d", "[", "ff", "]"}], "[", "x", "]"}], ")"}], ".", "v"}], 
     "]"}], ",", 
    RowBox[{
     RowBox[{"Print", "[", "\"\<Incorrect arguments!\>\"", "]"}], ";", " ", 
     RowBox[{"Return", "[", "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6330646365427036`*^9, 3.633064676635997*^9}, {
   3.6330647182693787`*^9, 3.633064737389472*^9}, {3.633065350446537*^9, 
   3.633065359981082*^9}, {3.6330654357834177`*^9, 3.6330654903245373`*^9}, {
   3.6330655395733542`*^9, 3.633065564084756*^9}, 3.633065990014118*^9, 
   3.6330661086619043`*^9, {3.6330661521243896`*^9, 3.6330661780598736`*^9}, {
   3.633066239284375*^9, 3.6330662440356474`*^9}, {3.633069040124574*^9, 
   3.6330690597416964`*^9}, 3.6338478114269147`*^9, {3.6338478766818295`*^9, 
   3.6338479010646725`*^9}, {3.6338479883312254`*^9, 
   3.6338480123708677`*^9}, {3.633848451480439*^9, 3.6338484692956705`*^9}, {
   3.6338740528472567`*^9, 3.6338740648436775`*^9}, {3.63387410050534*^9, 
   3.6338741513614297`*^9}, {3.633874186180691*^9, 3.6338742566616144`*^9}, {
   3.638945544860713*^9, 3.638945546779516*^9}, 3.6389456952917776`*^9, {
   3.638945775975119*^9, 3.6389459529418297`*^9}, {3.6389459989151106`*^9, 
   3.638946000771514*^9}, {3.6392838653985343`*^9, 3.639283897693591*^9}, 
   3.6392839482280807`*^9, {3.639284060016881*^9, 3.6392842137565517`*^9}, {
   3.6392842461930094`*^9, 3.6392842463958097`*^9}, {3.6392842973670993`*^9, 
   3.6392842975698996`*^9}, {3.639284336198969*^9, 3.6392844235213223`*^9}, {
   3.639284739301879*^9, 3.6392848347684474`*^9}, {3.639284865300701*^9, 
   3.639284980396104*^9}, 3.6392860969754753`*^9, 3.639286141253353*^9, {
   3.6664497769694505`*^9, 3.6664497790789347`*^9}, {3.666451833782463*^9, 
   3.666451957273241*^9}},ExpressionUUID->"bc1035cc-7df0-43a5-9ad9-\
1ff1a2135896"],

Cell[BoxData[
 RowBox[{
  RowBox[{"der", "[", 
   RowBox[{
    RowBox[{"vec", "[", "a", "]"}], ",", "h"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6452746328842597`*^9, 3.6452746671575193`*^9}, {
  3.666452015745019*^9, 
  3.666452056700265*^9}},ExpressionUUID->"e2376718-0015-441f-8eb6-\
eeb8a8d3f9ce"],

Cell["This coincides with the function calculated above:", "Text",
 CellChangeTimes->{{3.645274878522291*^9, 
  3.645274932623186*^9}},ExpressionUUID->"c0a36432-c9f9-4bac-985b-\
dfc9e50ccd1e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "h", "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], ".", 
    RowBox[{"vec", "[", "a", "]"}]}]}], "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.645274722646817*^9, 3.645274761724886*^9}, {
  3.666452121547371*^9, 3.6664521292665167`*^9}, {3.700805011139839*^9, 
  3.700805022507346*^9}},ExpressionUUID->"0309cd08-a77c-4f79-8a15-\
fa8189e06c05"],

Cell[TextData[{
 StyleBox["Examples",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.6452749859440794`*^9, 
  3.6452749972852993`*^9}},ExpressionUUID->"091a1bc5-f1bb-4f8e-ad90-\
57214a072e7e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"der", "[", 
   RowBox[{
    RowBox[{"vec", "[", "\[Xi]", "]"}], ",", 
    RowBox[{"plus", "[", 
     RowBox[{"f", ",", "h"}], "]"}]}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{3.640240141726615*^9, 
  3.6664520812327633`*^9},ExpressionUUID->"eb68afe3-bd50-4848-a1d4-\
334730874dbc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"der", "[", 
   RowBox[{"pp", ",", 
    RowBox[{"plus", "[", 
     RowBox[{"f", ",", "h"}], "]"}]}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6392855798261614`*^9, 3.6392855981269937`*^9}, 
   3.640240149573428*^9, 
   3.6664520812327633`*^9},ExpressionUUID->"91c92ed2-46e8-438b-a870-\
85b262af59c0"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"der", "[", 
     RowBox[{
      RowBox[{"vec", "[", "\[Xi]", "]"}], ",", 
      RowBox[{"plus", "[", 
       RowBox[{"f", ",", "h"}], "]"}]}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"der", "[", 
      RowBox[{
       RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "f"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"der", "[", 
      RowBox[{
       RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "h"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.632642871369364*^9, 3.6326429523717065`*^9}, {
  3.6381693996910458`*^9, 3.6381695448361015`*^9}, {3.6381695952905903`*^9, 
  3.6381696135768223`*^9}, {3.6389462397327337`*^9, 3.6389464037670217`*^9}, {
  3.6402401522410336`*^9, 3.640240155641839*^9}, {3.6664520812327633`*^9, 
  3.6664520812483873`*^9}},ExpressionUUID->"25353286-6c27-4885-aca9-\
d796db1751a3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"der", "[", 
   RowBox[{
    RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "one"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.632722213134819*^9, 3.632722220061231*^9}, {
   3.638169653109292*^9, 3.6381696626097083`*^9}, {3.6389464696303377`*^9, 
   3.6389464805659566`*^9}, 3.6402401567650414`*^9, 
   3.6664520812483873`*^9},ExpressionUUID->"5d06df75-0ee6-40b1-822e-\
7aafeb4f11d0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "a", "]"}], ";", 
  RowBox[{"SetAttributes", "[", 
   RowBox[{"a", ",", "Constant"}], "]"}], ";", 
  RowBox[{
   RowBox[{"h", "[", "x__", "]"}], " ", "=", " ", 
   RowBox[{"a", "*", 
    RowBox[{"f", "[", "x", "]"}]}]}], ";", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"a", "*", 
     RowBox[{
      RowBox[{"der", "[", 
       RowBox[{
        RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "f"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], " ", "\[Equal]", 
    RowBox[{
     RowBox[{"der", "[", 
      RowBox[{
       RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "h"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6326435853972225`*^9, 3.6326436661831646`*^9}, {
  3.6326437001328244`*^9, 3.6326437128508472`*^9}, {3.6326448411694183`*^9, 
  3.6326448875065002`*^9}, {3.632663619747343*^9, 3.63266366343142*^9}, {
  3.638169703795781*^9, 3.638169796793545*^9}, {3.6389465026555953`*^9, 
  3.6389465328260484`*^9}, {3.640240158200244*^9, 3.6402401595418463`*^9}, {
  3.6664520812640142`*^9, 
  3.6664520812640142`*^9}},ExpressionUUID->"da73a1bc-0636-42be-b7b0-\
4fe93794b60f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "a", "]"}], ";", 
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{"NumberQ", "[", "a", "]"}], ",", 
    RowBox[{"Simplify", "[", 
     RowBox[{
      RowBox[{"a", "*", 
       RowBox[{
        RowBox[{"der", "[", 
         RowBox[{
          RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "f"}], "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}]}], " ", "\[Equal]", 
      RowBox[{
       RowBox[{"der", "[", 
        RowBox[{
         RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "h"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6326429633229256`*^9, 3.6326430762041245`*^9}, {
   3.63264319262253*^9, 3.632643196491337*^9}, {3.632643375726453*^9, 
   3.632643398658493*^9}, {3.632643476654031*^9, 3.632643491630057*^9}, {
   3.632643529760524*^9, 3.632643545269952*^9}, 3.6326436210326853`*^9, {
   3.6326437977073965`*^9, 3.632643817679432*^9}, {3.6326438849069505`*^9, 
   3.6326438861861525`*^9}, {3.6326439186060095`*^9, 
   3.6326439551140738`*^9}, {3.6326642129177885`*^9, 3.6326642195946007`*^9}, 
   3.6381698426002254`*^9, {3.638169880057891*^9, 3.638169883224697*^9}, {
   3.6389465738073206`*^9, 3.638946612510988*^9}, {3.6402401610238485`*^9, 
   3.6402401633638525`*^9}, 3.645275141850754*^9, {3.6664520812640142`*^9, 
   3.6664520812796392`*^9}},ExpressionUUID->"c18a4bb1-dfab-4cfa-8c79-\
9cf2d4947bd5"],

Cell["\[FilledSmallSquare]", "Text",
 CellChangeTimes->{{3.6452762626347218`*^9, 
  3.6452762626347218`*^9}},ExpressionUUID->"2d5ef983-fcc0-404b-8b1e-\
f030842842cd"],

Cell[TextData[{
 "It follows\n",
 StyleBox["Proposition 1",
  FontColor->RGBColor[0, 0, 1]],
 ". For a fixed tangent vector with coordinates vec[\[Xi]] the map \n",
 "der[",
 "vec[\[Xi]],h]: h\[Element] \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "82c6e2be-1021-4e80-8b5a-edbb008a33a5"],
 "\[RightArrow] R is linear. Moreover it satisfies the following product rule"
}], "Text",
 CellChangeTimes->{{3.6326648170816545`*^9, 3.6326648498145123`*^9}, {
   3.6326648936711893`*^9, 3.6326648947319913`*^9}, {3.632664966441718*^9, 
   3.6326649784693394`*^9}, {3.6326650170054073`*^9, 3.632665165161268*^9}, 
   3.6326656164128637`*^9, {3.6326661050587254`*^9, 3.6326661331387744`*^9}, {
   3.6326671055414085`*^9, 3.6326671055414085`*^9}, 3.6330593199686337`*^9, {
   3.633064233811669*^9, 3.6330642447312937`*^9}, 3.6381699006207275`*^9, {
   3.638946640981038*^9, 3.638946708513557*^9}, {3.6392856911433573`*^9, 
   3.639285710724392*^9}, {3.6392857561244717`*^9, 3.6392857853005233`*^9}, 
   3.666452081357768*^9},ExpressionUUID->"0d72ef77-11f3-42ed-8a73-\
8c2f7d5c2b18"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"f", ",", "h"}], "]"}], ";", 
  RowBox[{
   RowBox[{"fh", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], " ", "*", 
    RowBox[{"h", "[", "x", "]"}]}]}], ";", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"der", "[", 
      RowBox[{
       RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "fh"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[Equal]", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"der", "[", 
        RowBox[{
         RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "h"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], " ", "+", 
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"der", "[", 
        RowBox[{
         RowBox[{"vec", "[", "\[Xi]", "]"}], ",", "f"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.632664349309229*^9, 3.632664359180447*^9}, {
   3.6326643933017073`*^9, 3.632664403504125*^9}, {3.632664492779483*^9, 
   3.6326645095807123`*^9}, {3.632664544232373*^9, 3.6326646124008937`*^9}, 
   3.632664752489541*^9, {3.638169945271006*^9, 3.6381700803746448`*^9}, {
   3.6381701118253*^9, 3.6381701257571244`*^9}, {3.638946747763226*^9, 
   3.6389468124097395`*^9}, {3.640240164955055*^9, 3.640240171741067*^9}, {
   3.6664520813734055`*^9, 
   3.6664520813890195`*^9}},ExpressionUUID->"241d686e-eba9-4ec3-9590-\
8301db9e6f92"],

Cell[TextData[{
 "\[FilledSquare] \n",
 StyleBox["Notation.",
  FontColor->RGBColor[0, 0, 1]],
 " The traditional notation is\nX[f] == ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Xi]", "i"], TraditionalForm]],ExpressionUUID->
  "7a03847a-a800-4376-9e6e-6d80ac96f924"],
 Cell[BoxData[
  StyleBox[
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], "f"}],
   FontWeight->"Bold"]],
  CellChangeTimes->{
   3.6345516019458685`*^9, {3.6345516322255216`*^9, 3.6345516884168205`*^9}, {
    3.6345517648101544`*^9, 3.634551780784582*^9}},
  TextAlignment->Center,ExpressionUUID->
  "b6fed91d-09aa-4e40-b1bf-e7d912831c5f"],
 " \[DotEqual] ",
 "der[",
 "X, f][x[1], ...,x[n]] "
}], "Text",
 CellChangeTimes->{{3.6326653014591084`*^9, 3.6326653014747086`*^9}, 
   3.632667816073413*^9, {3.6326678566414843`*^9, 3.632667948715247*^9}, 
   3.6330593380802655`*^9, {3.638170159004783*^9, 3.6381703032838373`*^9}, {
   3.6381703404939027`*^9, 3.638170365455947*^9}, {3.6381704073284206`*^9, 
   3.63817044078988*^9}, 3.638187455858329*^9, {3.638947010857688*^9, 
   3.6389470452245483`*^9}, {3.6392858441306276`*^9, 3.6392858441306276`*^9}, 
   3.666452081514027*^9},ExpressionUUID->"20909807-f591-4868-8e58-\
275f046f6ed2"],

Cell[BoxData[
 RowBox[{"Context", "[", "der", "]"}]], "Input",
 CellChangeTimes->{{3.6664491326157656`*^9, 3.6664492013380833`*^9}, {
  3.668583154380158*^9, 
  3.6685831545872307`*^9}},ExpressionUUID->"f1d864e4-6026-4caa-b60a-\
9190b73c744c"],

Cell["The product rule:", "Text",
 CellChangeTimes->{{3.6381788492293034`*^9, 
  3.638178864080529*^9}},ExpressionUUID->"07883c47-6a04-4f46-af1a-\
1bfae2b8e410"],

Cell[BoxData[
 RowBox[{
  RowBox[{"X", "[", 
   RowBox[{"f", "*", "h"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"X", "[", "f", "]"}], "*", "h"}], " ", "+", " ", 
   RowBox[{"f", "*", 
    RowBox[{
     RowBox[{"X", "[", "h", "]"}], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6381788697433395`*^9, 
  3.638178910680811*^9}},ExpressionUUID->"1d8ac57a-affe-477f-8e8c-\
012582bd723f"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6327283839579024`*^9, 3.6327284322781878`*^9}, {
  3.6329875171586676`*^9, 3.6329875261598835`*^9}, {3.6363850159934044`*^9, 
  3.636385050162465*^9}, {3.6381705450216637`*^9, 
  3.6381705463320656`*^9}},ExpressionUUID->"88fde5cd-ba81-4e15-9d3a-\
60ba69b3d781"],

Cell[TextData[{
 StyleBox["Remark.",
  FontColor->RGBColor[0, 0, 1]],
 " The differentiability classes of manifolds, functions, vector fields etc. \
fulfill the inclusions"
}], "Text",
 CellChangeTimes->{{3.6356662427264566`*^9, 3.6356663238077993`*^9}, {
   3.6356665026647644`*^9, 3.6356665130231824`*^9}, 
   3.635774299418092*^9},ExpressionUUID->"3e92c122-57eb-4b58-91b8-\
9ad76f2f3511"],

Cell[TextData[{
 "                                       ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Omega]"], TraditionalForm]],ExpressionUUID->
  "cd96e640-c0a4-480f-805e-607f30fa3db6"],
 " \[Subset] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Infinity]"], TraditionalForm]],ExpressionUUID->
  "a3fe39f0-b00d-4179-9173-61ea940f624d"],
 "\[Subset] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "543df677-3591-4d9b-83cd-84a156e79590"],
 "\[Subset] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "1c3a5fba-4d4e-4178-8059-6c382278c0fc"],
 " \[Subset] ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["C", "0"], " "}], TraditionalForm]],ExpressionUUID->
  "10606dcf-eab6-47b3-b828-6bce025edcc7"],
 ", k = 1,2,...."
}], "Text",
 CellChangeTimes->{{3.635666533601619*^9, 3.6356666209677725`*^9}, {
  3.6356666546034317`*^9, 3.635666871897015*^9}, {3.6356677381435485`*^9, 
  3.635667739875151*^9}},ExpressionUUID->"49d0b09b-5ff8-49f1-99d4-\
2db1ada947b8"],

Cell[TextData[{
 "Thus, if X, h are of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "6aec35c0-ded3-4537-94ee-f17cbdf3a383"],
 ", then Xh may belong to the class  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "24e27f8c-2bed-4809-b35e-8db9810a7a52"],
 " only. In concrete cases a control of the class property of Xh can be \
necessary. To avoid any complication we assume that the manifolds and the \
geometric objects defined on them are all of class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Infinity]"], TraditionalForm]],ExpressionUUID->
  "784dc1f9-4a45-4f8f-9ad5-e9ad0dbc9fdb"],
 ". In particular, the terminus \[OpenCurlyDoubleQuote]smooth\
\[CloseCurlyDoubleQuote] in the following means \[OpenCurlyDoubleQuote]of \
class ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "\[Infinity]"], TraditionalForm]],ExpressionUUID->
  "d5f42bd5-6352-46d2-82c9-546f8bb8221b"],
 "\[CloseCurlyDoubleQuote].  \[FilledSquare] \nWe summarize:\n",
 StyleBox["Proposition 2.",
  FontColor->RGBColor[0, 0, 1]],
 "\nFor  each smooth vector field X the map h \[Element]  \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "6d781639-5f55-48cd-a11f-4ee7636bd626"],
 " \[RightTeeArrow] Xh \[DotEqual] der[X,h] is a derivation of the real \
algebra   \[ScriptCapitalF]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "1b015711-0b6f-4858-a544-8e8760fb5aed"],
 ", it satisfies the rules:\nX[h1 + h2] == X[h1] + X[h2],                 \t\t\
\t (1)\nX[\[Alpha]\[CenterDot]h] == \[Alpha]\[CenterDot]X[h]  for all \
\[Alpha] \[Element] \[DoubleStruckCapitalR],                 \t\t (2)\nX[h1\
\[CenterDot]h2] == h1\[CenterDot]X[h2] +h2\[CenterDot]X[h1].\t\t\t (3)\nWith \
other words, X is a linear map fulfilling the product rule. \[FilledSquare]\n"
}], "Text",
 CellChangeTimes->{{3.635603994199234*^9, 3.6356040123108664`*^9}, 
   3.6356043733507013`*^9, 3.6356044970191193`*^9, {3.6356045681924443`*^9, 
   3.6356045718116508`*^9}, {3.6356046030615063`*^9, 
   3.6356046864340525`*^9}, {3.6356047261859226`*^9, 
   3.6356047651889915`*^9}, {3.6356047973406477`*^9, 
   3.6356050176466355`*^9}, {3.635605051813696*^9, 3.635605094326771*^9}, {
   3.635605141661254*^9, 3.635605500973487*^9}, {3.635605662771372*^9, 
   3.635605732977495*^9}, {3.635667143964295*^9, 3.6356675412256007`*^9}, {
   3.6356675957692966`*^9, 3.6356676655798206`*^9}, {3.6356677818784256`*^9, 
   3.6356677943740473`*^9}, {3.6356678398053274`*^9, 3.6356679072386465`*^9}, 
   3.635774322744133*^9, 3.639289619141088*^9, 3.6392897177744617`*^9, {
   3.6392899263834295`*^9, 3.6392899707851076`*^9}, 3.6392915132014265`*^9, 
   3.6664520816390324`*^9, {3.66859605053727*^9, 3.66859608777532*^9}, {
   3.7008053812823563`*^9, 
   3.700805394728948*^9}},ExpressionUUID->"fe96da2f-d15e-4896-a86a-\
a1ec6de564d1"],

Cell["We consider a second vector field Y defined by vy:", "Text",
 CellChangeTimes->{{3.635580294346589*^9, 3.635580308045413*^9}, {
  3.635605816507042*^9, 3.6356058217486515`*^9}, {3.645362386410612*^9, 
  3.6453624250362806`*^9}, {3.6685961350787477`*^9, 
  3.668596136238533*^9}},ExpressionUUID->"bf23632c-e095-4fdd-add7-\
c6b3e06c3e29"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vy", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"vv", "[", "fy", "]"}], "[", "x", "]"}]}], ";", 
  RowBox[{"vy", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6354915959465313`*^9, 3.635491615836566*^9}, {
   3.6354917824448586`*^9, 3.6354917893244705`*^9}, {3.635518632138204*^9, 
   3.635518637723014*^9}, 3.639290324587131*^9, {3.6452764517538543`*^9, 
   3.645276497040734*^9}, {3.7037463067783747`*^9, 
   3.703746353065791*^9}},ExpressionUUID->"d10e1261-0436-4e8b-a5f5-\
a65252ce64b9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"X", "[", "hh_", "]"}], "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"der", "[", 
     RowBox[{
      RowBox[{"vx", "[", "x", "]"}], ",", "hh"}], "]"}], "[", "x", "]"}]}], 
  ";", 
  RowBox[{
   RowBox[{"X", "[", "h", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.63549182319213*^9, 3.6354919137034893`*^9}, {
   3.635493696693021*^9, 3.6354937416835003`*^9}, {3.6354937752859592`*^9, 
   3.635493801447205*^9}, {3.6354938443628807`*^9, 3.6354938695101247`*^9}, {
   3.6354940029371586`*^9, 3.6354940149491796`*^9}, 3.6354940492848406`*^9, {
   3.635494085523704*^9, 3.6354941222617683`*^9}, 3.635518625289792*^9, 
   3.639290183448082*^9, {3.6392903589569917`*^9, 3.6392903600021935`*^9}, 
   3.639291525790649*^9, 3.6664520816859093`*^9, {3.668596627955793*^9, 
   3.66859663383445*^9}, {3.703746763775631*^9, 
   3.703746809453953*^9}},ExpressionUUID->"3f898990-4023-4b1b-acd3-\
37d482250088"],

Cell["\<\
and calculate the directional derivative of Xh with respect to Y:\
\>", "Text",
 CellChangeTimes->{{3.6355803875375533`*^9, 3.635580437186841*^9}, 
   3.635668026122856*^9},ExpressionUUID->"913aa89b-d2e8-4c95-ae83-\
34e9cb33a901"],

Cell[BoxData[
 RowBox[{
  RowBox[{"der", "[", 
   RowBox[{
    RowBox[{"vy", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"X", "[", "h", "]"}]}], "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635502876991579*^9, 3.6355029041044264`*^9}, {
   3.6355033296221046`*^9, 3.635503348014537*^9}, 3.6392901846492844`*^9, {
   3.6392903912676487`*^9, 3.6392903919696493`*^9}, 3.666452081701537*^9, {
   3.6685966652584877`*^9, 3.6685966719860773`*^9}, {3.7037468417185717`*^9, 
   3.7037468573094063`*^9}},ExpressionUUID->"460cd8b8-43a4-446c-b077-\
e6b30a0a12a6"],

Cell["\<\
The composition YXh of two directional derivatives is again a linear map, but \
not a directional derivative, since  it depends on the second derivatives of \
h. We set\
\>", "Text",
 CellChangeTimes->{{3.635580462618886*^9, 3.6355805294574037`*^9}, 
   3.635580611396841*^9, {3.635668188288943*^9, 3.635668251789055*^9}, {
   3.6356682869419165`*^9, 3.635668402737321*^9}, {3.6356684675634356`*^9, 
   3.635668470808241*^9}, 
   3.635752807379834*^9},ExpressionUUID->"ced227ab-8e15-49e6-a046-\
5bd57560aa4f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Y", "[", "hh_", "]"}], "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"der", "[", 
     RowBox[{
      RowBox[{"vy", "[", "x", "]"}], ",", "hh"}], "]"}], "[", "x", "]"}]}], 
  ";", 
  RowBox[{
   RowBox[{"Y", "[", "h", "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.635519507174941*^9, 3.63551952973258*^9}, 
   3.639290185788086*^9, {3.639290439226133*^9, 3.6392904406301355`*^9}, 
   3.666452081701537*^9, {3.668596705362729*^9, 3.668596719449841*^9}, {
   3.703746899486483*^9, 
   3.7037469910766287`*^9}},ExpressionUUID->"c93d0da1-4c27-4b4a-b8fd-\
804db86d03d8"],

Cell[TextData[{
 "and calculate  the ",
 StyleBox["commutator",
  FontColor->RGBColor[0, 0, 1]],
 " of the directional derivatives with respect to X, Y:"
}], "Text",
 CellChangeTimes->{{3.6356684812642593`*^9, 3.635668524485737*^9}, {
   3.6356687545819426`*^9, 3.6356687715275726`*^9}, {3.6356923485992227`*^9, 
   3.635692395433505*^9}, 3.6357743755416265`*^9, {3.6392905319026957`*^9, 
   3.6392905452105193`*^9}},ExpressionUUID->"54414ea9-b903-4017-82ff-\
2e6a012acf49"],

Cell[BoxData[
 RowBox[{
  RowBox[{"X", "[", 
   RowBox[{"Y", "[", "h", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",ExpressionUUID->"e60494a9-\
801b-40f4-9999-a189f663d345"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"X", "[", 
     RowBox[{"Y", "[", "h", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
   RowBox[{
    RowBox[{"Y", "[", 
     RowBox[{"X", "[", "h", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6355195622430377`*^9, 3.6355196215543413`*^9}, {
   3.6355783798963737`*^9, 3.635578403795616*^9}, {3.6392901868644876`*^9, 
   3.63929018795649*^9}, {3.6392905599681454`*^9, 3.6392905786131783`*^9}, 
   3.6392915267266507`*^9, {3.666452081701537*^9, 3.666452081701537*^9}, 
   3.66859674926685*^9, {3.6685967835136223`*^9, 3.66859682426642*^9}, {
   3.703747038964275*^9, 3.703747116099642*^9}, {3.7037471989869328`*^9, 
   3.703747244442795*^9}},ExpressionUUID->"fa448ddf-30a9-4f0c-a9a9-\
3a2852bd653b"],

Cell[TextData[{
 "This function depends linearly on the first derivatives of h. The \
coordinates of the derivative are the coordinates of a new vector field \
called  the ",
 StyleBox["commutator",
  FontColor->RGBColor[0, 0, 1]],
 " of the vector fields vx,vy:"
}], "Text",
 CellChangeTimes->{{3.635692430645767*^9, 3.63569246082022*^9}, {
   3.6452767222115293`*^9, 3.6452768911130257`*^9}, 3.660802564205418*^9, 
   3.660802601656567*^9},ExpressionUUID->"830e37bb-39fe-458c-956b-\
b25f6b47ac73"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"comm", "[", 
    RowBox[{"v_", ",", "w_"}], "]"}], "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Length", "[", 
        RowBox[{"v", "[", "x", "]"}], "]"}], "\[Equal]", " ", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], ")"}], "&&", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Length", "[", 
        RowBox[{"w", "[", "x", "]"}], "]"}], "\[Equal]", " ", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], ")"}]}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"Grad", "[", 
       RowBox[{
        RowBox[{"w", "[", "x", "]"}], ",", 
        RowBox[{"{", "x", "}"}]}], "]"}], ".", 
      RowBox[{"v", "[", "x", "]"}]}], " ", "-", 
     RowBox[{
      RowBox[{"Grad", "[", 
       RowBox[{
        RowBox[{"v", "[", "x", "]"}], ",", 
        RowBox[{"{", "x", "}"}]}], "]"}], ".", 
      RowBox[{"w", "[", "x", "]"}]}]}], ",", " ", 
    RowBox[{
     RowBox[{"Print", "[", "\"\<Invalid arguments!\>\"", "]"}], ";", 
     RowBox[{"Return", "[", "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.635520126652029*^9, 3.6355203351307955`*^9}, {
   3.6355203750356655`*^9, 3.6355204874650626`*^9}, {3.639291731769212*^9, 
   3.63929174187803*^9}, {3.6392917895079136`*^9, 3.6392918269519796`*^9}, {
   3.639313608876648*^9, 3.6393137171876383`*^9}, {3.639313753488902*^9, 
   3.639313856839083*^9}, 3.639313910908778*^9, 
   3.66859800576611*^9},ExpressionUUID->"007c2801-612a-43cc-8982-\
b0523d2ed67d"],

Cell[BoxData[
 RowBox[{"?", "comm"}]], "Input",
 CellChangeTimes->{{3.639313547084939*^9, 
  3.6393135504077454`*^9}},ExpressionUUID->"e2cb7b36-0981-4670-a921-\
6665f0f381cc"],

Cell[TextData[{
 StyleBox["Notation.",
  FontColor->RGBColor[0, 0, 1]],
 "    [X,Y] \[DotEqual]  comm[X,Y][x],   x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "0ead7f33-710a-4351-92f5-49a147927d1e"],
 "."
}], "Text",
 CellChangeTimes->{{3.635668837709889*^9, 3.635668971510725*^9}, {
  3.635669029596627*^9, 3.6356690366946397`*^9}, {3.639290620128851*^9, 
  3.639290626914863*^9}},ExpressionUUID->"53d841e9-fb94-4bd4-bb64-\
fd2b25960eb3"],

Cell["This is a vector field again, namely", "Text",
 CellChangeTimes->{{3.6356925333673477`*^9, 3.6356925506435785`*^9}, {
  3.635692697780438*^9, 
  3.6356927340525017`*^9}},ExpressionUUID->"df30b744-ed5a-4a76-a289-\
4c9ffd7dca22"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "Z", "]"}], ";", 
  RowBox[{
   RowBox[{"Zc", "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vx", ",", "vy"}], "]"}], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.635520506871497*^9, 3.635520552392377*^9}, {
   3.6355768977845707`*^9, 3.635576900779776*^9}, {3.6392906732333446`*^9, 
   3.6392906815209594`*^9}, 3.6392915284426537`*^9, {3.6392945707431374`*^9, 
   3.639294616974019*^9}, 
   3.703750635360194*^9},ExpressionUUID->"5aa8ec5b-9e63-4c41-94d3-\
164254a2051c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Zc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.645332267988242*^9, 3.645332280842665*^9}, {
   3.70374746202593*^9, 3.7037474700002747`*^9}, {3.703747504362109*^9, 
   3.703747507432527*^9}, 3.703747557817389*^9, 
   3.703750648688561*^9},ExpressionUUID->"89542d7c-82c9-46c4-b770-\
a46014c6f91f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Z", "[", "h_", "]"}], "[", "x__", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"der", "[", 
    StyleBox[
     RowBox[{
      RowBox[{"Zc", "[", "x", "]"}], ",", "h"}], "Output"], 
    StyleBox["]", "Output"]}], 
   StyleBox["[", "Output"], 
   StyleBox["x", "Output"], 
   StyleBox["]", "Output"]}]}]], "Input",
 CellChangeTimes->{{3.703750461545891*^9, 3.70375051835773*^9}, {
   3.70375058326409*^9, 3.703750600504019*^9}, 3.703750667848282*^9, {
   3.703750805206401*^9, 
   3.703750807566429*^9}},ExpressionUUID->"cbcac4bf-b95d-438a-8735-\
b0466c70991b"],

Cell[BoxData[
 RowBox[{"?", "der"}]], "Input",
 CellChangeTimes->{{3.703750761283383*^9, 
  3.703750772214497*^9}},ExpressionUUID->"852a4d28-11a4-4e98-b4c6-\
2babaa029a0e"],

Cell[TextData[{
 "We prove\n",
 StyleBox["Proposition 3.",
  FontColor->RGBColor[0, 0, 1]],
 " The directional derivative of h with respect to the commutator vector \
field [X, Y] equals the commutator of the directional derivatives of h with \
respect to the vector fields X, Y:\n\t\t\t[X, Y]h == X(Yh) - Y(Xh).   \t\t(4)\
\t\t\t"
}], "Text",
 CellChangeTimes->{{3.6356927501859303`*^9, 3.6356927537895365`*^9}, {
   3.635692788752198*^9, 3.635693159347251*^9}, {3.635773169917546*^9, 
   3.6357731761127567`*^9}, 3.635774389554451*^9, 
   3.6452771332254515`*^9},ExpressionUUID->"1deffc93-2f8c-4180-b619-\
65ee4b228cf7"],

Cell[TextData[StyleBox["Proof.",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.635693166882064*^9, 3.6356931945605125`*^9}, {
   3.6357623788225603`*^9, 3.635762388681778*^9}, {3.6357624220844364`*^9, 
   3.635762438482065*^9}, {3.635762470169721*^9, 3.635762530656028*^9}, 
   3.635763438114993*^9},ExpressionUUID->"e1f24039-6c87-429f-ab12-\
854435f4c2ed"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"X", "[", 
     RowBox[{"Y", "[", "h", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
   RowBox[{
    RowBox[{"Y", "[", 
     RowBox[{"X", "[", "h", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
   RowBox[{
    RowBox[{"Z", "[", "h", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703747645095579*^9, 
  3.7037476653910627`*^9}},ExpressionUUID->"2b23253a-923f-4a40-8a58-\
a33051cd28a3"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{
  3.6355207782339735`*^9, 3.635576984489523*^9, 3.6355770923793125`*^9, 
   3.6355772241059437`*^9, 3.6355772746032324`*^9, 3.6355781334315414`*^9, 
   3.635578227468506*^9, 3.6355784615001173`*^9, {3.635693246698805*^9, 
   3.635693254873219*^9}},ExpressionUUID->"167a5190-df2f-414e-ad84-\
1f758e8fda1c"],

Cell[TextData[{
 "Now we will show \n",
 StyleBox["Proposition 4.",
  FontColor->RGBColor[0, 0, 1]],
 " The real infinite dimensional vector space \[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "a9e60a98-56c2-41bf-9b2e-066dc5587746"],
 " of all smooth vector fields on the smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "efc6e2a2-448c-42de-9b18-ddadc09a12d5"],
 " with the commutator as operation is a ",
 ButtonBox["Lie algebra",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/LieAlgebra.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/LieAlgebra.html"],
 " over the real field \[DoubleStruckCapitalR].\n\n",
 StyleBox["Proof,",
  FontColor->RGBColor[0, 0, 1]],
 " We check the defining relations, see e. g. ",
 StyleBox[ButtonBox["MathWorld",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/LieAlgebra.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/LieAlgebra.html"],
  FontSlant->"Italic"],
 ButtonBox[" ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/LieAlgebra.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/LieAlgebra.html"],
 " or ",
 ButtonBox["S. Helgason",
  BaseStyle->"Hyperlink",
  ButtonData->"H78"],
 ", p. 99.",
 StyleBox[" ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["The relation [X,X] = = o, o the null vector field",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.635752607129481*^9, 3.635752730058098*^9}, {
   3.6357528665587387`*^9, 3.6357529045820055`*^9}, {3.635752953587692*^9, 
   3.6357529711417227`*^9}, {3.6357530797601147`*^9, 3.6357531012921524`*^9}, 
   3.635753142371025*^9, {3.6357532098792057`*^9, 3.6357532098792057`*^9}, {
   3.6357535151392584`*^9, 3.6357535600285378`*^9}, {3.6357536110290275`*^9, 
   3.635753678226346*^9}, 3.635753721122288*^9, {3.635753858481949*^9, 
   3.6357538696887693`*^9}, {3.6357551960074797`*^9, 3.635755196740681*^9}, {
   3.6357621954566374`*^9, 3.635762205208655*^9}, {3.6357622412799187`*^9, 
   3.6357622412799187`*^9}, {3.635762275383378*^9, 3.635762292574608*^9}, {
   3.6357630241036973`*^9, 3.6357630480839396`*^9}, {3.63576309937263*^9, 
   3.635763119703466*^9}, 3.635763153028125*^9, 3.6357633742530804`*^9, 
   3.6393134949652476`*^9},ExpressionUUID->"3665eeb7-5740-4a3f-82a7-\
1428b3e65a25"],

Cell[TextData[Cell[BoxData[
 RowBox[{
  RowBox[{"o", "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{"null", "[", 
   RowBox[{"Length", "[", 
    RowBox[{"{", "x", "}"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.635762538629642*^9, 3.6357625583188763`*^9}, {
   3.635762597759746*^9, 3.6357626388854184`*^9}, 
   3.635762777006462*^9},ExpressionUUID->"2a579df7-ea4f-436c-8da2-\
c630913ca485"]], "Text",
 Evaluatable->True,
 InitializationCell->True,
 CellChangeTimes->{{3.635752607129481*^9, 3.635752730058098*^9}, {
   3.6357528665587387`*^9, 3.6357529045820055`*^9}, {3.635752953587692*^9, 
   3.6357529711417227`*^9}, {3.6357530797601147`*^9, 3.6357531012921524`*^9}, 
   3.635753142371025*^9, {3.6357532098792057`*^9, 3.6357532098792057`*^9}, {
   3.6357535151392584`*^9, 3.6357535600285378`*^9}, {3.6357536110290275`*^9, 
   3.635753678226346*^9}, 3.635753721122288*^9, {3.635753858481949*^9, 
   3.6357538696887693`*^9}, {3.6357551960074797`*^9, 3.635755196740681*^9}, {
   3.6357621954566374`*^9, 3.635762205208655*^9}, {3.6357622412799187`*^9, 
   3.6357622412799187`*^9}, {3.635762275383378*^9, 3.635762292574608*^9}, {
   3.6357630241036973`*^9, 3.6357630480839396`*^9}, {3.63576309937263*^9, 
   3.635763119703466*^9}, {3.635763153028125*^9, 3.635763190833991*^9}, 
   3.6357632262334538`*^9},ExpressionUUID->"9c0db023-da41-4ab6-84a8-\
c5114a1b1015"],

Cell["is trivial by definition", "Text",
 CellChangeTimes->{{3.635763871824959*^9, 
  3.635763897865405*^9}},ExpressionUUID->"969983f2-1ea0-42fd-887a-\
a9c4ddb4edfa"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"comm", "[", 
    RowBox[{"vx", ",", "vx"}], "]"}], "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{"o", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.63931439015722*^9, 
  3.639314393963627*^9}},ExpressionUUID->"bd4f97d8-1e50-41ce-8f55-\
9aab448ab44a"],

Cell["This implies [Y,X] = - [x,y]:", "Text",
 CellChangeTimes->{{3.6357694359743843`*^9, 
  3.635769501252899*^9}},ExpressionUUID->"230f95f0-cb5d-4562-a783-\
69974597950c"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vx", ",", "vy"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "+", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vy", ",", "vx"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.639314020561371*^9, 3.6393140811050773`*^9}, {
   3.639314112351932*^9, 3.639314181194853*^9}, 
   3.6393142435949626`*^9},ExpressionUUID->"f7c2302a-5cd0-4a19-8ae0-\
a10baae5ffc8"],

Cell["The commutator is bilinear: [Z,x+\[Alpha]\[CenterDot]Y] =[Z,X] + \
\[Alpha]\[CenterDot][Z,Y]. Set", "Text",
 CellChangeTimes->{{3.6357697162034783`*^9, 3.6357697419777236`*^9}, {
   3.635769780155991*^9, 3.6357699106028214`*^9}, 
   3.7028929589502296`*^9},ExpressionUUID->"b29d711b-4076-453e-9573-\
a0dec06d4e82"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vz", "[", "x__", "]"}], ":=", 
   RowBox[{"vec", "[", 
    RowBox[{"fz", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"vz", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6357684419924297`*^9, {3.6357684779564934`*^9, 3.635768543718609*^9}, {
   3.635768606082919*^9, 3.6357686511105986`*^9}, 3.635768682283454*^9, 
   3.6393145025554175`*^9},ExpressionUUID->"8fd9da63-3e44-45b7-841a-\
231c81d3efcc"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vz", ",", 
      RowBox[{"plus", "[", 
       RowBox[{"vx", ",", "vy"}], "]"}]}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "-", " ", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vz", ",", "vx"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "-", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vz", ",", "vy"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635768801991865*^9, 3.635768886930415*^9}, {
  3.63576911651982*^9, 3.6357691283290415`*^9}, {3.635769183857539*^9, 
  3.635769308596759*^9}, {3.639314601319191*^9, 
  3.6393147056053743`*^9}},ExpressionUUID->"2d924770-284c-4b0b-b18c-\
59b95994b33f"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vz", ",", 
      RowBox[{"3", "*", "vx"}]}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "-", " ", 
   RowBox[{"3", "*", " ", 
    RowBox[{
     RowBox[{"comm", "[", 
      RowBox[{"vz", ",", "vx"}], "]"}], "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635768801991865*^9, 3.635768886930415*^9}, {
  3.63576911651982*^9, 3.6357691283290415`*^9}, {3.635769183857539*^9, 
  3.635769308596759*^9}, {3.635769591805459*^9, 3.635769692079836*^9}, {
  3.6393147452294436`*^9, 3.6393147747758956`*^9}, {3.639314850124028*^9, 
  3.63931487419487*^9}, {3.6393149057069254`*^9, 3.6393149253473597`*^9}, {
  3.6393149655018306`*^9, 3.63931502787074*^9}, {3.6393151500033545`*^9, 
  3.639315181921011*^9}},ExpressionUUID->"a3fc2d97-b5b2-46e8-92b2-\
662b87073480"],

Cell[TextData[{
 "Now we will prove the ",
 StyleBox["Jacobi identity",
  FontColor->RGBColor[0, 0, 1]],
 ":\n[X,[Y,Z]] + [Y,[Z,X]] + [Z,[X,Y]] = 0"
}], "Text",
 CellChangeTimes->{{3.635772024577848*^9, 3.6357720594001093`*^9}, 
   3.635772400626666*^9, {3.635772500189364*^9, 3.6357725882353196`*^9}, 
   3.635772670130864*^9, 
   3.635773561180005*^9},ExpressionUUID->"ad131832-2676-4828-a70a-\
ee179e3e95d8"],

Cell["We set", "Text",
 CellChangeTimes->{
  3.6357726948287077`*^9},ExpressionUUID->"8662f022-f186-4b62-a05f-\
b4f939a76322"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"x1", "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vy", ",", "vz"}], "]"}], "[", "x", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6357727311973715`*^9, 3.635772753962812*^9}, 
   3.635772788005072*^9, {3.6393152136202664`*^9, 
   3.639315285068392*^9}},ExpressionUUID->"1ff3237e-1ab0-4893-aefa-\
925840c480ea"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"y1", "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vz", ",", "vx"}], "]"}], "[", "x", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6357727621060266`*^9, 3.635772794463483*^9}, {
   3.639315323538059*^9, 3.6393153593713226`*^9}, 
   3.6393156863658967`*^9},ExpressionUUID->"eb7d2fee-dd45-4526-be06-\
bde1d1798145"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"z1", "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vx", ",", "vy"}], "]"}], "[", "x", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6357687135519094`*^9, 3.635768771675012*^9}, 
   3.635772799068492*^9, {3.6393153293256693`*^9, 3.6393153466729*^9}, {
   3.639315377046153*^9, 3.6393153817417617`*^9}, 
   3.6393156988303185`*^9},ExpressionUUID->"03e02d5e-fb2b-4ed9-887b-\
b7241b9ed73e"],

Cell["Then we calculate", "Text",
 CellChangeTimes->{{3.6357728273689413`*^9, 
  3.6357728373725595`*^9}},ExpressionUUID->"2d208611-6658-4af8-9118-\
182d308c0b14"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vx", ",", "x1"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vy", ",", "y1"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vz", ",", "z1"}], "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635772858542796*^9, 3.6357729510191593`*^9}, {
  3.6357729911932306`*^9, 3.635773003876053*^9}, {3.6357732484112844`*^9, 
  3.635773266307516*^9}, {3.6393154170290236`*^9, 
  3.6393154607559004`*^9}},ExpressionUUID->"d68fb522-b8bf-4cc1-87e6-\
9d263edaa1b5"],

Cell["\<\
This proves the Jacobi identity, and Proposition 4 is proved. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.635773360934683*^9, 3.6357733904237347`*^9}, {
  3.635773578422036*^9, 3.6357736253695188`*^9}, {3.6357737031194563`*^9, 
  3.6357737031194563`*^9}, {3.6393155449180484`*^9, 3.639315546696451*^9}, {
  3.6393157583600235`*^9, 
  3.6393157728524485`*^9}},ExpressionUUID->"84cfda87-95c8-4600-926c-\
006a0cf62fa6"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "x1", ",", "y1", ",", "z1", ",", "vx", ",", "vy", ",", "vz", ",", "X", ",", 
   "Y", ",", "Z", ",", "Zc"}], "]"}]], "Input",
 CellChangeTimes->{{3.635774560117353*^9, 3.6357746158516517`*^9}, {
  3.6393155725768967`*^9, 3.6393155964293385`*^9}, {3.7008061355227213`*^9, 
  3.700806143095348*^9}, {3.703750992788652*^9, 
  3.703751002284453*^9}},ExpressionUUID->"974de821-9faf-4fc1-97b0-\
7eb98496e636"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3.5. Local Bases. Commuting Properties", "Subsubsection",
 CellChangeTimes->{{3.654407306428541*^9, 3.654407323372377*^9}, {
   3.65440751092295*^9, 3.654407521563006*^9}, {3.654407597898327*^9, 
   3.654407636466126*^9}, {3.6544079281439447`*^9, 3.654408036406987*^9}, {
   3.6544914586025877`*^9, 3.654491461666752*^9}, {3.6634007369326572`*^9, 
   3.663400740164215*^9}, {3.6634010566897173`*^9, 3.663401059033663*^9}, 
   3.663402798480073*^9, {3.668597512449362*^9, 3.668597515480977*^9}, 
   3.668602079659835*^9, 3.700806208987597*^9, 3.700807104323606*^9, 
   3.702893108037691*^9, 3.702897871230674*^9, {3.7037510772116337`*^9, 
   3.703751079627425*^9}, 
   3.7037719177494173`*^9},ExpressionUUID->"deb96fd6-7865-420b-a5ec-\
796f17959ce5"],

Cell["We consider the local cobasis fields", "Text",
 CellChangeTimes->{{3.654351996446458*^9, 
  3.654352012366342*^9}},ExpressionUUID->"2f84f1c2-9701-48e2-b3bc-\
126a9ea17714"],

Cell[BoxData["dim"], "Input",
 CellChangeTimes->{{3.6634010438390837`*^9, 
  3.663401044857748*^9}},ExpressionUUID->"52df7798-ad6d-4f5b-a4ed-\
19c0f8f41be4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "dx", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"dx", "[", "i_", "]"}], "[", "x__", "]"}], ":=", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{"stb", "[", 
     RowBox[{"i", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}]}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.654350250478973*^9, 3.6543502779495373`*^9}, {
   3.6543509724721603`*^9, 3.654350979087448*^9}, {3.6543510184149227`*^9, 
   3.6543511091426353`*^9}, {3.654351188639935*^9, 3.6543512442138453`*^9}, 
   3.659426747539158*^9, {3.663401299976488*^9, 3.66340130516055*^9}, {
   3.663401338000621*^9, 
   3.6634014119601*^9}},ExpressionUUID->"9972e8ff-d9d2-4a7a-9523-\
21cb3955dc47"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{
   RowBox[{"dx", "[", "2", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.654351262468926*^9, 3.6543512722212143`*^9}, {
  3.66340143872011*^9, 3.6634014572957487`*^9}, {3.663402205933487*^9, 
  3.663402212180183*^9}},ExpressionUUID->"17cf600e-f4c7-4627-b63c-\
3f77ef3cdbd5"],

Cell["and the local basis fields", "Text",
 CellChangeTimes->{{3.654352046470023*^9, 
  3.6543520550461903`*^9}},ExpressionUUID->"b5e07a2c-8cf8-4196-acff-\
a25ad7390f94"],

Cell[BoxData[
 RowBox[{"?", "tx\[Iota]"}]], "Input",
 CellChangeTimes->{{3.6691898773841467`*^9, 
  3.669189893871702*^9}},ExpressionUUID->"008ccfb7-bc1e-40fa-99d1-\
dab59eb09c0b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tx\[Iota]", "[", "2", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6691899285936823`*^9, 3.6691899485596313`*^9}, {
  3.6691899937046556`*^9, 
  3.669189995655189*^9}},ExpressionUUID->"3210c851-0bb5-424a-a90e-\
21d4c8289776"],

Cell[BoxData[
 RowBox[{"?", "bx\[Iota]"}]], "Input",
 CellChangeTimes->{{3.668601572717271*^9, 3.6686015750584297`*^9}, 
   3.703741453039521*^9},ExpressionUUID->"661d65e5-1733-4269-b4cc-\
458f89c94c8b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bx\[Iota]", "[", "2", "]"}], "[", 
  RowBox[{
   RowBox[{"x", "[", "1", "]"}], ",", 
   RowBox[{"x", "[", "2", "]"}], ",", 
   RowBox[{"x", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.635141514952622*^9, 3.6351415544724913`*^9}, {
   3.6363810022930546`*^9, 3.636381017830682*^9}, {3.66860159282655*^9, 
   3.668601632898295*^9}, 
   3.7037414556150017`*^9},ExpressionUUID->"6be54b19-f566-433c-8e64-\
7d76caba0b02"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6351415709189205`*^9, 3.63514158757215*^9}, 
   3.6634022588140593`*^9},ExpressionUUID->"5b2c808b-661c-4790-8bf0-\
83e235462869"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ct", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"dx", "[", "i", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
      RowBox[{
       RowBox[{"bx\[Iota]", "[", "j", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", "1", ",", "1"}], "]"}], 
   "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.663401673479241*^9, 3.663401882430228*^9}, {
   3.663401941590436*^9, 3.6634019543817377`*^9}, {3.6686016849227*^9, 
   3.668601692010118*^9}, 
   3.7037414616083393`*^9},ExpressionUUID->"81712864-4be3-4442-8ff0-\
7a0ad48812eb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"ct", "[", 
     RowBox[{"i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6634018933941307`*^9, 3.663401899421689*^9}, {
  3.663401974198511*^9, 
  3.6634020739891567`*^9}},ExpressionUUID->"0f7bef9b-dbbb-4364-a329-\
dea677eae86e"],

Cell["\<\
The tensor product of contravariant tensors is not commutative:\
\>", "Text",
 CellChangeTimes->{{3.6543520964456167`*^9, 
  3.654352128725589*^9}},ExpressionUUID->"c319fe2a-296c-474d-bf10-\
6f12cb9f5c3a"],

Cell[BoxData[
 RowBox[{"t12", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"bx\[Iota]", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "\[CircleTimes]", 
   RowBox[{
    RowBox[{"bx\[Iota]", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.654350749166081*^9, 3.654350801385174*^9}, {
  3.668601817977141*^9, 3.668601840489119*^9}, {3.703741465535082*^9, 
  3.7037414673915787`*^9}},ExpressionUUID->"dc28981c-bf3e-4b4b-b7c6-\
ba0d10768fbf"],

Cell[BoxData[
 RowBox[{"t21", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"bx\[Iota]", "[", "2", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "\[CircleTimes]", 
   RowBox[{
    RowBox[{"bx\[Iota]", "[", "1", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.65435081118289*^9, 3.6543508543767757`*^9}, {
  3.668601863271875*^9, 3.668601879935093*^9}, {3.703741469350092*^9, 
  3.703741470383589*^9}},ExpressionUUID->"e4464476-49e2-4480-b464-\
a59d341733cb"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"t12", ",", "t21"}], "]"}]], "Input",
 CellChangeTimes->{{3.654350867824922*^9, 
  3.654350878872121*^9}},ExpressionUUID->"549431f2-bc2d-4a89-9b13-\
e91035ca81e6"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{"t12", "-", "t21"}], "]"}]], "Input",
 CellChangeTimes->{{3.6634023144460077`*^9, 
  3.6634023268918333`*^9}},ExpressionUUID->"364ade12-7ac0-40f5-a806-\
3bed901823a4"],

Cell["The same is true for the tensor product of covectors:", "Text",
 CellChangeTimes->{{3.654352293700042*^9, 
  3.654352314707747*^9}},ExpressionUUID->"386a88f0-39ee-4a43-9472-\
2242435aa375"],

Cell[BoxData[
 RowBox[{"td12", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dx", "[", "1", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
   RowBox[{
    RowBox[{"dx", "[", "3", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.654352172595365*^9, 3.6543522048613577`*^9}, {
   3.663402380467443*^9, 3.663402398965084*^9}, 
   3.668601951631404*^9},ExpressionUUID->"9b8514dd-e091-4432-953a-\
bebbb58bf306"],

Cell[BoxData[
 RowBox[{"td21", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dx", "[", "3", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
   RowBox[{
    RowBox[{"dx", "[", "1", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.654352172595365*^9, 3.654352235596807*^9}, {
  3.663402419467573*^9, 
  3.6634024329642887`*^9}},ExpressionUUID->"0f5ad227-670a-4e5a-bbb5-\
aef1ba12c295"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{"td12", " ", "-", "td21"}], "]"}]], "Input",
 CellChangeTimes->{{3.654352260982695*^9, 3.654352273316812*^9}, {
  3.6634024570211554`*^9, 
  3.6634024690910263`*^9}},ExpressionUUID->"de382ef4-a4cb-4368-a833-\
c3e5d2bcf4a2"],

Cell["But the tensor product of a vector and a covector commutes:", "Text",
 CellChangeTimes->{{3.654352343915429*^9, 
  3.6543523727233*^9}},ExpressionUUID->"25470bba-69fd-46cc-8a2e-6c4b28e5d9c0"],

Cell[BoxData[
 RowBox[{"tb12", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dx", "[", "1", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
   RowBox[{
    RowBox[{"bx\[Iota]", "[", "2", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6543513612852297`*^9, 3.654351409316658*^9}, {
   3.663402496573287*^9, 3.663402509483577*^9}, {3.663402593154537*^9, 
   3.663402603163509*^9}, 3.668601989477944*^9, 
   3.703741479943433*^9},ExpressionUUID->"223f3cc0-1874-44d0-8e53-\
5764a818e7ef"],

Cell[BoxData[
 RowBox[{"tb21", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"bx\[Iota]", "[", "2", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
   RowBox[{
    RowBox[{"dx", "[", "1", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6543513612852297`*^9, 3.654351465115878*^9}, {
   3.663402569156457*^9, 3.663402576075605*^9}, 3.668601996502818*^9, 
   3.703741481695735*^9},ExpressionUUID->"28684e08-8f9f-42ba-b4df-\
1d524b7b2fe1"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{"tb12", "-", "tb21"}], "]"}]], "Input",
 CellChangeTimes->{{3.654351480518528*^9, 3.6543515036508913`*^9}, {
  3.663402522635399*^9, 
  3.663402532314999*^9}},ExpressionUUID->"7fcacf90-296d-461d-8aae-\
9be6adcfe1fb"],

Cell[BoxData[
 RowBox[{"?", "covec"}]], "Input",
 CellChangeTimes->{{3.654352427439436*^9, 
  3.654352429875382*^9}},ExpressionUUID->"f784bb64-c835-4d90-9b8d-\
631968b147c2"],

Cell[BoxData[
 RowBox[{"covec", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], ")"}], ",", "3"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.654352459914579*^9, 3.6543524706590157`*^9}, 
   3.654352531691553*^9, {3.6543525662271147`*^9, 3.654352592274041*^9}, {
   3.654352635066104*^9, 
   3.6543527352893143`*^9}},ExpressionUUID->"cf1528c6-05c8-4a12-a914-\
5912fcb4b877"],

Cell["\<\
Generally, the tensor product of a vector with a covector commutes:\
\>", "Text",
 CellChangeTimes->{{3.654353093229456*^9, 
  3.654353133437132*^9}},ExpressionUUID->"d825f16c-1325-42ab-b8f5-\
12787c46cc65"],

Cell[BoxData[
 RowBox[{"w1", " ", "=", " ", 
  RowBox[{"covecTotensor", "[", 
   RowBox[{"covec", "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}], ",", 
        RowBox[{"x", "[", "3", "]"}]}], "]"}], ")"}], ",", "3"}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.654352392227189*^9, 3.654352418926034*^9}, {
  3.6543527698329763`*^9, 
  3.6543527839133253`*^9}},ExpressionUUID->"5c6eed8a-535c-4f44-92ef-\
e6c066d638f8"],

Cell[BoxData[
 RowBox[{"tx", " ", "=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vec", "[", 
    RowBox[{"b", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.654352816094161*^9, 3.6543528675678988`*^9}, 
   3.659426747553191*^9, {3.663402682962036*^9, 
   3.663402687593391*^9}},ExpressionUUID->"19eeb265-1c01-4e1d-89fd-\
a64d85bac786"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{"w1", "\[CircleTimes]", "tx"}], ",", 
   RowBox[{"tx", "\[CircleTimes]", "w1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6543528777794113`*^9, 
  3.654352919703177*^9}},ExpressionUUID->"470ddd12-ee2d-41d3-9280-\
f1d356ae1672"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{
   RowBox[{"w1", "\[CircleTimes]", "tx"}], "-", " ", 
   RowBox[{"tx", "\[CircleTimes]", "w1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.654352939027801*^9, 3.654352969199026*^9}, {
  3.663402726577486*^9, 
  3.663402765465905*^9}},ExpressionUUID->"37bf2a11-be96-4de4-b8bf-\
2f608d4f40c7"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.4. Exterior Differential in Tensor Description", "Subsection",
 CellChangeTimes->{{3.5725953757423115`*^9, 3.572595385461128*^9}, {
   3.58666439764649*^9, 3.5866644089011335`*^9}, {3.5872724756311846`*^9, 
   3.58727247921439*^9}, 3.599568542111173*^9, 3.599804993627187*^9, 
   3.6176277747189283`*^9, 3.617632840802114*^9, 3.627372710290759*^9, 
   3.6273740816423683`*^9, {3.635829521178998*^9, 3.6358295241742034`*^9}, {
   3.6403304970034513`*^9, 3.640330499624256*^9}, 3.6403413314538183`*^9, {
   3.6405844365717707`*^9, 3.640584438786975*^9}, 3.644933740880417*^9, 
   3.6453631363663297`*^9, 3.6454349169332085`*^9, 3.700807112692257*^9, {
   3.701512579023934*^9, 3.701512596359703*^9}, {3.70151266476719*^9, 
   3.7015126912470627`*^9}, 3.701834308475868*^9, {3.702020255250578*^9, 
   3.702020258234269*^9}, 3.702041611087635*^9, 3.7030429891242523`*^9, {
   3.703045798282001*^9, 3.703045820601934*^9}, 3.703771925526065*^9, 
   3.7040043517804193`*^9, {3.704030461034874*^9, 
   3.704030472843629*^9}},ExpressionUUID->"6ffe34a5-09ee-45cf-a5a8-\
cc8e19e6b1f5"],

Cell[TextData[{
 "Mathematica functions modelling the exterior differential calculus for \
p-forms and antisymmetric covariant tensors are contained in the package \
eudiffgeov3.m. Their applications to p-forms described in the notebook \
euklsfv3.nb in [",
 Cell[BoxData[
  FormBox[
   ButtonBox["EDG",
    BaseStyle->"Hyperlink",
    ButtonData->"EDG"], TraditionalForm]],ExpressionUUID->
  "11eb4b37-0c4e-4587-a634-9148e2ce0440"],
 "]. The exterior differential is denoted by dd.\nIt remains to consider the \
exterior differential in the realm of tensors."
}], "Text",
 CellChangeTimes->{{3.701500687594152*^9, 3.7015008098650217`*^9}, {
   3.701500959352166*^9, 3.70150107569534*^9}, {3.70150116107064*^9, 
   3.701501174125766*^9}, {3.7015012136707563`*^9, 3.701501289437789*^9}, {
   3.701501702922504*^9, 3.701501725938362*^9}, {3.701501772845037*^9, 
   3.701501772848337*^9}, 3.7015796609132233`*^9, {3.7016662571097183`*^9, 
   3.701666276524918*^9}, {3.702897999477808*^9, 3.702898021327345*^9}, {
   3.706245537385065*^9, 
   3.706245564765093*^9}},ExpressionUUID->"99a462a7-e692-4f51-b3e5-\
92b50475c105"],

Cell[BoxData[
 RowBox[{"?", "dd"}]], "Input",
 CellChangeTimes->{{3.705990860339202*^9, 
  3.7059908619437237`*^9}},ExpressionUUID->"287f5d50-cfeb-4a2a-982d-\
47765ee39670"],

Cell[CellGroupData[{

Cell["\<\
2.4.1. The  Exterior Algebra of Antisymmetric Covariant Tensor Fields\
\>", "Subsubsection",
 CellChangeTimes->{{3.586665161894202*^9, 3.586665168125559*^9}, {
   3.5872725143584003`*^9, 3.587272520677761*^9}, {3.60913869538486*^9, 
   3.6091387007281656`*^9}, 3.6092316026384597`*^9, {3.6093094046875467`*^9, 
   3.6093094080137367`*^9}, 3.609309574326249*^9, {3.6095608566429873`*^9, 
   3.6095608600281935`*^9}, 3.6177078269291887`*^9, 3.6177190550258956`*^9, 
   3.62744137111684*^9, 3.627442434835908*^9, 3.635839357673564*^9, {
   3.636124716479395*^9, 3.636124719786601*^9}, {3.639825166739862*^9, 
   3.639825174633476*^9}, {3.6398252063015313`*^9, 3.6398252091719365`*^9}, {
   3.6398252530392137`*^9, 3.639825256237219*^9}, 3.645436271891088*^9, 
   3.645588834302208*^9, 3.6455888654710627`*^9, 3.6686084046922483`*^9, 
   3.6686558244040823`*^9, 3.701002236813325*^9, 3.701006647977103*^9, {
   3.701512493080533*^9, 3.701512534520307*^9}, 3.701512641160029*^9, {
   3.701512709374626*^9, 3.701512743486472*^9}, {3.701512798797892*^9, 
   3.7015128516456947`*^9}, {3.701512899701085*^9, 3.701512908741053*^9}, 
   3.7015795823188877`*^9, 3.702897882351418*^9, 
   3.7029090864040403`*^9},ExpressionUUID->"62d19048-24bf-4f8b-8b0c-\
417b1343377a"],

Cell[TextData[{
 "The exterior algebra over finite-dimensional vector spaces has been \
presented in the notebook vectensalgv3.nb contained in the package \
riemann.zip, see [",
 ButtonBox["IDG",
  BaseStyle->"Hyperlink",
  ButtonData->"IDG"],
 "]. The complete tensor algebra over a finite-dimensional real vector space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "62f75cdf-65f7-4bb6-afea-00a1fbd900f7"],
 " exists at each point x of an n-dimensional smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "ad1eb038-e3a2-4e6e-9532-0ff0a0c201cc"],
 ": take  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "89c2cfd8-35c2-42af-9004-013f34577cad"],
 " = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "x"], TraditionalForm]],ExpressionUUID->
  "2c762c24-b1ca-432d-bcc9-93f42ad30a0c"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["M", "n"], " ", ".", " ", "We"}], " ", "used", " ", 
    "this", " ", "fact", " "}], TraditionalForm]],ExpressionUUID->
  "c8f3b437-2973-49f4-9649-b290c592b8c7"],
 " to define  the tensor bundles of the manifold and the tensor fields as \
smooth sections of the tensor bundles. Pointwise application of the algebraic \
tensor operation led to the corresponding algebraic operations for tensor \
fields on the manifold, see Proposition 1 in Subsection 2.3.1 above.  We \
consider the tensor bundle"
}], "Text",
 CellChangeTimes->{{3.7015138054052277`*^9, 3.7015139596046047`*^9}, {
   3.701514004076236*^9, 3.7015140067560883`*^9}, {3.701514077467602*^9, 
   3.70151410782734*^9}, {3.701514145982675*^9, 3.701514155683085*^9}, {
   3.701514353601512*^9, 3.7015144105845737`*^9}, 3.701514466944685*^9, {
   3.701514517146022*^9, 3.701514593087565*^9}, {3.701514640087483*^9, 
   3.70151468679895*^9}, {3.701514852837772*^9, 3.70151490751105*^9}, {
   3.701514951073934*^9, 3.701514952973165*^9}, {3.701515143646806*^9, 
   3.7015153519142714`*^9}, {3.7015154285617733`*^9, 3.701515512481223*^9}, {
   3.701515959068778*^9, 3.701515996509369*^9}, 3.7015160580928288`*^9, {
   3.701577831522688*^9, 3.701577835895813*^9}, 3.701577877370062*^9, {
   3.701578258382229*^9, 3.701578281286583*^9}, {3.7062456609727182`*^9, 
   3.7062456722740107`*^9}},ExpressionUUID->"865102fa-29e3-4ed3-b85a-\
d7cff6c68859"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["T", 
      RowBox[{"(", 
       RowBox[{"p", ",", "0"}], ")"}]], 
     SuperscriptBox["M", "n"]}], "  ", ":=", 
    RowBox[{
     FormBox[
      UnderscriptBox["\[Union]", 
       RowBox[{"x", "\[Element]", 
        SuperscriptBox["M", 
         RowBox[{"n", " "}]]}]],
      TraditionalForm], " ", 
     RowBox[{
      FormBox[
       SubsuperscriptBox["T", "x", 
        RowBox[{"(", 
         RowBox[{"p", ",", "0"}], ")"}]],
       TraditionalForm], 
      FormBox[
       SuperscriptBox["M", "n"],
       TraditionalForm]}]}]}], ";", "  ", 
   RowBox[{
    RowBox[{
     FormBox[
      SubsuperscriptBox["T", "x", 
       RowBox[{"(", 
        RowBox[{"p", ",", "0"}], ")"}]],
      TraditionalForm], 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}], "  ", ":=", " ", 
    RowBox[{
     OverscriptBox["\[CircleTimes]", "p"], " ", 
     RowBox[{
      SubsuperscriptBox["T", "x", "*"], 
      SuperscriptBox["M", 
       RowBox[{"n", " "}]]}]}]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.701578403226275*^9, 3.701578432570176*^9}, 
   3.70157854408764*^9, {3.701578579721982*^9, 3.7015786584324217`*^9}, {
   3.701578781543324*^9, 
   3.7015787977033052`*^9}},ExpressionUUID->"741a6a12-52e7-4e5f-9ea3-\
d42f7c8fffad"],

Cell[TextData[{
 "of p-fold covariant tensors over the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701578403226275*^9, 3.701578432570176*^9}, 
    3.70157854408764*^9, {3.701578579721982*^9, 3.7015786584324217`*^9}, {
    3.701578781543324*^9, 3.7015787977033052`*^9}},ExpressionUUID->
  "d1b48194-9bc4-4da7-8c6f-7c44b89b07ae"],
 ". The elements of the  exterior algebra over a vector space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"V", " "}], "n"], TraditionalForm]],ExpressionUUID->
  "21e71ae8-3286-4f8f-a7ba-f0d2457e66dd"],
 " are the antisymmetric covariant tensors \[Omega] \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Wedge]", "p"], TraditionalForm]],ExpressionUUID->
  "332f90ec-9475-483e-9e5d-7b7eae43a01b"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"V", " "}], 
    RowBox[{"n", "*"}]], TraditionalForm]],ExpressionUUID->
  "d87fb786-ead0-43c8-b564-92c94e77e2d3"],
 ", p = 0,1,...,n; remember that the antisymmetric tensors of degree p for \
p>n are the nulltensor. The exterior algebra over ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "d393281a-4abd-4709-83ad-65d50ede32b5"],
 " is the vector space of the direct sum"
}], "Text",
 CellChangeTimes->{{3.7015788451938963`*^9, 3.701578862270339*^9}, {
   3.701578926405764*^9, 3.7015789794453573`*^9}, {3.701579049900782*^9, 
   3.701579234763863*^9}, {3.701579265787715*^9, 3.701579350986731*^9}, {
   3.701579763927245*^9, 3.701579957845912*^9}, {3.701579989158063*^9, 
   3.701580081676476*^9}, {3.701580365165197*^9, 3.701580365165584*^9}, 
   3.701600320357636*^9, {3.701666568002577*^9, 
   3.7016665777218943`*^9}},ExpressionUUID->"0b1c6407-f2dc-43f9-930c-\
b021e0b6eaab"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Wedge]", 
   SuperscriptBox["V", "n"]}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalR]", "\[CirclePlus]", 
     SuperscriptBox[
      SuperscriptBox["V", "n"], "*"], "\[CirclePlus]", 
     FormBox[
      RowBox[{
       SuperscriptBox["\[Wedge]", "2"], 
       SuperscriptBox[
        RowBox[{"V", " "}], 
        RowBox[{"n", "*"}]]}],
      TraditionalForm], "\[CirclePlus]"}], "..."}], "\[CirclePlus]", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Wedge]", "n"], 
     SuperscriptBox[
      RowBox[{"V", " "}], 
      RowBox[{"n", "*"}]]}],
    TraditionalForm]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.7015801087241573`*^9, 3.701580150867963*^9}, {
   3.701580190149034*^9, 3.7015802102514067`*^9}, {3.701580244410735*^9, 
   3.701580259682967*^9}, 3.701580298751089*^9, {3.701580405902954*^9, 
   3.701580458321727*^9}, 3.701580520873383*^9, 3.701580614785187*^9, 
   3.7016665977067633`*^9},ExpressionUUID->"37627a9d-46c1-480b-9a36-\
9590baf77bef"],

Cell["provided with the wedge product", "Text",
 CellChangeTimes->{{3.70158086158762*^9, 3.701580990172077*^9}, {
  3.701666618192988*^9, 
  3.701666624032501*^9}},ExpressionUUID->"43fc2ecb-66ce-44c1-b0fc-\
eb84502398c9"],

Cell[BoxData[
 RowBox[{"?", "wedge"}]], "Input",
 CellChangeTimes->{{3.639825362099005*^9, 
  3.63982536490701*^9}},ExpressionUUID->"1b81d7d2-77d2-4a77-8d47-\
6f584387d7ee"],

Cell[BoxData[
 RowBox[{"Attributes", "[", "wedge", "]"}]], "Input",
 CellChangeTimes->{{3.6455891449299536`*^9, 
  3.6455891545083704`*^9}},ExpressionUUID->"69f19553-cf61-4e40-a028-\
d11b6357d894"],

Cell["\<\
The wedge product is associative, bilinear, and alternating; it satisfies\
\>", "Text",
 CellChangeTimes->{{3.701585468407844*^9, 3.701585549527438*^9}, 
   3.701586062331477*^9, {3.701586867548916*^9, 
   3.701586891300815*^9}},ExpressionUUID->"4804903a-3977-4f35-9a77-\
14f73a280099"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "\[Wedge]", "\[Tau]"}], "  ", "\[Equal]", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"p", "\[CenterDot]", "q"}]], 
     RowBox[{"\[Tau]", "\[Wedge]", "\[Sigma]"}], " ", "for", "  ", 
     "\[Sigma]"}]}], "\[Element]", " ", 
   RowBox[{
    SuperscriptBox["\[Wedge]", "p"], 
    SuperscriptBox[
     RowBox[{"V", " "}], 
     RowBox[{"n", "*"}]]}]}], " ", ",", " ", 
  RowBox[{"\[Tau]", "\[Element]", 
   RowBox[{
    SuperscriptBox["\[Wedge]", "q"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"V", " "}], 
      RowBox[{"n", "*"}]], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.701585680486411*^9, 
  3.701585929676659*^9}},ExpressionUUID->"65e6955e-5c66-483f-9b70-\
3680932009fe"],

Cell[TextData[{
 "Now we take ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "09fc92ce-1bdc-4b09-86b9-bcc90a969e1c"],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["T", "x"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701578403226275*^9, 3.701578432570176*^9}, 
    3.70157854408764*^9, {3.701578579721982*^9, 3.7015786584324217`*^9}, {
    3.701578781543324*^9, 3.7015787977033052`*^9}},ExpressionUUID->
  "37d32cbe-30af-4123-9067-917662104c9d"],
 " and obtain at each point x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701578403226275*^9, 3.701578432570176*^9}, 
    3.70157854408764*^9, {3.701578579721982*^9, 3.7015786584324217`*^9}, {
    3.701578781543324*^9, 3.7015787977033052`*^9}},ExpressionUUID->
  "6fc17e42-7c2c-4178-b908-74558afaef3e"],
 " the subspace of the antisymmetric tensors of degree q at this point "
}], "Text",
 CellChangeTimes->{{3.701587010467389*^9, 3.701587059730948*^9}, {
   3.7015871648509293`*^9, 3.701587289529415*^9}, {3.7015873554335537`*^9, 
   3.701587392392548*^9}, {3.7016000002809153`*^9, 3.70160000999969*^9}, 
   3.701600453092269*^9},ExpressionUUID->"bb78789f-9499-4e6a-ab46-\
06f096a38691"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["\[Wedge]", "q"], 
   RowBox[{
    SubsuperscriptBox["T", "x", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}]}], "\[Subset]", " ", 
  RowBox[{
   FormBox[
    SubsuperscriptBox["T", "x", 
     RowBox[{"(", 
      RowBox[{"q", ",", "0"}], ")"}]],
    TraditionalForm], 
   RowBox[{
    FormBox[
     SuperscriptBox["M", "n"],
     TraditionalForm], "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.7015874727257957`*^9, 3.7015874956*^9}, {
  3.701587535989147*^9, 3.701587565543221*^9}, {3.7016004652035627`*^9, 
  3.7016004751556683`*^9}},ExpressionUUID->"8579669f-1102-407e-8308-\
75951f2a0a37"],

Cell["The union of these spaces", "Text",
 CellChangeTimes->{{3.701587752790596*^9, 
  3.7015878171008863`*^9}},ExpressionUUID->"00574694-3818-46b6-af5f-\
f28b445fd57b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["\[Wedge]", "q"], 
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}]}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{
     UnderscriptBox[
      RowBox[{" ", "\[Union]"}], 
      RowBox[{"x", "\[Element]", 
       SuperscriptBox["M", 
        RowBox[{"n", " "}]]}]], " ", 
     SuperscriptBox["\[Wedge]", "q"], 
     SubsuperscriptBox["T", "x", "*"]}], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}], "\[Subset]", " ", 
   RowBox[{
    SuperscriptBox["T", 
     RowBox[{"(", 
      RowBox[{"q", ",", "0"}], ")"}]], 
    SuperscriptBox["M", "n"], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
   3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}, {
   3.701600485403276*^9, 
   3.701600502115911*^9}},ExpressionUUID->"70651000-eacf-4931-8b9c-\
7411a87a9177"],

Cell[TextData[{
 "is the bundle of antisymmetric covariant tensors of degree q on the \
manifold ",
 Cell[BoxData[
  SuperscriptBox["M", "n"]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
    3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}},
  ExpressionUUID->"b35a6baf-941d-419a-be03-d206904624ad"],
 ". Considering the coordinates of the tensors it is easy to prove that ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Wedge]", "q"], 
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
    3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}},
  ExpressionUUID->"f3c4e0b5-291f-4a35-9675-674d83862c7c"],
 " is a vector subbundle of ",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{
    SuperscriptBox["T", 
     RowBox[{"(", 
      RowBox[{"q", ",", "0"}], ")"}]], 
    SuperscriptBox["M", "n"], " "}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
    3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}},
  ExpressionUUID->"d6f43ee7-b4c8-4ede-8a12-38ec109c4f27"],
 "whose  projection is the restriction of the projection p: ",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{
    SuperscriptBox["T", 
     RowBox[{"(", 
      RowBox[{"q", ",", "0"}], ")"}]], 
    SuperscriptBox["M", "n"]}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
    3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}},
  ExpressionUUID->"6af143d3-060c-4efe-920d-564ad0904e14"],
 "\[RightArrow] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], TraditionalForm]],ExpressionUUID->
  "b9c0353a-3bad-4908-a48e-74c446cbf905"],
 " and whose fibers are ",
 Cell[BoxData[
  RowBox[{
   FormBox[
    SubsuperscriptBox["T", "x", 
     RowBox[{"(", 
      RowBox[{"q", ",", "0"}], ")"}]],
    TraditionalForm], 
   FormBox[
    SuperscriptBox["M", "n"],
    TraditionalForm]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.7015874727257957`*^9, 3.7015874956*^9}, {
   3.701587535989147*^9, 3.701587565543221*^9}, {3.7016004652035627`*^9, 
   3.7016004751556683`*^9}},ExpressionUUID->
  "5a516e26-b726-4576-b1f9-b1c756a92e39"],
 "."
}], "Text",
 CellChangeTimes->{{3.701588126362391*^9, 3.7015882073059397`*^9}, {
   3.701600082974821*^9, 3.7016001665262527`*^9}, {3.701600197349978*^9, 
   3.701600238285883*^9}, 3.701600432834673*^9, {3.701600515219694*^9, 
   3.701600736916995*^9}, {3.701600783392434*^9, 3.7016008144571667`*^9}, {
   3.701600937375978*^9, 3.701600954960381*^9}, {3.7016010364338503`*^9, 
   3.701601058399457*^9}, {3.701666762818149*^9, 
   3.701666764208082*^9}},ExpressionUUID->"6fda285f-5d9c-452b-a826-\
c45f2348a49c"],

Cell[TextData[{
 StyleBox["Proposition 1.",
  FontColor->RGBColor[0, 0, 1]],
 " If the manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.7015874727257957`*^9, 3.7015874956*^9}, {
   3.701587535989147*^9, 3.701587565543221*^9}, {3.7016004652035627`*^9, 
   3.7016004751556683`*^9}},ExpressionUUID->
  "900ba244-ba21-4254-afe5-03e451086748"],
 " is of class ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["C", "k"], TraditionalForm]],ExpressionUUID->
  "dde88eae-e41e-4f59-884b-6c59acd4f0e6"],
 ", the bundle ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Wedge]", "q"], 
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
    3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}},
  ExpressionUUID->"7f900cbf-0a33-4933-a229-f38d19fb60bf"],
 " is of class ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["C", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]],ExpressionUUID->
  "29fccaab-4c96-499c-bdde-1a90c5154287"],
 "; smooth manifolds have smooth tensor bundles. The smooth sections \
\[Tau][x] of ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Wedge]", "q"], 
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
    3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}, {
    3.701600485403276*^9, 3.701600502115911*^9}},ExpressionUUID->
  "063abc07-7b93-40b1-a799-68d88e7bd2f6"],
 ", p[\[Tau][x]]=x, of the bundle ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Wedge]", "q"], 
   RowBox[{
    SuperscriptBox["T", "*"], 
    SuperscriptBox["M", 
     RowBox[{"n", " "}]]}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701587858309106*^9, 3.7015879610108433`*^9}, 
    3.701588013985444*^9, {3.701588048019964*^9, 3.701588053097406*^9}, {
    3.701600485403276*^9, 3.701600502115911*^9}},ExpressionUUID->
  "22c67561-bb13-4ae2-b36e-d24572dd8dc4"],
 " are named differential forms of degree q on the manifold ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["M", 
    RowBox[{"n", " "}]], ","}]], "Input",ExpressionUUID->
  "43a0d53b-8afc-4e86-896f-dc4de6c1ae5a"],
 " or shortly q-forms on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "b17ec438-2da4-4fbc-bd3e-4369f6aad45f"],
 ". The set of all q-forms denoted by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Wedge]", "q"], 
    SuperscriptBox["M", "n"]}], TraditionalForm]],ExpressionUUID->
  "4a47e63c-e58b-4086-92b3-e07275384189"],
 " with pointwise defined algebraic operations (addition and multiplication \
with real numbers) is an infinite-dimensional real vector space. \
\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.701600990310708*^9, 3.7016010171186733`*^9}, {
  3.701601095154376*^9, 3.70160177196935*^9}, {3.701601802336829*^9, 
  3.70160197604663*^9}, {3.701602033855405*^9, 3.701602125879134*^9}, {
  3.701602160166546*^9, 3.701602171458662*^9}, {3.7016668771365623`*^9, 
  3.701666900247774*^9}},ExpressionUUID->"b29d4197-43d6-4073-a8c8-\
3bcbaaf60444"],

Cell[TextData[{
 "The proof is standard: The charts of ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["\[Wedge]", "q"], 
     RowBox[{
      SuperscriptBox["T", "*"], 
      SuperscriptBox["M", 
       RowBox[{"n", " "}]]}]}]}], TraditionalForm]],ExpressionUUID->
  "31dee222-b741-4ef1-b62b-f05d222bda3a"],
 " are defined by the charts of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "8d234298-a724-4dca-93d4-1c68e465794d"],
 " and the corresponding coordinates of the tensors. To define the algebra of \
differential forms of a manifold we introduce the direct sum"
}], "Text",
 CellChangeTimes->{{3.701602245380158*^9, 3.701602381171483*^9}, {
  3.7016024609864817`*^9, 3.701602467362378*^9}, {3.701610810396534*^9, 
  3.701610810396763*^9}, {3.7016108532276773`*^9, 3.701610895747096*^9}, {
  3.701611867235044*^9, 
  3.701611914338654*^9}},ExpressionUUID->"db1ce299-ad3c-497e-a23f-\
aaaab13cef5c"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Wedge]", 
    SuperscriptBox["M", "n"]}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Wedge]", "0"], 
       SuperscriptBox["M", "n"]}], "\[CirclePlus]", 
      RowBox[{
       SuperscriptBox["\[Wedge]", "1"], 
       SuperscriptBox["M", "n"]}], "\[CirclePlus]"}], "..."}], 
    "\[CirclePlus]", 
    RowBox[{
     SuperscriptBox["\[Wedge]", "n"], 
     RowBox[{
      SuperscriptBox["M", "n"], "."}]}]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
   3.701612161368088*^9},ExpressionUUID->"b4a68a61-6ba2-48bb-9714-\
81d4ae56ff0f"],

Cell[TextData[{
 "Here ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Wedge]", "0"], 
    SuperscriptBox["M", "n"]}], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "05fb01a3-3272-4d6e-906c-cf4c2df2e71f"],
 " is the space of all smooth functions on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "0a2d6eaa-3b91-4165-b43b-2329d8bdf6da"],
 " and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Wedge]", "q"], 
    SuperscriptBox["M", "n"]}], TraditionalForm]],ExpressionUUID->
  "3234006a-21af-4333-b34c-4ca4a75e21ad"],
 " the space of all smooth differential forms of degree q on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "a30965a2-93f3-4dde-839a-7b6f400f493c"],
 ".  Remember that all q-forms with q>n are null forms such that ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Wedge]", "q"], 
    SuperscriptBox["M", "n"]}], " ", "=", " ", "0.", " "}]], 
  "WolframAlphaShort",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "adbb5683-8cea-4c4e-8208-7b58f33af903"],
 "Clearly, this is an infinite dimensiional real vector space as the direct \
sum of such spaces. Furthermore the wedge product of tensor algebra operating \
pointwise defines a multiplication in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Wedge]", 
    SuperscriptBox["M", "n"]}], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "71d35037-6257-4690-98c4-9ef3d0f4f7f0"],
 " also denoted by wedge such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Wedge]", 
    SuperscriptBox["M", "n"]}], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "fb159556-c74a-42d7-9e45-9d70ee8938a6"],
 " becomes a ",
 StyleBox["Grassmann algebra",
  FontColor->RGBColor[0, 0, 1]],
 ", this is a real  graded alternating algebra. The operation wedge \
considered above in the realm of tensor algebra can also be applied to \
differential forms. "
}], "Text",
 CellChangeTimes->{{3.70161222412744*^9, 3.701612408993812*^9}, {
   3.701612439118722*^9, 3.701612547581442*^9}, {3.7016126199969788`*^9, 
   3.7016126507487783`*^9}, {3.701612687068136*^9, 3.701612881995532*^9}, {
   3.7016129349549837`*^9, 3.7016131370108633`*^9}, {3.70161318079416*^9, 
   3.701613257599605*^9}, {3.7016669710033283`*^9, 3.701667046356122*^9}, {
   3.701667104683073*^9, 3.701667105656889*^9}, 
   3.701681426048422*^9},ExpressionUUID->"de0a29e3-5e70-4064-81e8-\
051b4fa3529f"],

Cell[TextData[{
 "Proposition 2. The wedge product is well defined on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Wedge]", 
    SuperscriptBox["M", "n"]}], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "fdd23488-31ab-4616-9d14-282a9433af42"],
 ". It is bilinear, associative, and alternating:"
}], "Text",
 CellChangeTimes->{{3.7016814330049953`*^9, 
  3.70168165195599*^9}},ExpressionUUID->"ca953f36-f64c-4295-90e6-\
a3c8471d75f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Sigma]", "\[Wedge]", "\[Tau]"}], "  ", "\[Equal]", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"p", "\[CenterDot]", "q"}]], 
     RowBox[{"\[Tau]", "\[Wedge]", "\[Sigma]"}], " ", "for", "  ", 
     "\[Sigma]"}]}], "\[Element]", " ", 
   RowBox[{
    SuperscriptBox["\[Wedge]", "p"], 
    SuperscriptBox["M", "n"]}]}], " ", ",", " ", 
  RowBox[{"\[Tau]", "\[Element]", 
   RowBox[{
    SuperscriptBox["\[Wedge]", "q"], 
    SuperscriptBox[
     RowBox[{"M", " "}], "n"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.701585680486411*^9, 3.701585929676659*^9}, {
  3.7016817197473927`*^9, 
  3.7016818053307133`*^9}},ExpressionUUID->"5aebcea2-dcc1-4e3b-9a96-\
ca86e3363a43"],

Cell[TextData[{
 "With the wedge product ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Wedge]", 
    SuperscriptBox["M", "n"]}], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.701611966169958*^9, 3.701612120785117*^9}, 
    3.701612161368088*^9},ExpressionUUID->
  "67b79d98-a330-4ede-ba53-360cdfed291c"],
 " is is the Grasssmann algebra of differential forms on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "d98cf9b6-b9ff-4714-8a74-24ae78bafcb3"],
 ".\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.701681873025701*^9, 3.7016819164495792`*^9}, {
  3.70168195602499*^9, 
  3.701682048422452*^9}},ExpressionUUID->"28fa26b1-2de7-41f3-835c-\
d1b419ff4755"],

Cell["We will show this considering some examples.", "Text",
 CellChangeTimes->{{3.70161222412744*^9, 3.701612408993812*^9}, {
  3.701612439118722*^9, 3.701612547581442*^9}, {3.7016126199969788`*^9, 
  3.7016126507487783`*^9}, {3.701612687068136*^9, 3.701612881995532*^9}, {
  3.7016129349549837`*^9, 3.7016131370108633`*^9}, {3.70161318079416*^9, 
  3.701613257599605*^9}, {3.7016669710033283`*^9, 3.701667046356122*^9}, {
  3.701667104683073*^9, 3.701667105656889*^9}, {3.701682076599917*^9, 
  3.70168212088769*^9}},ExpressionUUID->"9e1e2ba6-148c-4c9d-a49c-\
636499cd424c"],

Cell["The wedge product of nothing equals the constant 1:", "Text",
 CellChangeTimes->{{3.701667338035795*^9, 
  3.701667380292053*^9}},ExpressionUUID->"df2ca357-5bc2-4cb4-9bfa-\
f09b0b867735"],

Cell[BoxData[
 RowBox[{"wedge", "[", "]"}]], "Input",
 CellChangeTimes->{{3.6398256482659073`*^9, 
  3.639825654334318*^9}},ExpressionUUID->"f603c444-9b35-4195-bf7c-\
0e6a69b1d2b1"],

Cell["The current dimension is", "Text",
 CellChangeTimes->{{3.701667557513194*^9, 
  3.7016675752215*^9}},ExpressionUUID->"8f077fe5-f5ef-4700-8d1d-04c9ac7679cd"],

Cell[BoxData["dim"], "Input",
 CellChangeTimes->{{3.701667545367248*^9, 
  3.701667546418971*^9}},ExpressionUUID->"6a20b488-7541-46ac-b088-\
be4e70b96a91"],

Cell["A function considered as a tensor field", "Text",
 CellChangeTimes->{{3.701667627722437*^9, 
  3.7016676586021*^9}},ExpressionUUID->"d5e89234-3573-4c5f-a87d-e963f91acb78"],

Cell[BoxData[
 RowBox[{"tfx", " ", "=", " ", 
  RowBox[{"tensor", "[", 
   RowBox[{
    RowBox[{"fx", "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", "0", ",", "0"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.639825697437194*^9, 3.639825726593645*^9}, {
  3.639825788463354*^9, 3.6398258166214037`*^9}, {3.701667679794957*^9, 
  3.701667680722733*^9}},ExpressionUUID->"29b9defe-ae97-4b3d-991a-\
0a5d8fd2fb1a"],

Cell["wedge acts on tensors only :", "Text",
 CellChangeTimes->{{3.7016678893751087`*^9, 
  3.701667918691677*^9}},ExpressionUUID->"cb77a343-99da-442c-a7eb-\
2d46e3571038"],

Cell[BoxData[
 RowBox[{"wedge", "[", 
  RowBox[{"fx", "[", 
   RowBox[{
    RowBox[{"x", "[", "1", "]"}], ",", 
    RowBox[{"x", "[", "2", "]"}], ",", 
    RowBox[{"x", "[", "3", "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6398258274790225`*^9, 
  3.639825849397061*^9}},ExpressionUUID->"50347746-52e5-4623-bd9e-\
fc0765d5a947"],

Cell[BoxData[
 RowBox[{
  RowBox[{"wedge", "[", "tfx", "]"}], "\[Equal]", "tfx"}]], "Input",
 CellChangeTimes->{{3.6398374996685314`*^9, 3.6398375153621593`*^9}, {
  3.701667934161243*^9, 
  3.701667943560459*^9}},ExpressionUUID->"865669ff-8a49-4469-9da0-\
3692aef7957e"],

Cell["another function :", "Text",
 CellChangeTimes->{{3.701668002472707*^9, 
  3.7016680193219757`*^9}},ExpressionUUID->"ceea080b-dd9c-46f5-9d2d-\
f27d4695229c"],

Cell[BoxData[
 RowBox[{"tft", "=", " ", 
  RowBox[{"tensor", "[", 
   RowBox[{
    RowBox[{"ft", "[", 
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], ",", 
      RowBox[{"x", "[", "2", "]"}], ",", 
      RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", "0", ",", "0"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.63988779611043*^9, 3.6398878463269176`*^9}, {
  3.701667992169087*^9, 
  3.701667994104596*^9}},ExpressionUUID->"9dbf833a-cba6-443d-8460-\
49970b5ec44a"],

Cell[BoxData[
 RowBox[{"wedge", "[", 
  RowBox[{"tfx", ",", "tft"}], "]"}]], "Input",
 CellChangeTimes->{{3.639887868806557*^9, 3.639887881177379*^9}, {
  3.701668037552535*^9, 
  3.701668044007988*^9}},ExpressionUUID->"8dd11ab6-2bf0-453c-89e4-\
ccad5d017b09"],

Cell[" wedge may be written as an Infix operator.", "Text",
 CellChangeTimes->{{3.701671248739588*^9, 3.701671257528627*^9}, {
   3.701671295976697*^9, 3.70167130406417*^9}, 3.701671394191752*^9, {
   3.701671431455536*^9, 
   3.701671481999057*^9}},ExpressionUUID->"7f1ddb64-b9c6-452f-97a9-\
575debba8b38"],

Cell[BoxData[
 RowBox[{"tfx", "\[Wedge]", "tft"}]], "Input",
 CellChangeTimes->{{3.7010034672648373`*^9, 3.7010035162341022`*^9}, {
  3.701668063879984*^9, 
  3.701668075199668*^9}},ExpressionUUID->"73b0cf54-83e8-475a-a685-\
73d005d761cd"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.701003525688048*^9, 
  3.701003528433384*^9}},ExpressionUUID->"b1f34224-5e1c-4fc1-88bc-\
c6fdd77498f8"],

Cell["\<\
The wedge product corresponds to the product of the functions:\
\>", "Text",
 CellChangeTimes->{{3.7016685150603867`*^9, 
  3.70166858187609*^9}},ExpressionUUID->"0a404402-f4c0-4da3-ac49-\
aeeef98e2127"],

Cell[BoxData[
 RowBox[{"coords", "[", 
  RowBox[{"tfx", "\[Wedge]", "tft"}], "]"}]], "Input",
 CellChangeTimes->{{3.701668590118417*^9, 
  3.701668614083788*^9}},ExpressionUUID->"398f578c-0a85-4763-a968-\
8ed982266c29"],

Cell[BoxData[
 RowBox[{"type", "[", 
  RowBox[{"tfx", "\[Wedge]", "tft"}], "]"}]], "Input",
 CellChangeTimes->{{3.701668590118417*^9, 3.701668614083788*^9}, {
  3.7016686478050756`*^9, 
  3.701668660195455*^9}},ExpressionUUID->"1a48dfc6-0d9b-46a3-a81e-\
c2381dc1838c"],

Cell["The only coordinate is", "Text",
 CellChangeTimes->{{3.701674149524231*^9, 
  3.701674160020196*^9}},ExpressionUUID->"c9ffe83f-641b-48cc-8f6a-\
077b32555861"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"tfx", "\[Wedge]", "tft"}], ")"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.701671586462593*^9},ExpressionUUID->"77e75887-37f7-412b-8d75-\
599af10ab443"],

Cell["Now we expect zero:", "Text",
 CellChangeTimes->{{3.70167419536375*^9, 
  3.7016742138918962`*^9}},ExpressionUUID->"dea36727-724b-41df-81a7-\
a4d45d0ad5ee"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"tfx", "\[Wedge]", "tft"}], "-", 
   RowBox[{"tft", "\[Wedge]", "tfx"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.701668691990034*^9, 
  3.701668763746667*^9}},ExpressionUUID->"fe7ebdba-5857-41a9-bd51-\
ea38313c3a20"],

Cell["\<\
but the obtained tensor is not identified with the nulltensor, although its \
only coordinate is zero:\
\>", "Text",
 CellChangeTimes->{{3.701674241395451*^9, 3.70167432756306*^9}, {
  3.701678721682489*^9, 
  3.7016787406581287`*^9}},ExpressionUUID->"97e7c6b8-b30b-4559-95c0-\
4dfac9494869"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.701669169781389*^9, 3.701669186167843*^9}, {
  3.701669426171864*^9, 
  3.7016694716461678`*^9}},ExpressionUUID->"cd745c01-eea0-469c-a457-\
c0c2a9ad861c"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{"tfx", "\[Wedge]", "tft"}], ",", 
   RowBox[{"tft", "\[Wedge]", "tfx"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.701668914633377*^9, 
  3.701668952945964*^9}},ExpressionUUID->"e9f42ac6-5de8-4462-8fce-\
035812480c86"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"tfx", "\[Wedge]", "tft"}], "-", 
    RowBox[{"tft", "\[Wedge]", "tfx"}]}], ")"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.701670268808422*^9, 3.7016702904643707`*^9}, {
  3.701670331135792*^9, 
  3.701670336895747*^9}},ExpressionUUID->"d6f46f86-38d7-4834-9f23-\
3f2f2d8a67fd"],

Cell[TextData[{
 StyleBox["Remark. ",
  FontColor->RGBColor[0, 0, 1]],
 "Considering functions as differential forms it is recommended first to \
transform them into a tensor field of type {0,0}, since often they are not \
interpreted as numeric quantities, e.g. "
}], "Text",
 CellChangeTimes->{{3.701679081199445*^9, 
  3.701679382757276*^9}},ExpressionUUID->"87463caa-aa9c-4e18-952f-\
d93f5be5deb9"],

Cell[BoxData[
 RowBox[{"wedge", "[", 
  RowBox[{
   RowBox[{"norm", "[", 
    RowBox[{"vec", "[", "x", "]"}], "]"}], ",", "tft"}], "]"}]], "Input",
 CellChangeTimes->{{3.70167939713118*^9, 3.7016794367650423`*^9}, 
   3.701679590289052*^9},ExpressionUUID->"297b5a25-d4b4-4fbc-a87e-\
6e3871766066"],

Cell[BoxData[
 RowBox[{"tnorm", " ", "=", 
  RowBox[{"tensor", "[", 
   RowBox[{
    RowBox[{"norm", "[", 
     RowBox[{"vec", "[", "x", "]"}], "]"}], ",", "0", ",", "0"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.701679446800365*^9, 3.701679495820997*^9}, 
   3.701679573113221*^9},ExpressionUUID->"a674dc77-66dd-44aa-b53b-\
b09ff5d6a8da"],

Cell[BoxData[
 RowBox[{"wedge", "[", 
  RowBox[{"tnorm", ",", "tft"}], "]"}]], "Input",
 CellChangeTimes->{{3.7016795145464*^9, 
  3.70167953415619*^9}},ExpressionUUID->"31ec0c0b-2074-4ecb-8cd6-\
960c46025693"],

Cell[BoxData[
 RowBox[{"dt0", "=", 
  RowBox[{"covecTotensor", "[", 
   RowBox[{
    RowBox[{"d", "[", "fx", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.639888208559554*^9, 3.6398882518496304`*^9}, {
   3.6398882831120853`*^9, 3.6398883068241267`*^9}, 
   3.639888708899233*^9},ExpressionUUID->"97cd9083-a553-41b6-b5c5-\
a65cba584c74"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6398884570367904`*^9, 
  3.6398884631520014`*^9}},ExpressionUUID->"89fc46d2-df1e-4a8b-ba8f-\
00a3097c628c"],

Cell[BoxData[
 RowBox[{"tfx", "\[Wedge]", "dt0"}]], "Input",
 CellChangeTimes->{{3.639888481638034*^9, 3.639888509702483*^9}, 
   3.701005410283084*^9, {3.7010054576349077`*^9, 3.701005477450399*^9}, {
   3.701679746570704*^9, 
   3.701679748970398*^9}},ExpressionUUID->"32a3bf82-0aee-44c8-a00a-\
10dc88886687"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6398885248813095`*^9, 
  3.6398885303569193`*^9}},ExpressionUUID->"871bf5b3-9ecd-4097-aa94-\
0b5630ba1da6"],

Cell[BoxData[
 RowBox[{"dt0", "\[Wedge]", "dt0"}]], "Input",
 CellChangeTimes->{{3.639888751612108*^9, 3.6398887653401318`*^9}, {
  3.701005522713997*^9, 
  3.7010055449217157`*^9}},ExpressionUUID->"252848c6-7362-44a2-86c1-\
7dfaa92a1239"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"nulltensor", "[", 
    RowBox[{"2", ",", "0"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6398887985681906`*^9, 
  3.639888848098277*^9}},ExpressionUUID->"8c4b561d-3f3e-4864-b27f-\
3928c56fb463"],

Cell[BoxData[
 RowBox[{"dtt0", "=", 
  RowBox[{"covecTotensor", "[", 
   RowBox[{
    RowBox[{"d", "[", "ft", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.639888208559554*^9, 3.6398882518496304`*^9}, {
   3.6398882831120853`*^9, 3.6398883068241267`*^9}, 
   3.6398886460779223`*^9},ExpressionUUID->"dcaf8615-697c-49c9-902e-\
fed3e027c084"],

Cell[BoxData["dt0"], "Input",
 CellChangeTimes->{{3.708418434078027*^9, 
  3.708418438037644*^9}},ExpressionUUID->"73527d5f-dc3c-41cb-96da-\
e5ebd6bb7754"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.708418474057067*^9, 
  3.708418503218093*^9}},ExpressionUUID->"2a7e6297-1b2e-4a2e-903d-\
face4d7e296d"],

Cell[BoxData[
 RowBox[{"dt0", "\[Wedge]", "dtt0", "\[Wedge]", "tfx"}]], "Input",
 CellChangeTimes->{{3.6398889107323875`*^9, 3.639888943024444*^9}, {
  3.701005571721695*^9, 3.701005617497129*^9}, {3.701679843007201*^9, 
  3.7016798442580547`*^9}},ExpressionUUID->"b2d0c320-2996-437e-88d1-\
895c1950f6cb"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.639888962087678*^9, 
  3.639888973194897*^9}},ExpressionUUID->"9ad57bad-45d6-4743-924c-\
264d654328b2"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.70100571844619*^9, 
  3.701005729376293*^9}},ExpressionUUID->"cddcbd10-4301-49c4-bae2-\
04986be234b9"],

Cell[BoxData[
 RowBox[{"?", "d"}]], "Input",
 CellChangeTimes->{{3.645587988827523*^9, 
  3.645587989763525*^9}},ExpressionUUID->"59f54002-0ac3-4b76-a5f6-\
b1c3054d584a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"x", "[", "2", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6455879322618237`*^9, 
  3.645587973399096*^9}},ExpressionUUID->"e953b9a4-acbc-4916-a5c7-\
d9256c2f14ae"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", "i_", "]"}], ":=", " ", 
  RowBox[{"covecTotensor", "[", 
   RowBox[{"stb", "[", "i", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6398890292613955`*^9, 3.639889096653514*^9}, {
  3.6398891476500034`*^9, 
  3.6398891841852674`*^9}},ExpressionUUID->"5e38bb58-0bfa-446b-8e6d-\
bce088dd2e25"],

Cell[BoxData[
 RowBox[{"dx", "[", "2", "]"}]], "Input",
 CellChangeTimes->{{3.639889198646493*^9, 
  3.6398892026245003`*^9}},ExpressionUUID->"19dfc202-e55f-4b48-b8ae-\
3d79413418be"],

Cell[BoxData[
 RowBox[{"covecTotensor", "[", 
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"x", "[", "2", "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.645588039340412*^9, 
  3.6455880892916994`*^9}},ExpressionUUID->"5a14acb8-4d78-4563-ac8a-\
a66f12f010fe"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%", ",", 
   RowBox[{"dx", "[", "2", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.645588111818139*^9, 
  3.6455881282605677`*^9}},ExpressionUUID->"439bd7d8-eccd-4254-8a14-\
96a0fe2f001d"],

Cell[BoxData[
 RowBox[{"coords", "[", "dt0", "]"}]], "Input",
 CellChangeTimes->{{3.6398892136225195`*^9, 3.6398892188485284`*^9}, {
  3.6398892605162015`*^9, 3.639889276989831*^9}, {3.7016801016162987`*^9, 
  3.7016801147916183`*^9}},ExpressionUUID->"6cf36911-8ef5-4bef-adfb-\
5ab52ae6a09d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", "2", "]"}], "\[Wedge]", "dt0", "\[Wedge]", 
  "dtt0"}]], "Input",
 CellChangeTimes->{{3.639889297831467*^9, 3.6398893253655157`*^9}, {
  3.701005814423666*^9, 
  3.7010058487353888`*^9}},ExpressionUUID->"48c88e04-02c3-4baf-8d7c-\
9087f0014966"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", "2", "]"}], "\[Wedge]", 
  RowBox[{"(", 
   RowBox[{"dt0", "\[Wedge]", "dtt0"}], ")"}]}]], "Input",
 CellChangeTimes->{{3.6455883591721735`*^9, 3.645588365193784*^9}, {
  3.645589202119654*^9, 3.6455892215260887`*^9}, {3.701003200769939*^9, 
  3.701003227436809*^9}, {3.701003281136701*^9, 
  3.7010033488272448`*^9}},ExpressionUUID->"fdfcbfe4-451c-4e50-8947-\
d57dffae0269"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%%", ",", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.668655528911788*^9, 3.668655531658819*^9}, {
  3.668655585156251*^9, 
  3.6686556041380463`*^9}},ExpressionUUID->"debc4168-3e7f-4cb3-8043-\
6b951e6da047"],

Cell["The exterior product wedge is Flat (associative).", "Text",
 CellChangeTimes->{{3.6455892702605743`*^9, 3.645589283395797*^9}, 
   3.7016821467404337`*^9, {3.702041200345338*^9, 
   3.7020412048013163`*^9}},ExpressionUUID->"10ea368e-91da-40c5-b35e-\
db758b5280fe"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%%", "]"}]], "Input",
 CellChangeTimes->{{3.6398893554267683`*^9, 3.639889365395186*^9}, 
   3.6455884819755893`*^9},ExpressionUUID->"dd78145a-0554-414b-bb5f-\
05c0c3e35e27"],

Cell[BoxData[
 RowBox[{"?", "strictcoords"}]], "Input",
 CellChangeTimes->{{3.6686556709366293`*^9, 
  3.668655677657028*^9}},ExpressionUUID->"f7ebf4a1-8bd9-4725-a993-\
74dc813e278b"],

Cell[BoxData[
 RowBox[{"dts", " ", "=", " ", 
  RowBox[{"covecTotensor", "[", 
   RowBox[{
    RowBox[{"d", "[", "fs", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.63988945316094*^9, 3.639889509523839*^9}, {
  3.6398895710815473`*^9, 
  3.6398895802075634`*^9}},ExpressionUUID->"79019b8f-ad42-4348-ae7e-\
cc79b70ccf83"],

Cell[BoxData[
 RowBox[{"dts", "\[Wedge]", "dt0", "\[Wedge]", "dtt0"}]], "Input",
 CellChangeTimes->{{3.639889297831467*^9, 3.6398893253655157`*^9}, {
  3.6398896008463993`*^9, 3.639889601985202*^9}, {3.70100604158175*^9, 
  3.701006066197654*^9}},ExpressionUUID->"bea764ad-94fb-402f-b66d-\
3d6eb5f8d975"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6398896160720263`*^9, 
  3.6398896241372404`*^9}},ExpressionUUID->"429eb26e-b1e6-4495-b4f9-\
32a3db90238b"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.701006109375578*^9, 
  3.701006120149199*^9}},ExpressionUUID->"c68b944b-b2d0-4775-8bd6-\
20c3298a1c0d"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"%", "[", 
    RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", 
   RowBox[{"Det", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"d", "[", "fs", "]"}], "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}], ",", 
        RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"d", "[", "fx", "]"}], "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}], ",", 
        RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"d", "[", "ft", "]"}], "[", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], ",", 
        RowBox[{"x", "[", "2", "]"}], ",", 
        RowBox[{"x", "[", "3", "]"}]}], "]"}]}], "}"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6398896483172827`*^9, 3.6398896843377457`*^9}, {
  3.6398897248042173`*^9, 3.6398897683594933`*^9}, {3.6398898054407587`*^9, 
  3.639889849542036*^9}, {3.701006198039871*^9, 
  3.701006204669621*^9}},ExpressionUUID->"73f79a23-4883-4611-b363-\
947528da41ff"],

Cell[BoxData[
 RowBox[{"dts", "\[Wedge]", "dt0", "\[Wedge]", "dtt0", "\[Wedge]", 
  RowBox[{"dx", "[", "3", "]"}]}]], "Input",
 CellChangeTimes->{{3.639889297831467*^9, 3.6398893253655157`*^9}, {
  3.6398896008463993`*^9, 3.639889601985202*^9}, {3.6398899821502695`*^9, 
  3.6398899899814835`*^9}, {3.701006502450192*^9, 
  3.701006538009842*^9}},ExpressionUUID->"cb4b8c2d-4c78-4a57-96a2-\
71c8ee54fbf6"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.639890026345147*^9, 
  3.639890066686818*^9}},ExpressionUUID->"413f6e38-7c61-49db-a571-\
cd5529667149"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"dt0", "*", "Pi"}], " ", "+", 
      RowBox[{"dtt0", "*", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], "^", "2"}]}]}], ")"}], "\[Wedge]", " ", 
    "dts"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{"Pi", "*", 
     RowBox[{
      RowBox[{"(", "dt0", ")"}], "\[Wedge]", " ", "dts"}]}], " ", "+", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], "^", "2"}], "*", 
     RowBox[{"dtt0", "\[Wedge]", " ", "dts"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7016804464382887`*^9, 3.701680737014145*^9}, {
  3.70168079955613*^9, 3.7016808433878593`*^9}, {3.701680877076828*^9, 
  3.7016808821393623`*^9}},ExpressionUUID->"641dec86-59d9-4820-9a77-\
b7894e4892d7"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"dt0", "*", "Pi"}], " ", "+", 
      RowBox[{"dtt0", "*", 
       RowBox[{
        RowBox[{"x", "[", "1", "]"}], "^", "2"}]}]}], ")"}], "\[Wedge]", " ", 
    "dts"}], ",", 
   RowBox[{
    RowBox[{"Pi", "*", 
     RowBox[{
      RowBox[{"(", "dt0", ")"}], "\[Wedge]", " ", "dts"}]}], " ", "+", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], "^", "2"}], "*", 
     RowBox[{"dtt0", "\[Wedge]", " ", "dts"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7016804464382887`*^9, 3.701680737014145*^9}, {
  3.70168079955613*^9, 3.7016808433878593`*^9}, {3.701680877076828*^9, 
  3.701680939338786*^9}},ExpressionUUID->"335336a3-1ff5-4f1c-8ae3-\
fae782451bf9"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"dt0", "*", "Pi"}], " ", "+", 
     RowBox[{"dtt0", "*", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], "^", "2"}]}]}], ")"}], "\[Wedge]", " ", 
   "dts"}], "]"}]], "Input",
 CellChangeTimes->{{3.7016804464382887`*^9, 3.701680737014145*^9}, {
  3.70168079955613*^9, 3.7016808433878593`*^9}, {3.701680877076828*^9, 
  3.701680939338786*^9}, {3.701680999702506*^9, 
  3.701681053577619*^9}},ExpressionUUID->"4fe14a03-9b55-420b-81ab-\
45dbf922500b"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
2.4.2. The Exterior Differential of Antisymmetric Arrays in n Dimensions\
\>", "Subsubsection",
 CellChangeTimes->{{3.5866649932855587`*^9, 3.586665013117693*^9}, {
   3.5872725258470573`*^9, 3.587272530654332*^9}, {3.5898752162352247`*^9, 
   3.589875218169628*^9}, 3.5899533142260494`*^9, {3.609309548254758*^9, 
   3.6093095610144877`*^9}, 3.6095615905629835`*^9, 3.6177190633933744`*^9, 
   3.6177712155926075`*^9, 3.6274424857387977`*^9, 3.627443068992622*^9, 
   3.635839359077566*^9, {3.6454362817971053`*^9, 3.645436327513186*^9}, 
   3.645588884269096*^9, {3.645594238601818*^9, 3.6455942534374447`*^9}, 
   3.645610929040021*^9, 3.6456110340382056`*^9, 3.646824320846387*^9, {
   3.646824396054119*^9, 3.646824442916601*^9}, {3.6468244997007008`*^9, 
   3.6468245299803543`*^9}, {3.6468965164180555`*^9, 3.6468965178688583`*^9}, 
   3.668657742898122*^9, 3.668690172261836*^9, 3.701757489893795*^9, 
   3.702909091473333*^9, 3.7030436926342287`*^9, 3.7040043634925127`*^9, 
   3.704005958726885*^9},ExpressionUUID->"da296b7d-deec-42d6-8b4f-\
c44eda59d06d"],

Cell["\<\
The exterior differentials of an antisymmetric forms, represented by their \
coordinate Arrays, has been treated in the notebook  euklsfv3.nb. We complete \
the test in this subsection.\
\>", "Text",
 CellChangeTimes->{{3.646646280696785*^9, 3.6466463762937527`*^9}, {
  3.646716304278878*^9, 3.6467163124844923`*^9}, {3.7017575785010138`*^9, 
  3.701757656652329*^9}, {3.701757712315794*^9, 
  3.701757795691349*^9}},ExpressionUUID->"f0b93caf-c643-4db1-8e4a-\
3bd85169631c"],

Cell[BoxData[
 RowBox[{"?", "altform"}]], "Input",
 CellChangeTimes->{{3.6456115380420933`*^9, 3.6456115417704997`*^9}, {
  3.7016822297106247`*^9, 
  3.701682236998608*^9}},ExpressionUUID->"33dd0c15-9c1d-4e11-aa23-\
bf58a4431ebb"],

Cell[BoxData[
 RowBox[{"Context", "[", "altform", "]"}]], "Input",
 CellChangeTimes->{{3.704004869782332*^9, 
  3.704004898335211*^9}},ExpressionUUID->"4950203b-4cad-418d-bf0f-\
3739dabda347"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"dim", ",", "af"}], "]"}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"f_", ",", "p_", ",", 
      RowBox[{"d_:", "dim"}]}], "]"}], "[", "x__", "]"}], ":=", " ", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"p", "\[Equal]", "0"}], ",", 
     RowBox[{"f", "[", "x", "]"}], ",", 
     RowBox[{
      RowBox[{"altform", "[", 
       RowBox[{"a", ",", "p", ",", "d"}], "]"}], "/.", 
      RowBox[{
       RowBox[{"a", "[", "i__", "]"}], "\[Rule]", " ", 
       RowBox[{
        RowBox[{"f", "[", "i", "]"}], "[", "x", "]"}]}]}]}], "]"}]}], ";", 
  " ", 
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.645611324456717*^9, 3.645611383497221*^9}, {
   3.6456114219698887`*^9, 3.6456114618309593`*^9}, {3.6456115111914463`*^9, 
   3.645611526900674*^9}, {3.6456115700563498`*^9, 3.6456115829419727`*^9}, 
   3.6456116144414277`*^9, {3.6456116471586857`*^9, 3.645611654911899*^9}, {
   3.646634665926436*^9, 3.6466346774860563`*^9}, {3.646634711790517*^9, 
   3.646634728872547*^9}, {3.6467163780202074`*^9, 3.6467164077538595`*^9}, {
   3.704005136774238*^9, 3.70400518555717*^9}, {3.704005235060533*^9, 
   3.7040052695726433`*^9}, {3.704005409683794*^9, 3.704005412826819*^9}, {
   3.7040058153453913`*^9, 3.704005831151688*^9}, {3.708417571715098*^9, 
   3.708417584460047*^9}, {3.708417622722979*^9, 
   3.708417671842547*^9}},ExpressionUUID->"20b6b1d4-92aa-4d0d-a90b-\
e5dbbdf98bb5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6456116748019342`*^9, 3.645611736162842*^9}, {
   3.645611914675357*^9, 3.645611952820424*^9}, {3.6456125307966456`*^9, 
   3.6456125911551514`*^9}, 3.645612665962283*^9, {3.6466347886362514`*^9, 
   3.646634790367855*^9}, 3.64663483326793*^9, {3.646634892002033*^9, 
   3.646634905542857*^9}, {3.6471531587203503`*^9, 
   3.647153218172055*^9}},ExpressionUUID->"4122fa67-c929-480e-88d7-\
0819cd214884"],

Cell[BoxData[
 RowBox[{"\[Lambda]", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.6456126101423855`*^9, {3.6456126870409203`*^9, 
   3.6456126928597307`*^9}},ExpressionUUID->"dfd191a2-8a24-4244-a77f-\
8bc6bdca270d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"af", "[", 
   RowBox[{"f", ",", "0"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.708417726060975*^9, 
  3.708417770519391*^9}},ExpressionUUID->"00616b49-b215-4979-83e8-\
a0b7ed256f76"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.668688730147881*^9, 
  3.6686887469033737`*^9}},ExpressionUUID->"d2954308-bdd8-448c-a448-\
bea60a0de51d"],

Cell["\<\
dd can act on Arrays and on tensors, In this subsection we show its action on \
Arrays.\
\>", "Text",
 CellChangeTimes->{{3.6471535190653834`*^9, 3.647153571699876*^9}, 
   3.70175424804655*^9},ExpressionUUID->"5e90db2e-2246-47f9-b345-\
1e371a4ad81c"],

Cell[TextData[{
 StyleBox["Example 1. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["dim = 2",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6466471691339455`*^9, 3.6466471788527627`*^9}, {
  3.70175660186856*^9, 
  3.701756603172813*^9}},ExpressionUUID->"c32ebcfc-628a-4533-b780-\
26f6303877f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{"Clear", "[", "sq", "]"}], ";", 
  RowBox[{
   RowBox[{"sq", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"var", "[", "x", "]"}], "}"}], ".", 
     RowBox[{"{", 
      RowBox[{"var", "[", "x", "]"}], "}"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.645623617600792*^9, 3.645623728329787*^9}, {
   3.645623763788649*^9, 3.6456238101363306`*^9}, {3.6457673167383943`*^9, 
   3.6457673273308125`*^9}, 3.645767585932467*^9, {3.6471539200328875`*^9, 
   3.6471539231840935`*^9}, {3.6471542451998587`*^9, 3.647154260441086*^9}, {
   3.6471548154900603`*^9, 3.647154819592868*^9}, {3.6471550456372643`*^9, 
   3.647155049147271*^9}},ExpressionUUID->"31a13052-2331-40c9-97ec-\
f84d3840ff1f"],

Cell[BoxData[
 RowBox[{"sq", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456238206663485`*^9, 3.645623837264778*^9}, {
  3.7017548179195747`*^9, 
  3.701754820755731*^9}},ExpressionUUID->"66d433f2-0a0c-4cfa-a86c-\
d80e1d851ea1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "sq", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456234702429333`*^9, 3.6456234912873707`*^9}, {
   3.6456235220194244`*^9, 3.645623550286674*^9}, 3.645623743742614*^9, {
   3.6456238531612062`*^9, 3.6456238850164623`*^9}, 3.645767611968913*^9, 
   3.701754248062735*^9},ExpressionUUID->"191f10bb-f6a6-4680-8901-\
0485f1212c89"],

Cell["\<\
To differentiate a function (as a 0-form) it must be included in curly \
brackets  to form an array. Then its exterior differential coincides with its \
differential:\
\>", "Text",
 CellChangeTimes->{{3.6471552433988123`*^9, 
  3.6471553905382705`*^9}},ExpressionUUID->"5a2a8dcf-bf59-4741-9562-\
440d2e7cbdc8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{"ff", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.6471555007524643`*^9, 
  3.6471555604381685`*^9}},ExpressionUUID->"e479b669-49ef-424e-aff3-\
ac99826623f3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "f", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456234702429333`*^9, 3.6456234912873707`*^9}, {
   3.6456235220194244`*^9, 3.645623550286674*^9}, 3.645623743742614*^9, {
   3.6456238531612062`*^9, 3.6456238850164623`*^9}, 3.645767611968913*^9, {
   3.64715542985034*^9, 3.6471554562611856`*^9}, 3.647155488366042*^9, 
   3.647155579220602*^9, 
   3.701754248069541*^9},ExpressionUUID->"9d60facf-b8dd-4420-b252-\
38f599c44316"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"d", "[", "ff", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.647155604477046*^9, 3.6471556414179115`*^9}, 
   3.647155674521169*^9},ExpressionUUID->"bb4a1b31-9636-4794-9b80-\
0c070719b2a5"],

Cell["", "Text",ExpressionUUID->"c5ce7564-a77a-48ef-a658-5b8cb0e17fcc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6467174302108555`*^9, 3.646717436341666*^9}, {
  3.6467174673857207`*^9, 
  3.646717479023341*^9}},ExpressionUUID->"5468408c-74a1-4a0f-8ea4-\
dac55438126e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6467175259170237`*^9, 3.646717541657451*^9}, 
   3.6468086978789635`*^9, 
   3.70175424807438*^9},ExpressionUUID->"18d6ffa3-a684-426e-9ac0-\
f6de3eddc20c"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6471558322686462`*^9, 3.6471558624390993`*^9}, 
   3.701754248078911*^9},ExpressionUUID->"49bff45b-5f07-44ae-a4ce-\
b6394a18578b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6467175259170237`*^9, 3.646717541657451*^9}, {
  3.646806497379592*^9, 3.646806515850024*^9}, {3.7017542480832043`*^9, 
  3.7017542480853777`*^9}},ExpressionUUID->"1c943038-629b-4b9a-8e25-\
276157bc5dbc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6467174302108555`*^9, 3.646717436341666*^9}, {
   3.6467174673857207`*^9, 3.646717479023341*^9}, 
   3.6471559210484023`*^9},ExpressionUUID->"8a01a6c3-1295-4b49-9ed8-\
554dc059ccca"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6467175259170237`*^9, 3.646717541657451*^9}, 
   3.6468086978789635`*^9, 
   3.701754248089205*^9},ExpressionUUID->"8fa90d40-a9c3-41d7-81a0-\
26899eb1f253"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6471558322686462`*^9, 3.6471558624390993`*^9}, 
   3.701754248093704*^9},ExpressionUUID->"a1652d76-ff80-4e24-8edf-\
bdc6ccc48ed0"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6467166633135085`*^9, 
  3.6467166633135085`*^9}},ExpressionUUID->"b91fb078-5a4f-48c3-a3ed-\
f987de8f1e07"],

Cell[TextData[{
 StyleBox["Example 2. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["dim = 3",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6466471691339455`*^9, 3.6466471788527627`*^9}, {
  3.6466472169036293`*^9, 3.646647218245232*^9}, {3.701756635236493*^9, 
  3.701756636188442*^9}},ExpressionUUID->"d8ea1c06-3296-4441-a1bd-\
7322c426f2f8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6468020910846524`*^9, {3.64680221519847*^9, 
   3.6468022231544847`*^9}},ExpressionUUID->"2e5d62ec-4bf3-464f-8c28-\
90b8d198b609"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456207910786276`*^9, 3.645620875833576*^9}, {
   3.645621252449438*^9, 3.645621287237499*^9}, 3.6456214668094144`*^9, {
   3.645621504436681*^9, 3.645621543467949*^9}, 3.6456221738494563`*^9, 
   3.6456222099323196`*^9, {3.6456223411129503`*^9, 3.6456223565569773`*^9}, {
   3.646646221619481*^9, 3.6466462286238937`*^9}, {3.646647045222928*^9, 
   3.646647048576934*^9}, 3.646808740233038*^9, 
   3.701754248098098*^9},ExpressionUUID->"de8bfe7a-d498-433b-8da6-\
898feba286e0"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6471560134785643`*^9, 
  3.6471560277369895`*^9}},ExpressionUUID->"1d0e10a5-af27-409b-8237-\
a6ace73f4bcf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6471560451154203`*^9, 3.6471560799034815`*^9}, {
  3.7017542481028423`*^9, 
  3.7017542481050367`*^9}},ExpressionUUID->"2d7ceeb6-a38c-4058-9ad4-\
51dfcab85e66"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6456116748019342`*^9, 3.645611736162842*^9}, {
   3.645611914675357*^9, 3.645611952820424*^9}, {3.6456125307966456`*^9, 
   3.6456125911551514`*^9}, 3.645612665962283*^9, {3.6456204964409103`*^9, 
   3.645620506674528*^9}, {3.646635017395054*^9, 3.6466350299374757`*^9}, {
   3.647156120869153*^9, 
   3.6471561221327553`*^9}},ExpressionUUID->"022853cf-fb7e-4a81-8782-\
990fdc21e324"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456207910786276`*^9, 3.645620875833576*^9}, {
   3.645621252449438*^9, 3.645621287237499*^9}, 3.6456214668094144`*^9, {
   3.645621504436681*^9, 3.645621543467949*^9}, 3.6456221738494563`*^9, 
   3.6456222099323196`*^9, {3.6456223411129503`*^9, 3.6456223565569773`*^9}, {
   3.646646221619481*^9, 3.6466462286238937`*^9}, {3.646647045222928*^9, 
   3.646647048576934*^9}, 
   3.701754248107789*^9},ExpressionUUID->"bbd7effa-666d-4cad-9f81-\
006f5370e3e4"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.645621676426983*^9, 
  3.6456216989690228`*^9}},ExpressionUUID->"4e473a9b-e45e-4939-9e03-\
c6b6473c195c"],

Cell[BoxData[
 RowBox[{"%%", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "]"}]], "Input",ExpressionUUID->\
"aa21de01-9283-49a5-ba8e-f93226947aab"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.645622478689592*^9, 3.6456225256924744`*^9}, {
   3.647156184423665*^9, 3.6471562316605477`*^9}, 
   3.701754248110447*^9},ExpressionUUID->"c45430bf-f2b6-400a-9433-\
34a704d4fb42"],

Cell[BoxData[
 RowBox[{"%", " ", "\[Equal]", 
  RowBox[{"{", "%%", "}"}]}]], "Input",
 CellChangeTimes->{{3.6471562461685734`*^9, 
  3.6471563090834837`*^9}},ExpressionUUID->"a2c567bf-3430-4489-bbc7-\
5b31bbb9d706"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"dd", "[", 
    RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646809008459909*^9, 3.646809021641932*^9}, {
  3.701754248113057*^9, 
  3.701754248114669*^9}},ExpressionUUID->"344f9d64-afa1-45c2-80e0-\
38eab00def0a"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6467168063345594`*^9, 
  3.6467168063345594`*^9}},ExpressionUUID->"18f3201e-84a9-4dd4-8cdb-\
4a90ecc462df"],

Cell[TextData[{
 StyleBox["Example 3. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["dd and Curl",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6467169212911615`*^9, 3.6467169464852057`*^9}, {
   3.6467472149359636`*^9, 3.6467472352159986`*^9}, 3.701754248121571*^9, 
   3.7017543983659353`*^9, {3.701756669644197*^9, 
   3.70175667245218*^9}},ExpressionUUID->"a14765e9-5d91-4394-9caf-\
8295fce71bfd"],

Cell["We yet consider the built-in function", "Text",
 CellChangeTimes->{{3.646746221721019*^9, 
  3.646746244621859*^9}},ExpressionUUID->"3e8396f1-5611-4101-85bc-\
07680004e6b8"],

Cell[BoxData[
 RowBox[{"?", "Curl"}]], "Input",
 CellChangeTimes->{{3.6456193620257177`*^9, 
  3.6456193664561253`*^9}},ExpressionUUID->"23904cda-a359-462b-a902-\
618f4d63a011"],

Cell[BoxData[
 RowBox[{"Curl", "[", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.645619537619626*^9, 3.6456196030461407`*^9}, {
  3.6456196380526023`*^9, 
  3.645619649425022*^9}},ExpressionUUID->"32ec612d-452d-4542-870e-\
4342fd0dc1e0"],

Cell["In this dimension it coincides with", "Text",
 CellChangeTimes->{{3.646746352847249*^9, 
  3.646746368806077*^9}},ExpressionUUID->"9637ee4b-6e7f-4025-8b0a-\
688ea08ceae5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646746379055295*^9, 
  3.646746418741765*^9}},ExpressionUUID->"1575a84c-e769-49cf-8d30-\
6a9b49506448"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"strictcoords", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.701755236492359*^9, 3.701755248503888*^9}, {
  3.701755484922332*^9, 
  3.701755500149687*^9}},ExpressionUUID->"0fb68b35-6cfd-4bfd-b6d4-\
e307e0454b31"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"%", "[", 
    RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", " ", 
   RowBox[{"Curl", "[", 
    RowBox[{
     RowBox[{"\[Lambda]", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.701755344315963*^9, 3.7017553875912323`*^9}, {
  3.7017554269296713`*^9, 
  3.701755459224201*^9}},ExpressionUUID->"16029fc6-a31b-4175-8991-\
7646d6f9b46f"],

Cell["\[FilledSquare] ", "Text",
 CellChangeTimes->{{3.646821842797634*^9, 3.6468218603476653`*^9}, {
  3.70175611745918*^9, 
  3.701756124000433*^9}},ExpressionUUID->"8ef59396-55b4-406e-a660-\
6898643d63fa"],

Cell[TextData[{
 StyleBox["Example 4.",
  FontColor->RGBColor[0, 0, 1]],
 " dd and Curl in  dimension 3."
}], "Text",
 CellChangeTimes->{{3.701756371413794*^9, 3.701756437654207*^9}, {
  3.701756707323848*^9, 
  3.701756708515913*^9}},ExpressionUUID->"87602684-a4b7-4977-b997-\
df31d25e46f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", " ", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6467473794226522`*^9, 3.646747385132262*^9}, 
   3.64674741800152*^9},ExpressionUUID->"ab6be232-23b4-444f-a076-\
adb3b62f63ab"],

Cell["The exterior differential of the 1-form is", "Text",
 CellChangeTimes->{{3.6468219618570433`*^9, 
  3.6468219828858805`*^9}},ExpressionUUID->"ebe767d8-35e3-42d8-8835-\
c54fdd5b99c2"],

Cell[BoxData[
 RowBox[{"daa1", "=", 
  RowBox[{
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6467474553635855`*^9, 3.6467474891844454`*^9}, 
   3.7017562829139547`*^9},ExpressionUUID->"933765ec-6d5d-42bf-8307-\
7694040443ee"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"strictcoords", "[", "daa1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6469145012700047`*^9, 
  3.646914541362075*^9}},ExpressionUUID->"ff8c9421-0ebc-4a58-b833-\
a4e454b23fae"],

Cell[BoxData[
 RowBox[{"Curl", "[", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6467477580509176`*^9, 
  3.6467478478758755`*^9}},ExpressionUUID->"5917bf2c-6d8c-4b63-90c2-\
9f0ee0debe60"],

Cell["\<\
Curl yields (up to the order and sign) the strict coordinates of this form:\
\>", "Text",
 CellChangeTimes->{{3.646822034755972*^9, 3.6468220530392036`*^9}, {
   3.6469147126503763`*^9, 3.646914722415993*^9}, {3.647738776290558*^9, 
   3.6477387796757636`*^9}, 
   3.6686897817744083`*^9},ExpressionUUID->"828fa8cf-687e-48b7-99b9-\
849edda09f0f"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"daa1", "[", 
    RowBox[{"[", 
     RowBox[{"3", ",", "2"}], "]"}], "]"}], ",", 
   RowBox[{"daa1", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "3"}], "]"}], "]"}], ",", 
   RowBox[{"daa1", "[", 
    RowBox[{"[", 
     RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.6468206454019313`*^9, 3.64682065625955*^9}, {
  3.646820765927743*^9, 3.646820801885806*^9}, {3.6468210362606173`*^9, 
  3.6468211225287695`*^9}, {3.646821159407234*^9, 
  3.646821162402439*^9}},ExpressionUUID->"27085d63-7332-4e7b-b22b-\
6a4fb8f0f9fe"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{"-", "%%"}]}]], "Input",
 CellChangeTimes->{{3.6468211792504687`*^9, 3.646821191683691*^9}, {
   3.646821681555751*^9, 3.6468216882481627`*^9}, 
   3.6477387563381224`*^9},ExpressionUUID->"490fee5a-b90f-4aec-8382-\
2d1401d876e2"],

Cell[BoxData["\[FilledSquare]"], "Text",
 CellChangeTimes->{{3.646822192831049*^9, 
  3.646822214951888*^9}},ExpressionUUID->"7e3aef51-30f9-4a74-a7ef-\
e723957b1dd5"],

Cell[TextData[{
 StyleBox["Example 5. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["dim = 5",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6468222614867697`*^9, 3.6468222815172048`*^9}, {
  3.701756726051785*^9, 
  3.7017567276598377`*^9}},ExpressionUUID->"3a6702d2-c7af-4c38-ac5a-\
11a390cefb71"],

Cell["\<\
In this example we check the function dd for all p-forms in dimension 5.\
\>", "Text",
 CellChangeTimes->{{3.6468966121086235`*^9, 3.6468966567871027`*^9}, 
   3.701754248285336*^9},ExpressionUUID->"5600ae40-a498-4905-9269-\
4e636686bbf1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "5"}], ";"}]], "Input",
 CellChangeTimes->{{3.6468223006428385`*^9, 
  3.6468223059780483`*^9}},ExpressionUUID->"fd2bbdd1-3925-4df8-951f-\
1c610e96f4f4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"af", "[", 
      RowBox[{"u", ",", "0"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "}"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6471573840525723`*^9, 3.6471574035838065`*^9}, {
  3.6471574838615475`*^9, 
  3.647157487402754*^9}},ExpressionUUID->"1da3a31d-ba67-4c06-9ee2-\
1ece512760ed"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646822311313257*^9, 3.6468223330440955`*^9}, 
   3.6471574279198494`*^9, 
   3.701754248307027*^9},ExpressionUUID->"453f72ee-267a-4c0b-9b43-\
d419c7f7c866"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468223466161194`*^9, 3.6468223653517523`*^9}, 
   3.646823658126023*^9, 3.647157512175597*^9, {3.701754248313917*^9, 
   3.701754248316256*^9}},ExpressionUUID->"aa6ecf72-21c8-4181-87c0-\
5ed6f72f02b8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",ExpressionUUID->\
"e0ba50b7-74b0-4837-bc1d-f6d310161dd0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 
   3.701754248322608*^9},ExpressionUUID->"ae40389f-a411-4815-9650-\
61afd2e79d07"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 
  3.6468225493696756`*^9}},ExpressionUUID->"234047b7-b570-4444-aca9-\
a8ea8d608623"],

Cell[BoxData[
 RowBox[{"%%", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646822559494093*^9, 
  3.646822596310158*^9}},ExpressionUUID->"287f82ef-2509-4878-915c-\
928933657817"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.646823602901926*^9, {3.701754248331914*^9, 
   3.7017542483342733`*^9}},ExpressionUUID->"8e8e0bca-4949-49b5-828a-\
366b14302d34"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6468229039270983`*^9},ExpressionUUID->"b8bc3bc9-7b71-47b2-858a-\
815b690f3a1e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 
   3.701754248339923*^9},ExpressionUUID->"4c0fe9c0-b3b8-424a-aafd-\
d03d8329d78f"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 
  3.6468225493696756`*^9}},ExpressionUUID->"e674ecaa-0c9d-4804-a3aa-\
d19be768c511"],

Cell[BoxData[
 RowBox[{"%%", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646822559494093*^9, 3.646822596310158*^9}, {
  3.6468229354859533`*^9, 
  3.646822937373557*^9}},ExpressionUUID->"52014414-976d-4ad1-aed9-\
676ed31ba785"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6471576700478745`*^9, 3.6471576756014843`*^9}, {
   3.647157707066739*^9, 3.647157717097557*^9}, 
   3.7017542483465853`*^9},ExpressionUUID->"345e4b9a-be36-4992-9336-\
dd0b3a3574f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646822559494093*^9, 3.646822596310158*^9}, {
   3.6468229354859533`*^9, 3.646822937373557*^9}, 3.6468235360402083`*^9, 
   3.646823778355434*^9, {3.7017542483494453`*^9, 
   3.701754248351049*^9}},ExpressionUUID->"df54c1c3-5819-498a-a1d1-\
9bfb7bebf253"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "3"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.646822980554433*^9},ExpressionUUID->"6353cb41-55e7-47a0-b060-\
e1b75e9acc4c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 
   3.701754248353744*^9},ExpressionUUID->"026d2654-7a72-43fe-b9c7-\
cdd673535b2c"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 
  3.6468225493696756`*^9}},ExpressionUUID->"4b0c8601-78cc-4c55-81c4-\
2de877ad24db"],

Cell[BoxData[
 RowBox[{"%%", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646822559494093*^9, 3.646822596310158*^9}, {
  3.646823008899682*^9, 
  3.646823013595291*^9}},ExpressionUUID->"a2dd2808-d0f7-4e4b-b30f-\
954b7a0ccbd2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.646823602901926*^9, {3.701754248356279*^9, 
   3.701754248357876*^9}},ExpressionUUID->"05cd66f9-40dc-48cb-87a5-\
d2f3bf6ae4b5"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.7017570112598333`*^9, 
  3.701757020369122*^9}},ExpressionUUID->"bbfb15fc-730a-432b-81ee-\
980e466181d3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "4"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6468230847002153`*^9},ExpressionUUID->"8c9c8c5b-adf2-4af0-871a-\
1df53fd1bcc1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 
   3.701754248359941*^9},ExpressionUUID->"0f38f281-dccc-4a24-b719-\
69ee3b8d6b34"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 
  3.6468225493696756`*^9}},ExpressionUUID->"64a354b5-2e0b-4cf6-bf3d-\
a25dfd8aa8cf"],

Cell[BoxData[
 RowBox[{"%%", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.646822559494093*^9, 3.646822596310158*^9}, 
   3.646823129409894*^9},ExpressionUUID->"a75e9c11-b646-4ffd-9bfc-\
7e758a6d4763"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "5"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{3.646823194883209*^9, 
  3.6471579149995046`*^9},ExpressionUUID->"3bc66be9-d3cc-4703-acb2-\
66dfa67b7032"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6471579284935284`*^9, 
  3.6471579394291477`*^9}},ExpressionUUID->"e614b9ec-dddc-4dfe-b7a3-\
052024eaf176"],

Cell[BoxData[
 RowBox[{"daa5", "=", 
  RowBox[{
   RowBox[{"dd", "[", "\[Lambda]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, {3.646824155345696*^9, 
   3.646824159105303*^9}, 
   3.701754248365937*^9},ExpressionUUID->"41f1745d-4927-4fef-8a0a-\
c8226d0f5e5a"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 
  3.6468225493696756`*^9}},ExpressionUUID->"3a18b5c4-fb9f-43e4-96d8-\
3178fad3312c"],

Cell[BoxData[
 RowBox[{"%%", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.646822559494093*^9, 3.646822596310158*^9}, {
   3.646823399212368*^9, 3.646823401536772*^9}, {3.6468239009248495`*^9, 
   3.6468239163844767`*^9}, 
   3.646823982169792*^9},ExpressionUUID->"90833d9a-efe5-46d2-9d09-\
0c6f871ec06f"],

Cell[BoxData[
 RowBox[{"TensorDimensions", "[", "daa5", "]"}]], "Input",
 CellChangeTimes->{{3.6468240632431345`*^9, 3.6468240924307857`*^9}, {
  3.701757276956669*^9, 
  3.701757278639168*^9}},ExpressionUUID->"dbb6ea42-dbe5-4689-8412-\
efcf3c66a45b"],

Cell[BoxData[
 RowBox[{"daa5", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "5"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.646824180929741*^9, 
  3.6468241953285666`*^9}},ExpressionUUID->"9aa145c7-bb6a-473b-acbb-\
3a6a227edc0a"],

Cell["OK. \[FilledSquare]", "Text",
 CellChangeTimes->{{3.646823445388449*^9, 
  3.646823455466067*^9}},ExpressionUUID->"6be41199-10ca-4762-aede-\
0b93d11551ea"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
2.4.3. The Exterior Differential of Antisymmetric Covariant Tensors\
\>", "Subsubsection",
 CellChangeTimes->{{3.5866649932855587`*^9, 3.586665013117693*^9}, {
   3.5872725258470573`*^9, 3.587272530654332*^9}, {3.5898752162352247`*^9, 
   3.589875218169628*^9}, 3.5899533142260494`*^9, {3.609309548254758*^9, 
   3.6093095610144877`*^9}, 3.6095615905629835`*^9, 3.6177190633933744`*^9, 
   3.6177712155926075`*^9, 3.6274424857387977`*^9, 3.627443068992622*^9, 
   3.635839359077566*^9, {3.6454362817971053`*^9, 3.645436327513186*^9}, 
   3.645588884269096*^9, {3.645594238601818*^9, 3.6455942534374447`*^9}, 
   3.645610929040021*^9, 3.6456110340382056`*^9, 3.646824320846387*^9, {
   3.646824396054119*^9, 3.646824442916601*^9}, {3.6468244997007008`*^9, 
   3.6468245299803543`*^9}, {3.6468246184013095`*^9, 3.646824690489036*^9}, {
   3.6468289408323607`*^9, 3.646828956869189*^9}, 3.647172964077033*^9, 
   3.6686902023402777`*^9, 3.668743309501213*^9, 3.701753242696518*^9, 
   3.7017575387011757`*^9, 3.701758555039019*^9, 3.703043707811245*^9, 
   3.703044404276865*^9, 3.704005964351521*^9, {3.704031057764558*^9, 
   3.704031070111843*^9}},ExpressionUUID->"1d2549d3-f1be-40fb-8d4f-\
c436c5833611"],

Cell["In this subsection we check the action of dd on tensors.", "Text",
 CellChangeTimes->{{3.64716731821129*^9, 3.647167361750966*^9}, 
   3.6471675097628207`*^9, 
   3.7017542483784227`*^9},ExpressionUUID->"e90e820a-17b2-49fa-95cd-\
5a75eea52836"],

Cell[TextData[{
 StyleBox["Example 1. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["dim = 2",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6466471691339455`*^9, 3.6466471788527627`*^9}, {
  3.701757947545947*^9, 
  3.7017579498577127`*^9}},ExpressionUUID->"13115d90-2faa-4679-90e0-\
dd26c16b07fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{"Clear", "[", "sq", "]"}], ";", 
  RowBox[{
   RowBox[{"sq", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"var", "[", "x", "]"}], "}"}], ".", 
    RowBox[{"{", 
     RowBox[{"var", "[", "x", "]"}], "}"}]}]}]}]], "Input",
 CellChangeTimes->{{3.645623617600792*^9, 3.645623728329787*^9}, {
   3.645623763788649*^9, 3.6456238101363306`*^9}, {3.6457673167383943`*^9, 
   3.6457673273308125`*^9}, 3.645767585932467*^9, {3.6471539200328875`*^9, 
   3.6471539231840935`*^9}, {3.6471542451998587`*^9, 3.647154260441086*^9}, {
   3.6471548154900603`*^9, 3.647154819592868*^9}, {3.6471550456372643`*^9, 
   3.647155049147271*^9}, {3.64716682301962*^9, 
   3.6471668336900387`*^9}},ExpressionUUID->"98fd457e-6f22-418d-af7b-\
0d0bebb1cf86"],

Cell[BoxData[
 RowBox[{
  RowBox[{"u", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}], "=", 
  RowBox[{"sq", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6456238206663485`*^9, 3.645623837264778*^9}, {
  3.6471667958599725`*^9, 
  3.6471668094319963`*^9}},ExpressionUUID->"517d4485-696c-48a7-8453-\
08afe4bfb347"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
  RowBox[{"tensor", "[", 
   RowBox[{"u", ",", "0", ",", "0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6471668624876895`*^9, 
  3.647166901394158*^9}},ExpressionUUID->"6c48ddb6-c9b2-4769-9243-\
4a2ad90c750b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456234702429333`*^9, 3.6456234912873707`*^9}, {
   3.6456235220194244`*^9, 3.645623550286674*^9}, 3.645623743742614*^9, {
   3.6456238531612062`*^9, 3.6456238850164623`*^9}, 3.645767611968913*^9, {
   3.647166915168982*^9, 3.647166915356182*^9}, 
   3.701754248397169*^9},ExpressionUUID->"31e88a8a-746f-4802-85fc-\
edc3671dd6f8"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"dd", "[", "tt", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647166933904615*^9, 3.64716705042122*^9}, 
   3.701754248410982*^9},ExpressionUUID->"690d7c18-2057-4c93-849f-\
9fcd4551aeaf"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dd", "[", 
    RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.6471555007524643`*^9, 3.6471555604381685`*^9}, {
  3.647167178591045*^9, 3.6471672391659513`*^9}, {3.701754248425806*^9, 
  3.701754248428018*^9}},ExpressionUUID->"3b290bc0-be84-4489-8926-\
b07a2eda3a05"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{
  3.647167276309616*^9},ExpressionUUID->"0f59b6e5-f995-4929-8be7-\
d647c30a170b"],

Cell["", "Text",ExpressionUUID->"693241a7-be51-4591-8ef4-e7b79b9c255e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6467174302108555`*^9, 3.646717436341666*^9}, {
  3.6467174673857207`*^9, 
  3.646717479023341*^9}},ExpressionUUID->"6476b383-7376-43c7-922a-\
51e14e1126f4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.64716823005289*^9, 
  3.647168298853011*^9}},ExpressionUUID->"d1d59da6-856f-4600-9ae9-\
41d91c5b04ca"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6467175259170237`*^9, 3.646717541657451*^9}, 
   3.6468086978789635`*^9, {3.6471683121306343`*^9, 3.6471683123178344`*^9}, 
   3.7017542484412746`*^9},ExpressionUUID->"61d7246b-6932-4437-83e4-\
eb37384f88b8"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",ExpressionUUID->"f3b0961a-dae7-4e03-8941-3621d9309620"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.701754248456657*^9, 
  3.701754248458889*^9}},ExpressionUUID->"bc3f4f42-d764-4931-b745-\
c34101d561b9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.647168865853611*^9},ExpressionUUID->"73ab22cd-9198-46e2-8434-\
dcf1a1698eb5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.647168435155252*^9},ExpressionUUID->"99fa35c4-1b2a-472e-b95e-\
79af4c781f1f"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.647168495905759*^9, 
  3.647168566118883*^9}},ExpressionUUID->"41620aeb-013b-452c-b811-\
bd834e188f1e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.701754248472609*^9},ExpressionUUID->"a6682abf-786e-4428-a8ed-\
b6bcefe14a63"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6467175259170237`*^9, 3.646717541657451*^9}, {
  3.646806497379592*^9, 3.646806515850024*^9}, {3.647168340870885*^9, 
  3.647168341042485*^9}, {3.70175424848674*^9, 
  3.701754248489069*^9}},ExpressionUUID->"888b770e-74db-4b1e-9150-\
c93a5c68e4ee"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6467166633135085`*^9, 
  3.6467166633135085`*^9}},ExpressionUUID->"fa2987f5-de11-4f6a-8ba6-\
8ffaad41fcc0"],

Cell[TextData[{
 StyleBox["Example 2. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["dim = 3",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6466471691339455`*^9, 3.6466471788527627`*^9}, {
  3.6466472169036293`*^9, 3.646647218245232*^9}, {3.7017580034730043`*^9, 
  3.701758004705311*^9}},ExpressionUUID->"8052dd80-4d4a-4e0d-a15a-\
975f003229e2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6468020910846524`*^9, {3.64680221519847*^9, 
   3.6468022231544847`*^9}},ExpressionUUID->"76258884-9e79-4b47-b1eb-\
650dc65a12d0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1"}], "]"}]}]], "Input",Expr\
essionUUID->"4c0f3e25-603e-4b69-84fd-a491c946ef6a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456207910786276`*^9, 3.645620875833576*^9}, {
   3.645621252449438*^9, 3.645621287237499*^9}, 3.6456214668094144`*^9, {
   3.645621504436681*^9, 3.645621543467949*^9}, 3.6456221738494563`*^9, 
   3.6456222099323196`*^9, {3.6456223411129503`*^9, 3.6456223565569773`*^9}, {
   3.646646221619481*^9, 3.6466462286238937`*^9}, {3.646647045222928*^9, 
   3.646647048576934*^9}, 3.646808740233038*^9, {3.647170628781516*^9, 
   3.647170628999916*^9}, 
   3.701754248503231*^9},ExpressionUUID->"8c61142f-3299-4154-b31b-\
071ccb679bb5"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6471560134785643`*^9, 
  3.6471560277369895`*^9}},ExpressionUUID->"bc18aba4-e3dc-45d4-a2a0-\
84d5097c2296"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6471560451154203`*^9, 3.6471560799034815`*^9}, {
  3.6471706591879697`*^9, 3.64717065939077*^9}, {3.701754248522105*^9, 
  3.701754248524428*^9}},ExpressionUUID->"bd5d4fa1-45aa-4376-bd89-\
0c265d32799c"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",ExpressionUUID->"342168ec-6db4-435f-9a9a-c6ad56e1f938"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6456116748019342`*^9, 3.645611736162842*^9}, {
   3.645611914675357*^9, 3.645611952820424*^9}, {3.6456125307966456`*^9, 
   3.6456125911551514`*^9}, 3.645612665962283*^9, {3.6456204964409103`*^9, 
   3.645620506674528*^9}, {3.646635017395054*^9, 3.6466350299374757`*^9}, {
   3.647156120869153*^9, 
   3.6471561221327553`*^9}},ExpressionUUID->"5c7a872b-7fc1-4e42-abef-\
7879b9def95e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6471707112960615`*^9},ExpressionUUID->"607091f6-d0ba-45c7-acae-\
6f2c577c0d14"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6456207910786276`*^9, 3.645620875833576*^9}, {
   3.645621252449438*^9, 3.645621287237499*^9}, 3.6456214668094144`*^9, {
   3.645621504436681*^9, 3.645621543467949*^9}, 3.6456221738494563`*^9, 
   3.6456222099323196`*^9, {3.6456223411129503`*^9, 3.6456223565569773`*^9}, {
   3.646646221619481*^9, 3.6466462286238937`*^9}, {3.646647045222928*^9, 
   3.646647048576934*^9}, {3.647170724602885*^9, 3.6471707247900853`*^9}, 
   3.701754248538477*^9},ExpressionUUID->"e9a7aa30-980c-4c82-8f2b-\
8fb4ccfc5fd0"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6471707745405726`*^9, 
  3.647170815024644*^9}},ExpressionUUID->"4bd988fe-a795-458d-9a01-\
6a7cfb8a93b9"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"dd", "[", "tt", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.645622478689592*^9, 3.6456225256924744`*^9}, {
   3.647156184423665*^9, 3.6471562316605477`*^9}, {3.6471708343686776`*^9, 
   3.6471708345558777`*^9}, 
   3.701754248555779*^9},ExpressionUUID->"4288c821-7fd6-4dcf-8706-\
021cb8afda05"],

Cell[BoxData[
 RowBox[{"%", " ", "\[Equal]", 
  RowBox[{"{", "%%", "}"}]}]], "Input",
 CellChangeTimes->{{3.6471562461685734`*^9, 
  3.6471563090834837`*^9}},ExpressionUUID->"1612d32f-eb01-4eb6-9c4a-\
040a322692cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"dd", "[", 
    RowBox[{"dd", "[", "tt", "]"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646809008459909*^9, 3.646809021641932*^9}, {
  3.647170870252741*^9, 3.6471708704399414`*^9}, {3.701754248571316*^9, 
  3.701754248573523*^9}},ExpressionUUID->"dc476540-2b6f-41b7-96f7-\
2cf18b1acf63"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6467168063345594`*^9, 
  3.6467168063345594`*^9}},ExpressionUUID->"a26248b7-52d5-49d8-b065-\
d64675e58ce7"],

Cell[TextData[{
 StyleBox["Example 3. ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["dim = 5",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.6468222614867697`*^9, 3.6468222815172048`*^9}, {
  3.701758108632208*^9, 
  3.701758109624406*^9}},ExpressionUUID->"7f0ff1be-7d08-41a9-8a74-\
e5a1d5480947"],

Cell["\<\
In this example we check the operation dd for all p-form tensors in dimension \
5.\
\>", "Text",
 CellChangeTimes->{{3.6468966121086235`*^9, 3.6468966567871027`*^9}, {
   3.668743002988604*^9, 3.668743007220459*^9}, 3.7017542485929403`*^9, {
   3.7030442350936003`*^9, 
   3.703044241157329*^9}},ExpressionUUID->"b5c75fcd-35f7-46e3-ae41-\
25682a319ee7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "5"}], ";"}]], "Input",
 CellChangeTimes->{{3.6468223006428385`*^9, 
  3.6468223059780483`*^9}},ExpressionUUID->"57eb84aa-c473-4766-88bf-\
ebaa4e0c762f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"af", "[", 
      RowBox[{"u", ",", "0"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "}"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6471573840525723`*^9, 3.6471574035838065`*^9}, {
  3.6471574838615475`*^9, 
  3.647157487402754*^9}},ExpressionUUID->"905dfc54-def5-442e-9568-\
487c29d4f7d0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"u", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}], "=", " ", 
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6471718088905983`*^9, 3.64717184371086*^9}, {
  3.6471720886914916`*^9, 
  3.647172091405896*^9}},ExpressionUUID->"8e365d66-14dd-41f5-b36d-\
6514cb23c2b0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"tensor", "[", 
   RowBox[{"u", ",", "0", ",", "0"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6471717314616623`*^9, {3.6471719011521606`*^9, 
   3.6471719160833874`*^9}},ExpressionUUID->"35e38cbf-8c41-4285-b6a7-\
1c453af7a80e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646822311313257*^9, 3.6468223330440955`*^9}, 
   3.6471574279198494`*^9, 3.647171575037387*^9, 
   3.7017542485973263`*^9},ExpressionUUID->"0add7885-ee88-4c73-968c-\
c30984ce9eee"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64717195747126*^9, 
  3.6471719822461033`*^9}},ExpressionUUID->"72a4df66-1284-46e9-b82a-\
357291662264"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468223466161194`*^9, 3.6468223653517523`*^9}, 
   3.646823658126023*^9, 3.647157512175597*^9, 3.647171577174591*^9, {
   3.701754248610415*^9, 
   3.701754248612423*^9}},ExpressionUUID->"a69a00f3-8528-4bc4-8b4a-\
8b13358b024b"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",ExpressionUUID->"aa33f034-c620-4b2b-b674-324a082abc60"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "1"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",ExpressionUUID->\
"4a8723f3-70c4-4ccc-a287-feb011886848"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.647172164917226*^9},ExpressionUUID->"0bebd4a7-1a13-446e-936e-\
68a7dbe62ca2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 3.64717159378862*^9, 
   3.7017542486266212`*^9},ExpressionUUID->"86cdcd13-7b87-41ed-92f2-\
4e7348266042"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 3.6468225493696756`*^9}, 
   3.647172195230079*^9},ExpressionUUID->"6652969f-902f-4695-987e-\
00451decda20"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.646823602901926*^9, 3.647171598921029*^9, {3.70175424864147*^9, 
   3.7017542486435547`*^9}},ExpressionUUID->"f791cbb0-b339-4e42-aa13-\
e36b94fd0281"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",ExpressionUUID->"8c23e298-d34b-49c3-a282-1151fdd7d686"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{3.6468229039270983`*^9, 
  3.6471723034836693`*^9},ExpressionUUID->"98dbef38-fb6c-415f-873b-\
d69aa08116e5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}]], "Input",Expr\
essionUUID->"4a63f35f-34d8-42ef-9aa7-4907838210c8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 3.647171614695657*^9, 
   3.701754248657229*^9},ExpressionUUID->"f208476c-8948-4a01-9d13-\
9ed5e19bcfd8"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", 
  RowBox[{
   RowBox[{"dd", "[", "tt", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6471576700478745`*^9, 3.6471576756014843`*^9}, {
   3.647157707066739*^9, 3.647157717097557*^9}, 3.647171617176061*^9, 
   3.701754248671422*^9},ExpressionUUID->"e022146e-d31e-463b-a056-\
941a189c75cc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.646822559494093*^9, 3.646822596310158*^9}, {
   3.6468229354859533`*^9, 3.646822937373557*^9}, 3.6468235360402083`*^9, 
   3.646823778355434*^9, 3.6471716190792646`*^9, {3.701754248686205*^9, 
   3.701754248688341*^9}},ExpressionUUID->"b6322536-035e-41f0-bd39-\
1d4a504df64a"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",ExpressionUUID->"b7aebcdf-8252-4e4d-83a2-bc25c5337741"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "3"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{3.646822980554433*^9, 
  3.6471725221440554`*^9},ExpressionUUID->"47fe2e6d-fe8b-4d57-bd4a-\
e8fdc0bc0ce7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "3"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.647172549446103*^9},ExpressionUUID->"9a26241e-d5d5-4d8c-bed8-\
271c9690216f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 3.6471716325732884`*^9, 
   3.701754248701509*^9},ExpressionUUID->"1fc6ffff-c49a-4587-a733-\
01daab4ac928"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 3.6468225493696756`*^9}, 
   3.647172565834132*^9},ExpressionUUID->"b04df6c9-caa8-4f1f-8e29-\
8c5636c1d677"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.646823602901926*^9, 3.64717163383689*^9, {3.70175424871462*^9, 
   3.701754248716689*^9}},ExpressionUUID->"d281fce9-71b4-47e8-ba96-\
1d1a1d78beeb"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",ExpressionUUID->"c9b9e957-5573-4763-8f76-e390b5551f47"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "4"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{3.6468230847002153`*^9, 
  3.6471728063849554`*^9},ExpressionUUID->"4e5b92e2-30bc-4ea5-9a1b-\
83aa7167772a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "4"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.64717279171893*^9},ExpressionUUID->"b29da9fd-5849-4612-9589-32228fe9d630"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, 3.64717164496171*^9, 
   3.701754248729589*^9},ExpressionUUID->"f8a9a846-41ff-4856-9151-\
57e7b4bc1392"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.646822537607255*^9, 3.6468225493696756`*^9}, 
   3.647172828084594*^9},ExpressionUUID->"47123094-2d24-4d90-a785-\
09be1fe4a4f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dd", "[", "tt", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468232794821577`*^9, 3.64682329200898*^9}, 
   3.6471716474421144`*^9, {3.701754248743712*^9, 
   3.701754248745944*^9}},ExpressionUUID->"4722396c-de9e-438d-8540-\
dabc81b8fc3f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Lambda]", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"u", ",", "5"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"\[Lambda]", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{3.646823194883209*^9, 
  3.6471579149995046`*^9},ExpressionUUID->"c94f88cd-002b-4f04-aed7-\
8cb459345268"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"\[Lambda]", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "5"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.647172874809676*^9},ExpressionUUID->"b8ef322e-5f6f-4cfd-acec-\
abdc60bd189a"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6471579284935284`*^9, 
  3.6471579394291477`*^9}},ExpressionUUID->"7e802a2b-3c6c-4fea-b3a6-\
6802566fbd28"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "tt", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6468224834751596`*^9, 3.6468224860959644`*^9}, {
   3.6468225180136204`*^9, 3.646822526188035*^9}, {3.646824155345696*^9, 
   3.646824159105303*^9}, 3.647171662964142*^9, 3.6471729028741255`*^9, 
   3.701754248759757*^9},ExpressionUUID->"d1b51cb5-132f-4a09-b8e3-\
bdcf257be4f0"],

Cell["OK. \[FilledSquare]", "Text",
 CellChangeTimes->{{3.646823445388449*^9, 
  3.646823455466067*^9}},ExpressionUUID->"df29ea15-74ec-479f-ab54-\
750bea604179"]
}, Closed]],

Cell[CellGroupData[{

Cell["2.4.4.  Linearity and Product Rules", "Subsubsection",
 CellChangeTimes->{{3.5866649932855587`*^9, 3.586665013117693*^9}, {
   3.5872725258470573`*^9, 3.587272530654332*^9}, {3.5898752162352247`*^9, 
   3.589875218169628*^9}, 3.5899533142260494`*^9, {3.609309548254758*^9, 
   3.6093095610144877`*^9}, 3.6095615905629835`*^9, 3.6177190633933744`*^9, 
   3.6177712155926075`*^9, 3.6274424857387977`*^9, 3.627443068992622*^9, 
   3.635839359077566*^9, {3.6454362817971053`*^9, 3.645436327513186*^9}, 
   3.645588884269096*^9, {3.645594238601818*^9, 3.6455942534374447`*^9}, 
   3.645610929040021*^9, 3.6456110340382056`*^9, 3.646824320846387*^9, {
   3.646824396054119*^9, 3.646824442916601*^9}, {3.6468244997007008`*^9, 
   3.6468245299803543`*^9}, {3.6468246184013095`*^9, 3.646824690489036*^9}, {
   3.6468289408323607`*^9, 3.646828956869189*^9}, 3.647172964077033*^9, {
   3.6473134672120247`*^9, 3.647313500112482*^9}, 3.6687433176990557`*^9, 
   3.668744733620945*^9, {3.701758561550065*^9, 3.701758574572652*^9}, 
   3.701759082848455*^9, 3.701760392893756*^9, 3.703044413429598*^9, 
   3.70304848281804*^9, {3.704031080463763*^9, 3.704031099775312*^9}, 
   3.704094279529212*^9},ExpressionUUID->"32fca9e3-fbff-4903-9bbb-\
10c2d8e524b1"],

Cell["\<\
In this subsection we demonstrate the rules of the exterior calculus. We \
introduce the standard basis forms and consider the basis representations of \
p-forms and  antisymmetric covariant tensors.\
\>", "Text",
 CellChangeTimes->{{3.701760166423328*^9, 
  3.701760366374086*^9}},ExpressionUUID->"a2de82e4-4cb5-48bb-9f15-\
5c2be9e4c8ce"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"wa", ",", "wb", ",", "wc"}], "]"}], ";", " ", 
  RowBox[{
   RowBox[{"wa", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"a", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"wb", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"b", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"wc", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], "  ", "=", " ", 
   RowBox[{
    RowBox[{"plus", "[", 
     RowBox[{"wa", " ", ",", "wb"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " "}]}]], "Input",
 CellChangeTimes->{{3.6473135327789392`*^9, 3.647313586349434*^9}, {
   3.647313622541497*^9, 3.6473137105568514`*^9}, {3.6473138213482466`*^9, 
   3.64731389767918*^9}, {3.647314608541229*^9, 3.6473146205064497`*^9}, {
   3.6473146668229313`*^9, 3.64731471167301*^9}, {3.7040929895823517`*^9, 
   3.704093001011888*^9}, 
   3.704093251642289*^9},ExpressionUUID->"86b285d3-2ba0-4ce0-b546-\
dfc8701f1191"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dd", "[", "wc", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.647315141391365*^9, 3.6473151559149904`*^9}, 
   3.701754248773567*^9},ExpressionUUID->"d834e9d3-6a31-4938-85fa-\
e7e9d73e7d98"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", " ", "-", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"dd", "[", "wa", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", 
     RowBox[{
      RowBox[{"dd", "[", "wb", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ")"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6473137521621246`*^9, 3.6473137897269907`*^9}, {
   3.6473147331854477`*^9, 3.647314768051509*^9}, 3.647314930354194*^9, 
   3.647314975781474*^9, {3.647315077571653*^9, 3.6473150862296677`*^9}, {
   3.6473152329323254`*^9, 3.647315244460746*^9}, {3.70175424878697*^9, 
   3.7017542487892036`*^9}},ExpressionUUID->"2b3749ee-29f8-4a8f-aba0-\
9fb2b220c00f"],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"NumberQ", "[", "xx", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"Simplify", "[", " ", 
    RowBox[{
     RowBox[{"dd", "[", 
      RowBox[{"times", "[", 
       RowBox[{"xx", ",", "wa"}], "]"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], " ", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647315317312874*^9, 3.6473153334121017`*^9}, {
   3.6473153828173885`*^9, 3.647315496354388*^9}, {3.6473155364308586`*^9, 
   3.647315547959279*^9}, {3.6473155962569637`*^9, 3.64731566216708*^9}, {
   3.64731569659634*^9, 3.6473157191539793`*^9}, 
   3.7017542488019743`*^9},ExpressionUUID->"fd12fb76-2451-4840-abb3-\
871c755d56d6"],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"NumberQ", "[", "xx", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"Simplify", "[", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"dd", "[", 
       RowBox[{"times", "[", 
        RowBox[{"xx", ",", "wa"}], "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], " ", "-", 
     RowBox[{"xx", "*", 
      RowBox[{
       RowBox[{"dd", "[", "wa", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647315317312874*^9, 3.6473153334121017`*^9}, {
  3.6473153828173885`*^9, 3.647315496354388*^9}, {3.6473155364308586`*^9, 
  3.647315547959279*^9}, {3.6473155962569637`*^9, 3.64731566216708*^9}, {
  3.64731569659634*^9, 3.6473157191539793`*^9}, {3.647315750260434*^9, 
  3.64731576501806*^9}, {3.6473157975129175`*^9, 3.647315826435368*^9}, {
  3.6473158615198298`*^9, 3.6473158801150627`*^9}, {3.701754248815402*^9, 
  3.701754248817666*^9}},ExpressionUUID->"c6aff3ae-cf9c-4e82-bbc6-\
8b0347a56396"],

Cell[TextData[{
 "These calculations illustrate \n",
 StyleBox["Proposition1.",
  FontColor->RGBColor[0, 0, 1]],
 " dd is a linear operator on the real vector space of all p-forms on a \
manifold. \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6473173588104596`*^9, 3.647317479757472*^9}, {
   3.64731754125278*^9, 3.6473175513615975`*^9}, {3.647580779711441*^9, 
   3.647580805139486*^9}, 
   3.701754248838291*^9},ExpressionUUID->"89d32340-62a3-4aa3-8c8a-\
79be568946b8"],

Cell["\<\
This proposition implies the same property for the corresponding tensors:\
\>", "Text",
 CellChangeTimes->{{3.6473176154777107`*^9, 3.6473177018706617`*^9}, 
   3.7040931106572237`*^9},ExpressionUUID->"af63d19b-4368-4451-93bd-\
aec8cf81820a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"wa", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6473177038830657`*^9, 
  3.647317771368784*^9}},ExpressionUUID->"be5c56a2-aab1-4f46-b8ad-\
3709b5455b64"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"wb", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6473177851592083`*^9, 
  3.6473178052208433`*^9}},ExpressionUUID->"7109e570-4477-4d00-9c83-\
1f60cc093034"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"twc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"wc", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6473177851592083`*^9, 
  3.6473178315380898`*^9}},ExpressionUUID->"87b33d26-a93f-4f9e-a19f-\
bec43d829594"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.647317867589753*^9, 
  3.647317885217784*^9}},ExpressionUUID->"f1377bed-4636-41c8-93d5-\
351f3be1c8f3"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", 
  RowBox[{"tensorToArray", "[", 
   RowBox[{"twc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647317906855022*^9, 
  3.6473180147604113`*^9}},ExpressionUUID->"44cc101b-ac5f-4bfd-8835-\
3b2b3be9a587"],

Cell[BoxData[
 RowBox[{
  RowBox[{"twc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647318032965644*^9, 
  3.6473180528400784`*^9}},ExpressionUUID->"b21bebee-04a2-47c4-8058-\
609aadffa340"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twc", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"dd", "[", "twa", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
    RowBox[{
     RowBox[{"dd", "[", "twb", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647318076099719*^9, 3.647318080062126*^9}, {
  3.6473181543106565`*^9, 3.647318219721572*^9}, {3.6473183585618153`*^9, 
  3.64731842941714*^9}, {3.701754248863369*^9, 
  3.701754248866785*^9}},ExpressionUUID->"a58ae349-f9c4-4d53-bfe1-\
f4ad08885e76"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{"3.", "*", "twa"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"3.", "*", 
    RowBox[{
     RowBox[{"dd", "[", "twa", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6473182430280128`*^9, 3.647318339982183*^9}, {
   3.647318468308008*^9, 3.6473185072924767`*^9}, 3.6473185407545357`*^9, 
   3.6473188515250816`*^9, {3.6473193076542826`*^9, 3.6473193794144087`*^9}, {
   3.647319438866113*^9, 3.647319445355725*^9}, {3.6473195558663187`*^9, 
   3.647319559953526*^9}, {3.701754248888246*^9, 
   3.701754248890078*^9}},ExpressionUUID->"fa6ad360-2266-4d53-9fd9-\
b24455df9738"],

Cell["\<\
\[FilledSquare]
Now we consider the product of  the p-form tensor twa[var[x]] with a function:\
\>", "Text",
 CellChangeTimes->{{3.6473212937959795`*^9, 3.647321368582511*^9}, {
  3.6476626652076893`*^9, 
  3.647662690557734*^9}},ExpressionUUID->"1d49fa75-e2c0-4e4c-bc9a-\
79ce585ffdd1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"times", "[", 
   RowBox[{
    RowBox[{"u", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "twa"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647319814717573*^9, 3.6473198186799803`*^9}, {
  3.647319876103681*^9, 3.647319895198115*^9}, {3.647319975101455*^9, 
  3.647320008719514*^9}},ExpressionUUID->"96d85545-4303-49d8-9a6f-\
7f1c0bdae72a"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6473199262733693`*^9, 3.6473199414677963`*^9}, {
  3.6475809509841423`*^9, 
  3.647580958440955*^9}},ExpressionUUID->"d8b58f58-4bb8-47e9-b049-\
43d4652a2b5d"],

Cell["We calculate the exterior differential of the product", "Text",
 CellChangeTimes->{{3.647322271498897*^9, 
  3.647322296848942*^9}},ExpressionUUID->"f56458c1-540e-4c7d-9580-\
afd8c2cf46d4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"times", "[", 
    RowBox[{
     RowBox[{"u", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "twa"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6473196409956684`*^9, 3.647319674426527*^9}, 
   3.6473197122565937`*^9, {3.6473201303529277`*^9, 3.647320144626953*^9}, 
   3.701754248910763*^9},ExpressionUUID->"9ead324d-abbe-4b97-bbab-\
5f79459ea064"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", 
  RowBox[{"tensorToArray", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647320207011462*^9, 
  3.6473202498803377`*^9}},ExpressionUUID->"8ae4f4ed-c973-4a75-9afb-\
ea95b0695d56"],

Cell[BoxData[
 RowBox[{"stp1", " ", "=", 
  RowBox[{"strictcoords", "[", "%%", "]"}]}]], "Input",
 CellChangeTimes->{{3.6473202632651615`*^9, 3.6473202726407776`*^9}, {
  3.647324233295943*^9, 
  3.6473242369775496`*^9}},ExpressionUUID->"74f9e614-fccb-465b-a08c-\
632188409afc"],

Cell["To use the Array forms we define", "Text",
 CellChangeTimes->{{3.6473243006100607`*^9, 
  3.6473243252893047`*^9}},ExpressionUUID->"19daf000-c7d2-45f9-b786-\
ecca967cdd5a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"aa", "[", 
   RowBox[{"var", "[", "x_", "]"}], "]"}], " ", "=", 
  RowBox[{"tensorToArray", "[", 
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.647320998908662*^9, {3.6473210666751804`*^9, 
   3.647321101572442*^9}},ExpressionUUID->"bf12110e-e521-44c5-981e-\
06254eb132f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"extprod", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "u", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"tensorToArray", "[", 
     RowBox[{"twa", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}], "]"}], " ", "+", " ", 
  RowBox[{
   RowBox[{"u", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
   RowBox[{
    RowBox[{"dd", "[", "aa", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.647320372449753*^9, 3.647320454755498*^9}, 
   3.6473204960799704`*^9, {3.6473205314538403`*^9, 3.647320602028364*^9}, 
   3.6473206668776784`*^9, 3.6473206982961335`*^9, {3.647321448532651*^9, 
   3.64732149308873*^9}, {3.6473215284851923`*^9, 3.6473216201821527`*^9}, {
   3.647321729366745*^9, 3.647321768008013*^9}, 
   3.701754248932926*^9},ExpressionUUID->"d4c63903-6b66-4325-a451-\
6791d09a9082"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64732182315411*^9, 
  3.647321833824528*^9}},ExpressionUUID->"eb7bd5bb-221f-4d8c-8385-\
39b7cbc8f7ed"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"stp1", "-", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.647321852482161*^9, 3.647321868409789*^9}, {
  3.6473244455811157`*^9, 
  3.647324475548768*^9}},ExpressionUUID->"7ba31c98-32b8-4a05-882d-\
f713374ad262"],

Cell["To calculate this result in tensor terms we set ", "Text",
 CellChangeTimes->{{3.6475813707340794`*^9, 
  3.647581395600523*^9}},ExpressionUUID->"0c4185d3-10d4-407c-81b2-\
061ba6d4507e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fu", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}]}], "]"}], "=", 
   RowBox[{"u", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"u0", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"fu", ",", "0", ",", "0"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.647581437096596*^9, 3.6475814493582172`*^9}, {
  3.647581502850711*^9, 3.647581524503549*^9}, {3.647581568230426*^9, 
  3.647581668991003*^9}},ExpressionUUID->"574cb52e-c7ca-4333-ae3d-\
f3e6108132d6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "u0", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647581690737441*^9, 3.6475817130610805`*^9}, 
   3.70175424895516*^9},ExpressionUUID->"ea495e04-907f-4d49-887c-\
b4c657e4bac3"],

Cell[BoxData[
 RowBox[{"su1", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "u0", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.647581754479153*^9, 3.6475818210756702`*^9}, 
   3.6475818998870087`*^9, 
   3.7017542489771147`*^9},ExpressionUUID->"ec297c91-0327-431f-b6a3-\
669927617ed4"],

Cell[BoxData[
 RowBox[{"su2", " ", "=", " ", 
  RowBox[{
   RowBox[{"u0", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{
    RowBox[{"dd", "[", "twa", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.647581922990649*^9, 3.647581927982658*^9}, {
   3.6475819699311314`*^9, 3.6475820131744075`*^9}, 
   3.701754249000742*^9},ExpressionUUID->"d6bae59a-a2c3-478d-874f-\
2c175ae9bd7a"],

Cell[BoxData[
 RowBox[{"su", " ", "=", " ", 
  RowBox[{"su1", "+", "su2"}]}]], "Input",
 CellChangeTimes->{{3.647582029351636*^9, 
  3.647582042393259*^9}},ExpressionUUID->"1e5fd443-9dd3-40d6-bb1e-\
937be15d529d"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"strictcoords", "[", "su", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647582084794133*^9, 3.6475821168989897`*^9}, 
   3.647582183620307*^9},ExpressionUUID->"7b4787f8-2da3-49dd-838e-\
9001f75e92f5"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", "stp1"}], "]"}]], "Input",
 CellChangeTimes->{{3.6475822066615477`*^9, 
  3.6475822206235723`*^9}},ExpressionUUID->"1cc26778-a47b-49db-859a-\
f5006fb4e7a0"],

Cell[TextData[{
 "Thus we got the following result formulated in traditional notation:\n\n",
 StyleBox["Propostion 2",
  FontColor->RGBColor[0, 0, 1]],
 ". The exterior differential of the product of a function f with a p-form w \
satisfies the product rule  "
}], "Text",
 CellChangeTimes->{{3.647582809337406*^9, 3.6475830674554596`*^9}, 
   3.647583130089569*^9, {3.647583163021227*^9, 3.6475832013972945`*^9}, 
   3.6475838280815954`*^9, 3.647583924271364*^9, 3.647662843063602*^9, 
   3.701759235879299*^9},ExpressionUUID->"345b0029-7b09-4c87-9230-\
d2f6f697d5c2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", 
   RowBox[{"f", "*", "w"}], "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"df", 
    FormBox["\[Wedge]",
     TraditionalForm], "w"}], " ", "+", " ", 
   RowBox[{"f", "*", 
    RowBox[{
     RowBox[{"d", "[", "w", "]"}], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6475838753028784`*^9, 
  3.6475838996857214`*^9}},ExpressionUUID->"c3ea2959-927c-4423-bf9f-\
e61a926b9346"],

Cell[" Here the traditional notations are", "Text",
 CellChangeTimes->{
  3.6475839340993814`*^9, {3.701759259559022*^9, 
   3.701759335069799*^9}},ExpressionUUID->"d76a08fc-3db3-42ed-a6dc-\
9d35ecee869f"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"d", "\[DotEqual]", " ", "dd"}], ";", "  ", 
  RowBox[{"f", "\[DotEqual]", " ", 
   RowBox[{"u0", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", " ", 
  RowBox[{"w", " ", "\[DotEqual]", " ", 
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"f", "*", "w"}], "]"}], " ", "\[DotEqual]", " ", 
   RowBox[{
    RowBox[{"dd", "[", 
     RowBox[{"times", "[", 
      RowBox[{
       RowBox[{"u", "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], ",", "twa"}], "]"}], "]"}], 
    "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"df", 
    FormBox["\[Wedge]",
     TraditionalForm], "w"}], " ", "\[DotEqual]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"dd", "[", "u0", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
    RowBox[{"twa", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f", "*", 
   RowBox[{"d", "[", "w", "]"}]}], " ", "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"u0", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
    RowBox[{
     RowBox[{"dd", "[", "twa", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}], "    ", 
   "\[FilledSquare]"}]}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.647583212364114*^9, 3.647583246138173*^9}, {
  3.6475832848262415`*^9, 3.6475834250704875`*^9}, {3.6475834639301558`*^9, 
  3.647583504084626*^9}, {3.6475835555179167`*^9, 3.6475837350430317`*^9}, {
  3.701754249025762*^9, 
  3.7017542490319643`*^9}},ExpressionUUID->"09658b9c-d464-4a0d-93d3-\
ca6846184ee7"],

Cell["\<\
Now we will define the standard basis differential forms depending on the \
charts of the manifold. We remember\
\>", "Text",
 CellChangeTimes->{{3.647586973853121*^9, 
  3.6475870614472747`*^9}},ExpressionUUID->"00590099-b738-4c73-8c0d-\
6ad3c7660878"],

Cell[BoxData[
 RowBox[{"?", "wtstb"}]], "Input",
 CellChangeTimes->{{3.6475839705566454`*^9, 
  3.647583974035452*^9}},ExpressionUUID->"2a4b6c2f-ae39-4ee2-b847-\
74a6bf9c1b76"],

Cell[BoxData[
 RowBox[{"wtstb", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], "]"}]], "Input",ExpressionUUID->"35fccf6c-\
3b94-4920-9f00-0a0c0030747b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "dx", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"dx", "[", "i_", "]"}], "[", "x__", "]"}], " ", ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{
       RowBox[{"Subsets", "[", 
        RowBox[{"Range", "[", 
         RowBox[{"Length", "[", 
          RowBox[{"{", "x", "}"}], "]"}], "]"}], "]"}], ",", "i"}], "]"}], 
     ",", 
     RowBox[{"wtstb", "[", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "]"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<Invalid argument i = \>\"", ",", " ", "i", ",", " ", 
        "\"\< !\>\""}], "]"}], ";", "Null"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<Invalid argument i = \>\"", ",", " ", "i", ",", " ", 
        "\"\< !\>\""}], "]"}], ";", "Null"}]}], "]"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6475846245253944`*^9, 3.6475846731818795`*^9}, {
   3.6475847250519705`*^9, 3.647584847636986*^9}, {3.6475852746877365`*^9, 
   3.647585302518185*^9}, {3.6475853329070387`*^9, 3.647585340239051*^9}, 
   3.6475853817819242`*^9, {3.6475854993125305`*^9, 3.647585571166257*^9}, {
   3.647585608341122*^9, 3.647585685218057*^9}, {3.6475857309105377`*^9, 
   3.647585748741369*^9}, {3.6475857833110294`*^9, 3.647585789644641*^9}, 
   3.647585837443125*^9, 3.647585883307205*^9, {3.6475863635228486`*^9, 
   3.6475864110873322`*^9}, 3.6476852180425014`*^9, {3.647685354870342*^9, 
   3.647685368754366*^9}, {3.647685469592943*^9, 3.6476854763009553`*^9}, {
   3.6476855108238153`*^9, 3.6476855569686966`*^9}, 3.647685696994543*^9, {
   3.6476858263031697`*^9, 3.647685840655195*^9}, 3.647685930183752*^9, {
   3.6476861163856792`*^9, 3.6476861205040865`*^9}, 
   3.6476861619377594`*^9},ExpressionUUID->"a25607f3-a1a2-40ac-ab5b-\
ed62da19a1f3"],

Cell[TextData[{
 "Since the coordinates of the basis differential forms are constant it \
follows\n\n",
 StyleBox["Proposition 3.",
  FontColor->RGBColor[0, 0, 1]],
 " The exterior differentials of the basis differential forms are null \
tensors. The ",
 StyleBox["strict basis decomposition",
  FontColor->RGBColor[0, 0, 1]],
 " with"
}], "Text",
 CellChangeTimes->{{3.6475871940631075`*^9, 3.6475872963524876`*^9}, 
   3.6475873272873416`*^9, {3.6476070946435156`*^9, 3.6476071008367267`*^9}, {
   3.647607134735586*^9, 3.647607157449226*^9}, {3.647608010474324*^9, 
   3.6476080138595304`*^9}, 
   3.703048046332737*^9},ExpressionUUID->"0faf638d-e4cd-4732-9ba8-\
d5789a1892cb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"subs", "[", 
   RowBox[{"d_", ",", "p_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Subsets", "[", 
   RowBox[{
    RowBox[{"Range", "[", "d", "]"}], ",", 
    RowBox[{"{", "p", "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.647607788407934*^9, 3.6476078169403844`*^9}, {
  3.6476078554100523`*^9, 
  3.647607872398482*^9}},ExpressionUUID->"317ee2b2-dcfa-4c7f-86dc-\
c3f6b7dc2285"],

Cell[BoxData[
 RowBox[{
  RowBox[{"twa", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "  ", "\[Equal]", " ", 
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"i", "\[Element]", " ", 
     RowBox[{"subs", "[", 
      RowBox[{"d", ",", "p"}], "]"}]}], " "], 
   RowBox[{
    RowBox[{
     RowBox[{"twa", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"i", ",", 
      RowBox[{"{", "}"}]}], "]"}], "*", 
    RowBox[{
     RowBox[{"dx", "[", "i", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.647607212002522*^9, 3.647607229240552*^9}, {
   3.6476073791412153`*^9, 3.647607466095768*^9}, {3.647607517669459*^9, 
   3.6476076012076054`*^9}, 3.64760772326222*^9, {3.6476079229737706`*^9, 
   3.6476079809278727`*^9}, {3.6476099996430635`*^9, 
   3.6476100436507406`*^9}},ExpressionUUID->"1b0386c9-96fa-4151-8c01-\
821b78aa0ffd"],

Cell["has the exterior differential", "Text",
 CellChangeTimes->{{3.647608145102561*^9, 
  3.647608180602623*^9}},ExpressionUUID->"eb7d096f-97e9-4241-b99a-\
594c5eacff41"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dd", "[", "twa", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "  ", "\[Equal]", "  ", 
  RowBox[{
   StyleBox[
    UnderscriptBox[
     StyleBox["\[CapitalSigma]",
      FontSize->24], 
     StyleBox[
      RowBox[{"i", "\[Element]", 
       RowBox[{"subs", "[", 
        RowBox[{"d", ",", "p"}], "]"}]}],
      FontSize->9]],
    FontSize->24], 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       StyleBox["d",
        FontSize->16,
        FontWeight->"Plain"], 
       StyleBox["[",
        FontSize->16,
        FontWeight->"Plain"], "twa", 
       StyleBox["]",
        FontSize->8]}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"i", ",", 
      RowBox[{"{", "}"}]}], "]"}], "\[Wedge]", 
    RowBox[{
     RowBox[{"dx", "[", "i", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.647608206654669*^9, 3.6476082092442737`*^9}, {
   3.647609090075465*^9, 3.647609214688484*^9}, 3.647609512227807*^9, 
   3.647609663267272*^9, {3.647609724076179*^9, 3.647609746259418*^9}, {
   3.647609795345104*^9, 3.6476098067955246`*^9}, {3.6476098450155916`*^9, 
   3.647609879023651*^9}, {3.6476099134373116`*^9, 3.647609915324915*^9}, 
   3.6476100825884085`*^9, 
   3.701754249051711*^9},ExpressionUUID->"da54320f-fc41-4f9d-9697-\
e841c99b68fb"],

Cell["We confirm this formula by examples", "Text",
 CellChangeTimes->{{3.701759459964891*^9, 
  3.701759491068932*^9}},ExpressionUUID->"70af1058-9bc3-4bd4-bb84-\
4bd003b51eae"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647584692806714*^9, 3.6475846950219183`*^9}, {
  3.6475859092968507`*^9, 
  3.6475859381725016`*^9}},ExpressionUUID->"5fddd404-7397-4db3-be39-\
da2010d38351"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dx", "[", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647586440462184*^9, 3.64758646103862*^9}, 
   3.64768572387339*^9, 3.6476864296342297`*^9, 
   3.7017542490742197`*^9},ExpressionUUID->"bd586231-0b45-4a1e-a9a1-\
cb2d3ca83b61"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64758647248904*^9, 
  3.6475864842046604`*^9}},ExpressionUUID->"1753ea7e-c910-469d-a103-\
174d40a1fe84"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", 
   RowBox[{"{", "2", "}"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.647586634620125*^9},ExpressionUUID->"dc42c7bb-377d-42a2-aad2-\
f14ccb9c605a"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64758647248904*^9, 
  3.6475864842046604`*^9}},ExpressionUUID->"c587ddc4-d249-4611-9bd0-\
d228eef7f3b9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dx", "[", 
    RowBox[{"{", "2", "}"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647586440462184*^9, 3.64758646103862*^9}, 
   3.647586543266364*^9, 
   3.701754249097307*^9},ExpressionUUID->"73784e7d-438d-4b05-96af-\
933200a4673a"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64758647248904*^9, 
  3.6475864842046604`*^9}},ExpressionUUID->"981ae393-5f6c-4801-bf48-\
ebb347ecb274"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dx", "[", 
    RowBox[{"{", "}"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647586440462184*^9, 3.64758646103862*^9}, 
   3.647586543266364*^9, 3.6475867569815397`*^9, 
   3.701754249120111*^9},ExpressionUUID->"b2d427b7-421d-4191-9a6c-\
9bacd9dea426"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64758647248904*^9, 
  3.6475864842046604`*^9}},ExpressionUUID->"b7a1af78-925b-4b68-bae6-\
9035b89bc8e6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647586440462184*^9, 3.64758646103862*^9}, 
   3.647586543266364*^9, {3.6475868256216607`*^9, 3.6475868308476696`*^9}, 
   3.6475868646685295`*^9},ExpressionUUID->"50009595-4337-4b82-a1dc-\
5df86913fb4a"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64758647248904*^9, 
  3.6475864842046604`*^9}},ExpressionUUID->"82925196-b0cd-4420-86d2-\
d5e63e53d30d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", 
   RowBox[{"dx", "[", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647586440462184*^9, 3.64758646103862*^9}, 
   3.647586543266364*^9, {3.6475867823627844`*^9, 3.647586792643203*^9}, {
   3.647686490708337*^9, 3.6476865150443797`*^9}, {3.647686561891262*^9, 
   3.647686564184466*^9}, 
   3.701754249144629*^9},ExpressionUUID->"46d58016-0f05-478c-9079-\
00c90bd3405b"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.64758647248904*^9, 
  3.6475864842046604`*^9}},ExpressionUUID->"e812efd4-5cc4-4f13-b364-\
e3b051f285a3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "2"}], "}"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6476865845893016`*^9, 
  3.6476865884113083`*^9}},ExpressionUUID->"8bc1b709-3de7-4fc1-a6fe-\
106d49178820"],

Cell["This confirms the first statement. We remember", "Text",
 CellChangeTimes->{{3.6475874178923006`*^9, 3.6475874178923006`*^9}, {
  3.6476102736731443`*^9, 3.6476102798351555`*^9}, {3.6476117617545586`*^9, 
  3.6476117766213846`*^9}},ExpressionUUID->"03443b96-984f-4652-898a-\
35cc950b3fe6"],

Cell[BoxData[
 RowBox[{"?", "subs"}]], "Input",
 CellChangeTimes->{{3.647587402697874*^9, 3.647587406691481*^9}, {
  3.6476866539938235`*^9, 
  3.6476866598750343`*^9}},ExpressionUUID->"d3c847a4-9b25-4ab8-8d72-\
c764583865e2"],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " definition of the strict basis representation of twa",
 "[var[x]] ",
 " is"
}], "Text",
 CellChangeTimes->{{3.6476118079618397`*^9, 3.6476118835127726`*^9}, 
   3.647611917895233*^9},ExpressionUUID->"b227e1ba-34e8-4f88-ac8e-\
57a7515f0d3b"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "basrepr", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"basrepr", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{"times", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"strictcoords", "[", 
        RowBox[{"twa", "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"dx", "[", 
        RowBox[{
         RowBox[{"subs", "[", 
          RowBox[{"3", ",", "2"}], "]"}], "[", 
         RowBox[{"[", "j", "]"}], "]"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", 
      RowBox[{"Binomial", "[", " ", 
       RowBox[{"3", ",", "2"}], "]"}]}], "}"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6475875230832853`*^9, 3.647587675464353*^9}, {
   3.6475877956625643`*^9, 3.6475878180174036`*^9}, {3.647587850839861*^9, 
   3.647587886532724*^9}, {3.647587922537587*^9, 3.647588171748025*^9}, {
   3.647610317088021*^9, 3.647610387943345*^9}, 3.6476104770663013`*^9, {
   3.6476105101539598`*^9, 
   3.647610539201211*^9}},ExpressionUUID->"50db4f01-08c9-43d4-829c-\
65a207c705e6"],

Cell["Indeed, it represents the tensor  twa[var[x]] :", "Text",
 CellChangeTimes->{{3.647612051415868*^9, 3.64761206428589*^9}, {
  3.647612097389148*^9, 
  3.6476121137847767`*^9}},ExpressionUUID->"87765663-f8c4-447e-af94-\
cfcb26c12557"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%", ",", 
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647587446923952*^9, 3.6475874893716264`*^9}, {
  3.6475881857412496`*^9, 3.647588216192503*^9}, {3.6476106885870733`*^9, 
  3.647610713484717*^9}},ExpressionUUID->"8b77e916-39a3-4ef2-a345-\
e7d3192e60e4"],

Cell["The exterior differential is the differential 3-form", "Text",
 CellChangeTimes->{{3.6476121324580097`*^9, 
  3.6476121655924683`*^9}},ExpressionUUID->"1044378f-a2e0-4a78-b7ac-\
bff1e92642d7"],

Cell[BoxData[
 RowBox[{"dbasrpr", "=", 
  RowBox[{
   RowBox[{"dd", "[", "basrepr", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.647610616546147*^9, 3.6476106708186417`*^9}, {
   3.6476107546219893`*^9, 3.647610767242411*^9}, 
   3.701754249168515*^9},ExpressionUUID->"34eef3fa-c9f5-48ae-84c3-\
18df6f587b15"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.647610779566433*^9, 
  3.647610785432043*^9}},ExpressionUUID->"a7b20c44-184b-4a20-885f-\
82a75a738cba"],

Cell["with the only strict coordinate", "Text",
 CellChangeTimes->{{3.647612179632493*^9, 
  3.6476121925805154`*^9}},ExpressionUUID->"c0d88b46-7eae-41cf-a70f-\
2244db848338"],

Cell[BoxData[
 RowBox[{"dbasrpr", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647610833573728*^9, 
  3.647610853073762*^9}},ExpressionUUID->"6dc50195-5dfb-4bed-b13e-\
915a134fe3a5"],

Cell["It coincides with the strict coordinate of dd[twa][var[x]]", "Text",
 CellChangeTimes->{{3.6476122376177945`*^9, 3.647612279347868*^9}, 
   3.701754249198138*^9},ExpressionUUID->"1774121f-2b48-4efd-9141-\
39103f0c6030"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twa", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.647611518846532*^9, 3.64761154631818*^9}, {
   3.6476115870342517`*^9, 3.6476115995922737`*^9}, 
   3.7017542492021217`*^9},ExpressionUUID->"ca73941e-5bb2-43d8-9e80-\
aa6cef8a49f2"],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " calculation of the sum yields"
}], "Text",
 CellChangeTimes->{{3.647612381848826*^9, 
  3.647612403501664*^9}},ExpressionUUID->"0e70e184-863b-4124-9d96-\
996f36891dde"],

Cell[BoxData[
 RowBox[{"sumrpr", " ", "=", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{"wedge", "[", 
     RowBox[{
      RowBox[{"covecTotensor", "[", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"strictcoords", "[", 
           RowBox[{"twa", "[", 
            RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "[", 
          RowBox[{"[", "j", "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"var", "[", "x", "]"}], "}"}], "}"}]}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"dx", "[", 
        RowBox[{
         RowBox[{"subs", "[", 
          RowBox[{"3", ",", "2"}], "]"}], "[", 
         RowBox[{"[", "j", "]"}], "]"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", 
      RowBox[{"Binomial", "[", " ", 
       RowBox[{"3", ",", "2"}], "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.647610886769821*^9, 3.647610913757869*^9}, {
   3.6476109818207884`*^9, 3.6476111131106186`*^9}, 3.647611155308693*^9, {
   3.647611230797226*^9, 
   3.647611252684064*^9}},ExpressionUUID->"b3044aff-fbb6-4f1a-898b-\
538a8934b76c"],

Cell["Also its strict coordinate has the expected value:", "Text",
 CellChangeTimes->{{3.6476124309265127`*^9, 
  3.647612459677363*^9}},ExpressionUUID->"6f346957-e2e6-45f7-b59c-\
395ea4930d9f"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6476112845861197`*^9, 
  3.6476113181105785`*^9}},ExpressionUUID->"62f35ea9-d995-4185-8352-\
c931126dc62d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dbasrpr", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}], "\[Equal]", " ", "%"}]], "Input",
 CellChangeTimes->{{3.647611354599043*^9, 
  3.647611363361059*^9}},ExpressionUUID->"779f50e1-cf40-4151-b46e-\
5837f98a087d"],

Cell[BoxData[
 RowBox[{"%%", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twa", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.647611662647585*^9, 
  3.701754249225568*^9},ExpressionUUID->"2f2166ff-e748-4d71-955d-\
6a8d0346a82b"],

Cell["Thus  Proposition 3 is confirmed. \[FilledSquare]", "Text",
 CellChangeTimes->{{3.6476114013159256`*^9, 3.647611401331526*^9}, {
  3.6476124976634293`*^9, 
  3.647612528114683*^9}},ExpressionUUID->"acd34583-cb20-414c-9be3-\
16a1db683bab"],

Cell[TextData[{
 StyleBox["The product rule for the exterior differential.",
  FontColor->RGBColor[0, 0, 1]],
 "\n\nWe consider the exterior product"
}], "Text",
 CellChangeTimes->{{3.6476919236502795`*^9, 3.647691925194682*^9}, {
  3.6477399256617765`*^9, 
  3.6477400032095127`*^9}},ExpressionUUID->"827dde3d-a3c7-438e-b879-\
8dffb1f07928"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", " ", 
  RowBox[{"Clear", "[", 
   RowBox[{"twa", ",", "twb", ",", "twc", ",", "aa", ",", "bb"}], "]"}], ";", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"a", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"bb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"altform", "[", 
    RowBox[{
     RowBox[{"b", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"bb", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6476869777879925`*^9, 3.647687054571327*^9}, {
   3.6476870953185987`*^9, 3.6476872076231956`*^9}, {3.647687294998949*^9, 
   3.6476873229229984`*^9}, {3.6476873827335033`*^9, 3.647687459735239*^9}, {
   3.647687497284505*^9, 3.6476875118237305`*^9}, {3.6476875848474584`*^9, 
   3.647687663327197*^9}, 
   3.6476887797767577`*^9},ExpressionUUID->"291683ad-76d8-407f-a465-\
867ccb3fe9cb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"twc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6476872255944276`*^9, 3.6476872568880825`*^9}, 
   3.647687337477824*^9, 3.647687716850891*^9, {3.6476877938838263`*^9, 
   3.647687846440318*^9}},ExpressionUUID->"9947f64a-8280-4859-af49-\
fe7ec12e6f6e"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6476902271473*^9, 
  3.6476902480669365`*^9}},ExpressionUUID->"b6edb116-fb0b-400a-ba78-\
24be80feb5a1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "twc", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647687876532771*^9, 3.6476879016956153`*^9}, 
   3.701754249251794*^9},ExpressionUUID->"1ae2e770-6157-41ef-831d-\
37acd3e8c4ff"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"strictcoords", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6476904625985136`*^9, 3.6476904744077344`*^9}, {
  3.6476905542018747`*^9, 
  3.647690562610289*^9}},ExpressionUUID->"dd1fddc3-dec5-4272-81e5-\
c67542b2810e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "twa", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.701754249277492*^9},ExpressionUUID->"c236f64c-dd7b-49a3-a32f-\
02455d739c40"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.647696897900297*^9, 
  3.6476969074475136`*^9}},ExpressionUUID->"46e1ec39-03b0-403b-ab51-\
04f4c70525e6"],

Cell[BoxData[
 RowBox[{"sc1", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twa", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6476879415848856`*^9, 3.647688022736228*^9}, 
   3.7017542493036537`*^9},ExpressionUUID->"f8456c78-8f77-4f82-ba3f-\
8c8c82750c6b"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"strictcoords", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.64769675729725*^9, 3.6476967633344607`*^9}, {
  3.6476967952053165`*^9, 
  3.6476968270605726`*^9}},ExpressionUUID->"c60e9b41-8e00-494b-b654-\
1e46da911c5a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "twb", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.7017542493272247`*^9},ExpressionUUID->"e1c4a0f6-002f-4805-9a83-\
d0f00e02e809"],

Cell[BoxData[
 RowBox[{"strictcoords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6476972735177565`*^9, 
  3.647697283688975*^9}},ExpressionUUID->"6e86706b-aaab-4b60-8004-\
a72fc497f2ee"],

Cell[BoxData[
 RowBox[{"sc2", " ", "=", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{
    RowBox[{"dd", "[", "twb", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.647688031253843*^9, 3.647688127646412*^9}, 
   3.647688878322131*^9, 
   3.701754249349175*^9},ExpressionUUID->"68cb480a-5592-47e0-8295-\
bdf5f7a10a4d"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"strictcoords", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647697329615455*^9, 
  3.6476973597859087`*^9}},ExpressionUUID->"3c30e460-2586-4211-a23a-\
c5ac6521d722"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"sc1", "-", "sc2"}]}]], "Input",
 CellChangeTimes->{{3.6476882628206496`*^9, 3.6476883081543293`*^9}, 
   3.6476889235934105`*^9, {3.647739727947029*^9, 
   3.647739736293044*^9}},ExpressionUUID->"8a7e862c-a14d-4902-bcb6-\
aa730cd1af99"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dd", "[", "twc", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
  RowBox[{"sc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6476884282145405`*^9, 3.6476884396649604`*^9}, 
   3.6476887238974595`*^9, 
   3.701754249373249*^9},ExpressionUUID->"e4375c8f-c20b-4992-94b4-\
1aa3140bc9a4"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"strictcoords", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647688455920189*^9, 3.647688469071012*^9}, {
  3.6476885063394775`*^9, 
  3.6476885155278935`*^9}},ExpressionUUID->"86f82528-fe6f-4d21-b4d4-\
3f1b0210a5ce"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twc", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"sc", " ", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647688144073241*^9, 3.647688198033736*^9}, {
   3.647688346093596*^9, 3.647688362723225*^9}, 
   3.701754249397485*^9},ExpressionUUID->"45e1270d-b4ba-4463-b8fe-\
e5742033e630"],

Cell[TextData[{
 "OK. We got the first example confirming \n",
 StyleBox["Proposition 4",
  FontColor->RGBColor[0, 0, 1]],
 ". The exterior product of differential forms satisfies the ",
 StyleBox["product rule",
  FontColor->RGBColor[0, 0, 1]],
 " (in traditional form)"
}], "Text",
 CellChangeTimes->{{3.647739769505502*^9, 3.6477397753087125`*^9}, {
  3.6477400347371683`*^9, 
  3.6477401558557806`*^9}},ExpressionUUID->"cfed021e-a68a-4b3b-90be-\
94a91fb9b971"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"v", "\[Wedge]", "w"}], "]"}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "v", "]"}], "\[Wedge]", "w"}], " ", "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "p"], 
     RowBox[{"v", "\[Wedge]", 
      RowBox[{"d", "[", "w", "]"}]}]}]}]}], ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6477401719394093`*^9, 3.647740202125462*^9}, {
  3.6477402322179146`*^9, 
  3.64774033219849*^9}},ExpressionUUID->"040c31d0-3153-4e38-b702-\
5ea98c7ff0e6"],

Cell["\<\
here p=type[v][[2,1]] denotes the covariance degree of the first factor. \
\[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.64774042281905*^9, 
  3.6477405305684385`*^9}},ExpressionUUID->"08069e32-5752-4c1d-b69f-\
ce34dda0d0bd"],

Cell[" Test more examples to confirm the proposition:", "Text",
 CellChangeTimes->{{3.64774593596154*^9, 3.6477459906868362`*^9}, 
   3.7030483577758093`*^9},ExpressionUUID->"b3354da2-9216-46e7-ba34-\
36b0dcec5da1"],

Cell[TextData[StyleBox["Example 1.",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6477424712240076`*^9, 3.6477424773080187`*^9}, {
  3.7017596685155888`*^9, 
  3.701759671251452*^9}},ExpressionUUID->"5d1e7379-e249-4451-90e6-\
f687b3094843"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dim", "=", "4"}], ";", " ", 
  RowBox[{"Clear", "[", 
   RowBox[{"twa", ",", "twb", ",", "twc", ",", "aa", ",", "bb"}], "]"}], ";", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"a", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"bb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"altform", "[", 
    RowBox[{
     RowBox[{"b", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"bb", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"twc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", " ", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.6476869777879925`*^9, 3.647687054571327*^9}, {
   3.6476870953185987`*^9, 3.6476872076231956`*^9}, {3.647687294998949*^9, 
   3.6476873229229984`*^9}, {3.6476873827335033`*^9, 3.647687459735239*^9}, {
   3.647687497284505*^9, 3.6476875118237305`*^9}, {3.6476875848474584`*^9, 
   3.647687663327197*^9}, 3.6476887797767577`*^9, {3.647742549224145*^9, 
   3.64774257508899*^9}, {3.6477449037701273`*^9, 
   3.647744967402639*^9}},ExpressionUUID->"529d6ff9-ce6c-4432-963b-\
33f68f3a7d08"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "twc", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647744996044289*^9, 3.6477450106927147`*^9}, 
   3.701754249421081*^9},ExpressionUUID->"7ef4fc5d-12db-4b3e-a461-\
e5d3bd7dd87b"],

Cell[BoxData[
 RowBox[{"stctwc", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{"strictcoords", "[", "%", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.647745066072812*^9, 
  3.647745118426504*^9}},ExpressionUUID->"5c11e937-1b4a-4c8d-95bc-\
6c604ad0d036"],

Cell[BoxData[
 RowBox[{"st1", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twa", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6477451525749645`*^9, 3.647745231464303*^9}, 
   3.701754249444202*^9},ExpressionUUID->"aeb500f0-3dad-48f6-a434-\
1cf540c24b71"],

Cell[BoxData[
 RowBox[{"st2", " ", "=", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{
    RowBox[{"dd", "[", "twb", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6477452465027294`*^9, 3.6477452943168135`*^9}, 
   3.701754249467197*^9},ExpressionUUID->"e420b803-c2e4-41de-b00a-\
f359d9f2996d"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twc", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"st1", "-", "st2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647745318153655*^9, 3.64774536668534*^9}, 
   3.70175424948936*^9},ExpressionUUID->"fe124cca-bab3-41ea-846b-\
4c44df12732e"],

Cell[TextData[{
 "OK. \[FilledSquare]\n",
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 "."
}], "Text",
 CellChangeTimes->{{3.647745853328195*^9, 3.6477458757610345`*^9}, {
  3.701759797938496*^9, 
  3.7017597991064997`*^9}},ExpressionUUID->"d2f5b448-8bd6-453a-8c50-\
6d65a8e2f0be"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dim", "=", "5"}], ";", " ", 
  RowBox[{"Clear", "[", 
   RowBox[{"twa", ",", "twb", ",", "twc", ",", "aa", ",", "bb"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"aa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"altform", "[", 
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"aa", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"bb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"altform", "[", 
    RowBox[{
     RowBox[{"b", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{"bb", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"twc", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", " ", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.6476869777879925`*^9, 3.647687054571327*^9}, {
   3.6476870953185987`*^9, 3.6476872076231956`*^9}, {3.647687294998949*^9, 
   3.6476873229229984`*^9}, {3.6476873827335033`*^9, 3.647687459735239*^9}, {
   3.647687497284505*^9, 3.6476875118237305`*^9}, {3.6476875848474584`*^9, 
   3.647687663327197*^9}, 3.6476887797767577`*^9, {3.647742549224145*^9, 
   3.64774257508899*^9}, {3.6477449037701273`*^9, 3.647744967402639*^9}, {
   3.647745450317087*^9, 3.6477455166016035`*^9}, {3.6477455654608893`*^9, 
   3.647745575491707*^9}},ExpressionUUID->"659619fa-3e40-4592-b875-\
3c1b0cfddd81"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "twc", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647744996044289*^9, 3.6477450106927147`*^9}, 
   3.7017542495115223`*^9},ExpressionUUID->"278405f1-ec3e-407f-a722-\
e853a702bcf3"],

Cell[BoxData[
 RowBox[{"st1", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twa", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{"twb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6477451525749645`*^9, 3.647745231464303*^9}, 
   3.701754249533927*^9},ExpressionUUID->"c4ce72f7-c3ed-4d20-89a9-\
51a79a814f90"],

Cell[BoxData[
 RowBox[{"st2", " ", "=", 
  RowBox[{
   RowBox[{"twa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
   RowBox[{
    RowBox[{"dd", "[", "twb", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6477452465027294`*^9, 3.6477452943168135`*^9}, 
   3.701754249556304*^9},ExpressionUUID->"0a15975f-c160-4e31-a9f1-\
0b35eaf11f66"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"dd", "[", "twc", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"st1", "+", "st2"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.647745318153655*^9, 3.64774536668534*^9}, 
   3.6477457590728292`*^9, 
   3.701754249578597*^9},ExpressionUUID->"1b35886b-4569-4cb2-8e9e-\
261f0519636a"],

Cell["OK. \[FilledSquare]", "Text",
 CellChangeTimes->{{3.647745819039335*^9, 
  3.6477458325489583`*^9}},ExpressionUUID->"d3135330-7558-459d-b291-\
50bb160d267c"],

Cell["Now we consider the basic p - form tensors, e.g.", "Text",
 CellChangeTimes->{{3.6477469712901583`*^9, 
  3.647747014096634*^9}},ExpressionUUID->"3b7e903b-0139-4f0a-95d3-\
8e443d685a1a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dx", "[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647746030342106*^9, 3.647746042432127*^9}, {
  3.6477460727573805`*^9, 
  3.647746079980193*^9}},ExpressionUUID->"19dfa0a7-f4f0-401f-a50f-\
4eb43eff9b3f"],

Cell["We show", "Text",
 CellChangeTimes->{{3.647747047324692*^9, 
  3.647747051255899*^9}},ExpressionUUID->"30640bf1-8506-4c08-8053-\
eadf8344afad"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"equi", "[", 
   RowBox[{"%", ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"dx", "[", 
         RowBox[{"{", "1", "}"}], "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "\[Wedge]", 
       RowBox[{
        RowBox[{"dx", "[", 
         RowBox[{"{", "2", "}"}], "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}]}], ")"}], "\[Wedge]", 
     RowBox[{
      RowBox[{"dx", "[", 
       RowBox[{"{", "3", "}"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6477461013834305`*^9, 
  3.6477462144992294`*^9}},ExpressionUUID->"12e6a77f-e240-4836-a32b-\
fb9a3d37e1f2"],

Cell["This is an example illustrating", "Text",
 CellChangeTimes->{{3.647747065030723*^9, 
  3.6477470941091743`*^9}},ExpressionUUID->"9e16d69a-630a-4af6-b6ea-\
cedc056b436c"],

Cell[TextData[{
 StyleBox["Lemma 5.",
  FontColor->RGBColor[0, 0, 1]],
 "  The basis p-form tensors are the exterior products of the corresponding \
coordinate differentials:"
}], "Text",
 CellChangeTimes->{{3.6477463944143453`*^9, 3.647746512740553*^9}, 
   3.6477468336667166`*^9},ExpressionUUID->"4a30d164-ebcc-405c-b8d1-\
987d33ff47f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dx", "[", 
    RowBox[{"{", 
     RowBox[{"i1", ",", "i2", ",", "...", ",", "ip"}], "}"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " ", "\[Equal]", "  ", 
  RowBox[{"wedge", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"dx", "[", 
       RowBox[{"{", "ij", "}"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "p"}], "}"}]}], "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.647746588434286*^9, 
  3.647746733654941*^9}},ExpressionUUID->"751ba374-1bf5-49f1-82f2-\
1fd3e18fec95"],

Cell["for i1<i2<...<ip.\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6477467750730143`*^9, 3.6477467960394506`*^9}, {
  3.647746903180439*^9, 
  3.647746903180439*^9}},ExpressionUUID->"f8b4bf50-ee91-41f4-8b6a-\
e04fd80fb218"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", " ", 
  RowBox[{"Clear", "[", 
   RowBox[{
   "twa", ",", "twb", ",", "twc", ",", "aa", ",", "bb", ",", "st1", ",", 
    "st2", ",", "sc1", ",", "sc2", ",", "su1", ",", "su2", ",", "stp1"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.64774712613603*^9, 3.647747174308915*^9}, {
  3.6477472360382233`*^9, 
  3.6477472616846685`*^9}},ExpressionUUID->"4834f760-bc11-4470-ae85-\
d3e2c381d98d"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
2.4.5. Exterior Differential of 1-Forms and Commutator of Vector Fields\
\>", "Subsubsection",
 CellChangeTimes->{{3.658560066360248*^9, 3.658560177743312*^9}, 
   3.668744743437334*^9, 3.66877579708286*^9, {3.701760401678841*^9, 
   3.7017604108535023`*^9}, 3.701834430362591*^9, 3.703048493064089*^9, 
   3.703049996795046*^9, 3.704094285265049*^9, 
   3.704105551630784*^9},ExpressionUUID->"cef993ed-997a-4adf-aad6-\
7def2cae63df"],

Cell["\<\
In this subsection we show an important formula relating the exterior \
differential of a 1-form \[Omega] with the commutator of vector fields X,Y, \
namely, in traditional notation\
\>", "Text",
 CellChangeTimes->{{3.6585602072876472`*^9, 3.6585603253821697`*^9}, {
  3.658560513876265*^9, 
  3.658560635739635*^9}},ExpressionUUID->"3b2fa659-5c33-4c68-a5cb-\
dbe2c277b4f3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dw", "[", 
   RowBox[{"X", ",", "Y"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"X", 
    RowBox[{"(", 
     RowBox[{"w", 
      RowBox[{"(", "Y", ")"}]}], ")"}]}], " ", "-", 
   RowBox[{"Y", 
    RowBox[{"(", 
     RowBox[{"w", 
      RowBox[{"(", "X", ")"}]}], ")"}]}], " ", "-", 
   RowBox[{"w", 
    RowBox[{"(", 
     RowBox[{"[", 
      RowBox[{"X", ",", "Y"}], "]"}], ")"}], "                               ", 
    RowBox[{"(", "1", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6585606474247437`*^9, 3.658560783906775*^9}, {
  3.65856082687648*^9, 3.658560836450137*^9}, {3.6585611300234423`*^9, 
  3.658561146087644*^9}, {3.658578536782384*^9, 3.658578544934428*^9}, {
  3.7018340834946947`*^9, 
  3.701834088069724*^9}},ExpressionUUID->"0f27bcfc-6291-4f36-9153-\
4cf098ded909"],

Cell[TextData[{
 "This formula can easily be proved by a straightforward  calculation in  \
local coordinates. We confirm it in the next \n",
 StyleBox["Example 1.",
  FontColor->RGBColor[0, 0, 1]],
 " Take a 1-form:"
}], "Text",
 CellChangeTimes->{{3.658588008043182*^9, 3.65858810504978*^9}, {
  3.658588541618177*^9, 3.658588685335763*^9}, {3.701760072142919*^9, 
  3.701760073312346*^9}},ExpressionUUID->"582096e3-a0c5-4c21-90b6-\
c1994edac61f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"a", ",", "w"}], "]"}], ";", " ", 
  RowBox[{
   RowBox[{"w", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"af", "[", 
     RowBox[{"a", ",", "1"}], "]"}], "[", "x", "]"}]}], ";", 
  RowBox[{"w", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65856087161843*^9, 3.658560882113689*^9}, {
  3.658560930267417*^9, 3.6585610245205812`*^9}, {3.6585610575861187`*^9, 
  3.658561087247735*^9}, {3.70841150467105*^9, 3.708411515549571*^9}, {
  3.708411574753543*^9, 3.708411576384301*^9}, {3.7084129904450502`*^9, 
  3.708413007356955*^9}, {3.708413037988719*^9, 3.70841304551682*^9}, {
  3.708413167981127*^9, 
  3.708413203274881*^9}},ExpressionUUID->"5bc4f8c7-a5a5-4e0d-a609-\
38b47da96d9b"],

Cell["Its exterior differential is", "Text",
 CellChangeTimes->{{3.658588122289267*^9, 
  3.658588132689043*^9}},ExpressionUUID->"947ed7c7-b718-4bb3-b9db-\
7252523dd174"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dd", "[", "w", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.658561188818655*^9, 3.658561199359276*^9}, 
   3.7017542496009007`*^9},ExpressionUUID->"0ab56e59-7641-411c-b23d-\
e331bd681619"],

Cell["Two vector fields are", "Text",
 CellChangeTimes->{{3.658588166160699*^9, 
  3.6585881805441923`*^9}},ExpressionUUID->"1c2df296-3edf-4ef8-91cc-\
1e74fcf4f60b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"vX", ",", "vY"}], "]"}], ";", 
  RowBox[{
   RowBox[{"vX", "[", "x__", "]"}], ":=", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Xi]", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"vY", "[", "x__", "]"}], ":=", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Eta]", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"vY", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658574658780465*^9, 3.658574783739725*^9}, {
   3.658574814573606*^9, 3.658574908614352*^9}, 3.658574942816668*^9, {
   3.708417040102951*^9, 
   3.708417054487406*^9}},ExpressionUUID->"a23baf47-df39-41ad-ad61-\
fc6f8bc1585c"],

Cell["We calculate the left side of equation (1)", "Text",
 CellChangeTimes->{{3.658588202151973*^9, 3.658588235047044*^9}, {
  3.668775623729241*^9, 
  3.6687756288571777`*^9}},ExpressionUUID->"53e285e0-be3e-4d98-97f6-\
ad374beb0764"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dwXY", "=", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"dd", "[", "w", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j"}], "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"vX", "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"vY", "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "dim"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6585749791747932`*^9, 3.658575000601149*^9}, {
   3.658575031355419*^9, 3.6585750507545147`*^9}, {3.658575091739724*^9, 
   3.658575193422319*^9}, 3.658575239577374*^9, 3.658575274168743*^9, 
   3.701754249622529*^9},ExpressionUUID->"7b39b87f-fc75-48bd-8b0c-\
d311d2637ad9"],

Cell["The three terms of the right side are", "Text",
 CellChangeTimes->{{3.658588311726491*^9, 
  3.658588336621499*^9}},ExpressionUUID->"745e51d9-4b29-4f4c-b530-\
bcee2a2a7f39"],

Cell[BoxData[
 RowBox[{
  RowBox[{"XwY", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"vX", "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"w", "[", 
             RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
            RowBox[{"[", "j", "]"}], "]"}], "*", 
           RowBox[{
            RowBox[{"vY", "[", 
             RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
            RowBox[{"[", "j", "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], ",", 
        RowBox[{"x", "[", "i", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.658575296996073*^9, 3.658575348803155*^9}, {
  3.65857541113169*^9, 3.6585754214433203`*^9}, {3.6585754627884283`*^9, 
  3.658575467564032*^9}, {3.658575513693304*^9, 3.658575575934793*^9}, {
  3.658575607039627*^9, 3.6585756633284597`*^9}, {3.658575712601832*^9, 
  3.65857572771378*^9}, {3.658575761458666*^9, 
  3.6585758155964117`*^9}},ExpressionUUID->"8c02c36c-c1f0-4f81-8969-\
c7762c03c361"],

Cell[BoxData[
 RowBox[{
  RowBox[{"YwX", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"vY", "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"w", "[", 
             RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
            RowBox[{"[", "j", "]"}], "]"}], "*", 
           RowBox[{
            RowBox[{"vX", "[", 
             RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
            RowBox[{"[", "j", "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], ",", 
        RowBox[{"x", "[", "i", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.658575296996073*^9, 3.658575348803155*^9}, {
   3.65857541113169*^9, 3.6585754214433203`*^9}, {3.6585754627884283`*^9, 
   3.658575467564032*^9}, {3.658575513693304*^9, 3.658575575934793*^9}, {
   3.658575607039627*^9, 3.6585756633284597`*^9}, {3.658575712601832*^9, 
   3.65857572771378*^9}, {3.658575761458666*^9, 3.658575885157527*^9}, 
   3.658576265261942*^9, 
   3.668775686883032*^9},ExpressionUUID->"c68f3506-abb9-44b3-9b86-\
8b3ee7a4d602"],

Cell[BoxData[
 RowBox[{
  RowBox[{"wcommXY", "=", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"w", "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{
        RowBox[{"comm", "[", 
         RowBox[{"vX", ",", "vY"}], "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "dim"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.658575924920074*^9, 3.658575929405786*^9}, {
   3.658575991855877*^9, 3.65857604039196*^9}, {3.6585760845768337`*^9, 
   3.658576132550818*^9}, {3.658576182634535*^9, 3.658576244787497*^9}, {
   3.658576279373548*^9, 3.658576367270046*^9}, 
   3.658578419447488*^9},ExpressionUUID->"c27da298-5b1f-47b4-a490-\
ad3ccbbddc95"],

Cell["Formula  (1) is confirmed:", "Text",
 CellChangeTimes->{{3.658588417668168*^9, 
  3.658588477603106*^9}},ExpressionUUID->"81e92da2-3a0d-4b49-8c46-\
302d5b316754"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"dwXY", "\[Equal]", " ", 
    RowBox[{"XwY", "-", "YwX", "-", "wcommXY"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.658576375764308*^9, 3.6585763870217943`*^9}, {
   3.6585764281272182`*^9, 3.658576499784021*^9}, {3.6585767026512012`*^9, 
   3.658576704282888*^9}, 3.65857675456441*^9, 3.658577508893711*^9, 
   3.658578157889536*^9},ExpressionUUID->"49eb3f0b-22b9-4b43-87de-\
d9b1053c456d"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.658588491866479*^9, 
  3.6585884918679037`*^9}},ExpressionUUID->"a8f0a34f-4c88-433c-adcc-\
47a47ccab5db"]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
3. Absolute Differentials and Gradients of Vector and Tensor Fields\
\>", "Section",
 CellChangeTimes->{{3.631859555473789*^9, 3.6318595915722523`*^9}, {
   3.644932830043013*^9, 3.6449328370182257`*^9}, {3.6477791587463894`*^9, 
   3.647779209633679*^9}, {3.65752663575072*^9, 3.657526660998356*^9}, 
   3.657604227178216*^9, {3.662908619379534*^9, 3.662908624259625*^9}, 
   3.66299402547756*^9, {3.6634857254778852`*^9, 3.663485732876852*^9}, {
   3.702041637270404*^9, 3.702041639726242*^9}, 3.7020418886683893`*^9, {
   3.703048544822234*^9, 3.703048566909946*^9}, {3.70857556504349*^9, 
   3.708575569753413*^9}},ExpressionUUID->"8444e20e-e68a-498b-a0e7-\
13d1e2769ffb"],

Cell[CellGroupData[{

Cell["3.1. The Differentiation of Vector Fields", "Subsection",
 CellChangeTimes->{{3.657526469807942*^9, 3.657526510175658*^9}, 
   3.668775805512735*^9, 3.668855213205536*^9, 3.701834477131157*^9, 
   3.7020196922155943`*^9, 3.70305007429325*^9, 3.703050578726018*^9, {
   3.708419177537333*^9, 3.708419183055635*^9}, 
   3.7084227765238743`*^9},ExpressionUUID->"bc7ff31c-8f62-4879-80e2-\
43b2a47f2ec1"],

Cell["We consider a vector field on a smooth manifold:", "Text",
 CellChangeTimes->{{3.647779371717963*^9, 
  3.6477794005000143`*^9}},ExpressionUUID->"73eabd3d-5689-4011-ba6f-\
5b31c7c627e8"],

Cell[BoxData[
 RowBox[{"?", "vv"}]], "Input",
 CellChangeTimes->{{3.6477794070988255`*^9, 3.6477794084248276`*^9}, 
   3.6477794435716896`*^9},ExpressionUUID->"2a7609dd-edf6-45a8-8c72-\
f2eac5f654c9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"aX", "[", 
   RowBox[{"var", "[", "x_", "]"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"vv", "[", "a", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6477796250624084`*^9, 
  3.6477796967289343`*^9}},ExpressionUUID->"6061f375-6524-4b11-ba09-\
f428b405f0bf"],

Cell[BoxData[
 RowBox[{"aX", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.647779764495453*^9},ExpressionUUID->"eb2b25f6-f85c-43c9-aafc-\
9e84b2238dcf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "aX", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.647779815601143*^9, 
  3.6477798345239763`*^9}},ExpressionUUID->"04ec3f93-92a8-4ee2-985c-\
1b5b2351e12e"],

Cell["Now we remember the coordinate transformation: vec[y]=", "Text",
 CellChangeTimes->{{3.6492289842305174`*^9, 3.649229023136918*^9}, 
   3.6492290671683264`*^9, 3.65631537402007*^9, {3.656315428835639*^9, 
   3.6563154577311983`*^9}, {3.668776045653553*^9, 
   3.6687760656693563`*^9}},ExpressionUUID->"c04c6dcf-fdc2-4a3b-8f74-\
808ce900a9de"],

Cell[BoxData[
 RowBox[{"\[Phi]\[Kappa]\[Iota]", "[", 
  RowBox[{"vec", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6492229492400064`*^9, 3.6492229582401185`*^9}, 
   3.6563153895338173`*^9, {3.668776002933949*^9, 
   3.6687760176310587`*^9}},ExpressionUUID->"530dc929-b5c2-4559-8e53-\
c0a37fca1594"],

Cell["and its inverse: vec[x]=", "Text",
 CellChangeTimes->{{3.649229040230728*^9, 3.649229080277732*^9}, {
  3.6563154786989307`*^9, 3.656315487867082*^9}, {3.668776085509141*^9, 
  3.668776096101039*^9}},ExpressionUUID->"a4eacb80-bb81-4c4e-8d60-\
41cbcf902ee1"],

Cell[BoxData[
 RowBox[{"\[Phi]\[Iota]\[Kappa]", "[", 
  RowBox[{"vec", "[", "y", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6492229492400064`*^9, 3.6492229582401185`*^9}, {
  3.6492236567113605`*^9, 3.649223677805189*^9}, {3.668776104589259*^9, 
  3.668776106820958*^9}},ExpressionUUID->"9a9ff88a-c924-4efa-b212-\
7d840cde2ba6"],

Cell["\<\
The vector coordinates are transformed with the Jacobian jac:\
\>", "Text",
 CellChangeTimes->{{3.6492291234029036`*^9, 
  3.6492291623092933`*^9}},ExpressionUUID->"64d4a194-0a1c-47c3-9b91-\
f0b308c80995"],

Cell[BoxData[
 RowBox[{"?", "jac"}]], "Input",
 CellChangeTimes->{{3.649224013743892*^9, 
  3.6492240174783*^9}},ExpressionUUID->"3a8bf759-e8b6-4624-b8e8-66e7b7f3a0d1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"jac", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
  RowBox[{"vec", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6492240430251884`*^9, 3.6492240476814547`*^9}, {
   3.64922408552542*^9, 3.6492240908535643`*^9}, {3.649224127853698*^9, 
   3.6492241348380976`*^9}, {3.668780054453676*^9, 3.668780055516127*^9}, {
   3.701842739100082*^9, 3.7018427399583817`*^9}, 3.7018433249185266`*^9, 
   3.7018435510307827`*^9, {3.701843723813092*^9, 3.701843730575612*^9}, {
   3.702017793814715*^9, 
   3.7020178162880907`*^9}},ExpressionUUID->"2dd8aa5e-b9cb-41b5-9e4d-\
0f480e79de94"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "\[Phi]\[Kappa]\[Iota]", "]"}], "[", 
  RowBox[{"vec", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6492240430251884`*^9, 3.6492240476814547`*^9}, {
   3.64922408552542*^9, 3.6492240908535643`*^9}, {3.649224127853698*^9, 
   3.6492241348380976`*^9}, {3.668780054453676*^9, 3.668780055516127*^9}, {
   3.701842739100082*^9, 3.7018427399583817`*^9}, 3.7018433249185266`*^9, 
   3.7018435510307827`*^9, {3.701843723813092*^9, 3.701843730575612*^9}, {
   3.701844517946844*^9, 3.701844541378005*^9}, {3.702017845598166*^9, 
   3.702017857902153*^9}},ExpressionUUID->"4e3609c0-915f-41b0-90d3-\
144af4446e29"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Transpose", "[", "%", "]"}], "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.7020178849358377`*^9, 
  3.70201789809303*^9}},ExpressionUUID->"9eee865f-1630-4ac3-a4c1-\
ba299c03df64"],

Cell["\<\
The coordinates of  the vector field in the new chart var[y] are given by\
\>", "Text",
 CellChangeTimes->{{3.649229217700117*^9, 3.6492292726534395`*^9}, 
   3.702018152363653*^9},ExpressionUUID->"56ddf38c-de52-43ab-b7fe-\
439fc8f6475c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"i", ",", "aY"}], "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"aY", "[", 
     RowBox[{"var", "[", "y_", "]"}], "]"}], "[", "i_", "]"}], ":=", 
   RowBox[{"Sum", "[", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"a", "[", "j", "]"}], "[", 
       RowBox[{
        RowBox[{"\[Phi]\[Iota]\[Kappa]", "[", 
         RowBox[{"vec", "[", "y", "]"}], "]"}], "/.", 
        RowBox[{"List", "\[Rule]", " ", "Sequence"}]}], "]"}], "*", 
      RowBox[{
       RowBox[{"A\[Kappa]\[Iota]", "[", 
        RowBox[{"j", ",", "i"}], "]"}], "[", 
       RowBox[{
        RowBox[{"\[Phi]\[Iota]\[Kappa]", "[", 
         RowBox[{"vec", "[", "y", "]"}], "]"}], "/.", 
        RowBox[{"List", "\[Rule]", " ", "Sequence"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6492237859617643`*^9, 3.6492238410869627`*^9}, {
   3.649223925009139*^9, 3.649223966399992*^9}, 3.6492244846205935`*^9, 
   3.649224516948846*^9, {3.649224976903624*^9, 3.64922505491953*^9}, {
   3.649225177748096*^9, 3.6492252163419847`*^9}, 3.649225296920333*^9, 
   3.6492253301861277`*^9, {3.6492257044687257`*^9, 3.6492257810002646`*^9}, {
   3.6492258231722913`*^9, 3.6492258492973847`*^9}, {3.649225881891252*^9, 
   3.649225958094651*^9}, {3.64922599711034*^9, 3.649226053532411*^9}, {
   3.649226097735778*^9, 3.6492261103451605`*^9}, {3.6492261799391985`*^9, 
   3.6492261848298407`*^9}, {3.6492266327377005`*^9, 3.649226653847102*^9}, {
   3.6492266895035167`*^9, 3.649226706581668*^9}, 3.6492267392067547`*^9, {
   3.66878008460456*^9, 3.6687800872522793`*^9}, {3.668780146868267*^9, 
   3.668780147875618*^9}, {3.668780284658869*^9, 3.6687802941948023`*^9}, {
   3.668780336674346*^9, 
   3.668780351922587*^9}},ExpressionUUID->"497ecb87-672f-4ca2-9ea4-\
3207458dfbf6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"aY", "[", 
   RowBox[{"var", "[", "y", "]"}], "]"}], "[", "1", "]"}]], "Input",
 CellChangeTimes->{
  3.649226518377915*^9},ExpressionUUID->"4f7be7d0-66a1-46f0-a1a2-\
f10663edb01a"],

Cell["The new coordinate vector is", "Text",
 CellChangeTimes->{{3.649229308825445*^9, 
  3.649229317856727*^9}},ExpressionUUID->"8671a1e6-6fed-4162-b328-\
becb3026457e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"yY", "[", 
   RowBox[{"var", "[", "y_", "]"}], " ", "]"}], ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"aY", "[", 
      RowBox[{"var", "[", "y", "]"}], "]"}], "[", "i", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6492246971837945`*^9, 3.6492247093713384`*^9}, {
  3.649226218454962*^9, 3.6492262300175037`*^9}, {3.64922631668969*^9, 
  3.6492264330650883`*^9}},ExpressionUUID->"ee9e8729-2de0-46b9-85ef-\
46a33a8765a1"],

Cell[BoxData[
 RowBox[{"yY", "[", 
  RowBox[{"var", "[", "y", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6492264408306828`*^9, 
  3.649226454408935*^9}},ExpressionUUID->"ea08e82f-a59e-428b-aab6-\
f5f0b42b32ae"],

Cell["\<\
The matrix of its partial derivatives - in Euclidean or affine geometry its \
gradient tensor - is very large, we show  a part of it only:\
\>", "Text",
 CellChangeTimes->{{3.6492294072632856`*^9, 
  3.6492295421075125`*^9}},ExpressionUUID->"69d58b0d-51e7-434b-a6dc-\
534d2423982b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", "[", "yY", "]"}], "[", 
  RowBox[{"var", "[", "y", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6492246381210775`*^9, 3.6492246605899663`*^9}, 
   3.6492268330040455`*^9},ExpressionUUID->"668e4791-f976-4ed2-ab88-\
ce68a36aba9e"],

Cell["The first row is", "Text",
 CellChangeTimes->{{3.6492295562325697`*^9, 
  3.649229564263863*^9}},ExpressionUUID->"66f97417-f507-4140-8e28-\
d3bc5c739fa1"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.649226884394781*^9, 3.6492268904261274`*^9}, {
  3.702018380117281*^9, 
  3.702018386894362*^9}},ExpressionUUID->"8da6bf85-7caf-4189-ada3-\
92bfed2afffa"],

Cell[TextData[{
 "The result contains second derivatives  of the transformation functions f\
\[Kappa]\[Iota], thus the result can be the coordinate matrix of a tensor \
only iff  all  admissible coordinate transformations are linear, what is not \
the case in general.  We derive  the transformation rules of the partial \
derivatives of  vector field components using traditional notations see the ",
 ButtonBox["Summary",
  BaseStyle->"Hyperlink",
  ButtonData->"transfcoords"],
 "."
}], "Text",
 CellChangeTimes->{{3.649229681108033*^9, 3.6492298263428793`*^9}, {
   3.6500907507271676`*^9, 3.6500907752428465`*^9}, {3.6500908851492763`*^9, 
   3.6500908872430925`*^9}, {3.650090917790035*^9, 3.650090979399545*^9}, {
   3.650091056212214*^9, 3.6500911046966953`*^9}, {3.6500911439311347`*^9, 
   3.6500911439311347`*^9}, {3.6500912290875654`*^9, 3.650091274431446*^9}, {
   3.668780476905057*^9, 3.668780477232553*^9}, {3.6687805088161077`*^9, 
   3.668780530127882*^9}, {3.668780715782577*^9, 3.668780735630701*^9}, {
   3.6687808133735723`*^9, 3.6687808133759203`*^9}, 3.702018630536207*^9, {
   3.702018664344614*^9, 3.702018684968047*^9}, {3.702018720112131*^9, 
   3.7020187791508713`*^9}},ExpressionUUID->"a69b7a2e-5efc-4cfc-840b-\
20aeaab58b4b"],

Cell["The transformation rule of the field components is", "Text",
 CellChangeTimes->{{3.6500912915564704`*^9, 3.650091349369108*^9}, {
  3.650091793338817*^9, 
  3.6500917943388367`*^9}},ExpressionUUID->"a4d5266c-3a56-4982-b8e3-\
0f9adeefe7de"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}], 
   RowBox[{
    SubsuperscriptBox["A", "j", "i"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.65009141144732*^9, 3.650091537728858*^9}, {
  3.6500915733539424`*^9, 
  3.6500916992135763`*^9}},ExpressionUUID->"9f2f1a14-b478-40dd-9151-\
82c8152887d1"],

Cell[TextData[{
 "Here ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}]],
  CellChangeTimes->{{3.65009141144732*^9, 3.650091537728858*^9}, {
   3.6500915733539424`*^9, 3.6500916992135763`*^9}},ExpressionUUID->
  "c8f72156-d02a-4af8-ad86-9a541bc0377d"],
 " are the components of the field in the new chart with the new coordinates \
y , ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Xi]", "j"], "[", "x"}]],
  CellChangeTimes->{{3.65009141144732*^9, 3.650091537728858*^9}, {
   3.6500915733539424`*^9, 3.6500916992135763`*^9}},ExpressionUUID->
  "39ae4cef-4f04-4d5a-a93e-76a356bb73f5"],
 "] are the components in the old coordinates x, and x[y] is the expression \
of the old coordinates as function of the news, i.e. the inverse coordinate \
transform. Differentiation of this formula with respect to a new coordinate ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "k"], TraditionalForm]],ExpressionUUID->
  "e4c1d7f5-898d-4d37-b277-f890f91f60cb"],
 " yields"
}], "Text",
 CellChangeTimes->{{3.6500918334482975`*^9, 3.650092050995593*^9}, {
  3.650092239209323*^9, 
  3.6500922926150575`*^9}},ExpressionUUID->"1bf8bf7a-f336-4c5f-9e73-\
1b6c406e77cf"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[PartialD]", 
    SuperscriptBox["\[Eta]", "i"]}], "/", 
   RowBox[{"\[PartialD]", 
    SuperscriptBox["y", "k"]}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SuperscriptBox["\[Xi]", "j"], "[", 
        RowBox[{"x", "[", "y", "]"}], "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "h"]}]}], ")"}], "\[CenterDot]", 
    RowBox[{
     RowBox[{"\[PartialD]", 
      SuperscriptBox["x", "h"]}], "/", 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["y", "k"]}]}], "\[CenterDot]", 
    RowBox[{
     SubsuperscriptBox["A", "j", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Xi]", "j"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[PartialD]", 
        RowBox[{
         SubsuperscriptBox["A", "j", "i"], "[", 
         RowBox[{"x", "[", "y", "]"}], "]"}]}], "/", 
       RowBox[{"\[PartialD]", 
        SuperscriptBox["x", "h"]}]}], ")"}], "\[CenterDot]", 
     RowBox[{
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "h"]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["y", "k"]}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6500923699589777`*^9, 3.650092556420314*^9}, 
   3.650092588701604*^9, {3.6500926222954855`*^9, 3.650092687624711*^9}, {
   3.6500927184060383`*^9, 3.6500927242653823`*^9}, {3.6500927577654886`*^9, 
   3.6500928510473766`*^9}},ExpressionUUID->"e1383cf7-8eb3-4d9a-aaf6-\
a0d4f97bb190"],

Cell[TextData[{
 "Remembering the definition of the transformation matrix  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["B", "k", "h"], TraditionalForm]],ExpressionUUID->
  "796632a8-a5a3-4ace-b1fb-814727a8125f"],
 " = ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"\[PartialD]", 
    SuperscriptBox["x", "h"]}], "/", 
   RowBox[{"\[PartialD]", 
    SuperscriptBox["y", "k"]}]}]],
  CellChangeTimes->{{3.6500923699589777`*^9, 3.650092556420314*^9}, 
    3.650092588701604*^9, {3.6500926222954855`*^9, 3.650092687624711*^9}, {
    3.6500927184060383`*^9, 3.6500927242653823`*^9}, {3.6500927577654886`*^9, 
    3.6500928510473766`*^9}},ExpressionUUID->
  "d9b495e2-29c1-4627-9471-4d39def1eca2"],
 ", we obtain\n",
 StyleBox["Lemma 1.",
  FontColor->RGBColor[0, 0, 1]],
 " The partial derivatives of the coordinates of a vector field on a smooth \
manifold transform as"
}], "Text",
 CellChangeTimes->{{3.650092936142394*^9, 3.650093095924658*^9}, {
  3.6501756392735543`*^9, 3.6501756858864613`*^9}, {3.6501757169955153`*^9, 
  3.65017572585491*^9}, {3.656137950075474*^9, 3.656137956386506*^9}, {
  3.702092049576008*^9, 
  3.702092051936105*^9}},ExpressionUUID->"f82626b6-4732-4d05-88f7-\
d56e3dc13799"],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[PartialD]", 
     SuperscriptBox["\[Eta]", "i"]}], "/", 
    RowBox[{"\[PartialD]", 
     SuperscriptBox["y", "k"]}]}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SuperscriptBox["\[Xi]", "j"], "[", "x", "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "h"]}]}], " ", 
     SubsuperscriptBox["B", "k", "h"], " ", 
     RowBox[{
      SubsuperscriptBox["A", "j", "i"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}]}], " ", "+", " ", 
    RowBox[{
     SuperscriptBox["\[Xi]", "j"], 
     RowBox[{
      RowBox[{
       RowBox[{"\[PartialD]", 
        SubsuperscriptBox["A", "j", "i"]}], "/", 
       RowBox[{"\[PartialD]", 
        SuperscriptBox["x", "h"]}]}], "\[CenterDot]", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["B", "k", "h"], "[", 
        RowBox[{"x", "[", "y", "]"}], "]"}], " ", ".", 
       "\[FilledSquare]"}]}]}]}]}],
  FontColor->GrayLevel[0]]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650093128112187*^9, 3.650093168173464*^9}, {
   3.650093218657909*^9, 3.6500933024237595`*^9}, {3.6500933330644464`*^9, 
   3.6500933473926353`*^9}, {3.6500933804239306`*^9, 3.650093391470775*^9}, 
   3.6500934376740837`*^9, 3.650093548591771*^9, {3.6501757692723494`*^9, 
   3.650175781037918*^9}},ExpressionUUID->"bf9af45b-2257-42c2-8d47-\
fb20c388f9ab"],

Cell[TextData[{
 "If in affine or Euclidean geometry only linear coordinate transformations \
are considered, the second term on the lefthand side of this equation \
vanishes since  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"\[PartialD]", 
    SubsuperscriptBox["A", "j", "i"]}], "/", 
   RowBox[{"\[PartialD]", 
    SuperscriptBox["x", "h"]}]}]],
  CellChangeTimes->{{3.650093128112187*^9, 3.650093168173464*^9}, {
    3.650093218657909*^9, 3.6500933024237595`*^9}, {3.6500933330644464`*^9, 
    3.6500933473926353`*^9}, {3.6500933804239306`*^9, 3.650093391470775*^9}, 
    3.6500934376740837`*^9, 3.650093548591771*^9},ExpressionUUID->
  "a9a6dc43-5d26-4b78-a4b1-1b213210b219"],
 " =  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[PartialD]", "2"], TraditionalForm]],ExpressionUUID->
  "9f7560ad-fc08-47bd-9dde-c29a6ec9e300"],
 "y/\[PartialD]",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "j"], TraditionalForm]],ExpressionUUID->
  "fa3462b0-1edd-4fd5-aaef-f52eb40ef5f7"],
 "\[PartialD]",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "h"], " "}], TraditionalForm]],ExpressionUUID->
  "662e454d-3793-46ab-8ab9-317e537d12ae"],
 "= 0. The remaining equation describes the transformation of  the \
coordinates of the gradient tensor, a tensor of type (1,1).  If the \
coordinate transformation isn\[CloseCurlyQuote]t linear, the second term may \
not vanish. For example, if the Cartesian coordinates of a vector field are \
constant (parallel vector field), its coordinates with respect to a \
curvilinear coordinate system will not vanish, in general. On a smooth \
manifold the partial derivatives  of  vector field coordinates are not the \
coordinates of a tensor. To get a gradient as a tensor field one has to \
introduce  an additional structure, a",
 StyleBox[" linear connection",
  FontColor->RGBColor[0, 0, 1]],
 ", see the next section."
}], "Text",
 CellChangeTimes->{{3.65009359696688*^9, 3.650093773873516*^9}, {
   3.6501743651088*^9, 3.65017442333912*^9}, {3.650174465201007*^9, 
   3.650174572935593*^9}, {3.6501746057318277`*^9, 3.6501749118520584`*^9}, {
   3.6501750840949364`*^9, 3.650175186118867*^9}, {3.6501752607306757`*^9, 
   3.6501752697307653`*^9}, {3.6501753413826733`*^9, 
   3.6501755404934807`*^9}, {3.6501756063638625`*^9, 3.6501756282579355`*^9}, 
   3.650175853998637*^9, 3.6501758868982797`*^9, {3.65017594042927*^9, 
   3.6501759576378202`*^9}, {3.6501762009718065`*^9, 3.6501762021749597`*^9}, 
   3.6501988228774543`*^9, 3.668855071014559*^9, 
   3.702019569629141*^9},ExpressionUUID->"3d07847c-dc3b-4608-b53b-\
7d0726cb9ade"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "3.2. Linear ",
 StyleBox["Connections", "Subsubsection"]
}], "Subsection",
 CellChangeTimes->{{3.647779258258964*^9, 3.647779260926569*^9}, {
   3.6477793039514446`*^9, 3.6477793541679325`*^9}, 3.65017631155942*^9, {
   3.650176361103219*^9, 3.6501763795465565`*^9}, {3.6501764117794824`*^9, 
   3.6501764337393317`*^9}, {3.703050586246426*^9, 3.703050613333578*^9}, {
   3.704105929239319*^9, 3.704105931622799*^9}, 
   3.7041060531186457`*^9},ExpressionUUID->"7c0f4aa8-e57e-4c5f-8ab4-\
52ebc8991276"],

Cell[CellGroupData[{

Cell["3.2.1. Introduction", "Subsubsection",
 CellChangeTimes->{{3.650199080937522*^9, 3.650199089051357*^9}, 
   3.668855224595316*^9, {3.668855680030759*^9, 3.668855685798607*^9}, {
   3.66886640824984*^9, 3.668866420689698*^9}, 3.702019699919251*^9, 
   3.702091732666394*^9, 3.703050599439516*^9, 3.703051188593014*^9, 
   3.704106060623019*^9, 
   3.704106136925856*^9},ExpressionUUID->"2053ffc7-e32b-4c2b-8df7-\
d11e8a53aa46"],

Cell[TextData[{
 "In this section we introduce  the concept of a linear connection on a \
smooth manifold. Such a geometric structure allows to define an ",
 StyleBox["absolute Differential",
  FontColor->RGBColor[0, 0, 1]],
 " for vector fields, and moreover for arbitrary tensor fields t[x]. The \
absolute Differential is a differential operator of first order on the \
manifold. Generally, a ",
 StyleBox["function or operator D",
  FontColor->RGBColor[0, 0, 1]],
 " acting on tensor fields as "
}], "Text",
 CellChangeTimes->{{3.6501987721979785`*^9, 3.650198879015436*^9}, {
   3.6501989110917544`*^9, 3.6501989125875134`*^9}, {3.650198958124126*^9, 
   3.6501990417635508`*^9}, {3.6501991284649954`*^9, 
   3.6501992134353456`*^9}, {3.6501992525234528`*^9, 3.650199301094574*^9}, {
   3.6501994571167717`*^9, 3.650199523364133*^9}, {3.6501995901430264`*^9, 
   3.6501997392679152`*^9}, {3.6501997804230556`*^9, 
   3.6501998852454214`*^9}, {3.6501999343810344`*^9, 
   3.6502000423364487`*^9}, {3.6502000752893815`*^9, 3.650200142278719*^9}, {
   3.6502001735670085`*^9, 3.6502004662143326`*^9}, {3.6502005495603037`*^9, 
   3.6502007174471245`*^9}, 3.650200801882692*^9, 3.6502008981775904`*^9, {
   3.650201537418646*^9, 3.650201571304493*^9}, {3.6502016142729635`*^9, 
   3.6502016142729635`*^9}, {3.650201649881037*^9, 3.650201649883027*^9}, {
   3.6502016875168705`*^9, 3.65020169509058*^9}, {3.6502017266503367`*^9, 
   3.650201726665962*^9}, 3.650262836277864*^9, {3.6560824257677937`*^9, 
   3.656082468704318*^9}, 3.656082566135847*^9, 3.656082682580011*^9, {
   3.656137029122912*^9, 3.656137029690713*^9}, {3.656157857961919*^9, 
   3.656157861953549*^9}, 3.6566606879720716`*^9, {3.656682216019473*^9, 
   3.656682280593749*^9}, {3.656682346169841*^9, 3.656682360193739*^9}, {
   3.6566823915616493`*^9, 3.656682399577374*^9}, {3.6566824446490602`*^9, 
   3.6566825275205927`*^9}, {3.6566825634578323`*^9, 3.656682596263338*^9}, 
   3.656683295697234*^9, 
   3.656747798130376*^9},ExpressionUUID->"bdcdaf63-ae37-409a-83b8-\
d6072910cc34"],

Cell[BoxData[
 RowBox[{"D", ":", " ", 
  RowBox[{"T", "\[Element]", " ", 
   RowBox[{
    RowBox[{
     FormBox[
      SuperscriptBox["\[ScriptCapitalJ]", 
       RowBox[{"(", 
        RowBox[{"p", ",", "q"}], ")"}]],
      TraditionalForm], 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}], " ", "\[RightTeeArrow]", "DT"}], " ", "\[Element]", 
   " ", 
   RowBox[{
    FormBox[
     SuperscriptBox["\[ScriptCapitalJ]", 
      RowBox[{"(", 
       RowBox[{"r", ",", "s"}], ")"}]],
     TraditionalForm], 
    FormBox[
     SuperscriptBox["M", "n"],
     TraditionalForm]}]}]}]], "Input",
 Evaluatable->False,ExpressionUUID->"3ed4db79-b202-468d-adca-bbc6ece279f8"],

Cell[TextData[{
 "is said to be ",
 StyleBox["of order k",
  FontColor->RGBColor[0, 0, 1]],
 " if the tensor coordinates of the values DT only depend on the derivatives \
of the  tensor coordinates of T with respect to the point coordinates ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "1"], TraditionalForm]],ExpressionUUID->
  "5a0cf52f-fb05-47bc-82ff-ac222184c622"],
 ",...,",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]],ExpressionUUID->
  "d0e94dcc-06bc-42bb-8007-9502df10c723"],
 " of the charts  up to order k and not only on  derivatives of order less \
than k.   For functions the absolute differential coincides with the usual \
differential considered in Subsection 2.4.2.  The absolute differential of a \
tensor field T of type (p, q) is a bilinear map between real vector spaces"
}], "Text",
 CellChangeTimes->{{3.6501987721979785`*^9, 3.650198879015436*^9}, {
   3.6501989110917544`*^9, 3.6501989125875134`*^9}, {3.650198958124126*^9, 
   3.6501990417635508`*^9}, {3.6501991284649954`*^9, 
   3.6501992134353456`*^9}, {3.6501992525234528`*^9, 3.650199301094574*^9}, {
   3.6501994571167717`*^9, 3.650199523364133*^9}, {3.6501995901430264`*^9, 
   3.6501997392679152`*^9}, {3.6501997804230556`*^9, 
   3.6501998852454214`*^9}, {3.6501999343810344`*^9, 
   3.6502000423364487`*^9}, {3.6502000752893815`*^9, 3.650200142278719*^9}, {
   3.6502001735670085`*^9, 3.6502004662143326`*^9}, {3.6502005495603037`*^9, 
   3.6502007174471245`*^9}, 3.650200801882692*^9, 3.6502008981775904`*^9, {
   3.650201537418646*^9, 3.650201571304493*^9}, {3.6502016142729635`*^9, 
   3.6502016142729635`*^9}, {3.650201649881037*^9, 3.650201649883027*^9}, {
   3.6502016875168705`*^9, 3.65020169509058*^9}, {3.6502017266503367`*^9, 
   3.650201726665962*^9}, 3.650262836277864*^9, {3.6560824257677937`*^9, 
   3.656082468704318*^9}, 3.656082566135847*^9, 3.656082682580011*^9, {
   3.656137029122912*^9, 3.656137029690713*^9}, {3.656157857961919*^9, 
   3.656157861953549*^9}, 3.6566606879720716`*^9, {3.656682216019473*^9, 
   3.656682280593749*^9}, {3.656682346169841*^9, 3.656682360193739*^9}, {
   3.6566823915616493`*^9, 3.656682399577374*^9}, {3.6566824446490602`*^9, 
   3.6566825275205927`*^9}, {3.6566825634578323`*^9, 
   3.6566825896164713`*^9}, {3.656682853469095*^9, 3.656682892604845*^9}, {
   3.656682923380574*^9, 3.656682994364008*^9}, {3.656683321504977*^9, 
   3.656683380968462*^9}, {3.656683638621537*^9, 3.6566836397892942`*^9}, {
   3.656683691460909*^9, 3.656683701436976*^9}, {3.656683734660902*^9, 
   3.6566839501467667`*^9}, {3.656684000930485*^9, 3.656684020042614*^9}, {
   3.656684118682686*^9, 3.656684147001844*^9}, {3.6566843831036367`*^9, 
   3.656684415497504*^9}, {3.656744791964096*^9, 3.656744822531929*^9}, {
   3.65674485681242*^9, 
   3.6567448586426983`*^9}},ExpressionUUID->"bc8b9592-1504-4276-8f9e-\
d04a450a3b84"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["D", "X"], ":", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"X", ",", "T"}], ")"}], "\[Element]", " ", 
    RowBox[{
     RowBox[{"\[ScriptCapitalX]", 
      RowBox[{
       FormBox[
        SuperscriptBox["M", "n"],
        TraditionalForm], " ", "\[Cross]", 
       FormBox[
        SuperscriptBox["\[ScriptCapitalJ]", 
         RowBox[{"(", 
          RowBox[{"p", ",", "q"}], ")"}]],
        TraditionalForm]}], 
      FormBox[
       SuperscriptBox["M", "n"],
       TraditionalForm]}], " ", "\[RightTeeArrow]", "  ", 
     RowBox[{
      SubscriptBox["D", "X"], "T"}]}], "\[Element]", " ", 
    RowBox[{
     FormBox[
      SuperscriptBox["\[ScriptCapitalJ]", 
       RowBox[{"(", 
        RowBox[{"p", ",", "q"}], ")"}]],
      TraditionalForm], 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}]}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6519077484399233`*^9, 3.65190774844132*^9}, {
   3.6519084533180428`*^9, 3.6519084566182737`*^9}, {3.651922369090623*^9, 
   3.651922410914259*^9}, {3.6519224599379807`*^9, 3.65192247265202*^9}, {
   3.651922511937611*^9, 3.651922596494038*^9}, {3.651922648376692*^9, 
   3.651922655781374*^9}, 3.6519226973689413`*^9, 3.651922776151458*^9, {
   3.652337428656632*^9, 3.652337461991699*^9}, {3.6560827553389072`*^9, 
   3.656082758994206*^9}, {3.656158011616036*^9, 3.65615803052672*^9}, 
   3.656682770022314*^9, 3.656747838770434*^9, {3.656747871681439*^9, 
   3.6567478745698147`*^9}},ExpressionUUID->"66a6e240-8ddb-45f8-befb-\
3f446977112d"],

Cell[TextData[{
 "X a vector field, where the absolute differential ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["D", "X"], "T"}]], "Input",
  CellChangeTimes->{{3.6519077484399233`*^9, 3.65190774844132*^9}, {
    3.6519084533180428`*^9, 3.6519084566182737`*^9}, {3.651922369090623*^9, 
    3.651922410914259*^9}, {3.6519224599379807`*^9, 3.65192247265202*^9}, {
    3.651922511937611*^9, 3.651922596494038*^9}, {3.651922648376692*^9, 
    3.651922655781374*^9}, 3.6519226973689413`*^9, 3.651922776151458*^9},
  ExpressionUUID->"31146376-80ef-4b69-9fd9-75f9f897f812"],
 " is related to the ",
 StyleBox["gradient tensor",
  FontColor->RGBColor[0, 0, 1]],
 " \[Del]T by a contraction"
}], "Text",
 CellChangeTimes->{{3.651922798582925*^9, 3.651922800270746*^9}, 
   3.6519265034561653`*^9, {3.651932038049959*^9, 3.651932077753664*^9}, {
   3.6519321086531963`*^9, 3.6519321440570908`*^9}, {3.6560827912907267`*^9, 
   3.656082831937333*^9}, 3.656157653258802*^9, {3.702091413309636*^9, 
   3.702091424709433*^9}},ExpressionUUID->"27a8db33-ed3a-48d2-a695-\
26b32b5ee7d4"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["D", "X"], "T"}], ":=", " ", 
    RowBox[{"contraction", "[", 
     RowBox[{"X", "\[CircleTimes]", 
      RowBox[{"\[Del]", "T"}]}], "]"}]}], ";"}], " "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6519228538304977`*^9, 3.65192285726292*^9}, {
   3.651923112036975*^9, 3.65192314152458*^9}, 3.6519232052202044`*^9, 
   3.6519321709056177`*^9, 3.656082955329113*^9, 
   3.656083192446879*^9},ExpressionUUID->"b753a09c-59fc-4d69-8aab-\
f575727ebe45"],

Cell["\<\
\[Del]T is a tensor field of type (p+1,q) on the manifold with coordinates\
\>", "Text",
 CellChangeTimes->{{3.6519322537281733`*^9, 3.651932283111724*^9}, 
   3.651932417374786*^9, {3.656082995417227*^9, 3.65608299566746*^9}, {
   3.656083029320177*^9, 3.656083119247126*^9}, 3.656083180535741*^9, {
   3.656157702922085*^9, 3.656157744963112*^9}, 3.6561578966805162`*^9, {
   3.656158042526422*^9, 3.65615805571137*^9}, {3.656684198504426*^9, 
   3.656684203455933*^9}},ExpressionUUID->"75482fa3-e411-471f-adc7-\
3f665112a237"],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["\[Del]", "h"], "T"}], ")"}], 
   RowBox[{
    RowBox[{
     SubscriptBox["j", "1"], "..."}], 
    SubscriptBox["j", "p"]}], 
   RowBox[{
    RowBox[{
     SubscriptBox["i", "1"], "..."}], 
    SubscriptBox["i", "q"]}]], " ", ":", "   ", 
  RowBox[{
   RowBox[{
    SubscriptBox["D", "X"], "T"}], " ", "\[Equal]", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", "h"], "*", 
    SubsuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[Del]", "h"], "T"}], ")"}], 
     RowBox[{
      RowBox[{
       SubscriptBox["j", "1"], "..."}], 
      SubscriptBox["j", "p"]}], 
     RowBox[{
      RowBox[{
       SubscriptBox["i", "1"], "..."}], 
      SubscriptBox["i", "q"]}]]}]}]}]], "Input",
 Evaluatable->False,ExpressionUUID->"00f0dd21-e3b1-499a-bb21-8966d229b5a8"],

Cell[TextData[{
 "where the tangent vector or the vector field X has the coordinates ",
 Cell[BoxData[
  SuperscriptBox["\[Xi]", "h"]], "Input",
  CellChangeTimes->{{3.656157824652059*^9, 3.656157824653529*^9}, {
   3.656157911376399*^9, 3.656157920128286*^9}, {3.6561579669979563`*^9, 
   3.656157988300477*^9}, {3.6561580764150667`*^9, 3.656158268366118*^9}, {
   3.6561583342849407`*^9, 3.656158450436617*^9}, {3.6561584902126102`*^9, 
   3.656158503548197*^9}},ExpressionUUID->
  "479b727f-b80b-4a90-8f73-2be0d31d1980"],
 ", and the operators ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", 
    RowBox[{"X", " "}]], TraditionalForm]],ExpressionUUID->
  "3e4526fd-e9fa-463b-bdab-82506a37e043"],
 " is of first order. For functions the absolute differential D equals the \
usual differential d, and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", 
    RowBox[{"X", " "}]], TraditionalForm]],ExpressionUUID->
  "02330725-fa72-48ee-b7e1-95ee72c3790d"],
 " corresponds to the directional derivative X]f][x] == d[f][x].X. The last \
three formulas as well as many formulas in the following are written in \
traditional notation. Here and furthermore in traditional notations we often \
omit the point variable x or its coordinates ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "i"], TraditionalForm]],ExpressionUUID->
  "b80963bb-367f-48df-a70d-e216c3122c4b"],
 ".  "
}], "Text",
 CellChangeTimes->{{3.6519322537281733`*^9, 3.651932283111724*^9}, 
   3.651932417374786*^9, {3.656082995417227*^9, 3.65608299566746*^9}, {
   3.656083029320177*^9, 3.656083119247126*^9}, 3.656083180535741*^9, {
   3.656157702922085*^9, 3.656157744963112*^9}, {3.656158535211598*^9, 
   3.6561585725310793`*^9}, {3.6566842384646263`*^9, 3.656684239927743*^9}, {
   3.656684480486474*^9, 3.6566844984702044`*^9}, {3.656684628077469*^9, 
   3.6566846451250687`*^9}, {3.656684724123876*^9, 3.6566848855478487`*^9}, {
   3.6567479423928347`*^9, 3.656748131479661*^9}, {3.656748874449914*^9, 
   3.656748875129539*^9}, 3.656916020485544*^9, {3.65691607716486*^9, 
   3.656916099748847*^9}, {3.656916464218116*^9, 3.656916601856819*^9}, {
   3.656916643545343*^9, 3.656916696657083*^9}, {3.7020915240560904`*^9, 
   3.702091525462175*^9}, {3.702091565238636*^9, 
   3.702091590828082*^9}},ExpressionUUID->"1e41cfa1-2e38-49f9-b623-\
8bdbc0362d4a"],

Cell[TextData[{
 "A conceptional global construction of the absolute differential one can \
find in many books about differential geometry, see e.g ",
 ButtonBox["S.Helgason",
  BaseStyle->"Hyperlink",
  ButtonData->"H78"],
 ", Section 1.4, or ",
 ButtonBox["Kobayashi, Nomizu",
  BaseStyle->"Hyperlink",
  ButtonData->"K-N1"],
 ", Chapter 3.  In  older literature one often denotes the linear connections \
 as affine connections what isn't quite correct from a modern point of view. \
To introduce the linear connections we follow here a classical way \
emphasizing the calculation with the coordinates of the tensor fields, see ",
 ButtonBox["J. A. Schouten",
  BaseStyle->"Hyperlink",
  ButtonData->"Sch54"],
 ", Chapter 3. "
}], "Text",
 CellChangeTimes->{{3.6501987721979785`*^9, 3.650198879015436*^9}, {
   3.6501989110917544`*^9, 3.6501989125875134`*^9}, {3.650198958124126*^9, 
   3.6501990417635508`*^9}, {3.6501991284649954`*^9, 
   3.6501992134353456`*^9}, {3.6501992525234528`*^9, 3.650199301094574*^9}, {
   3.6501994571167717`*^9, 3.650199523364133*^9}, {3.6501995901430264`*^9, 
   3.6501997392679152`*^9}, {3.6501997804230556`*^9, 
   3.6501998852454214`*^9}, {3.6501999343810344`*^9, 
   3.6502000423364487`*^9}, {3.6502000752893815`*^9, 3.650200142278719*^9}, {
   3.6502001735670085`*^9, 3.6502004662143326`*^9}, {3.6502005495603037`*^9, 
   3.6502007174471245`*^9}, 3.650200801882692*^9, 3.6502008981775904`*^9, {
   3.650201537418646*^9, 3.650201571304493*^9}, {3.6502016142729635`*^9, 
   3.6502016142729635`*^9}, {3.650201649881037*^9, 3.650201649883027*^9}, {
   3.6502016875168705`*^9, 3.65020169509058*^9}, {3.6502017266503367`*^9, 
   3.650201726665962*^9}, 3.650262836277864*^9, {3.6560824257677937`*^9, 
   3.656082468704318*^9}, 3.656082566135847*^9, {3.656082896706449*^9, 
   3.6560829002423553`*^9}, 3.656082943786875*^9, 3.656083230119783*^9, {
   3.656083284639888*^9, 
   3.656083292462549*^9}},ExpressionUUID->"2ae368db-f4e2-4ec4-988f-\
9b8623c6b299"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2.2. The Gradient of a Vector Field", "Subsubsection",
 CellChangeTimes->{{3.650199080937522*^9, 3.650199089051357*^9}, {
   3.6502628893869257`*^9, 3.650262928073685*^9}, {3.650864075009926*^9, 
   3.6508641010724707`*^9}, {3.656084926040357*^9, 3.6560849283523817`*^9}, 
   3.65615355305014*^9, {3.7020917002506847`*^9, 3.702091703546493*^9}, 
   3.702096893760199*^9, 3.703051198233882*^9, 3.703138829849955*^9, 
   3.704106145942668*^9, 
   3.704107056574736*^9},ExpressionUUID->"08f5f113-7e76-44fe-a6af-\
d61e398ba9ab"],

Cell[TextData[{
 "We consider the real vector space \[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "e79f0835-3cf8-452c-a75b-2de2ef56f677"],
 " of all smooth vector fields on the smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "b388eb95-d4d1-49a5-8450-117aae457a85"],
 " (Proposition 2.3.3).  As in Lemma 3.1 let X \[Element] \[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "7b922fb8-cba9-4cea-805a-a619ebcf5df4"],
 " be such a field with local coordinates \[Xi], \[Eta] transformed at the \
intersection of two charts by"
}], "Text",
 CellChangeTimes->{{3.650263332051921*^9, 3.6502634141020784`*^9}, {
  3.6502634450236025`*^9, 3.6502634578972225`*^9}, {3.650263856888285*^9, 
  3.65026401907924*^9}, {3.6502640499770107`*^9, 3.650264057211486*^9}, {
  3.6502641715364075`*^9, 3.6502641889114747`*^9}, {3.6508036852421637`*^9, 
  3.6508037311016817`*^9}, {3.6508629671308265`*^9, 3.65086296714642*^9}, {
  3.703051335703719*^9, 
  3.703051337079996*^9}},ExpressionUUID->"10bbc49e-1fca-4ea5-84c3-\
62a58b2979c2"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}], " ", "=", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["A", "j", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}], 
    RowBox[{
     SuperscriptBox["\[Xi]", "j"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}], "                              ", 
    RowBox[{"(", "1", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6502641571287565`*^9, 3.6502641639426765`*^9}, 
   3.6502641997078342`*^9, {3.650264352063588*^9, 3.6502644346745625`*^9}, {
   3.650264468282069*^9, 3.650264483152611*^9}, 3.6502645475732293`*^9, {
   3.65026458235839*^9, 3.650264602670939*^9}, {3.6502646429381514`*^9, 
   3.6502647858198223`*^9}, {3.6502734407665815`*^9, 
   3.6502734538254004`*^9}, {3.65608406784035*^9, 3.6560840769757423`*^9}, 
   3.656084302423015*^9, {3.702092260502816*^9, 
   3.7020922705561743`*^9}},ExpressionUUID->"cc4da17e-5602-4efc-b588-\
feb3cda05a79"],

Cell["\<\
(in traditional notation). The transformation formula of the first partial \
derivatives of the coordinates shows that there appears a disturbing second \
term  forbidding to consider the derivatives as the coordinates of a tensor \
(see Lemma 3.1.1):\
\>", "Text",
 CellChangeTimes->{{3.6502648361531687`*^9, 3.650264884871407*^9}, 
   3.650264945566358*^9, {3.650265000831976*^9, 3.6502650418514504`*^9}, {
   3.6502651979304247`*^9, 3.6502652384514275`*^9}, {3.6502654107747145`*^9, 
   3.650265456413926*^9}, 3.650265486800178*^9, {3.650265909662795*^9, 
   3.6502660692432137`*^9}, {3.6502661004317007`*^9, 
   3.6502662277802534`*^9}, {3.650266295543291*^9, 3.6502663326430454`*^9}, {
   3.650864132071072*^9, 3.650864168203903*^9}, 3.651895101809842*^9, {
   3.656083972914433*^9, 3.656084000519106*^9}, {3.6561381360494947`*^9, 
   3.6561381494174423`*^9}, {3.7020920774317417`*^9, 
   3.7020920788717337`*^9}},ExpressionUUID->"4c0d7bf1-4683-4d20-b55b-\
2cc422306ec4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[PartialD]", " ", 
     RowBox[{
      SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}]}], "/", 
    RowBox[{"\[PartialD]", 
     SuperscriptBox["y", "h"]}]}], " ", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubsuperscriptBox["A", "j", "i"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}], 
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SuperscriptBox["\[Xi]", "j"], "[", 
        RowBox[{"x", "[", "y", "]"}], "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "l"]}]}], "*", 
     RowBox[{
      SubsuperscriptBox["B", "h", "l"], "[", "y", "]"}]}], " ", "+", 
    RowBox[{
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SubsuperscriptBox["A", "j", "i"], "[", 
        RowBox[{"x", "[", "y", "]"}], "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "l"]}]}], "*", 
     RowBox[{
      SubsuperscriptBox["B", "h", "l"], "[", "y", "]"}], "*", 
     RowBox[{
      SuperscriptBox["\[Xi]", "j"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}]}]}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6502641571287565`*^9, 3.6502641639426765`*^9}, 
   3.6502641997078342`*^9, {3.650264352063588*^9, 3.6502644346745625`*^9}, {
   3.650264468282069*^9, 3.650264483152611*^9}, 3.6502645475732293`*^9, {
   3.65026458235839*^9, 3.650264602670939*^9}, {3.6502646429381514`*^9, 
   3.6502647858198223`*^9}, {3.6502734407665815`*^9, 
   3.6502734538254004`*^9}, {3.65608414948722*^9, 3.6560842217988243`*^9}, 
   3.656084296326646*^9, {3.656084339389889*^9, 3.65608439454135*^9}, {
   3.656084479652459*^9, 3.656084490640374*^9}, {3.656084550747917*^9, 
   3.6560845993635597`*^9}, {3.6560846485650387`*^9, 3.6560847304908047`*^9}, 
   3.6560847767298803`*^9},ExpressionUUID->"05fa5980-4986-4aa0-8d01-\
08614c4d8aba"],

Cell[TextData[{
 " here we used  ",
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["B", "h", "l"], "[", "y", "]"}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6502641571287565`*^9, 3.6502641639426765`*^9}, 
    3.6502641997078342`*^9, {3.650264352063588*^9, 3.6502644346745625`*^9}, {
    3.650264468282069*^9, 3.650264483152611*^9}, 3.6502645475732293`*^9, {
    3.65026458235839*^9, 3.650264602670939*^9}, {3.6502646429381514`*^9, 
    3.6502647858198223`*^9}, {3.6502734407665815`*^9, 
    3.6502734538254004`*^9}, {3.65608414948722*^9, 3.6560842217988243`*^9}, 
    3.656084296326646*^9, {3.656084339389889*^9, 3.65608439454135*^9}, {
    3.656084479652459*^9, 3.656084490640374*^9}, {3.656084550747917*^9, 
    3.6560845993635597`*^9}, {3.6560846485650387`*^9, 3.6560847304908047`*^9},
     3.6560847767298803`*^9},ExpressionUUID->
  "32496132-0109-4a17-bf79-3ea18df3cae3"],
 "= ",
 Cell[BoxData[
  RowBox[{"\[PartialD]", 
   SuperscriptBox["x", "l"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6502641571287565`*^9, 3.6502641639426765`*^9}, 
    3.6502641997078342`*^9, {3.650264352063588*^9, 3.6502644346745625`*^9}, {
    3.650264468282069*^9, 3.650264483152611*^9}, 3.6502645475732293`*^9, {
    3.65026458235839*^9, 3.650264602670939*^9}, {3.6502646429381514`*^9, 
    3.6502647858198223`*^9}, {3.6502734407665815`*^9, 
    3.6502734538254004`*^9}, {3.65608414948722*^9, 3.6560842217988243`*^9}, 
    3.656084296326646*^9, {3.656084339389889*^9, 3.65608439454135*^9}, {
    3.656084479652459*^9, 3.656084490640374*^9}, {3.656084550747917*^9, 
    3.6560845993635597`*^9}, {3.6560846485650387`*^9, 3.6560847304908047`*^9},
     3.6560847767298803`*^9},ExpressionUUID->
  "de8306bd-e36a-4e03-917b-bfd6dfa5e83d"],
 "/",
 Cell[BoxData[
  RowBox[{"\[PartialD]", 
   SuperscriptBox["y", "h"], " "}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6502641571287565`*^9, 3.6502641639426765`*^9}, 
    3.6502641997078342`*^9, {3.650264352063588*^9, 3.6502644346745625`*^9}, {
    3.650264468282069*^9, 3.650264483152611*^9}, 3.6502645475732293`*^9, {
    3.65026458235839*^9, 3.650264602670939*^9}, {3.6502646429381514`*^9, 
    3.6502647858198223`*^9}, {3.6502734407665815`*^9, 
    3.6502734538254004`*^9}, {3.65608414948722*^9, 3.6560842217988243`*^9}, 
    3.656084296326646*^9, {3.656084339389889*^9, 3.65608439454135*^9}, {
    3.656084479652459*^9, 3.656084490640374*^9}, {3.656084550747917*^9, 
    3.6560845993635597`*^9}, {3.6560846485650387`*^9, 3.6560847304908047`*^9},
     3.6560847767298803`*^9},ExpressionUUID->
  "69ee5780-f005-4af4-9159-db4991f6f6cd"],
 ",  remember also "
}], "Text",
 CellChangeTimes->{{3.6502648361531687`*^9, 3.650264884871407*^9}, 
   3.650264945566358*^9, {3.650265000831976*^9, 3.6502650418514504`*^9}, {
   3.6502651979304247`*^9, 3.6502652384514275`*^9}, {3.6502654107747145`*^9, 
   3.650265456413926*^9}, 3.650265486800178*^9, {3.650265909662795*^9, 
   3.6502660692432137`*^9}, {3.6502661004317007`*^9, 
   3.6502662277802534`*^9}, {3.650266295543291*^9, 3.6502663326430454`*^9}, {
   3.650864132071072*^9, 3.650864168203903*^9}, 3.651895101809842*^9, {
   3.656083972914433*^9, 3.656084000519106*^9}, {3.656084745379163*^9, 
   3.656084824233951*^9}, {3.656084863673717*^9, 3.6560849172412043`*^9}, {
   3.6561371460015507`*^9, 3.656137152689423*^9}, 
   3.656137183396002*^9},ExpressionUUID->"d9b9b4f5-8a1b-4e98-a221-\
5ad20c334423"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[PartialD]", 
    RowBox[{
     SubsuperscriptBox["A", "j", "i"], "[", "x", "]"}]}], "/", 
   RowBox[{"\[PartialD]", 
    SuperscriptBox["x", "l"]}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["\[PartialD]", "2"], 
     SuperscriptBox["y", "i"]}], "/", 
    RowBox[{"\[PartialD]", 
     SuperscriptBox["x", "l"]}]}], 
   RowBox[{
    RowBox[{"\[PartialD]", 
     SuperscriptBox["x", "j"]}], "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6561372183843718`*^9, 3.656137223960474*^9}, {
   3.656137332641967*^9, 3.6561374156948233`*^9}, 
   3.668861742199971*^9},ExpressionUUID->"bd9e17e3-e6ab-4fa3-befd-\
0332754fdb27"],

Cell["\<\
 To compensate the disturbing last  term we need a new geometric object whose \
coordinates transform uses second derivatives of the transformation in an \
appropriate way. The  gradient introduced for tangent vector fields of a \
surface or an immersion into the Euclidean space leads to the following \
ansatz for the components of the gradient tensor:\
\>", "Text",
 CellChangeTimes->{{3.6502648361531687`*^9, 3.650264884871407*^9}, 
   3.650264945566358*^9, {3.650265000831976*^9, 3.6502650418514504`*^9}, {
   3.6502651979304247`*^9, 3.6502652384514275`*^9}, {3.6502654107747145`*^9, 
   3.650265456413926*^9}, 3.650265486800178*^9, {3.650265909662795*^9, 
   3.6502660692432137`*^9}, {3.6502661004317007`*^9, 
   3.6502662277802534`*^9}, {3.650266295543291*^9, 3.6502663326430454`*^9}, {
   3.650864132071072*^9, 3.650864168203903*^9}, 3.651895101809842*^9, {
   3.656083972914433*^9, 3.656084000519106*^9}, {3.656084745379163*^9, 
   3.656084824233951*^9}, {3.656084863673717*^9, 3.6560849172412043`*^9}, {
   3.6561371460015507`*^9, 3.656137152689423*^9}, 3.656137183396002*^9, {
   3.656137488367096*^9, 3.65613749488608*^9}, {3.656137527630234*^9, 
   3.6561375343500853`*^9}, {3.656159020543783*^9, 
   3.6561590276000137`*^9}},ExpressionUUID->"59ed459b-d27b-40fa-8193-\
f992fe752df9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "h"], 
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], "[", "x", "]"}]}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "j"], "[", "x", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}], 
    RowBox[{
     SubsuperscriptBox["\[CapitalGamma]", "hk", "j"], "[", "x", "]"}], "                                                                ", 
    RowBox[{"(", "2", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6502664656646566`*^9, {3.650266529375502*^9, 3.6502666491873617`*^9}, 
   3.6502666870716534`*^9, {3.6502731006614904`*^9, 3.6502731908860517`*^9}, 
   3.6502732654402127`*^9, {3.6502734675989227`*^9, 3.650273486253156*^9}, {
   3.6502735242610545`*^9, 3.650273540209237*^9}, {3.650804193571299*^9, 
   3.6508041944150505`*^9}, {3.6534813187324142`*^9, 
   3.65348131955547*^9}},ExpressionUUID->"7fd6185d-510a-40a6-99a3-\
aaa1b48f15fa"],

Cell[TextData[{
 "the",
 StyleBox[" \[CapitalGamma]-symbols",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 Cell[BoxData[
  RowBox[{
   StyleBox[
    RowBox[{
     SubsuperscriptBox["\[CapitalGamma]", "hk", "j"], "[", "x", "]"}],
    FontWeight->"Bold"], 
   StyleBox[" ",
    FontWeight->"Bold"], "\[DotEqual]"}]],
  Evaluatable->False,
  CellChangeTimes->{
   3.6502664656646566`*^9, {3.650266529375502*^9, 3.6502666491873617`*^9}, 
    3.6502666870716534`*^9, {3.6502731006614904`*^9, 3.6502731908860517`*^9}, 
    3.6502732654402127`*^9, {3.6502734675989227`*^9, 3.650273486253156*^9}, {
    3.6502735242610545`*^9, 3.650273540209237*^9}},ExpressionUUID->
  "9daa0a41-22cb-480a-850d-9b7df1f6a824"],
 " \[CapitalGamma][j, h, k],  j , h, k = 1,...,n, depend on the charts. They \
define the linear connection.  The index h denotes the differentiation with \
respect to the coordinate ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "h"], TraditionalForm]],ExpressionUUID->
  "df078c05-aa3f-4415-8795-9492a24b30e9"],
 ": "
}], "Text",
 CellChangeTimes->{{3.6502736021471157`*^9, 3.650273772782195*^9}, 
   3.6502738602272778`*^9, {3.650273909411725*^9, 3.650274240024228*^9}, 
   3.650274299132992*^9, {3.650274351309549*^9, 3.650274599876934*^9}, 
   3.650274693919804*^9, 3.650803919914543*^9, {3.6508042159307156`*^9, 
   3.6508042163525095`*^9}, {3.653481338094226*^9, 3.653481338884144*^9}, {
   3.655883981374132*^9, 3.655883981374172*^9}, {3.655884016887941*^9, 
   3.6558840674275427`*^9}, {3.655884102998844*^9, 3.655884102998866*^9}, {
   3.6558841343961*^9, 3.6558841947323933`*^9}, {3.655884226250358*^9, 
   3.655884235692004*^9}, {3.655884266850662*^9, 3.655884308392803*^9}, {
   3.655884344457325*^9, 3.655884350978346*^9}, 3.655884390839574*^9, {
   3.655884430943632*^9, 3.655884489735219*^9}, {3.6558845199065857`*^9, 
   3.65588457972124*^9}, {3.655884621374852*^9, 3.655884649526826*^9}, {
   3.656083391862461*^9, 3.656083416287847*^9}, {3.656083897353065*^9, 
   3.656083900184999*^9}, {3.656137593509739*^9, 3.656137638820992*^9}, 
   3.7020922023827333`*^9},ExpressionUUID->"399db336-f712-4173-9e75-\
699911398bd4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "h"], " ", "\[Equal]", " ", 
   RowBox[{"\[Del]", " ", "/", 
    RowBox[{"\[PartialD]", 
     SuperscriptBox["x", "h"]}]}]}], ",", "    ", 
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], " ", 
   RowBox[{"\[Equal]", " ", 
    RowBox[{"\[PartialD]", " ", 
     RowBox[{"/", 
      RowBox[{
       RowBox[{"\[PartialD]", 
        SuperscriptBox["x", "h"]}], "."}]}]}]}]}]}]], "Input",
 Evaluatable->False,ExpressionUUID->"022ee80c-fd16-4cbe-ad36-afcf83870e48"],

Cell["\<\
Obviously, formula (2) defines a linear differential operator of the first \
order for vector fields: \
\>", "Text",
 CellChangeTimes->{{3.6502736021471157`*^9, 3.650273772782195*^9}, 
   3.6502738602272778`*^9, {3.650273909411725*^9, 3.650274240024228*^9}, 
   3.650274299132992*^9, {3.650274351309549*^9, 3.650274599876934*^9}, 
   3.650274693919804*^9, 3.650803919914543*^9, {3.6508042159307156`*^9, 
   3.6508042163525095`*^9}, {3.653481338094226*^9, 3.653481338884144*^9}, {
   3.655883981374132*^9, 3.655883981374172*^9}, {3.655884016887941*^9, 
   3.6558840674275427`*^9}, {3.655884102998844*^9, 3.655884102998866*^9}, {
   3.6558841343961*^9, 3.6558841947323933`*^9}, {3.655884226250358*^9, 
   3.655884235692004*^9}, {3.655884266850662*^9, 3.655884308392803*^9}, {
   3.655884344457325*^9, 3.655884350978346*^9}, 3.655884390839574*^9, {
   3.655884430943632*^9, 3.655884489735219*^9}, {3.6558845199065857`*^9, 
   3.65588457972124*^9}, {3.655884621374852*^9, 3.6558846719772797`*^9}, {
   3.655884708813793*^9, 3.655884716941655*^9}, {3.655884947684066*^9, 
   3.655884972500115*^9}, {3.656687115849594*^9, 3.6566871362974787`*^9}, {
   3.656687169944998*^9, 3.65668717535255*^9}, {3.656687211272711*^9, 
   3.6566872975926*^9}, 
   3.703051849555237*^9},ExpressionUUID->"6731714d-ac7c-4063-be83-\
d765cebd2fba"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Del]", ":", "X"}], "\[Element]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[ScriptCapitalX]", 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}], " ", "\[TildeEqual]", " ", 
    RowBox[{
     FormBox[
      SuperscriptBox["\[ScriptCapitalJ]", 
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}]],
      TraditionalForm], 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}]}], " ", "\[RightTeeArrow]", " ", 
   RowBox[{"\[Del]", "X"}]}], "\[Element]", 
  RowBox[{
   FormBox[
    SuperscriptBox["\[ScriptCapitalJ]", 
     RowBox[{"(", 
      RowBox[{"1", ",", "1"}], ")"}]],
    TraditionalForm], 
   FormBox[
    SuperscriptBox["M", "n"],
    TraditionalForm]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6566873355031357`*^9, 3.6566873536709843`*^9}, {
  3.656687400136572*^9, 3.6566874581182737`*^9}, {3.6566874937667313`*^9, 
  3.656687658084858*^9}},ExpressionUUID->"76240294-4c4d-41bc-abbc-\
36895f23ad70"],

Cell["\<\
  if the expressions (2) transform as tensor coordinates. Thus our task is to \
find the transformation behavior of the \[CapitalGamma]-symbols under a \
coordinate transformation y = y[x] and its inverse x = x[y] such that the \
following condition is fulfilled:
The expressions (2) transform as the coordinates of a tensor field of 
type (1, 1):\
\>", "Text",
 CellChangeTimes->{{3.6502736021471157`*^9, 3.650273772782195*^9}, 
   3.6502738602272778`*^9, {3.650273909411725*^9, 3.650274240024228*^9}, 
   3.650274299132992*^9, {3.650274351309549*^9, 3.650274599876934*^9}, 
   3.650274693919804*^9, 3.650803919914543*^9, {3.6508042159307156`*^9, 
   3.6508042163525095`*^9}, {3.653481338094226*^9, 3.653481338884144*^9}, {
   3.655883981374132*^9, 3.655883981374172*^9}, {3.655884016887941*^9, 
   3.6558840674275427`*^9}, {3.655884102998844*^9, 3.655884102998866*^9}, {
   3.6558841343961*^9, 3.6558841947323933`*^9}, {3.655884226250358*^9, 
   3.655884235692004*^9}, {3.655884266850662*^9, 3.655884308392803*^9}, {
   3.655884344457325*^9, 3.655884350978346*^9}, 3.655884390839574*^9, {
   3.655884430943632*^9, 3.655884489735219*^9}, {3.6558845199065857`*^9, 
   3.65588457972124*^9}, {3.655884621374852*^9, 3.6558846719772797`*^9}, {
   3.655884708813793*^9, 3.655884716941655*^9}, {3.655884947684066*^9, 
   3.655884972500115*^9}, {3.656687115849594*^9, 3.6566871362974787`*^9}, {
   3.656687169944998*^9, 3.65668717535255*^9}, {3.656687211272711*^9, 
   3.6566872975926*^9}, {3.6566877069559183`*^9, 3.656687733387817*^9}, {
   3.6566877725155582`*^9, 3.656687848387115*^9}, 
   3.702092335245777*^9},ExpressionUUID->"ee8677d8-db5a-4785-a8ef-\
5e8c9c8450b7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[Del]", "^"], "k"], 
   RowBox[{
    SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}]}], "=", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "h"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "j"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], 
   RowBox[{
    SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
   RowBox[{
    SubsuperscriptBox["A", "j", "i"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}], "                                             ", 
   RowBox[{"(", "3", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650274731847533*^9, 3.650274772983717*^9}, {
   3.6502748206464586`*^9, 3.6502748402631874`*^9}, {3.6502748844211926`*^9, 
   3.6502749576166954`*^9}, {3.6502749981199675`*^9, 
   3.6502751283733516`*^9}, {3.6502884619557095`*^9, 3.6502884688463116`*^9}, 
   3.650288686597*^9, {3.6508043011183434`*^9, 
   3.650804308790262*^9}},ExpressionUUID->"d57f0c42-fafb-4631-9824-\
2935f64941a9"],

Cell[TextData[{
 "Here",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  SubscriptBox[
   OverscriptBox["\[Del]", "^"], "k"]],
  CellChangeTimes->{{3.650274731847533*^9, 3.650274772983717*^9}, {
   3.6502748206464586`*^9, 3.6502748402631874`*^9}, {3.6502748844211926`*^9, 
   3.6502749576166954`*^9}, {3.6502749981199675`*^9, 3.6502751283733516`*^9}},
  FontWeight->"Bold",ExpressionUUID->"417c3722-c8f4-4d62-b172-4edd41fc0c23"],
 StyleBox[" = ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{"\[Del]", " "}]],
  Evaluatable->False,
  CellChangeTimes->{
   3.6502664656646566`*^9, {3.650266529375502*^9, 3.6502666491873617`*^9}, 
    3.6502666870716534`*^9, {3.6502731006614904`*^9, 3.6502731908860517`*^9}, 
    3.6502732654402127`*^9, {3.6502734675989227`*^9, 3.650273486253156*^9}, {
    3.6502735242610545`*^9, 3.650273540209237*^9}},
  FontWeight->"Bold",ExpressionUUID->"69cbd86b-f962-456d-b0f0-fe465d09f406"],
 StyleBox["/\[PartialD]",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "k"], TraditionalForm]],
  FontWeight->"Bold",ExpressionUUID->"1d0945ff-36b1-4ec7-aa68-6b6d8af3bc55"],
 " is the differentiation in the new chart y and ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.650274731847533*^9, 3.650274772983717*^9}, {
   3.6502748206464586`*^9, 3.6502748402631874`*^9}, {3.6502748844211926`*^9, 
   3.6502749576166954`*^9}, {3.6502749981199675`*^9, 3.6502751283733516`*^9}},
  ExpressionUUID->"0ae14fd6-2d32-49ec-9c92-f465930be3ea"],
 " are the coordinates of the vector field with  respect to the new chart y \
in which the equation corresponding to (2) is"
}], "Text",
 CellChangeTimes->{{3.6502855324315977`*^9, 3.6502855563535447`*^9}, {
   3.650285592119288*^9, 3.6502857733073125`*^9}, {3.6502858175418096`*^9, 
   3.6502858695732164`*^9}, 
   3.650286469793705*^9},ExpressionUUID->"c9891e55-67af-494a-b505-\
38854f12cc4d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Del]", "^"], "k"], 
    RowBox[{
     SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}]}], "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "k"], 
     RowBox[{
      SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}]}], " ", "+", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Eta]", "l"], "[", "y", "]"}], 
     RowBox[{
      SubsuperscriptBox[
       OverscriptBox["\[CapitalGamma]", "^"], "kl", "i"], "[", "y", 
      "]"}]}]}]}], ",", "   "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650286483606245*^9, 3.6502864867156143`*^9}, 
   3.650286527356372*^9, 3.650286574684623*^9, {3.650286620622233*^9, 
   3.650286655450411*^9}, {3.650286977123242*^9, 3.650287068123563*^9}, {
   3.650287103889264*^9, 3.6502871194674616`*^9}, {3.6508043585247293`*^9, 
   3.6508043588216047`*^9}, {3.6534815883852787`*^9, 
   3.653481588641274*^9}},ExpressionUUID->"44622928-49dd-476d-a8a4-\
4e7796d13fe8"],

Cell[TextData[{
 "where",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox["\[CapitalGamma]", "^"], "kl", "i"], "[", "y", "]"}]],
  CellChangeTimes->{{3.650286483606245*^9, 3.6502864867156143`*^9}, 
    3.650286527356372*^9, 3.650286574684623*^9, {3.650286620622233*^9, 
    3.650286655450411*^9}, {3.650286977123242*^9, 3.650287068123563*^9}, {
    3.650287103889264*^9, 3.6502871194674616`*^9}},
  FontWeight->"Bold",ExpressionUUID->"2f149f89-2544-45a6-ba3d-64da7927fdc6"],
 " are the \[CapitalGamma]-symbols in the new chart y. For the first term on \
the right side we insert the result of Lemma 3.1.1, and in the second the \
transformation of the vector coordinates at the point x = x(y):"
}], "Text",
 CellChangeTimes->{{3.650287136905012*^9, 3.650287138748768*^9}, {
   3.6502872251865177`*^9, 3.6502872705928464`*^9}, 3.6502874236401806`*^9, {
   3.650287706187896*^9, 3.650287871188389*^9}, 3.650290207148279*^9, {
   3.6508043854622784`*^9, 3.6508043858529043`*^9}, {3.653481601947838*^9, 
   3.653481602202365*^9}, 3.65613823137643*^9, {3.656744357121093*^9, 
   3.656744369727397*^9}, {3.702092447260743*^9, 
   3.702092448196354*^9}},ExpressionUUID->"b4e10f36-061f-4c0e-884f-\
53c9c8332a82"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[Del]", "^"], "k"], 
   RowBox[{
    SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}]}], "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SuperscriptBox["\[Xi]", "j"], "[", "x", "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "h"]}]}], ")"}], " ", 
    SubsuperscriptBox["B", "k", "h"], " ", 
    RowBox[{
     SubsuperscriptBox["A", "j", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["\[Xi]", "j"], 
      RowBox[{
       RowBox[{"\[PartialD]", 
        SubsuperscriptBox["A", "j", "i"]}], "/", 
       RowBox[{"\[PartialD]", 
        SuperscriptBox["x", "h"]}]}]}], ")"}], "\[CenterDot]", 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}]}], " ", "+", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], 
    RowBox[{
     SubsuperscriptBox["A", "j", "l"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}], " ", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox["\[CapitalGamma]", "^"], "kl", "i"], "[", "y", 
     "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6502879171572723`*^9, {3.6502879596573734`*^9, 3.650287963266794*^9}, 
   3.6502880160794334`*^9, 3.650288046392021*^9, {3.6502880781733637`*^9, 
   3.650288107345296*^9}, {3.6508046017438784`*^9, 3.6508046021813507`*^9}, {
   3.65348164756075*^9, 3.6534816478805656`*^9}, {3.668862300539925*^9, 
   3.668862315860221*^9}, {3.702092503444625*^9, 
   3.702092542333111*^9}},ExpressionUUID->"e81a175e-31dd-43af-80d1-\
3d7259f62d3c"],

Cell["Inserting (2) in (3) and applying condition (3) we obtain", "Text",
 CellChangeTimes->{{3.6502881598454742`*^9, 3.650288200361134*^9}, {
  3.6502902568046722`*^9, 
  3.650290273257846*^9}},ExpressionUUID->"a0804bed-7332-4eeb-aea0-\
bf535c002abe"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[Del]", "^"], "k"], 
   RowBox[{
    SuperscriptBox["\[Eta]", "i"], "[", "y", "]"}]}], "=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "h"], 
      RowBox[{
       SuperscriptBox["\[Xi]", "j"], "[", "x", "]"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Xi]", "l"], "[", "x", "]"}], 
      RowBox[{
       SubsuperscriptBox["\[CapitalGamma]", "lh", "j"], "[", "x", "]"}]}]}], 
    ")"}], 
   RowBox[{
    SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
   RowBox[{
    SubsuperscriptBox["A", "j", "i"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650288229189413*^9, 3.6502882631270237`*^9}, 
   3.650288296095869*^9, {3.6502884107836866`*^9, 3.650288417377388*^9}, {
   3.6502884803619456`*^9, 3.6502884907369747`*^9}, 3.650288721987709*^9, {
   3.6508047435566907`*^9, 3.6508047439941435`*^9}, {3.66886233538758*^9, 
   3.668862340067066*^9}},ExpressionUUID->"080375cc-8425-494c-a78d-\
b132b6ff3522"],

Cell["Comparing the last two equations  we get", "Text",
 CellChangeTimes->{{3.6502888713474617`*^9, 
  3.650288941207113*^9}},ExpressionUUID->"ab973c84-8d35-4a3e-86ba-\
c498c0e0113c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], 
    RowBox[{
     SubsuperscriptBox["A", "j", "l"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}], " ", 
    RowBox[{
     SubsuperscriptBox[
      OverscriptBox["\[CapitalGamma]", "^"], "kl", "i"], "[", "y", "]"}]}], 
   " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Xi]", "l"], "[", "x", "]"}], 
       RowBox[{
        SubsuperscriptBox["\[CapitalGamma]", "hl", "j"], "[", "x", "]"}]}], 
      ")"}], 
     RowBox[{
      SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
     RowBox[{
      SubsuperscriptBox["A", "j", "i"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}]}], "-", "  ", 
    RowBox[{
     SuperscriptBox["\[Xi]", "j"], 
     RowBox[{
      RowBox[{
       RowBox[{"\[PartialD]", 
        SubsuperscriptBox["A", "j", "i"]}], "/", 
       RowBox[{"\[PartialD]", 
        SuperscriptBox["x", "h"]}]}], "\[CenterDot]", 
      RowBox[{
       SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}]}]}]}]}], ",", 
  "\[LineSeparator]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650288969535236*^9, 3.650288970925949*^9}, {
   3.650289025801108*^9, 3.650289044066786*^9}, {3.6502890965825624`*^9, 
   3.6502890976450663`*^9}, 3.650290598274412*^9, {3.650804778369255*^9, 
   3.65080479218178*^9}, {3.653481674240066*^9, 3.65348168746381*^9}, {
   3.6688623793389072`*^9, 3.6688624062586727`*^9}, {3.668862538401765*^9, 
   3.6688625384412823`*^9}},ExpressionUUID->"fa664a8b-97a2-429f-98ee-\
c53dcbd9d69a"],

Cell["\<\
Renaming the summation indices in the right hand  first term according  l \
\[RightArrow] j and j\[RightArrow]l it follows\
\>", "Text",
 CellChangeTimes->{{3.650289237067318*^9, 3.6502893084269266`*^9}, {
   3.6502893384582634`*^9, 3.65028942639596*^9}, 
   3.651895164064172*^9},ExpressionUUID->"3c8e3d10-11a6-4ab3-bdb2-\
a562136f4fbd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["\[Xi]", "j"], 
   RowBox[{
    SubsuperscriptBox["A", "j", "l"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}], " ", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox["\[CapitalGamma]", "^"], "kl", "i"], "[", "y", "]"}]}], 
  " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Xi]", "j"], "[", 
       RowBox[{"x", "[", "y", "]"}], "]"}], 
      RowBox[{
       SubsuperscriptBox["\[CapitalGamma]", "hj", "l"], "[", 
       RowBox[{"x", "[", "y", "]"}], "]"}]}], ")"}], 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["A", "l", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], "-", "  ", 
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], 
    RowBox[{
     RowBox[{
      RowBox[{"\[PartialD]", 
       SubsuperscriptBox["A", "j", "i"]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "h"]}]}], "\[CenterDot]", 
     RowBox[{
      SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650288969535236*^9, 3.650288970925949*^9}, {
   3.650289025801108*^9, 3.650289044066786*^9}, {3.6502890965825624`*^9, 
   3.6502890976450663`*^9}, {3.6502894805836773`*^9, 3.6502894989899206`*^9}, 
   3.650290612477578*^9, {3.6508048160099497`*^9, 3.6508048163380756`*^9}, {
   3.6508048880100336`*^9, 3.6508048883694086`*^9}, {3.6508084668760757`*^9, 
   3.650808473282337*^9}, {3.6534817230162077`*^9, 3.6534817379038267`*^9}, {
   3.668864012526265*^9, 3.668864016094255*^9}, {3.668864060629879*^9, 
   3.668864067797584*^9}},ExpressionUUID->"cfd192c6-31a1-4379-970d-\
191a0fde0869"],

Cell["\<\
This equation is valid for arbitrary vector fields. Thus we conclude\
\>", "Text",
 CellChangeTimes->{{3.6502896073652925`*^9, 3.6502896638810625`*^9}, {
  3.6502906449464226`*^9, 
  3.6502906805558615`*^9}},ExpressionUUID->"00287024-e101-49a5-b8b9-\
305bed30476c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", "j", "l"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}], " ", 
   RowBox[{
    SubsuperscriptBox[
     OverscriptBox["\[CapitalGamma]", "^"], "kl", "i"], "[", "y", "]"}]}], 
  " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[CapitalGamma]", "hj", "l"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}], ")"}], 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["A", "l", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], "-", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"\[PartialD]", 
      SubsuperscriptBox["A", "j", "i"]}], "/", 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["x", "h"]}]}], "\[CenterDot]", 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650288969535236*^9, 3.650288970925949*^9}, {
   3.650289025801108*^9, 3.650289044066786*^9}, {3.6502890965825624`*^9, 
   3.6502890976450663`*^9}, {3.6502894805836773`*^9, 
   3.6502894989899206`*^9}, {3.650289549146346*^9, 3.6502895870058265`*^9}, 
   3.6502907032278056`*^9, {3.6502911645104265`*^9, 3.650291168307313*^9}, {
   3.650291404401721*^9, 3.650291415651779*^9}, {3.6508049034163165`*^9, 
   3.6508049145882573`*^9}, 3.6508084938136253`*^9, {3.6534817598074617`*^9, 
   3.653481769103468*^9}},ExpressionUUID->"149285cb-f07d-4c1d-898b-\
7d87a5f51db1"],

Cell[TextData[{
 "Multiplication with ",
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}]],
  CellChangeTimes->{{3.650288969535236*^9, 3.650288970925949*^9}, {
    3.650289025801108*^9, 3.650289044066786*^9}, {3.6502890965825624`*^9, 
    3.6502890976450663`*^9}, {3.6502894805836773`*^9, 
    3.6502894989899206`*^9}, {3.650289549146346*^9, 3.6502895870058265`*^9}, 
    3.6502907032278056`*^9, {3.6502911645104265`*^9, 3.650291168307313*^9}},
  ExpressionUUID->"286e0e2d-cca3-453a-96ce-4202a698182f"],
 " and summing over j yields since ",
 Cell[BoxData[
  SubsuperscriptBox["B", "m", "j"]],
  CellChangeTimes->{{3.650288969535236*^9, 3.650288970925949*^9}, {
    3.650289025801108*^9, 3.650289044066786*^9}, {3.6502890965825624`*^9, 
    3.6502890976450663`*^9}, {3.6502894805836773`*^9, 
    3.6502894989899206`*^9}, {3.650289549146346*^9, 3.6502895870058265`*^9}, 
    3.6502907032278056`*^9, {3.6502911645104265`*^9, 3.650291168307313*^9}},
  ExpressionUUID->"9b145ae0-ed6c-45f0-9463-ea392f811285"],
 Cell[BoxData[
  SubsuperscriptBox["A", "j", "l"]],
  CellChangeTimes->{{3.650288969535236*^9, 3.650288970925949*^9}, {
    3.650289025801108*^9, 3.650289044066786*^9}, {3.6502890965825624`*^9, 
    3.6502890976450663`*^9}, {3.6502894805836773`*^9, 
    3.6502894989899206`*^9}, {3.650289549146346*^9, 3.6502895870058265`*^9}, 
    3.6502907032278056`*^9, {3.6502911645104265`*^9, 3.650291168307313*^9}},
  ExpressionUUID->"0b03d7fc-0573-423a-a187-30eb216a2aac"],
 " =",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Delta]", "m", "l"], TraditionalForm]],ExpressionUUID->
  "fcb9f8cb-4689-40e6-9260-4c9fb5417d39"],
 ":"
}], "Text",
 CellChangeTimes->{{3.6502910822132874`*^9, 3.6502911136665163`*^9}, {
  3.6502911937605114`*^9, 
  3.6502913324327765`*^9}},ExpressionUUID->"c83175d7-b4db-411d-a380-\
ec881454f0af"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox["\[CapitalGamma]", "^"], "km", "i"], "[", "y", "]"}], " ", 
  "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[CapitalGamma]", "hj", "l"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}], ")"}], 
    RowBox[{
     SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["A", "l", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], "-", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[PartialD]", 
       SubsuperscriptBox["A", "j", "i"]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "h"]}]}], "\[CenterDot]", 
     RowBox[{
      SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}]}], 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650291361042246*^9, 3.650291388432951*^9}, 
   3.650291449354987*^9, 3.65029148137072*^9, 3.650291514073863*^9, {
   3.6508050408228498`*^9, 3.65080511611986*^9}, 3.650808507173025*^9, {
   3.653481790079063*^9, 
   3.653481798382739*^9}},ExpressionUUID->"7a39b636-4cdc-4cfb-9832-\
e190bbd1a7a6"],

Cell[TextData[{
 "Now deriving the equation ",
 Cell[BoxData[
  SubsuperscriptBox["A", "j", "i"]],ExpressionUUID->
  "f50fed5a-7835-4baf-bd40-2f5831718c5c"],
 "[x[y]]\[CenterDot]",
 Cell[BoxData[
  SubsuperscriptBox["B", "m", "j"]],
  CellChangeTimes->{{3.650288969535236*^9, 3.650288970925949*^9}, {
    3.650289025801108*^9, 3.650289044066786*^9}, {3.6502890965825624`*^9, 
    3.6502890976450663`*^9}, {3.6502894805836773`*^9, 
    3.6502894989899206`*^9}, {3.650289549146346*^9, 3.6502895870058265`*^9}, 
    3.6502907032278056`*^9, {3.6502911645104265`*^9, 3.650291168307313*^9}},
  ExpressionUUID->"6b5b3bc0-7189-444a-ac3b-60d1babbf7cf"],
 "[y] =",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Delta]", "m", "i"], TraditionalForm]],ExpressionUUID->
  "248fed3b-29bb-4939-9b3f-cf70a4cf3276"],
 " with respect to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "k"], TraditionalForm]],ExpressionUUID->
  "26092019-fe94-4fc2-ba3e-9cc51e8b978a"],
 " we obtain"
}], "Text",
 CellChangeTimes->{{3.6508076885152617`*^9, 3.6508077674059534`*^9}, {
  3.6508078008435936`*^9, 3.6508079466094885`*^9}, {3.656138681093772*^9, 
  3.656138690413609*^9}},ExpressionUUID->"997e9733-cc78-48c0-b988-\
b721cb5755ff"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[PartialD]", 
       SubsuperscriptBox["A", "j", "i"]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["x", "h"]}]}], "\[CenterDot]", 
     RowBox[{
      SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}]}], 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["A", "j", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}], "\[CenterDot]", 
    RowBox[{
     RowBox[{"\[PartialD]", 
      RowBox[{
       SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}]}], "/", 
     RowBox[{"\[PartialD]", 
      SuperscriptBox["y", "k"]}]}]}]}], " ", "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650808019422123*^9, 3.6508080249377584`*^9}, {
  3.65080805850032*^9, 3.6508080935785103`*^9}, {3.6508081270316205`*^9, 
  3.650808210156812*^9}},ExpressionUUID->"e07ac9cd-b98b-497b-b1be-\
1787c5976d85"],

Cell["\<\
Combining the last two  equations we obtain the transformation rules for the \
\[CapitalGamma]-symbols:\
\>", "Text",
 CellChangeTimes->{{3.6508082788444777`*^9, 
  3.650808348235231*^9}},ExpressionUUID->"ea68ca00-82c0-4bfa-9ebb-\
4068e93a0c23"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox["\[CapitalGamma]", "^"], "km", "i"], "[", "y", "]"}], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[CapitalGamma]", "hj", "l"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}], ")"}], 
    RowBox[{
     SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["A", "l", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubsuperscriptBox["A", "j", "i"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}], "\[CenterDot]", 
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["y", "k"]}]}]}], "                                               ", 
    RowBox[{"(", "4", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6508083699383965`*^9, 3.650808398313449*^9}, {
  3.6508085498449306`*^9, 3.6508085950325623`*^9}, {3.6508619541445956`*^9, 
  3.6508619604414825`*^9}, {3.653481814782988*^9, 3.653481827974868*^9}, {
  3.7020936232673893`*^9, 
  3.702093625260817*^9}},ExpressionUUID->"7fdd72b2-ef1e-435d-864c-\
57e4624837da"],

Cell[TextData[{
 StyleBox["Definition.",
  FontColor->RGBColor[0, 0, 1]],
 " A",
 StyleBox[" linear connection",
  FontColor->RGBColor[0, 0, 1]],
 " \[CapitalGamma] on a smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "af6131f9-bd4e-4b19-9506-66f44e7549e8"],
 " is a field of in each chart defined  smooth \[CapitalGamma]-symbols ",
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["\[CapitalGamma]", "hj", "l"], "[", "x", "]"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6508083699383965`*^9, 3.650808398313449*^9}, {
   3.6508085498449306`*^9, 3.6508085950325623`*^9}, {3.6508619541445956`*^9, 
   3.6508619604414825`*^9}},
  FontWeight->"Bold",ExpressionUUID->"4dfdda47-800e-4ba0-99fd-37022031e8c2"],
 StyleBox[", ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox["\[CapitalGamma]", "^"], "km", "i"], "[", "y", "]"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6508083699383965`*^9, 3.650808398313449*^9}, {
   3.6508085498449306`*^9, 3.6508085950325623`*^9}, {3.6508619541445956`*^9, 
   3.6508619604414825`*^9}},
  FontWeight->"Bold",ExpressionUUID->"fd2149bc-355f-4b6c-8614-ca16a064436a"],
 StyleBox[",",
  FontWeight->"Bold"],
 "..., related by (4) under the coordinate transformations of the atlas."
}], "Text",
 CellChangeTimes->{{3.6508620458322515`*^9, 3.650862054504156*^9}, {
  3.6508620921917257`*^9, 3.650862101301118*^9}, {3.650862143691821*^9, 
  3.6508621967543793`*^9}, {3.650862255082652*^9, 3.6508622640982776`*^9}, {
  3.6508622982077312`*^9, 3.650862500239355*^9}, {3.6508625680675874`*^9, 
  3.6508625983176613`*^9}, {3.6508627000678496`*^9, 3.650862703224106*^9}, {
  3.653481847721437*^9, 3.653481877719576*^9}, {3.70305390101893*^9, 
  3.7030539108106813`*^9}},ExpressionUUID->"84a7b250-057a-467f-b70c-\
24dee335391c"],

Cell[TextData[{
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ".  If for any smooth vector field X \[Element] \[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "ce61047d-5e23-403c-bd65-d188eb1e0b20"],
 " the ansatz (2) defines a tensor field of type (1,1)  on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "a78e1ec8-2d0c-4248-9dab-a8f75388fcf1"],
 ", then  the \[CapitalGamma]-symbols",
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  RowBox[{
   SubsuperscriptBox["\[CapitalGamma]", "hj", "l"], "[", "x", "]"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6508083699383965`*^9, 3.650808398313449*^9}, {
   3.6508085498449306`*^9, 3.6508085950325623`*^9}, {3.6508619541445956`*^9, 
   3.6508619604414825`*^9}},
  FontWeight->"Bold",ExpressionUUID->"b239defc-a3b2-4ae8-984e-cd6b56411035"],
 " fulfill the transformation rule (4) and thus define a linear connection on \
 ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "71b456c5-f269-4127-b4c8-36d5a9d58ae4"],
 ". Conversely, for each linear connection \[CapitalGamma] the definition (2) \
yields the coordinates of a tensor field \[Del]X of type (1,1) on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "360aa5d3-bb94-4e2f-9255-c3fb48db8a3a"],
 " called the",
 StyleBox[" gradient ",
  FontColor->RGBColor[0, 0, 1]],
 "of the vector field X \[Element] \[ScriptCapitalX]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "e574ace1-b7fd-414f-8519-57f50017ad81"],
 "."
}], "Text",
 CellChangeTimes->{{3.6508628986463356`*^9, 3.650862910958817*^9}, {
   3.650863034318396*^9, 3.650863192428135*^9}, {3.650863247021986*^9, 
   3.650863371490966*^9}, {3.6508634886786766`*^9, 3.6508634968706093`*^9}, {
   3.6508635299964*^9, 3.650863576922141*^9}, {3.65086363187259*^9, 
   3.6508637064557495`*^9}, {3.650863741910887*^9, 3.650863872430234*^9}, 
   3.650864304586581*^9, {3.6508845809685464`*^9, 3.6508845831111407`*^9}, 
   3.6508850268214293`*^9, {3.651239722586672*^9, 3.651239746977396*^9}, 
   3.651895172560698*^9, {3.65348194562436*^9, 3.653481946767639*^9}, 
   3.656138785117594*^9},ExpressionUUID->"7e4c49a9-af77-4b75-be3f-\
aaa5ac0665d0"],

Cell[TextData[{
 StyleBox["Proof. ",
  FontColor->RGBColor[0, 0, 1]],
 " We proved the first statement already finding the transformation rule (4). \
The second, converse statement can be proved by a straightforward calculation \
checking the transformation rule (3).\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.650883849095535*^9, 3.6508838902984896`*^9}, {
  3.6508839389600925`*^9, 3.650884055708911*^9}, {3.6508842114475126`*^9, 
  3.650884269521167*^9}, {3.651895176927978*^9, 3.651895180455701*^9}, {
  3.702094662819478*^9, 
  3.702094700922695*^9}},ExpressionUUID->"3071c131-a739-426b-82d5-\
6b3a770e4ddb"],

Cell[TextData[{
 StyleBox["Corollary",
  FontColor->RGBColor[0, 0, 1]],
 ".  The difference of the  \[CapitalGamma]-symbols"
}], "Text",
 CellChangeTimes->{{3.6508845635422173`*^9, 3.6508845988514476`*^9}, {
  3.6508849562382126`*^9, 3.6508849807599554`*^9}, {3.650885036094679*^9, 
  3.650885110998538*^9}, {3.6508851431585693`*^9, 
  3.6508851610335755`*^9}},ExpressionUUID->"eed96b68-31c3-4fc3-a32b-\
153bae128be5"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["T", "jk", 
     RowBox[{"i", " "}]], "[", "x", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[CapitalGamma]", "jk", "i"], "[", "x", "]"}], " ", 
    "-", " ", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["\[CapitalGamma]", "kj", "i"], "[", "x", "]"}], "                                                                                                                   ", 
     RowBox[{"(", "5", ")"}]}]}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6508851665860004`*^9, 3.650885166587001*^9}, {
  3.6508852096913395`*^9, 3.6508852439183984`*^9}, {3.650885279743926*^9, 
  3.650885319598962*^9}, {3.6508853540688562`*^9, 3.6508853678763504`*^9}, {
  3.6508856937926416`*^9, 3.6508857088254986`*^9}, {3.6508902432143717`*^9, 
  3.650890246834598*^9}},ExpressionUUID->"5f36ba5e-475e-4207-ba38-\
b9c236258dd0"],

Cell[TextData[{
 "are the coordinates of an antisymmetric tensor field of type (2,1) on the \
smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "ab41044c-089a-4317-8880-c676523bbf47"],
 " called the",
 StyleBox[" torsion tensor",
  FontColor->RGBColor[0, 0, 1]],
 " of the linear connection \[CapitalGamma]. A linear connection whose \
torsion tensor is the null tensor is said to be ",
 StyleBox["symmetric",
  FontColor->RGBColor[0, 0, 1]],
 "."
}], "Text",
 CellChangeTimes->{{3.6508854364701157`*^9, 3.6508855199623213`*^9}, {
   3.6508855688414297`*^9, 3.6508856278317986`*^9}, {3.65094358287633*^9, 
   3.6509436756967354`*^9}, {3.651895184656061*^9, 3.651895191503785*^9}, 
   3.703054023081571*^9},ExpressionUUID->"190d1bfc-f4fa-4224-8b06-\
2cf0c618f2c0"],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". Applying (4)  we obtain the tensor transformation"
}], "Text",
 CellChangeTimes->{{3.6508893643663054`*^9, 3.6508893738062744`*^9}, {
   3.6508894206775727`*^9, 3.6508894343103223`*^9}, {3.6508909708592377`*^9, 
   3.6508909740634246`*^9}, {3.65089119837103*^9, 3.6508912175249257`*^9}, 
   3.65189564831408*^9, 
   3.6523385150951643`*^9},ExpressionUUID->"31700f5f-3444-4838-9582-\
90667c1358a3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox["T", "^"], "jk", 
    RowBox[{"i", " "}]], "[", "y", "]"}], "  ", "=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["T", "mh", "l"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}], ")"}], 
   RowBox[{
    SubsuperscriptBox["B", "j", "m"], "[", "y", "]"}], 
   RowBox[{
    SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
   RowBox[{
    SubsuperscriptBox["A", "l", "i"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6508894750899134`*^9, 3.650889505730524*^9}, {
  3.650889557877827*^9, 3.6508895802484703`*^9}, {3.6508896131586895`*^9, 
  3.650889643743579*^9}},ExpressionUUID->"092a7833-f092-493a-99ac-\
629cdfdeb678"],

Cell["\<\
since the second terms of (4) yield zero under alternation; remembering\
\>", "Text",
 CellChangeTimes->{{3.6508898752375803`*^9, 3.6508898772698092`*^9}, {
  3.65089038091861*^9, 3.650890417128664*^9}, {3.650890756014285*^9, 
  3.6508907642065706`*^9}},ExpressionUUID->"8aea8363-0b4a-4de3-9c0e-\
ae0ea73339dc"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[PartialD]", 
    RowBox[{
     SubsuperscriptBox["B", "j", "m"], "[", "y", "]"}]}], "/", 
   RowBox[{"\[PartialD]", 
    SuperscriptBox["y", "k"]}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["\[PartialD]", "2"], "x"}], "/", 
    RowBox[{"\[PartialD]", 
     SuperscriptBox["y", "j"]}]}], 
   RowBox[{"\[PartialD]", 
    SuperscriptBox["y", "k"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6508894750899134`*^9, 3.650889505730524*^9}, {
  3.650889557877827*^9, 3.6508895802484703`*^9}, {3.6508896131586895`*^9, 
  3.650889643743579*^9}, {3.6508908350618963`*^9, 
  3.6508909484146366`*^9}},ExpressionUUID->"130de56b-d651-4258-a910-\
d65e3b65402b"],

Cell["we get", "Text",
 CellChangeTimes->{{3.65089098697323*^9, 
  3.650890989006212*^9}},ExpressionUUID->"a3f56db3-fe09-4bfd-8c7e-\
9713543acc95"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", "m", "i"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}], "\[CenterDot]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SubsuperscriptBox["B", "j", "m"], "[", "y", "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["y", "k"]}]}], "-", 
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SubsuperscriptBox["B", "k", "m"], "[", "y", "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["y", "j"]}]}]}], ")"}]}], " ", "=", " ", 
  RowBox[{"0.", " ", "\[FilledSquare]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.650890561162547*^9, 3.6508906046393642`*^9}, {
   3.6508906411490116`*^9, 3.650890717165707*^9}, 3.6508907872229757`*^9, {
   3.65089099979882*^9, 
   3.65089101397416*^9}},ExpressionUUID->"58e00696-96c6-410e-b6b0-\
bc82cfadd921"],

Cell[TextData[{
 "We define a Mathematica Module calculating the",
 StyleBox[" torsion tensor field",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.658663406219472*^9, 3.6586634203139343`*^9}, {
  3.668864464625935*^9, 
  3.668864498937318*^9}},ExpressionUUID->"d43ccc33-05b0-427b-b04b-\
803447ff3a1b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tors", "[", "\[CapitalGamma]_", "]"}], "[", "x__", "]"}], " ", ":=",
   " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"di", " ", "=", " ", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], ",", "to", ",", "i", ",", "j", ",", 
      "k"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"to", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"i_", ",", "j_"}], "}"}], ",", 
        RowBox[{"{", "k_", "}"}]}], "]"}], ":=", 
      RowBox[{
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "x", "]"}], "-", 
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tensor", "[", 
      RowBox[{"to", ",", "2", ",", "1", ",", "di"}], "]"}]}]}], 
   "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6586634364918823`*^9, 3.658663521969358*^9}, {
  3.658663588999679*^9, 3.658663624831602*^9}, {3.658663672623329*^9, 
  3.658663686830893*^9}, {3.658663725902643*^9, 3.65866385558921*^9}, {
  3.658663897045178*^9, 
  3.6586641479149513`*^9}},ExpressionUUID->"72536174-ab52-4a5b-ba11-\
83f032a3cec1"],

Cell[BoxData[
 RowBox[{"?", "tors"}]], "Input",
 CellChangeTimes->{{3.658664532447226*^9, 3.6586645632710752`*^9}, {
  3.658664728525716*^9, 3.6586648108608017`*^9}, {3.6586648462446823`*^9, 
  3.658664911204693*^9}, {3.662965844215248*^9, 
  3.66296586214219*^9}},ExpressionUUID->"b8ab1559-3846-4092-a52b-\
630bc8738161"],

Cell[TextData[StyleBox["Example 1.",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6586649543044767`*^9, 3.65866497678789*^9}, {
  3.7020948417929*^9, 
  3.7020948443846407`*^9}},ExpressionUUID->"2b52293b-a902-4df1-8b08-\
1ec6c0d32af2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"tors", "[", "\[CapitalGamma]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658664158109108*^9, 
  3.658664218290416*^9}},ExpressionUUID->"67671b47-1e6e-4426-910e-\
77ddf0448edb"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.658664244745801*^9, 
  3.658664279041821*^9}},ExpressionUUID->"fe834f22-d396-4e5d-b31c-\
7d000b15457f"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.662965928146706*^9, 
  3.6629659498532543`*^9}},ExpressionUUID->"8a73a7e3-3ba6-4222-ab37-\
f64d11262636"],

Cell["\<\
Clearly, this tensor is antisymmetric in the covariant indices.\
\>", "Text",
 CellChangeTimes->{{3.658665030050578*^9, 3.6586650618982983`*^9}, {
  3.668864673664194*^9, 
  3.66886467562411*^9}},ExpressionUUID->"96309d5c-6969-4ce5-a786-\
0183a86b23ab"],

Cell["\<\
\[FilledSquare]
A preliminary definition of the coordinates of the gradient of a vector field \
v[x] with respect to a given linear connection \[CapitalGamma] is\
\>", "Text",
 CellChangeTimes->{{3.656142104618433*^9, 3.656142169106102*^9}, {
  3.656142589750458*^9, 3.656142624302177*^9}, {3.6586650764263268`*^9, 
  3.6586650865361757`*^9}},ExpressionUUID->"caeeac11-398f-4d26-a7d8-\
255aa4887b57"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"nab1", "[", 
     RowBox[{"\[CapitalGamma]_", ",", "v_"}], "]"}], "[", "x__", "]"}], "[", 
   RowBox[{"h_", ",", "i_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"v", "[", "x", "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], ",", 
      RowBox[{"{", "x", "}"}]}], "]"}], "[", 
    RowBox[{"[", "h", "]"}], "]"}], " ", "+", 
   RowBox[{"Sum", " ", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"v", "[", "x", "]"}], "[", 
       RowBox[{"[", "l", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"i", ",", "h", ",", "l"}], "]"}], "[", "x", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"l", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.656141444087872*^9, 3.656141888060903*^9}, {
   3.656141936965157*^9, 3.6561419435246477`*^9}, {3.6561419858133793`*^9, 
   3.6561420251958933`*^9}, {3.656144470111723*^9, 3.6561444980955143`*^9}, {
   3.6561451925539083`*^9, 3.6561451950737343`*^9}, 3.6629670561172*^9, 
   3.662970197938498*^9, 3.6629759574644012`*^9, 3.6633093562913237`*^9, 
   3.70209514711027*^9, 
   3.702095182718708*^9},ExpressionUUID->"41657add-6cd5-43e5-90a2-\
7bc0186b9a77"],

Cell[BoxData[
 RowBox[{"?", "nab1"}]], "Input",
 CellChangeTimes->{{3.663310187596397*^9, 
  3.663310216970352*^9}},ExpressionUUID->"c009698c-18c2-467c-8b3c-\
7c103707477e"],

Cell[BoxData[
 RowBox[{"?", "nabla1"}]], "Input",ExpressionUUID->"ecf158f5-3069-4035-93eb-93d89128a49a"],

Cell[BoxData[
 RowBox[{"Context", "[", "nabla1", "]"}]], "Input",
 CellChangeTimes->{{3.66886477227413*^9, 
  3.668864793463665*^9}},ExpressionUUID->"bd4bcfd8-a998-4272-8c65-\
8c089230db30"],

Cell[TextData[{
 StyleBox["Remark. ",
  FontColor->RGBColor[0, 0, 1]],
 " The \[OpenCurlyDoubleQuote]symbol\[CloseCurlyDoubleQuote] \[Del] can\
\[CloseCurlyQuote]t be used as the symbol name of a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function (try it!); thus we use the name nabla to replace the traditional \
symbol \[Del]\[DotEqual]nabla."
}], "Text",
 CellChangeTimes->{{3.656142181485908*^9, 3.656142448063558*^9}, {
  3.656142486393363*^9, 3.656142556431097*^9}, {3.662979103675293*^9, 
  3.66297916138691*^9}, {3.704106589898089*^9, 
  3.7041065908103113`*^9}},ExpressionUUID->"4e139ab4-df47-44ac-836b-\
2dd9efaa9ecd"],

Cell[TextData[{
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 " We consider the n-dimensional affine space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", "n"], TraditionalForm]],ExpressionUUID->
  "33267dc3-b782-458a-9656-49841e711169"],
 " with Cartesian coordinates x[i], \ni = 1,...,n. and set the \
\[CapitalGamma]-symbols to be zero:"
}], "Text",
 CellChangeTimes->{{3.6509431745353546`*^9, 3.6509431952541666`*^9}, {
   3.6509432269261*^9, 3.6509432660823793`*^9}, 3.650943328694499*^9, {
   3.6509433614445596`*^9, 3.650943447099682*^9}, {3.650943479255045*^9, 
   3.6509435644850597`*^9}, {3.650943735124157*^9, 3.650943880951023*^9}, {
   3.6509439494698386`*^9, 3.650943960844869*^9}, {3.650945511851761*^9, 
   3.650945515101718*^9}, 3.651895657394869*^9, {3.656138890869315*^9, 
   3.656138891836619*^9}, 3.656746618086245*^9, 3.65778071450778*^9, 
   3.657780815940465*^9, 3.657780856882757*^9, 
   3.702095365275875*^9},ExpressionUUID->"28c46e1e-0f6c-4df0-9b41-\
3dbc3656f944"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]0", "[", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], "[", "x__", "]"}], " ", ":=",
   " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.657780875730495*^9, 3.657780920338277*^9}, 
   3.657780952146105*^9, {3.658211608999818*^9, 3.6582116160079317`*^9}, {
   3.658211646238563*^9, 3.658211681134699*^9}, 
   3.662966242691299*^9},ExpressionUUID->"4abb2a9f-eb98-4c67-a5a9-\
8f640e879caf"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]0"}]], "Input",
 CellChangeTimes->{{3.657780985209839*^9, 3.657781156544183*^9}, {
  3.657781351408091*^9, 3.6577813645991707`*^9}, {3.658211627615748*^9, 
  3.658211635216454*^9}, {3.6629662908431807`*^9, 
  3.662966300722609*^9}},ExpressionUUID->"7b631b08-cdac-484f-b0ca-\
51823aa99b54"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalGamma]0", "[", 
   RowBox[{"1", ",", "2", ",", "3"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.669032996382675*^9, 
  3.66903301539006*^9}},ExpressionUUID->"d2cec096-d8aa-41c0-ac36-\
1ab31314c28e"],

Cell[TextData[{
 "From (4) it follows that the \[CapitalGamma]-symbols in any Cartesian \
coordinate system are zero, since the matrices ",
 Cell[BoxData[
  SubsuperscriptBox["A", "l", "i"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6508894750899134`*^9, 3.650889505730524*^9}, {
   3.650889557877827*^9, 3.6508895802484703`*^9}, {3.6508896131586895`*^9, 
   3.650889643743579*^9}},
  FontWeight->"Bold",ExpressionUUID->"e802abe2-bb1c-491b-bab6-7df7d06fbd47"],
 StyleBox[", ",
  FontWeight->"Bold"],
 Cell[BoxData[
  SubsuperscriptBox["B", "k", "h"]],
  Evaluatable->False,
  CellChangeTimes->{{3.6508894750899134`*^9, 3.650889505730524*^9}, {
   3.650889557877827*^9, 3.6508895802484703`*^9}, {3.6508896131586895`*^9, 
   3.650889643743579*^9}},
  FontWeight->"Bold",ExpressionUUID->"8c07f28f-1a51-4ac4-89d1-dec35545d972"],
 StyleBox[" ",
  FontWeight->"Bold"],
 "are constant.   it follows"
}], "Text",
 CellChangeTimes->{{3.6509431745353546`*^9, 3.6509431952541666`*^9}, {
   3.6509432269261*^9, 3.6509432660823793`*^9}, 3.650943328694499*^9, {
   3.6509433614445596`*^9, 3.650943447099682*^9}, {3.650943479255045*^9, 
   3.6509435644850597`*^9}, {3.650943735124157*^9, 3.650943880951023*^9}, {
   3.6509439494698386`*^9, 3.650943960844869*^9}, {3.650945511851761*^9, 
   3.650945515101718*^9}, 3.651895657394869*^9, {3.656138890869315*^9, 
   3.656138891836619*^9}, 3.656746618086245*^9, 3.65778071450778*^9, 
   3.657780815940465*^9},ExpressionUUID->"a6cf954f-1210-4888-b6b8-\
88876c3d3b0e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "h"], 
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], "[", "x", "]"}]}], " ", "\[Equal]", "  ", 
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{
    SuperscriptBox["\[Xi]", "j"], "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6509439766417484`*^9, {3.6577821435769463`*^9, 
   3.657782155928557*^9}},ExpressionUUID->"8ac76c4f-c0f6-453e-a5ce-\
cd5ac8418007"],

Cell["\<\
the  derivatives with respect to this linear connection coincide with the \
usual partial derivatives with respect to the Cartesian coordinates, and the \
gradient of the vector field is the usual gradient:\
\>", "Text",
 CellChangeTimes->{{3.650944183417856*^9, 
  3.65094432957299*^9}},ExpressionUUID->"b30ba1be-f0fc-481e-96fc-\
46636bdec8a3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.650944371150196*^9, 
  3.65094441319491*^9}},ExpressionUUID->"1c0060f1-13fb-4cb0-8119-\
ead231561fb4"],

Cell[BoxData[
 RowBox[{"?", "Grad"}]], "Input",
 CellChangeTimes->{{3.6509444328476734`*^9, 3.650944435035206*^9}, {
  3.6688649544938583`*^9, 
  3.668864960911202*^9}},ExpressionUUID->"f7a5a2b4-f0e4-4554-a5b6-\
dce5c0107982"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"Grad", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65094452929959*^9, 3.65094456255789*^9}, {
   3.650944607131057*^9, 3.650944616974812*^9}, 3.650944661712579*^9, {
   3.650944693902235*^9, 3.650944700497114*^9}, {3.6633105912626762`*^9, 
   3.663310597726074*^9}, 
   3.702095508884597*^9},ExpressionUUID->"bf6c0cf5-f3f6-4592-a590-\
0c79077ea14c"],

Cell[BoxData[
 RowBox[{"Transpose", "[", 
  RowBox[{"tensorToArray", "[", 
   RowBox[{
    RowBox[{"nabla1", "[", 
     RowBox[{"\[CapitalGamma]0", ",", 
      RowBox[{"vv", "[", "\[Xi]", "]"}]}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6633104225381308`*^9, 3.663310542414618*^9}, {
   3.6633106577259817`*^9, 3.663310663373283*^9}, {3.663310804189097*^9, 
   3.663310824964446*^9}, 
   3.702095548330768*^9},ExpressionUUID->"91157414-e6f1-4135-b49e-\
bb610d5423f1"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "\[Equal]", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.6633106791728477`*^9, 
  3.663310709357252*^9}},ExpressionUUID->"eaa27ba4-804d-44a1-9ec0-\
b13b4ebac838"],

Cell[TextData[{
 "Clearly, the considered linear connection is symmetric; it is called the ",
 StyleBox["flat linear connection",
  FontColor->RGBColor[0, 0, 1]],
 ". \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6509454124449472`*^9, 
  3.6509454752880535`*^9}},ExpressionUUID->"4085e9aa-31f7-4c0c-a4d1-\
152fa7ded096"],

Cell[TextData[{
 StyleBox["Example 3.",
  FontColor->RGBColor[0, 0, 1]],
 " The \[CapitalGamma]-symbols of the flat affine plane in polar coordinates."
}], "Text",
 CellChangeTimes->{{3.6509797409711494`*^9, 3.650979829440081*^9}, {
  3.702095573850439*^9, 
  3.702095574858165*^9}},ExpressionUUID->"ca23dd82-e654-4a45-abca-\
fc4bde89b21c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "2"}], ";"}]], "Input",
 CellChangeTimes->{{3.650946133320168*^9, 
  3.6509461402470446`*^9}},ExpressionUUID->"9f273585-0100-4389-b631-\
f20505fd8178"],

Cell["We use the built - in function", "Text",
 CellChangeTimes->{{3.6561399736215277`*^9, 3.656140009837213*^9}, 
   3.702095597993949*^9},ExpressionUUID->"61134587-8c4d-4fbb-a468-\
0cf42e91238d"],

Cell[BoxData[
 RowBox[{"?", "CoordinateTransformData"}]], "Input",
 CellChangeTimes->{{3.656140021654957*^9, 
  3.65614003739673*^9}},ExpressionUUID->"4692dc75-3aae-4d41-b2c5-\
861011a3ca1d"],

Cell[BoxData[
 RowBox[{"CoordinateTransformData", "[", 
  RowBox[{
   RowBox[{"\"\<Polar\>\"", "\[Rule]", "\"\<Cartesian\>\""}], ",", " ", 
   "\"\<Mapping\>\"", ",", " ", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Theta]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.650946322459323*^9, 
  3.6509463496166973`*^9}},ExpressionUUID->"0df89fe7-1bdb-477e-bc8e-\
7fd5d2e89d3c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"polar", "[", 
   RowBox[{"x1_", ",", "x2_"}], "]"}], ":=", 
  RowBox[{"CoordinateTransformData", "[", 
   RowBox[{
    RowBox[{"\"\<Cartesian\>\"", " ", "\[Rule]", "\"\<Polar\>\""}], ",", " ", 
    "\"\<Mapping\>\"", ",", " ", 
    RowBox[{"{", 
     RowBox[{"x1", ",", "x2"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.650945836544608*^9, 3.65094592114433*^9}, {
  3.650945995516756*^9, 3.650946027082279*^9}, {3.6509461550283537`*^9, 
  3.650946162965885*^9}, {3.650976459365803*^9, 
  3.6509764697720723`*^9}},ExpressionUUID->"be92a03b-2804-4cf8-b93b-\
9f80d70b42aa"],

Cell[BoxData[
 RowBox[{"polar", "[", 
  RowBox[{"x1", ",", "x2"}], "]"}]], "Input",
 CellChangeTimes->{{3.6509460339416704`*^9, 
  3.650946051612855*^9}},ExpressionUUID->"ff4f20aa-8b0f-48a7-82dc-\
bfb903bb429c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"A", "[", 
   RowBox[{"x1_", ",", "x2_"}], "]"}], " ", ":=", " ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"polar", "[", 
     RowBox[{"x1", ",", "x2"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"x1", ",", "x2"}], "}"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6509471746867003`*^9, 
  3.6509472388361235`*^9}},ExpressionUUID->"e9f4c5fa-de14-4ae2-8c85-\
ad35c717838b"],

Cell[BoxData[
 RowBox[{"A", "[", 
  RowBox[{"x1", ",", "x2"}], "]"}]], "Input",
 CellChangeTimes->{{3.6509472475548716`*^9, 
  3.6509472569298515`*^9}},ExpressionUUID->"c97138df-378e-493a-acb8-\
cce6c64f972c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"invpolar", "[", 
   RowBox[{"r_", ",", "\[Theta]_"}], "]"}], " ", ":=", " ", 
  RowBox[{"CoordinateTransformData", "[", 
   RowBox[{
    RowBox[{"\"\<Polar\>\"", "\[Rule]", "\"\<Cartesian\>\""}], ",", " ", 
    "\"\<Mapping\>\"", ",", " ", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Theta]"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.650946285484874*^9, 3.650946305396841*^9}, {
  3.650946360772966*^9, 
  3.6509464086093407`*^9}},ExpressionUUID->"fabea072-344c-4cfc-b4f0-\
731d2b009550"],

Cell[BoxData[
 RowBox[{"invpolar", "[", 
  RowBox[{"r", ",", "\[Theta]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6509464171874695`*^9, 
  3.650946442053959*^9}},ExpressionUUID->"bf8c33d2-2441-4155-8c79-\
9554203dbd6d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"B", "[", 
   RowBox[{"r_", ",", "\[Theta]_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Transpose", "[", 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"invpolar", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"r", ",", "\[Theta]"}], "}"}], "}"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6509467863536468`*^9, 3.650946893483156*^9}, {
   3.650946997085986*^9, 3.6509470810881453`*^9}, {3.6509777053211513`*^9, 
   3.650977725352519*^9}, 
   3.650978418119368*^9},ExpressionUUID->"83d9afcc-ee3c-4c7b-af21-\
739f691f2654"],

Cell[BoxData[
 RowBox[{"B", "[", 
  RowBox[{"r", ",", "\[Theta]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6509470924787083`*^9, 
  3.6509471045779276`*^9}},ExpressionUUID->"5316be20-20e3-49ac-a47d-\
a03a3b7b78e9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", 
   RowBox[{"r_", ",", "\[Theta]_"}], "]"}], ":=", 
  RowBox[{"Transpose", "[", " ", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"A", "[", 
        RowBox[{"x1", ",", "x2"}], "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"x1", "\[Rule]", 
          RowBox[{"r", " ", 
           RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], ",", 
         RowBox[{"x2", "\[Rule]", 
          RowBox[{"r", " ", 
           RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}]}], "}"}]}], " ", ")"}], 
     ",", 
     RowBox[{"r", ">", "0"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.650948403253334*^9, 3.6509484428057914`*^9}, {
  3.650948538498659*^9, 3.650948589940057*^9}, {3.6509776575398936`*^9, 
  3.6509776718836412`*^9}},ExpressionUUID->"0d8cb6b7-c821-4052-aa09-\
613dff1203ab"],

Cell[BoxData[
 RowBox[{"a", "[", 
  RowBox[{"r", ",", "\[Theta]"}], "]"}]], "Input",
 CellChangeTimes->{{3.650948545936147*^9, 
  3.6509485560611916`*^9}},ExpressionUUID->"e83d2f26-d15b-4bff-b274-\
de7dcd5bbbcf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", 
   RowBox[{"r", ",", "\[Theta]"}], "]"}], ".", 
  RowBox[{"B", "[", 
   RowBox[{"r", ",", "\[Theta]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.650947349909602*^9, 3.6509474988356533`*^9}, 
   3.650947575277353*^9, {3.650947615572235*^9, 3.6509476199628687`*^9}, {
   3.6509479150169296`*^9, 3.65094791793881*^9}, 3.650947948081416*^9, 
   3.6509785146563845`*^9},ExpressionUUID->"87a901cf-b840-4a6f-b921-\
2bcf215868f1"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6509479639408007`*^9, 3.6509479990166883`*^9}, 
   3.7020957979845953`*^9},ExpressionUUID->"da44ab6a-a55f-448c-bead-\
eeb3e6b2f38c"],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox[
   OverscriptBox["\[CapitalGamma]", "^"], "mk", "i"], "=", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", "j", "i"], "[", 
    RowBox[{"x", "[", "y", "]"}], "]"}], "\[CenterDot]", 
   RowBox[{
    RowBox[{"\[PartialD]", 
     RowBox[{
      SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}]}], "/", 
    RowBox[{"\[PartialD]", 
     SuperscriptBox["y", "k"]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6509492252122746`*^9, 3.6509492272435284`*^9}, 
   3.662966532016618*^9, {3.668865706135872*^9, 
   3.6688657207439127`*^9}},ExpressionUUID->"9b11da8a-ecb9-4280-9abc-\
c1726ea420a9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"1", ",", "1", ",", "1"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "1"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", "r"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}], ",", "r"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.656140432769219*^9, 
   3.656140437208552*^9}},ExpressionUUID->"42c40674-6935-47d5-8057-\
2332ba3b57f1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"1", ",", "2", ",", "2"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "1"}], "]"}], "]"}], "*", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"B", "[", 
         RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", "\[Theta]"}], "]"}]}], 
    "+", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}], "*", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"B", "[", 
         RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"2", ",", "2"}], "]"}], "]"}], ",", "\[Theta]"}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.650979592908348*^9, 
   3.6509796810022774`*^9}, {3.6561404517049932`*^9, 3.656140458576416*^9}, {
   3.6561524305250053`*^9, 
   3.656152472244061*^9}},ExpressionUUID->"c57d3842-8295-4b21-81d0-\
aad896fece73"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"1", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "1"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", "\[Theta]"}], "]"}]}], 
   "+", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}], ",", "\[Theta]"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.650949149202374*^9, 
   3.650949161601396*^9}, {3.650949258665405*^9, 3.650949280205201*^9}, {
   3.650949572100582*^9, 3.650949581499173*^9}, {3.65614048196863*^9, 
   3.656140506855751*^9}},ExpressionUUID->"4239b410-c4f9-4703-a384-\
a02990eea94e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"1", ",", "2", ",", "1"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "1"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", "r"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "2"}], "]"}], "]"}], ",", "r"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.650949149202374*^9, 
   3.650949161601396*^9}, {3.650949258665405*^9, 3.650949322318084*^9}, {
   3.650949386825611*^9, 3.6509493905912433`*^9}, 3.650949422312962*^9, {
   3.6509494772405214`*^9, 3.650949480959298*^9}, {3.6509495929367313`*^9, 
   3.6509496124523726`*^9}, 3.6509759565211234`*^9, {3.6509760085680947`*^9, 
   3.6509760181618347`*^9}, 3.650976154521443*^9, {3.6509769748823614`*^9, 
   3.6509769805385914`*^9}, {3.650977030163767*^9, 3.650977050069973*^9}, {
   3.6561405416076097`*^9, 
   3.656140546255451*^9}},ExpressionUUID->"20733ffc-fc21-43b0-8bf0-\
d3d478a1c01b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"2", ",", "1", ",", "1"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", "r"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "2"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "2"}], "]"}], "]"}], ",", "r"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.6509496255021553`*^9, 
   3.650949632377153*^9}, 3.65094966317143*^9, {3.6509786572660017`*^9, 
   3.6509786913598022`*^9}, {3.6561405623757687`*^9, 
   3.656140566967547*^9}},ExpressionUUID->"c3bd6aae-1d8a-4425-9ead-\
8398c77cdb67"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"2", ",", "2", ",", "2"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "2"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", "\[Theta]"}], "]"}]}], 
   "+", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "2"}], "]"}], "]"}], "*", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"B", "[", 
        RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "2"}], "]"}], "]"}], ",", "\[Theta]"}], 
     "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.6509496255021553`*^9, 
   3.650949632377153*^9}, 3.65094966317143*^9, {3.6509786572660017`*^9, 
   3.6509786913598022`*^9}, 3.650979450658131*^9, {3.650979488970702*^9, 
   3.650979554205165*^9}, {3.656140584287601*^9, 
   3.656140587727557*^9}},ExpressionUUID->"f94901c3-2955-4562-be01-\
bce15423725a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"2", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}], "*", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"B", "[", 
         RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", "\[Theta]"}], "]"}]}], 
    "+", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "2"}], "]"}], "]"}], "*", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"B", "[", 
         RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "2"}], "]"}], "]"}], ",", "\[Theta]"}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.650949149202374*^9, 
   3.650949161601396*^9}, {3.650949258665405*^9, 3.650949280205201*^9}, {
   3.6509787887193966`*^9, 3.650978802735058*^9}, {3.656140607095511*^9, 
   3.656140610934696*^9}, {3.65615249030867*^9, 
   3.65615250346782*^9}},ExpressionUUID->"48f290e1-f9e9-4bcd-86ff-\
036e2b26e638"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]polar", "[", 
    RowBox[{"2", ",", "2", ",", "1"}], "]"}], "[", 
   RowBox[{"r_", ",", " ", "\[Theta]_"}], "]"}], " ", "=", " ", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", "2"}], "]"}], "]"}], "*", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"B", "[", 
         RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"2", ",", "1"}], "]"}], "]"}], ",", "r"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "2"}], "]"}], "]"}], "*", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"B", "[", 
         RowBox[{"r", ",", "\[Theta]"}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"2", ",", "2"}], "]"}], "]"}], ",", "r"}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6509482867907934`*^9, 3.650948343496112*^9}, {
   3.650948471494114*^9, 3.6509484971312895`*^9}, 3.6509485315923834`*^9, {
   3.6509486163773317`*^9, 3.6509486539659977`*^9}, {3.6509487046154633`*^9, 
   3.6509489033608694`*^9}, {3.65094893410265*^9, 3.650948999168452*^9}, {
   3.650949097297903*^9, 3.650949103766713*^9}, {3.650949149202374*^9, 
   3.650949161601396*^9}, {3.650949258665405*^9, 3.650949280205201*^9}, {
   3.6509787887193966`*^9, 3.650978802735058*^9}, {3.6509788591100845`*^9, 
   3.65097888514141*^9}, {3.6509789182196465`*^9, 3.65097896762595*^9}, {
   3.65097903387609*^9, 3.6509791072512465`*^9}, 3.6509791744232454`*^9, 
   3.65097928684533*^9, {3.656140626367366*^9, 3.6561406409026213`*^9}, {
   3.656152532651697*^9, 
   3.656152554019725*^9}},ExpressionUUID->"13fce8bb-490c-4e58-ac77-\
e80dabf73150"],

Cell["\<\
We verify that the \[CapitalGamma]-symbols  calculated by the transformation \
rule coincide with the Christoffel-symbols  of the Euclidean metric in polar \
coordinates. The polar coordinates of the Euclidean metric are\
\>", "Text",
 CellChangeTimes->{{3.651040145506809*^9, 3.651040206882845*^9}, {
  3.6510419447369328`*^9, 3.6510419509087496`*^9}, {3.6510420125967503`*^9, 
  3.651042064424953*^9}, {3.651042096425001*^9, 3.6510422327846584`*^9}, {
  3.651895681010137*^9, 
  3.651895685738008*^9}},ExpressionUUID->"2699d75f-fe2b-4dbf-a150-\
660f961d4d0f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "gp", "]"}], ";", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"gp", "[", 
     RowBox[{"r_", ",", "\[Theta]_"}], "]"}], "[", 
    RowBox[{"1", ",", "1"}], "]"}], " ", ":=", "1"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"gp", "[", 
     RowBox[{"r_", ",", "\[Theta]_"}], "]"}], "[", 
    RowBox[{"1", ",", "2"}], "]"}], " ", ":=", "0"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"gp", "[", 
     RowBox[{"r_", ",", "\[Theta]_"}], "]"}], "[", 
    RowBox[{"2", ",", "1"}], "]"}], " ", ":=", "0"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"gp", "[", 
     RowBox[{"r_", ",", "\[Theta]_"}], "]"}], "[", 
    RowBox[{"2", ",", "2"}], "]"}], " ", ":=", 
   RowBox[{"r", "^", "2"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6510354950751357`*^9, 3.651035685973342*^9}, {
  3.6510357198327904`*^9, 3.6510358422861843`*^9}, {3.6510399574596415`*^9, 
  3.651039996459714*^9}, {3.703134241159252*^9, 
  3.70313435222136*^9}},ExpressionUUID->"7b415449-55f1-4e11-bf51-\
665957c59840"],

Cell["\<\
The Christoffel-symbols of the second kind are defined in riemannv2.m:\
\>", "Text",
 CellChangeTimes->{{3.6510422507533913`*^9, 3.651042290534765*^9}, {
  3.651895738135242*^9, 3.651895740094953*^9}, {3.703054381502207*^9, 
  3.7030543919659433`*^9}},ExpressionUUID->"388174cc-9e63-4c03-983a-\
f0897fc9d053"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]g"}]], "Input",
 CellChangeTimes->{{3.651042295128499*^9, 3.651042306737837*^9}, 
   3.6544349965811996`*^9, 3.65614075147829*^9, 
   3.70313502096154*^9},ExpressionUUID->"ab0408c4-4c61-408c-b878-\
c4d23131e1ad"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]g", "[", "gp", "]"}], "[", 
   RowBox[{"1", ",", "2", ",", "2"}], "]"}], "[", 
  RowBox[{"r", ",", "\[Theta]"}], "]"}]], "Input",
 CellChangeTimes->{{3.703133838425894*^9, 3.703133841329154*^9}, {
  3.7031340284331293`*^9, 3.7031340636004333`*^9}, {3.708423949018736*^9, 
  3.708423980417335*^9}},ExpressionUUID->"7c3cc434-77b2-471f-b9dd-\
4b281b1a4207"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]polar", "[", 
       RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
      RowBox[{"r", ",", " ", "\[Theta]"}], "]"}], " ", "-", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]g", "[", "gp", "]"}], "[", 
       RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6510416276108627`*^9, 3.651041698095393*^9}, {
  3.6510417365642138`*^9, 3.6510417717361546`*^9}, {3.6510418254083414`*^9, 
  3.6510418921272645`*^9}, {3.6561406684467916`*^9, 3.656140673790509*^9}, {
  3.703134152943494*^9, 
  3.70313418654386*^9}},ExpressionUUID->"4f21547f-82f3-46c3-a277-\
222f3d0b082e"],

Cell[" OK. \[FilledSquare]", "Text",
 CellChangeTimes->{
  3.656142676486669*^9},ExpressionUUID->"023427e8-9cb6-4e72-ba6d-\
2537157b182b"],

Cell[TextData[StyleBox["Example 4 The built-in operator Grad",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.6512156374942904`*^9, 3.6512156582912536`*^9}, {
   3.651216373464451*^9, 3.65121638371447*^9}, {3.6688658817101183`*^9, 
   3.66886588588591*^9}, {3.669036031324463*^9, 3.669036050180366*^9}, 
   3.7020961500857573`*^9, {3.708424431393497*^9, 
   3.708424449890648*^9}},ExpressionUUID->"0fd1dc46-c146-4d71-93d5-\
51f40d3c0d62"],

Cell[BoxData[
 RowBox[{"?", "Grad"}]], "Input",
 CellChangeTimes->{{3.6510332648147807`*^9, 
  3.651033274846032*^9}},ExpressionUUID->"c92663d9-fb00-454e-b6f3-\
49f1fcb7905e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CoordinateChartData", "[", "]"}], "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.651033500481738*^9, 
  3.651033573528761*^9}},ExpressionUUID->"fb3fb0dd-e6e7-4086-b34a-\
50800dd3bb81"],

Cell[BoxData[
 RowBox[{
  RowBox[{"v\[Xi]", "[", 
   RowBox[{"r_", ",", "\[Theta]_"}], "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"v1", "[", 
     RowBox[{"r", ",", "\[Theta]"}], "]"}], ",", 
    RowBox[{"v2", "[", 
     RowBox[{"r", ",", "\[Theta]"}], "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.708424290375246*^9, 3.708424318252336*^9}, {
  3.708424352901677*^9, 3.708424360988894*^9}, {3.708424460050469*^9, 
  3.708424516093354*^9}},ExpressionUUID->"ece9c4af-b548-49e1-8039-\
a12c0fc365bf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "2"}], ";", 
  RowBox[{"Gv\[Xi]", "=", 
   RowBox[{"Grad", "[", 
    RowBox[{
     RowBox[{"v\[Xi]", "[", 
      RowBox[{"r", ",", "\[Theta]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "\[Theta]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<Polar\>\"", ",", "\"\<Euclidean\>\"", ",", "2"}], "}"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.651033809884302*^9, 3.6510338167124434`*^9}, {
  3.6510338634156756`*^9, 3.6510339268376317`*^9}, {3.65103395993143*^9, 
  3.6510339648689175`*^9}, {3.6688659850794992`*^9, 3.668866004501855*^9}, {
  3.703137820651585*^9, 
  3.7031378527778*^9}},ExpressionUUID->"e4688d89-3a00-4a3a-94cf-5bfc3828bc80"],

Cell["This doesn't coincide with the absolute differential nabla1:", "Text",
 CellChangeTimes->{{3.6512161775733833`*^9, 3.651216259073557*^9}, 
   3.651895851253661*^9, {3.70209623906885*^9, 3.702096240084684*^9}, {
   3.703135967448174*^9, 
   3.703135970823985*^9}},ExpressionUUID->"57b334b3-a9cc-451e-ab4f-\
623ec5a98479"],

Cell[BoxData[
 RowBox[{"?", "nabla1"}]], "Input",
 CellChangeTimes->{{3.7031356798266287`*^9, 
  3.703135685426097*^9}},ExpressionUUID->"95671b88-6d21-40a5-b830-\
3ba03b840b6e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla1", "[", 
   RowBox[{"\[CapitalGamma]polar", ",", "v\[Xi]"}], "]"}], "[", 
  RowBox[{"r", ",", " ", "\[Theta]"}], "]"}]], "Input",
 CellChangeTimes->{{3.703135709177883*^9, 3.703135717142551*^9}, {
  3.703135778171576*^9, 3.7031357983137007`*^9}, {3.7031358354100313`*^9, 
  3.703135854409614*^9}},ExpressionUUID->"ee8daf7f-84ba-493e-8a35-\
d71fdfbcb4e9"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.703135878842827*^9, 
  3.7031358869849052`*^9}},ExpressionUUID->"81bc86b3-ce8f-4c28-9587-\
cdbdd5f0cdfb"],

Cell[BoxData[""], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.70313598397641*^9},ExpressionUUID->"1589ecd1-93e0-4212-ae69-7baf2715abba"],

Cell["The reason will be explained in the next example.   \[FilledSquare]", \
"Text",
 CellChangeTimes->{{3.651237915489616*^9, 3.651237940145918*^9}, {
  3.6512394777580657`*^9, 
  3.6512394933674946`*^9}},ExpressionUUID->"f2c3b787-1c3f-47df-99c8-\
a1e5531929ba"],

Cell[TextData[{
 StyleBox["Example 5",
  FontColor->RGBColor[0, 0, 1]],
 ". A parallel field"
}], "Text",
 CellChangeTimes->{{3.651216395917596*^9, 3.651216447605213*^9}, 
   3.6512165104803295`*^9, 
   3.702096323796397*^9},ExpressionUUID->"f9136cab-f083-494a-b8a9-\
6c8bea74555c"],

Cell[TextData[{
 "A vector field  in the affine space is said to be ",
 StyleBox["parallel",
  FontColor->RGBColor[0, 0, 1]],
 " iff its Cartesian coordinates are constant; e.g. in the plane"
}], "Text",
 CellChangeTimes->{{3.651235248140879*^9, 3.651235307000436*^9}, {
   3.651235348578638*^9, 3.651235450063201*^9}, 
   3.702096370796185*^9},ExpressionUUID->"80ad1153-59a7-4a59-9cc2-\
59e09f178d12"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vparallel", "[", 
   RowBox[{"x_", ",", " ", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{"a", ",", "b"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6512354823757353`*^9, 
  3.6512354823757353`*^9}},ExpressionUUID->"f094e4fb-fd64-4882-9f65-\
f08303ab991a"],

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"vparallel", "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6512354931569777`*^9, 
  3.6512355668602915`*^9}},ExpressionUUID->"8b68f03f-611c-4383-97f2-\
0813ee2bc91b"],

Cell["\<\
Clearly, an arbitrary vector field in the affine space is parallel iff its \
gradient with respect to the flat linear connection vanishes, since the \
absolute differential coincides with the usual differential in Cartesian \
coordinates, remember example 2.\
\>", "Text",
 CellChangeTimes->{{3.6512356306728897`*^9, 3.6512356803292513`*^9}, {
   3.6512357289543414`*^9, 3.6512358004232235`*^9}, {3.651235870610839*^9, 
   3.6512358816733737`*^9}, 3.6518959130074797`*^9, {3.656153353500808*^9, 
   3.656153356156662*^9}, 
   3.70209642461985*^9},ExpressionUUID->"e95e79e5-35e4-4700-b5ce-\
55fe80c1d8a5"],

Cell["\<\
The polar coordinates of a parallel vector field  are not constant; by the \
transformation law of vector coordinates they are\
\>", "Text",
 CellChangeTimes->{{3.6512360192829533`*^9, 
  3.6512361148456306`*^9}},ExpressionUUID->"22b5139d-726d-4367-9489-\
904d89309cf4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv1", "[", 
   RowBox[{"r_", ",", "\[Theta]_"}], "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}], ".", 
   RowBox[{"a", "[", 
    RowBox[{"r", ",", "\[Theta]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6512165172771916`*^9, 3.651216581246073*^9}, {
  3.651216652527467*^9, 3.65121669691817*^9}, {3.651229283799775*^9, 
  3.6512292975810504`*^9}, {3.651229861113324*^9, 
  3.651229864160205*^9}},ExpressionUUID->"ffee442f-beaf-4631-9ec0-\
87853fdc6253"],

Cell[TextData[{
 "Since the absolute differential is represented by the gradient tensor {",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Del]", "h"], TraditionalForm]],ExpressionUUID->
  "da11f03a-ae69-416e-9c64-8e50d56fc2d5"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["vv1", "i"], TraditionalForm]],ExpressionUUID->
  "8d8730f7-e57c-4c4f-8f24-151aa8dfee72"],
 "} the calculation in polar coordinates must return the null tensor again, \
what is confirmed here:"
}], "Text",
 CellChangeTimes->{{3.6512363993618155`*^9, 3.651236418299366*^9}, {
   3.651236452346304*^9, 3.651236590393385*^9}, {3.6512366381591473`*^9, 
   3.65123671634678*^9}, 3.65189591779947*^9, 
   3.703136531528211*^9},ExpressionUUID->"6810e564-e63d-4646-b601-\
b19889f66b0f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla1", "[", 
   RowBox[{"\[CapitalGamma]polar", ",", "vv1"}], "]"}], "[", 
  RowBox[{"r", ",", "\[Theta]"}], "]"}]], "Input",
 CellChangeTimes->{{3.651229163565144*^9, 3.651229163877681*^9}, {
  3.651229215502774*^9, 3.651229242018448*^9}, {3.7031365585319242`*^9, 
  3.703136561811872*^9}, {3.7031366030204906`*^9, 
  3.703136650507123*^9}},ExpressionUUID->"1c94ff7d-0a53-42fd-be83-\
20598ad0e137"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.703136671296762*^9, 
  3.703136678338892*^9}},ExpressionUUID->"7e306c15-0da7-432a-ac81-\
cada6d3c956e"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", "%", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"b", ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"\[Theta]", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.70313670321025*^9, 3.703136713490561*^9}, {
  3.703136749538836*^9, 
  3.703136836897921*^9}},ExpressionUUID->"84721f9a-043b-4c6f-b094-\
e939756ed923"],

Cell["\<\
OK.
  The Mathematica built-in gradient  Grad in polar coordinates doesn\
\[CloseCurlyQuote]t return the null tensor:\
\>", "Text",
 CellChangeTimes->{{3.6512367296748695`*^9, 3.6512368388313937`*^9}, 
   3.703138133279611*^9},ExpressionUUID->"9b75b06c-0fc5-4373-8b11-\
7c92c7c3e414"],

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"vv1", "[", 
    RowBox[{"r", ",", "\[Theta]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Theta]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Polar\>\"", ",", "\"\<Euclidean\>\"", ",", "2"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{
  3.6512294613313537`*^9},ExpressionUUID->"2233aafd-877c-43a3-ad75-\
aeae19f7a7e8"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Together", "[", "%", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", ">", "0"}], ",", 
     RowBox[{"b", ">", "0"}], ",", 
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"\[Theta]", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6512299020508375`*^9, 3.651229911050935*^9}, 
   3.7031369255145893`*^9},ExpressionUUID->"e25299c1-18c9-42d8-8af1-\
82c3005eb52a"],

Cell["\<\
The reason is that it is defined with respect to the local orthonormal bases \
corresponding to the coordinate system. The coordinates of the field vv1 with \
respect to these bases are\
\>", "Text",
 CellChangeTimes->{{3.6512368589719734`*^9, 3.651236953597176*^9}, {
   3.6512373791605177`*^9, 3.651237407426197*^9}, 
   3.6512391518825455`*^9},ExpressionUUID->"37eba8c3-0483-4678-bf0b-\
f8f2973e4f07"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv2", "[", 
   RowBox[{"r_", ",", "\[Theta]_"}], "]"}], ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", 
      RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
     RowBox[{"b", " ", 
      RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ",", 
    RowBox[{
     RowBox[{"b", " ", 
      RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "-", 
     RowBox[{"a", " ", 
      RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6512371668476253`*^9, 3.6512371675351267`*^9}, {
   3.6512372264102354`*^9, 3.651237253097785*^9}, 
   3.6512373259572945`*^9},ExpressionUUID->"e19af47b-4875-4c0e-8a71-\
ed4d3ab15a3c"],

Cell["Now we obtain the expected result:", "Text",
 CellChangeTimes->{{3.6512374284262333`*^9, 
  3.651237441879342*^9}},ExpressionUUID->"4e8d1b7f-fd0c-4543-b6a9-\
b436765d9d40"],

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"vv2", "[", 
    RowBox[{"r", ",", "\[Theta]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Theta]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Polar\>\"", ",", "\"\<Euclidean\>\"", ",", "2"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6512372974415665`*^9, 3.6512373141760225`*^9}, 
   3.6512373533635364`*^9},ExpressionUUID->"83e188b1-6d1c-41fd-b780-\
da2fc7a1fa26"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.7031387260744057`*^9, 
  3.703138726076552*^9}},ExpressionUUID->"a7dfabc7-c0fa-4ef9-bc4e-\
7231bd351c0b"],

Cell["", "Text",ExpressionUUID->"5f465b5c-66df-4b2b-ad8c-e174afab3cbb"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.2.3. The Absolute Differential of Functions, Vector, and Covector Fields\
\>", "Subsubsection",
 CellChangeTimes->{{3.650199080937522*^9, 3.650199089051357*^9}, {
   3.6502628893869257`*^9, 3.650262928073685*^9}, {3.650864075009926*^9, 
   3.6508641010724707`*^9}, {3.651894858289116*^9, 3.651894860872778*^9}, {
   3.651894901200581*^9, 3.651894912160174*^9}, 3.652338574806452*^9, {
   3.655885249441699*^9, 3.655885251785349*^9}, 3.656921630631371*^9, {
   3.656921765694251*^9, 3.656921802845566*^9}, 3.6572629775939074`*^9, {
   3.6688664366897*^9, 3.6688664427535133`*^9}, 3.669124445361519*^9, {
   3.669181354455949*^9, 3.6691814011187696`*^9}, {3.70209686572036*^9, 
   3.702096866648239*^9}, 3.7020969021854*^9, 3.7021041671109343`*^9, 
   3.703138837131016*^9, 3.703220817418668*^9, 3.7041070710314627`*^9, 
   3.704107742753441*^9},ExpressionUUID->"53a6ac1f-4f4e-4dc2-aa22-\
b6184f4402fe"],

Cell[TextData[{
 " We assume that on the n-dimensional smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6519077484399233`*^9, 3.65190774844132*^9}, {
    3.6519084533180428`*^9, 3.6519084566182737`*^9}, {3.651922369090623*^9, 
    3.651922410914259*^9}, {3.6519224599379807`*^9, 3.65192247265202*^9}, {
    3.651922511937611*^9, 3.651922596494038*^9}, {3.651922648376692*^9, 
    3.651922655781374*^9}, 3.6519226973689413`*^9, 3.651922776151458*^9, {
    3.652337428656632*^9, 3.652337461991699*^9}},ExpressionUUID->
  "73c1ef25-3966-4ab4-b3ef-e9710a136334"],
 " a linear connection \[CapitalGamma] is defined  with local coordinates"
}], "Text",
 CellChangeTimes->{{3.651932953242281*^9, 3.651933042457714*^9}, {
  3.652338351428577*^9, 3.652338351562481*^9}, {3.652338387832119*^9, 
  3.652338422463729*^9}, {3.6524177481002073`*^9, 3.65241774834066*^9}, {
  3.652418777653577*^9, 
  3.652418784021207*^9}},ExpressionUUID->"33c28844-756c-4515-b8a7-\
b46b55c461f8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox["\[CapitalGamma]", "kh", "j"], "[", "x", "]"}], "  ", 
  "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{"\[CapitalGamma]", "[", 
    RowBox[{"j", ",", "k", ",", "h"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6519331977047367`*^9, 3.651933228832294*^9}, 
   3.6519332708079348`*^9, {3.651933415337541*^9, 3.651933456990685*^9}, {
   3.655733956662973*^9, 
   3.6557339609989967`*^9}},ExpressionUUID->"f916e2c2-a4c1-4e13-a9dd-\
f90c0e89e286"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"\[CapitalGamma]", "[", 
    RowBox[{"j", ",", "k", ",", "h"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",ExpressionUUID->\
"f23fbeb5-8042-4fc5-a8be-d7c07c0a32c9"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]"}]], "Input",
 CellChangeTimes->{{3.669024299430175*^9, 
  3.669024306832868*^9}},ExpressionUUID->"a3ae5515-b7fa-4e1f-9415-\
fdc2ac454466"],

Cell["satisfying the transformation rules 3.2.2 (4):", "Text",
 CellChangeTimes->{{3.65233844358984*^9, 3.652338463343783*^9}, {
   3.6523385915664043`*^9, 3.652338836508197*^9}, {3.652339837546164*^9, 
   3.652339872729493*^9}, 3.652340498603204*^9, 3.652340697401107*^9, 
   3.65348221105077*^9},ExpressionUUID->"2c330baa-afcf-49a0-a330-\
db6a5a0eb25a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox[
    OverscriptBox["\[CapitalGamma]", "^"], "km", "i"], "[", "y", "]"}], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[CapitalGamma]", "hj", "l"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}], ")"}], 
    RowBox[{
     SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["B", "k", "h"], "[", "y", "]"}], 
    RowBox[{
     SubsuperscriptBox["A", "l", "i"], "[", 
     RowBox[{"x", "[", "y", "]"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubsuperscriptBox["A", "j", "i"], "[", 
      RowBox[{"x", "[", "y", "]"}], "]"}], "\[CenterDot]", 
     RowBox[{
      RowBox[{"\[PartialD]", 
       RowBox[{
        SubsuperscriptBox["B", "m", "j"], "[", "y", "]"}]}], "/", 
      RowBox[{"\[PartialD]", 
       SuperscriptBox["y", "k"]}]}]}], "                                                   ", 
    RowBox[{"(", "4", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6508083699383965`*^9, 3.650808398313449*^9}, {
  3.6508085498449306`*^9, 3.6508085950325623`*^9}, {3.6508619541445956`*^9, 
  3.6508619604414825`*^9}, {3.653482248467016*^9, 3.6534822622585173`*^9}, {
  3.704107159462723*^9, 3.704107165342165*^9}, {3.708425093811366*^9, 
  3.708425122361498*^9}, {3.708425159594702*^9, 3.708425161925318*^9}},
 CellTags->"gammatr",ExpressionUUID->"934eceb0-0287-4178-8b43-85613c87c2ef"],

Cell[BoxData[
 RowBox[{"Context", "[", "\[CapitalGamma]", "]"}]], "Input",
 CellChangeTimes->{{3.652340298233325*^9, 
  3.652340367804675*^9}},ExpressionUUID->"ab64016a-26d0-4588-b2e1-\
cf619ae32a50"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"\[CapitalGamma]", "[", 
    RowBox[{"j", ",", "k", ",", "h"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6544361096866093`*^9, 3.6544362248108*^9}, 
   3.654436360873564*^9, {3.654436437880865*^9, 3.6544366316255713`*^9}, {
   3.654436743704029*^9, 3.654436769984906*^9}, {3.654436804114821*^9, 
   3.6544368222830553`*^9}, {3.654436856837637*^9, 3.65443688366195*^9}, {
   3.654436919278461*^9, 3.654436922629221*^9}, 3.65573319677798*^9, {
   3.656660902329509*^9, 3.656660940969514*^9}, {3.662988244295952*^9, 
   3.662988270538219*^9}, {3.6629883205195503`*^9, 
   3.662988325287409*^9}},ExpressionUUID->"5a19c509-932c-4e98-b455-\
e53c344294bc"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"j", ",", "k", ",", "h"}], "}"}], "\[Epsilon]", " ", 
    RowBox[{"Tuples", "[", 
     RowBox[{
      RowBox[{"Range", "[", "dim", "]"}], ",", "3"}], "]"}]}], ";"}], 
  " "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6544361096866093`*^9, 3.6544362248108*^9}, 
   3.654436360873564*^9, {3.654436437880865*^9, 3.6544366316255713`*^9}, {
   3.654436743704029*^9, 3.654436769984906*^9}, {3.654436804114821*^9, 
   3.6544368222830553`*^9}, {3.654436856837637*^9, 3.65443688366195*^9}, {
   3.654436919278461*^9, 3.654436922629221*^9}, 3.65573319677798*^9, {
   3.656660902329509*^9, 3.656660940969514*^9}, {3.662988244295952*^9, 
   3.662988270538219*^9}, 
   3.702100265063381*^9},ExpressionUUID->"a5953371-67b2-4c7e-aabd-\
b7bea8f7012b"],

Cell[BoxData[
 RowBox[{"?", "Range"}]], "Input",
 CellChangeTimes->{{3.6544366671583652`*^9, 3.6544367052151814`*^9}, {
  3.654436835182538*^9, 
  3.6544368377179213`*^9}},ExpressionUUID->"41fbadf1-86c5-4766-8d06-\
d5f806e74ce6"],

Cell[BoxData[
 RowBox[{"?", "Tuples"}]], "Input",
 CellChangeTimes->{{3.669089049068039*^9, 
  3.66908905696449*^9}},ExpressionUUID->"087e3758-c892-4640-b13e-\
c36ca6972052"],

Cell[BoxData["dim"], "Input",
 CellChangeTimes->{{3.702100377151059*^9, 
  3.702100378533246*^9}},ExpressionUUID->"6ad26a66-8177-47d3-ac21-\
c356384f1de3"],

Cell[BoxData[
 RowBox[{"Tuples", "[", 
  RowBox[{
   RowBox[{"Range", "[", "dim", "]"}], ",", "3"}], "]"}]], "Input",ExpressionU\
UID->"45be3b67-c890-4796-82e3-c08f452dcbe2"],

Cell["\<\
In general, the  \[CapitalGamma]-symbols are a family of smooth functions \
given on the chart neighborhoods and obeying the transformation rule (4).  In \
this subsection we will prove the following \
\>", "Text",
 CellChangeTimes->{{3.652340421859564*^9, 3.65234045116324*^9}, 
   3.65234071960071*^9, {3.6523408161758432`*^9, 3.652341021678062*^9}, 
   3.652416866696587*^9, {3.652416901336308*^9, 3.652416930839699*^9}, {
   3.652417797219993*^9, 3.652417833147772*^9}, {3.654342815036409*^9, 
   3.654342857555779*^9}, {3.654435633063623*^9, 3.654435635831217*^9}, 
   3.654435681859083*^9, 3.654436090723713*^9, 3.6690901894939823`*^9, {
   3.669090251901676*^9, 3.6690902742933702`*^9}, {3.669090441084271*^9, 
   3.6690905409152317`*^9}, {3.669090630106563*^9, 3.669090698500901*^9}, {
   3.669090761218231*^9, 3.669090799257967*^9}, {3.66909096640804*^9, 
   3.6690909926078587`*^9}, 
   3.702100546901404*^9},ExpressionUUID->"11e72b91-ba17-469b-9dda-\
f3f81f2587da"],

Cell[TextData[{
 StyleBox["Theorem.",
  FontColor->RGBColor[0, 0, 1]],
 " Given a smooth n-dimensional manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6519077484399233`*^9, 3.65190774844132*^9}, {
    3.6519084533180428`*^9, 3.6519084566182737`*^9}, {3.651922369090623*^9, 
    3.651922410914259*^9}, {3.6519224599379807`*^9, 3.65192247265202*^9}, {
    3.651922511937611*^9, 3.651922596494038*^9}, {3.651922648376692*^9, 
    3.651922655781374*^9}, 3.6519226973689413`*^9, 3.651922776151458*^9, {
    3.652337428656632*^9, 3.652337461991699*^9}},ExpressionUUID->
  "6af4a950-6e5f-47ca-a117-ab1c3415f760"],
 " with a smooth linear connection \[CapitalGamma]. Then there exists a \
uniquely defined linear differential operator of first order called the  ",
 StyleBox["absolute differential",
  FontColor->RGBColor[0, 0, 1]],
 " ",
 "absD",
 " over the real algebra of all smooth tensor fields on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6519077484399233`*^9, 3.65190774844132*^9}, {
    3.6519084533180428`*^9, 3.6519084566182737`*^9}, {3.651922369090623*^9, 
    3.651922410914259*^9}, {3.6519224599379807`*^9, 3.65192247265202*^9}, {
    3.651922511937611*^9, 3.651922596494038*^9}, {3.651922648376692*^9, 
    3.651922655781374*^9}, 3.6519226973689413`*^9, 3.651922776151458*^9, {
    3.652337428656632*^9, 3.652337461991699*^9}},ExpressionUUID->
  "d08ac902-a311-4399-bbc1-a9a699315890"],
 " "
}], "Text",
 CellChangeTimes->{{3.652417847259348*^9, 3.652417853042938*^9}, {
   3.6524186751337957`*^9, 3.652418703293661*^9}, {3.6524187502228537`*^9, 
   3.652418769477344*^9}, {3.652418833268812*^9, 3.652418954611896*^9}, {
   3.652419011005618*^9, 3.652419180018436*^9}, {3.6524192259635963`*^9, 
   3.652419230665592*^9}, {3.652419272193777*^9, 3.652419462152554*^9}, {
   3.652419513568042*^9, 3.652419517528101*^9}, {3.6524204628973293`*^9, 
   3.652420578568014*^9}, {3.6524206105833673`*^9, 3.652420646142799*^9}, 
   3.652421900731571*^9, 3.6524221093947268`*^9, {3.6561560887195663`*^9, 
   3.656156105750985*^9}, 3.6561563450374527`*^9, {3.6567454576535683`*^9, 
   3.656745522436994*^9}, {3.6691120171259117`*^9, 
   3.669112027778998*^9}},ExpressionUUID->"6058aaa5-4206-4f7e-95b5-\
1ae1f177ff3c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"absD", ":", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"X", ",", "T"}], ")"}], "\[Element]", " ", 
    RowBox[{
     RowBox[{"\[ScriptCapitalX]", 
      RowBox[{
       FormBox[
        SuperscriptBox["M", "n"],
        TraditionalForm], " ", "\[Cross]", 
       FormBox[
        SuperscriptBox["\[ScriptCapitalJ]", 
         RowBox[{"(", 
          RowBox[{"k", ",", " ", "h"}], ")"}]],
        TraditionalForm]}], 
      FormBox[
       SuperscriptBox["M", "n"],
       TraditionalForm]}], " ", "\[RightTeeArrow]", "  ", 
     RowBox[{
      SubscriptBox["absD", "X"], "T"}]}], "\[Element]", " ", 
    RowBox[{
     FormBox[
      SuperscriptBox["\[ScriptCapitalJ]", 
       RowBox[{"(", 
        RowBox[{"k", ",", " ", "h"}], ")"}]],
      TraditionalForm], 
     FormBox[
      SuperscriptBox["M", "n"],
      TraditionalForm]}]}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6519077484399233`*^9, 3.65190774844132*^9}, {
   3.6519084533180428`*^9, 3.6519084566182737`*^9}, {3.651922369090623*^9, 
   3.651922410914259*^9}, {3.6519224599379807`*^9, 3.65192247265202*^9}, {
   3.651922511937611*^9, 3.651922596494038*^9}, {3.651922648376692*^9, 
   3.651922655781374*^9}, 3.6519226973689413`*^9, 3.651922776151458*^9, {
   3.652337428656632*^9, 3.652337461991699*^9}, {3.6560827553389072`*^9, 
   3.656082758994206*^9}, {3.6567456321640053`*^9, 3.656745637292364*^9}, {
   3.669112029692151*^9, 3.669112034010316*^9}, {3.669112397580724*^9, 
   3.669112406500237*^9}},ExpressionUUID->"8f47c1aa-d0aa-40b9-ab0a-\
5f7c0987ff06"],

Cell["satisfying the following conditions:", "Text",
 CellChangeTimes->{{3.652417847259348*^9, 3.652417853042938*^9}, {
   3.6524186751337957`*^9, 3.652418703293661*^9}, {3.6524187502228537`*^9, 
   3.652418769477344*^9}, {3.652418833268812*^9, 3.652418954611896*^9}, {
   3.652419011005618*^9, 3.652419180018436*^9}, {3.6524192259635963`*^9, 
   3.652419230665592*^9}, {3.652419272193777*^9, 3.652419462152554*^9}, {
   3.652419513568042*^9, 3.652419517528101*^9}, {3.6524204628973293`*^9, 
   3.652420578568014*^9}, {3.6524206105833673`*^9, 3.652420646142799*^9}, 
   3.652421900731571*^9, 3.6524221093947268`*^9, {3.6561560887195663`*^9, 
   3.656156105750985*^9}, 
   3.6561563450374527`*^9},ExpressionUUID->"864df167-993d-4cce-9596-\
390abdf3cc8c"],

Cell["1. absD is \[DoubleStruckCapitalR]-linear in X and in T.", "Text",
 CellChangeTimes->{{3.652417847259348*^9, 3.652417853042938*^9}, {
   3.6524186751337957`*^9, 3.652418703293661*^9}, {3.6524187502228537`*^9, 
   3.652418769477344*^9}, {3.652418833268812*^9, 3.652418954611896*^9}, {
   3.652419011005618*^9, 3.652419180018436*^9}, {3.6524192259635963`*^9, 
   3.652419230665592*^9}, {3.652419272193777*^9, 3.652419462152554*^9}, {
   3.652419513568042*^9, 3.652419517528101*^9}, {3.6524204628973293`*^9, 
   3.652420578568014*^9}, {3.6524206105833673`*^9, 3.652420646142799*^9}, {
   3.652421900731571*^9, 3.652421938100853*^9}, {3.656156410413206*^9, 
   3.656156425379882*^9}, {3.656745663227735*^9, 3.656745680683392*^9}, 
   3.669112038644884*^9, 
   3.702100609651216*^9},ExpressionUUID->"c39c23d2-c0ca-4726-8bc1-\
676a9630b47f"],

Cell["\<\
2. Applied to  functions (as tensor fields of type (0,0)) absD coincides with \
the differential:\
\>", "Text",
 CellChangeTimes->{{3.652417847259348*^9, 3.652417853042938*^9}, {
   3.6524186751337957`*^9, 3.652418703293661*^9}, {3.6524187502228537`*^9, 
   3.652418769477344*^9}, {3.652418833268812*^9, 3.652418954611896*^9}, {
   3.652419011005618*^9, 3.652419180018436*^9}, {3.6524192259635963`*^9, 
   3.652419230665592*^9}, {3.652419272193777*^9, 3.652419462152554*^9}, {
   3.652419513568042*^9, 3.652419517528101*^9}, {3.6524204628973293`*^9, 
   3.652420578568014*^9}, {3.6524206105833673`*^9, 3.652420646142799*^9}, {
   3.652421900731571*^9, 3.652421965123683*^9}, 3.656156702522163*^9, 
   3.656157026719551*^9, 
   3.669112041611806*^9},ExpressionUUID->"a527f0a3-0e32-4d2a-bc7e-\
3556d2fe24fd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], 
   RowBox[{"f", "[", "x", "]"}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", "x", "]"}], ".", "X"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.652420654945572*^9, 3.652420713174815*^9}, 
   3.65615648344411*^9, {3.6561565197312317`*^9, 3.656156554835333*^9}, 
   3.6567460048413143`*^9, 3.6567482946784573`*^9, {3.656748720386908*^9, 
   3.656748748930306*^9}, {3.669089958967628*^9, 3.669089985639645*^9}, 
   3.669112044610271*^9},
 TextAlignment->Center,ExpressionUUID->"27a2cab6-a298-4c05-af95-35ec07b9231a"],

Cell[TextData[{
 "3,  Applied to  vector fields v[x] (as tensor fields of type (0, 1)) ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["absD", "X"], TraditionalForm]],ExpressionUUID->
  "541726c0-84d0-4464-9c58-d3217711b83e"],
 " is defined on the base of the gradient tensor"
}], "Text",
 CellChangeTimes->{{3.656745747602746*^9, 3.656745752282531*^9}, {
  3.656745799451703*^9, 3.656745830842073*^9}, {3.656745883798892*^9, 
  3.656745945609034*^9}, {3.6567473725575457`*^9, 3.656747377189466*^9}, {
  3.6691120471468678`*^9, 
  3.6691120471469088`*^9}},ExpressionUUID->"bb552de7-1852-4e0f-b42b-\
2d1443e83bf5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["absD", "X"], 
    RowBox[{"v", "[", "x", "]"}]}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Del]", "h"], 
     RowBox[{
      SuperscriptBox["v", "i"], "[", "x", "]"}]}], 
    SuperscriptBox["\[Xi]", "h"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], "x"}]}]}], ",", "   "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6567460500487137`*^9, 3.65674609353649*^9}, {
   3.656746224468548*^9, 3.656746437205608*^9}, {3.65674647582078*^9, 
   3.656746490260861*^9}, {3.656746736475091*^9, 3.656746743227028*^9}, 
   3.6567468502021427`*^9, 3.656747363429922*^9, {3.6568358220161037`*^9, 
   3.6568358356775312`*^9}, 
   3.669112061490155*^9},ExpressionUUID->"d565a965-0431-4902-8df8-\
05319e463131"],

Cell[TextData[{
 "where the ",
 Cell[BoxData[
  SuperscriptBox["\[Xi]", "h"]], "Input",
  CellChangeTimes->{{3.6567460500487137`*^9, 3.65674609353649*^9}, {
    3.656746224468548*^9, 3.656746437205608*^9}, {3.65674647582078*^9, 
    3.656746490260861*^9}, {3.656746736475091*^9, 3.656746743227028*^9}, 
    3.6567468502021427`*^9, 3.656747363429922*^9},ExpressionUUID->
  "cc5cdca8-56a3-40e7-ba17-ee98dba25d45"],
 " are the coordinates of the tangent vector or vector field X and ",
 Cell[BoxData[
  RowBox[{"  ", 
   RowBox[{
    SubscriptBox["\[Del]", "h"], 
    RowBox[{
     SuperscriptBox["v", "i"], "[", "x", "]"}]}]}]], "Input",
  CellChangeTimes->{{3.6567460500487137`*^9, 3.65674609353649*^9}, {
    3.656746224468548*^9, 3.656746437205608*^9}, {3.65674647582078*^9, 
    3.656746490260861*^9}, {3.656746736475091*^9, 3.656746743227028*^9}, 
    3.6567468502021427`*^9},ExpressionUUID->
  "d62c2644-c6fc-49fa-9a2f-0a4c57c31659"],
 " are the coordinates of the gradient tensor defined in Subsection 3.2.2:"
}], "Text",
 CellChangeTimes->{{3.6567472229596148`*^9, 3.6567472979661913`*^9}, {
  3.6567473294618053`*^9, 3.65674734166197*^9}, {3.656747612888049*^9, 
  3.656747612888068*^9}, {3.656747651781898*^9, 
  3.656747685242517*^9}},ExpressionUUID->"5322cad6-207b-41eb-ab94-\
dad7457bdf0b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "h"], 
    RowBox[{
     SuperscriptBox["v", "i"], "[", "x", "]"}]}], "\[DotEqual]", "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"nab1", "[", 
      RowBox[{"\[CapitalGamma]_", ",", "v_"}], "]"}], "[", "x__", "]"}], "[", 
    RowBox[{"h_", ",", "i_"}], "]"}]}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"v", "[", "x", "]"}], "[", 
      RowBox[{"[", "i", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"{", "x", "}"}], "[", 
      RowBox[{"[", "h", "]"}], "]"}]}], "]"}], " ", "+", 
   RowBox[{"Sum", " ", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"v", "[", "x", "]"}], "[", 
       RowBox[{"[", "l", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"i", ",", "h", ",", "l"}], "]"}], "[", "x", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"l", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.656746700731824*^9, 3.6567467013636227`*^9}, {
   3.6567467785393476`*^9, 3.656746790830647*^9}, 3.656746830930542*^9, {
   3.656746861538372*^9, 3.656746932841977*^9}, {3.6567470210571117`*^9, 
   3.656747031712488*^9}, {3.656747084400077*^9, 3.656747141913146*^9}, 
   3.6629885692673273`*^9, 
   3.663309510401091*^9},ExpressionUUID->"d4dc10e4-451c-46c4-8fd9-\
61038a01e372"],

Cell["4. For all contractions \[FormalCapitalY]T of tensor fields T one has", \
"Text",
 CellChangeTimes->{{3.6524207426295433`*^9, 3.652420745197649*^9}, {
   3.652421319032557*^9, 3.652421337304871*^9}, {3.652421438631689*^9, 
   3.652421478127287*^9}, 3.6561570313514853`*^9, 3.656745703923317*^9, {
   3.6629886144116096`*^9, 
   3.66298861482756*^9}},ExpressionUUID->"5a776ee5-f32d-4837-8d16-\
26a831c2436f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], "[", "\[FormalCapitalY]T", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"\[FormalCapitalY]", "[", 
    RowBox[{
     SubscriptBox["absD", "X"], "T"}], "]"}], "."}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.652421493290246*^9, 3.652421567735425*^9}, 
   3.656156504236162*^9, {3.656156582868298*^9, 3.656156589980475*^9}, {
   3.66911207925837*^9, 3.669112080090538*^9}},
 TextAlignment->Center,ExpressionUUID->"7ab197d7-07fa-4128-8616-da835b51ca14"],

Cell["\<\
5.  The product rule is valid for arbitrary tensor fields A,B\
\>", "Text",
 CellChangeTimes->{{3.6524216256304197`*^9, 3.652421689781563*^9}, 
   3.656157035655376*^9, 3.656745710467207*^9, {3.656835960219985*^9, 
   3.656835990667589*^9}},ExpressionUUID->"fdcac56c-3635-42ac-aef9-\
7e42bdfaf5be"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], "[", 
   RowBox[{"A", "\[CircleTimes]", 
    StyleBox["B",
     FontWeight->"Plain"]}], 
   StyleBox["]",
    FontWeight->"Plain"]}], 
  StyleBox[" ",
   FontWeight->"Plain"], "=", 
  StyleBox[" ",
   FontWeight->"Plain"], 
  RowBox[{
   RowBox[{
    RowBox[{
     StyleBox["(",
      FontWeight->"Plain"], 
     RowBox[{
      SubscriptBox["absD", "X"], "A"}], ")"}], "\[CircleTimes]", " ", "B"}], 
   " ", "+", " ", 
   RowBox[{
    RowBox[{"A", "\[CircleTimes]", 
     SubscriptBox["absD", "X"]}], 
    RowBox[{"B", "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6524217027390327`*^9, 3.652421866949074*^9}, {
  3.656156608107471*^9, 3.656156621884164*^9}, {3.669112083329936*^9, 
  3.669112086770149*^9}},
 TextAlignment->Center,ExpressionUUID->"a6a01e57-0b47-43f0-9816-1cfbfa46b808"],

Cell[TextData[{
 "6. For any open submanifold U\[Subset]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6519077484399233`*^9, 3.65190774844132*^9}, {
    3.6519084533180428`*^9, 3.6519084566182737`*^9}, {3.651922369090623*^9, 
    3.651922410914259*^9}, {3.6519224599379807`*^9, 3.65192247265202*^9}, {
    3.651922511937611*^9, 3.651922596494038*^9}, {3.651922648376692*^9, 
    3.651922655781374*^9}, 3.6519226973689413`*^9, 3.651922776151458*^9, {
    3.652337428656632*^9, 3.652337461991699*^9}},ExpressionUUID->
  "3f25100d-39bc-4f87-94ab-51faa55f9641"],
 " denote by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["|", 
    RowBox[{"U", " "}]], TraditionalForm]],ExpressionUUID->
  "8a6c5316-5768-4901-9606-104370036a37"],
 "  the restrictions of tensor fields A and linear connections to U. Clearly \
\[CapitalGamma]",
 Cell[BoxData[
  FormBox[
   SubscriptBox["|", 
    RowBox[{"U", " "}]], TraditionalForm]],ExpressionUUID->
  "0d77cbc0-34be-4c70-af1a-b9d45e2abcf8"],
 " is a linear connection on U. Then the restriction of the absolute \
differential of a tensor field is the restricted absolute differential of the \
restricted field:"
}], "Text",
 CellChangeTimes->{{3.652675482617928*^9, 3.652675513164654*^9}, {
   3.652675586300227*^9, 3.652675634338429*^9}, {3.652675664382886*^9, 
   3.652675819752997*^9}, {3.652675852602003*^9, 3.652675992209136*^9}, {
   3.6526763858856*^9, 3.652676414645467*^9}, 3.654406380935895*^9, {
   3.6561566514512053`*^9, 3.656156685386742*^9}, {3.656156729418352*^9, 
   3.6561567376260777`*^9}, 3.656157039552032*^9, 3.6566610761606503`*^9, {
   3.65666112854454*^9, 3.656661137200695*^9}, 
   3.6567457259960012`*^9},ExpressionUUID->"b90ac6ab-428b-4007-8cf4-\
1d0e660e4a7d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   StyleBox["(",
    FontWeight->"Plain"], 
   RowBox[{
    SubscriptBox["absD", "X"], "A"}], ")"}], 
  SubscriptBox["|", 
   RowBox[{"U", " "}]], "  ", 
  RowBox[{"\[Equal]", " ", 
   RowBox[{
    SubscriptBox[
     SubscriptBox["absD", 
      RowBox[{"X", "|"}]], 
     RowBox[{"U", " "}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"A", 
       SubscriptBox["|", 
        RowBox[{"U", " "}]]}], ")"}], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.652676057320887*^9, 3.6526760669604883`*^9}, {
  3.652676146071782*^9, 3.652676194687641*^9}, {3.656156756330867*^9, 
  3.656156888760708*^9}, {3.669112088930203*^9, 
  3.6691120905303097`*^9}},ExpressionUUID->"397b7350-b0ee-4de7-886b-\
d031bc559fb1"],

Cell[TextData[{
 StyleBox["Proof.",
  FontColor->RGBColor[0, 0, 1]],
 " First we assume that the absolute differential corresponding to a given \
linear connection satisfies the conditions 1 - 6 and show that the \
coordinates of ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["absD", "X"], "T"}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.652421493290246*^9, 3.652421567735425*^9}, 
    3.656156504236162*^9, {3.656156582868298*^9, 3.656156589980475*^9}},
  TextAlignment->Center,ExpressionUUID->
  "0c8d1006-3fe6-4969-8ac5-f1e5cefc3b36"],
 " are uniquely defined. Then the expressions obtained in this part of the \
proof will be shown to be the coordinates of a tensor field satisfying 1. - \
6.  Thus the operator ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["absD", "X"], "T"}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.652421493290246*^9, 3.652421567735425*^9}, 
    3.656156504236162*^9, {3.656156582868298*^9, 3.656156589980475*^9}},
  TextAlignment->Center,ExpressionUUID->
  "1aa35940-ba54-491d-b098-ad3e22b71519"],
 " defined this way fulfills the statement of the theorem.\nFor functions f \
as tensors of type (0,0) condition 2 defines ",
 Cell[BoxData[
  SubscriptBox["absD", "X"]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.652421493290246*^9, 3.652421567735425*^9}, 
    3.656156504236162*^9, {3.656156582868298*^9, 3.656156589980475*^9}},
  TextAlignment->Center,ExpressionUUID->
  "c30eed1d-69ba-4bb0-ab63-0b87d340382d"],
 "f such that conditions 1, 2, 5, and 6 for functions are fulfilled:"
}], "Text",
 CellChangeTimes->{{3.652422423551714*^9, 3.652422478239637*^9}, 
   3.652422508311203*^9, {3.6524226005503607`*^9, 3.652422665677822*^9}, {
   3.652422699637463*^9, 3.652422764284708*^9}, {3.652422823588141*^9, 
   3.652422959426772*^9}, {3.652423737123698*^9, 3.652423749171473*^9}, {
   3.652444308523034*^9, 3.652444496969111*^9}, {3.652676318006133*^9, 
   3.652676325854145*^9}, {3.6526764809887753`*^9, 3.652676620763959*^9}, {
   3.652676850864827*^9, 3.652676851720171*^9}, 3.6526768933353*^9, {
   3.652676945310589*^9, 3.6526769583181887`*^9}, {3.6526769885018187`*^9, 
   3.652677063084757*^9}, {3.652677097140314*^9, 3.652677126668044*^9}, {
   3.652677166683552*^9, 3.6526773634570723`*^9}, {3.654343007483583*^9, 
   3.654343016226591*^9}, {3.654406421567011*^9, 3.654406429566905*^9}, {
   3.656156941865844*^9, 3.6561569492969112`*^9}, 3.656157071600759*^9, {
   3.656157113687706*^9, 3.656157152111272*^9}, 3.65615722722615*^9, {
   3.656157257503121*^9, 3.6561572958405237`*^9}, {3.656157330303698*^9, 
   3.6561573616062403`*^9}, {3.65615745454147*^9, 3.656157463229467*^9}, {
   3.6561575093004303`*^9, 3.6561575143084307`*^9}, 3.656158832577936*^9, 
   3.656316827552226*^9, {3.656748217848658*^9, 3.6567482439438972`*^9}, {
   3.6567484045740747`*^9, 3.6567484105105953`*^9}, 3.656748454733399*^9, 
   3.6567485065652657`*^9, 3.6569172997482033`*^9, {3.669112094236183*^9, 
   3.6691121000602207`*^9}},ExpressionUUID->"a2aa218c-a659-4462-ac24-\
3101dd4f163c"],

Cell[BoxData[
 RowBox[{"?", "d"}]], "Input",
 CellChangeTimes->{{3.6567485144212303`*^9, 
  3.656748515883954*^9}},ExpressionUUID->"c5565668-27c9-4e68-a019-\
6bc0e39bff1d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", "f", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], ".", 
  RowBox[{"vec", "[", "\[Xi]", "]"}]}]], "Input",
 CellChangeTimes->{{3.656748604440564*^9, 3.656748617371562*^9}, {
   3.656748653075727*^9, 3.656748687810999*^9}, {3.669112988743318*^9, 
   3.6691130538145638`*^9}, {3.708425625333254*^9, 3.708425628489615*^9}, 
   3.708425674524395*^9},ExpressionUUID->"08c0d59d-f938-4acf-9f70-\
217c2b855b0f"],

Cell["The corresponding gradient tensor is ", "Text",
 CellChangeTimes->{{3.652422423551714*^9, 3.652422478239637*^9}, 
   3.652422508311203*^9, {3.6524226005503607`*^9, 3.652422665677822*^9}, {
   3.652422699637463*^9, 3.652422764284708*^9}, {3.652422823588141*^9, 
   3.652422959426772*^9}, {3.652423737123698*^9, 3.652423749171473*^9}, {
   3.652444308523034*^9, 3.652444496969111*^9}, {3.652676318006133*^9, 
   3.652676325854145*^9}, {3.6526764809887753`*^9, 3.652676620763959*^9}, {
   3.652676850864827*^9, 3.652676851720171*^9}, 3.6526768933353*^9, {
   3.652676945310589*^9, 3.6526769583181887`*^9}, {3.6526769885018187`*^9, 
   3.652677063084757*^9}, {3.652677097140314*^9, 3.652677126668044*^9}, {
   3.652677166683552*^9, 3.6526773634570723`*^9}, {3.654343007483583*^9, 
   3.654343016226591*^9}, {3.654406421567011*^9, 3.654406429566905*^9}, {
   3.656156941865844*^9, 3.6561569492969112`*^9}, 3.656157071600759*^9, {
   3.656157113687706*^9, 3.656157152111272*^9}, 3.65615722722615*^9, {
   3.656157257503121*^9, 3.6561572958405237`*^9}, {3.656157330303698*^9, 
   3.6561573616062403`*^9}, {3.65615745454147*^9, 3.656157463229467*^9}, {
   3.6561575093004303`*^9, 3.6561575143084307`*^9}, 3.656158832577936*^9, 
   3.656316827552226*^9, {3.656748217848658*^9, 3.6567482439438972`*^9}, {
   3.6567484045740747`*^9, 3.6567484105105953`*^9}, 3.656748454733399*^9, 
   3.6567485065652657`*^9},ExpressionUUID->"919d8e58-2c12-44ae-8ea4-\
d9bb093c50a0"],

Cell[BoxData[
 RowBox[{"?", "nabla0"}]], "Input",
 CellChangeTimes->{{3.669113136964487*^9, 
  3.6691131429819117`*^9}},ExpressionUUID->"934182ea-0430-44dd-8d1b-\
d0c9312a87e1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nabla0", "[", "f_", "]"}], "[", "x__", "]"}], ":=", 
  RowBox[{"covecTotensor", "[", 
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", "x", "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.652423006059102*^9, 3.652423099058041*^9}, {
   3.652423165953165*^9, 3.652423192400968*^9}, {3.6524243957028723`*^9, 
   3.6524244156146584`*^9}, 3.652424554030555*^9, {3.652424624829391*^9, 
   3.652424644388485*^9}, {3.652424678237733*^9, 3.652424682853071*^9}, {
   3.6524454629155197`*^9, 3.652445477947034*^9}, {3.652707219020443*^9, 
   3.652707232059682*^9}, 3.662967216915649*^9, 3.662970417880039*^9, {
   3.662989280613188*^9, 3.66298929819678*^9}, {3.662989449020054*^9, 
   3.6629895069310017`*^9}, {3.669113085609253*^9, 3.669113100670312*^9}, {
   3.708425748571392*^9, 
   3.708425799753936*^9}},ExpressionUUID->"65018730-3f63-4828-a49d-\
3c236ce264ac"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "f", "]"}], ";", 
  RowBox[{
   RowBox[{"nabla0", "[", "f", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65242325939367*^9, 3.652423277552004*^9}, {
   3.6524245709652967`*^9, 3.65242457486882*^9}, 3.6524247363798647`*^9, {
   3.652445495051313*^9, 3.652445504602859*^9}, 3.6629704343998423`*^9, 
   3.708425807164311*^9},ExpressionUUID->"012e8d39-d447-458d-b74c-\
fb7458a0d6c7"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.652423295494874*^9, 
  3.6524233030802803`*^9}},ExpressionUUID->"fa253dc1-134c-4d2c-badf-\
1ed7d9804652"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nabla0", "[", "f", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.652423371751286*^9, 3.652423395015458*^9}, 
   3.652423438279714*^9, {3.652423547510706*^9, 3.652423556549603*^9}, {
   3.6524455237958384`*^9, 3.652445560554572*^9}, 3.653797150199676*^9, 
   3.654343079425802*^9, 
   3.662970462039399*^9},ExpressionUUID->"d4d26d46-2553-49f6-8dd7-\
93858da51ba9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla0", "[", 
   RowBox[{"plus", "[", 
    RowBox[{"f", ",", "g"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.652445610216058*^9, 3.652445617808168*^9}, {
   3.652497601257401*^9, 3.6524976242966843`*^9}, 
   3.662970475687386*^9},ExpressionUUID->"939ac70e-3c92-41cf-9413-\
48ccaf1661a3"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.652497638084215*^9, 
  3.652497646144717*^9}},ExpressionUUID->"804c24f2-02ce-4294-9905-\
6a4426993ccf"],

Cell["\<\
We must use plus for the addition of functions or fields since the \
built-inPlus  is not evaluated for the addition of these objects, it acts on \
their values only:\
\>", "Text",
 CellChangeTimes->{{3.6526775048782253`*^9, 3.6526776035011597`*^9}, {
  3.669113781061699*^9, 3.6691137950053053`*^9}, {3.669113830240547*^9, 
  3.669113838678071*^9}},ExpressionUUID->"1aef142b-eb8a-4b0e-9942-\
01e515bc9d41"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nabla0", "[", 
    RowBox[{"f", "+", "g"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.652423371751286*^9, 3.652423395015458*^9}, 
   3.652423438279714*^9, {3.652423547510706*^9, 3.652423556549603*^9}, {
   3.6524455237958384`*^9, 3.652445560554572*^9}, {3.652675082200927*^9, 
   3.6526750830490847`*^9}, 
   3.662976459620583*^9},ExpressionUUID->"24838d19-4dac-4d42-8c8a-\
01831a9ff29b"],

Cell[BoxData[
 RowBox[{"coords", "[", 
  RowBox[{
   RowBox[{"nabla0", "[", 
    RowBox[{"f", "+", "g"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.669113241928241*^9, 3.669113249173053*^9}, 
   3.702101023466298*^9},ExpressionUUID->"1c120c38-120d-4a14-8c61-\
6a893c8a4bc4"],

Cell["but (f+g)[var[x]] isn\[CloseCurlyQuote]t defined:", "Text",
 CellChangeTimes->{{3.7084270656688547`*^9, 
  3.7084271131236057`*^9}},ExpressionUUID->"a47eec1f-fd8a-47fd-a4ce-\
79225a1ee2cc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"f", "+", "g"}], ")"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.708427137529853*^9, 
  3.708427137531767*^9}},ExpressionUUID->"f4c4f731-e4a5-44d5-9648-\
2cc0d05acf0e"],

Cell[BoxData[
 RowBox[{"TrueQ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"f", "+", "g"}], ")"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
    RowBox[{"g", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.708427137529853*^9, 
  3.708427216958827*^9}},ExpressionUUID->"f6a6fd25-b012-4301-861c-\
b66f4453acd4"],

Cell[BoxData[
 RowBox[{"TrueQ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"plus", "[", 
     RowBox[{"f", ",", "g"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
    RowBox[{"g", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.708427137529853*^9, 
  3.7084272716160307`*^9}},ExpressionUUID->"19dd409d-40f4-4427-a1f9-\
a2a6b5f89b7c"],

Cell["Multiplying the fields with real numbers, e.g. Pi:", "Text",
 CellChangeTimes->{{3.652678276789916*^9, 
  3.652678307974001*^9}},ExpressionUUID->"f9899ad1-844f-4d51-a1bc-\
1c636fa34bab"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nabla0", "[", 
    RowBox[{"Pi", "*", "f"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "\[Equal]", " ", 
  RowBox[{"Pi", "*", 
   RowBox[{
    RowBox[{"nabla0", "[", "f", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{
  3.652675350615377*^9, {3.652677702548273*^9, 3.652677756515798*^9}, 
   3.662976497067515*^9, 
   3.6629890009438457`*^9},ExpressionUUID->"3a834201-ddec-49f9-adb5-\
45a909daa5cc"],

Cell["\<\
doesn't give the expected linearity since the formal definitions of the \
tensor fields differ.  But remember \
\>", "Text",
 CellChangeTimes->{{3.652678420141942*^9, 3.652678581667445*^9}, {
  3.657614562710887*^9, 
  3.657614609847198*^9}},ExpressionUUID->"0c736222-df65-4e03-b583-\
8eade4903d0d"],

Cell[BoxData[
 RowBox[{"?", "equi"}]], "Input",
 CellChangeTimes->{{3.6576146206274242`*^9, 
  3.657614622734117*^9}},ExpressionUUID->"709c9708-409c-4bf2-be4d-\
4afd79f7480c"],

Cell["Both fields have the same coordinates:", "Text",
 CellChangeTimes->{{3.652678420141942*^9, 3.652678581667445*^9}, {
  3.657614562710887*^9, 
  3.657614609847198*^9}},ExpressionUUID->"1a2f6c49-0372-4122-91f2-\
9ba2458bb9d6"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla0", "[", 
     RowBox[{"Pi", "*", "f"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", " ", 
   RowBox[{"Pi", "*", 
    RowBox[{
     RowBox[{"nabla0", "[", "f", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.652675350615377*^9, {3.652677702548273*^9, 3.6526777808082333`*^9}, {
   3.6526778174512367`*^9, 3.652677849658783*^9}, 3.652677887328351*^9, {
   3.662976534466917*^9, 
   3.662976538554463*^9}},ExpressionUUID->"4c112bf9-c909-4411-af87-\
aaeab9598b5d"],

Cell["analogously", "Text",
 CellChangeTimes->{{3.708426833358048*^9, 
  3.70842683944619*^9}},ExpressionUUID->"c1fda347-73a0-41dc-b324-\
8190f1bf3486"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla0", "[", 
     RowBox[{"plus", "[", 
      RowBox[{"f", ",", "g"}], "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"nabla0", "[", "f", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
    RowBox[{
     RowBox[{"nabla0", "[", "g", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.652675167344481*^9, 3.6526751924960318`*^9}, 
   3.662970507357437*^9, {3.708426037707121*^9, 3.7084260967769537`*^9}, {
   3.70842613552337*^9, 3.708426162288718*^9}, {3.7084262427779617`*^9, 
   3.708426273160411*^9}},ExpressionUUID->"33806a95-f381-46e6-b20e-\
484be37ba6cf"],

Cell["\<\
we have equivalent definitions of the same tensor field. Thus nabla0 is real \
linear as we know already for the differential d of functions. For an \
arbitrary tangent vector vX or a field vX[var[x]]\
\>", "Text",
 CellChangeTimes->{{3.652678593795025*^9, 3.652678620754641*^9}, {
   3.652678655650702*^9, 3.6526786924980907`*^9}, 3.656661670850724*^9, {
   3.6566617123302603`*^9, 3.6566618228889008`*^9}, {3.669113926188423*^9, 
   3.669113945564538*^9}, 
   3.708426811055622*^9},ExpressionUUID->"6323726a-111e-423e-ba08-\
567e78fee6e8"],

Cell[BoxData[
 RowBox[{"vX", "=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vec", "[", "\[Xi]", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65666183926819*^9, 
  3.656661922744179*^9}},ExpressionUUID->"bfb69729-4812-43d9-a165-\
9c043390885d"],

Cell["\<\
we obtain  the directional derivative as the only coordinate of the \
contraction, as expected:\
\>", "Text",
 CellChangeTimes->{
  3.656661941492207*^9, {3.656662010439938*^9, 3.656662019574676*^9}, {
   3.656662385083716*^9, 3.656662417307497*^9}, {3.656662561146493*^9, 
   3.656662591714466*^9}},ExpressionUUID->"c1df7b53-95fd-40fe-97b2-\
65f78a3d2024"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ".", 
   RowBox[{"vec", "[", "\[Xi]", "]"}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"nabla0", "[", "f", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", "vX"}], ",", 
     "1", ",", "1"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.656662025894168*^9, 3.656662162365744*^9}, {
   3.656662256310433*^9, 3.6566622569970083`*^9}, {3.6566623335974493`*^9, 
   3.656662350348425*^9}, 3.662976574369719*^9, {3.66911400239701*^9, 
   3.669114045844043*^9}},ExpressionUUID->"8bbb9630-e752-4a42-b627-\
6c26d1e50a04"],

Cell[TextData[{
 "Now we consider a vector field Y[x] with coordinates ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SuperscriptBox["v", "k"], "[", "x", "]"}], "\[DotEqual]", " ", 
   RowBox[{
    RowBox[{"\[Eta]", "[", "k", "]"}], "[", 
    RowBox[{"x", ",", ".."}], "]"}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{
   3.6502664656646566`*^9, {3.650266529375502*^9, 3.6502666491873617`*^9}, 
    3.6502666870716534`*^9, {3.6502731006614904`*^9, 3.6502731908860517`*^9}, 
    3.6502732654402127`*^9, {3.6502734675989227`*^9, 3.650273486253156*^9}, {
    3.6502735242610545`*^9, 3.650273540209237*^9}, {3.650804193571299*^9, 
    3.6508041944150505`*^9}},ExpressionUUID->
  "0b7f8b72-ac59-4555-b467-29ad1d604876"],
 ". Proposition 3.2.2 shows that an operator ",
 "absD",
 " satisfying the conditions 1 - 6 defines a tensor field \[Del]Y  whose \
coordinates have the shape"
}], "Text",
 CellChangeTimes->{{3.652678756417199*^9, 3.652678814480547*^9}, {
   3.652678893151895*^9, 3.652679045790443*^9}, 3.656159368173643*^9, 
   3.656749099400242*^9, {3.656749190711216*^9, 3.6567491966949577`*^9}, {
   3.656749239538206*^9, 3.656749239538245*^9}, 3.656749739322815*^9, 
   3.6567512684951687`*^9, 3.656918162108535*^9, {3.656918294251377*^9, 
   3.6569183176110373`*^9}, {3.6629779812902527`*^9, 3.662977981290292*^9}, {
   3.662978013639084*^9, 3.6629780576775293`*^9}, 
   3.669112101775833*^9},ExpressionUUID->"4dabdeee-62d2-4b8a-92a6-\
2ad7b92ec267"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "h"], 
   RowBox[{
    SuperscriptBox["v", "j"], "[", "x", "]"}]}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{
     SuperscriptBox["v", "j"], "[", "x", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     SuperscriptBox["v", "k"], "[", "x", "]"}], 
    RowBox[{
     SubsuperscriptBox["\[CapitalGamma]", "hk", "j"], "[", "x", "]"}], "                      ", 
    RowBox[{"(", "2", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6502664656646566`*^9, {3.650266529375502*^9, 3.6502666491873617`*^9}, 
   3.6502666870716534`*^9, {3.6502731006614904`*^9, 3.6502731908860517`*^9}, 
   3.6502732654402127`*^9, {3.6502734675989227`*^9, 3.650273486253156*^9}, {
   3.6502735242610545`*^9, 3.650273540209237*^9}, {3.650804193571299*^9, 
   3.6508041944150505`*^9}, {3.6534824488732367`*^9, 3.653482449384453*^9}, {
   3.656749241559985*^9, 3.656749244709979*^9}, {3.7084274207912073`*^9, 
   3.70842742445022*^9}},ExpressionUUID->"59dbd65a-1c4f-4fe6-b9c6-\
b512d8220529"],

Cell[TextData[{
 "We introduce a preliminary corresponding ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " definition being obsolete later since it is included into the general \
definition for tensor fields. For the vector field"
}], "Text",
 CellChangeTimes->{{3.652705080379346*^9, 3.652705083036367*^9}, {
  3.652705348571054*^9, 3.652705407082685*^9}, {3.652705446938195*^9, 
  3.652705470761992*^9}, {3.6527055741054983`*^9, 3.652705579697289*^9}, {
  3.656751314142529*^9, 3.656751380013919*^9}, {3.656751411709807*^9, 
  3.656751418029572*^9}, {3.702101815725697*^9, 
  3.702101830234475*^9}},ExpressionUUID->"8a54859d-1ad2-4e6a-82ab-\
5beef36ed253"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6629687825703487`*^9, 
  3.6629687877018013`*^9}},ExpressionUUID->"cb550792-17cb-434d-bceb-\
1479bfe17e3e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Y", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"vv", "[", "\[Eta]", "]"}], "[", "x", "]"}]}], ";", 
  RowBox[{"Y", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.652705699418322*^9, 3.652705708832561*^9}, {
   3.656749383853791*^9, 3.656749388613441*^9}, {3.66296888680582*^9, 
   3.6629689691962757`*^9}, {3.66297718322087*^9, 3.66297719230709*^9}, {
   3.662977232283511*^9, 3.662977238540073*^9}, 3.662977275882807*^9, {
   3.662977332081031*^9, 3.662977351672718*^9}, {3.6629901798600073`*^9, 
   3.6629902026832533`*^9}},ExpressionUUID->"540fa58b-db8f-4e1f-b358-\
39ef67d88bc8"],

Cell[BoxData[
 RowBox[{"?", "nabla1"}]], "Input",
 CellChangeTimes->{{3.669114146901379*^9, 
  3.669114150489884*^9}},ExpressionUUID->"086c54cf-c073-4c43-be2b-\
f8d24e9053a4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nabla1", "[", 
    RowBox[{"\[CapitalGamma]_", ",", "zz_"}], "]"}], "[", "x__", "]"}], " ", ":=",
   " ", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"zz", "[", "x", "]"}], "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", "x", "}"}], "[", 
          RowBox[{"[", "h", "]"}], "]"}]}], "]"}], "+", " ", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"zz", "[", "x", "]"}], "[", 
           RowBox[{"[", "k", "]"}], "]"}], "*", 
          RowBox[{
           RowBox[{"\[CapitalGamma]", "[", 
            RowBox[{"i", ",", "h", ",", "k"}], "]"}], "[", "x", "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "1", ",", 
           RowBox[{"Length", "[", 
            RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"h", ",", " ", "1", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}], ",", "1"}], 
   "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.652706827696953*^9, 3.652706836815072*^9}, {
   3.652706871886722*^9, 3.652706891518133*^9}, {3.652706966333794*^9, 
   3.652707083677209*^9}, {3.6527071926591578`*^9, 3.652707192782361*^9}, {
   3.652707381747512*^9, 3.652707407370923*^9}, {3.6567515284135427`*^9, 
   3.6567515545731792`*^9}, {3.662969465970572*^9, 3.662969477752174*^9}, {
   3.66297684224205*^9, 3.662976851097802*^9}, {3.662990328619638*^9, 
   3.662990396937199*^9}, {3.662991237103883*^9, 3.662991248831821*^9}, {
   3.662991403999537*^9, 3.662991406542947*^9}, 3.662991476359304*^9, {
   3.6629916123502607`*^9, 3.6629916277180634`*^9}, 3.663079693856637*^9, {
   3.663079870912787*^9, 3.663079888000586*^9}, 
   3.66308162139473*^9},ExpressionUUID->"c62f3dff-27bd-47c3-ba52-\
7a36622aa450"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla1", "[", 
   RowBox[{"\[CapitalGamma]", ",", "Y"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.652707321374048*^9, 3.652707336650268*^9}, 
   3.656751574388637*^9, {3.662990417695477*^9, 
   3.6629904240564528`*^9}},ExpressionUUID->"dc433e04-c252-4d80-abd5-\
68a7f48a853f"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6527074705244083`*^9, 
  3.652707470525223*^9}},ExpressionUUID->"ce2efe45-fe46-4899-8711-\
f07f76c9a690"],

Cell[TextData[{
 "From the definition of nabla1 it is immediately clear that the conditions \
1, 3, and 6 are fulfilled, conditions 2,4, and 5 can\[CloseCurlyQuote]t be \
applied to vector fields. It remains to show condition 5 for the product of \
functions with vector fields. We show\n",
 StyleBox["Lemma 1",
  FontColor->RGBColor[0, 0, 1]],
 ". \[Del]\[DotEqual] nabla1 satisfies the product rule for the product of a \
vector field ",
 "Y",
 " with a function f:"
}], "Text",
 CellChangeTimes->{{3.6527084468570337`*^9, 3.652708488224794*^9}, {
   3.652708522640815*^9, 3.652708729103401*^9}, {3.6527088283265944`*^9, 
   3.65270885068616*^9}, {3.6543433582650137`*^9, 3.6543433795828123`*^9}, 
   3.6544064457968407`*^9, 3.6563172227643003`*^9, 3.656751654300271*^9, {
   3.656751719372245*^9, 3.656751787540676*^9}, 3.656847388095695*^9, 
   3.656849074201438*^9, 
   3.656849263601019*^9},ExpressionUUID->"c8bdc31e-559c-42a4-bf5e-\
c9e338598b47"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Del]", 
   RowBox[{"[", 
    RowBox[{
     RowBox[{"f", "[", "x", "]"}], "*", 
     RowBox[{"Y", "[", "x", "]"}]}], "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", "f", "]"}], "[", "x", "]"}], "\[CircleTimes]", 
    RowBox[{"Y", "[", "x", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], "*", 
    RowBox[{
     RowBox[{"\[Del]", 
      RowBox[{"Y", "[", "x", "]"}]}], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.652708880470109*^9, 3.652709000581114*^9}, {
  3.656849108992024*^9, 3.656849117256371*^9}, {3.6568492652976313`*^9, 
  3.656849287239245*^9}},ExpressionUUID->"cf982ebb-ecb2-4de7-a21f-\
74c82bdf3440"],

Cell[BoxData[
 RowBox[{
  RowBox[{"fY", "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], "*", 
   RowBox[{"Y", "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.652709104434045*^9, 3.652709114483951*^9}, {
   3.652709147331842*^9, 3.6527091651878157`*^9}, {3.6527092425229692`*^9, 
   3.65270926745037*^9}, 3.656849218063991*^9, 
   3.656849268791772*^9},ExpressionUUID->"c087bf19-ecb0-4e9f-a14e-\
55ac99ea39ca"],

Cell[BoxData[
 RowBox[{"fY", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.652709278135688*^9, 3.652709285722692*^9}, 
   3.6568492707980967`*^9},ExpressionUUID->"73c79f09-e9d8-42cb-8491-\
5fe15ec21b81"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla1", "[", 
   RowBox[{"\[CapitalGamma]", ",", "fY"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.652709304642984*^9, 3.652709326577914*^9}, 
   3.65684927381532*^9, {3.6629919069630623`*^9, 
   3.662991910468495*^9}},ExpressionUUID->"7f728713-496e-49ad-a810-\
fe9757d1aae2"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.652709479022574*^9, 3.652709506888782*^9}, 
   3.663080807631967*^9},ExpressionUUID->"8899c57c-f6df-4842-ac73-\
d6596d9bb3bf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vY", "[", 
   RowBox[{"var", "[", "x_", "]"}], "]"}], " ", "=", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"Y", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.652710110864912*^9, 3.652710242363161*^9}, {
  3.6568496115479918`*^9, 3.656849652171155*^9}, {3.65684970724389*^9, 
  3.6568497188035088`*^9}},ExpressionUUID->"5913e5d5-8084-459d-aaf9-\
db701ac6ceac"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nabla0", "[", "f", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
  RowBox[{"vY", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.652710348713008*^9, 3.652710389273119*^9}, 
   3.656849748057096*^9, {3.662969696578822*^9, 3.6629697013269787`*^9}, 
   3.66296993763623*^9, {3.6629784061508207`*^9, 
   3.662978413110511*^9}},ExpressionUUID->"7e2ae6e0-a241-4311-be7d-\
358972894888"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.652710413299086*^9, 
  3.6527104325284843`*^9}},ExpressionUUID->"e4e44838-c7cd-470d-ba58-\
7a56b4208716"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla1", "[", 
     RowBox[{"\[CapitalGamma]", ",", "fY"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"nabla0", "[", "f", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
      RowBox[{"vY", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"nabla1", "[", 
        RowBox[{"\[CapitalGamma]", ",", "Y"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ")"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.652710477769946*^9, 3.652710481235405*^9}, {
   3.6527105121766567`*^9, 3.652710514376432*^9}, {3.652710579488647*^9, 
   3.652710617704105*^9}, 3.65321654297153*^9, 3.653483488562058*^9, {
   3.656849753130238*^9, 3.656849757475013*^9}, 3.662978459206092*^9, {
   3.662992100844733*^9, 3.662992130314926*^9}, 3.6630806262494593`*^9, 
   3.6630806706972313`*^9},ExpressionUUID->"03738c90-23be-44b0-9624-\
21f7c2427b5f"],

Cell["\<\
This equivalence gives in traditional notation the product rule\
\>", "Text",
 CellChangeTimes->{{3.663081103855184*^9, 3.663081154143532*^9}, 
   3.663081385408011*^9},ExpressionUUID->"f10a4007-4dac-4be9-a7bd-\
9e6a9eef2644"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Del]", 
   RowBox[{"[", 
    RowBox[{"f", "*", "Y"}], "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Del]", "f"}], "\[CircleTimes]", "Y"}], " ", "+", 
   RowBox[{"f", "*", 
    RowBox[{"\[Del]", "Y"}]}]}]}]], "Input",
 Evaluatable->False,ExpressionUUID->"9cb91603-6dbd-46df-8f15-016e7ef4a584"],

Cell["\<\
Now we calculate the absolute differential with respect to the tangent vector\
\>", "Text",
 CellChangeTimes->{{3.663081413816184*^9, 3.663081432016308*^9}, 
   3.6691121449086857`*^9, {3.702102996928423*^9, 
   3.702103060847973*^9}},ExpressionUUID->"7acb1531-f527-48b3-8fb1-\
7f0e3670f805"],

Cell[BoxData[
 RowBox[{"X", "=", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vec", "[", "\[Xi]", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65691176865387*^9, 
  3.65691180566323*^9}},ExpressionUUID->"dcdf1fd3-2c79-4ccd-8f34-\
3af86aeee0e3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nabla1", "[", 
    RowBox[{"\[CapitalGamma]", ",", "fY"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", "X"}]], "Input",
 CellChangeTimes->{{3.656911908162526*^9, 3.656911913182375*^9}, {
  3.662992170728245*^9, 
  3.662992173674335*^9}},ExpressionUUID->"c63a05e5-962f-4f0b-bcab-\
a5574721de13"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.656911940166418*^9, 3.656911973894012*^9}, 
   3.662970005940596*^9},ExpressionUUID->"16c08e41-fdcc-4f45-ba48-\
9dbbc1cab950"],

Cell[BoxData[
 RowBox[{"prab", "=", 
  RowBox[{"contraction", "[", 
   RowBox[{"%%", ",", "1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6569120376809807`*^9, 3.65691212181271*^9}, {
  3.656912205333468*^9, 
  3.6569122094120893`*^9}},ExpressionUUID->"9d34e733-0ed3-46b2-89ac-\
706250c8c06a"],

Cell[BoxData[
 RowBox[{"prab", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.656912139672464*^9, 3.656912153188158*^9}, {
  3.656912253996262*^9, 
  3.656912254891423*^9}},ExpressionUUID->"bf1d932b-f866-4070-bcdd-\
c4a8fb5bef8b"],

Cell[BoxData[
 RowBox[{"prs", "=", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"nabla0", "[", "f", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
       RowBox[{"vY", "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", "X"}], "+", 
      RowBox[{
       RowBox[{"times", "[", 
        RowBox[{
         RowBox[{"f", "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"nabla1", "[", 
           RowBox[{"\[CapitalGamma]", ",", "Y"}], "]"}], "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "\[CircleTimes]", 
       "X"}]}], ")"}], ",", "1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6569123709387293`*^9, 3.656912428938203*^9}, 
   3.6569124626514893`*^9, {3.656912590160633*^9, 3.656912611663484*^9}, {
   3.66299231981806*^9, 
   3.662992343041361*^9}},ExpressionUUID->"60eda08f-094e-4afa-ac6a-\
fdf565db830a"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"prab", ",", "prs"}], "]"}]], "Input",
 CellChangeTimes->{{3.656912486638316*^9, 
  3.656912504633698*^9}},ExpressionUUID->"df5d395b-51b0-4f6b-8c58-\
707dc15c24ef"],

Cell["The last equivalence corresponds in traditional notation to", "Text",
 CellChangeTimes->{{3.6569128442501802`*^9, 3.656912881697837*^9}, 
   3.65691861673586*^9, {3.6630815223125134`*^9, 
   3.663081541168645*^9}},ExpressionUUID->"e94c7ad4-ceac-4b12-9ccb-\
5003f750e2df"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], 
   RowBox[{"(", 
    RowBox[{"f", "*", "Y"}], ")"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["absD", "X"], 
    RowBox[{"(", "f", ")"}], "*", "Y"}], " ", "+", " ", 
   RowBox[{"f", "*", 
    SubscriptBox["absD", "X"], 
    RowBox[{"(", "Y", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6569128913170357`*^9, 3.656912961257217*^9}, {
  3.6569130070174847`*^9, 3.6569130190800943`*^9}, {3.6569131851018887`*^9, 
  3.656913212037579*^9}, {3.656913257869981*^9, 3.6569132721092253`*^9}, {
  3.669112246746128*^9, 
  3.669112250842227*^9}},ExpressionUUID->"74abf3c3-ba1e-4b4a-9ff0-\
be850bbfc8cf"],

Cell["\<\
which is the product rule 5 for the considered tensors. This shows Lemma 1. \
\[FilledSquare]
For the tensor product of a tensor of type (0, 0), where the function is the \
only component, with a tensor of type (0,1) the product rule will be treated \
later in a more general context.\
\>", "Text",
 CellChangeTimes->{{3.6527106543906603`*^9, 3.652710679284216*^9}, 
   3.65271073245391*^9, {3.653214751745233*^9, 3.653214869487979*^9}, {
   3.654346775761962*^9, 3.654346819409751*^9}, {3.6568505915071363`*^9, 
   3.656850800322785*^9}, {3.656913360122992*^9, 3.6569134167627897`*^9}, 
   3.656918661895883*^9, {3.702103215718075*^9, 
   3.7021032204872437`*^9}},ExpressionUUID->"a3a0675e-138d-4cb5-92e0-\
b8ef6cf3c9aa"],

Cell["\<\
The next step is to find a local expression for the absolute differential of \
a covector field \
\>", "Text",
 CellChangeTimes->{{3.6527106543906603`*^9, 3.652710679284216*^9}, 
   3.65271073245391*^9, {3.653214751745233*^9, 3.653214869487979*^9}, {
   3.654346775761962*^9, 3.654346819409751*^9}, {3.6568505915071363`*^9, 
   3.656850805571011*^9}},ExpressionUUID->"b4452612-4152-410e-851a-\
38cbe2a19456"],

Cell[BoxData["dim"], "Input",
 CellChangeTimes->{{3.6691199917306843`*^9, 
  3.6691199931973658`*^9}},ExpressionUUID->"1034435a-b973-441d-ab93-\
4f8745661be7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "w", "]"}], ";", 
  RowBox[{
   RowBox[{"w", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"vv", "[", "a", "]"}], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.653214901090413*^9, 3.65321495120072*^9}, {
  3.653214992415463*^9, 3.6532151021271772`*^9}, {3.662992707822968*^9, 
  3.662992769894197*^9}, {3.669119799896166*^9, 3.669119805599223*^9}, {
  3.6691200052061462`*^9, 
  3.669120019189351*^9}},ExpressionUUID->"716debf5-18ad-4cf1-899d-\
d242ce1e2e85"],

Cell[BoxData[
 RowBox[{"w", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.662993026017847*^9, 
  3.662993033083021*^9}},ExpressionUUID->"46d52936-462b-405c-8e7f-\
4aff2f173ccf"],

Cell["\<\
For an arbitrary vector field Y[x] we consider the function (in traditional \
notation)\
\>", "Text",
 CellChangeTimes->{{3.653215316836391*^9, 3.653215380763886*^9}, {
   3.653215686600903*^9, 3.6532157031128893`*^9}, 
   3.656850379879331*^9},ExpressionUUID->"c1f35e68-8467-4d22-a4cc-\
f4bae4ad6235"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ff", "[", "x", "]"}], " ", "=", " ", 
   RowBox[{"\[FormalCapitalY]", 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{"w", "[", "x", "]"}], "\[CircleTimes]", 
      RowBox[{"Y", "[", "x", "]"}]}], ")"}]}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.653215388412421*^9, 3.653215404811666*^9}, {
   3.653215549155059*^9, 3.653215617194066*^9}, {3.6532156484978113`*^9, 
   3.6532156623539543`*^9}, {3.653215732290238*^9, 3.653215742081232*^9}, 
   3.653215802289804*^9, {3.653216657617981*^9, 3.653216665680491*^9}, 
   3.656850381339911*^9, {3.6629932348029623`*^9, 3.662993246718689*^9}, {
   3.662993315529105*^9, 
   3.66299335204163*^9}},ExpressionUUID->"adbfa6cb-0d61-4f2a-9635-\
6a39d6e1d91e"],

Cell["\<\
where \[FormalCapitalY] denotes the contraction.  From conditions 1 - 6 it \
follows\
\>", "Text",
 CellChangeTimes->{{3.6532157552082987`*^9, 3.65321575839994*^9}, {
   3.653215808168795*^9, 3.653215828607627*^9}, {3.6532158692872667`*^9, 
   3.6532159145266953`*^9}, 3.653216196156205*^9, {3.653216690279772*^9, 
   3.653216700487844*^9}, 
   3.656850348197353*^9},ExpressionUUID->"c928ba60-bd95-40a6-8c5a-\
f9f399191a21"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["d", "X"], "[", "ff", "]"}], "[", "x", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["absD", "X"], "[", "ff", "]"}], "[", "x", "]"}], " ", "=", 
   RowBox[{
    RowBox[{"\[FormalCapitalY]", "[", 
     RowBox[{
      SubscriptBox["absD", "X"], 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{"w", "[", "x", "]"}], "\[CircleTimes]", 
        RowBox[{"Y", "[", "x", "]"}]}], ")"}]}], "]"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[FormalCapitalY]", "[", 
      RowBox[{
       SubscriptBox["absD", "X"], 
       RowBox[{
        RowBox[{"(", " ", 
         RowBox[{"w", "[", "x", "]"}], ")"}], "\[CircleTimes]", 
        RowBox[{"Y", "[", "x", "]"}]}]}], "]"}], "+", " ", 
     RowBox[{
      RowBox[{"\[FormalCapitalY]", "[", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"w", "[", "x", "]"}], "\[CircleTimes]", 
         SubscriptBox["absD", "X"]}], 
        RowBox[{"Y", "[", "x", "]"}]}], "]"}], "."}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.653215959559926*^9, 3.6532159762546873`*^9}, {
   3.653216033005919*^9, 3.6532160445338707`*^9}, {3.653216228732835*^9, 
   3.653216234276623*^9}, {3.653216321372836*^9, 3.653216346651238*^9}, 
   3.653216384819622*^9, {3.653216421818882*^9, 3.6532164772827463`*^9}, {
   3.653216509875463*^9, 3.6532165274184732`*^9}, {3.653216559906045*^9, 
   3.653216609265436*^9}, {3.6568503830374823`*^9, 3.656850386907267*^9}, {
   3.656913825987649*^9, 3.656913889942074*^9}, {3.656914045870542*^9, 
   3.656914097775282*^9}, {3.6655607470062943`*^9, 3.6655607555242643`*^9}, {
   3.669112260537191*^9, 3.669112260547681*^9}, 
   3.6691204660027*^9},ExpressionUUID->"5dad3c1f-bbb9-4fc8-83f2-37d31b814b74"],

Cell["\<\
In local coordinates these equations imply  (remember the sum convention,\
\>", "Text",
 CellChangeTimes->{{3.65321690761403*^9, 3.653216958309421*^9}, 
   3.653217021965187*^9, {3.653217903413844*^9, 3.6532179299177322`*^9}, {
   3.6629929615715714`*^9, 3.66299296762778*^9}, {3.6629930011177483`*^9, 
   3.6629930157277327`*^9}, {3.6629931484283953`*^9, 3.662993205969644*^9}, {
   3.662993385568474*^9, 3.6629933871237698`*^9}, {3.6629934352325993`*^9, 
   3.662993483368124*^9}, 3.662993530168269*^9, {3.662993607074091*^9, 
   3.6629936558711*^9}, {3.6629936963432817`*^9, 3.662993698593213*^9}, {
   3.6629938046972446`*^9, 
   3.6629938351957283`*^9}},ExpressionUUID->"eff78d44-7400-410b-90f5-\
c2e93d84673e"],

Cell[TextData[{
 Cell[BoxData[
  SubscriptBox["a", "j"]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6527092168731937`*^9, 3.652709230778975*^9}, {
    3.653217055427621*^9, 3.653217174112215*^9}, {3.653217232534514*^9, 
    3.653217237004012*^9}, {3.653217284467392*^9, 3.653217292411338*^9}, {
    3.653217342307827*^9, 3.653217386009572*^9}, {3.653217433362357*^9, 
    3.653217435907441*^9}, {3.65321779033324*^9, 3.6532178213274527`*^9}, 
    3.653217860463846*^9, 3.6532178934472723`*^9, {3.65321793872591*^9, 
    3.653217997398817*^9}, {3.656860217556101*^9, 3.656860220412565*^9}, {
    3.6569142868936567`*^9, 3.656914325141862*^9}, {3.656914373726487*^9, 
    3.6569143886379223`*^9}, {3.656914427117989*^9, 3.656914439686001*^9}},
  FontSize->12,
  FontWeight->"Plain",ExpressionUUID->"d296a234-5356-4976-9e31-5dd4861b8761"],
 StyleBox[" \[DotEqual] a[j][x[1], x[2], x[3]]",
  FontSize->12],
 ",  and ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["v", "j"], "\[DotEqual]", " ", 
   RowBox[{
    RowBox[{"\[Eta]", "[", "j", "]"}], "[", 
    RowBox[{
     RowBox[{"x", "[", "1", "]"}], ",", 
     RowBox[{"x", "[", "2", "]"}], ",", 
     RowBox[{"x", "[", "3", "]"}]}], "]"}]}]], "Input",ExpressionUUID->
  "3199a9a7-e24f-446b-9780-ce6e54060f5b"],
 "), "
}], "Text",
 CellChangeTimes->{{3.65321690761403*^9, 3.653216958309421*^9}, 
   3.653217021965187*^9, {3.653217903413844*^9, 3.6532179299177322`*^9}, {
   3.6629929615715714`*^9, 3.66299296762778*^9}, {3.6629930011177483`*^9, 
   3.6629930157277327`*^9}, {3.6629931484283953`*^9, 3.662993205969644*^9}, {
   3.662993385568474*^9, 3.6629933871237698`*^9}, {3.6629934352325993`*^9, 
   3.662993483368124*^9}, 3.662993530168269*^9, {3.662993607074091*^9, 
   3.6629936558711*^9}, {3.6629936963432817`*^9, 3.662993698593213*^9}, {
   3.6629938046972446`*^9, 3.662993868158457*^9}, {3.669181071343329*^9, 
   3.669181104123941*^9}},ExpressionUUID->"8b53e040-c0c5-44a2-ac16-\
b337743d521a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[PartialD]", "h"], 
        SubscriptBox["a", "j"]}], ")"}], 
      SuperscriptBox["v", "j"]}], "+", 
     RowBox[{
      SubscriptBox["a", "j"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "h"], 
       SuperscriptBox["v", "j"]}]}]}], ")"}], 
   SuperscriptBox["\[Xi]", "h"]}], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Del]", "h"], 
       RowBox[{"(", " ", 
        SubscriptBox["a", "j"], ")"}]}], 
      SuperscriptBox["\[Xi]", "h"]}], ")"}], 
    SuperscriptBox["v", "j"]}], " ", "+", 
   RowBox[{
    SubscriptBox["a", "j"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Del]", "h"], 
       SuperscriptBox["v", "j"]}], 
      SuperscriptBox["\[Xi]", "h"]}], ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6527092168731937`*^9, 3.652709230778975*^9}, {
   3.653217055427621*^9, 3.653217174112215*^9}, {3.653217232534514*^9, 
   3.653217237004012*^9}, {3.653217284467392*^9, 3.653217292411338*^9}, {
   3.653217342307827*^9, 3.653217386009572*^9}, {3.653217433362357*^9, 
   3.653217435907441*^9}, {3.65321779033324*^9, 3.6532178213274527`*^9}, 
   3.653217860463846*^9, 3.6532178934472723`*^9, {3.65321793872591*^9, 
   3.653217997398817*^9}, {3.656860217556101*^9, 3.656860220412565*^9}, {
   3.6569142868936567`*^9, 3.656914325141862*^9}, {3.656914373726487*^9, 
   3.6569143886379223`*^9}, {3.656914427117989*^9, 
   3.656914439686001*^9}},ExpressionUUID->"8496916b-82cb-4684-9845-\
fccd7fee13c2"],

Cell["\<\
Now we will use formula (2). Since the last equation is valid for all tangent \
vectors X, we conclude\
\>", "Text",
 CellChangeTimes->{{3.653218179741681*^9, 3.6532181973957157`*^9}, {
  3.653218229275133*^9, 3.653218246580207*^9}, {3.656914532901209*^9, 
  3.6569146314528427`*^9}, {3.703220227364108*^9, 
  3.7032202336037397`*^9}},ExpressionUUID->"fb4c4692-f636-4673-a455-\
b33c228d4284"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[PartialD]", "h"], 
      SubscriptBox["a", "j"]}], ")"}], 
    SuperscriptBox["v", "j"]}], "+", 
   RowBox[{
    SubscriptBox["a", "j"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "h"], 
     SuperscriptBox["v", "j"]}]}]}], "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[Del]", "h"], 
     RowBox[{"(", " ", 
      SubscriptBox["a", "j"], ")"}]}], 
    SuperscriptBox["v", "j"]}], " ", "+", 
   RowBox[{
    SubscriptBox["a", "j"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", "h"], 
       SuperscriptBox["v", "j"]}], "+", " ", 
      RowBox[{
       SuperscriptBox["v", "k"], 
       SubsuperscriptBox["\[CapitalGamma]", "hk", "j"]}]}], " ", 
     ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6527092168731937`*^9, 3.652709230778975*^9}, {
   3.653217055427621*^9, 3.653217174112215*^9}, {3.653217232534514*^9, 
   3.653217237004012*^9}, {3.653217284467392*^9, 3.653217292411338*^9}, {
   3.653217342307827*^9, 3.653217386009572*^9}, {3.653217433362357*^9, 
   3.653217435907441*^9}, {3.65321779033324*^9, 3.6532178213274527`*^9}, 
   3.653217860463846*^9, 3.6532178934472723`*^9, {3.65321793872591*^9, 
   3.653217997398817*^9}, {3.653218101518901*^9, 3.6532181215084057`*^9}, 
   3.65321816299684*^9, {3.653218362802144*^9, 3.6532184159062767`*^9}, {
   3.6532185011049147`*^9, 3.653218507728731*^9}, {3.653483526322016*^9, 
   3.653483526953677*^9}, {3.6568602214846363`*^9, 
   3.656860226684391*^9}},ExpressionUUID->"bba67c90-c4f1-4c47-866d-\
9c6deb5a1a42"],

Cell["and", "Text",
 CellChangeTimes->{{3.653218262378469*^9, 3.65321826771412*^9}, {
  3.6569146782365723`*^9, 
  3.6569146803805037`*^9}},ExpressionUUID->"5d6e120a-1921-4196-b2cf-\
3eb7cdefb8e4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "h"], 
    RowBox[{"(", " ", 
     SubscriptBox["a", "j"], ")"}]}], 
   SuperscriptBox["v", "j"]}], "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[PartialD]", "h"], 
      SubscriptBox["a", "j"]}], ")"}], 
    SuperscriptBox["v", "j"]}], "-", "  ", 
   RowBox[{
    SubscriptBox["a", "j"], 
    SuperscriptBox["v", "k"], 
    RowBox[{
     SubsuperscriptBox["\[CapitalGamma]", "hk", "j"], "."}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6532183176425037`*^9, 3.653218345978126*^9}, 
   3.6532184475070877`*^9, {3.653218535240794*^9, 3.6532185495542793`*^9}, {
   3.65321858106524*^9, 3.653218604624299*^9}, {3.653483545761414*^9, 
   3.65348354633739*^9}, {3.656860228235858*^9, 
   3.656860230521244*^9}},ExpressionUUID->"829a1ac2-e4c3-4936-a0c4-\
37fd07aa854e"],

Cell[TextData[{
 "In the last term of this equation we change the notation of the summation \
indices: k -> j, j->k, Since then the equation must be satisfied for \
arbitrary vector fields with coordinates ",
 Cell[BoxData[
  SuperscriptBox["v", "j"]], "Input",
  CellChangeTimes->{{3.6532183176425037`*^9, 3.653218345978126*^9}, 
    3.6532184475070877`*^9, {3.653218535240794*^9, 3.6532185495542793`*^9}, {
    3.65321858106524*^9, 3.653218604624299*^9}},ExpressionUUID->
  "b08f168b-c5f4-4990-a1fa-1514da821fdd"],
 " it follows\n\n",
 StyleBox["Lemma 2.",
  FontColor->RGBColor[0, 0, 1]],
 " If \[Del] is an absolute differential satisfying  the conditions 1 - 6 the \
local coordinates of the gradient tensor \[Del]w of a covector field w with \
coordinates ",
 Cell[BoxData[
  SubscriptBox["a", "j"]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6532183176425037`*^9, 3.653218345978126*^9}, 
    3.6532184475070877`*^9, {3.653218535240794*^9, 3.6532185495542793`*^9}, {
    3.65321858106524*^9, 3.653218604624299*^9}, {3.6532852186619797`*^9, 
    3.653285254365849*^9}, {3.653285291789157*^9, 3.653285322405327*^9}, {
    3.6532854670035343`*^9, 3.6532854849077797`*^9}, {3.653285548684759*^9, 
    3.653285583498811*^9}, {3.653285616627145*^9, 3.6532856705615063`*^9}},
  TextAlignment->Center,ExpressionUUID->
  "2d6b5270-849a-420d-a992-c861a7528816"],
 " are"
}], "Text",
 CellChangeTimes->{{3.6532845892503633`*^9, 3.653284654634203*^9}, {
   3.6532846948337812`*^9, 3.6532847348652678`*^9}, {3.6532847663612223`*^9, 
   3.653284934535967*^9}, {3.653284975159156*^9, 3.6532851060065947`*^9}, 
   3.653285500227488*^9, {3.653286132054192*^9, 3.6532861408136873`*^9}, {
   3.653286172568027*^9, 3.653286175189541*^9}, 3.654347038504456*^9, {
   3.656860232148609*^9, 3.656860232148649*^9}, {3.6569156828399057`*^9, 
   3.656915683415435*^9}},ExpressionUUID->"c8e5a626-6e06-43ad-9b02-\
8d7afec227eb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "h"], 
    SubscriptBox["a", "j"]}], "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "h"], 
     SubscriptBox["a", "j"]}], "-", "  ", 
    RowBox[{
     SubscriptBox["a", "k"], 
     SubsuperscriptBox["\[CapitalGamma]", "hj", "k"]}]}]}], ",", 
  "                            ", 
  RowBox[{"(", "3", ")"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6532183176425037`*^9, 3.653218345978126*^9}, 
   3.6532184475070877`*^9, {3.653218535240794*^9, 3.6532185495542793`*^9}, {
   3.65321858106524*^9, 3.653218604624299*^9}, {3.6532852186619797`*^9, 
   3.653285254365849*^9}, {3.653285291789157*^9, 3.653285322405327*^9}, {
   3.6532854670035343`*^9, 3.6532854849077797`*^9}, {3.653285548684759*^9, 
   3.653285583498811*^9}, {3.653285616627145*^9, 3.6532856705615063`*^9}, {
   3.653483590705222*^9, 3.6534835912893133`*^9}, {3.654347631354821*^9, 
   3.6543476402354393`*^9}, {3.65691477345295*^9, 3.65691477685256*^9}, 
   3.656915622975309*^9},
 TextAlignment->Center,ExpressionUUID->"ee156436-1c5e-4af9-9b99-4c5c1d56ca24"],

Cell[TextData[{
 "The corresponding  absolute differential is ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["absD", "X"], 
   RowBox[{"w", "[", "x", "]"}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.653215959559926*^9, 3.6532159762546873`*^9}, {
    3.653216033005919*^9, 3.6532160445338707`*^9}, {3.653216228732835*^9, 
    3.653216234276623*^9}, {3.653216321372836*^9, 3.653216346651238*^9}, 
    3.653216384819622*^9, {3.653216421818882*^9, 3.6532164772827463`*^9}, {
    3.653216509875463*^9, 3.6532165274184732`*^9}, {3.653216559906045*^9, 
    3.653216609265436*^9}, {3.6568503830374823`*^9, 3.656850386907267*^9}, {
    3.656913825987649*^9, 3.656913889942074*^9}, {3.656914045870542*^9, 
    3.656914097775282*^9}},ExpressionUUID->
  "57272ff4-d2a0-4d1b-9a93-86bcfa4daaa0"],
 " has the coordinates  ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Del]", "h"], 
   SubscriptBox["a", "j"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6532183176425037`*^9, 3.653218345978126*^9}, 
    3.6532184475070877`*^9, {3.653218535240794*^9, 3.6532185495542793`*^9}, {
    3.65321858106524*^9, 3.653218604624299*^9}, {3.6532852186619797`*^9, 
    3.653285254365849*^9}, {3.653285291789157*^9, 3.653285322405327*^9}, {
    3.6532854670035343`*^9, 3.6532854849077797`*^9}, {3.653285548684759*^9, 
    3.653285583498811*^9}, {3.653285616627145*^9, 3.6532856705615063`*^9}, {
    3.653483590705222*^9, 3.6534835912893133`*^9}, {3.654347631354821*^9, 
    3.6543476402354393`*^9}, {3.65691477345295*^9, 3.65691477685256*^9}},
  TextAlignment->Center,ExpressionUUID->
  "b2bc7dee-2364-42f4-859c-7255ac7930e6"],
 Cell[BoxData[
  SuperscriptBox["\[Xi]", "h"]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6527092168731937`*^9, 3.652709230778975*^9}, {
    3.653217055427621*^9, 3.653217174112215*^9}, {3.653217232534514*^9, 
    3.653217237004012*^9}, {3.653217284467392*^9, 3.653217292411338*^9}, {
    3.653217342307827*^9, 3.653217386009572*^9}, {3.653217433362357*^9, 
    3.653217435907441*^9}, {3.65321779033324*^9, 3.6532178213274527`*^9}, 
    3.653217860463846*^9, 3.6532178934472723`*^9, {3.65321793872591*^9, 
    3.653217997398817*^9}, {3.656860217556101*^9, 3.656860220412565*^9}, {
    3.6569142868936567`*^9, 3.656914325141862*^9}, {3.656914373726487*^9, 
    3.6569143886379223`*^9}, {3.656914427117989*^9, 3.656914439686001*^9}},
  ExpressionUUID->"eed4dc66-7d2e-47a7-926b-0ede4732587d"],
 ". ",
 "\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.656915346672845*^9, 3.65691540697655*^9}, {
  3.656915443700303*^9, 3.656915498699584*^9}, {3.656915563371421*^9, 
  3.65691564678192*^9}, {3.656915708190978*^9, 3.6569157081910257`*^9}, {
  3.6691122605682583`*^9, 
  3.669112260568294*^9}},ExpressionUUID->"262baec1-e56f-448d-9a11-\
8ad87faffadf"],

Cell["We continue the proof of the theorem in the next subsection.", "Text",
 CellChangeTimes->{{3.656921899788843*^9, 
  3.6569219305086*^9}},ExpressionUUID->"65fb4404-3f9a-44a1-8616-bb9f4fd3cffe"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.2.4. The Absolute Differential of a Tensor Field of Type (p, q)\
\>", "Subsubsection",
 CellChangeTimes->{{3.6568604419512453`*^9, 3.65686048624162*^9}, 
   3.656916790048291*^9, {3.6569216841598263`*^9, 3.65692174550215*^9}, 
   3.6569219946202507`*^9, 3.657262660708332*^9, {3.662993920022049*^9, 
   3.662993922669943*^9}, 3.6631591439305897`*^9, 3.669112260650052*^9, 
   3.669112515643536*^9, 3.669124454913671*^9, 3.669191359747057*^9, 
   3.702104147983664*^9, 3.702105674538538*^9, 3.703220825095845*^9, 
   3.703223135597801*^9, 3.704107759274082*^9, 3.704118297382864*^9, {
   3.708428171276835*^9, 3.708428173082209*^9}, 
   3.708437737275558*^9},ExpressionUUID->"ccea431e-814b-4fd9-8b8e-\
175ef1fecc27"],

Cell[TextData[{
 "For a given linear connection \[CapitalGamma] we introduce the ",
 StyleBox["connection forms",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.656922082723364*^9, 
  3.656922177290629*^9}},ExpressionUUID->"ce75c213-a9ba-4a1e-9643-\
084fbd86b056"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"cnf", "[", "\[Gamma]_", "]"}], "[", 
      RowBox[{"i_", ",", "j_"}], "]"}], "[", "x__", "]"}], "[", "v_", "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Not", "[", 
      RowBox[{
       RowBox[{"MemberQ", "[", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"Length", "[", 
           RowBox[{"{", "x", "}"}], "]"}], "]"}], ",", "i"}], "]"}], "&&", 
       RowBox[{"MemberQ", "[", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"Length", "[", 
           RowBox[{"{", "x", "}"}], "]"}], "]"}], ",", "j"}], "]"}]}], "]"}], 
     ",", " ", "\n", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<Invalid indices \>\"", ",", " ", "i", ",", " ", "\"\<, \>\"", ",",
         " ", "j", ",", " ", "\"\< !\>\""}], "]"}], ";", 
      RowBox[{"Return", "[", "]"}]}], ",", "\n", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Head", "[", "v", "]"}], "===", "List"}], ")"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", 
           RowBox[{"{", "x", "}"}], "]"}], " ", "==", " ", 
          RowBox[{"Length", "[", "v", "]"}]}], ")"}]}], ",", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Gamma]", "[", 
            RowBox[{"j", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}], "*", 
          RowBox[{"v", "[", 
           RowBox[{"[", "h", "]"}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"h", ",", 
           RowBox[{"Length", "[", 
            RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}], ",", "\n", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Invalid argument \>\"", ",", " ", "v", ",", " ", "\"\< !\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}]}], "]"}]}], ";"}]], "Input",
 Evaluatable->False,ExpressionUUID->"3a412b48-3824-4374-ba56-0bec986544c8"],

Cell[BoxData[
 RowBox[{"?", "cnf"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.656999002092044*^9, 3.656999058807818*^9}, {
   3.656999105023759*^9, 3.656999341550022*^9}, {3.6569994150870934`*^9, 
   3.656999467734548*^9}, 3.656999521070836*^9, {3.663312670364493*^9, 
   3.663312678763624*^9}},ExpressionUUID->"2dcc55b5-b1ce-4db9-b9e2-\
a87ef0081005"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cnf", "[", "\[CapitalGamma]", "]"}], "[", 
   RowBox[{"1", ",", "2"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.656942547593587*^9, 3.656942553540283*^9}, {
   3.656942672867462*^9, 3.656942697884145*^9}, {3.6569430477459183`*^9, 
   3.656943108024582*^9}, 3.6569437815407877`*^9, 3.656945163039654*^9, 
   3.656945215335672*^9, 3.663143818252393*^9, {3.704117444630773*^9, 
   3.70411745028582*^9}},ExpressionUUID->"3f78e6cc-3515-4c3b-8ffa-\
96856daa7ccf"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{"vec", "[", "u", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.656943704625729*^9, 3.6569437402514887`*^9}, {
   3.656943806315249*^9, 3.656943811947106*^9}, {3.656943848163574*^9, 
   3.656943889491647*^9}, {3.656943976594118*^9, 3.6569439819056387`*^9}, {
   3.656944414445957*^9, 3.656944455901882*^9}, 3.656945123984172*^9, 
   3.656945174775894*^9, 3.663144438502857*^9, 
   3.6633127441486*^9},ExpressionUUID->"8c8f0d2d-4e7b-4dd6-ad51-67829fbcf51d"],

Cell[BoxData[
 RowBox[{"%%", "[", 
  RowBox[{"stb", "[", 
   RowBox[{"3", ",", "3"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.656943704625729*^9, 3.6569437402514887`*^9}, {
   3.656943806315249*^9, 3.656943811947106*^9}, {3.656943848163574*^9, 
   3.656943889491647*^9}, {3.656943976594118*^9, 3.6569439819056387`*^9}, {
   3.656944414445957*^9, 3.656944455901882*^9}, 3.656945123984172*^9, 
   3.656945174775894*^9, {3.6631439291467943`*^9, 
   3.663143979145918*^9}},ExpressionUUID->"ae63c8d8-6cb2-4362-bc6e-\
826c50255f9a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"cnf", "[", "\[CapitalGamma]", "]"}], "[", 
     RowBox[{"1", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"vec", "[", "u", "]"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"2", ",", "1", ",", "1"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}], ",", 
       RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"2", ",", "2", ",", "1"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}], ",", 
       RowBox[{"x", "[", "3", "]"}]}], "]"}], ",", 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"2", ",", "3", ",", "1"}], "]"}], "[", 
      RowBox[{
       RowBox[{"x", "[", "1", "]"}], ",", 
       RowBox[{"x", "[", "2", "]"}], ",", 
       RowBox[{"x", "[", "3", "]"}]}], "]"}]}], "}"}], ".", 
   RowBox[{"vec", "[", "u", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6631443251129913`*^9, 3.663144421254478*^9}, {
  3.663144451461979*^9, 3.663144508957973*^9}, {3.702104468390195*^9, 
  3.70210447667636*^9}, {3.704117493446473*^9, 
  3.7041175040375843`*^9}},ExpressionUUID->"e578d9b3-3e52-49b8-a059-\
598399c90b1d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cnf", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"1", ",", "4"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"vec", "[", "u", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.656942547593587*^9, 3.656942553540283*^9}, {
   3.656942672867462*^9, 3.656942697884145*^9}, {3.6569430477459183`*^9, 
   3.6569431405280943`*^9}, {3.656943172832436*^9, 3.6569431839999733`*^9}, 
   3.656944535341311*^9, {3.704117532582066*^9, 
   3.704117538444725*^9}},ExpressionUUID->"1a360956-0385-419f-bba1-\
3bd7dbca109b"],

Cell["In traditional notation we write", "Text",
 CellChangeTimes->{{3.656999782786672*^9, 
  3.6569997960905533`*^9}},ExpressionUUID->"24277c02-25f5-4d9f-acc4-\
d2559629625d"],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Omega]", "i", 
   RowBox[{" ", "j"}]], " ", "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"cnf", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"i", ",", "j"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6569998175910053`*^9, 3.656999928458393*^9}, 
   3.6570000110819407`*^9, 3.657000444813261*^9, {3.6691847835458813`*^9, 
   3.6691847904824123`*^9}, {3.704117565155775*^9, 
   3.70411757146104*^9}},ExpressionUUID->"ab8995c2-d2c9-4693-9779-\
9d704a6da185"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"cnf", "[", "\[CapitalGamma]", "]"}], "[", 
   RowBox[{"i", ",", "j"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7041175874848967`*^9, 
  3.704117592676508*^9}},ExpressionUUID->"57339c28-fa15-4c3c-8ab6-\
f5ed39ebe126"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cnf", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"1", ",", "2"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"tx\[Iota]", "[", "2", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.669184999705647*^9, 3.6691850013603287`*^9}, {
   3.6691850402337217`*^9, 3.669185045520657*^9}, {3.669185122863779*^9, 
   3.669185149151894*^9}, {3.6691852300479*^9, 3.669185280527421*^9}, {
   3.669185322423849*^9, 3.66918537091064*^9}, {3.669185491606353*^9, 
   3.6691854925812674`*^9}, {3.669185539518148*^9, 3.66918557471741*^9}, {
   3.669190099006126*^9, 3.669190118445725*^9}, 3.6691901695339603`*^9, {
   3.7041176240692244`*^9, 
   3.7041176322121763`*^9}},ExpressionUUID->"62fade79-c796-4c37-b6f1-\
ef2e70bb67ee"],

Cell["\<\
Then the absolute differentials of vector and covector fields can be \
traditionally notated as\
\>", "Text",
 CellChangeTimes->{{3.657000030441176*^9, 
  3.6570001182725677`*^9}},ExpressionUUID->"55f1fdea-0df5-4b95-b54c-\
b338b084af33"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], "Y"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["d", "X"], 
      SuperscriptBox["v", "i"]}], "+", " ", 
     RowBox[{
      SuperscriptBox["v", "s"], 
      RowBox[{
       SubsuperscriptBox["\[Omega]", "s", 
        RowBox[{" ", "i"}]], "[", "X", "]"}]}]}], ")"}], 
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], "x", "  "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.657000131311385*^9, 3.657000324878745*^9}, {
   3.657000388142742*^9, 3.657000430861926*^9}, {3.65700054726746*^9, 
   3.657000558180908*^9}, {3.6570010928003893`*^9, 3.657001114048114*^9}, 
   3.657001154775881*^9, 3.6571780487393923`*^9, 
   3.669112260708768*^9},ExpressionUUID->"46520824-e62e-4ffc-b7c6-\
622bb909ee9b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], "u"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["d", "X"], 
      SubscriptBox["u", "i"]}], "-", " ", 
     RowBox[{
      SubscriptBox["u", "s"], 
      RowBox[{
       SubsuperscriptBox["\[Omega]", "i", 
        RowBox[{" ", "s"}]], "[", "X", "]"}]}]}], ")"}], 
   SuperscriptBox["dx", "i"]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.657000131311385*^9, 3.657000324878745*^9}, {
   3.657000388142742*^9, 3.657000430861926*^9}, {3.65700054726746*^9, 
   3.657000558180908*^9}, {3.657000603836319*^9, 3.657000734434884*^9}, 
   3.669112260747731*^9},ExpressionUUID->"33b01767-18c9-4406-afa4-\
11c9e19170a8"],

Cell["\<\
We apply these formulas to the basis decompositions of the base vectors and \
covectors themselves\
\>", "Text",
 CellChangeTimes->{{3.65700116635942*^9, 3.657001242878365*^9}, 
   3.657001298880439*^9},ExpressionUUID->"0b370000-fef4-4368-857f-\
b43d8e25fda3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], "x"}], "  ", "\[Equal]", "  ", 
   RowBox[{
    SubsuperscriptBox["\[Delta]", "j", "i"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], "x"}]}]}], " ", ",", "     ", 
  RowBox[{
   SuperscriptBox["dx", "j"], " ", "\[Equal]", "  ", 
   RowBox[{
    SubsuperscriptBox["\[Delta]", "i", "j"], 
    SuperscriptBox["dx", "i"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.657001388846099*^9, 3.657001586997802*^9}, {
  3.6570022349033737`*^9, 
  3.657002252423661*^9}},ExpressionUUID->"e043ced8-7ca0-4720-95bc-\
f8aa2aec8448"],

Cell[TextData[{
 "Since the coordinates are constant, we obtain\n",
 StyleBox["Lemma 3",
  FontColor->RGBColor[0, 0, 1]],
 ". In the  linear connection with the connection forms ",
 Cell[BoxData[
  SubsuperscriptBox["\[Omega]", "j", 
   RowBox[{" ", "i"}]]], "Input",
  CellChangeTimes->{{3.657177888406492*^9, 3.657177929387047*^9}, {
   3.657178022162005*^9, 3.657178024491088*^9}, {3.657178085713378*^9, 
   3.657178099553705*^9}, {3.657178133024953*^9, 3.657178203736478*^9}},
  ExpressionUUID->"c0ba2db4-6c4a-49c2-b0b3-537a3aa149b9"],
 " the absolute differentials of the basis vector fields and covector fields \
are"
}], "Text",
 CellChangeTimes->{{3.6571778295956383`*^9, 3.65717785222726*^9}, {
  3.657179388101717*^9, 3.657179390302927*^9}, {3.657179428061756*^9, 
  3.6571795008451*^9}},ExpressionUUID->"779f7c7c-15eb-43e9-a21e-da75cd1c2f1c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["absD", "X"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "j"], "x"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    SubsuperscriptBox["\[Delta]", "j", "s"], 
    RowBox[{
     SubsuperscriptBox["\[Omega]", "s", 
      RowBox[{" ", "i"}]], "[", "X", "]"}], 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], "x"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Omega]", "j", 
      RowBox[{" ", "i"}]], "[", "X", "]"}], 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], "x"}]}]}], ",", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["absD", "X"], " ", 
    SuperscriptBox["dx", "j"]}], "\[Equal]", " ", 
   RowBox[{
    RowBox[{"-", 
     SubsuperscriptBox["\[Delta]", "s", "j"]}], 
    RowBox[{
     SubsuperscriptBox["\[Omega]", "i", 
      RowBox[{" ", "s"}]], "[", "X", "]"}], 
    SuperscriptBox["dx", "i"]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      SubsuperscriptBox["\[Omega]", "i", 
       RowBox[{" ", "j"}]], "[", "X", "]"}]}], 
    SuperscriptBox["dx", "i"], "   ", "\[FilledSquare]", "            ", 
    RowBox[{"(", "4", ")"}], "                         "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.657177888406492*^9, 3.657177929387047*^9}, {
   3.657178022162005*^9, 3.657178024491088*^9}, {3.657178085713378*^9, 
   3.657178099553705*^9}, {3.657178133024953*^9, 3.657178203736478*^9}, {
   3.657179522460808*^9, 3.6571795686687317`*^9}, {3.6571796170204782`*^9, 
   3.657179630904572*^9}, 3.657179667624715*^9, {3.6691122607587833`*^9, 
   3.669112260759914*^9}, {3.702104798625983*^9, 
   3.7021048104768353`*^9}},ExpressionUUID->"2043de11-dd63-43bd-aae0-\
76260e9b16d3"],

Cell[TextData[{
 "Now we remember the coordinate representation of a tensor ta[x] of\ntype \
(p, q) with respect to the local basis {",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[PartialD]", "j"], "x"}]], "Input",
  CellChangeTimes->{{3.65364494817089*^9, 3.653645047846068*^9}, 
    3.653645082374243*^9, {3.653645120069531*^9, 3.653645151076767*^9}, {
    3.65364518605205*^9, 3.653645224916737*^9}, {3.6536452594202757`*^9, 
    3.653645289364173*^9}, {3.653645724501967*^9, 3.653645777368388*^9}, {
    3.6536458115597486`*^9, 3.653645833727903*^9}, {3.653645879362185*^9, 
    3.653645901200007*^9}, {3.6536459332874413`*^9, 3.6536460054596*^9}, {
    3.653646221581079*^9, 3.65364630906019*^9}},ExpressionUUID->
  "73e42016-125c-4b9d-9d2c-17445a6aee75"],
 "} and cobasis {",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["dx", "i"], "}"}]], "Input",
  CellChangeTimes->{{3.65364494817089*^9, 3.653645047846068*^9}, 
    3.653645082374243*^9, {3.653645120069531*^9, 3.653645151076767*^9}, {
    3.65364518605205*^9, 3.653645224916737*^9}, {3.6536452594202757`*^9, 
    3.653645289364173*^9}, {3.653645724501967*^9, 3.653645777368388*^9}, {
    3.6536458115597486`*^9, 3.653645833727903*^9}, {3.653645879362185*^9, 
    3.653645901200007*^9}, {3.6536459332874413`*^9, 3.6536460054596*^9}, {
    3.653646221581079*^9, 3.65364630906019*^9}},ExpressionUUID->
  "aced308e-4e96-42d7-915d-506d3cb1919e"],
 " at the point x corresponding to a chart:"
}], "Text",
 CellChangeTimes->{{3.6536329066450977`*^9, 3.653632994347782*^9}, {
   3.653644922694639*^9, 3.653644939206197*^9}, {3.653646333227131*^9, 
   3.653646333987255*^9}, {3.653646374522925*^9, 3.6536464732537603`*^9}, {
   3.6536465034293413`*^9, 3.653646559441558*^9}, 3.6536466502254353`*^9, 
   3.657178859914407*^9, {3.702104842537756*^9, 
   3.7021048499774733`*^9}},ExpressionUUID->"926a1097-fa2e-438b-873c-\
638470094b9f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ta", "[", "x", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox[
         SubscriptBox["a", 
          RowBox[{
           RowBox[{
            SubscriptBox["i", "1"], "..."}], 
           SubscriptBox["i", "p"]}]], 
         RowBox[{
          RowBox[{
           SubscriptBox["j", "1"], "..."}], 
          SubscriptBox["j", "q"]}]], "[", "x", "]"}], 
       RowBox[{
        SuperscriptBox["dx", 
         SubscriptBox["i", "1"]], "\[CircleTimes]"}]}], "..."}], 
     "\[CircleTimes]", 
     SuperscriptBox["dx", 
      SubscriptBox["i", "p"]], "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
   "\[CirclePlus]", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     SubscriptBox["j", "q"]], "x"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.65364494817089*^9, 3.653645047846068*^9}, 
   3.653645082374243*^9, {3.653645120069531*^9, 3.653645151076767*^9}, {
   3.65364518605205*^9, 3.653645224916737*^9}, {3.6536452594202757`*^9, 
   3.653645289364173*^9}, {3.653645724501967*^9, 3.653645777368388*^9}, {
   3.6536458115597486`*^9, 3.653645833727903*^9}, {3.653645879362185*^9, 
   3.653645901200007*^9}, {3.6536459332874413`*^9, 3.6536460054596*^9}, {
   3.653646221581079*^9, 3.65364630906019*^9}, {3.653646604633304*^9, 
   3.653646611633601*^9}, {3.657178887754752*^9, 
   3.657178892554873*^9}},ExpressionUUID->"93d78259-2c1d-4418-9791-\
fcb303d68626"],

Cell["\<\
Assuming conditions 1 - 6 of the theorem 3.2.3 the absolute absDifferential is\
\>", "Text",
 CellChangeTimes->{{3.65364691576602*^9, 3.653646998909822*^9}, {
   3.653800060851028*^9, 3.653800064098741*^9}, 3.654347502435828*^9, {
   3.6571789362988167`*^9, 3.6571789556811943`*^9}, {3.657179894489493*^9, 
   3.6571798979133263`*^9}, 
   3.669112260779313*^9},ExpressionUUID->"232a30b0-6e0c-4a74-a0eb-\
4bea6de2e490"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], 
   RowBox[{"ta", "[", "x", "]"}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SubscriptBox["d", "X"], "[", 
         SuperscriptBox[
          SubscriptBox["a", 
           RowBox[{
            RowBox[{
             SubscriptBox["i", "1"], "..."}], 
            SubscriptBox["i", "p"]}]], 
          RowBox[{
           RowBox[{
            SubscriptBox["j", "1"], "..."}], 
           SubscriptBox["j", "q"]}]], "]"}], "\[CircleTimes]", 
        SuperscriptBox["dx", 
         SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
      "\[CircleTimes]", 
      SuperscriptBox["dx", 
       SubscriptBox["i", "p"]], "\[CircleTimes]", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
    "\[CirclePlus]", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      SubscriptBox["j", "q"]], "x"}]}], " ", "+", " ", "\[IndentingNewLine]", 
   RowBox[{
    SuperscriptBox[
     SubscriptBox["a", 
      RowBox[{
       RowBox[{
        SubscriptBox["i", "1"], "..."}], 
       SubscriptBox["i", "p"]}]], 
     RowBox[{
      RowBox[{
       SubscriptBox["j", "1"], "..."}], 
      SubscriptBox["j", "q"]}]], 
    RowBox[{
     SubscriptBox["absD", "X"], "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["dx", 
           SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
        "\[CircleTimes]", 
        SuperscriptBox["dx", 
         SubscriptBox["i", "p"]], "\[CircleTimes]", 
        RowBox[{
         SubscriptBox["\[PartialD]", 
          SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
      "\[CirclePlus]", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["j", "q"]], "x"}]}], "]"}], "                        ", 
    RowBox[{"(", "5", ")"}], "            "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.65364494817089*^9, 3.653645047846068*^9}, 
   3.653645082374243*^9, {3.653645120069531*^9, 3.653645151076767*^9}, {
   3.65364518605205*^9, 3.653645224916737*^9}, {3.6536452594202757`*^9, 
   3.653645289364173*^9}, {3.653645724501967*^9, 3.653645777368388*^9}, {
   3.6536458115597486`*^9, 3.653645833727903*^9}, {3.653645879362185*^9, 
   3.653645901200007*^9}, {3.6536459332874413`*^9, 3.6536460054596*^9}, {
   3.653646221581079*^9, 3.65364630906019*^9}, {3.653646604633304*^9, 
   3.653646611633601*^9}, {3.653647116877762*^9, 3.653647214997335*^9}, {
   3.653647285420951*^9, 3.65364730474063*^9}, {3.653798925187096*^9, 
   3.653798939483246*^9}, {3.657179036009933*^9, 3.6571791029296417`*^9}, 
   3.657179713532518*^9, {3.657204044212661*^9, 3.6572040832604723`*^9}, {
   3.669112260848542*^9, 
   3.669112260849785*^9}},ExpressionUUID->"6217052a-65e5-4b49-b594-\
3ed58464e7eb"],

Cell[TextData[{
 "To evaluate this formula we have to calculate the absolute differential ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["absD", "X"], "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["dx", 
         SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
      "\[CircleTimes]", 
      SuperscriptBox["dx", 
       SubscriptBox["i", "p"]], "\[CircleTimes]", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
    "\[CirclePlus]", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      SubscriptBox["j", "q"]], "x"}]}], "]"}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.65364494817089*^9, 3.653645047846068*^9}, 
    3.653645082374243*^9, {3.653645120069531*^9, 3.653645151076767*^9}, {
    3.65364518605205*^9, 3.653645224916737*^9}, {3.6536452594202757`*^9, 
    3.653645289364173*^9}, {3.653645724501967*^9, 3.653645777368388*^9}, {
    3.6536458115597486`*^9, 3.653645833727903*^9}, {3.653645879362185*^9, 
    3.653645901200007*^9}, {3.6536459332874413`*^9, 3.6536460054596*^9}, {
    3.653646221581079*^9, 3.65364630906019*^9}, {3.653646604633304*^9, 
    3.653646611633601*^9}, {3.653647116877762*^9, 3.653647214997335*^9}, {
    3.653647285420951*^9, 3.65364730474063*^9}},ExpressionUUID->
  "435ca1b0-9950-4f1b-94c1-c0fa12889f06"],
 " of the local basis tensor fields. "
}], "Text",
 CellChangeTimes->{{3.6536475704952393`*^9, 3.653647672624339*^9}, {
   3.65364818487576*^9, 3.653648199339631*^9}, {3.653648243356084*^9, 
   3.653648257139257*^9}, {3.653648313066887*^9, 3.653648524457267*^9}, {
   3.653648562541026*^9, 3.653648593281302*^9}, {3.6536487860877333`*^9, 
   3.6536488923661833`*^9}, {3.653648935037939*^9, 3.6536489966853657`*^9}, 
   3.6536508117993727`*^9, {3.6536589060498962`*^9, 3.653658920681649*^9}, {
   3.653658967488748*^9, 3.653658989240625*^9}, 3.653796861339964*^9, {
   3.654347587763624*^9, 3.654347593165537*^9}, {3.657179240747678*^9, 
   3.6571793033832293`*^9}, {3.657179740195023*^9, 3.6571797432427692`*^9}, {
   3.669112261042287*^9, 
   3.669112261042328*^9}},ExpressionUUID->"b1711eb8-44c5-4fc9-80df-\
e68840e7d203"],

Cell["Using the product rule 5 we get", "Text",
 CellChangeTimes->{{3.6537971879769*^9, 
  3.653797228312728*^9}},ExpressionUUID->"8ec42df4-7d4f-4e11-aacb-\
dc71c90fe48e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["dx", 
         SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
      "\[CircleTimes]", 
      SuperscriptBox["dx", 
       SubscriptBox["i", "p"]], "\[CircleTimes]", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
    "\[CircleTimes]", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      SubscriptBox["j", "q"]], "x"}]}], "]"}], "=", " ", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  UnderoverscriptBox["\[Sum]", 
                   RowBox[{"\[Alpha]", "=", "1"}], "p"], 
                  RowBox[{
                   SuperscriptBox["dx", 
                    SubscriptBox["i", "1"]], "\[CircleTimes]"}]}], "..."}], 
                "\[CircleTimes]", 
                SuperscriptBox["dx", 
                 SubscriptBox["i", 
                  RowBox[{"\[Alpha]", "-", "1"}]]], "\[CircleTimes]", 
                RowBox[{
                 SubscriptBox["absD", "X"], "[", 
                 SuperscriptBox["dx", 
                  SubscriptBox["i", "\[Alpha]"]], "]"}], "\[CircleTimes]", 
                SuperscriptBox["dx", 
                 SubscriptBox["i", 
                  RowBox[{"\[Alpha]", "+", "1"}]]], "\[CircleTimes]"}], 
               "..."}], "\[CircleTimes]", 
              SuperscriptBox["dx", 
               SubscriptBox["i", "p"]]}], ")"}], "\[CircleTimes]", 
            RowBox[{
             SubscriptBox["\[PartialD]", 
              SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
          "\[CircleTimes]", 
          RowBox[{
           SubscriptBox["\[PartialD]", 
            SubscriptBox["j", "q"]], "x"}]}], " ", "+", "\[IndentingNewLine]", 
         RowBox[{
          SuperscriptBox["dx", 
           SubscriptBox["i", "1"]], "\[CircleTimes]"}]}], "..."}], 
       "\[CircleTimes]", 
       SuperscriptBox["dx", 
        SubscriptBox["i", "p"]], "\[CircleTimes]", " ", 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"\[Beta]", "=", "1"}], "q"], 
        RowBox[{
         RowBox[{
          SubscriptBox["\[PartialD]", 
           SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}]}]}], "..."}], 
     "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["j", 
        RowBox[{"\[Beta]", "-", "1"}]]], "x"}], " ", "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["absD", "X"], "[", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["j", "\[Beta]"]], "x"}], " ", "]"}], "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["j", 
        RowBox[{"\[Beta]", "+", "1"}]]], "x"}], " ", "\[CircleTimes]"}], 
    "..."}], "\[CircleTimes]", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     SubscriptBox["j", "q"]], "x", " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.669112261065875*^9, 3.669112261071932*^9}, {
  3.669190468763735*^9, 
  3.6691905175390663`*^9}},ExpressionUUID->"15326de1-eca2-4d30-8bc4-\
791706042f15"],

Cell["Inserting the expressions (4) into this formula we get", "Text",
 CellChangeTimes->{{3.653798710195958*^9, 3.653798858922676*^9}, {
   3.653799032865518*^9, 3.653799204447706*^9}, {3.653799241927559*^9, 
   3.653799283863227*^9}, 3.653799496627665*^9, {3.653799549442841*^9, 
   3.6537998123269176`*^9}, {3.653799958964963*^9, 3.653800019476101*^9}, {
   3.653800086707531*^9, 3.653800122403183*^9}, {3.653800152627008*^9, 
   3.653800258976536*^9}, {3.65380057034256*^9, 3.6538006989738693`*^9}, {
   3.6538022667654877`*^9, 3.6538022878455753`*^9}, {3.654349190240724*^9, 
   3.654349256693667*^9}, {3.6544066241576853`*^9, 3.654406632702991*^9}, {
   3.654407536563085*^9, 3.6544075591255007`*^9}, 3.6544077374987907`*^9, 
   3.654408127689076*^9, 3.654408168797821*^9, {3.654417561944024*^9, 
   3.654417753237488*^9}, 3.657180066373914*^9, 3.657180133841015*^9, {
   3.657181208926897*^9, 
   3.657181210910261*^9}},ExpressionUUID->"77d5d27d-d326-443d-aae5-\
7e9654af1f75"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["dx", 
         SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
      "\[CircleTimes]", 
      SuperscriptBox["dx", 
       SubscriptBox["i", "p"]], "\[CircleTimes]", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
    "\[CircleTimes]", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      SubscriptBox["j", "q"]], "x"}]}], "]"}], "=", " ", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   UnderoverscriptBox["\[Sum]", 
                    RowBox[{"\[Alpha]", "=", "1"}], "p"], 
                   RowBox[{
                    RowBox[{
                    SubsuperscriptBox["\[Omega]", "i", 
                    RowBox[{" ", 
                    SubscriptBox["i", "\[Alpha]"]}]], "[", "X", "]"}], 
                    RowBox[{
                    SuperscriptBox["dx", 
                    SubscriptBox["i", "1"]], "\[CircleTimes]"}]}]}], "..."}], 
                 "\[CircleTimes]", 
                 SuperscriptBox["dx", 
                  SubscriptBox["i", 
                   RowBox[{"\[Alpha]", "-", "1"}]]], "\[CircleTimes]", 
                 SuperscriptBox["dx", "i"], "\[CircleTimes]", 
                 SuperscriptBox["dx", 
                  SubscriptBox["i", 
                   RowBox[{"\[Alpha]", "+", "1"}]]], "\[CircleTimes]"}], 
                "..."}], "\[CircleTimes]", 
               SuperscriptBox["dx", 
                SubscriptBox["i", "p"]]}], ")"}]}], "\[CircleTimes]", 
            RowBox[{
             SubscriptBox["\[PartialD]", 
              SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], "..."}], 
          "\[CircleTimes]", 
          RowBox[{
           SubscriptBox["\[PartialD]", 
            SubscriptBox["j", "q"]], "x"}]}], " ", "+", "\[IndentingNewLine]", 
         RowBox[{
          SuperscriptBox["dx", 
           SubscriptBox["i", "1"]], "\[CircleTimes]"}]}], "..."}], 
       "\[CircleTimes]", 
       SuperscriptBox["dx", 
        SubscriptBox["i", "p"]], "\[CircleTimes]", " ", 
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"\[Beta]", "=", "1"}], "q"], 
        RowBox[{
         RowBox[{
          SubsuperscriptBox["\[Omega]", 
           SubscriptBox["j", "\[Beta]"], 
           RowBox[{" ", "i"}]], "[", "X", "]"}], 
         RowBox[{
          RowBox[{
           SubscriptBox["\[PartialD]", 
            SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}]}]}]}], "..."}],
      "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["j", 
        RowBox[{"\[Beta]", "-", "1"}]]], "x"}], " ", "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["\[PartialD]", "i"], "x"}], " ", "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["j", 
        RowBox[{"\[Beta]", "+", "1"}]]], "x"}], " ", "\[CircleTimes]"}], 
    "..."}], "\[CircleTimes]", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      SubscriptBox["j", "q"]], "x"}], "  ", ".", "   "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6572035460238857`*^9, {3.6572036122724237`*^9, 3.657203747999387*^9}, {
   3.657203877063265*^9, 3.657203941174062*^9}, 3.657205029638899*^9, {
   3.6572063604682198`*^9, 3.657206363708749*^9}, 3.669112261081253*^9, {
   3.669190640361765*^9, 
   3.669190669769896*^9}},ExpressionUUID->"42677f9d-5329-4b6d-b58e-\
f2cc4c69d7b3"],

Cell["\<\
Now we insert the last result into formula (5) and consider the arising terms \
in detail. The terms of the first sum are\
\>", "Text",
 CellChangeTimes->{{3.657204695167507*^9, 3.657204770143017*^9}, {
  3.6572049108382072`*^9, 3.657204943077932*^9}, {3.669190751872191*^9, 
  3.6691907585680723`*^9}},ExpressionUUID->"35afb994-3d31-4bdd-bbd9-\
0b55bff41554"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          SubscriptBox["a", 
           RowBox[{
            RowBox[{
             SubscriptBox["i", "1"], ".."}], 
            RowBox[{
             SubscriptBox["i", "\[Alpha]"], ".", 
             SubscriptBox["i", "p"]}]}]], 
          RowBox[{
           RowBox[{
            SubscriptBox["j", "1"], "..."}], 
           SubscriptBox["j", "q"]}]]}], 
        SubsuperscriptBox["\[Omega]", "i", 
         RowBox[{" ", 
          SubscriptBox["i", "\[Alpha]"]}]], 
        RowBox[{
         SuperscriptBox["dx", 
          SubscriptBox["i", "1"]], "\[CircleTimes]"}]}], ".."}], 
      "\[CircleTimes]", 
      SuperscriptBox["dx", 
       SubscriptBox["i", 
        RowBox[{"\[Alpha]", "-", "1"}]]], "\[CircleTimes]", 
      SuperscriptBox["dx", "i"], "\[CircleTimes]", 
      SuperscriptBox["dx", 
       SubscriptBox["i", 
        RowBox[{"\[Alpha]", "+", "1"}]]], "\[CircleTimes]"}], ".."}], 
    "\[CircleTimes]", 
    SuperscriptBox["dx", 
     SubscriptBox["i", "p"]], "\[CircleTimes]", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], ".."}], 
  "\[CircleTimes]", 
  RowBox[{
   SubscriptBox["\[PartialD]", 
    SubscriptBox["j", "q"]], "x"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.657205054830636*^9, {3.657205092110668*^9, 
   3.657205176316752*^9}},ExpressionUUID->"8e65af67-3d81-48f8-a7c8-\
76c4d1c52d3f"],

Cell[TextData[{
 "Renaming in each of these terms the summation indices  ",
 Cell[BoxData[
  SubscriptBox["i", "\[Alpha]"]], "Input",
  CellChangeTimes->{
   3.657205054830636*^9, {3.657205092110668*^9, 3.657205176316752*^9}},
  ExpressionUUID->"15888fd5-d259-4116-8007-fda140f9e434"],
 " -> i, i -> ",
 Cell[BoxData[
  SubscriptBox["i", "\[Alpha]"]], "Input",
  CellChangeTimes->{
   3.657205054830636*^9, {3.657205092110668*^9, 3.657205176316752*^9}},
  ExpressionUUID->"d18b0f15-33a1-492b-84b6-222d9b5b187c"],
 ",  we get in each term the same tensor base elements "
}], "Text",
 CellChangeTimes->{{3.657205231691882*^9, 3.6572052750114098`*^9}, {
   3.657205309694812*^9, 3.657205337811718*^9}, {3.657205372450796*^9, 
   3.657205386706637*^9}, {3.657205517705611*^9, 3.657205549953369*^9}, 
   3.6572055995370293`*^9},ExpressionUUID->"5e82926a-444b-4cfc-a81d-\
95ace259d95a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["dx", 
          SubscriptBox["i", "1"]], "\[CircleTimes]"}], ".."}], 
       "\[CircleTimes]", 
       SuperscriptBox["dx", 
        SubscriptBox["i", 
         RowBox[{"\[Alpha]", "-", "1"}]]], "\[CircleTimes]", 
       SuperscriptBox["dx", 
        SubscriptBox["i", "\[Alpha]"]], "\[CircleTimes]", 
       SuperscriptBox["dx", 
        SubscriptBox["i", 
         RowBox[{"\[Alpha]", "+", "1"}]]], "\[CircleTimes]"}], ".."}], 
     "\[CircleTimes]", 
     SuperscriptBox["dx", 
      SubscriptBox["i", "p"]], "\[CircleTimes]", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       SubscriptBox["j", "1"]], "x"}], "\[CircleTimes]"}], ".."}], 
   "\[CircleTimes]", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     SubscriptBox["j", "q"]], "x"}]}], "   ", 
  RowBox[{"(", "6", ")"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6572056188510113`*^9, {3.657205653313623*^9, 3.657205663069845*^9}, {
   3.657206717840644*^9, 3.6572067343076563`*^9}, {3.702105205030744*^9, 
   3.7021052654941397`*^9}},ExpressionUUID->"e9f9fdde-8161-4220-963f-\
91164e44f143"],

Cell["with the coordinates", "Text",
 CellChangeTimes->{{3.6572056790968237`*^9, 
  3.657205714081406*^9}},ExpressionUUID->"a57339d0-9789-48fc-b66f-\
728d2da73e0b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   SuperscriptBox[
    SubscriptBox["a", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SubscriptBox["i", "1"], ".."}], 
        SubscriptBox["i", 
         RowBox[{"\[Alpha]", "-", "1"}]], 
        SubscriptBox["ii", 
         RowBox[{"\[Alpha]", "+", "1"}]]}], ".."}], 
      SubscriptBox["i", "p"]}]], 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "1"], "..."}], 
     SubscriptBox["j", "q"]}]]}], 
  RowBox[{
   SubsuperscriptBox["\[Omega]", 
    SubscriptBox["i", "\[Alpha]"], 
    RowBox[{" ", "i"}]], "[", "X", "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6572057941053743`*^9, 3.657205946312561*^9}, {
  3.657206446338973*^9, 
  3.657206449882663*^9}},ExpressionUUID->"ee447a4d-a1c1-4665-bf11-\
f6a1ec8f7fa0"],

Cell["\<\
Analogously, the terms of the second summand get the same basis elements with \
the coordinates\
\>", "Text",
 CellChangeTimes->{{3.657205980269972*^9, 
  3.657206059733204*^9}},ExpressionUUID->"1ac13719-c01c-4afa-b38b-\
3e41ab5c486c"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   SubscriptBox["a", 
    RowBox[{
     RowBox[{
      SubscriptBox["i", "1"], "..."}], 
     SubscriptBox["i", "p"]}]], 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["j", "1"], ".."}], 
      SubscriptBox["j", 
       RowBox[{"\[Beta]", "-", "1"}]], 
      SubscriptBox["ij", 
       RowBox[{"\[Beta]", "+", "1"}]]}], ".."}], 
    SubscriptBox["j", "q"]}]], 
  RowBox[{
   SubsuperscriptBox["\[Omega]", "i", 
    RowBox[{" ", 
     SubscriptBox["j", "\[Beta]"]}]], "[", "X", "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6572062073649*^9, 3.6572062437724447`*^9}, {
   3.657206285419887*^9, 3.6572062877337303`*^9}, {3.657206402596417*^9, 
   3.657206427411188*^9}, 
   3.657207079829876*^9},ExpressionUUID->"6e83f775-6e39-4d79-8633-\
fdb2eb79ef77"],

Cell["\<\
Collecting all these terms we obtain the coordinates of the absolute \
differential  in the tensor basis (5) as\
\>", "Text",
 CellChangeTimes->{{3.657206557905671*^9, 3.6572066716969147`*^9}, {
  3.657206747111618*^9, 
  3.657206763287504*^9}},ExpressionUUID->"0cdf68d9-967a-4a9a-86a0-\
e8c146da0d58"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], 
   SuperscriptBox[
    SubscriptBox[
     RowBox[{"ta", "[", "x", "]"}], 
     RowBox[{
      RowBox[{
       SubscriptBox["i", "1"], "..."}], 
      SubscriptBox["i", "p"]}]], 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "1"], "..."}], 
     SubscriptBox["j", "q"]}]]}], " ", "=", "  ", 
  RowBox[{
   RowBox[{
    SubscriptBox["d", "X"], "[", 
    SuperscriptBox[
     SubscriptBox["a", 
      RowBox[{
       RowBox[{
        SubscriptBox["i", "1"], "..."}], 
       SubscriptBox["i", "p"]}]], 
     RowBox[{
      RowBox[{
       SubscriptBox["j", "1"], "..."}], 
      SubscriptBox["j", "q"]}]], "]"}], " ", "-", " ", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"\[Alpha]", "=", "1"}], "p"], " ", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["a", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           SubscriptBox["i", "1"], ".."}], 
          SubscriptBox["i", 
           RowBox[{"\[Alpha]", "-", "1"}]], 
          SubscriptBox["ii", 
           RowBox[{"\[Alpha]", "+", "1"}]]}], ".."}], 
        SubscriptBox["i", "p"]}]], 
      RowBox[{
       RowBox[{
        SubscriptBox["j", "1"], "..."}], 
       SubscriptBox["j", "q"]}]], 
     RowBox[{
      SubsuperscriptBox["\[Omega]", 
       SubscriptBox["i", "\[Alpha]"], 
       RowBox[{" ", "i"}]], "[", "X", "]"}]}]}], "  ", "+", "  ", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"\[Beta]", "=", "1"}], "q"], " ", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["a", 
       RowBox[{
        RowBox[{
         SubscriptBox["i", "1"], "..."}], 
        SubscriptBox["i", "p"]}]], 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["j", "1"], ".."}], 
         SubscriptBox["j", 
          RowBox[{"\[Beta]", "-", "1"}]], 
         SubscriptBox["ij", 
          RowBox[{"\[Beta]", "+", "1"}]]}], ".."}], 
       SubscriptBox["j", "q"]}]], 
     RowBox[{
      SubsuperscriptBox["\[Omega]", "i", 
       RowBox[{" ", 
        SubscriptBox["j", "\[Beta]"]}]], "[", "X", "]"}], "                                          ", 
     RowBox[{"(", "7", ")"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6572068532252293`*^9, 3.657206878383407*^9}, {
   3.6572069329748993`*^9, 3.657206941959277*^9}, {3.657206986598702*^9, 
   3.657207059927412*^9}, {3.657207111669792*^9, 3.65720722506961*^9}, 
   3.669112261105041*^9},ExpressionUUID->"3288d436-d27f-4921-b254-\
71aea81db48b"],

Cell[TextData[{
 "We see that the coordinates of the absolute differential  ",
 Cell[BoxData[
  SubscriptBox["absD", "X"]], "Input",
  CellChangeTimes->{{3.6572068532252293`*^9, 3.657206878383407*^9}, {
   3.6572069329748993`*^9, 3.657206941959277*^9}, {3.657206986598702*^9, 
   3.657207059927412*^9}, {3.657207111669792*^9, 3.65720722506961*^9}},
  ExpressionUUID->"1e051d38-8bc5-4a98-8160-a186509ebf35"],
 "ta  are expressed by the coordinates of X, ta, their first partial \
derivatives, and the \[CapitalGamma]-symbols of the linear connection \
contained in the connection forms. Thus the coordinates of the absolute \
differential sufficing the conditions 1- 6 are uniquely defined by the linear \
connection, and the operation ",
 Cell[BoxData[
  SubscriptBox["absD", "X"]], "Input",
  CellChangeTimes->{{3.6572068532252293`*^9, 3.657206878383407*^9}, {
   3.6572069329748993`*^9, 3.657206941959277*^9}, {3.657206986598702*^9, 
   3.657207059927412*^9}, {3.657207111669792*^9, 3.65720722506961*^9}},
  ExpressionUUID->"95e839b3-d157-41e3-adcc-0e3bcfec9fee"],
 " is uniquely defined.  To prove the existence of such an operator one has \
to take formula (7) as the definition of the tensor  coordinates of the \
absolute differential. Then one has to prove that they fulfil the correct \
transformation rule and that the so defined tensor satisfies the conditions 1 \
-6 of Theorem 3.2.3. We leave this part of the proof to the reader. \
\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.653798710195958*^9, 3.653798858922676*^9}, {
   3.653799032865518*^9, 3.653799204447706*^9}, {3.653799241927559*^9, 
   3.653799283863227*^9}, 3.653799496627665*^9, {3.653799549442841*^9, 
   3.6537998123269176`*^9}, {3.653799958964963*^9, 3.653800019476101*^9}, {
   3.653800086707531*^9, 3.653800122403183*^9}, {3.653800152627008*^9, 
   3.653800258976536*^9}, {3.65380057034256*^9, 3.6538006989738693`*^9}, {
   3.6538022667654877`*^9, 3.6538022878455753`*^9}, {3.654349190240724*^9, 
   3.654349256693667*^9}, {3.6544066241576853`*^9, 3.654406632702991*^9}, {
   3.654407536563085*^9, 3.6544075591255007`*^9}, 3.6544077374987907`*^9, 
   3.654408127689076*^9, 3.654408168797821*^9, {3.654417561944024*^9, 
   3.654417753237488*^9}, 3.657180066373914*^9, 3.657180133841015*^9, {
   3.657207254073189*^9, 3.657207599730513*^9}, {3.657293040248876*^9, 
   3.65729304963579*^9}, {3.6572930879471607`*^9, 3.6572932589553633`*^9}, {
   3.6572933228900213`*^9, 3.6572933888317738`*^9}, {3.657364535639022*^9, 
   3.657364537957891*^9}, {3.66911226111331*^9, 3.669112261301072*^9}, 
   3.669191247391554*^9, 3.7021055391994257`*^9, {3.703222954866518*^9, 
   3.703222979739606*^9}},ExpressionUUID->"2c64f5da-6aad-47a6-8fd8-\
4062f7af49f1"],

Cell[TextData[{
 " We go here another, an experimental way: We translate formula (7) into a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " definition and verify that this so defined  absolute differential \
satisfies the conditions 1-6 in many examples.  This  will be done in the \
next subsection. Of course, the experimental method may not replace a \
rigorous mathematical proof, but it has the advantage to show that the \
created ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " modules calculate the absolute differentials in a correct way. Moreover, \
it shows the power of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in symbolic calculations."
}], "Text",
 CellChangeTimes->{{3.653798710195958*^9, 3.653798858922676*^9}, {
   3.653799032865518*^9, 3.653799204447706*^9}, {3.653799241927559*^9, 
   3.653799283863227*^9}, 3.653799496627665*^9, {3.653799549442841*^9, 
   3.6537998123269176`*^9}, {3.653799958964963*^9, 3.653800019476101*^9}, {
   3.653800086707531*^9, 3.653800122403183*^9}, {3.653800152627008*^9, 
   3.653800258976536*^9}, {3.65380057034256*^9, 3.6538006989738693`*^9}, {
   3.6538022667654877`*^9, 3.6538022878455753`*^9}, {3.654349190240724*^9, 
   3.654349256693667*^9}, {3.6544066241576853`*^9, 3.654406632702991*^9}, {
   3.654407536563085*^9, 3.6544075591255007`*^9}, 3.6544077374987907`*^9, 
   3.654408127689076*^9, 3.654408168797821*^9, {3.654417561944024*^9, 
   3.654417753237488*^9}, 3.657180066373914*^9, 3.657180133841015*^9, {
   3.657207254073189*^9, 3.657207599730513*^9}, 3.657293040248876*^9, {
   3.657293421103949*^9, 3.657293524423233*^9}, {3.65729356404713*^9, 
   3.6572936360864973`*^9}, {3.657293685854638*^9, 3.657293876197467*^9}, {
   3.657293913365162*^9, 3.657293915301063*^9}, {3.6572939779887743`*^9, 
   3.65729400839649*^9}, {3.7032230420081787`*^9, 
   3.703223044615141*^9}},ExpressionUUID->"90565454-becf-4bbe-9ac1-\
52ad0ada41a7"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2.5. The Absolute Differential, Modules absD, tensorD", \
"Subsubsection",
 CellChangeTimes->{{3.6568604419512453`*^9, 3.65686048624162*^9}, 
   3.656916790048291*^9, {3.6569216841598263`*^9, 3.65692174550215*^9}, 
   3.6569219946202507`*^9, 3.657262652204883*^9, {3.657262695620166*^9, 
   3.6572627379794827`*^9}, {3.657280937149122*^9, 3.657280940205999*^9}, 
   3.6574384420890636`*^9, 3.659788193327013*^9, 3.6631591603866034`*^9, 
   3.663312330214357*^9, {3.669112261334654*^9, 3.6691122614171677`*^9}, {
   3.669112547539403*^9, 3.669112571771269*^9}, 3.669191367755207*^9, 
   3.669212722620142*^9, 3.702105685659779*^9, 3.70221286151537*^9, 
   3.703220920766397*^9, 3.703223156198848*^9, 3.703228254730618*^9, 
   3.704118306999086*^9, 
   3.704162358080977*^9},ExpressionUUID->"cee23004-312f-4c52-a619-\
5e59b950abe5"],

Cell["\<\
The last formula 3.2.4 (7) is the base for the next modules calculating the \
absolute differential of arbitrary tensor fields:\
\>", "Text",
 CellChangeTimes->{{3.6572810945552797`*^9, 3.657281111931048*^9}, {
   3.6572811498668413`*^9, 3.657281168194961*^9}, 3.657281240777845*^9, {
   3.657281276905466*^9, 
   3.657281333017214*^9}},ExpressionUUID->"e258f474-de26-439b-a721-\
1452abb9c6dc"],

Cell[BoxData[
 RowBox[{"?", "absD"}]], "Input",
 CellChangeTimes->{{3.657281364298539*^9, 3.657281643518072*^9}, {
   3.657281697726363*^9, 3.65728172755*^9}, {3.657281762957602*^9, 
   3.657281808089693*^9}, {3.657281842189096*^9, 3.657281912972012*^9}, 
   3.657281950291802*^9, {3.657282246770093*^9, 3.65728224706539*^9}, 
   3.657282286232348*^9, {3.657364704493528*^9, 3.65736471757218*^9}, 
   3.657435863465494*^9, {3.663312007506358*^9, 3.66331202733781*^9}, 
   3.663312096929076*^9, 3.669112261426806*^9, {3.669208715926908*^9, 
   3.669208720054491*^9}},ExpressionUUID->"c6a2fcc3-1430-4723-939b-\
3656f5d32c27"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"absD", "[", 
      RowBox[{"\[CapitalGamma]_", ",", "tens_"}], "]"}], "[", "x__", "]"}], 
    "[", "v_", "]"}], "[", 
   RowBox[{"ii_", ",", "ji_"}], "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"di", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], ",", "pt", ",", "qt", ",", "wv", 
      ",", "dt", ",", "dti", ",", "dtj", ",", "i", ",", "j", ",", "s", ",", 
      "l"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Syntax", " ", "control"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"tens", "[", "x", "]"}], "]"}], "===", "tensor"}], ",", " ", 
       RowBox[{
        RowBox[{"pt", " ", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
        RowBox[{"qt", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
          RowBox[{"tens", "[", "x", "]"}], ",", "\"\< is not a tensor!\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"Tuples", "[", 
           RowBox[{
            RowBox[{"Range", "[", "di", "]"}], ",", "pt"}], "]"}], ",", 
          "ii"}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Invalid argument \>\"", ",", 
          RowBox[{"ii", ";"}], ",", " ", "\"\< !\>\""}], "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"Tuples", "[", 
           RowBox[{
            RowBox[{"Range", "[", "di", "]"}], ",", "qt"}], "]"}], ",", 
          "ji"}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Invalid argument \>\"", ",", "ji", ",", " ", "\"\< !\>\""}], 
         "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"di", " ", "\[Equal]", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", "3", "]"}], "]"}]}], "]"}], ",", " ", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Tensor dimension \>\"", ",", 
          RowBox[{
           RowBox[{"type", "[", 
            RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", " ", 
          "\"\< is not the manifold dimension\>\"", ",", " ", "di", ",", 
          "\"\< !\>\""}], "]"}], ";", " ", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Head", "[", "v", "]"}], " ", "===", " ", "List"}], ")"}], "&&", 
        RowBox[{
         RowBox[{"Length", "[", "v", "]"}], " ", "\[Equal]", " ", "di"}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"wv", "[", 
         RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalGamma]", "[", 
             RowBox[{"j", ",", "s", ",", "i"}], "]"}], "[", "x", "]"}], "*", 
           RowBox[{"v", "[", 
            RowBox[{"[", "s", "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"s", ",", 
            RowBox[{"Length", "[", 
             RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Invalid argument \>\"", ",", " ", "v", ",", " ", "\"\< !\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"The", " ", "components"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"dt", " ", "=", " ", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"absD", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"tens", "[", "x", "]"}], "[", 
            RowBox[{"ii", ",", "ji"}], "]"}], ",", 
           RowBox[{
            RowBox[{"{", "x", "}"}], "[", 
            RowBox[{"[", "s", "]"}], "]"}]}], "]"}], "*", 
         RowBox[{"v", "[", 
          RowBox[{"[", "s", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", "di"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dti", " ", "=", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"tens", "[", "x", "]"}], "[", 
            RowBox[{
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"ii", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ";;", 
                  RowBox[{"l", "-", "1"}]}], "]"}], "]"}], ",", 
               RowBox[{"{", "s", "}"}], ",", 
               RowBox[{"ii", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"l", "+", "1"}], ";;"}], "]"}], "]"}]}], "]"}], ",",
              "ji"}], "]"}], "*", 
           RowBox[{"wv", "[", 
            RowBox[{
             RowBox[{"ii", "[", 
              RowBox[{"[", "l", "]"}], "]"}], ",", "s"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"s", ",", "di"}], "}"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"l", ",", "pt"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dtj", " ", "=", " ", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"tens", "[", "x", "]"}], "[", 
            RowBox[{"ii", ",", 
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"ji", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ";;", 
                  RowBox[{"l", "-", "1"}]}], "]"}], "]"}], ",", 
               RowBox[{"{", "s", "}"}], ",", 
               RowBox[{"ji", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"l", "+", "1"}], ";;"}], "]"}], "]"}]}], "]"}]}], 
            "]"}], "*", 
           RowBox[{"wv", "[", 
            RowBox[{"s", ",", 
             RowBox[{"ji", "[", 
              RowBox[{"[", "l", "]"}], "]"}]}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"s", ",", "di"}], "}"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"l", ",", "qt"}], "}"}]}], "]"}]}], ";", " ", 
     "\[IndentingNewLine]", 
     RowBox[{"dt", "-", "dti", "+", "dtj"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.656331605889989*^9, 3.6563316513029213`*^9}, {
   3.656331692798827*^9, 3.656331720318149*^9}, 3.656331977701541*^9, {
   3.656332010324163*^9, 3.6563320250613613`*^9}, {3.6563320864130297`*^9, 
   3.6563320894046297`*^9}, {3.656332131275894*^9, 3.65633216551546*^9}, {
   3.65633224124648*^9, 3.6563323669298058`*^9}, {3.6563355433736753`*^9, 
   3.656335548901659*^9}, {3.656335634668845*^9, 3.6563356367565804`*^9}, {
   3.656335671468384*^9, 3.6563356846203003`*^9}, {3.656335720843706*^9, 
   3.6563357852839127`*^9}, {3.656335829579122*^9, 3.656335838107562*^9}, {
   3.6563358991624517`*^9, 3.656336044705237*^9}, {3.656336933042362*^9, 
   3.6563370021623573`*^9}, 3.6563370408505573`*^9, {3.65633718344843*^9, 
   3.6563373372794027`*^9}, {3.656337375407099*^9, 3.656337517293624*^9}, {
   3.6563375628455753`*^9, 3.656337613964919*^9}, {3.656338030002207*^9, 
   3.656338047489258*^9}, 3.656338147057521*^9, {3.656338189352157*^9, 
   3.656338445294128*^9}, {3.6563384821189938`*^9, 3.656338503029792*^9}, {
   3.656339230215904*^9, 3.6563392344685583`*^9}, {3.656339309295796*^9, 
   3.656339332024498*^9}, 3.656339386911826*^9, {3.656339577147262*^9, 
   3.656339621645112*^9}, {3.656400472127962*^9, 3.65640047916794*^9}, {
   3.6564006029586143`*^9, 3.656400606894191*^9}, {3.656400758413766*^9, 
   3.6564008158366117`*^9}, {3.65640085957303*^9, 3.656400899859363*^9}, {
   3.6564874044780207`*^9, 3.656487547348946*^9}, {3.656487652614419*^9, 
   3.6564877742997637`*^9}, 3.656487933137908*^9, {3.6572645029399433`*^9, 
   3.657264502942608*^9}, {3.657266519419777*^9, 3.657266525353076*^9}, 
   3.657280971409759*^9, {3.66911226147086*^9, 3.6691122614739*^9}, 
   3.669208746393618*^9},ExpressionUUID->"e4b41b89-89b4-4e6f-af54-\
63635837e8c6"],

Cell[BoxData[
 RowBox[{"?", "tensorD"}]], "Input",
 CellChangeTimes->{{3.657282340348958*^9, 3.6572823869986563`*^9}, {
   3.65728245056746*^9, 3.657282533335187*^9}, {3.657282574285859*^9, 
   3.657282592725545*^9}, {3.657282665333427*^9, 3.657282757435487*^9}, {
   3.657282818700899*^9, 3.657282905231942*^9}, {3.6572829361388073`*^9, 
   3.657282958690646*^9}, 3.657283028729323*^9, {3.663312042450453*^9, 
   3.663312054546008*^9}, 3.66911226150021*^9, 
   3.6692092914983597`*^9},ExpressionUUID->"a07f51b1-261c-4efd-9bca-\
c7459a44df95"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]_", ",", "tens_"}], "]"}], "[", "x__", "]"}], 
   "[", "v_", "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"di", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], ",", "pt", ",", "qt"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Syntax", " ", "control"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"tens", "[", "x", "]"}], "]"}], "===", "tensor"}], ",", " ", 
       RowBox[{
        RowBox[{"pt", " ", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
        RowBox[{"qt", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
          RowBox[{"tens", "[", "x", "]"}], ",", "\"\< is not a tensor!\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Head", "[", "v", "]"}], " ", "===", " ", "List"}], ")"}], 
         "&&", 
         RowBox[{
          RowBox[{"Length", "[", "v", "]"}], " ", "\[Equal]", " ", "di"}]}], 
        "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Invalid argument \>\"", ",", " ", "v", ",", " ", "\"\< !\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"tensor", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"absD", "[", 
          RowBox[{"\[CapitalGamma]", ",", "tens"}], "]"}], "[", "x", "]"}], 
        "[", "v", "]"}], ",", "pt", ",", "qt", ",", "di"}], "]"}]}]}], 
   "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6563427833560266`*^9, 3.656342789447337*^9}, {
   3.656342843158556*^9, 3.656342963941431*^9}, 3.656343021040022*^9, {
   3.6563430564532223`*^9, 3.656343106269249*^9}, {3.65634319672395*^9, 
   3.6563432219262238`*^9}, {3.656343270432033*^9, 3.656343276293867*^9}, {
   3.65634335880473*^9, 3.656343507274365*^9}, 3.657264502947132*^9, 
   3.657264588222106*^9, {3.657266592530807*^9, 3.657266599574698*^9}, {
   3.657267243744074*^9, 3.657267246825692*^9}, {3.657267889516656*^9, 
   3.657267903396482*^9}, 3.657281044125877*^9, {3.669112261526486*^9, 
   3.669112261528077*^9}, 3.669209221925199*^9, 
   3.669209293065049*^9},ExpressionUUID->"744efc89-5a46-4f3c-92b5-\
5842cee5fea5"],

Cell[TextData[{
 StyleBox["Example 1.",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" A covector field, (p, q) = (1, 0)",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.657290877656283*^9, 3.6572909582557783`*^9}, {
  3.657365012889072*^9, 3.6573650759528303`*^9}, {3.657366077303913*^9, 
  3.657366086591525*^9}, {3.657366432660926*^9, 
  3.657366433268894*^9}},ExpressionUUID->"303eebbb-5fdc-42f3-b316-\
b2bd6c8cf6b8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", "3"}], ";", " ", 
  RowBox[{"Clear", "[", "ta", "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{"covec", "[", 
     RowBox[{"a", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6572909620029907`*^9, 3.6572909620045424`*^9}, {
   3.657291695103586*^9, 3.6572917445023212`*^9}, {3.657291963564336*^9, 
   3.657292025011381*^9}, {3.702203576960803*^9, 3.7022036619521027`*^9}, 
   3.702203747238988*^9, {3.702208405511715*^9, 
   3.7022084214319487`*^9}},ExpressionUUID->"f216830f-ef0e-40b8-bb58-\
ccf2afacaf50"],

Cell[BoxData[
 RowBox[{"ta", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7022034540717783`*^9, 3.702203527303945*^9}, 
   3.702203717776103*^9},ExpressionUUID->"eb1223b1-f406-43b2-a374-\
acc7f59e5eb7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ta", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7022034540717783`*^9, 3.702203527303945*^9}, 
   3.702203717776103*^9, {3.702203762639226*^9, 
   3.7022037772380447`*^9}},ExpressionUUID->"a8e6988f-16e4-4df1-9ae5-\
a5dfa632b95c"],

Cell[BoxData[
 RowBox[{"vx", " ", "=", " ", 
  RowBox[{"vec", "[", "\[Xi]", "]"}]}]], "Input",
 CellChangeTimes->{{3.6572910348828*^9, 3.657291035255374*^9}, {
  3.6572913733713827`*^9, 
  3.657291396907179*^9}},ExpressionUUID->"de650108-e0db-4100-b12e-\
9fa72f5e9873"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "dta", "]"}], ";", 
  RowBox[{"dta", "=", 
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.657290982681526*^9, 3.657291014215515*^9}, {
   3.657291056727507*^9, 3.657291058326824*^9}, {3.6572911860697613`*^9, 
   3.657291189253972*^9}, {3.657292060915298*^9, 3.6572920815459013`*^9}, {
   3.6572921315617933`*^9, 3.6572921602415237`*^9}, 3.657292292040619*^9, 
   3.657292464736109*^9, 3.6691122615397987`*^9, 3.669209311294772*^9, {
   3.702208516143931*^9, 
   3.702208535678212*^9}},ExpressionUUID->"188facb9-72a5-471e-9922-\
49eae190c266"],

Cell[BoxData[
 RowBox[{"type", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.70220324221239*^9, 
  3.702203253146687*^9}},ExpressionUUID->"b0b98d1f-9498-4755-8c37-\
4e760f0dbd69"],

Cell["\<\
The result is not a tensor, but a tensor form: for each tangent vector X it \
yields a tensor at the point x:\
\>", "Text",
 CellChangeTimes->{{3.6573653810460873`*^9, 3.6573654601896*^9}, {
  3.6573659023558073`*^9, 
  3.657365907241681*^9}},ExpressionUUID->"b630c1c6-400f-4e71-a7af-\
9be3e3382105"],

Cell[BoxData[
 RowBox[{"dta", "[", "vx", "]"}]], "Input",
 CellChangeTimes->{{3.6572924794199247`*^9, 
  3.65729249079064*^9}},ExpressionUUID->"7465a973-98a1-45a5-8bdf-\
032a3bd00c97"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.663307950046338*^9, 
  3.663307963444542*^9}},ExpressionUUID->"dbcc1da9-84c7-4da2-bbaf-\
3dbd253c1f67"],

Cell["\<\
and for a vector field  a tensor field of the same type as ta (here a \
covector field):\
\>", "Text",
 CellChangeTimes->{{3.6573655177667418`*^9, 3.657365615068301*^9}, {
   3.657366062719965*^9, 3.657366063456192*^9}, 
   3.657435775473917*^9},ExpressionUUID->"43398a01-72d1-48de-b6d8-\
255940202fe6"],

Cell[BoxData[
 RowBox[{"dta", "[", 
  RowBox[{"vec", "[", 
   RowBox[{"v", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6572924794199247`*^9, 3.65729249079064*^9}, {
   3.657365638605101*^9, 3.65736570679598*^9}, {3.657365749619529*^9, 
   3.6573657723235807`*^9}, 3.657365806434905*^9, {3.704161888321986*^9, 
   3.704161904100092*^9}},ExpressionUUID->"001ba2e8-d18f-464e-a09a-\
f1ed930cfc9b"],

Cell["with local coordinates, e.g. for i = 2", "Text",
 CellChangeTimes->{{3.657366155007172*^9, 
  3.657366193630577*^9}},ExpressionUUID->"1f0c07df-732f-4851-a3fb-\
9668e23d90ac"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dta", "[", 
   RowBox[{"vec", "[", 
    RowBox[{"v", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657291103078464*^9, 3.657291119086581*^9}, 
   3.657366135258588*^9},ExpressionUUID->"b7626b69-315b-46a5-9d17-\
3d50418860ab"],

Cell[BoxData[
 RowBox[{"?", "absD"}]], "Input",
 CellChangeTimes->{{3.657291504031287*^9, 3.657291507852705*^9}, 
   3.669112261554429*^9, 
   3.669209221980894*^9},ExpressionUUID->"74669ee7-44bd-4bc2-98d0-\
ecc82f9855b0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"absD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"vec", "[", 
    RowBox[{"v", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.65729152478093*^9, 3.65729152841693*^9}, {
   3.657291594961164*^9, 3.6572916035605297`*^9}, {3.657291639344905*^9, 
   3.65729164670332*^9}, 3.657366259975456*^9, 3.669112261559004*^9, 
   3.669209222012373*^9},ExpressionUUID->"ccb98d5c-65c3-48a7-bb82-\
6fbdef235fbb"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"dta", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"v", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "2", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65729189255068*^9, 3.657291897947876*^9}, {
   3.657292398943852*^9, 3.657292413495126*^9}, 
   3.6573662943679657`*^9},ExpressionUUID->"acb25c72-74f2-4f5f-a3c1-\
c12158f783a7"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.657366418117402*^9, 
  3.6573664181186132`*^9}},ExpressionUUID->"99018c2d-47c8-45c4-a79e-\
549a6cdb45b7"],

Cell[TextData[{
 StyleBox["Example 2.",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" A vector field, (p, q) = (0, 1)",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{{3.657290877656283*^9, 3.6572909582557783`*^9}, {
   3.657365012889072*^9, 3.6573650759528303`*^9}, {3.657366077303913*^9, 
   3.657366086591525*^9}, 3.657366398669787*^9, {3.657366457317089*^9, 
   3.657366469340456*^9}, 3.66330829265681*^9, 
   3.7022125140544033`*^9},ExpressionUUID->"b9130476-82b0-4b2f-8a73-\
5c2bd31dc984"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", "3"}], ";", " ", 
  RowBox[{"Clear", "[", "ta", "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", "x", "]"}], ",", "0", ",", "1"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6572909620029907`*^9, 3.6572909620045424`*^9}, {
  3.657291695103586*^9, 3.6572917445023212`*^9}, {3.657291963564336*^9, 
  3.657292025011381*^9}, {3.657366480413506*^9, 
  3.657366484588719*^9}},ExpressionUUID->"e49814f5-bfcf-43a5-aaf9-\
5e6bb982ed5b"],

Cell[BoxData[
 RowBox[{"vx", " ", "=", " ", 
  RowBox[{"vec", "[", "\[Xi]", "]"}]}]], "Input",
 CellChangeTimes->{{3.6572910348828*^9, 3.657291035255374*^9}, {
  3.6572913733713827`*^9, 
  3.657291396907179*^9}},ExpressionUUID->"497de2c2-e067-4e51-a53b-\
5efaa6fb1809"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "dta", "]"}], ";", 
  RowBox[{"dta", "=", 
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.657290982681526*^9, 3.657291014215515*^9}, {
   3.657291056727507*^9, 3.657291058326824*^9}, {3.6572911860697613`*^9, 
   3.657291189253972*^9}, {3.657292060915298*^9, 3.6572920815459013`*^9}, {
   3.6572921315617933`*^9, 3.6572921602415237`*^9}, 3.657292292040619*^9, 
   3.657292464736109*^9, 3.6691122615641613`*^9, 
   3.669209311302525*^9},ExpressionUUID->"38970397-b044-4fd9-a95c-\
d623cecfe97d"],

Cell["\<\
The result is not a tensor, but a tensor form: for each tangent vector X it \
yields a tensor at the point x:\
\>", "Text",
 CellChangeTimes->{{3.6573653810460873`*^9, 3.6573654601896*^9}, {
  3.6573659023558073`*^9, 
  3.657365907241681*^9}},ExpressionUUID->"507fc7d4-732c-43d7-812e-\
8d99ad7e2e16"],

Cell[BoxData[
 RowBox[{"dta", "[", "vx", "]"}]], "Input",
 CellChangeTimes->{{3.6572924794199247`*^9, 
  3.65729249079064*^9}},ExpressionUUID->"7bf3d7da-8eee-432e-8545-\
9624269836f1"],

Cell["\<\
and for a vector field  a tensor field of the same type as ta (here a vector \
field):\
\>", "Text",
 CellChangeTimes->{{3.6573655177667418`*^9, 3.657365615068301*^9}, {
   3.657366062719965*^9, 3.657366063456192*^9}, 3.657366548860076*^9, 
   3.6574359582316303`*^9},ExpressionUUID->"a683cd60-6d49-441a-a8b6-\
d0c3ef4cca00"],

Cell[BoxData[
 RowBox[{"dta", "[", 
  RowBox[{"vec", "[", 
   RowBox[{"v", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6572924794199247`*^9, 3.65729249079064*^9}, {
   3.657365638605101*^9, 3.65736570679598*^9}, {3.657365749619529*^9, 
   3.6573657723235807`*^9}, 
   3.657365806434905*^9},ExpressionUUID->"de8ec17d-cccb-4ec1-81da-\
20d5bbc46fec"],

Cell["with local coordinates, e.g. for i = 1", "Text",
 CellChangeTimes->{{3.657366155007172*^9, 3.657366193630577*^9}, 
   3.657436032206443*^9},ExpressionUUID->"13b781b2-7237-4828-ae8d-\
37a3bb3ce1e5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dta", "[", 
   RowBox[{"vec", "[", 
    RowBox[{"v", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657291103078464*^9, 3.657291119086581*^9}, 
   3.657366135258588*^9, {3.657366566156417*^9, 
   3.6573665740558987`*^9}},ExpressionUUID->"529bb0da-38a7-4ccd-8070-\
c248f86e21cf"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"absD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"vec", "[", 
    RowBox[{"v", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.65729152478093*^9, 3.65729152841693*^9}, {
   3.657291594961164*^9, 3.6572916035605297`*^9}, {3.657291639344905*^9, 
   3.65729164670332*^9}, 3.657366259975456*^9, {3.657366665787352*^9, 
   3.657366670378615*^9}, 3.66911226157128*^9, 
   3.669209222034814*^9},ExpressionUUID->"630407b7-22ab-4bd2-81e6-\
d7c70732a92a"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"dta", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"v", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "1", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65729189255068*^9, 3.657291897947876*^9}, {
   3.657292398943852*^9, 3.657292413495126*^9}, 3.6573662943679657`*^9, {
   3.6573666841954317`*^9, 
   3.6573666900668917`*^9}},ExpressionUUID->"d1b61daa-4536-4c53-ad65-\
7ce2a8f938ce"],

Cell["\<\
In Subsection  3.2.2 we  preliminarily defined  the gradient nabla1 of a \
vector field. We will show that this definition admits to consider the \
absolute differential of the vector field as a contraction. Aiming this we \
define the vector field Xa by its tensor coordinates:\
\>", "Text",
 CellChangeTimes->{{3.6574362196199703`*^9, 3.657436301995431*^9}, {
   3.657436358882865*^9, 3.657436595321047*^9}, {3.663308393920376*^9, 
   3.663308423495943*^9}, 
   3.702212668503401*^9},ExpressionUUID->"2ad2f582-a6c1-4644-a88a-\
ae6c704a2c31"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Xa", "[", "x__", "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", "x", "]"}], "[", 
     RowBox[{
      RowBox[{"{", "}"}], ",", 
      RowBox[{"{", "i", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.657367281054158*^9, 3.657367329037388*^9}, {
  3.6573685161145983`*^9, 
  3.6573686056261463`*^9}},ExpressionUUID->"c07f5be1-ed3f-47f9-9a5d-\
5057bd5892eb"],

Cell["\<\
Its gradient is a tensor field of type (1,1) with the coordinates, e.g.\
\>", "Text",
 CellChangeTimes->{{3.657436626896307*^9, 3.6574366569281683`*^9}, {
  3.657436690671878*^9, 
  3.65743669619978*^9}},ExpressionUUID->"f5964b1f-20bc-4cc1-a1b2-\
769e3691f0c8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nab1", "[", 
    RowBox[{"\[CapitalGamma]", ",", "Xa"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"3", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.657366888725246*^9, 3.6573668936090803`*^9}, {
   3.65736694074405*^9, 3.657367057432192*^9}, {3.657367349333733*^9, 
   3.657367388756873*^9}, 3.663308537726831*^9, 3.663308593184146*^9, 
   3.6633095422726097`*^9},ExpressionUUID->"0a2ce3c5-e16e-44f6-96f7-\
a93f562777cb"],

Cell["\<\
The first coordinate of its contraction with the vector field vec[v[var[x]]] \
is\
\>", "Text",
 CellChangeTimes->{{3.657436855774351*^9, 
  3.657436916029997*^9}},ExpressionUUID->"4a59e054-cba9-4f80-ba23-\
6cc74257d3f3"],

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"nab1", "[", 
       RowBox[{"\[CapitalGamma]", ",", "Xa"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"i", ",", "1"}], "]"}], "*", 
    RowBox[{
     RowBox[{"vec", "[", 
      RowBox[{"v", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "[", 
     RowBox[{"[", "i", "]"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657367487877262*^9, 3.657367527539564*^9}, {
   3.657367557715816*^9, 3.6573676212747507`*^9}, 3.663308629918635*^9, 
   3.663309575600189*^9},ExpressionUUID->"e85c8dd7-4573-4f23-95e8-\
3827afed1e8f"],

Cell["\<\
The coordinates coincide with the coordinates of the absolute differential \
calculated above:\
\>", "Text",
 CellChangeTimes->{{3.657437058948897*^9, 
  3.657437110028532*^9}},ExpressionUUID->"cd5a733f-ffd1-43fc-a813-\
6f04bd3e0326"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"dta", "[", 
     RowBox[{"vec", "[", 
      RowBox[{"v", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "1", "}"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657436948423511*^9, 
  3.657436986262989*^9}},ExpressionUUID->"1c0a6998-c07f-4281-8088-\
eeb721e17082"],

Cell["\<\
\[FilledSquare]
Further examples are contained in the next Subsection and in the subnotebook \
testabsD.nb.\
\>", "Text",
 CellChangeTimes->{{3.657437409714274*^9, 3.65743743027388*^9}, {
   3.6574382704761667`*^9, 3.657438312698119*^9}, 
   3.669112261593503*^9},ExpressionUUID->"ee7cfc4f-8d35-4444-acf3-\
5a15cbc282ef"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2.6. Properties of the Absolute Differential", "Subsubsection",
 CellChangeTimes->{{3.6568604419512453`*^9, 3.65686048624162*^9}, 
   3.656916790048291*^9, {3.6569216841598263`*^9, 3.65692174550215*^9}, 
   3.6569219946202507`*^9, 3.657262652204883*^9, {3.657262695620166*^9, 
   3.6572627379794827`*^9}, {3.657280937149122*^9, 3.657280940205999*^9}, {
   3.657438456929121*^9, 3.657438490136627*^9}, 3.6574651119894953`*^9, 
   3.669112261632042*^9, 3.669112588035142*^9, 3.669212731916664*^9, 
   3.669443128117676*^9, 3.7022128682763433`*^9, 3.7032356053842573`*^9, 
   3.704162376881473*^9, 
   3.704163645344043*^9},ExpressionUUID->"ec685c16-5a1b-4342-9b56-\
7d036647d570"],

Cell[TextData[{
 "In this subsection we check the properties 1 - 5 of the absolute \
differential stated in Theorem 3.2.3, property 6 is obvious by the \
definition. \n",
 StyleBox["Property 1.",
  FontColor->RGBColor[0, 0, 1]],
 " The real linearity. "
}], "Text",
 CellChangeTimes->{{3.657440240659668*^9, 3.65744026342586*^9}, {
  3.657440297201777*^9, 3.65744036405715*^9}, {3.657440395283504*^9, 
  3.65744048233615*^9}},ExpressionUUID->"52eb8786-5706-4cde-86db-\
8f62889de769"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb"}], "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", "x__", "]"}], "=", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", "x", "]"}], ",", " ", "2", ",", "1"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tb", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"tensor", " ", "[", 
    RowBox[{
     RowBox[{"b", "[", "x", "]"}], ",", "2", ",", "1"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.657440502526444*^9, 3.657440682391099*^9}, {
  3.669439926506855*^9, 
  3.669439943127308*^9}},ExpressionUUID->"82c332cc-4056-4125-967d-\
27feeab83bf8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tc", "]"}], ";", 
  RowBox[{
   RowBox[{"tc", "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"ta", "[", "x", "]"}], " ", "+", 
    RowBox[{"tb", "[", "x", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.65744077223917*^9, 3.6574408108771877`*^9}, {
  3.6694396290088167`*^9, 3.669439663927986*^9}, {3.669439749032422*^9, 
  3.669439759575438*^9}, {3.6694398115201397`*^9, 
  3.669439839719552*^9}},ExpressionUUID->"ab2ebfc7-ec19-43bd-8977-\
4a108e38c768"],

Cell[BoxData[
 RowBox[{"tc", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6694394793995647`*^9, 
  3.669439511320895*^9}},ExpressionUUID->"3704354b-89b3-4867-8b7c-\
4ae4823197ab"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.669439717616346*^9, 
  3.669439725415915*^9}},ExpressionUUID->"a2495668-2782-4b33-84cc-\
cdc96c0da667"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vx", "]"}], ";", " ", 
  RowBox[{"vx", ":=", " ", 
   RowBox[{"vec", "[", 
    RowBox[{"fx", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.64527588009085*^9, 3.64527589147887*^9}, {
  3.669438678985944*^9, 3.669438722233759*^9}, {3.6694388226095667`*^9, 
  3.6694389017045727`*^9}},ExpressionUUID->"9b837fc4-e532-4e90-9e89-\
5ea0e65a5b15"],

Cell[BoxData["vx"], "Input",
 CellChangeTimes->{{3.6452758978904815`*^9, 3.6452759147541113`*^9}, 
   3.669438882025489*^9},ExpressionUUID->"ab3441e9-1adf-4dd1-8cfc-\
01944b4186a1"],

Cell[BoxData[
 RowBox[{"ttc", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}]}]], "Input",
 CellChangeTimes->{{3.657440843089847*^9, 3.657440934100749*^9}, {
   3.6574409934370728`*^9, 3.65744101970892*^9}, 3.669112261672463*^9, 
   3.6692093113108063`*^9},ExpressionUUID->"e5fce69b-4123-41d4-8181-\
5e11dda70367"],

Cell[BoxData[
 RowBox[{"tta", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}]}]], "Input",
 CellChangeTimes->{{3.657440843089847*^9, 3.657440934100749*^9}, {
   3.6574409934370728`*^9, 3.657441047451598*^9}, 3.669112261706134*^9, 
   3.669209311365856*^9, 
   3.6694402300479593`*^9},ExpressionUUID->"6b4ed6e1-3b65-41b3-9e56-\
5e2fe418850a"],

Cell[BoxData[
 RowBox[{"ttb", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}]}]], "Input",
 CellChangeTimes->{{3.657440843089847*^9, 3.657440934100749*^9}, {
   3.6574409934370728`*^9, 3.6574411190591803`*^9}, 3.6691122617113132`*^9, 
   3.669209311427396*^9},ExpressionUUID->"00aca6a1-821c-4e3b-99de-\
c532ca310ea9"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{"ttc", ",", 
    RowBox[{"tta", "+", "ttb"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6574411280795097`*^9, 
  3.657441181353985*^9}},ExpressionUUID->"067d72f7-f0c8-414c-bc3b-\
9404bb113d19"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tensorD", "[", 
    RowBox[{"\[CapitalGamma]", ",", 
     RowBox[{"Pi", "*", "ta"}]}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}]], "Input",
 CellChangeTimes->{{3.657441253268112*^9, 3.657441292193202*^9}, {
   3.65744154067669*^9, 3.6574415722543373`*^9}, 3.669112261715682*^9, 
   3.6692093114857817`*^9},ExpressionUUID->"9d54fc56-5590-4648-b152-\
9638ba20bdb7"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%", ",", 
   RowBox[{"Pi", "*", "tta"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6574413116963177`*^9, 
  3.657441339752721*^9}},ExpressionUUID->"8ababded-ea0b-4118-b4f3-\
8d285446fdf0"],

Cell["\<\
Thus the linearity in the tensor element ta is confirmed.  The linearity in \
the vector argument X is trivial\
\>", "Text",
 CellChangeTimes->{{3.657454552112212*^9, 3.657454619215816*^9}, {
  3.669440144596093*^9, 
  3.669440159131949*^9}},ExpressionUUID->"171c1a6e-91ba-4f47-91ec-\
ca98f1e5f80e"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{"vec", "[", "\[Xi]", "]"}], "+", " ", 
     RowBox[{"Pi", "*", 
      RowBox[{"vec", "[", "\[Eta]", "]"}]}]}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tensorD", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"vec", "[", "\[Xi]", "]"}], "]"}], "+", 
    RowBox[{"Pi", "*", 
     RowBox[{
      RowBox[{
       RowBox[{"tensorD", "[", 
        RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{"vec", " ", "[", "\[Eta]", "]"}], "]"}]}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6694405819226847`*^9, 
  3.6694406061672*^9}},ExpressionUUID->"d18ab5c9-0ccd-405d-af48-8ed3cb0e0224"],

Cell[TextData[{
 "\[FilledSquare]\n",
 StyleBox["Property 2.  ",
  FontColor->RGBColor[0, 0, 1]],
 "Consider a function as  a tensor of type (0, 0):"
}], "Text",
 CellChangeTimes->{{3.6574594340432568`*^9, 3.657459540659812*^9}, 
   3.657461379594716*^9},ExpressionUUID->"5673aa98-7ec7-4eff-8e4d-\
7e10ebecb3fd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tf", "[", "x__", "]"}], ":=", " ", 
  RowBox[{"tensor", "[", 
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], ",", "0", ",", "0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6574595499870777`*^9, 3.657459603827827*^9}, 
   3.6574596603874474`*^9, {3.657461693673398*^9, 
   3.6574617489770403`*^9}},ExpressionUUID->"c907a563-ae9c-47ea-b692-\
e66c31c3b637"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]", ",", " ", "tf"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657460963251807*^9, 3.6574610750782824`*^9}, 
   3.657461702674065*^9, 3.66911226172577*^9, 
   3.66920931160257*^9},ExpressionUUID->"17e3e144-5282-4608-a694-\
72fa3c4638b1"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", "f", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ".", "vx"}]}]], "Input",
 CellChangeTimes->{{3.657461106223879*^9, 3.6574611485336523`*^9}, {
  3.6694408167022457`*^9, 
  3.669440840788835*^9}},ExpressionUUID->"527d97c9-9469-4bd1-84cc-\
f7db43476a12"],

Cell[TextData[{
 "\[FilledSquare]\n",
 StyleBox["Property 3",
  FontColor->RGBColor[0, 0, 1]],
 " has been confirmed in 3.2.3, see the considerations before Lemma 1. \
\[GraySquare]\n",
 StyleBox["Property 4",
  FontColor->RGBColor[0, 0, 1]],
 ". We consider a contraction of the tensor"
}], "Text",
 CellChangeTimes->{{3.657461361721057*^9, 3.657461371042889*^9}, {
   3.6574615003058023`*^9, 3.657461548939386*^9}, {3.6574615791053867`*^9, 
   3.65746162560121*^9}, 3.669441662205386*^9, {3.669441694796814*^9, 
   3.669441743219836*^9}},ExpressionUUID->"abeb41fd-cfc3-49af-a081-\
64d01315cd0b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc", ",", "tf"}], "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", "x", "]"}], ",", "2", ",", "2"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.657461636705422*^9, 3.657461654848914*^9}, {
  3.657461837847707*^9, 
  3.6574618962233553`*^9}},ExpressionUUID->"f4767e8a-3289-4804-af9e-\
640796466189"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ta1", "]"}], ";", 
  RowBox[{
   RowBox[{"ta1", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], " ", ":=", " ", 
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{"ta", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "2"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.657461934977556*^9, 3.657461984658825*^9}, {
  3.657462041094301*^9, 3.657462076517885*^9}, {3.657462365612021*^9, 
  3.657462432859436*^9}},ExpressionUUID->"c2506a6f-d46e-4472-a679-\
a1673a3e0664"],

Cell[BoxData[
 RowBox[{"ta1", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657462441460599*^9, 
  3.657462455035348*^9}},ExpressionUUID->"3996e12b-cb1c-4f85-a783-\
60aaa8233f73"],

Cell[BoxData[
 RowBox[{"ta2", " ", "=", " ", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tensorD", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}], ",", "1", ",", 
    "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6574621098688602`*^9, 3.657462142288342*^9}, {
   3.657462184909668*^9, 3.657462218077199*^9}, 3.669112261730258*^9, 
   3.6692093116616287`*^9},ExpressionUUID->"0e06c69a-e065-4b24-8ab9-\
e6e5ea9a1741"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ta3", "]"}], ";", " ", 
  RowBox[{
   RowBox[{"ta3", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "ta1"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6574622312894993`*^9, 3.657462242988655*^9}, {
   3.657462274165307*^9, 3.657462286684849*^9}, {3.657462330748239*^9, 
   3.657462336596346*^9}, {3.657462606794541*^9, 3.6574626514178963`*^9}, 
   3.669112261734467*^9, 
   3.669209311720149*^9},ExpressionUUID->"917779d6-e0b3-4af0-bac9-\
fef06e4b4dfb"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"equi", "[", 
    RowBox[{
     RowBox[{"ta3", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "ta2"}], "]"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.657462483323181*^9, 3.657462541586266*^9}, 
   3.657462790970182*^9},ExpressionUUID->"07d40dc0-bc34-4157-8c67-\
029dab9248ab"],

Cell["This confirms property 4. \[FilledSquare]", "Text",
 CellChangeTimes->{{3.657462966302544*^9, 
  3.6574629964672337`*^9}},ExpressionUUID->"bea88556-ae26-4ebb-be1f-\
a60f22a29246"],

Cell[TextData[{
 StyleBox["Property 5",
  FontColor->RGBColor[0, 0, 1]],
 ". The ",
 StyleBox["product rule",
  FontColor->RGBColor[0, 0, 1]],
 "."
}], "Text",
 CellChangeTimes->{{3.657463116004675*^9, 3.657463132205648*^9}, 
   3.6574631636870623`*^9},ExpressionUUID->"082d653b-b5fd-4c15-8b27-\
17afbc528e4e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{"Clear", "[", "vx", "]"}], ";", 
  RowBox[{"vx", "=", " ", 
   RowBox[{"vec", "[", "\[Xi]", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.708582575694684*^9, 3.7085825764609127`*^9}, {
   3.708582622013834*^9, 3.7085826571947517`*^9}, 3.708582707161311*^9, {
   3.7085827441537523`*^9, 
   3.708582768595336*^9}},ExpressionUUID->"ec57f9ea-6fcc-458d-aed3-\
7eddd693c944"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc"}], "]"}], ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", "x__", "]"}], ":=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", "x", "]"}], ",", "2", ",", "1"}], "]"}]}], ";", "  ", 
  RowBox[{
   RowBox[{"tb", "[", "x__", "]"}], ":=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"b", "[", "x", "]"}], ",", "1", ",", "2"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tc", "[", 
    RowBox[{"var", "[", "x_", "]"}], "]"}], "=", " ", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.657463216548376*^9, 3.657463300596349*^9}, {
  3.657463359323868*^9, 3.657463383571685*^9}, {3.657463439659734*^9, 
  3.657463555690907*^9}},ExpressionUUID->"cfa086bf-7f66-4297-96ea-\
31aed62554f0"],

Cell[BoxData[
 RowBox[{"tcc", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"tensorD", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}]}]], "Input",
 CellChangeTimes->{{3.657463605156432*^9, 3.65746361264223*^9}, {
   3.6574636514503593`*^9, 3.65746366252185*^9}, 3.66911226173951*^9, 
   3.669209311776779*^9, {3.669441899490205*^9, 
   3.669441905050561*^9}},ExpressionUUID->"9627c997-b717-4a32-b532-\
965125f9d21d"],

Cell[BoxData[
 RowBox[{"taa", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"tensorD", "[", 
       RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}], ")"}], 
   "\[CircleTimes]", 
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.657463676041737*^9, 3.657463679849588*^9}, {
   3.657463724273121*^9, 3.6574638124484377`*^9}, 3.6691122617432547`*^9, 
   3.669209311832961*^9},ExpressionUUID->"d992c87c-ff15-4947-ba98-\
a54117713ef1"],

Cell[BoxData[
 RowBox[{"tbb", " ", "=", "  ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"tensorD", "[", 
       RowBox[{"\[CapitalGamma]", ",", " ", "tb"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.657463835342472*^9, 3.657463912496191*^9}, 
   3.669112261748328*^9, 
   3.669209311890979*^9},ExpressionUUID->"b67575bc-d2f1-44c5-974f-\
ce8d9f1ede7e"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"equi", "[", 
    RowBox[{"tcc", ",", 
     RowBox[{"taa", " ", "+", " ", "tbb"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657462483323181*^9, 3.657462541586266*^9}, 
   3.657462790970182*^9, {3.657463965271634*^9, 
   3.657463981687089*^9}},ExpressionUUID->"4faea4e2-2c0b-460f-b463-\
1a49864e26a1"],

Cell[TextData[{
 StyleBox["Mathematica needed 15 min, but v. 9.0.1 only 11.420 s, to confirm \
this result.",
  FontColor->RGBColor[1, 0, 0]],
 "  \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.657465035956985*^9, 3.657465035958047*^9}, {
   3.704163568025332*^9, 3.704163603312673*^9}, {3.708582852998823*^9, 
   3.708582888845627*^9}, {3.7085829188851347`*^9, 3.7085829285975227`*^9}, 
   3.7085829643247213`*^9},ExpressionUUID->"0ed10460-2993-4fa1-9859-\
ca1e58db856d"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.3. The Gradient Tensor", "Subsection",
 CellChangeTimes->{{3.657465181525076*^9, 3.657465203652673*^9}, 
   3.657531026665142*^9, {3.7022173341707153`*^9, 3.702217336410695*^9}, 
   3.7022642081149073`*^9},ExpressionUUID->"3e6ccc2e-8ca9-49f0-bfe2-\
d2cf2ff22bc3"],

Cell[CellGroupData[{

Cell["3.3.1. Definition and Examples", "Subsubsection",
 CellChangeTimes->{{3.6576043105770206`*^9, 3.657604362105603*^9}, 
   3.657639513003221*^9, {3.663082261673394*^9, 3.663082264184663*^9}, 
   3.663139778388404*^9, 3.669112261763117*^9, 3.669112597467101*^9, 
   3.669443140158263*^9, 3.669444837752057*^9, 3.702264217656707*^9, 
   3.702265872650099*^9, 3.703235615744652*^9, 3.703236294019886*^9, {
   3.704163625176409*^9, 3.7041636268801107`*^9}, 
   3.704164076591999*^9},ExpressionUUID->"74e42549-b4c9-4225-b16d-\
5bc5a8e88294"],

Cell[TextData[{
 "Already in Subsection 3.2.1 we mentioned the relation between the absolute \
differential and the gradient tensor. Now we define the coordinates of the \
gradient of a tensor ta evaluating ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["absD", "X"], TraditionalForm]],ExpressionUUID->
  "679599ab-b564-47e1-b033-1c8056f159f4"],
 "[ta] for the basis vector fields ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[PartialD]", "h"], TraditionalForm]],ExpressionUUID->
  "387133ac-371c-4bd0-87be-7b2c24bb3c47"],
 "x"
}], "Text",
 CellChangeTimes->{{3.65649139537873*^9, 3.6564914009144487`*^9}, {
  3.6564914351542673`*^9, 3.6564914701564293`*^9}, {3.657604447896173*^9, 
  3.657604660878953*^9}, {3.657604757482592*^9, 3.657604763474123*^9}, {
  3.657604794889645*^9, 3.657604805350277*^9}, {3.669112261832165*^9, 
  3.669112261853591*^9}, {3.6694436361216803`*^9, 3.6694436985937223`*^9}, {
  3.702264268463479*^9, 3.702264278630213*^9}, {3.7032356800158167`*^9, 
  3.703235694759547*^9}},ExpressionUUID->"37cfc1e2-cb3d-49bc-a625-\
7ee7b2c0d4cf"],

Cell[BoxData[
 RowBox[{"?", "nab"}]], "Input",
 CellChangeTimes->{{3.657523132494933*^9, 3.657523246236598*^9}, {
   3.657523281364259*^9, 3.657523398235338*^9}, {3.657524046461885*^9, 
   3.657524073197365*^9}, {3.662908336127256*^9, 3.662908343847746*^9}, 
   3.663078064653371*^9, {3.663078588171939*^9, 3.6630786056742287`*^9}, 
   3.663078726777981*^9},ExpressionUUID->"74da2d45-dd1a-4aa7-b7f1-\
0c773fbd130f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"nab", "[", 
     RowBox[{"\[CapitalGamma]_", ",", " ", "tens_"}], "]"}], "[", "x__", 
    "]"}], "[", 
   RowBox[{"ii_", ",", "ji_"}], "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"di", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], ",", "pt", ",", "qt"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Syntax", " ", "control"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"tens", "[", "x", "]"}], "]"}], "===", "tensor"}], ",", " ", 
       RowBox[{
        RowBox[{"pt", " ", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
        RowBox[{"qt", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
          RowBox[{"tens", "[", "x", "]"}], ",", "\"\< is not a tensor!\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"Tuples", "[", 
           RowBox[{
            RowBox[{"Range", "[", "di", "]"}], ",", 
            RowBox[{"pt", "+", "1"}]}], "]"}], ",", "ii"}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Invalid argument \>\"", ",", 
          RowBox[{"ii", ";"}], ",", " ", "\"\< !\>\""}], "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"Tuples", "[", 
           RowBox[{
            RowBox[{"Range", "[", "di", "]"}], ",", "qt"}], "]"}], ",", 
          "ji"}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Invalid argument \>\"", ",", "ji", ",", " ", "\"\< !\>\""}], 
         "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"di", " ", "\[Equal]", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
          RowBox[{"[", "3", "]"}], "]"}]}], "]"}], ",", " ", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Tensor dimension \>\"", ",", 
          RowBox[{
           RowBox[{"type", "[", 
            RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", " ", 
          "\"\< is not the manifold dimension\>\"", ",", " ", "di", ",", 
          "\"\< !\>\""}], "]"}], ";", " ", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"absD", "[", 
         RowBox[{"\[CapitalGamma]", ",", " ", "tens"}], "]"}], "[", "x", 
        "]"}], "[", 
       RowBox[{"stb", "[", 
        RowBox[{
         RowBox[{"ii", "[", 
          RowBox[{"[", 
           RowBox[{"pt", "+", "1"}], "]"}], "]"}], ",", "di"}], "]"}], "]"}], 
      "[", 
      RowBox[{
       RowBox[{"ii", "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", "pt"}], "]"}], "]"}], ",", "ji"}], "]"}]}]}], 
   "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.656488947792346*^9, 3.656488970589985*^9}, {
   3.656489035958049*^9, 3.656489038374073*^9}, {3.65648909224609*^9, 
   3.656489118110442*^9}, {3.6564891713976107`*^9, 3.65648917390975*^9}, {
   3.656489204669435*^9, 3.656489211235785*^9}, {3.6564892518613787`*^9, 
   3.6564892598607607`*^9}, {3.6564893207002974`*^9, 3.656489360596406*^9}, {
   3.6564898111765823`*^9, 3.656489923007687*^9}, {3.656489960735326*^9, 
   3.6564900611261063`*^9}, {3.6564901035258703`*^9, 3.656490165493618*^9}, {
   3.6564904901157513`*^9, 3.656490541842905*^9}, {3.656490577026573*^9, 
   3.656490623418254*^9}, {3.656490699325241*^9, 3.656490719117638*^9}, 
   3.656490795338358*^9, {3.656491049543851*^9, 3.65649110635087*^9}, 
   3.6572645034096193`*^9, 3.657267451693083*^9, 3.657522214402823*^9, 
   3.6575224766441727`*^9, 3.6630780555694103`*^9, 3.669112261868601*^9, 
   3.669209222055462*^9},ExpressionUUID->"802dc61a-e959-408f-9c29-\
dd66c33c90a7"],

Cell["The gradient tensor of ta is defined in the next module:", "Text",
 CellChangeTimes->{{3.657604860861328*^9, 3.657604880061565*^9}, {
  3.65760491025333*^9, 
  3.6576049269812737`*^9}},ExpressionUUID->"38efc0a2-63fa-449f-a574-\
5124b5f4b155"],

Cell[BoxData[
 RowBox[{"?", "nabla"}]], "Input",
 CellChangeTimes->{{3.657523455738923*^9, 3.6575234631787863`*^9}, {
   3.657523513811049*^9, 3.657523543090246*^9}, {3.657523585809845*^9, 
   3.657523602697648*^9}, 3.657523663865156*^9, {3.657523792216058*^9, 
   3.6575238580470543`*^9}, 3.657523936295124*^9, {3.6629083948547287`*^9, 
   3.662908405214514*^9}, 
   3.6630782775505247`*^9},ExpressionUUID->"e4653645-4c52-46e9-9ff2-\
e0b75f93c947"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]_", ",", " ", "tens_"}], "]"}], "[", "x__", 
    "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"di", "=", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], ",", "pt", ",", "qt"}], "}"}], ",",
      "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Syntax", " ", "control"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"tens", "[", "x", "]"}], "]"}], "===", "tensor"}], ",", " ", 
        RowBox[{
         RowBox[{"pt", " ", "=", " ", 
          RowBox[{
           RowBox[{"type", "[", 
            RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
         RowBox[{"qt", "=", " ", 
          RowBox[{
           RowBox[{"type", "[", 
            RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
           RowBox[{"tens", "[", "x", "]"}], ",", 
           "\"\< is not a tensor!\>\""}], "]"}], ";", 
         RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"Not", "[", 
         RowBox[{"di", " ", "\[Equal]", " ", 
          RowBox[{
           RowBox[{"type", "[", 
            RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}]}], "]"}], ",", " ", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{"\"\<Tensor dimension \>\"", ",", 
           RowBox[{
            RowBox[{"type", "[", 
             RowBox[{"tens", "[", "x", "]"}], "]"}], "[", 
            RowBox[{"[", "3", "]"}], "]"}], ",", " ", 
           "\"\< is not the manifold dimension\>\"", ",", " ", "di", ",", 
           "\"\< !\>\""}], "]"}], ";", " ", 
         RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"The", " ", "definition"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"tensor", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"nab", "[", 
          RowBox[{"\[CapitalGamma]", ",", " ", "tens"}], "]"}], "[", "x", 
         "]"}], ",", 
        RowBox[{"pt", "+", "1"}], ",", "qt", ",", "di"}], "]"}]}]}], "]"}]}], 
  "\[IndentingNewLine]"}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6565100888730087`*^9, 3.65651012561441*^9}, {
   3.656511818472228*^9, 3.6565118484479933`*^9}, {3.656511908794376*^9, 
   3.6565119226405153`*^9}, 3.656511989519878*^9, {3.656512041841549*^9, 
   3.656512098255233*^9}, {3.656512133695133*^9, 3.6565121602708178`*^9}, 
   3.656512509951805*^9, {3.656512617947453*^9, 3.656512645922299*^9}, 
   3.657521833438445*^9, 3.657521939840366*^9, 3.65752221635462*^9, 
   3.6630780431749*^9, 
   3.66307827755879*^9},ExpressionUUID->"81125c3a-e193-46e7-8864-\
225853cca2aa"],

Cell[TextData[{
 StyleBox["Example 1.",
  FontColor->RGBColor[0, 0, 1]],
 " Functions"
}], "Text",
 CellChangeTimes->{{3.657525476079919*^9, 
  3.657525523384944*^9}},ExpressionUUID->"f1c3f4bc-d5e0-4e0e-9180-\
157a66b42523"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{"Clear", "[", "tf", "]"}], ";", 
  RowBox[{
   RowBox[{"tf", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "0", ",", "0"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6575250495407677`*^9, 3.657525066948503*^9}, {
  3.7022644045745487`*^9, 
  3.702264411822651*^9}},ExpressionUUID->"17dd5ccf-5916-4b37-b025-\
6d90c2e7d791"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{"\[CapitalGamma]", ",", "tf"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657525104866123*^9, 3.6575251555956697`*^9}, 
   3.663078277578661*^9},ExpressionUUID->"91f974cf-767c-4835-a962-\
962c902ce789"],

Cell[BoxData[
 RowBox[{"?", "nabla0"}]], "Input",
 CellChangeTimes->{{3.663137905207178*^9, 
  3.6631379182044764`*^9}},ExpressionUUID->"8dcdb7d2-02db-45d0-8ff3-\
397c8a50eae3"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%%", ",", 
   RowBox[{
    RowBox[{"nabla0", "[", "f", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.663137870965889*^9, 3.663137888905428*^9}, {
   3.6631379424602213`*^9, 3.663137990804035*^9}, 
   3.669444223421376*^9},ExpressionUUID->"d16158a2-5558-472a-9ecc-\
1745760303c3"],

Cell["\<\
The two functions nabla0[f] and nabla[\[CapitalGamma], tf] give the Gradient \
Grad of the  function f:\
\>", "Text",
 CellChangeTimes->{{3.6631382337935143`*^9, 3.663138367719974*^9}, {
   3.70226448268512*^9, 3.702264490437806*^9}, 
   3.7041637171517277`*^9},ExpressionUUID->"aa8dbc96-69cf-47ce-9d41-\
47c0ae70faa0"],

Cell[BoxData[
 RowBox[{"Flatten", "[", 
  RowBox[{"coords", "[", 
   RowBox[{
    RowBox[{"nabla0", "[", "f", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657525170405305*^9, 3.6575251760753717`*^9}, {
  3.6575252147632647`*^9, 3.657525222659289*^9}, {3.663138039212689*^9, 
  3.663138089003478*^9}},ExpressionUUID->"6f5e5b40-3272-4a0c-83a2-\
5749b5d7abc0"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{"Grad", "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65752538071891*^9, 
  3.657525428481388*^9}},ExpressionUUID->"42ff2084-7d7c-47d7-8601-\
c444a6584331"],

Cell[BoxData[
 RowBox[{"?", "Grad"}]], "Input",
 CellChangeTimes->{{3.6575252718611727`*^9, 
  3.657525273650532*^9}},ExpressionUUID->"0ad306cd-cec1-4111-8686-\
b08d279c6723"],

Cell[TextData[{
 "\[FilledSquare]\n",
 StyleBox["Example 2. ",
  FontColor->RGBColor[0, 0, 1]],
 "Vector fields"
}], "Text",
 CellChangeTimes->{{3.657525823399506*^9, 
  3.6575258537731752`*^9}},ExpressionUUID->"9fecbaa0-bc40-4ec3-9de0-\
57da8658e3bb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "a", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",ExpressionUUID->"8fb9a62d-\
fbcf-47ce-9fb2-82ba028161e1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ta", "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"vv", "[", "a", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.656409851247612*^9, 3.656409974154014*^9}, {
   3.6565129026476107`*^9, 3.6565129069665937`*^9}, {3.657524765455882*^9, 
   3.6575248070385027`*^9}, 3.657525038645211*^9, {3.6575258850615673`*^9, 
   3.657525907517067*^9}, {3.663138702097953*^9, 
   3.6631387287501373`*^9}},ExpressionUUID->"6cc0960d-bafa-4f8d-9fd0-\
adef6700d8f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.65752595764637*^9, 3.6575260029962263`*^9}, 
   3.663078277608337*^9},ExpressionUUID->"14e678a8-19ab-40e0-b4f7-\
a4759658ff33"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657526089742125*^9, 
  3.6575261221636543`*^9}},ExpressionUUID->"925920df-52e3-46d7-9665-\
656398634354"],

Cell["The coordinates coincide with those of formula 3.2.2, (2).", "Text",
 CellChangeTimes->{{3.657526823372922*^9, 3.657526954892213*^9}, {
   3.657527496159328*^9, 3.6575275009512653`*^9}, 3.657527542815431*^9, 
   3.66313855685469*^9, 
   3.663138597129222*^9},ExpressionUUID->"2e907707-a259-4b32-9ee6-\
ccd5aa88ea56"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla1", "[", 
   RowBox[{"\[CapitalGamma]", ",", 
    RowBox[{"vv", "[", "a", "]"}]}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663138638185005*^9, 3.663138643243969*^9}, {
  3.6631387996367197`*^9, 
  3.663138842788267*^9}},ExpressionUUID->"3500fb91-ce24-40d1-8d2e-\
e5566e7efbb2"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%", ",", 
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.663138860293771*^9, 
  3.663138892275996*^9}},ExpressionUUID->"f4623480-dedf-4b74-b9b6-\
c81aaef3d183"],

Cell["\<\
 Both operators nabla1[\[CapitalGamma],vv[a]] and nabla[\[CapitalGamma], ta] \
give the same gradient tensor field. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.657526823372922*^9, 3.657526954892213*^9}, {
   3.657527496159328*^9, 3.6575275009512653`*^9}, 3.657527542815431*^9, 
   3.66313855685469*^9, 3.663138597129222*^9, {3.663138939579068*^9, 
   3.663139060542082*^9}},ExpressionUUID->"9c8782e1-ca34-4a80-a1b4-\
dc2b20bc6159"],

Cell[TextData[{
 "\n",
 StyleBox["Example 3.",
  FontColor->RGBColor[0, 0, 1]],
 " Covector fields."
}], "Text",
 CellChangeTimes->{{3.657526823372922*^9, 3.657526954892213*^9}, {
   3.657527496159328*^9, 3.6575275009512653`*^9}, 3.657527542815431*^9, 
   3.66313855685469*^9, 3.663138597129222*^9, {3.663138939579068*^9, 
   3.663139060542082*^9}},ExpressionUUID->"74fcfa85-5d9c-4429-ab4d-\
d0249ade6e26"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ta", "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{
     RowBox[{"vv", "[", "a", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.656409851247612*^9, 3.656409974154014*^9}, {
   3.6565129026476107`*^9, 3.6565129069665937`*^9}, {3.657524765455882*^9, 
   3.6575248070385027`*^9}, 3.657525038645211*^9, {3.6575258850615673`*^9, 
   3.657525907517067*^9}, {3.657527022027502*^9, 3.657527035491722*^9}, {
   3.663139104738104*^9, 
   3.663139168370037*^9}},ExpressionUUID->"79916365-90ca-4df3-948f-\
387df29e7681"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.65752595764637*^9, 3.6575260029962263`*^9}, 
   3.6630782776348953`*^9},ExpressionUUID->"03a5f5ed-f4d0-430b-af34-\
6f17acbb0d88"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657526089742125*^9, 3.6575261221636543`*^9}, {
  3.6575270603549967`*^9, 
  3.657527069715199*^9}},ExpressionUUID->"aff62a19-0a34-4ad6-8d93-\
03fd69aa5429"],

Cell["\<\
The coordinates coincide with those of formula 3.2.3, (3), Lemma 2. \
\[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.657526823372922*^9, 3.657526954892213*^9}, {
   3.65752734586471*^9, 3.6575273539846067`*^9}, {3.657527554591529*^9, 
   3.657527604502324*^9}, 
   3.657530896967448*^9},ExpressionUUID->"e51e4a9c-5f92-4033-862e-\
f3776fa3ddc3"],

Cell[TextData[{
 StyleBox["Example 4.",
  FontColor->RGBColor[0, 0, 1]],
 " Tensor fields of type (0,2)."
}], "Text",
 CellChangeTimes->{{3.657526823372922*^9, 3.657526954892213*^9}, {
   3.65752734586471*^9, 3.6575273539846067`*^9}, {3.657527554591529*^9, 
   3.657527604502324*^9}, {3.657530896967448*^9, 3.657530908130253*^9}, 
   3.657609227070928*^9},ExpressionUUID->"f104ee0a-a905-4abd-8acd-\
21a00131ee98"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "th", "]"}], ";", 
  RowBox[{
   RowBox[{"th", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"h", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "0", ",", "2"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.656409851247612*^9, 3.656409974154014*^9}, {
   3.6565129026476107`*^9, 3.6565129069665937`*^9}, {3.657524765455882*^9, 
   3.6575248070385027`*^9}, 
   3.657525038645211*^9},ExpressionUUID->"5674bd26-8562-4123-b38a-\
4a0b78040a44"],

Cell[BoxData[
 RowBox[{"?", "nab"}]], "Input",
 CellChangeTimes->{{3.663139340643252*^9, 
  3.6631393423124743`*^9}},ExpressionUUID->"d7f1f285-a6d8-442e-8860-\
247cb0cf4bb7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nab", "[", 
    RowBox[{"\[CapitalGamma]", ",", " ", "th"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.656512765396839*^9, 3.656512832935441*^9}, {
   3.656512867407143*^9, 3.656512881758539*^9}, 3.657522220242201*^9, 
   3.663078032293741*^9},ExpressionUUID->"02056ba7-370d-44ec-9b36-\
e6043a3ad13b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{"\[CapitalGamma]", ",", " ", "th"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6565131337678547`*^9, 3.656513156372128*^9}, 
   3.6575218348381767`*^9, 
   3.6630782776397057`*^9},ExpressionUUID->"90916214-caf5-4e4c-8dec-\
d4bcd78942e7"],

Cell[BoxData[
 RowBox[{"%%", "\[Equal]", " ", 
  RowBox[{"%", "[", 
   RowBox[{
    RowBox[{"{", "2", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.656513178072464*^9, 
  3.656513205965015*^9}},ExpressionUUID->"3262e4e2-f222-4c19-adbc-\
a88c4dcd66a0"],

Cell[TextData[{
 StyleBox["Example 5.",
  FontColor->RGBColor[0, 0, 1]],
 " Tensor fields of type (1, 1)."
}], "Text",
 CellChangeTimes->{{3.657526823372922*^9, 3.657526954892213*^9}, {
   3.65752734586471*^9, 3.6575273539846067`*^9}, {3.657527554591529*^9, 
   3.657527604502324*^9}, {3.657530896967448*^9, 3.657530908130253*^9}, {
   3.657530945857669*^9, 3.6575309534177094`*^9}, 
   3.65760921512633*^9},ExpressionUUID->"65d2997c-8b12-4d73-853d-\
b008b6d28361"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ta", "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "1"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.65649163067452*^9, 3.656491686816806*^9}, 
   3.657524893302342*^9},ExpressionUUID->"cc525ae5-15b8-4483-89d8-\
caee58cad504"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "dta", "]"}], ";", 
  RowBox[{
   RowBox[{"dta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"dta", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.657607270106449*^9, 3.65760731650417*^9}, 
   3.663078277647299*^9, {3.704163828881736*^9, 3.7041638336071053`*^9}, {
   3.7041638837427197`*^9, 3.704163897486781*^9}, {3.7041639339838753`*^9, 
   3.70416393515313*^9}},ExpressionUUID->"8098f8c6-bf39-4b69-9d45-\
d0dcc410cfca"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nab", "[", 
    RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.656491697937336*^9, 3.656491718632066*^9}, {
   3.656491759233148*^9, 3.656491788183885*^9}, 3.6575222238103*^9, 
   3.663078018628847*^9},ExpressionUUID->"ceb480f8-e79d-4f76-ad5c-\
b55ecfb4d0af"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"dta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], " ", "\[Equal]", " ", "%"}]], "Input",
 CellChangeTimes->{{3.656513275214595*^9, 3.6565133204424133`*^9}, 
   3.657521836357819*^9, {3.657607344385026*^9, 
   3.657607345638996*^9}},ExpressionUUID->"f4757d7d-779c-4170-87c4-\
82f746a3e0d4"],

Cell["\<\
Using this example we confirm the relation between the gradient and the \
absolute differential mentioned in Subsection 3.2.1. We transform the vector \
field\
\>", "Text",
 CellChangeTimes->{{3.657607909577036*^9, 
  3.6576080002161503`*^9}},ExpressionUUID->"df8673ac-2093-4c73-ac25-\
31e956e3a97d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vx", "=", " ", 
   RowBox[{
    RowBox[{"vv", "[", "v", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.657607065408299*^9, 3.65760713265703*^9}, {
  3.663139600446905*^9, 
  3.6631396205498943`*^9}},ExpressionUUID->"30f70875-3d97-4a88-a0d9-\
92e9ea32b6ac"],

Cell["into the corresponding tensor", "Text",
 CellChangeTimes->{{3.657608015912006*^9, 
  3.657608026336039*^9}},ExpressionUUID->"3a7f44d8-dd3e-42d3-8fdd-\
4b3036cc52d0"],

Cell[BoxData[
 RowBox[{"vX", " ", "=", " ", 
  RowBox[{"vecTotensor", "[", "vx", "]"}]}]], "Input",
 CellChangeTimes->{{3.657607150746668*^9, 
  3.657607187880372*^9}},ExpressionUUID->"52202446-2f0f-402f-ab76-\
d968c623d1a5"],

Cell["Form the tensor product", "Text",
 CellChangeTimes->{{3.657608056735568*^9, 
  3.65760806643155*^9}},ExpressionUUID->"86db9880-cffe-47ba-a760-\
28089e3e0fc4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dta", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", "vX"}]], "Input",
 CellChangeTimes->{{3.657607381468659*^9, 
  3.657607430598431*^9}},ExpressionUUID->"912faccd-29c0-4da6-8cc2-\
f7a3ede84d02"],

Cell["and take the contraction", "Text",
 CellChangeTimes->{{3.657608083639443*^9, 
  3.6576080955351877`*^9}},ExpressionUUID->"7c9328b6-e5de-4e69-bcf9-\
93759b0bbfa2"],

Cell[BoxData[
 RowBox[{"contraction", "[", 
  RowBox[{"%", ",", " ", "2", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.657607454236561*^9, 
  3.657607503421866*^9}},ExpressionUUID->"bdace28f-8771-4fb3-b87b-\
a66c7413724a"],

Cell[TextData[{
 "It coincides with the absolute differential ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["absD", "vx"], TraditionalForm]],ExpressionUUID->
  "41d2e21d-7c5b-4a30-a19d-508c94c41152"],
 "[ta]:"
}], "Text",
 CellChangeTimes->{{3.657608128383156*^9, 3.6576081961663837`*^9}, {
   3.669112261919044*^9, 3.6691122619190903`*^9}, 
   3.702265799798047*^9},ExpressionUUID->"bab8e2ad-7b9a-45ab-92e2-\
be2371bed55c"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"equi", "[", 
    RowBox[{"%", ",", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"tensorD", "[", 
        RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", "vx", "]"}]}], "]"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657607525031991*^9, 3.657607589043571*^9}, {
   3.657607628850986*^9, 3.657607684219551*^9}, 3.669112261934012*^9, 
   3.6692093119500647`*^9},ExpressionUUID->"113f3611-9b10-4319-9ebe-\
4ccd74044ddf"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6576092009598827`*^9, 
  3.657609200961212*^9}},ExpressionUUID->"541c8bce-142d-4fb4-8f83-\
9e1599a1aaa6"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ta", ",", "vx", ",", "vX"}], "]"}]], "Input",
 CellChangeTimes->{{3.702267056539125*^9, 
  3.7022670902479486`*^9}},ExpressionUUID->"3ec82f34-20d7-41ec-b52b-\
4adabe23d29d"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3.2. Properties of Gradient Tensors", "Subsubsection",
 CellChangeTimes->{{3.6576043105770206`*^9, 3.657604362105603*^9}, {
   3.657609288381632*^9, 3.657609353397019*^9}, {3.6576146815336447`*^9, 
   3.65761468434126*^9}, 3.657704459362054*^9, 3.663139789556982*^9, 
   3.663140540501378*^9, 3.669444845832922*^9, 3.669446590837955*^9, 
   3.702265879498847*^9, 3.702268596452388*^9, 3.703236303956588*^9, 
   3.703236738608526*^9, 3.704164084160982*^9, 
   3.704164295322793*^9},ExpressionUUID->"d32f9290-2228-44fc-a786-\
d7f7798ad5a5"],

Cell[TextData[{
 "The basic properties of the gradient operator ",
 "nabla",
 " are inherited from the properties of the absolute differential expressed \
in conditions 1 - 6 of Theorem 3.2.3. \n",
 StyleBox["Proposition.",
  FontColor->RGBColor[0, 0, 1]],
 " the gradient tensor  of a smooth tensor field ta[x] of type (p, q)"
}], "Text",
 CellChangeTimes->{{3.657633948964363*^9, 3.6576339530444927`*^9}, {
   3.657634058451627*^9, 3.657634080907124*^9}, {3.657634136034874*^9, 
   3.657634258793848*^9}, {3.6576342923376007`*^9, 3.657634315265512*^9}, {
   3.657634369312875*^9, 3.6576343777688026`*^9}, {3.657634420224381*^9, 
   3.65763444178408*^9}, {3.65763474943752*^9, 3.6576347950130796`*^9}, {
   3.657634946036006*^9, 3.6576349517716913`*^9}, 3.6576950336585197`*^9, 
   3.663078277682548*^9},ExpressionUUID->"e2f624b9-9064-4701-9ce1-\
b96d480ff710"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Del]", 
   RowBox[{"ta", "[", "x", "]"}]}], " ", "\[DotEqual]", 
  RowBox[{
   RowBox[{"nabla", "[", 
    RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6576345354559317`*^9, 3.6576346431911287`*^9}, 
   3.6576346792463713`*^9, 
   3.6630782777374897`*^9},ExpressionUUID->"7ef882da-6136-4f14-8145-\
4d00f3f7f732"],

Cell["\<\
is a smooth tensor field of type (p+1, q). The operator \
\[Del]\[DotEqual]nabla has the following properties:
1. \[Del] is a linear differential operator of the first order depending on \
the linear connection \[CapitalGamma]:\
\>", "Text",
 CellChangeTimes->{{3.657634875828306*^9, 3.657634929763562*^9}, {
   3.657634961971599*^9, 3.657635008843029*^9}, {3.657635044819515*^9, 
   3.657635249201025*^9}, {3.657695460304226*^9, 3.657695476743351*^9}, 
   3.663078277765214*^9},ExpressionUUID->"d4cb35c1-f1a7-4aee-b722-\
6caba800b29d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Del]", 
    RowBox[{"[", 
     RowBox[{"ta", " ", "+", "tb"}], "]"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"\[Del]", "ta"}], " ", "+", "  ", 
    RowBox[{"\[Del]", "tb"}]}]}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Del]", 
     RowBox[{"[", "\[Alpha]ta", "]"}]}], " ", "\[Equal]", " ", 
    RowBox[{"\[Alpha]", 
     RowBox[{"\[Del]", "ta"}], " ", 
     StyleBox["for",
      FontColor->GrayLevel[0]], " ", "\[Alpha]"}]}], "\[Element]", 
   RowBox[{"\[DoubleStruckCapitalR]", "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.657635264505069*^9, 
  3.657635433231834*^9}},ExpressionUUID->"4e9d961f-50b8-44f1-a506-\
bb654ecc2739"],

Cell["\<\
2. For a function f[x] the coordinates of \[Del]tensor[f[x],0,0] coincide \
with those of d[f][x].
3. The gradient tensor of a vector field has the coordinates given in formula \
(2) of Subsection 3.2.2.
4. For all contractions of tensor fields \[FormalCapitalY]ta one has\
\>", "Text",
 CellChangeTimes->{{3.657635551252048*^9, 3.657635568630146*^9}, {
  3.6576356008059263`*^9, 3.65763560204567*^9}, {3.657635635413393*^9, 
  3.657635867403468*^9}, {3.657635974306609*^9, 3.657636054609748*^9}, {
  3.657636185680629*^9, 3.657636204288679*^9}, {3.6576364335507193`*^9, 
  3.657636460654108*^9}},ExpressionUUID->"92dca7db-d277-46e3-9f18-\
4a9eecb6d076"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Del]", 
   RowBox[{"[", "\[FormalCapitalY]ta", "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{"\[FormalCapitalY]", 
   RowBox[{
    RowBox[{"\[Del]", "ta"}], "."}]}]}]], "Input",
 Evaluatable->False,ExpressionUUID->"2a717274-1fa6-4542-bc8a-72e1ed3d3d65"],

Cell["5. The product rule ", "Text",
 CellChangeTimes->{{3.657636666900165*^9, 3.6576366786519747`*^9}, {
   3.6576367278116503`*^9, 3.657636823818822*^9}, 
   3.657636859024355*^9},ExpressionUUID->"6c37a75e-b752-498d-ae96-\
eb06704b2952"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Del]", 
   RowBox[{"[", 
    RowBox[{"ta", "\[CircleTimes]", "tb"}], "]"}]}], " ", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Del]", 
      RowBox[{"[", "ta", "]"}]}], ")"}], "\[CircleTimes]", "tb"}], "  ", "+", 
   " ", 
   RowBox[{"ta", "\[CircleTimes]", 
    RowBox[{"\[Del]", 
     RowBox[{"[", "tb", "]"}]}]}]}]}]], "Input",
 Evaluatable->False,ExpressionUUID->"fe176c96-af79-4e31-90df-633a14a47553"],

Cell["\<\
is valid for functions and contravariant tensors,  and not in general for \
products of covariant or mixed type tensors.\
\>", "Text",
 CellChangeTimes->{{3.657636666900165*^9, 3.6576366786519747`*^9}, {
   3.6576367278116503`*^9, 3.657636823818822*^9}, 3.657636859024355*^9, {
   3.657637981592765*^9, 3.6576380193042088`*^9}, 
   3.657695549270464*^9},ExpressionUUID->"05db0c7c-9ff1-4e9d-a3b7-\
d3a9c334bdb4"],

Cell["\<\
6. The relation between the absolute differential and the gradient tensor is \
the contraction\
\>", "Text",
 CellChangeTimes->{{3.65763742111732*^9, 3.6576374256132383`*^9}, {
  3.6576374565090427`*^9, 3.657637504900352*^9}, {3.6576955764462147`*^9, 
  3.657695582486183*^9}},ExpressionUUID->"7c9c85b6-47cf-4c2c-abf3-\
00a2f7c1cc26"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["absD", "X"], 
   RowBox[{"ta", "[", "x", "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{"\[FormalCapitalY]", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"ta", "[", "x", "]"}], "\[CircleTimes]", "X"}], ",", 
    RowBox[{"p", "+", "1"}], ",", 
    RowBox[{"q", "+", "1"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.657637545546153*^9, 3.657637595740447*^9}, {
   3.657637641188078*^9, 3.657637702363364*^9}, 
   3.6691122619433527`*^9},ExpressionUUID->"6bd33721-2105-4082-a5e1-\
4b65efc9e71e"],

Cell["\<\
if ta[x] is a tensor field of type (p, q), and a statement   analogous to \
condition 6 of Theorem 3.2.3 is satisfied for the restricted operator \
\[Del]|U.\
\>", "Text",
 CellChangeTimes->{{3.657637717762844*^9, 3.657637903897456*^9}, {
  3.657695609966387*^9, 
  3.6576956223500013`*^9}},ExpressionUUID->"9533c0f1-ac64-4ba7-bae2-\
7f0707201299"],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ".  Instead of a proof we confirm the statements only calculating \
appropriate examples."
}], "Text",
 CellChangeTimes->{{3.657637948248877*^9, 3.657637952752717*^9}, 
   3.657695652405369*^9, {3.657704341474708*^9, 
   3.657704398050461*^9}},ExpressionUUID->"96fa8f29-28a2-460d-ab08-\
4a170fb265fe"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb"}], "]"}], ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "1"}], "]"}]}], 
  ";", " ", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"b", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "1"}], "]"}]}], 
  ";", 
  RowBox[{
   RowBox[{"ts", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", "  ", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.6576387218778276`*^9, 3.657638837465934*^9}, {
  3.657638951937969*^9, 
  3.657639001804124*^9}},ExpressionUUID->"f65d1300-6c6b-4092-b4b4-\
b613b00548f2"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", " ", "ts"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", " ", "tb"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657638702921576*^9, 3.657638710034409*^9}, {
  3.6576388599770727`*^9, 3.657638940840817*^9}, {3.657639032897147*^9, 
  3.657639198806818*^9}, {3.6576392389901457`*^9, 3.657639267822234*^9}, {
  3.6630782778161983`*^9, 
  3.663078277819593*^9}},ExpressionUUID->"31e83532-019a-4357-94ac-\
6fb94f7f032c"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", 
       RowBox[{"Pi", "*", " ", "ta"}]}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"Pi", "*", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657638702921576*^9, 3.657638710034409*^9}, {
  3.6576388599770727`*^9, 3.657638940840817*^9}, {3.657639032897147*^9, 
  3.657639198806818*^9}, {3.6576392389901457`*^9, 3.657639267822234*^9}, {
  3.657639299037702*^9, 3.657639373934216*^9}, {3.6630782778252363`*^9, 
  3.663078277827297*^9}},ExpressionUUID->"617f4ae1-5ef0-4d7b-aa6c-\
278c5ec23d47"],

Cell["\<\
This  confirms the linearity. Properties 2 and 3 are confirmed in the \
Examples 1 and 2 of the last Subsection 3.3.1. We consider an example \
confirming the property 4:\
\>", "Text",
 CellChangeTimes->{{3.657639430611718*^9, 3.657639489107016*^9}, {
  3.657639542522835*^9, 3.657639585122748*^9}, {3.657695863043722*^9, 
  3.6576958902915907`*^9}},ExpressionUUID->"92f4623b-ef17-4b43-8818-\
1c54ac505d5f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ta", "]"}], ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "2"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6576958956155376`*^9, 
  3.657695981923099*^9}},ExpressionUUID->"05c2f64e-54a1-43a3-9c25-\
4a70e0f114ae"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.657695995106694*^9, 3.657695998114787*^9}, {
   3.657696037284212*^9, 3.657696061564619*^9}, 
   3.6630782778325243`*^9},ExpressionUUID->"337f7fcf-414e-449d-b799-\
e50077b014c9"],

Cell[BoxData[
 RowBox[{"contraction", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "2"}], 
  "]"}]], "Input",
 CellChangeTimes->{
  3.663078277835977*^9},ExpressionUUID->"3ac169d9-07bb-42cf-813e-\
44562c902579"],

Cell["We define a contraction of the tensor ta[var[x]]:", "Text",
 CellChangeTimes->{{3.6576968301473017`*^9, 
  3.6576968642117*^9}},ExpressionUUID->"7c1f5ac1-121f-4492-ae2f-0aef6c5a0b24"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tb", "]"}], ";", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{"ta", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "2"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.657696537750183*^9, 3.657696568967754*^9}, {
  3.657696869764686*^9, 
  3.657696892540462*^9}},ExpressionUUID->"22833cf1-de06-4d09-b1e8-\
c3a137cd0ea0"],

Cell["The difference", "Text",
 CellChangeTimes->{{3.6576969214030647`*^9, 
  3.657696927403194*^9}},ExpressionUUID->"7116973f-7fe7-4aa9-ab4e-\
61cc7c5f765a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", "1", ",", "2"}], "]"}], "-", 
  RowBox[{
   RowBox[{"nabla", "[", 
    RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.657695995106694*^9, 3.657695998114787*^9}, {
  3.657696037284212*^9, 3.657696210386676*^9}, {3.6576962442099323`*^9, 
  3.6576962706250668`*^9}, {3.65769660253566*^9, 3.657696662126629*^9}, {
  3.663078277839499*^9, 
  3.663078277840885*^9}},ExpressionUUID->"112654e3-9c07-4831-ace0-\
4eb2b7afe123"],

Cell["equals the nulltensor", "Text",
 CellChangeTimes->{{3.657696969482769*^9, 
  3.657696980386632*^9}},ExpressionUUID->"6c514204-2873-4c5f-925d-\
9ce7dce34050"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%", ",", 
   RowBox[{"nulltensor", "[", 
    RowBox[{"1", ",", "1", ",", "3"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657696693390196*^9, 
  3.657696728589445*^9}},ExpressionUUID->"5c59df49-f438-49dc-97bb-\
ec386e31adff"],

Cell["\<\
The second part of Property 6 is a consequence of property 6 in Theorem \
3.2.3; it has been confirmed in example 5 of Subsection 3.3.1. We conclude \
with confirming property 5 in a series of examples:\
\>", "Text",
 CellChangeTimes->{{3.657697220608768*^9, 3.6576972977920837`*^9}, {
   3.65769735454428*^9, 3.6576973789835*^9}, {3.65769743547106*^9, 
   3.657697565989406*^9}, {3.669445699414319*^9, 3.669445710213999*^9}, 
   3.7022673781502237`*^9, {3.703236529146468*^9, 
   3.703236530458105*^9}},ExpressionUUID->"0b938766-0152-4a6d-98ee-\
7bdd12231f19"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc", ",", " ", "taa", ",", "tbb"}], "]"}], 
  ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"a", ",", "0", ",", "0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"b", ",", "0", ",", "0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ")"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tbb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6576976147743387`*^9, 3.657697839043828*^9}, 
   3.657697876403973*^9, {3.6576979682356997`*^9, 3.657697987754698*^9}, {
   3.657698031514449*^9, 3.657698156353023*^9}, {3.657698188888818*^9, 
   3.657698236472454*^9}, {3.663078277844997*^9, 
   3.66307827784653*^9}},ExpressionUUID->"ddbf5bf5-bf27-49b5-8421-\
09ae26935722"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{"taa", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", 
    RowBox[{"tbb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6576982601929197`*^9, 3.657698358247448*^9}, 
   3.663078277853006*^9},ExpressionUUID->"a0f79c7e-ed4c-42b7-9da7-\
477dcf1f8215"],

Cell["\<\
If one of the factors is of type (0, 0) the product rule is fulfilled:\
\>", "Text",
 CellChangeTimes->{{3.657702003863244*^9, 3.657702070566979*^9}, 
   3.657702129720874*^9},ExpressionUUID->"4b1c83ca-d27d-4c52-9d3d-\
b3aa58a53f8a"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc", ",", " ", "taa", ",", "tbb"}], "]"}], 
  ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"a", ",", "2", ",", "1"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"b", ",", "0", ",", "0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ")"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tbb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6576976147743387`*^9, 3.657697839043828*^9}, 
   3.657697876403973*^9, {3.6576979682356997`*^9, 3.657697987754698*^9}, {
   3.657698031514449*^9, 3.657698156353023*^9}, {3.657698188888818*^9, 
   3.657698236472454*^9}, {3.657701434765357*^9, 3.657701465173251*^9}, 
   3.65770190140147*^9, {3.6630782778640823`*^9, 
   3.6630782778660717`*^9}},ExpressionUUID->"ec4128ed-e3d9-45d1-a610-\
39c8fdfaf36f"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{"taa", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", 
    RowBox[{"tbb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6576982601929197`*^9, 3.657698358247448*^9}, 
   3.663078277880403*^9},ExpressionUUID->"e7d30bd1-4add-440b-ad8d-\
6a79df6c36f1"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc", ",", " ", "taa", ",", "tbb"}], "]"}], 
  ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"a", ",", "0", ",", "0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"b", ",", "2", ",", "2"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ")"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tbb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6576976147743387`*^9, 3.657697839043828*^9}, 
   3.657697876403973*^9, {3.6576979682356997`*^9, 3.657697987754698*^9}, {
   3.657698031514449*^9, 3.657698156353023*^9}, {3.657698188888818*^9, 
   3.657698236472454*^9}, 3.6577014993325567`*^9, {3.657701546076996*^9, 
   3.657701549948515*^9}, {3.657701623499886*^9, 3.6577016388514557`*^9}, 
   3.657701935433589*^9, {3.6630782778928213`*^9, 
   3.663078277894939*^9}},ExpressionUUID->"f97c2e43-f0ba-43f3-acc0-\
1a38d5c2a9ba"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{"taa", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", 
    RowBox[{"tbb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6576982601929197`*^9, 3.657698358247448*^9}, 
   3.663078277906439*^9},ExpressionUUID->"0df311ab-b3fb-47b9-9ce5-\
d951b28dd8f7"],

Cell["Both factors contravariant:", "Text",
 CellChangeTimes->{{3.657702272613228*^9, 
  3.65770229142898*^9}},ExpressionUUID->"ecce5bb0-01e9-4289-bb98-\
b701efaf3f6c"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc", ",", " ", "taa", ",", "tbb"}], "]"}], 
  ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"a", ",", "0", ",", "1"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"b", ",", "0", ",", "2"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ")"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tbb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6576976147743387`*^9, 3.657697839043828*^9}, 
   3.657697876403973*^9, {3.6576979682356997`*^9, 3.657697987754698*^9}, {
   3.657698031514449*^9, 3.657698156353023*^9}, {3.657698188888818*^9, 
   3.657698236472454*^9}, 3.6577014993325567`*^9, {3.657701546076996*^9, 
   3.657701549948515*^9}, {3.657701623499886*^9, 3.6577016388514557`*^9}, 
   3.657701755562749*^9, {3.6577022103033533`*^9, 3.657702213574727*^9}, {
   3.663078277918462*^9, 
   3.6630782779204884`*^9}},ExpressionUUID->"2bc1f3c9-4003-422d-86c5-\
445a931e9050"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{"taa", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", 
    RowBox[{"tbb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6576982601929197`*^9, 3.657698358247448*^9}, 
   3.66307827793073*^9},ExpressionUUID->"f5107067-3c48-43c5-b92f-\
089fede12b33"],

Cell["\<\
The product rule isn\[CloseCurlyQuote]t valid for the tensor product of a \
vector and a covector field:\
\>", "Text",
 CellChangeTimes->{{3.657703287988164*^9, 
  3.657703349619741*^9}},ExpressionUUID->"2716b97f-3919-4a4e-8ca4-\
086f92fce384"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc", ",", " ", "taa", ",", "tbb"}], "]"}], 
  ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"a", ",", "0", ",", "1"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"b", ",", "1", ",", "0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ")"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tbb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6576976147743387`*^9, 3.657697839043828*^9}, 
   3.657697876403973*^9, {3.6576979682356997`*^9, 3.657697987754698*^9}, {
   3.657698031514449*^9, 3.657698156353023*^9}, {3.657698188888818*^9, 
   3.657698236472454*^9}, 3.6577014993325567`*^9, {3.657701546076996*^9, 
   3.657701549948515*^9}, {3.657701623499886*^9, 3.6577016388514557`*^9}, {
   3.6577023076779833`*^9, 3.657702316061275*^9}, {3.657702627651301*^9, 
   3.6577026367144747`*^9}, {3.66307827794141*^9, 
   3.6630782779432917`*^9}},ExpressionUUID->"853db18d-5910-431f-8c2b-\
672af0fa8a1c"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"taa", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], " ", "+", 
     RowBox[{"tbb", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ")"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6576982601929197`*^9, 3.657698358247448*^9}, {
   3.6577023438301687`*^9, 3.65770237765306*^9}, 
   3.66307827795161*^9},ExpressionUUID->"cb7f4ff9-d5c5-434a-afce-\
3b253a0523f5"],

Cell["and this isn\[CloseCurlyQuote]t the null tensor, since e.g.", "Text",
 CellChangeTimes->{{3.657703164253497*^9, 3.657703190132613*^9}, 
   3.7032366911605873`*^9},ExpressionUUID->"c1dfe928-5054-4ce3-ab88-\
00d6526982f0"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], ",", 
    RowBox[{"{", "1", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6577024100385447`*^9, 3.657702437300263*^9}, 
   3.657702813706006*^9, {3.669446082427569*^9, 3.6694461063466187`*^9}, 
   3.669446191698605*^9},ExpressionUUID->"5590544c-1896-49d7-bafa-\
1445e0e6cb3c"],

Cell["\<\
isn\[CloseCurlyQuote]t zero  in general.
Also it isn\[CloseCurlyQuote]t valid in the case of two covector fields:\
\>", "Text",
 CellChangeTimes->{{3.657703201676015*^9, 3.657703211613263*^9}, 
   3.657703243246662*^9, {3.65770389251884*^9, 3.657703904910864*^9}, {
   3.657703994469983*^9, 3.6577040233898087`*^9}, 3.657704084407605*^9, 
   3.6577041222471533`*^9, {3.657704168012384*^9, 3.657704185540572*^9}, 
   3.7022679288337812`*^9},ExpressionUUID->"8373f3ba-708a-42b2-b857-\
6700522dc63f"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ta", ",", "tb", ",", "tc", ",", " ", "taa", ",", "tbb"}], "]"}], 
  ";", " ", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"a", ",", "1", ",", "0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{"tensor", "[", 
    RowBox[{"b", ",", "1", ",", "0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"tc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"nabla", "[", 
       RowBox[{"\[CapitalGamma]", ",", "ta"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ")"}], "\[CircleTimes]", 
    RowBox[{"tb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tbb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
   RowBox[{
    RowBox[{"ta", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
    RowBox[{
     RowBox[{"nabla", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tb"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6576976147743387`*^9, 3.657697839043828*^9}, 
   3.657697876403973*^9, {3.6576979682356997`*^9, 3.657697987754698*^9}, {
   3.657698031514449*^9, 3.657698156353023*^9}, {3.657698188888818*^9, 
   3.657698236472454*^9}, 3.6577014993325567`*^9, {3.657701546076996*^9, 
   3.657701549948515*^9}, {3.657701623499886*^9, 3.6577016388514557`*^9}, {
   3.65770337370864*^9, 3.65770338380445*^9}, {3.663078277960066*^9, 
   3.663078277961926*^9}},ExpressionUUID->"2097513a-db25-459a-ae46-\
36a1e9ca87ce"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tc"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
   RowBox[{"taa", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "-", 
   RowBox[{"tbb", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6576982601929197`*^9, 3.657698358247448*^9}, 
   3.663078277970189*^9, {3.66944624736208*^9, 
   3.669446278153296*^9}},ExpressionUUID->"07fd15d0-5d52-4424-b77f-\
4be3b03f7c77"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "1"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6576982601929197`*^9, 3.657698358247448*^9}, 
   3.663078277970189*^9, {3.66944624736208*^9, 3.669446278153296*^9}, 
   3.669446316209684*^9, {3.66944636086392*^9, 
   3.669446413000861*^9}},ExpressionUUID->"41bc771a-ac88-4773-9c57-\
999f5516b4d6"],

Cell["\<\
and this coordinate isn\[CloseCurlyQuote]t  zero in general.
Further examples see and explore in the subnotebook testD.nb.\
\>", "Text",
 CellChangeTimes->{{3.657704202100152*^9, 3.657704291403482*^9}, 
   3.669112261979897*^9, 3.669446573997799*^9, {3.702268513860059*^9, 
   3.702268530212747*^9}},ExpressionUUID->"933df079-4a70-4642-a284-\
698afe15a73b"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4. Parallelism", "Subsection",
 CellChangeTimes->{{3.657776594638961*^9, 3.6577766079907417`*^9}, {
   3.65777717053061*^9, 3.657777177834587*^9}, {3.7084379696733503`*^9, 
   3.708437971856511*^9}, 
   3.708480627513241*^9},ExpressionUUID->"f97efea7-fac4-413c-b55d-\
1c5eb2f12604"],

Cell[CellGroupData[{

Cell["3.4.1.  Absolute Parallel Vector Fields", "Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, {
   3.5972984401750546`*^9, 3.59729846072029*^9}, {3.5972984954771514`*^9, 
   3.5972985062723703`*^9}, {3.5973320886172123`*^9, 3.5973320905123205`*^9}, 
   3.5973952636966553`*^9, 3.6132229905260477`*^9, 3.6132797959577127`*^9, 
   3.6171013236210327`*^9, 3.6171717453084965`*^9, {3.6188032063086033`*^9, 
   3.6188032409562645`*^9}, 3.618817261348339*^9, {3.657777713518718*^9, 
   3.657777719622416*^9}, 3.657778396824327*^9, 3.669446597640415*^9, 
   3.669528122455851*^9, 3.702268604196622*^9, 3.702275129045824*^9, 
   3.7032367966890297`*^9, 3.7032434720197363`*^9, 3.7041643171880074`*^9, 
   3.704164797688899*^9},ExpressionUUID->"8c27d539-2c29-4e5b-8cef-\
cfe163df29b4"],

Cell[TextData[{
 "Already in the notebook euklidsfv3. nb (see [",
 ButtonBox["E",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "D",
 ButtonBox["G",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "] ) on surfaces in the Euclidean space we considered basic concepts of \
parallelism. The next three subsections are generalizations of the \
subsections with the same title in the mentioned notebook. "
}], "Text",
 CellChangeTimes->{{3.657777257389125*^9, 3.657777325153277*^9}, {
   3.657777364889122*^9, 3.657777369601035*^9}, {3.6577774258256197`*^9, 
   3.657777430464757*^9}, {3.657777468497836*^9, 3.657777468499263*^9}, {
   3.657777528103983*^9, 3.657777580631167*^9}, {3.6577776149037333`*^9, 
   3.657777697542715*^9}, {3.6577787066463423`*^9, 3.657778729437894*^9}, 
   3.658120650370913*^9, 3.663140622509186*^9, 3.669112261994947*^9, 
   3.6694467146053543`*^9, 
   3.702272867882976*^9},ExpressionUUID->"6c2848c4-a337-47b7-8241-\
34721c357eab"],

Cell["\<\
For the following we assume that a smooth n-dimensional manifold provided \
with a smooth linear connection \[CapitalGamma] is given. A  tensor field, e. \
g. the vector field\
\>", "Text",
 CellChangeTimes->{{3.5972985733524885`*^9, 3.5972986623350444`*^9}, {
   3.5972987038467174`*^9, 3.597298884011434*^9}, 3.5972989562083607`*^9, 
   3.6132230395101337`*^9, {3.657778901651731*^9, 3.657779085242001*^9}, {
   3.657779743758381*^9, 3.657779747772336*^9}, {3.657784635944704*^9, 
   3.657784660088266*^9}},ExpressionUUID->"f5b15f5c-e173-4a3f-beb6-\
a75f158319eb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"Clear", "[", "vx", "]"}], ";", 
  RowBox[{
   RowBox[{"vx", "[", "x__", "]"}], " ", ":=", 
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5956590461387978`*^9, 3.5956590587255173`*^9}, 
   3.595924528295635*^9, {3.597299146747095*^9, 3.5972992078212023`*^9}, {
   3.597300291508306*^9, 3.5973003191827545`*^9}, {3.5973004499577847`*^9, 
   3.5973004648714104`*^9}, {3.657779096626377*^9, 3.657779121778213*^9}, {
   3.6577791802014313`*^9, 3.657779347648019*^9}, 3.663140688036812*^9, {
   3.663140731292418*^9, 3.66314074126823*^9}, {3.66314078542854*^9, 
   3.663140797947753*^9}, {3.663140841820014*^9, 3.663140856134142*^9}, {
   3.66314088752323*^9, 
   3.663140912642973*^9}},ExpressionUUID->"989d756d-ea2c-438c-9f71-\
0acdde76efd5"],

Cell[BoxData[
 RowBox[{"vx", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663140919161334*^9, 
  3.663140929202702*^9}},ExpressionUUID->"07e05d51-d8ca-485d-8719-\
ae0919e3ba36"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vX", "]"}], ";", 
  RowBox[{
   RowBox[{"vX", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{"vx", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5956590461387978`*^9, 3.5956590587255173`*^9}, 
   3.595924528295635*^9, {3.597299146747095*^9, 3.5972992078212023`*^9}, {
   3.597300291508306*^9, 3.5973003191827545`*^9}, {3.5973004499577847`*^9, 
   3.5973004648714104`*^9}, {3.657779096626377*^9, 3.657779121778213*^9}, {
   3.6577791802014313`*^9, 3.657779347648019*^9}, 3.663140688036812*^9, {
   3.663140731292418*^9, 3.66314074126823*^9}, {3.66314078542854*^9, 
   3.663140797947753*^9}, {3.663140841820014*^9, 3.663140856134142*^9}, 
   3.663140969619285*^9, {3.663141025331306*^9, 
   3.663141025929863*^9}},ExpressionUUID->"55813b9b-85cc-4d60-b1e6-\
a5b776aa39e2"],

Cell[TextData[{
 "is said to be ",
 StyleBox["absolute parallel",
  FontColor->RGBColor[0, 0, 1]],
 " if its gradient tensor equals the nulltensor, in traditional notation"
}], "Text",
 CellChangeTimes->{
  3.5972989670035796`*^9, {3.597299243826066*^9, 3.5972992583808913`*^9}, {
   3.657779416687078*^9, 3.657779458774632*^9}, {3.6577794940786133`*^9, 
   3.657779504725933*^9}, 3.6577797625085287`*^9, 
   3.657784685704193*^9},ExpressionUUID->"4468828d-9089-4f15-844d-\
ea646c2632d9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Del]", 
   RowBox[{"vX", "[", "x", "]"}]}], "\[Equal]", " ", "0."}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5972993146657906`*^9, 3.5972993494538517`*^9}, {
   3.597299402821545*^9, 3.597299430792394*^9}, 3.5972995294157677`*^9, 
   3.5973003459836016`*^9, 3.6132230887126207`*^9, {3.624339079502738*^9, 
   3.62433908661635*^9}, 3.6304781022676134`*^9, {3.6577795747420063`*^9, 
   3.657779616277816*^9}, 
   3.657779658909153*^9},ExpressionUUID->"1f4ca29d-fcce-40ee-af0a-\
dee15afef468"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". In the ",
 StyleBox["affine space",
  FontColor->RGBColor[0, 0, 1]],
 "  (s. Example 1 in 3.3.3) the absolute differential coincides with the \
total differential. Therefore the absolute parallel vector fields are the \
vector fields with constant Cartesian coordinates.\nThe coordinates of the \
gradient tensor are"
}], "Text",
 CellChangeTimes->{{3.5973040529501977`*^9, 3.597304222288495*^9}, {
  3.6577822979677258`*^9, 3.657782372662259*^9}, {3.657782404782179*^9, 
  3.6577824175260363`*^9}, {3.657783187415016*^9, 3.657783209174737*^9}, {
  3.657784006247472*^9, 3.6577840162873487`*^9}, {3.7022732367911463`*^9, 
  3.702273238118081*^9}},ExpressionUUID->"97c1e5c8-2b33-4e78-945b-\
652fe06a77cd"],

Cell[BoxData[
 RowBox[{"?", "nab"}]], "Input",
 CellChangeTimes->{{3.658055567927423*^9, 3.6580555711003513`*^9}, 
   3.663078010940448*^9},ExpressionUUID->"52e097c0-0cc3-4e2d-8be0-\
448372ecfe8c"],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"nab", "[", 
      RowBox[{"\[CapitalGamma]0", ",", " ", "vX"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", "i", "}"}], ",", 
     RowBox[{"{", "j", "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "dim"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.59730430471904*^9, 3.5973043671191497`*^9}, {
   3.6577825232453938`*^9, 3.6577826001084747`*^9}, {3.657782640299621*^9, 
   3.65778271657935*^9}, {3.6577830475768147`*^9, 3.657783052465279*^9}, 
   3.663078006433963*^9},ExpressionUUID->"8c6035a6-8ede-40fb-9294-\
3581a4a3925c"],

Cell["This matrix coincides with the built-in", "Text",
 CellChangeTimes->{{3.657783230302144*^9, 
  3.65778324756579*^9}},ExpressionUUID->"a62ac8bc-113c-4469-a052-\
f6c997fbb82a"],

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"vx", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657782968440633*^9, 
  3.657783020480575*^9}},ExpressionUUID->"0fa093e9-bbb4-45b5-87d6-\
b7aaf84409f4"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "\[Equal]", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.624340005551564*^9, 3.6243400125091763`*^9}, {
  3.657782837673809*^9, 3.657782877809792*^9}, {3.6577829234331284`*^9, 
  3.657782937953404*^9}, {3.6577830932890873`*^9, 
  3.657783097671336*^9}},ExpressionUUID->"ddcfb97f-651f-4705-a1f8-\
f4489a589378"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"c", "[", "x__", "]"}], "[", "1", "]"}], ":=", " ", "a1"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"c", "[", "x__", "]"}], "[", "2", "]"}], ":=", " ", "a2"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"c", "[", "x__", "]"}], "[", "3", "]"}], ":=", " ", "a3"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5972997287685175`*^9, 3.597299731280122*^9}, {
  3.597299801963846*^9, 3.5972999486041036`*^9}, {3.657783295062289*^9, 
  3.65778331574192*^9}, {3.6577833539028273`*^9, 3.657783372925378*^9}, {
  3.657783657139282*^9, 
  3.657783699010421*^9}},ExpressionUUID->"32436526-9f94-425b-ab78-\
d2afdcf98459"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vc", "[", 
   RowBox[{"var", "[", "x_", "]"}], "]"}], " ", "=", " ", 
  RowBox[{"vec", "[", 
   RowBox[{"c", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6577834521385593`*^9, 3.657783511331834*^9}, {
  3.657783546731839*^9, 
  3.657783577171596*^9}},ExpressionUUID->"d2e11b27-b057-426e-86e8-\
820ad8ed1fc7"],

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"vc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5972999616457267`*^9, 3.5972999994601927`*^9}, {
  3.6577837468746*^9, 
  3.657783757025793*^9}},ExpressionUUID->"24dcff7d-0808-42e5-8aba-\
be48cdf1f29a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vcX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vc", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.657783782156748*^9, 
  3.657783876664316*^9}},ExpressionUUID->"8405d5c0-4419-4c25-bc4f-\
7737573b5d05"],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"nab", "[", 
      RowBox[{"\[CapitalGamma]0", ",", " ", "vcX"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", "i", "}"}], ",", 
     RowBox[{"{", "j", "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "dim"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.6577839385276403`*^9, 
  3.663077981735462*^9},ExpressionUUID->"77d2f613-270e-4239-8397-\
f320e4e8d3dd"],

Cell["\<\
We conclude : In the n-dimensional affine space there exist parallel  basis \
vector fields\
\>", "Text",
 CellChangeTimes->{{3.657784214812504*^9, 3.6577842572285213`*^9}, {
  3.657784287828476*^9, 3.657784393291482*^9}, {3.6578635061611023`*^9, 
  3.657863519849275*^9}},ExpressionUUID->"2a58fdb9-1090-4a67-aea9-\
cc4819d36b5b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vb", "[", "x__", "]"}], "[", "i_", "]"}], ":=", " ", 
  RowBox[{"stb", "[", "i", "]"}]}]], "Input",
 CellChangeTimes->{{3.657784424991912*^9, 
  3.65778449953083*^9}},ExpressionUUID->"99c239b2-70b8-4933-a95f-\
e14fb984ef7e"],

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"vb", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", "2", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"var", "[", "x", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657784504090435*^9, 
  3.6577845925534163`*^9}},ExpressionUUID->"182e8740-aa3c-4fc8-8fd4-\
6c895f67cf53"],

Cell[TextData[{
 "One easily proves: In an affine space with the flat affine connection \
\[CapitalGamma]0 a tensor field is parallel iff its coordinates with respect \
to a parallel basis field are constants.\n",
 StyleBox["Remark:",
  FontColor->RGBColor[0, 0, 1]],
 " First one sees that the cobasis to a parallel basis field consists of \
parallel covector fields. \[FilledSquare] "
}], "Text",
 CellChangeTimes->{{3.6577847212805777`*^9, 3.6577849910353928`*^9}, 
   3.6578635363212357`*^9, {3.6578635690098457`*^9, 3.657863618736594*^9}, 
   3.702273856353457*^9},ExpressionUUID->"a3206238-8b95-4a4b-bc75-\
7e9239bc5e63"],

Cell["\<\
Now we consider the general case and ask whether there exist absolute \
parallel vector fields. The system of linear partial differential equations \
of first order to find them is:\
\>", "Text",
 CellChangeTimes->{
  3.6244561411776648`*^9, {3.6244578001781783`*^9, 3.6244578152010045`*^9}, 
   3.6580555232379723`*^9, {3.658055614004425*^9, 
   3.658055616308076*^9}},ExpressionUUID->"79527412-39e2-4427-9112-\
a04eb67fb7f0"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"nab", "[", 
        RowBox[{"\[CapitalGamma]", ",", "vX"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", "i", "}"}], ",", 
       RowBox[{"{", "j", "}"}]}], "]"}], "\[Equal]", " ", "0"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5973046678096776`*^9, 3.5973048127651324`*^9}, 
   3.597305117074867*^9, {3.5973055552952366`*^9, 3.597305559070443*^9}, 
   3.614861862498745*^9, 3.6148635021396246`*^9, 3.6188037315971227`*^9, 
   3.6188037658630824`*^9, {3.624355033463996*^9, 3.624355045413617*^9}, 
   3.624355133787772*^9, {3.624456851836912*^9, 3.6244568734897504`*^9}, {
   3.624456941880271*^9, 3.6244569555926948`*^9}, 3.624457008055587*^9, 
   3.626010748996644*^9, {3.657864020221589*^9, 3.657864076885758*^9}, {
   3.6578641774367657`*^9, 3.657864305211672*^9}, 
   3.663077971470063*^9},ExpressionUUID->"cd80d172-3f5a-4dff-b7e6-\
9ed5d39bb07a"],

Cell[TextData[{
 "Using the definition of the absolute differential we may rewrite this \
system in traditional form as a system of differential equations for the \
coordinates ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}},ExpressionUUID->
  "be47260b-42f1-4011-9f33-3871c8e0ea58"],
 " of the vector field "
}], "Text",
 CellChangeTimes->{{3.597306271258494*^9, 3.5973063053289537`*^9}, {
   3.6244579954437213`*^9, 3.624457998626127*^9}, 3.6578683545693398`*^9, 
   3.6578684317635107`*^9, {3.657868483547271*^9, 
   3.6578684953601103`*^9}},ExpressionUUID->"aae788e6-72a7-4122-9a74-\
accaae21d55a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{"-", " ", 
     SubsuperscriptBox["\[CapitalSigma]", 
      RowBox[{"i", "=", "1"}], "dim"]}], 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "[", 
      RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}], " ", 
    RowBox[{
     SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}], ",", "    ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"k", ",", 
     RowBox[{"j", "=", "1"}], ",", "..", ",", "dim"}], ")"}], ".", 
   "                                             ", 
   RowBox[{"(", "1", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}, {3.5973064685364404`*^9, 3.597306522949336*^9}, {
   3.5973134928002768`*^9, 3.597313703088646*^9}, {3.6243553099588814`*^9, 
   3.624355313094487*^9}, 3.6244574499575634`*^9, 3.6244577130520253`*^9, {
   3.6244580529610224`*^9, 3.624458069403451*^9}, 3.626010754051053*^9, {
   3.657864422498458*^9, 3.657864454466215*^9}, {3.6578688787820683`*^9, 
   3.657868882445883*^9}, {3.7022740179913387`*^9, 3.702274033298581*^9}, {
   3.702274113094756*^9, 
   3.702274130942346*^9}},ExpressionUUID->"af6680c9-940b-44ac-ab48-\
e654e344d65e"],

Cell[TextData[{
 "Assume that we found a solution (",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}], ",", " ", 
   RowBox[{"k", " ", "=", " ", "1"}], ",", "...", ",", " ", "dim"}]],
  Evaluatable->False,
  CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}, {3.5973064685364404`*^9, 3.597306522949336*^9}},
  ExpressionUUID->"8ca36392-5fc2-425e-88f0-3209a47411d2"],
 ") of this system. Then these functions satisfy also the following \
derivations of the equations:"
}], "Text",
 CellChangeTimes->{{3.5973065921510572`*^9, 3.5973067715513725`*^9}, 
   3.5973137235558815`*^9, {3.657868678582477*^9, 
   3.657868700727544*^9}},ExpressionUUID->"6268b5cc-f6b0-4f7f-8a67-\
e876be63bd2d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]}]}], " ", "\[Equal]", " ", 
  RowBox[{"-", " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["\[CapitalSigma]", 
       RowBox[{"i", "=", "1"}], "dim"], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}], " ", 
      RowBox[{
       SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}], ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973058712737913`*^9, 3.5973059305226955`*^9}, {
   3.597305962658752*^9, 3.5973060124072394`*^9}, {3.5973060436228943`*^9, 
   3.5973060926225805`*^9}, {3.5973064685364404`*^9, 3.597306522949336*^9}, {
   3.597313407951728*^9, 3.5973134700554366`*^9}, {3.597313752103932*^9, 
   3.5973138567021155`*^9}, {3.5973138906945753`*^9, 3.5973138933309803`*^9}, 
   3.62445771568843*^9, 3.626010757545459*^9, 3.6578687515182657`*^9, {
   3.657868807237688*^9, 3.657868811573596*^9}, 3.6578688847571898`*^9},
 FontSize->16,ExpressionUUID->"0591743e-82b9-445f-bd80-b46a1a0ae16b"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"\[Equal]", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SubsuperscriptBox["\[CapitalSigma]", 
       RowBox[{"i", "=", "1"}], "dim"]}], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", "h"], 
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}]}], ")"}], 
     RowBox[{
      SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}], "-", " ", 
    RowBox[{
     SubsuperscriptBox["\[CapitalSigma]", 
      RowBox[{"i", "=", "1"}], "dim"], 
     RowBox[{
      RowBox[{"\[CapitalGamma]", "[", 
       RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}], " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", "h"], 
      RowBox[{
       SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597313955278689*^9, 3.5973139664483085`*^9}, {
   3.5973140016887703`*^9, 3.597314030439621*^9}, {3.5973140614368753`*^9, 
   3.597314082777713*^9}, 3.5973141235249844`*^9, 3.597314177547879*^9, {
   3.5973142101831365`*^9, 3.597314235361581*^9}, {3.597315982396927*^9, 
   3.5973159873392096`*^9}, {3.6243553651517787`*^9, 
   3.6243553788330026`*^9}, {3.6244574555423727`*^9, 
   3.6244574570555754`*^9}, {3.624457717810034*^9, 3.624457722209241*^9}, {
   3.6260107592926617`*^9, 3.6260107606186647`*^9}, {3.6578688867067957`*^9, 
   3.657868888668687*^9}, {3.657868956796336*^9, 3.657868987028034*^9}, {
   3.657869017907654*^9, 3.6578690324114637`*^9}},
 FontSize->16,ExpressionUUID->"c3f7b561-bdd9-459c-8c22-d798c6b051cc"],

Cell["\<\
for  j ,k ,h= 1,..,dim. Inserting equation (1) with renamed indices  \
\>", "Text",
 CellChangeTimes->{{3.5973139027065964`*^9, 3.597313937791058*^9}, {
   3.5973141374558086`*^9, 3.597314150653432*^9}, 3.5973145527441382`*^9, {
   3.597314592071807*^9, 3.597314653442315*^9}, 3.597314768898118*^9, 
   3.597315097138294*^9, {3.617168754455643*^9, 3.61716875815285*^9}, 
   3.6171688253109674`*^9, 
   3.6578688905832872`*^9},ExpressionUUID->"c8fc2628-4f7f-4206-8523-\
9ce131be871e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{
    SuperscriptBox["\[Xi]", "l"], "[", "x", "]"}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"-", 
    StyleBox[
     SubsuperscriptBox["\[CapitalSigma]", 
      RowBox[{"i", "=", "1"}], "dim"],
     FontSize->14]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "[", 
     RowBox[{"l", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}], 
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597314922839188*^9, 3.597314928517598*^9}, 
   3.5973149670652657`*^9, {3.5973150064397354`*^9, 3.5973150128513465`*^9}, {
   3.597315389576408*^9, 3.5973154186548595`*^9}, 3.5973156631852884`*^9, {
   3.624355399549839*^9, 3.6243554015622425`*^9}, 3.624457465822791*^9, 
   3.6244577240812445`*^9, 3.6260107708366823`*^9, 3.657868892540552*^9, {
   3.657869077795073*^9, 3.6578690918108253`*^9}},
 FontSize->16,ExpressionUUID->"dab90942-56e8-44b3-a851-fedd90a9f1fc"],

Cell["into the second term we obtain for this term", "Text",
 CellChangeTimes->{
  3.597315106404711*^9, {3.6171688319097795`*^9, 3.6171688324869804`*^9}, 
   3.6304784009925375`*^9},ExpressionUUID->"0dfb44af-73e8-4417-87de-\
560dfbe65cf1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", " ", 
    SubsuperscriptBox["\[CapitalSigma]", 
     RowBox[{"l", "=", "1"}], "dim"]}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "[", 
     RowBox[{"k", ",", "j", ",", "l"}], "]"}], "[", "x", "]"}], " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "l"], "[", "x", "]"}]}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"+", 
    SubsuperscriptBox["\[CapitalSigma]", 
     RowBox[{"l", "=", "1"}], "dim"]}], 
   SubsuperscriptBox["\[CapitalSigma]", 
    RowBox[{"i", "=", "1"}], "dim"], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "[", 
     RowBox[{"k", ",", "j", ",", "l"}], "]"}], "[", "x", "]"}], 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "[", 
     RowBox[{"l", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}], 
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973146918963823`*^9, 3.597314697137992*^9}, {
   3.5973147316608524`*^9, 3.597314749491684*^9}, 3.597314788897353*^9, {
   3.5973148208618093`*^9, 3.597314844620651*^9}, 3.5973150316181793`*^9, {
   3.5973154540045214`*^9, 3.5973154840501738`*^9}, {3.597315550833891*^9, 
   3.597315574561533*^9}, {3.5973156251212215`*^9, 3.597315676570112*^9}, {
   3.6243554243694825`*^9, 3.624355459876145*^9}, {3.6244574673671937`*^9, 
   3.6244574700191984`*^9}, {3.6244577257816477`*^9, 
   3.6244577292916536`*^9}, {3.626010772287485*^9, 3.6260107749862895`*^9}, {
   3.657868894770891*^9, 3.6578689004283648`*^9}, {3.6578691155232267`*^9, 
   3.657869128250454*^9}, {3.657869175386836*^9, 3.657869179610553*^9}, 
   3.657869253353953*^9, {3.657869322497311*^9, 
   3.6578693446013536`*^9}},ExpressionUUID->"065bbdba-75c5-42c0-b0af-\
d652daa78f18"],

Cell[TextData[{
 "Adding the results for the two terms and collecting the terms at ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]],
  CellChangeTimes->{{3.5973146918963823`*^9, 3.597314697137992*^9}, {
    3.5973147316608524`*^9, 3.597314749491684*^9}, 3.597314788897353*^9, {
    3.5973148208618093`*^9, 3.597314844620651*^9}, 3.5973150316181793`*^9, {
    3.5973154540045214`*^9, 3.5973154840501738`*^9}, {3.597315550833891*^9, 
    3.597315574561533*^9}, {3.5973156251212215`*^9, 3.597315676570112*^9}},
  ExpressionUUID->"70227ab7-1b35-4465-b755-fd4193359ece"],
 " yields"
}], "Text",
 CellChangeTimes->{{3.597315754383049*^9, 3.5973157695774755`*^9}, {
  3.597315915735114*^9, 
  3.597315949999074*^9}},ExpressionUUID->"97e274f3-2e3b-4e09-a941-\
aa799ee1a088"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    RowBox[{
     SuperscriptBox["\[Xi]", "k"], "[", "x", "]"}]}]}], " ", "\[Equal]", 
  RowBox[{
   RowBox[{"-", " ", 
    SubsuperscriptBox["\[CapitalSigma]", 
     RowBox[{"i", "=", "1"}], "dim"]}], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "h"], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}]}], "-", 
     RowBox[{
      SubsuperscriptBox["\[CapitalSigma]", 
       RowBox[{"l", "=", "1"}], "dim"], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "j", ",", "l"}], "]"}], "[", "x", "]"}], 
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"l", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}]}]}], ")"}], 
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973160039211583`*^9, 3.5973160378971014`*^9}, {
   3.5973160743631873`*^9, 3.5973160847617817`*^9}, {3.597316129803358*^9, 
   3.5973162206325536`*^9}, {3.5973162523453674`*^9, 
   3.5973163002021046`*^9}, {3.5973163303778305`*^9, 3.597316379267627*^9}, {
   3.624355489484997*^9, 3.624355507643429*^9}, {3.624457473201604*^9, 
   3.624457475775608*^9}, {3.6244577309452567`*^9, 3.6244577351572638`*^9}, {
   3.6260107762030916`*^9, 3.6260107782622957`*^9}, {3.657868901892771*^9, 
   3.657868903444703*^9}, {3.6578691886819277`*^9, 3.6578691967946167`*^9}, {
   3.657869409369123*^9, 3.6578694580323467`*^9}, 3.657896409213023*^9, 
   3.657896516299144*^9},ExpressionUUID->"f0b6a4ea-5696-49f9-9d25-\
ac5cd5c1072b"],

Cell[TextData[{
 "Permuting h and j and subtracting from the last equation the left side \
becomes zero, and we obtain a system of linear equations for the coordinates ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}],
   FontWeight->"Bold"]],
  CellChangeTimes->{{3.5973160039211583`*^9, 3.5973160378971014`*^9}, {
   3.5973160743631873`*^9, 3.5973160847617817`*^9}, {3.597316129803358*^9, 
   3.5973162206325536`*^9}, {3.5973162523453674`*^9, 
   3.5973163002021046`*^9}, {3.5973163303778305`*^9, 3.597316379267627*^9}},
  ExpressionUUID->"3bcef173-9209-47a5-ac7c-6bd6e6cf22e7"],
 ":"
}], "Text",
 CellChangeTimes->{{3.5973174735432158`*^9, 3.5973175753910413`*^9}, {
   3.5973176149943066`*^9, 3.597317622350727*^9}, 3.5973279986752806`*^9, 
   3.61744129428474*^9, {3.618804143009654*^9, 3.618804146752868*^9}, 
   3.6188232318984947`*^9},ExpressionUUID->"d64f0830-b565-49c6-b0cc-\
729f7bb219c6"],

Cell[BoxData[
 RowBox[{"0", " ", "=", 
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"i", "=", "1"}], "dim"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", "h"], 
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", "j"], 
       RowBox[{
        RowBox[{"\[CapitalGamma]", "[", 
         RowBox[{"k", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}]}], "-", 
      " ", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"l", "=", "1"}], "dim"], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[CapitalGamma]", "[", 
            RowBox[{"k", ",", "j", ",", "l"}], "]"}], "[", "x", "]"}], 
          RowBox[{
           RowBox[{"\[CapitalGamma]", "[", 
            RowBox[{"l", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}]}], "-", 
         " ", 
         RowBox[{
          RowBox[{
           RowBox[{"\[CapitalGamma]", "[", 
            RowBox[{"k", ",", "h", ",", "l"}], "]"}], "[", "x", "]"}], 
          RowBox[{
           RowBox[{"\[CapitalGamma]", "[", 
            RowBox[{"l", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}]}]}], 
        ")"}]}]}], ")"}], "*", 
    RowBox[{
     SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5973280116780243`*^9, 3.5973280157692585`*^9}, {
   3.597328075913699*^9, 3.5973280771907716`*^9}, {3.597328179819642*^9, 
   3.5973283575508075`*^9}, 3.597328398054124*^9, {3.6243555266598625`*^9, 
   3.6243555695755377`*^9}, {3.624457481126418*^9, 3.624457494230441*^9}, {
   3.624457739135271*^9, 3.6244577508508916`*^9}, {3.62599134814475*^9, 
   3.625991350297554*^9}, {3.625991389609623*^9, 3.6259914949098077`*^9}, {
   3.6259915253142614`*^9, 3.625991539354286*^9}, 3.625991618711625*^9, {
   3.6259916918601537`*^9, 3.6259917018441715`*^9}, {3.625991864521257*^9, 
   3.6259918794816833`*^9}, 3.6259922141334715`*^9, 3.625992261089554*^9, {
   3.625992315455649*^9, 3.6259923329744797`*^9}, {3.625992396872192*^9, 
   3.6259924242658405`*^9}, 3.6260011022667756`*^9, {3.6260011462432528`*^9, 
   3.626001173870901*^9}, {3.626001322273962*^9, 3.6260013379519897`*^9}, {
   3.6260013718040495`*^9, 3.6260014063581095`*^9}, {3.6260014403193693`*^9, 
   3.626001453704193*^9}, {3.6260107796350975`*^9, 3.6260107865303097`*^9}, {
   3.657868905044663*^9, 3.657868906556864*^9}, {3.657869513952508*^9, 
   3.657869673247216*^9}},ExpressionUUID->"f3dc24d1-0455-41c3-b8c3-\
3ab277cdc21c"],

Cell[TextData[{
 "The coefficients of this linear system are the coordinates of the",
 StyleBox["  curvature tensor",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[", ",
  FontColor->GrayLevel[0]],
 "defined for a  space with a linear connection ",
 "\[CapitalGamma]"
}], "Text",
 CellChangeTimes->{{3.5973884413774414`*^9, 3.597388472552224*^9}, {
   3.5973885165367403`*^9, 3.5973885696807795`*^9}, 3.5973913039921737`*^9, {
   3.624355928769169*^9, 3.6243559380511847`*^9}, {3.6244583522007475`*^9, 
   3.62445836482117*^9}, {3.6244585643767204`*^9, 3.6244586082907977`*^9}, {
   3.6578697288457937`*^9, 3.657869802413093*^9}},
 CellTags->"Rtens",ExpressionUUID->"a89a289d-3039-4de5-bd14-d0772c1828b2"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.657983571313335*^9, 3.6579836012263107`*^9}, {
   3.65798382850333*^9, 3.657984057892851*^9}, {3.6579843425068893`*^9, 
   3.6579843845063972`*^9}, 3.657984480698185*^9, {3.662618417078274*^9, 
   3.662618430646068*^9}, 
   3.662622984915092*^9},ExpressionUUID->"1b7c1ea7-78ba-4f61-91f3-\
6fafb959dd14"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "RC", "]"}], ";", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"RC", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"[", "\[CapitalGamma]_", "]"}], "k_"}], ",", "i_", ",", "h_", 
      ",", "j_"}], "]"}], "[", "x__", "]"}], ":=", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}], ",", 
      RowBox[{
       RowBox[{"{", "x", "}"}], "[", 
       RowBox[{"[", "h", "]"}], "]"}]}], "]"}], "-", " ", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"k", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}], ",", 
      RowBox[{
       RowBox[{"{", "x", "}"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], "]"}], "+", "\[IndentingNewLine]", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"k", ",", "h", ",", "l"}], "]"}], "[", "x", "]"}], 
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"l", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}]}], "-", 
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"k", ",", "j", ",", "l"}], "]"}], "[", "x", "]"}], 
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"l", ",", "h", ",", "i"}], "]"}], "[", "x", "]"}]}]}], 
       ")"}], ",", 
      RowBox[{"{", 
       RowBox[{"l", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6595356352763023`*^9, 3.659535747407957*^9}, {
   3.659535861072917*^9, 3.659535869274328*^9}, {3.659535910009501*^9, 
   3.659535960329501*^9}, {3.6595360213197117`*^9, 3.659536144835541*^9}, {
   3.659536175370736*^9, 3.6595362112503757`*^9}, {3.659536250947853*^9, 
   3.659536313658745*^9}, {3.659536350483696*^9, 3.659536416178493*^9}, 
   3.6626229849931717`*^9, 3.662623303390524*^9, {3.6634877757508297`*^9, 
   3.663487782687522*^9}, {3.669112262048911*^9, 3.669112262049838*^9}, {
   3.669526858940096*^9, 
   3.669526864755772*^9}},ExpressionUUID->"3e6c329e-7896-4aa2-bd6f-\
9e0c77fe2277"],

Cell[TextData[{
 StyleBox["Remark",
  FontColor->RGBColor[0, 0, 1]],
 ". This well known expression coincides with the formula contained in \
Proposition 7.6, p.145 in  ",
 ButtonBox["S. Kobayashi, K. Nomizu I",
  BaseStyle->"Hyperlink",
  ButtonData->"K-N1"],
 "."
}], "Text",
 CellChangeTimes->{{3.5973885997855015`*^9, 3.5973886194826283`*^9}, {
   3.5973886805381203`*^9, 3.597388763016838*^9}, {3.597388807320372*^9, 
   3.5973888593523483`*^9}, {3.5973892481165843`*^9, 3.597389248119584*^9}, {
   3.5974686722211447`*^9, 3.5974687529512863`*^9}, 3.597471054875729*^9, {
   3.6259248594520626`*^9, 3.625924876177292*^9}, {3.626001672478977*^9, 
   3.6260016794989896`*^9}, 3.657948664731814*^9, 3.6579487484184217`*^9, {
   3.663141449166141*^9, 
   3.663141450309968*^9}},ExpressionUUID->"992f270a-cb80-4587-9755-\
54e42a70ad87"],

Cell["The corresponding curvature tensor is", "Text",
 CellChangeTimes->{{3.657983437505641*^9, 
  3.6579834603376503`*^9}},ExpressionUUID->"abb95b64-1e7f-47f0-a991-\
044ff99005ce"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.662618503050748*^9, 3.6626185209099503`*^9}, 
   3.662618589486178*^9, 
   3.662622985019661*^9},ExpressionUUID->"35651188-95c6-4887-9071-\
dbce2fa19d99"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RCtensor", "[", "\[CapitalGamma]_", "]"}], "[", "x__", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"arr", ",", 
       RowBox[{"di", "=", 
        RowBox[{"Length", "[", 
         RowBox[{"{", "x", "}"}], "]"}]}], ",", "k", ",", "h", ",", "i", ",", 
       "j"}], "}"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"arr", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"h_", ",", "i_", ",", "j_"}], "}"}], ",", 
         RowBox[{"{", "k_", "}"}]}], "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{"RC", "[", "\[CapitalGamma]", "]"}], "[", 
         RowBox[{"k", ",", "h", ",", "i", ",", "j"}], "]"}], "[", "x", 
        "]"}]}], ";", 
      RowBox[{"tensor", "[", 
       RowBox[{"arr", ",", "3", ",", "1", ",", "di"}], "]"}]}]}], "]"}]}], 
  ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6626186492681093`*^9, {3.662622985048908*^9, 3.662622985051586*^9}, {
   3.6634858644848137`*^9, 
   3.663485872861246*^9}},ExpressionUUID->"76792c45-d722-4fef-a664-\
d521deaea38d"],

Cell["\<\
The integrability condition for the existence of parallel vector fields is\
\>", "Text",
 CellChangeTimes->{{3.5973885997855015`*^9, 3.5973886194826283`*^9}, {
   3.5973886805381203`*^9, 3.597388763016838*^9}, {3.597388807320372*^9, 
   3.5973888593523483`*^9}, {3.5973892481165843`*^9, 3.597389248119584*^9}, {
   3.5974686722211447`*^9, 3.5974687529512863`*^9}, 3.597471054875729*^9, {
   3.6259248594520626`*^9, 3.625924876177292*^9}, {3.626001672478977*^9, 
   3.6260016794989896`*^9}, 3.657948664731814*^9, {3.658054146657074*^9, 
   3.658054166567882*^9}},ExpressionUUID->"bc0091e9-d2ff-4419-9e46-\
ddeb23111be9"],

Cell[TextData[{
 StyleBox["Proposition 1",
  FontColor->RGBColor[0, 0, 1]],
 ". An absolute parallel vector field with coordinates ",
 Cell[BoxData[
  SubscriptBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}], ")"}], 
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", 
    RowBox[{"--", 
     RowBox[{",", "n"}]}]}]]],
  CellChangeTimes->{{3.5973280116780243`*^9, 3.5973280157692585`*^9}, {
    3.597328075913699*^9, 3.5973280771907716`*^9}, {3.597328179819642*^9, 
    3.5973283575508075`*^9}, 3.597328398054124*^9},ExpressionUUID->
  "4f6cb728-6e0c-4e72-851e-ffdd9897a307"],
 "  on a n-dimensional smooth manifold with a linear connection ",
 "\[CapitalGamma] ",
 "satisfies the system of linear equations"
}], "Text",
 CellChangeTimes->{{3.597468651223508*^9, 3.5974686651231318`*^9}, {
  3.5974687616873016`*^9, 3.5974687668353105`*^9}, {3.59746886327468*^9, 
  3.5974690072941327`*^9}, {3.5974692417313447`*^9, 3.5974692445393496`*^9}, {
  3.59746943280048*^9, 3.59746948360977*^9}, {3.59747449972058*^9, 
  3.5974745004849815`*^9}, {3.624458646869665*^9, 3.6244587141837835`*^9}, {
  3.66952710216124*^9, 3.6695271492086353`*^9}, {3.6695272463365602`*^9, 
  3.6695272684793663`*^9}},ExpressionUUID->"9f3791f0-99ef-4673-b105-\
29878ac9dc31"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"RC", "[", "\[CapitalGamma]", "]"}], "[", 
       RowBox[{"k", ",", "i", ",", "j", ",", "h"}], "]"}], "[", "x", "]"}], 
     RowBox[{
      SuperscriptBox["\[Xi]", "i"], "[", "x", "]"}]}]}], "\[Equal]", " ", 
   "0"}], ",", "           ", 
  RowBox[{"(", 
   RowBox[{"k", ",", " ", "j", ",", " ", 
    RowBox[{"h", " ", "\[Equal]", " ", "1"}], ",", "...", ",", "n"}], 
   ")"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.5974690605526266`*^9, 3.5974691322035522`*^9}, {
   3.597469170220819*^9, 3.5974692133236947`*^9}, {3.5974693011986494`*^9, 
   3.5974693148330727`*^9}, {3.669527174744669*^9, 3.6695271857209272`*^9}, 
   3.669527306887145*^9, {3.703242264965454*^9, 
   3.703242311165227*^9}},ExpressionUUID->"e7b7c53a-06c5-47fc-b7d7-\
3b15dc8f9c43"],

Cell["\<\
A necessary condition for the existence of such a non zero vector field is \
that the rank of this system is less the dimension dim of the manifold. \
Obviously, the set of all absolute parallel vector fields is a finite \
dimensional vector space under pointwise addition and multiplication with \
real numbers. \[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.597469516494627*^9, 3.597469627332822*^9}, 
   3.597471074609764*^9, {3.597471155386706*^9, 3.5974713113089795`*^9}, 
   3.624458742887834*^9, 
   3.6578689086405582`*^9},ExpressionUUID->"f563c600-3a3c-4735-b5d4-\
6abeaadb2f49"],

Cell[TextData[{
 StyleBox["Example 1.",
  FontColor->RGBColor[0, 0, 1]],
 " The flat linear connection. "
}], "Text",
 CellChangeTimes->{{3.6579846885673513`*^9, 
  3.657984763662321*^9}},ExpressionUUID->"a51a3582-ef88-4192-bf98-\
2eb214048a88"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]0"}]], "Input",
 CellChangeTimes->{{3.657981990187101*^9, 
  3.65798200595037*^9}},ExpressionUUID->"7fb766d3-8360-4c13-900b-\
98eeb017fd23"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalGamma]0", "[", 
   RowBox[{"1", ",", "2", ",", "1"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.66348602437122*^9, 
  3.663486041442956*^9}},ExpressionUUID->"91581f02-b9bf-4f47-9509-\
47e4c2a7b54e"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.6634860853003683`*^9, 
  3.663486087380945*^9}},ExpressionUUID->"25a67263-89d0-4443-9f85-\
55b00b09d552"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"RC", "[", "\[CapitalGamma]0", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.657982052705742*^9, 3.6579821138775387`*^9}, {
   3.658211832158313*^9, 3.658211837893098*^9}, 3.66262298505976*^9, {
   3.663485933973254*^9, 3.663485941204619*^9}, 3.663485983636218*^9, {
   3.703242981360964*^9, 
   3.703242985568036*^9}},ExpressionUUID->"302ba1cb-a812-41ee-961b-\
f69785b56c18"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCtensor", "[", "\[CapitalGamma]0", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6579830014141693`*^9, 3.657983003518161*^9}, 
   3.662622985068623*^9},ExpressionUUID->"dd8448d4-8a3c-48ac-a2b6-\
e497dcbb5d68"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"%", ",", 
   RowBox[{"nulltensor", "[", 
    RowBox[{"3", ",", "1", ",", "dim"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.657983018227825*^9, 
  3.657983065077528*^9}},ExpressionUUID->"eabe7d56-8be0-4eec-83c9-\
dfb3db8f65cb"],

Cell[TextData[{
 "\[FilledSquare]\n",
 StyleBox["Example 2.",
  FontColor->RGBColor[0, 0, 1]],
 " A general unspecified linear connection"
}], "Text",
 CellChangeTimes->{{3.6579849744847183`*^9, 3.6579850280461597`*^9}, {
  3.662618122896182*^9, 
  3.6626181290325203`*^9}},ExpressionUUID->"b653c3a7-52b1-4388-aa58-\
f3cfc54c3330"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "2"}], ";", " ", 
  RowBox[{"R2", "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6579830014141693`*^9, 3.657983003518161*^9}, {
   3.657983116733531*^9, 3.657983144333126*^9}, 3.662620949783338*^9, 
   3.6626229850798397`*^9, 
   3.662624622864542*^9},ExpressionUUID->"eb79c592-8a6d-443e-9577-\
f15c915a4112"],

Cell[BoxData[
 RowBox[{
  RowBox[{"R2", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
    RowBox[{"{", "1", "}"}]}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"RC", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"1", ",", "1", ",", "1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.657983691536312*^9, 3.657983693456108*^9}, {
   3.657983731513092*^9, 3.65798377675915*^9}, {3.662620961816133*^9, 
   3.662620966951714*^9}, {3.6626210126797667`*^9, 3.6626210326071663`*^9}, 
   3.662622985088999*^9, {3.662624642240266*^9, 3.662624654456579*^9}, {
   3.663487872917891*^9, 3.6634878799183207`*^9}, {3.7032432047661867`*^9, 
   3.7032432092947407`*^9}},ExpressionUUID->"c6746995-a3aa-490f-ba6d-\
dfd999e36c1a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"R2", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
    RowBox[{"{", "1", "}"}]}], "]"}], " ", "+", 
  RowBox[{"R2", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "1"}], "}"}], ",", 
    RowBox[{"{", "1", "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.704164726281125*^9, 
  3.704164743041*^9}},ExpressionUUID->"02cb4797-b5bf-4097-bd25-90f12fa3b3f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "5"}], ";", 
  RowBox[{"TensorSymmetry", "[", 
   RowBox[{"tensorToArray", "[", 
    RowBox[{
     RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.662624710024238*^9, 
  3.6626248051421757`*^9}},ExpressionUUID->"ccf55731-82cd-4f6f-9723-\
c6310b351e05"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6626248409800034`*^9, 
  3.6626248464056463`*^9}},ExpressionUUID->"a6a2c12b-ef2a-4dc6-9041-\
632e1cdc3fde"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.703243437646666*^9, 
  3.7032434376488*^9}},ExpressionUUID->"34e4c7a1-77a6-4445-8ae6-40c20a53c44d"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4.2. Parallel Displacement Along Curves", "Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, {
   3.5972984401750546`*^9, 3.59729846072029*^9}, {3.5972984954771514`*^9, 
   3.5972985062723703`*^9}, {3.5973320886172123`*^9, 3.5973320905123205`*^9}, 
   3.5973952636966553`*^9, {3.597729800827932*^9, 3.59772986233884*^9}, 
   3.6132798036301517`*^9, 3.6132847212454233`*^9, 3.6171717529369097`*^9, 
   3.61717393361794*^9, 3.6188231691395845`*^9, 3.6188258241208286`*^9, {
   3.6245360039693832`*^9, 3.6245360091017923`*^9}, 3.6245420308786135`*^9, {
   3.657778345065104*^9, 3.657778348225174*^9}, {3.6580655394070883`*^9, 
   3.658065542191123*^9}, 3.6581542382495728`*^9, {3.6631592697611732`*^9, 
   3.663159282528934*^9}, 3.6633400616517467`*^9, 3.6691122620697927`*^9, 
   3.669446789211906*^9, 3.669528131256102*^9, 3.669614028821847*^9, 
   3.702275142486474*^9, 3.702345845365279*^9, 3.7032434813720217`*^9, 
   3.703685341692194*^9, 3.704164805761888*^9, 
   3.704166842386614*^9},ExpressionUUID->"50e25c95-63c6-4fd0-b1f0-\
46a071506155"],

Cell["\<\
In the last subsection we have seen that non zero absolute parallel vector \
fields exist only under certain integrability conditions expressed as \
restrictions to the curvature tensor. In dimension 1 these conditions don\
\[CloseCurlyQuote]t appear. Thus it is possible for manifolds with a linear \
connection to define parallel vector functions along curves. Aiming at this \
we first define the absolute derivative of a vector function  vx[t] along a \
curve \[Gamma][t] with the coordinate representation\
\>", "Text",
 CellChangeTimes->{{3.597735507062501*^9, 3.5977355331009903`*^9}, {
   3.5977355962056*^9, 3.597735634300779*^9}, {3.5977358303089895`*^9, 
   3.5977359083014507`*^9}, {3.59773603346961*^9, 3.597736128501045*^9}, {
   3.5977362858940477`*^9, 3.5977364648612843`*^9}, {3.5978475796643496`*^9, 
   3.5978476595364895`*^9}, {3.6132799006777024`*^9, 3.613279904972948*^9}, {
   3.6260716008312755`*^9, 3.626071601205676*^9}, 3.658065644711104*^9, {
   3.658120754729314*^9, 3.658120808432866*^9}, {3.658120865536499*^9, 
   3.65812094392738*^9}, 3.658120977823554*^9, {3.658121352212626*^9, 
   3.658121374171877*^9}, 3.658121408899905*^9, {3.658121446164083*^9, 
   3.6581214908832808`*^9}, 3.658121634297441*^9, {3.658121689705737*^9, 
   3.6581217114014997`*^9}, 
   3.66961084690104*^9},ExpressionUUID->"d1a68a8f-4b4e-49ae-bd3d-\
9454f4f7e9cb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{"Clear", "[", "\[Gamma]", "]"}], ";", " ", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "i", "]"}], "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}], ";", 
  RowBox[{"\[Gamma]", "[", "t", "]"}], " "}]], "Input",
 CellChangeTimes->{{3.5977318613653574`*^9, 3.597731863773495*^9}, {
   3.597736504464549*^9, 3.597736506142645*^9}, 3.597736550155163*^9, {
   3.6245408600029573`*^9, 3.6245408613757596`*^9}, {3.6580657281340437`*^9, 
   3.658065756821476*^9}, {3.658121162862288*^9, 3.658121172294119*^9}, 
   3.6581215412268877`*^9, {3.6581505075708303`*^9, 3.658150660033287*^9}, 
   3.658150907480309*^9, {3.663325298592568*^9, 3.6633253412395353`*^9}, 
   3.704165437227973*^9},ExpressionUUID->"a27e57fe-e1d4-46e4-9923-\
06dc6965bb0a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "\[Xi]", "]"}], ";", 
  RowBox[{
   RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704165896777336*^9, 
  3.704165950361376*^9}},ExpressionUUID->"da20e81c-bc6e-4d7c-97fc-\
44161d6b737f"],

Cell["\<\
We define a vector field as a tensor of type (0,1) in a neighborhood of the \
curve:\
\>", "Text",
 CellChangeTimes->{{3.669539997206479*^9, 3.669540055094241*^9}, {
  3.702344447173304*^9, 3.702344480425062*^9}, {3.702344521652466*^9, 
  3.702344523540296*^9}},ExpressionUUID->"8454562a-5d06-4f2e-ae7c-\
7cc9e5e541f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vX", "]"}], ";", 
  RowBox[{
   RowBox[{"vX", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"vv", "[", "\[Xi]", "]"}], "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"vX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658122643000293*^9, 3.658122733840217*^9}, {
   3.658150776656516*^9, 3.6581508155046577`*^9}, 3.663325711229546*^9, {
   3.663325769956048*^9, 
   3.663325785540489*^9}},ExpressionUUID->"d7469a0c-1511-4c27-b8c3-\
5c58c47c25f6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.704165767922922*^9, 
  3.7041657967383013`*^9}},ExpressionUUID->"7b061eef-ef0c-44cb-a241-\
9fc42d10ae16"],

Cell["\<\
We evaluate the absolute differential of the tensor field vX  for the vector \
field\
\>", "Text",
 CellChangeTimes->{{3.669539997206479*^9, 3.669540055094241*^9}, {
  3.669540229917254*^9, 3.669540304020588*^9}, {3.669610984537856*^9, 
  3.669610985865518*^9}},ExpressionUUID->"6cc4409d-ba64-4bce-b7f5-\
2d49b74bb998"],

Cell[BoxData[
 RowBox[{"?", "tensorD"}]], "Input",
 CellChangeTimes->{{3.663326388930376*^9, 3.6633263942793503`*^9}, 
   3.6691122622285137`*^9, 
   3.669209312011492*^9},ExpressionUUID->"2dfab3c6-4b13-4f16-bca2-\
8eda94dc9875"],

Cell[BoxData[
 RowBox[{"DvX", " ", "=", 
  RowBox[{
   RowBox[{"tensorD", "[", 
    RowBox[{"\[CapitalGamma]", ",", " ", "vX"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.669540467188991*^9, 3.6695404739718647`*^9}, 
   3.703251840191514*^9},ExpressionUUID->"740ab6a5-3b9e-4265-a2d0-\
c15977c74702"],

Cell["Now we restrict this tensor field to the curve \[Gamma][t]", "Text",
 CellChangeTimes->{{3.669540569586135*^9, 
  3.6695406499781427`*^9}},ExpressionUUID->"94c053fe-6605-496d-b2c4-\
79dc75466a2b"],

Cell[BoxData[
 RowBox[{"DvX", " ", "/.", " ", 
  RowBox[{
   RowBox[{"x", "[", "i_", "]"}], " ", "->", " ", 
   RowBox[{
    RowBox[{"x", "[", "i", "]"}], "[", "t", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.669540669541798*^9, 
  3.669540669543236*^9}},ExpressionUUID->"6192f700-f09f-47ed-ba13-\
1e8570fb6316"],

Cell["\<\
and apply it to the tangent vector of the curve. We obtain the resulting  \
tensor field along the curve:\
\>", "Text",
 CellChangeTimes->{{3.66954073164847*^9, 3.669540770082602*^9}, {
   3.669540801689178*^9, 3.6695409939360447`*^9}, {3.669541055985441*^9, 
   3.669541069071309*^9}, 3.6696110511932707`*^9, {3.703252711365868*^9, 
   3.703252745077441*^9}, 
   3.7032541633935537`*^9},ExpressionUUID->"88fec714-593f-471f-9e2c-\
2ca3e0353bff"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vvX", "]"}], ";", 
  RowBox[{
   RowBox[{"vvX", "[", "t_", "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"DvX", " ", "/.", " ", 
      RowBox[{
       RowBox[{"x", "[", "i_", "]"}], " ", "->", " ", 
       RowBox[{
        RowBox[{"x", "[", "i", "]"}], "[", "t", "]"}]}]}], ")"}], "[", 
    RowBox[{
     RowBox[{"d", "[", "\[Gamma]", "]"}], "[", "t", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.669541074747177*^9, 3.669541074748497*^9}, {
   3.703251276299513*^9, 3.703251311297982*^9}, {3.7032513819715157`*^9, 
   3.703251391410102*^9}, 3.7032515376746063`*^9, 3.703252195491091*^9, {
   3.7032541823875628`*^9, 
   3.703254195281971*^9}},ExpressionUUID->"f081e364-84f3-430b-b627-\
3b6676736f6f"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703248011574251*^9, 3.703248023844346*^9}, {
  3.70325196978909*^9, 3.70325197526795*^9}, {3.7032520145235*^9, 
  3.703252049026943*^9}, {3.703252250051332*^9, 
  3.703252250913371*^9}},ExpressionUUID->"cb9b0395-2972-465a-a601-\
a8dacb97ac99"],

Cell[TextData[{
 "We defined in riemannv2.m the ",
 StyleBox["absolute derivative ",
  FontColor->RGBColor[0, 0, 1]],
 "tensorD1 of a tensor function ta[t] given along a curve cu[t] in the \
manifold:"
}], "Text",
 CellChangeTimes->{{3.5978243856151857`*^9, 3.597824491751256*^9}, 
   3.597847681766528*^9, 3.624540411361769*^9, 3.6245407807392178`*^9, {
   3.6260721218721905`*^9, 3.626072135865415*^9}, 3.663325542469625*^9, {
   3.6691122621185226`*^9, 3.669112262155981*^9}, 3.6695282833019953`*^9, {
   3.669612101103298*^9, 3.6696121748458014`*^9}, {3.703243924679944*^9, 
   3.703243930647958*^9}},ExpressionUUID->"8b159e08-9dab-4100-b9d8-\
14ca4182fd23"],

Cell[BoxData[
 RowBox[{"?", "tensorD1"}]], "Input",
 CellChangeTimes->{{3.597847771310686*^9, 3.597847775990694*^9}, 
   3.597847810685155*^9, 3.597931689455128*^9, {3.6245404552134457`*^9, 
   3.6245404956175175`*^9}, 3.6245407109290953`*^9, {3.624540760833583*^9, 
   3.6245407743588066`*^9}, 3.624540881811795*^9, 3.62454095280752*^9, {
   3.626072002859582*^9, 3.626072006369588*^9}, {3.669112262158568*^9, 
   3.669112262159234*^9}, 3.669528306381941*^9, {3.6696120749112577`*^9, 
   3.669612079063184*^9}},ExpressionUUID->"4f5be41b-f42f-4dcf-ae7b-\
77410dc24101"],

Cell[BoxData[
 RowBox[{"?", "absD1"}]], "Input",
 CellChangeTimes->{{3.703248414827076*^9, 
  3.703248429265699*^9}},ExpressionUUID->"8decd58e-b064-4272-b567-\
39a146d116f6"],

Cell["Let the tensor to differentiate along the curve \[Gamma] be", "Text",
 CellChangeTimes->{{3.5977365672371397`*^9, 3.5977365831090474`*^9}, {
   3.658065774965391*^9, 3.6580657932770987`*^9}, {3.658121739472457*^9, 
   3.6581220318217*^9}, {3.66333787936736*^9, 3.663337880863405*^9}, 
   3.6695408244329*^9, {3.6696114389570847`*^9, 3.669611485148653*^9}, 
   3.702344795248701*^9},ExpressionUUID->"dcf6dbc9-03c6-40dd-aed0-\
d8a2c818ee74"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tvX", "]"}], ";", 
  RowBox[{
   RowBox[{"tvX", "[", "t_", "]"}], ":=", 
   RowBox[{"vX", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], "[", "t", "]"}], ",", 
     RowBox[{
      RowBox[{"x", "[", "2", "]"}], "[", "t", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.669539120008315*^9, 3.6695391569097853`*^9}, {
  3.6695391929093943`*^9, 3.6695392342209797`*^9}, {3.669539271509523*^9, 
  3.669539293804552*^9}, {3.704166249369731*^9, 
  3.704166277336801*^9}},ExpressionUUID->"6134dbfd-407c-42f0-8168-\
505dc57d16b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tvX", "[", "t", "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6695393156583157`*^9, 3.6695393262199297`*^9}, {
  3.7032542954418507`*^9, 
  3.703254314880268*^9}},ExpressionUUID->"f3bb1777-ff83-4917-8bdf-\
6af78b3d6a3d"],

Cell["\<\
then the absolute differential of  vX with respect to the curve \[Gamma] is\
\>", "Text",
 CellChangeTimes->{{3.6581228242382717`*^9, 3.658122891400406*^9}, 
   3.6581234551400127`*^9, 3.6691122621825037`*^9, 3.66952845954023*^9, {
   3.6696117460982723`*^9, 3.669611748002117*^9}, {3.704166133888423*^9, 
   3.704166148304421*^9}},ExpressionUUID->"75c2d370-0f22-4bc8-9a24-\
6585c960e664"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ttvX", "]"}], ";", 
  RowBox[{"ttvX", "=", 
   RowBox[{
    RowBox[{"tensorD1", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tvX"}], "]"}], "[", 
    RowBox[{"\[Gamma]", ",", "t"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6695394637059727`*^9, 3.6695395905376463`*^9}, 
   3.669611760058482*^9, {3.669611865754583*^9, 3.6696119116819487`*^9}, {
   3.669612197271318*^9, 3.669612199918517*^9}, {3.703254422664876*^9, 
   3.703254440335552*^9}},ExpressionUUID->"6f936f12-8d76-4659-a400-\
d62f685e2b5f"],

Cell[BoxData[
 RowBox[{"tensor", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"absD1", "[", 
     RowBox[{"\[CapitalGamma]", ",", "tvX"}], "]"}], "[", 
    RowBox[{"\[Gamma]", ",", "t"}], "]"}], ",", "0", ",", "1", ",", "2"}], 
  "]"}]], "Input",
 CellChangeTimes->{
  3.704166296041539*^9},ExpressionUUID->"e8898910-caeb-4566-b12b-\
be6495edde72"],

Cell[BoxData[
 RowBox[{"ttvX", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.703253106184679*^9, 
  3.70325314270592*^9}},ExpressionUUID->"c8048a28-7ce2-436b-90c1-\
6f5a488f1541"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"ttvX", ",", 
   RowBox[{"vvX", "[", "t", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.669539624292351*^9, 3.66953967635297*^9}, {
  3.669612222206007*^9, 3.669612272653569*^9}, {3.703254502362492*^9, 
  3.703254504199068*^9}},ExpressionUUID->"7dd39255-0c1f-47b0-a709-\
4dd984dc23f1"],

Cell[TextData[{
 "The calculation shows\n",
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". The absolute differential of the restriction of a tensor field ta[x] to a \
curve calculated by tensor D1 (with coordinates calculated by absD1) \
coincides with the restriction of the absolute differential of the field ta \
to the curve. \[FilledSquare]\nThe definition of the functions absd1, \
tensorD1 are given in the package riemannv2.m:"
}], "Text",
 CellChangeTimes->{{3.6696125075147676`*^9, 3.669612918415966*^9}, {
  3.70234511212921*^9, 
  3.702345113393393*^9}},ExpressionUUID->"81b481dd-089e-4f3e-9c97-\
5855e1957754"],

Cell[BoxData[
 RowBox[{"?", "absD1"}]], "Input",
 CellChangeTimes->{{3.663339005288879*^9, 3.663339009727353*^9}, 
   3.6691122623015823`*^9, 
   3.6692092221047287`*^9},ExpressionUUID->"fd8c960e-4591-46f5-a8fc-\
14da236ec158"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"absD1", "[", 
     RowBox[{"\[CapitalGamma]_", ",", "tens_"}], "]"}], "[", 
    RowBox[{"cu_", ",", "t_"}], "]"}], "[", 
   RowBox[{"ii_", ",", "ji_"}], "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "di", ",", "pt", ",", "qt", ",", "wv", ",", "dt", ",", "dti", ",", "dtj",
       ",", "i", ",", "j", ",", "s", ",", "l", ",", "v", ",", "xt"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Syntax", " ", "control"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"cu", "[", "t", "]"}], "]"}], "===", "List"}], ",", " ", 
       RowBox[{"di", " ", "=", " ", 
        RowBox[{"Length", "[", 
         RowBox[{"cu", "[", "t", "]"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
          RowBox[{"cu", "[", "t", "]"}], ",", "\"\< is not a curve!\>\""}], 
         "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"tens", "[", "t", "]"}], "]"}], "===", "tensor"}], ",", " ", 
       RowBox[{
        RowBox[{"pt", " ", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "t", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
        RowBox[{"qt", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "t", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
          RowBox[{"tens", "[", "t", "]"}], ",", "\"\< is not a tensor!\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"di", " ", "\[Equal]", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "t", "]"}], "]"}], "[", 
          RowBox[{"[", "3", "]"}], "]"}]}], "]"}], ",", " ", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Tensor dimension \>\"", ",", 
          RowBox[{
           RowBox[{"type", "[", 
            RowBox[{"tens", "[", "t", "]"}], "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", " ", 
          "\"\< is not the manifold dimension\>\"", ",", " ", "di", ",", 
          "\"\< !\>\""}], "]"}], ";", " ", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"Tuples", "[", 
           RowBox[{
            RowBox[{"Range", "[", "di", "]"}], ",", "pt"}], "]"}], ",", 
          "ii"}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Invalid argument \>\"", ",", "ii", ",", " ", "\"\< !\>\""}], 
         "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"Tuples", "[", 
           RowBox[{
            RowBox[{"Range", "[", "di", "]"}], ",", "qt"}], "]"}], ",", 
          "ji"}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
         "\"\<Invalid argument \>\"", ",", "ji", ",", " ", "\"\< !\>\""}], 
         "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"v", " ", "=", " ", 
      RowBox[{"absD", "[", 
       RowBox[{
        RowBox[{"cu", "[", "t", "]"}], ",", "t"}], "]"}]}], ";", 
     RowBox[{"xt", " ", "=", 
      RowBox[{
       RowBox[{"cu", "[", "t", "]"}], "/.", 
       RowBox[{"List", "\[Rule]", "Sequence"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"wv", "[", 
       RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[CapitalGamma]", "[", 
           RowBox[{"j", ",", "s", ",", "i"}], "]"}], "[", "xt", "]"}], "*", 
         RowBox[{"v", "[", 
          RowBox[{"[", "s", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", "di"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"The", " ", "components"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"dt", " ", "=", " ", 
      RowBox[{"absD", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"tens", "[", "t", "]"}], "[", 
         RowBox[{"ii", ",", "ji"}], "]"}], ",", "t"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dti", " ", "=", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"tens", "[", "t", "]"}], "[", 
            RowBox[{
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"ii", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ";;", 
                  RowBox[{"l", "-", "1"}]}], "]"}], "]"}], ",", 
               RowBox[{"{", "s", "}"}], ",", 
               RowBox[{"ii", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"l", "+", "1"}], ";;"}], "]"}], "]"}]}], "]"}], ",",
              "ji"}], "]"}], "*", 
           RowBox[{"wv", "[", 
            RowBox[{
             RowBox[{"ii", "[", 
              RowBox[{"[", "l", "]"}], "]"}], ",", "s"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"s", ",", "di"}], "}"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"l", ",", "pt"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"dtj", " ", "=", " ", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"tens", "[", "t", "]"}], "[", 
            RowBox[{"ii", ",", 
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"ji", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ";;", 
                  RowBox[{"l", "-", "1"}]}], "]"}], "]"}], ",", 
               RowBox[{"{", "s", "}"}], ",", 
               RowBox[{"ji", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"l", "+", "1"}], ";;"}], "]"}], "]"}]}], "]"}]}], 
            "]"}], "*", 
           RowBox[{"wv", "[", 
            RowBox[{"s", ",", 
             RowBox[{"ji", "[", 
              RowBox[{"[", "l", "]"}], "]"}]}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"s", ",", "di"}], "}"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"l", ",", "qt"}], "}"}]}], "]"}]}], ";", " ", 
     "\[IndentingNewLine]", 
     RowBox[{"dt", "-", "dti", "+", "dtj"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.658204808826778*^9, 3.6582048298592157`*^9}, {
   3.65820486445114*^9, 3.658204921474236*^9}, {3.658205019273486*^9, 
   3.658205039489333*^9}, 3.6582052081907377`*^9, {3.658205314058037*^9, 
   3.658205431799797*^9}, {3.65820547611967*^9, 3.6582056050363827`*^9}, 
   3.658205660119402*^9, {3.658205718198739*^9, 3.658205730852972*^9}, {
   3.6582057851635427`*^9, 3.6582057875656433`*^9}, {3.65820582072647*^9, 
   3.658205919673932*^9}, {3.658206112547024*^9, 3.6582061147869*^9}, {
   3.658206151522327*^9, 3.6582062440413227`*^9}, {3.658206308888897*^9, 
   3.658206378135593*^9}, {3.658206844956012*^9, 3.658206901529976*^9}, {
   3.6582069334499283`*^9, 3.658206972267239*^9}, 3.658207015235273*^9, {
   3.6582070895542393`*^9, 3.658207143882482*^9}, 3.658207188625695*^9, 
   3.658212261437223*^9, {3.6582125211229258`*^9, 3.65821254458742*^9}, {
   3.669112262332016*^9, 3.669112262338778*^9}, 
   3.669209222163434*^9},ExpressionUUID->"fa3da4c2-93f5-4b34-af68-\
481760184c2a"],

Cell["The corresponding tensor field is", "Text",
 CellChangeTimes->{{3.663339112027256*^9, 3.663339123947125*^9}, {
  3.663339154210107*^9, 
  3.663339160281798*^9}},ExpressionUUID->"14f303dc-8379-4997-836c-\
1f1659b3526f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tensorD1", "[", 
    RowBox[{"\[CapitalGamma]_", ",", "tens_"}], "]"}], "[", 
   RowBox[{"cu_", ",", "t_"}], "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"di", ",", "pt", ",", "qt"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{"Syntax", " ", "control"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"cu", "[", "t", "]"}], "]"}], "===", "List"}], ",", " ", 
       RowBox[{"di", " ", "=", " ", 
        RowBox[{"Length", "[", 
         RowBox[{"cu", "[", "t", "]"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
          RowBox[{"cu", "[", "t", "]"}], ",", "\"\< is not a curve!\>\""}], 
         "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"tens", "[", "t", "]"}], "]"}], "===", "tensor"}], ",", " ", 
       RowBox[{
        RowBox[{"pt", " ", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "t", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
        RowBox[{"qt", "=", " ", 
         RowBox[{
          RowBox[{"type", "[", 
           RowBox[{"tens", "[", "t", "]"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "2"}], "]"}], "]"}]}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{
          RowBox[{"tens", "[", "t", "]"}], ",", "\"\< is not a tensor!\>\""}],
          "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"tensor", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"absD1", "[", 
         RowBox[{"\[CapitalGamma]", ",", "tens"}], "]"}], "[", 
        RowBox[{"cu", ",", "t"}], "]"}], ",", "pt", ",", "qt", ",", "di"}], 
      "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 InitializationCell->True,
 CellChangeTimes->{{3.6563427833560266`*^9, 3.656342789447337*^9}, {
   3.656342843158556*^9, 3.656342963941431*^9}, 3.656343021040022*^9, {
   3.6563430564532223`*^9, 3.656343106269249*^9}, {3.65634319672395*^9, 
   3.6563432219262238`*^9}, {3.656343270432033*^9, 3.656343276293867*^9}, {
   3.65634335880473*^9, 3.656343507274365*^9}, 3.657264502947132*^9, 
   3.657264588222106*^9, {3.657266592530807*^9, 3.657266599574698*^9}, {
   3.657267243744074*^9, 3.657267246825692*^9}, {3.657267889516656*^9, 
   3.657267903396482*^9}, 3.657281044125877*^9, {3.658213272745513*^9, 
   3.65821328452922*^9}, {3.658213337697052*^9, 3.658213378768393*^9}, {
   3.658213437279313*^9, 3.658213542111087*^9}, {3.669112262377688*^9, 
   3.669112262381872*^9}, 3.669209222242198*^9, 
   3.669209312187055*^9},ExpressionUUID->"20c30fa4-df77-479f-a479-\
5ec2b7c1abfc"],

Cell["\<\
The definitions generalize the concept of the absolute differential of a \
vector field along the curve. 
 We consider further examples.\
\>", "Text",
 CellChangeTimes->{{3.663339335723093*^9, 3.663339467479291*^9}, {
  3.6696130507338877`*^9, 3.669613052854163*^9}, {3.669613087093629*^9, 
  3.6696131097179623`*^9}},ExpressionUUID->"ac7578cf-ae22-4071-b909-\
dfc9d9c6b625"],

Cell[TextData[StyleBox["Example 1.",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.5978245313115187`*^9, 3.597824542687169*^9}, 
   3.5979097307832108`*^9, {3.6582119273812304`*^9, 
   3.658211942211858*^9}},ExpressionUUID->"18141161-2c79-4898-bce8-\
b39fb5ce5fe9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"y", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s1", "[", "t", "]"}], ",", 
     RowBox[{"s2", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{"Clear", "[", "vX", "]"}], ";", 
  RowBox[{
   RowBox[{"vX", "[", "t_", "]"}], " ", ":=", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{"y", "[", "t", "]"}], " ", "]"}]}], ";", 
  RowBox[{"vX", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.5977317427255716`*^9, 3.5977317893472385`*^9}, {
   3.597731905438878*^9, 3.5977319912867885`*^9}, 3.5977320952957373`*^9, {
   3.5977321323428564`*^9, 3.5977321664548073`*^9}, {3.597732200380748*^9, 
   3.59773233731258*^9}, {3.5977323700014496`*^9, 3.5977323837522364`*^9}, 
   3.5977615795014133`*^9, {3.5977631817334275`*^9, 3.597763189424241*^9}, 
   3.597814977779808*^9, {3.5978168288634596`*^9, 3.5978168347290697`*^9}, {
   3.658207314013188*^9, 3.658207338725256*^9}, {3.658207434685601*^9, 
   3.6582074818521748`*^9}, {3.6582075130128613`*^9, 3.658207615330698*^9}, {
   3.704166610908877*^9, 
   3.704166655683948*^9}},ExpressionUUID->"65b990a2-3e12-4a63-bbaa-\
9dd6dee88265"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"absD1", "[", 
     RowBox[{"\[CapitalGamma]", ",", "vX"}], "]"}], "[", 
    RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5977317427255716`*^9, 3.5977317893472385`*^9}, {
   3.597731905438878*^9, 3.5977319912867885`*^9}, 3.5977320952957373`*^9, {
   3.5977321323428564`*^9, 3.5977321664548073`*^9}, {3.597732200380748*^9, 
   3.59773233731258*^9}, {3.5977323700014496`*^9, 3.5977323837522364`*^9}, 
   3.5977615795014133`*^9, {3.5977631817334275`*^9, 3.597763189424241*^9}, 
   3.597814977779808*^9, {3.5978168288634596`*^9, 3.5978168347290697`*^9}, {
   3.658207314013188*^9, 3.658207338725256*^9}, {3.658207434685601*^9, 
   3.6582074818521748`*^9}, {3.6582075130128613`*^9, 3.658207545780037*^9}, {
   3.658207650298895*^9, 3.658207651722266*^9}, {3.6582076883031282`*^9, 
   3.65820774632997*^9}, {3.658207809785429*^9, 3.6582078378009653`*^9}, 
   3.658208200254364*^9, 3.66911226241886*^9, 
   3.6692092224002047`*^9},ExpressionUUID->"74c29bfb-32a4-4948-b3f6-\
abb89ee84b68"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tensorD1", "[", 
    RowBox[{"\[CapitalGamma]", ",", "vX"}], "]"}], "[", 
   RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.658213953018485*^9, 3.6582140994971046`*^9}, 
   3.6691122624382153`*^9, 
   3.669209312246004*^9},ExpressionUUID->"61f64db1-0120-4722-bea6-\
5230b6254ed0"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "\[Equal]", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.658214125790906*^9, 
  3.658214151472101*^9}},ExpressionUUID->"e0f064db-52c0-4561-9f6d-\
3ef7ecf0263b"],

Cell["OK\[FilledSquare]", "Text",
 CellChangeTimes->{{3.6582082497306356`*^9, 3.658208267541827*^9}, {
   3.658213227735901*^9, 3.658213234562566*^9}, 3.704166503413342*^9, 
   3.704166711555232*^9},ExpressionUUID->"5d3a56d4-876f-4084-abd5-\
be2ce2c311ec"],

Cell[TextData[{
 StyleBox["Example 2.",
  FontColor->RGBColor[0, 0, 1]],
 " Euclidean plane"
}], "Text",
 CellChangeTimes->{{3.624541561504989*^9, 3.6245415878066354`*^9}, {
  3.658211954379628*^9, 
  3.65821196089151*^9}},ExpressionUUID->"ac82b029-c96a-490b-a6fb-\
af7f5f7f3f4f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "[", "t", "]"}], ",", 
     RowBox[{"v", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{
   RowBox[{"y", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s1", "[", "t", "]"}], ",", 
     RowBox[{"s2", "[", "t", "]"}]}], "}"}]}], ";", 
  RowBox[{"vX", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.5977317427255716`*^9, 3.5977317893472385`*^9}, {
   3.597731905438878*^9, 3.5977319912867885`*^9}, 3.5977320952957373`*^9, {
   3.5977321323428564`*^9, 3.5977321664548073`*^9}, {3.597732200380748*^9, 
   3.59773233731258*^9}, {3.5977323700014496`*^9, 3.5977323837522364`*^9}, 
   3.5977615795014133`*^9, {3.5977631817334275`*^9, 3.597763189424241*^9}, 
   3.597814977779808*^9, {3.5978153495284615`*^9, 3.5978153652688885`*^9}, {
   3.5978154316938057`*^9, 3.5978154432534256`*^9}, 3.597816947501668*^9, {
   3.624541491679267*^9, 3.624541495501273*^9}, {3.6582073534372597`*^9, 
   3.658207357653213*^9}, {3.658211429281122*^9, 3.6582114305930243`*^9}, {
   3.6582114716906548`*^9, 3.6582115013446503`*^9}, 3.669112262460231*^9, 
   3.669209222484893*^9, {3.669613402676886*^9, 3.669613439018993*^9}, 
   3.704166564181932*^9},ExpressionUUID->"97aeac8c-d45d-4f8e-9256-\
13f1773c7461"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"absD1", "[", 
     RowBox[{"\[CapitalGamma]0", ",", "vX"}], "]"}], "[", 
    RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "]"}]], "Input",ExpressionUUID->\
"e62cf7be-3122-4a94-833f-8cea73bbe357"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"coords", "[", 
   RowBox[{
    RowBox[{"tensorD1", "[", 
     RowBox[{"\[CapitalGamma]0", ",", "vX"}], "]"}], "[", 
    RowBox[{"\[Gamma]", ",", "t"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.658213953018485*^9, 3.6582140994971046`*^9}, 
   3.658214201432225*^9, 3.669112262482916*^9, 3.669209312302656*^9, {
   3.702345504191102*^9, 3.702345530189597*^9}, 
   3.7023455774789248`*^9},ExpressionUUID->"5292dcb7-ef16-42ee-99c7-\
6206265d6216"],

Cell["\<\
In the Euclidean plane with Cartesian coordinates the absolute derivative \
coincides with the standard derivative.
\[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.597824599532421*^9, 
  3.5978246899355917`*^9}},ExpressionUUID->"e33ee919-812c-475a-85cc-\
434c1a8d2b78"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.658211990670682*^9, 
  3.658212005939391*^9}},ExpressionUUID->"39a8e65c-cc6d-4a47-805f-\
fa077092e747"],

Cell[TextData[{
 "D",
 StyleBox["efinition",
  FontColor->RGBColor[0, 0, 1]],
 ". The tensor ta[t] is said to be",
 StyleBox[" parallel displaced",
  FontColor->RGBColor[0, 0, 1]],
 " along the curve \[Gamma][t], if its absolute derivative is zero: tensorD1[\
\[CapitalGamma], ta][\[Gamma], t] ==  nulltensor."
}], "Text",
 CellChangeTimes->{{3.597824912191304*^9, 3.5978250944877305`*^9}, {
   3.5978251325839095`*^9, 3.5978253947519045`*^9}, {3.613282563068982*^9, 
   3.6132825845572114`*^9}, {3.658214300357005*^9, 3.658214319462657*^9}, {
   3.658214355030683*^9, 3.658214477445404*^9}, 3.658214519325049*^9, {
   3.669112262527379*^9, 3.6691122625431023`*^9}, 3.669209312387857*^9, 
   3.669613614945306*^9},ExpressionUUID->"66f2f1b7-e18b-4ba8-b5ab-\
6f99cf73cdf3"],

Cell[TextData[{
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". Let \[Gamma][t0, t1] be the arc of the curve \[Gamma] between \
\[Gamma][t0], \[Gamma][t1]. Then for any tangent vector \n\[Xi]  \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"\[Gamma]", "[", "t0", "]"}]], TraditionalForm]],ExpressionUUID->
  "918b0488-1b6f-4983-b276-b68161a09f7b"],
 " with coordinates {si} there exists a uniquely defined vector function \
\[Xi][t], parallel displaced along \[Gamma][t], with  \[Xi][t0] == \[Xi].  \
The map"
}], "Text",
 CellChangeTimes->{{3.5978254006782436`*^9, 3.597825430982977*^9}, {
   3.5978254653109407`*^9, 3.5978256295313334`*^9}, {3.597827809533633*^9, 
   3.5978281050655365`*^9}, {3.597828142609684*^9, 3.5978284203455696`*^9}, 
   3.597848111297683*^9, {3.5978482153030653`*^9, 3.5978482290622897`*^9}, {
   3.597909746031082*^9, 3.597909760134889*^9}, {3.6132826504209785`*^9, 
   3.6132826656368484`*^9}, 3.617441476236147*^9, {3.6582169993621893`*^9, 
   3.658217019167859*^9}, 
   3.658217066047262*^9},ExpressionUUID->"b9c83474-dd16-459a-bec4-\
73cd7e9f92e2"],

Cell[TextData[{
 "\[Gamma][t0, t1]:   \[Xi]  \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"\[Gamma]", "[", "t0", "]"}]], TraditionalForm]],ExpressionUUID->
  "e0a05a0e-fe93-4154-8c8c-13289526f3e6"],
 "  \[RightTeeArrow] \[Xi][t1] \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"[", "t1", "]"}]], TraditionalForm]],ExpressionUUID->
  "5eca667c-bc2f-4340-a8f5-a57df957a697"]
}], "Text",
 CellChangeTimes->{{3.5978254006782436`*^9, 3.597825430982977*^9}, {
  3.5978254653109407`*^9, 3.5978256295313334`*^9}, {3.597827809533633*^9, 
  3.5978281050655365`*^9}, {3.597828142609684*^9, 3.5978284203455696`*^9}, {
  3.597848111297683*^9, 3.5978481458205433`*^9}, {3.5978491789011583`*^9, 
  3.597849186342371*^9}, {3.6582170970087423`*^9, 
  3.658217120343421*^9}},ExpressionUUID->"b01c7c0b-0346-4ed6-967b-\
2a647467c560"],

Cell["\<\
is an isomorphism of vector spaces, depending on the curve \[Gamma].\
\>", "Text",
 CellChangeTimes->{{3.5978254006782436`*^9, 3.597825430982977*^9}, {
  3.5978254653109407`*^9, 3.5978256295313334`*^9}, {3.597827809533633*^9, 
  3.5978281050655365`*^9}, {3.597828142609684*^9, 3.5978284203455696`*^9}, {
  3.597848111297683*^9, 3.59784815508696*^9}, {3.658217156085594*^9, 
  3.6582171655012913`*^9}},ExpressionUUID->"687865e3-4af9-4503-a6f4-\
4f962d790242"],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". Analyzing the definition or looking at the examples we see that the \
equation "
}], "Text",
 CellChangeTimes->{{3.5978482749731703`*^9, 3.5978482821647835`*^9}, {
  3.597848335095676*^9, 3.5978483637373266`*^9}, {3.5978484072926025`*^9, 
  3.597848416122218*^9}},ExpressionUUID->"80f09319-fa97-4e75-93e5-\
db5ec6e0d915"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"absD1", "[", 
    RowBox[{"\[CapitalGamma]", ",", "\[Xi]"}], "]"}], "[", 
   RowBox[{"\[Gamma]", ",", "t"}], "]"}], " ", "\[Equal]", "  ", 
  "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597848445029069*^9, 3.5978484840291376`*^9}, {
   3.658217200005604*^9, 3.65821723925443*^9}, 3.669112262547571*^9, 
   3.669209222562606*^9},ExpressionUUID->"2b6116aa-11b5-4ac8-8f67-\
3f5098319694"],

Cell["\<\
Is a homogeneous system of ordinary linear differential equations of first \
order for the coordinates s[i][t] of \[Xi][t}. Therefore the solutions are \
uniquely defined by the initial conditions. Since\
\>", "Text",
 CellChangeTimes->{{3.597848493233154*^9, 3.597848651495432*^9}, {
  3.597848698732315*^9, 3.597848732397174*^9}, {3.618824036118561*^9, 
  3.618824043533985*^9}},ExpressionUUID->"b971e5f2-d12e-48f5-9875-\
57f00da220b9"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"absD1", "[", 
    RowBox[{"\[CapitalGamma]", ",", 
     RowBox[{"\[Xi]", "+", "\[Eta]"}]}], "]"}], "\[Equal]", 
   RowBox[{
    RowBox[{"absD1", "[", 
     RowBox[{"\[CapitalGamma]", ",", "\[Xi]"}], "]"}], "+", 
    RowBox[{
     RowBox[{"absD1", "[", 
      RowBox[{"\[CapitalGamma]", ",", "\[Eta]"}], "]"}], " ", 
     StyleBox["and",
      FontColor->GrayLevel[0]], " ", 
     RowBox[{"absD1", "[", 
      RowBox[{"\[CapitalGamma]", ",", 
       RowBox[{"a", " ", "\[Xi]"}]}], "]"}]}]}], "\[Equal]", 
   RowBox[{"a", " ", 
    RowBox[{"absD1", "[", 
     RowBox[{"\[CapitalGamma]", ",", "\[Xi]"}], "]"}]}]}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   StyleBox["for",
    FontColor->GrayLevel[0]], " ", "a"}], "\[Element]", 
  "\[DoubleStruckCapitalR]"}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.597848753691211*^9, 3.5978489453375483`*^9}, {
  3.5978490043368516`*^9, 3.59784902028008*^9}, {3.658217299837265*^9, 
  3.65821735997272*^9}, {3.658217400188498*^9, 3.6582174763551693`*^9}, {
  3.669112262552373*^9, 3.669112262556511*^9}, {3.669209222585373*^9, 
  3.669209222593398*^9}},ExpressionUUID->"87c05fa4-d719-40f5-b03a-\
80abe28dfe91"],

Cell[TextData[{
 "the map \[Gamma][t0, t1] defined above is a linear isomorphism. \
\[FilledSquare]\n\nThe family of linear isomorphisms \[Gamma][t0, t], t0 \
\[LessEqual] t \[LessEqual] t1, is called the ",
 StyleBox["parallel displacement",
  FontColor->RGBColor[0, 0, 1]],
 "  along the curve \[Gamma].\n",
 StyleBox["Corollary.",
  FontColor->RGBColor[0, 0, 1]],
 " For any smooth curve on a smooth manifold  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "36838cb1-3174-439a-8c86-72f68fe98638"],
 " with a linear connection there exist a family of  basis vectors b[t][i], i \
= 1,...,n, parallel displaced along the curve. A tensor ta[t] is parallel \
displaced along the curve iff its coordinates with respect to a parallel \
displaced basis are constant.\n",
 StyleBox["Proof. ",
  FontColor->RGBColor[0, 0, 1]],
 "Derive the basis representation of the tensor  in the parallel displaced \
basis. \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.5978492407084665`*^9, 3.597849289240152*^9}, {
   3.597903092985965*^9, 3.597903098055974*^9}, {3.5979033790748672`*^9, 
   3.5979034785094423`*^9}, {3.597904186319085*^9, 3.597904199182821*^9}, {
   3.6260729171615877`*^9, 3.6260729195327916`*^9}, {3.65821755652779*^9, 
   3.658217585329893*^9}, {3.658217657209292*^9, 3.658217689680933*^9}, {
   3.6582177314807587`*^9, 3.65821791827953*^9}, {3.658218015086732*^9, 
   3.658218176189393*^9}, {3.658218207293181*^9, 3.6582184005890102`*^9}, 
   3.66333999063862*^9, 3.6655583316520967`*^9, 3.6691122626213284`*^9, {
   3.6696138739124117`*^9, 3.66961388531175*^9}, 
   3.702345798769829*^9},ExpressionUUID->"516abf66-6ba8-4594-a331-\
ed36b1ecc95c"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4.3. Auto-Parallel Curves", "Subsubsection",
 CellChangeTimes->{{3.595076945363204*^9, 3.5950769905720835`*^9}, 
   3.5953174793534184`*^9, {3.5953175263051033`*^9, 3.595317587064579*^9}, 
   3.5956008724322104`*^9, {3.5965213907316623`*^9, 3.5965214041788855`*^9}, {
   3.5972984401750546`*^9, 3.59729846072029*^9}, {3.5972984954771514`*^9, 
   3.5972985062723703`*^9}, {3.5973320886172123`*^9, 3.5973320905123205`*^9}, 
   3.5973952636966553`*^9, {3.597729800827932*^9, 3.59772986233884*^9}, {
   3.597910399537461*^9, 3.597910436118553*^9}, {3.5979116271266747`*^9, 
   3.5979116482228813`*^9}, 3.613284728941863*^9, 3.6132874017987423`*^9, 
   3.6171739408407526`*^9, 3.617195815150076*^9, 3.6188258784059334`*^9, 
   3.618830173270586*^9, 3.6245420369782248`*^9, 3.6245441638235598`*^9, {
   3.657778358968999*^9, 3.6577783631606827`*^9}, {3.658218442666534*^9, 
   3.6582184448425713`*^9}, 3.658234725827528*^9, 3.658311411471614*^9, {
   3.663340009061323*^9, 3.663340020132846*^9}, 3.663393630867839*^9, 
   3.669614038078107*^9, 3.6696167490624943`*^9, 3.70234587997163*^9, 
   3.702346938468135*^9, 3.704166849946302*^9, 
   3.704167991555395*^9},ExpressionUUID->"a86f9de3-5343-444f-b7c7-\
c1db1510a921"],

Cell[TextData[{
 StyleBox["Definition",
  FontColor->RGBColor[0, 0, 1]],
 ". A regular curve \[Gamma][t] on a smooth manifold with a linear connection \
is said to be ",
 StyleBox["auto-parallel",
  FontColor->RGBColor[0, 0, 1]],
 "  or a ",
 StyleBox["geodesic",
  FontColor->RGBColor[0, 0, 1]],
 " if its tangent vector is parallel displaced along the curve. \nAlready in \
the notebook euklidsfv3. nb (see [",
 ButtonBox["E",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "D",
 ButtonBox["G",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "] )  about the Euclidean geometry of surfaces  we considered the geodesics \
of an immersion as  auto-parallel curves."
}], "Text",
 CellChangeTimes->{{3.597911657441409*^9, 3.597911675247427*^9}, {
   3.597911706147195*^9, 3.59791200922353*^9}, {3.597928848436631*^9, 
   3.5979288904770355`*^9}, {3.597928956356804*^9, 3.5979290724444437`*^9}, {
   3.6132847851830797`*^9, 3.6132847983008304`*^9}, 3.6132848615014453`*^9, 
   3.617441713205701*^9, {3.658234807302924*^9, 3.658234861100473*^9}, {
   3.6582351383782*^9, 3.658235344500518*^9}, {3.6582353865789948`*^9, 
   3.6582354498102703`*^9}, 3.658235544352624*^9, {3.658239938059885*^9, 
   3.658239938060895*^9}, 3.658239998718988*^9, {3.6583038431148*^9, 
   3.658303848459002*^9}, {3.665558132276923*^9, 3.665558143660755*^9}, {
   3.669112262654335*^9, 3.6691122626792316`*^9}, 3.669614591225792*^9, {
   3.702345962707896*^9, 3.702345964131119*^9}, {3.702346012932127*^9, 
   3.702346080234644*^9}},ExpressionUUID->"feef1412-f468-4dc9-b0fd-\
9f59d94ab034"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"{", 
    RowBox[{"var", "[", 
     RowBox[{"x", "[", "t", "]"}], "]"}], "}"}]}], ";", 
  RowBox[{"\[Gamma]", "[", "t", "]"}], " "}]], "Input",
 CellChangeTimes->{{3.5977318613653574`*^9, 3.597731863773495*^9}, {
   3.597736504464549*^9, 3.597736506142645*^9}, 3.597736550155163*^9, {
   3.658235630171179*^9, 3.658235659266376*^9}, {3.658235874811743*^9, 
   3.658235934593399*^9}},ExpressionUUID->"ac6c2b74-d5ba-4aaf-b5ff-\
55211f5090f6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], " "}]], "Input",
 CellChangeTimes->{
  3.65823600890333*^9},ExpressionUUID->"7dc31d24-457c-459b-91a1-9b11eb39429a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vX", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], "]"}]}], ";", 
  RowBox[{"vX", "[", "t", "]"}], " "}]], "Input",
 CellChangeTimes->{{3.658236024632043*^9, 3.658236064421708*^9}, {
  3.6582360985642776`*^9, 
  3.658236148418549*^9}},ExpressionUUID->"7e9ab71e-8c4f-446e-885c-\
6dd7f26d975b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"absD1", "[", 
    RowBox[{"\[CapitalGamma]", ",", " ", "vX"}], "]"}], "[", 
   RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6582361695287952`*^9, 3.6582362003049383`*^9}, {
   3.6582362347088013`*^9, 3.658236268327718*^9}, {3.658236298777464*^9, 
   3.658236308062914*^9}, 3.6691122626823177`*^9, 
   3.669209222648939*^9},ExpressionUUID->"dbbb1e64-bf33-4935-a3e4-\
c788aaa3d1d9"],

Cell["\<\
To find the autoparallel curves one has to solve the system of differential \
equations of second order\
\>", "Text",
 CellChangeTimes->{{3.658236433824544*^9, 3.658236545556757*^9}, {
   3.6582365773573103`*^9, 3.6582365943339777`*^9}, 3.658309020960354*^9, {
   3.663392633937379*^9, 
   3.663392640337287*^9}},ExpressionUUID->"6adc52fd-b65c-4a1a-b783-\
283db0d7c2b7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"absD1", "[", 
        RowBox[{"\[CapitalGamma]", ",", " ", "vX"}], "]"}], "[", 
       RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "i", "}"}]}], "]"}], "\[Equal]", " ", "0"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}]}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.658236602464999*^9, 3.65823668855942*^9}, 
   3.658236728041276*^9, 3.6691122626962976`*^9, 
   3.669209222673534*^9},ExpressionUUID->"33cf35cd-37f5-4dd8-82f1-\
b268cd1a9a21"],

Cell["In traditional notation this system can be written as", "Text",
 CellChangeTimes->{{3.658309038487607*^9, 3.658309063127305*^9}, {
  3.702346171520323*^9, 
  3.7023461859606943`*^9}},ExpressionUUID->"f5540eec-c73c-457d-84f7-\
9a5d90a253df"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    SuperscriptBox["d", "2"], 
    RowBox[{"x", "[", "i", "]"}]}], 
   SuperscriptBox["dt", "2"]], " ", "\[Equal]", " ", 
  RowBox[{"-", " ", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"j", ",", 
      RowBox[{"k", " ", "=", "1"}]}], "n"], 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["\[CapitalGamma]", 
       RowBox[{"j", " ", "k"}], "i"], "[", "x", "]"}], 
     FractionBox[
      RowBox[{"dx", "[", "j", "]"}], "dt"], 
     FractionBox[
      RowBox[{"dx", "[", "k", "]"}], "dt"], "                                             ", 
     RowBox[{"(", "1", ")"}]}]}]}]}]], "Input",
 Evaluatable->False,ExpressionUUID->"267cd09e-e728-44a7-9b0c-7846a6472e2c"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". The affine space\nIn Cartesian coordinates the equations are very simple;"
}], "Text",
 CellChangeTimes->{{3.5979304075968103`*^9, 3.5979304432118473`*^9}, {
  3.658235563988928*^9, 3.658235590132141*^9}, {3.658239271574217*^9, 
  3.658239299958024*^9}, {3.658308911072977*^9, 
  3.65830891664887*^9}},ExpressionUUID->"7032096e-4b63-4f27-96d4-\
d934b73035da"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"absD1", "[", 
       RowBox[{"\[CapitalGamma]0", ",", " ", "vX"}], "]"}], "[", 
      RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", "}"}], ",", 
      RowBox[{"{", "i", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5979292059430795`*^9, 3.5979292420701456`*^9}, {
   3.597929272325876*^9, 3.597929298654382*^9}, {3.597929354638584*^9, 
   3.597929492815488*^9}, {3.597930356399882*^9, 3.597930367070492*^9}, {
   3.624543465207533*^9, 3.6245434701371417`*^9}, {3.6582368658031893`*^9, 
   3.6582368744418097`*^9}, {3.658236912169846*^9, 3.658236933754101*^9}, 
   3.669112262716856*^9, 
   3.669209222730904*^9},ExpressionUUID->"76f4dafb-e6dd-436b-93fc-\
41b0c0efc347"],

Cell[BoxData[
 RowBox[{
  RowBox[{"di", "=", "dim"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"\[Gamma]", ",", "dim"}], "]"}], ";", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], ":=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"fx", "[", "i", "]"}], "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}], ";", 
  RowBox[{"dim", "=", " ", "di"}], ";", 
  RowBox[{"\[Gamma]", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.5979292059430795`*^9, 3.5979292420701456`*^9}, {
   3.597929272325876*^9, 3.597929298654382*^9}, {3.597929354638584*^9, 
   3.597929492815488*^9}, {3.597930356399882*^9, 3.597930367070492*^9}, {
   3.624543465207533*^9, 3.6245434701371417`*^9}, {3.6582368658031893`*^9, 
   3.6582368744418097`*^9}, {3.658236912169846*^9, 3.658236933754101*^9}, 
   3.669112262716856*^9, 3.669209222730904*^9, {3.7041672676674337`*^9, 
   3.704167284655033*^9}, {3.704167329651287*^9, 3.70416738559415*^9}, {
   3.7041674222919683`*^9, 3.704167458954069*^9}, {3.704167506730369*^9, 
   3.704167646755011*^9}},ExpressionUUID->"29288992-21ef-4edd-8176-\
79cf0975bfab"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"absD1", "[", 
       RowBox[{"\[CapitalGamma]0", ",", " ", "vX"}], "]"}], "[", 
      RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", "}"}], ",", 
      RowBox[{"{", "i", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5979292059430795`*^9, 3.5979292420701456`*^9}, {
   3.597929272325876*^9, 3.597929298654382*^9}, {3.597929354638584*^9, 
   3.597929492815488*^9}, {3.597930356399882*^9, 3.597930367070492*^9}, {
   3.624543465207533*^9, 3.6245434701371417`*^9}, {3.6582368658031893`*^9, 
   3.6582368744418097`*^9}, {3.658236912169846*^9, 3.658236933754101*^9}, 
   3.669112262716856*^9, 3.669209222730904*^9, {3.7041672676674337`*^9, 
   3.704167284655033*^9}, {3.704167329651287*^9, 3.70416738559415*^9}, {
   3.7041674222919683`*^9, 3.704167458954069*^9}, {3.704167506730369*^9, 
   3.704167673004244*^9}},ExpressionUUID->"0664b9da-8479-4862-884d-\
407bc7e204a9"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"absD1", "[", 
        RowBox[{"\[CapitalGamma]0", ",", " ", "vX"}], "]"}], "[", 
       RowBox[{"\[Gamma]", ",", "t"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "i", "}"}]}], "]"}], "\[Equal]", " ", "0"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5979292059430795`*^9, 3.5979292420701456`*^9}, {
   3.597929272325876*^9, 3.597929298654382*^9}, {3.597929354638584*^9, 
   3.597929492815488*^9}, {3.597930356399882*^9, 3.597930367070492*^9}, {
   3.624543465207533*^9, 3.6245434701371417`*^9}, {3.6582368658031893`*^9, 
   3.658236897594481*^9}, 3.658237315802019*^9, 3.669112262736991*^9, 
   3.669209222810616*^9},ExpressionUUID->"03ab63ff-3ed7-4b29-a46b-\
bec1088c8b38"],

Cell[BoxData[
 RowBox[{"DSolve", "[", 
  RowBox[{"%", ",", 
   RowBox[{"\[Gamma]", "[", "t", "]"}], ",", 
   RowBox[{"{", "t", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.658237844320169*^9, 3.658237985647746*^9}, 
   3.669112262783463*^9, 3.66961480389583*^9, 3.7041677640587482`*^9, 
   3.704167803185199*^9},ExpressionUUID->"d8687bbc-2a44-466a-8687-\
0e20723ac97e"],

Cell["\<\
In the affine space the lines with natural affine parameter t are the \
autoparallel curves. \[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.5979305555082703`*^9, 3.5979306105174165`*^9}, {
  3.597930665788578*^9, 3.597930668044707*^9}, {3.658239830736166*^9, 
  3.658239887135201*^9}},ExpressionUUID->"ca538265-0b12-40f7-92f0-\
34ffd6b0d3a7"],

Cell["\<\
Since the tangent vector depends on the parameter representation, the \
condition of the definition fixes also the parameter t of the curve up to an \
affine transformation s=s[t] :=a\[CenterDot]t+b. 
Indeed, assuming s =  s[t] is a smooth parameter transformation with s\
\[CloseCurlyQuote][t]\[NotEqual]0 such that\
\>", "Text",
 CellChangeTimes->{{3.597911657441409*^9, 3.597911675247427*^9}, {
   3.597911706147195*^9, 3.59791200922353*^9}, {3.597928848436631*^9, 
   3.5979288904770355`*^9}, {3.597928956356804*^9, 3.5979290724444437`*^9}, {
   3.6132847851830797`*^9, 3.6132847983008304`*^9}, 3.6132848615014453`*^9, 
   3.617441713205701*^9, {3.658234807302924*^9, 3.658234861100473*^9}, {
   3.6582351383782*^9, 3.658235344500518*^9}, {3.6582353865789948`*^9, 
   3.6582354498102703`*^9}, 3.658235544352624*^9, {3.658240195811388*^9, 
   3.6582402170670223`*^9}, {3.658240258227495*^9, 3.658240267459969*^9}, {
   3.658240391980744*^9, 3.658240488893304*^9}, 3.6582405331656713`*^9, {
   3.6582409085971947`*^9, 3.658240966685204*^9}, {3.658241000611578*^9, 
   3.658241021980783*^9}, {3.658300156041696*^9, 3.658300186456892*^9}, {
   3.658308888001495*^9, 3.658308889057474*^9}, 
   3.663392990072619*^9},ExpressionUUID->"c55a33bc-d1b4-4923-b843-\
01df38a7e474"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t", "]"}], " ", "=", 
   RowBox[{
    SubscriptBox["\[Gamma]", "1"], "[", 
    RowBox[{"s", "[", "t", "]"}], "]"}]}], ",", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], " ", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["\[Gamma]", "1"], "'"}], "[", 
      RowBox[{"s", "[", "t", "]"}], "]"}], "*", 
     RowBox[{"h", "[", "t", "]"}], 
     StyleBox[" ",
      FontColor->GrayLevel[0]], 
     StyleBox["with",
      FontColor->GrayLevel[0]], " ", 
     RowBox[{"h", "[", "t", "]"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"ds", "/", "dt"}], " ", "\[NotEqual]", " ", 
     RowBox[{"0.", "     ", 
      RowBox[{"(", "+", ")"}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6582410973972816`*^9, 3.658241110117888*^9}, {
   3.65830022598886*^9, 3.6583004623768377`*^9}, {3.658300500122272*^9, 
   3.658300529977416*^9}, {3.658300766852262*^9, 3.6583008032844353`*^9}, 
   3.6583008402031507`*^9, {3.658301570414311*^9, 3.658301576046728*^9}, {
   3.658301607166019*^9, 3.658301618014513*^9}, {3.65830178404663*^9, 
   3.658301786117344*^9}, 3.658301826461961*^9, {3.658301869989958*^9, 
   3.658301892181809*^9}, 3.6583028451933193`*^9, {3.658302888472539*^9, 
   3.658302981504304*^9}},ExpressionUUID->"26e83810-b4f5-49c0-82dd-\
ebd760b27ec5"],

Cell[TextData[{
 "We assume that \[Gamma][t] and ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Gamma]", "1"], "[", "s", "]"}]], "Input",
  CellChangeTimes->{{3.6582410973972816`*^9, 3.658241110117888*^9}, {
    3.65830022598886*^9, 3.6583004623768377`*^9}, {3.658300500122272*^9, 
    3.658300529977416*^9}, {3.658300766852262*^9, 3.6583008032844353`*^9}, 
    3.6583008402031507`*^9},ExpressionUUID->
  "e3de6409-f7aa-40b1-8989-40ffdfc02eba"],
 " both are autoparallel curves, that means (in traditional notation)"
}], "Text",
 CellChangeTimes->{{3.597911657441409*^9, 3.597911675247427*^9}, {
   3.597911706147195*^9, 3.59791200922353*^9}, {3.597928848436631*^9, 
   3.5979288904770355`*^9}, {3.597928956356804*^9, 3.5979290724444437`*^9}, {
   3.6132847851830797`*^9, 3.6132847983008304`*^9}, 3.6132848615014453`*^9, 
   3.617441713205701*^9, {3.658234807302924*^9, 3.658234861100473*^9}, {
   3.6582351383782*^9, 3.658235344500518*^9}, {3.6582353865789948`*^9, 
   3.6582354498102703`*^9}, 3.658235544352624*^9, {3.658240195811388*^9, 
   3.6582402170670223`*^9}, {3.658240258227495*^9, 3.658240267459969*^9}, {
   3.658240391980744*^9, 3.658240488893304*^9}, 3.6582405331656713`*^9, {
   3.658301113474463*^9, 3.658301150010138*^9}, {3.658301190629654*^9, 
   3.658301269857524*^9}},ExpressionUUID->"4174d0a1-24c5-4538-b2b4-\
75ec1a6f5a04"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["D", 
     RowBox[{"\[Gamma]", "'"}]], 
    RowBox[{
     RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]}], " ", "=", " ", "0"}], ",", 
  " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["D", 
     RowBox[{
      SubscriptBox["\[Gamma]", "1"], "'"}]], 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Gamma]", "1"], "'"}], "[", "s", "]"}]}], " ", "=", " ", 
   "0."}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6583012823557253`*^9, {3.658301321199192*^9, 3.658301375016993*^9}, {
   3.658301499793236*^9, 3.658301538278985*^9}, 3.658301646965857*^9, {
   3.669112262804957*^9, 3.669112262809033*^9}, {3.669615015581662*^9, 
   3.669615026381371*^9}},ExpressionUUID->"11665257-7942-4f00-9b16-\
6eed15f11de7"],

Cell["\<\
Deriving the second equation of (+) along the curve \[Gamma][t] we get using \
the product rule\
\>", "Text",
 CellChangeTimes->{{3.6583016897896852`*^9, 3.658301702709697*^9}, {
   3.658301761181698*^9, 3.6583018061426287`*^9}, {3.658301900292912*^9, 
   3.6583019694282713`*^9}, {3.6583020568273*^9, 3.658302068603266*^9}, 
   3.6691122628412313`*^9, 
   3.6696150437809563`*^9},ExpressionUUID->"0ed09564-172a-45cc-88f2-\
5c5fe33efe34"],

Cell[BoxData[
 RowBox[{"0", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["D", 
     RowBox[{"\[Gamma]", "'"}]], 
    RowBox[{
     RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["D", 
      RowBox[{
       SubscriptBox["\[Gamma]", "\[IndentingNewLine]"], "'"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         SubscriptBox["\[Gamma]", "1"], "'"}], "[", 
        RowBox[{"s", "[", "t", "]"}], "]"}], "*", 
       RowBox[{"h", "[", "t", "]"}]}], ")"}]}], "=", "  ", 
    RowBox[{
     RowBox[{
      SubscriptBox["D", 
       RowBox[{
        SubscriptBox["\[Gamma]", "\[IndentingNewLine]"], "'"}]], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["\[Gamma]", "1"], "'"}], "[", 
        RowBox[{"s", "[", "t", "]"}], "]"}], ")"}], "*", 
      RowBox[{"h", "[", "t", "]"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["\[Gamma]", "1"], "'"}], "[", 
       RowBox[{"s", "[", "t", "]"}], "]"}], "*", 
      RowBox[{"h", "'"}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.658302105834805*^9, {3.658302141498671*^9, 3.6583023187688227`*^9}, {
   3.65830236824692*^9, 3.658302372209395*^9}, {3.658302440431172*^9, 
   3.658302462102911*^9}, {3.658302636268434*^9, 3.6583026908286657`*^9}, {
   3.658302996712658*^9, 3.6583029996320953`*^9}, 3.658303658268338*^9, {
   3.66911226284581*^9, 3.669112262848394*^9}, {3.6696150905816927`*^9, 
   3.6696151102692738`*^9}, 
   3.6696151633166847`*^9},ExpressionUUID->"d406fcc2-162d-4293-bb21-\
3ffe1137f603"],

Cell[TextData[{
 "Since , by the linearity  of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["D", "X"], " "}], TraditionalForm]],ExpressionUUID->
  "4a77bcd9-3d39-415f-9e1c-f91e6c243716"],
 " in X, we have"
}], "Text",
 CellChangeTimes->{{3.6583030341032887`*^9, 3.658303094910597*^9}, {
  3.658303148811913*^9, 3.658303214350028*^9}, {3.669112262871518*^9, 
  3.669112262871551*^9}, {3.6696155881708593`*^9, 
  3.669615591929771*^9}},ExpressionUUID->"37c7bd1a-db81-4c34-95f7-\
516dd16b4d3b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["D", 
     RowBox[{
      SubscriptBox["\[Gamma]", "\[IndentingNewLine]"], "'"}]], 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Gamma]", "1"], "'"}], "[", 
     RowBox[{"s", "[", "t", "]"}], "]"}]}], "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "/", "h"}], ")"}], "*", " ", 
     SubscriptBox["D", 
      RowBox[{
       SubscriptBox["\[Gamma]", "1"], "'"}]], 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Gamma]", "1"], "'"}], "[", 
      RowBox[{"s", "[", "t", "]"}], "]"}]}], "=", "0"}]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6583034511804667`*^9, 3.65830355986106*^9}, {
   3.658303621067893*^9, 3.658303632675346*^9}, 3.658303664763445*^9, {
   3.66911226288728*^9, 3.669112262891451*^9}, {3.669615600272931*^9, 
   3.669615611288509*^9}},ExpressionUUID->"75c05c75-1559-4742-b82a-\
66197442b144"],

Cell[TextData[{
 "and since for regular curves ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubscriptBox["\[Gamma]", "1"], "'"}], "[", 
   RowBox[{"s", "[", "t", "]"}], "]"}]], "Input",
  CellChangeTimes->{
   3.658302105834805*^9, {3.658302141498671*^9, 3.6583023187688227`*^9}, {
    3.65830236824692*^9, 3.658302372209395*^9}, {3.658302440431172*^9, 
    3.658302462102911*^9}, {3.658302636268434*^9, 3.6583026908286657`*^9}, {
    3.658302996712658*^9, 3.6583029996320953`*^9}, 3.658303658268338*^9},
  ExpressionUUID->"3f83f2e2-39a2-483c-a90f-f28cda2c7d13"],
 "\[NotEqual] null it follows h\[CloseCurlyQuote] = s\[CloseCurlyQuote]\
\[CloseCurlyQuote] = 0, thus the parameter transformation is"
}], "Text",
 CellChangeTimes->{{3.658303669072719*^9, 3.6583036816126833`*^9}, {
   3.658303712626768*^9, 3.65830371401096*^9}, {3.658303879394197*^9, 
   3.658303892745826*^9}, {3.658303941756195*^9, 3.658304035649454*^9}, {
   3.658308685986989*^9, 3.658308700370987*^9}, {3.6583087312264833`*^9, 
   3.658308737986288*^9}, 3.65830877348223*^9, {3.66339330540694*^9, 
   3.6633933220468073`*^9}, {3.6633933720224752`*^9, 
   3.663393380950244*^9}},ExpressionUUID->"8bfe4204-004f-4e3e-a501-\
90080b1bf396"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"s", "[", "t", "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "*", "t"}], "+", 
     RowBox[{"b", " ", 
      StyleBox["with",
       FontColor->GrayLevel[0]], "  ", "a"}]}], "\[NotEqual]", "0"}]}], ",", 
  " ", "a", ",", 
  RowBox[{"b", " ", "\[Element]", " ", 
   RowBox[{"\[DoubleStruckCapitalR]", ".", "                         ", 
    RowBox[{"(", "2", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.658304038600905*^9, 3.658304100641611*^9}, 
   3.658304147385357*^9, 3.658308665999215*^9, 3.658308791026305*^9, 
   3.658308837474036*^9, {3.65830989736303*^9, 3.658309920612887*^9}, 
   3.6696162484668627`*^9},ExpressionUUID->"597fab26-6e8f-45a8-a6be-\
397dc9215bcf"],

Cell[TextData[{
 "Collecting the results we formulate\n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ".  Let ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "b330426d-f3a1-4d0d-b2d1-0d654cef324d"],
 " be a smooth manifold with a linear connection \[CapitalGamma]. Then at \
each point x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "80934023-509e-4e3d-83bb-5415f20c2a48"],
 " and any vector X \[NotEqual] null vector there exists a uniquely defined \
autoparrallel curve \[Gamma][t]  with \[Gamma][",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "0"], TraditionalForm]],ExpressionUUID->
  "12753699-01f1-4326-a45a-0505f1b5aa82"],
 "] =x and \[Gamma]\[CloseCurlyQuote][",
 Cell[BoxData[
  FormBox[
   SubscriptBox["t", "0"], TraditionalForm]],ExpressionUUID->
  "d1c159b8-eac8-45ec-a484-ae23a7b0c7d9"],
 "] = X as the solution of the system of differential equations (1). The \
parameter t of this solution is defined up to an affine transformation (2); \
it is named an ",
 StyleBox["affine  Parameter",
  FontColor->RGBColor[0, 0, 1]],
 " of the autoparallel curve. \[FilledSquare]\n",
 StyleBox["Remarks",
  FontColor->RGBColor[0, 0, 1]],
 ". The first statement follows from general existence- and uniqueness \
theorems about the solutions of the differential. equations (1). \nIn \
difference to the arc length in Euclidean geometry an affine parameter isn\
\[CloseCurlyQuote]t defined for arbitrary smooth curves,  it exists for \
autoparallel curves only.\nThe autoparallel curves are generalizations of the \
geodesics on surfaces in the Euclidean space or Riemannian manifolds treated \
in the notebook ",
 ButtonBox["euklsfv3.nb",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 ", subsection 4.2.5, containing some examples."
}], "Text",
 CellChangeTimes->{{3.658310067184272*^9, 3.658310310956297*^9}, {
   3.6583103606441298`*^9, 3.658310450806446*^9}, {3.6583104876755047`*^9, 
   3.658310537579447*^9}, {3.658310595187443*^9, 3.6583106814705057`*^9}, {
   3.658310715005468*^9, 3.6583108489657183`*^9}, 3.6583109361394463`*^9, {
   3.6583109770805798`*^9, 3.6583111318797283`*^9}, {3.658311196384447*^9, 
   3.658311365185004*^9}, {3.658334596980464*^9, 3.658334791614559*^9}, {
   3.658334826858049*^9, 3.658334826859201*^9}, {3.658466711527404*^9, 
   3.65846672779105*^9}, {3.65846684089555*^9, 3.65846692198976*^9}, {
   3.6655582068941507`*^9, 3.665558210837351*^9}, 3.665558250916477*^9, {
   3.6696166699292793`*^9, 3.669616685159359*^9}, {3.702346838657391*^9, 
   3.7023468590638123`*^9}, 3.702346890439303*^9, 
   3.70416793134394*^9},ExpressionUUID->"e0690bd4-abf2-438e-96a7-\
4fd8376885e8"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.5. Properties of the Curvature and Torsion Tensors", "Subsection",
 CellChangeTimes->{{3.658467127563079*^9, 3.658467154426777*^9}, {
  3.658658932256463*^9, 
  3.6586589527004557`*^9}},ExpressionUUID->"149f77cd-e0e3-4247-945f-\
d91122eb30ad"],

Cell[CellGroupData[{

Cell["3.5.1. Symmetry Properties", "Subsubsection",
 CellChangeTimes->{{3.65847059770016*^9, 3.658470625331717*^9}, 
   3.6696167576712008`*^9, 3.669619769021575*^9, 3.702346946092609*^9, 
   3.7023561849615993`*^9, {3.7041680134027643`*^9, 3.70416801522655*^9}, 
   3.704199153756626*^9},ExpressionUUID->"57e5b13d-6b0b-49bf-bb11-\
d56b7286abea"],

Cell[TextData[{
 "We consider an arbitrary linear connection \[CapitalGamma] on a \
n-dimensional smooth manifold ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "57a370c8-5e5f-473b-bb7d-c4ad11a64f40"],
 ", n=3. The torsion tensor field is antisymmetric by definition, see   \
Subsection 3.2.2.(5). The curvature tensor  is"
}], "Text",
 CellChangeTimes->{{3.658470351621616*^9, 3.658470392869267*^9}, {
  3.65847047540473*^9, 3.658470542948287*^9}, {3.658659516721113*^9, 
  3.6586595654552803`*^9}, {3.658659649471035*^9, 3.658659786893787*^9}, {
  3.6586598527736053`*^9, 3.658659861461033*^9}, {3.65866002092417*^9, 
  3.6586600364677677`*^9}},
 TextJustification->1.,ExpressionUUID->"90d55f84-8532-4f36-8bf0-2e51543f4471"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.658467170641263*^9, 3.658467189290949*^9}, {
   3.6584673033076763`*^9, 3.658467314185235*^9}, 
   3.662622985329588*^9},ExpressionUUID->"9b4ec66e-fde7-40f9-8a8d-\
d10e28808823"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"RT", " ", "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.658467391992221*^9, 3.65846745886467*^9}, 
   3.6626229853846817`*^9},ExpressionUUID->"e13ad7e0-ca04-42c2-9870-\
1575ae47e600"],

Cell["\<\
To find its symmetry properties we consider its array of coordinates\
\>", "Text",
 CellChangeTimes->{{3.658470679971315*^9, 
  3.6584707361227694`*^9}},ExpressionUUID->"340bb2e2-9b06-42f1-a0cc-\
53aa8054afd0"],

Cell[BoxData[
 RowBox[{"?", "tensorToArray"}]], "Input",
 CellChangeTimes->{{3.6584673415945*^9, 
  3.658467349349908*^9}},ExpressionUUID->"269d1c5a-0494-4a67-8e0d-\
454cde31f1f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ar", "[", "ta_", "]"}], " ", ":=", " ", 
  RowBox[{"tensorToArray", "[", "ta", "]"}]}]], "Input",
 CellChangeTimes->{{3.6584674737654753`*^9, 3.6584675413924437`*^9}, {
  3.658660149218993*^9, 3.6586601564589357`*^9}, {3.658660235146182*^9, 
  3.6586602624980583`*^9}},ExpressionUUID->"d32d189e-627a-48c1-8eae-\
276760c207b0"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". Low dimensions"
}], "Text",
 CellChangeTimes->{{3.658471421543256*^9, 
  3.6584714561007566`*^9}},ExpressionUUID->"7c599f82-c96a-4024-82b3-\
ee2f3c7343c9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "0"}], ";", 
  RowBox[{"RT", " ", "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{3.658471031961135*^9, 
  3.662622985443783*^9},ExpressionUUID->"8a427ea5-5e8a-4a16-bb14-\
7360108c30fd"],

Cell[BoxData[
 RowBox[{"type", "[", "RT", "]"}]], "Input",
 CellChangeTimes->{{3.702354844495144*^9, 
  3.702354864908657*^9}},ExpressionUUID->"988543ed-90da-4bf3-9ad9-\
0502fffe1e45"],

Cell[BoxData[
 RowBox[{"ar", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6584710447370462`*^9, 3.658471056528573*^9}, {
  3.658660292201562*^9, 
  3.658660292697824*^9}},ExpressionUUID->"8ac913be-ed02-421f-ac36-\
284773d28381"],

Cell[BoxData[
 RowBox[{"Head", "[", "%%", "]"}]], "Input",
 CellChangeTimes->{{3.6584712505416527`*^9, 
  3.6584712612071733`*^9}},ExpressionUUID->"90a44dfd-2b31-4cf8-8f3a-\
38fdf390af10"],

Cell["Of course, a point has no curvature.", "Text",
 CellChangeTimes->{{3.658471643531331*^9, 
  3.658471668843116*^9}},ExpressionUUID->"1d1c0fe6-fd0c-4f5c-a5b4-\
7be030adf55d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "1"}], ";", 
  RowBox[{"RT", " ", "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{3.658471095768264*^9, 3.658471206344737*^9, 
  3.6626229854984407`*^9},ExpressionUUID->"4ee86e00-7c00-45f3-996f-\
7c3c685757c3"],

Cell[BoxData[
 RowBox[{"ar", "[", "RT", "]"}]], "Input",
 CellChangeTimes->{{3.6586601823138323`*^9, 3.6586602071220217`*^9}, {
  3.658660322593972*^9, 
  3.658660325849196*^9}},ExpressionUUID->"594b32f0-b68a-4b4b-a2be-\
a20ec2b31953"],

Cell[BoxData[
 RowBox[{"RT", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7023549719349422`*^9, 
  3.70235502043543*^9}},ExpressionUUID->"925b54b7-0762-4deb-a458-\
5de483058a5f"],

Cell[TextData[{
 "For a curve the curvature tensor is trivial. The dimension dim = 2 has been \
discussed in details  in the notebook euklidsfv3. nb (see [",
 ButtonBox["E",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "D",
 ButtonBox["G",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "] )  "
}], "Text",
 CellChangeTimes->{{3.6584717006986513`*^9, 3.658471770370098*^9}, {
   3.658471839201743*^9, 3.65847185579375*^9}, 3.6691122629185266`*^9, 
   3.70235504012386*^9, 3.704179375023918*^9, {3.704179430791381*^9, 
   3.704179436831208*^9}},ExpressionUUID->"4efda785-76ab-4901-a06e-\
12c037f46518"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "2"}], ";", 
  RowBox[{"RT", " ", "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{3.658471095768264*^9, 3.658471206344737*^9, 
  3.658471360270076*^9, 
  3.6626229855528593`*^9},ExpressionUUID->"0e4b650b-bb51-4fbc-af4a-\
e748f9f02aab"],

Cell[BoxData[
 RowBox[{"ar", "[", "RT", "]"}]], "Input",
 CellChangeTimes->{{3.6584711762957487`*^9, 3.6584711795517483`*^9}, {
  3.658660359433324*^9, 3.658660369433835*^9}, {3.658660404281535*^9, 
  3.658660412776598*^9}, {3.658660460609083*^9, 
  3.658660490505261*^9}},ExpressionUUID->"d3e4a387-ad24-4d49-9bec-\
add815ba99f6"],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.658471884833453*^9, 3.6584718936575947`*^9}, 
   3.6691122629213123`*^9, 
   3.669617029580558*^9},ExpressionUUID->"ab98ed13-743b-4b8f-983e-\
1502a373f9ef"],

Cell[BoxData[
 RowBox[{"?", "TensorSymmetry"}]], "Input",
 CellChangeTimes->{{3.663394781556542*^9, 
  3.663394788563088*^9}},ExpressionUUID->"efe69fab-201a-4206-9d84-\
6458450c46e2"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", 
  RowBox[{"ar", "[", "RT", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663394614249857*^9, 3.663394632348525*^9}, 
   3.663394765771956*^9, 
   3.663394828453308*^9},ExpressionUUID->"d66f27f9-cee6-445e-ad05-\
c96eb3513938"],

Cell[BoxData[
 RowBox[{"type", "[", "RT", "]"}]], "Input",
 CellChangeTimes->{{3.702355132553955*^9, 
  3.702355141362344*^9}},ExpressionUUID->"1b770c94-2f8b-4472-8f03-\
61b348d2b7b3"],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"RT", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "i", ",", "j"}], "}"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}], "+", 
    RowBox[{"RT", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "j", ",", "i"}], "}"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "dim"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "dim"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7023551716012383`*^9, 3.7023552226014347`*^9}, {
  3.702355601495771*^9, 3.7023556227593317`*^9}, {3.7023556555188303`*^9, 
  3.702355667990549*^9}, {3.702355699693943*^9, 
  3.702355967475429*^9}},ExpressionUUID->"b81edeeb-db5a-418d-813c-\
ac947033a44b"],

Cell["\<\
\[FilledSquare]
We return to the case dim \[GreaterEqual] 3.\
\>", "Text",
 CellChangeTimes->{{3.658470764146576*^9, 3.658470811522201*^9}, {
  3.658471936432115*^9, 3.6584719852006683`*^9}, {3.6584725913017893`*^9, 
  3.65847259130297*^9}, {3.6696197202380342`*^9, 
  3.669619728637744*^9}},ExpressionUUID->"f563e640-3f31-41a4-b6ab-\
5bc23f941a38"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "5"}], ";", 
  RowBox[{"RT", " ", "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{
  3.658471095768264*^9, 3.658471206344737*^9, 3.658471360270076*^9, 
   3.6584720634406853`*^9, {3.658472498032893*^9, 3.658472500382497*^9}, 
   3.66262298560612*^9},ExpressionUUID->"1a666fe3-8238-4b67-864a-\
2ac82922548f"],

Cell[BoxData[
 RowBox[{"arRT", "=", 
  RowBox[{"tensorToArray", "[", "RT", "]"}]}]], "Input",
 CellChangeTimes->{{3.6584711762957487`*^9, 3.6584711795517483`*^9}, {
  3.6584721028166723`*^9, 
  3.658472122447927*^9}},ExpressionUUID->"6450f189-c289-4ba9-980a-\
0e9ad9f67368"],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.658471884833453*^9, 3.6584718936575947`*^9}, 
   3.669112262934657*^9, 
   3.66961963670331*^9},ExpressionUUID->"16b21c5e-db5b-4bbf-81cb-\
e62b09e336d9"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "arRT", "]"}]], "Input",
 CellChangeTimes->{{3.658467675133802*^9, 
  3.6584677289980288`*^9}},ExpressionUUID->"bff9010b-6396-4471-ae4c-\
3b6f4f855621"],

Cell["\<\
This function finds an antisymmetry in the slots 2 and 3. We test it defining\
\>", "Text",
 CellChangeTimes->{{3.658470845853154*^9, 
  3.658470926977353*^9}},ExpressionUUID->"c91a98ce-da09-4c0c-852b-\
bdd7648bbad9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"testf", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"RT", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"i", ",", "l", ",", "k"}], "}"}], ",", 
       RowBox[{"{", "j", "}"}]}], "]"}], "+", 
     RowBox[{"RT", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"i", ",", "k", ",", "l"}], "}"}], ",", 
       RowBox[{"{", "j", "}"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"l", ",", "dim"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658467797359783*^9, 
  3.658467988267696*^9}},ExpressionUUID->"ffe5ed06-44ce-4bf4-99fa-\
92115e10babe"],

Cell[BoxData[
 RowBox[{
  RowBox[{"testf", "[", 
   RowBox[{"3", ",", "5"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.658467999637817*^9, 3.658468111202888*^9}, 
   3.658472554926291*^9},ExpressionUUID->"6616315e-3db3-46db-9fec-\
1805543c35eb"],

Cell[TextData[{
 "Evaluate the last cell for all values of i,j. Consider other dimensions, if \
wanted. We conclude\n",
 StyleBox["Lemma.",
  FontColor->RGBColor[0, 0, 1]],
 " The curvature tensor RCtensor of any smooth linear connection is \
antisymmetric in the slots 2,3:"
}], "Text",
 CellChangeTimes->{{3.65847234255055*^9, 3.658472464021978*^9}, {
   3.658472611653442*^9, 3.658472748436187*^9}, 3.702356108821946*^9, {
   3.7041797160135517`*^9, 
   3.7041797203329563`*^9}},ExpressionUUID->"ccb9543f-6487-45e7-bcc4-\
208d28a20758"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox["RT", "hij", 
    RowBox[{"       ", "k"}]], " ", "+", " ", 
   SubsuperscriptBox["RT", "hji", 
    RowBox[{"       ", "k"}]]}], "  ", "\[Equal]", "  ", 
  RowBox[{"0.", "                     ", "\[FilledSquare]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.65847276168016*^9, 3.658472762156052*^9}, {
  3.6584727989398108`*^9, 
  3.658472978062779*^9}},ExpressionUUID->"e49716c0-0f8b-42de-b2a0-\
b0f24710fd6a"],

Cell["A look on the definition shows the statement being obvious.", "Text",
 CellChangeTimes->{{3.658477621371602*^9, 3.6584776533313837`*^9}, 
   3.658659452328361*^9},ExpressionUUID->"79a2f3ba-9116-40aa-9926-\
bb8c3060b029"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.5.2. Alternated Twofold Absolute Derivations", "Subsection",
 CellChangeTimes->{{3.65847059770016*^9, 3.658470625331717*^9}, {
   3.658660814261071*^9, 3.65866088327617*^9}, 3.669112262971304*^9, 
   3.669619774622693*^9, 3.669619814749362*^9, 3.669620698934136*^9, 
   3.702356196130374*^9, 
   3.702357878803643*^9},ExpressionUUID->"481fb439-6833-470c-a98c-\
42b0037e5e06"],

Cell[CellGroupData[{

Cell["3.5.2.1. Functions", "Subsubsection",
 CellChangeTimes->{{3.658671633652501*^9, 3.658671684556116*^9}, 
   3.658741552296008*^9, {3.6587425966541147`*^9, 3.6587426163815603`*^9}, {
   3.6587426473016157`*^9, 3.6587426530614653`*^9}, 3.6593405440657263`*^9, 
   3.6696207087830563`*^9, 3.669631108307435*^9, {3.704198931970333*^9, 
   3.704198960432886*^9}, {3.7084810059316597`*^9, 3.7084810107631083`*^9}, 
   3.708481126082541*^9},ExpressionUUID->"443b7d8f-1d2d-48cb-8d7a-\
d8eab76a0da8"],

Cell["The tensor of type (0,0) corresponding to a function is", "Text",
 CellChangeTimes->{{3.658671633652501*^9, 3.658671684556116*^9}, 
   3.658741552296008*^9, {3.6587425966541147`*^9, 3.6587426163815603`*^9}, {
   3.6587426473016157`*^9, 3.6587426530614653`*^9}, {3.6593405440657263`*^9, 
   3.659340566121642*^9}},ExpressionUUID->"d22c4538-c5ae-4780-a18a-\
550be2ab57b4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"tf", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{
     RowBox[{"f", "[", "x", "]"}], ",", "0", ",", "0"}], "]"}]}], ";", 
  RowBox[{"tf", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658661467675105*^9, 3.658661541967236*^9}, {
  3.658661950180245*^9, 
  3.6586619554599857`*^9}},ExpressionUUID->"eb6ce697-e40b-4362-8cb4-\
21f801ee4fb6"],

Cell["Its gradient tensor", "Text",
 CellChangeTimes->{{3.658671699356159*^9, 3.658671715347849*^9}, 
   3.7023563260320797`*^9},ExpressionUUID->"a89d64de-9334-4fe9-894a-\
db029256968e"],

Cell[BoxData[
 RowBox[{"?", "nabla"}]], "Input",
 CellChangeTimes->{{3.658661715678054*^9, 3.6586617446291027`*^9}, 
   3.6630782779789677`*^9},ExpressionUUID->"e237fc3a-b5bf-492f-9666-\
6252736f6dad"],

Cell[BoxData[
 RowBox[{
  RowBox[{"grtf", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{
   RowBox[{"nabla", "[", 
    RowBox[{"\[CapitalGamma]", ",", "tf"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658661559177373*^9, 3.658661589006831*^9}, {
   3.6586616434304857`*^9, 3.658661695390131*^9}, {3.658661764914978*^9, 
   3.658661769269031*^9}, {3.658661818411132*^9, 3.6586618628607883`*^9}, {
   3.658662491856347*^9, 3.658662492351204*^9}, 
   3.663078277983244*^9},ExpressionUUID->"e08b5a84-acdd-4697-8da1-\
f10fe9ebb001"],

Cell["\<\
has the same coordinates as the differential (or gradient) of f :\
\>", "Text",
 CellChangeTimes->{{3.6586717329281187`*^9, 3.65867177285316*^9}, 
   3.702356368991933*^9},ExpressionUUID->"8023998f-061a-403f-8e70-\
f80085833bd6"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.658661899194841*^9, 
  3.6586619260529633`*^9}},ExpressionUUID->"0239468d-c99a-4afa-86bd-\
e3e5dc7d08b2"],

Cell["The gradient tensor of grtf is", "Text",
 CellChangeTimes->{{3.658671808595088*^9, 3.658671859882469*^9}, {
  3.669619875700549*^9, 
  3.669619883628519*^9}},ExpressionUUID->"effcdb2a-e291-4039-a1f6-\
0c3037882d3a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"gr2tf", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{
   RowBox[{"nabla", "[", 
    RowBox[{"\[CapitalGamma]", ",", "grtf"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658662011989534*^9, 3.658662025539806*^9}, {
   3.658662514383918*^9, 3.6586625266067657`*^9}, 
   3.663078278021863*^9},ExpressionUUID->"ab24b8bd-df8a-48c1-b402-\
616030787757"],

Cell["with the coordinates", "Text",
 CellChangeTimes->{{3.6586719251883717`*^9, 
  3.658671948243846*^9}},ExpressionUUID->"ddbf48e3-8f2d-42c1-b7da-\
a0c11eb68e0b"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.658662613303919*^9, 3.658662618741989*^9}, {
  3.658671893875684*^9, 
  3.658671906642765*^9}},ExpressionUUID->"fc5209ee-81f9-40d8-84c5-\
96278f837e97"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.658671980009968*^9, 
  3.658671989625869*^9}},ExpressionUUID->"4655ddd6-a582-4446-b7af-\
749d0f8f79b4"],

Cell["In traditional notation the coordinates are ", "Text",
 CellChangeTimes->{{3.6586721062564793`*^9, 3.6586721256802473`*^9}, {
   3.658674698237508*^9, 3.6586747464853697`*^9}, 
   3.658741813716419*^9},ExpressionUUID->"865c645f-ee64-4633-a71a-\
6f389494e8e6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "i"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], "f"}]}], " ", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "j"], "f"}]}], "  ", "-", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "l"], "f"}], " ", 
    SubsuperscriptBox["\[CapitalGamma]", "ij", "l"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.658672240560411*^9, 3.658672245799884*^9}, {
  3.658672286619232*^9, 3.658672428910529*^9}, {3.658672472318424*^9, 
  3.6586725049673233`*^9}, {3.65874166818913*^9, 3.658741668908948*^9}, {
  3.6696203204654207`*^9, 
  3.669620323337384*^9}},ExpressionUUID->"4a02cf69-648d-4d21-b6bf-\
71c7a569747f"],

Cell[TextData[StyleBox["Example 1",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.658742111168602*^9, 3.658742115225081*^9}, {
  3.6587433161635427`*^9, 3.658743320595471*^9}, {3.7041992670763597`*^9, 
  3.704199267491678*^9}},ExpressionUUID->"3e701042-5fb9-4d34-aaa8-\
794eaf7e16fa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "ta", "]"}], ";", 
  RowBox[{
   RowBox[{"ta", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"b", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.658674779360259*^9, 3.6586748154370747`*^9}, {
  3.65867484863728*^9, 3.658674889508889*^9}, {3.658674952132423*^9, 
  3.658674975964546*^9}, {3.658675032908291*^9, 
  3.65867506150917*^9}},ExpressionUUID->"5301a5f5-e590-4ba6-bd8d-\
0ff1dcb8ad91"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{"\[CapitalGamma]", ",", " ", "ta"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.658674906968555*^9, 3.658674934532529*^9}, {
   3.6586750005094967`*^9, 3.6586750037640133`*^9}, 
   3.663078278065851*^9},ExpressionUUID->"ae82ea5d-a0ec-4b2c-9264-\
6dff7950cc5c"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.658675085532612*^9, {3.658675136403624*^9, 
   3.658675144082629*^9}},ExpressionUUID->"8bd86b72-56a3-4d88-949e-\
e22d35dd794b"],

Cell["\<\
The derivation index (here f = 2) is always the last of the covariant \
indices.   \[FilledSquare]
Return to\
\>", "Text",
 CellChangeTimes->{{3.658672565005231*^9, 3.658672644364586*^9}, {
   3.6587421725924187`*^9, 3.6587422666839523`*^9}, {3.658742306264423*^9, 
   3.658742333390641*^9}, 3.658742775323962*^9, {3.658743378282483*^9, 
   3.658743379130526*^9}},ExpressionUUID->"c3bb5b1a-ebb1-4105-b5e9-\
ec96146ed089"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "i"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], "f"}]}], " ", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "j"], "f"}]}], "  ", "-", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "l"], "f"}], " ", 
    SubsuperscriptBox["\[CapitalGamma]", "ij", "l"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.658672240560411*^9, 3.658672245799884*^9}, {
  3.658672286619232*^9, 3.658672428910529*^9}, {3.658672472318424*^9, 
  3.6586725049673233`*^9}, {3.65874166818913*^9, 3.658741668908948*^9}, {
  3.669620200626285*^9, 
  3.669620201386297*^9}},ExpressionUUID->"5bdbba99-61ff-4869-b064-\
43ab4c85655b"],

Cell["\<\
Alternating the tensor  the first term of the right side gives zero:\
\>", "Text",
 CellChangeTimes->{{3.658672565005231*^9, 3.658672644364586*^9}, {
   3.6587421725924187`*^9, 3.6587422666839523`*^9}, {3.658742306264423*^9, 
   3.658742333390641*^9}, {3.6587423986796017`*^9, 3.658742399207246*^9}, 
   3.658742793972641*^9},ExpressionUUID->"9b043ff3-0dee-4aca-9f96-\
3f359c699762"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "i"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "j"], "f"}]}], "-", 
   RowBox[{
    SubscriptBox["\[Del]", "j"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], "f"}]}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      SubscriptBox["\[PartialD]", "l"], "f"}]}], 
    RowBox[{"(", " ", 
     RowBox[{
      SubsuperscriptBox["\[CapitalGamma]", "ij", "l"], " ", "-", " ", 
      SubsuperscriptBox["\[CapitalGamma]", "ji", "l"]}], ")"}]}], " ", "=", 
   RowBox[{
    RowBox[{"-", " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", "l"], "f"}]}], " ", 
    SubsuperscriptBox["T", "ij", 
     RowBox[{"    ", "l"}]], "                                                          ", 
    RowBox[{"(", "1", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.65867266676499*^9, 3.658672729588616*^9}, {
  3.658672759605713*^9, 3.6586730137386513`*^9}, {3.658742432535933*^9, 
  3.658742460719318*^9}, {3.658742953338633*^9, 3.658742974890462*^9}, {
  3.659249047182032*^9, 3.659249056926158*^9}, {3.6696205060800753`*^9, 
  3.6696205405839767`*^9}},ExpressionUUID->"39a077c0-5b07-4351-ab5f-\
260adb776883"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  SubsuperscriptBox["T", "ij", 
   RowBox[{"    ", "l"}]]], "Input",
  CellChangeTimes->{{3.65867266676499*^9, 3.658672729588616*^9}, {
   3.658672759605713*^9, 3.6586730137386513`*^9}},ExpressionUUID->
  "949fd2aa-e8c0-444f-bee2-cc5133a66f30"],
 " stands for the coordinates of the torsion tensor"
}], "Text",
 CellChangeTimes->{{3.658673224928301*^9, 3.6586732770080147`*^9}, 
   3.658742849084012*^9, {3.659249084921927*^9, 3.659249084921968*^9}, {
   3.663395501182076*^9, 
   3.6633955021977463`*^9}},ExpressionUUID->"995671b6-2ba2-4f30-bf6a-\
666188771dc9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tors", "[", "\[CapitalGamma]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6586730359709167`*^9, 
  3.658673167162034*^9}},ExpressionUUID->"37eb81d2-33a8-4540-b529-\
401a6546cec3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"altgr2tf", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", 
  RowBox[{"covalt", "[", 
   RowBox[{"gr2tf", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6586629473760242`*^9, 3.658662953142563*^9}, {
  3.65866299091359*^9, 3.658662994216814*^9}, {3.658663071416519*^9, 
  3.658663088479434*^9}},ExpressionUUID->"d02cb2f9-db62-4367-b595-\
a5a650db96d3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"altgr2tf", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"3", ",", "2"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}], " ", "+", 
  RowBox[{
   RowBox[{"altgr2tf", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.65866301844134*^9, 3.658663046855939*^9}, {
  3.65866311452015*^9, 
  3.658663188974855*^9}},ExpressionUUID->"404cda0f-6adf-4d54-acbd-\
23f1b297bbc9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"altgr2tf", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.658665259662407*^9, 3.658665306912529*^9}, {
  3.658669327536717*^9, 
  3.6586693381687803`*^9}},ExpressionUUID->"0429d500-3ee6-43a7-a4e7-\
588d9ab12ba4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tors", "[", "\[CapitalGamma]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
  RowBox[{"grtf", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658669531719795*^9, 3.6586695855982523`*^9}, 
   3.658669659806603*^9},ExpressionUUID->"deb205c2-ac68-4ea1-9534-\
9d4962aa0475"],

Cell[BoxData[
 RowBox[{"tt2", "=", " ", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tors", "[", "\[CapitalGamma]", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
     RowBox[{"grtf", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", "3", ",", "1"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.658669689521648*^9, 3.658669724044985*^9}, {
   3.658669762453905*^9, 3.658669791885386*^9}, {3.6586698679972982`*^9, 
   3.6586699152121572`*^9}, 
   3.6586702302740717`*^9},ExpressionUUID->"74b85758-367f-4bb8-93b0-\
e06d2cda01a1"],

Cell[BoxData[
 RowBox[{"tt2", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.658669947822657*^9, 3.6586699684834013`*^9}, 
   3.658670061347677*^9, {3.658670292353219*^9, 3.658670294616905*^9}, {
   3.6586736392305193`*^9, 
   3.6586736413097773`*^9}},ExpressionUUID->"36681036-24fc-402f-bee3-\
82effc04a94b"],

Cell[BoxData[
 RowBox[{"equi", "[", 
  RowBox[{"tt2", ",", 
   RowBox[{"altgr2tf", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6586703235770283`*^9, 3.6586703486801367`*^9}, {
  3.658670389607869*^9, 
  3.658670393599852*^9}},ExpressionUUID->"811db6df-0dcd-4ac5-8eb9-\
c586e5082051"],

Cell["\<\
OK. The calculation confirms formula (1). In particular, for symmetric linear \
connections one has\
\>", "Text",
 CellChangeTimes->{{3.658743029016638*^9, 3.658743065743861*^9}, {
   3.658743103679339*^9, 3.658743140374539*^9}, 
   3.659102588857151*^9},ExpressionUUID->"555d7fac-98a4-4d28-a9c4-\
acda3a881377"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "i"], 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], "f"}]}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "j"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], "f"}]}], "       ", 
   "\[FilledSquare]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.65867266676499*^9, 3.658672729588616*^9}, {
  3.658672759605713*^9, 3.6586730137386513`*^9}, {3.658742432535933*^9, 
  3.658742460719318*^9}, {3.658742953338633*^9, 3.658742974890462*^9}, {
  3.658743197526553*^9, 
  3.658743218025516*^9}},ExpressionUUID->"801c749c-c20d-4237-87ac-\
557ad1a8c6d7"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.5.2.2. Curvature and Torsion Operators", "Subsubsection",
 CellChangeTimes->{{3.659340672904992*^9, 3.659340691248678*^9}, 
   3.659423681209342*^9, 3.66963111578575*^9, 3.6696386371065063`*^9, {
   3.7041803098566303`*^9, 3.704180313576373*^9}, 3.704183034231265*^9, {
   3.704198982841341*^9, 3.7041989894564342`*^9}, 3.7084811330679007`*^9, 
   3.7084886770975733`*^9},ExpressionUUID->"bc3ce4c6-cf72-4c86-807d-\
e2371cc82ec9"],

Cell[TextData[{
 "In the following we need two algebraic operators defined by the torsion  \
tensor T and the curvature tensor R.  S. ",
 ButtonBox["Helgason",
  BaseStyle->"Hyperlink",
  ButtonData->"H78"],
 ", \[Section] 1.8,  defines for two vector fields X,Y  in  traditional \
notation"
}], "Text",
 CellChangeTimes->{{3.659103084005867*^9, 3.6591031577656183`*^9}, {
   3.659103192125338*^9, 3.659103197221344*^9}, 3.659103643858983*^9, {
   3.659103689338135*^9, 3.659103691163323*^9}, {3.659103731415831*^9, 
   3.65910379739371*^9}, {3.659104117455599*^9, 3.6591041347596273`*^9}, {
   3.659104597220776*^9, 3.659104612252246*^9}, {3.659248904414661*^9, 
   3.65924891352591*^9}, {3.659249184509343*^9, 3.659249201652952*^9}, {
   3.6593407574889317`*^9, 3.6593407810249166`*^9}, {3.663397834804859*^9, 
   3.663397876988006*^9}, {3.669637507251767*^9, 
   3.669637508747686*^9}},ExpressionUUID->"0df0a3d4-7bb3-4c8e-b507-\
56c46f87f354"],

Cell[BoxData[
 RowBox[{
  RowBox[{"T", 
   RowBox[{"(", 
    RowBox[{"X", ",", "Y"}], ")"}]}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "X"], 
    RowBox[{"(", "Y", ")"}]}], " ", "-", "      ", 
   RowBox[{
    SubscriptBox["\[Del]", "Y"], 
    RowBox[{"(", "X", ")"}]}], "  ", "-", " ", 
   RowBox[{
    RowBox[{"[", 
     RowBox[{"X", ",", "Y"}], "]"}], "                                                       ", 
    RowBox[{"(", "2", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.659103807871719*^9, 3.659103859369636*^9}, {
  3.659103937903619*^9, 3.659104032240687*^9}, {3.65910407455234*^9, 
  3.659104084583899*^9}, {3.659104190823513*^9, 3.6591041978476963`*^9}, {
  3.659250072998001*^9, 
  3.659250078046061*^9}},ExpressionUUID->"e8c12f95-418d-47f7-bdd4-\
e7ddd29e3ce7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"R", 
   RowBox[{"(", 
    RowBox[{"X", ",", " ", "Y"}], ")"}]}], " ", "=", " ", 
  RowBox[{
   SubscriptBox["\[Del]", "X"], 
   RowBox[{
    SubscriptBox["\[Del]", "Y"], "  ", 
    RowBox[{"-", " ", 
     RowBox[{
      SubscriptBox["\[Del]", "Y"], 
      RowBox[{
       SubscriptBox["\[Del]", 
        RowBox[{"X", " "}]], 
       RowBox[{"-", "  ", 
        RowBox[{
         SubscriptBox["\[Del]", 
          RowBox[{"[", 
           RowBox[{"X", ",", "Y"}], "]"}]], 
         "                                                                    \
", 
         RowBox[{"(", "3", ")"}]}]}]}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.659104154105115*^9, 3.6591041751115522`*^9}, {
  3.659104237160205*^9, 3.659104316446753*^9}, {3.65910436406329*^9, 
  3.6591045597247343`*^9}, {3.659254939436763*^9, 
  3.6592549482685413`*^9}},ExpressionUUID->"5102a98a-3494-4d71-9f47-\
996173284475"],

Cell[TextData[{
 "In Helgason\[CloseCurlyQuote]s book these equations serve as definitions of \
the torsion tensor T and the curvature tenso R. Since in our presentation \
both tensors are defined already in other ways,  these equations become \
statements of properties of T and R;  we will interpret the left sides of \
these formulas  in terms of the torsion and curvature tensors defined before \
and then confirm the statements. The proof by calculations in local \
coordinates are left to the user. We begin with (2) translating it into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " concepts.  Let"
}], "Text",
 CellChangeTimes->{{3.659104719867259*^9, 3.6591047640831337`*^9}, {
   3.65924924832446*^9, 3.6592493492754707`*^9}, {3.659249401979087*^9, 
   3.659249430179221*^9}, {3.659249516466531*^9, 3.659249549802248*^9}, 
   3.659249589161982*^9, {3.659249690137045*^9, 3.659249885391501*^9}, {
   3.6592499398234463`*^9, 3.65924999098267*^9}, 3.659250092957409*^9, {
   3.659340841648065*^9, 3.659340857384046*^9}, {3.659340900223219*^9, 
   3.659340923062999*^9}, {3.65934228474186*^9, 3.6593422878763847`*^9}, {
   3.65934233802007*^9, 3.659342378667859*^9}, 3.6633979113085127`*^9, {
   3.7023568885822353`*^9, 
   3.702356908405981*^9}},ExpressionUUID->"54a12b20-a3cd-4514-8cda-\
29755d97e19b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"vX", ",", "\[Xi]"}], "]"}], ";", 
  RowBox[{
   RowBox[{"vX", "[", "x__", "]"}], ":=", 
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", "x", "]"}]}], ";", 
  RowBox[{"vX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658823430721339*^9, 3.6588235923806543`*^9}, 
   3.6595140348043003`*^9, 3.665651225022389*^9, {3.6656512608216677`*^9, 
   3.665651268949911*^9}, {3.6656513487403393`*^9, 3.665651468154565*^9}, {
   3.665653213440683*^9, 3.665653266030534*^9}, {3.702357032100669*^9, 
   3.70235709805033*^9}, {3.702357144185479*^9, 
   3.702357186617242*^9}},ExpressionUUID->"b49b5393-e39b-487e-9916-\
52f7944769b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vA", "]"}], ";", 
  RowBox[{
   RowBox[{"vA", "[", "x__", "]"}], ":=", 
   RowBox[{
    RowBox[{"vv", "[", "a", "]"}], "[", "x", "]"}]}], ";", 
  RowBox[{"vA", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658823430721339*^9, 3.6588235923806543`*^9}, 
   3.6595140348043003`*^9, 3.665651225022389*^9, {3.6656512608216677`*^9, 
   3.665651268949911*^9}, {3.6656513487403393`*^9, 3.665651468154565*^9}, {
   3.665653213440683*^9, 3.665653266030534*^9}, {3.665653315262239*^9, 
   3.6656533704609756`*^9}},ExpressionUUID->"3e2145ae-2c80-46e9-a332-\
62f80c19ac34"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vB", "]"}], ";", 
  RowBox[{
   RowBox[{"vB", "[", "x__", "]"}], ":=", 
   RowBox[{
    RowBox[{"vv", "[", "b", "]"}], "[", "x", "]"}]}], ";", 
  RowBox[{"vB", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658823430721339*^9, 3.6588235923806543`*^9}, 
   3.6595140348043003`*^9, 3.665651225022389*^9, {3.6656512608216677`*^9, 
   3.665651268949911*^9}, {3.6656513487403393`*^9, 3.665651468154565*^9}, {
   3.665653213440683*^9, 3.665653266030534*^9}, {3.665653315262239*^9, 
   3.6656534324027443`*^9}},ExpressionUUID->"9af131e2-cb5f-4ce1-aef2-\
f381b396dfc2"],

Cell[BoxData[
 RowBox[{"Clear", "[", "vY", "]"}]], "Input",
 CellChangeTimes->{{3.658823430721339*^9, 3.658823657739585*^9}, 
   3.659514042644701*^9, {3.6656516667425327`*^9, 
   3.665651675454282*^9}},ExpressionUUID->"1600aece-f75c-40fd-9b23-\
4edb352ab193"],

Cell["be vector fields on the manifold. Then", "Text",
 CellChangeTimes->{{3.6591032357182503`*^9, 3.659103269070619*^9}, {
   3.659103437459689*^9, 3.659103462275361*^9}, {3.659103566227195*^9, 
   3.659103580826643*^9}, 3.659103628074532*^9, 3.659103677978236*^9, {
   3.659104783955122*^9, 3.6591048417706747`*^9}, {3.659341063589283*^9, 
   3.6593410827974167`*^9}, 
   3.665651691957178*^9},ExpressionUUID->"b42d08ae-f99f-4118-a9ca-\
a66db54fb5c2"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.5.2.3. T(X,Y)", "Subsubsection",
 CellChangeTimes->{{3.65934102865352*^9, 3.6593410466932917`*^9}, {
  3.659711499580008*^9, 3.6597115022440157`*^9}, {3.704199006210343*^9, 
  3.70419901662451*^9}},ExpressionUUID->"5c26802e-4df9-4988-9b0c-\
4ae4c33dd392"],

Cell["corresponds to", "Text",
 CellChangeTimes->{{3.659341096957922*^9, 
  3.659341107685116*^9}},ExpressionUUID->"b969d8f0-c960-4b29-99a6-\
8101bae588f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vtors", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"vtors", "[", 
     RowBox[{"\[CapitalGamma]_", ",", "v_", ",", "w_"}], "]"}], "[", "x__", 
    "]"}], " ", ":=", " ", 
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tors", "[", "\[CapitalGamma]", "]"}], "[", "x", "]"}], 
      "\[CircleTimes]", 
      RowBox[{"vecTotensor", "[", 
       RowBox[{"v", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
      RowBox[{"vecTotensor", "[", 
       RowBox[{"w", "[", "x", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], "}"}]}], "]"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.6591050568264637`*^9, 3.659105194176456*^9}, {
   3.659105347104501*^9, 3.659105503998913*^9}, 3.659105543806645*^9, {
   3.6591055923187447`*^9, 3.6591056207100067`*^9}, {3.6591056794301987`*^9, 
   3.6591057050455523`*^9}, 3.6591057544853086`*^9, {3.665651765493293*^9, 
   3.6656517704613523`*^9}, {3.665653499762974*^9, 3.665653553280035*^9}, {
   3.6656536192949133`*^9, 
   3.6656536312142763`*^9}},ExpressionUUID->"551f87b6-2145-445c-812e-\
6186c58e143a"],

Cell["Remember", "Text",
 CellChangeTimes->{{3.6591062260010633`*^9, 
  3.6591062284090967`*^9}},ExpressionUUID->"339be7fd-7c6f-4c41-9e93-\
c5c8a6338765"],

Cell[BoxData[
 RowBox[{"?", "tors"}]], "Input",
 CellChangeTimes->{{3.659106241971888*^9, 
  3.6591062449131308`*^9}},ExpressionUUID->"982d9a7f-fbe2-40ca-9f55-\
45b8886eba0a"],

Cell["The result is a vector field on the manifold, for example", "Text",
 CellChangeTimes->{{3.6591059270511093`*^9, 
  3.65910596171511*^9}},ExpressionUUID->"dac68073-ab04-4071-8f5a-\
99666a6bc131"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vtors", "[", 
   RowBox[{"\[CapitalGamma]", ",", "vA", ",", "vB"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.659105795826338*^9, 3.659105844396267*^9}, {
  3.665651798876237*^9, 
  3.665651806716691*^9}},ExpressionUUID->"a834b3c0-85c4-4d24-9545-\
5cfa0e942b11"],

Cell[BoxData[
 RowBox[{
  RowBox[{"torab", "=", 
   RowBox[{"tensorToArray", "[", "%", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6656518492625113`*^9, 3.6656518721160297`*^9}, {
  3.665652081966695*^9, 
  3.665652092989135*^9}},ExpressionUUID->"65781268-106d-48a3-a117-\
321fabf60b43"],

Cell["The coordinates of this vector field are given by", "Text",
 CellChangeTimes->{{3.659106026501295*^9, 
  3.659106056171728*^9}},ExpressionUUID->"4568f9dc-a415-4c89-8335-\
5a09be0ef713"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "torsop", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"torsop", "[", 
      RowBox[{"\[CapitalGamma]_", ",", "v_", ",", "w_"}], "]"}], "[", "x__", 
     "]"}], "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "k_", "}"}]}], "]"}], " ", ":=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[CapitalGamma]", "[", 
          RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "x", "]"}], "-", 
        RowBox[{
         RowBox[{"\[CapitalGamma]", "[", 
          RowBox[{"k", ",", "j", ",", "i"}], "]"}], "[", "x", "]"}]}], ")"}], 
      "*", 
      RowBox[{
       RowBox[{"v", "[", "x", "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{
       RowBox[{"w", "[", "x", "]"}], "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.659092651164105*^9, {3.659092684756003*^9, 3.659092730019205*^9}, {
   3.6590929316253633`*^9, 3.659092952953476*^9}, {3.659092996864588*^9, 
   3.659093098312606*^9}, {3.6590931289762163`*^9, 3.65909324525489*^9}, {
   3.65909327955088*^9, 3.659093296686742*^9}, 3.659093336063566*^9, {
   3.659094631427335*^9, 3.6590946465712337`*^9}, {3.6590947740899467`*^9, 
   3.659094782153141*^9}, {3.659096194446372*^9, 3.65909620441434*^9}, {
   3.659106808566431*^9, 3.6591068253175983`*^9}, {3.665652005067586*^9, 
   3.66565201037219*^9}, {3.665653706006359*^9, 3.6656537138391113`*^9}, {
   3.665653771756865*^9, 
   3.665653814644554*^9}},ExpressionUUID->"0ea215eb-eaf7-4bc4-978c-\
a9ee7d42fe30"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"torsop", "[", 
    RowBox[{"\[CapitalGamma]", ",", "vA", ",", "vB"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.658900056897686*^9, 3.658900080330883*^9}, {
   3.6589001214995613`*^9, 3.65890016220936*^9}, {3.659093396606748*^9, 
   3.659093406014143*^9}, {3.6590947531941557`*^9, 3.6590947580658493`*^9}, 
   3.6590966677387733`*^9, 3.659096807970168*^9, {3.6591069363410273`*^9, 
   3.6591069674055147`*^9}, {3.665652036418543*^9, 
   3.665652051048976*^9}},ExpressionUUID->"955b0e27-7670-497a-9868-\
ccb23670410e"],

Cell["Indeed", "Text",
 CellChangeTimes->{{3.659107016707346*^9, 
  3.6591070187793713`*^9}},ExpressionUUID->"d7734be4-0aaa-48c7-8943-\
a3b74c088e0a"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "-", 
   RowBox[{"torab", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.65909668665932*^9, 3.659096695769539*^9}, {
  3.6590968347612133`*^9, 3.659096896216526*^9}, {3.6591063240980673`*^9, 
  3.6591064353689137`*^9}, {3.6656521481543407`*^9, 
  3.66565216034536*^9}},ExpressionUUID->"0120b477-1a96-45be-a2d1-\
39d01911893c"],

Cell["or, more general", "Text",
 CellChangeTimes->{{3.659107027531377*^9, 
  3.659107033210885*^9}},ExpressionUUID->"8b65ce77-e8a8-481a-b777-\
a81052a01c9d"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vtors", "[", 
      RowBox[{"\[CapitalGamma]", ",", "vA", ",", "vB"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"tensor", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"torsop", "[", 
        RowBox[{"\[CapitalGamma]", ",", "vA", ",", "vB"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], ",", "0", ",", "1"}], "]"}]}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.658900230963407*^9, 3.658900254417049*^9}, 
   3.658900289864593*^9, {3.659093760434811*^9, 3.6590937661943007`*^9}, {
   3.659096939961108*^9, 3.659096966416375*^9}, 3.659106555233213*^9, {
   3.659106592255842*^9, 3.659106595287366*^9}, {3.6591066313591967`*^9, 
   3.6591067305426292`*^9}, {3.665652208006939*^9, 
   3.665652229654147*^9}},ExpressionUUID->"0051ea99-700d-49a9-8289-\
75358ace5e8f"],

Cell[TextData[{
 "The operators ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Del]", "X"], ",", 
   SubscriptBox["\[Del]", "Y"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.659103807871719*^9, 3.659103859369636*^9}, {
   3.659103937903619*^9, 3.659104032240687*^9}, {3.65910407455234*^9, 
   3.659104084583899*^9}, {3.659104190823513*^9, 3.6591041978476963`*^9}},
  ExpressionUUID->"27bf655c-37b1-4ab7-bc0c-83d11f7fc549"],
 " are absolute differentials and [X,Y] is the commutator of the vector \
fields,; equation (2) can be verified by a calculation  in coordinates. We \
confirm the equation defining the right side as the tensor"
}], "Text",
 CellChangeTimes->{{3.659107293409492*^9, 3.65910734906531*^9}, {
  3.6591074322446747`*^9, 3.659107688999165*^9}, {3.659250145477521*^9, 
  3.659250194564363*^9}},ExpressionUUID->"6430cd1f-5e33-41c9-b0d5-\
2d065acfce91"],

Cell[BoxData[
 RowBox[{"?", " ", "tensorD"}]], "Input",
 CellChangeTimes->{{3.6591081509029837`*^9, 3.659108152300983*^9}, 
   3.669112263013233*^9, 
   3.6692092640054283`*^9},ExpressionUUID->"b254e680-4837-4aff-a4b4-\
cee785b5ea12"],

Cell[BoxData[
 RowBox[{"vA", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{
  3.665663596516995*^9},ExpressionUUID->"97975377-fa90-4fdf-a23d-\
95c387f263b3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tva", "]"}], ";", 
  RowBox[{
   RowBox[{"tva", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{"vA", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"tva", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6591081987995853`*^9, 3.6591082925560427`*^9}, {
  3.665652378478601*^9, 3.665652422812846*^9}, {3.6656524618746223`*^9, 
  3.665652526431924*^9}, {3.665661030883491*^9, 3.66566103973934*^9}, {
  3.665662567850463*^9, 3.665662572234275*^9}, {3.6656626040829372`*^9, 
  3.665662672787456*^9}, {3.6656637180916862`*^9, 
  3.665663745906295*^9}},ExpressionUUID->"76edb91f-8026-4366-9a0c-\
c5d98b02ce93"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tvb", "]"}], ";", 
  RowBox[{
   RowBox[{"tvb", "[", "x__", "]"}], "  ", ":=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{"vB", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{"tvb", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6591081987995853`*^9, 3.6591082925560427`*^9}, {
   3.665652378478601*^9, 3.665652422812846*^9}, {3.6656524618746223`*^9, 
   3.665652520043202*^9}, {3.665652741047104*^9, 3.665652775678609*^9}, 
   3.665661094988652*^9, {3.665663787579384*^9, 3.665663831681738*^9}, 
   3.6656640440158453`*^9},ExpressionUUID->"c4383f21-8211-435e-b926-\
b3759b0e067e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tensorD", "[", 
    RowBox[{"\[CapitalGamma]", ",", "tvb"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"vX", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6656615150420313`*^9, 3.665661531882193*^9}, {
   3.66566156278553*^9, 3.665661572313298*^9}, 3.6656629100274277`*^9, 
   3.665662940641533*^9, {3.66566299255276*^9, 3.665663039584654*^9}, 
   3.665663862468136*^9, 3.6691122630183363`*^9, 
   3.6692092640591583`*^9},ExpressionUUID->"586d6241-8695-4804-845f-\
ec423a090472"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tab", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tvb"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
    RowBox[{"vA", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "-", 
   RowBox[{
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "tva"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
    RowBox[{"vB", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], " ", "-", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"comm", "[", 
      RowBox[{"vA", ",", "vB"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.659107714307805*^9, 3.659107735590814*^9}, {
   3.659107935343775*^9, 3.659108100333378*^9}, 3.659108316163705*^9, 
   3.6591083764673862`*^9, {3.659108415498699*^9, 3.6591084932761307`*^9}, {
   3.6591085439301977`*^9, 3.659108593272705*^9}, {3.665652790021772*^9, 
   3.665652942664812*^9}, {3.665664273119399*^9, 3.665664284966649*^9}, 
   3.665664954079068*^9, {3.6691122630495663`*^9, 3.669112263051195*^9}, {
   3.669209264138212*^9, 
   3.669209264158935*^9}},ExpressionUUID->"63717011-6059-4cfe-934e-\
8867e5e9adbf"],

Cell[BoxData[
 RowBox[{"vecTotensor", "[", 
  RowBox[{
   RowBox[{"comm", "[", 
    RowBox[{"vA", ",", "vB"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",ExpressionUUID->\
"c7bf32b8-32a3-4e11-9ec7-5114805cd65d"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.665664903192565*^9, 
  3.665664914206321*^9}},ExpressionUUID->"c0f68023-78b1-4153-b3cf-\
8601cb6c6ad3"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{
    RowBox[{"tab", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{
     RowBox[{"vtors", "[", 
      RowBox[{"\[CapitalGamma]", ",", "vA", ",", "vB"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6591086114395514`*^9, 3.6591086808324738`*^9}, {
  3.659108735313501*^9, 3.659108749592222*^9}, {3.665664328005475*^9, 
  3.665664352508877*^9}, {3.665664385213543*^9, 3.665664493691854*^9}, {
  3.6656650105896053`*^9, 
  3.665665097925288*^9}},ExpressionUUID->"04c81128-d2d0-4a4d-a0f5-\
0cae3f78e079"],

Cell["This confirms equation (2). \[FilledSquare]", "Text",
 CellChangeTimes->{{3.65934217130824*^9, 
  3.6593422105997877`*^9}},ExpressionUUID->"f8b39721-ae5e-4aef-9e75-\
2b1cee6c10f6"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.5.2.4. R(X,Y). Vector Fields", "Subsubsection",
 CellChangeTimes->{{3.659342142652657*^9, 3.65934215422849*^9}, {
   3.659513198162767*^9, 3.659513219187017*^9}, {3.659711513019806*^9, 
   3.659711517475721*^9}, 3.6696386529624643`*^9, 3.669643921325242*^9, 
   3.704183044405652*^9, 3.7041904579828577`*^9, {3.704199026066113*^9, 
   3.704199034272423*^9}, 3.7084886846429777`*^9, 
   3.708488951295842*^9},ExpressionUUID->"b78fe128-339a-4d0e-a47e-\
dbc4b375560c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.65936493316055*^9, 3.659364937130145*^9}, 
   3.65941745799722*^9, 
   3.6594185462352257`*^9},ExpressionUUID->"0482099b-7c8e-4899-9f4b-\
5e1072ed45ad"],

Cell["Now we consider the equation", "Text",
 CellChangeTimes->{{3.65910876192731*^9, 3.6591087639032516`*^9}, {
  3.659251428497086*^9, 
  3.659251445656817*^9}},ExpressionUUID->"108f6fff-4c74-4619-84c3-\
f162ee060b83"],

Cell[BoxData[
 RowBox[{
  RowBox[{"R", 
   RowBox[{"(", 
    RowBox[{"X", ",", " ", "Y"}], ")"}]}], " ", "=", " ", 
  RowBox[{
   SubscriptBox["\[Del]", "X"], 
   RowBox[{
    SubscriptBox["\[Del]", "Y"], "  ", 
    RowBox[{"-", " ", 
     RowBox[{
      SubscriptBox["\[Del]", "Y"], 
      RowBox[{
       SubscriptBox["\[Del]", 
        RowBox[{"X", " "}]], 
       RowBox[{"-", "  ", 
        RowBox[{
         SubscriptBox["\[Del]", 
          RowBox[{"[", 
           RowBox[{"X", ",", "Y"}], "]"}]], 
         "                                                                    \
", 
         RowBox[{"(", "3", ")"}]}]}]}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.659104154105115*^9, 3.6591041751115522`*^9}, {
  3.659104237160205*^9, 3.659104316446753*^9}, {3.65910436406329*^9, 
  3.6591045597247343`*^9}, {3.659251558984684*^9, 3.659251561175674*^9}, {
  3.65936573864364*^9, 
  3.659365745571465*^9}},ExpressionUUID->"56dd80e0-0fbf-4d16-89de-\
2181d479e792"],

Cell["R is interpreted as the curvature tensor RC", "Text",
 CellChangeTimes->{{3.659251665447177*^9, 3.6592517050797253`*^9}, 
   3.662622985679522*^9},ExpressionUUID->"620805f8-3731-425b-bf7c-\
9a481dbf48a6"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.6588232241899548`*^9, 3.6588232341551733`*^9}, 
   3.6626229857068863`*^9},ExpressionUUID->"98b7a4ec-a5d6-4bcb-beb8-\
fbc80b8de605"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.6588232241899548`*^9, 3.6588232341551733`*^9}, {
   3.658823351071334*^9, 3.6588233579569187`*^9}, 
   3.662622985733693*^9},ExpressionUUID->"20c9433a-cdf2-4adf-9808-\
85670c3a7b99"],

Cell["\<\
applied to the pairs of vector fields (X, Y) \[DotEqual] (v,w)  forming the \
curvature operator\
\>", "Text",
 CellChangeTimes->{{3.6592517334224787`*^9, 3.6592517919094133`*^9}, {
   3.6592518252456713`*^9, 3.6592519287250357`*^9}, 
   3.663398281953216*^9},ExpressionUUID->"82fa3612-9114-4c02-a79b-\
b890173f8715"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "RCop", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"RCop", "[", 
      RowBox[{"\[CapitalGamma]_", ",", "v_", ",", "w_"}], "]"}], "[", "x__", 
     "]"}], "[", 
    RowBox[{
     RowBox[{"{", "h_", "}"}], ",", 
     RowBox[{"{", "k_", "}"}]}], "]"}], ":=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"RC", "[", "\[CapitalGamma]", "]"}], "[", "x", "]"}], "[", 
       RowBox[{"k", ",", "h", ",", "i", ",", "j"}], "]"}], "*", 
      RowBox[{"v", "[", 
       RowBox[{"[", "i", "]"}], "]"}], "*", 
      RowBox[{"w", "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.658823715978299*^9, 3.658824041552599*^9}, {
  3.658824097920638*^9, 3.6588241077359037`*^9}, {3.65882416290366*^9, 
  3.6588242317594757`*^9}, {3.658824498032752*^9, 3.658824502973007*^9}, {
  3.6588245640845337`*^9, 3.658824570156527*^9}, {3.6588987134713793`*^9, 
  3.658898728414789*^9}, {3.662622985764929*^9, 3.6626229857679253`*^9}, {
  3.663488147620801*^9, 3.6634881532285643`*^9}, {3.704188397681901*^9, 
  3.704188454153023*^9}},ExpressionUUID->"850ec18f-3d90-459e-867a-\
1697c4b1ea60"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RCoptensor", "[", 
    RowBox[{"\[CapitalGamma]_", ",", "v_", ",", "w_"}], "]"}], "[", "x__", 
   "]"}], ":=", " ", 
  RowBox[{"tensor", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"RCop", "[", 
      RowBox[{"\[CapitalGamma]", ",", "v", ",", "w"}], "]"}], "[", "x", "]"}],
     ",", "1", ",", "1", ",", 
    RowBox[{"Length", "[", 
     RowBox[{"{", "x", "}"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.658824355942143*^9, 3.658824459661421*^9}, {
  3.6588245981249*^9, 3.658824624412126*^9}, {3.662622985796418*^9, 
  3.662622985799224*^9}},ExpressionUUID->"2d442add-9304-40df-bdc1-\
51cdb8bbd812"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCoptensor", "[", 
   RowBox[{"\[CapitalGamma]", ",", 
    RowBox[{"vA", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
    RowBox[{"vB", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6588989459875193`*^9, 3.658898996786648*^9}, {
   3.658899167707798*^9, 3.658899197130176*^9}, 3.662622985825248*^9, {
   3.665671087239037*^9, 
   3.665671099014578*^9}},ExpressionUUID->"4fdd20e2-54de-4af3-b096-\
0cedc6dc2f69"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6656509312007113`*^9, 
  3.665650964079659*^9}},ExpressionUUID->"03fd6c37-4436-485b-b0d9-\
a2e7c1793ffc"],

Cell["\<\
The arguments need not to be vector fields, they may be  tangent vectors at \
the point x given by their coordinate vectors, e.g.\
\>", "Text",
 CellChangeTimes->{{3.6592529927079773`*^9, 3.659253083035533*^9}, 
   3.6593581539072237`*^9},ExpressionUUID->"d02a9271-b479-4321-bcef-\
d0f27f2c66c1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RCop", "[", 
    RowBox[{"\[CapitalGamma]", ",", 
     RowBox[{"vec", "[", "\[Xi]", "]"}], ",", 
     RowBox[{"vec", "[", "\[Eta]", "]"}]}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "1", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6588989459875193`*^9, 3.658898996786648*^9}, {
   3.658899167707798*^9, 3.658899197130176*^9}, {3.659252600857253*^9, 
   3.659252656367446*^9}, {3.659252701759897*^9, 3.6592527220709352`*^9}, 
   3.6592531244761257`*^9, 
   3.662622985852903*^9},ExpressionUUID->"f6d8003c-2f83-40dd-918f-\
7b181e7a7fd9"],

Cell["This operator is antisymmetric in the arguments vX, vY", "Text",
 CellChangeTimes->{{3.659252309945793*^9, 3.659252390793436*^9}, 
   3.659358203138661*^9},ExpressionUUID->"ce4885f2-fcfe-4419-836f-\
df166e2b68ea"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RCoptensor", "[", 
    RowBox[{"\[CapitalGamma]", ",", 
     RowBox[{"vA", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
     RowBox[{"vB", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
  RowBox[{
   RowBox[{"RCoptensor", "[", 
    RowBox[{"\[CapitalGamma]", ",", 
     RowBox[{"vB", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
     RowBox[{"vA", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658899277924871*^9, 3.6588992809687443`*^9}, {
   3.658899311896975*^9, 3.658899343072225*^9}, {3.6588994505603848`*^9, 
   3.658899479416341*^9}, 3.6592521628518963`*^9, 3.659252252219399*^9, {
   3.662622985882412*^9, 3.662622985884593*^9}, {3.665671197629293*^9, 
   3.6656712173100557`*^9}},ExpressionUUID->"dfa7ad1e-1185-4477-b53e-\
4fd6060e4f11"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{"%", ",", 
    RowBox[{"nulltensor", "[", 
     RowBox[{"1", ",", "1", ",", "dim"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6588993680942497`*^9, 3.658899400567925*^9}, {
  3.659365014338038*^9, 
  3.6593650149381247`*^9}},ExpressionUUID->"85f3ce9b-4d03-460b-9a88-\
c1d736997f98"],

Cell["\<\
Equation (3) has to be understood as an operator equation. The operator can \
be applied to vector or covector fields. We begin with the vector field \
\>", "Text",
 CellChangeTimes->{{3.659255112850486*^9, 3.659255195985861*^9}, {
   3.659255236617799*^9, 3.659255343457149*^9}, {3.659358231842568*^9, 
   3.6593582504026957`*^9}, 
   3.6594176329150476`*^9},ExpressionUUID->"11a33e23-2969-4e5e-9a57-\
fc9585f36c04"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vz", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"z", "[", "x", "]"}], "]"}], "]"}]}], ";", 
  RowBox[{"vz", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659255388926888*^9, 3.659255435182228*^9}, {
   3.659255475464315*^9, 3.659255583799409*^9}, {3.659417658963517*^9, 
   3.6594176807713013`*^9}, 3.659417724447998*^9, 
   3.659417764915595*^9},ExpressionUUID->"74713db7-0934-4fea-8f83-\
db74cbaa51e4"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.665671291315721*^9, 
  3.665671304811445*^9}},ExpressionUUID->"d6636c39-33e2-48df-aa4d-\
4eca9c25d274"],

Cell[BoxData[
 RowBox[{"Rxyu", " ", "=", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RCoptensor", "[", 
       RowBox[{"\[CapitalGamma]", ",", 
        RowBox[{"vA", "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
        RowBox[{"vB", "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
     RowBox[{"vz", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", "1", ",", "2"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.659255667594878*^9, 3.659255761837414*^9}, {
   3.659255796406947*^9, 3.6592558289573927`*^9}, {3.659255893468808*^9, 
   3.6592559020527067`*^9}, {3.659255954613348*^9, 3.659255993811859*^9}, 
   3.659417728038761*^9, {3.65941782728343*^9, 3.659417835426923*^9}, 
   3.662622985911306*^9, {3.6656713255235367`*^9, 
   3.6656713314575233`*^9}},ExpressionUUID->"1a308e58-0a5a-43db-afa1-\
fb016e249d46"],

Cell["This is a vector field with the coordinates, e.g.", "Text",
 CellChangeTimes->{{3.6592561314258137`*^9, 3.659256157017887*^9}, 
   3.659256609240807*^9, 
   3.6594178544895973`*^9},ExpressionUUID->"733dca98-344f-45c6-9614-\
1d1d1f65b965"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Rxyu", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"RCop", "[", 
        RowBox[{"\[CapitalGamma]", ",", 
         RowBox[{"vA", "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
         RowBox[{"vB", "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", "k", "}"}], ",", 
       RowBox[{"{", "2", "}"}]}], "]"}], "*", 
     RowBox[{
      RowBox[{"z", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", "k", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6592561783808413`*^9, 3.659256224534574*^9}, {
   3.659256306549008*^9, 3.659256309214703*^9}, {3.659256357969585*^9, 
   3.65925636778538*^9}, {3.659256400514323*^9, 3.659256530903426*^9}, {
   3.659417862345158*^9, 3.6594178720415993`*^9}, {3.65941807192034*^9, 
   3.659418098775978*^9}, 3.662622985938715*^9, {3.665671397627057*^9, 
   3.6656714062349157`*^9}},ExpressionUUID->"f6570945-d38f-403a-a7dd-\
44fb27717048"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vat", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vA", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264334113666*^9, 3.6592644021953363`*^9}, {
  3.665671446041984*^9, 
  3.665671459001775*^9}},ExpressionUUID->"52790cdb-a49c-40ff-92ac-\
856a77ba8a8e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vbt", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vB", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264334113666*^9, 3.659264434762969*^9}, {
  3.665671496777626*^9, 
  3.665671515625142*^9}},ExpressionUUID->"d997fdee-9e87-4034-a833-\
b3aa2f24867f"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dyw", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "vz"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vB", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, 3.6594177296436243`*^9, 
   3.6656715530654087`*^9, 3.669112263139368*^9, 
   3.669209264266617*^9},ExpressionUUID->"62830b2d-72d5-40d7-964f-\
4b2e5f6b532e"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dxw", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "vz"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vA", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, {3.65926508434908*^9, 
   3.6592651049660807`*^9}, 3.659417733612548*^9, 3.665671569616934*^9, 
   3.669112263168626*^9, 
   3.669209264345477*^9},ExpressionUUID->"5d280c0f-5581-4a97-acef-\
76bfb1316430"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dxdyw", 
   StyleBox["[", "MSG"], 
   StyleBox[
    RowBox[{"var", "[", "x", "]"}], "MSG"], 
   StyleBox["]", "MSG"]}], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "dyw"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vA", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264974795933*^9, 3.659265052878395*^9}, 
   3.6656716043608017`*^9, 3.6691122631981564`*^9, 
   3.6692092644274197`*^9},ExpressionUUID->"8968fe1e-2d40-4480-a078-\
a6b321512593"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dydxw", 
   StyleBox["[", "MSG"], 
   StyleBox[
    RowBox[{"var", "[", "x", "]"}], "MSG"], 
   StyleBox["]", "MSG"]}], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "dxw"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vB", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264974795933*^9, 3.659265052878395*^9}, {
   3.659265138680681*^9, 3.659265184317321*^9}, 3.665671621840016*^9, 
   3.669112263234514*^9, 
   3.669209264506597*^9},ExpressionUUID->"97911a3c-d978-4623-9326-\
43b3e980f0d6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xcy", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vA", ",", "vB"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6592652223826036`*^9, 3.659265311708168*^9}, {
  3.665671640327404*^9, 3.665671651571597*^9}, {3.6656717040545387`*^9, 
  3.66567171548695*^9}},ExpressionUUID->"138ecd18-f5a0-4adc-a39b-\
e927ca29dac4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dxcyw", 
   StyleBox["[", "MSG"], 
   StyleBox[
    RowBox[{"var", "[", "x", "]"}], "MSG"], 
   StyleBox["]", "MSG"]}], "=", 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "vz"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"xcy", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659265324055689*^9, 3.659265328411195*^9}, {
   3.6592653664030647`*^9, 3.659265413706882*^9}, 3.659417736155983*^9, 
   3.6691122632665863`*^9, 
   3.6692092645853157`*^9},ExpressionUUID->"b8a552a1-fc5c-4ae6-a337-\
5f4c7515c59a"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"equi", "[", 
    RowBox[{"Rxyu", ",", 
     RowBox[{
      RowBox[{"dxdyw", 
       StyleBox["[", "MSG"], 
       StyleBox[
        RowBox[{"var", "[", "x", "]"}], "MSG"], 
       StyleBox["]", "MSG"]}], 
      StyleBox["-", "MSG"], 
      RowBox[{"dydxw", 
       StyleBox["[", "MSG"], 
       StyleBox[
        RowBox[{"var", "[", "x", "]"}], "MSG"], 
       StyleBox["]", "MSG"]}], 
      StyleBox[" ", "MSG"], 
      StyleBox["-", "MSG"], 
      RowBox[{"dxcyw", 
       StyleBox["[", "MSG"], 
       StyleBox[
        RowBox[{"var", "[", "x", "]"}], "MSG"], 
       StyleBox["]", "MSG"]}]}]}], 
    StyleBox["]", "MSG"]}], 
   StyleBox["]", "MSG"]}], 
  StyleBox["]", "MSG"]}]], "Input",
 CellChangeTimes->{{3.659265445392846*^9, 3.6592654579060183`*^9}, {
  3.6592655226492357`*^9, 3.659265533057201*^9}, {3.659265651153475*^9, 
  3.659265708928019*^9}, {3.6592657513685637`*^9, 3.659265772456167*^9}, {
  3.659359765927826*^9, 3.659359807950728*^9}, {3.659359853389882*^9, 
  3.659359883885879*^9}, {3.659361327506352*^9, 3.6593613360180693`*^9}, {
  3.659364536231144*^9, 3.659364558974835*^9}, {3.659418370325959*^9, 
  3.6594184062844343`*^9}},ExpressionUUID->"60bcbd94-9ebb-41ee-b7e8-\
e1d8e39e2334"],

Cell["\<\
After 44.616 s the result was True. Formula (3) applied to a vector field \
Z[x] yields the alternated differential\
\>", "Text",
 CellChangeTimes->{{3.6595132630105658`*^9, 3.659513345674157*^9}, {
   3.6656717895652857`*^9, 3.665671819956852*^9}, 3.669643885253875*^9, {
   3.704190378760339*^9, 3.704190379727172*^9}, {3.708488900728401*^9, 
   3.7084889016554213`*^9}},ExpressionUUID->"168858d5-6e8c-4f66-b3df-\
b32600797a91"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[Del]", "X"], 
     RowBox[{
      SubscriptBox["\[Del]", "Y"], "Z"}]}], "  ", "-", " ", 
    RowBox[{
     SubscriptBox["\[Del]", "Y"], 
     RowBox[{
      SubscriptBox["\[Del]", "X"], "Z"}]}]}], " ", "=", 
   RowBox[{
    RowBox[{"R", 
     RowBox[{"(", 
      RowBox[{"X", ",", " ", "Y"}], ")"}], " ", "Z"}], " ", "+", "  ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Del]", 
       RowBox[{"[", 
        RowBox[{"X", ",", "Y"}], "]"}]], "Z"}], "                                                               ", 
     RowBox[{"(", "4", ")"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.659104154105115*^9, 3.6591041751115522`*^9}, {
  3.659104237160205*^9, 3.659104316446753*^9}, {3.65910436406329*^9, 
  3.6591045597247343`*^9}, {3.659251558984684*^9, 3.659251561175674*^9}, {
  3.65936573864364*^9, 3.659365745571465*^9}, {3.659513423890057*^9, 
  3.659513524489314*^9}, {3.704190418175712*^9, 
  3.704190420623622*^9}},ExpressionUUID->"462496a3-7142-4873-9995-\
92baa10cace9"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.5.2.5. A Covector Field on the Sphere", "Subsubsection",
 CellChangeTimes->{{3.659342142652657*^9, 3.65934215422849*^9}, {
   3.6594185986908503`*^9, 3.659418608434808*^9}, {3.659425057416567*^9, 
   3.659425099196392*^9}, {3.6594251745638638`*^9, 3.659425197675734*^9}, 
   3.6696439095900307`*^9, 3.6696976964386044`*^9, 3.702357905411895*^9, 
   3.7023608458187532`*^9, 3.7041904708574247`*^9, 3.704193014893911*^9, {
   3.70419904594442*^9, 3.7041990611679487`*^9}, 3.70848895749642*^9, 
   3.70848905673489*^9},ExpressionUUID->"6e3f7d8a-dda7-4fa6-b40d-\
0a3ef65b1321"],

Cell["\<\
The operators in formula (3) can be applied to covector fields too, but in \
this case formula (3) is false. We give here a counterexample.\
\>", "Text",
 CellChangeTimes->{{3.659425269258844*^9, 
  3.659425444449044*^9}},ExpressionUUID->"9bdc7e2a-1d85-471d-87b5-\
87c9f8075a13"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";"}]], "Input",
 CellChangeTimes->{{3.65936493316055*^9, 3.659364937130145*^9}, 
   3.65942189582349*^9, 3.659423748368477*^9, 
   3.659425455449564*^9},ExpressionUUID->"52a17136-fc4b-4c02-a9ee-\
f320db894f66"],

Cell[TextData[{
 "The Riemann metric of the sphere of radius r is (see ",
 ButtonBox["Subsection 1.2",
  BaseStyle->"Hyperlink",
  ButtonData->"rgpos"],
 ")"
}], "Text",
 CellChangeTimes->{{3.659425487576549*^9, 3.659425557792029*^9}, {
   3.659425589247754*^9, 3.659425597215773*^9}, {3.659426000227933*^9, 
   3.65942601740382*^9}, {3.659426075887661*^9, 3.659426075888485*^9}, 
   3.659426244162064*^9},ExpressionUUID->"6c68f853-1df0-45e2-9a06-\
215cdf023ee4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "rgpos", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"rgpos", "[", "s__", "]"}], "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"RGposk", "[", "r", "]"}], "[", "s", "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"i", ",", "j"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{
  3.659420482819964*^9, 3.669695668857924*^9, {3.7041910138756332`*^9, 
   3.704191022131605*^9}, {3.704191123154315*^9, 
   3.704191141689728*^9}},ExpressionUUID->"065cdce0-44d3-4eb2-b2b7-\
f89fb96277d6"],

Cell["\<\
We suppressed the explicit argument r to have the correct form of the metric \
in the arguments of \[CapitalGamma]; it can be set in the global context.\
\>", "Text",
 CellChangeTimes->{{3.659426212145418*^9, 3.659426340624893*^9}, {
  3.704190678812557*^9, 
  3.704190679564829*^9}},ExpressionUUID->"1783a8eb-c1b3-46c1-a521-\
f9dd6e2ced0a"],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"rgpos", "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"i", ",", "j"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "//", "MatrixForm"}], 
  "MSG"]], "Input",
 CellChangeTimes->{{3.659420194007251*^9, 3.6594202482533827`*^9}, 
   3.659420496563946*^9, {3.659421912207061*^9, 3.659421979158699*^9}, 
   3.659423772344714*^9, {3.659426364161935*^9, 3.659426419328814*^9}, 
   3.669695763636733*^9, {3.7041910525887003`*^9, 
   3.704191062011307*^9}},ExpressionUUID->"1001e136-985c-4dde-8f7a-\
5f6af14ef9fa"],

Cell["Now we specify the \[CapitalGamma]-symbols:", "Text",
 CellChangeTimes->{{3.659426493431398*^9, 
  3.659426538567072*^9}},ExpressionUUID->"c3b8c4c1-f7db-46f6-b52b-\
d788ddeba3a7"],

Cell[BoxData[
 RowBox[{"?", 
  StyleBox[" ", "MSG"], 
  StyleBox["\[CapitalGamma]g", "MSG"]}]], "Input",
 CellChangeTimes->{{3.704190907208457*^9, 
  3.704190928203371*^9}},ExpressionUUID->"1512a414-5cd0-46a6-be63-\
3ba598df4349"],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalGamma]p", "[", 
     RowBox[{"k_", " ", ",", "i_", ",", " ", "j_"}], "]"}], "[", "x__", "]"}],
    ":=", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]g", "[", "rgpos", "]"}], "[", 
     RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", "x", "]"}], " "}], 
  "MSG"]], "Input",
 CellChangeTimes->{{3.65941943131767*^9, 3.6594194563241034`*^9}, {
   3.659419493052603*^9, 3.6594195051713247`*^9}, {3.659419545523901*^9, 
   3.659419575530952*^9}, {3.659419645395409*^9, 3.659419671178093*^9}, {
   3.659419922074193*^9, 3.659419951232253*^9}, {3.659420007569851*^9, 
   3.659420025983303*^9}, 3.659420341357622*^9, 3.659420377109268*^9, 
   3.659420527915909*^9, 3.6696957705640078`*^9, 3.704190791462585*^9, 
   3.704190834052762*^9, 
   3.7041908679403477`*^9},ExpressionUUID->"d9402bd3-7e71-4171-88d7-\
f9f0477d7f21"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalGamma]p", "[", 
   RowBox[{"2", ",", "1", ",", "1"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6594200867108393`*^9, 3.659420101214411*^9}, {
  3.6594216996327057`*^9, 3.659421748121655*^9}, {3.659422012229889*^9, 
  3.6594220347186737`*^9}, {3.65942384671966*^9, 
  3.659423888472198*^9}},ExpressionUUID->"5da481ec-175f-40cd-b377-\
d9da69ff448e"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.704191948494358*^9, 
  3.704191950068186*^9}},ExpressionUUID->"45aea6b1-1e87-47b1-8cea-\
3f0590e447f6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RC", "[", "\[CapitalGamma]p", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.704191917310163*^9, 3.704191942709806*^9}, {
  3.704191977563883*^9, 
  3.704192010571898*^9}},ExpressionUUID->"213fb1f7-37a0-48d6-803c-\
9d44338c7436"],

Cell["We again  consider the equation", "Text",
 CellChangeTimes->{{3.65910876192731*^9, 3.6591087639032516`*^9}, {
   3.659251428497086*^9, 3.659251445656817*^9}, {3.659426577862927*^9, 
   3.6594265902464857`*^9}, 
   3.663398934580432*^9},ExpressionUUID->"bc310361-a5d3-4076-981d-\
88c10ce3c81f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"R", 
   RowBox[{"(", 
    RowBox[{"X", ",", " ", "Y"}], ")"}]}], " ", "=", " ", 
  RowBox[{
   SubscriptBox["\[Del]", "X"], 
   RowBox[{
    SubscriptBox["\[Del]", "Y"], "  ", 
    RowBox[{"-", " ", 
     RowBox[{
      SubscriptBox["\[Del]", "Y"], 
      RowBox[{
       SubscriptBox["\[Del]", 
        RowBox[{"X", " "}]], 
       RowBox[{"-", "  ", 
        RowBox[{
         SubscriptBox["\[Del]", 
          RowBox[{"[", 
           RowBox[{"X", ",", "Y"}], "]"}]], 
         "                                                                    \
", 
         RowBox[{"(", "3", ")"}]}]}]}]}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.659104154105115*^9, 3.6591041751115522`*^9}, {
  3.659104237160205*^9, 3.659104316446753*^9}, {3.65910436406329*^9, 
  3.6591045597247343`*^9}, {3.659251558984684*^9, 3.659251561175674*^9}, {
  3.65936573864364*^9, 
  3.659365745571465*^9}},ExpressionUUID->"0a19aff3-6841-4389-82b0-\
77c9ebf03486"],

Cell["R is interpreted as the curvature tensor RC", "Text",
 CellChangeTimes->{{3.659251665447177*^9, 3.6592517050797253`*^9}, 
   3.662622985981196*^9},ExpressionUUID->"b013b0e5-d6c4-41d3-bb24-\
01ec7f23df5a"],

Cell["\<\
applied onto the pairs of vector fields (X, Y) \[DotEqual] (v,w)  forming the \
curvature operator as in the last SubSubsection \
\[OpenCurlyDoubleQuote]R(X,Y). Vector Fields\[CloseCurlyDoubleQuote], for \
example\
\>", "Text",
 CellChangeTimes->{{3.6592517334224787`*^9, 3.6592517919094133`*^9}, {
  3.6592518252456713`*^9, 3.6592519287250357`*^9}, {3.704193091445538*^9, 
  3.704193102269123*^9}, {3.7041931481409597`*^9, 3.704193174460878*^9}, {
  3.704193212262103*^9, 3.7041932246447*^9}, {3.704193257004529*^9, 
  3.704193264068472*^9}},ExpressionUUID->"9690a441-3af3-4ac0-b42b-\
40fc4d127ad9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"vX", ",", "vY"}], "]"}], ";", 
  RowBox[{
   RowBox[{"vX", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Xi]", "[", "x", "]"}], "]"}]}], ";", " ", 
  RowBox[{
   RowBox[{"vY", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Eta]", "[", "x", "]"}], "]"}]}], ";", " ", 
  RowBox[{"vY", "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.669696173366599*^9, 3.66969637878642*^9}, {
  3.6696964165696*^9, 3.669696417593577*^9}, {3.669696449874342*^9, 
  3.66969646654636*^9}},ExpressionUUID->"cc3e0535-b461-49eb-8422-\
753df91cc121"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RCop", "[", 
    RowBox[{"\[CapitalGamma]p", ",", 
     RowBox[{"vX", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"vY", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7041924531452713`*^9, 
  3.70419254738478*^9}},ExpressionUUID->"2aaf7fdc-1bd8-440b-ba48-\
bd8ccc646025"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RCoptensor", "[", 
    RowBox[{"\[CapitalGamma]p", ",", 
     RowBox[{"vX", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{"vY", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7041924531452713`*^9, 3.70419254738478*^9}, {
  3.704192609000733*^9, 
  3.704192612648582*^9}},ExpressionUUID->"cb6ef28b-7426-456e-a756-\
15a862ea69ef"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", "%%"}]], "Input",
 CellChangeTimes->{{3.704193307838914*^9, 
  3.704193311292338*^9}},ExpressionUUID->"c887f94b-1606-446f-a144-\
1ae1e208aa9a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCoptensor", "[", 
   RowBox[{"\[CapitalGamma]p", ",", 
    RowBox[{"vX", "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], ",", 
    RowBox[{"vY", "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}]}], "]"}], "[", 
  RowBox[{"u", ",", " ", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.6588989459875193`*^9, 3.658898996786648*^9}, {
   3.658899167707798*^9, 3.658899197130176*^9}, 3.659421790952231*^9, {
   3.6594267034755163`*^9, 3.659426710984194*^9}, 3.662622986042741*^9, {
   3.7041922088368464`*^9, 
   3.704192245635564*^9}},ExpressionUUID->"cae193cd-b575-4f69-9c7e-\
1356fb722ee7"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"%", "[", 
     RowBox[{
      RowBox[{"{", "i", "}"}], ",", 
      RowBox[{"{", "j", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "dim"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.659422093880991*^9, 3.65942218037287*^9}, 
   3.659423964567515*^9, {3.669696525857424*^9, 
   3.6696965388012543`*^9}},ExpressionUUID->"95427180-f8f9-4b6d-af21-\
bee77beca8b1"],

Cell["\<\
The arguments need not to be vector fields, they may be  tangent vectors at \
the point x given by their coordinate vectors, e.g.\
\>", "Text",
 CellChangeTimes->{{3.6592529927079773`*^9, 3.659253083035533*^9}, 
   3.6593581539072237`*^9},ExpressionUUID->"251cea54-603a-47f2-b923-\
f3f56d80b67a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RCop", "[", 
    RowBox[{"\[CapitalGamma]p", ",", 
     RowBox[{"vec", "[", "a", "]"}], ",", 
     RowBox[{"vec", "[", "b", "]"}]}], "]"}], "[", 
   RowBox[{"u", ",", " ", "v"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6588989459875193`*^9, 3.658898996786648*^9}, {
   3.658899167707798*^9, 3.658899197130176*^9}, {3.659252600857253*^9, 
   3.659252656367446*^9}, {3.659252701759897*^9, 3.6592527220709352`*^9}, 
   3.6592531244761257`*^9, 3.659424015649536*^9, 3.659426747339239*^9, {
   3.659426827860216*^9, 3.659426909516485*^9}, 
   3.662622986072082*^9},ExpressionUUID->"ce8ab5b7-5702-40da-b3bc-\
7fab6578fd40"],

Cell["This operator is antisymmetric in the arguments vX, vY", "Text",
 CellChangeTimes->{{3.659252309945793*^9, 3.659252390793436*^9}, 
   3.659358203138661*^9},ExpressionUUID->"9911a75a-1edd-460d-9b42-\
1a6605d21e62"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RCoptensor", "[", 
    RowBox[{"\[CapitalGamma]p", ",", 
     RowBox[{"vX", "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], ",", 
     RowBox[{"vY", "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}]}], "]"}], "[", 
   RowBox[{"u", ",", " ", "v"}], "]"}], "+", 
  RowBox[{
   RowBox[{"RCoptensor", "[", 
    RowBox[{"\[CapitalGamma]p", ",", 
     RowBox[{"vY", "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], ",", 
     RowBox[{"vX", "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}]}], "]"}], "[", 
   RowBox[{"u", ",", " ", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.658899277924871*^9, 3.6588992809687443`*^9}, {
   3.658899311896975*^9, 3.658899343072225*^9}, {3.6588994505603848`*^9, 
   3.658899479416341*^9}, 3.6592521628518963`*^9, 3.659252252219399*^9, {
   3.6594222804679327`*^9, 3.659422285211774*^9}, {3.659426747356247*^9, 
   3.659426747367692*^9}, {3.662622986101959*^9, 
   3.66262298610395*^9}},ExpressionUUID->"02d04763-0ee4-486b-a9b0-\
35629f962437"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{"%", ",", 
    RowBox[{"nulltensor", "[", 
     RowBox[{"1", ",", "1", ",", "dim"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6588993680942497`*^9, 3.658899400567925*^9}, {
  3.659365014338038*^9, 
  3.6593650149381247`*^9}},ExpressionUUID->"885bfc6a-daac-48ad-b12e-\
cfb6204bfbcf"],

Cell["\<\
Equation (3) has to be understood as an operator equation. The operator can \
be applied to vector or covector fields. We consider now the covector field \
\>", "Text",
 CellChangeTimes->{{3.659255112850486*^9, 3.659255195985861*^9}, {
   3.659255236617799*^9, 3.659255343457149*^9}, {3.659358231842568*^9, 
   3.6593582504026957`*^9}, {3.659422342748559*^9, 3.659422349266927*^9}, 
   3.65942694154784*^9},ExpressionUUID->"293b5f25-d903-4024-b1b4-\
829832de9b0d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ww", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"w", "[", "x", "]"}], "]"}], "]"}]}], ";", 
  RowBox[{"ww", "[", 
   RowBox[{"u", ",", " ", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659255388926888*^9, 3.659255435182228*^9}, {
   3.659255475464315*^9, 3.659255583799409*^9}, 3.65942674738416*^9, 
   3.669697782382737*^9},ExpressionUUID->"9cc829e9-3ef2-4447-b8b9-\
0bcaa6f2563a"],

Cell[BoxData[
 RowBox[{"Rxyu", " ", "=", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RCoptensor", "[", 
       RowBox[{"\[CapitalGamma]p", ",", 
        RowBox[{"vX", "[", 
         RowBox[{"u", ",", " ", "v"}], "]"}], ",", 
        RowBox[{"vY", "[", 
         RowBox[{"u", ",", " ", "v"}], "]"}]}], "]"}], "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}], "\[CircleTimes]", 
     RowBox[{"ww", "[", 
      RowBox[{"u", ",", " ", "v"}], "]"}]}], ",", "2", ",", "1"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.659255667594878*^9, 3.659255761837414*^9}, {
   3.659255796406947*^9, 3.6592558289573927`*^9}, {3.659255893468808*^9, 
   3.6592559020527067`*^9}, {3.659255954613348*^9, 3.659255993811859*^9}, 
   3.659424109831002*^9, {3.6594267473948183`*^9, 3.659426747400433*^9}, 
   3.662622986127611*^9},ExpressionUUID->"336d038a-e183-4154-a129-\
b5504c1a34c8"],

Cell["This is a covector field with the coordinates, e.g.", "Text",
 CellChangeTimes->{{3.6592561314258137`*^9, 3.659256157017887*^9}, 
   3.659256609240807*^9},ExpressionUUID->"77f4a822-9ecc-4209-b967-\
72ec654f71f1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Rxyu", "[", 
   RowBox[{
    RowBox[{"{", "2", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}], "\[Equal]", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"RCop", "[", 
        RowBox[{"\[CapitalGamma]p", ",", 
         RowBox[{"vX", "[", 
          RowBox[{"u", ",", " ", "v"}], "]"}], ",", 
         RowBox[{"vY", "[", 
          RowBox[{"u", ",", " ", "v"}], "]"}]}], "]"}], "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "[", 
      RowBox[{
       RowBox[{"{", "2", "}"}], ",", 
       RowBox[{"{", "k", "}"}]}], "]"}], "*", 
     RowBox[{
      RowBox[{"w", "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "[", "k", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "dim"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6592561783808413`*^9, 3.659256224534574*^9}, {
   3.659256306549008*^9, 3.659256309214703*^9}, {3.659256357969585*^9, 
   3.65925636778538*^9}, {3.659256400514323*^9, 3.659256530903426*^9}, 
   3.6594241343339357`*^9, {3.659426747407099*^9, 3.659426747413229*^9}, 
   3.662622986159295*^9, 
   3.6696978401505947`*^9},ExpressionUUID->"7786d576-d0d6-4036-8516-\
29ec98f37e17"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vxt", "[", 
   RowBox[{"u", ",", " ", "v"}], "]"}], " ", "=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vX", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264334113666*^9, 3.6592644021953363`*^9}, {
  3.659426747420291*^9, 
  3.6594267474222383`*^9}},ExpressionUUID->"b4b7c65b-008e-456a-84fc-\
0bbaabbd4e8b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vyt", "[", 
   RowBox[{"u", ",", " ", "v"}], "]"}], " ", "=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{"vY", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264334113666*^9, 3.659264434762969*^9}, {
  3.6594267474274387`*^9, 
  3.659426747429378*^9}},ExpressionUUID->"f046d043-9c88-453e-9b1d-\
336ee82512d9"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dyw", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]p", ",", "ww"}], "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vY", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, 3.659424164880913*^9, {
   3.659426747435482*^9, 3.65942674743985*^9}, 3.6691122633112288`*^9, 
   3.669209264664373*^9},ExpressionUUID->"0c58baba-7874-4a01-b97d-\
ff93a0e96a8d"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dxw", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]p", ",", "ww"}], "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vX", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, {3.65926508434908*^9, 
   3.6592651049660807`*^9}, 3.659424168711389*^9, {3.6594267474456367`*^9, 
   3.659426747449214*^9}, 3.6691122633498573`*^9, 
   3.6692092647432833`*^9},ExpressionUUID->"646afc17-aa33-43f6-8deb-\
2ce9cf4426e4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dxdyw", 
   StyleBox["[", "MSG"], 
   StyleBox[
    RowBox[{"u", ",", " ", "v"}], "MSG"], 
   StyleBox["]", "MSG"]}], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]p", ",", "dyw"}], "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vX", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264974795933*^9, 3.659265052878395*^9}, 
   3.659424171406163*^9, {3.659426747455312*^9, 3.659426747459053*^9}, 
   3.6691122633879232`*^9, 
   3.669209264822031*^9},ExpressionUUID->"e1f526f3-e4e9-48b5-bece-\
39dd0cf4c913"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dydxw", 
   StyleBox["[", "MSG"], 
   StyleBox[
    RowBox[{"u", ",", " ", "v"}], "MSG"], 
   StyleBox["]", "MSG"]}], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]p", ",", "dxw"}], "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vY", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659264974795933*^9, 3.659265052878395*^9}, {
   3.659265138680681*^9, 3.659265184317321*^9}, 3.659424174408784*^9, {
   3.659426747465856*^9, 3.659426747469545*^9}, 3.669112263421433*^9, 
   3.6692092649010277`*^9},ExpressionUUID->"963827b4-cb06-40e5-a5f8-\
c16864d5e0b4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xcy", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], " ", "=", " ", 
   RowBox[{
    RowBox[{"comm", "[", 
     RowBox[{"vX", ",", "vY"}], "]"}], "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6592652223826036`*^9, 3.659265311708168*^9}, {
  3.659426747476467*^9, 
  3.6594267474789467`*^9}},ExpressionUUID->"1beec8bc-ece5-4aec-839b-\
cf203105e7a9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dxcyw", 
   StyleBox["[", "MSG"], 
   StyleBox[
    RowBox[{"u", ",", " ", "v"}], "MSG"], 
   StyleBox["]", "MSG"]}], "=", 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]p", ",", "ww"}], "]"}], "[", 
     RowBox[{"u", ",", " ", "v"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"xcy", "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659265324055689*^9, 3.659265328411195*^9}, {
   3.6592653664030647`*^9, 3.659265413706882*^9}, 3.659424177951645*^9, {
   3.659426747485476*^9, 3.659426747489439*^9}, 3.669112263451515*^9, 
   3.669209264980102*^9},ExpressionUUID->"b2d620cf-8acb-4065-bf41-\
86b104e3c39f"],

Cell["\<\
If formula (3) would be valid for covector fields too, then the coordinates \
of the difference tensor should be zero. We calculate\
\>", "Text",
 CellChangeTimes->{{3.659427171088908*^9, 3.659427282376639*^9}, {
  3.6633997619250193`*^9, 
  3.6633997729329433`*^9}},ExpressionUUID->"5c08c79f-0b40-466a-9f6c-\
ba9e34418f57"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"coords", "[", 
    RowBox[{"Rxyu", "-", 
     RowBox[{"dxdyw", 
      StyleBox["[", "MSG"], 
      StyleBox[
       RowBox[{"u", ",", " ", "v"}], "MSG"], 
      StyleBox["]", "MSG"]}], 
     StyleBox["+", "MSG"], 
     RowBox[{"dydxw", 
      StyleBox["[", "MSG"], 
      StyleBox[
       RowBox[{"u", ",", " ", "v"}], "MSG"], 
      StyleBox["]", "MSG"]}], 
     StyleBox[" ", "MSG"], 
     StyleBox["+", "MSG"], 
     RowBox[{"dxcyw", 
      StyleBox["[", "MSG"], 
      StyleBox[
       RowBox[{"u", ",", " ", "v"}], "MSG"], 
      StyleBox["]", "MSG"]}]}], 
    StyleBox["]", "MSG"]}], 
   StyleBox["]", "MSG"]}], 
  StyleBox["]", "MSG"]}]], "Input",
 CellChangeTimes->{{3.659265445392846*^9, 3.6592654579060183`*^9}, {
  3.6592655226492357`*^9, 3.659265533057201*^9}, {3.659265651153475*^9, 
  3.659265708928019*^9}, {3.6592657513685637`*^9, 3.659265772456167*^9}, {
  3.659359765927826*^9, 3.659359807950728*^9}, {3.659359853389882*^9, 
  3.659359883885879*^9}, {3.659361327506352*^9, 3.6593613360180693`*^9}, {
  3.659364536231144*^9, 3.659364558974835*^9}, {3.659426747495976*^9, 
  3.659426747500251*^9}},ExpressionUUID->"1f38dcad-338d-4bdf-972a-\
649c2e7357f0"],

Cell[BoxData[
 RowBox[{"err", "=", 
  RowBox[{"%", "[", 
   RowBox[{"[", "2", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659424529330638*^9, 
  3.659424540834529*^9}},ExpressionUUID->"eab5a534-184c-4bce-9173-\
1557b7cbf099"],

Cell["This expression isn\[CloseCurlyQuote]t the null matrix, in general:", \
"Text",
 CellChangeTimes->{{3.6594272991042557`*^9, 3.659427341472053*^9}, {
  3.659429423344705*^9, 
  3.659429426248145*^9}},ExpressionUUID->"6b346247-db4e-4fb4-b505-\
bca0035b3dba"],

Cell[BoxData[
 RowBox[{"err", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Eta]", "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "[", "2", "]"}], "\[Rule]", "1"}],
     ",", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Xi]", "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "[", "1", "]"}], "\[Rule]", "1"}],
     ",", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Eta]", "[", 
       RowBox[{"u", ",", " ", "v"}], "]"}], "[", "1", "]"}], "\[Rule]", 
     "0"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.659424559720992*^9, 3.6594245727382917`*^9}, {
  3.659424636641437*^9, 3.6594246934888678`*^9}, {3.659424724393586*^9, 
  3.6594247384008207`*^9}, {3.65942710244902*^9, 
  3.659427124522439*^9}},ExpressionUUID->"2a3ddfc1-9cef-49ce-bdc0-\
8a875c9aeb36"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.659427359606104*^9, 
  3.65942735960776*^9}},ExpressionUUID->"13021cc8-e50c-432c-a282-\
f673bf81d5c0"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.5.2.6. Covector Fields", "Subsubsection",
 CellChangeTimes->{{3.659512627511612*^9, 3.6595126567270727`*^9}, 
   3.669697706894676*^9, 3.669699975029106*^9, {3.7041990826241283`*^9, 
   3.704199108343731*^9}, 3.708489063055708*^9, 
   3.708489548299189*^9},ExpressionUUID->"8e7aab53-4f1e-4074-a6e9-\
3e68a95408e5"],

Cell["\<\
In this example we derive a formula for the alternated absolute differential \
of a covector field\
\>", "Text",
 CellChangeTimes->{{3.65951368952698*^9, 3.65951374628666*^9}, 
   3.659687083862915*^9, 
   3.669699120891444*^9},ExpressionUUID->"f46e4d2d-19da-45a0-9513-\
1b4224166b7f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{
   RowBox[{"ww", "[", "x__", "]"}], " ", ":=", " ", 
   RowBox[{"covecTotensor", "[", 
    RowBox[{"vec", "[", 
     RowBox[{"u", "[", "x", "]"}], "]"}], "]"}]}], ";", 
  RowBox[{"ww", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659513765914359*^9, 
  3.65951390237361*^9}},ExpressionUUID->"2a5ac0ed-608d-4836-9b13-\
16b00cbc0b72"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.659513914575933*^9, 
  3.659513920381404*^9}},ExpressionUUID->"e68bfa13-1685-4088-8274-\
bb6cce22e749"],

Cell["\<\
Calculating in local coordinates one obtains  - in traditional notation -\
\>", "Text",
 CellChangeTimes->{{3.659514171514674*^9, 
  3.6595142145225163`*^9}},ExpressionUUID->"ee019aff-cb2d-40c9-8278-\
3db7d9e671b2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["absD", "X"], 
    SubscriptBox["absD", "Y"], 
    SubscriptBox["u", "i"]}], "-", " ", 
   RowBox[{
    SubscriptBox["absD", "Y"], 
    SubscriptBox["absD", "X"], 
    SubscriptBox["u", "i"]}]}], " ", "=", "   ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", " ", 
     SubscriptBox["u", "l"]}], 
    SubsuperscriptBox[
     RowBox[{"R", "[", 
      RowBox[{"X", ",", "Y"}], "]"}], "i", 
     RowBox[{"   ", "l"}]]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Del]", "m"], 
     SubscriptBox["u", "i"]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"[", 
       RowBox[{"X", ",", "Y"}], "]"}], ")"}], "m"]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.659514321179336*^9, 3.6595145737358427`*^9}, {
  3.6595146145517473`*^9, 3.659514696734583*^9}, {3.6597093118556023`*^9, 
  3.659709404438818*^9}, {3.659709457486343*^9, 3.659709485006062*^9}, {
  3.659709562781431*^9, 3.659709728612014*^9}, {3.659709766315564*^9, 
  3.65970984589098*^9}, {3.669112263489826*^9, 
  3.669112263494369*^9}},ExpressionUUID->"c9c61513-f476-4b0e-bc2b-\
f24434fb8e4a"],

Cell[TextData[{
 "Here ",
 Cell[BoxData[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"[", 
     RowBox[{"X", ",", "Y"}], "]"}], ")"}], "m"]], "Input",
  CellChangeTimes->{{3.659514321179336*^9, 3.6595145737358427`*^9}, {
   3.6595146145517473`*^9, 3.659514696734583*^9}},ExpressionUUID->
  "50675df9-75c3-4dae-9bde-0a763e7d7e33"],
 " are the coordinates of the commutator of the vector fields X,Y. This leads \
to the tensor formula"
}], "Text",
 CellChangeTimes->{{3.659514720301921*^9, 3.659514828781032*^9}, {
  3.659709908941629*^9, 
  3.659709963313652*^9}},ExpressionUUID->"ee414ecf-5ad9-4fe9-b13d-\
6b0ad3629766"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "X"], 
    RowBox[{
     SubscriptBox["\[Del]", "Y"], "ww"}]}], "-", " ", 
   RowBox[{
    SubscriptBox["\[Del]", "Y"], 
    RowBox[{
     SubscriptBox["\[Del]", "X"], "ww"}]}]}], "=", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "R"}], 
    RowBox[{"(", 
     RowBox[{"X", ",", "Y"}], ")"}], " ", "ww"}], "  ", "+", " ", 
   RowBox[{
    SubscriptBox["absD", 
     RowBox[{"[", 
      RowBox[{"X", ",", "Y"}], "]"}]], "ww", "                             ", 
    RowBox[{"(", "5", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.659514926452684*^9, 3.659515030355571*^9}, {
   3.659515061803668*^9, 3.659515161202573*^9}, {3.659515197194545*^9, 
   3.659515207258247*^9}, {3.659600185510453*^9, 3.659600190918429*^9}, {
   3.659710015585878*^9, 3.659710085089129*^9}, 
   3.669112263524479*^9},ExpressionUUID->"bc5cc6cd-d6e7-41f1-b152-\
cedd21cbe444"],

Cell["\<\
which we are going too confirm now. Consider again the vector fields\
\>", "Text",
 CellChangeTimes->{{3.6595178797332573`*^9, 3.6595179377621717`*^9}, {
  3.659710109952703*^9, 
  3.6597101323120747`*^9}},ExpressionUUID->"6b08b7f5-2b2c-4463-b4c1-\
d24d12b57206"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vX", "[", "x__", "]"}], ":=", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Xi]", "[", "x", "]"}], "]"}]}], ";", " ", 
  RowBox[{
   RowBox[{"vxt", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{"vX", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.658823430721339*^9, 3.6588235923806543`*^9}, 
   3.6595140348043003`*^9, {3.659517954186281*^9, 3.659518038441491*^9}, 
   3.659518140377281*^9},ExpressionUUID->"0b55490f-9970-4708-a6da-\
b559f741d616"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vY", "[", "x__", "]"}], ":=", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Eta]", "[", "x", "]"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"vyt", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "=", " ", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{"vY", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.658823430721339*^9, 3.658823657739585*^9}, 
   3.659514042644701*^9, {3.659518055017819*^9, 3.659518117585437*^9}, {
   3.659518186776147*^9, 
   3.65951820010375*^9}},ExpressionUUID->"1f32bced-7a87-4f71-ac57-\
2772e86faeb1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"xcy", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "=", " ", 
  RowBox[{
   RowBox[{"comm", "[", 
    RowBox[{"vX", ",", "vY"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6596003227612534`*^9, 
  3.659600370960105*^9}},ExpressionUUID->"67fd6a89-05e4-40e0-887a-\
f5075efa16ef"],

Cell["The last term of (5)  is the tensor of type (1,0)", "Text",
 CellChangeTimes->{{3.6597102409588833`*^9, 3.659710298702374*^9}, {
  3.669699319073838*^9, 
  3.669699320962009*^9}},ExpressionUUID->"34e56bcd-3eae-46b1-953c-\
0267fcaf7111"],

Cell[BoxData[
 RowBox[{"xcyw", " ", "=", " ", 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "ww"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"xcy", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659600795288459*^9, 3.659600851918037*^9}, 
   3.669112263531678*^9, 
   3.669209265058824*^9},ExpressionUUID->"2d4149e4-345d-4568-a36c-\
b8d8295b4cd2"],

Cell["Now we calculate the left side of (5)", "Text",
 CellChangeTimes->{{3.659710321407304*^9, 
  3.6597103802130947`*^9}},ExpressionUUID->"8842c0ff-a903-42b1-babd-\
a80612aa5252"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dyw", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "ww"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vY", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, 3.659424164880913*^9, {
   3.659426747435482*^9, 3.65942674743985*^9}, {3.659518598390316*^9, 
   3.65951863293371*^9}, 3.669112263563777*^9, 
   3.6692092651405783`*^9},ExpressionUUID->"afb0c7e2-d396-48f7-8b2d-\
4f4e540d1adb"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dxyw", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "dyw"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vX", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, 3.659424164880913*^9, {
   3.659426747435482*^9, 3.65942674743985*^9}, {3.659518598390316*^9, 
   3.65951863293371*^9}, {3.659518686557281*^9, 3.659518757435994*^9}, 
   3.6691122635930643`*^9, 
   3.6692092652193193`*^9},ExpressionUUID->"ee850ca5-c176-4249-a725-\
17435ba01f4c"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dxw", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "ww"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vX", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, 3.659424164880913*^9, {
   3.659426747435482*^9, 3.65942674743985*^9}, {3.659518598390316*^9, 
   3.659518656352779*^9}, {3.659518995050178*^9, 3.659519003178692*^9}, 
   3.6691122636206007`*^9, 
   3.669209265298835*^9},ExpressionUUID->"538566a8-09e3-4989-923e-\
044c8f28daf9"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"dyxw", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
  StyleBox["=", "MSG"], 
  RowBox[{
   StyleBox[
    RowBox[{
     RowBox[{"tensorD", "[", 
      RowBox[{"\[CapitalGamma]", ",", "dxw"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "MSG"], 
   StyleBox["[", "MSG"], 
   RowBox[{"vY", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.659264263797426*^9, {3.659264512139284*^9, 3.6592646139215403`*^9}, 
   3.659264651289257*^9, {3.65926482505719*^9, 3.659264825327557*^9}, {
   3.659264905840147*^9, 3.65926495486395*^9}, 3.659424164880913*^9, {
   3.659426747435482*^9, 3.65942674743985*^9}, {3.659518598390316*^9, 
   3.659518656352779*^9}, 3.659518956598325*^9, {3.659519018161676*^9, 
   3.6595190324576607`*^9}, 3.669112263644099*^9, 
   3.6692092653775597`*^9},ExpressionUUID->"c8a02f95-d00e-465b-ad74-\
b710cfeb595f"],

Cell["The following difference is the wanted tensor", "Text",
 CellChangeTimes->{{3.659710463196649*^9, 
  3.6597104917246943`*^9}},ExpressionUUID->"da88a6bb-385d-4594-b174-\
3f5217d3096a"],

Cell[BoxData[
 StyleBox[
  RowBox[{"dxydiffww", "=", 
   RowBox[{
    RowBox[{"dxyw", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
    RowBox[{"dyxw", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}]}]}], "MSG"]], "Input",
 CellChangeTimes->{{3.659519161782003*^9, 3.659519180637156*^9}, {
  3.659519213028234*^9, 3.659519216412208*^9}, {3.6595339537224483`*^9, 
  3.659533964002022*^9}},ExpressionUUID->"20371cf7-c4a2-46a0-b6bc-\
00e6b6e5a731"],

Cell["\<\
It remains to calculate the curvature operator applied to the  tensor ww of \
type (1,0):\
\>", "Text",
 CellChangeTimes->{{3.6597105166363983`*^9, 
  3.659710579939959*^9}},ExpressionUUID->"ae44ded9-1d2b-4453-8adf-\
9b2f31862542"],

Cell[BoxData[
 RowBox[{"xyR", "=", " ", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
     RowBox[{"vxt", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
     RowBox[{"vyt", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"3", ",", "3"}], "}"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6595196984632072`*^9, 3.659519788544404*^9}, {
   3.6595198196141148`*^9, 3.659519830373617*^9}, {3.659519860501526*^9, 
   3.659519865316757*^9}, {3.659519904756442*^9, 3.659519927514844*^9}, 
   3.659710587636547*^9, 
   3.662622986191092*^9},ExpressionUUID->"a524c4aa-c71f-4468-a065-\
53877e27ca1b"],

Cell[BoxData[
 RowBox[{"xywR", " ", "=", " ", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"ww", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", "xyR"}], ",", 
    " ", "1", ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.659519951620454*^9, 
  3.659520040407152*^9}},ExpressionUUID->"d31de32d-3439-4995-ad1e-\
307b44966b1c"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Flatten", "[", 
    RowBox[{"coords", "[", 
     RowBox[{"dxydiffww", "+", "xywR", " ", "-", "xcyw"}], "]"}], "]"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6595200927111692`*^9, 3.659520165067254*^9}, {
   3.659520423588647*^9, 3.659520438819585*^9}, {3.659520584831602*^9, 
   3.6595205989519672`*^9}, 3.659534362401775*^9, {3.659534576295855*^9, 
   3.659534603207088*^9}, {3.659534653822751*^9, 3.659534665606421*^9}, {
   3.659537327881111*^9, 3.6595373468802*^9}, 3.659537508831812*^9, {
   3.659537670662133*^9, 3.659537761069228*^9}, {3.659600911965783*^9, 
   3.659600982869115*^9}, {3.6596010615745993`*^9, 3.65960106330822*^9}, {
   3.659601538937314*^9, 3.659601592207266*^9}, 
   3.6596836498752604`*^9},ExpressionUUID->"fbfa7637-bf64-4bd9-97dd-\
0fdd6084ce99"],

Cell["The result {0,0,0} obtained after 0.652 s.", "Text",
 CellChangeTimes->{{3.669699785382208*^9, 3.669699830765746*^9}, {
  3.704193758136997*^9, 3.704193758368804*^9}, {3.708489436691271*^9, 
  3.708489437627186*^9}},ExpressionUUID->"7b54953a-d852-4b6c-9d35-\
a389e9c157c1"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"equi", "[", 
    RowBox[{"dxydiffww", ",", 
     RowBox[{
      RowBox[{"-", "xywR"}], " ", "+", "xcyw"}]}], "]"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6595200927111692`*^9, 3.659520165067254*^9}, {
   3.659520423588647*^9, 3.659520438819585*^9}, {3.659520584831602*^9, 
   3.6595205989519672`*^9}, 3.659534362401775*^9, {3.659534576295855*^9, 
   3.659534603207088*^9}, {3.659534653822751*^9, 3.659534665606421*^9}, {
   3.659537327881111*^9, 3.6595373468802*^9}, 3.659537508831812*^9, {
   3.659537670662133*^9, 3.659537761069228*^9}, {3.659600911965783*^9, 
   3.659600982869115*^9}, {3.6596010615745993`*^9, 3.65960106330822*^9}, {
   3.659601538937314*^9, 3.659601592207266*^9}, 3.6596836498752604`*^9, {
   3.6597106445187607`*^9, 
   3.6597106990990667`*^9}},ExpressionUUID->"dedb2292-1299-4465-bc1b-\
c9f4372ff509"],

Cell["The result True obtained after 228.204 s.", "Text",
 CellChangeTimes->{{3.669699785382208*^9, 3.669699860557535*^9}, 
   3.6696998995667152`*^9, 3.702360809051827*^9, {3.7041938428980303`*^9, 
   3.7041938446489162`*^9}, 
   3.70848947306833*^9},ExpressionUUID->"1aef4976-53e7-4177-8d48-\
ca6382519dda"],

Cell["\<\
Both results confirm the statement (5). One realizes that it is often faster \
to calculate the difference of two equivalent tensors than to check their \
equivalence using the function equi. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.6597109772801323`*^9, 3.659710999688129*^9}, {
   3.659711310589732*^9, 3.659711443180306*^9}, 3.663400067825601*^9, {
   3.669699923837304*^9, 
   3.669699946476959*^9}},ExpressionUUID->"ffa3d755-ef25-47a2-8984-\
ee9c2fe354c7"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.5.3. The Ricci Tensor", "Subsubsection",
 CellChangeTimes->{{3.6656720097704067`*^9, 3.665672026402179*^9}, 
   3.669699982485704*^9, 3.669700592119833*^9, 3.7023608572353277`*^9, 
   3.702361884177775*^9},ExpressionUUID->"ce5ba630-4c77-431c-81c4-\
160c87c0dea8"],

Cell["\<\
The curvature tensor of a linear connection  can be considered as a field of \
linear maps of the tangent spaces of the manifold into themselves depending \
on two vector fields A,B:\
\>", "Text",
 CellChangeTimes->{{3.665672076863224*^9, 3.665672196536072*^9}, 
   3.665672861914673*^9, {3.665720957399267*^9, 3.665720980150449*^9}, {
   3.665721023201576*^9, 3.665721054165997*^9}, {3.665721084285707*^9, 
   3.665721100429431*^9}, {3.6657211429011097`*^9, 
   3.665721163996994*^9}},ExpressionUUID->"ccb08f6f-a252-4aa1-96e8-\
49a693e12006"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"lm", "[", 
    RowBox[{"A", ",", "B"}], "]"}], ":", " ", 
   RowBox[{"X", "\[Element]", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["T", "x"], 
      SuperscriptBox["M", "n"]}], " ", "\[RightTeeArrow]", " ", "Y"}]}]}], 
  " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"R", "[", 
     RowBox[{"X", ",", "A"}], "]"}], "[", "B", "]"}], " ", "\[Element]", " ", 
   RowBox[{
    SubscriptBox["T", "x"], 
    SuperscriptBox["M", "n"], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.665672210177058*^9, 3.665672230567824*^9}, {
  3.6656722707433243`*^9, 3.665672417118915*^9}, {3.66572126489259*^9, 
  3.665721301196493*^9}, {3.665721387907468*^9, 3.6657214020913343`*^9}, {
  3.665723236219602*^9, 
  3.66572325602732*^9}},ExpressionUUID->"e1a8982e-48a3-4da4-84c6-\
fffc5ab582fd"],

Cell["\<\
In traditional coordinate notation the map lm[A,B]  can be written as\
\>", "Text",
 CellChangeTimes->{{3.665723274498475*^9, 3.665723285098392*^9}, {
  3.665724286232813*^9, 
  3.665724370344304*^9}},ExpressionUUID->"7e6e30dc-0634-4c80-96a3-\
5d12ec9fdf5b"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[Eta]", "k"], "=", " ", 
  RowBox[{
   SubsuperscriptBox["R", "hij", 
    RowBox[{"       ", "k"}]], 
   SuperscriptBox["\[Alpha]", "h"], 
   SuperscriptBox["\[Beta]", "j"], 
   RowBox[{
    SuperscriptBox["\[Xi]", "i"], "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.665724411996249*^9, 
  3.665724670069748*^9}},ExpressionUUID->"80fd0fe0-6a09-46da-9780-\
10e30a43320b"],

Cell["\<\
The Ricci curvature tensor is defined to be the trace of this map:\
\>", "Text",
 CellChangeTimes->{{3.665724739244453*^9, 
  3.665724792124257*^9}},ExpressionUUID->"063aef7e-9e24-447a-bcb6-\
14641f027a9e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Ric", "[", 
   RowBox[{"A", ",", "B"}], "]"}], " ", "=", "  ", 
  RowBox[{
   SubsuperscriptBox["R", "hij", 
    RowBox[{"       ", "i"}]], 
   SuperscriptBox["\[Alpha]", "h"], 
   SuperscriptBox["\[Beta]", "j"]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.66572479887009*^9, 3.665724800580071*^9}, {
  3.665724864387657*^9, 
  3.665724932227456*^9}},ExpressionUUID->"6b181e7a-5110-4888-8e3a-\
96ce75f0bad2"],

Cell[TextData[{
 "The definition in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " terms is"
}], "Text",
 CellChangeTimes->{{3.665735859290763*^9, 
  3.6657358816355953`*^9}},ExpressionUUID->"93ea2530-14c4-4b0e-a36d-\
53527d3a43b2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Riccitensor", "[", "\[CapitalGamma]_", "]"}], "[", "x__", "]"}], 
  " ", ":=", " ", 
  RowBox[{"contraction", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"RCtensor", "[", "\[CapitalGamma]", "]"}], "[", "x", "]"}], ",", 
    "2", ",", "1"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.665725237814007*^9, 3.66572524771244*^9}, {
  3.66572538767914*^9, 3.665725398303235*^9}, {3.66572598592457*^9, 
  3.665726071386093*^9}, {3.665726204841543*^9, 
  3.665726208753202*^9}},ExpressionUUID->"0d9e1a2a-ff0c-469f-af9c-\
f633c1d135d6"],

Cell[BoxData[
 RowBox[{"?", "Riccitensor"}]], "Input",
 CellChangeTimes->{{3.665733267356821*^9, 3.6657334203297663`*^9}, {
  3.665735619398315*^9, 
  3.665735634317774*^9}},ExpressionUUID->"ed2e84f6-962d-4b70-a09a-\
dc462c8aa5de"],

Cell[BoxData[
 RowBox[{"Context", "[", "Riccitensor", "]"}]], "Input",
 CellChangeTimes->{{3.665733267356821*^9, 3.6657334203297663`*^9}, {
  3.665735619398315*^9, 3.665735634317774*^9}, {3.7041941811207457`*^9, 
  3.70419419946253*^9}},ExpressionUUID->"a57a2bd9-e8d2-4de7-9827-\
6cd67b6215fe"],

Cell[BoxData[
 RowBox[{"Rct", " ", "=", 
  RowBox[{
   RowBox[{"Riccitensor", "[", "\[CapitalGamma]", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.665726088631867*^9, 3.665726114545061*^9}, {
  3.6657264004756117`*^9, 
  3.665726403063346*^9}},ExpressionUUID->"91703f5e-44b5-4461-9643-\
1b45702084e4"],

Cell["The coordinates of this tensor are", "Text",
 CellChangeTimes->{{3.665726271511252*^9, 
  3.665726282407649*^9}},ExpressionUUID->"33aff587-586d-4984-a428-\
7c3856f6a2c8"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.704194322004717*^9, 
  3.704194324629999*^9}},ExpressionUUID->"78255c54-a95a-456a-a176-\
f8d2d43e11a1"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.704198204706811*^9, 
  3.7041982161743507`*^9}},ExpressionUUID->"b70c4350-18b8-4db3-a9cf-\
6d6bab86f6d6"],

Cell[BoxData[
 RowBox[{"?", "rct"}]], "Input",
 CellChangeTimes->{{3.6657334539780083`*^9, 3.6657335262165947`*^9}, {
  3.665733559441351*^9, 3.6657335903683977`*^9}, {3.6657356761096*^9, 
  3.665735703333067*^9}},ExpressionUUID->"a20d8b36-5fca-4c38-be5b-\
b05363cb3db6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rct", "[", "\[CapitalGamma]_", "]"}], "[", "x__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", " ", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RC", "[", "\[CapitalGamma]", "]"}], "[", "x", "]"}], "[", 
     RowBox[{"h", ",", "i", ",", "h", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"h", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"{", "x", "}"}], "]"}]}], "}"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.665726667571447*^9, 3.665726849660121*^9}, {
  3.665726886220049*^9, 3.6657269540987453`*^9}, {3.704194370086849*^9, 
  3.704194396879356*^9}, {3.704198274830373*^9, 
  3.7041982875265627`*^9}},ExpressionUUID->"89168fa7-796a-445f-8b90-\
72dda3bb7a02"],

Cell[BoxData[
 RowBox[{"Rct", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.665726414625355*^9, 
  3.6657264407511463`*^9}},ExpressionUUID->"7563ce59-d725-4289-a622-\
50024af7bf6c"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{
    RowBox[{"rct", "[", "\[CapitalGamma]", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"1", ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.665726983758362*^9, 3.665726989866343*^9}, {
  3.66572702813966*^9, 3.665727082186101*^9}, {3.7041986518853292`*^9, 
  3.7041986614128733`*^9}},ExpressionUUID->"691e3d77-8e7e-4782-8b5a-\
76d5329f300b"],

Cell[BoxData[
 RowBox[{"rctt", ":=", " ", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{"Table", " ", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rct", "[", "\[CapitalGamma]", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
       RowBox[{"i", ",", "j"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "dim"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], ",", "2"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.66572712485538*^9, 3.665727275616077*^9}, {
  3.665728064632842*^9, 3.665728066569887*^9}, {3.665728116825099*^9, 
  3.665728117545237*^9}, {3.704198703692573*^9, 
  3.704198719260322*^9}},ExpressionUUID->"b2b9dc29-3b4c-4f67-b4cc-\
d6fa15b48adc"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"equi", "[", 
   RowBox[{"rctt", ",", "Rct"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.665727290013184*^9, 3.66572733492873*^9}, {
  3.665727727252102*^9, 3.665727764908944*^9}, {3.665727812100802*^9, 
  3.6657278170118437`*^9}, {3.6657279279711227`*^9, 3.6657279483861103`*^9}, {
  3.665728142465352*^9, 3.665728147800723*^9}, {3.6657282200478497`*^9, 
  3.665728235936098*^9}},ExpressionUUID->"a900a32a-a8e7-4b1a-80bb-\
2f4607cfe44f"],

Cell[TextData[{
 "The Ricci tensor is often used in Riemannian geometry and relativity \
theory. We show here\n",
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". The Ricci-tensor of the Levi-Civita connection of a pseudo-Riemannian \
space (see Section 4.1 below) is symmetric.\n",
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". The Levi-Civita connection is given by"
}], "Text",
 CellChangeTimes->{{3.665735970450363*^9, 3.665736156120966*^9}, {
   3.6657361931447277`*^9, 3.66573629484769*^9}, 
   3.704194134447743*^9},ExpressionUUID->"39a3dfbc-5dc7-4d38-aea6-\
15987c85afd0"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]g"}]], "Input",
 CellChangeTimes->{{3.6657317173975897`*^9, 
  3.665731730512595*^9}},ExpressionUUID->"83b3832a-6421-4987-bc71-\
e996dd94dde1"],

Cell[BoxData[
 RowBox[{"?", "rg"}]], "Input",
 CellChangeTimes->{{3.6657318826365957`*^9, 
  3.665731883606328*^9}},ExpressionUUID->"a4c221c0-ac4b-4a32-a83a-\
e33d9bd10fd5"],

Cell["The Ricci tensor of the pseudo-Riemannian manifold is", "Text",
 CellChangeTimes->{{3.665736332318906*^9, 3.665736387198698*^9}, 
   3.708489915591778*^9},ExpressionUUID->"f895b392-2fd9-4418-9d8c-\
40a9c6445825"],

Cell[BoxData[
 RowBox[{"Rgeoct", "=", 
  RowBox[{
   RowBox[{"Riccitensor", "[", 
    RowBox[{"\[CapitalGamma]g", "[", "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6657291454549026`*^9, 3.665729166297607*^9}, {
  3.66572925034699*^9, 3.665729314798023*^9}, {3.665731971509972*^9, 
  3.665731974693921*^9}, {3.6657325428976393`*^9, 
  3.665732553329088*^9}},ExpressionUUID->"210f651a-f908-4725-8ebe-\
948b992737f9"],

Cell["To confirm its symmetry we need its coordinate array", "Text",
 CellChangeTimes->{{3.665736618284485*^9, 
  3.665736653268042*^9}},ExpressionUUID->"c876757a-9e89-4a00-9231-\
b2bd707b36d9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rctarr", "=", 
   RowBox[{"tensorToArray", "[", "%", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.665728608510314*^9, 3.665728627133397*^9}, {
   3.665728657220145*^9, 3.6657286638357887`*^9}, 
   3.665729338733613*^9},ExpressionUUID->"b2ed8aab-8f85-47f7-b802-\
029ecb497868"],

Cell["Indeed:", "Text",
 CellChangeTimes->{{3.665736680683732*^9, 
  3.665736684316209*^9}},ExpressionUUID->"c8fba943-1010-441f-b7bf-\
6b6f89141bee"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "rctarr", "]"}]], "Input",
 CellChangeTimes->{{3.665728669539564*^9, 
  3.665728686252729*^9}},ExpressionUUID->"72cb038e-2197-41a0-8a3f-\
8bf9378f421c"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.665736703635852*^9, 
  3.6657367036375513`*^9}},ExpressionUUID->"b0883ef1-fa5a-4745-99f7-\
f89d54bae29a"]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4. Pseudo-Riemannian Spaces", "Section",
 CellChangeTimes->{{3.661586315494672*^9, 3.6615863388461657`*^9}, {
  3.670568418319591*^9, 3.6705684388714523`*^9}, {3.6706497940966682`*^9, 
  3.670649825896896*^9}},ExpressionUUID->"17943592-7608-4a77-8384-\
de67e5b47af4"],

Cell[CellGroupData[{

Cell["4.1. The Levi-Civita Connection", "Subsection",
 CellChangeTimes->{{3.6615874460422792`*^9, 3.6615874819137487`*^9}, 
   3.669700546337277*^9, 3.6697280843361397`*^9, {3.702362001000907*^9, 
   3.7023620025207148`*^9}, 3.7042625563488903`*^9, 3.7084895650597897`*^9, 
   3.708496558224681*^9},ExpressionUUID->"dd38dc69-f399-4245-8867-\
d88e7c690bc1"],

Cell[TextData[{
 "In this Section we consider pseudo-Riemannian manifolds defined in ",
 ButtonBox["Section 1.1",
  BaseStyle->"Hyperlink",
  ButtonData->"defRsp"],
 ". We will prove the ",
 Cell[BoxData[
  FormBox[
   ButtonBox[
    RowBox[{"Levi", "-", "Civita"}],
    BaseStyle->"Hyperlink",
    ButtonData->{
      URL[
      "http://www-history.mcs.st-andrews.ac.uk/Biographies/Levi-Civita.html"],
       None},
    ButtonNote->
     "http://www-history.mcs.st-andrews.ac.uk/Biographies/Levi-Civita.html"], 
   TraditionalForm]],ExpressionUUID->"6f8ea082-dd22-4149-aa46-c2187660982c"],
 " Theorem, sometimes named the ",
 ButtonBox["Fundamental Theorem of Riemannian Geometry ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/FundamentalTheoremofRiemannianGeometry.\
html"], None},
  ButtonNote->
   "http://mathworld.wolfram.com/FundamentalTheoremofRiemannianGeometry.html"],
 "stating the existence and uniqueness of a canonical symmetric linear \
connection on  the pseudo-Riemannian space. "
}], "Text",
 CellChangeTimes->{{3.661586369733899*^9, 3.661586452973571*^9}, {
   3.661586486397147*^9, 3.6615866245080347`*^9}, {3.661586698349266*^9, 
   3.66158669835135*^9}, {3.661586741235362*^9, 3.6615867523632307`*^9}, {
   3.6615868573379793`*^9, 3.6615868900093193`*^9}, {3.661586942111771*^9, 
   3.661587155542378*^9}, {3.661587253722044*^9, 3.661587253724102*^9}, 
   3.661587373803754*^9, 3.6705682405453577`*^9, {3.702448183004505*^9, 
   3.702448186211524*^9}, {3.70252500788054*^9, 3.702525038568679*^9}, {
   3.702526165936371*^9, 3.7025261770081882`*^9}, 3.7025262431758347`*^9, {
   3.704285904380271*^9, 3.70428590438337*^9}, {3.704286598260326*^9, 
   3.704286598271447*^9}},ExpressionUUID->"7691c6c1-c61d-4433-b249-\
e7f5e545bc16"],

Cell[TextData[{
 StyleBox["Theorem",
  FontColor->RGBColor[0, 0, 1]],
 " (",
 ButtonBox["Levi-Civita",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://de.wikipedia.org/wiki/Tullio_Levi-Civita"], None},
  ButtonNote->"https://de.wikipedia.org/wiki/Tullio_Levi-Civita"],
 "). On each pseudo-Riemannian manifold there exists a uniquely defined \
linear connection \[CapitalGamma] whose torsion tensor is the null tensor \
such that the absolute differential of the metric tensor  is zero."
}], "Text",
 CellChangeTimes->{{3.6615876178081627`*^9, 3.661587794382444*^9}, {
   3.6615879048053083`*^9, 3.661588009428344*^9}, {3.6615882859779453`*^9, 
   3.6615882859806147`*^9}, 3.661595262685974*^9, 
   3.702448460176195*^9},ExpressionUUID->"2a8affa5-48c2-4b7d-8c61-\
086687576fef"],

Cell[TextData[{
 StyleBox["Definition",
  FontColor->RGBColor[0, 0, 1]],
 ". This uniquely defined symmetric linear connection is called the ",
 StyleBox["Levi-Civita connection",
  FontColor->RGBColor[0, 0, 1]],
 " (sometimes also the ",
 StyleBox["Riemann connection",
  FontColor->RGBColor[0, 0, 1]],
 ") of the pseudo-Riemannian manifold. "
}], "Text",
 CellChangeTimes->{{3.675917807901376*^9, 3.675917843755643*^9}, 
   3.6759178741637363`*^9},ExpressionUUID->"726ad21e-78f1-425b-bfc5-\
a0e94099f3e4"],

Cell[TextData[{
 StyleBox["Proof of the Theorem,",
  FontColor->RGBColor[0, 0, 1]],
 ". We consider a general pseudo-Riemannian metric with the unspecified \
coordinates "
}], "Text",
 CellChangeTimes->{{3.6615991523355227`*^9, 3.6615991581110573`*^9}, {
   3.661599795578628*^9, 3.661599856640383*^9}, {3.67591790911504*^9, 
   3.675917916146853*^9}, 3.702448497375491*^9, {3.702526336254228*^9, 
   3.702526342533889*^9}},ExpressionUUID->"6217ab73-3b4d-480d-b0aa-\
95a731dc219e"],

Cell[BoxData[
 RowBox[{"?", "RG"}]], "Input",
 CellChangeTimes->{{3.7042040121325502`*^9, 
  3.704204014396936*^9}},ExpressionUUID->"59125225-32db-4394-84b3-\
122a455364d8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{
   RowBox[{"RG", "[", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.661599862192931*^9, 3.6615998638565607`*^9}, {
   3.661599970234028*^9, 3.661599993223673*^9}, {3.662789756844782*^9, 
   3.662789759108511*^9}, {3.702474308796722*^9, 3.702474309484091*^9}, 
   3.702474733617841*^9, {3.7042039869103107`*^9, 3.704203987829121*^9}, {
   3.704204041677809*^9, 
   3.704204080556388*^9}},ExpressionUUID->"98964d3f-3303-472b-ade4-\
1316dbf1836e"],

Cell["\<\
We assume that the he coordinates of the absolute differential of the metric \
tensor rg with respect to the standard  basis of a chart fulfill  the \
conditions of the theorem:\
\>", "Text",
 CellChangeTimes->{{3.661600088341168*^9, 3.661600237913581*^9}, {
   3.6616007400447206`*^9, 3.66160075757225*^9}, 3.661753034989335*^9, {
   3.6697247614193707`*^9, 3.669724831306252*^9}, 
   3.669726230487035*^9},ExpressionUUID->"e4fa0fd7-c00d-4608-86a0-\
8e3f56b0a395"],

Cell[BoxData[
 RowBox[{"?", "rg"}]], "Input",
 CellChangeTimes->{{3.702448573737006*^9, 
  3.702448575238779*^9}},ExpressionUUID->"632b447e-8d45-4dc8-92ef-\
771312771121"],

Cell["The rg-matrix is always symmetric", "Text",
 CellChangeTimes->{{3.7024764555856237`*^9, 
  3.7024764732256327`*^9}},ExpressionUUID->"7777274f-7da7-4007-97e6-\
5e02324d4f3d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rg", "[", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"2", ",", "1"}], "]"}], "-", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"rg", "[", "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.702448604041354*^9, 3.7024487039190893`*^9}, {
  3.70244874768563*^9, 3.7024488136611843`*^9}, {3.7024488639416933`*^9, 
  3.702448896660409*^9}, {3.70244897537204*^9, 3.7024490261469717`*^9}, {
  3.7024490799730186`*^9, 3.7024490902909393`*^9}, {3.702451038981187*^9, 
  3.702451104202816*^9}, {3.702474774001412*^9, 3.7024747897043047`*^9}, {
  3.702524719538362*^9, 3.702524760945978*^9}, {3.7025247909869423`*^9, 
  3.702524829306971*^9}, {3.702526409710401*^9, 
  3.7025264244058*^9}},ExpressionUUID->"43172ae8-bc68-479f-876d-bf7caee8deb6"],

Cell["\<\
Of course, Mathematica can\[CloseCurlyQuote]t decide wether the metric is \
regular:\
\>", "Text",
 CellChangeTimes->{{3.702476880878202*^9, 
  3.702476955501932*^9}},ExpressionUUID->"989dbb75-f7c0-407d-9082-\
7c5925ce586e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Det", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"rg", "[", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{"i", ",", "j"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "]"}], "\[NotEqual]", 
  "0"}]], "Input",
 CellChangeTimes->{{3.702476485750103*^9, 3.70247649090584*^9}, {
   3.7024765478019667`*^9, 3.702476648136444*^9}, {3.70247678990557*^9, 
   3.702476796487193*^9}, {3.702524851449251*^9, 3.7025248731054773`*^9}, 
   3.7025265148221292`*^9},ExpressionUUID->"94b10152-a51c-4497-a32f-\
5edc1464724c"],

Cell["\<\
what is assumed in the following. We need the inverse matrix of the metric. \
It can be  calculated by\
\>", "Text",
 CellChangeTimes->{{3.702476966783558*^9, 3.702477020269223*^9}, {
  3.702526581884727*^9, 3.702526649412019*^9}, {3.702526696731843*^9, 
  3.70252670909972*^9}, {3.70253082215524*^9, 
  3.702530897714316*^9}},ExpressionUUID->"e8f88c78-f6d9-430c-9062-\
cf03c70d8535"],

Cell[BoxData[
 RowBox[{"?", "INV"}]], "Input",
 CellChangeTimes->{{3.702526821447302*^9, 
  3.702526824979251*^9}},ExpressionUUID->"998b75c0-1c1b-4fa4-b4ac-\
09702f35b2fd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"INV", "[", 
   RowBox[{"RG", "[", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.702526854073457*^9, 3.702526889026853*^9}, 
   3.70272488457332*^9, {3.702724939279587*^9, 
   3.7027249402454967`*^9}},ExpressionUUID->"28dcf2ea-f3e7-4973-9764-\
96bcc544f32c"],

Cell[BoxData[
 RowBox[{"?", "inv"}]], "Input",
 CellChangeTimes->{{3.7025305059383183`*^9, 
  3.702530508564727*^9}},ExpressionUUID->"d33ed4ef-fcc7-4058-a446-\
d94b0a0b922b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"inv", "[", 
    RowBox[{"RG", "[", "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"1", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.702530518855257*^9, 3.702530551261096*^9}, 
   3.7027243374802933`*^9, {3.70420436200215*^9, 
   3.7042043637619543`*^9}},ExpressionUUID->"8b12f922-205d-4c01-82b5-\
507a42050d6d"],

Cell["\<\
We consider the gradient tensor of the metric in traditional notation:\
\>", "Text",
 CellChangeTimes->{{3.702531175264059*^9, 
  3.702531210295843*^9}},ExpressionUUID->"96778aaa-9e33-471d-8c42-\
9fa907243a9f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "h"], 
   RowBox[{"g", "[", 
    RowBox[{"i", ",", "j"}], "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{"g", "[", 
     RowBox[{"i", ",", "j"}], "]"}]}], " ", "-", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"l", ",", "j"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", "hi", 
     RowBox[{"l", " "}]]}], " ", "-", " ", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"i", ",", "l"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", 
     RowBox[{"hj", " "}], 
     RowBox[{"l", " "}]]}]}], " ", "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6616003867175303`*^9, 3.6616004386522303`*^9}, {
  3.661600469850914*^9, 3.661600716565433*^9}, {3.662385244476645*^9, 
  3.662385246844475*^9}},ExpressionUUID->"81bab147-70e0-4741-a02c-\
d892851f2341"],

Cell["or", "Text",
 CellChangeTimes->{{3.661600848265563*^9, 
  3.661600868291366*^9}},ExpressionUUID->"e44dbf17-99b7-4146-9f10-\
cfc176b02dac"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "h"], 
   RowBox[{"g", "[", 
    RowBox[{"i", ",", "j"}], "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"l", ",", "j"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", "hi", 
     RowBox[{"l", " "}]]}], " ", "+", " ", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"i", ",", "l"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", 
     RowBox[{"hj", " "}], 
     RowBox[{"l", " "}]], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.661600906337352*^9, 3.661600936417*^9}, {
  3.662385249173073*^9, 
  3.662385251117736*^9}},ExpressionUUID->"1f5db553-28ba-4399-bb34-\
fdfe16883440"],

Cell["\<\
Permuting the indices (h,i,j) cyclically twice in this equation we get\
\>", "Text",
 CellChangeTimes->{{3.661601858459262*^9, 3.661601916947587*^9}, {
  3.661753087252552*^9, 3.6617531084690733`*^9}, {3.662446685889743*^9, 
  3.662446698817843*^9}, {3.66244674097334*^9, 
  3.6624467562901497`*^9}},ExpressionUUID->"aae3f91f-6170-4182-811e-\
7fbac954926b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "i"], 
   RowBox[{"g", "[", 
    RowBox[{"j", ",", "h"}], "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"l", ",", "h"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", "ij", 
     RowBox[{"l", " "}]]}], " ", "+", " ", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"j", ",", "l"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", 
     RowBox[{"ih", " "}], 
     RowBox[{"l", " "}]], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.661601947932416*^9, 3.661601982795932*^9}, {
  3.661602013532112*^9, 3.661602056859898*^9}, {3.662385252373735*^9, 
  3.6623852536927233`*^9}},ExpressionUUID->"72f1e6eb-89e9-47bc-9b31-\
19c5fa585456"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[PartialD]", "j"], 
   RowBox[{"g", "[", 
    RowBox[{"h", ",", "i"}], "]"}]}], " ", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"l", ",", "i"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", "jh", 
     RowBox[{"l", " "}]]}], " ", "+", " ", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"h", ",", "l"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", "ji", 
     RowBox[{"l", " "}]], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.661602114236454*^9, 3.661602207836411*^9}, {
  3.661602241030756*^9, 3.661602264508644*^9}, {3.66238525466183*^9, 
  3.662385255997534*^9}},ExpressionUUID->"ae2479bd-2244-44e9-a28a-\
21e155edbbf0"],

Cell["Having in mind the symmetry property", "Text",
 CellChangeTimes->{{3.661602282148245*^9, 3.661602312765647*^9}, 
   3.7025312629036093`*^9},ExpressionUUID->"91118914-431f-49f2-ae94-\
17210cf472d7"],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[CapitalGamma]", "ij", 
   RowBox[{"l", " "}]], " ", "\[Equal]", " ", 
  SubsuperscriptBox["\[CapitalGamma]", "ji", 
   RowBox[{"l", " "}]]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.661602342859733*^9, 3.6616023978117533`*^9}, {
  3.6616024358045263`*^9, 3.661602447836439*^9}, {3.661753147100116*^9, 
  3.6617531476758833`*^9}},ExpressionUUID->"94d07adf-c65c-48b2-80ca-\
cade0c309797"],

Cell["\<\
since  the torsion tensor is zero, and g[i,j] == g[j,i], we conclude\
\>", "Text",
 CellChangeTimes->{{3.6616024682033367`*^9, 3.661602526403265*^9}, {
  3.6617531374760857`*^9, 3.661753139731922*^9}, {3.662385274124423*^9, 
  3.662385286528764*^9}, {3.669724871489884*^9, 
  3.669724880265943*^9}},ExpressionUUID->"376b9ac6-69e9-4f16-865b-\
c4baecd37e02"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", "h"], 
    RowBox[{"g", "[", 
     RowBox[{"i", ",", "j"}], "]"}]}], " ", "+", "  ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], 
    RowBox[{"g", "[", 
     RowBox[{"j", ",", "h"}], "]"}]}], " ", "-", " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "j"], 
    RowBox[{"g", "[", 
     RowBox[{"h", ",", "i"}], "]"}]}]}], " ", "\[Equal]", " ", 
  RowBox[{"2", 
   RowBox[{"g", "[", 
    RowBox[{"j", ",", "l"}], "]"}], 
   SubsuperscriptBox["\[CapitalGamma]", "hi", 
    RowBox[{"l", " "}]]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.661602566291513*^9, 3.661602568771818*^9}, {
  3.66160261196414*^9, 3.6616027082112207`*^9}, {3.66160275085248*^9, 
  3.661602782835724*^9}, {3.662385290949288*^9, 
  3.662385292941031*^9}},ExpressionUUID->"e08fd4ff-1121-4ec4-addf-\
f4aff4151d37"],

Cell[TextData[{
 "Multiplying both sides of this equation from the left with the coordinate \
matrix {",
 Cell[BoxData[
  RowBox[{
   OverscriptBox["g", 
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}]], "[", 
   RowBox[{"k", ",", "j"}], "]"}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.661607834220664*^9, 3.6616078809162083`*^9}, {
   3.661607948404129*^9, 3.6616080877396593`*^9}, {3.662385294383587*^9, 
   3.66238530947665*^9}},ExpressionUUID->
  "cf0b09fb-d108-4e3c-830d-6ec391ee9535"],
 "} of the inverse metric tensor and with 1/2 one obtains"
}], "Text",
 CellChangeTimes->{{3.661603094203581*^9, 3.6616031569162683`*^9}, {
  3.6617532414825983`*^9, 3.661753269370459*^9}, {3.66972499887298*^9, 
  3.669725004056821*^9}, {3.669725034128662*^9, 3.669725040616535*^9}, {
  3.6759185455988083`*^9, 
  3.6759185668310213`*^9}},ExpressionUUID->"08f41583-4ec7-4e2c-8bd4-\
5e897c81d73e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     OverscriptBox["g", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]], "[", 
     RowBox[{"k", ",", "j"}], "]"}], 
    RowBox[{"g", "[", 
     RowBox[{"j", ",", "l"}], "]"}], 
    SubsuperscriptBox["\[CapitalGamma]", "hi", 
     RowBox[{"l", " "}]]}], "==", " ", 
   RowBox[{
    SubsuperscriptBox["\[Delta]", "l", "k"], 
    SubsuperscriptBox["\[CapitalGamma]", "hi", 
     RowBox[{"l", " "}]]}], "\[Equal]", " ", 
   SubsuperscriptBox["\[CapitalGamma]", "hi", 
    RowBox[{"k", " "}]]}], ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.661607834220664*^9, 3.6616078809162083`*^9}, {
  3.661607948404129*^9, 3.6616080877396593`*^9}, {3.662385294383587*^9, 
  3.66238530947665*^9}},ExpressionUUID->"848c3341-7104-4e52-ad8b-\
72ca563409bf"],

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[CapitalGamma]", "hi", 
   RowBox[{"k", " "}]], " ", "\[Equal]", 
  RowBox[{
   RowBox[{
    OverscriptBox["g", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]], "[", 
    RowBox[{"k", ",", "j"}], "]"}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", "h"], 
       RowBox[{"g", "[", 
        RowBox[{"i", ",", "j"}], "]"}]}], " ", "+", "  ", 
      RowBox[{
       SubscriptBox["\[PartialD]", "i"], 
       RowBox[{"g", "[", 
        RowBox[{"j", ",", "h"}], "]"}]}], " ", "-", " ", 
      RowBox[{
       SubscriptBox["\[PartialD]", "j"], 
       RowBox[{"g", "[", 
        RowBox[{"h", ",", "i"}], "]"}]}]}], ")"}], "/", "2"}], "          ", 
   RowBox[{"(", "1", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.661608110811425*^9, 3.6616081181636677`*^9}, {
  3.661608165939457*^9, 3.661608260970874*^9}, {3.661608293092252*^9, 
  3.661608297386895*^9}, {3.661608556105802*^9, 3.66160857662777*^9}, {
  3.6617661279998837`*^9, 3.661766155399336*^9}, {3.662385318700341*^9, 
  3.662385334285211*^9}},ExpressionUUID->"365f55ae-525f-4f18-b69c-\
fc507e379474"],

Cell[TextData[{
 "It follows: If a linear connection satisfies the assumptions of the theorem \
then  the coefficients of the connection fulfill equation (1) and thus are \
uniquely defined by the Riemannian metric tensor. To prove the existence one \
uses equation (1) as the definition of the connection coefficients and checks \
the transformation rule ",
 ButtonBox["3.2.3, (4)",
  BaseStyle->"Hyperlink",
  ButtonData->"gammatr"],
 ", this is left to the reader. In all the formulas of the proof we used \
traditional notation and the sum convention. All the formulas aren\
\[CloseCurlyQuote]t evaluatable by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.661608598352891*^9, 3.6616086287851067`*^9}, {
   3.66160873837679*^9, 3.66160897703198*^9}, {3.661609235686281*^9, 
   3.661609457398328*^9}, 3.6616095428207817`*^9, {3.661753368161395*^9, 
   3.6617533857925262`*^9}},ExpressionUUID->"dae376f1-356b-42ce-a92e-\
19ac19c780b5"],

Cell[TextData[{
 "The next statement clears the geometric meaning of the second condition of \
the Theorem: ",
 StyleBox["Angles and length of vectors remain invariant under parallel \
displacement s along curves.",
  FontSlant->"Italic"],
 "\n\n",
 StyleBox["Corollary.",
  FontColor->RGBColor[0, 0, 1]],
 " Let \[Gamma][t] be a smooth curve and X[t], Y[t] two vector functions with \
\nX[t] \[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"\[Gamma]", "[", "t", "]"}]], TraditionalForm]],ExpressionUUID->
  "83f42b39-affe-4ad0-a072-4fb8f118c704"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "9663b723-75a9-496a-a3a2-1b1eb085eeaa"],
 ", Y[t]\[Element] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"\[Gamma]", "[", "t", "]"}]], TraditionalForm]],ExpressionUUID->
  "271a6142-92d4-4fc9-b113-82fab4976f54"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "52a8d385-65d6-42c0-be18-4d9bcb5ba2e4"],
 ". If these vector functions are parallel displacements with respect to the \
Levi-Civita connection,  the scalar product of these vectors is constant; \
thus the parallel displacements are pseudo-orthogonal maps, orthogonal in the \
Riemannian case. . Conversely, if for a given linear connection this is the \
case for all smooth curves and parallel displaced vector functions along the \
curves, then absD[g] = 0,\n",
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". The scalar product of the vector functions is the contraction"
}], "Text",
 CellChangeTimes->{{3.66555941913654*^9, 3.6655594464187307`*^9}, {
   3.665559558953979*^9, 3.665559693496621*^9}, {3.665559818927219*^9, 
   3.6655598758708363`*^9}, {3.665559906062261*^9, 3.665560020029251*^9}, {
   3.6655600626449947`*^9, 3.6655601293879766`*^9}, {3.665560166153813*^9, 
   3.665560228131139*^9}, {3.6655602642206087`*^9, 3.665560310098995*^9}, {
   3.665562300326523*^9, 3.665562572240793*^9}, 3.665573618317013*^9, 
   3.66557365326093*^9, {3.665573685244322*^9, 3.6655737059162807`*^9}, 
   3.6691122637248087`*^9, 3.669726321822959*^9, {3.669726723132346*^9, 
   3.669726736596175*^9}, {3.6759180130022907`*^9, 3.67591813556242*^9}, 
   3.675918243649411*^9, {3.7025315949253674`*^9, 3.702531596510338*^9}, {
   3.702531778237225*^9, 3.702531884810933*^9}, 3.7025319167952538`*^9, {
   3.702531972147256*^9, 3.7025319880825863`*^9}, 
   3.702532050604375*^9},ExpressionUUID->"eaa03464-85e4-4570-a8df-\
13ead35cb50c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "t", "]"}], ":=", 
   RowBox[{"<", 
    RowBox[{"X", "[", "t", "]"}]}]}], ",", 
  RowBox[{
   RowBox[{"Y", "[", "t", "]"}], ">", "\[DotEqual]", "  ", 
   RowBox[{"\[FormalCapitalY]", " ", 
    RowBox[{
     RowBox[{"\[FormalCapitalY]g", "[", 
      RowBox[{"\[Gamma]", "[", "t", "]"}], "]"}], "\[CircleTimes]", 
     RowBox[{"X", "[", "t", "]"}], "\[CircleTimes]", 
     RowBox[{"Y", "[", "t", "]"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.665560332090993*^9, 3.6655603627863293`*^9}, {
  3.665560397321212*^9, 3.665560401649377*^9}, {3.665560625225297*^9, 
  3.665560647560985*^9}, {3.665560827553973*^9, 3.665560922104518*^9}, {
  3.6655610651912394`*^9, 
  3.665561072807026*^9}},ExpressionUUID->"9e1c6ea8-8dda-4eae-8f90-\
e712fcc17358"],

Cell["\<\
where g denotes the Riemannian metric tensor and  \[FormalCapitalY] \
\[FormalCapitalY] the complete contraction of the tensor product. Obviously, \
the defining derivation rules of a linear connection can be applied to the \
absolute differential along a curve. It follows \
\>", "Text",
 CellChangeTimes->{{3.665560948960391*^9, 3.665560986679756*^9}, {
   3.665561024985454*^9, 3.66556105434371*^9}, {3.665561868938092*^9, 
   3.665561940425256*^9}, {3.665561972424921*^9, 3.665561993617021*^9}, 
   3.665562605667898*^9, 
   3.6655737307321663`*^9},ExpressionUUID->"bb6428dc-19f9-4264-81d0-\
fd354edaba07"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"f", "[", "t", "]"}], "]"}], "/", "dt"}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"absD", "[", 
     RowBox[{"f", "[", "t", "]"}], "]"}], "/", "dt"}], " ", "=", " ", 
   RowBox[{"\[FormalCapitalY]", " ", "\[FormalCapitalY]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"absD", "[", "g", "]"}], "/", "dt"}], ")"}], "\[CircleTimes]", 
     RowBox[{"X", "[", "t", "]"}], "\[CircleTimes]", 
     RowBox[{"Y", "[", "t", "]"}]}], "                                     ", 
    RowBox[{"(", "2", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.665562003074119*^9, 3.6655621154633904`*^9}, {
   3.66556223216582*^9, 3.665562234086286*^9}, 3.665562270271618*^9, {
   3.665573811754859*^9, 3.665573822122983*^9}, 3.665574257769473*^9, {
   3.6655790695222692`*^9, 3.665579082993642*^9}, {3.669112263751931*^9, 
   3.669112263753248*^9}},ExpressionUUID->"f8e01acd-6867-43fa-9d3a-\
5a55d2e9b961"],

Cell["\<\
Here we applied the commutation of absD with the contractiond  \
\[FormalCapitalY] \[FormalCapitalY], he product rule, and absDX = absDY=0 for \
parallel displaced vectors. If absDg is zero too, then f\.b4[t] = 0 and the \
scalar product is constant. Conversely,expression (2) depends linearly on \
X,Y, and the tangent vector \[Gamma]\[CloseCurlyQuote][t]  of the curve.  In \
local coordinates one has\
\>", "Text",
 CellChangeTimes->{{3.665574201128693*^9, 3.665574245303936*^9}, {
   3.665574279264103*^9, 3.665574537910285*^9}, {3.665574571318324*^9, 
   3.66557461626196*^9}, 3.6655782407453403`*^9, {3.6655790044828787`*^9, 
   3.665579012234995*^9}, {3.665579104257909*^9, 3.665579283928331*^9}, {
   3.66557986709793*^9, 3.665579870569695*^9}, {3.669112263814781*^9, 
   3.6691122639383087`*^9}, {3.702532237704689*^9, 3.7025322645359993`*^9}, {
   3.702532311248597*^9, 3.702532322352414*^9}, {3.70253238071004*^9, 
   3.702532426414687*^9}},ExpressionUUID->"c63d2f15-73c8-4366-ac48-\
f7972341e6d1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "'"}], "[", "t", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Del]", "i"], 
    SubscriptBox["g", "jk"]}], 
   SuperscriptBox[
    RowBox[{"\[Gamma]", "'"}], "i"], 
   SuperscriptBox["\[Xi]", "j"], 
   RowBox[{
    SuperscriptBox["\[Eta]", "k"], "."}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6655792996096983`*^9, 3.665579311943762*^9}, {
   3.665579368063973*^9, 3.665579454869541*^9}, {3.665579538755106*^9, 
   3.665579584491878*^9}, {3.665579623051005*^9, 3.665579659051749*^9}, 
   3.665579733770228*^9, {3.702559994581006*^9, 
   3.7025600082447853`*^9}},ExpressionUUID->"0fa7ded4-5873-4d50-83d5-\
54f7cc4a06b6"],

Cell[TextData[{
 "If this three-linear form is zero for all vectors \[Gamma]\.b4, X, Y, its \
coefficients ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Del]", "i"], 
   SubscriptBox["g", "jk"]}]], "Input",
  CellChangeTimes->{{3.6655792996096983`*^9, 3.665579311943762*^9}, {
    3.665579368063973*^9, 3.665579454869541*^9}, {3.665579538755106*^9, 
    3.665579584491878*^9}, {3.665579623051005*^9, 3.665579659051749*^9}, 
    3.665579733770228*^9},ExpressionUUID->
  "486312a4-58bf-4fd2-a04e-d97c8ccbc99d"],
 " must be zero, and this means absDg is the null tensor. \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.665579678402989*^9, 3.665579681842861*^9}, 
   3.6655797133319187`*^9, {3.665579745226515*^9, 3.6655798529294767`*^9}, {
   3.665579891698591*^9, 3.665580064295267*^9}, 3.669112263956128*^9, {
   3.702532480141529*^9, 
   3.702532509205106*^9}},ExpressionUUID->"b6e921d7-cf85-415a-ba8a-\
c074c228ceee"],

Cell[TextData[{
 "As an application we mention the definition of the ",
 StyleBox["arc length ",
  FontColor->RGBColor[0, 0, 1]],
 "parameter s of a regular  curve \[Gamma][t] in a Riemannian space, in \
traditional notation:"
}], "Text",
 CellChangeTimes->{{3.67056891277103*^9, 3.6705690398740177`*^9}, {
  3.6705696837800417`*^9, 3.67056972630772*^9}, {3.670569933794332*^9, 
  3.670569964873783*^9}, {3.7025325438046417`*^9, 
  3.70253256405247*^9}},ExpressionUUID->"bd449bd0-9504-4cf1-beca-\
92da18ba1c07"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"s", "[", "t", "]"}], ":=", " ", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", 
     SubscriptBox["t", "0"], "t"], 
    RowBox[{
     SqrtBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], ",", 
        RowBox[{
         RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]}], ")"}]], "dt"}]}]}], 
  ",", "          ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     OverscriptBox["\[Gamma]", "\[CenterDot]"], ",", 
     OverscriptBox["\[Gamma]", "\[CenterDot]"]}], ")"}], ">", " ", "0"}], 
  ","}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.670569071748131*^9, 3.6705690838178253`*^9}, {
  3.6705691230606003`*^9, 3.670569138417362*^9}, {3.670569264258337*^9, 
  3.6705693466651173`*^9}, {3.670569434916954*^9, 3.670569466978548*^9}, {
  3.6705695279839067`*^9, 3.6705695337422256`*^9}, {3.670569592526773*^9, 
  3.670569655012623*^9}, {3.670570139448845*^9, 3.6705701685203743`*^9}, {
  3.7084946671269608`*^9, 3.7084946931276093`*^9}, {3.7084952004191628`*^9, 
  3.708495223746952*^9}},ExpressionUUID->"a38f31b3-3bb3-48b7-984f-\
0bddc952ef90"],

Cell["Here", "Text",
 CellChangeTimes->{{3.708495360529148*^9, 
  3.7084953618088427`*^9}},ExpressionUUID->"e60cef49-33ed-4fa9-a5b1-\
2f1286d8735b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "\[Gamma]", "]"}], ";", 
  RowBox[{
   RowBox[{"\[Gamma]", "[", "t_", "]"}], ":=", " ", 
   RowBox[{"vec", "[", 
    RowBox[{"\[Xi]", "[", "t", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.702557851813799*^9, 3.7025578543662*^9}, {
  3.702557897903934*^9, 
  3.702558002524765*^9}},ExpressionUUID->"c11ca2fb-36f4-46ed-9c38-\
9ec4af0d568f"],

Cell["is a curve given by its coordinates \[Xi][t][i],", "Text",
 CellChangeTimes->{{3.70849540220856*^9, 3.708495455824601*^9}, {
  3.708495514889703*^9, 
  3.708495522968231*^9}},ExpressionUUID->"bdbe32fb-f0e8-4ce3-a79d-\
fcc3f378b0fc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]], "Input",ExpressionUUID->\
"1667737e-62ad-42e2-867e-79aa814e8e28"],

Cell[" with  tangent vector", "Text",
 CellChangeTimes->{{3.708495569663529*^9, 
  3.708495579551646*^9}},ExpressionUUID->"ec17c8e6-c256-4b59-852e-\
fc08af8208a0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.702558046003847*^9, 3.702558138403206*^9}, {
  3.702558281035862*^9, 3.702558295266114*^9}, {3.7025584028331347`*^9, 
  3.702558441608501*^9}},ExpressionUUID->"6803a08e-0d6b-4218-8b0d-\
6d4d4df066ec"],

Cell["\<\
and  (\[Gamma]\[CloseCurlyQuote][t],\[Gamma]\[CloseCurlyQuote][t]) denotes \
the scalar square of this vector wit respect  to the pseudo-Riemannian \
metric: \
\>", "Text",
 CellChangeTimes->{{3.7084956220511513`*^9, 3.708495637838809*^9}, {
   3.708495668656619*^9, 3.7084957813021507`*^9}, 
   3.708495817614276*^9},ExpressionUUID->"fee48c3d-2d43-4e01-baa0-\
36a2108eb1b0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}], ",", 
    RowBox[{
     RowBox[{"\[Gamma]", "'"}], "[", "t", "]"}]}], ")"}], " ", 
  "\[DotEqual]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.708495824892874*^9, 3.708495824893783*^9}, 
   3.708495856629422*^9},ExpressionUUID->"ef3733c8-65e6-4e04-afc8-\
76beed6bb262"],

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"rg", "[", "]"}], "[", 
      RowBox[{
       RowBox[{"\[Gamma]", "[", "t", "]"}], "/.", 
       RowBox[{"List", "\[Rule]", "Sequence"}]}], "]"}], "[", 
     RowBox[{"i", ",", "j"}], "]"}], "*", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Xi]", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "i", "]"}], "*", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Xi]", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}], "[", "j", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "dim"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "dim"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7025586256805964`*^9, 3.702558649769609*^9}, {
   3.702558681207732*^9, 3.702558739742076*^9}, {3.702558780551137*^9, 
   3.7025588778294353`*^9}, 3.7084947786845827`*^9, {3.708494814889962*^9, 
   3.7084948196531*^9}, 
   3.708494880795381*^9},ExpressionUUID->"0f0ffcdd-6833-43e3-a460-\
09d215ad1082"],

Cell[TextData[{
 "In pseudo-Riemannian spaces this definition is valid for ",
 StyleBox["spacelike curves",
  FontColor->RGBColor[0, 0, 1]],
 " only, whose tangent vectors have positive scalar square. This formula is \
well known in Euclidean differential geometry.\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.670569752437118*^9, 3.670569851962627*^9}, {
   3.6705699984654503`*^9, 3.6705701099605207`*^9}, 3.67057019228787*^9, 
   3.67057027579918*^9, {3.670570333886518*^9, 3.6705703700226173`*^9}, 
   3.702532608917102*^9, {3.7025600704948273`*^9, 3.7025600704970093`*^9}, 
   3.708495962461677*^9},ExpressionUUID->"7e0394ca-9ebb-49d3-b3a0-\
06b73b06c012"],

Cell[TextData[{
 "Formula (1) used as the definition of the \[CapitalGamma]-symbols, the ",
 StyleBox["Christoffel symbol of the second kind ",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox["of a",
  FontColor->GrayLevel[0]],
 StyleBox[" pseudo-Riemannian space, ",
  FontColor->RGBColor[0, 0, 1]],
 "translated into  a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " module is contained in the package riemannv2.m. This definition appeared \
already in the notebook euklsfv3.nb of [",
 ButtonBox["E",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "D",
 ButtonBox["G",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "] where the absolute differential for immersions into the Euclidean space \
is introduced projecting the total differential into the tangent spaces of \
the immersion:"
}], "Text",
 CellChangeTimes->{{3.661761382537861*^9, 3.661761631199931*^9}, 
   3.661761803646393*^9, {3.661765047225173*^9, 3.661765076480112*^9}, {
   3.661765185519442*^9, 3.661765246927232*^9}, {3.661765285658181*^9, 
   3.661765394342267*^9}, 3.661765862249572*^9, {3.661777714597129*^9, 
   3.661777732723928*^9}, {3.661779391764028*^9, 3.661779537892495*^9}, {
   3.661955908205174*^9, 3.6619559407727013`*^9}, {3.6623861371983223`*^9, 
   3.6623861604133463`*^9}, 3.662446909834491*^9, 3.669112264111836*^9, {
   3.669726891786569*^9, 3.669726899498115*^9}, 3.67591763551158*^9, 
   3.6759177318782454`*^9, {3.7025590022290993`*^9, 3.702559003173086*^9}, {
   3.7042545045173798`*^9, 3.704254588220595*^9}, {3.70425462234809*^9, 
   3.704254650355102*^9}, {3.704254687501067*^9, 
   3.7042546958033133`*^9}},ExpressionUUID->"5d6939ad-d95d-4bb1-96b2-\
1a3d746b954a"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]g"}]], "Input",
 CellChangeTimes->{{3.661765435012085*^9, 3.661765440147094*^9}, 
   3.7042547628194647`*^9},ExpressionUUID->"5890b25b-2bdf-4634-9cd5-\
a4b144a50ea1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]g", "[", 
       RowBox[{"g_:", 
        RowBox[{"rg", "[", "]"}]}], "]"}], "[", 
      RowBox[{"k_", ",", "i_", ",", "j_"}], "]"}], "[", "x__", "]"}], " ", ":=",
     " ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"invg", ",", "gg", ",", 
        RowBox[{"dd", " ", "=", " ", 
         RowBox[{"Length", "[", 
          RowBox[{"{", "x", "}"}], "]"}]}], ",", "h", ",", "l", ",", "u"}], 
       "}"}], ",", "\n", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"Not", "[", 
          RowBox[{"MemberQ", "[", 
           RowBox[{
            RowBox[{"Tuples", "[", 
             RowBox[{
              RowBox[{"Range", "[", 
               RowBox[{"Length", "[", 
                RowBox[{"{", "x", "}"}], "]"}], "]"}], ",", "3"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"k", ",", "i", ",", "j"}], "}"}]}], "]"}], "]"}], ",", 
         "\n", 
         RowBox[{
          RowBox[{"Print", "[", 
           RowBox[{
           "\"\<Invalid indices \>\"", ",", " ", "k", ",", "\"\<,\>\"", ",", 
            "i", ",", "\"\<,\>\"", ",", "j"}], "]"}], ";", " ", 
          RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\n", 
       RowBox[{"gg", ":=", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"g", "[", "x", "]"}], "[", 
           RowBox[{"h", ",", "l"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"h", ",", "dd"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"l", ",", "dd"}], "}"}]}], "]"}]}], ";", "\n", 
       RowBox[{"invg", " ", "=", " ", 
        RowBox[{"Inverse", "[", "gg", "]"}]}], ";", "\n", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"invg", "[", 
           RowBox[{"[", 
            RowBox[{"k", ",", "h"}], "]"}], "]"}], "*", 
          RowBox[{
           RowBox[{"chr", "[", 
            RowBox[{"h", ",", "i", ",", "j", ",", "g"}], "]"}], "[", "x", 
           "]"}]}], ",", " ", 
         RowBox[{"{", 
          RowBox[{"h", ",", "dd"}], "}"}]}], "]"}]}]}], "]"}]}], ";"}], 
  "\n"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.704255019391468*^9, 3.70425502315786*^9}, {
  3.7042550673201427`*^9, 
  3.7042550766805887`*^9}},ExpressionUUID->"bbb40af1-bcb2-463a-bd62-\
7677f721a0e2"],

Cell[BoxData[
 RowBox[{"?", "Inverse"}]], "Input",
 CellChangeTimes->{{3.662386224173827*^9, 3.662386227549506*^9}, 
   3.704255154190186*^9},ExpressionUUID->"cfba3574-a772-4f1c-96f7-\
fb4a20db3051"],

Cell["Here", "Text",
 CellChangeTimes->{{3.6617659611846113`*^9, 
  3.6617659622082767`*^9}},ExpressionUUID->"622607bd-8c0d-44ee-8c5f-\
8976e8a613a4"],

Cell[BoxData[
 RowBox[{"?", "chr"}]], "Input",
 CellChangeTimes->{{3.661765799452292*^9, 3.661765823306634*^9}, 
   3.704255252589261*^9},ExpressionUUID->"cfad8cee-28b9-4a02-8923-\
223f95927566"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"chr", "[", 
     RowBox[{"j_", ",", "k_", ",", "i_", ",", 
      RowBox[{"g_:", 
       RowBox[{"rg", "[", "]"}]}]}], "]"}], "[", "x__", "]"}], ":=", "\n", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"g", "[", "x", "]"}], "[", 
         RowBox[{"i", ",", "j"}], "]"}], ",", 
        RowBox[{
         RowBox[{"{", "x", "}"}], "[", 
         RowBox[{"[", "k", "]"}], "]"}]}], "]"}], " ", "+", 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"g", "[", "x", "]"}], "[", 
         RowBox[{"j", ",", "k"}], "]"}], ",", 
        RowBox[{
         RowBox[{"{", "x", "}"}], "[", 
         RowBox[{"[", "i", "]"}], "]"}]}], "]"}], " ", "-", 
      RowBox[{"D", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"g", "[", "x", "]"}], "[", 
         RowBox[{"k", ",", "i"}], "]"}], ",", 
        RowBox[{
         RowBox[{"{", "x", "}"}], "[", 
         RowBox[{"[", "j", "]"}], "]"}]}], "]"}]}], ")"}], "/", "2"}]}], 
  ";"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.704255019391468*^9, 3.70425502315786*^9}, 
   3.7042550673201427`*^9},ExpressionUUID->"eddbbb05-2c2f-4b59-b97c-\
57af282eba10"],

Cell["is defined by the second factor on the right side of (1). ", "Text",
 CellChangeTimes->{{3.661766026983899*^9, 3.661766032255951*^9}, {
   3.661766073319765*^9, 3.6617661057513037`*^9}, {3.66176618685464*^9, 
   3.661766189542612*^9}, 3.6617778103791637`*^9, {3.661777973321579*^9, 
   3.661778205359243*^9}, {3.661778303366715*^9, 3.6617783249263897`*^9}, 
   3.661778424077301*^9, 3.661948407733389*^9, {3.6619484879891253`*^9, 
   3.661948550012274*^9}, {3.662386351202052*^9, 3.662386352281724*^9}, 
   3.7042554173958893`*^9},ExpressionUUID->"180b5732-5335-47e3-8f90-\
3210b23b84ce"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". Pseudo-Euclidean space. Its Levi-Civita connection is the flat linear \
connection."
}], "Text",
 CellChangeTimes->{{3.704262750363442*^9, 
  3.704262870930827*^9}},ExpressionUUID->"a53e651d-39b5-40f7-ac07-\
d1f7bf1433da"],

Cell[BoxData[
 RowBox[{"?", "lor"}]], "Input",
 CellChangeTimes->{{3.704262890126904*^9, 3.704262894890649*^9}, {
  3.70426321623442*^9, 
  3.704263219296179*^9}},ExpressionUUID->"198905c3-68e8-4469-b115-\
bb79b12ed59c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"glor", "[", "x__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{"lor", "[", 
   RowBox[{"i", ",", "j"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704263235236096*^9, 
  3.7042633377831507`*^9}},ExpressionUUID->"43d5a27a-99dc-4ac8-929d-\
f7625b9a19e3"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RG", "[", "glor", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704263357778017*^9, 
  3.704263385286888*^9}},ExpressionUUID->"313866fa-dc0b-48a8-9347-\
063dc10e9c03"],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]g", "[", "glor", "]"}], "[", 
     RowBox[{"i", ",", "j", ",", "k"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "dim"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "dim"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "dim"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.704263398342967*^9, 3.7042634067021103`*^9}, {
  3.704263473525877*^9, 3.704263531157753*^9}, {3.7042636039812613`*^9, 
  3.7042637155408897`*^9}, {3.704263806452302*^9, 
  3.7042639675793962`*^9}},ExpressionUUID->"8d83da17-b4cf-4fcd-adf4-\
a1976665e86b"],

Cell["\[FilledSmallSquare]", "Text",
 CellChangeTimes->{{3.7042640755330973`*^9, 
  3.704264089953473*^9}},ExpressionUUID->"ac4e7aed-dbdc-46b3-b66e-\
2af54180a2df"],

Cell["\<\
Already in the notebook euklsfv3.nb of [EDG] we defined an absolute \
differential DD for tangent vector field of immersions  in an outer way using \
the above mentioned projection of the total differential and Christoffel \
symbols for surfaces in the Euclidean space. We leave it to the user to \
verify that these definitions are special cases of the concepts presented \
more generally now.\
\>", "Text",
 CellChangeTimes->{{3.661778639703635*^9, 3.661778739042396*^9}, {
   3.661778817161541*^9, 3.66177885584896*^9}, {3.661779656346148*^9, 
   3.661779656705184*^9}, {3.66244813828094*^9, 3.662448148928875*^9}, 
   3.662893491337946*^9, {3.669112264221987*^9, 3.669112264261044*^9}, {
   3.669727345142023*^9, 3.669727377037545*^9}, {3.702559366968299*^9, 
   3.702559368440381*^9}, {3.7042555959791307`*^9, 
   3.704255754234043*^9}},ExpressionUUID->"351cc2b2-1810-43a2-89bc-\
ecdde9640d6c"],

Cell[TextData[{
 "We consider an immersion smf:",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["M", "m"]}], TraditionalForm]],ExpressionUUID->
  "77d31db4-03ad-4f75-b335-2a4b0c012fcf"],
 "\[LongRightArrow] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["E", "n"], TraditionalForm]],ExpressionUUID->
  "03b4e22b-dce4-484b-bd90-e06da3f7ad44"],
 " into the Euclidean space with its induced Riemannian metric"
}], "Text",
 CellChangeTimes->{{3.6617789741919928`*^9, 3.6617791248883038`*^9}, {
  3.661779161498863*^9, 3.661779212733871*^9}, {3.7042641110829678`*^9, 
  3.704264111083063*^9}},ExpressionUUID->"8a1c7222-d857-468a-a1de-\
a729d36c5d65"],

Cell[BoxData[
 RowBox[{"?", "RGT"}]], "Input",
 CellChangeTimes->{{3.661778475080469*^9, 3.6617784832678633`*^9}, {
  3.6617785324059553`*^9, 3.661778532876058*^9}, {3.662130007025405*^9, 
  3.662130008408992*^9}},ExpressionUUID->"714439da-152a-4bb7-8142-\
646bbb6bb969"],

Cell[BoxData[
 RowBox[{"?", "rgt"}]], "Input",
 CellChangeTimes->{{3.662902342880908*^9, 
  3.662902349956698*^9}},ExpressionUUID->"633c6770-3381-4970-9de9-\
285673555b0f"],

Cell[BoxData[
 RowBox[{"Context", "[", "rgt", "]"}]], "Input",
 CellChangeTimes->{{3.665034498869282*^9, 
  3.665034512797179*^9}},ExpressionUUID->"aa682f77-40da-4a95-a9c9-\
2fe252d1944d"],

Cell["We consider the surface", "Text",
 CellChangeTimes->{{3.661780749542766*^9, 3.661780782167144*^9}, {
   3.661780820326054*^9, 3.661780822238615*^9}, {3.661780919669557*^9, 
   3.66178092373372*^9}, {3.661780973981366*^9, 3.661780975957322*^9}, {
   3.6617810677401733`*^9, 3.6617810718684*^9}, 
   3.704255967304439*^9},ExpressionUUID->"0011247f-39ba-4e87-b683-\
9e05d60d5e00"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", 
  RowBox[{"smf", "[", 
   RowBox[{"u1", ",", "u2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.661780824980423*^9, 3.661780825846779*^9}, {
  3.661780991853307*^9, 
  3.661781045996917*^9}},ExpressionUUID->"754aa680-71d6-470e-ba59-\
f0668ad27814"],

Cell["with the induced Riemannian metric", "Text",
 CellChangeTimes->{{3.661781122486554*^9, 
  3.661781156340637*^9}},ExpressionUUID->"27aef91f-98a3-4333-b1c7-\
4eff01b6dc5a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGT", "[", "smf", "]"}], "[", 
  RowBox[{"u1", ",", "u2"}], "]"}]], "Input",
 CellChangeTimes->{{3.6621328472859898`*^9, 3.6621328594076643`*^9}, {
   3.66213303858252*^9, 3.662133042718524*^9}, {3.662133072959144*^9, 
   3.662133103070414*^9}, {3.6621911586808558`*^9, 3.662191163248475*^9}, 
   3.662192162984638*^9, {3.662198141126223*^9, 
   3.6621981481008587`*^9}},ExpressionUUID->"9c0c68c5-6fed-4527-8887-\
fe706d1c9c5d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "[", 
   RowBox[{"[", 
    RowBox[{"1", ",", "2"}], "]"}], "]"}], "==", 
  RowBox[{
   RowBox[{
    RowBox[{"rgt", "[", "smf", "]"}], "[", 
    RowBox[{"u1", ",", "u2"}], "]"}], "[", 
   RowBox[{"2", ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6621911812077713`*^9, 3.66219121957708*^9}, {
   3.662191312592332*^9, 3.662191331183317*^9}, 3.6621921284174633`*^9, 
   3.662198170182901*^9, {3.6624482083044987`*^9, 3.6624482105926733`*^9}, {
   3.662902416854587*^9, 3.662902431486257*^9}, {3.6650351398897743`*^9, 
   3.665035144081457*^9}},ExpressionUUID->"92da660f-d902-4871-9be6-\
a029c509918d"],

Cell[BoxData[
 RowBox[{"?", "rgt"}]], "Input",
 CellChangeTimes->{{3.704256193011846*^9, 3.704256195030099*^9}, {
  3.704256237720531*^9, 
  3.70425623905368*^9}},ExpressionUUID->"929fb7e3-a76c-4407-b667-\
4e7296588931"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rgt", "[", "smf", "]"}], "[", 
   RowBox[{"u1", ",", "u2"}], "]"}], "[", 
  RowBox[{"1", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.7042564078720293`*^9, 3.70425643362061*^9}, {
  3.7042583424335747`*^9, 
  3.70425835642316*^9}},ExpressionUUID->"4679c75b-f3fe-46f2-9ba6-\
dda532d69f8d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGT", "[", "smf", "]"}], "[", 
   RowBox[{"u1", ",", "u2"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"RG", "[", 
    RowBox[{"rgt", "[", "smf", "]"}], "]"}], "[", 
   RowBox[{"u1", ",", "u2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.66178119883016*^9, 3.661781230859469*^9}, {
  3.661781281738987*^9, 3.661781293202855*^9}, {3.662903000910047*^9, 
  3.662903004357477*^9}, {3.662903048651309*^9, 3.6629030909123287`*^9}, {
  3.704256121647295*^9, 3.704256157758746*^9}, {3.704258399800077*^9, 
  3.704258407694468*^9}},ExpressionUUID->"fc59c945-ddab-448e-a030-\
c4c837c49d71"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", 
  RowBox[{
   RowBox[{"RGT", "[", "smf", "]"}], "[", 
   RowBox[{"u1", ",", "u2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.662303516234413*^9, 3.662303529091091*^9}, 
   3.662903114206719*^9},ExpressionUUID->"7f202afd-20ce-4d37-b802-\
fa6e262a11bd"],

Cell["The Levi-Civita connection of the immersion ff is", "Text",
 CellChangeTimes->{{3.662449641253025*^9, 3.6624496668450127`*^9}, {
  3.66290258022978*^9, 3.662902594125807*^9}, {3.7042585301575127`*^9, 
  3.704258532357583*^9}},ExpressionUUID->"53de238a-8cd1-4912-aba5-\
52083cd755d9"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]f"}]], "Input",
 CellChangeTimes->{{3.66290692503286*^9, 
  3.6629069364120483`*^9}},ExpressionUUID->"c33a9761-b55d-49d8-983c-\
5c3cbbf05192"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]f", "[", "ff_", "]"}], "[", 
     RowBox[{"h_", ",", "i_", ",", "j_"}], "]"}], "[", "u__", "]"}], ":=", 
   " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalGamma]g", "[", 
       RowBox[{"rgt", "[", "ff", "]"}], "]"}], "[", 
      RowBox[{"h", ",", "i", ",", "j"}], "]"}], "[", 
     RowBox[{"rgt", "[", "ff", "]"}], "]"}], "[", "u", "]"}]}], 
  "\[LineSeparator]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6617818969091063`*^9, 3.66178192669237*^9}, {
   3.661781961540745*^9, 3.661782056683663*^9}, 3.66195694459608*^9, {
   3.6619572970949097`*^9, 3.661957337253935*^9}, {3.662203684045409*^9, 
   3.662203705452559*^9}, 3.662448479358225*^9, {3.662451899029317*^9, 
   3.662451922942337*^9}, {3.662452014412271*^9, 3.662452073540215*^9}, {
   3.662452105683856*^9, 3.6624523641537724`*^9}, {3.662452410005486*^9, 
   3.662452415744884*^9}, {3.662452488233354*^9, 3.662452520264963*^9}, {
   3.6624527568307133`*^9, 3.662452804197858*^9}, 3.662453197306637*^9, 
   3.662880870643223*^9, {3.662902519981741*^9, 3.662902524154255*^9}, {
   3.662902605757997*^9, 3.6629026697983923`*^9}, {3.6629035843223343`*^9, 
   3.662903618248233*^9}, 3.662906043690123*^9, 3.663483465681418*^9, 
   3.6650354311904*^9},ExpressionUUID->"b1a095ec-6844-4bc9-befc-663287384c45"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]f", "[", "smf", "]"}], "[", 
   RowBox[{"1", ",", "1", ",", "2"}], "]"}], "[", 
  RowBox[{"u1", ",", "u2"}], "]"}]], "Input",
 CellChangeTimes->{{3.662902695474374*^9, 3.662902726550888*^9}, {
   3.6629027620713863`*^9, 3.662902834814323*^9}, 3.662903183395236*^9, 
   3.662903647537198*^9, 
   3.662903697406439*^9},ExpressionUUID->"a85e960f-22e8-4830-9af3-\
d087bf73a2f3"],

Cell["\<\
The coordinates of a tangent vector field in a given chart are\
\>", "Text",
 CellChangeTimes->{{3.662448869314443*^9, 3.662448898586068*^9}, 
   3.662449014369245*^9, {3.704258692868565*^9, 
   3.704258698428203*^9}},ExpressionUUID->"07678d2a-9eb0-4268-a228-\
401ab8cc8bef"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"u1", ",", "u2"}], "]"}]], "Input",
 CellChangeTimes->{
  3.662448840523101*^9},ExpressionUUID->"75cc2d62-2a42-4a93-9383-\
8fd006161528"],

Cell["The corresponding tensor field is", "Text",
 CellChangeTimes->{{3.704258726911271*^9, 
  3.704258749853075*^9}},ExpressionUUID->"3254d41f-2ca3-4b13-b74b-\
cef0665c6678"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vX", "[", 
   RowBox[{"u1_", ",", "u2_"}], "]"}], " ", ":=", " ", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"u1", ",", "u2"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6617823314679213`*^9, 3.661782373843379*^9}, {
  3.661782838141383*^9, 
  3.661782898604289*^9}},ExpressionUUID->"4f676b5c-52f6-40e3-b7a5-\
a6562525c80c"],

Cell[BoxData[
 RowBox[{"vX", "[", 
  RowBox[{"u1", ",", "u2"}], "]"}]], "Input",
 CellChangeTimes->{{3.661782916044217*^9, 
  3.66178292480357*^9}},ExpressionUUID->"9abc857d-93e3-4d9d-af0e-\
1e5209045831"],

Cell["\<\
The tensor coordinates of this field coincide with the vector coordinates:\
\>", "Text",
 CellChangeTimes->{{3.662449074312674*^9, 
  3.662449108344393*^9}},ExpressionUUID->"4b319eff-d0d6-48e7-99a0-\
18c619bec183"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", 
  RowBox[{"vX", "[", 
   RowBox[{"u1", ",", "u2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.662449138357437*^9, 
  3.6624492002482443`*^9}},ExpressionUUID->"88260fde-3d88-431c-9cdf-\
463cc1d0809f"],

Cell[BoxData[
 RowBox[{"?", "nabla"}]], "Input",
 CellChangeTimes->{{3.661783442153122*^9, 3.661783443168088*^9}, 
   3.663078278147311*^9},ExpressionUUID->"07a3be50-629f-4431-8fb2-\
c41e71d160a2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vvX", "[", 
   RowBox[{"u1_", ",", "u2_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"nabla", "[", 
    RowBox[{
     RowBox[{"\[CapitalGamma]f", "[", "smf", "]"}], ",", "vX"}], "]"}], "[", 
   RowBox[{"u1", ",", "u2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.661782416602684*^9, 3.661782422952323*^9}, {
   3.6617824562565928`*^9, 3.661782515592368*^9}, {3.661782779582272*^9, 
   3.661782804069817*^9}, 3.661783002419964*^9, {3.661783171065557*^9, 
   3.661783184105816*^9}, {3.661783224576888*^9, 3.6617832457685337`*^9}, {
   3.661783523999393*^9, 3.661783580581984*^9}, 3.6623881900787077`*^9, {
   3.662388252725892*^9, 3.6623882690299873`*^9}, {3.6624485810127277`*^9, 
   3.66244858937313*^9}, {3.66290337738632*^9, 3.662903388625296*^9}, 
   3.6629038375555897`*^9, 
   3.663078278183826*^9},ExpressionUUID->"faa836e9-8395-4e22-af30-\
9d36b442bab6"],

Cell[BoxData[
 RowBox[{"vvX", "[", 
  RowBox[{"u1", ",", "u2"}], "]"}]], "Input",
 CellChangeTimes->{{3.6617833402883873`*^9, 3.66178338096005*^9}, {
   3.6617836574858503`*^9, 3.661783669277935*^9}, {3.662129863651156*^9, 
   3.6621298654350433`*^9}, 3.662388454525219*^9, 
   3.662903858138383*^9},ExpressionUUID->"a7c584b1-66d8-4c8d-89c3-\
a5602d19319c"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"vvX", "[", 
    RowBox[{"u1", ",", "u2"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "1", "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.662903878884108*^9, 3.662903880433857*^9}, {
  3.704259049563691*^9, 
  3.704259086505393*^9}},ExpressionUUID->"0212bc59-cd4e-4492-993b-\
149a41482505"],

Cell[TextData[{
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". The sphere of radius r."
}], "Text",
 CellChangeTimes->{{3.704259347643931*^9, 3.7042593997584553`*^9}, {
  3.704264262136209*^9, 
  3.704264306599875*^9}},ExpressionUUID->"113007e9-5f6a-4580-b7ed-\
3c872dda95ee"],

Cell[BoxData[
 RowBox[{"?", "spher"}]], "Input",
 CellChangeTimes->{{3.7042592757821817`*^9, 
  3.7042593045755663`*^9}},ExpressionUUID->"dd862fe5-5511-464b-a3f8-\
e7b25eb20cd8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGT", "[", 
   RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.704259414405099*^9, 
  3.704259436182358*^9}},ExpressionUUID->"305ca37a-52d4-4f4e-abe7-\
f85adb225358"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vvX", "]"}], ";", 
  RowBox[{
   RowBox[{"vvX", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "=", 
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{
      RowBox[{"\[CapitalGamma]f", "[", 
       RowBox[{"spher", "[", "r", "]"}], "]"}], ",", "vX"}], "]"}], "[", 
    RowBox[{"u", ",", " ", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.704259477971488*^9, 
  3.704259601989872*^9}},ExpressionUUID->"245c42f7-5086-4d84-878f-\
85fa657c06b3"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"vvX", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", "1", "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.70425962137323*^9, 
  3.7042596843964443`*^9}},ExpressionUUID->"190db4de-ac4d-4824-9219-\
fbb51f544565"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"tensorToArray", "[", 
    RowBox[{"vvX", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellChangeTimes->{{3.70425962137323*^9, 3.7042596843964443`*^9}, {
  3.704259730225726*^9, 3.7042597872672167`*^9}, {3.704259873428401*^9, 
  3.704259876530753*^9}},ExpressionUUID->"0b2e6821-1d6c-4db0-801d-\
a81bddbbd010"],

Cell[TextData[{
 StyleBox["Example 3",
  FontColor->RGBColor[0, 0, 1]],
 ". The 3-sphere  in the 4-space."
}], "Text",
 CellChangeTimes->{{3.704260275686552*^9, 3.704260376246072*^9}, {
  3.704261313278308*^9, 3.70426133007026*^9}, {3.704264376495303*^9, 
  3.7042643786315804`*^9}},ExpressionUUID->"48e3ce41-71a7-4e1c-a6dc-\
23b7aedf9211"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}]], "Input",
 CellChangeTimes->{{3.704260389922105*^9, 
  3.704260411279749*^9}},ExpressionUUID->"b0a6fa9a-0c8f-4e43-bbfb-\
098c840e0e91"],

Cell["The metric tensor is", "Text",
 CellChangeTimes->{{3.704261352605932*^9, 
  3.704261363037475*^9}},ExpressionUUID->"77622bf5-dafd-4e4b-83d8-\
9f93f053f35d"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"RGT", "[", 
    RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704259414405099*^9, 3.704259436182358*^9}, {
  3.704260485159027*^9, 3.704260521294772*^9}, {3.7042605540380297`*^9, 
  3.704260564454604*^9}},ExpressionUUID->"76c663b7-1b2f-4a06-a350-\
abeba30b386f"],

Cell["We consider the vector field", "Text",
 CellChangeTimes->{{3.704261370650503*^9, 
  3.7042613952959337`*^9}},ExpressionUUID->"ea117296-1ba2-4ca9-bdb1-\
d49b62d4197a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
  RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}]], "Input",
 CellChangeTimes->{{3.704260055858068*^9, 3.704260094736692*^9}, {
   3.704260129681863*^9, 3.704260179079595*^9}, 3.7042602559613457`*^9, 
   3.704260468029859*^9},ExpressionUUID->"ece8ae50-6d13-495f-83aa-\
8433334384cb"],

Cell["and the corresponding tensor field", "Text",
 CellChangeTimes->{{3.704261411749486*^9, 
  3.704261425461557*^9}},ExpressionUUID->"ef106d2a-534b-4945-91f6-\
142321951114"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vX", "[", 
   RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}], "=", 
  RowBox[{"vecTotensor", "[", 
   RowBox[{
    RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
    RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704260689883997*^9, 
  3.704260759067214*^9}},ExpressionUUID->"35644a50-285b-4745-b4e9-\
235a7d07da7a"],

Cell[BoxData[
 RowBox[{"coords", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.70426083507994*^9, 
  3.7042608558343363`*^9}},ExpressionUUID->"11cc9d9d-6ab3-498b-b356-\
e57bb466f4e7"],

Cell["The gradient tensor is", "Text",
 CellChangeTimes->{{3.704261446800233*^9, 
  3.7042614698472977`*^9}},ExpressionUUID->"36ebbad5-2dd7-462a-ae28-\
0bce6d88f0a9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vvX", "]"}], ";", 
  RowBox[{
   RowBox[{"vvX", "[", 
    RowBox[{"s1_", ",", "s2_", ",", "s3_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"nabla", "[", 
     RowBox[{
      RowBox[{"\[CapitalGamma]f", "[", 
       RowBox[{"spher", "[", "r", "]"}], "]"}], ",", "vX"}], "]"}], "[", 
    RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.704259477971488*^9, 3.704259601989872*^9}, {
   3.7042605977491283`*^9, 3.704260607492436*^9}, 3.704260644037984*^9, 
   3.7042611693454027`*^9},ExpressionUUID->"6259d59d-2ab5-4f72-870e-\
804e5d615c10"],

Cell[BoxData[
 RowBox[{"vvX", "[", 
  RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}]], "Input",
 CellChangeTimes->{{3.704261177617496*^9, 
  3.7042611888725*^9}},ExpressionUUID->"f60674af-0847-4c50-ab21-13de0e19799a"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "[", 
   RowBox[{
    RowBox[{"{", "1", "}"}], ",", 
    RowBox[{"{", "3", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704261199602089*^9, 3.7042612198651457`*^9}, {
  3.704261501574494*^9, 
  3.704261511541359*^9}},ExpressionUUID->"9de3350e-cd09-4b58-9c6c-\
b14a967101dd"],

Cell["Here is the coordinate Array of the gradient tensor:", "Text",
 CellChangeTimes->{{3.704262278782568*^9, 
  3.704262324574795*^9}},ExpressionUUID->"efd416f3-de22-4c82-93f3-\
f08d87c262aa"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"tensorToArray", "[", 
   RowBox[{"vvX", "[", 
    RowBox[{"s1", ",", "s2", ",", "s3"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.70425962137323*^9, 3.7042596843964443`*^9}, {
   3.704259730225726*^9, 3.7042597872672167`*^9}, {3.704259873428401*^9, 
   3.704259876530753*^9}, 3.7042616415406513`*^9, 
   3.704261673140689*^9},ExpressionUUID->"d36feeef-c51b-4e3f-8acf-\
afa04492543b"],

Cell["OK. \[FilledRectangle]", "Text",
 CellChangeTimes->{{3.704262381246108*^9, 
  3.704262398893804*^9}},ExpressionUUID->"a85b513e-e6a2-4b31-9df1-\
5d483027b7b7"],

Cell[TextData[{
 StyleBox["Example 4",
  FontColor->RGBColor[0, 0, 1]],
 ". The Gradient of the metric tensor."
}], "Text",
 CellChangeTimes->{{3.7042644565264482`*^9, 
  3.704264510078416*^9}},ExpressionUUID->"1331139a-0881-449f-9df9-\
5e3d68437d2d"],

Cell[BoxData[
 RowBox[{"?", "RGT"}]], "Input",
 CellChangeTimes->{{3.704265378283255*^9, 
  3.7042653796336*^9}},ExpressionUUID->"659773af-5cdf-4158-8f9d-9e251d3cbc14"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGT", "[", 
   RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7042645143193808`*^9, 3.704264546950294*^9}, 
   3.704265354170568*^9, {3.7042654011838207`*^9, 
   3.704265415072238*^9}},ExpressionUUID->"f93b9a4b-88d7-497d-b67a-\
a1ee2dcb2050"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGTT", "[", "x__", "]"}], ":=", " ", 
  RowBox[{"ArrayTotensor", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"RGT", "[", 
      RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "x", "]"}], ",", "2"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.704265451467166*^9, 3.70426549726652*^9}, {
  3.7042655365675907`*^9, 3.704265548359264*^9}, {3.704265790087902*^9, 
  3.704265816334309*^9}},ExpressionUUID->"dec74bf8-3e4b-4233-bf36-\
d84367842070"],

Cell[BoxData[
 RowBox[{"RGTT", "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704265557858389*^9, 
  3.704265570822809*^9}},ExpressionUUID->"418e0011-9589-4c95-93f2-\
9d3e165e600d"],

Cell[BoxData[
 RowBox[{"?", "nabla"}]], "Input",
 CellChangeTimes->{{3.704264799249785*^9, 3.7042648016360607`*^9}, {
  3.704265661007728*^9, 
  3.704265670222609*^9}},ExpressionUUID->"61a269ab-a520-40d1-b2be-\
d1fdbe245e65"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{
    RowBox[{"\[CapitalGamma]f", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], ",", "RGTT"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704264627237688*^9, 3.704264660893743*^9}, {
  3.704264777559477*^9, 3.7042647893726597`*^9}, {3.704264824196978*^9, 
  3.704264848756278*^9}, {3.704264888964073*^9, 3.704264917419538*^9}, {
  3.704265185195033*^9, 3.7042651859776917`*^9}, {3.70426589211016*^9, 
  3.704265936051649*^9}},ExpressionUUID->"970eed8a-7c1b-448a-ab67-\
1781bb9b6e7a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorToArray", "[", "%", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.704264938375512*^9, 3.704264950043355*^9}, 
   3.704266324361834*^9},ExpressionUUID->"fad48116-b7a1-43d9-8db5-\
76a2092e3228"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "[", "1", "]"}], "|", 
      RowBox[{"x", "[", "2", "]"}], "|", 
      RowBox[{"x", "[", "3", "]"}]}], ",", "Reals"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.704266013443944*^9, 3.704266048323366*^9}, {
  3.704266080699061*^9, 3.704266149178842*^9}, {3.704266181220503*^9, 
  3.7042662002767067`*^9}},ExpressionUUID->"be9a7f10-fc5a-4da8-9588-\
0767a74b7b7f"],

Cell[TextData[{
 "Clearly, by the Levi-Civita Theorem we have  \[Del]RG = nulltensor; this \
equation one sometimes cites as the \[OpenCurlyDoubleQuote]Lemma of ",
 ButtonBox["Ricci",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://de.wikipedia.org/wiki/Gregorio_Ricci-Curbastro"], None},
  ButtonNote->"https://de.wikipedia.org/wiki/Gregorio_Ricci-Curbastro"],
 "\[CloseCurlyDoubleQuote]."
}], "Text",
 CellChangeTimes->{{3.704266405616301*^9, 3.704266419304101*^9}, {
  3.7042664893997927`*^9, 3.704266526359453*^9}, {3.704266576823141*^9, 
  3.704266665334495*^9}, {3.704286978698495*^9, 
  3.7042869787008963`*^9}},ExpressionUUID->"e012fde6-a20c-42d7-b5f4-\
13be03a6ce3a"],

Cell["OK. \[FilledRectangle]", "Text",
 CellChangeTimes->{{3.704262381246108*^9, 
  3.704262398893804*^9}},ExpressionUUID->"4948fc32-08c2-4231-b732-\
811fcd136130"]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2. The Curvature Tensor", "Subsection",
 CellChangeTimes->{{3.662617109080329*^9, 3.662617129096106*^9}, {
   3.704266694710362*^9, 3.704266696350065*^9}, 
   3.704287038005934*^9},ExpressionUUID->"fdc14203-1f8d-4821-8649-\
13ec4fe915b9"],

Cell[CellGroupData[{

Cell["4.2.1. Ricci\[CloseCurlyQuote]s Lemma", "Subsubsection",
 CellChangeTimes->{{3.663511296663538*^9, 3.6635113753506947`*^9}, 
   3.66972810036901*^9, {3.6697961979264183`*^9, 3.669796202709854*^9}, 
   3.7042870481478853`*^9, {3.704345721396508*^9, 3.704345770059887*^9}, 
   3.708496530602133*^9, 
   3.7084980042625303`*^9},ExpressionUUID->"19fe0e55-b5a7-4717-8115-\
fcbe85092cbf"],

Cell["\<\
As a test of the defined Modules we verify the Lemma of Ricci\
\>", "Text",
 CellChangeTimes->{{3.704345815987596*^9, 3.704345855723279*^9}, 
   3.704346014554368*^9},ExpressionUUID->"93a1a3c3-2366-4afd-8043-\
a0bd02f9645a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Del]", "h"], 
   SubscriptBox["g", "ij"]}], "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.7043459091621313`*^9, 
  3.704345973658448*^9}},ExpressionUUID->"498e5183-9ccc-4b06-81a7-\
65f96244289b"],

Cell["(see Theorem 4.1) in dimensions 2 - 5.", "Text",
 CellChangeTimes->{{3.70434603214919*^9, 3.704346057897647*^9}, {
  3.704346148361002*^9, 
  3.704346198625018*^9}},ExpressionUUID->"7d9fd0f9-64fc-4cd5-9926-\
f37cbd6a6281"],

Cell[BoxData[
 RowBox[{"dim", "=", "2"}]], "Input",
 CellChangeTimes->{{3.7042872793926497`*^9, 3.704287281057185*^9}, {
  3.704287599688031*^9, 
  3.7042876034787197`*^9}},ExpressionUUID->"3bb76997-e431-4181-b74b-\
69e2414f5775"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.6634818618868027`*^9, 
  3.663481868220975*^9}},ExpressionUUID->"64b11e51-a6b7-4044-ada2-\
fe542726c96d"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]g"}]], "Input",
 CellChangeTimes->{
  3.663481617868066*^9},ExpressionUUID->"d7cb96a7-f87c-478f-bd89-\
a191a2e374ff"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCtensor", "[", 
   RowBox[{"\[CapitalGamma]g", "[", 
    RowBox[{"rg", "[", "]"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663477842837433*^9, 3.6634778438284473`*^9}, {
   3.665043106760045*^9, 3.665043110887701*^9}, 3.665044221694536*^9, 
   3.665044263735827*^9, {3.665044310185071*^9, 
   3.6650443192803783`*^9}},ExpressionUUID->"ccb6f454-1d34-4d0a-bacc-\
0b2807087395"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "1"}], "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7042873558225937`*^9, 3.704287389336447*^9}, {
  3.70428762616803*^9, 3.704287632190637*^9}, {3.7042883060340548`*^9, 
  3.704288321049262*^9}},ExpressionUUID->"abfe0f8d-13de-4c04-9726-\
01a8c49581bb"],

Cell[BoxData[
 RowBox[{"?", "rg"}]], "Input",
 CellChangeTimes->{{3.665043356646996*^9, 
  3.665043357903337*^9}},ExpressionUUID->"9d783c07-6a93-4fd2-aae3-\
01d28fa2379b"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGm", "[", "x__", "]"}], ":=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"RG", "[", "]"}], "[", "x", "]"}], ",", "2"}], "]"}]}], ";", 
  RowBox[{"RGm", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704287721658629*^9, 3.7042879091005774`*^9}, {
  3.704287962652893*^9, 
  3.7042879639399548`*^9}},ExpressionUUID->"0fe63de1-b042-4cca-93e2-\
7c56ffc63744"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGm", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.7042879438758698`*^9, {3.704287974075782*^9, 
   3.7042879874324913`*^9}},ExpressionUUID->"d697fefb-c663-49f1-aa77-\
e4256d174eb1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{
    RowBox[{"\[CapitalGamma]g", "[", "]"}], ",", "RGm"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704288023638361*^9, 3.7042880383713093`*^9}, {
  3.704288081531061*^9, 
  3.704288129306827*^9}},ExpressionUUID->"05a28c49-8479-444d-ab7a-\
9de92af1d365"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"tensorToArray", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7042881608414*^9, 
  3.704288234730165*^9}},ExpressionUUID->"17759222-2396-43ce-95fe-\
15bafcffa5a6"],

Cell[TextData[StyleBox["dim= 4",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.7042883931761723`*^9, 3.704288396488029*^9}, 
   3.7043446327878*^9},ExpressionUUID->"2bccf7c7-13f1-43a7-a860-6dfa1a7ba3ef"],

Cell[BoxData[
 RowBox[{"dim", " ", "=", "4"}]], "Input",
 CellChangeTimes->{{3.704288415206204*^9, 3.7042884201284847`*^9}, 
   3.704288643696512*^9, 3.7042891418558407`*^9, 3.704290108829727*^9, 
   3.70429982651818*^9},ExpressionUUID->"17ce4c46-9231-4ace-8677-\
7a1863313090"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCtensor", "[", 
   RowBox[{"\[CapitalGamma]g", "[", 
    RowBox[{"rg", "[", "]"}], "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663477842837433*^9, 3.6634778438284473`*^9}, {
   3.665043106760045*^9, 3.665043110887701*^9}, 3.665044221694536*^9, 
   3.665044263735827*^9, {3.665044310185071*^9, 
   3.6650443192803783`*^9}},ExpressionUUID->"41d76c66-f965-4c04-9a67-\
943d26bd3806"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"%", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
      RowBox[{"{", "4", "}"}]}], "]"}], ",", 
    RowBox[{"TimeConstraint", "\[Rule]", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.7042873558225937`*^9, 3.704287389336447*^9}, {
   3.70428762616803*^9, 3.704287632190637*^9}, {3.7042883060340548`*^9, 
   3.704288321049262*^9}, {3.704288441607939*^9, 3.70428847333608*^9}, 
   3.704288679656266*^9, {3.70428904806561*^9, 3.704289068884533*^9}, 
   3.704289158172821*^9, {3.704290141550151*^9, 3.7042901739255047`*^9}, {
   3.704344703016818*^9, 
   3.704344722028338*^9}},ExpressionUUID->"4ed68f9f-8db7-4aef-8bbb-\
e7bbab148027"],

Cell["\<\
Mathematica v. 11.1.1needed 122.008 s to calculate a single coordinate of the \
general curvature tensor in dimension 4.\
\>", "Text",
 CellChangeTimes->{{3.7042895415280733`*^9, 3.704289628199397*^9}, {
  3.704345014137196*^9, 3.7043450267531357`*^9}, {3.70849786804101*^9, 
  3.708497889383232*^9}},ExpressionUUID->"91c5daf3-b10b-4bf3-b743-\
da56a6f7bd19"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGm", "[", "x__", "]"}], ":=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"RG", "[", "]"}], "[", "x", "]"}], ",", "2"}], "]"}]}], ";", 
  RowBox[{"RGm", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704287721658629*^9, 3.7042879091005774`*^9}, {
  3.704287962652893*^9, 
  3.7042879639399548`*^9}},ExpressionUUID->"0c1657f6-c380-4d5b-8973-\
839ee7814e40"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGm", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.7042879438758698`*^9, {3.704287974075782*^9, 3.7042879874324913`*^9}, 
   3.7042885436238422`*^9, {3.7043450464648314`*^9, 
   3.704345053873815*^9}},ExpressionUUID->"20877207-d484-49ad-a05c-\
85b58e2238e9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{
    RowBox[{"\[CapitalGamma]g", "[", "]"}], ",", "RGm"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704288023638361*^9, 3.7042880383713093`*^9}, {
  3.704288081531061*^9, 
  3.704288129306827*^9}},ExpressionUUID->"899a2174-7208-4d30-b730-\
ad8f3a615ef7"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"tensorToArray", "[", "%", "]"}], ",", 
    RowBox[{"TimeConstraint", "\[Rule]", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.7042881608414*^9, 3.704288234730165*^9}, {
  3.704288583239382*^9, 3.704288593799177*^9}, {3.70428969216019*^9, 
  3.704289706663082*^9}},ExpressionUUID->"3ab49ce7-f2f2-4751-9011-\
388f31d2ec9c"],

Cell["\<\
This result- the Lemma of Ricci in dimension 4 - was obtained using 18.428 s.\
\>", "Text",
 CellChangeTimes->{{3.7042899861407537`*^9, 3.70429008379624*^9}, 
   3.7043451305855627`*^9, 
   3.708497952320068*^9},ExpressionUUID->"c6bbbd5b-d46f-4395-bc97-\
aad5abd52838"],

Cell[TextData[StyleBox["dim= 5",
 FontColor->RGBColor[0, 0, 1]]], "Text",
 CellChangeTimes->{{3.704345180639723*^9, 
  3.7043451854159822`*^9}},ExpressionUUID->"254255ca-12fb-4280-aff8-\
bd8768ac1b23"],

Cell[BoxData[
 RowBox[{"dim", "=", "5"}]], "Input",
 CellChangeTimes->{{3.7043452383054323`*^9, 
  3.704345241447463*^9}},ExpressionUUID->"19527d26-459a-4fa4-b4f3-\
adb5747aefff"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGm", "[", "x__", "]"}], ":=", 
   RowBox[{"ArrayTotensor", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"RG", "[", "]"}], "[", "x", "]"}], ",", "2"}], "]"}]}], ";", 
  RowBox[{"RGm", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704287721658629*^9, 3.7042879091005774`*^9}, {
  3.704287962652893*^9, 
  3.7042879639399548`*^9}},ExpressionUUID->"41e338df-e08d-4b6a-a1fb-\
2ccb80e735ac"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGm", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.7042879438758698`*^9, {3.704287974075782*^9, 3.7042879874324913`*^9}, 
   3.7042885436238422`*^9, {3.7043450464648314`*^9, 3.704345053873815*^9}, 
   3.7043453724551563`*^9},ExpressionUUID->"195eec04-70bf-4302-bdf3-\
ec26fd456f29"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nabla", "[", 
   RowBox[{
    RowBox[{"\[CapitalGamma]g", "[", "]"}], ",", "RGm"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704288023638361*^9, 3.7042880383713093`*^9}, {
  3.704288081531061*^9, 
  3.704288129306827*^9}},ExpressionUUID->"66f88158-8523-451e-9e0c-\
15fa43afc4ad"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"tensorToArray", "[", "%", "]"}], ",", 
    RowBox[{"TimeConstraint", "\[Rule]", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.7042881608414*^9, 3.704288234730165*^9}, {
  3.704288583239382*^9, 3.704288593799177*^9}, {3.70428969216019*^9, 
  3.704289706663082*^9}},ExpressionUUID->"8ffd4b24-577b-4b28-85b9-\
cc775febebae"],

Cell["\<\
This result- the Lemma of Ricci in dimension 5 - was obtained using 215.596 s.\
\>", "Text",
 CellChangeTimes->{{3.7042899861407537`*^9, 3.70429008379624*^9}, 
   3.7043451305855627`*^9, 3.704345404125939*^9, {3.70434566505284*^9, 
   3.7043456659095097`*^9}, 
   3.708497983327338*^9},ExpressionUUID->"150652dd-3a21-421d-8c55-\
14aeb4425d68"]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.2.  Curvature of Immersions. Examples", "Subsubsection",
 CellChangeTimes->{{3.6634264420351143`*^9, 3.6634264529705267`*^9}, {
   3.663511863906427*^9, 3.6635118806489983`*^9}, 3.669796208366578*^9, 
   3.669818004477478*^9, {3.670649628369945*^9, 3.670649635865732*^9}, 
   3.704345736308922*^9, {3.7043472765616493`*^9, 3.704347287825356*^9}, 
   3.704692515661504*^9, 3.708498011663468*^9, 
   3.708498844241144*^9},ExpressionUUID->"664fdd2d-e31e-49f3-a455-\
18e1689f0283"],

Cell[BoxData["psrgt"], "Input",ExpressionUUID->"d0c28cff-542f-4b4d-8bd4-709213066919"],

Cell[TextData[{
 "Now we consider  the",
 StyleBox[" curvature of an immersion",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" f of manifolds into pseudo-Euclidean spaces. If the embedding \
space is Euclidean, its Riemannian metric - the first fundamental form - is ",
  FontColor->GrayLevel[0]],
 "defined by"
}], "Text",
 CellChangeTimes->{{3.6244573675114183`*^9, 3.624457369913822*^9}, {
   3.624458808236349*^9, 3.6244588092191505`*^9}, {3.6244588694976563`*^9, 
   3.6244591352285233`*^9}, 3.624459228485487*^9, {3.624459263975549*^9, 
   3.6244592902303953`*^9}, {3.6580649231398163`*^9, 3.658064975155493*^9}, {
   3.658065006291267*^9, 3.658065024794977*^9}, {3.658065075978606*^9, 
   3.658065087098549*^9}, 3.662622985265234*^9, {3.663141606500415*^9, 
   3.663141609156509*^9}, {3.663426296494563*^9, 3.663426305190179*^9}, 
   3.663426341072866*^9, {3.704347412513311*^9, 3.7043474570168123`*^9}, {
   3.7043477876220922`*^9, 3.7043479313412046`*^9}, {3.7043483569064093`*^9, 
   3.70434839738663*^9}, {3.705643654727603*^9, 
   3.70564365744726*^9}},ExpressionUUID->"4bb49ba7-6965-404f-b13d-\
6a3d4c7aa2b1"],

Cell[BoxData[
 RowBox[{"?", "rgt"}]], "Input",
 CellChangeTimes->{{3.6580650992662067`*^9, 3.658065104626511*^9}, {
  3.704347942903598*^9, 
  3.704347944229162*^9}},ExpressionUUID->"a55699ca-d998-4ead-906e-\
3678e9409597"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rgt", "[", "f_", "]"}], "[", "x__", "]"}], "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "f", "]"}], "[", "i", "]"}], "[", "x", "]"}], ".", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "f", "]"}], "[", "j", "]"}], "[", "x", "]"}]}]}], 
  ";"}]], "Input",
 Evaluatable->False,ExpressionUUID->"7ea2ec74-a4d8-4fce-bb66-3594ad9a9f5b"],

Cell[BoxData[
 RowBox[{"Context", "[", "rgt", "]"}]], "Input",
 CellChangeTimes->{{3.669814723388529*^9, 3.669814735281488*^9}, {
  3.7043484204338503`*^9, 
  3.704348422529858*^9}},ExpressionUUID->"80c9e52a-90e6-4c18-9ff0-\
83e472b6e3f6"],

Cell[BoxData[
 RowBox[{"?", "tv"}]], "Input",
 CellChangeTimes->{{3.704348537921228*^9, 
  3.704348540568797*^9}},ExpressionUUID->"712dbdfa-0066-4041-adbe-\
24e028cf9881"],

Cell["\<\
If the embedding space is pseudo-Euclidean, the induced metric is defined \
using the scalar product\
\>", "Text",
 CellChangeTimes->{{3.704348572080464*^9, 
  3.704348655503943*^9}},ExpressionUUID->"2b1fd965-b4d8-4fa9-a217-\
642bcbed38ba"],

Cell[BoxData[
 RowBox[{"?", "pssp"}]], "Input",
 CellChangeTimes->{{3.704348659714817*^9, 
  3.704348661608161*^9}},ExpressionUUID->"0291efe2-5a2a-4c19-b905-\
ee8255501d40"],

Cell["instead of the Dot product. We set", "Text",
 CellChangeTimes->{{3.7043486772157173`*^9, 
  3.70434870229543*^9}},ExpressionUUID->"a06fa3d4-ad66-4e74-9927-\
fe67a483e9e0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"psrgt", "[", "f_", "]"}], "[", "x__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "f", "]"}], "[", "i", "]"}], "[", "x", "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "f", "]"}], "[", "j", "]"}], "[", "x", "]"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.7043487560479107`*^9, 
  3.704348820455398*^9}},ExpressionUUID->"b48661e8-ca05-4986-93e7-\
e480bf45fd2c"],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{"psrgt", "::", "usage"}], " ", "=", "\"\<\!\(\*
StyleBox[\(ps\*
StyleBox[\"rgt\", \"MSG\"]\)]\)\!\(\*
StyleBox[\"[\", \"MSG\"]\)\!\(\*
StyleBox[\"f\", \"MSG\"]\)\!\(\*
StyleBox[\"]\", \"MSG\"]\)\!\(\*
StyleBox[\"[\", \"MSG\"]\)\!\(\*
StyleBox[\"x\", \"MSG\"]\)\!\(\*
StyleBox[\",\", \"MSG\"]\)\!\(\*
StyleBox[\"y\", \"MSG\"]\)\!\(\*
StyleBox[\",\", \"MSG\"]\)\!\(\*
StyleBox[\"...\", \"MSG\"]\)\!\(\*
StyleBox[\"]\", \"MSG\"]\)\!\(\*
StyleBox[\"[\", \"MSG\"]\)\!\(\*
StyleBox[\"i\", \"MSG\"]\)\!\(\*
StyleBox[\",\", \"MSG\"]\)\!\(\*
StyleBox[\"j\", \"MSG\"]\)\!\(\*
StyleBox[\"]\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"is\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"the\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"(\", \"MSG\"]\)\!\(\*
StyleBox[\"i\", \"MSG\"]\)\!\(\*
StyleBox[\",\", \"MSG\"]\)\!\(\*
StyleBox[\"j\", \"MSG\"]\)\!\(\*
StyleBox[\")\", \"MSG\"]\)\!\(\*
StyleBox[\"-\", \"MSG\"]\)\!\(\*
StyleBox[\"coordinate\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"of\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"the\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"pseudo\", \"MSG\"]\)\!\(\*
StyleBox[\"-\", \"MSG\"]\)\!\(\*
StyleBox[\"Riemannian\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"metric\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"of\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"the\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"immersion\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"f\", \"MSG\"]\)\!\(\*
StyleBox[\"[\", \"MSG\"]\)\!\(\*
StyleBox[\"x\", \"MSG\"]\)\!\(\*
StyleBox[\",\", \"MSG\"]\)\!\(\*
StyleBox[\"y\", \"MSG\"]\)\!\(\*
StyleBox[\",\", \"MSG\"]\)\!\(\*
StyleBox[\"...\", \"MSG\"]\)\!\(\*
StyleBox[\"]\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"into\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)a\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"pseudo\", \"MSG\"]\)\!\(\*
StyleBox[\"-\", \"MSG\"]\)\!\(\*
StyleBox[\"Euclidean\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"space\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"with\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"scalar\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"product\", \"MSG\"]\)\!\(\*
StyleBox[\" \", \"MSG\"]\)\!\(\*
StyleBox[\"pssp\", \"MSG\"]\)\!\(\*
StyleBox[\".\", \"MSG\"]\)\>\""}], ";"}],
  FontFamily->"Nimbus Sans L",
  FontSize->14,
  FontWeight->"Plain"]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.7043488744198227`*^9, 3.704348928774268*^9}, {
   3.704348962262994*^9, 3.7043490382374277`*^9}, {3.7043492975903177`*^9, 
   3.704349298396208*^9}, {3.7043493644791*^9, 3.704349364609911*^9}, {
   3.704349424494079*^9, 3.704349445891451*^9}, {3.704349483059764*^9, 
   3.704349493898507*^9}, 3.704349545987604*^9, 
   3.7043496186812973`*^9},ExpressionUUID->"3c70b097-be37-4c80-85a8-\
039bc712b444"],

Cell["\<\
Remember: To work with pseudo-Euclidean spaces the invariants dimension dim \
and index in must be defined in the Global Context:\
\>", "Text",
 CellChangeTimes->{{3.70434971423834*^9, 
  3.704349834909583*^9}},ExpressionUUID->"5c47c694-2f70-4da2-ac97-\
4678ea3e2ac4"],

Cell[BoxData[
 RowBox[{"?", "dim"}]], "Input",
 CellChangeTimes->{{3.704349842995694*^9, 
  3.704349845573539*^9}},ExpressionUUID->"a9235f6c-e219-45af-b6af-\
ab86a1b3250d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.704349859424633*^9, 
  3.704349865364993*^9}},ExpressionUUID->"73e1612a-fd2b-4c38-9629-\
5fcf1a189eed"],

Cell[BoxData[
 RowBox[{"?", "ind"}]], "Input",
 CellChangeTimes->{{3.704349873997596*^9, 
  3.7043498758052177`*^9}},ExpressionUUID->"860b54dd-153f-4db6-985a-\
5cf3f90aef3a"],

Cell[BoxData[
 RowBox[{"pssp", "[", 
  RowBox[{
   RowBox[{"vec", "[", "x", "]"}], ",", 
   RowBox[{"vec", "[", "x", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.704349887813055*^9, 
  3.704349910533042*^9}},ExpressionUUID->"0fea26d3-5448-4985-af79-\
87f0ab3140d8"],

Cell["The coordinate matrix of the induced metric is given by", "Text",
 CellChangeTimes->{{3.704350066547389*^9, 
  3.704350132026832*^9}},ExpressionUUID->"af428530-f87d-4d35-bd31-\
806645bde68a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RG", "[", 
     RowBox[{"rgt", "[", "f", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], " ", "or", " ", 
   RowBox[{
    RowBox[{"RG", "[", 
     RowBox[{"psrgt", "[", "f", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", " "}]], "Input",
 Evaluatable->False,ExpressionUUID->"df0c3fd4-5b37-4396-a33d-a8912a8d695b"],

Cell["respectively.", "Text",
 CellChangeTimes->{{3.704350141156892*^9, 3.7043502818184223`*^9}, 
   3.704350323235248*^9},ExpressionUUID->"7fbc0411-8c77-48bd-8450-\
cfaba0230f4a"],

Cell["In the Euclidean case this matrix is obtained also by", "Text",
 CellChangeTimes->{{3.704350661238731*^9, 
  3.7043507039506397`*^9}},ExpressionUUID->"95c4ea5d-21f3-43a6-b09c-\
607717179288"],

Cell[BoxData[
 RowBox[{"?", "RGT"}]], "Input",
 CellChangeTimes->{{3.704350709995203*^9, 
  3.704350715533782*^9}},ExpressionUUID->"80c09b33-db97-44f4-be59-\
2d44366cf859"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"imm", "[", "f", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704350744985137*^9, 
  3.704350828989818*^9}},ExpressionUUID->"b684eee0-ad94-42d9-b080-\
920269b7333c"],

Cell[BoxData[
 RowBox[{"sf", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.704350835582209*^9, 3.704350856918536*^9}, {
  3.70564420753934*^9, 
  3.7056442191855288`*^9}},ExpressionUUID->"0287ca86-bc0c-4015-8b03-\
24e3804dcf26"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGT", "[", "sf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.7045236797443447`*^9, 
  3.704523713001319*^9},ExpressionUUID->"7007e40d-d420-4df8-839a-\
70e58fa53a44"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGT", "[", "sf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"RG", "[", 
    RowBox[{"rgt", "[", "sf", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704350868163685*^9, 3.704350902374344*^9}, {
  3.704350979724889*^9, 
  3.704351006748378*^9}},ExpressionUUID->"e6c20897-f9e7-4dec-8058-\
66fc01d32782"],

Cell["\<\
The induced metric in case of a pseudo-Euclidean embedding space wit index 1 \
is\
\>", "Text",
 CellChangeTimes->{{3.7045237873993807`*^9, 
  3.704523872247069*^9}},ExpressionUUID->"738a8e59-f42c-43ef-b259-\
cb689c66c54a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RG", "[", 
   RowBox[{"psrgt", "[", "sf", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.70452389791897*^9, 
  3.704523898894781*^9}},ExpressionUUID->"b6e186a5-c574-4607-9fa7-\
d59fecf3b75e"],

Cell["\<\
it may be positive definite (Riemannian), indefinite (pseudo-Riemannian of \
index 1), or isotropic, at regular points.\
\>", "Text",
 CellChangeTimes->{{3.7045239433505*^9, 3.704524019237927*^9}, {
  3.704524081965873*^9, 3.7045241290214663`*^9}, {3.704524200933572*^9, 
  3.704524220069414*^9}, {3.704524351196994*^9, 3.704524354036887*^9}, {
  3.704524392548848*^9, 
  3.704524545356329*^9}},ExpressionUUID->"f7c762c8-6b3a-4cc5-bc8b-\
0919c9e5067c"],

Cell["\<\
To obtain the curvature tensor of the immersion f one has to insert the \
Christoffel symbols\
\>", "Text",
 CellChangeTimes->{{3.704351178970468*^9, 
  3.704351279393878*^9}},ExpressionUUID->"1eb25575-af4d-4529-9b95-\
eafdde23b92a"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]f"}]], "Input",
 CellChangeTimes->{{3.704351293256061*^9, 
  3.704351300298019*^9}},ExpressionUUID->"769ff9ea-d72a-47d4-a774-\
142e07e7ba6e"],

Cell[" into the functions", "Text",
 CellChangeTimes->{{3.704351384048802*^9, 
  3.704351399576556*^9}},ExpressionUUID->"7bdb8e70-7d89-430e-98da-\
70788c79fbfb"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.704351406335559*^9, 
  3.704351407888628*^9}},ExpressionUUID->"4ed4b3a7-4fd8-41f7-b6b2-\
7aea52985db3"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.704351436909645*^9, 
  3.704351459505105*^9}},ExpressionUUID->"7e6671c1-569e-447c-9285-\
8fa41c3053fd"],

Cell[TextData[{
 "In the notebook euklsfv3.nb about surface theory contained in [",
 ButtonBox["EDG]",
  BaseStyle->"Hyperlink",
  ButtonData->"EDG"],
 "\nhas been introduced as Module Rimm, compare it with RC. \nA first \
orientation in pseudo-Euclidean geometry gives the following"
}], "Text",
 CellChangeTimes->{{3.704351611519034*^9, 3.704351628166786*^9}, {
   3.7043517186619873`*^9, 3.704351779645824*^9}, {3.704351829551485*^9, 
   3.7043519094929256`*^9}, {3.704351953333406*^9, 3.7043519686124983`*^9}, 
   3.704352186388462*^9, {3.705717684985057*^9, 
   3.70571779804805*^9}},ExpressionUUID->"4384d8be-8078-4e2d-900b-\
da9fff326090"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". The circle  in the pseudo - Euclidean plane. "
}], "Text",
 CellChangeTimes->{{3.705714773970354*^9, 
  3.705714846619977*^9}},ExpressionUUID->"157f7feb-ac7b-4dfc-b574-\
c35340c66f0b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";"}]], "Input",
 CellChangeTimes->{{3.7057148553842916`*^9, 
  3.7057148710833197`*^9}},ExpressionUUID->"bcf24422-5d45-4271-b9ba-\
c17c43828d13"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.705714877692521*^9, 
  3.705714903019484*^9}},ExpressionUUID->"3b796942-307f-4738-8741-\
9eb856ffefa4"],

Cell["The points have different character", "Text",
 CellChangeTimes->{{3.7057149406911716`*^9, 
  3.7057149678990498`*^9}},ExpressionUUID->"699d6d87-9cbb-4a87-9d13-\
99205abbea62"],

Cell[BoxData[
 RowBox[{"?", "ch"}]], "Input",
 CellChangeTimes->{{3.7057149815561867`*^9, 
  3.705714983562896*^9}},ExpressionUUID->"ca5a44f8-6b9d-4523-9bdf-\
30b17960b4fa"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{
   RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705714997373274*^9, 
  3.7057150152273283`*^9}},ExpressionUUID->"64eba363-1e97-40c0-abde-\
7ce8a4d4f5b7"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{
   RowBox[{"spher", "[", "r", "]"}], "[", "0", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705714997373274*^9, 
  3.705715036331298*^9}},ExpressionUUID->"ed082231-2f37-4e7a-a67b-\
bea7f5f1d2b5"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{
   RowBox[{"spher", "[", "r", "]"}], "[", 
   RowBox[{"Pi", "/", "2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705714997373274*^9, 
  3.70571507819471*^9}},ExpressionUUID->"3cafab1c-7b87-41ce-9662-\
98202fe9763e"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{
   RowBox[{"spher", "[", "r", "]"}], "[", 
   RowBox[{"Pi", "/", "4"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705714997373274*^9, 
  3.705715091154962*^9}},ExpressionUUID->"2b821239-3666-480f-95f6-\
60ae95ae592c"],

Cell["\<\
The tangents are spacelike, timelike or isotropic correspondingly:\
\>", "Text",
 CellChangeTimes->{{3.705715136546032*^9, 3.705715154097962*^9}, {
  3.705715321313324*^9, 
  3.705715394048916*^9}},ExpressionUUID->"fd387e9d-787f-4c40-87b2-\
343ea8163770"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "/.", 
   RowBox[{"t", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705715158802615*^9, 
  3.7057152151465597`*^9}},ExpressionUUID->"2b1af099-199c-43ff-870d-\
ad7cdc8a0044"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "/.", 
   RowBox[{"t", "\[Rule]", 
    RowBox[{"Pi", "/", "2"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705715158802615*^9, 3.7057152151465597`*^9}, {
  3.705715252043706*^9, 
  3.705715265434434*^9}},ExpressionUUID->"b8b8521a-c18d-4d3f-bc20-\
aaeaf0b146e0"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"d", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "/.", 
   RowBox[{"t", "\[Rule]", 
    RowBox[{"Pi", "/", "4"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705715284542255*^9, 
  3.7057153023860683`*^9}},ExpressionUUID->"c4258a43-a4db-4fb2-b383-\
2aeb573d8138"],

Cell[BoxData[
 RowBox[{"?", "cross"}]], "Input",
 CellChangeTimes->{{3.7057155203985786`*^9, 
  3.705715524528216*^9}},ExpressionUUID->"94bb5383-2b88-4066-aedf-\
e027b625b1fe"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "\[Equal]", " ", 
  RowBox[{"cross", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.705715434201309*^9, 3.705715489497115*^9}, {
  3.70571555580226*^9, 
  3.705715568464217*^9}},ExpressionUUID->"ceb6f223-d202-4a57-bef2-\
10aab2b735db"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "\[Equal]", 
    " ", 
    RowBox[{"cross", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}], "}"}], "]"}]}], 
   ")"}], "/.", 
  RowBox[{"t", "\[Rule]", " ", 
   RowBox[{"Pi", "/", "4"}]}]}]], "Input",
 CellChangeTimes->{{3.70571564021775*^9, 
  3.70571566585555*^9}},ExpressionUUID->"58a657a5-7435-4934-baa5-\
76a5573d5d49"],

Cell[BoxData[
 RowBox[{"?", "pscross"}]], "Input",
 CellChangeTimes->{{3.7057155769335747`*^9, 
  3.705715585520117*^9}},ExpressionUUID->"519a801c-44b7-4b81-ae91-\
0f6858dcce28"],

Cell["The identity", "Text",
 CellChangeTimes->{{3.7057158662936153`*^9, 
  3.705715872581555*^9}},ExpressionUUID->"f7f2f569-8ebc-49d2-b067-\
ae6f93da5dc8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"d", "[", 
    RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "\[Equal]", " ", 
  RowBox[{"pscross", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.705715434201309*^9, 3.705715489497115*^9}, {
  3.70571555580226*^9, 3.705715568464217*^9}, {3.705715713519261*^9, 
  3.705715714422844*^9}},ExpressionUUID->"a15053b3-68f4-4b6a-b0ab-\
baafad673ad6"],

Cell["is not True in general", "Text",
 CellChangeTimes->{{3.705715881451137*^9, 
  3.7057159039360533`*^9}},ExpressionUUID->"22ddc0bb-fa57-457c-bdf9-\
3ed05e2a1463"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "\[Equal]", 
    " ", 
    RowBox[{"pscross", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}], "}"}], "]"}]}], 
   ")"}], "/.", 
  RowBox[{"t", "\[Rule]", "0"}]}]], "Input",
 CellChangeTimes->{{3.7057157375944242`*^9, 3.705715796968512*^9}, 
   3.705715933861897*^9},ExpressionUUID->"bc180588-65a0-4127-86b1-\
1bcad4756efa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "\[Equal]", 
    " ", 
    RowBox[{"pscross", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}], "}"}], "]"}]}], 
   ")"}], "/.", 
  RowBox[{"t", "\[Rule]", 
   RowBox[{"Pi", "/", "2"}]}]}]], "Input",
 CellChangeTimes->{{3.7057157375944242`*^9, 3.705715796968512*^9}, 
   3.7057159478551207`*^9},ExpressionUUID->"1ffc0a04-9a74-4804-9161-\
eb02644a80f8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "t", "]"}], "\[Equal]", 
    " ", 
    RowBox[{"pscross", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"spher", "[", "r", "]"}], "[", "t", "]"}], "}"}], "]"}]}], 
   ")"}], "/.", 
  RowBox[{"t", "\[Rule]", 
   RowBox[{"Pi", "/", "4"}]}]}]], "Input",
 CellChangeTimes->{{3.7057157375944242`*^9, 3.705715796968512*^9}, {
   3.7057159478551207`*^9, 3.7057159650210953`*^9}, {3.7057160431104927`*^9, 
   3.705716052700327*^9}, 
   3.7057160940459557`*^9},ExpressionUUID->"b56face1-d716-4f16-bdd5-\
caa2a51aa720"],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"ch", "[", 
    RowBox[{
     RowBox[{"d", "[", 
      RowBox[{"spher", "[", "1", "]"}], "]"}], "[", "t", "]"}], "]"}], ",", 
   " ", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{
      RowBox[{"-", "3"}], "*", 
      RowBox[{"Pi", "/", "4"}]}], ",", 
     RowBox[{"5", "*", 
      RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705716108115591*^9, 3.705716117883994*^9}, {
   3.705716151493125*^9, 3.7057162154954767`*^9}, {3.705716528363862*^9, 
   3.7057165664642344`*^9}, {3.705716615226351*^9, 3.705716633567979*^9}, 
   3.705716721291211*^9},ExpressionUUID->"77ca85cf-7aae-4bb2-8103-\
778f3be8648f"],

Cell["\<\
At the isotropic points \[PlusMinus] Pi/4+k*Pi, k integer, the tangents are \
isotropic too. These points divide the circle into 4 segments at which the \
tangents are either spacelike or timelike. \[FilledRectangle]\
\>", "Text",
 CellChangeTimes->{{3.705716774126919*^9, 3.7057167894308643`*^9}, {
  3.705716834615115*^9, 3.705716906198235*^9}, {3.705716945061934*^9, 
  3.705716972141539*^9}, {3.705717168612137*^9, 3.705717294227676*^9}, {
  3.705717343521471*^9, 
  3.705717343525546*^9}},ExpressionUUID->"fd7922a9-d01e-4ace-b743-\
f7f5ee22517d"],

Cell[TextData[{
 StyleBox["Example 2. ",
  FontColor->RGBColor[0, 0, 1]],
 "The sphere in the pseudo-Euclidean 3-space. "
}], "Text",
 CellChangeTimes->{{3.705717485411498*^9, 
  3.7057175255140657`*^9}},ExpressionUUID->"4da14bd7-3168-4b0a-9be1-\
c2c82026839f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.705717825577055*^9, 
  3.705717833631523*^9}},ExpressionUUID->"f7c5b726-4860-48ea-8d4a-\
3eb1d552f69b"],

Cell["\<\
The standard parameter representation of the hypersphere in the n-dimensional \
Euclidean space is defined as\
\>", "Text",
 CellChangeTimes->{{3.705718011061878*^9, 3.705718098581416*^9}, {
  3.705734112076921*^9, 
  3.7057341123079443`*^9}},ExpressionUUID->"9db65050-df54-4d1f-b3d1-\
6ce535986901"],

Cell[BoxData[
 RowBox[{"?", "spher"}]], "Input",
 CellChangeTimes->{{3.705718107049494*^9, 
  3.705718110228979*^9}},ExpressionUUID->"3b9c6669-c453-4e5a-837d-\
5b92d285c0b0"],

Cell["A basis of the tangent spaces is defined by", "Text",
 CellChangeTimes->{{3.705718321579445*^9, 
  3.705718387739493*^9}},ExpressionUUID->"b2b84e6d-1b54-4dde-9e90-\
7b29afabe947"],

Cell[BoxData[
 RowBox[{"?", "tv"}]], "Input",
 CellChangeTimes->{{3.705718274951517*^9, 
  3.705718277364214*^9}},ExpressionUUID->"af00bdaf-7791-4862-aca7-\
1d784e4e7245"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.705717845328005*^9, 3.70571786572766*^9}, {
  3.7057178994332657`*^9, 
  3.705717899687133*^9}},ExpressionUUID->"538be12c-17ab-43d6-923a-\
701b6107d18f"],

Cell["The parameter region is {u,0,2*Pi},{v,-Pi/2,Pi/2}:", "Text",
 CellChangeTimes->{{3.705718629097372*^9, 
  3.7057186924091263`*^9}},ExpressionUUID->"33b9d32b-8e0f-43ab-840c-\
dba093a6b702"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",ExpressionUUID->"c9128e94-bb4b-\
4c6b-a271-8447e3733d84"],

Cell["\<\
For a better adaption of the coordinates we change the order of the axes:\
\>", "Text",
 CellChangeTimes->{{3.7057562967284317`*^9, 
  3.705756347624412*^9}},ExpressionUUID->"24240ce7-fc61-4ff2-be54-\
fdd61c364a82"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sphera", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Sin", "[", "v", "]"}], ",", 
    RowBox[{
     RowBox[{"Cos", "[", "u", "]"}], " ", 
     RowBox[{"Cos", "[", "v", "]"}]}], ",", 
    RowBox[{
     RowBox[{"Cos", "[", "v", "]"}], " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.705753805840394*^9, 
  3.705753907065592*^9}},ExpressionUUID->"57a7c119-3362-4392-9b14-\
2d533cf695f7"],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"sphera", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}], ",", 
     RowBox[{"Pi", "/", "2"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Opacity", "[", ".5", "]"}]}], "}"}]}], ",", 
   RowBox[{"Mesh", "\[Rule]", "None"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7057187007802267`*^9, 3.705718752121017*^9}, {
   3.705735638835519*^9, 3.705735727299685*^9}, {3.705735832818421*^9, 
   3.705735872073473*^9}, 
   3.705753928523943*^9},ExpressionUUID->"13ded31e-d4a6-41f1-9e6b-\
72674d0ef793"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sph1", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.705734138342495*^9, 
  3.705734148300449*^9}},ExpressionUUID->"3c7b1b2f-800b-498a-9721-\
b1c428038361"],

Cell["A Euclidean normal vector is", "Text",
 CellChangeTimes->{{3.70571878273637*^9, 3.705718807104314*^9}, {
  3.705812118253048*^9, 
  3.70581212391711*^9}},ExpressionUUID->"d2cdca3d-7757-448a-b4c9-\
e57cd3e70cda"],

Cell[BoxData[
 RowBox[{
  RowBox[{"norsph", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Cross", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tv", "[", "sphera", "]"}], "[", "1", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"tv", "[", "sphera", "]"}], "[", "2", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.705718233673059*^9, 3.7057182675648947`*^9}, {
  3.7057183994114103`*^9, 3.705718549482934*^9}, {3.70575395273135*^9, 
  3.705753961472788*^9}},ExpressionUUID->"f6a9de95-73f9-440c-9f30-\
f32b9cb811cf"],

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{"Cos", "[", "v", "]"}], "*", 
   RowBox[{"sphera", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.705719292307214*^9, 3.70571930368451*^9}, {
   3.7057193457010508`*^9, 3.705719373204329*^9}, 3.7057194497333803`*^9, {
   3.7057540005159597`*^9, 
   3.705754008265181*^9}},ExpressionUUID->"88074aed-26ee-4e05-b451-\
96f033c0e700"],

Cell["\<\
Thus the only singularities of the parameter representation are the poles v= \
\[PlusMinus] Pi/2.\
\>", "Text",
 CellChangeTimes->{{3.705719491306896*^9, 
  3.705719575234604*^9}},ExpressionUUID->"38fa4ba3-104d-49f3-8684-\
066e3da07c71"],

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"norsph", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", " ", 
    RowBox[{"null", "[", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], ",", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.705718850770501*^9, 3.705719038495139*^9}, {
  3.705719075528634*^9, 3.70571911867852*^9}, {3.705754035232952*^9, 
  3.7057540482008553`*^9}},ExpressionUUID->"428f3ddc-1f2e-4a1f-a680-\
3d85a9e1c141"],

Cell["\<\
The intersection with the isotropic cone is the set of isotropic points of \
the sphere:\
\>", "Text",
 CellChangeTimes->{{3.705734667935974*^9, 
  3.705734735440051*^9}},ExpressionUUID->"403b2399-e09c-4d9d-8001-\
22454287761b"],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", 
      RowBox[{"Cos", "[", "s", "]"}], ",", 
      RowBox[{"Sin", "[", "s", "]"}]}], "}"}], "*", "t"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "1.5"}], ",", "1.5"}], "}"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Green", ",", 
      RowBox[{"Opacity", "[", ".5", "]"}]}], "}"}]}], ",", 
   RowBox[{"Mesh", "\[Rule]", "None"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7057348362895107`*^9, 3.705734850639686*^9}, {
  3.705734897095462*^9, 3.705735003990787*^9}, {3.7057350370306473`*^9, 
  3.705735120614653*^9}, {3.705735265925303*^9, 3.705735301709063*^9}, {
  3.7057353943883657`*^9, 3.705735449323949*^9}, {3.70573578836985*^9, 
  3.7057357963697453`*^9}},ExpressionUUID->"002934b8-22b4-43b5-98f5-\
a1b689ed965e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"cone1", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.705735491666354*^9, 
  3.705735498354999*^9}},ExpressionUUID->"98fcc2da-8b46-47a2-8e45-\
5971ce58b508"],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"sph1", ",", "cone1"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705735504434895*^9, 
  3.705735579234703*^9}},ExpressionUUID->"e5bc0459-1ead-41f2-b28f-\
5c63cb66929b"],

Cell["To find the isotropic points we consider", "Text",
 CellChangeTimes->{{3.705756437599329*^9, 3.705756449255457*^9}, {
  3.7057565848545713`*^9, 
  3.705756598166255*^9}},ExpressionUUID->"162bd240-349d-4e2f-beb5-\
e6c6f472c5f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"psf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"pssp", "[", 
     RowBox[{
      RowBox[{"sphera", "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{"sphera", "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", ">", "0"}], ",", 
      RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7057485727107058`*^9, 3.705748622383954*^9}, {
  3.705748670689591*^9, 3.705748698544879*^9}, {3.7057487297753983`*^9, 
  3.705748893253731*^9}, {3.705754301784727*^9, 
  3.705754307318996*^9}},ExpressionUUID->"428a9fe6-05f4-4db1-99c8-\
d15f2e59a8a0"],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"ch", "[", 
    RowBox[{"sphera", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}], ",", 
     RowBox[{"Pi", "/", "2"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705734435582925*^9, 3.705734446890429*^9}, 
   3.70575438864065*^9},ExpressionUUID->"e41efcb7-987a-46fb-a4d1-\
71ef73e8f453"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{"sphera", "[", 
   RowBox[{"u", ",", 
    RowBox[{
     RowBox[{"-", "Pi"}], "/", "4"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705755872228114*^9, 
  3.705755886380268*^9}},ExpressionUUID->"842e7693-fde4-4798-b281-\
7492ab3d8a26"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{"sphera", "[", 
   RowBox[{"u", ",", 
    RowBox[{"Pi", "/", "4"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705755872228114*^9, 
  3.705755906596236*^9}},ExpressionUUID->"483aa711-420e-4098-b281-\
88aa6d22131a"],

Cell["\<\
The isotropic points are the latitude circles sphera[u, \[PlusMinus]Pi/4]:\
\>", "Text",
 CellChangeTimes->{{3.705755928571452*^9, 3.705756015914631*^9}, {
   3.70575611156201*^9, 3.705756147345785*^9}, 
   3.705756637382176*^9},ExpressionUUID->"4b0e1e74-fc89-4800-8704-\
c744a787bff6"],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"sphera", "[", 
      RowBox[{"u", ",", 
       RowBox[{
        RowBox[{"-", "Pi"}], "/", "4"}]}], "]"}], ",", 
     RowBox[{"sphera", "[", 
      RowBox[{"u", ",", 
       RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Black", ",", "Thick"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Black", ",", "Thick"}], "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7057352424957733`*^9, 3.705735273894474*^9}, {
  3.705735933842359*^9, 3.705736045055709*^9}, {3.7057474134896393`*^9, 
  3.705747465137463*^9}, {3.705747499297505*^9, 3.705747517385191*^9}, {
  3.705747691761034*^9, 3.705747723775771*^9}, {3.7057478923526907`*^9, 
  3.705747934750675*^9}, {3.705748004799317*^9, 3.7057480197974157`*^9}, {
  3.705748151428708*^9, 3.7057482303794737`*^9}, {3.705748263285632*^9, 
  3.705748305341145*^9}, {3.705748373627994*^9, 3.705748416258296*^9}, {
  3.705754148738346*^9, 3.7057541952962313`*^9}, {3.7057542280255203`*^9, 
  3.705754250632052*^9}},ExpressionUUID->"dd1be7ba-5dd9-4246-963c-\
9437153930a4"],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "sph1", ",", "cone1"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.70573538197644*^9, {3.7057477418147173`*^9, 
   3.705747827502183*^9}},ExpressionUUID->"31a281a4-3ecb-4de1-b904-\
fa25fae68374"],

Cell["\<\
This picture shows the position of the Euclidean unit sphere with the \
isotropic cone of the pseudo-Euclidean space. Now we consider the \
pseudo-Euclidean normal of the sphere:\
\>", "Text",
 CellChangeTimes->{{3.705756688565427*^9, 3.705756902004156*^9}, 
   3.705757609894369*^9},ExpressionUUID->"3d68e5ec-c9a3-42f0-a128-\
6feadafba15e"],

Cell[BoxData[
 RowBox[{"?", "pscross"}]], "Input",
 CellChangeTimes->{{3.705756959322214*^9, 
  3.7057569666199417`*^9}},ExpressionUUID->"2c5c7c56-08de-44cd-8ae7-\
637b078ad256"],

Cell[BoxData[
 RowBox[{
  RowBox[{"psnor", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"pscross", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "sphera", "]"}], "[", "1", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", "sphera", "]"}], "[", "2", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "}"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7057569101931477`*^9, 3.705756953128049*^9}, {
   3.705756990659645*^9, 3.7057569990914803`*^9}, 3.7057570393632193`*^9, {
   3.7057575875904207`*^9, 3.7057576004544086`*^9}, {3.705757673822736*^9, 
   3.705757712694175*^9}, {3.7057577755180607`*^9, 
   3.705757791916835*^9}},ExpressionUUID->"2a3d196e-41c0-4a2c-bc78-\
9d2e2c68e3d8"],

Cell["\<\
Up to the sign of the first component (reflection at the y,z-plane) it \
coincides with\
\>", "Text",
 CellChangeTimes->{{3.705758103017395*^9, 
  3.70575818548882*^9}},ExpressionUUID->"c2096ea8-56c3-4ccc-b977-\
c403165f7293"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Cos", "[", "v", "]"}], "*", 
  RowBox[{"sphera", "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.705757881024375*^9, 3.705757885779339*^9}, {
  3.7057579524751053`*^9, 3.705757964955008*^9}, {3.705758021557019*^9, 
  3.7057580243306847`*^9}},ExpressionUUID->"340ef0e1-213c-4ce8-a8a4-\
86491125a6b7"],

Cell["This vector is pseudo-orthogonal to the tangent plane:", "Text",
 CellChangeTimes->{{3.70575857074911*^9, 
  3.705758621709044*^9}},ExpressionUUID->"0a4096ce-f7ee-4f18-9de3-\
7410a9c478e9"],

Cell[BoxData[
 RowBox[{"pssp", "[", 
  RowBox[{
   RowBox[{"psnor", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"tv", "[", "sphera", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705758630085546*^9, 3.705758633456411*^9}, {
  3.7057586730134573`*^9, 
  3.705758740500512*^9}},ExpressionUUID->"41842303-678a-41a7-9343-\
9a9c5edad15c"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"pssp", "[", 
   RowBox[{
    RowBox[{"psnor", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"tv", "[", "sphera", "]"}], "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705758630085546*^9, 3.705758633456411*^9}, {
  3.7057586730134573`*^9, 3.705758713004624*^9}, {3.705758761701173*^9, 
  3.70575877543601*^9}},ExpressionUUID->"1681b9df-5a80-41d4-9af3-\
7c37e156fe6d"],

Cell["\<\
The character of this vector and thus the character of the corresponding \
tangent plane  can be recognized from the sign of the function\
\>", "Text",
 CellChangeTimes->{{3.705758403091254*^9, 
  3.705758554493689*^9}},ExpressionUUID->"f27ec42e-27f7-4259-bb81-\
a626b80410fa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"chnor", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"pssp", "[", 
    RowBox[{
     RowBox[{"psnor", "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", ",", 
     RowBox[{"psnor", "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.705758201964539*^9, 
  3.7057583291033783`*^9}},ExpressionUUID->"e94cc71e-13d5-444d-a9c5-\
460f96bcddbe"],

Cell["\<\
This function doesn\[CloseCurlyQuote]t depend on u. it is zero at the \
singularities v=\[PlusMinus]Pi/2 and at the isotropic points v= Pi/4 + k*Pi, \
k integer. This shows that the tangent planes of the sphere at the isotropic \
points are isotropic. The character of all tangent planes can be seen from \
the plot\
\>", "Text",
 CellChangeTimes->{{3.705758928146302*^9, 
  3.7057592371522408`*^9}},ExpressionUUID->"4d2425f2-d6a1-4b31-8778-\
4ead8891cf5f"],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"chnor", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{
      RowBox[{"-", "Pi"}], "/", "2"}], ",", 
     RowBox[{"Pi", "/", "2"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705759240182128*^9, 
  3.705759299359405*^9}},ExpressionUUID->"96cda99f-39da-413b-aeea-\
3c60eb5068ac"],

Cell[BoxData[
 RowBox[{"chnor", "[", 
  RowBox[{"u", ",", 
   RowBox[{
    RowBox[{"Pi", "/", "4"}], " ", "+", ".1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7057594219666977`*^9, 
  3.705759441798558*^9}},ExpressionUUID->"d325dacc-03b1-462d-9207-\
16f869cf2488"],

Cell[BoxData[
 RowBox[{"chnor", "[", 
  RowBox[{"u", ",", 
   RowBox[{
    RowBox[{"Pi", "/", "4"}], " ", "-", ".1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7057594219666977`*^9, 
  3.7057594651024237`*^9}},ExpressionUUID->"b7f4e3f6-160c-419d-aba9-\
cdbfe734ba0e"],

Cell[BoxData[
 RowBox[{"chnor", "[", 
  RowBox[{"u", ",", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.705759489981914*^9, 
  3.7057594940060883`*^9}},ExpressionUUID->"ca219996-8915-411f-8309-\
a869809401ca"],

Cell["\<\
Now we calculate the curvature tensor of the sphere inmmersed into the \
pseudo-Euclidean space:\
\>", "Text",
 CellChangeTimes->{{3.705815175982264*^9, 
  3.705815241270014*^9}},ExpressionUUID->"9afda1e5-3556-4b24-beb6-\
efe3236f1353"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.7058124197300167`*^9, 
  3.705812422835505*^9}},ExpressionUUID->"d896164a-4dd1-49bd-a167-\
c09ba28e1b0a"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.705813485184155*^9, 
  3.705813498557785*^9}},ExpressionUUID->"72e498ff-aa3d-477a-8c73-\
3d3c187896de"],

Cell["We need the \[CapitalGamma]-symbols", "Text",
 CellChangeTimes->{{3.705815261749958*^9, 
  3.705815293741787*^9}},ExpressionUUID->"d4087a73-4e12-4756-8f3a-\
5c7de33c49a9"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]g"}]], "Input",
 CellChangeTimes->{{3.705812442322912*^9, 
  3.7058124769335423`*^9}},ExpressionUUID->"b818966c-5e9b-44c8-9b18-\
c3acdc5937bb"],

Cell["of the induced pseudo-Riemannian metric:", "Text",
 CellChangeTimes->{{3.7058153046373167`*^9, 
  3.7058153345975723`*^9}},ExpressionUUID->"014b0cca-c288-4655-9be8-\
6e077b23756d"],

Cell[BoxData[
 RowBox[{"?", "psrgt"}]], "Input",
 CellChangeTimes->{{3.705812489202498*^9, 
  3.705812494314941*^9}},ExpressionUUID->"d8ef1732-13fa-4927-8c36-\
474498b3ab5d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"psph", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"psrgt", "[", "sphera", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"i", ",", "j"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.705812519730475*^9, 3.705812525490559*^9}, {
  3.705812563410287*^9, 3.705812660609877*^9}, {3.70581270087318*^9, 
  3.705812716489318*^9}, {3.705813205543674*^9, 3.705813221990539*^9}, {
  3.7058133268073177`*^9, 
  3.705813334590536*^9}},ExpressionUUID->"a042f272-545b-410a-9c16-\
72a973972bc1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]sph", "[", 
    RowBox[{"k_", ",", "i_", ",", "j_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalGamma]g", "[", "psph", "]"}], "[", 
    RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7058127672093*^9, 3.705812772137124*^9}, {
  3.70581284865757*^9, 
  3.70581296985649*^9}},ExpressionUUID->"94f33afc-85d9-4eed-ac4d-\
1b88b6c66319"],

Cell[BoxData[
 RowBox[{"curvtens", "=", 
  RowBox[{
   RowBox[{"RCtensor", "[", "\[CapitalGamma]sph", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.705813413308064*^9, 3.705813455756351*^9}, {
  3.705813526227673*^9, 
  3.705813548003635*^9}},ExpressionUUID->"4356102b-d56a-432a-a31b-\
ac8b38262e22"],

Cell["Its coordinates ", "Text",
 CellChangeTimes->{{3.705815589283617*^9, 3.7058155956031857`*^9}, 
   3.705815631803342*^9},ExpressionUUID->"50ba99a6-630b-4ca5-bf40-\
c88db777be05"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"tensorToArray", "[", "curvtens", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705815398224133*^9, 
  3.7058154381253233`*^9}},ExpressionUUID->"ec0f8cbf-21b5-46f7-b3be-\
8d1630264f6f"],

Cell["\<\
depend on v only. Of course, they are not defined at isotropic points, \
because the induced metric degenerates at these points\
\>", "Text",
 CellChangeTimes->{{3.705815642099284*^9, 
  3.705815791258288*^9}},ExpressionUUID->"d56b4d25-e269-4db9-97ed-\
2a963bc0253f"],

Cell[BoxData[
 RowBox[{"Det", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"psph", "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", "[", 
     RowBox[{"i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "2"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705815847365262*^9, 
  3.705816003249329*^9}},ExpressionUUID->"91acc6f6-fcfa-435a-91ad-\
7246b72cdab8"],

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"v", "\[Rule]", 
   RowBox[{"Pi", "/", "4"}]}]}]], "Input",
 CellChangeTimes->{{3.7058160204407387`*^9, 
  3.7058160336006107`*^9}},ExpressionUUID->"2e961375-5c74-49b3-9640-\
42f5da75cc53"],

Cell[BoxData[
 RowBox[{
  RowBox[{"curvtens", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "/.", " ", 
  RowBox[{"v", "\[Rule]", 
   RowBox[{"Pi", "/", "4"}]}]}]], "Input",
 CellChangeTimes->{{3.705814098654458*^9, 3.70581416657482*^9}, {
  3.70581561525375*^9, 
  3.705815619163818*^9}},ExpressionUUID->"6b21176e-7961-4fc8-9be3-\
849dcf5fe0e8"],

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"curvtens", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}], ",", " ", 
   RowBox[{"v", "\[Rule]", 
    RowBox[{"Pi", "/", "4"}]}], ",", 
   RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705814098654458*^9, 3.70581416657482*^9}, {
  3.705814228471081*^9, 3.70581425874902*^9}, {3.7058143640453253`*^9, 
  3.7058144042215*^9}},ExpressionUUID->"1359e4d6-98fb-4d4d-af49-81476d3fe523"],

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"curvtens", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}], ",", " ", 
   RowBox[{"v", "\[Rule]", 
    RowBox[{"Pi", "/", "4"}]}], ",", 
   RowBox[{"Direction", "\[Rule]", 
    RowBox[{"-", "1"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705814098654458*^9, 3.70581416657482*^9}, {
  3.705814228471081*^9, 3.70581425874902*^9}, {3.7058143640453253`*^9, 
  3.705814419212852*^9}},ExpressionUUID->"0c580310-d945-4dc8-9626-\
c6ee855abd7b"],

Cell["We calculate yet the", "Text",
 CellChangeTimes->{{3.705816154943358*^9, 
  3.705816172599328*^9}},ExpressionUUID->"903d314b-339b-4e15-b8e1-\
c3129544c8ef"],

Cell[BoxData[
 RowBox[{"?", "Riccitensor"}]], "Input",
 CellChangeTimes->{{3.705816051823344*^9, 3.705816070561595*^9}, {
  3.705816136169374*^9, 
  3.7058161395518217`*^9}},ExpressionUUID->"05ca2c3c-80c8-41ee-91fe-\
adf00c3f6b38"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Riccitensor", "[", "\[CapitalGamma]sph", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.705816183778566*^9, 3.705816194293345*^9}, {
  3.705816228144277*^9, 
  3.705816238823203*^9}},ExpressionUUID->"1b5def44-4ed6-4583-b4dd-\
df067695d5ef"],

Cell["Its coordinates are", "Text",
 CellChangeTimes->{{3.705816577372364*^9, 
  3.705816585452248*^9}},ExpressionUUID->"750bf6e8-dea2-4de6-9f68-\
9d00b86681bd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"tensorToArray", "[", "%%", "]"}], "]"}], "//", 
  "TableForm"}]], "Input",
 CellChangeTimes->{{3.705816254056172*^9, 
  3.705816337158538*^9}},ExpressionUUID->"69af780e-7bf5-4270-9ac4-\
7f63e5635b5a"],

Cell["\<\
Of course, it isn\[CloseCurlyQuote]t defined at isotropic points:
\
\>", "Text",
 CellChangeTimes->{{3.705816639481456*^9, 
  3.7058167395948963`*^9}},ExpressionUUID->"b8b61af8-aae4-4b72-b5d8-\
378eced5584b"],

Cell[BoxData[
 RowBox[{"%", "/.", " ", 
  RowBox[{"v", "\[Rule]", 
   RowBox[{"Pi", "/", "4"}]}]}]], "Input",
 CellChangeTimes->{{3.7058166039215317`*^9, 
  3.7058166164444*^9}},ExpressionUUID->"f96748ff-d8dc-4c50-976c-20696c54144e"],

Cell[TextData[{
 StyleBox["Exercise",
  FontColor->RGBColor[0, 0, 1]],
 ". Calculate the curvature tensor and the Ricci-tensor  of the standard \
sphere embedded into the Euclidean 3-space and compare the results with those \
of the considered immersion into the pseudo-Euclidean space. \
\[FilledRectangle]"
}], "Text",
 CellChangeTimes->{{3.70581674783515*^9, 
  3.7058169286299963`*^9}},ExpressionUUID->"227ed8ae-fcfc-4dbe-9cfb-\
2cb371e4c35f"],

Cell[TextData[{
 StyleBox["Example 4. ",
  FontColor->RGBColor[0, 0, 1]],
 "The 3-sphere in the pseudo-Euclidean 4-space. \nIn Example 4.1.3 we \
considered the 3-sphere as a submanifold of the Euclidean 4-space. We could \
proceed as in Example 3 calculating the induced metric, the curvature tensor \
and the Ricci-tensor of the corresponding embedding into the pseudo-Euclidean \
space of index 1; we leave this to the reader as an ",
 StyleBox["exercise",
  FontColor->RGBColor[0, 0, 1]],
 " and consider instead the immersion"
}], "Text",
 CellChangeTimes->{{3.658059160354829*^9, 3.65805918253055*^9}, 
   3.6706498604563847`*^9, {3.7045250840168858`*^9, 3.70452524623213*^9}, {
   3.704525279455752*^9, 3.7045253798152037`*^9}, {3.704525544950562*^9, 
   3.7045255925340357`*^9}, {3.7045967041304827`*^9, 3.704596736505954*^9}, 
   3.705714799532864*^9, 3.705817312039742*^9, {3.7058174382708473`*^9, 
   3.70581774042129*^9}},ExpressionUUID->"0b1e30a5-2821-4533-8511-\
857c93f0d6c0"],

Cell[BoxData[
 RowBox[{"?", "sph"}]], "Input",
 CellChangeTimes->{{3.705818231163155*^9, 
  3.705818233472637*^9}},ExpressionUUID->"35ec399a-496a-4a63-991a-\
f96618cbf825"],

Cell["introduced already in Subsection 1.2", "Text",
 CellChangeTimes->{{3.7058182582115793`*^9, 
  3.705818297457419*^9}},ExpressionUUID->"a5a4e0b7-478d-45e5-a919-\
82ab3af32627"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "4"}], ";"}]], "Input",
 CellChangeTimes->{{3.705817402987241*^9, 
  3.705817409422503*^9}},ExpressionUUID->"8e9c825d-64c5-4e4e-bd41-\
2a7c6dbe890c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sph", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.704525656283592*^9, 3.704525672813353*^9}, 
   3.705818335127936*^9},ExpressionUUID->"50f6f1de-ae96-467b-bb8b-\
42dd956aec09"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "mde", "]"}], ";", 
  RowBox[{"mde", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"d", "[", 
       RowBox[{"sph", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
     RowBox[{"Element", "[", 
      RowBox[{
       RowBox[{"u", "|", "v", "|", "w"}], ",", "Reals"}], "]"}]}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.704534158597312*^9, 3.7045342139652967`*^9}, {
   3.7045355655099773`*^9, 3.704535589388322*^9}, {3.7045356354768057`*^9, 
   3.704535640420103*^9}, {3.704535842612586*^9, 3.7045358771313972`*^9}, {
   3.7046110452123203`*^9, 3.704611143977804*^9}, 3.7058183672961473`*^9, 
   3.705818876997713*^9, 3.705826630365603*^9, {3.7058267432712107`*^9, 
   3.705826745877593*^9}},ExpressionUUID->"6aebe8b6-edab-455d-9c8b-\
0ae4b57faad0"],

Cell[BoxData[
 RowBox[{"Dimensions", "[", "mde", "]"}]], "Input",
 CellChangeTimes->{{3.704535644709197*^9, 
  3.7045356586519423`*^9}},ExpressionUUID->"4ce115f7-41eb-419f-9f78-\
0786bcfef7d2"],

Cell["\<\
To find the rank of mde depending on u,v,w we have to consider the minors of \
mde.\
\>", "Text",
 CellChangeTimes->{{3.70453828079008*^9, 3.704538343069909*^9}, {
  3.704539256775133*^9, 
  3.704539272838777*^9}},ExpressionUUID->"05043410-5d79-4fa9-b263-\
fc5862c089a2"],

Cell[BoxData[
 RowBox[{"?", "Minors"}]], "Input",
 CellChangeTimes->{{3.704538368381703*^9, 3.704538371285058*^9}, 
   3.7045384825023613`*^9},ExpressionUUID->"372bb2b6-869d-4d37-8f59-\
51cc1296efad"],

Cell[BoxData[
 RowBox[{"Minors", "[", 
  RowBox[{"mde", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.704538511479149*^9, 
  3.704538525716889*^9}},ExpressionUUID->"dbde3088-bec7-4263-a474-\
247b4175e13c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "dee", "]"}], ";", 
  RowBox[{"dee", "=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"Flatten", "[", "%", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.7045390378365507`*^9, 3.7045390608001747`*^9}, {
   3.7045392984163322`*^9, 3.7045393173183403`*^9}, {3.704539516909882*^9, 
   3.704539521068748*^9}, 
   3.7046111935544243`*^9},ExpressionUUID->"4ef8d81e-f1da-48ee-9fef-\
df7bb1efe4c1"],

Cell["\<\
The singularities are those points {u,v,w} for which the vector dee is the \
null vector. We try\
\>", "Text",
 CellChangeTimes->{{3.70453963660363*^9, 3.704539790498233*^9}, 
   3.7045968376728363`*^9, 3.7046112376800947`*^9, {3.705818570917953*^9, 
   3.705818575117858*^9}},ExpressionUUID->"c073e965-a6ff-445e-bf9f-\
8d0d3c33fef5"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"dee", ".", "dee"}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"u", "|", "v", "|", "w"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.7058187208977118`*^9, 
  3.705818780364852*^9}},ExpressionUUID->"a9be7d85-e9d3-433e-bd24-\
3183c4f6e0e5"],

Cell["The immersion is regular at all points (r>0).", "Text",
 CellChangeTimes->{{3.705818813987829*^9, 3.7058188465157213`*^9}, {
  3.70581893463667*^9, 
  3.705818961778696*^9}},ExpressionUUID->"5e4744ea-669c-47a9-9819-\
186922acb92f"],

Cell["\<\
Now we calculate the induced metric on the immersion of the 3-sphere:\
\>", "Text",
 CellChangeTimes->{{3.70453993988869*^9, 3.7045400665202208`*^9}, 
   3.704543496102324*^9, {3.705834369092916*^9, 
   3.7058343742612534`*^9}},ExpressionUUID->"af9e4a8e-e89a-4505-93da-\
a61297c33ca5"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"RG", "[", 
     RowBox[{"psrgt", "[", 
      RowBox[{"sph", "[", "r", "]"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"u", "|", "v", "|", "w"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.704525724593692*^9, 3.704525798021266*^9}, {
   3.7046113441526833`*^9, 3.7046113771521387`*^9}, {3.704611428744973*^9, 
   3.704611472279352*^9}, {3.7058190244515657`*^9, 3.705819033331229*^9}, 
   3.705819077628212*^9},ExpressionUUID->"127c1f40-1cf8-443f-bf0e-\
119d12b4eb49"],

Cell[BoxData[
 RowBox[{
  RowBox[{"psdet", "[", 
   RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"Det", "[", "%", "]"}], ",", 
    RowBox[{"Element", "[", 
     RowBox[{
      RowBox[{"u", "|", "v", "|", "w"}], ",", "Reals"}], "]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.704529648922339*^9, 3.704529871044331*^9}, {
   3.704529901924323*^9, 3.704529965740238*^9}, {3.704611559426074*^9, 
   3.704611576846788*^9}, 3.704611620663704*^9, 
   3.7058191248917017`*^9},ExpressionUUID->"e137c6c2-ced4-4c30-8502-\
3f1bb89977d7"],

Cell["\<\
The isotropic points are points {u,v,w} for which this function equals zero. \
Consider the function\
\>", "Text",
 CellChangeTimes->{{3.704543809358163*^9, 3.704543828196333*^9}, 
   3.7045438734780807`*^9, {3.704544039970199*^9, 3.7045441633532887`*^9}, {
   3.705819220313026*^9, 
   3.70581923938489*^9}},ExpressionUUID->"5031ac18-4674-4615-9c02-\
8a218eaacd6a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"fpsdet", "[", 
   RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], ":=", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"8", " ", 
     SuperscriptBox["u", "2"]}], "-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SuperscriptBox["u", "2"], "+", 
       SuperscriptBox["v", "2"], "+", 
       SuperscriptBox["w", "2"]}], ")"}], "2"]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.7045439218654623`*^9, 3.704543961163438*^9}, {
   3.704543998314666*^9, 3.70454400587545*^9}, 
   3.705819290739259*^9},ExpressionUUID->"4701bc36-1a36-490d-8ae0-\
4293b2a0d51d"],

Cell["It suffices to find the zeros of fpsdet:", "Text",
 CellChangeTimes->{{3.704544517439138*^9, 3.7045445317346487`*^9}, {
  3.704544569472925*^9, 3.704544665005704*^9}, {3.705819349576248*^9, 
  3.705819383863412*^9}},ExpressionUUID->"d631035f-dc96-4edf-856c-\
5c1aeeadbaba"],

Cell[BoxData[
 RowBox[{"fpsdet", "[", 
  RowBox[{
   RowBox[{"-", 
    SqrtBox[
     RowBox[{"3", "+", 
      RowBox[{"2", " ", 
       SqrtBox["2"]}]}]]}], ",", "0", ",", 
   SqrtBox[
    RowBox[{
     RowBox[{"-", "4"}], "-", 
     RowBox[{"2", " ", 
      SqrtBox["2"]}], "+", 
     RowBox[{"2", " ", 
      SqrtBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{"3", "+", 
          RowBox[{"2", " ", 
           SqrtBox["2"]}]}], ")"}]}]]}]}]]}], "]"}]], "Input",
 CellChangeTimes->{{3.70583486472586*^9, 3.705834911361312*^9}, {
   3.705834941880988*^9, 3.705834975104904*^9}, 
   3.705835030738406*^9},ExpressionUUID->"965ea9ac-a27b-41f0-82e5-\
dcd27fd42a4c"],

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"fpsdet", "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}], "\[Equal]", "0"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "v", ",", "w"}], "}"}], ",", "Reals"}], "]"}]], "Input",
 CellChangeTimes->{{3.704525820343902*^9, 3.7045258678448*^9}, {
  3.704530003939745*^9, 3.704530077113846*^9}, {3.704544787382079*^9, 
  3.7045448124052067`*^9}, {3.704544864524459*^9, 3.70454487674098*^9}, {
  3.704544928813054*^9, 3.7045449391328077`*^9}, {3.70454606650611*^9, 
  3.70454607229972*^9}},ExpressionUUID->"4ec44543-94e0-48c8-9efb-\
6581d4c39588"],

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", "True"}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{"fpsdet", "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.704611894030854*^9, 
  3.7046119538753443`*^9}},ExpressionUUID->"7530aa09-a37f-41f0-a8cc-\
76671dd554a8"],

Cell[BoxData[
 RowBox[{"?", "ContourPlot3D"}]], "Input",
 CellChangeTimes->{{3.70582053892249*^9, 
  3.70582054929465*^9}},ExpressionUUID->"eb1a6d81-f31c-431b-a305-\
85cf2c53e89d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"xx", "=", "3"}], ";", 
  RowBox[{"ContourPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"fpsdet", "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "xx"}], ",", "xx"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", 
      RowBox[{"-", "xx"}], ",", "xx"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"w", ",", 
      RowBox[{"-", "xx"}], ",", "xx"}], "}"}], ",", 
    RowBox[{"PlotPoints", " ", "\[Rule]", " ", "100"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.70582053892249*^9, 3.70582054929465*^9}, {
   3.705820598192668*^9, 3.705820753101548*^9}, {3.705820814038576*^9, 
   3.705820941964595*^9}, 3.705820982589794*^9, {3.705821228963256*^9, 
   3.705821240313835*^9}},ExpressionUUID->"834c0012-c50b-45d5-9422-\
deb15153b185"],

Cell[BoxData[
 RowBox[{"fpsdet", "[", 
  RowBox[{"0", ",", "0", ",", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.705821288276103*^9, 
  3.705821308457486*^9}},ExpressionUUID->"baa91dee-8c02-4d40-bff2-\
c163a1a0f7ab"],

Cell[BoxData[
 RowBox[{"fpsdet", "[", 
  RowBox[{"1", ",", "0", ",", "0"}], "]"}]], "Input",
 CellChangeTimes->{{3.705821288276103*^9, 3.705821308457486*^9}, {
  3.705821339234165*^9, 3.705821359560925*^9}, {3.705821437905513*^9, 
  3.7058214551920643`*^9}, {3.705821487048998*^9, 
  3.7058215137278433`*^9}},ExpressionUUID->"ef1a3a77-ec22-4012-9cf0-\
759f200ff18f"],

Cell[BoxData[
 RowBox[{"fpsdet", "[", 
  RowBox[{"1", ",", "0", ",", ".10"}], "]"}]], "Input",
 CellChangeTimes->{{3.705821288276103*^9, 3.705821308457486*^9}, {
  3.705821339234165*^9, 3.705821359560925*^9}, {3.705821437905513*^9, 
  3.7058214551920643`*^9}, {3.705821487048998*^9, 3.7058215137278433`*^9}, {
  3.7058352641522217`*^9, 
  3.705835294663993*^9}},ExpressionUUID->"efb1ed3a-b1b2-4634-9552-\
4b9e682fe8b9"],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"fpsdet", "[", 
    RowBox[{"u", ",", "0", ",", "0"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.705835315258025*^9, 3.7058354345898123`*^9}, {
  3.70583557459827*^9, 3.7058356359977493`*^9}, {3.70583567271782*^9, 
  3.705835673546891*^9}, {3.7058357109414263`*^9, 3.7058357537715673`*^9}, {
  3.705835790565873*^9, 3.70583583397886*^9}, {3.705836099786825*^9, 
  3.705836104464532*^9}},ExpressionUUID->"6ce2acc4-da4a-427d-a31f-\
e5a69e531993"],

Cell[TextData[{
 "We conclude: There exist two spheres in the parameter space {u,v,w}\
\[Epsilon] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]],
  ExpressionUUID->"eecfbfbd-d7f4-4533-a18e-4acf6514ccc3"],
 " at which the tangent spaces of the immersions are isotropic;  Inside the \
spheres the induced metric  is positive definite, and outside  indefinite of \
index 1."
}], "Text",
 CellChangeTimes->{{3.705835976393104*^9, 3.705835978088916*^9}, {
  3.7058360149128857`*^9, 3.705836084456348*^9}, {3.705836121407948*^9, 
  3.705836138488058*^9}, {3.7058361921256523`*^9, 3.7058362692310467`*^9}, {
  3.7058403891145687`*^9, 
  3.705840501192377*^9}},ExpressionUUID->"29737a65-fad1-4721-a93a-\
5728539ce843"],

Cell["\<\
Another method to dfind the character of the induced metric of a hypersurface \
is to consider its normal vector:\
\>", "Text",
 CellChangeTimes->{{3.705840772597666*^9, 
  3.705840850429397*^9}},ExpressionUUID->"7cfc6582-3848-4085-82b7-\
05b7a65e877f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "nvec", "]"}], ";", " ", 
  RowBox[{
   RowBox[{"nvec", "[", 
    RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], " ", "=", 
   RowBox[{"FullSimplify", "[", " ", 
    RowBox[{"pscross", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"tv", "[", 
          RowBox[{"sph", "[", "1", "]"}], "]"}], "[", "i", "]"}], "[", 
        RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.705840863695498*^9, 3.7058409781486397`*^9}, {
  3.7058410092053623`*^9, 
  3.705841102779409*^9}},ExpressionUUID->"14b41f39-4b6a-4c6f-a498-\
78861343f813"],

Cell["\<\
The vector nvec[u,v,w] is pseudo-orthogonal to the tangent space at this \
point:\
\>", "Text",
 CellChangeTimes->{{3.7046187070760403`*^9, 3.704618717118908*^9}, {
  3.704618749184825*^9, 3.7046188376375237`*^9}, {3.705841306471208*^9, 
  3.705841325647868*^9}},ExpressionUUID->"ac4e918a-04ec-4966-95d5-\
573aaeb221d1"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"pssp", "[", 
     RowBox[{
      RowBox[{"nvec", "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"tv", "[", 
         RowBox[{"sph", "[", "1", "]"}], "]"}], "[", "i", "]"}], "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705841161742852*^9, 
  3.705841267975862*^9}},ExpressionUUID->"55c5859b-6c59-49ef-9d26-\
9479edb20423"],

Cell["\<\
Thus its character defines the type of the induced metric at regular points:\
\>", "Text",
 CellChangeTimes->{{3.704618849409149*^9, 3.7046189063513727`*^9}, {
   3.704618951668577*^9, 3.70461896954066*^9}, 
   3.7058413586555967`*^9},ExpressionUUID->"35bd2a02-2da1-4f7e-ab6a-\
cd5cd96718cd"],

Cell["\<\
The vector nvec[u,v,w] is orthogonal to the tangent space at this point:\
\>", "Text",
 CellChangeTimes->{{3.7046187070760403`*^9, 3.704618717118908*^9}, {
  3.704618749184825*^9, 
  3.7046188376375237`*^9}},ExpressionUUID->"4c2cce90-8abc-438c-9ecc-\
220cfc4b8a98"],

Cell[BoxData[
 RowBox[{"?", "ch"}]], "Input",
 CellChangeTimes->{{3.704618974137849*^9, 
  3.704618974139593*^9}},ExpressionUUID->"3523d1e8-b2fa-45bb-bfa5-\
1ecd43ed019e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"xx", "=", "3"}], ";", 
  RowBox[{"ContourPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"ch", "[", 
      RowBox[{"nvec", "[", 
       RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}], "\[Equal]", "0"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "xx"}], ",", "xx"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", 
      RowBox[{"-", "xx"}], ",", "xx"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"w", ",", 
      RowBox[{"-", "xx"}], ",", "xx"}], "}"}], ",", 
    RowBox[{"PlotPoints", " ", "\[Rule]", " ", "30"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.70462059066827*^9, 3.704620656071968*^9}, {
   3.705841414633081*^9, 3.7058414924727097`*^9}, 3.705841560055437*^9, {
   3.705841628277464*^9, 3.7058416396826363`*^9}, 3.705842221235201*^9, 
   3.705842600048829*^9, 
   3.705842867390399*^9},ExpressionUUID->"b83b8347-76e7-4c55-ae5f-\
59de20d8d7e1"],

Cell["\<\
The result coincides with that obtained above considering the determinant. \
\>", "Text",
 CellChangeTimes->{{3.704621363913353*^9, 3.704621550872204*^9}, {
   3.7046217967107973`*^9, 3.704621912997801*^9}, {3.705841703681676*^9, 
   3.70584176886611*^9}, 3.7058418253945723`*^9, 3.7058426198419*^9, {
   3.7058434185660477`*^9, 3.705843435765813*^9}, 
   3.7058435282216463`*^9},ExpressionUUID->"28eb5a73-a084-4002-a715-\
58b482a4cca9"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{"nvec", "[", 
   RowBox[{"0", ",", "0", ",", "0"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704621555089979*^9, 
  3.704621578153347*^9}},ExpressionUUID->"ce99d229-cda6-4d61-8531-\
f804d50f2ec6"],

Cell["\<\
Outside of the spheres tne normal vector is spacelike  the corresponding  \
tangent spaces are pseudo - Euclidean of index 1, and inside timelike:\
\>", "Text",
 CellChangeTimes->{{3.7046219231180067`*^9, 3.7046220114525642`*^9}, {
  3.705843479189745*^9, 3.705843516205517*^9}, {3.70584362092498*^9, 
  3.7058436667967854`*^9}, {3.7058437514841843`*^9, 
  3.705843753757361*^9}},ExpressionUUID->"3612731b-2d7d-458d-b5e6-\
bcffa75dad9a"],

Cell[BoxData[
 RowBox[{"ch", "[", 
  RowBox[{"nvec", "[", 
   RowBox[{"1", ",", "0", ",", "0"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.704621555089979*^9, 3.704621578153347*^9}, {
  3.704621745712616*^9, 
  3.704621761695263*^9}},ExpressionUUID->"6e565048-1775-4bbe-adf4-\
28643be166ab"],

Cell["\<\
thus the corresponding  tangent spaces are Euclidean. This example shows the \
typical behavior of an immersion into the pseudo-Euclidean space: the induced \
metric may change its index and have isotropic points, where the metric \
degenerates. \
\>", "Text",
 CellChangeTimes->{{3.704622044291202*^9, 3.7046222945704327`*^9}, {
   3.704622337553981*^9, 3.704622346654373*^9}, {3.704689453404405*^9, 
   3.70468946995601*^9}, 3.704690319686118*^9, {3.7058436821733103`*^9, 
   3.70584369632976*^9}, {3.7058437726927*^9, 
   3.705843782220656*^9}},ExpressionUUID->"80f435e9-a1fb-4a7e-8b99-\
b6d87a20e7ec"],

Cell["The curvature tensor", "Text",
 CellChangeTimes->{{3.704622044291202*^9, 3.7046222945704327`*^9}, {
   3.704622337553981*^9, 3.704622346654373*^9}, {3.704689453404405*^9, 
   3.70468946995601*^9}, 
   3.704690326821773*^9},ExpressionUUID->"8a9586ec-6970-4cb4-b450-\
a245ebae946f"],

Cell[BoxData[
 RowBox[{"?", "RC"}]], "Input",
 CellChangeTimes->{{3.7046894754462833`*^9, 
  3.704689479315488*^9}},ExpressionUUID->"dbe52ea1-aa79-4d1f-8771-\
0f34854ee1cd"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.704690906754785*^9, 
  3.704690915025597*^9}},ExpressionUUID->"1253d1fa-7bd4-4d46-9c08-\
6167f97b457a"],

Cell["\<\
We consider the sphere in the 4-dimensional  pseudo-Euclidean space of index \
1 and calculate the \[CapitalGamma]-symbols\
\>", "Text",
 CellChangeTimes->{{3.704690356733247*^9, 3.704690458100697*^9}, 
   3.7056630476785107`*^9},ExpressionUUID->"105b22cf-3054-49b1-9702-\
9129bab5d5b6"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]g"}]], "Input",
 CellChangeTimes->{{3.704689511899499*^9, 
  3.704689555797769*^9}},ExpressionUUID->"93a5e33d-8eeb-4400-9730-\
434d28fc420f"],

Cell["The pseudo-Riemannian metric is", "Text",
 CellChangeTimes->{{3.704690473703404*^9, 
  3.7046905130202513`*^9}},ExpressionUUID->"006545c8-ec83-4a94-ba96-\
2660207935ea"],

Cell[BoxData[
 RowBox[{"?", "psrgt"}]], "Input",
 CellChangeTimes->{{3.70468958386294*^9, 
  3.7046895933554163`*^9}},ExpressionUUID->"42e6d407-6b20-4d25-8d07-\
8882d34d4b6d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalGamma]sph", "[", "r_", "]"}], "[", 
    RowBox[{"k_", ",", "i_", ",", "j_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], ":=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalGamma]g", "[", 
      RowBox[{"psrgt", "[", 
       RowBox[{"sph", "[", "r", "]"}], "]"}], "]"}], "[", 
     RowBox[{"k", ",", "i", ",", "j"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704689832682082*^9, 3.704689871121876*^9}, {
   3.704689907185824*^9, 3.704689944761466*^9}, {3.704690055561879*^9, 
   3.704690060072124*^9}, 3.704690179792659*^9, 3.7046902125044127`*^9, {
   3.7046902442563353`*^9, 3.704690249174952*^9}, {3.7046905946653633`*^9, 
   3.704690750227049*^9}, {3.705663067023488*^9, 3.705663087886592*^9}, 
   3.7058427972411137`*^9},ExpressionUUID->"ec5a75dd-b21b-426a-85be-\
223ca964151c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalGamma]sph", "[", "r", "]"}], "[", 
   RowBox[{"1", ",", "2", ",", "2"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.7046908158665257`*^9, 3.704690868641994*^9}, {
  3.705663098607334*^9, 
  3.7056631026619797`*^9}},ExpressionUUID->"2cc48b3a-ce4f-4979-b778-\
09fbb90a578c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "RCspher", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"RCsph", "[", "r_", "]"}], "[", 
    RowBox[{"u_", ",", "v_", ",", "w_"}], "]"}], "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", 
     RowBox[{"\[CapitalGamma]sph", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.704690936665924*^9, 3.704691027704768*^9}, {
   3.7056621987792883`*^9, 3.705662242994294*^9}, {3.705662274577026*^9, 
   3.705662294001363*^9}, {3.705663161767233*^9, 3.705663176557844*^9}, 
   3.705844098261314*^9},ExpressionUUID->"77bdb8fb-0553-4fb7-a82f-\
6e9c90cacbf5"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RCsph", "[", "r", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "i", ",", "j"}], "}"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7046910743363543`*^9, 3.704691137975937*^9}, 
   3.704691195944716*^9, 3.7046912427207613`*^9, 3.7056623915661287`*^9, {
   3.705663193733657*^9, 3.705663195205563*^9}, {3.70584397805945*^9, 
   3.705844047579383*^9}, {3.70584411107557*^9, 
   3.705844163682707*^9}},ExpressionUUID->"bd45df9e-6ecd-4414-ab8d-\
09ca2e6f023a"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RCsph", "[", "r", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "i", ",", "j"}], "}"}], ",", 
      RowBox[{"{", "2", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7046910743363543`*^9, 3.704691137975937*^9}, 
   3.704691195944716*^9, 3.7046912427207613`*^9, 3.7056623915661287`*^9, {
   3.705663193733657*^9, 3.705663195205563*^9}, {3.70584397805945*^9, 
   3.705844047579383*^9}, {3.70584411107557*^9, 3.705844163682707*^9}, {
   3.705844200059667*^9, 
   3.705844208258842*^9}},ExpressionUUID->"81e07d7b-ff14-4f45-9ef6-\
7da7d9be2ca5"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RCsph", "[", "r", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "i", ",", "j"}], "}"}], ",", 
      RowBox[{"{", "3", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7046910743363543`*^9, 3.704691137975937*^9}, 
   3.704691195944716*^9, 3.7046912427207613`*^9, 3.7056623915661287`*^9, {
   3.705663193733657*^9, 3.705663195205563*^9}, {3.70584397805945*^9, 
   3.705844047579383*^9}, {3.70584411107557*^9, 3.705844163682707*^9}, {
   3.705844200059667*^9, 
   3.7058442406027327`*^9}},ExpressionUUID->"39b1b43c-9f65-4fda-92bb-\
9ddda0b2deb8"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"RCsph", "[", "r", "]"}], "[", 
      RowBox[{"u", ",", "v", ",", "w"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "i", ",", "j"}], "}"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7046910743363543`*^9, 3.704691137975937*^9}, 
   3.704691195944716*^9, 3.7046912427207613`*^9, 3.7056623915661287`*^9, {
   3.705663193733657*^9, 3.705663195205563*^9}, {3.70584397805945*^9, 
   3.705844047579383*^9}, {3.70584411107557*^9, 3.705844163682707*^9}, {
   3.705844200059667*^9, 3.7058442406027327`*^9}, {3.705844272387144*^9, 
   3.705844288738406*^9}},ExpressionUUID->"af1144ae-72bd-4624-9857-\
76da944d3820"],

Cell["\<\
Calculate other coordinates of the curvature tensor and verify, that the \
coordinates are not defined at isotropic points, where the induced metric \
degenerates.\
\>", "Text",
 CellChangeTimes->{{3.705844665348724*^9, 
  3.705844795341323*^9}},ExpressionUUID->"b67a68fb-8f6f-432e-94ae-\
2531ab3ed57c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCtensor", "[", 
   RowBox[{"\[CapitalGamma]sph", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.705662583967556*^9, 3.705662586127149*^9}, {
   3.705662645040855*^9, 3.705662680446899*^9}, 3.705662772849166*^9, 
   3.7056633146084013`*^9},ExpressionUUID->"7af72437-81a9-4b20-aa8f-\
0b9c7b72e514"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorToArray", "[", "%", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.705662606483526*^9, 3.7056626234791193`*^9}, 
   3.705664198825049*^9},ExpressionUUID->"0e162878-9706-4280-b244-\
ac8cecb052d3"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"Element", "[", 
    RowBox[{
     RowBox[{"u", "|", "v", "|", "w"}], ",", "Reals"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.705662787383109*^9, 3.7056628029989758`*^9}, {
  3.705664451654956*^9, 3.7056645193896637`*^9}, {3.7058443623845463`*^9, 
  3.705844392209238*^9}},ExpressionUUID->"aa1fb8ba-7a6c-4312-aa32-\
d9e6f5cbd602"],

Cell["We calculate also the", "Text",
 CellChangeTimes->{{3.705894869521133*^9, 
  3.705894879921282*^9}},ExpressionUUID->"56bd5a2f-4d5d-4526-9f47-\
5e165314a2a1"],

Cell[BoxData[
 RowBox[{"?", "Riccitensor"}]], "Input",
 CellChangeTimes->{{3.705894546237419*^9, 
  3.7058945542119017`*^9}},ExpressionUUID->"e93a93a1-0505-4c00-bb53-\
4342910740d6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Riccitensor", "[", 
   RowBox[{"\[CapitalGamma]sph", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.7058946007710667`*^9, 3.705894609329825*^9}, 
   3.705894645597365*^9},ExpressionUUID->"b5c00c6d-038c-4c40-958c-\
46aa141e60ab"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"tensorToArray", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705894653539872*^9, 
  3.70589469081181*^9}},ExpressionUUID->"0c87ebed-df2e-4cb4-998f-\
87fa0b6db116"],

Cell[BoxData[
 RowBox[{"TensorSymmetry", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.705894761421488*^9, 
  3.705894783114463*^9}},ExpressionUUID->"06069958-a0c7-42ae-9cca-\
31c3cf060e36"],

Cell["\<\
The curvature tensor and Ricci tensor don\[CloseCurlyQuote]t depend on the \
radius r; this is the case also for the curvature tensor of the hypersphere \
in the Euclidean space:\
\>", "Text",
 CellChangeTimes->{{3.704691686627182*^9, 3.704691812026629*^9}, {
  3.7046922483037558`*^9, 3.704692271343329*^9}, {3.704692306751092*^9, 
  3.704692364326784*^9}, {3.705894903345245*^9, 3.705894919673647*^9}, {
  3.705895218935245*^9, 
  3.705895220966941*^9}},ExpressionUUID->"8bc58364-57d6-4324-8bb2-\
25416153f900"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]f"}]], "Input",
 CellChangeTimes->{{3.704691902789288*^9, 
  3.704691917322945*^9}},ExpressionUUID->"de6486b7-9005-4e5a-8b6d-\
c56dddd6b0cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCtensor", "[", 
   RowBox[{"\[CapitalGamma]f", "[", 
    RowBox[{"sph", "[", "r", "]"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.7046918178872623`*^9, 3.704691878386121*^9}, {
   3.7046920006670847`*^9, 3.704692004289441*^9}, 3.705663325782331*^9, 
   3.70589374356178*^9},ExpressionUUID->"33c3377c-b9fd-452e-8cba-\
0f44c1c633ef"],

Cell[BoxData[
 RowBox[{"%", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7058557000168753`*^9, 
  3.705855792270916*^9}},ExpressionUUID->"76f87f4a-76fc-4f53-9a97-\
37d12f06cf50"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.7058558321252127`*^9, 
  3.7058558459817457`*^9}},ExpressionUUID->"cc1350a5-4ceb-4975-949b-\
e0e8cffb3052"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"tensorToArray", "[", 
    RowBox[{
     RowBox[{"RCtensor", "[", 
      RowBox[{"\[CapitalGamma]f", "[", 
       RowBox[{"sph", "[", "r", "]"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w"}], "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7046920132196198`*^9, 3.704692068640499*^9}, {
  3.7058938323377247`*^9, 
  3.7058938505299187`*^9}},ExpressionUUID->"44beb3bf-9d6d-457d-bcff-\
7b765e8054c8"],

Cell["Mathematica needed 66.58 s for this calculation.", "Text",
 CellChangeTimes->{{3.705894467196142*^9, 
  3.705894520851871*^9}},ExpressionUUID->"a03f081b-5640-420d-b5fd-\
2c9340d834b2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Riccitensor", "[", 
   RowBox[{"\[CapitalGamma]f", "[", 
    RowBox[{"sph", "[", "r", "]"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w"}], "]"}]], "Input",
 CellChangeTimes->{{3.7058949401410933`*^9, 3.705894947306549*^9}, {
  3.7058949860349073`*^9, 3.705895002977174*^9}, {3.705895052762437*^9, 
  3.7058950634890013`*^9}},ExpressionUUID->"d85c80b6-5269-4bf9-a30d-\
7c84776428ab"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"tensorToArray", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7058951013264914`*^9, 
  3.705895129240868*^9}},ExpressionUUID->"04619cfe-1312-4663-a75e-\
5c147db8d5e2"],

Cell["\[FilledRectangle]", "Text",
 CellChangeTimes->{{3.704692447698883*^9, 
  3.704692447702105*^9}},ExpressionUUID->"c5d9e997-6ea8-4141-96fe-\
a5ce7cfe795a"]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.3. The Covariant Curvature Tensor", "Subsubsection",
 CellChangeTimes->{{3.6635117359545794`*^9, 3.663511766290524*^9}, {
   3.663513341663433*^9, 3.663513343959806*^9}, 3.663766506922921*^9, 
   3.669818010438081*^9, 3.669878281268444*^9, {3.705895280422608*^9, 
   3.7058952821502237`*^9}, 3.705930759079752*^9, 3.70849885705746*^9, 
   3.708567901398068*^9},ExpressionUUID->"52f7478a-ab6f-4799-bdc3-\
aa23b0d409c2"],

Cell["\<\
In this subsection we introduce the covariant curvature tensor.\
\>", "Text",
 CellChangeTimes->{{3.6634831286506*^9, 3.663483217777864*^9}, {
  3.705898413278664*^9, 
  3.7058984467424717`*^9}},ExpressionUUID->"0d41a29e-330d-4ab4-9261-\
b60e634985ee"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6634832225330763`*^9, 3.663483226266313*^9}, 
   3.663508317478022*^9, 3.6635929298041153`*^9, 3.663594349057375*^9, 
   3.6636528464148207`*^9},ExpressionUUID->"c96b5017-3329-47e1-9449-\
372c804334f4"],

Cell["The metric tensor is ", "Text",
 CellChangeTimes->{{3.663570645711315*^9, 
  3.66357065515973*^9}},ExpressionUUID->"b234e0e8-18df-4761-9652-\
907d96e93e9e"],

Cell[BoxData[
 RowBox[{"?", "RGtensor"}]], "Input",
 CellChangeTimes->{{3.663498295395432*^9, 3.6634982983556767`*^9}, {
  3.663591414530204*^9, 
  3.6635914173060102`*^9}},ExpressionUUID->"6083f9ab-303f-4776-80d3-\
6720404734c6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGtensor", "[", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663591599028181*^9, 3.6635916213673487`*^9}, {
   3.66359166498285*^9, 3.6635917431825323`*^9}, 
   3.6636530206941633`*^9},ExpressionUUID->"feed0615-81ea-49a6-8249-\
16f2c8b7c428"],

Cell[BoxData[
 RowBox[{"tensorToArray", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.669868760263647*^9, 
  3.669868775085992*^9}},ExpressionUUID->"ad59759a-a3e7-4125-b12c-\
dff79be96772"],

Cell["\<\
The Riemannian curvature tensor of an unspecified pseudo-Riemannian metric is\
\>", "Text",
 CellChangeTimes->{{3.663652898876539*^9, 3.663652934431237*^9}, {
  3.6636531133244743`*^9, 
  3.663653142748025*^9}},ExpressionUUID->"04002039-4b95-4309-86e0-\
bdcfeca44feb"],

Cell[BoxData[
 RowBox[{"?", "RCtensor"}]], "Input",
 CellChangeTimes->{{3.663591503796645*^9, 
  3.663591511230419*^9}},ExpressionUUID->"77931ced-5338-4fa1-94c3-\
8049d7b20f9b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RCtensor", "[", 
   RowBox[{"\[CapitalGamma]g", "[", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663591599028181*^9, 3.6635916213673487`*^9}, {
   3.66359166498285*^9, 3.6635917431825323`*^9}, 
   3.6636530771576357`*^9},ExpressionUUID->"324b0f66-4f28-4576-be3d-\
e30ad1160dea"],

Cell["with", "Text",
 CellChangeTimes->{{3.6636531634040422`*^9, 
  3.6636531650680656`*^9}},ExpressionUUID->"b55bde4f-4759-493e-8eef-\
0555e9c5ebc0"],

Cell[BoxData[
 RowBox[{"?", "\[CapitalGamma]g"}]], "Input",
 CellChangeTimes->{{3.663591558588146*^9, 
  3.6635915697519493`*^9}},ExpressionUUID->"e5ab71f1-8cd2-4005-b6a4-\
652f82e80c87"],

Cell["It is a tensor field of type (3,1) with coordinates", "Text",
 CellChangeTimes->{{3.6636532102836113`*^9, 
  3.6636532844985847`*^9}},ExpressionUUID->"7333555b-08e7-402b-9391-\
39818646a440"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubsuperscriptBox["R", 
    RowBox[{"  ", "hij"}], "k"], "[", "x", "]"}], "  ", "\[DotEqual]", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"RCtensor", "[", 
     RowBox[{"\[CapitalGamma]g", "[", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"h", ",", "i", ",", "j"}], "}"}], ",", 
    RowBox[{"{", "k", "}"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6636532954053097`*^9, 3.663653344851122*^9}, {
   3.663653375274926*^9, 3.663653461617854*^9}, 3.663653524078583*^9, 
   3.663653584240539*^9, {3.66365361961629*^9, 
   3.663653622072485*^9}},ExpressionUUID->"33e5e57e-8942-4567-bd69-\
bbb850a12897"],

Cell["We consider the tensor product with the coordinates", "Text",
 CellChangeTimes->{{3.663653653647748*^9, 3.66365368237558*^9}, {
  3.671683835137738*^9, 
  3.671683838833645*^9}},ExpressionUUID->"a4151cd7-0627-4e4b-b298-\
1c92c2c843cf"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", "ks"], "[", "x", "]"}], 
  RowBox[{
   SubsuperscriptBox["R", 
    RowBox[{"  ", "hij"}], "l"], "[", "x", "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663653696047441*^9, 
  3.663653788631715*^9}},ExpressionUUID->"f0926389-940d-4baa-9320-\
266901780ab7"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RGtensor", "[", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "\[CircleTimes]", 
  RowBox[{
   RowBox[{"RCtensor", "[", 
    RowBox[{"\[CapitalGamma]g", "[", "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.663591599028181*^9, 3.6635916213673487`*^9}, {
  3.66359166498285*^9, 
  3.6635917431825323`*^9}},ExpressionUUID->"410fa450-a057-47dc-96a9-\
7e6abc4443d1"],

Cell[TextData[{
 "The following contraction is a covariant tensor field of degree 4 named the \
",
 StyleBox["covariant curvature tensor",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.6636790998086443`*^9, 
  3.663679183903713*^9}},ExpressionUUID->"ea2d891b-ed3c-461a-a5bb-\
1c06bf83697b"],

Cell[BoxData[
 RowBox[{"?", "covRCtensor"}]], "Input",
 CellChangeTimes->{{3.6636792617166643`*^9, 3.663679264319961*^9}, 
   3.6636831547843027`*^9, 
   3.665065262454735*^9},ExpressionUUID->"fc5d51ad-69a8-495d-80bc-\
560e983c0c16"],

Cell["defined in", "Text",
 CellChangeTimes->{{3.7058988508830357`*^9, 
  3.705898855499167*^9}},ExpressionUUID->"6cf9c327-d28f-40f5-9c8e-\
9887bae70bf9"],

Cell[BoxData[
 RowBox[{"Context", "[", "covRCtensor", "]"}]], "Input",
 CellChangeTimes->{{3.705898859653637*^9, 
  3.7058988919388027`*^9}},ExpressionUUID->"a801cf83-c9ed-4406-b1be-\
f80bd9b07746"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"covRCtensor", "[", 
     RowBox[{"gg_:", 
      RowBox[{"rg", "[", "]"}]}], "]"}], "[", "x__", "]"}], ":=", " ", 
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"RGtensor", "[", "gg", "]"}], "[", "x", "]"}], 
      "\[CircleTimes]", 
      RowBox[{
       RowBox[{"RCtensor", "[", 
        RowBox[{"\[CapitalGamma]g", "[", "gg", "]"}], "]"}], "[", "x", 
       "]"}]}], ",", "2", ",", "1"}], "]"}]}], ";"}]], "Input",
 Evaluatable->False,ExpressionUUID->"256c74b3-910b-4742-86df-9d71579acd04"],

Cell[TextData[{
 "The operation of",
 ButtonBox[" lowering the index",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/IndexLowering.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/IndexLowering.html"],
 " k"
}], "Text",
 CellChangeTimes->{{3.663680436339463*^9, 3.6636804531316423`*^9}, {
  3.6636807312582483`*^9, 3.663680764954425*^9}, {3.663680845641803*^9, 
  3.663680845643096*^9}},ExpressionUUID->"cdaa0ea5-c7e1-481e-9905-\
26bcddf63ca9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SubsuperscriptBox["R", 
     RowBox[{"  ", "hij"}], "k"], "[", "x", "]"}], " ", "\[RightTeeArrow]", 
   " ", 
   RowBox[{
    SubscriptBox["R", 
     RowBox[{" ", "khij"}]], "[", "x", "]"}]}], "=", " ", 
  RowBox[{
   RowBox[{
    SubscriptBox["g", "kl"], "[", "x", "]"}], 
   RowBox[{
    SubsuperscriptBox["R", 
     RowBox[{"  ", "hij"}], "l"], "[", "x", "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663680498643361*^9, 3.6636805972994747`*^9}, {
  3.663680637124598*^9, 
  3.663680647970809*^9}},ExpressionUUID->"2f9afa65-7e1b-479f-9584-\
e0ac8e6ae70c"],

Cell[TextData[{
 "is a part of the ",
 ButtonBox["Index Gymnastics",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/IndexGymnastics.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/IndexGymnastics.html"],
 ". It can be done for any contravariant index of tensor fields in a \
pseudo-Riemannian space.  The inverse operation is the ",
 ButtonBox["index raising",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/IndexRaising.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/IndexRaising.html"],
 ", done with the",
 StyleBox[" inverse metric tensor:",
  FontColor->GrayLevel[0]]
}], "Text",
 CellChangeTimes->{
  3.66368070690633*^9, {3.663680962600772*^9, 3.6636810121044292`*^9}, {
   3.6636810664022427`*^9, 3.663681212039151*^9}, 3.6636812965827103`*^9, {
   3.663682951265263*^9, 3.66368302051274*^9}, {3.663684799189735*^9, 
   3.663684799192668*^9}},ExpressionUUID->"c90f3545-786d-418a-9774-\
e046cca260ed"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["R", 
      RowBox[{" ", "khij"}]], "[", "x", "]"}], "\[RightTeeArrow]", " ", 
    RowBox[{
     SubsuperscriptBox["R", 
      RowBox[{"  ", "hij"}], "k"], "[", "x", "]"}]}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     SuperscriptBox["g", "kl"], "[", "x", "]"}], " ", 
    RowBox[{
     SubscriptBox["R", 
      RowBox[{" ", "lhij"}]], "[", "x", "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663683063768306*^9, 3.663683065464872*^9}, {
  3.6636830965375547`*^9, 3.663683135123877*^9}, {3.663683177823773*^9, 
  3.6636831804312687`*^9}, {3.663683211967482*^9, 
  3.663683251447426*^9}},ExpressionUUID->"d122c82b-e83b-4f9b-82dc-\
78060c4d9907"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["g", "kl"], "[", "x", "]"}], " ", "\[DotEqual]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"inv", "[", "genRG", "]"}], "[", 
    RowBox[{"k", ",", "l"}], "]"}], "[", "x", "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663683356902357*^9, 3.663683372117762*^9}, {
  3.663683488013842*^9, 3.663683526316818*^9}, {3.6636844892051077`*^9, 
  3.6636844908530493`*^9}},ExpressionUUID->"2d83c6b9-4f61-4d9d-8b51-\
c85d7008201a"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["g", "kl"], "[", "x", "]"}]], "Input",
  CellChangeTimes->{{3.663683063768306*^9, 3.663683065464872*^9}, {
   3.6636830965375547`*^9, 3.663683135123877*^9}, {3.663683177823773*^9, 
   3.6636831804312687`*^9}, {3.663683211967482*^9, 3.663683251447426*^9}},
  ExpressionUUID->"f21e2646-22ce-4c33-9b1e-f9fe4a16f691"],
 " denotes the coordinates of a twofold contravariant tensor, the ",
 StyleBox["inverse metric tensor",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.663683305054186*^9, 3.6636833287419863`*^9}, {
  3.663684607092128*^9, 3.663684622219635*^9}, {3.663686504341464*^9, 
  3.66368656782691*^9}},ExpressionUUID->"1a2db856-fc3a-48d6-aa42-\
099f43f7d556"],

Cell[BoxData[
 RowBox[{"?", "INV"}]], "Input",
 CellChangeTimes->{{3.705902207911343*^9, 
  3.7059022098151493`*^9}},ExpressionUUID->"60632459-bbc3-4d79-8d66-\
8860b123ea35"],

Cell[BoxData[
 RowBox[{
  RowBox[{"INV", "[", 
   RowBox[{"RG", "[", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705902002529395*^9, 
  3.705902009184887*^9}},ExpressionUUID->"01d94126-b562-46cb-a681-\
c90bfac891cf"],

Cell[BoxData[
 RowBox[{"ArrayTotensor", "[", 
  RowBox[{
   RowBox[{"INV", "[", 
    RowBox[{"RG", "[", "]"}], "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663683894092723*^9, 3.6636839665373077`*^9}, 
   3.6636840269694557`*^9, {3.663684306070784*^9, 3.6636843186278143`*^9}, 
   3.7059020525768337`*^9, {3.705902102354528*^9, 
   3.705902103248095*^9}},ExpressionUUID->"d45ce338-2b01-494e-a443-\
588e6a8be573"],

Cell[BoxData[
 RowBox[{"?", "inv"}]], "Input",
 CellChangeTimes->{{3.6650651678796577`*^9, 
  3.665065169484475*^9}},ExpressionUUID->"a3b8379d-df80-40bc-9622-\
c753b9d3969c"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"inv", "[", 
     RowBox[{"RG", "[", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
   RowBox[{"1", ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6650656706095448`*^9, 3.665065676089532*^9}, {
  3.70590216000751*^9, 
  3.7059021709517393`*^9}},ExpressionUUID->"cdda1d4d-604f-472d-8389-\
72539beec0b0"],

Cell[BoxData[
 RowBox[{"Context", "[", "INV", "]"}]], "Input",
 CellChangeTimes->{{3.6698693361103992`*^9, 
  3.6698693520960827`*^9}},ExpressionUUID->"760d3a0c-b5a6-4af8-ad59-\
e9aea235016d"],

Cell["\<\
Every covariant index can be raised. Tensors which differ by raising or \
lowering of indices only represent the same object: the transition between \
them is canonically given by the pseudo-Riemannian metric tensor of the  \
considered space.
We return to the covariant curvature tensor which has more interesting \
symmetry properties than the original curvature tensor defined by the \
Levy-Civita connection.\
\>", "Text",
 CellChangeTimes->{{3.663686768779673*^9, 3.663687103293667*^9}, 
   3.6698692514011993`*^9, 3.669869317432547*^9, {3.705902316279766*^9, 
   3.70590236662261*^9}},ExpressionUUID->"a657d169-d6a8-4a9b-b4c7-\
5f4da0289fbe"],

Cell[BoxData[
 RowBox[{
  RowBox[{"covRCtensor", "[", 
   RowBox[{"rg", "[", "]"}], "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663591993479333*^9, 3.663592023331283*^9}, {
  3.663593811467861*^9, 3.6635938119245*^9}, {3.6636488875136957`*^9, 
  3.6636489000976553`*^9}, {3.66506591539284*^9, 3.665065915688271*^9}, {
  3.7059024077674017`*^9, 
  3.7059024176854267`*^9}},ExpressionUUID->"2a6fdf82-8686-4d27-a7b0-\
8be40b6fa078"],

Cell["The coordinates of this tensor are", "Text",
 CellChangeTimes->{{3.6636871348933983`*^9, 
  3.663687146829459*^9}},ExpressionUUID->"a718a955-040e-4752-99e8-\
1b1fb54e59d8"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"covrc", "[", 
     RowBox[{"gg_:", 
      RowBox[{"rg", "[", "]"}]}], "]"}], "[", 
    RowBox[{"k_", ",", "h_", ",", "i_", ",", "j_"}], "]"}], "[", "x__", "]"}],
   ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"covRCtensor", "[", "gg", "]"}], "[", "x", "]"}], "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"k", ",", "h", ",", "i", ",", "j"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663485250127709*^9, 3.663485256744276*^9}, 
   3.6634852922395687`*^9, {3.663498331086689*^9, 3.6634984657065*^9}, {
   3.663498688528874*^9, 3.663498786072043*^9}, {3.663498835247759*^9, 
   3.663498849543777*^9}, {3.663498926351234*^9, 3.663499027070335*^9}, {
   3.663499064949574*^9, 3.663499289460431*^9}, {3.6634995470192623`*^9, 
   3.663499647201833*^9}, {3.6635037881118097`*^9, 3.663503881672029*^9}, {
   3.663649312614892*^9, 3.663649412493531*^9}, 3.6636502818307467`*^9, {
   3.665065958290298*^9, 
   3.6650659593704453`*^9}},ExpressionUUID->"5ba2ebc7-8306-4653-be23-\
5c7c14276bab"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"1", ",", "2", ",", "2", ",", "3"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.66364943800315*^9, 3.663649523844922*^9}, {
  3.663649640493535*^9, 3.663649646611808*^9}, {3.665065972931692*^9, 
  3.665065973386888*^9}, {3.70590247619916*^9, 3.705902483821302*^9}, {
  3.7059025641251574`*^9, 
  3.7059025834443293`*^9}},ExpressionUUID->"703caf05-63a3-4c46-bb98-\
608fc41ba81d"],

Cell["\<\
The time needed for  the last calculation was 5.52 s, 12.216 s with v. 9.0.1.\
\>", "Text",
 CellChangeTimes->{
  3.663649788106453*^9, {3.7059026420211697`*^9, 3.7059026482275953`*^9}, {
   3.705902753796*^9, 3.705902754322608*^9}, 3.7084992335980473`*^9, {
   3.708514551368541*^9, 
   3.70851458852059*^9}},ExpressionUUID->"e468ba03-d213-4c17-b75f-\
30fe5c6a8bd3"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"tensorToArray", "[", 
   RowBox[{
    RowBox[{"covRCtensor", "[", 
     RowBox[{"rg", "[", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663650403908434*^9, 3.6636504168926277`*^9}, {
   3.663687247423656*^9, 3.663687283846882*^9}, {3.665066158881179*^9, 
   3.665066159096587*^9}, 3.705902779122937*^9, 
   3.708499128930702*^9},ExpressionUUID->"0f398b96-d319-41b6-864b-\
248b7d12290c"],

Cell[TextData[StyleBox["Needed time:  32.064 s witn v. 9.0.1, but 252.352 s \
with v. 11.1.1.",
 FontColor->RGBColor[1, 0, 0]]], "Text",
 CellChangeTimes->{{3.6636877600243*^9, 3.6636877894241877`*^9}, 
   3.6650660874463778`*^9, 3.6650668924397717`*^9, 3.705902811434263*^9, 
   3.7059028631141663`*^9, {3.7085002429257383`*^9, 3.708500281478281*^9}, {
   3.708514639704221*^9, 
   3.7085146643271503`*^9}},ExpressionUUID->"8d4c54e6-f3be-42a8-855a-\
b09d00312cbd"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"TensorSymmetry", "[", 
   RowBox[{"%", "[", 
    RowBox[{"[", "2", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663500516686882*^9, 3.6635005283458138`*^9}, {
   3.66368737211091*^9, 3.6636873890467377`*^9}, 3.705902890538097*^9, 
   3.708499158351904*^9},ExpressionUUID->"bee1d934-093a-459a-9205-\
5344016d905f"],

Cell["\<\
Result:
{57.480000,{{Cycles[{{3,4}}],-1},{Cycles[{{1,2}}],-1},{Cycles[{{1,3},{2,4}}],\
1}}}\
\>", "Text",
 CellChangeTimes->{{3.708515688216173*^9, 
  3.708515696816307*^9}},ExpressionUUID->"dd845c15-af31-44b4-bb4f-\
1b1969fd072b"],

Cell[TextData[{
 "Needed time:  252.352 with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 11.1.1, but only  57.48 s  with v. 9.0.1. Clearly, for tensors of \
higher degrees or dimensions the calculation needs more time and the \
expressions obtained are very large."
}], "Text",
 CellChangeTimes->{{3.6636877600243*^9, 3.6636877894241877`*^9}, {
  3.663687860625917*^9, 3.663687949017159*^9}, {3.665067535571228*^9, 
  3.6650675435228367`*^9}, {3.705902910185261*^9, 3.705902921353034*^9}, {
  3.705902972601387*^9, 3.705903009057333*^9}, {3.70590304540091*^9, 
  3.705903047721002*^9}, {3.7085067936347427`*^9, 3.70850680495463*^9}, {
  3.7085147556165257`*^9, 3.708514846638743*^9}, {3.708516001703967*^9, 
  3.7085160434051943`*^9}},ExpressionUUID->"d86b8d45-f10d-4183-8bdc-\
a2a3daaf5464"],

Cell["\<\
We confirm the symmetry properties of the covariant curvature tensor in \
details:\
\>", "Text",
 CellChangeTimes->{{3.663650614458943*^9, 3.663650662970304*^9}, {
   3.6636507093301373`*^9, 3.6636507284336643`*^9}, 
   3.663687977817422*^9},ExpressionUUID->"2f1e332a-6e6f-4825-b19c-\
8210fa96a75d"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"covrc", "[", 
        RowBox[{"rg", "[", "]"}], "]"}], "[", 
       RowBox[{"1", ",", "2", ",", "2", ",", "3"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"covrc", "[", 
        RowBox[{"rg", "[", "]"}], "]"}], "[", 
       RowBox[{"2", ",", "1", ",", "2", ",", "3"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
    RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.663650743994884*^9, 3.6636507922817183`*^9}, {
  3.665067606203052*^9, 3.66506760693751*^9}, {3.708514902068516*^9, 
  3.7085149183567867`*^9}, {3.70851496911683*^9, 
  3.708514992243971*^9}},ExpressionUUID->"e17363d6-fdc9-4788-be45-\
39d619cfe243"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"1", ",", "2", ",", "2", ",", "3"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"1", ",", "2", ",", "3", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.663650743994884*^9, 3.6636508328493977`*^9}, {
  3.6650676076004333`*^9, 
  3.6650676083044567`*^9}},ExpressionUUID->"91a8aeb1-b9fa-49fd-a0e9-\
3f42f501db40"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"1", ",", "2", ",", "2", ",", "3"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "-", 
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"2", ",", "3", ",", "1", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.663650743994884*^9, 3.6636508328493977`*^9}, {
  3.663650864331414*^9, 3.663650904993143*^9}, {3.6650676090565577`*^9, 
  3.665067610391941*^9}},ExpressionUUID->"dbe44363-0348-415f-a517-\
bce756fdb765"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"1", ",", "2", ",", "2", ",", "3"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"1", ",", "2", ",", "3", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"covrc", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"1", ",", "3", ",", "2", ",", "2"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.663650743994884*^9, 3.6636508328493977`*^9}, {
  3.663650864331414*^9, 3.663650904993143*^9}, {3.663650971331414*^9, 
  3.663651025047757*^9}, {3.6636510719764843`*^9, 3.6636510933745203`*^9}, {
  3.665067611392989*^9, 
  3.665067617696678*^9}},ExpressionUUID->"1d218a0b-6ab1-412e-8eea-\
a3607fa9502b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"tensorToArray", "[", 
    RowBox[{
     RowBox[{"covRCtensor", "[", 
      RowBox[{"rg", "[", "]"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6636511234036093`*^9, 3.663651218021834*^9}, {
   3.663652389028165*^9, 3.66365241557971*^9}, {3.6636880028977137`*^9, 
   3.663688023265685*^9}, {3.665067662832502*^9, 3.6650676655364523`*^9}, {
   3.669870915836277*^9, 3.669870925368874*^9}, 3.7059033369508343`*^9, 
   3.7085161326903543`*^9},ExpressionUUID->"105dd5be-9748-44bb-8a0a-\
89cf94822815"],

Cell["After 561.58 s.  Mathematica showed a very large result.", "Text",
 CellChangeTimes->{{3.669876399569289*^9, 3.6698765505375633`*^9}, 
   3.7059037933727503`*^9, 
   3.7085177568654118`*^9},ExpressionUUID->"ba8bac2e-5f10-4669-ae41-\
7368062c593b"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"TensorSymmetry", "[", 
   RowBox[{"%", "[", 
    RowBox[{"[", "2", "]"}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.663500516686882*^9, 3.6635005283458138`*^9}, {
  3.66368737211091*^9, 
  3.6636873890467377`*^9}},ExpressionUUID->"903249df-e36b-4d81-a176-\
0428ff1a2be7"],

Cell["The result was", "Text",
 CellChangeTimes->{{3.669876594763661*^9, 
  3.669876612346513*^9}},ExpressionUUID->"21b354a1-d03e-4e49-ad4d-\
ce000a9f66f7"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2214.26`", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cycles", "[", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"3", ",", "4"}], "}"}], "}"}], "]"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cycles", "[", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"1", ",", "2"}], "}"}], "}"}], "]"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cycles", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"1", ",", "3"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"2", ",", "4"}], "}"}]}], "}"}], "]"}], ",", "1"}], 
      "}"}]}], "}"}]}], "}"}]], "Input",
 Evaluatable->False,ExpressionUUID->"f65096ec-d67d-4e9f-8073-845f4b741269"],

Cell["\<\
It is well known that the covariant curvature tensor satisfies the Bianchi \
identity, namely\
\>", "Text",
 CellChangeTimes->{{3.669876868670918*^9, 3.669876953414461*^9}, {
  3.669877056700716*^9, 3.669877171540516*^9}, {3.669877240388336*^9, 
  3.669877282971798*^9}, {3.669877318715622*^9, 
  3.669877330235435*^9}},ExpressionUUID->"99c13766-2916-48ba-b91a-\
7644b586ecab"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"covrc", "[", 
        RowBox[{"rg", "[", "]"}], "]"}], "[", 
       RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"covrc", "[", 
        RowBox[{"rg", "[", "]"}], "]"}], "[", 
       RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"covrc", "[", 
        RowBox[{"rg", "[", "]"}], "]"}], "[", 
       RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
    RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6636511234036093`*^9, 3.663651218021834*^9}, {
  3.663652389028165*^9, 3.66365241557971*^9}, {3.6636880028977137`*^9, 
  3.663688023265685*^9}, {3.665067662832502*^9, 3.6650676655364523`*^9}, {
  3.669870915836277*^9, 
  3.669870925368874*^9}},ExpressionUUID->"829b7f60-9393-4370-84cb-\
abd0893cb9d6"],

Cell[BoxData[
 RowBox[{"Result", ":", "\[IndentingNewLine]", 
  RowBox[{"{", 
   RowBox[{
   "13591.63199999999960709828883409500122070313`10.153871520566971", ",", 
    "0"}], "}"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.708538839139428*^9, 
  3.708538847786375*^9}},ExpressionUUID->"e1c8f6d5-06ec-4510-a8ab-\
647f4b774068"],

Cell["\<\
The time needed was  332.936 s, and the result was 0, of course. The option 
TimeConstraint\[Rule] Infinity has been essential for getting the result.\
\>", "Text",
 CellChangeTimes->{{3.6636886369493103`*^9, 3.663688643716257*^9}, {
   3.663688687997151*^9, 3.663688761062792*^9}, {3.663688792911601*^9, 
   3.663688845384534*^9}, 3.705904396168517*^9, {3.708525125476673*^9, 
   3.708525129396451*^9}},ExpressionUUID->"99368ce6-1b3b-4852-81cb-\
06c913a64177"],

Cell[TextData[{
 "The last calculations confirm the following \n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". The covariant curvature tensor of a pseudo-Riemannian manifold has the \
following symmetry properties:"
}], "Text",
 CellChangeTimes->{{3.663765576673501*^9, 
  3.6637657283060513`*^9}},ExpressionUUID->"76fd1dd9-4771-430e-b4f3-\
722d346a930d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["R", 
    RowBox[{" ", "khij"}]], " ", "+", " ", 
   SubscriptBox["R", 
    RowBox[{" ", "hkij"}]]}], " ", "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663765822986397*^9, 3.6637658592662687`*^9}, {
  3.663765932468609*^9, 
  3.66376593388408*^9}},ExpressionUUID->"6bf3e1ca-f182-4104-9eef-\
d4efeac830de"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["R", 
    RowBox[{" ", "khij"}]], " ", "+", " ", 
   SubscriptBox["R", 
    RowBox[{" ", "khji"}]]}], " ", "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663765880835608*^9, 3.663765884871099*^9}, {
  3.663765956577495*^9, 
  3.66376595779272*^9}},ExpressionUUID->"ddd9cea6-4fe3-42d1-87a0-\
985b75974919"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["R", 
   RowBox[{" ", "khij"}]], " ", "\[Equal]", "  ", 
  SubscriptBox["R", 
   RowBox[{" ", "ijkh"}]], " "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663765880835608*^9, 3.663765892540791*^9}, {
  3.6637659717189713`*^9, 
  3.663766022912692*^9}},ExpressionUUID->"8ed0ca5f-e44b-4d10-8384-\
4bba31308644"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["R", 
    RowBox[{" ", "khij"}]], " ", "+", " ", 
   SubscriptBox["R", 
    RowBox[{" ", "kijh"}]], " ", "+", " ", 
   SubscriptBox["R", 
    RowBox[{" ", "kjhi"}]]}], "\[Equal]", " ", "0"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663765880835608*^9, 3.663765918159452*^9}, {
  3.663766039214686*^9, 3.663766049765482*^9}, {3.663766093120865*^9, 
  3.663766094112503*^9}},ExpressionUUID->"d282ba9d-dc73-46b1-8a58-\
84c68a5205d9"],

Cell[TextData[{
 "The last equation is known under the name ",
 StyleBox["Bianchi\[CloseCurlyQuote]s identity",
  FontColor->RGBColor[0, 0, 1]],
 ". A proof of this proposition can be found e.g. in [",
 ButtonBox["K-N1",
  BaseStyle->"Hyperlink",
  ButtonData->"K-N1"],
 "], \[Section]\[Section]V.1-2, or in [",
 ButtonBox["H78",
  BaseStyle->"Hyperlink",
  ButtonData->"H78"],
 "], \[Section] I.12. \[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6637662674645643`*^9, 3.663766325692718*^9}, {
  3.663766394685707*^9, 3.663766420925709*^9}, {3.665581994291835*^9, 
  3.665582035643982*^9}, {3.665582117987571*^9, 3.665582164148252*^9}, {
  3.665582194432274*^9, 3.6655821944384947`*^9}, {3.669877972599221*^9, 
  3.669877979830185*^9}},ExpressionUUID->"a752b111-1e98-441d-a22a-\
9d873a4ec4f4"],

Cell["\<\
Raising the index k in the last equation one sees that Bianchi\
\[CloseCurlyQuote]s identity is valid for the Riemannian curvature tensor \
too, e.g.:\
\>", "Text",
 CellChangeTimes->{{3.669878049109788*^9, 
  3.6698781547490377`*^9}},ExpressionUUID->"018f8be0-fe13-4d73-90a4-\
fadddf7945d2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{"RCT", " ", "=", 
   RowBox[{
    RowBox[{"RCtensor", "[", 
     RowBox[{"\[CapitalGamma]g", "[", "]"}], "]"}], "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.669877648369664*^9, 3.669877690920684*^9}, 
   3.705904825275896*^9},ExpressionUUID->"6203105c-407f-4320-88fa-\
bbfa54613cab"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"RCT", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "4"}], "}"}], ",", 
       RowBox[{"{", "1", "}"}]}], "]"}], "+", 
     RowBox[{"RCT", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "2"}], "}"}], ",", 
       RowBox[{"{", "1", "}"}]}], "]"}], "+", 
     RowBox[{"RCT", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"4", ",", "2", ",", "3"}], "}"}], ",", 
       RowBox[{"{", "1", "}"}]}], "]"}]}], ",", 
    RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6636511234036093`*^9, 3.663651218021834*^9}, {
  3.663652389028165*^9, 3.66365241557971*^9}, {3.6636880028977137`*^9, 
  3.663688023265685*^9}, {3.665067662832502*^9, 3.6650676655364523`*^9}, {
  3.669870915836277*^9, 3.669870925368874*^9}, {3.66987774299331*^9, 
  3.6698777870803013`*^9}, {3.669877823496284*^9, 
  3.6698778973915453`*^9}},ExpressionUUID->"041d6265-c1c2-49f0-926d-\
aa4e401d7fb3"],

Cell["\<\
The result zero appeared very fast already after 1.784 s (v. 9.0.1) !\
\>", "Text",
 CellChangeTimes->{{3.669878166812804*^9, 3.6698782594763803`*^9}, {
  3.705904891092166*^9, 3.7059049062350883`*^9}, {3.708567557080814*^9, 
  3.708567586719611*^9}},ExpressionUUID->"43ad9b66-a0df-4a9e-a94d-\
d6a0ae3a707b"],

Cell["\<\
Already in the notebook euklsfv3.nb of [EDG], Subsection 4.2.3, Lemmata 5,6 \
we considered the co variant curvature tensor of a surface in the Euclidean \
3-space, introduced the \[OpenCurlyDoubleQuote]only essential component\
\[CloseCurlyDoubleQuote] of this tensor and related it to the Gaussian \
curvature The surface (Lemma 6 there). The notation changed in the \
meantime;the correspondence is  \
\>", "Text",
 CellChangeTimes->{{3.7059297706889687`*^9, 3.705929875390749*^9}, {
  3.705929967709839*^9, 3.7059302768043137`*^9}, {3.705930565882265*^9, 
  3.70593057596201*^9}},ExpressionUUID->"fbba4795-2117-4761-b7a0-\
368082a79087"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"covRimm", "[", 
     RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", "sf", "]"}], 
   "[", 
   RowBox[{"u", ",", "v"}], "]"}], "\[Equilibrium]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"covrc", "[", 
     RowBox[{"rg", "[", "sf", "]"}], "]"}], "[", 
    RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6991626490288982`*^9, 3.6991626927080803`*^9}, 
   3.705929615377143*^9, {3.705930310803422*^9, 3.705930312811532*^9}, 
   3.705930424716436*^9, {3.7059305073065653`*^9, 3.705930530866269*^9}, 
   3.705930606580737*^9},ExpressionUUID->"74f1f239-5b11-492b-a87b-\
f821f3661530"],

Cell["We apply the mentioned Lemma 6 in the next two examples.", "Text",
 CellChangeTimes->{{3.70593065384046*^9, 
  3.705930694248407*^9}},ExpressionUUID->"5ba84d14-750d-438c-a546-\
d8ccb4d0e6dd"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[".",
  FontColor->GrayLevel[0]],
 " Curvature of the sphere."
}], "Text",
 CellChangeTimes->{{3.70273548318359*^9, 3.702735526519287*^9}, 
   3.702785571260413*^9, {3.70278560282021*^9, 
   3.702785609356266*^9}},ExpressionUUID->"2e7d692d-90b0-4379-982e-\
a49619643de0"],

Cell["We consider the sphere in the  Euclidean space", "Text",
 CellChangeTimes->{{3.631428802084368*^9, 3.6314289416798143`*^9}, {
  3.6315141990931654`*^9, 3.6315142007155685`*^9}, {3.705906076138177*^9, 
  3.705906092034186*^9}, {3.7059061432255363`*^9, 
  3.705906172529624*^9}},ExpressionUUID->"93ac2b99-3b53-4cc2-9710-\
f66ab3b7b2ea"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.705905968971739*^9, 
  3.7059059804750757`*^9}},ExpressionUUID->"5fb1f786-3253-437b-b939-\
9a06a9873ec2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.70590619195273*^9, 
  3.705906233601365*^9}},ExpressionUUID->"95104063-7219-4fbd-bf11-\
ff71ebb0242d"],

Cell["The induced metric is", "Text",
 CellChangeTimes->{{3.705906247728662*^9, 
  3.7059062574489107`*^9}},ExpressionUUID->"3aa00645-6b21-4ab2-84e1-\
93702f853878"],

Cell[BoxData[
 RowBox[{"?", "gsph"}]], "Input",
 CellChangeTimes->{{3.705905923584272*^9, 
  3.7059059257011557`*^9}},ExpressionUUID->"a4012c21-d704-4e87-8b24-\
c8122180a9b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGtensor", "[", 
   RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.70273540833707*^9, 3.702735418274446*^9}, 
   3.705663327782754*^9, {3.705905894115741*^9, 
   3.70590590169592*^9}},ExpressionUUID->"bac076d2-aec1-46f5-b8c0-\
e7711e158951"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"tensorToArray", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705906279907263*^9, 
  3.7059063459458323`*^9}},ExpressionUUID->"6aac1c09-18a5-435b-9de6-\
17f867c199f6"],

Cell["The covariant curvature tensor is", "Text",
 CellChangeTimes->{{3.705906383039589*^9, 
  3.705906407239788*^9}},ExpressionUUID->"1befac1a-5375-40e0-b568-\
bc9721f12e6a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"covRCtensor", "[", 
   RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.7059064143654213`*^9, 3.705906421823914*^9}, 
   3.705906479673373*^9},ExpressionUUID->"ad220d01-b7f4-429f-a848-\
2e92cb1278e9"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"tensorToArray", "[", "%", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.7059064917859707`*^9, 3.705906507231279*^9}, {
  3.705906618815405*^9, 
  3.705906641094103*^9}},ExpressionUUID->"c2e1d7b9-4f95-45cc-9701-\
ed15ce68c5da"],

Cell[TextData[{
 "All the non-zero coordinates are \[PlusMinus]",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["r", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "v", "]"}], "2"]}]], "Output",
  CellChangeTimes->{3.70590651401206*^9, 3.705906647652142*^9},
  ExpressionUUID->"8e249f03-6cc4-443f-a4c8-359545493ca1"],
 "; thus the \[OpenCurlyDoubleQuote]only essential component\
\[CloseCurlyDoubleQuote] of this tensor is"
}], "Text",
 CellChangeTimes->{{3.6314291057773046`*^9, 3.631429110365713*^9}, {
  3.631429186260311*^9, 3.631429202936741*^9}, {3.705906747085223*^9, 
  3.705906824900457*^9}, {3.705906880656763*^9, 
  3.705906929740333*^9}},ExpressionUUID->"11cdd1e9-ec45-4b72-9fbf-\
a016957470f4"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"covRCtensor", "[", 
      RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.631368432361788*^9, 3.63136846797865*^9}, {
   3.631368573029636*^9, 3.631368592984071*^9}, {3.6313689434484887`*^9, 
   3.63136894386969*^9}, 3.6314259378889036`*^9, {3.631514232773625*^9, 
   3.63151428703052*^9}, 3.664895108715498*^9, 3.66489540556221*^9, {
   3.664947896620805*^9, 3.66494791171658*^9}, {3.702733959231509*^9, 
   3.702733962765532*^9}, {3.7027340011732407`*^9, 3.70273400984527*^9}, {
   3.702734062869156*^9, 3.702734085084784*^9}, {3.705907353210507*^9, 
   3.7059073628895903`*^9}, {3.705928598170951*^9, 3.70592860597725*^9}, {
   3.705928702296001*^9, 
   3.705928702471826*^9}},ExpressionUUID->"1b120a17-7d1b-40d9-8aa3-\
19aa6502ba67"],

Cell["The determinant of the induced metric is", "Text",
 CellChangeTimes->{{3.705907248713407*^9, 
  3.705907279865294*^9}},ExpressionUUID->"7e2a7759-1435-4c6b-9688-\
96fc19d760d8"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"Det", "[", 
    RowBox[{
     RowBox[{"RG", "[", 
      RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"u", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.631369135230626*^9, 3.6313691548564606`*^9}, 
   3.6314257715770116`*^9, {3.631514412907141*^9, 3.631514450924408*^9}, 
   3.661510065597473*^9, {3.705907118444551*^9, 
   3.705907133907372*^9}},ExpressionUUID->"3f54ad7a-ce3c-483b-8e94-\
f8047f53647b"],

Cell["s is the sqare of the area element of the sphere:", "Text",
 CellChangeTimes->{{3.705907972509763*^9, 3.705907972528288*^9}, {
  3.7059080052864037`*^9, 
  3.705908029995799*^9}},ExpressionUUID->"ab078bd9-5b3b-4703-b021-\
5b19d7a7102b"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"%", "==", 
    RowBox[{
     RowBox[{
      RowBox[{"dVol", "[", 
       RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7059077928546*^9, 
  3.7059078984777517`*^9}},ExpressionUUID->"f7a8e405-8f8d-429b-8aad-\
79d3743ac424"],

Cell[TextData[{
 "Thus the  Gaussian curvature of the sphere K=1/",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["r", "2"], " ", "equals"}], TraditionalForm]],
  ExpressionUUID->"0cda7511-66ab-4a18-99e9-f0c687da9ea7"],
 " "
}], "Text",
 CellChangeTimes->{{3.6313695367461357`*^9, 3.631369574750803*^9}, {
   3.631369716255052*^9, 3.6313697243670664`*^9}, {3.6315144650892324`*^9, 
   3.631514466852036*^9}, {3.6315147712633705`*^9, 3.6315148428830967`*^9}, {
   3.705907416424336*^9, 3.705907485025865*^9}, 
   3.705907770413847*^9},ExpressionUUID->"cb9246d1-ae8d-439f-8d27-\
004e8795df58"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"covRCtensor", "[", 
       RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "}"}], ",", 
      RowBox[{"{", "}"}]}], "]"}], "/", 
    RowBox[{
     RowBox[{
      RowBox[{"dVol", "[", 
       RowBox[{"gsph", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"v", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6313696001028476`*^9, 3.631369700920225*^9}, {
  3.6314259657505527`*^9, 3.631425971007762*^9}, {3.631514493028882*^9, 
  3.6315145388149624`*^9}, {3.664947940933242*^9, 3.664947955653405*^9}, {
  3.6649495573798027`*^9, 3.66494956989855*^9}, {3.7059075881006317`*^9, 
  3.705907591639468*^9}, {3.705907662737307*^9, 
  3.7059076915286093`*^9}},ExpressionUUID->"3d4cfb4a-c4c4-4104-a944-\
e578ccc611ec"],

Cell["\<\
\[FilledSquare]
 Later, after introducing the concept of  the sectonal curvature, we shall \
show that the n-dimensional spherical Riemannian spaces have constant \
positive sectional curvature.\
\>", "Text",
 CellChangeTimes->{{3.6313697566309233`*^9, 3.6313697789741626`*^9}, {
   3.6313703847829247`*^9, 3.631370574302459*^9}, {3.6313706272433524`*^9, 
   3.631370661007812*^9}, 3.6314287632518997`*^9, {3.631514649692357*^9, 
   3.631514684090418*^9}, 3.6377293274935703`*^9, {3.700128899676736*^9, 
   3.7001289084002237`*^9}},ExpressionUUID->"bed4d801-5154-4522-afbc-\
c2fc47c4c72c"],

Cell[TextData[{
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". Analogously we calculate the Gaussian curvature of the hyperbolic plane. \
Its Riemannian metric is"
}], "Text",
 CellChangeTimes->{{3.631428802084368*^9, 3.6314289416798143`*^9}, {
   3.661240389855194*^9, 3.66124047201423*^9}, {3.661240536581463*^9, 
   3.661240608429428*^9}, {3.661240714956036*^9, 3.661240720500054*^9}, 
   3.661240756492819*^9, 3.666104907388955*^9, {3.7027854395107117`*^9, 
   3.702785462470047*^9}, {3.702785690852407*^9, 3.70278570964357*^9}, {
   3.7059081442919807`*^9, 3.7059081523713818`*^9}, {3.705908636872038*^9, 
   3.7059086477915783`*^9}, 
   3.706073264754362*^9},ExpressionUUID->"88ddc8ab-00a2-4e0d-b564-\
cbb3acc78121"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RG", "[", 
   RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.705908656346147*^9, 
  3.705908676737486*^9}},ExpressionUUID->"8e01afb4-209e-42af-ba83-\
ef0123d7ef65"],

Cell["\<\
The only singularities of this metric are the points{0,v}. The covariant \
curvature tensor is\
\>", "Text",
 CellChangeTimes->{{3.705908721078887*^9, 
  3.705908854758059*^9}},ExpressionUUID->"2f88ec8f-0950-4077-9e36-\
d5c371ba494d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"covRCtensor", "[", 
   RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.7059083513195877`*^9, 3.705908351321101*^9}, {
  3.705908429889248*^9, 
  3.705908441776904*^9}},ExpressionUUID->"8029cc97-dadb-408f-b946-\
4cc6b86dc4fb"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"tensorToArray", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.705908501303787*^9, 
  3.70590853945658*^9}},ExpressionUUID->"dfd96f57-4739-42fe-9424-\
0bc9a718eae2"],

Cell["\<\
The only essential component of the tensor covRCtensor in dimension 2 is\
\>", "Text",
 CellChangeTimes->{{3.6314291057773046`*^9, 3.631429110365713*^9}, {
   3.631429186260311*^9, 3.631429202936741*^9}, {3.66074777905348*^9, 
   3.6607477878532248`*^9}, {3.666105851452797*^9, 3.666105860756486*^9}, {
   3.666162803294634*^9, 3.666162806686541*^9}, {3.705909354866671*^9, 
   3.7059093582985353`*^9}, 3.7059287274472322`*^9, {3.705928766407333*^9, 
   3.70592878044698*^9}},ExpressionUUID->"c7cbec85-bd64-48f2-b62d-\
847ec26a767d"],

Cell[BoxData[
 RowBox[{"?", "covrc"}]], "Input",
 CellChangeTimes->{{3.705908889795991*^9, 
  3.705908898550161*^9}},ExpressionUUID->"52b72a8f-a4fa-400a-b2a5-\
10883e01c44d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"covrc", "[", 
    RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.631368432361788*^9, 3.63136846797865*^9}, {
   3.631368573029636*^9, 3.631368592984071*^9}, {3.6313689434484887`*^9, 
   3.63136894386969*^9}, 3.6314259378889036`*^9, 3.661240234934801*^9, 
   3.666105615545343*^9, {3.70590892428587*^9, 
   3.705908982325601*^9}},ExpressionUUID->"87dccf21-e1c3-4f4f-a17f-\
7ecf09e96e99"],

Cell["\<\
The volume (here area) element of the surface in this metric is\
\>", "Text",
 CellChangeTimes->{{3.6314292434112124`*^9, 3.63142927614607*^9}, 
   3.661410763787815*^9, 
   3.661411107208527*^9},ExpressionUUID->"287dd801-650c-4632-bc61-\
3fa824f21166"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dVol", "[", 
   RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.631369439801565*^9, 3.631369489708053*^9}, 
   3.631425951601328*^9, 3.6612402364131613`*^9, 3.66610565412888*^9, {
   3.7059090093411837`*^9, 
   3.705909011085412*^9}},ExpressionUUID->"06601980-8794-44f4-ae80-\
523c1d1699cf"],

Cell[BoxData[
 RowBox[{"Det", "[", 
  RowBox[{
   RowBox[{"RGhypsp", "[", "r", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.631369135230626*^9, 3.6313691548564606`*^9}, 
   3.6314257715770116`*^9, 3.66124010711022*^9, {3.666105673439042*^9, 
   3.666105691190319*^9}},ExpressionUUID->"914559e4-d602-4f7e-a1f9-\
46e0086ae1aa"],

Cell["\<\
Thus the  Gaussian curvature of this metric is negative constant:\
\>", "Text",
 CellChangeTimes->{{3.6313695367461357`*^9, 3.631369574750803*^9}, {
  3.631369716255052*^9, 
  3.6313697243670664`*^9}},ExpressionUUID->"4a88a6a7-ba35-455d-8be6-\
f07bd6499aa7"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"covrc", "[", 
       RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"1", ",", "2", ",", "1", ",", "2"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "/", 
    RowBox[{
     RowBox[{
      RowBox[{"dVol", "[", 
       RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"r", ">", "0"}], ",", 
     RowBox[{"u", ">", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6313696001028476`*^9, 3.631369700920225*^9}, {
   3.6314259657505527`*^9, 3.631425971007762*^9}, {3.661240238277309*^9, 
   3.661240240723728*^9}, {3.6661057140869093`*^9, 3.666105724630336*^9}, {
   3.705909051349442*^9, 3.705909063157524*^9}, 
   3.7059091095683813`*^9},ExpressionUUID->"5ca54463-9137-4053-8f0a-\
e8acd8e07200"],

Cell["\<\
\[FilledSquare] 
Later, after introducing the concept of  the sectional curvature, we shall \
show that the hyperbolic Riemannian spaces have constant negative sectional \
curvature.\
\>", "Text",
 CellChangeTimes->{{3.6313697566309233`*^9, 3.6313697789741626`*^9}, {
   3.6313703847829247`*^9, 3.631370574302459*^9}, {3.6313706272433524`*^9, 
   3.631370661007812*^9}, 3.6314287632518997`*^9, {3.702785669060183*^9, 
   3.7027856806913223`*^9}, 
   3.705909171276347*^9},ExpressionUUID->"b0f0606b-e246-4b6c-b570-\
289f2b12ad30"]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.4. Duality and Tensor Metric", "Subsubsection",
 CellChangeTimes->{{3.663829235269321*^9, 3.6638292680206823`*^9}, {
   3.665388995035955*^9, 3.665388997356093*^9}, 3.665495658614378*^9, 
   3.66911226451782*^9, 3.66972818591121*^9, 3.669878288524803*^9, 
   3.6699036263348722`*^9, {3.705930721736144*^9, 3.705930726200116*^9}, 
   3.706060536796853*^9, 3.708567911830207*^9, 
   3.708568147971096*^9},ExpressionUUID->"8b434415-766b-4fe8-ad80-\
d06aae5827a6"],

Cell[TextData[{
 "This subsection begins with a lemma  of tensor algebra. We know that the \
space  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ScriptCapitalT]", 
    RowBox[{"(", 
     RowBox[{"k", ",", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "954f72ce-5eee-4fa4-a652-19a18220e80f"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "74aa7a09-86f4-4941-8aef-90d592d9997c"],
 " of tensors of type (k,h) over the n-dimensional vector space V is a ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["n", 
    RowBox[{"k", "+", "h"}]], TraditionalForm]],ExpressionUUID->
  "c4da0e27-8be5-4bef-bd55-f00a4fc9a2bb"],
 "-dimensional vector space over the field K, here the field of real numbers. \
We show the following \n",
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ".  The tensor spaces ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ScriptCapitalT]", 
    RowBox[{"(", 
     RowBox[{"k", ",", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "1d86738a-f9bd-4194-a4ef-d120af37890c"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "d54fb0d9-6b15-47ed-9d34-34894b47d255"],
 "  and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ScriptCapitalT]", 
    RowBox[{"(", 
     RowBox[{"h", ",", "k"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "78cc40b9-9c62-498c-ac12-3ffbb593d1fb"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "1c9724b1-d851-494b-8559-c72cbc75caa1"],
 "  are dual to each other in a canonical way.\n",
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ". Let {",
 Cell[BoxData[
  FormBox[
   SubscriptBox["b", "i"], TraditionalForm]],ExpressionUUID->
  "38469d90-1243-49e0-b8d0-a8522b638499"],
 "} be a basis of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "6ed7499d-4b40-4188-9519-86d59ddba987"],
 " and {",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["v", "j"], "}"}], " "}], TraditionalForm]],
  ExpressionUUID->"5a59f7cd-f5bf-4a95-8769-1678e170f677"],
 "be the dual basis in the dual space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "2bb7ec32-f1d3-4d13-b312-ca7baefb62e6"],
 "*:"
}], "Text",
 CellChangeTimes->{{3.663829843232057*^9, 3.6638298891596737`*^9}, {
   3.6638299249434977`*^9, 3.663829968838893*^9}, {3.6638300712810163`*^9, 
   3.6638301160494213`*^9}, {3.663830152614237*^9, 3.663830166663398*^9}, {
   3.663830458075137*^9, 3.663830640346039*^9}, {3.663830789720687*^9, 
   3.663830818136986*^9}, {3.663830855610737*^9, 3.663830876239935*^9}, {
   3.663830912680067*^9, 3.663831020383288*^9}, {3.6638310700542603`*^9, 
   3.66383108809479*^9}, {3.6638413398678293`*^9, 3.6638413497667503`*^9}, 
   3.6638413905635223`*^9, {3.6638414628191013`*^9, 3.663841473290724*^9}, {
   3.6638415049800386`*^9, 3.663841543789851*^9}, {3.663841576029621*^9, 
   3.663841586221485*^9}, {3.663841685302638*^9, 3.663841899875145*^9}, 
   3.665127171032085*^9, 3.7059817641006393`*^9, {3.70598180184424*^9, 
   3.705981808123439*^9}},ExpressionUUID->"33f72e6c-057b-4418-8372-\
d4343191321a"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SuperscriptBox["v", "j"], "(", 
     SubscriptBox["b", "i"], ")"}], "  ", "=", " ", 
    RowBox[{"<", 
     SuperscriptBox["v", "j"]}]}], ",", 
   RowBox[{
    RowBox[{
     SubscriptBox["b", "i"], ">"}], " ", "=", " ", 
    SubsuperscriptBox["\[Delta]", "i", "j"]}], ","}], 
  TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663841938811152*^9, 
  3.663842111337484*^9}},ExpressionUUID->"adf68e03-f4ce-4cf4-9550-\
65ddc64fcf5f"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Delta]", "i", "j"], TraditionalForm]], "Input",
  CellChangeTimes->{{3.663841938811152*^9, 3.663842111337484*^9}},
  ExpressionUUID->"11a5af8f-d60d-4c70-bd66-766910926b39"],
 " the Kronecker symbol. Then consider the canonical corresponding tensor \
bases"
}], "Text",
 CellChangeTimes->{{3.6638421549396048`*^9, 3.663842173864579*^9}, {
  3.663842222896454*^9, 
  3.6638422836000853`*^9}},ExpressionUUID->"c95d59ec-a280-4c51-96a2-\
6b150b2fe772"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["v", 
           SubscriptBox["j", "1"]], " ", "\[CircleTimes]"}], "..."}], 
        "\[CircleTimes]", 
        SuperscriptBox["v", 
         SubscriptBox["j", "k"]], "\[CircleTimes]", " ", 
        SubscriptBox["b", 
         SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
      "\[CircleTimes]", " ", 
      SubscriptBox["b", 
       SubscriptBox["i", "h"]]}], " ", "}"}], "  ", "of", "  ", 
    SuperscriptBox["\[ScriptCapitalT]", 
     RowBox[{"(", 
      RowBox[{"k", ",", "h"}], ")"}]], 
    SuperscriptBox["V", "n"]}], ",", " "}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6638423454106693`*^9, 3.663842470779087*^9}, {
   3.663842502651578*^9, 3.663842686915042*^9}, {3.663842872829878*^9, 
   3.663842885582006*^9}, 
   3.663843023492765*^9},ExpressionUUID->"10122f7e-9061-4a88-9183-\
bcb38727724f"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["v", 
          SubscriptBox["j", "1"]], " ", "\[CircleTimes]"}], "..."}], 
       "\[CircleTimes]", 
       SuperscriptBox["v", 
        SubscriptBox["j", "h"]], "\[CircleTimes]", " ", 
       SubscriptBox["b", 
        SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
     "\[CircleTimes]", " ", 
     SubscriptBox["b", 
      SubscriptBox["i", "k"]]}], " ", "}"}], "  ", "of", "  ", 
   SuperscriptBox["\[ScriptCapitalT]", 
    RowBox[{"(", 
     RowBox[{"h", ",", "k"}], ")"}]], 
   RowBox[{
    SuperscriptBox["V", "n"], ".", " "}]}], TraditionalForm]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6638423454106693`*^9, 3.663842470779087*^9}, {
   3.663842502651578*^9, 3.663842707994739*^9}, 3.663842758894875*^9, {
   3.6638427928946447`*^9, 3.663842815125955*^9}, {3.663842894621957*^9, 
   3.663842916485773*^9}, 
   3.663843033900599*^9},ExpressionUUID->"7e79ea62-6275-43a4-908d-\
c72f91f0b905"],

Cell["Consider the tensors", "Text",
 CellChangeTimes->{{3.663843064204467*^9, 
  3.663843072732339*^9}},ExpressionUUID->"ba4a219d-4564-4b47-8264-\
fdc9242f8784"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sa", " ", "=", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["v", 
           SubscriptBox["j", "1"]], " ", "\[CircleTimes]"}], "..."}], 
        "\[CircleTimes]", 
        SuperscriptBox["v", 
         SubscriptBox["j", "k"]], "\[CircleTimes]", " ", 
        SubscriptBox["b", 
         SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
      "\[CircleTimes]", " ", 
      SubscriptBox["b", 
       SubscriptBox["i", "h"]]}], 
     SuperscriptBox[
      SubsuperscriptBox["a", 
       RowBox[{
        RowBox[{
         SubscriptBox["j", "1"], "..."}], 
        SubscriptBox["j", "k"]}], "       "], 
      RowBox[{
       RowBox[{
        SubscriptBox["i", "1"], "..."}], 
       SubscriptBox["i", "h"], "    "}]]}], "\[Element]", " ", 
    RowBox[{
     SuperscriptBox["\[ScriptCapitalT]", 
      RowBox[{"(", 
       RowBox[{"k", ",", "h"}], ")"}]], 
     SuperscriptBox["V", "n"]}]}]}], ",", " "}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6638431421457567`*^9, 3.6638431999241667`*^9}, {
   3.663843241132163*^9, 3.663843255464082*^9}, {3.6638432895246487`*^9, 
   3.6638433377475023`*^9}, {3.66384338814023*^9, 3.6638434681706047`*^9}, {
   3.663843499410858*^9, 3.6638435689543047`*^9}, {3.663843610770308*^9, 
   3.663843613291429*^9}, {3.663843912197686*^9, 3.663843916404347*^9}, 
   3.663843964838213*^9, {3.663910190960559*^9, 3.663910194760065*^9}, 
   3.7059820152035217`*^9, {3.7059820725195837`*^9, 
   3.705982076011271*^9}},ExpressionUUID->"17e3d223-3660-4cd1-8ba5-\
abedd4705616"],

Cell[BoxData[
 RowBox[{"tc", " ", "=", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["v", 
          SubscriptBox["j", "1"]], " ", "\[CircleTimes]"}], "..."}], 
       "\[CircleTimes]", 
       SuperscriptBox["v", 
        SubscriptBox["j", "h"]], "\[CircleTimes]", " ", 
       SubscriptBox["b", 
        SubscriptBox["i", "1"]], "\[CircleTimes]"}], "..."}], 
     "\[CircleTimes]", " ", 
     SubscriptBox["b", 
      SubscriptBox["i", "k"]]}], 
    SuperscriptBox[
     SubsuperscriptBox["c", 
      RowBox[{
       RowBox[{
        SubscriptBox["j", "1"], "..."}], 
       SubscriptBox["j", "h"]}], "    "], 
     RowBox[{
      RowBox[{
       SubscriptBox["i", "1"], "..."}], 
      SubscriptBox["i", "k"]}]]}], "    ", "\[Element]", "  ", 
   RowBox[{
    SuperscriptBox["\[ScriptCapitalT]", 
     RowBox[{"(", 
      RowBox[{"h", ",", "k"}], ")"}]], 
    RowBox[{
     SuperscriptBox["V", "n"], ".", " "}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6638431421457567`*^9, 3.6638431999241667`*^9}, {
  3.663843241132163*^9, 3.663843255464082*^9}, {3.6638432895246487`*^9, 
  3.6638433377475023`*^9}, {3.66384338814023*^9, 3.6638434681706047`*^9}, {
  3.663843499410858*^9, 3.6638435689543047`*^9}, {3.663843610770308*^9, 
  3.6638436959934177`*^9}, {3.6638437381537313`*^9, 3.663843744905613*^9}, {
  3.6638438429933567`*^9, 3.663843848881638*^9}, {3.663843979103251*^9, 
  3.6638440085900917`*^9}, {3.663910224608274*^9, 3.663910233255637*^9}, {
  3.70598214462652*^9, 
  3.705982155474195*^9}},ExpressionUUID->"8e0226e1-904a-478d-a003-\
9c0f67aae883"],

Cell["Then we define the scalar product", "Text",
 CellChangeTimes->{{3.663844090181422*^9, 
  3.66384411802739*^9}},ExpressionUUID->"23e2abe0-de0a-45b9-95bc-\
91f7a12501f9"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"tc", ",", "sa"}], ")"}], " ", "\[Element]", "  ", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[ScriptCapitalT]", 
      RowBox[{"(", 
       RowBox[{"h", ",", "k"}], ")"}]], 
     SuperscriptBox["V", "n"], "  ", "\[Times]", "   ", 
     SuperscriptBox["\[ScriptCapitalT]", 
      RowBox[{"(", 
       RowBox[{"k", ",", "h"}], ")"}]], 
     SuperscriptBox["V", "n"]}], " ", "\[RightTeeArrow]", 
    "\[IndentingNewLine]", 
    RowBox[{"<", "tc"}]}]}], ",", 
  RowBox[{
   RowBox[{"sa", ">"}], ":=", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      SubsuperscriptBox["c", 
       RowBox[{
        RowBox[{
         SubscriptBox["i", "1"], "..."}], 
        SubscriptBox["i", "h"]}], "      "], 
      RowBox[{
       RowBox[{
        SubscriptBox["j", "1"], "..."}], 
       SubscriptBox["j", "k"]}]], " ", 
     SuperscriptBox[
      SubsuperscriptBox["a", 
       RowBox[{
        RowBox[{
         SubscriptBox["j", "1"], "..."}], 
        SubscriptBox["j", "k"]}], "      "], 
      RowBox[{
       RowBox[{
        SubscriptBox["i", "1"], "..."}], 
       SubscriptBox["i", "h"]}]]}], " ", "\[Element]", " ", 
    RowBox[{"K", ".", "                  ", 
     RowBox[{"(", "1", ")"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.663844130696533*^9, 3.663844160922092*^9}, {
   3.6638441988180847`*^9, 3.663844256865671*^9}, {3.663844291546966*^9, 
   3.6638443278983603`*^9}, 3.663844376643506*^9, {3.6638444187397633`*^9, 
   3.6638445305622168`*^9}, {3.663844564386043*^9, 3.663844570874506*^9}, {
   3.66384460643537*^9, 3.663844678610381*^9}, {3.663910248288351*^9, 
   3.66391026831995*^9}, {3.663910347862337*^9, 3.663910360551199*^9}, {
   3.705982192642272*^9, 
   3.705982238761693*^9}},ExpressionUUID->"4e2d748b-2081-4ab0-a88e-\
6d634e2f6315"],

Cell[TextData[{
 "All these formulas are written in traditional notation. Remember the sum \
convention: about indices appearing twice is to sum over the whole range of \
the index. The definition of the scalar product (1) is a complete contraction \
of the tensor product tc",
 Cell[BoxData[
  FormBox["\[CircleTimes]", TraditionalForm]],ExpressionUUID->
  "b02a1511-d881-400b-af29-8d421325c33f"],
 "sa. For all tc \[Element] ",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{
    SuperscriptBox["\[ScriptCapitalT]", 
     RowBox[{"(", 
      RowBox[{"h", ",", "k"}], ")"}]], 
    SuperscriptBox["V", "n"]}]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6638431421457567`*^9, 3.6638431999241667`*^9}, {
   3.663843241132163*^9, 3.663843255464082*^9}, {3.6638432895246487`*^9, 
   3.6638433377475023`*^9}, {3.66384338814023*^9, 3.6638434681706047`*^9}, {
   3.663843499410858*^9, 3.6638435689543047`*^9}, {3.663843610770308*^9, 
   3.6638436959934177`*^9}, {3.6638437381537313`*^9, 3.663843744905613*^9}, {
   3.6638438429933567`*^9, 3.663843848881638*^9}, {3.663843979103251*^9, 
   3.6638440085900917`*^9}, {3.663910224608274*^9, 3.663910233255637*^9}},
  ExpressionUUID->"dfbedbdb-73e7-4c04-8a37-99e6a2986f93"],
 " the function tc(sa) := <tc,sa> is a linear form on ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[ScriptCapitalT]", 
    RowBox[{"(", 
     RowBox[{"k", ",", "h"}], ")"}]], 
   SuperscriptBox["V", "n"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6638431421457567`*^9, 3.6638431999241667`*^9}, {
    3.663843241132163*^9, 3.663843255464082*^9}, {3.6638432895246487`*^9, 
    3.6638433377475023`*^9}, {3.66384338814023*^9, 3.6638434681706047`*^9}, {
    3.663843499410858*^9, 3.6638435689543047`*^9}, {3.663843610770308*^9, 
    3.663843613291429*^9}, {3.663843912197686*^9, 3.663843916404347*^9}, 
    3.663843964838213*^9, {3.663910190960559*^9, 3.663910194760065*^9}},
  ExpressionUUID->"a1f8f504-bdb2-4a8c-a142-a6ade35af3d0"],
 " being the null form only iff tc equals the null tensor. Since the \
dimensions of ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[ScriptCapitalT]", 
    RowBox[{"(", 
     RowBox[{"h", ",", "k"}], ")"}]], 
   SuperscriptBox["V", "n"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6638431421457567`*^9, 3.6638431999241667`*^9}, {
   3.663843241132163*^9, 3.663843255464082*^9}, {3.6638432895246487`*^9, 
   3.6638433377475023`*^9}, {3.66384338814023*^9, 3.6638434681706047`*^9}, {
   3.663843499410858*^9, 3.6638435689543047`*^9}, {3.663843610770308*^9, 
   3.6638436959934177`*^9}, {3.6638437381537313`*^9, 3.663843744905613*^9}, {
   3.6638438429933567`*^9, 3.663843848881638*^9}, {3.663843979103251*^9, 
   3.6638440085900917`*^9}, {3.663910224608274*^9, 3.663910233255637*^9}},
  ExpressionUUID->"6d593a2e-9201-46e6-9507-dbd3adc3213a"],
 " equals the dimension of ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox["\[ScriptCapitalT]", 
    RowBox[{"(", 
     RowBox[{"k", ",", "h"}], ")"}]], 
   SuperscriptBox["V", "n"]}]], "Input",
  Evaluatable->False,
  CellChangeTimes->{{3.6638431421457567`*^9, 3.6638431999241667`*^9}, {
    3.663843241132163*^9, 3.663843255464082*^9}, {3.6638432895246487`*^9, 
    3.6638433377475023`*^9}, {3.66384338814023*^9, 3.6638434681706047`*^9}, {
    3.663843499410858*^9, 3.6638435689543047`*^9}, {3.663843610770308*^9, 
    3.663843613291429*^9}, {3.663843912197686*^9, 3.663843916404347*^9}, 
    3.663843964838213*^9, {3.663910190960559*^9, 3.663910194760065*^9}},
  ExpressionUUID->"d8136d9f-f0b0-47c6-9b3b-5519f00973db"],
 " the spaces are dual to each other, and the tensor bases we used to define \
(1) are dual bases. ",
 "\[FilledSquare]",
 " "
}], "Text",
 CellChangeTimes->{{3.6638448285956287`*^9, 3.663844897330134*^9}, {
  3.663910120079932*^9, 3.663910125823793*^9}, {3.66391037661432*^9, 
  3.663910407182125*^9}, {3.663910438838045*^9, 3.663910535197536*^9}, {
  3.663910567117437*^9, 3.6639106779324083`*^9}, {3.66391815539039*^9, 
  3.6639184096925097`*^9}, {3.663918450535307*^9, 
  3.663918590245755*^9}},ExpressionUUID->"1f251c70-c0a7-40a8-b069-\
98abdfca6379"],

Cell["\<\
For tensor fields tc = tc[x],sa = sa[x] on a manifold  formula (1) can be \
applied pointwise . Then the scalar product is a function\
\>", "Text",
 CellChangeTimes->{{3.663920507321344*^9, 3.663920600016417*^9}, 
   3.663920791366954*^9, {3.664005700929626*^9, 
   3.664005717281155*^9}},ExpressionUUID->"0dd57a04-4a88-404d-9590-\
f6b4bea71420"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], " ", ":=", " ", 
   RowBox[{"<", 
    RowBox[{"tc", "[", "x", "]"}]}]}], ",", 
  RowBox[{
   RowBox[{"sa", "[", "x", "]"}], ">", 
   RowBox[{
   ".", "                                                               ", 
    RowBox[{"(", "2", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6639206373875303`*^9, 3.6639206537685013`*^9}, {
   3.663920696897401*^9, 3.6639207580638657`*^9}, 
   3.663920808391181*^9},ExpressionUUID->"6bf58f21-b1b7-4d93-8bae-\
be10e7219ead"],

Cell[TextData[{
 "As we know, the spaces ",
 Cell[BoxData[
  RowBox[{
   FormBox[
    SuperscriptBox["\[ScriptCapitalJ]", 
     RowBox[{"(", 
      RowBox[{"h", ",", "k"}], ")"}]],
    TraditionalForm], 
   FormBox[
    SuperscriptBox["M", "n"],
    TraditionalForm]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"aab57a6e-5e01-483e-b447-ca7a793cb0ce"],
 " of tensor fields  on a smooth manifold are infinite dimensional (see \
Proposition 2.3.1), and their exist much more linear functions on ",
 Cell[BoxData[
  RowBox[{
   FormBox[
    SuperscriptBox["\[ScriptCapitalJ]", 
     RowBox[{"(", 
      RowBox[{"k", ",", " ", "h"}], ")"}]],
    TraditionalForm], 
   FormBox[
    SuperscriptBox["M", "n"],
    TraditionalForm]}]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"26c9784c-753c-40b8-8047-93c01abf92af"],
 " as those defined by formula (2). \n\nNow we consider a pseudo-Riemannian \
manifold. Each tangent space of this manifold is a pseudo-Euclidean vector \
space. Combining formula (1) at each point x \[Element] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],
  Evaluatable->False,
  CellChangeTimes->{{3.6349722024639835`*^9, 3.634972385838107*^9}, {
   3.6349724173727627`*^9, 3.634972473638462*^9}},
  FontWeight->"Bold",ExpressionUUID->"e5398cbb-4eb3-4416-9baf-c6b9a068c0c7"],
 " with the index gymnastics described in the last subsection  we will in a \
canonical way define a pseudo-Euclidean metric in each local tensor space ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"k", ",", " ", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "cd7ee092-124b-4acb-a28f-28a2afc69e57"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "1e7c410a-3b4e-4266-bd54-e8d014801d69"],
 " .To explain the method we consider"
}], "Text",
 CellChangeTimes->{{3.663921004109354*^9, 3.663921109692218*^9}, {
   3.66392115144377*^9, 3.663921189587488*^9}, {3.6639212346537533`*^9, 
   3.663921261387006*^9}, {3.663921314802668*^9, 3.663921501169348*^9}, {
   3.663921580643159*^9, 3.663921611088552*^9}, {3.663921657568274*^9, 
   3.663921720647862*^9}, 3.664005913847394*^9, {3.6640059645437117`*^9, 
   3.664005979414961*^9}, {3.6651274722316628`*^9, 3.665127515847637*^9}, {
   3.6651276116236258`*^9, 3.6651276748556843`*^9}, {3.665127762360037*^9, 
   3.6651278562002993`*^9}, {3.665127893088377*^9, 3.6651279143213987`*^9}, 
   3.665127944351945*^9},ExpressionUUID->"4c53e128-ad50-4606-a39d-\
7b8bcc110b5b"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"2", ",", "1"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "579f439f-b9db-4b20-bded-0ef11381d4ec"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "bfd6bf35-5a99-40c3-966e-b1a72faf1504"],
 ".  Let sa be a tensor of type (2,1) and tc \[Element] ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"1", ",", "2"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "c96eab6d-8273-4342-81b2-0eac2933dbcb"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "67b94002-10d2-411b-8f65-1dc33e799638"],
 " an element of the dual space. Then the map"
}], "Text",
 CellChangeTimes->{
  3.6640059213041077`*^9, 3.664005997166175*^9, {3.6640060781093884`*^9, 
   3.6640060852932568`*^9}, {3.664006617945067*^9, 3.664006862754068*^9}, {
   3.664006922399222*^9, 3.664006925222879*^9}, {3.664620836681107*^9, 
   3.66462084212121*^9}, {3.66462129696907*^9, 
   3.664621309569249*^9}},ExpressionUUID->"3768a896-00b5-4f44-902f-\
55da59df0a0b"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tc", " ", "=", " ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["c", "k", 
      RowBox[{"  ", "ij"}]], 
     RowBox[{
      SuperscriptBox["v", "k"], "\[CircleTimes]", 
      SubscriptBox["b", "i"], "\[CircleTimes]", 
      SubscriptBox["b", "j"]}]}], " ", "\[RightTeeArrow]", " ", 
    RowBox[{
     SubsuperscriptBox["c", "ij", 
      RowBox[{"    ", "k"}]], 
     RowBox[{
      SuperscriptBox["v", "i"], "\[CircleTimes]", 
      SuperscriptBox["v", "j"], "\[CircleTimes]", 
      SubscriptBox["b", "k"]}], "  ", "with"}]}]}], 
  "  "}], "\[IndentingNewLine]", 
 RowBox[{
  SubsuperscriptBox["c", "ij", 
   RowBox[{"    ", "k"}]], " ", ":=", " ", 
  RowBox[{
   SubscriptBox["g", "il"], 
   SubscriptBox["g", "jm"], 
   SuperscriptBox["g", 
    RowBox[{"kh", " "}]], 
   SubsuperscriptBox["c", "h", 
    RowBox[{"  ", "lm"}]], "                               ", 
   RowBox[{"(", "3", ")"}]}]}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.66400695324058*^9, 3.664007054054577*^9}, {
   3.664007115822296*^9, 3.664007261611101*^9}, {3.66400732862893*^9, 
   3.664007331284556*^9}, {3.664007362229805*^9, 3.664007507771495*^9}, {
   3.6640075589640703`*^9, 3.66400756639537*^9}, {3.6640076248106537`*^9, 
   3.664007717106395*^9}, {3.664007747638747*^9, 3.664007755729692*^9}, 
   3.664007786874115*^9, {3.6640078203133507`*^9, 3.664007830689434*^9}, {
   3.664204730434966*^9, 3.66420473801101*^9}, 
   3.705982871932741*^9},ExpressionUUID->"7be41465-e682-45a0-83d1-\
721b67e9c07b"],

Cell["\<\
is defined by raising the index k and lowering the indices i,j. This map is \
canonical, since it is defined by tensor operations with the fixed given \
metric tensor RG. It can be used to identify the tensor space with its dual:\
\>", "Text",
 CellChangeTimes->{{3.664007902793469*^9, 3.664008100759265*^9}, 
   3.664022211733103*^9, {3.664023753523013*^9, 3.664023756483245*^9}, {
   3.665128049048265*^9, 3.6651280546397743`*^9}, 
   3.705982902195546*^9},ExpressionUUID->"b5cc0087-8898-4eca-8500-\
2ece96fa4512"],

Cell[BoxData[
 RowBox[{"tc", " ", "=", " ", 
  RowBox[{
   RowBox[{
    SubsuperscriptBox["c", "k", 
     RowBox[{"  ", "ij"}]], 
    RowBox[{
     SuperscriptBox["v", "k"], "\[CircleTimes]", 
     SubscriptBox["b", "i"], "\[CircleTimes]", 
     SubscriptBox["b", "j"]}]}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["c", "ij", 
      RowBox[{"    ", "k"}]], 
     RowBox[{
      SuperscriptBox["v", "i"], "\[CircleTimes]", 
      SuperscriptBox["v", "j"], "\[CircleTimes]", 
      SubscriptBox["b", "k"]}], "  ", "with", "  ", 
     SubsuperscriptBox["c", "ij", 
      RowBox[{"    ", "k"}]]}], " ", ":=", " ", 
    RowBox[{
     SubscriptBox["g", "il"], 
     SubscriptBox["g", "jm"], 
     SuperscriptBox["g", 
      RowBox[{"kh", " "}]], 
     SubsuperscriptBox["c", "h", 
      RowBox[{"  ", "lm"}]], "                              ", 
     RowBox[{"(", "4", ")"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.6640224310151987`*^9, {3.664023093673732*^9, 3.664023095957938*^9}, {
   3.664204750546267*^9, 
   3.664204759594553*^9}},ExpressionUUID->"fefea4e6-b9f5-485b-a9ac-\
6687ab8ce92c"],

Cell[TextData[{
 "The two decompositions describe the same tensor in different  \
representations. In the case of Riemannian manifolds the tangent spaces are \
Euclidean vector spaces; if we choose orthonormal bases in the vector space \
the coordinates of the metric tensor are given by the Kronecker symbol ",
 Cell[BoxData[
  SubscriptBox["g", "ij"]], "Input",
  CellChangeTimes->{
   3.6640224310151987`*^9, {3.664023093673732*^9, 3.664023095957938*^9}},
  ExpressionUUID->"b7ec4fde-71ae-4ab6-8979-9e6ceab2cb98"],
 " = ",
 Cell[BoxData[
  SubscriptBox["\[Delta]", "ij"]], "Input",
  CellChangeTimes->{
   3.6640224310151987`*^9, {3.664023093673732*^9, 3.664023095957938*^9}},
  ExpressionUUID->"e3f3d7ca-9bf4-46fb-8a7e-582e9d423c19"],
 ", and in this case we get ",
 Cell[BoxData[
  SubsuperscriptBox["c", "ij", 
   RowBox[{"    ", "k"}]]], "Input",
  CellChangeTimes->{
   3.6640224310151987`*^9, {3.664023093673732*^9, 3.664023095957938*^9}},
  ExpressionUUID->"5ad71d18-3a66-4867-ac18-115c43f1609d"],
 " = ",
 Cell[BoxData[
  SubsuperscriptBox["c", "k", 
   RowBox[{"  ", "ij"}]]], "Input",
  CellChangeTimes->{
   3.6640224310151987`*^9, {3.664023093673732*^9, 3.664023095957938*^9}},
  ExpressionUUID->"3b7b17df-fb84-4c7e-a9af-bb90ab0a909a"],
 ", both representations coincide. Therefore in elemtary Euclidean geometry \
it is not necessary to distinguish covariant and contravariant objects or \
indices. "
}], "Text",
 CellChangeTimes->{{3.664022477076602*^9, 3.664022579136477*^9}, {
   3.6640227292679777`*^9, 3.664022770930683*^9}, 3.664023056570444*^9, {
   3.6640231063931217`*^9, 3.664023258565741*^9}, 3.664023295484741*^9, {
   3.6640233344599953`*^9, 3.664023465524743*^9}, {3.664023498371565*^9, 
   3.664023615406733*^9}, {3.6640237919477243`*^9, 3.664023803404191*^9}, {
   3.664023858593573*^9, 3.664023897993297*^9}, {3.664024230764023*^9, 
   3.664024230766774*^9}, 3.6642004150699177`*^9, {3.665128111496861*^9, 
   3.6651281179055347`*^9}},ExpressionUUID->"a5c1d256-ee91-4d14-9bac-\
6e5e042ff645"],

Cell[TextData[{
 "Now we take a second tensor sa \[Element] ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"1", ",", "2"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "8de52235-c3c6-4f34-9d2b-9ae88cfac69a"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "0dbae415-6497-4401-ad9f-b374ee5e3050"],
 " :"
}], "Text",
 CellChangeTimes->{{3.66408399801693*^9, 3.66408404537955*^9}, {
  3.66408408381739*^9, 
  3.664084102889407*^9}},ExpressionUUID->"1c49490b-051d-49e7-a66c-\
f7213a945055"],

Cell[BoxData[
 RowBox[{"sa", " ", "=", " ", 
  RowBox[{
   SubsuperscriptBox["a", "k", 
    RowBox[{"  ", "ij"}]], 
   RowBox[{
    SuperscriptBox["v", "k"], "\[CircleTimes]", 
    SubscriptBox["b", "i"], "\[CircleTimes]", 
    SubscriptBox["b", "j"], " "}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.664084106973917*^9, 3.6640841874183702`*^9}, {
  3.6640842355865192`*^9, 
  3.664084240299505*^9}},ExpressionUUID->"71f8e3a8-6e26-4e3e-aa33-\
0e618903f574"],

Cell["and form the scalar product (1)", "Text",
 CellChangeTimes->{{3.664084282482917*^9, 
  3.664084303875176*^9}},ExpressionUUID->"4afd1f27-5f88-4f96-889f-\
261051da5ed6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"<", "tc"}], ",", 
  RowBox[{
   RowBox[{"sa", ">"}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["c", "ij", 
      RowBox[{"    ", "k"}]], " ", 
     SubsuperscriptBox["a", "k", 
      RowBox[{"  ", "ij"}]]}], "=", " ", 
    RowBox[{
     SubscriptBox["g", "il"], 
     SubscriptBox["g", "jm"], 
     SuperscriptBox["g", 
      RowBox[{"kh", " "}]], 
     SubsuperscriptBox["c", "h", 
      RowBox[{"  ", "lm"}]], 
     SubsuperscriptBox["a", "k", 
      RowBox[{"  ", "ij"}]], "                ", 
     RowBox[{"(", "5", ")"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6640843245917683`*^9, 3.664084432421517*^9}, 
   3.664084496727462*^9, 3.664084552223637*^9, {3.664204785601816*^9, 
   3.664204796601132*^9}, 3.66990198304206*^9, {3.705983050019186*^9, 
   3.7059831714500437`*^9}},ExpressionUUID->"08786d32-27ac-43e1-88db-\
cc15fc0c0d23"],

Cell[TextData[{
 "Obviously, this equation defines a bilinear scalar product on the vector \
space ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"1", ",", "2"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "f72ba5da-8a56-450c-a1a9-f76d23c89aa3"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "07c9c309-6595-43ae-9369-b740ab0ad61f"],
 ". Now raising the indices h,k we obtain"
}], "Text",
 CellChangeTimes->{{3.664084834773961*^9, 3.664084965540934*^9}, {
  3.664193606947385*^9, 
  3.664193665322947*^9}},ExpressionUUID->"58a1869f-0193-4e3f-a53e-\
8e8b4be10626"],

Cell[BoxData[
 RowBox[{
  RowBox[{"<", "tc"}], ",", 
  RowBox[{
   RowBox[{"sa", ">"}], " ", "=", "  ", 
   RowBox[{
    RowBox[{
     SubscriptBox["g", "il"], 
     SubscriptBox["g", "jm"], 
     SuperscriptBox["g", 
      RowBox[{"kh", " "}]], 
     SuperscriptBox["c", 
      RowBox[{" ", 
       RowBox[{
        SubscriptBox["h", "1"], "lm"}]}]], 
     SuperscriptBox["a", 
      RowBox[{" ", 
       RowBox[{
        SubscriptBox["k", "1"], "ij"}]}]], 
     SubscriptBox["g", 
      SubscriptBox["hh", "1"]], 
     SubscriptBox["g", 
      SubscriptBox["kk", "1"]]}], "\[Equal]", " ", 
    RowBox[{
     SubscriptBox["g", "il"], 
     SubscriptBox["g", "jm"], 
     SubscriptBox["g", "kh"], 
     SuperscriptBox["c", 
      RowBox[{" ", "klm"}]], 
     SuperscriptBox["a", 
      RowBox[{" ", "hij"}]], " ", 
     RowBox[{"(", "6", ")"}]}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6641937059390163`*^9, 3.664193731636018*^9}, {
  3.6641937783949327`*^9, 3.664193962394906*^9}, {3.664194039714149*^9, 
  3.664194072257825*^9}, {3.664194499160828*^9, 3.664194567112462*^9}, {
  3.664194604304858*^9, 3.664194638848271*^9}, {3.664204810905386*^9, 
  3.664204820785033*^9}, {3.705983238585925*^9, 
  3.7059833383771257`*^9}},ExpressionUUID->"857c5caa-a371-43c6-af0c-\
44dbf9d8ba6a"],

Cell[TextData[{
 "To get  the right side of the equation we used ",
 Cell[BoxData[
  SuperscriptBox["g", 
   RowBox[{"kh", " "}]]], "Input",
  CellChangeTimes->{{3.6641937059390163`*^9, 3.664193731636018*^9}, {
   3.6641937783949327`*^9, 3.664193962394906*^9}, {3.664194039714149*^9, 
   3.664194072257825*^9}, {3.664194499160828*^9, 3.664194567112462*^9}, {
   3.664194604304858*^9, 3.664194638848271*^9}},ExpressionUUID->
  "61ab72dd-e91f-4a7b-80eb-78117da651ae"],
 Cell[BoxData[
  SubscriptBox["g", 
   SubscriptBox["kk", "1"]]], "Input",
  CellChangeTimes->{{3.6641937059390163`*^9, 3.664193731636018*^9}, {
   3.6641937783949327`*^9, 3.664193962394906*^9}, {3.664194039714149*^9, 
   3.664194072257825*^9}, {3.664194499160828*^9, 3.664194567112462*^9}, {
   3.664194604304858*^9, 3.664194638848271*^9}},ExpressionUUID->
  "0a77f2aa-b9fb-4bfd-9c3a-0b53e6146938"],
 " = ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Delta]", 
    SubscriptBox["k", "1"], "h"], TraditionalForm]],ExpressionUUID->
  "980dc88e-01c5-44ec-8dc6-6de7517bfdef"],
 " and renamed k1 \[RightArrow]h. \[FilledSquare]\nWe conclude:"
}], "Text",
 CellChangeTimes->{{3.664194780655301*^9, 3.664194837635062*^9}, {
  3.6641949171221848`*^9, 3.664195055191627*^9}, {3.664195193967767*^9, 
  3.664195282856042*^9}, {3.664200428064457*^9, 
  3.664200431462325*^9}},ExpressionUUID->"eb2511c8-ada9-426a-858d-\
7cff2a6d68c7"],

Cell[TextData[{
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ". Let ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "a7ce7541-9ea0-4ad4-88ad-419a5634814b"],
 " be an n-dimensional pseudo-Euclidean vector space of index k. \na) Then \
each tensor space ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["T", 
    RowBox[{"(", 
     RowBox[{"h", ",", "k"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "4469857d-67f9-407f-bcf7-9d437ac39eaf"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "f514a9b5-de61-4209-8dc3-d6f447fa5706"],
 " is a pseudo-Euclidean vector space of dimension ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["n", 
    RowBox[{"k", "+", "h"}]], TraditionalForm]],ExpressionUUID->
  "6a8923ce-9e20-4988-8af5-901b04d8dd21"],
 ";  its metric tensor is a (k+h)-times tensor product of the metric tensor \
in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "d863e61e-451b-41bb-967a-da86552e04d1"],
 ", namely"
}], "Text",
 CellChangeTimes->{{3.66419529425464*^9, 3.66419531991921*^9}, {
   3.6641999011103067`*^9, 3.664199903439062*^9}, {3.664199944952327*^9, 
   3.664199982822606*^9}, {3.664200013678822*^9, 3.664200093023511*^9}, {
   3.664200125720808*^9, 3.6642001569931507`*^9}, 3.6642002538223047`*^9, {
   3.664200370510128*^9, 3.6642003921422663`*^9}, {3.664200564094339*^9, 
   3.6642007744706*^9}, {3.6642011010610447`*^9, 3.664201153644264*^9}, 
   3.66420217426819*^9, 3.6642052248073378`*^9, {3.664255123316637*^9, 
   3.6642551233171062`*^9}},ExpressionUUID->"427a589e-f7dc-4ce2-8e99-\
9450f7a8cf93"],

Cell[BoxData[
 RowBox[{
  RowBox[{"<", "tc"}], ",", " ", 
  RowBox[{
   RowBox[{"sa", ">"}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["g", "il"], 
        SubscriptBox["g", "jm"]}], "..."}], 
      SuperscriptBox["g", "kh"]}], "..."}], 
    SubsuperscriptBox["c", 
     RowBox[{"k", "..."}], 
     RowBox[{"    ", 
      RowBox[{"lm", "..."}]}]], 
    SubsuperscriptBox["a", 
     RowBox[{"h", "..."}], 
     RowBox[{"    ", 
      RowBox[{"ij", "..."}]}]], "                 ", 
    RowBox[{"(", "7", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.664200809486517*^9, 3.66420081148621*^9}, {
  3.6642008748061733`*^9, 3.6642009450785923`*^9}, {3.6642009854219427`*^9, 
  3.664201026558631*^9}, {3.664201059614401*^9, 3.664201070678039*^9}, {
  3.664204832712904*^9, 3.6642048412244997`*^9}, {3.7059834227204313`*^9, 
  3.705983435888021*^9}},ExpressionUUID->"99ef9d1b-29ba-4c7c-83f0-\
c353aaf27086"],

Cell["\<\
b) The tensor spaces of the same degree are canonically isomorphic as \
pseudo-Euclidean vector spaces.\
\>", "Text",
 CellChangeTimes->{{3.664201210122731*^9, 3.664201302943699*^9}, 
   3.6642021538604603`*^9},ExpressionUUID->"80e370e9-4df3-4bed-85a4-\
893df1e74814"],

Cell[TextData[{
 "c) These statements are true also for the tangent spaces of \
pseudo-Riemannian manifolds. In particular,  the local tensor  spaces ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["T", "x", 
    RowBox[{"(", 
     RowBox[{"k", ",", "h"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "bc8f210b-61ce-4d29-961e-02e292d89c84"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "n"], TraditionalForm]],ExpressionUUID->
  "d634e1cd-a1e8-4f21-9413-b2b5287f3635"],
 " of a Riemannian manifold  are Euclidean vector spaces with a positive \
definite scalar product."
}], "Text",
 CellChangeTimes->{{3.664201330686797*^9, 3.6642013325265493`*^9}, {
   3.6642013879012814`*^9, 3.664201493835091*^9}, {3.664201549954368*^9, 
   3.664201699286274*^9}, 3.664202163732368*^9, {3.664205027285829*^9, 
   3.6642050289729433`*^9}, 3.66420513520333*^9, 3.6651283058324747`*^9, 
   3.70598352529526*^9},ExpressionUUID->"c4cc497f-c601-434d-8744-\
b8fa77349f07"],

Cell["\<\
d) The pointwise application of the scalar product to tensor fields of the \
same type (k,h) \
\>", "Text",
 CellChangeTimes->{{3.6642017570695877`*^9, 3.664201890753935*^9}, 
   3.664201936604828*^9},ExpressionUUID->"fdc44b4e-844c-4d6f-b1d9-\
8f8ee16edba1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"<", 
   RowBox[{"tc", "[", "x", "]"}]}], ",", " ", 
  RowBox[{
   RowBox[{"sa", "[", "x", "]"}], ">", 
   "                                                                  ", 
   RowBox[{"(", "8", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.664201989272251*^9, 3.664202028607395*^9}, {
   3.664204855648546*^9, 3.664204866552402*^9}, {3.66990222214395*^9, 
   3.669902258623727*^9}, 
   3.7059835828148203`*^9},ExpressionUUID->"ec5a04a4-edd0-4a16-8842-\
02b0bbf61193"],

Cell["are invariant under isometric transformations.", "Text",
 CellChangeTimes->{{3.6642017570695877`*^9, 3.664201890753935*^9}, 
   3.664201936604828*^9, {3.664202043974667*^9, 3.664202061942322*^9}, 
   3.7059836092615147`*^9},ExpressionUUID->"0e01e8f4-17f1-4036-92dc-\
fd7584dd7838"],

Cell[TextData[{
 StyleBox["Proof",
  FontColor->RGBColor[0, 0, 1]],
 ".  The definition (5) can be written as a complete contraction of a tensor \
product. Therefore the result doesn\[CloseCurlyQuote]t depend on the \
coordinate system and it is bilinear and symmetric. The statement b) is shown \
in the last Example, formula (6). Thus we may restrict to the case of \
contravariant tensors, type (0,k). Furthermore we may assume that the basis \
of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "884caeae-f274-4f30-8e54-12add1a66492"],
 " is standard pseudo-Euclidean, that means "
}], "Text",
 CellChangeTimes->{{3.664254521795733*^9, 3.664254542663971*^9}, {
   3.6642546156553926`*^9, 3.664254732344861*^9}, {3.664254791917954*^9, 
   3.6642548879733553`*^9}, {3.664254957461048*^9, 3.664255054644455*^9}, {
   3.664255135230053*^9, 3.6642551756596737`*^9}, 3.664256729438861*^9, {
   3.665128339639597*^9, 
   3.665128359990946*^9}},ExpressionUUID->"4440c5c4-ea80-4dca-ae3a-\
e2c75b68256d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["g", "ij"], "=", " ", 
   RowBox[{
    SubscriptBox["\[Delta]", "ij"], 
    SubscriptBox["\[Epsilon]", "i"]}]}], ",", " ", 
  RowBox[{
   SubscriptBox["\[Epsilon]", "i"], " ", "=", " ", 
   RowBox[{"\[PlusMinus]", "1"}]}], ",", "  ", "i", ",", 
  RowBox[{"j", " ", "=", " ", "1"}], ",", "...", ",", 
  RowBox[{"n", ".", "                           ", 
   RowBox[{"(", "9", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6642552195721827`*^9, 3.66425536233733*^9}, {
   3.664255424386524*^9, 3.664255468177327*^9}, {3.664256295202078*^9, 
   3.6642563248012457`*^9}, 
   3.7059836696698627`*^9},ExpressionUUID->"e43d669f-4066-4a22-aece-\
bbfb785d72fb"],

Cell["\<\
Then also the corresponding tensor basis is standard pseudo-Euclidean:\
\>", "Text",
 CellChangeTimes->{{3.664256384016568*^9, 
  3.664256430144182*^9}},ExpressionUUID->"92d1ac19-7cf8-4306-aae5-\
d55f12fa18fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"<", " ", 
     RowBox[{
      SubscriptBox["b", 
       SubscriptBox["i", "1"]], "\[CircleTimes]"}]}], "..."}], 
   "\[CircleTimes]", " ", 
   SubscriptBox["b", 
    SubscriptBox["i", "k"]]}], ",", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["b", 
        SubscriptBox["j", "1"]], "\[CircleTimes]"}], "..."}], 
     "\[CircleTimes]", " ", 
     SubscriptBox["b", 
      SubscriptBox["j", "k"]]}], ">"}], " ", "=", " ", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["i", "1"], "..."}], 
       SubscriptBox["i", "k"]}], ",", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["j", "1"], "..."}], 
       SubscriptBox["j", "k"]}]}]], 
    SubscriptBox["\[Epsilon]", 
     RowBox[{
      RowBox[{
       SubscriptBox["i", "1"], "..."}], 
      SubscriptBox["i", "k"]}]], "   ", 
    RowBox[{"(", "10", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.664256464365237*^9, {3.6642565363371353`*^9, 3.664256604086912*^9}, {
   3.664256659959496*^9, 3.664256689974784*^9}, {3.6642567510143213`*^9, 
   3.6642568522695227`*^9}, 
   3.705983724382196*^9},ExpressionUUID->"c4ffb0fe-82ef-4621-a40f-\
21023db09e7b"],

Cell[TextData[{
 "where \[Delta] denotes the ",
 StyleBox["generalized Kronecker symbol",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.664256912242311*^9, 3.6642569856275263`*^9}, {
  3.664257167713999*^9, 
  3.664257190649967*^9}},ExpressionUUID->"3dc8a407-e2ac-48ac-b7c9-\
5309a38c0111"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Delta]", 
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["i", "1"], "..."}], 
     SubscriptBox["i", "k"]}], ",", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["j", "1"], "..."}], 
     SubscriptBox["j", "k"]}]}]], " ", "=", 
  RowBox[{
   UnderoverscriptBox[
    RowBox[{" ", "\[Product]"}], 
    RowBox[{"s", "=", "1"}], "k"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     SubscriptBox["i", "s"], " ", 
     SubscriptBox["j", "s"]}]], "       ", 
   StyleBox["and",
    FontColor->GrayLevel[0]], "                         ", 
   RowBox[{"(", "11", ")"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.664257240210002*^9, 3.664257245194147*^9}, {
  3.664257313715796*^9, 3.664257466056069*^9}, {3.664257651078392*^9, 
  3.664257660750198*^9}, {3.664257711901916*^9, 3.664257730774221*^9}, {
  3.6642582976888227`*^9, 3.664258306238304*^9}, {3.705983766029813*^9, 
  3.7059837702452803`*^9}},ExpressionUUID->"63e44bc7-32c0-4db9-9b70-\
009f5565af28"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Epsilon]", 
   RowBox[{
    RowBox[{
     SubscriptBox["i", "1"], "..."}], 
    SubscriptBox["i", "k"]}]], " ", "=", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox[
     RowBox[{" ", "\[Product]"}], 
     RowBox[{"s", "=", "1"}], "k"], " ", 
    SubscriptBox["\[Epsilon]", 
     SubscriptBox["i", "s"]]}], " ", "=", " ", 
   RowBox[{
    RowBox[{"\[PlusMinus]", "1."}], "                                          ", 
    RowBox[{"(", "12", ")"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{
  3.664257544167377*^9, {3.6642575768403997`*^9, 3.664257639830391*^9}, {
   3.664257676934372*^9, 3.664257697646413*^9}, 3.664257740141913*^9, {
   3.664257781565784*^9, 3.664257809277199*^9}, {3.664258314424737*^9, 
   3.664258317248158*^9}, {3.705983807325691*^9, 
   3.705983816189282*^9}},ExpressionUUID->"c06a100c-358a-4666-8b32-\
8c3286260305"],

Cell[TextData[{
 "It follows, that the tensor metric is not degenerated, so it is \
pseudo-Euclidean. In particular, if ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["V", "n"], TraditionalForm]],ExpressionUUID->
  "8b0706cf-e051-4442-ba4b-9dee620a153a"],
 " is Euclidean or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["M", "n"], " "}], TraditionalForm]],ExpressionUUID->
  "87d55d5a-fa58-4184-ae7c-fe4c5c209c82"],
 "is a Riemannian manifold, all the ",
 Cell[BoxData[
  SubscriptBox["\[Epsilon]", 
   SubscriptBox["i", "s"]]], "Input",
  CellChangeTimes->{
   3.664257544167377*^9, {3.6642575768403997`*^9, 3.664257639830391*^9}, {
    3.664257676934372*^9, 3.664257697646413*^9}, 3.664257740141913*^9, {
    3.664257781565784*^9, 3.664257809277199*^9}},ExpressionUUID->
  "32f83663-b9e4-4685-b9c5-ee489fa073b8"],
 " are 1, and the tensor spaces are Euclidean too. The other statements are \
obvious consequences. ",
 "\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.66425798181888*^9, 3.6642582651846313`*^9}, {
  3.664258353898724*^9, 3.664258353898768*^9}, {3.664258405679346*^9, 
  3.664258443500464*^9}},ExpressionUUID->"07d7de2b-1725-4361-9ff1-\
21c97f9151da"],

Cell["\<\
To calculate the index of a pseudo - Euclidean Tensor metric first open the \
subnotebook index.nb and evaluate its initialization cells. Then apply the \
function index\
\>", "Text",
 CellChangeTimes->{{3.664259259815917*^9, 3.664259455141815*^9}, {
  3.665129995025229*^9, 
  3.665129999057239*^9}},ExpressionUUID->"0f49f8c4-733b-4193-84a9-\
7983a1baf93a"],

Cell[BoxData[
 RowBox[{"?", "index"}]], "Input",
 CellChangeTimes->{{3.6699024830901318`*^9, 
  3.6699024965571547`*^9}},ExpressionUUID->"35043fb3-9259-4400-b067-\
67e775abf9b8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ind", ",", "dim"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.664259460941621*^9, 3.664259463334203*^9}, {
   3.664538318174011*^9, 3.664538423117285*^9}, {3.664538479300488*^9, 
   3.664538564709467*^9}, 
   3.666159379773774*^9},ExpressionUUID->"5dd66b6a-c627-44b6-a22c-\
2744578c57b2"],

Cell[TextData[{
 "For example, in case of the 4-dimensional ",
 ButtonBox["Lorentzian space,   ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/LorentzianSpace.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/LorentzianSpace.html"]
}], "Text",
 CellChangeTimes->{{3.664259484581193*^9, 3.664259490941291*^9}, {
   3.6642622688619127`*^9, 3.664262305773621*^9}, {3.6642623537893343`*^9, 
   3.6642623537907543`*^9}, {3.664262530331646*^9, 3.6642625516677103`*^9}, {
   3.6642626731628428`*^9, 3.66426269641022*^9}, {3.66426278264179*^9, 
   3.6642627969223623`*^9}, 
   3.664262856369216*^9},ExpressionUUID->"572f6111-bfc2-44d3-98ea-\
13a2da4d3d2d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "4"}], ";", 
  RowBox[{"ind", "=", "1"}], ";"}]], "Input",
 CellChangeTimes->{{3.66434440758661*^9, 
  3.6643444407647953`*^9}},ExpressionUUID->"54e558e3-9995-4e88-a694-\
af48d8beb432"],

Cell["we calculate", "Text",
 CellChangeTimes->{{3.664262900048231*^9, 3.664262911368236*^9}, 
   3.665130059904394*^9},ExpressionUUID->"bfba0ceb-0113-4296-bfba-\
c53c47ed102e"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"index", "[", 
     RowBox[{"4", ",", "1", ",", "h"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"h", ",", "1", ",", "5"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.664259496108252*^9, 3.6642595664134808`*^9}, 
   3.6642618378033543`*^9},ExpressionUUID->"f3bea879-53e8-47a7-863e-\
98fb45db9f73"],

Cell["\<\
The program needed 46.344 s to calculate the 5 indices. The result is
{1,6,28,120,496}, and the dimensions of the tensor spaces are\
\>", "Text",
 CellChangeTimes->{{3.664263693173977*^9, 3.6642637061660633`*^9}, {
   3.664263741170162*^9, 3.6642638886285353`*^9}, 
   3.705989677800869*^9},ExpressionUUID->"ba17d4c6-514c-4db6-80a2-\
c75c66cb3b42"],

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"4", "^", "h"}], ",", 
   RowBox[{"{", 
    RowBox[{"h", ",", "5"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6642619066411667`*^9, 
  3.664261936480712*^9}},ExpressionUUID->"cb1dc3b7-95b5-4043-81ac-\
70752256deb5"],

Cell[TextData[{
 "\[FilledSquare]\n",
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". Tensors over the Lorentzian space"
}], "Text",
 CellChangeTimes->{{3.664284595361926*^9, 3.66428461618373*^9}, {
  3.664284705543132*^9, 3.66428473433508*^9}, {3.664451923665303*^9, 
  3.664451951960578*^9}, {3.66462092609929*^9, 
  3.6646209294700747`*^9}},ExpressionUUID->"ef70f698-f7ee-4a33-b11a-\
4cc1798c4df2"],

Cell["The metric of the Lorentzian space has the coordinates", "Text",
 CellChangeTimes->{{3.664265171754492*^9, 
  3.664265335449246*^9}},ExpressionUUID->"1e2dbbd1-f7d8-4fc6-b6ab-\
ad0d97a232b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"lor", ",", "ind"}], "]"}], ";", 
  RowBox[{
   RowBox[{"\"\<Lorewntzian space\>\"", 
    RowBox[{"lor", "[", 
     RowBox[{"i_", ",", "j_", ",", 
      RowBox[{"k_:", "ind"}]}], "]"}]}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Not", "[", 
          RowBox[{"IntegerQ", "[", "i", "]"}], "]"}], "||", 
         RowBox[{"Not", "[", 
          RowBox[{"IntegerQ", "[", "j", "]"}], "]"}], "||", 
         RowBox[{"(", 
          RowBox[{"i", "<=", "0"}], ")"}], "||", 
         RowBox[{"(", 
          RowBox[{"j", "<=", "0"}], ")"}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<Invalid arguments!\>\"", "]"}], ";", 
         RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"i", "\[Equal]", "j"}], ")"}], "&&", 
         RowBox[{"(", 
          RowBox[{"i", "\[LessEqual]", "k"}], ")"}]}], ",", 
        RowBox[{"-", "1"}], ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"i", "\[Equal]", "j"}], ")"}], "&&", 
           RowBox[{"(", 
            RowBox[{"i", ">", "k"}], ")"}]}], ",", "1", ",", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"i", "!=", "j"}], ")"}], ",", "0"}], "]"}]}], "]"}]}], 
       "]"}]}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.664265340051578*^9, 3.664265383744734*^9}, {
   3.664347480300942*^9, 3.6643474926766768`*^9}, {3.664347639723579*^9, 
   3.664347826050449*^9}, {3.664347912433794*^9, 3.664347916913764*^9}, {
   3.664347972873768*^9, 3.664348189136011*^9}, {3.664348366503089*^9, 
   3.664348377478903*^9}, {3.664348412885973*^9, 3.664348416773444*^9}, 
   3.6643485316405287`*^9, 3.664348634859029*^9, {3.664348733310651*^9, 
   3.6643488650892553`*^9}, {3.6643488988235826`*^9, 3.664349021603578*^9}, 
   3.664349063234145*^9, {3.6643491499597893`*^9, 3.6643491741820707`*^9}, {
   3.664349292593761*^9, 3.664349316545395*^9}, {3.664349652847859*^9, 
   3.664349658910927*^9}, 3.664349765196621*^9, {3.664349823491426*^9, 
   3.66434984917019*^9}, 3.66437389064791*^9, {3.66453693895509*^9, 
   3.6645369901624403`*^9}, {3.664539169839159*^9, 3.664539200582755*^9}, 
   3.6645392342947607`*^9, {3.670076553020965*^9, 
   3.6700765663891773`*^9}},ExpressionUUID->"7d63b13c-572a-49d3-8b8c-\
846fb7782027"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"lorg", "[", 
     RowBox[{"i_", ",", "j_", ",", 
      RowBox[{"k_:", "ind"}]}], "]"}], "[", "x__", "]"}], ":=", " ", 
   RowBox[{"lor", "[", 
    RowBox[{"i", ",", "j", ",", "k"}], "]"}]}], "\[LineSeparator]"}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.664373966630209*^9, 3.6643740294774523`*^9}, {
   3.6645370250255003`*^9, 3.66453704290547*^9}, {3.664537237944076*^9, 
   3.664537241423624*^9}, 
   3.669902649711439*^9},ExpressionUUID->"e158f5a3-909c-48c7-b6e2-\
01324b9de1ec"],

Cell[BoxData[
 RowBox[{"?", "lor"}]], "Input",
 CellChangeTimes->{{3.664620953669963*^9, 
  3.6646209613481407`*^9}},ExpressionUUID->"0fdae242-f41c-4475-847f-\
c2f861f28a65"],

Cell[BoxData[
 RowBox[{"?", "lorg"}]], "Input",
 CellChangeTimes->{{3.6651302571595583`*^9, 
  3.665130259549741*^9}},ExpressionUUID->"ec31e670-9dc3-4fc7-867a-\
a5c64ec2ba7b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"lorg", "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.705991830233848*^9, 3.7059918744423656`*^9}, {
  3.7059919050821133`*^9, 
  3.7059919260421762`*^9}},ExpressionUUID->"dec86dce-fd77-41a9-bcbf-\
533134da0bf9"],

Cell[BoxData[
 RowBox[{"Context", "[", "lor", "]"}]], "Input",
 CellChangeTimes->{{3.705991493732052*^9, 
  3.705991507844108*^9}},ExpressionUUID->"aaee6dd0-aabb-4094-b0f9-\
08c95c391cee"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ind", " ", "=", " ", "1"}], ";", 
  RowBox[{"dim", "=", " ", "4"}], ";", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"lor", "[", 
      RowBox[{"i", ",", "j"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "//", "\[LineSeparator]", 
   "MatrixForm"}]}]], "Input",
 CellChangeTimes->{{3.664265396639043*^9, 3.664265450416418*^9}, {
   3.6643486674425087`*^9, 3.6643486936083193`*^9}, {3.664349876594008*^9, 
   3.664349882953003*^9}, 3.664374043141941*^9, {3.66443360455956*^9, 
   3.6644336060145063`*^9}, {3.664433679174349*^9, 3.664433680477479*^9}, {
   3.664433713885404*^9, 3.6644337210528297`*^9}, {3.664537084042366*^9, 
   3.664537086129119*^9}, {3.664537348743617*^9, 3.664537372223256*^9}, 
   3.664538105897153*^9, {3.664538190151544*^9, 3.664538252111783*^9}, 
   3.664538686715621*^9, {3.664538974329777*^9, 3.6645390056170177`*^9}, {
   3.664539049192007*^9, 3.664539096903207*^9}, {3.664539294758896*^9, 
   3.664539320589806*^9}, 3.664539506932652*^9, {3.664539541987595*^9, 
   3.664539559908215*^9}, {3.664542548689662*^9, 3.6645425579455976`*^9}, {
   3.66454263641684*^9, 3.66454264743357*^9}, {3.66513010608176*^9, 
   3.665130122105523*^9}},ExpressionUUID->"9d2e1865-1a37-49b3-982e-\
a23ea0d6c17b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ind", " ", "=", " ", "2"}], ";", 
  RowBox[{"dim", "=", " ", "4"}], ";", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"lorg", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{"i", ",", "j"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "dim"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "//", 
   "MatrixForm"}]}]], "Input",
 CellChangeTimes->{{3.664538758025543*^9, 3.6645387935699873`*^9}, {
  3.665130140335932*^9, 3.665130167096126*^9}, {3.665130274214856*^9, 
  3.665130294749861*^9}},ExpressionUUID->"6d54fc20-d053-4f35-8869-\
3bfab7e13874"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ind", "=", "1"}], ";"}]], "Input",
 CellChangeTimes->{{3.664538722284161*^9, 3.664538724450449*^9}, {
   3.664541102790053*^9, 3.664541103878051*^9}, 
   3.6651303079965973`*^9},ExpressionUUID->"bdb7d5c4-5735-4f61-bf93-\
0326121a3d05"],

Cell[BoxData[
 RowBox[{"?", "RG"}]], "Input",
 CellChangeTimes->{{3.6643496821277857`*^9, 
  3.664349683397684*^9}},ExpressionUUID->"657846f4-6900-440d-93d2-\
4014f7a60195"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ind", "=", "2"}], ";", 
  RowBox[{
   RowBox[{"RG", "[", "lorg", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.664349181900681*^9, 3.664349202540962*^9}, {
   3.664349422678228*^9, 3.6643494493004227`*^9}, {3.6643497211980267`*^9, 
   3.664349742788838*^9}, 3.664374068485712*^9, {3.66453729144746*^9, 
   3.664537311159275*^9}, {3.705991701939024*^9, 3.705991735130253*^9}, {
   3.705991943642559*^9, 
   3.705991952977189*^9}},ExpressionUUID->"005769f8-e013-4629-80f4-\
7381b99d01db"],

Cell["Consider two vectors:", "Text",
 CellChangeTimes->{{3.6642848987173777`*^9, 
  3.6642849094856367`*^9}},ExpressionUUID->"711f4dde-6e86-4156-b396-\
c54cc8424e62"],

Cell[BoxData[
 RowBox[{"?", "pssp"}]], "Input",
 CellChangeTimes->{{3.664537448998974*^9, 
  3.664537451829782*^9}},ExpressionUUID->"343adcad-b96d-490a-b058-\
29b6201ffa38"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ind", "=", "1"}], ";", 
  RowBox[{"va", "=", " ", 
   RowBox[{"vec", "[", "a", "]"}]}], ";", " ", 
  RowBox[{"vb", "=", "  ", 
   RowBox[{"vec", "[", "b", "]"}]}], ";", " ", 
  RowBox[{"pssp", "[", 
   RowBox[{"va", ",", "vb"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.664263999106452*^9, 3.6642640094523973`*^9}, {
  3.664264162371587*^9, 3.664264336465802*^9}, {3.6642646670872097`*^9, 
  3.664264754678443*^9}, {3.664269797352306*^9, 3.664269830720903*^9}, {
  3.6642724564307747`*^9, 3.664272490781694*^9}, {3.664619289669265*^9, 
  3.664619297852379*^9}},ExpressionUUID->"841372dd-4acf-43fd-baf0-\
1e5be9540041"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vva", "=", " ", 
    RowBox[{"vecTotensor", "[", 
     RowBox[{"va", "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", "3", "]"}], "\[Rule]", "0"}], ",", 
        RowBox[{
         RowBox[{"a", "[", "4", "]"}], "\[Rule]", "0"}]}], "}"}]}], "]"}]}], 
   ";", " ", 
   RowBox[{"vvb", "=", "  ", 
    RowBox[{"vecTotensor", "[", 
     RowBox[{"vb", "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", "3", "]"}], "\[Rule]", "0"}], ",", 
        RowBox[{
         RowBox[{"b", "[", "4", "]"}], "\[Rule]", "0"}]}], "}"}]}], "]"}]}], 
   ";"}], " "}]], "Input",
 CellChangeTimes->{{3.664263999106452*^9, 3.6642640094523973`*^9}, {
  3.664264162371587*^9, 3.664264336465802*^9}, {3.6642646670872097`*^9, 
  3.664264754678443*^9}, {3.664272523830612*^9, 3.664272569965584*^9}, {
  3.6642751974733143`*^9, 3.6642752422725687`*^9}, {3.6642753572483873`*^9, 
  3.6642753706787567`*^9}},ExpressionUUID->"d83f5ec5-4e42-49bd-b80e-\
faa335d921d6"],

Cell["Its wedge product", "Text",
 CellChangeTimes->{{3.664284929189443*^9, 
  3.664284935989326*^9}},ExpressionUUID->"7d66be6c-ea1b-4191-baeb-\
b4fc4e1e70c7"],

Cell[BoxData[
 RowBox[{"vab", "=", 
  RowBox[{"vva", "\[Wedge]", "vvb"}]}]], "Input",
 CellChangeTimes->{{3.664264363235574*^9, 3.6642644135927153`*^9}, {
   3.66426462808012*^9, 3.664264642415407*^9}, {3.6642648027983913`*^9, 
   3.664264806301467*^9}, {3.6642649897031403`*^9, 3.6642649897852993`*^9}, {
   3.705992195023642*^9, 3.705992203734939*^9}, 
   3.705992246048108*^9},ExpressionUUID->"85c89594-dbc9-4c35-9535-\
8b21a75f641b"],

Cell["with the coordinates", "Text",
 CellChangeTimes->{{3.6642852897758827`*^9, 
  3.664285303947773*^9}},ExpressionUUID->"a372bcdc-4410-46ae-8efc-\
fce3ce9fa551"],

Cell[BoxData[
 RowBox[{"vabarr", "=", 
  RowBox[{"tensorToArray", "[", "%", "]"}]}]], "Input",
 CellChangeTimes->{{3.664264841378982*^9, 3.66426485774175*^9}, 
   3.664264889149539*^9, {3.664265123795727*^9, 3.664265142843503*^9}, {
   3.664265473561379*^9, 
   3.664265474448332*^9}},ExpressionUUID->"272a2255-2e4e-442d-a9ab-\
db8ee7fb5077"],

Cell["\<\
The scalar sqare of this tensor is - 2*(Area[va,vb])^2, Area[va,vb] the area \
of the parallelogram spanned by va,vb:\
\>", "Text",
 CellChangeTimes->{{3.664284962924889*^9, 3.664284979844865*^9}, 
   3.664285030628749*^9, {3.664285067700204*^9, 3.664285209843601*^9}, {
   3.66428553466472*^9, 
   3.6642855580247602`*^9}},ExpressionUUID->"a842cd84-881f-448a-a40f-\
e65d65d04d81"],

Cell[BoxData[
 RowBox[{"sqvab", "=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"lorg", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{"i", ",", "k"}], "]"}], "*", 
     RowBox[{
      RowBox[{"lorg", "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{"j", ",", "l"}], "]"}], "*", 
     RowBox[{"vabarr", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "j"}], "]"}], "]"}], "*", 
     RowBox[{"vabarr", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "j"}], "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"l", ",", "4"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.664265500905938*^9, 3.664265504239942*^9}, {
   3.664268257623623*^9, 3.664268513581657*^9}, {3.664275943283423*^9, 
   3.664275951363379*^9}, {3.6643499824082613`*^9, 3.664349990936142*^9}, 
   3.664374138509609*^9, 3.664374176757159*^9, {3.665130412739455*^9, 
   3.665130452738697*^9}},ExpressionUUID->"0bbcf20c-65e7-4005-849f-\
591b4707ae48"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.664350065477647*^9, 
  3.664350080292511*^9}},ExpressionUUID->"e81b30a6-a11a-4d89-83d6-\
cc02c3394d55"],

Cell["Now consider its tensor product", "Text",
 CellChangeTimes->{{3.664285226106908*^9, 
  3.664285241634495*^9}},ExpressionUUID->"350fbb15-58ff-47ba-9cf7-\
142fc7f53d35"],

Cell[BoxData[
 RowBox[{"tab", " ", "=", " ", 
  RowBox[{"vva", "\[CircleTimes]", "vvb"}]}]], "Input",
 CellChangeTimes->{{3.664268625797866*^9, 
  3.6642686460418253`*^9}},ExpressionUUID->"6a7d3b75-9efc-4fa3-8a26-\
15951598ff76"],

Cell["with the coordinates", "Text",
 CellChangeTimes->{{3.6642852897758827`*^9, 
  3.664285303947773*^9}},ExpressionUUID->"4bd0758e-e911-420d-b3eb-\
aee1c2998ff8"],

Cell[BoxData[
 RowBox[{"tabarr", "=", " ", 
  RowBox[{"tensorToArray", "[", "%", "]"}]}]], "Input",
 CellChangeTimes->{{3.664268672739436*^9, 
  3.664268731441463*^9}},ExpressionUUID->"3b967c8c-7c49-4384-b915-\
7bce0213806d"],

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"lorg", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"i", ",", "k"}], "]"}], "*", 
    RowBox[{
     RowBox[{"lorg", "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{"j", ",", "l"}], "]"}], "\[IndentingNewLine]", "*", 
    RowBox[{"tabarr", "[", 
     RowBox[{"[", 
      RowBox[{"i", ",", "j"}], "]"}], "]"}], "*", 
    RowBox[{"tabarr", "[", 
     RowBox[{"[", 
      RowBox[{"i", ",", "j"}], "]"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"l", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.66426877966558*^9, 3.6643443426515503`*^9, {3.664350014910975*^9, 
   3.664350021767071*^9}, {3.664374213668446*^9, 3.664374219483822*^9}, {
   3.665130492858357*^9, 
   3.665130519817572*^9}},ExpressionUUID->"312f0d9c-4e00-44cb-8c07-\
bac70966eb16"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.664268811459984*^9, 
  3.664268826112651*^9}},ExpressionUUID->"a8b7e6a1-9759-4841-ac28-\
9316b4fae40a"],

Cell[" \[FilledSmallSquare]", "Text",
 CellChangeTimes->{{3.664284962924889*^9, 3.664284979844865*^9}, 
   3.664285030628749*^9, {3.664285067700204*^9, 3.664285209843601*^9}, {
   3.6642854171936407`*^9, 3.664285516760809*^9}, {3.6642856202958508`*^9, 
   3.6642856212915277`*^9}, 
   3.664345297113027*^9},ExpressionUUID->"2f6c07ce-9556-4367-8835-\
260a97f075da"],

Cell[TextData[{
 "In the following we need ",
 StyleBox["multi-indices",
  FontColor->RGBColor[0, 0, 1]],
 ": k-tuples of integers:"
}], "Text",
 CellChangeTimes->{{3.664453937769443*^9, 
  3.6644540006171513`*^9}},ExpressionUUID->"984da056-f391-468b-a48f-\
4800a85cb754"],

Cell[BoxData[
 RowBox[{"?", "tpl"}]], "Input",
 CellChangeTimes->{{3.664454007932858*^9, 
  3.6644540098175917`*^9}},ExpressionUUID->"2e3e7954-9e69-46e1-b329-\
bc52ad906ead"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tpl", "[", 
   RowBox[{"5", ",", "3"}], "]"}], "[", 
  RowBox[{"[", "7", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.664454022464218*^9, 
  3.664454042857353*^9}},ExpressionUUID->"f8c6291c-dd0b-4683-ae6a-\
ad378033baf1"],

Cell["\<\
Here is the definition of a Module  for the calculation of tensor scalar \
products:\
\>", "Text",
 CellChangeTimes->{{3.6646216420102654`*^9, 3.664621665282359*^9}, {
   3.664621714314527*^9, 3.664621745338634*^9}, 
   3.664621782482789*^9},ExpressionUUID->"2c53acae-1e9d-4252-8873-\
344d67397176"],

Cell[BoxData[
 RowBox[{"?", "tensorscpr"}]], "Input",
 CellChangeTimes->{{3.664460752954329*^9, 3.664460759218433*^9}, 
   3.665496171307517*^9},ExpressionUUID->"5cd15086-036a-45b2-8442-\
e975070bbe49"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"tensorscpr", ",", "ta", ",", "tb"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tensorscpr", "[", "g_", "]"}], "[", 
   RowBox[{"ta_", ",", "tb_"}], "]"}], ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "pp", ",", "qq", ",", "nn", ",", "h", ",", "gg", ",", "hh", ",", "mg", 
      ",", "i", ",", "j", ",", "s", ",", "t", ",", "tt", ",", "tta", ",", 
      "ttb", ",", "tfa", ",", "tfb"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Head", "[", "ta", "]"}], "===", "tensor"}], ")"}], "&&", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Head", "[", "tb", "]"}], " ", "===", "tensor"}], ")"}]}], 
        "]"}], ",", " ", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Print", "[", "\"\<Arguments must be tensors!\>\"", "]"}], 
        ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"type", "[", "ta", "]"}], "[", 
           RowBox[{"[", "2", "]"}], "]"}], "===", 
          RowBox[{
           RowBox[{"type", "[", "tb", "]"}], "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"type", "[", "ta", "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}], "===", 
          RowBox[{
           RowBox[{"type", "[", "tb", "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}]}], ")"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"pp", "=", 
         RowBox[{
          RowBox[{"type", "[", "ta", "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ";", 
        RowBox[{"qq", "=", 
         RowBox[{
          RowBox[{"type", "[", "ta", "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "2"}], "]"}], "]"}]}], ";", 
        RowBox[{"nn", "=", 
         RowBox[{
          RowBox[{"type", "[", "ta", "]"}], "[", 
          RowBox[{"[", "3", "]"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
        "Print", "[", "\"\<tensors belong to different spaces!\>\"", "]"}], 
        ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"pp", "\[Equal]", "0"}], ",", 
       RowBox[{
        RowBox[{"tta", " ", "=", " ", "ta"}], ";", 
        RowBox[{"ttb", "=", "tb"}], ";", 
        RowBox[{"Goto", "[", "SUM", "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"mg", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"g", "[", 
         RowBox[{"i", ",", "j"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "nn"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "nn"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"h", "[", 
       RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", 
      RowBox[{
       RowBox[{"Inverse", "[", " ", "mg", "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"hh", "[", 
       RowBox[{"i_List", ",", "j_List"}], "]"}], ":=", 
      RowBox[{"Product", "[", 
       RowBox[{
        RowBox[{"h", "[", 
         RowBox[{
          RowBox[{"i", "[", 
           RowBox[{"[", "s", "]"}], "]"}], ",", 
          RowBox[{"j", "[", 
           RowBox[{"[", "s", "]"}], "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", "pp"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"tfa", "[", 
       RowBox[{
        RowBox[{"{", "}"}], ",", "i_List"}], "]"}], " ", ":=", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"hh", "[", 
          RowBox[{
           RowBox[{"i", "[", 
            RowBox[{"[", 
             RowBox[{"1", ";;", "pp"}], "]"}], "]"}], ",", 
           RowBox[{
            RowBox[{"tpl", "[", 
             RowBox[{"nn", ",", "pp"}], "]"}], "[", 
            RowBox[{"[", "s", "]"}], "]"}]}], "]"}], "*", 
         RowBox[{"ta", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"tpl", "[", 
             RowBox[{"nn", ",", "pp"}], "]"}], "[", 
            RowBox[{"[", "s", "]"}], "]"}], ",", 
           RowBox[{"i", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"pp", "+", "1"}], ";;", 
              RowBox[{"pp", "+", "qq"}]}], "]"}], "]"}]}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", 
          RowBox[{"nn", "^", "pp"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"tfb", "[", 
       RowBox[{
        RowBox[{"{", "}"}], ",", "i_List"}], "]"}], " ", ":=", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"hh", "[", 
          RowBox[{
           RowBox[{"i", "[", 
            RowBox[{"[", 
             RowBox[{"1", ";;", "pp"}], "]"}], "]"}], ",", 
           RowBox[{
            RowBox[{"tpl", "[", 
             RowBox[{"nn", ",", "pp"}], "]"}], "[", 
            RowBox[{"[", "s", "]"}], "]"}]}], "]"}], "*", 
         RowBox[{"tb", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"tpl", "[", 
             RowBox[{"nn", ",", "pp"}], "]"}], "[", 
            RowBox[{"[", "s", "]"}], "]"}], ",", 
           RowBox[{"i", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"pp", "+", "1"}], ";;", 
              RowBox[{"pp", "+", "qq"}]}], "]"}], "]"}]}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", 
          RowBox[{"nn", "^", "pp"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tta", " ", "=", 
      RowBox[{"tensor", "[", 
       RowBox[{"tfa", ",", "0", ",", 
        RowBox[{"pp", "+", "qq"}], ",", "nn"}], "]"}]}], ";", 
     RowBox[{"ttb", " ", "=", " ", 
      RowBox[{"tensor", "[", 
       RowBox[{"tfb", ",", "0", ",", 
        RowBox[{"pp", "+", "qq"}], ",", "nn"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Label", "[", "SUM", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"gg", "[", 
       RowBox[{"i_List", ",", "j_List"}], "]"}], ":=", 
      RowBox[{"Product", "[", 
       RowBox[{
        RowBox[{"g", "[", 
         RowBox[{
          RowBox[{"i", "[", 
           RowBox[{"[", "s", "]"}], "]"}], ",", 
          RowBox[{"j", "[", 
           RowBox[{"[", "s", "]"}], "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", 
          RowBox[{"pp", "+", "qq"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Sum", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"gg", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"tpl", "[", 
            RowBox[{"nn", ",", 
             RowBox[{"pp", "+", "qq"}]}], "]"}], "[", 
           RowBox[{"[", "t", "]"}], "]"}], ",", 
          RowBox[{
           RowBox[{"tpl", "[", 
            RowBox[{"nn", ",", 
             RowBox[{"pp", "+", "qq"}]}], "]"}], "[", 
           RowBox[{"[", "tt", "]"}], "]"}]}], "]"}], "*", 
        RowBox[{"tta", "[", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{
           RowBox[{"tpl", "[", 
            RowBox[{"nn", ",", 
             RowBox[{"pp", "+", "qq"}]}], "]"}], "[", 
           RowBox[{"[", "t", "]"}], "]"}]}], "]"}], "*", 
        RowBox[{"ttb", "[", 
         RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{
           RowBox[{"tpl", "[", 
            RowBox[{"nn", ",", 
             RowBox[{"pp", "+", "qq"}]}], "]"}], "[", 
           RowBox[{"[", "tt", "]"}], "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"t", ",", 
         RowBox[{"nn", "^", 
          RowBox[{"(", 
           RowBox[{"pp", "+", "qq"}], ")"}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"tt", ",", 
         RowBox[{"nn", "^", 
          RowBox[{"(", 
           RowBox[{"pp", "+", "qq"}], ")"}]}]}], "}"}]}], "]"}]}]}], 
   "]"}]}]}], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6642857187007504`*^9, 3.6642857802027073`*^9}, {
   3.664285822891304*^9, 3.664285958199726*^9}, {3.664285991221971*^9, 
   3.664286105670766*^9}, {3.664286159391725*^9, 3.664286162551791*^9}, {
   3.6642862434649343`*^9, 3.6642862651706257`*^9}, {3.664286372994541*^9, 
   3.66428644764565*^9}, {3.664286562844726*^9, 3.6642865823005543`*^9}, {
   3.6642866409408216`*^9, 3.664286889271896*^9}, {3.6642869981218843`*^9, 
   3.6642870084042063`*^9}, {3.664287077122094*^9, 3.664287114722148*^9}, {
   3.664288821702916*^9, 3.664288836549671*^9}, {3.664288868812543*^9, 
   3.664289001432778*^9}, {3.664289036000741*^9, 3.66428909449485*^9}, {
   3.6642891929644403`*^9, 3.664289195404285*^9}, {3.664289228315431*^9, 
   3.6642892351311483`*^9}, {3.664289274769977*^9, 3.664289339368354*^9}, {
   3.664289560498212*^9, 3.664289590305518*^9}, {3.664289706294881*^9, 
   3.664289722316575*^9}, 3.664289772276997*^9, {3.664289810667952*^9, 
   3.664289841491467*^9}, {3.6642898845540743`*^9, 3.66428998650359*^9}, {
   3.664290039462442*^9, 3.664290253729694*^9}, {3.664290290776849*^9, 
   3.664290306535905*^9}, {3.664290342437887*^9, 3.66429035960739*^9}, {
   3.664290411181888*^9, 3.664290416233733*^9}, {3.664290451517291*^9, 
   3.664290625065798*^9}, 3.664290660999975*^9, {3.664290693991951*^9, 
   3.664290900395042*^9}, {3.66429104690648*^9, 3.664291069290247*^9}, {
   3.6642912203605347`*^9, 3.664291232344493*^9}, 3.664291266296445*^9, {
   3.664291336431892*^9, 3.6642913731833076`*^9}, {3.66429168324413*^9, 
   3.664291765660879*^9}, 3.664291860827447*^9, {3.6642919062518387`*^9, 
   3.6642919279980507`*^9}, 3.6642919757871313`*^9, {3.664344004102652*^9, 
   3.664344056399169*^9}, {3.6643440937264338`*^9, 3.6643441002148314`*^9}, 
   3.664346297539352*^9, 3.664346342915722*^9, {3.6643463752482157`*^9, 
   3.664346388577615*^9}, {3.664346441905676*^9, 3.664346456664579*^9}, {
   3.664351053011832*^9, 3.664351061084052*^9}, {3.664351146986307*^9, 
   3.664351376302362*^9}, {3.66435210801007*^9, 3.6643522086384172`*^9}, {
   3.664352240358779*^9, 3.664352268489407*^9}, {3.6643523075609503`*^9, 
   3.664352448311934*^9}, {3.664352502622758*^9, 3.664352571847464*^9}, 
   3.664352611719079*^9, {3.664352687040722*^9, 3.664352704839114*^9}, {
   3.664352739372727*^9, 3.6643527396374083`*^9}, {3.6643527734058723`*^9, 
   3.664352809086413*^9}, {3.664352847964806*^9, 3.664352947332302*^9}, 
   3.664352979187916*^9, {3.664353020620079*^9, 3.6643531037910013`*^9}, {
   3.6643531486509943`*^9, 3.664353236490905*^9}, {3.664353273369936*^9, 
   3.664353341983953*^9}, {3.664353389472208*^9, 3.6643534144482718`*^9}, 
   3.6643666425759487`*^9, {3.6643669695827017`*^9, 3.66436697738337*^9}, {
   3.664367037870905*^9, 3.664367108062895*^9}, {3.664367229317019*^9, 
   3.664367246614602*^9}, 3.6643673309333277`*^9, {3.6643712693552713`*^9, 
   3.664371280162347*^9}, 3.664371630376596*^9, {3.664371669559948*^9, 
   3.664371693751205*^9}, {3.664371786511599*^9, 3.664371819606851*^9}, {
   3.664371875231902*^9, 3.664371886303226*^9}, 3.664371921790793*^9, {
   3.664371990735695*^9, 3.664372014914888*^9}, {3.664372072006484*^9, 
   3.6643720973871183`*^9}, {3.664376377934003*^9, 3.66437638213377*^9}, {
   3.6643765106525106`*^9, 3.664376530219623*^9}, 3.664376573453745*^9, {
   3.664376616644661*^9, 3.664376655083663*^9}, {3.664376695963793*^9, 
   3.6643768010593348`*^9}, 3.66443386319846*^9, 3.664433896831183*^9, {
   3.664433937118705*^9, 3.6644339519745417`*^9}, 3.66443408194219*^9, {
   3.664434211401683*^9, 3.664434216940393*^9}, {3.66443425403578*^9, 
   3.664434256531673*^9}, 3.664434290316369*^9, {3.664434402676312*^9, 
   3.6644344311999073`*^9}, 3.66443447539398*^9, 3.66443450781031*^9, {
   3.6644345564969673`*^9, 3.664434620041284*^9}, {3.664434681487358*^9, 
   3.664434682816444*^9}, {3.6644347134949093`*^9, 3.664434880390831*^9}, 
   3.66443520968645*^9, 3.664435246101514*^9, {3.66443527906068*^9, 
   3.664435332956764*^9}, {3.6644355614868793`*^9, 3.664435567115823*^9}, {
   3.664435598810738*^9, 3.664435605706509*^9}, {3.664435770816984*^9, 
   3.6644357870255127`*^9}, {3.664435920713017*^9, 3.664436002912328*^9}, {
   3.664436092471809*^9, 3.664436144239458*^9}, {3.664436186975739*^9, 
   3.664436202197565*^9}, {3.66443633942279*^9, 3.664436357302771*^9}, {
   3.664436402990617*^9, 3.664436427047884*^9}, {3.6644366589400873`*^9, 
   3.6644366638758183`*^9}, {3.664436704379753*^9, 3.6644367071778727`*^9}, {
   3.664436759203589*^9, 3.66443678192656*^9}, 3.6644368261771393`*^9, {
   3.664437276617015*^9, 3.6644372865238333`*^9}, 3.664437331378154*^9, {
   3.664437389263322*^9, 3.6644374359767523`*^9}, {3.664437659136572*^9, 
   3.664437670695812*^9}, 3.6644383233492107`*^9, {3.664438427221676*^9, 
   3.66443860902631*^9}, {3.664438842546858*^9, 3.6644388655147123`*^9}, {
   3.664438902375016*^9, 3.6644389036238203`*^9}, {3.664439124976075*^9, 
   3.664439202203639*^9}, 3.664439331441472*^9, 3.664439400088485*^9, 
   3.664449116190012*^9, {3.6644491686078167`*^9, 3.664449170877532*^9}, 
   3.6644495432471046`*^9, {3.66444957949553*^9, 3.664449589886722*^9}, 
   3.664449637696501*^9, {3.6644496740379667`*^9, 3.664449692943984*^9}, {
   3.664449725947554*^9, 3.664449727373472*^9}, {3.664449884513694*^9, 
   3.664449901521688*^9}, {3.664449954688121*^9, 3.664450068473919*^9}, {
   3.664450117776703*^9, 3.664450180256349*^9}, 3.66445028516607*^9, {
   3.664450378523424*^9, 3.664450399483307*^9}, {3.664450774107753*^9, 
   3.664450794573082*^9}, {3.66445090574716*^9, 3.664450934977447*^9}, 
   3.664451307566779*^9, 3.6644513790133553`*^9, 3.664451468627101*^9, {
   3.6644515320022297`*^9, 3.664451533922193*^9}, 3.664718185498026*^9, {
   3.665496171327148*^9, 
   3.665496171339924*^9}},ExpressionUUID->"57503aed-2746-4720-9af9-\
c9f30c0f7834"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "lor", "]"}], "[", 
  RowBox[{"tab", ",", "tab"}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664291114031996*^9, 3.664291114823557*^9}, {
   3.664291539822062*^9, 3.664291547676365*^9}, {3.664345463053019*^9, 
   3.664345474444188*^9}, 3.664350628913548*^9, 3.664374312540564*^9, 
   3.66437627829832*^9, 
   3.6654961713527737`*^9},ExpressionUUID->"feb0d7ca-0aa8-4120-b953-\
6b4738f572d1"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6642919961996727`*^9, 
  3.664292004224164*^9}},ExpressionUUID->"93279211-b95a-45cf-b61c-\
c696fd45c80b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "lor", "]"}], "[", 
  RowBox[{"vab", ",", "tab"}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664291114031996*^9, 3.664291114823557*^9}, {
   3.664291539822062*^9, 3.664291547676365*^9}, {3.664345463053019*^9, 
   3.664345474444188*^9}, 3.664350628913548*^9, 3.664350709143223*^9, 
   3.664374335043262*^9, 3.664619610830298*^9, 
   3.665496171366929*^9},ExpressionUUID->"b4c3496a-9c00-4d16-a12e-\
98b5d16f928d"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6642919961996727`*^9, 
  3.664292004224164*^9}},ExpressionUUID->"1a9bf318-63c2-4c50-bfb8-\
a4bcba77dfe9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "lor", "]"}], "[", 
  RowBox[{"vab", ",", "vab"}], "]"}]], "Input",
 CellChangeTimes->{
  3.7059927323798656`*^9},ExpressionUUID->"7679782e-5b02-4d05-811a-\
09a103ab3e03"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6642919961996727`*^9, 
  3.664292004224164*^9}},ExpressionUUID->"dc984ff6-12f3-4d8d-966b-\
1d24b30b7e45"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "lor", "]"}], "[", 
  RowBox[{"vab", ",", "vb"}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664287141690444*^9, 3.664287148897708*^9}, {
   3.665130720689632*^9, 3.665130722302067*^9}, 3.6654961713795853`*^9, 
   3.7059926735096197`*^9, 
   3.705992807204528*^9},ExpressionUUID->"83ea3e46-e65d-4cbe-af27-\
28f3a7372e7e"],

Cell[TextData[{
 StyleBox["Example 3",
  FontColor->RGBColor[0, 0, 1]],
 ". Tensor tc of type (2, 1)"
}], "Text",
 CellChangeTimes->{{3.6646210622474318`*^9, 3.664621096359054*^9}, 
   3.664621149575638*^9},ExpressionUUID->"5d72d28e-0377-4515-9023-\
368339d218bf"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "tc", "]"}], ";", " ", 
  RowBox[{"tc", " ", "=", " ", 
   RowBox[{"tensor", "[", 
    RowBox[{"fc", ",", " ", "2", ",", " ", "1", ",", " ", "4"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.664621136307434*^9, 
  3.664621136308606*^9}},ExpressionUUID->"8d4c122b-751a-48f7-8dee-\
c96d690c0115"],

Cell[BoxData[
 RowBox[{"type", "[", "tc", "]"}]], "Input",
 CellChangeTimes->{{3.664366758070612*^9, 
  3.664366768800971*^9}},ExpressionUUID->"595d1686-1ca1-47f1-900f-\
0a2d91efcf20"],

Cell[BoxData[
 RowBox[{"tc", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.664366777640436*^9, 
  3.6643668141456223`*^9}},ExpressionUUID->"e82cd2ca-9b59-45fa-8b3e-\
e1a6fd85197a"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"tensorscpr", "[", "lor", "]"}], "[", 
   RowBox[{"tc", ",", "tc"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664291114031996*^9, 3.664291114823557*^9}, {
   3.664291539822062*^9, 3.664291547676365*^9}, 3.664350613689481*^9, {
   3.664366527797489*^9, 3.6643665500057173`*^9}, 3.664374436602026*^9, {
   3.664449836660577*^9, 3.664449839374939*^9}, {3.664451568782834*^9, 
   3.6644516029338713`*^9}, 
   3.6654961713918247`*^9},ExpressionUUID->"3afbaf52-cf0d-4fea-b1c6-\
6593f6bfe8a0"],

Cell["\<\
The time needed for this calculation was 27.832 s. Clearly, by the definition \
of the tensor scalar product, it is obvious that the standard tensor basis is \
pseudo-orthonormal. Thus the value is a sum of signed squares of the \
coordinates of tc. The dimension of the tensor space is\
\>", "Text",
 CellChangeTimes->{{3.664462968976166*^9, 3.664462987848106*^9}, {
   3.664463019960071*^9, 3.664463208782646*^9}, {3.664463478932578*^9, 
   3.664463496643825*^9}, 3.665130849605147*^9, {3.705992955244122*^9, 
   3.70599296177836*^9}},ExpressionUUID->"b44fe447-48fb-4184-8c80-\
9a7249de9a84"],

Cell[BoxData[
 RowBox[{"4", "^", "3"}]], "Input",
 CellChangeTimes->{{3.664463505962945*^9, 
  3.664463508636589*^9}},ExpressionUUID->"c9cc5aa4-9174-4a3e-8163-\
0a1567701b20"],

Cell["\<\
Its index is (First load and initialize the notebook index.nb!)\
\>", "Text",
 CellChangeTimes->{{3.6644636286826057`*^9, 3.664463634530876*^9}, {
  3.664463720010043*^9, 3.664463776505273*^9}, {3.705993387927061*^9, 
  3.705993388086862*^9}},ExpressionUUID->"48b16d7e-40ba-470e-9e85-\
b7dae55f8038"],

Cell[BoxData[
 RowBox[{"index", "[", 
  RowBox[{"4", ",", "1", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.664463675279429*^9, 
  3.6644637033153267`*^9}},ExpressionUUID->"b4cc2081-033d-45c4-a6b7-\
dfe8ae8194ca"],

Cell[TextData[{
 "\[FilledSquare]\n",
 StyleBox["Example 4",
  FontColor->RGBColor[0, 0, 1]],
 ". Bivectors"
}], "Text",
 CellChangeTimes->{{3.6644638167365637`*^9, 3.6644638276812*^9}, {
  3.6646213789211082`*^9, 
  3.664621452891972*^9}},ExpressionUUID->"68bbe90c-96d0-4a96-8160-\
f8e9ecc1aa14"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "lor", "]"}], "[", 
  RowBox[{"vva", ",", "vvb"}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664291114031996*^9, 3.664291114823557*^9}, {
   3.664291539822062*^9, 3.664291547676365*^9}, {3.664345463053019*^9, 
   3.664345474444188*^9}, 3.664350628913548*^9, 3.664374312540564*^9, {
   3.66461916154408*^9, 3.6646191724389153`*^9}, 
   3.6654961714069366`*^9},ExpressionUUID->"90e62ce0-e326-4c6a-9816-\
a0af471dddb5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "lor", "]"}], "[", 
  RowBox[{
   RowBox[{"vva", "\[Wedge]", "vvb"}], ",", 
   RowBox[{"vva", "\[Wedge]", "vvb"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664291114031996*^9, 3.664291114823557*^9}, {
   3.664291539822062*^9, 3.664291547676365*^9}, {3.664345463053019*^9, 
   3.664345474444188*^9}, 3.664350628913548*^9, 3.664374312540564*^9, {
   3.66461916154408*^9, 3.6646191724389153`*^9}, {3.664619451302413*^9, 
   3.6646194744137087`*^9}, {3.664620035143021*^9, 3.664620043645952*^9}, 
   3.665496171426375*^9},ExpressionUUID->"7ce8d94f-115c-4b37-9c47-\
00816f74056d"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6642919961996727`*^9, 3.664292004224164*^9}, 
   3.664619436805431*^9, 
   3.664619542510051*^9},ExpressionUUID->"d0e35929-7ed6-469f-9e00-\
22ab8d04f270"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "\[Delta]", "]"}], "[", 
  RowBox[{
   RowBox[{"vva", "\[Wedge]", "vvb"}], ",", 
   RowBox[{"vva", "\[Wedge]", "vvb"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664291114031996*^9, 3.664291114823557*^9}, {
   3.664291539822062*^9, 3.664291547676365*^9}, {3.664345463053019*^9, 
   3.664345474444188*^9}, 3.664350628913548*^9, 3.664374312540564*^9, {
   3.66461916154408*^9, 3.6646191724389153`*^9}, {3.664619451302413*^9, 
   3.6646194744137087`*^9}, {3.664620035143021*^9, 3.664620043645952*^9}, 
   3.6646206800037317`*^9, 
   3.6654961714478083`*^9},ExpressionUUID->"ec6e88db-ca21-44f9-aecd-\
2fb19dc5e13c"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.664630324159132*^9, 
  3.664630335249558*^9}},ExpressionUUID->"2b86c7da-b25d-425b-8c12-\
207135d9df7d"],

Cell["\<\
The tensor scalar square of the bivector vva\[Wedge]vvb is 2 times the square \
of the area of the parallelogram spanned by the vectors vva,vvb, measured in \
the given metric.  For arbitrary metric g we obtain\
\>", "Text",
 CellChangeTimes->{{3.6646303489687033`*^9, 3.664630484143591*^9}, {
  3.664634411891224*^9, 3.664634430214815*^9}, {3.665411857010846*^9, 
  3.665411896786394*^9}, {3.6654119741375847`*^9, 3.6654119762736588`*^9}, {
  3.665412283214699*^9, 
  3.665412333462038*^9}},ExpressionUUID->"bc513a25-f037-4164-99d2-\
909e3809ec29"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", "g", "]"}], "[", 
  RowBox[{
   RowBox[{"vva", "\[Wedge]", "vvb"}], ",", 
   RowBox[{"vva", "\[Wedge]", "vvb"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6642862025431347`*^9, 3.664286219672996*^9}, {
   3.664286481851029*^9, 3.664286486906835*^9}, {3.664287047809309*^9, 
   3.664287048312582*^9}, {3.664291114031996*^9, 3.664291114823557*^9}, {
   3.664291539822062*^9, 3.664291547676365*^9}, {3.664345463053019*^9, 
   3.664345474444188*^9}, 3.664350628913548*^9, 3.664374312540564*^9, {
   3.66461916154408*^9, 3.6646191724389153`*^9}, {3.664619451302413*^9, 
   3.6646194744137087`*^9}, {3.664620035143021*^9, 3.664620043645952*^9}, 
   3.6646206800037317`*^9, {3.665412074961296*^9, 3.665412077704937*^9}, {
   3.665412131089438*^9, 3.66541214474472*^9}, 
   3.66549617146602*^9},ExpressionUUID->"3709a327-4979-4a60-9f8b-\
f82f0c680bea"],

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.665412151628076*^9, 
  3.66541215953564*^9}},ExpressionUUID->"a9542336-b94f-4fa4-acba-\
e1ca858758d3"],

Cell["\[FilledSquare]", "Text",
 CellChangeTimes->{{3.665412243224667*^9, 
  3.665412243226886*^9}},ExpressionUUID->"f6792673-06bc-4a81-b4c9-\
b41fb60c3c35"],

Cell[TextData[{
 StyleBox["Example 5",
  FontColor->RGBColor[0, 0, 1]],
 ". Tensor fields\nUp to now we considered tensors over a vector space with \
scalar product only. The Module ",
 "tensorscpr",
 " can be applied to tensor fields too; it operates pointwise."
}], "Text",
 CellChangeTimes->{{3.664719218531402*^9, 3.664719259356824*^9}, {
   3.6647192963656073`*^9, 3.664719309732173*^9}, {3.664719522034459*^9, 
   3.664719656881621*^9}, {3.6651309520352097`*^9, 3.665130954202839*^9}, 
   3.665133030801165*^9, 
   3.665496171495409*^9},ExpressionUUID->"9622ca59-aa05-4d4a-9094-\
ef3fec5fa98e"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "va", ",", "vb", ",", "vva", ",", "vvb", ",", "vab", ",", "tab", ",", 
   "tc"}], "]"}]], "Input",
 CellChangeTimes->{{3.664719669998083*^9, 3.6647196745095863`*^9}, {
   3.664719754914681*^9, 3.6647197830966578`*^9}, {3.6647198164799633`*^9, 
   3.664719827576201*^9}, 
   3.665133046593647*^9},ExpressionUUID->"ecbbc950-2439-427a-8da3-\
e9d1baf45a1a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "3"}], ";", " ", 
  RowBox[{"ind", " ", "=", "1"}], ";"}]], "Input",
 CellChangeTimes->{{3.6647198637759113`*^9, 3.66471987337535*^9}, 
   3.664720084838005*^9, 
   3.6647987495803328`*^9},ExpressionUUID->"9a733735-aaef-4b10-a1fe-\
b3dee41ca4f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vv", "[", "a", "]"}], "[", 
  RowBox[{"var", "[", "x", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.664635171452095*^9, 3.664635184437282*^9}, {
  3.66463521904558*^9, 
  3.664635233037586*^9}},ExpressionUUID->"7f5b6e4f-f5ce-479d-b63e-\
fa3b21d18fb0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vX", " ", ":=", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}], ";", 
  RowBox[{"vY", ":=", 
   RowBox[{"vecTotensor", "[", 
    RowBox[{
     RowBox[{"vv", "[", "\[Eta]", "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}]}], ";", "vX"}]], "Input",
 CellChangeTimes->{{3.664635557957609*^9, 3.664635703168705*^9}, {
   3.66463580685576*^9, 3.664635862559226*^9}, {3.664718326693881*^9, 
   3.6647183338859243`*^9}, 3.664718389117535*^9, {3.664718878337607*^9, 
   3.6647189139209633`*^9}},ExpressionUUID->"8a87f69a-7c12-4d56-be75-\
0c306e100db8"],

Cell["\<\
Before applying tensorscpr it is necessary to define the Riemannian metric \
explicitly, e.g. for a general, unspecified  pseudo-Riemannian metric:\
\>", "Text",
 CellChangeTimes->{{3.664799320057662*^9, 3.664799438009478*^9}, 
   3.665496171600646*^9, {3.6654967339620447`*^9, 
   3.6654967370172997`*^9}},ExpressionUUID->"148002a2-83ff-401b-8c25-\
4a69c05c557c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h", "[", "x__", "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"RG", "[", "]"}], "[", "x", "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"i", ",", "j"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.66472025104161*^9, 3.6647202904527073`*^9}, {
  3.664720444762773*^9, 3.664720501090355*^9}, {3.664808049016542*^9, 
  3.6648080995196*^9}, {3.706060079878297*^9, 3.706060087149538*^9}, {
  3.706060154629964*^9, 
  3.706060155356668*^9}},ExpressionUUID->"fb2f5f72-58b6-4c92-aa7e-\
b36afa85665a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", 
   RowBox[{"h", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "[", 
  RowBox[{"vX", ",", "vY"}], "]"}]], "Input",
 CellChangeTimes->{{3.6647205435575647`*^9, 3.664720548641712*^9}, {
   3.664720815695772*^9, 3.664720844527608*^9}, {3.664720910661788*^9, 
   3.664720938334681*^9}, {3.664808119807252*^9, 3.6648081436071033`*^9}, 
   3.665496171678445*^9, {3.66549683908862*^9, 
   3.665496843063599*^9}},ExpressionUUID->"e7bdc993-e980-4c67-b39f-\
ed7e734d7cda"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", 
   RowBox[{"h", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"vX", "\[CircleTimes]", "vY"}], ",", 
   RowBox[{"vX", "\[CircleTimes]", "vY"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6647205435575647`*^9, 3.664720548641712*^9}, {
   3.664720815695772*^9, 3.664720844527608*^9}, {3.664720910661788*^9, 
   3.664720938334681*^9}, {3.664721027529348*^9, 3.664721094653935*^9}, {
   3.664808327173751*^9, 3.6648083604056253`*^9}, 3.665496171694223*^9, {
   3.6654968735037193`*^9, 
   3.665496883171838*^9}},ExpressionUUID->"ba75894b-46b2-44e8-b6a0-\
c4740d436368"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6647211661779137`*^9, 
  3.664721178628456*^9}},ExpressionUUID->"7ebe3030-67f0-470e-864a-\
5a516f9559d1"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "\[Equal]", 
   RowBox[{
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"h", "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
        RowBox[{"i", ",", "j"}], "]"}], "*", 
       RowBox[{
        RowBox[{"\[Xi]", "[", "i", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
       RowBox[{
        RowBox[{"\[Xi]", "[", "j", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "dim"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "*", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"h", "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
        RowBox[{"i", ",", "j"}], "]"}], "*", 
       RowBox[{
        RowBox[{"\[Eta]", "[", "i", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], "*", 
       RowBox[{
        RowBox[{"\[Eta]", "[", "j", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "dim"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "dim"}], "}"}]}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6647218755071898`*^9, 3.6647219224221487`*^9}, {
  3.664721959366671*^9, 3.664722079949418*^9}, {3.664722148293796*^9, 
  3.6647222380360823`*^9}, {3.664722319020029*^9, 3.6647223241241503`*^9}, {
  3.664808443469391*^9, 
  3.664808455692566*^9}},ExpressionUUID->"70a84bc4-327f-4f7b-b17f-\
e1eb60e80753"],

Cell[BoxData[
 RowBox[{"?", "lorg"}]], "Input",
 CellChangeTimes->{{3.664808627683503*^9, 3.664808630890998*^9}, {
  3.669903451742594*^9, 
  3.669903463725997*^9}},ExpressionUUID->"f6d6c5dc-143b-40b5-99da-\
e3bc70368fdb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tensorscpr", "[", 
   RowBox[{"lorg", "[", 
    RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{"vX", "\[CircleTimes]", "vY"}], ",", 
   RowBox[{"vX", "\[CircleTimes]", "vY"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6647205435575647`*^9, 3.664720548641712*^9}, {
   3.664720815695772*^9, 3.664720844527608*^9}, {3.664720910661788*^9, 
   3.664720938334681*^9}, {3.664721027529348*^9, 3.664721094653935*^9}, {
   3.664721478458296*^9, 3.664721479161841*^9}, 3.6651332555990458`*^9, {
   3.665133362366845*^9, 3.6651333690705547`*^9}, 3.665133417086927*^9, {
   3.66513359878119*^9, 3.665133606668355*^9}, 3.665496171707676*^9, {
   3.6654969429105186`*^9, 
   3.665496953597707*^9}},ExpressionUUID->"e3a70614-d9f8-40ed-bd5a-\
0aafbd465e87"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.6647214358357773`*^9, 3.6647214494020348`*^9}, 
   3.664721507026676*^9},ExpressionUUID->"b7b8e66f-b690-4ea8-8b5e-\
39ad9bce2f14"],

Cell["In dimension 2 we obtain", "Text",
 CellChangeTimes->{{3.664799215778057*^9, 
  3.664799227129758*^9}},ExpressionUUID->"efd50e89-8a1d-4502-b0a8-\
fdb340728f59"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{
   RowBox[{"tensorscpr", "[", 
    RowBox[{"lorg", "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "[", 
   RowBox[{
    RowBox[{"vX", "\[Wedge]", "vY"}], ",", 
    RowBox[{"vX", "\[Wedge]", "vY"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6647205435575647`*^9, 3.664720548641712*^9}, {
   3.664720815695772*^9, 3.664720844527608*^9}, {3.664720910661788*^9, 
   3.664720938334681*^9}, {3.664721027529348*^9, 3.664721094653935*^9}, {
   3.664721478458296*^9, 3.664721479161841*^9}, {3.664722467610059*^9, 
   3.664722474990055*^9}, {3.664799086794977*^9, 3.664799091723132*^9}, 
   3.664799152555653*^9, {3.665133849130372*^9, 3.6651338586573687`*^9}, 
   3.665133897009679*^9, 3.665496171720689*^9, {3.665497019229848*^9, 
   3.6654970300933247`*^9}},ExpressionUUID->"ad0088be-609c-41db-a1f4-\
69bb66cf8934"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", "-", 
   RowBox[{"2", 
    RowBox[{"Det", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"lorg", "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
        RowBox[{"i", ",", "j"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "dim"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "dim"}], "}"}]}], "]"}], "]"}], "*", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Det", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"vv", "[", "\[Xi]", "]"}], "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"vv", "[", "\[Eta]", "]"}], "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}]}], "}"}], "]"}], ")"}], "^", 
     "2"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6647226978236732`*^9, 3.664722718991556*^9}, {
   3.6647227507124653`*^9, 3.664722811534919*^9}, {3.6647228550869703`*^9, 
   3.664722884958712*^9}, {3.664722920256048*^9, 3.664722944654978*^9}, 
   3.6647230041432657`*^9, 3.664723039296462*^9, 3.664723113384574*^9, {
   3.664723260607769*^9, 3.664723275615251*^9}, 3.664723348136294*^9, 
   3.664809140095459*^9, {3.665133690740251*^9, 3.665133695619911*^9}, 
   3.6651337295245256`*^9, {3.665133769212349*^9, 3.665133785762739*^9}, 
   3.665133955056241*^9, {3.665133991423675*^9, 3.665134014807829*^9}, 
   3.6651343013241463`*^9, 3.665134339019105*^9, {3.6654971803660383`*^9, 
   3.66549720210324*^9}, {3.669112264587098*^9, 3.669112264588323*^9}, {
   3.66990351511874*^9, 
   3.66990352368576*^9}},ExpressionUUID->"2837d718-cf78-4301-b231-\
7ed929b0e876"],

Cell["\<\
Often it is convenient to use the following Module for a calculation of  the \
scalar product of tensors:\
\>", "Text",
 CellChangeTimes->{{3.665497450249531*^9, 
  3.6654975256968613`*^9}},ExpressionUUID->"016e2faf-8fa7-4160-b204-\
258362fce79c"],

Cell[BoxData[
 RowBox[{"?", "tensorfieldscpr"}]], "Input",
 CellChangeTimes->{{3.665497883531571*^9, 
  3.6654978933391933`*^9}},ExpressionUUID->"5efe970a-92ee-44ca-90cc-\
5d7596c5a3dc"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"tensorfieldscpr", "[", 
     RowBox[{"gg_:", 
      RowBox[{"rg", "[", "]"}]}], "]"}], "[", 
    RowBox[{"ta_", ",", "tb_"}], "]"}], "[", "x__", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"gh", ",", "i", ",", "j"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"gh", "[", 
       RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
      RowBox[{
       RowBox[{"gg", "[", "x", "]"}], "[", 
       RowBox[{"i", ",", "j"}], "]"}]}], ";", 
     RowBox[{
      RowBox[{"tensorscpr", "[", "gh", "]"}], "[", 
      RowBox[{"ta", ",", "tb"}], "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.665210145310944*^9, 3.66521029299897*^9}, {
   3.665210338686432*^9, 3.665210368414509*^9}, {3.665210421957889*^9, 
   3.665210536077223*^9}, 3.665212539910582*^9, 3.665496171564361*^9, 
   3.66549636398952*^9},ExpressionUUID->"ad8743fb-87f3-4bb8-b2ec-\
32aacc7f9d7e"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rg", "[", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.665241072001178*^9, 3.665241114563546*^9}, {
  3.6652411484043827`*^9, 
  3.665241150259371*^9}},ExpressionUUID->"d0c01528-e73f-4b86-a48f-\
0a7062d213cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "2"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"tensorfieldscpr", "[", "]"}], "[", 
      RowBox[{
       RowBox[{"vX", "\[Wedge]", "vY"}], ",", 
       RowBox[{"vX", "\[Wedge]", "vY"}]}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6652142768506527`*^9, 3.665214395949971*^9}, {
   3.66521459060424*^9, 3.66521459469219*^9}, 3.665496365845537*^9, {
   3.665497552087908*^9, 
   3.665497567598915*^9}},ExpressionUUID->"c6eaf78a-d1cd-475f-a743-\
21b0b6569795"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", " ", "=", " ", "4"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"tensorfieldscpr", "[", "lorg", "]"}], "[", 
      RowBox[{
       RowBox[{"vX", "\[Wedge]", "vY"}], ",", 
       RowBox[{"vX", "\[Wedge]", "vY"}]}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6652142768506527`*^9, 3.665214395949971*^9}, {
   3.66521459060424*^9, 3.66521459469219*^9}, 3.665496365845537*^9, {
   3.665497552087908*^9, 3.665497567598915*^9}, {3.6654976175003567`*^9, 
   3.665497628102563*^9}, {3.665497722524987*^9, 
   3.665497723845159*^9}},ExpressionUUID->"44ae17bf-4ac5-4899-9f40-\
5b7658f71afe"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"vX", ",", "vY"}], "]"}], ";", " ", 
  RowBox[{"dim", "=", " ", "3"}], ";"}]], "Input",
 CellChangeTimes->{{3.6647995742240677`*^9, 3.664799621656623*^9}, {
  3.665497230186846*^9, 
  3.665497240891055*^9}},ExpressionUUID->"bacd62d8-8939-46f1-8ef6-\
b3d4f6dafaee"]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2.5. Sectional Curvature", "Subsubsection",
 CellChangeTimes->{{3.663829235269321*^9, 3.6638292680206823`*^9}, {
   3.6646344864669743`*^9, 3.664634507394854*^9}, {3.6651344877362623`*^9, 
   3.665134490464149*^9}, 3.665495615890316*^9, 3.669903636180914*^9, 
   3.670000951960124*^9, 3.706060543026783*^9, 
   3.7060991720631*^9},ExpressionUUID->"352a209a-1d66-4f4f-99c6-9684ae0182c9"],

Cell[TextData[{
 "The sectional curvature evaluates the curvature tensor of a Riemannian \
manifold in dependence on the 2-dimensional tangent subspaces. For surfaces \
of the Euclidean space the tangent spaces are 2-dimensional, and the only \
sectional curvature coincides with the Gaussian curvature, see Example 5 \
below.  A geometric definition and interpretation of the sectional curvature \
can be found e.g. in S. Helgason [",
 ButtonBox["H78",
  BaseStyle->"Hyperlink",
  ButtonData->"H78"],
 "], \[Section]I.12. The definition of the sectional curvature  in tensor \
terms is the complete contraction "
}], "Text",
 CellChangeTimes->{{3.6653983731223173`*^9, 3.665398415138217*^9}, {
   3.665398449730289*^9, 3.6653984885380573`*^9}, {3.665398579401465*^9, 
   3.66539866732053*^9}, {3.665398709127536*^9, 3.6653988097658978`*^9}, {
   3.665398850085065*^9, 3.6653988702449007`*^9}, {3.6653989250997467`*^9, 
   3.665398985378665*^9}, {3.6653990563773317`*^9, 3.665399074368919*^9}, {
   3.6653991792313433`*^9, 3.665399195166912*^9}, {3.6653992377043943`*^9, 
   3.6653992377057467`*^9}, 3.6653993333885117`*^9, {3.6653994750980883`*^9, 
   3.6653995434564457`*^9}, {3.665399587880073*^9, 3.665399614376318*^9}, {
   3.665400237349147*^9, 3.665400297908474*^9}, {3.665400338659686*^9, 
   3.6654003601314793`*^9}, {3.665400419026494*^9, 3.665400444875409*^9}, 
   3.6654126902754602`*^9, 3.670650010201048*^9, 
   3.706073264782029*^9},ExpressionUUID->"64f33308-eac1-4816-b26e-\
c524bcd248d6"],

Cell[BoxData[
 RowBox[{"2", "*", 
  RowBox[{
   RowBox[{"contraction", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"covRCtensor", "[", "g", "]"}], "[", "x", "]"}], 
      "\[CircleTimes]", "X", "\[CircleTimes]", "Y", "\[CircleTimes]", "X", 
      "\[CircleTimes]", "Y"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "4"}], "}"}]}], "}"}]}], "]"}], "/", 
   RowBox[{
    RowBox[{"tensorscpr", "[", "g", "]"}], "[", 
    RowBox[{
     RowBox[{"X", "\[Wedge]", "Y"}], ",", 
     RowBox[{"X", "\[Wedge]", "Y"}]}], "]"}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.6654004674606647`*^9, 3.665400767628237*^9}, {
   3.6654112720411158`*^9, 3.665411435511878*^9}, 
   3.6654961717359877`*^9},ExpressionUUID->"33b77a6e-e711-4ca1-a8fe-\
1a820e7506ac"],

Cell["\<\
divided by the square of the area of the parallelogram spanned by the vectors \
X,Y, see Example 4 in the last Subsection 4.2.4. The numerator is a tensor of \
type (0,0), whose only coordinate normed by the area square ensures that the \
sectional curvature depends only on the subspace of the tangent space and not \
on the special elements X, Y spanning it.. Unfortunately, the evaluation of \
this expression even in dimension 3 needs so much time that it becomes almost \
unusable. The following Module calculates the sectional curvature of a \
tangent plane at the point x spanned by the linearly independent tangent \
vectors X,Y; it is about 50 times faster.\
\>", "Text",
 CellChangeTimes->{{3.6653983731223173`*^9, 3.665398415138217*^9}, {
   3.665398449730289*^9, 3.6653984885380573`*^9}, {3.665398579401465*^9, 
   3.66539866732053*^9}, {3.665398709127536*^9, 3.6653988097658978`*^9}, {
   3.665398850085065*^9, 3.6653988702449007`*^9}, {3.6653989250997467`*^9, 
   3.665398985378665*^9}, {3.6653990563773317`*^9, 3.665399074368919*^9}, {
   3.6653991792313433`*^9, 3.665399195166912*^9}, {3.6653992377043943`*^9, 
   3.6653992377057467`*^9}, 3.6653993333885117`*^9, {3.6653994750980883`*^9, 
   3.6653995434564457`*^9}, {3.665399587880073*^9, 3.665399614376318*^9}, {
   3.665400237349147*^9, 3.665400297908474*^9}, {3.665400338659686*^9, 
   3.6654003601314793`*^9}, {3.665400419026494*^9, 3.665400444875409*^9}, {
   3.6654008155788*^9, 3.6654008742582073`*^9}, {3.66540932179987*^9, 
   3.6654093496954317`*^9}, {3.6654094346786213`*^9, 3.665409496501814*^9}, {
   3.6654095647497187`*^9, 3.6654095812770033`*^9}, {3.665409627324679*^9, 
   3.6654096429165077`*^9}, 3.665409681140315*^9, {3.665412468109353*^9, 
   3.66541254250902*^9}, {3.6654125809887857`*^9, 3.665412653123926*^9}, {
   3.665412700292756*^9, 3.665412717842948*^9}, {3.6654127857464447`*^9, 
   3.6654127903386517`*^9}, {3.706068641806179*^9, 3.706068661637467*^9}, {
   3.706068693732292*^9, 3.7060687144606333`*^9}, {3.706068753825181*^9, 
   3.7060687756365557`*^9}, {3.7060724493927183`*^9, 3.706072568543836*^9}, {
   3.706072649848167*^9, 
   3.706072728791987*^9}},ExpressionUUID->"95c98e83-08fb-449e-998a-\
700dcae10705"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"sectionalcurvature", "[", 
     RowBox[{"gg_:", 
      RowBox[{"rg", "[", "]"}]}], "]"}], "[", "x__", "]"}], "[", 
   RowBox[{"X_", ",", " ", "Y_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"di", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"{", "x", "}"}], "]"}]}], ",", "tX", ",", "tY", ",", "gh", 
      ",", "crt", ",", "nn", ",", "zz", ",", "i", ",", "j", ",", "k", ",", 
      "l"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Head", "[", "X", "]"}], "===", " ", "List"}], ")"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", "X", "]"}], "\[Equal]", "di"}], ")"}]}], ",",
        " ", 
       RowBox[{"tX", "=", 
        RowBox[{"vecTotensor", "[", "X", "]"}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Invalid argument X = \>\"", ",", "X"}], "]"}], ";", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Head", "[", "Y", "]"}], "===", " ", "List"}], ")"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", "Y", "]"}], "\[Equal]", "di"}], ")"}]}], ",",
        " ", 
       RowBox[{"tY", "=", 
        RowBox[{"vecTotensor", "[", "Y", "]"}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Invalid argument Y = \>\"", ",", "Y"}], "]"}], ";", " ", 
        RowBox[{"Return", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"gh", "[", 
       RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
      RowBox[{
       RowBox[{"gg", "[", "x", "]"}], "[", 
       RowBox[{"i", ",", "j"}], "]"}]}], ";", 
     RowBox[{"nn", "=", 
      RowBox[{
       RowBox[{"tensorscpr", "[", "gh", "]"}], "[", 
       RowBox[{
        RowBox[{"tX", "\[Wedge]", "tY"}], ",", 
        RowBox[{"tX", "\[Wedge]", "tY"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"crt", " ", "=", 
      RowBox[{
       RowBox[{"covRCtensor", "[", "gg", "]"}], "[", "x", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"zz", "=", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"crt", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"i", ",", "j", ",", "k", ",", "l"}], "}"}], ",", 
           RowBox[{"{", "}"}]}], "]"}], "*", 
         RowBox[{"X", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "*", 
         RowBox[{"Y", "[", 
          RowBox[{"[", "j", "]"}], "]"}], "*", 
         RowBox[{"X", "[", 
          RowBox[{"[", "k", "]"}], "]"}], "*", 
         RowBox[{"Y", "[", 
          RowBox[{"[", "l", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "di"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "di"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "di"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"l", ",", "di"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"2", "*", 
      RowBox[{"zz", "/", "nn"}]}]}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.66523977201795*^9, 3.66523977861242*^9}, {
   3.6652398927240562`*^9, 3.665239958603343*^9}, {3.6652401912589912`*^9, 
   3.665240256858665*^9}, {3.6652403008176937`*^9, 3.665240917285803*^9}, {
   3.665240965510687*^9, 3.665241012749486*^9}, {3.665241252050742*^9, 
   3.665241376602439*^9}, {3.665241437209405*^9, 3.6652414526808443`*^9}, {
   3.6652414828581448`*^9, 3.665241635240377*^9}, {3.6652416747514*^9, 
   3.665241733831397*^9}, {3.665241862654479*^9, 3.665241887526432*^9}, {
   3.665242292346745*^9, 3.665242314178728*^9}, {3.665293556329933*^9, 
   3.665293560657115*^9}, {3.665293608744343*^9, 3.665293613392763*^9}, {
   3.6652936670172873`*^9, 3.6652937169445972`*^9}, {3.665294480665073*^9, 
   3.6652944821614943`*^9}, {3.665318091094858*^9, 3.6653181072239733`*^9}, {
   3.665320629257977*^9, 3.665320630226667*^9}, 3.665320836118442*^9, {
   3.665320875901802*^9, 3.66532102195677*^9}, {3.665326100197081*^9, 
   3.6653261272680073`*^9}, {3.665383983343977*^9, 3.665383986081149*^9}, 
   3.665384021544218*^9, {3.665384056903311*^9, 3.665384059902906*^9}, {
   3.6653849582556953`*^9, 3.665384965015607*^9}, 
   3.665496171809636*^9},ExpressionUUID->"0fc27a7a-9055-4fb1-a949-\
b3fc3cf94980"],

Cell[BoxData[
 RowBox[{"?", "sectionalcurvature"}]], "Input",
 CellChangeTimes->{{3.665384472993779*^9, 3.665384481361849*^9}, {
   3.66538455433068*^9, 3.665384788415217*^9}, {3.665384861262279*^9, 
   3.6653849009579906`*^9}, 3.6653850271893387`*^9, {3.665400147486251*^9, 
   3.665400155469883*^9}},ExpressionUUID->"cbc5fa72-78f2-4336-ba4d-\
d59abf5de9db"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sectionalcurvature", "[", "]"}], "[", 
     RowBox[{"x", ",", "y", ",", "z"}], "]"}], "[", 
    RowBox[{
     RowBox[{"vec", "[", "a", "]"}], ",", 
     RowBox[{"vec", "[", "b", "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.665241935893012*^9, 3.665242038643977*^9}, {
   3.665321073827632*^9, 3.665321074627421*^9}, 3.66538418859518*^9, 
   3.665387644178988*^9, {3.665387730134879*^9, 3.6653877318303328`*^9}, {
   3.669958995976655*^9, 3.6699590039221315`*^9}, {3.6699590363516316`*^9, 
   3.669959040724484*^9}, 
   3.669983016593491*^9},ExpressionUUID->"9cb6fd3d-4b87-4d25-8f24-\
14ba9e44db2d"],

Cell["\<\
In  60.088 s. we got a \[OpenCurlyDoubleQuote]large output\
\[CloseCurlyDoubleQuote].Simplifying this expression needs much time and \
brings little insight.\
\>", "Text",
 CellChangeTimes->{{3.665388389552717*^9, 3.665388433433831*^9}, {
   3.665388474754846*^9, 3.665388514091673*^9}, 3.665397593639365*^9, {
   3.665397681110928*^9, 3.665397704959001*^9}, {3.6653977524464912`*^9, 
   3.6653977669665203`*^9}, {3.6654098933687143`*^9, 3.6654098976965723`*^9}, 
   3.669959672211987*^9, {3.6699830667596397`*^9, 3.6699831314317627`*^9}, 
   3.6699831715359592`*^9, {3.7060729287489157`*^9, 3.7060729294435863`*^9}, 
   3.70856831775307*^9},ExpressionUUID->"75b42b60-02d5-4de3-ae16-\
95283da0f982"],

Cell[TextData[{
 StyleBox["Example 1",
  FontColor->RGBColor[0, 0, 1]],
 ". The sphere of radius r."
}], "Text",
 CellChangeTimes->{{3.6652201704460278`*^9, 
  3.665220206139907*^9}},ExpressionUUID->"e1b15781-5598-4963-9f8d-\
1ea9644d0432"],

Cell[BoxData[
 RowBox[{
  RowBox[{"spher", "[", "r", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.66480536159304*^9, 
  3.664805386481803*^9}},ExpressionUUID->"7ff73e06-22d1-4363-b25b-\
303ed1217404"],

Cell[BoxData[
 RowBox[{"?", "RGT"}]], "Input",
 CellChangeTimes->{{3.6648055051657057`*^9, 
  3.664805508463529*^9}},ExpressionUUID->"3c534d59-517f-4fd7-8612-\
6f8b5f486ba8"],

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"RGT", "[", 
     RowBox[{"spher", "[", "r", "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.664714472709392*^9, 3.664714519927878*^9}, {
  3.664805400864204*^9, 3.6648054434637957`*^9}, {3.664805588135786*^9, 
  3.664805619703302*^9}, {3.6648059095169983`*^9, 3.66480591350816*^9}, {
  3.6648062400250607`*^9, 3.664806275280105*^9}, {3.665154225498925*^9, 
  3.665154228066105*^9}},ExpressionUUID->"620b81af-c254-4bfd-9d69-\
cbc7519750c1"],

Cell["\<\
Sometimes it is necessary or at least useful to define the Riemannian metric \
explicitly. The  correct format is\
\>", "Text",
 CellChangeTimes->{{3.6654100552785063`*^9, 3.6654101080061407`*^9}, {
  3.6654102010765963`*^9, 3.66541022020479*^9}, {3.6654102566205072`*^9, 
  3.6654102663560247`*^9}},ExpressionUUID->"6c25b7b1-6fbe-4298-a499-\
d2fcee936228"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "h", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"h", "[", "r_", "]"}], "[", "x__", "]"}], "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"RGT", "[", 
        RowBox[{"spher", "[", "r", "]"}], "]"}], "[", "x", "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "j"}], "]"}], "]"}], ",", 
     RowBox[{"r", ">", "0"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.664714571224259*^9, 3.6647146335277643`*^9}, {
   3.664805945899349*^9, 3.664806032178396*^9}, 3.66480616242584*^9, 
   3.664806346096407*^9, {3.665153652621505*^9, 3.665153674557283*^9}, {
   3.665153723519299*^9, 3.665153732919465*^9}, {3.665154374113172*^9, 
   3.665154397367853*^9}, {3.665155101010906*^9, 
   3.6651551065471973`*^9}},ExpressionUUID->"103fe61c-3bfa-4fd4-bd17-\
e9d2571f5051"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"h", "[", "r", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.665154602563137*^9, 3.665154649122306*^9}, {
  3.665213091817803*^9, 
  3.665213097065044*^9}},ExpressionUUID->"d5eae301-428a-4d22-b05a-\
fccb52f14024"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", " ", 
  RowBox[{
   RowBox[{"vv", "[", "a", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6652941523815117`*^9, 3.6652941815304527`*^9}, {
  3.665410356971439*^9, 
  3.665410361115563*^9}},ExpressionUUID->"ded6e5e0-89cc-42cf-a0a2-\
3ee0aa293b2f"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sectionalcurvature", "[", 
      RowBox[{"h", "[", "r", "]"}], "]"}], "[", 
     RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"vv", "[", "a", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"vv", "[", "b", "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6652096375705214`*^9, 3.665209778371078*^9}, {
   3.665214715947653*^9, 3.665214728995122*^9}, {3.6652940563155518`*^9, 
   3.665294066443132*^9}, {3.665294133635438*^9, 3.665294143235124*^9}, {
   3.6652942796349916`*^9, 3.6652942958091097`*^9}, {3.6652943581290703`*^9, 
   3.66529436901789*^9}, {3.6653214189979477`*^9, 3.6653214200066338`*^9}, 
   3.6653841886834497`*^9, {3.665394844443334*^9, 3.66539485177134*^9}, 
   3.706073033275585*^9},ExpressionUUID->"a7bd82b0-24ab-4d00-8508-\
5fe62acf5a87"],

Cell["\<\
The time needed: 42.496 s. The result is the Gaussian curvature of the \
sphere. \[FilledSquare]\
\>", "Text",
 CellChangeTimes->{{3.66521729194851*^9, 3.665217325548472*^9}, {
   3.665219969670827*^9, 3.665220021301435*^9}, {3.665220272011508*^9, 
   3.665220279805127*^9}, 3.665294683574709*^9, 3.6653893609494257`*^9, 
   3.665395060721374*^9, 3.66998341714294*^9, 
   3.706073264341761*^9},ExpressionUUID->"b8b583c6-3b68-4ccc-9588-\
9bc5dec54137"],

Cell["\<\
The next two examples show that the sectional curvature for spaces of \
constant curvature is really constant.\
\>", "Text",
 CellChangeTimes->{{3.670649478299395*^9, 3.6706495496830187`*^9}, {
  3.706073338440715*^9, 
  3.706073342280579*^9}},ExpressionUUID->"a8b44049-d527-47f7-8f54-\
e5496aae908f"],

Cell[TextData[{
 StyleBox["Example 2",
  FontColor->RGBColor[0, 0, 1]],
 ". Spaces of constant positive curvature"
}], "Text",
 CellChangeTimes->{{3.6652202883633223`*^9, 
  3.6652203178749228`*^9}},ExpressionUUID->"173478ab-16e8-49bd-9a31-\
80d800d9f965"],

Cell[BoxData[
 RowBox[{"?", "rgposk"}]], "Input",
 CellChangeTimes->{{3.665220909104726*^9, 
  3.665220912431101*^9}},ExpressionUUID->"54816db4-fe88-4c43-9d19-\
2046963ba708"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", " ", 
  RowBox[{
   RowBox[{"vv", "[", "a", "]"}], "[", 
   RowBox[{"var", "[", "x", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.665221007414852*^9, 3.6652210080305567`*^9}, 
   3.665294894253153*^9},ExpressionUUID->"62336ec0-ec3e-458f-8b77-\
c065d63e591c"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"sectionalcurvature", "[", 
       RowBox[{"rgposk", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"vv", "[", "a", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"vv", "[", "b", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], ",", 
    RowBox[{"r", ">", "0"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.665220922488048*^9, 3.665220999327064*^9}, {
   3.665221036934848*^9, 3.665221116254575*^9}, {3.665294793093227*^9, 
   3.6652948193173943`*^9}, 3.665294872117437*^9, {3.6653216495751467`*^9, 
   3.665321650487421*^9}, 3.665384188686241*^9, {3.669983501798814*^9, 
   3.669983507358955*^9}},ExpressionUUID->"b74799c8-8637-44e9-927f-\
6e82959d56be"],

Cell["The time needed: 0. s. The result is the constant 1/r^2.", "Text",
 CellChangeTimes->{{3.66521729194851*^9, 3.665217325548472*^9}, {
   3.665219969670827*^9, 3.665220021301435*^9}, {3.665220272011508*^9, 
   3.665220279805127*^9}, {3.665223972896968*^9, 3.665224018405445*^9}, {
   3.6652949726917067`*^9, 3.6652949785071287`*^9}, {3.6653216305422373`*^9, 
   3.665321633886435*^9}, {3.665395526812117*^9, 3.665395548244114*^9}, 
   3.665395651011058*^9, {3.669983586526383*^9, 3.6699835933575068`*^9}, {
   3.706073390304367*^9, 
   3.706073399736013*^9}},ExpressionUUID->"afc044c8-0244-4430-b085-\
e616b47dd12b"],

Cell[TextData[{
 StyleBox["Example 3",
  FontColor->RGBColor[0, 0, 1]],
 ". Spaces of constant negative curvature"
}], "Text",
 CellChangeTimes->{{3.6652202883633223`*^9, 3.6652203178749228`*^9}, {
   3.6652240300216312`*^9, 3.665224042060905*^9}, 3.6654129485363693`*^9, 
   3.665468391560659*^9},ExpressionUUID->"e7b9241a-ce4a-4469-85f6-\
23a2dc66060c"],

Cell[BoxData[
 RowBox[{"?", "rghypcnf"}]], "Input",
 CellChangeTimes->{{3.665224281266224*^9, 
  3.665224284202715*^9}},ExpressionUUID->"e472c92b-424b-4581-aeb9-\
64fe48dddba3"],

Cell[BoxData[
 RowBox[{"?", "RGhypcnf"}]], "Input",
 CellChangeTimes->{{3.665224281266224*^9, 3.665224284202715*^9}, {
  3.6652243793300333`*^9, 
  3.665224392080701*^9}},ExpressionUUID->"70179bec-f4bf-44ca-9ec7-\
f890b33b03b4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"sectionalcurvature", "[", 
       RowBox[{"rghypcnf", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"vv", "[", "a", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"vv", "[", "b", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.665220922488048*^9, 3.665220999327064*^9}, {
   3.665221036934848*^9, 3.665221116254575*^9}, {3.665224342809914*^9, 
   3.665224347257821*^9}, {3.665238145567711*^9, 3.6652381627994013`*^9}, {
   3.6652950517393913`*^9, 3.665295088915887*^9}, 3.665298999433454*^9, {
   3.6653836613932323`*^9, 3.665383670776434*^9}, 
   3.665384188689745*^9},ExpressionUUID->"7073bf72-22f4-4fa3-9286-\
fcadb9949025"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"sectionalcurvature", "[", 
       RowBox[{"rghypcnf", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"vv", "[", "a", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"vv", "[", "b", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.665220922488048*^9, 3.665220999327064*^9}, {
   3.665221036934848*^9, 3.665221116254575*^9}, {3.665224342809914*^9, 
   3.665224347257821*^9}, {3.665238145567711*^9, 3.6652381627994013`*^9}, {
   3.6652950517393913`*^9, 3.665295088915887*^9}, 3.665298999433454*^9, {
   3.6653836613932323`*^9, 3.665383670776434*^9}, 3.665384188689745*^9, {
   3.669984091676713*^9, 
   3.669984103636498*^9}},ExpressionUUID->"d84f03cc-8eb7-4d76-aaea-\
b297b93ee85e"],

Cell["\<\
Time needed 59.5 min (dim = 3), 27 min with v. 9.0.1. Repeated with dim 2: \
Time 33.276  s, result -1/r^2.\
\>", "Text",
 CellChangeTimes->{
  3.665230105587511*^9, {3.665235433852387*^9, 3.6652354430280437`*^9}, 
   3.6652381256242933`*^9, {3.665239062027841*^9, 3.665239147338874*^9}, 
   3.665239317848921*^9, {3.665240111059133*^9, 3.665240113531148*^9}, {
   3.665298945896347*^9, 3.665298975343848*^9}, 3.665300575387127*^9, {
   3.6653967400628366`*^9, 3.6653967449744473`*^9}, {3.669984061732832*^9, 
   3.6699840684916286`*^9}, {3.7085712898489113`*^9, 
   3.7085712920658207`*^9}, {3.708571343488488*^9, 
   3.708571361648242*^9}},ExpressionUUID->"095cb941-246b-4048-9d9f-\
15db743eea7f"],

Cell[BoxData[
 RowBox[{"?", "rghypsp"}]], "Input",
 CellChangeTimes->{{3.665230358580886*^9, 3.665230359509132*^9}, {
  3.669982265443413*^9, 
  3.6699822845063887`*^9}},ExpressionUUID->"701807d5-a7ed-4d74-bd57-\
d428a06ee44c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "2"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"sectionalcurvature", "[", 
       RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"vv", "[", "a", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"vv", "[", "b", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.665220922488048*^9, 3.665220999327064*^9}, {
   3.665221036934848*^9, 3.665221116254575*^9}, {3.665224342809914*^9, 
   3.665224347257821*^9}, {3.665238145567711*^9, 3.6652381627994013`*^9}, {
   3.6652950517393913`*^9, 3.665295088915887*^9}, 3.665298999433454*^9, {
   3.665300695122085*^9, 3.6653007358571463`*^9}, 3.665301207893897*^9, {
   3.665321749183455*^9, 3.665321750207285*^9}, 3.665384188697482*^9, 
   3.665396975444901*^9},ExpressionUUID->"ccaa69ce-2f65-4b1b-8029-\
56cbee3aa296"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", "3"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"sectionalcurvature", "[", 
        RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"vv", "[", "a", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"vv", "[", "b", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], ",", 
     RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.665220922488048*^9, 3.665220999327064*^9}, {
   3.665221036934848*^9, 3.665221116254575*^9}, {3.665224342809914*^9, 
   3.665224347257821*^9}, {3.665238145567711*^9, 3.6652381627994013`*^9}, {
   3.6652950517393913`*^9, 3.665295088915887*^9}, 3.665298999433454*^9, {
   3.665300695122085*^9, 3.6653007358571463`*^9}, 3.665301207893897*^9, {
   3.665321749183455*^9, 3.665321750207285*^9}, 3.665384188697482*^9, 
   3.665396975444901*^9, {3.669984336683154*^9, 3.6699843474433727`*^9}, {
   3.70607401479673*^9, 3.706074016755597*^9}, {3.7060740575636683`*^9, 
   3.70607405889319*^9}},ExpressionUUID->"588190bd-362f-4713-9e58-\
cd4bad0f2ae2"],

Cell["\<\
Time needed 40 min (v. 9.0.1.). Repeated with dim 2: Time 38.276 s, result \
-1/r^2.\
\>", "Text",
 CellChangeTimes->{
  3.665230105587511*^9, {3.665235433852387*^9, 3.6652354430280437`*^9}, {
   3.665238090567298*^9, 3.6652380996233873`*^9}, {3.665240072819332*^9, 
   3.665240102803364*^9}, 3.6653011864377003`*^9, {3.6653057864416227`*^9, 
   3.665305808040449*^9}, 3.665396945173216*^9, 3.669984311620071*^9, {
   3.706077104253049*^9, 3.706077113476679*^9}, {3.708572686406863*^9, 
   3.708572714439658*^9}},ExpressionUUID->"741edd22-e41c-4d11-a156-\
78f656d83327"],

Cell["Parallelize does not work:", "Text",
 CellChangeTimes->{{3.665383278588125*^9, 3.66538334271517*^9}, 
   3.7060771405483007`*^9},ExpressionUUID->"25dfcf20-0012-4dbd-a02f-\
73154986577e"],

Cell[BoxData[
 RowBox[{"?", "Parallelize"}]], "Input",
 CellChangeTimes->{{3.6653255678603992`*^9, 
  3.665325599360785*^9}},ExpressionUUID->"d3364065-5946-40e3-ad6a-\
6d97f4da0362"],

Cell[BoxData[
 RowBox[{"LaunchKernels", "[", "8", "]"}]], "Input",
 CellChangeTimes->{{3.665325907833005*^9, 
  3.665325931055479*^9}},ExpressionUUID->"e13dd0b3-acc0-459e-865c-\
8e2bed8c56be"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"Parallelize", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"sectionalcurvature", "[", 
       RowBox[{"rghypsp", "[", "r", "]"}], "]"}], "[", 
      RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       RowBox[{"vv", "[", "a", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"vv", "[", "b", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}]}], "]"}], ",", 
    RowBox[{"Method", " ", "\[Rule]", "\"\<FinestGrained\>\""}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.665220922488048*^9, 3.665220999327064*^9}, {
   3.665221036934848*^9, 3.665221116254575*^9}, {3.665224342809914*^9, 
   3.665224347257821*^9}, {3.665238145567711*^9, 3.6652381627994013`*^9}, {
   3.6652950517393913`*^9, 3.665295088915887*^9}, 3.665298999433454*^9, {
   3.665300695122085*^9, 3.6653007358571463`*^9}, 3.665301207893897*^9, {
   3.665321749183455*^9, 3.665321750207285*^9}, {3.665326005694543*^9, 
   3.665326022221744*^9}, 3.6653841887000437`*^9, {3.706077821050557*^9, 
   3.7060778339763308`*^9}, {3.706077888440744*^9, 3.706077898759774*^9}, {
   3.7060779726070004`*^9, 
   3.706077974768833*^9}},ExpressionUUID->"65a94f07-41cb-401f-9132-\
91311475162c"],

Cell[TextData[{
 "Got a message: ",
 Cell[BoxData[
  RowBox[{
   StyleBox[
    RowBox[{"Parallelize", "::", "nopar1"}], "MessageName"], ":", 
   " ", "\<\"\[NoBreak]\\!\\(\\*RowBox[{RowBox[{RowBox[{\\\"sectionalcurv\\\",\
 \\\"[\\\", RowBox[{\\\"rghypsp\\\", \\\"[\\\", \\\"r\\\", \\\"]\\\"}], \\\"]\
\\\"}], \\\"[\\\", RowBox[{\\\"var\\\", \\\"[\\\", \\\"x\\\", \\\"]\\\"}], \\\
\"]\\\"}], \\\"[\\\", RowBox[{RowBox[{RowBox[{\\\"vv\\\", \\\"[\\\", \
\\\"a\\\", \\\"]\\\"}], \\\"[\\\", RowBox[{\\\"var\\\", \\\"[\\\", \\\"x\\\", \
\\\"]\\\"}], \\\"]\\\"}], \\\",\\\", RowBox[{RowBox[{\\\"vv\\\", \\\"[\\\", \
\\\"b\\\", \\\"]\\\"}], \\\"[\\\", RowBox[{\\\"var\\\", \\\"[\\\", \\\"x\\\", \
\\\"]\\\"}], \\\"]\\\"}]}], \\\"]\\\"}]\\)\[NoBreak] cannot be parallelized; \
proceeding with sequential evaluation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/Parallelize\\\", ButtonNote -> \
\\\"Parallelize::nopar1\\\"]\\)\"\>"}]], "Message", "MSG",
  CellChangeTimes->{3.665326025150035*^9},ExpressionUUID->
  "71767c99-d88c-421c-9851-9ef8e6615bd1"],
 "\naborted. ",
 "\[FilledSquare]"
}], "Text",
 CellChangeTimes->{{3.6653833869067*^9, 3.665383419946466*^9}, {
  3.665468583007874*^9, 
  3.665468585972537*^9}},ExpressionUUID->"fa6ac2ed-9b6f-4d45-9b4c-\
45ab0852775a"],

Cell[TextData[{
 StyleBox["Example 4",
  FontSlant->"Plain",
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[".",
  FontSlant->"Plain"],
 " ",
 StyleBox["Independence on the basis",
  FontSlant->"Plain"]
}], "Text",
 CellChangeTimes->{{3.665468473048113*^9, 3.6654685537773237`*^9}},
 FontSlant->"Italic",ExpressionUUID->"b24e16fb-6079-4ef0-a286-c3f50ff69637"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "2"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"sectionalcurvature", "[", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{
       RowBox[{
        RowBox[{"vv", "[", "a", "]"}], "[", 
        RowBox[{"var", "[", "x", "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"vv", "[", "b", "]"}], "[", 
         RowBox[{"var", "[", "x", "]"}], "]"}], "+", 
        RowBox[{"t", "*", 
         RowBox[{
          RowBox[{"vv", "[", "a", "]"}], "[", 
          RowBox[{"var", "[", "x", "]"}], "]"}]}]}]}], "]"}], ",", 
     RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6654130107478123`*^9, 3.665413051559461*^9}, {
   3.665413098648209*^9, 3.665413216358552*^9}, {3.665413832066362*^9, 
   3.665413847177391*^9}, 
   3.669984786026026*^9},ExpressionUUID->"45112a41-ef40-4d94-8414-\
a6c790c889f2"],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"%", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.665467913405582*^9, 3.665467925588311*^9}, 
   3.66911226461202*^9, 3.66998465358493*^9, {3.669984833391683*^9, 
   3.6699848365036783`*^9}},ExpressionUUID->"19cbc687-9fa6-42ee-8b5d-\
526737d78940"],

Cell["\<\
We try the same with two vectors in the {x1, x3} coordinate plane:\
\>", "Text",
 CellChangeTimes->{{3.66546865074192*^9, 3.665468710484214*^9}, {
  3.669985481410324*^9, 
  3.6699855336815643`*^9}},ExpressionUUID->"28512def-014b-4734-aa6d-\
397188b86fb9"],

Cell[BoxData[
 RowBox[{"vX", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0", ",", "s"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.665467456919257*^9, 3.665467476038458*^9}, {
  3.669985156167336*^9, 
  3.669985167293457*^9}},ExpressionUUID->"3e198ea3-a26b-4ffc-ac8e-\
31a00091ada3"],

Cell[BoxData[
 RowBox[{"vY", " ", "=", " ", 
  RowBox[{"{", " ", 
   RowBox[{"t", ",", "0", ",", "1"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.66546748142387*^9, 3.6654675623241653`*^9}, {
  3.669985181149955*^9, 
  3.6699851862931633`*^9}},ExpressionUUID->"d1233411-bca0-4cb6-8033-\
d62581282886"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"sectionalcurvature", "[", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{"vX", ",", "vY"}], "]"}], ",", 
     RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6654130107478123`*^9, 3.665413051559461*^9}, {
   3.665413098648209*^9, 3.665413216358552*^9}, {3.665413832066362*^9, 
   3.665413847177391*^9}, 3.665467426922014*^9, {3.665467580916201*^9, 
   3.665467655835711*^9}},ExpressionUUID->"22057347-347c-47b0-af00-\
df43f00f2b53"],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"%", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "t"}], "]"}]], "Input",
 CellChangeTimes->{{3.665467913405582*^9, 3.665467925588311*^9}, 
   3.66911226461202*^9, 3.66998465358493*^9, {3.669985043238099*^9, 
   3.669985046629814*^9}},ExpressionUUID->"082171f8-bd6d-4538-a76d-\
4c3ec181ec73"],

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"%%", "[", 
    RowBox[{"[", "2", "]"}], "]"}], ",", "s"}], "]"}]], "Input",
 CellChangeTimes->{{3.665467913405582*^9, 3.665467925588311*^9}, 
   3.66911226461202*^9, 3.66998465358493*^9, {3.669985043238099*^9, 
   3.669985046629814*^9}, {3.669985290804762*^9, 
   3.669985311476035*^9}},ExpressionUUID->"f9bcbcf2-76d6-4fb5-9104-\
ac65fd8bcede"],

Cell[TextData[{
 "Time needed: ",
 "56.416 ",
 "s.  The result  does not depend on s or t. This confirms the\n",
 StyleBox["Lemma",
  FontColor->RGBColor[0, 0, 1]],
 ". The sectional curvature depends on the tangent plane only, not on the \
chosen basis X, Y of the plane. \[FilledSmallSquare]"
}], "Text",
 CellChangeTimes->{{3.6654680969550962`*^9, 3.665468118993449*^9}, {
   3.665468159462591*^9, 3.665468338891241*^9}, {3.665495081110128*^9, 
   3.665495081485689*^9}, 3.669985034598653*^9, {3.6699853963001347`*^9, 
   3.6699854492263937`*^9}},ExpressionUUID->"e7e83712-2803-4c3a-9f0d-\
7e262d4703be"],

Cell[TextData[{
 StyleBox["Remark.",
  FontColor->RGBColor[0, 0, 1]],
 " In the case that the vectors vX,vY are linearly dependent one gets an \
error message:"
}], "Text",
 CellChangeTimes->{{3.6699857713913593`*^9, 
  3.6699858494947767`*^9}},ExpressionUUID->"73eec0a4-fc15-43f0-bee5-\
15bf5cb29919"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dim", "=", " ", "3"}], ";", 
  RowBox[{"Timing", "[", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"sectionalcurvature", "[", "]"}], "[", 
       RowBox[{"var", "[", "x", "]"}], "]"}], "[", 
      RowBox[{"vX", ",", "vX"}], "]"}], ",", 
     RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6654130107478123`*^9, 3.665413051559461*^9}, {
   3.665413098648209*^9, 3.665413216358552*^9}, {3.665413832066362*^9, 
   3.665413847177391*^9}, 3.665467426922014*^9, {3.665467580916201*^9, 
   3.665467655835711*^9}, 
   3.6699856460574512`*^9},ExpressionUUID->"e75bdb5f-a98c-4060-b3bc-\
913b6a668809"],

Cell[TextData[{
 StyleBox["Example 5",
  FontColor->RGBColor[0, 0, 1]],
 ". The Gaussian curvature of a general surface. The general metric tensor of \
the surface has the coordinates"
}], "Text",
 CellChangeTimes->{{3.665495411121272*^9, 3.6654954705606337`*^9}, 
   3.7060732643528767`*^9, {3.7060989989029427`*^9, 
   3.706099053310841*^9}},ExpressionUUID->"b764f670-e7c0-444c-bfe5-\
c5f7158a6ca8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RGT", "[", "smf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.66546900913544*^9, 3.66546901996623*^9}, {
  3.665469132553668*^9, 
  3.6654691630452347`*^9}},ExpressionUUID->"b559afa1-497d-41da-aad8-\
782f3aaf31da"],

Cell[BoxData[
 RowBox[{"?", "rgt"}]], "Input",
 CellChangeTimes->{{3.6654691839566298`*^9, 
  3.6654691867108*^9}},ExpressionUUID->"40157670-a3e1-4e96-a13f-975b77e97aa3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"gh", "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"rgt", "[", "smf", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "[", 
   RowBox[{"i", ",", "j"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.665469605807938*^9, 3.665469669876974*^9}, {
  3.665470121980071*^9, 
  3.665470131429616*^9}},ExpressionUUID->"ad46037a-1d57-42ae-af12-\
e56a7d88ef88"],

Cell[BoxData[
 RowBox[{
  RowBox[{"gh", "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"1", ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.66546986835655*^9, 3.665469877189332*^9}, {
  3.665470149822192*^9, 
  3.665470154534254*^9}},ExpressionUUID->"4947bc45-49c7-403b-879f-\
23896213d7bc"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"sectionalcurvature", "[", "gh", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{
     RowBox[{
      RowBox[{"vv", "[", "a", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}], ",", 
     RowBox[{
      RowBox[{"vv", "[", "b", "]"}], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6652096375705214`*^9, 3.665209778371078*^9}, {
   3.665214715947653*^9, 3.665214728995122*^9}, {3.6652940563155518`*^9, 
   3.665294066443132*^9}, {3.665294133635438*^9, 3.665294143235124*^9}, {
   3.6652942796349916`*^9, 3.6652942958091097`*^9}, {3.6652943581290703`*^9, 
   3.66529436901789*^9}, {3.6653214189979477`*^9, 3.6653214200066338`*^9}, 
   3.6653841886834497`*^9, {3.665394844443334*^9, 3.66539485177134*^9}, {
   3.6654703149338818`*^9, 3.6654703412614813`*^9}, {3.665470374692848*^9, 
   3.665470377692583*^9}, {3.665470812190475*^9, 
   3.665470829870311*^9}},ExpressionUUID->"49a0efa8-ccac-478b-a4e7-\
5d946f05dfa5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Ksf", " ", "=", " ", 
   RowBox[{"%", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.665495174070166*^9, 3.665495189500765*^9}, 
   3.669987231978516*^9, {3.7060808911226063`*^9, 3.70608090045848*^9}, 
   3.706098908624379*^9},ExpressionUUID->"64e30d49-70fa-41b0-a936-\
61cce4ec675c"],

Cell["\<\
The time needed was 307.196 s. The result is the Gaussian curvature of the \
surface:\
\>", "Text",
 CellChangeTimes->{{3.665474140696239*^9, 3.6654741933282213`*^9}, {
   3.665481696842823*^9, 3.6654817030188103`*^9}, 3.706073264381637*^9, {
   3.706101093024974*^9, 
   3.706101095496991*^9}},ExpressionUUID->"bd231e8d-20ba-42dd-9420-\
49f56526e22c"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"gauss", "[", "smf", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7060810645316153`*^9, 3.7060810790522957`*^9}, 
   3.706081155187201*^9, 
   3.7085742368417*^9},ExpressionUUID->"4a391c56-39d4-4f83-a252-27cab9fc8140"],

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"Together", "[", 
     RowBox[{"Ksf", " ", "-", " ", 
      RowBox[{
       RowBox[{"gauss", "[", "smf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}], "]"}], ",", 
    RowBox[{"TimeConstraint", "\[Rule]", " ", "Infinity"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.665474199402516*^9, 3.66547427082346*^9}, {
  3.6654755177142143`*^9, 3.665475597723257*^9}, {3.669987264706503*^9, 
  3.669987267258237*^9}},ExpressionUUID->"3986b2c9-4796-41db-a570-\
d55f51d7477d"],

Cell[TextData[{
 "After  42 min, ",
 "0.332 s with v. 9.0.1,",
 " the correct result: 0. \[FilledSquare]\nThis calculation is a proof of ",
 StyleBox["Gauss\[CloseCurlyQuote]s Theorema Egregium",
  FontColor->RGBColor[0, 0, 1]],
 ": The Gaussian curvature, here defined as the quotient of the determinants \
of the second and first fundamental forms of the surface, is by Ksf expressed \
in terms of the first fundamental form only. Thus isometric surfaces have the \
same Gaussian curvature."
}], "Text",
 CellChangeTimes->{{3.66549530782224*^9, 3.6654953767600727`*^9}, {
   3.670000754049816*^9, 3.67000076608957*^9}, {3.706091599844638*^9, 
   3.706091823797367*^9}, 3.706091856212461*^9, {3.7060987282321672`*^9, 
   3.706098807000654*^9}, {3.708574285236198*^9, 3.708574285754902*^9}, {
   3.7085743220997877`*^9, 3.708574347715616*^9}, 
   3.708574380026944*^9},ExpressionUUID->"a525efbb-c10d-4721-8eeb-\
cd1efaec1967"]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\

Epilog\
\>", "Subtitle",
 CellChangeTimes->{{3.6716876228171463`*^9, 3.6716876277928467`*^9}, {
   3.671687750239933*^9, 3.671687751856036*^9}, {3.671688091293305*^9, 
   3.6716881070619087`*^9}, 3.67220196902218*^9, {3.706161938616461*^9, 
   3.7061619443364964`*^9}},ExpressionUUID->"78de34fe-e325-486c-9d5d-\
8bf92c00f0d9"],

Cell[TextData[{
 "In this Epilog I mention some important subjects not treated in the \
notebooks;  the user may continue the study of differential geometry in one \
or several of the mentioned directions and  elaborate Mathematica functions \
for the corresponding calculations. The references given in the epilog as \
well as in the Notebooks are very subjective and in no way complete. Please \
use the mathematical databases ",
 ButtonBox["Mathematical Reviews",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.ams.org/mr-database"], None},
  ButtonNote->"http://www.ams.org/mr-database"],
 " or ",
 ButtonBox["Zentralblatt",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://zbmath.org/"], None},
  ButtonNote->"https://zbmath.org/"],
 " to find the rich actual literature. Furthermore I recommend to look into \
the Wolfram Library Archive to find information on Mathematica projects and \
packages for the interested item.  In particular, search there for \
\[OpenCurlyDoubleQuote]Tensor\[CloseCurlyDoubleQuote] tom find  information \
and sources on work based  on Mathematica, e. g.  ",
 ButtonBox["MathTensor",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://library.wolfram.com/infocenter/TechNotes/4697/"], None},
  ButtonNote->"http://library.wolfram.com/infocenter/TechNotes/4697/"],
 ", ",
 ButtonBox["H. Soleng",
  BaseStyle->"Hyperlink",
  ButtonData->"Soleng"],
 ", ",
 ButtonBox["Jos\[EAcute] M. Mart\[IAcute]n-Garc\[IAcute]a",
  BaseStyle->"Hyperlink",
  ButtonData->"xAct"],
 ", and others."
}], "Text",
 CellChangeTimes->{{3.671688213783787*^9, 3.671688222742345*^9}, {
   3.671688290548703*^9, 3.671688343484438*^9}, {3.6716884032201033`*^9, 
   3.6716884246279507`*^9}, {3.671688456171796*^9, 3.671688687794319*^9}, {
   3.671689012600731*^9, 3.671689149855845*^9}, {3.6716891903678226`*^9, 
   3.6716892776947603`*^9}, {3.6716894540527353`*^9, 3.671689454055334*^9}, {
   3.671689584596363*^9, 3.671689584599622*^9}, {3.671690002480818*^9, 
   3.671690103472639*^9}, {3.671690196496668*^9, 3.671690196499049*^9}, 
   3.671706282135581*^9, {3.6717142610949583`*^9, 3.671714265908896*^9}, 
   3.672202033327105*^9, 3.672202752168088*^9, {3.672203095943861*^9, 
   3.6722032016453342`*^9}, {3.67220328066039*^9, 3.672203289244514*^9}, {
   3.6722033697241087`*^9, 3.672203511755501*^9}, {3.672203544482704*^9, 
   3.672203575659651*^9}, {3.672203760699835*^9, 3.672203763523252*^9}, {
   3.672203815737454*^9, 3.672203877106618*^9}, {3.67220392568254*^9, 
   3.672204062050118*^9}, {3.6722041281491547`*^9, 3.6722041381298237`*^9}, {
   3.672204214361268*^9, 3.672204218625204*^9}, {3.672204519251274*^9, 
   3.672204540486314*^9}, {3.672204836498383*^9, 3.672204836501189*^9}, {
   3.672205028294256*^9, 3.672205028296596*^9}, {3.6722056316811438`*^9, 
   3.67220564176864*^9}, 3.672205713511763*^9, 3.6728161685721817`*^9, {
   3.7060920752950993`*^9, 3.706092081838668*^9}, {3.708575030324274*^9, 
   3.708575033934956*^9}, {3.7085750810378532`*^9, 3.7085751016459703`*^9}, {
   3.708575207085425*^9, 
   3.7085752662197857`*^9}},ExpressionUUID->"a4354efa-c7c2-4293-a141-\
bee3036b4ef9"],

Cell[CellGroupData[{

Cell["1. Relativity Theory", "Subsubtitle",
 CellChangeTimes->{{3.671715206330577*^9, 3.67171522531428*^9}, {
  3.671715264297999*^9, 
  3.671715276161693*^9}},ExpressionUUID->"beacea25-3f6f-4bc1-84dc-\
383d09f44305"],

Cell[TextData[{
 "The space-time world of special relativity theory is the Lorentz space: the \
four-dimensional pseudo-Euclidean vector space of index 1, and that of the \
general relativity theory is a 4-dimensional pseudo-Riemannian manifold of \
index 1. Thus the Mathematica concepts contained in this notebook and the \
accompanying packages may serve as a starting point to go ahead into this \
field. There exists a lot of applications of Mathematica to this field. Many \
concrete calculations cam be found in the work of  ",
 ButtonBox["Sortirios Bonanos",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.inp.demokritos.gr/~sbonano/"], None},
  ButtonNote->"http://www.inp.demokritos.gr/~sbonano/"],
 " who unfortunately already  ",
 ButtonBox["died",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://hyperspace.uni-frankfurt.de/2013/10/22/death-of-sotirios-\
bonanos/"], None},
  ButtonNote->
   "http://hyperspace.uni-frankfurt.de/2013/10/22/death-of-sotirios-bonanos/"],
 " in 2013. Further see the commercial package ",
 ButtonBox["Cartan",
  BaseStyle->"Hyperlink",
  ButtonData->"Soleng"],
 " of Harald. H. Soleng, the package ",
 ButtonBox["ccgrg",
  BaseStyle->"Hyperlink",
  ButtonData->"ccgrg"],
 " of Andrzej Woszczyna and others."
}], "Text",
 CellChangeTimes->{{3.671715322881287*^9, 3.671715532847365*^9}, {
   3.6717159414352093`*^9, 3.671716102721624*^9}, {3.6717161554891577`*^9, 
   3.67171627087232*^9}, {3.6722030515972347`*^9, 3.672203089580769*^9}, {
   3.67220603276295*^9, 3.672206181313596*^9}, {3.6722062227778254`*^9, 
   3.672206273356099*^9}, {3.672208911659809*^9, 3.6722089154838543`*^9}, {
   3.672208946438724*^9, 3.672208946440905*^9}, {3.6722090390956173`*^9, 
   3.6722090390979643`*^9}, {3.672210860019474*^9, 3.672210862363492*^9}, {
   3.672210921826956*^9, 3.672210990674408*^9}, {3.67221102261765*^9, 
   3.672211022619481*^9}, {3.67221120006479*^9, 3.672211218472829*^9}, {
   3.6722112590325727`*^9, 3.6722112890395937`*^9}, {3.672211366053814*^9, 
   3.67221136605651*^9}, 3.672236975435512*^9, {3.6723184217714777`*^9, 
   3.6723184289395533`*^9}, {3.672318484220792*^9, 3.672318544062438*^9}, {
   3.672380740051495*^9, 
   3.672380771137138*^9}},ExpressionUUID->"8bb73728-9aab-4c65-9b93-\
badd74c80982"]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.  Lie Groups and Lie Algebras", "Subsubtitle",
 CellChangeTimes->{{3.671715206330577*^9, 3.67171522531428*^9}, {
   3.671715264297999*^9, 3.671715276161693*^9}, {3.6723811311966143`*^9, 
   3.672381138860183*^9}, {3.672381267067646*^9, 3.672381294251239*^9}, 
   3.672383640246601*^9},ExpressionUUID->"2393b25f-3c7f-4fac-b90c-\
01cce9c3a584"],

Cell[TextData[{
 "The theory of Lie groups and algebras is needed for a wide field of \
developments in differential geometry, geometric analysis, and algebra with \
important applications to physics. There are some special programs written in \
Mathematica, e. g. the work ",
 ButtonBox["LieArt",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://lieart.hepforge.org/"], None},
  ButtonNote->"http://lieart.hepforge.org/"],
 " of ",
 ButtonBox["Robert Feger and Thomas w. Kephart",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://library.wolfram.com/infocenter/Articles/9174/"], None},
  ButtonNote->"http://library.wolfram.com/infocenter/Articles/9174/"],
 ", or my old ",
 ButtonBox["package,",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://library.wolfram.com/infocenter/MathSource/677/"], None},
  ButtonNote->"http://library.wolfram.com/infocenter/MathSource/677/"],
 " but a systematic treatment of this theory in Mathematica concepts seems to \
be missing yet. An interactive textbook developed along the lines of ",
 ButtonBox["S. Helgason\[CloseCurlyQuote]s book",
  BaseStyle->"Hyperlink",
  ButtonData->"H78"],
 " would be an important step in using Mathematica methods in transformation \
groups, fiber bundles and holonomy theory, homogeneous, in particular \
symmetric spaces."
}], "Text",
 CellChangeTimes->{{3.672381422433049*^9, 3.67238146512354*^9}, {
   3.672381536689439*^9, 3.6723815549771023`*^9}, {3.672381620136437*^9, 
   3.672381802999544*^9}, {3.672381844463162*^9, 3.6723819441022787`*^9}, {
   3.6723819764700613`*^9, 3.672381999421865*^9}, {3.672382047757482*^9, 
   3.67238214181283*^9}, 3.672382197572144*^9, {3.672382254283856*^9, 
   3.67238226377175*^9}, {3.672382432514201*^9, 3.672382596496859*^9}, {
   3.6723826737525578`*^9, 3.67238269117594*^9}, {3.6723827405197973`*^9, 
   3.6723827871836777`*^9}, {3.672382818943042*^9, 3.672382905254567*^9}, {
   3.6723829626474648`*^9, 3.6723829662297487`*^9}, {3.6723830546073647`*^9, 
   3.6723830546094017`*^9}, {3.672383206183859*^9, 3.672383206185788*^9}, {
   3.6723834339392138`*^9, 3.672383433941656*^9}, {3.672383487169426*^9, 
   3.67238349541707*^9}, {3.672383540555155*^9, 3.672383540557588*^9}, {
   3.672383572873807*^9, 3.672383584248475*^9}, 3.6728162302980824`*^9, 
   3.6728163136976233`*^9, {3.672829995137208*^9, 3.672829999015523*^9}, {
   3.672830191517701*^9, 3.6728302058379383`*^9}, 3.672835135456682*^9, {
   3.673873002770631*^9, 3.673873005587339*^9}, 3.673873901724256*^9, 
   3.673875021393856*^9, {3.673875054038423*^9, 3.6738750540419064`*^9}, {
   3.675916213233891*^9, 3.6759162171592073`*^9}, 3.706092227569968*^9, 
   3.7060923773130293`*^9},ExpressionUUID->"ea0d6266-3ab4-4405-a82b-\
f7ac50e3c990"]
}, Open  ]],

Cell[CellGroupData[{

Cell["3. Submanifolds of Homogeneous Spaces with Cartan Methods ", \
"Subsubtitle",
 CellChangeTimes->{{3.671715206330577*^9, 3.67171522531428*^9}, {
  3.671715264297999*^9, 3.671715276161693*^9}, {3.6723811311966143`*^9, 
  3.672381138860183*^9}, {3.672381267067646*^9, 3.672381294251239*^9}, {
  3.672383655246419*^9, 
  3.672383736557804*^9}},ExpressionUUID->"0679a443-6b10-4959-b6a4-\
0e03e76059de"],

Cell[TextData[{
 "Here I have in mind less the general theory but the investigation of \
special problems in projective, affine, symplectic or M\[ODoubleDot]bius \
spaces. For these fields there exists an overwhelming large number of \
publications, but not so much is done, outside of Euclidean geometry,  with \
Mathematica. Just appeared the book of ",
 ButtonBox["G. R. Jensen, E. Musso, and L. Nicolodi",
  BaseStyle->"Hyperlink",
  ButtonData->"JMN"],
 " which could stimulate interesting Mathematica projects in this field. \
Clearly there are many further subjects which could be mentioned, like \
complex geometry, geometric theory of differential equations, Hamiltonian \
equations and calculus of variations,  but to discuss them here would break \
the limits of this notebook."
}], "Text",
 CellChangeTimes->{{3.672383826336176*^9, 3.672384079435029*^9}, {
   3.67238418613811*^9, 3.672384248713423*^9}, {3.672384283193425*^9, 
   3.672384292593459*^9}, {3.672384490519846*^9, 3.6723845119034567`*^9}, {
   3.672384557327046*^9, 3.67238472118196*^9}, {3.672384753821766*^9, 
   3.6723847932454987`*^9}, {3.672829915119772*^9, 3.672829961974832*^9}, {
   3.67283057325832*^9, 3.672830592562158*^9}, 3.672835168374473*^9, {
   3.673873147816848*^9, 3.673873162041031*^9}, {3.673873290719969*^9, 
   3.6738733535115013`*^9}, {3.6738736211497917`*^9, 3.673873647069478*^9}, {
   3.673873689893002*^9, 3.673873748780725*^9}, {3.673873789900321*^9, 
   3.673873799828907*^9}, {3.6738739735313*^9, 3.673874009994541*^9}, {
   3.6738749330260277`*^9, 
   3.673874933027952*^9}},ExpressionUUID->"56bcdf2f-f257-42f8-a881-\
6b636ff1a158"],

Cell[TextData[StyleBox["I  will be very grateful for all  comments, critical \
remarks or hints to errors. Reports on experiences of using the notebooks in \
teaching differential geometry are welcome.  Please, write me an e-mail to \
sulanke@mathematik.hu-berlin.de.",
 FontSlant->"Plain"]], "Text",
 CellChangeTimes->{{3.672830728303721*^9, 3.672830772927446*^9}, {
  3.672830842871059*^9, 3.672830863464491*^9}, {3.6728308965190983`*^9, 
  3.672830913262577*^9}, {3.672831011429667*^9, 3.672831113172864*^9}, {
  3.672831198988104*^9, 3.672831199916016*^9}, {3.672831232963868*^9, 
  3.672831315155322*^9}, {3.6728313556750402`*^9, 3.672831402386644*^9}, {
  3.6728316466481237`*^9, 3.6728316481524067`*^9}, {3.7062458296714287`*^9, 
  3.70624583057506*^9}},
 FontSlant->"Italic",ExpressionUUID->"e9efd352-0f45-434b-8785-b7691ed3c2d7"],

Cell["\<\
Berlin. July 10, 2017.
Rolf Sulanke\
\>", "Text",
 CellChangeTimes->{{3.67283156863279*^9, 3.672831606232538*^9}, {
  3.7060986525113373`*^9, 3.706098676631462*^9}, {3.708575442059402*^9, 
  3.708575447786562*^9}},ExpressionUUID->"9d6d2c2a-8f99-4c5d-bd6e-\
0f6ba62976fa"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["References", "Subtitle",
 CellChangeTimes->{{3.5544745369306817`*^9, 3.554474559500721*^9}, {
  3.554527079991947*^9, 3.55452712145682*^9}, {3.554528488699799*^9, 
  3.5545285128486414`*^9}, {3.554531186249479*^9, 3.554531220616339*^9}, {
  3.554617243384263*^9, 3.554617271261512*^9}},
 FontFamily->"Times",ExpressionUUID->"082926a3-0911-46b8-bd79-e390d7d3fe79"],

Cell["\<\
The References contain only citations used in this notebook. In many \
textbooks or monographs about differential geometry one finds comprehensive \
bibliographies.\
\>", "Text",
 CellChangeTimes->{{3.670129448859424*^9, 3.670129658042801*^9}, {
   3.670129950720845*^9, 3.6701299721285954`*^9}, {3.670130569213179*^9, 
   3.6701305789572277`*^9}, 
   3.670396165997735*^9},ExpressionUUID->"b219b1b3-91a7-4877-8602-\
d309442b73be"],

Cell["\<\
The newest versions of the notebooks and packages on differential geometry \
are available on my homepage:\
\>", "Text",
 CellChangeTimes->{{3.701499617826499*^9, 3.701499654441882*^9}, {
  3.701499768321471*^9, 3.70149983980879*^9}, {3.7014999574483223`*^9, 
  3.7014999792480087`*^9}, {3.701500094758868*^9, 3.7015001051749067`*^9}, {
  3.7060986094416428`*^9, 3.706098612079434*^9}, {3.706163269677351*^9, 
  3.706163271453485*^9}, {3.706251796911285*^9, 
  3.7062517980791693`*^9}},ExpressionUUID->"9febeb6a-722c-46c8-a41f-\
083491e845e4"],

Cell[TextData[{
 "R. Sulanke. ",
 ButtonBox["Differential Geometry",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www-irm.mathematik.hu-berlin.de/~sulanke/diffgeo.html"], None},
  ButtonNote->"http://www-irm.mathematik.hu-berlin.de/~sulanke/diffgeo.html"],
 ". Since 2016."
}], "Text",
 CellChangeTimes->{{3.701499851933041*^9, 3.701499944735998*^9}, {
  3.7015000115914516`*^9, 3.701500076751062*^9}, {3.701500111526679*^9, 
  3.701500128310445*^9}, {3.701500319285164*^9, 3.701500342748609*^9}, {
  3.701500426041005*^9, 3.701500426043584*^9}, {3.706162969729157*^9, 
  3.706162976137117*^9}, {3.706163141215125*^9, 3.706163171710301*^9}, {
  3.706163226517386*^9, 3.706163226519713*^9}},
 CellTags->"IDG",ExpressionUUID->"7faa1803-e9ac-4398-8bb1-84f3acca0425"],

Cell["\<\
W. Blaschke, H. Reichardt. Einf\[UDoubleDot]hrung in die \
Differentialgeometrie.2. Auflage. Springer-Verlag. \
Berlin.G\[ODoubleDot]ttingen.Heidelberg. 1960.\
\>", "Text",
 CellChangeTimes->{{3.5866621958935566`*^9, 3.5866623742377577`*^9}, 
   3.5875491110897255`*^9, {3.588167593643005*^9, 3.5881676056026893`*^9}, 
   3.669112264695797*^9, 3.670126131388859*^9, 3.6738751397273817`*^9, {
   3.700220907102117*^9, 3.7002209081337852`*^9}},
 CellTags->"B-R",ExpressionUUID->"21df92d0-0d9b-47dd-9f83-0bd9b35930f9"],

Cell["\<\
W. Blaschke, K. Leichtwei\[SZ]. Elementare Differentialgeometrie. 5. Auflage. \
Springer-Verlag. Berlin.G\[ODoubleDot]ttingen.Heidelberg. 1973.\
\>", "Text",
 CellChangeTimes->{{3.7002207819024963`*^9, 3.700220896901835*^9}, {
  3.700220934677589*^9, 3.700220948981552*^9}, {3.7002211110206957`*^9, 
  3.700221112284165*^9}},
 CellTags->"B-L",ExpressionUUID->"1f8500fc-8832-44fb-ae96-c5038fa453db"],

Cell[TextData[{
 "Steven. M. Christensen & Associates. ",
 ButtonBox["MathTensor",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://library.wolfram.com/infocenter/TechNotes/4697/"], None},
  ButtonNote->"http://library.wolfram.com/infocenter/TechNotes/4697/"],
 "."
}], "Text",
 CellChangeTimes->{{3.568446200711132*^9, 3.568446259272482*^9}, {
   3.5684465141440597`*^9, 3.5684465276968346`*^9}, {3.5684467203818555`*^9, 
   3.568446726824224*^9}, 3.6029191105032673`*^9, {3.6701309056495028`*^9, 
   3.670130917936844*^9}, {3.670130953205494*^9, 3.670130953208911*^9}, 
   3.6722041141127167`*^9},
 CellTags->"Christ",ExpressionUUID->"0667904c-1335-41aa-be8f-da12f32b1340"],

Cell[TextData[{
 "Remark. This softtware seems to be not available now, \nsee ",
 ButtonBox["http://community.wolfram.com/groups/-/m/t/339315",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://community.wolfram.com/groups/-/m/t/339315"], None},
  ButtonNote->"http://community.wolfram.com/groups/-/m/t/339315"]
}], "Text",
 CellChangeTimes->{{3.670131481911681*^9, 3.6701315838454323`*^9}, {
  3.670131618217527*^9, 3.670131627826985*^9}, {3.670131661965198*^9, 
  3.6701316619679403`*^9}},ExpressionUUID->"4ba80fa6-4efa-4c51-a683-\
312397d6b91d"],

Cell[TextData[{
 "[G06] Alfred Gray, Simon Salamon, Elsa Abbena. Modern ",
 "D",
 "ifferential Geometry of Curves and Surfaces with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".\nThird ed. CRC Press. 2006. "
}], "Text",
 CellChangeTimes->{{3.4481738125625*^9, 3.448173840671875*^9}, {
   3.44817393246875*^9, 3.44817394509375*^9}, {3.448173979828125*^9, 
   3.448173980734375*^9}, {3.4481759745*^9, 3.448176007265625*^9}, 
   3.450237117953125*^9, {3.450237154*^9, 3.450237191546875*^9}, {
   3.450238953984375*^9, 3.450238956734375*^9}, 3.541836408394127*^9, 
   3.5418367181953497`*^9, 3.669112264715403*^9, 3.67012613898711*^9},
 CellTags->"Gray",ExpressionUUID->"cc8232d6-5c7f-4187-ace0-7b06ffe46740"],

Cell[TextData[{
 "[G06nb] Alfred Gray, Simon Salamon, Elsa Abbena. A series of 27 ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebooks corresponding to the chapters of [G06], available from the \
website \n",
 ButtonBox["http://webmath2.unito.it/paginepersonali/abbena/gray/",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://webmath2.unito.it/paginepersonali/abbena/gray/"], None},
  ButtonNote->"http://webmath2.unito.it/paginepersonali/abbena/gray/"]
}], "Text",
 CellChangeTimes->{{3.599201219155733*^9, 3.599201261780171*^9}, {
   3.5992013329092393`*^9, 3.599201340627681*^9}, {3.5992013935957108`*^9, 
   3.5992014308198395`*^9}, {3.599201571187868*^9, 3.599201593564148*^9}, {
   3.599201693291852*^9, 3.5992016987161627`*^9}, {3.599201729499923*^9, 
   3.599201765534984*^9}, {3.6701321544067*^9, 3.670132157131933*^9}, 
   3.670308111582549*^9, {3.670308173229368*^9, 3.6703081732317953`*^9}},
 CellTags->"Gray2",ExpressionUUID->"898a7ded-4561-4d52-b930-9f4af91f2b54"],

Cell["\<\
[G94] Alfred Gray. Differentialgeometrie. Klassische Theorie in moderner \
Darstellung. (\[CapitalUDoubleDot]bersetzung aus dem Amerikanischen H. Gollek).
Spektrum Akademischer Verlag, Heidelberg.Berlin.Oxford. 1994.\
\>", "Text",
 CellChangeTimes->{{3.450237198875*^9, 3.45023720096875*^9}, {
   3.450237249703125*^9, 3.450237440828125*^9}, {3.450238940359375*^9, 
   3.45023894278125*^9}, {3.454066758375*^9, 3.4540668540625*^9}, 
   3.45407173278125*^9, 3.541836408394155*^9, 3.5418367181953783`*^9, {
   3.669112264736435*^9, 3.669112264753516*^9}, {3.670126142300949*^9, 
   3.6701261435583763`*^9}},
 CellTags->"Gray3",ExpressionUUID->"c6706746-5ca2-46ac-9e8d-809a3ea97382"],

Cell["\<\
S. Helgason.Differential Geometry, Lie Groups, and Symmetric Spaces.
Academic Press, 1978.\
\>", "Text",
 CellChangeTimes->{
  3.6492278754609175`*^9, {3.649228192336979*^9, 3.6492281995401945`*^9}, {
   3.649228584760295*^9, 3.6492286043229027`*^9}, {3.650198204150859*^9, 
   3.6501982744354153`*^9}, 3.669112264673501*^9, 3.670126133790217*^9, 
   3.6738750761682167`*^9},
 CellTags->"H78",ExpressionUUID->"0758737c-9bc5-44ba-a44c-6f7e319bf127"],

Cell["\<\
G. R. Jensen, E. Musso, L. Nicolodi. Surfaces in Classical Geometries. A \
Treatment by Moving Frames. Universitext. Springer-Verlag. 2016.\
\>", "Text",
 CellChangeTimes->{{3.673874105593801*^9, 3.673874130256372*^9}, {
  3.673874374750423*^9, 3.673874428662014*^9}, {3.6738744765734386`*^9, 
  3.673874489901372*^9}, {3.673874615676599*^9, 3.673874666139975*^9}},
 CellTags->"JMN",ExpressionUUID->"268e697c-a6d7-4db2-99be-89bde2586826"],

Cell["\<\
S. Kobayashi, K. Nomizu. Foundations of Differential Geometry, Vol.  I. \
Interscience Publishers. New York - London. 1963.\
\>", "Text",
 CellChangeTimes->{{3.597388908321149*^9, 3.5973889165686207`*^9}, {
   3.5973889798572407`*^9, 3.597389168017003*^9}, 3.669112264790337*^9, 
   3.6701261495935793`*^9, 3.673875331966544*^9},
 CellTags->"K-N1",ExpressionUUID->"f4362455-c4bb-4a38-a039-80dd10285d0d"],

Cell["\<\
Erwin Kreyszig. Differentialgeometrie. Akademische Verlagsgesellschaft Geest \
& Portig K. G. Leipzig 1957.\
\>", "Text",
 CellChangeTimes->{{3.5906430418435745`*^9, 3.5906431143993025`*^9}, {
   3.5906432147074785`*^9, 3.590643286436404*^9}, 3.6691122647704077`*^9, 
   3.670126148070321*^9, 3.673875365253952*^9},
 CellTags->"Kr",ExpressionUUID->"84379d9b-123f-495a-9ac6-282ae8d762cd"],

Cell["\<\
A. I. Maltzev. Grundlagen der linearen Algebra (Russ.) , Moskau 1956.\
\>", "Text",
 CellChangeTimes->{3.673875399565633*^9},
 CellTags->"Mal",ExpressionUUID->"e515a207-0109-4bdc-a391-6e6cc86c3d63"],

Cell[TextData[{
 "Jos\[EAcute] M.Mart\[IAcute]n - Garc\[IAcute]a.",
 ButtonBox[" xAct",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.xact.es/"], None},
  ButtonNote->"http://www.xact.es/"],
 ":  Efficient tensor computer algebra for the Wolfram Language. "
}], "Text",
 CellChangeTimes->{{3.672204573863549*^9, 3.672204600717012*^9}, {
  3.672204653628922*^9, 3.672204670652895*^9}, {3.672204742463983*^9, 
  3.672204742465872*^9}},
 CellTags->"xAct",ExpressionUUID->"0033d828-935c-4125-a93a-682f3fe8b07f"],

Cell["\<\
A.L.Onishchik, Sulanke, R.  Algebra und Geometrie, Einf\[UDoubleDot]hrung. \
VEB Deutscher Verlag der Wissenschaften,Berlin 1977. (2. Aufl.Berlin 1986, \
Russ. Moscow 2004)\
\>", "Text",
 CellChangeTimes->{
  3.558420210558278*^9, {3.5584202541427712`*^9, 3.55842026933464*^9}, {
   3.558420425654581*^9, 3.5584204384943156`*^9}, {3.558420483111868*^9, 
   3.5584204900312634`*^9}, 3.6029190720390673`*^9, 3.669112264845334*^9, 
   3.67012615333636*^9, 3.670568347480423*^9, {3.672202266190864*^9, 
   3.672202279336891*^9}, 3.672316871744972*^9, 3.673875435277327*^9},
 CellTags->"OSI",ExpressionUUID->"3cdfef22-aea4-4a65-aa81-8bca3fd44206"],

Cell["\<\
A.L.Onishchik, Sulanke,R. Algebra und Geometrie, 2. Moduln und Algebren. VEB \
Deutscher Verlag der Wissenschaften, Berlin 1988, (Russ. Moscow 2008). \
\>", "Text",
 CellChangeTimes->{{3.5584202018147783`*^9, 3.5584202427751207`*^9}, {
   3.5584202745509386`*^9, 3.5584203272139506`*^9}, {3.55842050308601*^9, 
   3.5584205424142594`*^9}, 3.558420596694364*^9, 3.6691122648690853`*^9, 
   3.6701261567875423`*^9, 3.673875475493106*^9},
 CellTags->"OSII",ExpressionUUID->"4ae8ad1d-c3c6-4659-88df-49dee0826979"],

Cell["\<\
[OSIII] A.L. Onishchik,  Sulanke, R. Projective and Cayley-Klein Geometries, \
Springer Monographs in Mathematics, Berlin Heidelberg 2006.\
\>", "Text",
 CellChangeTimes->{
  3.558420632032385*^9, {3.5584206638472047`*^9, 3.558420707766717*^9}, {
   3.5584207826470003`*^9, 3.5584208113106394`*^9}, {3.558420924839133*^9, 
   3.5584209540868053`*^9}, {3.558421144135676*^9, 3.5584211873778887`*^9}, 
   3.644757937343561*^9, 3.6723169003129873`*^9, {3.704179511878263*^9, 
   3.704179535846389*^9}},
 CellTags->"OSIII",ExpressionUUID->"d8d078d5-e367-4e9b-af60-df057e345174"],

Cell["\<\
 J. A. Schouten. Ricci-Calculus. Grundlehren der mathematischen \
Wissenschaften Vol. X, 2nd. edition. Springer-Verlag 1954.\
\>", "Text",
 CellChangeTimes->{{3.6502012768965425`*^9, 3.6502012785969076`*^9}, {
   3.6502013124747396`*^9, 3.6502014803563447`*^9}, 3.673875626548039*^9},
 CellTags->"Sch54",ExpressionUUID->"7e0f735d-2d39-4c01-8b29-f97ba112e151"],

Cell[TextData[{
 "  ",
 ButtonBox["Harald H. Soleng. Tensors in Physics",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://store.wolfram.com/view/book/D0709.str"], None},
  ButtonNote->"https://store.wolfram.com/view/book/D0709.str"],
 ": CARTAN 1.8. December 28, 2011\n    Copyright \[Copyright] 2002-2011 by Ad \
Infinitum\n",
 ButtonBox["http://arxiv.org/abs/gr-qc/9502035",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://arxiv.org/abs/gr-qc/9502035"], None},
  ButtonNote->"http://arxiv.org/abs/gr-qc/9502035"]
}], "Text",
 CellChangeTimes->{{3.564386869358122*^9, 3.5643869116525407`*^9}, {
   3.5643869469485598`*^9, 3.564386981804553*^9}, {3.5684478444059258`*^9, 
   3.5684478444099255`*^9}, 3.669112264888083*^9, 3.67012615823658*^9, {
   3.67031137522351*^9, 3.670311387775066*^9}, 3.6703114279833317`*^9, {
   3.670311464277316*^9, 3.670311464279087*^9}, {3.67221008989155*^9, 
   3.672210089893211*^9}, 3.6738756588678207`*^9},
 CellTags->"Soleng",ExpressionUUID->"07049e92-b028-40ff-a664-c65723372cee"],

Cell[TextData[{
 "[EDG] R.Sulanke. An Interactive Textbook on Euclidean Differential \
Geometry. Version 1.  ",
 ButtonBox["Wolfram Library Archive 2015. ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://library.wolfram.com/infocenter/MathSource/9115/"], None},
  ButtonNote->"http://library.wolfram.com/infocenter/MathSource/9115/"],
 "\n",
 "New versions: ",
 ButtonBox["Introduction to Differential Geometry",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www-irm.mathematik.hu-berlin.de/~sulanke/diffgeo.html#\
IntroDG"], None},
  ButtonNote->
   "http://www-irm.mathematik.hu-berlin.de/~sulanke/diffgeo.html#IntroDG"],
 ". Section Downloads, EDG.zip."
}], "Text",
 CellChangeTimes->{{3.652339338223857*^9, 3.652339372999291*^9}, {
   3.6523395126215553`*^9, 3.652339544269264*^9}, {3.652339646387846*^9, 
   3.652339671212287*^9}, {3.6523398867694483`*^9, 3.652339894857216*^9}, {
   3.66911226480753*^9, 3.6691122648208513`*^9}, 3.6694467352766447`*^9, 
   3.670126151454809*^9, {3.706100823789588*^9, 3.706100837573846*^9}, {
   3.706100892238106*^9, 3.706100900406974*^9}, 3.706246283299902*^9, 
   3.706251361139742*^9, {3.70840500105009*^9, 3.708405010729574*^9}},
 CellTags->"EDG",ExpressionUUID->"4cddb376-10e3-4ed2-a93e-c692609c788c"],

Cell[TextData[{
 "Tutorial ",
 ButtonBox["Symbolic Tensors",
  BaseStyle->"Link",
  ButtonData->"paclet:tutorial/SymbolicTensors"]
}], "Text",
 CellChangeTimes->{{3.5701704833495164`*^9, 3.5701705055327854`*^9}},
 CellTags->"Tensors",ExpressionUUID->"8f055b0c-6a33-4b5e-9a2f-1896ad783681"],

Cell[TextData[{
 "Andrzej Woszczyna and others.  ccgrg - the symbolic tensor analysis \
package, with tools for general relativity.",
 ButtonBox[" Wolfram Library Archive. MathSource/8848.",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://library.wolfram.com/infocenter/MathSource/8848/"], None},
  ButtonNote->"http://library.wolfram.com/infocenter/MathSource/8848/"],
 "\n"
}], "Text",
 CellChangeTimes->{{3.672316825409959*^9, 3.6723168299771013`*^9}, {
  3.6723179861502037`*^9, 3.672318060365601*^9}, {3.672318112909089*^9, 
  3.672318179580916*^9}, {3.672318221596582*^9, 3.672318228076406*^9}, {
  3.672318263997683*^9, 3.6723182639998713`*^9}},
 CellTags->"ccgrg",ExpressionUUID->"bcec088b-3c67-4ac3-80fe-012dbf099f00"]
}, Closed]],

Cell[CellGroupData[{

Cell["Homepage", "Subtitle",
 CellChangeTimes->{
  3.445933589770261*^9, {3.4459337168429823`*^9, 3.445933720398094*^9}, 
   3.541836408394314*^9, 
   3.541836718195538*^9},ExpressionUUID->"01187eef-dc97-40c3-b2ea-\
8daa6aa6539d"],

Cell["\<\
The  newest versions and all my public Mathematica notebooks and packages can \
be downloaded from my\
\>", "Text",
 CellChangeTimes->{{3.70860369634648*^9, 3.7086037216903477`*^9}, {
  3.708603770346141*^9, 
  3.708603836529463*^9}},ExpressionUUID->"4a5970fe-41dd-4517-a508-\
eed629f1cefc"],

Cell[TextData[ButtonBox["http://www-irm.mathematik.hu-berlin.de/~sulanke",
 BaseStyle->"Hyperlink",
 ButtonData->{
   URL["http://www-irm.mathematik.hu-berlin.de/~sulanke"], None}]], "Text",
 CellChangeTimes->{{3.4459337369318686`*^9, 3.4459338102072334`*^9}, {
   3.4465638794375*^9, 3.4465638794375*^9}, 3.54183640839435*^9, 
   3.541836718195591*^9},
 CellTags->"Home",ExpressionUUID->"c7ecf6e1-c5e5-4aab-ae7b-974105bcc91c"],

Cell["\<\
I will be grateful for all feedback, in particular for hints to errors, or \
reports about problems, failures, or success at work with my packages or \
notebooks. Please write to\
\>", "Text",
 CellChangeTimes->{{3.708603868438356*^9, 3.708603908144747*^9}, {
  3.70860394762455*^9, 3.708604001719892*^9}, {3.708604105959395*^9, 
  3.708604113767239*^9}, {3.708604366173559*^9, 3.708604388661172*^9}, {
  3.708604431125073*^9, 3.708604473972559*^9}, {3.708604938713274*^9, 
  3.70860495133745*^9}, {3.708605007681033*^9, 
  3.708605038928814*^9}},ExpressionUUID->"bc072768-af5b-4c61-bd00-\
88ab0e68f38e"],

Cell["e-mail: sulanke@mathematik.hu-berlin.de", "Text",
 CellChangeTimes->{{3.618995363041403*^9, 3.618995394834259*^9}, {
  3.706245879750105*^9, 
  3.706245891166106*^9}},ExpressionUUID->"70179e99-2791-4430-9ac8-\
557d515121f6"]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{992, 647},
WindowMargins->{{4, Automatic}, {4, Automatic}},
PrintingCopies->1,
PrintingPageRange->{1, 32000},
PrintingOptions->{"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{594.75, 842.25},
"PostScriptOutputFile"->"",
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->False,
"PrintingMargins"->{{72.00000078000001, 72.00000078000001}, {54.000000585, \
54.000000585}}},
ShowSelection->True,
ShowCellTags->True,
Magnification:>1.5 Inherited,
FrontEndVersion->"11.1 for Linux x86 (64-bit) (April 18, 2017)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "defRsp"->{
  Cell[74246, 1627, 2246, 39, 
  7909, "Text", "ExpressionUUID" -> "95da0e7c-02d8-4123-8770-dd98e0202c16",
   CellTags->"defRsp"]},
 "rgpos"->{
  Cell[147282, 3584, 798, 18, 
  3250, "Input", "ExpressionUUID" -> "538ba085-94cb-4c7c-abf1-67e28fe5526b",
   Evaluatable->False,
   CellTags->"rgpos"],
  Cell[158501, 3900, 548, 8, 
  2254, "Input", "ExpressionUUID" -> "df75de80-ddb4-4456-a9ac-fdbae91eba86",
   CellTags->"rgpos"]},
 "transfcoords"->{
  Cell[526113, 13182, 238, 4, 
  2254, "Text", "ExpressionUUID" -> "9ebfee58-3354-49b5-a02b-11a7150a83ac",
   CellTags->"transfcoords"]},
 "gammatr"->{
  Cell[1012524, 26322, 1455, 36, 
  3837, "Input", "ExpressionUUID" -> "934eceb0-0287-4178-8b43-85613c87c2ef",
   Evaluatable->False,
   CellTags->"gammatr"]},
 "Rtens"->{
  Cell[1266750, 32985, 702, 14, 
  1533, "Text", "ExpressionUUID" -> "a89a289d-3039-4de5-bd14-d0772c1828b2",
   CellTags->"Rtens"]},
 "IDG"->{
  Cell[1821396, 47881, 776, 15, 
  72, "Text", "ExpressionUUID" -> "7faa1803-e9ac-4398-8bb1-84f3acca0425",
   CellTags->"IDG"]},
 "B-R"->{
  Cell[1822175, 47898, 525, 9, 
  105, "Text", "ExpressionUUID" -> "21df92d0-0d9b-47dd-9f83-0bd9b35930f9",
   CellTags->"B-R"]},
 "B-L"->{
  Cell[1822703, 47909, 408, 7, 
  105, "Text", "ExpressionUUID" -> "1f8500fc-8832-44fb-ae96-c5038fa453db",
   CellTags->"B-L"]},
 "Christ"->{
  Cell[1823114, 47918, 684, 14, 
  72, "Text", "ExpressionUUID" -> "0667904c-1335-41aa-be8f-da12f32b1340",
   CellTags->"Christ"]},
 "Gray"->{
  Cell[1824360, 47947, 715, 14, 
  137, "Text", "ExpressionUUID" -> "cc8232d6-5c7f-4187-ace0-7b06ffe46740",
   CellTags->"Gray"]},
 "Gray2"->{
  Cell[1825078, 47963, 999, 18, 
  137, "Text", "ExpressionUUID" -> "898a7ded-4561-4d52-b930-9f4af91f2b54",
   CellTags->"Gray2"]},
 "Gray3"->{
  Cell[1826080, 47983, 690, 11, 
  137, "Text", "ExpressionUUID" -> "c6706746-5ca2-46ac-9e8d-809a3ea97382",
   CellTags->"Gray3"]},
 "H78"->{
  Cell[1826773, 47996, 458, 9, 
  105, "Text", "ExpressionUUID" -> "0758737c-9bc5-44ba-a44c-6f7e319bf127",
   CellTags->"H78"]},
 "JMN"->{
  Cell[1827234, 48007, 448, 7, 
  105, "Text", "ExpressionUUID" -> "268e697c-a6d7-4db2-99be-89bde2586826",
   CellTags->"JMN"]},
 "K-N1"->{
  Cell[1827685, 48016, 413, 7, 
  105, "Text", "ExpressionUUID" -> "f4362455-c4bb-4a38-a039-80dd10285d0d",
   CellTags->"K-N1"]},
 "Kr"->{
  Cell[1828101, 48025, 397, 7, 
  105, "Text", "ExpressionUUID" -> "84379d9b-123f-495a-9ac6-282ae8d762cd",
   CellTags->"Kr"]},
 "Mal"->{
  Cell[1828501, 48034, 208, 4, 
  72, "Text", "ExpressionUUID" -> "e515a207-0109-4bdc-a391-6e6cc86c3d63",
   CellTags->"Mal"]},
 "xAct"->{
  Cell[1828712, 48040, 523, 12, 
  105, "Text", "ExpressionUUID" -> "0033d828-935c-4125-a93a-682f3fe8b07f",
   CellTags->"xAct"]},
 "OSI"->{
  Cell[1829238, 48054, 652, 11, 
  105, "Text", "ExpressionUUID" -> "3cdfef22-aea4-4a65-aa81-8bca3fd44206",
   CellTags->"OSI"]},
 "OSII"->{
  Cell[1829893, 48067, 519, 8, 
  105, "Text", "ExpressionUUID" -> "4ae8ad1d-c3c6-4659-88df-49dee0826979",
   CellTags->"OSII"]},
 "OSIII"->{
  Cell[1830415, 48077, 584, 10, 
  105, "Text", "ExpressionUUID" -> "d8d078d5-e367-4e9b-af60-df057e345174",
   CellTags->"OSIII"]},
 "Sch54"->{
  Cell[1831002, 48089, 369, 6, 
  105, "Text", "ExpressionUUID" -> "7e0f735d-2d39-4c01-8b29-f97ba112e151",
   CellTags->"Sch54"]},
 "Soleng"->{
  Cell[1831374, 48097, 1036, 21, 
  137, "Text", "ExpressionUUID" -> "07049e92-b028-40ff-a664-c65723372cee",
   CellTags->"Soleng"]},
 "EDG"->{
  Cell[1832413, 48120, 1270, 26, 
  137, "Text", "ExpressionUUID" -> "4cddb376-10e3-4ed2-a93e-c692609c788c",
   CellTags->"EDG"]},
 "Tensors"->{
  Cell[1833686, 48148, 289, 7, 
  72, "Text", "ExpressionUUID" -> "8f055b0c-6a33-4b5e-9a2f-1896ad783681",
   CellTags->"Tensors"]},
 "ccgrg"->{
  Cell[1833978, 48157, 737, 14, 
  137, "Text", "ExpressionUUID" -> "bcec088b-3c67-4ac3-80fe-012dbf099f00",
   CellTags->"ccgrg"]},
 "Home"->{
  Cell[1835289, 48192, 427, 7, 
  72, "Text", "ExpressionUUID" -> "c7ecf6e1-c5e5-4aab-ae7b-974105bcc91c",
   CellTags->"Home"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"defRsp", 1837347, 48247},
 {"rgpos", 1837491, 48251},
 {"transfcoords", 1837795, 48259},
 {"gammatr", 1837947, 48263},
 {"Rtens", 1838119, 48268},
 {"IDG", 1838262, 48272},
 {"B-R", 1838401, 48276},
 {"B-L", 1838540, 48280},
 {"Christ", 1838682, 48284},
 {"Gray", 1838825, 48288},
 {"Gray2", 1838968, 48292},
 {"Gray3", 1839112, 48296},
 {"H78", 1839254, 48300},
 {"JMN", 1839393, 48304},
 {"K-N1", 1839533, 48308},
 {"Kr", 1839672, 48312},
 {"Mal", 1839810, 48316},
 {"xAct", 1839949, 48320},
 {"OSI", 1840090, 48324},
 {"OSII", 1840231, 48328},
 {"OSIII", 1840373, 48332},
 {"Sch54", 1840517, 48336},
 {"Soleng", 1840661, 48340},
 {"EDG", 1840805, 48344},
 {"Tensors", 1840950, 48348},
 {"ccgrg", 1841094, 48352},
 {"Home", 1841237, 48356}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 346, 8, 394, "Title", "ExpressionUUID" -> \
"f245cfa7-6f2d-4053-bfb9-544cdb93418d"],
Cell[929, 32, 163, 2, 50, "Text", "ExpressionUUID" -> \
"e1d4ab09-06cd-4640-ad33-9103460785ed"],
Cell[1095, 36, 233, 4, 57, "Text", "ExpressionUUID" -> \
"31f51928-44c4-4750-bbfa-bea043be87b9"],
Cell[1331, 42, 1012, 18, 213, "Text", "ExpressionUUID" -> \
"e906996d-cb76-4e39-98f7-24de483b8a52"],
Cell[CellGroupData[{
Cell[2368, 64, 152, 3, 97, "Section", "ExpressionUUID" -> \
"7b87f206-691e-4a74-9ada-aa5d6daf4aee"],
Cell[2523, 69, 5041, 81, 700, "Text", "ExpressionUUID" -> \
"092495f2-d158-4942-85fc-9faf682756c7"],
Cell[7567, 152, 2934, 42, 196, "Text", "ExpressionUUID" -> \
"ff8a0b50-daad-4a8f-b2f1-a95d8b6cf590"],
Cell[10504, 196, 3383, 48, 409, "Text", "ExpressionUUID" -> \
"0d9221c7-1d1e-4201-8992-d8a9e21733ef"],
Cell[13890, 246, 3377, 48, 409, "Text", "ExpressionUUID" -> \
"2162a546-d69c-47fa-b1d9-6ef8657075bf"],
Cell[17270, 296, 3543, 51, 453, "Text", "ExpressionUUID" -> \
"f1db59fd-78da-4e26-84f5-100598a34761"],
Cell[20816, 349, 1359, 21, 367, "Text", "ExpressionUUID" -> \
"1a0e824a-c136-42b9-94ec-3384ef6c372d"],
Cell[22178, 372, 2918, 42, 153, "Text", "ExpressionUUID" -> \
"b3f80b45-87f1-446d-a368-4f594eec3dc5"],
Cell[CellGroupData[{
Cell[25121, 418, 205, 4, 96, "Subsection", "ExpressionUUID" -> \
"78c89644-4717-437a-98bb-478565d2b179"],
Cell[25329, 424, 3658, 66, 2400, "Output", "ExpressionUUID" -> \
"d189f472-d1e8-42f4-a379-e96d4931b127"],
Cell[CellGroupData[{
Cell[29012, 494, 160, 3, 75, "Subsubsection", "ExpressionUUID" -> \
"3a867856-9761-40de-b5a9-9e92f03e9daf"],
Cell[29175, 499, 4597, 73, 95985, "Input", "ExpressionUUID" -> \
"1479cf29-903f-43d7-8e93-08f5a499067f",
 InitializationCell->True],
Cell[33775, 574, 411, 7, 3573, "Input", "ExpressionUUID" -> \
"f32f98eb-b8f9-47b2-9799-4694e9d3f9ed"],
Cell[34189, 583, 221, 5, 3573, "Input", "ExpressionUUID" -> \
"7e9e469d-a1f4-449d-9150-73893e5032dd"]
}, Closed]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[34471, 595, 299, 5, 74, "Section", "ExpressionUUID" -> \
"9c233649-9d70-46cd-9739-be724a14842a"],
Cell[34773, 602, 780, 13, 213, "Text", "ExpressionUUID" -> \
"ee24a9fa-e3df-454e-a893-eeec2a0e794f"]
}, Closed]],
Cell[CellGroupData[{
Cell[35590, 620, 159, 3, 74, "Section", "ExpressionUUID" -> \
"640d0647-09af-4bda-b542-9ac0559f2824"],
Cell[35752, 625, 2563, 37, 2499, "Text", "ExpressionUUID" -> \
"28434613-4ecc-47a5-bc48-13cd99cb42a0"],
Cell[38318, 664, 586, 10, 3349, "Text", "ExpressionUUID" -> \
"8e157162-6c03-4082-8f18-ee903aa16911"],
Cell[38907, 676, 504, 9, 2499, "Text", "ExpressionUUID" -> \
"d94f51c1-2e28-4a81-b237-469bf98ad2f7"],
Cell[CellGroupData[{
Cell[39436, 689, 294, 5, 2109, "Subsection", "ExpressionUUID" -> \
"e9d3cdde-0e3a-44c2-b0ea-2c47039ded1c"],
Cell[39733, 696, 532, 10, 2431, "Text", "ExpressionUUID" -> \
"68fe155c-3183-41e8-8030-73fe8a4a50b7"],
Cell[40268, 708, 716, 13, 1464, "Input", "ExpressionUUID" -> \
"55b3da63-534d-49b6-80aa-11113072b5a9",
 Evaluatable->False,
 InitializationCell->True],
Cell[40987, 723, 206, 4, 1464, "Text", "ExpressionUUID" -> \
"69f184f2-515b-4538-af1c-acf2da5f7769"],
Cell[41196, 729, 210, 5, 1464, "Input", "ExpressionUUID" -> \
"512b4d3d-d104-42e7-9d6f-8e0cfc6a4e41",
 InitializationCell->True],
Cell[41409, 736, 853, 14, 1464, "Input", "ExpressionUUID" -> \
"73b46175-db8e-432d-bdb0-3b16f2ad23df",
 InitializationCell->True],
Cell[42265, 752, 664, 11, 3436, "Text", "ExpressionUUID" -> \
"6d86f52a-8ee5-4b72-af5f-c36d06953102"],
Cell[42932, 765, 86, 0, 1464, "Input", "ExpressionUUID" -> \
"c5fa054e-1c0e-4d80-b7e4-2f7455babc89"],
Cell[43021, 767, 1312, 22, 6385, "Text", "ExpressionUUID" -> \
"1ba463ec-8c37-406d-b954-1ad88a18cddf"],
Cell[44336, 791, 602, 10, 2499, "Text", "ExpressionUUID" -> \
"67c14754-a3c2-47be-8941-b2e449fc7cd1"],
Cell[44941, 803, 210, 3, 1464, "Text", "ExpressionUUID" -> \
"d28249fa-a763-4eaa-800c-10c51d58f5b4"]
}, Closed]],
Cell[CellGroupData[{
Cell[45188, 811, 221, 4, 1639, "Subsection", "ExpressionUUID" -> \
"2002252d-de38-4d9a-af4d-654011d91217"],
Cell[45412, 817, 309, 7, 2431, "Text", "ExpressionUUID" -> \
"0c045186-26b2-455b-94e6-ad823389187f"],
Cell[45724, 826, 685, 11, 1464, "Input", "ExpressionUUID" -> \
"4e55b106-846f-4a11-8a4b-bd0eed2553d1",
 InitializationCell->True],
Cell[46412, 839, 489, 9, 2431, "Text", "ExpressionUUID" -> \
"cd8194fa-1ada-4875-965b-e529088ac7d9"],
Cell[46904, 850, 484, 9, 2431, "Text", "ExpressionUUID" -> \
"1b749b2d-d4be-4d2f-ab47-124ba6f57e3d"],
Cell[47391, 861, 199, 4, 1464, "Input", "ExpressionUUID" -> \
"7c458d99-4217-4ad1-9770-204c102e3cc8"],
Cell[47593, 867, 249, 5, 1464, "Input", "ExpressionUUID" -> \
"a2222035-8284-4504-9441-b752c79e7c3d"],
Cell[47845, 874, 401, 7, 2431, "Text", "ExpressionUUID" -> \
"8edff557-e991-4985-897f-c730339b724e"],
Cell[48249, 883, 480, 10, 1464, "Input", "ExpressionUUID" -> \
"b8ef2a5a-230b-4106-9342-7f12370b8b84",
 InitializationCell->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[48778, 899, 225, 3, 74, "Section", "ExpressionUUID" -> \
"5514a4bb-f51d-4089-be08-0b8030a9340b"],
Cell[49006, 904, 649, 11, 83, "Text", "ExpressionUUID" -> \
"6c7490d8-d8d0-4e7c-850b-57881e9037b0"],
Cell[CellGroupData[{
Cell[49680, 919, 153, 3, 67, "Subsection", "ExpressionUUID" -> \
"3a45bfd3-bacd-4184-819a-ad8a5e6d7476"],
Cell[49836, 924, 536, 9, 8299, "Text", "ExpressionUUID" -> \
"a9e00951-3d5a-4751-b399-23afaac4aa08"],
Cell[50375, 935, 176, 4, 8037, "Input", "ExpressionUUID" -> \
"9f7bb773-ef8e-4a8a-8ffd-478c936396ea"],
Cell[50554, 941, 172, 4, 8037, "Input", "ExpressionUUID" -> \
"4cc5fe02-72a1-44f9-a945-c66779054e8e"],
Cell[50729, 947, 449, 10, 8299, "Text", "ExpressionUUID" -> \
"d008c924-d960-4663-90d5-95f081303a10"],
Cell[51181, 959, 168, 4, 8037, "Input", "ExpressionUUID" -> \
"80d87ada-d210-4527-a3f8-3777e239215e"],
Cell[51352, 965, 541, 15, 11083, "Text", "ExpressionUUID" -> \
"b1004939-087a-4602-9516-03be1f72f39f"],
Cell[51896, 982, 175, 4, 8037, "Input", "ExpressionUUID" -> \
"890c9506-4798-4842-b1f2-5bcc89b7c1cf"],
Cell[52074, 988, 636, 11, 8299, "Text", "ExpressionUUID" -> \
"80093576-2503-4816-bb10-e3f7962790b1"],
Cell[52713, 1001, 185, 4, 8037, "Input", "ExpressionUUID" -> \
"cc12674d-a261-47dd-8d20-f13755e10d56"],
Cell[52901, 1007, 234, 5, 8037, "Input", "ExpressionUUID" -> \
"fe16f2e6-2e46-43f0-a0a3-b5591c975454"],
Cell[53138, 1014, 236, 5, 8037, "Input", "ExpressionUUID" -> \
"c92854ba-754c-4d25-8597-b16e9db609ac"],
Cell[53377, 1021, 383, 8, 13768, "Text", "ExpressionUUID" -> \
"2ea5f129-4f27-4685-9ed6-fda2c120f84d"],
Cell[53763, 1031, 198, 3, 8299, "Text", "ExpressionUUID" -> \
"f4dab96b-3814-448e-acc5-687f68b81328"],
Cell[53964, 1036, 183, 4, 8037, "Input", "ExpressionUUID" -> \
"546c0b25-86c0-4f95-912f-39c51530f560"],
Cell[54150, 1042, 327, 5, 8299, "Text", "ExpressionUUID" -> \
"537a641c-41fc-44ef-bf00-92841a61031d"],
Cell[54480, 1049, 248, 5, 8037, "Input", "ExpressionUUID" -> \
"d4aaf5d2-e51f-4846-88fc-0a7b23b6736d"],
Cell[54731, 1056, 715, 12, 19110, "Text", "ExpressionUUID" -> \
"33ae1aad-a17a-4aa3-bf8f-7ae8c6d290e0"],
Cell[55449, 1070, 296, 6, 13768, "Text", "ExpressionUUID" -> \
"59b9f869-4840-40e3-b6fc-50907cf066f4"],
Cell[55748, 1078, 171, 4, 8037, "Input", "ExpressionUUID" -> \
"0c6c773c-14aa-49e3-a82d-c3805ad9b5d4"],
Cell[55922, 1084, 182, 4, 8037, "Input", "ExpressionUUID" -> \
"46478d2e-dbde-4f34-86ad-149162ce26e4"],
Cell[56107, 1090, 225, 5, 8037, "Input", "ExpressionUUID" -> \
"be503de2-51fc-4f73-a4e1-60c528badd9b"],
Cell[56335, 1097, 322, 8, 13768, "Text", "ExpressionUUID" -> \
"6bd8cef0-8a44-40b9-9dbd-726a4a051303"],
Cell[56660, 1107, 182, 4, 8037, "Input", "ExpressionUUID" -> \
"51798630-7a72-45ab-b99f-4b221506e836"]
}, Closed]],
Cell[CellGroupData[{
Cell[56879, 1116, 170, 3, 53, "Subsection", "ExpressionUUID" -> \
"9c1f3292-e616-46e9-9469-334cef4b062c"],
Cell[CellGroupData[{
Cell[57074, 1123, 267, 4, 53, "Subsubsection", "ExpressionUUID" -> \
"a05a83fa-1765-4bc5-916d-aeba5441df04"],
Cell[57344, 1129, 568, 11, 3349, "Text", "ExpressionUUID" -> \
"8c67220c-bd8a-488a-acf1-b99e0cc847fa"],
Cell[57915, 1142, 255, 6, 1464, "Input", "ExpressionUUID" -> \
"60b3295e-abdc-46e0-868d-f1814927fa93",
 InitializationCell->True],
Cell[58173, 1150, 205, 5, 1464, "Input", "ExpressionUUID" -> \
"ac3b1292-5917-44a1-b4f8-3aa54f4f62ef"],
Cell[58381, 1157, 198, 5, 1464, "Input", "ExpressionUUID" -> \
"cc877281-3ca5-408d-90b4-5adeb85085eb",
 InitializationCell->True],
Cell[58582, 1164, 449, 9, 13768, "Text", "ExpressionUUID" -> \
"562a137b-285f-417d-9c32-9d2c79c458c8"]
}, Closed]],
Cell[CellGroupData[{
Cell[59068, 1178, 295, 5, 42, "Subsubsection", "ExpressionUUID" -> \
"a1daffb9-3624-4369-9338-e5fe5108ba9a"],
Cell[59366, 1185, 576, 11, 7303, "Text", "ExpressionUUID" -> \
"afee2d48-505f-4a8b-8af7-b22512757069"],
Cell[59945, 1198, 173, 4, 2382, "Input", "ExpressionUUID" -> \
"f2348077-d15c-4b18-aca6-49bb727c097c"],
Cell[60121, 1204, 1097, 34, 6913, "Input", "ExpressionUUID" -> \
"1453cfc6-8bbc-4b10-999f-de20555f399d",
 InitializationCell->True],
Cell[61221, 1240, 376, 8, 11787, "Input", "ExpressionUUID" -> \
"a3cbe7de-b1f2-4ee9-aa00-5429224d1d1b",
 InitializationCell->True],
Cell[61600, 1250, 747, 24, 5272, "Input", "ExpressionUUID" -> \
"25307d71-9366-40b4-91fb-655b87303bce",
 InitializationCell->True],
Cell[62350, 1276, 350, 8, 10165, "Input", "ExpressionUUID" -> \
"840491e8-3592-45bd-951e-7fddf27a9151",
 InitializationCell->True],
Cell[62703, 1286, 2019, 68, 8524, "Input", "ExpressionUUID" -> \
"937a557a-614d-433d-be2b-22ed607f0dbb",
 InitializationCell->True],
Cell[64725, 1356, 429, 10, 13446, "Input", "ExpressionUUID" -> \
"52db5bf5-c8c9-44dd-a6d4-b5d178e2bbaf",
 InitializationCell->True],
Cell[65157, 1368, 1161, 35, 7041, "Input", "ExpressionUUID" -> \
"2eafcd5a-25ab-4a67-a8a1-ab7505abcb71",
 InitializationCell->True],
Cell[66321, 1405, 362, 9, 8524, "Input", "ExpressionUUID" -> \
"d691ca25-517b-4a94-97cc-d8826734d97f",
 InitializationCell->True],
Cell[66686, 1416, 312, 8, 2382, "Input", "ExpressionUUID" -> \
"805598d0-c135-4527-8a76-e5a4fd6db37d"]
}, Closed]],
Cell[CellGroupData[{
Cell[67035, 1429, 175, 3, 42, "Subsubsection", "ExpressionUUID" -> \
"277c8a40-5505-4565-8a9d-d185439778bd"],
Cell[67213, 1434, 234, 4, 8299, "Text", "ExpressionUUID" -> \
"af2255e3-3d37-4749-96f2-02cbce3f5d44"],
Cell[67450, 1440, 258, 6, 12049, "Input", "ExpressionUUID" -> \
"8091782b-8ec7-402d-b948-cdc0e2ea3ea8",
 InitializationCell->True],
Cell[67711, 1448, 197, 4, 8037, "Input", "ExpressionUUID" -> \
"e6c8bad7-9828-49e5-b0fe-242e63804845"]
}, Closed]],
Cell[CellGroupData[{
Cell[67945, 1457, 167, 3, 42, "Subsubsection", "ExpressionUUID" -> \
"5f67b2d7-c1d1-44f7-9ded-e903a5b063e3"],
Cell[68115, 1462, 283, 5, 18670, "Text", "ExpressionUUID" -> \
"c9aa5cc1-d3fb-479e-bf6d-91859599400d"],
Cell[68401, 1469, 336, 7, 27108, "Input", "ExpressionUUID" -> \
"6c787980-f460-4acc-b4f4-b9eec8570621",
 InitializationCell->True],
Cell[68740, 1478, 255, 6, 18075, "Input", "ExpressionUUID" -> \
"33e434f2-4b10-4073-b502-de5b26497592"],
Cell[68998, 1486, 194, 4, 18075, "Input", "ExpressionUUID" -> \
"038a3dc3-0660-4766-a782-81c3a4bbd438"]
}, Closed]],
Cell[CellGroupData[{
Cell[69229, 1495, 184, 3, 42, "Subsubsection", "ExpressionUUID" -> \
"4717da45-b0b8-4ed8-9afc-d46336401675"],
Cell[69416, 1500, 234, 4, 18670, "Text", "ExpressionUUID" -> \
"216ab05c-7349-4945-8dc3-27f925e663f1"],
Cell[69653, 1506, 361, 7, 27108, "Input", "ExpressionUUID" -> \
"2e10e346-7fbc-4a61-a393-7fbf8be17574",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[70051, 1518, 217, 4, 42, "Subsubsection", "ExpressionUUID" -> \
"d419db38-a77a-4402-92db-1691da4d2c6f"],
Cell[70271, 1524, 321, 7, 2147483647, "Text", "ExpressionUUID" -> \
"f7a5bbcd-8546-4aaa-af65-f6a24861be9b"],
Cell[70595, 1533, 186, 3, 2147483647, "Text", "ExpressionUUID" -> \
"68460d51-8e98-4775-9673-3c87de96ed52"],
Cell[70784, 1538, 179, 4, 2147483647, "Input", "ExpressionUUID" -> \
"fbfa0e09-6a10-4f36-9710-978c1eb6a626"],
Cell[70966, 1544, 160, 3, 2147483647, "Text", "ExpressionUUID" -> \
"2e2a45e5-5536-4692-a93a-499928fcecea"],
Cell[71129, 1549, 279, 5, 2147483647, "Input", "ExpressionUUID" -> \
"c6463c20-aecf-441a-a90d-4befb76dfa9e"]
}, Closed]],
Cell[CellGroupData[{
Cell[71445, 1559, 171, 3, 42, "Subsubsection", "ExpressionUUID" -> \
"04a037da-fcbb-4cc4-bf12-b18fe067cda0"],
Cell[71619, 1564, 411, 9, 2147483647, "Text", "ExpressionUUID" -> \
"b9668115-04c1-44f7-a2bb-a14b4d234e19"],
Cell[72033, 1575, 181, 4, 2147483647, "Input", "ExpressionUUID" -> \
"fca3fac9-89eb-4f19-b4d4-48a41b967751"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[72275, 1586, 561, 8, 97, "Section", "ExpressionUUID" -> \
"dd3f0dc2-9417-4d27-8eba-46ce977e86ae"],
Cell[72839, 1596, 545, 9, 148, "Text", "ExpressionUUID" -> \
"79bb2371-ee3a-439c-8dea-d1d1960146b7"],
Cell[CellGroupData[{
Cell[73409, 1609, 614, 9, 67, "Subsection", "ExpressionUUID" -> \
"441f0c9b-50c9-4b41-99af-07daf17bcea3"],
Cell[CellGroupData[{
Cell[74048, 1622, 195, 3, 53, "Subsubsection", "ExpressionUUID" -> \
"84d1382d-5539-4dc9-a1e1-8c01905d929c"],
Cell[74246, 1627, 2246, 39, 7909, "Text", "ExpressionUUID" -> \
"95da0e7c-02d8-4123-8770-dd98e0202c16",
 CellTags->"defRsp"],
Cell[76495, 1668, 857, 15, 2431, "Text", "ExpressionUUID" -> \
"378e3208-251f-4cf0-a705-c4a69ada3d94"],
Cell[77355, 1685, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"c0a4c9a7-3f4f-446e-a561-91805953babb"],
Cell[77531, 1691, 1026, 30, 4365, "Input", "ExpressionUUID" -> \
"8c37ab0a-2e8c-40bf-baae-8b16a141c639",
 Evaluatable->False],
Cell[78560, 1723, 326, 7, 1464, "Input", "ExpressionUUID" -> \
"db96acfd-88b6-4584-95b4-09d97e2ee34d"],
Cell[78889, 1732, 282, 7, 1464, "Text", "ExpressionUUID" -> \
"195f635b-b38e-42bc-a75c-cfbc7dbaa661"],
Cell[79174, 1741, 450, 13, 1464, "Input", "ExpressionUUID" -> \
"d29b7fa4-1dd1-4202-99e4-474c2d26b8f1"],
Cell[79627, 1756, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"3de5355a-158a-491b-9b5c-97af2cc30a9c"],
Cell[79817, 1762, 303, 7, 1503, "Text", "ExpressionUUID" -> \
"ec389161-d5b0-4f52-a245-b525966f7344"],
Cell[80123, 1771, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"be1550a9-6866-4f5c-a22f-189d7d5b26e9"],
Cell[80299, 1777, 578, 18, 2460, "Input", "ExpressionUUID" -> \
"9dcaa8ce-da6d-4f21-b53b-60f8e6de9ee0",
 Evaluatable->False],
Cell[80880, 1797, 152, 3, 1464, "Text", "ExpressionUUID" -> \
"1e8a26dd-5a76-46fd-b00f-5dd9d5697851"],
Cell[81035, 1802, 743, 18, 1464, "Input", "ExpressionUUID" -> \
"270aff66-2802-4124-b233-a59f4d1d775c"],
Cell[81781, 1822, 628, 13, 1464, "Input", "ExpressionUUID" -> \
"e1db6e81-0b90-442d-b260-4c395d38ba97"],
Cell[82412, 1837, 568, 12, 2499, "Text", "ExpressionUUID" -> \
"ab79f887-68a2-4cde-9e5e-1a99dded2433"],
Cell[82983, 1851, 263, 7, 1464, "Input", "ExpressionUUID" -> \
"50d91f22-6611-4b13-857b-664ce00654bb"],
Cell[83249, 1860, 597, 18, 1464, "Input", "ExpressionUUID" -> \
"a75f9fa0-d908-4ac8-b6b8-8dbb3b53e961"],
Cell[83849, 1880, 232, 7, 1464, "Input", "ExpressionUUID" -> \
"8317e8b8-1b91-433f-ab13-085fa6398e6f"],
Cell[84084, 1889, 208, 5, 1464, "Input", "ExpressionUUID" -> \
"5f179aaf-f11a-4ec9-9329-1f443ae904a4"],
Cell[84295, 1896, 480, 9, 1464, "Text", "ExpressionUUID" -> \
"7f35f241-a59d-44d3-8822-f2aaf880f78b"],
Cell[84778, 1907, 627, 17, 1464, "Input", "ExpressionUUID" -> \
"dc1b6662-213c-436f-a9c1-f15f3f03df84"],
Cell[85408, 1926, 184, 3, 1464, "Text", "ExpressionUUID" -> \
"4bdcac04-548c-43b1-9e54-01ad16330424"],
Cell[85595, 1931, 198, 4, 1464, "Input", "ExpressionUUID" -> \
"a6890570-58d5-45e1-b072-461a85f4473a"],
Cell[85796, 1937, 165, 3, 1464, "Text", "ExpressionUUID" -> \
"823829ee-4bdc-4379-8aec-586e40fedd81"],
Cell[85964, 1942, 539, 15, 1464, "Input", "ExpressionUUID" -> \
"4674f1d1-aeb5-4981-b47b-514d3a4d0dff"],
Cell[86506, 1959, 244, 6, 1464, "Input", "ExpressionUUID" -> \
"9d382ddd-57e0-4125-8324-c5d509031c1d"],
Cell[86753, 1967, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"8d52235b-333d-4b3b-ac1a-9e35817ec12e"],
Cell[86929, 1973, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"eba3e470-9d6d-4b31-84b0-ef84813de998"],
Cell[87105, 1979, 957, 19, 3349, "Text", "ExpressionUUID" -> \
"be9ca9b7-360d-4907-8551-09d52236d629"],
Cell[88065, 2000, 739, 20, 1464, "Input", "ExpressionUUID" -> \
"9bcd6162-47c5-4862-afda-18429ecd543a"],
Cell[88807, 2022, 891, 23, 1464, "Input", "ExpressionUUID" -> \
"2077a8e3-17d2-46d8-9f21-a4a9f97a76a3"],
Cell[89701, 2047, 172, 3, 1464, "Text", "ExpressionUUID" -> \
"41f47e45-fdc9-40f2-84ae-d57a8f63b134"],
Cell[89876, 2052, 372, 8, 1464, "Input", "ExpressionUUID" -> \
"ce215825-9caa-4422-9bd0-bda7dd06d039"],
Cell[90251, 2062, 195, 3, 1464, "Text", "ExpressionUUID" -> \
"25d8df9c-75e1-4b8e-8b07-1104ab64807e"],
Cell[90449, 2067, 595, 13, 1464, "Input", "ExpressionUUID" -> \
"27691efd-61b0-4c76-b738-456b53eafd65"],
Cell[91047, 2082, 170, 4, 1464, "Input", "ExpressionUUID" -> \
"9f4b534b-b74f-433c-9b85-1755c535c6eb"],
Cell[91220, 2088, 191, 4, 1464, "Input", "ExpressionUUID" -> \
"f1ebc9a0-2a97-47d2-8966-c027be40445d"],
Cell[91414, 2094, 633, 12, 1464, "Input", "ExpressionUUID" -> \
"ce1bbfcb-b8be-467e-874e-d294d648adf3"],
Cell[92050, 2108, 159, 3, 1239, "Text", "ExpressionUUID" -> \
"fc95dcbe-224b-44a8-abb9-ba0ab0260fc9"]
}, Closed]],
Cell[CellGroupData[{
Cell[92246, 2116, 265, 4, 42, "Subsubsection", "ExpressionUUID" -> \
"7ad4e423-5768-4761-8c2f-949c277a82d9"],
Cell[92514, 2122, 375, 8, 1464, "Text", "ExpressionUUID" -> \
"f5b02f9a-e2b1-41f1-9413-3fdd9fc36485"],
Cell[92892, 2132, 477, 12, 1464, "Input", "ExpressionUUID" -> \
"cafe88d7-6388-45d6-b402-b22d1310c3eb",
 InitializationCell->True],
Cell[93372, 2146, 555, 9, 1464, "Input", "ExpressionUUID" -> \
"aa3b89f4-829a-42e0-b08d-ba137e078a6d"],
Cell[93930, 2157, 233, 4, 1464, "Text", "ExpressionUUID" -> \
"d1ff09c1-5f79-44bf-a52c-70858a671e41"],
Cell[94166, 2163, 469, 10, 1464, "Input", "ExpressionUUID" -> \
"d0156e66-b19c-492c-9f8a-2c3dccba5cb9"],
Cell[94638, 2175, 496, 11, 2431, "Text", "ExpressionUUID" -> \
"e206bc09-db05-4491-b4c5-3bc4a8046f30"],
Cell[95137, 2188, 171, 4, 1464, "Input", "ExpressionUUID" -> \
"0d314787-2fe0-46d5-bcc7-a62d8c4ccf02"],
Cell[95311, 2194, 320, 7, 1464, "Input", "ExpressionUUID" -> \
"8e4292c8-3ad9-470e-affb-ad5cd9a534f0"],
Cell[95634, 2203, 191, 4, 1464, "Input", "ExpressionUUID" -> \
"be4dfcbc-60b5-41c4-ace5-6e16d8c2403e"],
Cell[95828, 2209, 488, 10, 3349, "Text", "ExpressionUUID" -> \
"146421fd-67fa-40f2-bab7-c190959ce6fb"],
Cell[96319, 2221, 558, 12, 1464, "Input", "ExpressionUUID" -> \
"385d2460-2f7e-4153-8fb2-923d63aecf2b"],
Cell[96880, 2235, 269, 4, 1464, "Text", "ExpressionUUID" -> \
"bbf0f0e4-af41-480a-8062-eb87173f5a2b"],
Cell[97152, 2241, 194, 4, 1464, "Input", "ExpressionUUID" -> \
"67b1f8a2-090e-4e29-bb1d-1cd6ab5dab79"],
Cell[97349, 2247, 512, 10, 1464, "Input", "ExpressionUUID" -> \
"02054863-a9c7-49a9-a4cd-74e56a914b08"],
Cell[97864, 2259, 331, 9, 1464, "Input", "ExpressionUUID" -> \
"a6017ee3-2dba-44ef-815e-05d94296a29a"],
Cell[98198, 2270, 895, 21, 3436, "Text", "ExpressionUUID" -> \
"77397915-ef5d-42c0-9b7e-079246c1ad8b"],
Cell[99096, 2293, 195, 5, 1464, "Input", "ExpressionUUID" -> \
"7759ac12-dafe-4fde-88d7-f769b74ab5fb"],
Cell[99294, 2300, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"4b5ce838-e0ff-4021-99d5-1570b26da0fd"],
Cell[99472, 2306, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"0a77dad5-1cbe-490d-9d12-2f510c58e9ff"],
Cell[99662, 2312, 762, 15, 1464, "Input", "ExpressionUUID" -> \
"6b386ecf-4ac1-442b-bcd1-f4b846e700fd"],
Cell[100427, 2329, 753, 19, 3349, "Text", "ExpressionUUID" -> \
"bd66aff0-f532-4260-81b9-a8e102ba34af"],
Cell[101183, 2350, 194, 4, 1464, "Input", "ExpressionUUID" -> \
"6fd40046-d8ea-422e-964c-685c37b00be8"],
Cell[101380, 2356, 266, 6, 1464, "Input", "ExpressionUUID" -> \
"6901e04f-c38f-490c-b2c4-3dea2f1e5ad4"],
Cell[101649, 2364, 1229, 29, 2460, "Input", "ExpressionUUID" -> \
"32a1e55d-4ddd-43aa-a07e-8bc94ab0cb55"],
Cell[102881, 2395, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"23745606-4aeb-41ca-a7e4-85f88f1d9412"],
Cell[103058, 2401, 190, 4, 1464, "Input", "ExpressionUUID" -> \
"6ae4f03b-ad59-4202-b628-bd0de546e24f"],
Cell[103251, 2407, 503, 10, 1464, "Input", "ExpressionUUID" -> \
"f1e54b7d-bb59-494e-9ec1-cbe523eb2d04"],
Cell[103757, 2419, 248, 6, 1464, "Input", "ExpressionUUID" -> \
"04c379ef-5376-43bf-a7d9-84e39c3950c8"],
Cell[104008, 2427, 528, 13, 1464, "Input", "ExpressionUUID" -> \
"c403d3b7-280b-46e5-a7d6-ca3d74f0d6e6"],
Cell[104539, 2442, 498, 11, 1503, "Text", "ExpressionUUID" -> \
"4522dbf9-373a-4292-9fcf-62bf4dd9f953"],
Cell[105040, 2455, 263, 7, 1464, "Input", "ExpressionUUID" -> \
"8aea0237-6785-4209-8e25-5ce14f0eeaeb"],
Cell[105306, 2464, 223, 5, 1464, "Input", "ExpressionUUID" -> \
"5b28fd98-745a-44d8-b7a9-639ffbb6b6f0"],
Cell[105532, 2471, 734, 20, 1464, "Input", "ExpressionUUID" -> \
"bc67166f-e3e0-4cf0-8a6b-7fcc4820e1a0",
 Evaluatable->False,
 InitializationCell->True],
Cell[106269, 2493, 345, 9, 1464, "Input", "ExpressionUUID" -> \
"68c04a67-0968-4ac9-924a-b0e7286d700e"],
Cell[106617, 2504, 373, 9, 1464, "Text", "ExpressionUUID" -> \
"a831f033-5d6e-4a01-9220-cef37e537211"],
Cell[106993, 2515, 171, 4, 1464, "Input", "ExpressionUUID" -> \
"f2875e0b-a841-4889-84a0-450cf43c083f"],
Cell[107167, 2521, 368, 8, 1464, "Input", "ExpressionUUID" -> \
"9dfd27ce-21cd-426a-991f-b2aef4cf25db"],
Cell[107538, 2531, 1106, 25, 1464, "Input", "ExpressionUUID" -> \
"bcb88286-88be-4d51-99fd-0dcae68e3f0c"],
Cell[108647, 2558, 157, 3, 1464, "Text", "ExpressionUUID" -> \
"38cf99cc-d25e-4a45-b4b4-c40cc029581e"],
Cell[108807, 2563, 170, 4, 1464, "Input", "ExpressionUUID" -> \
"0f2c9937-7c05-4e43-84df-6b64ba012046"],
Cell[108980, 2569, 446, 9, 1464, "Input", "ExpressionUUID" -> \
"803c6313-5df1-4911-90b7-296952e64b99"],
Cell[109429, 2580, 327, 8, 1464, "Input", "ExpressionUUID" -> \
"5fe97dde-3db7-4aa8-b4ec-2f4c16ccd70e"],
Cell[109759, 2590, 367, 7, 1464, "Text", "ExpressionUUID" -> \
"a6184300-a4f4-42d6-8d1f-664b644f5012"],
Cell[110129, 2599, 172, 4, 1464, "Input", "ExpressionUUID" -> \
"5256a8d4-84cd-4b84-9d2e-f1d8d99b1e73"],
Cell[110304, 2605, 647, 19, 3349, "Text", "ExpressionUUID" -> \
"f169b392-4076-46ce-aa05-fa6ce17c1390"]
}, Closed]],
Cell[CellGroupData[{
Cell[110988, 2629, 427, 6, 42, "Subsubsection", "ExpressionUUID" -> \
"ce2ba340-decb-4d39-9e9d-0b5560dbb4d7"],
Cell[111418, 2637, 522, 9, 2431, "Text", "ExpressionUUID" -> \
"f6de9f42-8517-4134-bce8-dac8a1dae453"],
Cell[111943, 2648, 1292, 25, 1639, "Input", "ExpressionUUID" -> \
"e8e0de0e-20b6-4342-94c0-d8d5da6919a3",
 Evaluatable->False],
Cell[113238, 2675, 551, 12, 1464, "Input", "ExpressionUUID" -> \
"46a35793-9a35-4854-a53c-ff95bd70434a"],
Cell[113792, 2689, 288, 8, 1503, "Text", "ExpressionUUID" -> \
"e9a73472-edae-42f0-9de6-b1ba7820e48c"],
Cell[114083, 2699, 239, 5, 1464, "Input", "ExpressionUUID" -> \
"108ff74f-0c6a-49a7-9671-8d99c5d124d7"],
Cell[114325, 2706, 294, 6, 1464, "Input", "ExpressionUUID" -> \
"43ca271b-6ded-4583-83d7-57447b518d91"],
Cell[114622, 2714, 153, 3, 1464, "Text", "ExpressionUUID" -> \
"fad61af5-c418-47d0-bd6a-45bbb40cb0ad"],
Cell[114778, 2719, 273, 7, 1464, "Input", "ExpressionUUID" -> \
"fcdff5f6-4e85-46c1-adbc-e229f2f6d817"],
Cell[115054, 2728, 460, 10, 2431, "Text", "ExpressionUUID" -> \
"a0643915-b27e-4e11-81e6-b8aacdacd9e2"],
Cell[115517, 2740, 231, 5, 1464, "Input", "ExpressionUUID" -> \
"02c8881b-29e2-4b42-b131-3f70f08c4652"],
Cell[115751, 2747, 169, 3, 1464, "Text", "ExpressionUUID" -> \
"fee4fcda-171e-4101-9cd0-3e3dec357cee"],
Cell[115923, 2752, 245, 5, 1464, "Input", "ExpressionUUID" -> \
"3657b87b-9609-49ac-97df-a5b195ecbe5a"],
Cell[116171, 2759, 1717, 41, 4365, "Input", "ExpressionUUID" -> \
"27e81bff-73b8-4883-a0d7-b97cfea30adc",
 Evaluatable->False],
Cell[117891, 2802, 226, 5, 1464, "Input", "ExpressionUUID" -> \
"c099d38d-b16b-419d-a88c-cb61d487f72f"],
Cell[118120, 2809, 518, 14, 1464, "Input", "ExpressionUUID" -> \
"570e6dc9-daa1-4efc-87db-20959866ed0a",
 Evaluatable->False],
Cell[118641, 2825, 662, 12, 1464, "Input", "ExpressionUUID" -> \
"e43c35ba-1ae7-41dc-8601-0a9213c78574"],
Cell[119306, 2839, 495, 11, 1464, "Input", "ExpressionUUID" -> \
"3e9de83a-dc4c-4ca7-8c78-c97d8fb81d6a"],
Cell[119804, 2852, 259, 7, 1464, "Input", "ExpressionUUID" -> \
"adfb14fb-50cb-4af6-bfcb-aafaf7da2a78"],
Cell[120066, 2861, 438, 10, 1464, "Input", "ExpressionUUID" -> \
"4f613020-0357-456b-93ca-fb5da2602645"],
Cell[120507, 2873, 1222, 21, 1464, "Input", "ExpressionUUID" -> \
"5596b70b-52c6-46e5-aa9e-374165faf5b3"],
Cell[121732, 2896, 499, 10, 1464, "Input", "ExpressionUUID" -> \
"ad9d171c-4218-4d62-9e1e-0797fb2f45d0"],
Cell[122234, 2908, 217, 5, 1464, "Input", "ExpressionUUID" -> \
"af117bd5-32c0-4b3a-bdd5-ffca210e4536"],
Cell[122454, 2915, 440, 7, 1464, "Text", "ExpressionUUID" -> \
"b693736d-a499-4ce7-bc55-ef21c7f6f1a4"]
}, Closed]],
Cell[CellGroupData[{
Cell[122931, 2927, 300, 5, 42, "Subsubsection", "ExpressionUUID" -> \
"cdb18208-8a2a-4b7e-aef2-cfb13a396b07"],
Cell[123234, 2934, 451, 10, 2431, "Text", "ExpressionUUID" -> \
"8b700843-b43b-454e-a793-7a45aaef8d50"],
Cell[123688, 2946, 202, 4, 1464, "Input", "ExpressionUUID" -> \
"a7435dfe-a71c-44aa-92ed-d33540671757"],
Cell[123893, 2952, 200, 4, 1464, "Input", "ExpressionUUID" -> \
"f7d19208-e8e9-4f94-b9bf-5c2db154d06d"],
Cell[124096, 2958, 488, 12, 1464, "Input", "ExpressionUUID" -> \
"3670dd9b-9e21-43a6-a546-4835d47e5048",
 Evaluatable->False,
 InitializationCell->True],
Cell[124587, 2972, 877, 16, 1464, "Input", "ExpressionUUID" -> \
"02fc6bb2-f4b3-4c69-82cc-307e0380e289",
 Evaluatable->False,
 InitializationCell->True],
Cell[125467, 2990, 794, 18, 1464, "Input", "ExpressionUUID" -> \
"99d118ec-8411-4966-9822-98e4b5156f97"],
Cell[126264, 3010, 279, 6, 1464, "Input", "ExpressionUUID" -> \
"406e17ad-8824-4428-9763-d3cbda822c41"],
Cell[126546, 3018, 208, 5, 1464, "Input", "ExpressionUUID" -> \
"5fb42b0f-fbf2-43e3-8211-18a34bfce625"],
Cell[126757, 3025, 235, 5, 1464, "Text", "ExpressionUUID" -> \
"9f3485e5-1478-429e-9dfe-951d9ec54bff"],
Cell[126995, 3032, 176, 4, 1464, "Input", "ExpressionUUID" -> \
"37b4c92e-d740-4bbc-b102-d4fecf9c3293"],
Cell[127174, 3038, 432, 10, 1464, "Input", "ExpressionUUID" -> \
"79c89278-01a6-4826-8a21-99910c8eb9d8"],
Cell[127609, 3050, 120, 3, 1444, "Item", "ExpressionUUID" -> \
"1eff9810-5f5c-4b70-a93a-df618f0fc71e"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[127778, 3059, 857, 15, 67, "Subsection", "ExpressionUUID" -> \
"01a311f9-759b-4a7f-b4c1-d49abfcb635e"],
Cell[128638, 3076, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"79def7db-7997-44c9-a61b-d532b701243c"],
Cell[128815, 3082, 207, 4, 1464, "Text", "ExpressionUUID" -> \
"3d01d160-4626-4649-abe7-3ed263ce6c98"],
Cell[129025, 3088, 779, 16, 1464, "Input", "ExpressionUUID" -> \
"b999c22f-0a4e-40ca-9554-cc6f8f2d47fb",
 Evaluatable->False,
 InitializationCell->True],
Cell[129807, 3106, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"f4bb5388-dc87-46b9-94e1-6e959059863d"],
Cell[129985, 3112, 193, 4, 1464, "Input", "ExpressionUUID" -> \
"cf909c2e-1a6a-45f0-9ce4-dc6305049b6a"],
Cell[130181, 3118, 570, 12, 1464, "Text", "ExpressionUUID" -> \
"19f454b7-57a9-451a-ad15-b29531279e0a"],
Cell[130754, 3132, 235, 6, 1464, "Input", "ExpressionUUID" -> \
"d64fedfa-5c0f-49ed-b226-afa7ffa26702"],
Cell[130992, 3140, 459, 10, 1464, "Input", "ExpressionUUID" -> \
"be1b28f0-366c-4e64-8ed1-bcfdf9a0d5fc"],
Cell[131454, 3152, 628, 19, 1639, "Input", "ExpressionUUID" -> \
"02a876b5-26d1-4fd1-99d0-1cefeff5c6ce"],
Cell[132085, 3173, 172, 4, 1464, "Input", "ExpressionUUID" -> \
"aa72aad6-0815-4339-9359-c7a1522e6043"],
Cell[132260, 3179, 322, 8, 1464, "Input", "ExpressionUUID" -> \
"706c4a2e-bec0-4e8c-86c4-664f67cfd1d1"],
Cell[132585, 3189, 303, 8, 1464, "Input", "ExpressionUUID" -> \
"d696339e-16aa-4f9d-a088-12a349f3b02a"],
Cell[132891, 3199, 223, 5, 1464, "Input", "ExpressionUUID" -> \
"fca73f55-1cc0-488f-bda0-f541ec48ad4d"],
Cell[133117, 3206, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"8e6351bf-285d-4e95-85ac-4ff654d2b231"],
Cell[133294, 3212, 895, 21, 1464, "Input", "ExpressionUUID" -> \
"fad5713d-9dfd-42df-9150-a0fb9a2c73f7",
 InitializationCell->True],
Cell[134192, 3235, 752, 15, 1464, "Input", "ExpressionUUID" -> \
"2d5bec5d-457c-4d17-88b9-41973afdd130"],
Cell[134947, 3252, 375, 10, 1464, "Input", "ExpressionUUID" -> \
"780dcf04-f410-47ee-8cbe-b1e30d6aa6f6"],
Cell[135325, 3264, 224, 5, 1464, "Input", "ExpressionUUID" -> \
"33ecf0d5-aeee-4486-a3db-a8d4926bfb86"],
Cell[135552, 3271, 971, 21, 1464, "Input", "ExpressionUUID" -> \
"5799f2e3-a6dd-4984-a461-4c3a8521e5ca"],
Cell[136526, 3294, 194, 4, 1464, "Input", "ExpressionUUID" -> \
"ed785b61-4611-4e8c-b517-8510f14a6a70"],
Cell[136723, 3300, 494, 14, 1464, "Input", "ExpressionUUID" -> \
"e5f3358b-cf41-4cc2-a180-d5cc7ad4deca"],
Cell[137220, 3316, 413, 10, 3349, "Text", "ExpressionUUID" -> \
"ba343d45-e598-47b4-b620-4a8d27cfb17b"],
Cell[137636, 3328, 494, 9, 1464, "Input", "ExpressionUUID" -> \
"0c5ea8e9-683d-40ac-b9c5-fae6fdb23ad3"],
Cell[138133, 3339, 509, 9, 1464, "Input", "ExpressionUUID" -> \
"25b6dbf2-b768-4ebc-931b-60c1b43401dc"],
Cell[138645, 3350, 677, 17, 1464, "Input", "ExpressionUUID" -> \
"ca544bf8-76be-4114-ba20-fc104977c5a6"],
Cell[139325, 3369, 604, 17, 1464, "Input", "ExpressionUUID" -> \
"8dffc159-8d45-400d-9fee-fd3501125097"],
Cell[139932, 3388, 180, 4, 1464, "Input", "ExpressionUUID" -> \
"e510a47a-7662-4673-bf7d-ef5d2ca51d2c"],
Cell[140115, 3394, 214, 5, 1464, "Text", "ExpressionUUID" -> \
"5029235b-58c6-4fd3-9c34-35e04aa4c436"],
Cell[140332, 3401, 1102, 23, 1464, "Input", "ExpressionUUID" -> \
"6e46d93e-76bd-44ae-ab26-57486277e304",
 Evaluatable->False,
 InitializationCell->True],
Cell[141437, 3426, 472, 8, 1464, "Input", "ExpressionUUID" -> \
"e28295eb-4fc0-4c2c-b919-044be0087def"],
Cell[141912, 3436, 208, 5, 1464, "Input", "ExpressionUUID" -> \
"a1a89896-09f7-41b1-8385-f14b7793ee20"],
Cell[142123, 3443, 348, 7, 1464, "Input", "ExpressionUUID" -> \
"171a7027-6043-450f-88de-6d145fd8a4cd"],
Cell[142474, 3452, 275, 8, 1464, "Input", "ExpressionUUID" -> \
"142a328c-2682-4052-8b45-7c93c7e9a36d"],
Cell[142752, 3462, 338, 9, 1464, "Input", "ExpressionUUID" -> \
"49ae794e-22e2-4d00-b79b-7edfdb479973"],
Cell[143093, 3473, 427, 10, 1464, "Input", "ExpressionUUID" -> \
"e78d32a2-20c6-4ea7-9b0f-1872b9e9e710"],
Cell[143523, 3485, 487, 11, 1464, "Input", "ExpressionUUID" -> \
"3430de38-afad-4a04-aade-9e8cdc97b5a9"],
Cell[144013, 3498, 342, 9, 1464, "Input", "ExpressionUUID" -> \
"d4502f79-951e-4d70-8a87-70a0bbaed379"],
Cell[144358, 3509, 445, 9, 4404, "Text", "ExpressionUUID" -> \
"640fe3ae-d063-4a89-8b6f-6f9dcd06dac3"],
Cell[144806, 3520, 1653, 45, 3427, "Input", "ExpressionUUID" -> \
"58cef54b-7b77-4d86-bbfe-7f5eecdf8976",
 Evaluatable->False],
Cell[146462, 3567, 579, 9, 1464, "Input", "ExpressionUUID" -> \
"d0b621ed-022e-4f36-b4b7-20eea4b3de01"],
Cell[147044, 3578, 235, 4, 1464, "Text", "ExpressionUUID" -> \
"047b97cb-4f30-4caf-99e1-6b42e2e58403"],
Cell[147282, 3584, 798, 18, 3250, "Input", "ExpressionUUID" -> \
"538ba085-94cb-4c7c-abf1-67e28fe5526b",
 Evaluatable->False,
 CellTags->"rgpos"],
Cell[148083, 3604, 319, 6, 1464, "Input", "ExpressionUUID" -> \
"e2f75c0c-d089-4c14-9634-4baa0f93ba1e"],
Cell[148405, 3612, 193, 4, 1464, "Input", "ExpressionUUID" -> \
"b8e909d0-67a1-4369-9bef-f2830686c8b0"],
Cell[148601, 3618, 374, 9, 1464, "Input", "ExpressionUUID" -> \
"3a127bfd-304b-424f-9c83-ee009862f710"],
Cell[148978, 3629, 356, 8, 1464, "Input", "ExpressionUUID" -> \
"15ffae13-b35e-4969-a60a-12dadbd19275"],
Cell[149337, 3639, 501, 9, 3349, "Text", "ExpressionUUID" -> \
"e47159ae-b164-42bf-b70e-f17cc1534d96"],
Cell[149841, 3650, 531, 15, 2460, "Input", "ExpressionUUID" -> \
"e167f2f6-4cb9-4d1f-9f26-49d90d391fd4"],
Cell[150375, 3667, 188, 3, 1503, "Text", "ExpressionUUID" -> \
"ca7c967d-3e1f-4e53-8b2e-10fffadc57f4"],
Cell[150566, 3672, 331, 9, 1464, "Input", "ExpressionUUID" -> \
"57369869-d626-46e1-be37-eb2f37888356"],
Cell[150900, 3683, 164, 3, 1464, "Text", "ExpressionUUID" -> \
"7ec08ec5-4b2e-4a95-8192-19be04368f37"],
Cell[151067, 3688, 299, 8, 1464, "Input", "ExpressionUUID" -> \
"e186045b-02bf-4eae-9be2-c7159c78825e"],
Cell[151369, 3698, 172, 3, 1464, "Text", "ExpressionUUID" -> \
"757b68a9-0250-4521-ab6c-39cb0ba53415"],
Cell[151544, 3703, 838, 20, 2460, "Input", "ExpressionUUID" -> \
"735a2445-eb41-4759-89ae-b3b6673b06d8"],
Cell[152385, 3725, 427, 9, 2499, "Text", "ExpressionUUID" -> \
"ae2eb508-22e6-47ff-83b9-49564b41146c"],
Cell[152815, 3736, 257, 6, 1464, "Input", "ExpressionUUID" -> \
"003a3a4a-feb9-4d09-998a-f02b9bb685eb"],
Cell[153075, 3744, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"739d1c10-f5a3-4dfe-ad50-7e33ed816c3c"],
Cell[153252, 3750, 188, 4, 1464, "Input", "ExpressionUUID" -> \
"605ba5de-9491-44e9-829f-8bdcf18eed04"],
Cell[153443, 3756, 650, 14, 1464, "Input", "ExpressionUUID" -> \
"122a60f1-012a-4764-a4f9-3c48f6037e03",
 Evaluatable->False],
Cell[154096, 3772, 235, 6, 1464, "Input", "ExpressionUUID" -> \
"4d4d8e3b-eb4b-4a36-9407-82dc59782678"],
Cell[154334, 3780, 212, 5, 1464, "Input", "ExpressionUUID" -> \
"b2d68867-08e3-4b4a-adf3-fb9f1736650e"],
Cell[154549, 3787, 759, 18, 2460, "Input", "ExpressionUUID" -> \
"ef02f9d5-a6c5-49d9-9425-a0fb5832acd1"],
Cell[155311, 3807, 331, 8, 2431, "Text", "ExpressionUUID" -> \
"78214f63-ff6e-41ab-8d8a-002a310f3ab8"],
Cell[155645, 3817, 407, 11, 1464, "Input", "ExpressionUUID" -> \
"14a83bc8-fe6d-47e2-a479-dc482ac8e88e"],
Cell[156055, 3830, 400, 10, 1464, "Input", "ExpressionUUID" -> \
"9e14719d-aaf2-4717-b7a2-c4ae414ff45e"],
Cell[156458, 3842, 547, 12, 3349, "Text", "ExpressionUUID" -> \
"5468ad8f-8651-40ab-bd0a-bbaa002eef4a"],
Cell[157008, 3856, 901, 24, 1464, "Input", "ExpressionUUID" -> \
"f0d84094-63ea-46ac-b2a7-7d91efdfea5b",
 Evaluatable->False],
Cell[157912, 3882, 586, 16, 1503, "Text", "ExpressionUUID" -> \
"f6c1c216-45b0-415e-97e8-7e002887a59c"],
Cell[158501, 3900, 548, 8, 2254, "Input", "ExpressionUUID" -> \
"df75de80-ddb4-4456-a9ac-fdbae91eba86",
 CellTags->"rgpos"],
Cell[159052, 3910, 247, 6, 1464, "Input", "ExpressionUUID" -> \
"2cc6bc13-68fa-4d2b-bce9-47f14bc5b0f5"],
Cell[159302, 3918, 205, 3, 1464, "Text", "ExpressionUUID" -> \
"5f7b1ee8-ba06-4583-8a40-e2547ad17773"],
Cell[159510, 3923, 367, 8, 1464, "Input", "ExpressionUUID" -> \
"8f8e8981-482b-46f6-901d-5ca88efc1303"]
}, Closed]],
Cell[CellGroupData[{
Cell[159914, 3936, 389, 7, 53, "Subsection", "ExpressionUUID" -> \
"e1f1d7ac-4fd3-496e-b154-d2377bcfea2f"],
Cell[CellGroupData[{
Cell[160328, 3947, 700, 12, 53, "Subsubsection", "ExpressionUUID" -> \
"978b4fea-2cc3-4702-85c0-717f04191d88"],
Cell[161031, 3961, 868, 18, 3349, "Text", "ExpressionUUID" -> \
"c15f0098-2c3d-4531-8ece-8a5b2679b6da"],
Cell[161902, 3981, 227, 5, 1464, "Input", "ExpressionUUID" -> \
"653f5140-44c7-4577-9b67-8e67950d1c71"],
Cell[162132, 3988, 268, 7, 1464, "Text", "ExpressionUUID" -> \
"27b51963-d5e0-4f6a-b9ae-7f2776b0007a"],
Cell[162403, 3997, 401, 11, 1464, "Input", "ExpressionUUID" -> \
"01e0011c-aa2a-4d23-abfc-c7c58e3f74f3"],
Cell[162807, 4010, 996, 27, 2460, "Input", "ExpressionUUID" -> \
"ae15a7ca-9627-42c2-be07-e99b7cbe1918"],
Cell[163806, 4039, 616, 11, 3349, "Text", "ExpressionUUID" -> \
"bbf64666-1533-4bcd-9ce2-85e34935c486"],
Cell[164425, 4052, 3133, 72, 4365, "Input", "ExpressionUUID" -> \
"44235e6e-2ee7-455e-b368-0a7ec30510e5",
 Evaluatable->False,
 InitializationCell->True],
Cell[167561, 4126, 352, 6, 1464, "Input", "ExpressionUUID" -> \
"5d3c57cf-16a6-490a-bb1f-8c7550a0b78f"],
Cell[167916, 4134, 378, 9, 1464, "Input", "ExpressionUUID" -> \
"5b441746-a001-48b9-a3fd-666bfae576a7"],
Cell[168297, 4145, 245, 6, 1464, "Input", "ExpressionUUID" -> \
"758c8b06-bd15-493d-a35c-c2c58974862a"],
Cell[168545, 4153, 334, 11, 1464, "Text", "ExpressionUUID" -> \
"e7313dbe-8cf0-48bb-b9dc-32fa4dd96a48"],
Cell[168882, 4166, 712, 18, 1464, "Input", "ExpressionUUID" -> \
"23f6d40a-df40-41a4-b341-3c5d20f12065"],
Cell[169597, 4186, 277, 8, 1464, "Text", "ExpressionUUID" -> \
"3651f5c8-931f-4f27-8181-0e1fa9ecf6c6"],
Cell[169877, 4196, 691, 17, 1464, "Input", "ExpressionUUID" -> \
"0c1f8ec5-3f8d-4e63-bdcf-4115fcbd5a4d"],
Cell[170571, 4215, 719, 13, 1503, "Text", "ExpressionUUID" -> \
"ccc3c961-e0d8-4843-9711-57eb7ce04ab5"],
Cell[171293, 4230, 237, 6, 1464, "Input", "ExpressionUUID" -> \
"cafe0ba5-f35b-4f74-99e0-4e6e9a3d01e4"],
Cell[171533, 4238, 195, 5, 1464, "Input", "ExpressionUUID" -> \
"9d02c2df-0a40-41ec-b892-baef8302ed9f"],
Cell[171731, 4245, 429, 10, 1464, "Input", "ExpressionUUID" -> \
"e6135231-5cce-4adb-b244-9e03cc586a51"],
Cell[172163, 4257, 548, 9, 1464, "Text", "ExpressionUUID" -> \
"d136f692-e4bc-43c0-861e-ef70a1362711"],
Cell[172714, 4268, 417, 11, 1464, "Input", "ExpressionUUID" -> \
"b2481e1f-96c3-4801-91ea-478e8c975658"],
Cell[173134, 4281, 362, 7, 1464, "Text", "ExpressionUUID" -> \
"afb566c6-26df-46e0-80c3-6ccef413be3a"],
Cell[173499, 4290, 514, 10, 1464, "Input", "ExpressionUUID" -> \
"933b153b-a6dd-4bc4-beb3-6f6b9f559376"],
Cell[174016, 4302, 638, 15, 1464, "Input", "ExpressionUUID" -> \
"5d4cc8cd-3607-4a42-aad5-1fb899255467"],
Cell[174657, 4319, 796, 13, 4306, "Text", "ExpressionUUID" -> \
"9f4f0194-a1d8-4593-9649-706ea9352240"],
Cell[175456, 4334, 2147, 51, 5331, "Input", "ExpressionUUID" -> \
"5e8f257b-e0d6-404e-b7e0-aff5f4389aba",
 Evaluatable->False],
Cell[177606, 4387, 225, 5, 1464, "Input", "ExpressionUUID" -> \
"e504eb73-470c-459f-af76-1125a4c859a5"],
Cell[177834, 4394, 289, 5, 1464, "Input", "ExpressionUUID" -> \
"d0c14470-8d8c-4a99-9960-ca12570927da"],
Cell[178126, 4401, 260, 4, 1464, "Text", "ExpressionUUID" -> \
"e90df04f-45cc-4962-9815-d2aa4e2bd7fa"],
Cell[178389, 4407, 771, 18, 1464, "Input", "ExpressionUUID" -> \
"5d1a645c-882f-4dc3-831d-dd5323ace927",
 Evaluatable->False],
Cell[179163, 4427, 178, 4, 1464, "Input", "ExpressionUUID" -> \
"17849d3e-cb1f-4c71-8e61-a96773e17f90"],
Cell[179344, 4433, 347, 8, 1464, "Input", "ExpressionUUID" -> \
"5a3e5029-9cbf-4aa7-a2f6-181b81648c98"],
Cell[179694, 4443, 424, 9, 1464, "Input", "ExpressionUUID" -> \
"f99be15a-358c-48b5-b305-6dc4babef287"],
Cell[180121, 4454, 492, 10, 1464, "Input", "ExpressionUUID" -> \
"aa9069be-eb3f-424c-9ddd-b550c7f2e0af"],
Cell[180616, 4466, 567, 11, 1464, "Input", "ExpressionUUID" -> \
"9e7b7c75-c3ed-4b38-add9-c5cd8d0fe869"],
Cell[181186, 4479, 319, 8, 1464, "Input", "ExpressionUUID" -> \
"16a5725b-0597-4d8a-8cdc-0bdf2984deb5"],
Cell[181508, 4489, 461, 12, 1464, "Input", "ExpressionUUID" -> \
"40a3f313-2a16-4433-aebe-6ba209a03fa2"],
Cell[181972, 4503, 523, 10, 2499, "Text", "ExpressionUUID" -> \
"c3201a21-40da-47a7-b814-708bf9f4164d"]
}, Closed]],
Cell[CellGroupData[{
Cell[182532, 4518, 832, 12, 42, "Subsubsection", "ExpressionUUID" -> \
"3b370f27-9736-43f5-ac66-de5d79c402ec"],
Cell[183367, 4532, 878, 14, 3349, "Text", "ExpressionUUID" -> \
"88c81b18-cf5f-4008-bc16-fd134fcd66f1"],
Cell[184248, 4548, 2920, 57, 2460, "Input", "ExpressionUUID" -> \
"c538fb76-5bfb-4468-b536-cd48f808ec76",
 Evaluatable->False],
Cell[187171, 4607, 425, 7, 1464, "Input", "ExpressionUUID" -> \
"467e01ad-79e3-4e7e-9b2c-69eafbd3a791"],
Cell[187599, 4616, 390, 9, 1464, "Text", "ExpressionUUID" -> \
"29468eb8-ef2b-42c6-81d1-cc3cfec31b2e"],
Cell[187992, 4627, 193, 4, 1464, "Input", "ExpressionUUID" -> \
"c7c00d2f-a68a-4f5b-9091-e8e5eca1abe0"],
Cell[188188, 4633, 214, 5, 1464, "Input", "ExpressionUUID" -> \
"775b6d18-0dbf-41f9-81ba-69740f75f03e"],
Cell[188405, 4640, 542, 11, 3349, "Text", "ExpressionUUID" -> \
"6b92b38e-bed4-4bb7-a69b-2437d3ffeb23"],
Cell[188950, 4653, 544, 12, 1464, "Input", "ExpressionUUID" -> \
"5d7fed49-27e2-486a-92ea-0c5a6a81728d"],
Cell[189497, 4667, 362, 7, 1464, "Text", "ExpressionUUID" -> \
"7c9fd821-c5f1-4e35-a9e6-e83681c5f962"],
Cell[189862, 4676, 379, 9, 1464, "Input", "ExpressionUUID" -> \
"b898c814-8354-42b0-b453-c3cdd42c890b"],
Cell[190244, 4687, 245, 6, 1464, "Input", "ExpressionUUID" -> \
"f8ff48c0-a2fe-414c-92d9-f1ce0751a046"],
Cell[190492, 4695, 556, 14, 2431, "Text", "ExpressionUUID" -> \
"e29ed14a-ab98-4af7-bb34-430bb4e357cd"],
Cell[191051, 4711, 710, 18, 1464, "Input", "ExpressionUUID" -> \
"7d85b310-1ee8-4d0a-b604-5c7f35ad1049"],
Cell[191764, 4731, 561, 16, 1464, "Input", "ExpressionUUID" -> \
"81de89f4-0e41-44b0-94ca-2df5c1364523"],
Cell[192328, 4749, 226, 5, 1464, "Input", "ExpressionUUID" -> \
"b799f208-a64c-4433-8d96-5e195410b4cd"],
Cell[192557, 4756, 805, 19, 1464, "Input", "ExpressionUUID" -> \
"7da509b0-2620-4f97-8cb9-4a4fb8bc522a"],
Cell[193365, 4777, 739, 17, 1464, "Input", "ExpressionUUID" -> \
"f093ab04-50d7-4387-b2b5-e97abd965d59"],
Cell[194107, 4796, 741, 17, 1464, "Input", "ExpressionUUID" -> \
"2ef60533-a995-42e4-8ebe-60bdb5bc17ef"],
Cell[194851, 4815, 883, 21, 2460, "Input", "ExpressionUUID" -> \
"24989352-766a-4403-8d4c-5e87cb002992"],
Cell[195737, 4838, 905, 14, 5224, "Text", "ExpressionUUID" -> \
"98c8b6bf-614e-4f1c-a0e7-3b02fe4e3756"],
Cell[196645, 4854, 2201, 51, 5331, "Input", "ExpressionUUID" -> \
"1e7fcd78-8a67-49f2-87b4-807d69c312a9",
 Evaluatable->False],
Cell[198849, 4907, 177, 4, 1464, "Input", "ExpressionUUID" -> \
"2a1e5ca9-503a-4a20-851e-dfb5e726d15f"],
Cell[199029, 4913, 258, 4, 1464, "Text", "ExpressionUUID" -> \
"d0307092-8324-4fef-bbb1-f3a9b2563174"],
Cell[199290, 4919, 177, 4, 1464, "Input", "ExpressionUUID" -> \
"df11a945-78a1-4653-865e-86c450f73c3e"],
Cell[199470, 4925, 822, 19, 1464, "Input", "ExpressionUUID" -> \
"d537a068-f01a-422e-878c-c4f88f35373d",
 Evaluatable->False],
Cell[200295, 4946, 374, 9, 1464, "Input", "ExpressionUUID" -> \
"cc242382-eb48-4219-b5a0-9b07073fc4f6"],
Cell[200672, 4957, 443, 8, 1464, "Input", "ExpressionUUID" -> \
"d439d34a-f036-4940-9020-ac18efb6f46a"],
Cell[201118, 4967, 515, 10, 1464, "Input", "ExpressionUUID" -> \
"e417010c-ce44-4b0b-ac79-10feed6ef13d"],
Cell[201636, 4979, 594, 12, 1464, "Input", "ExpressionUUID" -> \
"58a2c136-08a6-4352-8059-499db85bec67"],
Cell[202233, 4993, 1422, 31, 4432, "Text", "ExpressionUUID" -> \
"f76f3999-b47d-493f-9c79-c7f8be256a2e"],
Cell[203658, 5026, 670, 12, 3349, "Text", "ExpressionUUID" -> \
"99d1edcc-ea58-4c1b-8c5e-9473f9fe9ac4"],
Cell[204331, 5040, 579, 12, 1464, "Input", "ExpressionUUID" -> \
"8407a6d2-dc7d-4fc8-a071-140cd0b38eef"],
Cell[204913, 5054, 641, 12, 1464, "Input", "ExpressionUUID" -> \
"6915cbde-4325-47fb-898b-bef906922c66"],
Cell[205557, 5068, 829, 24, 2460, "Input", "ExpressionUUID" -> \
"ef180ff5-a362-4df6-8076-3d1cb286d556",
 Evaluatable->False,
 InitializationCell->True],
Cell[206389, 5094, 530, 9, 1464, "Input", "ExpressionUUID" -> \
"610548f7-30e3-4aa6-a268-002574233537"],
Cell[206922, 5105, 267, 6, 1464, "Input", "ExpressionUUID" -> \
"846e93bd-e2be-4886-980b-83951ca4ab94"],
Cell[207192, 5113, 337, 7, 1464, "Input", "ExpressionUUID" -> \
"78aa0d09-2da9-4b02-bca9-a5f1a817eb2b"],
Cell[207532, 5122, 709, 16, 1464, "Input", "ExpressionUUID" -> \
"5f8d9a17-4b00-444e-b5f1-cfd39e005b6f"],
Cell[208244, 5140, 259, 4, 1464, "Text", "ExpressionUUID" -> \
"cfba0477-6399-42e1-b777-9f1409e33aff"],
Cell[208506, 5146, 740, 18, 1464, "Input", "ExpressionUUID" -> \
"14f9a960-d255-4ee7-b12f-67a5c5cfa241",
 Evaluatable->False,
 InitializationCell->True],
Cell[209249, 5166, 341, 6, 1464, "Input", "ExpressionUUID" -> \
"45546446-9c86-4d6c-98dd-11a510b7474e"],
Cell[209593, 5174, 314, 8, 1464, "Input", "ExpressionUUID" -> \
"398d5116-45a7-43f0-8695-7c0b0daa3d6e"],
Cell[209910, 5184, 282, 6, 1464, "Input", "ExpressionUUID" -> \
"7807709c-52c8-4f32-bb54-2a7370db2fb3"],
Cell[210195, 5192, 441, 9, 1464, "Input", "ExpressionUUID" -> \
"6f472502-a66c-4b30-9a8b-a8baf3488f07"],
Cell[210639, 5203, 410, 9, 1464, "Input", "ExpressionUUID" -> \
"303d0de5-30ea-45a8-bafc-a833aad07ffa"],
Cell[211052, 5214, 381, 9, 1464, "Input", "ExpressionUUID" -> \
"25993e05-5f13-41dc-95a8-f2d96c6ec2ef"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[211494, 5230, 412, 6, 97, "Section", "ExpressionUUID" -> \
"3f95adbb-87f1-4de1-88f4-6b20aa097185"],
Cell[211909, 5238, 736, 12, 115, "Text", "ExpressionUUID" -> \
"65e460b7-cff1-476c-a89b-7372f165b5fc"],
Cell[CellGroupData[{
Cell[212670, 5254, 215, 4, 67, "Subsection", "ExpressionUUID" -> \
"6693537b-4750-493f-a5b2-eb56b165335a"],
Cell[CellGroupData[{
Cell[212910, 5262, 225, 4, 53, "Subsubsection", "ExpressionUUID" -> \
"531d3913-c29f-4f16-9ea9-42b70fd62574"],
Cell[213138, 5268, 2545, 50, 1955401035, "Text", "ExpressionUUID" -> \
"033432b0-6e36-467d-8360-4bf94fae822b"],
Cell[215686, 5320, 1903, 34, 933494238, "Text", "ExpressionUUID" -> \
"b30b9351-0ec5-4748-ac0f-7211251d5753"],
Cell[217592, 5356, 878, 24, 302773095, "Input", "ExpressionUUID" -> \
"8bd26fb1-08d3-4a09-b733-5af4bc302f84",
 Evaluatable->False],
Cell[218473, 5382, 4389, 105, 1753149960, "Text", "ExpressionUUID" -> \
"3530bbbc-cf69-4bc9-8074-08aeb4437b20"],
Cell[222865, 5489, 1007, 30, 337376484, "Input", "ExpressionUUID" -> \
"cc30da6b-9027-4d99-8d01-6779d804a0d9",
 Evaluatable->False],
Cell[223875, 5521, 371, 9, 456322069, "Input", "ExpressionUUID" -> \
"9bca7013-910f-4873-ad23-36894c7cdad2",
 InitializationCell->True],
Cell[224249, 5532, 490, 10, 883389960, "Input", "ExpressionUUID" -> \
"f8eb65fc-d012-4be3-b50a-02426df9bc37",
 InitializationCell->True],
Cell[224742, 5544, 2795, 64, 1135399980, "Text", "ExpressionUUID" -> \
"ae6ff1a4-7af8-4489-8a6c-0d62cfa7ef40"],
Cell[227540, 5610, 329, 6, 304214716, "Input", "ExpressionUUID" -> \
"d8142666-f6b0-4883-9c0e-5d782cb17ed6"],
Cell[227872, 5618, 205, 3, 314284872, "Text", "ExpressionUUID" -> \
"14e347bd-8327-4e2f-99d1-5b0e039613d6"],
Cell[228080, 5623, 673, 18, 304214716, "Input", "ExpressionUUID" -> \
"d5a9f928-e79b-4624-8765-556be2b9f73a",
 Evaluatable->False],
Cell[228756, 5643, 1101, 28, 554723260, "Text", "ExpressionUUID" -> \
"97fcf730-36c7-491e-82b4-b75fbbeb56e3"],
Cell[229860, 5673, 461, 11, 489643632, "Input", "ExpressionUUID" -> \
"b8e229bc-887e-495b-944c-ad30b86d8886",
 Evaluatable->False],
Cell[230324, 5686, 972, 22, 521394706, "Text", "ExpressionUUID" -> \
"00d7858e-2db1-455a-b96d-1ea46a14101b"],
Cell[231299, 5710, 1348, 39, 337376484, "Input", "ExpressionUUID" -> \
"de61ed76-c2e3-4ec3-99fd-789a7e2f05c1",
 Evaluatable->False],
Cell[232650, 5751, 609, 16, 539491561, "Text", "ExpressionUUID" -> \
"22211072-c44b-4da7-80fa-35c1779f4c1a"],
Cell[233262, 5769, 948, 20, 329846406, "Input", "ExpressionUUID" -> \
"8dfb344e-3c37-4338-91bf-20f0e3b1377e",
 Evaluatable->False],
Cell[234213, 5791, 1782, 47, 337376484, "Input", "ExpressionUUID" -> \
"4a871fae-563e-4209-b8db-54cdc4c4821b",
 Evaluatable->False],
Cell[235998, 5840, 1616, 35, 742154413, "Text", "ExpressionUUID" -> \
"503b7910-2fae-486b-af5f-8e9efa1fa61f"],
Cell[237617, 5877, 835, 23, 329846406, "Input", "ExpressionUUID" -> \
"fadb6786-29a7-47e1-984d-bd430d350815",
 Evaluatable->False],
Cell[238455, 5902, 523, 12, 521394706, "Text", "ExpressionUUID" -> \
"fff60825-2031-45f5-ab42-56c6500567e9"],
Cell[238981, 5916, 1015, 26, 539491561, "Input", "ExpressionUUID" -> \
"386496b9-9b8c-4241-a9af-ad9baa9e8a05",
 Evaluatable->False],
Cell[239999, 5944, 484, 11, 521394706, "Text", "ExpressionUUID" -> \
"cd7124b0-fa0e-4197-952a-574eb61e4657"],
Cell[240486, 5957, 204, 4, 314284872, "Text", "ExpressionUUID" -> \
"4a9a1ff6-b2ef-48e5-996e-dbad7959887b"],
Cell[240693, 5963, 189, 3, 314284872, "Text", "ExpressionUUID" -> \
"f8120f9c-e8c7-4c30-9666-a09f2d958f35"],
Cell[240885, 5968, 338, 7, 329846406, "Input", "ExpressionUUID" -> \
"61da7909-b736-4a77-9e17-45f13697adb0"],
Cell[241226, 5977, 907, 23, 832084891, "Text", "ExpressionUUID" -> \
"afc10a8b-fa35-4666-9d6d-04f0ea2bdce7"],
Cell[242136, 6002, 245, 6, 329846406, "Input", "ExpressionUUID" -> \
"103c3f7b-d419-4f42-b7f3-c93f92bd79a3"],
Cell[242384, 6010, 456, 11, 304214716, "Text", "ExpressionUUID" -> \
"777a7fa8-fa57-4cdf-b813-15cc0a5a5be0"],
Cell[242843, 6023, 246, 6, 329846406, "Input", "ExpressionUUID" -> \
"29815633-2fac-4efe-b0b3-fbb132a873e7"],
Cell[243092, 6031, 439, 11, 304214716, "Text", "ExpressionUUID" -> \
"6fe9c71a-2919-429a-b8f3-fa96fa829613"],
Cell[243534, 6044, 157, 3, 314284872, "Text", "ExpressionUUID" -> \
"a0f23a35-cc24-4d5f-9b64-b512ec1fa7d3"],
Cell[243694, 6049, 297, 9, 314284872, "Text", "ExpressionUUID" -> \
"d74e4c7e-807d-4a7d-855e-601d03612932"],
Cell[243994, 6060, 963, 21, 456322069, "Input", "ExpressionUUID" -> \
"a6e2abbd-90fa-4fbe-8c9c-75082344aab1",
 InitializationCell->True],
Cell[244960, 6083, 1342, 22, 1088942167, "Input", "ExpressionUUID" -> \
"20e13410-a2d5-4953-9fe3-cb7f57e11431",
 InitializationCell->True],
Cell[246305, 6107, 232, 5, 304214716, "Input", "ExpressionUUID" -> \
"fbb9f95c-7a10-4ded-a3a2-ee11849853f9"],
Cell[246540, 6114, 244, 5, 304214716, "Input", "ExpressionUUID" -> \
"6df6a618-4cbc-4006-8b57-5dd229a256cf"],
Cell[246787, 6121, 205, 3, 314284872, "Text", "ExpressionUUID" -> \
"c25d13b2-fa19-4e16-8f05-6129678946f5"],
Cell[246995, 6126, 559, 11, 304214716, "Input", "ExpressionUUID" -> \
"47ddf962-186b-42a4-a411-0694d73351be"],
Cell[247557, 6139, 198, 3, 314284872, "Text", "ExpressionUUID" -> \
"f88383d2-8feb-4f70-8b83-ef5833128680"],
Cell[247758, 6144, 824, 17, 456322069, "Input", "ExpressionUUID" -> \
"c09b0872-d7c9-4646-b6eb-eb53fffd68d6",
 InitializationCell->True],
Cell[248585, 6163, 412, 9, 304214716, "Input", "ExpressionUUID" -> \
"912ba9e2-fce0-48bc-abfc-e4f8e15b0b80"],
Cell[249000, 6174, 420, 9, 304214716, "Input", "ExpressionUUID" -> \
"4db165cd-9c7a-4dd3-850a-f977c2c1fc48"],
Cell[249423, 6185, 371, 9, 304214716, "Input", "ExpressionUUID" -> \
"8eadc1db-6ae4-44b5-99e0-af651e826143"],
Cell[249797, 6196, 410, 8, 304214716, "Input", "ExpressionUUID" -> \
"3aa829e6-1781-4171-9633-3be19f4169f8"],
Cell[250210, 6206, 187, 4, 304214716, "Input", "ExpressionUUID" -> \
"c6c005e4-afd2-4c05-917e-0a8330278fd9"],
Cell[250400, 6212, 317, 8, 304214716, "Input", "ExpressionUUID" -> \
"8c11a6f6-8dad-4e05-b554-0b8e48ef8677"],
Cell[250720, 6222, 3610, 76, 2147483647, "Text", "ExpressionUUID" -> \
"e4ace768-f644-4408-b7c7-b6a74b99f163"],
Cell[254333, 6300, 1744, 28, 2147483647, "Text", "ExpressionUUID" -> \
"bc3ec288-a9bb-4572-b0c8-e860fe75e19b"],
Cell[256080, 6330, 2161, 40, 1543302606, "Text", "ExpressionUUID" -> \
"b95d796d-0552-472c-a6e5-5debfb5cc219"]
}, Closed]],
Cell[CellGroupData[{
Cell[258278, 6375, 907, 23, 42, "Subsubsection", "ExpressionUUID" -> \
"4f23d230-3d20-441d-816f-972c90b11eaf"],
Cell[259188, 6400, 2179, 51, 2147483647, "Text", "ExpressionUUID" -> \
"edc38707-abba-472f-ab39-599df1ab0d7b"],
Cell[261370, 6453, 2018, 46, 2147483647, "Text", "ExpressionUUID" -> \
"30d016f5-c011-4189-947d-8a438697fde3"],
Cell[263391, 6501, 178, 4, 2147483647, "Input", "ExpressionUUID" -> \
"582a96fa-2573-4bed-823c-676635aac83a"],
Cell[263572, 6507, 162, 3, 2147483647, "Text", "ExpressionUUID" -> \
"b64f2663-054e-44d8-bcef-99d3bf23d495"],
Cell[263737, 6512, 184, 4, 2147483647, "Input", "ExpressionUUID" -> \
"b1679efc-9dda-4b65-a87f-0ddb9283cd78"],
Cell[263924, 6518, 1141, 27, 2147483647, "Text", "ExpressionUUID" -> \
"4f6994a6-4bc7-44bb-8eb8-18c45ec88210"],
Cell[265068, 6547, 207, 5, 2147483647, "Input", "ExpressionUUID" -> \
"63b56bcc-3745-4e60-a57e-11284f059d8d"],
Cell[265278, 6554, 560, 11, 2147483647, "Input", "ExpressionUUID" -> \
"0a8a6444-2b4f-4a43-80a9-125b78d8e026"],
Cell[265841, 6567, 214, 5, 2147483647, "Input", "ExpressionUUID" -> \
"978df08a-67b4-4ebb-a9ac-52d59ee269cb"],
Cell[266058, 6574, 540, 14, 2147483647, "Text", "ExpressionUUID" -> \
"59523128-ef4b-4ab4-8860-525f1233e72d"],
Cell[266601, 6590, 1408, 33, 2147483647, "Input", "ExpressionUUID" -> \
"49299524-6786-4ebd-b617-e0eb6cc470cd"],
Cell[268012, 6625, 986, 22, 2147483647, "Input", "ExpressionUUID" -> \
"532e14a6-6440-440d-b952-aaf3bdcd941a"],
Cell[269001, 6649, 329, 7, 2147483647, "Text", "ExpressionUUID" -> \
"cd2df1ac-b4d3-4a6f-9590-415bec0f538d"],
Cell[269333, 6658, 386, 10, 2147483647, "Input", "ExpressionUUID" -> \
"dce0a7f4-9ba5-49e9-9210-9956a53c4c99"],
Cell[269722, 6670, 770, 17, 2147483647, "Input", "ExpressionUUID" -> \
"997a0895-d0a8-434b-ba1a-369719798c91"],
Cell[270495, 6689, 593, 12, 2147483647, "Text", "ExpressionUUID" -> \
"a99cdda6-08cf-4d80-9c13-fc69e8903cde"],
Cell[271091, 6703, 545, 13, 2147483647, "Input", "ExpressionUUID" -> \
"1360eb1f-427c-45b1-bff8-338d78052b05"],
Cell[271639, 6718, 856, 23, 2147483647, "Input", "ExpressionUUID" -> \
"ff869e2f-d477-4f37-9ba8-2a1e425a6355"],
Cell[272498, 6743, 193, 3, 2147483647, "Text", "ExpressionUUID" -> \
"609588fb-5b32-411b-8210-5561b50bd19b"],
Cell[272694, 6748, 339, 7, 2147483647, "Input", "ExpressionUUID" -> \
"c3daf031-1654-4132-b456-cd317a0a4675"],
Cell[273036, 6757, 263, 6, 2147483647, "Text", "ExpressionUUID" -> \
"9b3e9824-027f-4077-b668-0296df0be610"],
Cell[273302, 6765, 469, 11, 2147483647, "Input", "ExpressionUUID" -> \
"4dc1b5d4-1840-4783-8a11-781b319ca0e5"],
Cell[273774, 6778, 1424, 32, 2147483647, "Input", "ExpressionUUID" -> \
"6eb70c81-04e4-41f6-b4af-8ca4e7a5b7b7"],
Cell[275201, 6812, 236, 4, 2147483647, "Text", "ExpressionUUID" -> \
"7b758477-2377-44c8-9621-27c20854e910"],
Cell[275440, 6818, 240, 6, 2147483647, "Input", "ExpressionUUID" -> \
"2103cb9c-405c-434c-bea0-3e2dd26ce52f"],
Cell[275683, 6826, 347, 5, 2147483647, "Text", "ExpressionUUID" -> \
"56056e78-f89c-474e-af71-3b661ef5b234"],
Cell[276033, 6833, 311, 8, 2147483647, "Input", "ExpressionUUID" -> \
"b2712fc8-30d8-4b48-94f3-5f0232816419"],
Cell[276347, 6843, 209, 3, 2147483647, "Text", "ExpressionUUID" -> \
"466883f6-6d81-446e-80e3-508adc171e70"],
Cell[276559, 6848, 315, 8, 2147483647, "Input", "ExpressionUUID" -> \
"75821b7e-0f4b-486a-8d11-932d1c140a53"],
Cell[276877, 6858, 274, 7, 2147483647, "Input", "ExpressionUUID" -> \
"348ee08e-3b73-497b-b83c-974d02c2989a"],
Cell[277154, 6867, 364, 10, 2147483647, "Input", "ExpressionUUID" -> \
"e67935ec-8635-4a02-8a8b-0d779f9e41c8"],
Cell[277521, 6879, 359, 7, 2147483647, "Text", "ExpressionUUID" -> \
"3029407b-4f8f-440a-9111-34ea64ff951c"],
Cell[277883, 6888, 1533, 36, 2147483647, "Input", "ExpressionUUID" -> \
"7c655ab4-a1a4-4424-ae8a-d336346c9e0a"],
Cell[279419, 6926, 833, 20, 2147483647, "Input", "ExpressionUUID" -> \
"35b9c5be-8310-46da-8923-26ee8aedc32e"],
Cell[280255, 6948, 412, 10, 2147483647, "Input", "ExpressionUUID" -> \
"b2404933-c726-46fd-8676-0267ef46112c"],
Cell[280670, 6960, 172, 3, 2147483647, "Text", "ExpressionUUID" -> \
"39a0baee-222b-4ec3-93a2-78777e04f8c1"],
Cell[280845, 6965, 810, 18, 2147483647, "Input", "ExpressionUUID" -> \
"fb2db198-1598-4812-83de-75e25384673b"],
Cell[281658, 6985, 293, 7, 2147483647, "Input", "ExpressionUUID" -> \
"2ed3b6b7-5058-4eb9-9d93-30886a1fdc9d"],
Cell[281954, 6994, 252, 5, 2147483647, "Text", "ExpressionUUID" -> \
"0be5db9f-31ad-4c81-a0d3-c4010a60d334"],
Cell[282209, 7001, 521, 13, 2147483647, "Input", "ExpressionUUID" -> \
"fc44f6bd-8e0b-4052-a65d-58842f185d20"],
Cell[282733, 7016, 1002, 26, 2147483647, "Input", "ExpressionUUID" -> \
"a55fdda3-d4a4-4e0d-9f3e-22867fa511f0"],
Cell[283738, 7044, 369, 8, 2147483647, "Input", "ExpressionUUID" -> \
"1069c15a-4881-4082-91cf-4d53f1a65d34"],
Cell[284110, 7054, 267, 6, 2147483647, "Text", "ExpressionUUID" -> \
"3d5cab4d-1135-435d-9e47-a902c94e0fd7"],
Cell[284380, 7062, 499, 12, 2147483647, "Input", "ExpressionUUID" -> \
"f47825e6-19b8-4540-b119-bae40562ccac"],
Cell[284882, 7076, 882, 23, 2147483647, "Input", "ExpressionUUID" -> \
"6ebe4028-73e5-4b18-ba8d-8f2e8f5f8c64"],
Cell[285767, 7101, 363, 8, 2147483647, "Input", "ExpressionUUID" -> \
"891f5ee4-1679-4ce7-817d-cc0fde78de52"],
Cell[286133, 7111, 249, 5, 2147483647, "Text", "ExpressionUUID" -> \
"ca131b83-7593-4474-a5b6-8d96cf9e9f72"],
Cell[286385, 7118, 414, 9, 2147483647, "Input", "ExpressionUUID" -> \
"c4b82d5b-d61f-4067-b254-3fc06eb49cb0"],
Cell[286802, 7129, 315, 8, 2147483647, "Input", "ExpressionUUID" -> \
"5ee56e23-c0bb-44ca-9a5a-b52f42b8a5e7"],
Cell[287120, 7139, 276, 6, 2147483647, "Text", "ExpressionUUID" -> \
"68b5157a-7c54-4da4-ae94-6be42124729a"],
Cell[287399, 7147, 1828, 37, 2147483647, "Text", "ExpressionUUID" -> \
"025f729f-99d9-44fd-8b65-bcb1950343a9"],
Cell[289230, 7186, 297, 7, 2147483647, "Text", "ExpressionUUID" -> \
"56244ab9-9f95-4d1e-82d2-d00413a3c8a9"],
Cell[289530, 7195, 629, 18, 2147483647, "Input", "ExpressionUUID" -> \
"38f404b7-10aa-4502-93ef-05efb9809935"],
Cell[290162, 7215, 168, 3, 2147483647, "Text", "ExpressionUUID" -> \
"60dfba30-2d59-4458-8630-c0fdad9d55b7"],
Cell[290333, 7220, 263, 6, 2147483647, "Input", "ExpressionUUID" -> \
"9d601cc9-40ed-4df1-9a26-92dee37d3bd8"],
Cell[290599, 7228, 175, 3, 2147483647, "Text", "ExpressionUUID" -> \
"bb4c511c-5ba8-42d6-a30f-ecd9f06486a9"],
Cell[290777, 7233, 350, 10, 2147483647, "Input", "ExpressionUUID" -> \
"409f561c-516f-4542-864b-3e656da8fec7"],
Cell[291130, 7245, 234, 6, 2147483647, "Text", "ExpressionUUID" -> \
"0c400c2f-b3e9-4bf8-a629-3a3d420ef43b"],
Cell[291367, 7253, 471, 11, 2147483647, "Input", "ExpressionUUID" -> \
"a22dd5fd-aa32-437d-abbb-1d6a0f7f1ddd"],
Cell[291841, 7266, 403, 11, 2147483647, "Input", "ExpressionUUID" -> \
"7cbcdef1-3e07-448b-b531-72ddcdd5e72d"],
Cell[292247, 7279, 266, 7, 2147483647, "Input", "ExpressionUUID" -> \
"a377952b-f661-4804-8327-e5c728ccab44"],
Cell[292516, 7288, 205, 3, 2147483647, "Text", "ExpressionUUID" -> \
"8efacf69-871b-4ea5-9c88-d4bea2d7ec59"],
Cell[292724, 7293, 289, 7, 2147483647, "Input", "ExpressionUUID" -> \
"a9dad553-d839-405c-848b-93112a9b7c99"],
Cell[293016, 7302, 261, 7, 2147483647, "Input", "ExpressionUUID" -> \
"24ab5b4e-a6eb-40a5-8892-edd13df92b73"],
Cell[293280, 7311, 578, 17, 2147483647, "Input", "ExpressionUUID" -> \
"966dffeb-7553-4ac7-a052-004f5cb96ced"],
Cell[293861, 7330, 515, 14, 2147483647, "Input", "ExpressionUUID" -> \
"1844a4d8-d6fa-4324-8709-13eff44cee35"],
Cell[294379, 7346, 239, 4, 2147483647, "Text", "ExpressionUUID" -> \
"e7a21303-7e03-4432-ad87-07530109472e"]
}, Closed]],
Cell[CellGroupData[{
Cell[294655, 7355, 251, 4, 42, "Subsubsection", "ExpressionUUID" -> \
"38955002-8be9-4962-a642-97f4e7e107d5"],
Cell[294909, 7361, 701, 18, 2147483647, "Text", "ExpressionUUID" -> \
"a24031f4-ee12-41c4-8530-9c609349adae"],
Cell[295613, 7381, 989, 23, 2147483647, "Input", "ExpressionUUID" -> \
"27b3477b-63fb-4466-95cf-bd46b5c94b36",
 Evaluatable->False],
Cell[296605, 7406, 4819, 112, 2147483647, "Text", "ExpressionUUID" -> \
"1df99378-6c88-4db7-949a-71a0af520d73"],
Cell[301427, 7520, 686, 15, 2147483647, "Input", "ExpressionUUID" -> \
"4bd4f1b6-a90f-4c0c-bced-5595116a56ef",
 Evaluatable->False],
Cell[302116, 7537, 1415, 36, 2147483647, "Text", "ExpressionUUID" -> \
"6e947f95-e0c8-4832-a67d-0c2a6d30f5b0"],
Cell[303534, 7575, 2756, 57, 2147483647, "Text", "ExpressionUUID" -> \
"7801a86f-5bbc-44a2-a0ac-c89802ca0090"],
Cell[306293, 7634, 580, 10, 2147483647, "Input", "ExpressionUUID" -> \
"16e17323-4efb-4273-a86c-ca271eed84d8",
 InitializationCell->True],
Cell[306876, 7646, 184, 4, 2147483647, "Input", "ExpressionUUID" -> \
"3ec82258-0aef-4843-b2f4-3fa4133bb13d"],
Cell[307063, 7652, 186, 3, 2147483647, "Text", "ExpressionUUID" -> \
"ee66fd10-1cfe-4705-8998-a05883d976ba"],
Cell[307252, 7657, 84, 0, 2147483647, "Input", "ExpressionUUID" -> \
"256e393e-71c7-42f2-b979-a4aa613994aa"],
Cell[307339, 7659, 1118, 26, 2147483647, "Text", "ExpressionUUID" -> \
"8005adbe-ffd4-4676-b87f-610411ff7a9a"],
Cell[308460, 7687, 957, 26, 2147483647, "Input", "ExpressionUUID" -> \
"0748a434-dcbe-406d-8a93-2a770dae632b",
 Evaluatable->False],
Cell[309420, 7715, 1411, 30, 2147483647, "Text", "ExpressionUUID" -> \
"8444435c-7667-40b3-9816-085f53140f92"],
Cell[310834, 7747, 662, 13, 2147483647, "Text", "ExpressionUUID" -> \
"67e7135c-5da0-4a84-82c2-517a9696ffbb"],
Cell[311499, 7762, 172, 4, 2147483647, "Input", "ExpressionUUID" -> \
"7af43380-8bcb-4f79-88b2-42ea0eda47e3"],
Cell[311674, 7768, 642, 13, 2147483647, "Input", "ExpressionUUID" -> \
"a592e2f5-1f11-4356-a2a9-584c14a90c8f"],
Cell[312319, 7783, 243, 6, 2147483647, "Input", "ExpressionUUID" -> \
"533155ac-b1da-4314-9468-b9395fcff90b"],
Cell[312565, 7791, 348, 8, 2147483647, "Input", "ExpressionUUID" -> \
"05584663-723a-4920-b2d5-03f68f2d31c4"],
Cell[312916, 7801, 237, 5, 2147483647, "Input", "ExpressionUUID" -> \
"db3d7b06-c603-49a8-92e2-7b1fd66de3c5"],
Cell[313156, 7808, 1347, 27, 2147483647, "Text", "ExpressionUUID" -> \
"aed93678-4f36-4ecf-8bda-dca47835d953"],
Cell[314506, 7837, 269, 6, 2147483647, "Input", "ExpressionUUID" -> \
"83bad27b-25a2-4d0f-ba1e-bf8b8ebd4bbf"],
Cell[314778, 7845, 803, 12, 2147483647, "Text", "ExpressionUUID" -> \
"79709955-c57d-4f60-bd6d-190a19a21324"],
Cell[315584, 7859, 1297, 31, 2147483647, "Text", "ExpressionUUID" -> \
"1e332206-5ce5-453b-b503-ee111374bb1a"],
Cell[316884, 7892, 740, 21, 2147483647, "Input", "ExpressionUUID" -> \
"258305ee-72e6-4d9c-acdc-2cc9ac0e951b",
 Evaluatable->False],
Cell[317627, 7915, 759, 18, 2147483647, "Text", "ExpressionUUID" -> \
"39b13252-8189-4883-b231-404e153f06c4"],
Cell[318389, 7935, 883, 19, 2147483647, "Text", "ExpressionUUID" -> \
"9c98862a-8107-400b-b02b-c5fd4a2bbe3b"],
Cell[319275, 7956, 628, 18, 2147483647, "Input", "ExpressionUUID" -> \
"532e2e5a-fe72-4132-9215-4b42a930cbf2",
 Evaluatable->False],
Cell[319906, 7976, 372, 10, 2147483647, "Input", "ExpressionUUID" -> \
"cd810c5b-2da3-455c-b58d-ce66f2a9bc8a"],
Cell[320281, 7988, 185, 3, 2147483647, "Text", "ExpressionUUID" -> \
"feea3c8d-5470-462b-b842-412eebe13ddd"],
Cell[320469, 7993, 864, 22, 2147483647, "Input", "ExpressionUUID" -> \
"330a1525-543d-466d-ab85-7c2170aedfee",
 Evaluatable->False],
Cell[321336, 8017, 371, 8, 2147483647, "Input", "ExpressionUUID" -> \
"d442cab8-e3ab-4d5b-b167-4b7b85b04c94"],
Cell[321710, 8027, 478, 13, 2147483647, "Input", "ExpressionUUID" -> \
"8b051998-2f1e-4936-a6ff-3d26c9f58546"],
Cell[322191, 8042, 258, 6, 2147483647, "Input", "ExpressionUUID" -> \
"50967f8a-ab9c-4437-9672-87a9a845a529"],
Cell[322452, 8050, 188, 4, 2147483647, "Input", "ExpressionUUID" -> \
"ded2c081-0310-4b4f-a6c9-03542bbbb6f2"],
Cell[322643, 8056, 183, 3, 2147483647, "Text", "ExpressionUUID" -> \
"8f7e593e-e0e0-4c76-8a22-c7659f8255f2"],
Cell[322829, 8061, 685, 19, 2147483647, "Input", "ExpressionUUID" -> \
"6343515e-d599-42f3-bdc9-df7e6b55d382",
 Evaluatable->False],
Cell[323517, 8082, 462, 12, 2147483647, "Input", "ExpressionUUID" -> \
"1e4809e5-0110-47ce-bd83-2eacddc2aba9"],
Cell[323982, 8096, 455, 11, 2147483647, "Input", "ExpressionUUID" -> \
"a82f43de-27f2-44f2-bd1f-36af4f40b6df"],
Cell[324440, 8109, 1164, 26, 2147483647, "Text", "ExpressionUUID" -> \
"98dd7a73-7008-4380-856a-759710c79159"],
Cell[325607, 8137, 352, 7, 2147483647, "Input", "ExpressionUUID" -> \
"ab37dac7-adda-401f-9286-57018235960e",
 InitializationCell->True],
Cell[325962, 8146, 305, 8, 2147483647, "Input", "ExpressionUUID" -> \
"196d31e7-6660-403a-aaea-870f756a639d"],
Cell[326270, 8156, 542, 11, 2147483647, "Text", "ExpressionUUID" -> \
"da411071-95c7-4a3f-97ca-9e7c7f14861e"],
Cell[326815, 8169, 181, 3, 2147483647, "Text", "ExpressionUUID" -> \
"f88621b4-a914-4f5c-801c-7f34010cbe77"],
Cell[326999, 8174, 172, 4, 2147483647, "Input", "ExpressionUUID" -> \
"facaf512-e10c-44c9-a7fa-1a7e26506fc8"],
Cell[327174, 8180, 208, 4, 2147483647, "Input", "ExpressionUUID" -> \
"4d2cef79-52b6-4cfd-bf9d-052c5a827cb4"],
Cell[327385, 8186, 213, 5, 2147483647, "Input", "ExpressionUUID" -> \
"afdc1536-2a01-4a76-8875-53f537f05bfb"],
Cell[327601, 8193, 790, 21, 2147483647, "Input", "ExpressionUUID" -> \
"b8dbb4e3-20f1-4d9b-a78e-fbdc775b0e86"],
Cell[328394, 8216, 223, 5, 2147483647, "Text", "ExpressionUUID" -> \
"d8923b52-0651-40c2-9c39-d9cb208471f7"],
Cell[328620, 8223, 522, 13, 2147483647, "Input", "ExpressionUUID" -> \
"32b99913-3275-4d4c-a74c-70e5b4d59c14"],
Cell[329145, 8238, 186, 4, 2147483647, "Input", "ExpressionUUID" -> \
"d9c018f0-433a-4588-bbe3-064a74fad491"],
Cell[329334, 8244, 189, 4, 2147483647, "Input", "ExpressionUUID" -> \
"1a0d6f57-2c36-4f84-bc4d-afd8c7d17d16"],
Cell[329526, 8250, 213, 5, 2147483647, "Text", "ExpressionUUID" -> \
"1b4e2f63-fa66-4817-8eef-43d61195e957"],
Cell[329742, 8257, 275, 8, 2147483647, "Input", "ExpressionUUID" -> \
"c9ad1601-17fd-41c5-8b02-32ca5deb4b8d"],
Cell[330020, 8267, 190, 4, 2147483647, "Input", "ExpressionUUID" -> \
"183de157-e0b1-4ce6-87cf-07d3e850d1cf"],
Cell[330213, 8273, 187, 4, 2147483647, "Input", "ExpressionUUID" -> \
"6a47be53-5b01-41d3-97a7-c1a76483bb9a"],
Cell[330403, 8279, 217, 3, 2147483647, "Text", "ExpressionUUID" -> \
"476e3ed9-ca37-4efa-bd7c-604bd0a89dc7"],
Cell[330623, 8284, 191, 4, 2147483647, "Input", "ExpressionUUID" -> \
"dc80b096-395d-4f56-bcd6-fc63aeffcb8d"],
Cell[330817, 8290, 185, 4, 2147483647, "Input", "ExpressionUUID" -> \
"8128a42d-6db5-4707-b7ac-082749faac15"],
Cell[331005, 8296, 235, 5, 2147483647, "Input", "ExpressionUUID" -> \
"a6eb8279-3bbf-48ef-a6df-c62a0585c2c5"],
Cell[331243, 8303, 187, 4, 2147483647, "Input", "ExpressionUUID" -> \
"f3a2cbe0-2031-4c43-9d00-5961d2753f98"],
Cell[331433, 8309, 343, 7, 2147483647, "Input", "ExpressionUUID" -> \
"068f5bff-f38b-41d0-87a2-879b80686189"],
Cell[331779, 8318, 351, 7, 2147483647, "Text", "ExpressionUUID" -> \
"81ec28e4-1502-4284-b686-cb66cbe1a90e"],
Cell[332133, 8327, 184, 4, 2147483647, "Input", "ExpressionUUID" -> \
"f4c5ca7f-c739-47e8-bcb7-26c63ad7ff27"],
Cell[332320, 8333, 410, 8, 2147483647, "Input", "ExpressionUUID" -> \
"6c7f5d77-ab6c-4549-b0ae-558c6f170253"],
Cell[332733, 8343, 384, 7, 2147483647, "Input", "ExpressionUUID" -> \
"a2efd4a7-b20b-4c21-b43c-f5cb6366f1a0"],
Cell[333120, 8352, 175, 4, 2147483647, "Input", "ExpressionUUID" -> \
"185fd295-f7e7-4407-9bf9-669d0025f9d9"]
}, Closed]],
Cell[CellGroupData[{
Cell[333332, 8361, 669, 10, 42, "Subsubsection", "ExpressionUUID" -> \
"c4140200-0697-47d1-a1f3-334c5e74e322"],
Cell[334004, 8373, 2595, 55, 5419, "Text", "ExpressionUUID" -> \
"7236a6ce-c270-4af5-b699-10605ae42b81"],
Cell[336602, 8430, 584, 11, 1464, "Input", "ExpressionUUID" -> \
"58013f82-2f08-4538-a399-173862d05b23",
 InitializationCell->True],
Cell[337189, 8443, 972, 21, 2499, "Text", "ExpressionUUID" -> \
"774b057c-97ca-43e3-8574-5a634635185a"],
Cell[338164, 8466, 197, 5, 1464, "Input", "ExpressionUUID" -> \
"7f002176-85f4-465f-9091-5606b6f32373"],
Cell[338364, 8473, 280, 7, 1464, "Input", "ExpressionUUID" -> \
"b4706a9f-7d65-4187-83ad-e0038445d7f9",
 InitializationCell->True],
Cell[338647, 8482, 193, 4, 1464, "Input", "ExpressionUUID" -> \
"0ad48493-0dea-4659-806a-55bdf341391d"],
Cell[338843, 8488, 330, 7, 1464, "Input", "ExpressionUUID" -> \
"0a66bf0c-10f5-40e9-8e60-cef192f02da6"],
Cell[339176, 8497, 240, 4, 1503, "Text", "ExpressionUUID" -> \
"ab1aa335-79b7-45f9-ae68-a94b8cf27053"],
Cell[339419, 8503, 523, 10, 1464, "Input", "ExpressionUUID" -> \
"4c0746e3-838c-42b0-93a2-85b29710cf0e"],
Cell[339945, 8515, 530, 13, 2460, "Text", "ExpressionUUID" -> \
"7fc5c4df-90c6-419d-848b-27ca18292410"],
Cell[340478, 8530, 437, 11, 1464, "Input", "ExpressionUUID" -> \
"38a29dcc-1ded-4eb7-bd3c-7eeeb1a1f563"],
Cell[340918, 8543, 1853, 40, 7528, "Text", "ExpressionUUID" -> \
"026ca68c-155b-4588-88f6-77eb5faf9a74"],
Cell[342774, 8585, 167, 4, 1464, "Input", "ExpressionUUID" -> \
"65449225-69ee-4540-bf86-a616e8fdea46"],
Cell[342944, 8591, 733, 13, 1464, "Input", "ExpressionUUID" -> \
"5adb4e6f-9f17-410b-a421-c6d520e79230"],
Cell[343680, 8606, 415, 11, 1533, "Text", "ExpressionUUID" -> \
"4adef16e-c099-4200-a5e4-b8c94838af94"],
Cell[344098, 8619, 318, 6, 1464, "Input", "ExpressionUUID" -> \
"7120ef26-0c03-46d1-bb64-c5d8fbd557f7"],
Cell[344419, 8627, 407, 11, 1503, "Text", "ExpressionUUID" -> \
"37a51847-5f07-4862-a4b4-54d661e4f5f5"],
Cell[344829, 8640, 344, 7, 1464, "Input", "ExpressionUUID" -> \
"a0d680b9-962f-4f80-836a-2cceaf75c8b9"],
Cell[345176, 8649, 156, 3, 1464, "Text", "ExpressionUUID" -> \
"08791b6a-115a-4fc8-900e-7452942d31c6"],
Cell[345335, 8654, 123, 1, 1464, "Input", "ExpressionUUID" -> \
"922f0e15-d242-41fa-a9d8-f73b17c7ab9a"],
Cell[345461, 8657, 2321, 43, 1591, "Text", "ExpressionUUID" -> \
"16846dd9-bb2e-4524-b516-fc0f716b9c68"],
Cell[347785, 8702, 410, 9, 1464, "Input", "ExpressionUUID" -> \
"9cd6aeb7-d75c-46a3-b2d1-adb801859967"],
Cell[348198, 8713, 1702, 39, 3514, "Text", "ExpressionUUID" -> \
"643bcf8d-2426-48ee-897d-f219f8e128da"],
Cell[349903, 8754, 726, 20, 1728, "Input", "ExpressionUUID" -> \
"909bb338-2a42-42d0-871c-d9ade6473f92",
 Evaluatable->False],
Cell[350632, 8776, 949, 19, 1533, "Text", "ExpressionUUID" -> \
"903a08dd-9b33-44a2-a042-5e2dd19c4ed1"],
Cell[351584, 8797, 464, 9, 1464, "Input", "ExpressionUUID" -> \
"68deeaef-3537-43b7-b75b-8745c4661f2c"],
Cell[352051, 8808, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"f095a16e-13b1-44ad-9224-bd31dfb669e2"],
Cell[352249, 8814, 124, 1, 1464, "Input", "ExpressionUUID" -> \
"378986a7-7b84-47d1-9344-af5568779b2b"],
Cell[352376, 8817, 473, 10, 1464, "Input", "ExpressionUUID" -> \
"a296bb4b-10bb-4f9c-bc2b-b187f9eb35a8"],
Cell[352852, 8829, 1811, 31, 1533, "Text", "ExpressionUUID" -> \
"3fde278b-d77a-4c87-8397-e6d340dba51c"],
Cell[354666, 8862, 172, 4, 1464, "Input", "ExpressionUUID" -> \
"a5d2ae90-54c0-4c62-8883-1542e4482677"],
Cell[354841, 8868, 190, 4, 1464, "Input", "ExpressionUUID" -> \
"e9fb0e48-82b0-4923-90c1-63e3be6f673c"],
Cell[355034, 8874, 220, 5, 1464, "Input", "ExpressionUUID" -> \
"3df6b0ee-a975-4116-8eb8-87b9ad059818"],
Cell[355257, 8881, 573, 11, 1464, "Input", "ExpressionUUID" -> \
"8093857e-2cd4-4228-8fbc-ba02bf03108c"],
Cell[355833, 8894, 721, 14, 1464, "Input", "ExpressionUUID" -> \
"8829b0d4-e8cc-4c9f-9462-fe4c2188ee3b"],
Cell[356557, 8910, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"c36d935c-08d4-4638-90c8-d8043dc4b85c"],
Cell[356745, 8916, 124, 1, 1464, "Input", "ExpressionUUID" -> \
"2f492352-a39e-4b91-a012-a044eb7403f7"],
Cell[356872, 8919, 223, 5, 1464, "Input", "ExpressionUUID" -> \
"294f8699-9eff-41ae-8267-b2e48259a1b1"],
Cell[357098, 8926, 1200, 25, 1464, "Input", "ExpressionUUID" -> \
"ef39f231-a95e-42c9-95ae-77b4ba4f26af"],
Cell[358301, 8953, 1108, 21, 1464, "Input", "ExpressionUUID" -> \
"c1b40804-6f27-4d18-aae5-b2111a9bbab3"],
Cell[359412, 8976, 402, 7, 1464, "Input", "ExpressionUUID" -> \
"bb30eb67-848c-4c3a-b8b6-2344717da1c1"],
Cell[359817, 8985, 540, 11, 1533, "Text", "ExpressionUUID" -> \
"4c059bd6-bf2b-4775-934e-d2851828f2b3"],
Cell[360360, 8998, 927, 20, 1464, "Input", "ExpressionUUID" -> \
"2d96c593-cefe-4fe9-9a01-8e2f2b757345",
 InitializationCell->True],
Cell[361290, 9020, 510, 9, 1464, "Input", "ExpressionUUID" -> \
"a3189ff5-7fa5-4e3a-9946-618ce57df771"],
Cell[361803, 9031, 705, 16, 1533, "Text", "ExpressionUUID" -> \
"a70bb0f8-4293-4921-b6c9-0ce0c73d3038"],
Cell[362511, 9049, 197, 5, 1464, "Input", "ExpressionUUID" -> \
"0f18a37a-d1a0-4898-8145-990408458d7a"],
Cell[362711, 9056, 583, 11, 1464, "Input", "ExpressionUUID" -> \
"2557bdda-5790-4d22-8eec-a6c22e1700a8"],
Cell[363297, 9069, 304, 8, 1464, "Input", "ExpressionUUID" -> \
"31c0d5bf-0fa7-4f54-b308-fc93a3c614b9"],
Cell[363604, 9079, 219, 4, 1464, "Text", "ExpressionUUID" -> \
"0b055a4a-7508-4aa6-b563-0c901bce4e66"],
Cell[363826, 9085, 336, 8, 1464, "Input", "ExpressionUUID" -> \
"af0305b6-b907-42b9-b490-801d1b0caa28"],
Cell[364165, 9095, 164, 3, 1464, "Text", "ExpressionUUID" -> \
"451561ac-ffc4-4255-8384-ce619208e0c3"],
Cell[364332, 9100, 354, 9, 1464, "Input", "ExpressionUUID" -> \
"99de920d-aa47-446a-9758-a6613c6ee602"],
Cell[364689, 9111, 321, 7, 2431, "Text", "ExpressionUUID" -> \
"b0350ee5-e307-442c-bd73-0981f2e76001"],
Cell[365013, 9120, 1821, 39, 3427, "Input", "ExpressionUUID" -> \
"74b19941-18ec-45bc-827e-56778059163b"],
Cell[366837, 9161, 267, 4, 1464, "Text", "ExpressionUUID" -> \
"a4a89818-cbfd-4628-99e6-6df4f6998c42"],
Cell[367107, 9167, 785, 20, 1464, "Input", "ExpressionUUID" -> \
"7ea7dac1-31e0-4a05-9da5-49d8198932a2",
 Evaluatable->False],
Cell[367895, 9189, 1034, 29, 2899, "Input", "ExpressionUUID" -> \
"b91e1ee3-27b9-4464-ad11-2c1d63082e02"],
Cell[368932, 9220, 6596, 141, 5545, "Text", "ExpressionUUID" -> \
"e08a3368-282f-40c2-9f67-a80a1697191e"],
Cell[375531, 9363, 527, 11, 1464, "Input", "ExpressionUUID" -> \
"60a719e9-04d0-40ed-8332-c9dc193be3dd",
 Evaluatable->False],
Cell[376061, 9376, 177, 3, 1464, "Text", "ExpressionUUID" -> \
"d2676eb9-1ed0-4b0f-982a-3ae6b567a790"],
Cell[376241, 9381, 330, 8, 2431, "Text", "ExpressionUUID" -> \
"4041a681-2400-4bf6-9bfd-5699366f1301"],
Cell[376574, 9391, 184, 4, 1464, "Input", "ExpressionUUID" -> \
"ae3ccd1f-13a7-45be-a22e-07073e69fab9"],
Cell[376761, 9397, 2623, 64, 5361, "Text", "ExpressionUUID" -> \
"30b21e58-fdfb-4ac1-b511-91047a91f5f8"],
Cell[379387, 9463, 1448, 27, 1464, "Input", "ExpressionUUID" -> \
"8a0260be-16cf-49bf-b066-ce13e0aa3ec6",
 InitializationCell->True],
Cell[380838, 9492, 369, 10, 1464, "Input", "ExpressionUUID" -> \
"179b0095-aba3-4aa9-ad07-308e8bcf3f8f"],
Cell[381210, 9504, 1444, 38, 2382, "Text", "ExpressionUUID" -> \
"e44b5ae7-72b0-49e7-ad89-2bb2cbdef1a7"],
Cell[382657, 9544, 1540, 28, 1464, "Input", "ExpressionUUID" -> \
"9b927eea-9275-4dd0-bbd5-3d4d22692ec1",
 InitializationCell->True],
Cell[384200, 9574, 614, 13, 1464, "Input", "ExpressionUUID" -> \
"387fa57b-23e5-422f-ab2b-76f54990eebf"],
Cell[384817, 9589, 747, 16, 1464, "Input", "ExpressionUUID" -> \
"f7014c7d-c29c-415c-86cf-0031e722e168",
 InitializationCell->True],
Cell[385567, 9607, 664, 14, 1464, "Input", "ExpressionUUID" -> \
"675f8a4b-6622-44b0-9425-101d1bfa58dd"],
Cell[386234, 9623, 676, 15, 1464, "Input", "ExpressionUUID" -> \
"b58f61fb-daf3-4ec8-9a04-15c9a6470710"],
Cell[386913, 9640, 152, 3, 1464, "Text", "ExpressionUUID" -> \
"2361787f-86ae-4a4c-954d-1f1166a5cfc0"],
Cell[387068, 9645, 282, 7, 1464, "Input", "ExpressionUUID" -> \
"e15dce52-af7c-44dc-a31a-ed1da83cfff9"],
Cell[387353, 9654, 709, 16, 1903, "Input", "ExpressionUUID" -> \
"ccc05aa2-73cc-4a7a-899a-068b60cc5845"],
Cell[388065, 9672, 740, 16, 1464, "Input", "ExpressionUUID" -> \
"fa38e938-962e-44e5-a259-31d0b8b3ab5d"],
Cell[388808, 9690, 871, 22, 2460, "Input", "ExpressionUUID" -> \
"bb4ccb57-f842-45e3-964b-ea19d1b6413d"],
Cell[389682, 9714, 169, 3, 1464, "Text", "ExpressionUUID" -> \
"83854caa-1319-4640-b1d3-a6270b0e4ba6"],
Cell[389854, 9719, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"1f30cfc3-9f71-4339-a9d9-0284620cf0f5"],
Cell[390044, 9725, 177, 4, 1464, "Input", "ExpressionUUID" -> \
"227d3ee0-cb57-4b0d-b597-72872e492152"],
Cell[390224, 9731, 162, 3, 1464, "Text", "ExpressionUUID" -> \
"54673cd7-2c0c-49d7-8ea5-3ddd1e665080"],
Cell[390389, 9736, 462, 11, 1464, "Input", "ExpressionUUID" -> \
"fce61884-9f64-476d-9db3-0942dc5e3370",
 Evaluatable->False],
Cell[390854, 9749, 210, 5, 1464, "Input", "ExpressionUUID" -> \
"c9d7161e-cd30-4395-a830-7075f86656e0"],
Cell[391067, 9756, 581, 13, 2724, "Text", "ExpressionUUID" -> \
"b9a19f14-8b97-4712-8141-69475667b797"],
Cell[391651, 9771, 1593, 38, 3349, "Text", "ExpressionUUID" -> \
"1c9261d6-0bf7-4a7f-8bc2-2162833b9b5a"],
Cell[393247, 9811, 973, 27, 2460, "Input", "ExpressionUUID" -> \
"e6a8da2e-0ee9-4ecc-8192-2f2278baeb62",
 Evaluatable->False],
Cell[394223, 9840, 172, 3, 1464, "Text", "ExpressionUUID" -> \
"4f78100f-8411-415c-a1e8-2ba10a84f6d9"],
Cell[394398, 9845, 932, 27, 1767, "Input", "ExpressionUUID" -> \
"4b1b41e3-07f1-4efe-8778-daf29e85f419",
 Evaluatable->False],
Cell[395333, 9874, 466, 10, 3349, "Text", "ExpressionUUID" -> \
"d09d4479-8c1f-404a-9bad-d8cc64459fa8"],
Cell[395802, 9886, 159, 3, 1464, "Text", "ExpressionUUID" -> \
"25213464-1076-4bba-ba34-69067b4e544b"],
Cell[395964, 9891, 933, 23, 2499, "Text", "ExpressionUUID" -> \
"eafae50c-24c5-4c3f-846c-19b6a7f33e98"],
Cell[396900, 9916, 561, 15, 1581, "Input", "ExpressionUUID" -> \
"2cc3c8ac-a5ca-48dc-9bb3-f48a397bb413",
 Evaluatable->False],
Cell[397464, 9933, 183, 3, 1464, "Text", "ExpressionUUID" -> \
"3c177fbe-18b5-4bd0-9541-912c6dca9092"],
Cell[397650, 9938, 840, 20, 1464, "Input", "ExpressionUUID" -> \
"caabc340-2b3b-4904-aa2b-695dbd55bc01",
 Evaluatable->False],
Cell[398493, 9960, 3093, 64, 5487, "Text", "ExpressionUUID" -> \
"ec210915-76dc-4469-a34f-7ddcd7df3f19"],
Cell[401589, 10026, 191, 3, 1464, "Input", "ExpressionUUID" -> \
"1c1642a4-8a8c-4c11-85c8-b7bcc3ac403c"],
Cell[401783, 10031, 706, 15, 1464, "Input", "ExpressionUUID" -> \
"4adf81f9-054b-4d86-80a4-b70665d0d95a"],
Cell[402492, 10048, 304, 9, 1464, "Input", "ExpressionUUID" -> \
"ae56db06-9e54-46ff-bb60-c760f89c4228"],
Cell[402799, 10059, 1291, 31, 3427, "Input", "ExpressionUUID" -> \
"d56af982-6ac6-445f-ab19-8efeef4d748b"],
Cell[404093, 10092, 1169, 32, 4482, "Text", "ExpressionUUID" -> \
"4f700bde-ee18-4f15-a25d-aaabdae567a8"],
Cell[405265, 10126, 384, 13, 1728, "Input", "ExpressionUUID" -> \
"6529259f-3ccb-4c66-bd24-3695926610ed",
 Evaluatable->False],
Cell[405652, 10141, 947, 23, 1533, "Text", "ExpressionUUID" -> \
"e3f64e1f-d301-4874-95bd-c3b93358e74a"],
Cell[406602, 10166, 646, 12, 1464, "Input", "ExpressionUUID" -> \
"7ac34348-fcdb-4bfb-9429-9e57faf897d1"],
Cell[407251, 10180, 880, 22, 1533, "Text", "ExpressionUUID" -> \
"1df8c3d4-1d35-4ab4-90ec-3352cab76665"],
Cell[408134, 10204, 867, 21, 1464, "Input", "ExpressionUUID" -> \
"82302d86-7ce8-40b9-9dd3-f53125102fed",
 Evaluatable->False],
Cell[409004, 10227, 1230, 32, 1591, "Text", "ExpressionUUID" -> \
"b5be5825-915f-4342-a039-c7e8784c52fc"],
Cell[410237, 10261, 1702, 31, 1591, "Text", "ExpressionUUID" -> \
"c4dd7142-9831-466f-93de-4e7e309d0d45"],
Cell[411942, 10294, 7847, 172, 5683, "Text", "ExpressionUUID" -> \
"7ef21eb7-e8f4-4590-a9ca-ea6027a29dc2"],
Cell[419792, 10468, 858, 20, 1464, "Input", "ExpressionUUID" -> \
"16163534-ffa3-440d-b6b6-a6dbbfefeab7",
 Evaluatable->False],
Cell[420653, 10490, 751, 18, 1464, "Input", "ExpressionUUID" -> \
"3f56989a-08a6-43d8-916e-506dddf7bb5c",
 Evaluatable->False],
Cell[421407, 10510, 1256, 22, 3436, "Text", "ExpressionUUID" -> \
"6abbfa2d-d5c9-4185-9e15-b6597d497bbb"],
Cell[422666, 10534, 4987, 119, 5625, "Text", "ExpressionUUID" -> \
"3d745f63-1935-43e7-8ebb-c511a865c08f"],
Cell[427656, 10655, 998, 31, 1464, "Input", "ExpressionUUID" -> \
"56e99d69-6a23-4613-b290-1aabd53ebc73",
 Evaluatable->False],
Cell[428657, 10688, 3697, 98, 8446, "Text", "ExpressionUUID" -> \
"2b86b916-b995-4db3-a5a3-e824b0e36898"],
Cell[432357, 10788, 182, 4, 1464, "Input", "ExpressionUUID" -> \
"55e715e0-7050-4a7c-aa15-5165972f86ef"],
Cell[432542, 10794, 1487, 34, 2460, "Input", "ExpressionUUID" -> \
"bccf41a0-ce24-4cf0-b505-42102a025f55",
 Evaluatable->False,
 InitializationCell->True],
Cell[434032, 10830, 194, 4, 1464, "Input", "ExpressionUUID" -> \
"3999c239-3f0c-41f3-a8d1-95a3fabf674c"],
Cell[434229, 10836, 374, 9, 1464, "Text", "ExpressionUUID" -> \
"9bd2b47a-9d87-44a3-806f-0e77ca35eb2e"],
Cell[434606, 10847, 1419, 33, 3427, "Input", "ExpressionUUID" -> \
"59233c31-38d9-41a7-85e3-738d7ff32298",
 Evaluatable->False],
Cell[436028, 10882, 179, 4, 1464, "Input", "ExpressionUUID" -> \
"70779eab-a454-4869-b9e1-1b8253cdec55"],
Cell[436210, 10888, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"d4a628af-e6cf-4ea0-bc65-c7831b3d3cc0"],
Cell[436408, 10894, 278, 5, 1464, "Text", "ExpressionUUID" -> \
"5706883f-b437-49ab-9d23-4928f753b0db"],
Cell[436689, 10901, 749, 19, 1464, "Input", "ExpressionUUID" -> \
"18b6b422-ee0f-4776-9e45-9ee2816f000e"],
Cell[437441, 10922, 395, 11, 1464, "Input", "ExpressionUUID" -> \
"be12dd12-6e67-4e9a-8740-549d750bfbbf"],
Cell[437839, 10935, 912, 20, 1464, "Input", "ExpressionUUID" -> \
"93daec75-ba8c-4fe0-9474-6be684a5f53e"],
Cell[438754, 10957, 273, 7, 1464, "Input", "ExpressionUUID" -> \
"f4598c45-9349-4abf-95fe-b5b1950c615f"],
Cell[439030, 10966, 358, 8, 1464, "Input", "ExpressionUUID" -> \
"ce9dd74e-7ac3-4b56-90df-26fff1e742a3"],
Cell[439391, 10976, 240, 6, 1464, "Input", "ExpressionUUID" -> \
"1e73f315-2bca-408e-bcf9-3c0a240c1857"],
Cell[439634, 10984, 276, 6, 1464, "Input", "ExpressionUUID" -> \
"a478193a-783b-4783-8133-b031d7ac0930"],
Cell[439913, 10992, 412, 8, 1464, "Text", "ExpressionUUID" -> \
"5c53f304-06ad-46a5-a9a2-cd9b34382abf"],
Cell[440328, 11002, 441, 13, 1464, "Input", "ExpressionUUID" -> \
"208dacc3-7c24-436c-a738-33aae448bc67"],
Cell[440772, 11017, 1134, 24, 1464, "Input", "ExpressionUUID" -> \
"8987f044-680d-455a-84dc-142683a21d5d"],
Cell[441909, 11043, 1094, 22, 1464, "Input", "ExpressionUUID" -> \
"8688feae-8d20-4979-adbe-79f2eb555543"],
Cell[443006, 11067, 162, 3, 1464, "Text", "ExpressionUUID" -> \
"e01f48df-8988-4c6e-883b-ab87dd30426c"],
Cell[443171, 11072, 840, 18, 2499, "Text", "ExpressionUUID" -> \
"ea97bcb5-d9dc-46b6-b7c4-b9ce08f5170c"],
Cell[444014, 11092, 613, 15, 1464, "Input", "ExpressionUUID" -> \
"1200bd37-447c-4b1c-ba4e-4da6182a8905",
 InitializationCell->True],
Cell[444630, 11109, 646, 14, 1464, "Text", "ExpressionUUID" -> \
"43ba257e-c56a-4bb7-9467-3566c2512534"],
Cell[445279, 11125, 352, 8, 1464, "Input", "ExpressionUUID" -> \
"4cb635ea-cea8-45e0-9f75-06e29cabf66f"],
Cell[445634, 11135, 716, 16, 1464, "Input", "ExpressionUUID" -> \
"d3efaec7-ea16-40b5-9200-e93e5a92971e"],
Cell[446353, 11153, 790, 19, 1464, "Input", "ExpressionUUID" -> \
"84979253-76ec-45d6-87f0-57784aa72eac"],
Cell[447146, 11174, 391, 10, 1464, "Input", "ExpressionUUID" -> \
"ff02c909-febe-45ab-88fd-d8a57eed0b79"],
Cell[447540, 11186, 571, 13, 1503, "Text", "ExpressionUUID" -> \
"61c68199-c2fc-4fc2-92f0-a4c39a4886a3"],
Cell[448114, 11201, 198, 4, 1464, "Input", "ExpressionUUID" -> \
"0ac55096-9cc1-44e9-925b-f00c399793eb"],
Cell[448315, 11207, 1676, 46, 2635, "Text", "ExpressionUUID" -> \
"092cc3ee-d793-4a15-ae61-a1b6ffb96443"],
Cell[449994, 11255, 1153, 26, 1464, "Input", "ExpressionUUID" -> \
"81baf7f1-4e66-4f68-ab9a-d57b19d6c713",
 Evaluatable->False],
Cell[451150, 11283, 12542, 288, 13593, "Text", "ExpressionUUID" -> \
"ae87acca-be61-4a97-8a41-6ece8fd2487d"],
Cell[463695, 11573, 2317, 44, 3514, "Text", "ExpressionUUID" -> \
"f9618acb-b706-4810-95f5-bc4b51f5f1bd"],
Cell[466015, 11619, 1263, 25, 3436, "Text", "ExpressionUUID" -> \
"ff87f2d7-3659-4f03-86e9-8aed90c59fe4"],
Cell[467281, 11646, 795, 16, 1464, "Input", "ExpressionUUID" -> \
"3c0f4236-3ea7-4989-99ec-f9dceabaf5bc",
 Evaluatable->False],
Cell[468079, 11664, 676, 12, 2431, "Text", "ExpressionUUID" -> \
"13685f11-a516-4039-9ac7-b86529bf1d05"],
Cell[468758, 11678, 2001, 44, 2557, "Input", "ExpressionUUID" -> \
"fb64c834-ac42-4a41-a4f5-aa006ac3c266",
 Evaluatable->False],
Cell[470762, 11724, 1107, 26, 1503, "Text", "ExpressionUUID" -> \
"abe4f94b-b5b4-43a7-8e5e-3a05387a6acc"],
Cell[471872, 11752, 1183, 27, 2460, "Input", "ExpressionUUID" -> \
"7b27c4f2-dd76-4f71-b9e8-7a7268a29a7b"],
Cell[473058, 11781, 513, 12, 2499, "Text", "ExpressionUUID" -> \
"13fe156e-0814-42bc-948c-2e3d4d196c4d"],
Cell[473574, 11795, 880, 22, 1708, "Input", "ExpressionUUID" -> \
"4865def5-8f83-43b5-ba35-470a3f295bd6",
 Evaluatable->False],
Cell[474457, 11819, 330, 8, 2431, "Text", "ExpressionUUID" -> \
"18aebd85-dd0e-4aa8-beef-b56e31a6e500"],
Cell[474790, 11829, 523, 14, 1533, "Input", "ExpressionUUID" -> \
"d7a76882-4434-43b1-88b7-0cf7bbe09346",
 Evaluatable->False],
Cell[475316, 11845, 159, 3, 1464, "Text", "ExpressionUUID" -> \
"098b3cee-2cd4-4beb-aab8-180a6c4df35e"],
Cell[475478, 11850, 1440, 38, 1708, "Input", "ExpressionUUID" -> \
"b3955108-8ffe-442b-8d12-5edcca06bde8",
 Evaluatable->False],
Cell[476921, 11890, 311, 7, 2431, "Text", "ExpressionUUID" -> \
"0108450b-6517-43b3-9b35-48d11611ea95"],
Cell[477235, 11899, 520, 11, 1464, "Input", "ExpressionUUID" -> \
"8aff232b-45f6-4970-aee5-e1d45501e4f4"],
Cell[477758, 11912, 387, 8, 1464, "Input", "ExpressionUUID" -> \
"27a184b9-9450-4d34-b829-8fec6a6c3706"],
Cell[478148, 11922, 245, 5, 1464, "Input", "ExpressionUUID" -> \
"cbb6227c-e230-4bc4-9466-87e0347f339f"],
Cell[478396, 11929, 329, 7, 2431, "Text", "ExpressionUUID" -> \
"a0b96707-d961-4d64-ab6c-01a9fa669f8f"],
Cell[478728, 11938, 919, 20, 2499, "Input", "ExpressionUUID" -> \
"c03d566b-9419-4ace-aa76-dbbcf56956b2"],
Cell[479650, 11960, 305, 7, 1464, "Input", "ExpressionUUID" -> \
"899b0383-9597-4d2f-8a6a-c0b0c359f9bb"],
Cell[479958, 11969, 219, 4, 1464, "Input", "ExpressionUUID" -> \
"b7eb1b6e-09a3-49cf-a102-bbef73bd8fe2"],
Cell[480180, 11975, 331, 7, 1464, "Input", "ExpressionUUID" -> \
"94ccdc5d-3984-4476-8182-ea5505df7454"],
Cell[480514, 11984, 162, 3, 1464, "Text", "ExpressionUUID" -> \
"f78a0619-23c8-4f3b-9ded-fdfb761c05bc"]
}, Closed]],
Cell[CellGroupData[{
Cell[480713, 11992, 795, 11, 42, "Subsubsection", "ExpressionUUID" -> \
"4194d1d4-5865-4f7c-bb1e-0031d8756a6d"],
Cell[481511, 12005, 2679, 76, 3339, "Text", "ExpressionUUID" -> \
"79054dae-2910-462a-9337-017aef3c4e63"],
Cell[484193, 12083, 1143, 33, 1093, "Text", "ExpressionUUID" -> \
"6785a327-3bfd-4fa6-88c6-d0ecbb415af7"],
Cell[485339, 12118, 1993, 52, 2518, "Text", "ExpressionUUID" -> \
"62867d42-d138-4033-a3c3-909e2ca3f813"],
Cell[487335, 12172, 1303, 29, 2518, "Text", "ExpressionUUID" -> \
"56ff3fef-8530-43a6-af4f-ae8112c2ef52"],
Cell[488641, 12203, 569, 14, 1591, "Input", "ExpressionUUID" -> \
"dc3617a3-7126-4dd9-906c-7c80a38d813e",
 InitializationCell->True],
Cell[489213, 12219, 350, 8, 1464, "Input", "ExpressionUUID" -> \
"c7d6e2c7-c730-43e9-8cbc-18b82fc4d21c"],
Cell[489566, 12229, 2038, 56, 2587, "Text", "ExpressionUUID" -> \
"bb49681c-ff63-4961-b897-2f844917bf87"],
Cell[491607, 12287, 1303, 28, 1464, "Input", "ExpressionUUID" -> \
"0b10b6aa-3615-4d11-b640-d76881e55b31",
 Evaluatable->False],
Cell[492913, 12317, 1815, 39, 1639, "Text", "ExpressionUUID" -> \
"bab097ad-7325-4dae-bd08-4b158ff86d5c"],
Cell[494731, 12358, 1227, 27, 1464, "Input", "ExpressionUUID" -> \
"c8e8f107-cc80-4287-acb6-0f3d5d38c2b4",
 Evaluatable->False],
Cell[495961, 12387, 836, 23, 1533, "Text", "ExpressionUUID" -> \
"d4e9cc6b-dc07-4e66-bb9f-ddf4c6f7d6c4"],
Cell[496800, 12412, 1855, 50, 2851, "Input", "ExpressionUUID" -> \
"a223765a-cd01-4075-9a29-e7361555ed61",
 Evaluatable->False],
Cell[498658, 12464, 344, 7, 1503, "Text", "ExpressionUUID" -> \
"c691f1db-12bc-47c0-8d7a-202a9ec25e89"],
Cell[499005, 12473, 1485, 34, 2518, "Text", "ExpressionUUID" -> \
"daeac705-5863-4d7b-86e1-cf8c411dfa02"],
Cell[500493, 12509, 567, 14, 1464, "Input", "ExpressionUUID" -> \
"bbb5776a-e5ae-4ca7-ab56-f5c28a33af17",
 Evaluatable->False],
Cell[501063, 12525, 1144, 27, 1533, "Text", "ExpressionUUID" -> \
"b380eab5-7486-4cc3-a7f8-a25c96ac9387"],
Cell[502210, 12554, 740, 22, 1464, "Input", "ExpressionUUID" -> \
"4b1d8871-494f-444d-892f-8f4bd004ef6f"],
Cell[502953, 12578, 750, 16, 1533, "Text", "ExpressionUUID" -> \
"e8f38c75-c6a5-4a16-9a04-6b941d725b5f"],
Cell[503706, 12596, 566, 13, 1464, "Input", "ExpressionUUID" -> \
"f06a9e62-8fbd-4d4f-81be-ef419129c459"],
Cell[504275, 12611, 218, 5, 1464, "Text", "ExpressionUUID" -> \
"4f9f30da-8796-4280-8f1d-c737c2e2afb7"],
Cell[504496, 12618, 919, 22, 1464, "Input", "ExpressionUUID" -> \
"9187e75b-cbf4-498a-9d1d-df0dce6c636f"],
Cell[505418, 12642, 398, 10, 1464, "Input", "ExpressionUUID" -> \
"6b7b67ec-7b60-47f8-83a3-1db8e5ee0333"],
Cell[505819, 12654, 1542, 38, 2460, "Input", "ExpressionUUID" -> \
"440b080d-9ee3-4640-b610-195085b4f10f"],
Cell[507364, 12694, 1904, 40, 2460, "Input", "ExpressionUUID" -> \
"55846baf-98c9-4fc1-a056-d3dcfd50fc15"],
Cell[509271, 12736, 1770, 41, 2460, "Input", "ExpressionUUID" -> \
"208a69e7-6c21-4c12-bcee-742355c73d99"],
Cell[511044, 12779, 226, 5, 1464, "Text", "ExpressionUUID" -> \
"5c2b170a-b5cb-4749-893b-d45d75e239a1"],
Cell[511273, 12786, 586, 18, 2782, "Input", "ExpressionUUID" -> \
"672c527f-f763-48d7-8fb6-70c32c031700",
 Evaluatable->False],
Cell[511862, 12806, 369, 8, 2431, "Text", "ExpressionUUID" -> \
"2cb584c1-c998-4307-b74d-c47145f1e2d9"],
Cell[512234, 12816, 518, 14, 1708, "Input", "ExpressionUUID" -> \
"3ac3f793-fab2-4ec6-902a-814c03e2cc8f",
 Evaluatable->False],
Cell[512755, 12832, 576, 15, 1533, "Text", "ExpressionUUID" -> \
"8cdabd4e-8f4e-4bbf-9118-a1c4ca8d741b"],
Cell[513334, 12849, 1234, 34, 1767, "Input", "ExpressionUUID" -> \
"72d30928-c31d-4194-bd82-1f7ac5ded7e8",
 Evaluatable->False],
Cell[514571, 12885, 1752, 41, 2724, "Text", "ExpressionUUID" -> \
"36773d0a-de20-4cad-b4a7-775b3c9e070f"],
Cell[516326, 12928, 403, 11, 1464, "Input", "ExpressionUUID" -> \
"cadb1adf-a54b-4949-a406-81f8b8a6a23a"],
Cell[516732, 12941, 688, 19, 1464, "Input", "ExpressionUUID" -> \
"54abcfda-5d65-4dfd-aa6b-046455d6795d"],
Cell[517423, 12962, 393, 9, 1464, "Input", "ExpressionUUID" -> \
"67461b42-52fb-4d7d-a0bd-b9a1d7f18da5"],
Cell[517819, 12973, 291, 7, 1464, "Input", "ExpressionUUID" -> \
"38165874-be2c-4d7f-9c4b-9b798b7384c1"],
Cell[518113, 12982, 294, 7, 1464, "Input", "ExpressionUUID" -> \
"a06f9ed5-d5b9-43b6-8911-615c39a2525c"],
Cell[518410, 12991, 764, 19, 1464, "Input", "ExpressionUUID" -> \
"95aaad1e-5ac6-4be3-999c-1d66b63f476e"],
Cell[519177, 13012, 1253, 32, 3700, "Text", "ExpressionUUID" -> \
"95556943-49be-4987-9e17-7d3af3d4c918"],
Cell[520433, 13046, 1145, 27, 1533, "Input", "ExpressionUUID" -> \
"ec406c26-201c-43ab-80fb-5533373ba07b",
 Evaluatable->False],
Cell[521581, 13075, 1843, 33, 2499, "Text", "ExpressionUUID" -> \
"12630cc3-9fc5-4685-a31d-2a3b44d981d0"],
Cell[523427, 13110, 2495, 65, 1767, "Input", "ExpressionUUID" -> \
"ea29f98e-51e4-4e49-832f-3157025009ad",
 Evaluatable->False],
Cell[525925, 13177, 185, 3, 1464, "Text", "ExpressionUUID" -> \
"9de29ff2-f230-4031-8eda-f488da907c1b"],
Cell[526113, 13182, 238, 4, 2254, "Text", "ExpressionUUID" -> \
"9ebfee58-3354-49b5-a02b-11a7150a83ac",
 CellTags->"transfcoords"],
Cell[526354, 13188, 2269, 66, 4168, "Text", "ExpressionUUID" -> \
"745ee643-2389-41f2-8cc1-d73789b814d6"],
Cell[528626, 13256, 205, 4, 1464, "Text", "ExpressionUUID" -> \
"82eaeabf-716c-45ee-a4b9-289e6dc95cf0"],
Cell[528834, 13262, 1621, 45, 1708, "Input", "ExpressionUUID" -> \
"7746d7af-4ad7-4ebb-bc89-12a4ad3d8dc1",
 Evaluatable->False],
Cell[530458, 13309, 1726, 41, 1708, "Input", "ExpressionUUID" -> \
"b37673f2-4137-4e4e-adae-9c700f3448ee",
 Evaluatable->False],
Cell[532187, 13352, 769, 20, 1767, "Input", "ExpressionUUID" -> \
"520b3dd6-b1fc-44bb-a470-99216ef8cda2",
 Evaluatable->False],
Cell[532959, 13374, 408, 10, 1669, "Input", "ExpressionUUID" -> \
"0dd55052-7043-44b8-b396-fedb4dd42277",
 Evaluatable->False],
Cell[533370, 13386, 446, 10, 2499, "Text", "ExpressionUUID" -> \
"c48a65b3-6b66-456e-a023-4cf9da9a53db"],
Cell[533819, 13398, 130, 3, 1464, "Input", "ExpressionUUID" -> \
"99ded239-bc1a-4fed-a812-29aae7c7068b"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[533998, 13407, 609, 9, 67, "Subsection", "ExpressionUUID" -> \
"093cf404-3037-4811-bba6-b8fee14d6727"],
Cell[CellGroupData[{
Cell[534632, 13420, 334, 5, 53, "Subsubsection", "ExpressionUUID" -> \
"3156ffd5-d6ab-40fd-8ad5-173a643bf2de"],
Cell[534969, 13427, 887, 14, 3349, "Text", "ExpressionUUID" -> \
"20c9ba73-9e3d-43c2-b542-8bcddb750185"],
Cell[535859, 13443, 176, 4, 1464, "Input", "ExpressionUUID" -> \
"a092e73d-5412-442f-b207-8d39f212e761"],
Cell[536038, 13449, 2350, 47, 3427, "Text", "ExpressionUUID" -> \
"117e7905-2c79-4b02-9749-e7ad3e535bc2"],
Cell[538391, 13498, 3039, 66, 1708, "Text", "ExpressionUUID" -> \
"2bc223df-c635-4321-bbe5-b963f58cdb71"],
Cell[541433, 13566, 4242, 99, 9228, "Text", "ExpressionUUID" -> \
"ea87918c-5a25-48de-abe5-a364cd3ea116"],
Cell[545678, 13667, 1041, 33, 1591, "Input", "ExpressionUUID" -> \
"8247b670-51b3-4fd1-b930-de45227351c7",
 Evaluatable->False],
Cell[546722, 13702, 5020, 139, 7918, "Text", "ExpressionUUID" -> \
"9ee7503a-60ff-4d1f-bd48-0024848c7feb"],
Cell[551745, 13843, 2788, 43, 13710, "Text", "ExpressionUUID" -> \
"b22c783b-e63d-4cfe-8d41-e169a6089adc"],
Cell[554536, 13888, 362, 7, 2431, "Text", "ExpressionUUID" -> \
"a485dbde-48c7-4469-ad91-43233d83f70c"],
Cell[554901, 13897, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"02664ef6-cb42-4718-96f2-4032bb95f8e9"],
Cell[555079, 13903, 496, 13, 1533, "Text", "ExpressionUUID" -> \
"b13da655-c767-4b2a-bf85-b66ba29d2728"],
Cell[555578, 13918, 520, 15, 1464, "Input", "ExpressionUUID" -> \
"6d0101d5-afd6-42e0-b9fd-0643a8d3ad05"],
Cell[556101, 13935, 184, 4, 1464, "Input", "ExpressionUUID" -> \
"b8d2050b-b885-45fd-afaf-f5bc332c73b6"],
Cell[556288, 13941, 232, 5, 1464, "Text", "ExpressionUUID" -> \
"01ec1685-7afa-47d7-80cf-53871aa09dd3"],
Cell[556523, 13948, 264, 7, 1464, "Input", "ExpressionUUID" -> \
"cf7f0198-479c-40c2-aa70-d2302b71e07e"],
Cell[556790, 13957, 551, 15, 2557, "Text", "ExpressionUUID" -> \
"3f91d9fa-dab5-4695-aaa9-5b880711b521"],
Cell[557344, 13974, 363, 11, 1464, "Input", "ExpressionUUID" -> \
"2936ddc1-732b-47c2-abed-8969ede99132"],
Cell[557710, 13987, 189, 3, 1464, "Text", "ExpressionUUID" -> \
"dd2def27-90e5-4462-8fbd-b983a9b848fb"],
Cell[557902, 13992, 347, 10, 1464, "Input", "ExpressionUUID" -> \
"2a71a65a-93ae-406a-8630-e1a823af5b13"],
Cell[558252, 14004, 164, 3, 1464, "Text", "ExpressionUUID" -> \
"c1a32448-64ca-40bb-9af4-bac7399a4f56"],
Cell[558419, 14009, 268, 6, 1464, "Input", "ExpressionUUID" -> \
"068d3355-09e3-4b88-9db7-7cf3b66928e7"],
Cell[558690, 14017, 719, 12, 3427, "Text", "ExpressionUUID" -> \
"a791571a-1afd-4a05-95a9-26a73c8cdef7"],
Cell[559412, 14031, 1550, 30, 6327, "Text", "ExpressionUUID" -> \
"4830ae9f-ce28-42ff-ab20-f608094b5216"],
Cell[560965, 14063, 399, 10, 1581, "Input", "ExpressionUUID" -> \
"b28b1ec0-27f1-48a9-9d37-1d2435482987",
 Evaluatable->False],
Cell[561367, 14075, 184, 4, 1464, "Input", "ExpressionUUID" -> \
"798155c1-5804-4828-8616-86ddf74d1863"],
Cell[561554, 14081, 2781, 61, 6279, "Input", "ExpressionUUID" -> \
"a12ae26a-8ca2-4826-b310-6b879740c6c4",
 Evaluatable->False],
Cell[564338, 14144, 535, 10, 2431, "Text", "ExpressionUUID" -> \
"d62b8885-19df-4fe5-8ad7-89d736251e6b"],
Cell[564876, 14156, 407, 11, 1464, "Input", "ExpressionUUID" -> \
"06ee9352-f67d-4ca5-a91e-f73e8de0e55f"],
Cell[565286, 14169, 382, 11, 1464, "Input", "ExpressionUUID" -> \
"793175d5-60db-4ae9-bda7-9db4a76bdc5b"],
Cell[565671, 14182, 228, 5, 1464, "Text", "ExpressionUUID" -> \
"db9035dd-9a54-48b9-8ba7-33682e14804d"],
Cell[565902, 14189, 186, 4, 1464, "Input", "ExpressionUUID" -> \
"78b5502f-4876-4331-b336-68118394cc7d"],
Cell[566091, 14195, 391, 11, 1464, "Input", "ExpressionUUID" -> \
"93adcf7a-b7eb-4897-8559-5f2d9a055e5f"],
Cell[566485, 14208, 338, 8, 1464, "Text", "ExpressionUUID" -> \
"3a84f648-b44b-4c0d-97cb-3b2edbc0db52"],
Cell[566826, 14218, 563, 13, 1464, "Input", "ExpressionUUID" -> \
"c37d2151-c628-47bb-98a7-6fe810728944"],
Cell[567392, 14233, 210, 5, 1464, "Input", "ExpressionUUID" -> \
"ba25a44d-c01f-4169-bb7e-4153d441b77e"],
Cell[567605, 14240, 171, 4, 1464, "Input", "ExpressionUUID" -> \
"d21edf42-4774-4273-a3bd-0c69af5a4ba7"],
Cell[567779, 14246, 345, 9, 1464, "Input", "ExpressionUUID" -> \
"9da5338f-f6ed-4592-8b3e-f79eb6d72ad9"],
Cell[568127, 14257, 213, 5, 1464, "Input", "ExpressionUUID" -> \
"0d5f033e-31da-4af9-818d-55b5223b7821"],
Cell[568343, 14264, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"75b06aa6-e643-465b-88c2-f6637d7de6a7"],
Cell[568533, 14270, 317, 8, 1464, "Input", "ExpressionUUID" -> \
"3ff02fb1-25b9-44eb-bb65-56c5f4be610c"],
Cell[568853, 14280, 210, 5, 1464, "Input", "ExpressionUUID" -> \
"f93ddfe3-5b83-4407-b316-fce664ccba14"],
Cell[569066, 14287, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"983523cf-95ef-44d3-b050-8092d72125bd"],
Cell[569254, 14293, 178, 3, 1464, "Text", "ExpressionUUID" -> \
"a7dd1077-29ab-445d-b527-f356ffc285b8"],
Cell[569435, 14298, 1075, 28, 2460, "Input", "ExpressionUUID" -> \
"9aa0fbe0-09d1-4d4c-8604-3516f9e2385c"],
Cell[570513, 14328, 303, 6, 2431, "Text", "ExpressionUUID" -> \
"1b71103b-3e4a-4cb4-9706-986648852a74"],
Cell[570819, 14336, 182, 4, 1464, "Input", "ExpressionUUID" -> \
"4da19528-a478-47da-ba1c-8e156d8dd791"],
Cell[571004, 14342, 157, 3, 1464, "Text", "ExpressionUUID" -> \
"2d6572a5-d17f-4b7c-8de6-8089fd5032f7"],
Cell[571164, 14347, 1016, 24, 2460, "Input", "ExpressionUUID" -> \
"703c0e55-0490-460a-b642-39b0dc585e81"],
Cell[572183, 14373, 214, 5, 1464, "Input", "ExpressionUUID" -> \
"c688fa23-46f6-4569-bc22-bbad35475653"],
Cell[572400, 14380, 248, 7, 1464, "Input", "ExpressionUUID" -> \
"9426b89d-a9cf-4990-9ae5-2960219b5b3a"],
Cell[572651, 14389, 332, 9, 1464, "Input", "ExpressionUUID" -> \
"3b57150f-1c30-4ce1-ac10-7dc0a8947917"],
Cell[572986, 14400, 626, 14, 1464, "Input", "ExpressionUUID" -> \
"25a18c20-bafb-4d20-a061-c5f65ecae157"],
Cell[573615, 14416, 286, 6, 1464, "Input", "ExpressionUUID" -> \
"61a80af1-8d3a-4e19-a36d-0692620a6d27"],
Cell[573904, 14424, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"2c6e43ef-0aeb-4796-b7a3-b387fae38ede"],
Cell[574094, 14430, 356, 10, 1464, "Input", "ExpressionUUID" -> \
"1b95c7ca-31a3-43c1-913e-33fe38ea716b"],
Cell[574453, 14442, 402, 10, 1464, "Input", "ExpressionUUID" -> \
"26da4945-766d-49f2-af78-402e753061f2"],
Cell[574858, 14454, 226, 6, 1464, "Input", "ExpressionUUID" -> \
"69c4070c-5158-4ea3-87cb-86fa7eaf1a56"],
Cell[575087, 14462, 248, 7, 1464, "Input", "ExpressionUUID" -> \
"7ef659b0-927b-408d-a165-dc6d3e5065aa"],
Cell[575338, 14471, 214, 6, 1464, "Input", "ExpressionUUID" -> \
"4a18cfd4-8b20-444f-9794-e22f5f03e4aa"],
Cell[575555, 14479, 248, 7, 1464, "Input", "ExpressionUUID" -> \
"eec46947-8754-43a7-8f9b-abfac0e72dcb"],
Cell[575806, 14488, 323, 9, 1464, "Input", "ExpressionUUID" -> \
"cf2b7c79-dd86-41df-9f39-ca823932094b"],
Cell[576132, 14499, 248, 7, 1464, "Input", "ExpressionUUID" -> \
"8212bc13-ed24-4005-8e7e-5acc2070e945"],
Cell[576383, 14508, 158, 3, 1464, "Text", "ExpressionUUID" -> \
"7b753feb-d8f7-4c10-a173-d5a01f732977"],
Cell[576544, 14513, 1104, 16, 1464, "Text", "ExpressionUUID" -> \
"1bcb556a-4272-400f-84f3-be3c60f34bde"],
Cell[577651, 14531, 1744, 42, 8358, "Text", "ExpressionUUID" -> \
"0e21e038-a23e-4e93-b159-c2aa018ef44d"],
Cell[579398, 14575, 262, 7, 1503, "Text", "ExpressionUUID" -> \
"e3f22b8e-c27f-46d2-bd65-39d53bf8df9f"],
Cell[579663, 14584, 180, 4, 1464, "Input", "ExpressionUUID" -> \
"a31d4010-8701-4614-947e-e76d7e99f71a"],
Cell[579846, 14590, 347, 9, 1464, "Input", "ExpressionUUID" -> \
"a852b5ba-3a3c-49c8-a428-1b9d8bc83c80"],
Cell[580196, 14601, 392, 10, 1464, "Input", "ExpressionUUID" -> \
"f3c15fd9-95f9-469c-be5d-9f613e84bd32"],
Cell[580591, 14613, 273, 7, 1464, "Input", "ExpressionUUID" -> \
"b4953cb5-c9ed-4884-ac8d-3fc5738c9c30"],
Cell[580867, 14622, 356, 10, 1464, "Input", "ExpressionUUID" -> \
"c57e9627-81cf-409b-9303-1083c065c6f2"],
Cell[581226, 14634, 135, 3, 1464, "Text", "ExpressionUUID" -> \
"37506cdd-e4a9-48a0-bb54-59a5ea1ebf37"],
Cell[581364, 14639, 957, 18, 2431, "Text", "ExpressionUUID" -> \
"22c9a429-a827-42d9-9c2c-af1b106c4908"],
Cell[582324, 14659, 1464, 38, 3427, "Input", "ExpressionUUID" -> \
"15f8bd5e-22dc-4315-b205-ed4bbfdfb1d5"],
Cell[583791, 14699, 188, 3, 1464, "Text", "ExpressionUUID" -> \
"e9590be3-aab7-476a-8bbc-3c450ab70182"],
Cell[583982, 14704, 535, 11, 1464, "Input", "ExpressionUUID" -> \
"b27f6966-5700-4111-9134-ef65c3e6d441"],
Cell[584520, 14717, 428, 10, 1464, "Input", "ExpressionUUID" -> \
"d6d9db29-d9fc-4843-9b1f-e8e4ddf5abc1"],
Cell[584951, 14729, 270, 6, 1464, "Input", "ExpressionUUID" -> \
"a96e6eb7-df1f-4eb3-806b-6416d6fe7a0d"],
Cell[585224, 14737, 431, 11, 1464, "Input", "ExpressionUUID" -> \
"09633979-3c87-436e-a290-2ea64df9b0fb"],
Cell[585658, 14750, 571, 12, 1464, "Input", "ExpressionUUID" -> \
"58037ba5-a6e0-4228-953f-5cfd23f1e7d6"],
Cell[586232, 14764, 289, 8, 1464, "Input", "ExpressionUUID" -> \
"4ac80fcf-4761-43c8-912a-d4e05fcaa845"],
Cell[586524, 14774, 212, 3, 1464, "Text", "ExpressionUUID" -> \
"4ea11926-820a-478c-ba05-7c0ddbc11c40"],
Cell[586739, 14779, 180, 4, 1464, "Input", "ExpressionUUID" -> \
"301fa0e8-5abe-472c-955f-93a14dd5b61e"],
Cell[586922, 14785, 247, 5, 1464, "Input", "ExpressionUUID" -> \
"98bdebf2-d8f5-4d5c-87e5-564cd8a156ea"],
Cell[587172, 14792, 875, 17, 1464, "Input", "ExpressionUUID" -> \
"544c5d1f-7604-428c-97a5-2c88c16bfb2c"],
Cell[588050, 14811, 372, 8, 1464, "Input", "ExpressionUUID" -> \
"5a1e4630-4404-4b49-8cf9-590056dbaf54"],
Cell[588425, 14821, 503, 12, 1464, "Input", "ExpressionUUID" -> \
"275b53fc-b6cf-48ae-bcbd-90bdfd8e96b6"],
Cell[588931, 14835, 470, 8, 2431, "Text", "ExpressionUUID" -> \
"fb2043b1-8df0-48fe-bbe8-942019cd9f07"],
Cell[589404, 14845, 609, 14, 1464, "Input", "ExpressionUUID" -> \
"5009f4c0-10c6-40b0-a4a4-15d766ee92df"],
Cell[590016, 14861, 186, 4, 1464, "Input", "ExpressionUUID" -> \
"b59fcdd8-1461-45b6-b02a-de12c85b763a"],
Cell[590205, 14867, 279, 8, 1464, "Input", "ExpressionUUID" -> \
"d2491b7c-51b3-4586-8ac4-770cd9e9c3d7"],
Cell[590487, 14877, 180, 4, 1464, "Input", "ExpressionUUID" -> \
"7af8418d-7477-460d-bc84-e4e564027457"],
Cell[590670, 14883, 343, 8, 1464, "Input", "ExpressionUUID" -> \
"7be178b9-1d06-4c5d-b6eb-dd614d8434ca"],
Cell[591016, 14893, 352, 7, 1464, "Input", "ExpressionUUID" -> \
"594ce1e2-1af2-4bef-b926-0b07b66d43de"],
Cell[591371, 14902, 157, 3, 1464, "Text", "ExpressionUUID" -> \
"7e9107a0-d0de-4807-b7de-8e3b4dbd2afc"],
Cell[591531, 14907, 365, 11, 1464, "Text", "ExpressionUUID" -> \
"13577411-c5f2-41b6-a1ae-ec065d4cf8e1"],
Cell[591899, 14920, 289, 5, 1464, "Input", "ExpressionUUID" -> \
"60011c43-5450-45c2-9a01-8824b449c730"],
Cell[592191, 14927, 228, 5, 1464, "Text", "ExpressionUUID" -> \
"dd4d82a4-efb0-4729-8f8e-c8e30d31333e"],
Cell[592422, 14934, 392, 9, 1464, "Input", "ExpressionUUID" -> \
"57bc9335-c016-44a9-92b1-dee0e65a6633"],
Cell[592817, 14945, 381, 8, 1464, "Input", "ExpressionUUID" -> \
"171289e8-e7c2-4387-a43a-a1d82b123ca1"],
Cell[593201, 14955, 324, 9, 1464, "Input", "ExpressionUUID" -> \
"e5ba6ea3-e4e0-4636-9cbc-bb7a59c8dab4"],
Cell[593528, 14966, 458, 10, 1464, "Input", "ExpressionUUID" -> \
"90df1b1e-f263-4eab-b244-cf7ae9ec031f"],
Cell[593989, 14978, 236, 5, 1464, "Input", "ExpressionUUID" -> \
"1247936d-a29e-4635-9397-17ff9b1c91b3"],
Cell[594228, 14985, 378, 10, 1464, "Input", "ExpressionUUID" -> \
"372a70eb-c593-4b9b-844d-3410aaf79db4"],
Cell[594609, 14997, 211, 5, 1464, "Text", "ExpressionUUID" -> \
"9240e47d-4ba4-479b-8277-841bcf2522ec"],
Cell[594823, 15004, 572, 17, 1464, "Input", "ExpressionUUID" -> \
"a7dd1885-c556-488c-839b-4f32e6eaefb3"],
Cell[595398, 15023, 505, 14, 1464, "Input", "ExpressionUUID" -> \
"fea4cc2c-8459-43d4-864b-0501a3f0b294"],
Cell[595906, 15039, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"437320f5-5f07-4dd6-b2eb-433eaa314409"],
Cell[596094, 15045, 157, 3, 1464, "Text", "ExpressionUUID" -> \
"2a1645c0-9d04-41ad-b457-0ad36044dc4b"],
Cell[596254, 15050, 193, 3, 1464, "Text", "ExpressionUUID" -> \
"4b5c28da-aa01-4226-8a90-cc4fc0bce4bb"],
Cell[596450, 15055, 200, 4, 1464, "Input", "ExpressionUUID" -> \
"fcb5dd19-501d-4941-8ed6-99f8972d1d7b"],
Cell[596653, 15061, 220, 5, 1464, "Text", "ExpressionUUID" -> \
"552fbba7-7979-4583-a1c3-f59a2e6dd474"],
Cell[596876, 15068, 245, 5, 1464, "Input", "ExpressionUUID" -> \
"b4f01a6d-64f6-4847-a1df-0a6a489e69b0"]
}, Closed]],
Cell[CellGroupData[{
Cell[597158, 15078, 618, 9, 42, "Subsubsection", "ExpressionUUID" -> \
"30ea83d1-6b9b-46ca-bace-e28b6c7e1865"],
Cell[597779, 15089, 400, 8, 3427, "Text", "ExpressionUUID" -> \
"3f27dc33-1e6b-407b-9215-2cb4b162218e"],
Cell[598182, 15099, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"8ba4d4be-7415-4ab0-83eb-583d39c78446"],
Cell[598360, 15105, 290, 8, 1464, "Text", "ExpressionUUID" -> \
"bfe078c0-2d7f-4c04-9f7d-14f749b0771b"],
Cell[598653, 15115, 247, 5, 1464, "Input", "ExpressionUUID" -> \
"1c6bf720-9038-4dfb-98d0-700a4bb3155a"],
Cell[598903, 15122, 619, 16, 1464, "Input", "ExpressionUUID" -> \
"9f1fde78-9d33-4891-97d4-d75231edf67c"],
Cell[599525, 15140, 159, 4, 1464, "Input", "ExpressionUUID" -> \
"bfb46763-6664-492a-a3e8-400e21072512"],
Cell[599687, 15146, 236, 4, 1464, "Text", "ExpressionUUID" -> \
"fe2f7234-0018-4c50-9f30-01566df9b3c9"],
Cell[599926, 15152, 273, 8, 1464, "Input", "ExpressionUUID" -> \
"5247c221-a1ec-4bb3-89f2-74a0be8ca142"],
Cell[600202, 15162, 220, 5, 1464, "Input", "ExpressionUUID" -> \
"c85b4acc-0a96-427c-9906-804c57594e7a"],
Cell[600425, 15169, 199, 3, 1464, "Text", "ExpressionUUID" -> \
"d2685cbf-7ad7-4670-a837-262f29bae5a4"],
Cell[600627, 15174, 189, 4, 1464, "Input", "ExpressionUUID" -> \
"da380492-ba0a-435b-9678-1292a44e3da2"],
Cell[600819, 15180, 319, 7, 2460, "Text", "ExpressionUUID" -> \
"596e031e-79ac-4ac0-8ab1-1751e9080222"],
Cell[601141, 15189, 895, 16, 1464, "Input", "ExpressionUUID" -> \
"4de4c56c-6b3b-4fa5-a13a-c9e217248659"],
Cell[602039, 15207, 186, 4, 1464, "Input", "ExpressionUUID" -> \
"e1c5b9ac-938d-4ae3-bf68-321a8294e5b2"],
Cell[602228, 15213, 202, 3, 1464, "Text", "ExpressionUUID" -> \
"0c5addfe-836f-428e-a39a-bdfaa0e56f23"],
Cell[602433, 15218, 247, 7, 1464, "Input", "ExpressionUUID" -> \
"f058e315-83aa-475a-b9f5-a143a6d7ecd5"],
Cell[602683, 15227, 189, 4, 1464, "Input", "ExpressionUUID" -> \
"8717f4c5-11fd-4af2-85d0-1e12fee86a91"],
Cell[602875, 15233, 238, 4, 1464, "Text", "ExpressionUUID" -> \
"8462682c-a212-48d3-bae9-8c425cd95838"],
Cell[603116, 15239, 1141, 23, 2460, "Input", "ExpressionUUID" -> \
"ee2e791e-35ba-4a77-9462-7bec5b87b412"],
Cell[604260, 15264, 285, 5, 1464, "Input", "ExpressionUUID" -> \
"fe7e5504-0aaf-4f64-85aa-fc7401e5d830"],
Cell[604548, 15271, 186, 3, 1464, "Text", "ExpressionUUID" -> \
"ef9da763-626c-4cf7-b675-f032f8186fa2"],
Cell[604737, 15276, 296, 8, 1464, "Input", "ExpressionUUID" -> \
"d3d077d1-64e9-498e-8278-8db962a811c6"],
Cell[605036, 15286, 198, 4, 1464, "Input", "ExpressionUUID" -> \
"f36f7046-eeaa-4252-a00f-a53d5e277ca5"],
Cell[605237, 15292, 184, 4, 1464, "Input", "ExpressionUUID" -> \
"f7bce746-9b6f-468e-b307-8ad8cc2cc1da"],
Cell[605424, 15298, 285, 7, 1464, "Text", "ExpressionUUID" -> \
"a3d21d76-94a2-47df-8b72-5422e9c5eb6a"],
Cell[605712, 15307, 179, 4, 1464, "Input", "ExpressionUUID" -> \
"f47eaad9-29d3-458c-a294-0e9fa58ebec1"],
Cell[605894, 15313, 265, 7, 1464, "Text", "ExpressionUUID" -> \
"b2ef2566-243d-4a1f-bea9-f36d5fddf120"],
Cell[606162, 15322, 239, 6, 1464, "Input", "ExpressionUUID" -> \
"9a1ea5e6-74fa-4431-84ef-8586a7b85f8c"],
Cell[606404, 15330, 172, 3, 1464, "Text", "ExpressionUUID" -> \
"3c08ad06-c3f6-4905-bac7-a2387c7eaca5"],
Cell[606579, 15335, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"5eab7dda-76a8-42a5-a7ee-24366e2477a3"],
Cell[606755, 15341, 179, 4, 1464, "Input", "ExpressionUUID" -> \
"475bc898-03c5-4b14-ae15-d8652874048b"],
Cell[606937, 15347, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"ec0fff87-28cf-472c-a6e5-003e56d0e8fe"],
Cell[607125, 15353, 236, 5, 1464, "Input", "ExpressionUUID" -> \
"b1995bab-81d6-432b-8485-faf817142528"],
Cell[607364, 15360, 282, 6, 1464, "Input", "ExpressionUUID" -> \
"568fd2e4-77b4-488a-99ac-1a237f8199b0"],
Cell[607649, 15368, 225, 3, 1464, "Text", "ExpressionUUID" -> \
"c8cf9ab6-00c4-40ed-a720-2ac92992330c"],
Cell[607877, 15373, 269, 5, 1464, "Input", "ExpressionUUID" -> \
"ed099c49-4fe4-4a5e-b843-221f567c7d30"],
Cell[608149, 15380, 157, 3, 1464, "Text", "ExpressionUUID" -> \
"7bb33fa7-b4e9-4dbe-a7a3-0411a8b28330"],
Cell[608309, 15385, 273, 5, 1464, "Input", "ExpressionUUID" -> \
"e954fa52-09d8-4743-9beb-3a2d11c32ecf"],
Cell[608585, 15392, 193, 4, 1464, "Input", "ExpressionUUID" -> \
"8a96c9de-1213-4fa6-a477-bf45a463ee29"],
Cell[608781, 15398, 244, 6, 2431, "Text", "ExpressionUUID" -> \
"e62402db-7be2-4370-a4f5-0c0c21d13b4a"],
Cell[609028, 15406, 184, 4, 1464, "Input", "ExpressionUUID" -> \
"112baf00-a63a-4d9d-839f-e7af4f7bf196"],
Cell[609215, 15412, 428, 8, 3349, "Text", "ExpressionUUID" -> \
"c63c3d7a-09fd-4192-b7c1-065b8f807cdb"],
Cell[609646, 15422, 715, 15, 1464, "Input", "ExpressionUUID" -> \
"d86dcb7b-4e68-49fa-872d-461827b0d582"],
Cell[610364, 15439, 182, 4, 1464, "Input", "ExpressionUUID" -> \
"3a3f7003-52b3-449d-9fc7-904614e2e388"],
Cell[610549, 15445, 213, 4, 1464, "Input", "ExpressionUUID" -> \
"3cba6f9e-0f5f-4bc4-90b7-4b9141d9b312"],
Cell[610765, 15451, 189, 4, 1464, "Input", "ExpressionUUID" -> \
"5c80a848-0a2e-4006-bd4f-602db235db5f"],
Cell[610957, 15457, 664, 11, 4404, "Text", "ExpressionUUID" -> \
"5a7a946e-f919-4cfe-8977-c967dbbb55b9"],
Cell[611624, 15470, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"406a219d-90ec-4d08-b92d-f2046763d080"],
Cell[611800, 15476, 217, 5, 1464, "Input", "ExpressionUUID" -> \
"61fb1dd0-be63-4134-bf2e-b2782e4833c2"],
Cell[612020, 15483, 202, 3, 1464, "Text", "ExpressionUUID" -> \
"9653728d-3b47-4679-a49f-f730ac76b1bf"],
Cell[612225, 15488, 403, 11, 1464, "Input", "ExpressionUUID" -> \
"d6e9e28b-e2e1-4b67-b259-541f95660446"],
Cell[612631, 15501, 307, 6, 1464, "Text", "ExpressionUUID" -> \
"87457212-effe-4e30-9e71-9c53b847b3e3"],
Cell[612941, 15509, 713, 17, 1464, "Input", "ExpressionUUID" -> \
"75a78d40-52c8-43fe-87ab-ece11d2aca9d"],
Cell[613657, 15528, 240, 5, 1464, "Input", "ExpressionUUID" -> \
"a80da443-6007-45a1-b965-88443bc7424b"],
Cell[613900, 15535, 273, 6, 1464, "Input", "ExpressionUUID" -> \
"2bb34560-24cd-44d3-88ab-1f15afc63832"],
Cell[614176, 15543, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"2cb77d44-d05e-4b8e-894f-b9a66bcab571"],
Cell[614364, 15549, 212, 4, 1464, "Input", "ExpressionUUID" -> \
"d9fd8f9e-a860-4bb4-acfd-fe66a815130b"],
Cell[614579, 15555, 265, 5, 1464, "Input", "ExpressionUUID" -> \
"0654f70d-9db9-4790-b20c-73c446c91351"],
Cell[614847, 15562, 301, 8, 1464, "Input", "ExpressionUUID" -> \
"51d72c4b-b809-4cae-89cf-1082cdb4d60b"],
Cell[615151, 15572, 245, 5, 1464, "Input", "ExpressionUUID" -> \
"4ce908a8-ab29-4869-8dd8-5531f8e48c39"],
Cell[615399, 15579, 184, 4, 1464, "Input", "ExpressionUUID" -> \
"feaa0509-9920-4b3e-9334-1d2de0a23fb5"],
Cell[615586, 15585, 213, 5, 1464, "Input", "ExpressionUUID" -> \
"fe590d4e-3bab-44a7-ba98-e516ebdd799e"],
Cell[615802, 15592, 233, 6, 1464, "Text", "ExpressionUUID" -> \
"9a21e201-2fe0-4ea2-8bdd-5d0bb2b8c4cb"],
Cell[616038, 15600, 283, 7, 1464, "Input", "ExpressionUUID" -> \
"37bc547c-680f-4db6-8272-391e008845f0"],
Cell[616324, 15609, 180, 3, 1464, "Text", "ExpressionUUID" -> \
"0f7c00c5-2998-4d04-86d9-dc7b7ac3432a"],
Cell[616507, 15614, 240, 7, 1464, "Input", "ExpressionUUID" -> \
"47a4e1f6-c798-4616-bc63-8bfd5fe6d338"],
Cell[616750, 15623, 331, 6, 1464, "Input", "ExpressionUUID" -> \
"f754692e-4e6c-4166-af21-1b7c0f635b91"]
}, Closed]],
Cell[CellGroupData[{
Cell[617118, 15634, 440, 7, 42, "Subsubsection", "ExpressionUUID" -> \
"cca10944-0ff7-4d52-af43-fd3f72025de0"],
Cell[617561, 15643, 1525, 39, 3553, "Text", "ExpressionUUID" -> \
"af53f264-8cb6-4ec9-81b5-4dc372e56d54"],
Cell[619089, 15684, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"15a20442-5a70-4daa-a8fa-3ae3cf37f514"],
Cell[619265, 15690, 238, 6, 1464, "Input", "ExpressionUUID" -> \
"9f054607-1f8e-4240-ba86-f93203ab898b"],
Cell[619506, 15698, 619, 15, 1464, "Input", "ExpressionUUID" -> \
"8d5ecbd8-7453-46d7-8982-9c8f29b93cd2"],
Cell[620128, 15715, 278, 4, 1464, "Text", "ExpressionUUID" -> \
"200c5a41-db1c-4d7a-833f-4cdc80012192"],
Cell[620409, 15721, 512, 10, 1464, "Input", "ExpressionUUID" -> \
"5eb6f2bc-41ec-43ea-a97b-15a4503b5523"],
Cell[620924, 15733, 247, 7, 1464, "Input", "ExpressionUUID" -> \
"13571538-53ad-4671-a4a8-b66ae391e66b"],
Cell[621174, 15742, 188, 3, 1464, "Text", "ExpressionUUID" -> \
"3225d699-1c40-4de4-8384-0c678353402e"],
Cell[621365, 15747, 290, 5, 1464, "Input", "ExpressionUUID" -> \
"d6aabe8c-2e5a-4df9-bc2e-7dfdcbc90363"],
Cell[621658, 15754, 144, 3, 1464, "Text", "ExpressionUUID" -> \
"d93e486c-d845-4889-9abc-da7f1ebdc3ed"],
Cell[621805, 15759, 186, 4, 1464, "Input", "ExpressionUUID" -> \
"4415a86d-3d14-41e9-b6c3-441422cf43d0"],
Cell[621994, 15765, 175, 3, 1464, "Text", "ExpressionUUID" -> \
"07fd3d0f-0055-4a8e-a666-90ac1857c9a0"],
Cell[622172, 15770, 385, 9, 1464, "Input", "ExpressionUUID" -> \
"a0d2a35e-e2cc-4524-af97-d0ff0ecb3803"],
Cell[622560, 15781, 174, 3, 1464, "Text", "ExpressionUUID" -> \
"184c5d99-9f31-4898-8c76-7a138883bda9"],
Cell[622737, 15786, 208, 5, 1464, "Input", "ExpressionUUID" -> \
"ba6c5ca4-db96-4a40-b59e-7e197f656aac"],
Cell[622948, 15793, 171, 3, 1464, "Text", "ExpressionUUID" -> \
"807e8db8-d458-4fe0-b00d-15aa83e95c62"],
Cell[623122, 15798, 853, 21, 2460, "Input", "ExpressionUUID" -> \
"5c014661-36a1-4f45-84ec-b23e12e9aeb4"],
Cell[623978, 15821, 212, 4, 1464, "Input", "ExpressionUUID" -> \
"20e82aae-3628-4e19-a4fd-2bed51d311b7"],
Cell[624193, 15827, 260, 6, 1464, "Input", "ExpressionUUID" -> \
"d652a3f8-8b09-4e64-8265-ddea12f070a2"],
Cell[624456, 15835, 167, 3, 1464, "Text", "ExpressionUUID" -> \
"a17d9ddd-c21b-42a4-8f36-acfe31df0c0e"],
Cell[624626, 15840, 761, 16, 1464, "Input", "ExpressionUUID" -> \
"2f29c827-38dc-4dcb-bc1b-065216a736e2"],
Cell[625390, 15858, 267, 5, 1464, "Input", "ExpressionUUID" -> \
"2dc44828-feeb-4c4d-8b5e-c3132b752abc"],
Cell[625660, 15865, 172, 3, 1464, "Text", "ExpressionUUID" -> \
"619212b3-4044-4b6b-8b1a-3d26593acf3f"],
Cell[625835, 15870, 310, 6, 1464, "Input", "ExpressionUUID" -> \
"447eef19-4f4f-4209-a363-fde47665bc06"],
Cell[626148, 15878, 190, 3, 1464, "Text", "ExpressionUUID" -> \
"57543813-4634-49a5-8e21-7d9cd6099674"],
Cell[626341, 15883, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"6d2d987e-93be-42ee-913e-0d440041f67b"],
Cell[626531, 15889, 320, 6, 1464, "Text", "ExpressionUUID" -> \
"98976727-3fbe-49a7-a096-05d4df5a09f3"],
Cell[626854, 15897, 260, 5, 1464, "Input", "ExpressionUUID" -> \
"f365051b-bbd5-4a8b-96fe-9761fd85fa69"],
Cell[627117, 15904, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"e00c5af2-e9eb-4f68-bd66-77e30f554182"],
Cell[627307, 15910, 435, 10, 1464, "Input", "ExpressionUUID" -> \
"5ce94aff-d033-482c-9b85-3f0fc419c131"],
Cell[627745, 15922, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"ae9b9014-77ca-4786-bdec-95410a75e306"],
Cell[627933, 15928, 1618, 36, 4432, "Text", "ExpressionUUID" -> \
"5ce97ab5-91db-4595-bb0b-11d0c53e2dc7"],
Cell[629554, 15966, 369, 9, 2431, "Text", "ExpressionUUID" -> \
"b7b7b290-f809-4fa7-9d17-ad4449d65cab"],
Cell[629926, 15977, 527, 14, 1464, "Input", "ExpressionUUID" -> \
"d7945e21-c0be-4946-83ed-7af69c14ecb6"],
Cell[630456, 15993, 186, 4, 1464, "Input", "ExpressionUUID" -> \
"c4f415b4-f936-47c0-97a7-dcd71e64db1a"],
Cell[630645, 15999, 274, 6, 1464, "Input", "ExpressionUUID" -> \
"e8e4245e-353f-4727-84bd-3fa6613d6f35"],
Cell[630922, 16007, 212, 4, 1464, "Input", "ExpressionUUID" -> \
"e19118a3-c07b-40c4-8297-76090a6af51b"],
Cell[631137, 16013, 414, 11, 1464, "Input", "ExpressionUUID" -> \
"35ea65a7-d6ed-4ef6-9529-f2a7a52e3bf1"],
Cell[631554, 16026, 706, 13, 2460, "Text", "ExpressionUUID" -> \
"26f0526e-0286-4442-b392-9bce877a1f2c"],
Cell[632263, 16041, 419, 8, 2431, "Text", "ExpressionUUID" -> \
"083c501e-9f79-49dd-8d83-65258cbaf13c"],
Cell[632685, 16051, 201, 4, 1464, "Input", "ExpressionUUID" -> \
"f6ed04b5-299d-4909-bb31-df1706073f07"],
Cell[632889, 16057, 319, 7, 1464, "Input", "ExpressionUUID" -> \
"755d4fed-feaa-49c7-a17b-5f5eb9860138"],
Cell[633211, 16066, 896, 18, 1464, "Input", "ExpressionUUID" -> \
"96564f62-e20f-4c64-a7e0-8290266535a7",
 InitializationCell->True],
Cell[634110, 16086, 519, 11, 3427, "Input", "ExpressionUUID" -> \
"93ec5fb0-713e-4989-962c-31d4ab258cd3",
 InitializationCell->True],
Cell[634632, 16099, 463, 11, 1464, "Input", "ExpressionUUID" -> \
"bd885846-6f0f-448d-973d-ed409009d603"],
Cell[635098, 16112, 186, 4, 1464, "Input", "ExpressionUUID" -> \
"8b60cd5e-986d-47bc-a5cb-874ad54c977e"],
Cell[635287, 16118, 484, 11, 1503, "Text", "ExpressionUUID" -> \
"29d23457-1fc2-4585-9f05-0895f05b222f"],
Cell[635774, 16131, 962, 21, 1464, "Input", "ExpressionUUID" -> \
"a3d93c5f-2bbe-4b5c-97df-6a24d8c40327"],
Cell[636739, 16154, 259, 6, 2431, "Text", "ExpressionUUID" -> \
"56c0b95b-a738-4656-b7f8-5d340b376c77"],
Cell[637001, 16162, 307, 6, 1464, "Input", "ExpressionUUID" -> \
"79c211b6-abca-4979-95a9-eda40dc175b7"],
Cell[637311, 16170, 930, 15, 5224, "Text", "ExpressionUUID" -> \
"764513ab-4286-49c1-a3e0-195814617e27"],
Cell[638244, 16187, 497, 11, 1464, "Input", "ExpressionUUID" -> \
"47d8779b-6b72-4851-8f50-f92e0634eefe"]
}, Closed]],
Cell[CellGroupData[{
Cell[638778, 16203, 526, 8, 42, "Subsubsection", "ExpressionUUID" -> \
"5c24a4a7-efb4-44db-898c-7c67905ceadf"],
Cell[639307, 16213, 239, 4, 1464, "Text", "ExpressionUUID" -> \
"b132afd9-fca9-4865-8817-8ff9fcec7ee7"],
Cell[639549, 16219, 195, 5, 1464, "Input", "ExpressionUUID" -> \
"7c4cd890-3187-4383-aeac-6650d659e293"],
Cell[639747, 16226, 343, 9, 1464, "Input", "ExpressionUUID" -> \
"409e115b-c4e7-4dea-b179-bc25ab07d971"],
Cell[640093, 16237, 305, 8, 1464, "Input", "ExpressionUUID" -> \
"fb04d67d-c1b1-4452-bc8b-42c9c8821172"],
Cell[640401, 16247, 306, 9, 1464, "Input", "ExpressionUUID" -> \
"7e12e89b-bbec-4dfb-a4ff-a427f2bb22eb"],
Cell[640710, 16258, 383, 10, 1464, "Input", "ExpressionUUID" -> \
"4b78b8ff-cfa1-45f2-bf92-51c355d18df4"],
Cell[641096, 16270, 986, 23, 2460, "Text", "ExpressionUUID" -> \
"7646310c-b6f2-4e2b-9ed5-a1dc56e1a43d"],
Cell[642085, 16295, 1107, 26, 3427, "Text", "ExpressionUUID" -> \
"3155f4d4-d39a-4081-9a63-d38f360bf048"],
Cell[643195, 16323, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"c4be2812-b797-4237-9503-c70b4c2d5f3d"],
Cell[643372, 16329, 2371, 47, 3427, "Input", "ExpressionUUID" -> \
"bc1035cc-7df0-43a5-9ad9-1ff1a2135896",
 Evaluatable->False,
 InitializationCell->True],
Cell[645746, 16378, 349, 9, 1464, "Input", "ExpressionUUID" -> \
"e2376718-0015-441f-8eb6-eeb8a8d3f9ce"],
Cell[646098, 16389, 192, 3, 1464, "Text", "ExpressionUUID" -> \
"c0a36432-c9f9-4bac-985b-dfc9e50ccd1e"],
Cell[646293, 16394, 569, 14, 2460, "Input", "ExpressionUUID" -> \
"0309cd08-a77c-4f79-8a15-fa8189e06c05"],
Cell[646865, 16410, 217, 7, 1464, "Text", "ExpressionUUID" -> \
"091a1bc5-f1bb-4f8e-ad90-57214a072e7e"],
Cell[647085, 16419, 359, 10, 1464, "Input", "ExpressionUUID" -> \
"eb68afe3-bd50-4848-a1d4-334730874dbc"],
Cell[647447, 16431, 379, 10, 1464, "Input", "ExpressionUUID" -> \
"91c92ed2-46e8-438b-a870-85b262af59c0"],
Cell[647829, 16443, 1059, 26, 2460, "Input", "ExpressionUUID" -> \
"25353286-6c27-4885-aca9-d796db1751a3"],
Cell[648891, 16471, 457, 10, 1464, "Input", "ExpressionUUID" -> \
"5d06df75-0ee6-40b1-822e-7aafeb4f11d0"],
Cell[649351, 16483, 1203, 29, 2460, "Input", "ExpressionUUID" -> \
"da73a1bc-0636-42be-b7b0-4fe93794b60f"],
Cell[650557, 16514, 1439, 30, 2460, "Input", "ExpressionUUID" -> \
"c18a4bb1-dfab-4cfa-8c79-9cf2d4947bd5"],
Cell[651999, 16546, 166, 3, 1464, "Text", "ExpressionUUID" -> \
"2d5ef983-fcc0-404b-8b1e-f030842842cd"],
Cell[652168, 16551, 1137, 22, 3427, "Text", "ExpressionUUID" -> \
"0d72ef77-11f3-42ed-8a73-8c2f7d5c2b18"],
Cell[653308, 16575, 1614, 41, 4365, "Input", "ExpressionUUID" -> \
"241d686e-eba9-4ec3-9590-8301db9e6f92"],
Cell[654925, 16618, 1214, 30, 3514, "Text", "ExpressionUUID" -> \
"20909807-f591-4868-8e58-275f046f6ed2"],
Cell[656142, 16650, 243, 5, 1464, "Input", "ExpressionUUID" -> \
"f1d864e4-6026-4caa-b60a-9190b73c744c"],
Cell[656388, 16657, 161, 3, 1464, "Text", "ExpressionUUID" -> \
"07883c47-6a04-4f46-af1a-1bfae2b8e410"],
Cell[656552, 16662, 429, 13, 1464, "Input", "ExpressionUUID" -> \
"1d8ac57a-affe-477f-8e8c-012582bd723f",
 Evaluatable->False],
Cell[656984, 16677, 315, 5, 1464, "Text", "ExpressionUUID" -> \
"88fde5cd-ba81-4e15-9d3a-60ba69b3d781"],
Cell[657302, 16684, 391, 9, 2431, "Text", "ExpressionUUID" -> \
"3e92c122-57eb-4b58-91b8-9ad76f2f3511"],
Cell[657696, 16695, 1095, 33, 1533, "Text", "ExpressionUUID" -> \
"49d0b09b-5ff8-49f1-99d4-2db1ada947b8"],
Cell[658794, 16730, 3004, 61, 14188, "Text", "ExpressionUUID" -> \
"fe96da2f-d15e-4896-a86a-a1ec6de564d1"],
Cell[661801, 16793, 342, 5, 1464, "Text", "ExpressionUUID" -> \
"bf23632c-e095-4fdd-add7-c6b3e06c3e29"],
Cell[662146, 16800, 684, 16, 1464, "Input", "ExpressionUUID" -> \
"d10e1261-0436-4e8b-a5f5-a65252ce64b9"],
Cell[662833, 16818, 1101, 25, 2460, "Input", "ExpressionUUID" -> \
"3f898990-4023-4b1b-acd3-37d482250088"],
Cell[663937, 16845, 240, 5, 1464, "Text", "ExpressionUUID" -> \
"913aa89b-d2e8-4c95-ae83-34e9cb33a901"],
Cell[664180, 16852, 707, 16, 1464, "Input", "ExpressionUUID" -> \
"460cd8b8-43a4-446c-b077-e6b30a0a12a6"],
Cell[664890, 16870, 519, 10, 2431, "Text", "ExpressionUUID" -> \
"ced227ab-8e15-49e6-a046-5bd57560aa4f"],
Cell[665412, 16882, 770, 21, 2460, "Input", "ExpressionUUID" -> \
"c93d0da1-4c27-4b4a-b8fd-804db86d03d8"],
Cell[666185, 16905, 474, 10, 1464, "Text", "ExpressionUUID" -> \
"54414ea9-b903-4017-82ff-2e6a012acf49"],
Cell[666662, 16917, 198, 5, 1464, "Input", "ExpressionUUID" -> \
"e60494a9-801b-40f4-9999-a189f663d345"],
Cell[666863, 16924, 860, 18, 1464, "Input", "ExpressionUUID" -> \
"fa448ddf-30a9-4f0c-a9a9-3a2852bd653b"],
Cell[667726, 16944, 499, 11, 3349, "Text", "ExpressionUUID" -> \
"830e37bb-39fe-458c-956b-b25f6b47ac73"],
Cell[668228, 16957, 1613, 43, 4365, "Input", "ExpressionUUID" -> \
"007c2801-612a-43cc-8982-b0523d2ed67d",
 Evaluatable->False],
Cell[669844, 17002, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"e2cb7b36-0981-4670-a921-6665f0f381cc"],
Cell[670022, 17008, 501, 13, 1503, "Text", "ExpressionUUID" -> \
"53d841e9-fb94-4bd4-bb64-fd2b25960eb3"],
Cell[670526, 17023, 233, 4, 1464, "Text", "ExpressionUUID" -> \
"df30b744-ed5a-4a76-a289-4c9ffd7dca22"],
Cell[670762, 17029, 563, 13, 1464, "Input", "ExpressionUUID" -> \
"5aa8ec5b-9e63-4c41-94d3-164254a2051c"],
Cell[671328, 17044, 409, 9, 1464, "Input", "ExpressionUUID" -> \
"89542d7c-82c9-46c4-b770-a46014c6f91f"],
Cell[671740, 17055, 602, 17, 1464, "Input", "ExpressionUUID" -> \
"cbcac4bf-b95d-438a-8735-b0466c70991b"],
Cell[672345, 17074, 172, 4, 1464, "Input", "ExpressionUUID" -> \
"852a4d28-11a4-4e98-b4c6-2babaa029a0e"],
Cell[672520, 17080, 622, 13, 5224, "Text", "ExpressionUUID" -> \
"1deffc93-2f8c-4180-b619-65ee4b228cf7"],
Cell[673145, 17095, 374, 6, 1464, "Text", "ExpressionUUID" -> \
"e1f24039-6c87-429f-ab12-854435f4c2ed"],
Cell[673522, 17103, 561, 16, 1464, "Input", "ExpressionUUID" -> \
"2b23253a-923f-4a40-8a58-a33051cd28a3"],
Cell[674086, 17121, 357, 6, 1464, "Text", "ExpressionUUID" -> \
"167a5190-df2f-414e-ad84-1f758e8fda1c"],
Cell[674446, 17129, 2453, 57, 7245, "Text", "ExpressionUUID" -> \
"3665eeb7-5740-4a3f-82a7-1428b3e65a25"],
Cell[676902, 17188, 1366, 24, 1444, "Text", "ExpressionUUID" -> \
"9c0db023-da41-4ab6-84a8-c5114a1b1015",
 Evaluatable->True,
 InitializationCell->True],
Cell[678271, 17214, 166, 3, 1464, "Text", "ExpressionUUID" -> \
"969983f2-1ea0-42fd-887a-a9c4ddb4edfa"],
Cell[678440, 17219, 549, 16, 1464, "Input", "ExpressionUUID" -> \
"bd4f97d8-1e50-41ce-8f55-9aab448ab44a"],
Cell[678992, 17237, 173, 3, 1464, "Text", "ExpressionUUID" -> \
"230f95f0-cb5d-4562-a783-69974597950c"],
Cell[679168, 17242, 723, 20, 2460, "Input", "ExpressionUUID" -> \
"f7c2302a-5cd0-4a19-8ae0-a10baae5ffc8"],
Cell[679894, 17264, 320, 5, 1503, "Text", "ExpressionUUID" -> \
"b29d711b-4076-453e-9573-a0dec06d4e82"],
Cell[680217, 17271, 574, 15, 1464, "Input", "ExpressionUUID" -> \
"8fd9da63-3e44-45b7-841a-231c81d3efcc"],
Cell[680794, 17288, 1094, 30, 2460, "Input", "ExpressionUUID" -> \
"2d924770-284c-4b0b-b18c-59b95994b33f"],
Cell[681891, 17320, 1112, 26, 2460, "Input", "ExpressionUUID" -> \
"a3fc2d97-b5b2-46e8-92b2-662b87073480"],
Cell[683006, 17348, 411, 10, 2431, "Text", "ExpressionUUID" -> \
"ad131832-2676-4828-a70a-ee179e3e95d8"],
Cell[683420, 17360, 126, 3, 1464, "Text", "ExpressionUUID" -> \
"8662f022-f186-4b62-a05f-b4f939a76322"],
Cell[683549, 17365, 398, 10, 1464, "Input", "ExpressionUUID" -> \
"1ff3237e-1ab0-4893-aefa-925840c480ea"],
Cell[683950, 17377, 400, 10, 1464, "Input", "ExpressionUUID" -> \
"eb7d2fee-dd45-4526-be06-bde1d1798145"],
Cell[684353, 17389, 472, 11, 1464, "Input", "ExpressionUUID" -> \
"03e02d5e-fb2b-4ed9-887b-b7241b9ed73e"],
Cell[684828, 17402, 163, 3, 1464, "Text", "ExpressionUUID" -> \
"2d208611-6658-4af8-9118-182d308c0b14"],
Cell[684994, 17407, 1052, 28, 2460, "Input", "ExpressionUUID" -> \
"d68fb522-b8bf-4cc1-87e6-9d263edaa1b5"],
Cell[686049, 17437, 434, 8, 1503, "Text", "ExpressionUUID" -> \
"84cfda87-95c8-4600-926c-006a0cf62fa6"],
Cell[686486, 17447, 453, 9, 1464, "Input", "ExpressionUUID" -> \
"974de821-9faf-4fc1-97b0-7eb98496e636"]
}, Closed]],
Cell[CellGroupData[{
Cell[686976, 17461, 757, 11, 42, "Subsubsection", "ExpressionUUID" -> \
"deb96fd6-7865-420b-a5ec-796f17959ce5"],
Cell[687736, 17474, 178, 3, 1464, "Text", "ExpressionUUID" -> \
"2f84f1c2-9701-48e2-b3bc-126a9ea17714"],
Cell[687917, 17479, 157, 3, 1464, "Input", "ExpressionUUID" -> \
"52df7798-ad6d-4f5b-a4ed-19c0f8f41be4"],
Cell[688077, 17484, 735, 17, 1464, "Input", "ExpressionUUID" -> \
"9972e8ff-d9d2-4a7a-9523-21cb3955dc47"],
Cell[688815, 17503, 379, 8, 1464, "Input", "ExpressionUUID" -> \
"17cf600e-f4c7-4627-b63c-3f77ef3cdbd5"],
Cell[689197, 17513, 170, 3, 1464, "Text", "ExpressionUUID" -> \
"b5e07a2c-8cf8-4196-acff-a25ad7390f94"],
Cell[689370, 17518, 180, 4, 1464, "Input", "ExpressionUUID" -> \
"008ccfb7-bc1e-40fa-99d1-dab59eb09c0b"],
Cell[689553, 17524, 301, 7, 1464, "Input", "ExpressionUUID" -> \
"3210c851-0bb5-424a-a90e-21d4c8289776"],
Cell[689857, 17533, 203, 4, 1464, "Input", "ExpressionUUID" -> \
"661d65e5-1733-4269-b4cc-458f89c94c8b"],
Cell[690063, 17539, 464, 11, 1464, "Input", "ExpressionUUID" -> \
"6be54b19-f566-433c-8e64-7d76caba0b02"],
Cell[690530, 17552, 218, 4, 1464, "Input", "ExpressionUUID" -> \
"5b2c808b-661c-4790-8bf0-83e235462869"],
Cell[690751, 17558, 779, 22, 2460, "Input", "ExpressionUUID" -> \
"81712864-4be3-4442-8ff0-7a0ad48812eb"],
Cell[691533, 17582, 467, 13, 1464, "Input", "ExpressionUUID" -> \
"0f7bef9b-dbbb-4364-a329-dea677eae86e"],
Cell[692003, 17597, 215, 5, 1464, "Text", "ExpressionUUID" -> \
"c319fe2a-296c-474d-bf10-6f12cb9f5c3a"],
Cell[692221, 17604, 710, 18, 1464, "Input", "ExpressionUUID" -> \
"dc28981c-bf3e-4b4b-b7c6-ba0d10768fbf"],
Cell[692934, 17624, 709, 18, 1464, "Input", "ExpressionUUID" -> \
"e4464476-49e2-4480-b464-a59d341733cb"],
Cell[693646, 17644, 210, 5, 1464, "Input", "ExpressionUUID" -> \
"549431f2-bc2d-4a89-9b13-e91035ca81e6"],
Cell[693859, 17651, 223, 5, 1464, "Input", "ExpressionUUID" -> \
"364ade12-7ac0-40f5-a806-3bed901823a4"],
Cell[694085, 17658, 195, 3, 1464, "Text", "ExpressionUUID" -> \
"386a88f0-39ee-4a43-9472-2242435aa375"],
Cell[694283, 17663, 483, 12, 1464, "Input", "ExpressionUUID" -> \
"9b8514dd-e091-4432-953a-bebbb58bf306"],
Cell[694769, 17677, 459, 12, 1464, "Input", "ExpressionUUID" -> \
"0f5ad227-670a-4e5a-bbb5-aef1ba12c295"],
Cell[695231, 17691, 279, 6, 1464, "Input", "ExpressionUUID" -> \
"de382ef4-a4cb-4368-a833-c3e5d2bcf4a2"],
Cell[695513, 17699, 197, 2, 1464, "Text", "ExpressionUUID" -> \
"25470bba-69fd-46cc-8a2e-6c4b28e5d9c0"],
Cell[695713, 17703, 562, 13, 1464, "Input", "ExpressionUUID" -> \
"223f3cc0-1874-44d0-8e53-5764a818e7ef"],
Cell[696278, 17718, 512, 12, 1464, "Input", "ExpressionUUID" -> \
"28684e08-8f9f-42ba-b4df-1d524b7b2fe1"],
Cell[696793, 17732, 272, 6, 1464, "Input", "ExpressionUUID" -> \
"7fcacf90-296d-461d-8aae-9be6adcfe1fb"],
Cell[697068, 17740, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"f784bb64-c835-4d90-9b8d-631968b147c2"],
Cell[697245, 17746, 525, 14, 1464, "Input", "ExpressionUUID" -> \
"cf1528c6-05c8-4a12-a914-5912fcb4b877"],
Cell[697773, 17762, 217, 5, 1464, "Text", "ExpressionUUID" -> \
"d825f16c-1325-42ab-b8f5-12787c46cc65"],
Cell[697993, 17769, 539, 15, 1464, "Input", "ExpressionUUID" -> \
"5c6eed8a-535c-4f44-92ef-e6c066d638f8"],
Cell[698535, 17786, 393, 9, 1464, "Input", "ExpressionUUID" -> \
"19eeb265-1c01-4e1d-89fd-a64d85bac786"],
Cell[698931, 17797, 286, 7, 1464, "Input", "ExpressionUUID" -> \
"470ddd12-ee2d-41d3-9280-f1d356ae1672"],
Cell[699220, 17806, 347, 8, 1464, "Input", "ExpressionUUID" -> \
"37bf2a11-be96-4de4-b8bf-2f608d4f40c7"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[699616, 17820, 1082, 15, 67, "Subsection", "ExpressionUUID" -> \
"6ffe34a5-09ee-45cf-a5a8-cc8e19e6b1f5"],
Cell[700701, 17837, 1119, 22, 180, "Text", "ExpressionUUID" -> \
"99a462a7-e692-4f51-b3e5-92b50475c105"],
Cell[701823, 17861, 173, 4, 49, "Input", "ExpressionUUID" -> \
"287f5d50-cfeb-4a2a-982d-47765ee39670"],
Cell[CellGroupData[{
Cell[702021, 17869, 1267, 19, 90, "Subsubsection", "ExpressionUUID" -> \
"62d19048-24bf-4f8b-8b0c-417b1343377a"],
Cell[703291, 17890, 2412, 52, 6249, "Text", "ExpressionUUID" -> \
"865102fa-29e3-4ed3-b85a-d7cff6c68859"],
Cell[705706, 17944, 1358, 46, 2851, "Input", "ExpressionUUID" -> \
"741a6a12-52e7-4e5f-9ea3-d42f7c8fffad",
 Evaluatable->False],
Cell[707067, 17992, 1855, 43, 3436, "Text", "ExpressionUUID" -> \
"0b1c6407-f2dc-43f9-930c-b021e0b6eaab"],
Cell[708925, 18037, 1038, 29, 1464, "Input", "ExpressionUUID" -> \
"37627a9d-46c1-480b-9a36-9590baf77bef",
 Evaluatable->False],
Cell[709966, 18068, 221, 4, 1464, "Text", "ExpressionUUID" -> \
"43fc2ecb-66ce-44c1-b0fc-eb84502398c9"],
Cell[710190, 18074, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"1b81d7d2-77d2-4a77-8d47-6f584387d7ee"],
Cell[710366, 18080, 197, 4, 1464, "Input", "ExpressionUUID" -> \
"69f19553-cf61-4e40-a028-d11b6357d894"],
Cell[710566, 18086, 296, 6, 1464, "Text", "ExpressionUUID" -> \
"4804903a-3977-4f35-9a77-14f73a280099"],
Cell[710865, 18094, 840, 27, 1464, "Input", "ExpressionUUID" -> \
"65e6955e-5c66-483f-9b70-3680932009fe",
 Evaluatable->False],
Cell[711708, 18123, 1421, 35, 2499, "Text", "ExpressionUUID" -> \
"bb78789f-9499-4e6a-ab46-06f096a38691"],
Cell[713132, 18160, 676, 22, 1591, "Input", "ExpressionUUID" -> \
"8579669f-1102-407e-8308-75951f2a0a37",
 Evaluatable->False],
Cell[713811, 18184, 169, 3, 1464, "Text", "ExpressionUUID" -> \
"00574694-3818-46b6-af5f-f28b445fd57b"],
Cell[713983, 18189, 939, 30, 2499, "Input", "ExpressionUUID" -> \
"70651000-eacf-4931-8b9c-7411a87a9177",
 Evaluatable->False],
Cell[714925, 18221, 2923, 76, 3514, "Text", "ExpressionUUID" -> \
"6fda285f-5d9c-452b-a826-c45f2348a49c"],
Cell[717851, 18299, 3326, 89, 5487, "Text", "ExpressionUUID" -> \
"b29d4197-43d6-4073-a8c8-3bcbaaf60444"],
Cell[721180, 18390, 983, 25, 2431, "Text", "ExpressionUUID" -> \
"db1ce299-ad3c-497e-a23f-aaaab13cef5c"],
Cell[722166, 18417, 701, 22, 1728, "Input", "ExpressionUUID" -> \
"b4a68a61-6ba2-48bb-9714-81d4ae56ff0f",
 Evaluatable->False],
Cell[722870, 18441, 3126, 79, 6249, "Text", "ExpressionUUID" -> \
"de0a29e3-5e70-4064-81e8-051b4fa3529f"],
Cell[725999, 18522, 547, 14, 1464, "Text", "ExpressionUUID" -> \
"ca953f36-f64c-4295-90e6-a3c8471d75f7"],
Cell[726549, 18538, 807, 24, 1464, "Input", "ExpressionUUID" -> \
"5aebcea2-dcc1-4e3b-9a96-ca86e3363a43",
 Evaluatable->False],
Cell[727359, 18564, 729, 20, 1503, "Text", "ExpressionUUID" -> \
"28fa26b1-2de7-41f3-835c-d1b419ff4755"],
Cell[728091, 18586, 576, 8, 1464, "Text", "ExpressionUUID" -> \
"9e1e2ba6-148c-4c9d-a49c-636499cd424c"],
Cell[728670, 18596, 193, 3, 1464, "Text", "ExpressionUUID" -> \
"df2ca357-5bc2-4cb4-9bfa-f09b0b867735"],
Cell[728866, 18601, 181, 4, 1464, "Input", "ExpressionUUID" -> \
"f603c444-9b35-4195-bf7c-0e6a69b1d2b1"],
Cell[729050, 18607, 162, 2, 1464, "Text", "ExpressionUUID" -> \
"8f077fe5-f5ef-4700-8d1d-04c9ac7679cd"],
Cell[729215, 18611, 155, 3, 1464, "Input", "ExpressionUUID" -> \
"6a20b488-7541-46ac-b088-be4e70b96a91"],
Cell[729373, 18616, 177, 2, 1464, "Text", "ExpressionUUID" -> \
"d5e89234-3573-4c5f-a87d-e963f91acb78"],
Cell[729553, 18620, 518, 13, 1464, "Input", "ExpressionUUID" -> \
"29b9defe-ae97-4b3d-991a-0a5d8fd2fb1a"],
Cell[730074, 18635, 172, 3, 1464, "Text", "ExpressionUUID" -> \
"cb77a343-99da-442c-a7eb-2d46e3571038"],
Cell[730249, 18640, 339, 9, 1464, "Input", "ExpressionUUID" -> \
"50347746-52e5-4623-bd9e-fc0765d5a947"],
Cell[730591, 18651, 271, 6, 1464, "Input", "ExpressionUUID" -> \
"865669ff-8a49-4469-9da0-3692aef7957e"],
Cell[730865, 18659, 162, 3, 1464, "Text", "ExpressionUUID" -> \
"ceea080b-dd9c-46f5-9d2d-f27d4695229c"],
Cell[731030, 18664, 466, 13, 1464, "Input", "ExpressionUUID" -> \
"9dbf833a-cba6-443d-8460-49970b5ec44a"],
Cell[731499, 18679, 260, 6, 1464, "Input", "ExpressionUUID" -> \
"8dd11ab6-2bf0-453c-89e4-ccad5d017b09"],
Cell[731762, 18687, 307, 5, 1464, "Text", "ExpressionUUID" -> \
"7f1ddb64-b9c6-452f-97a9-575debba8b38"],
Cell[732072, 18694, 239, 5, 1464, "Input", "ExpressionUUID" -> \
"73b0cf54-83e8-475a-a685-73d005d761cd"],
Cell[732314, 18701, 183, 4, 1464, "Input", "ExpressionUUID" -> \
"b1f34224-5e1c-4fc1-88bc-c6fdd77498f8"],
Cell[732500, 18707, 213, 5, 1464, "Text", "ExpressionUUID" -> \
"0a404402-f4c0-4da3-ac49-aeeef98e2127"],
Cell[732716, 18714, 219, 5, 1464, "Input", "ExpressionUUID" -> \
"398f578c-0a85-4763-a968-8ed982266c29"],
Cell[732938, 18721, 268, 6, 1464, "Input", "ExpressionUUID" -> \
"1a48dfc6-0d9b-46a3-a81e-c2381dc1838c"],
Cell[733209, 18729, 164, 3, 1464, "Text", "ExpressionUUID" -> \
"c9ffe83f-641b-48cc-8f6a-077b32555861"],
Cell[733376, 18734, 275, 9, 1464, "Input", "ExpressionUUID" -> \
"77e75887-37f7-412b-8d75-599af10ab443"],
Cell[733654, 18745, 162, 3, 1464, "Text", "ExpressionUUID" -> \
"dea36727-724b-41df-81a7-a4d45d0ad5ee"],
Cell[733819, 18750, 284, 7, 1464, "Input", "ExpressionUUID" -> \
"fe7ebdba-5857-41a9-bd51-ea38313c3a20"],
Cell[734106, 18759, 302, 7, 1464, "Text", "ExpressionUUID" -> \
"97e7c6b8-b30b-4559-95c0-4dfac9494869"],
Cell[734411, 18768, 236, 5, 1464, "Input", "ExpressionUUID" -> \
"cd745c01-eea0-469c-a457-c0c2a9ad861c"],
Cell[734650, 18775, 276, 7, 1464, "Input", "ExpressionUUID" -> \
"e9f42ac6-5de8-4462-8fce-035812480c86"],
Cell[734929, 18784, 411, 12, 1464, "Input", "ExpressionUUID" -> \
"d6f46f86-38d7-4834-9f23-3f2f2d8a67fd"],
Cell[735343, 18798, 402, 9, 2431, "Text", "ExpressionUUID" -> \
"87463caa-aa9c-4e18-952f-d93f5be5deb9"],
Cell[735748, 18809, 297, 7, 1464, "Input", "ExpressionUUID" -> \
"297b5a25-d4b4-4fbc-a87e-6e3871766066"],
Cell[736048, 18818, 344, 9, 1464, "Input", "ExpressionUUID" -> \
"a674dc77-66dd-44aa-b53b-b09ff5d6a8da"],
Cell[736395, 18829, 210, 5, 1464, "Input", "ExpressionUUID" -> \
"31ec0c0b-2074-4ecb-8cd6-960c46025693"],
Cell[736608, 18836, 484, 12, 1464, "Input", "ExpressionUUID" -> \
"97cd9083-a553-41b6-b5c5-a65cba584c74"],
Cell[737095, 18850, 189, 4, 1464, "Input", "ExpressionUUID" -> \
"89fc46d2-df1e-4a8b-ba8f-00a3097c628c"],
Cell[737287, 18856, 311, 6, 1464, "Input", "ExpressionUUID" -> \
"32a3bf82-0aee-44c8-a00a-10dc88886687"],
Cell[737601, 18864, 189, 4, 1464, "Input", "ExpressionUUID" -> \
"871bf5b3-9ecd-4097-aa94-0b5630ba1da6"],
Cell[737793, 18870, 239, 5, 1464, "Input", "ExpressionUUID" -> \
"252848c6-7362-44a2-86c1-7dfaa92a1239"],
Cell[738035, 18877, 276, 7, 1464, "Input", "ExpressionUUID" -> \
"8c4b561d-3f3e-4864-b27f-3928c56fb463"],
Cell[738314, 18886, 487, 12, 1464, "Input", "ExpressionUUID" -> \
"dcaf8615-697c-49c9-902e-fed3e027c084"],
Cell[738804, 18900, 155, 3, 1464, "Input", "ExpressionUUID" -> \
"73527d5f-dc3c-41cb-96da-e5ebd6bb7754"],
Cell[738962, 18905, 183, 4, 1464, "Input", "ExpressionUUID" -> \
"2a7e6297-1b2e-4a2e-903d-face4d7e296d"],
Cell[739148, 18911, 305, 5, 1464, "Input", "ExpressionUUID" -> \
"b2d0c320-2996-437e-88d1-895c1950f6cb"],
Cell[739456, 18918, 191, 4, 1464, "Input", "ExpressionUUID" -> \
"9ad57bad-45d6-4743-924c-264d654328b2"],
Cell[739650, 18924, 190, 4, 1464, "Input", "ExpressionUUID" -> \
"cddcbd10-4301-49c4-bae2-04986be234b9"],
Cell[739843, 18930, 170, 4, 1464, "Input", "ExpressionUUID" -> \
"59f54002-0ac3-4b76-a5f6-b1c3054d584a"],
Cell[740016, 18936, 269, 7, 1464, "Input", "ExpressionUUID" -> \
"e953b9a4-acbc-4916-a5c7-d9256c2f14ae"],
Cell[740288, 18945, 337, 8, 1464, "Input", "ExpressionUUID" -> \
"5e38bb58-0bfa-446b-8e6d-bce088dd2e25"],
Cell[740628, 18955, 183, 4, 1464, "Input", "ExpressionUUID" -> \
"19dfc202-e55f-4b48-b8ae-3d79413418be"],
Cell[740814, 18961, 312, 8, 1464, "Input", "ExpressionUUID" -> \
"5a14acb8-4d78-4563-ac8a-a66f12f010fe"],
Cell[741129, 18971, 238, 6, 1464, "Input", "ExpressionUUID" -> \
"439bd7d8-eccd-4254-8a14-96a0fe2f001d"],
Cell[741370, 18979, 292, 5, 1464, "Input", "ExpressionUUID" -> \
"6cf36911-8ef5-4bef-adfb-5ab52ae6a09d"],
Cell[741665, 18986, 289, 7, 1464, "Input", "ExpressionUUID" -> \
"48c88e04-02c3-4baf-8d7c-9087f0014966"],
Cell[741957, 18995, 420, 9, 1464, "Input", "ExpressionUUID" -> \
"fdfcbfe4-451c-4e50-8947-d57dffae0269"],
Cell[742380, 19006, 258, 6, 1464, "Input", "ExpressionUUID" -> \
"debc4168-3e7f-4cb3-8043-6b951e6da047"],
Cell[742641, 19014, 270, 4, 1464, "Text", "ExpressionUUID" -> \
"10ea368e-91da-40c5-b35e-db758b5280fe"],
Cell[742914, 19020, 219, 4, 1464, "Input", "ExpressionUUID" -> \
"dd78145a-0554-414b-bb5f-05c0c3e35e27"],
Cell[743136, 19026, 183, 4, 1464, "Input", "ExpressionUUID" -> \
"f7ebf4a1-8bd9-4725-a993-74dc813e278b"],
Cell[743322, 19032, 467, 12, 1464, "Input", "ExpressionUUID" -> \
"79019b8f-ad42-4348-ae7e-cc79b70ccf83"],
Cell[743792, 19046, 304, 5, 1464, "Input", "ExpressionUUID" -> \
"bea764ad-94fb-402f-b66d-3d6eb5f8d975"],
Cell[744099, 19053, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"429eb26e-b1e6-4495-b4f9-32a3db90238b"],
Cell[744297, 19059, 191, 4, 1464, "Input", "ExpressionUUID" -> \
"c68b944b-b2d0-4775-8bd6-20c3298a1c0d"],
Cell[744491, 19065, 1142, 31, 3427, "Input", "ExpressionUUID" -> \
"73f79a23-4883-4611-b363-947528da41ff"],
Cell[745636, 19098, 404, 7, 1464, "Input", "ExpressionUUID" -> \
"cb4b8c2d-4c78-4a57-96a2-71c8ee54fbf6"],
Cell[746043, 19107, 291, 8, 1464, "Input", "ExpressionUUID" -> \
"413f6e38-7c61-49db-a571-cd5529667149"],
Cell[746337, 19117, 793, 22, 1464, "Input", "ExpressionUUID" -> \
"641dec86-59d9-4820-9a77-b7894e4892d7"],
Cell[747133, 19141, 775, 22, 1464, "Input", "ExpressionUUID" -> \
"335336a3-1ff5-4f1c-8ae3-fae782451bf9"],
Cell[747911, 19165, 551, 14, 1464, "Input", "ExpressionUUID" -> \
"4fe14a03-9b55-420b-81ab-45dbf922500b"]
}, Closed]],
Cell[CellGroupData[{
Cell[748499, 19184, 1076, 16, 79, "Subsubsection", "ExpressionUUID" -> \
"da296b7d-deec-42d6-8b4f-c44eda59d06d"],
Cell[749578, 19202, 485, 9, 2431, "Text", "ExpressionUUID" -> \
"f0b93caf-c643-4db1-8e4a-3bd85169631c"],
Cell[750066, 19213, 231, 5, 1464, "Input", "ExpressionUUID" -> \
"33dd0c15-9c1d-4e11-aa23-bf58a4431ebb"],
Cell[750300, 19220, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"4950203b-4cad-418d-bf0f-3739dabda347"],
Cell[750495, 19226, 1531, 34, 3427, "Input", "ExpressionUUID" -> \
"20b6b1d4-92aa-4d0d-a90b-e5dbbdf98bb5",
 InitializationCell->True],
Cell[752029, 19262, 722, 16, 1464, "Input", "ExpressionUUID" -> \
"4122fa67-c929-480e-88d7-0819cd214884"],
Cell[752754, 19280, 250, 6, 1464, "Input", "ExpressionUUID" -> \
"dfd191a2-8a24-4244-a77f-8bc6bdca270d"],
Cell[753007, 19288, 263, 7, 1464, "Input", "ExpressionUUID" -> \
"00616b49-b215-4979-83e8-a0b7ed256f76"],
Cell[753273, 19297, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"d2954308-bdd8-448c-a448-bea60a0de51d"],
Cell[753471, 19303, 261, 6, 1464, "Text", "ExpressionUUID" -> \
"5e90db2e-2246-47f9-b345-1e371a4ad81c"],
Cell[753735, 19311, 311, 9, 1464, "Text", "ExpressionUUID" -> \
"c32ebcfc-628a-4533-b780-26f6303877f5"],
Cell[754049, 19322, 835, 19, 1464, "Input", "ExpressionUUID" -> \
"31a13052-2331-40c9-97ec-f84d3840ff1f"],
Cell[754887, 19343, 264, 6, 1464, "Input", "ExpressionUUID" -> \
"66d433f2-0a0c-4cfa-a86c-d80e1d851ea1"],
Cell[755154, 19351, 417, 8, 1464, "Input", "ExpressionUUID" -> \
"191f10bb-f6a6-4680-8901-0485f1212c89"],
Cell[755574, 19361, 320, 7, 2431, "Text", "ExpressionUUID" -> \
"5a2a8dcf-bf59-4741-9562-440d2e7cbdc8"],
Cell[755897, 19370, 393, 11, 1464, "Input", "ExpressionUUID" -> \
"e479b669-49ef-424e-aff3-ac99826623f3"],
Cell[756293, 19383, 515, 10, 1464, "Input", "ExpressionUUID" -> \
"9d60facf-b8dd-4420-b252-38f599c44316"],
Cell[756811, 19395, 301, 7, 1464, "Input", "ExpressionUUID" -> \
"bb4a1b31-9636-4794-9b80-0c070719b2a5"],
Cell[757115, 19404, 71, 0, 1464, "Text", "ExpressionUUID" -> \
"c5ce7564-a77a-48ef-a658-5b8cb0e17fcc"],
Cell[757189, 19406, 552, 15, 1464, "Input", "ExpressionUUID" -> \
"5468408c-74a1-4a0f-8ea4-dac55438126e"],
Cell[757744, 19423, 299, 7, 1464, "Input", "ExpressionUUID" -> \
"18d6ffa3-a684-426e-9ac0-f6de3eddc20c"],
Cell[758046, 19432, 315, 7, 1464, "Input", "ExpressionUUID" -> \
"49bff45b-5f07-44ae-a4ce-b6394a18578b"],
Cell[758364, 19441, 378, 8, 1464, "Input", "ExpressionUUID" -> \
"1c943038-629b-4b9a-8e25-276157bc5dbc"],
Cell[758745, 19451, 578, 15, 1464, "Input", "ExpressionUUID" -> \
"8a01a6c3-1295-4b49-9ed8-554dc059ccca"],
Cell[759326, 19468, 300, 7, 1464, "Input", "ExpressionUUID" -> \
"8fa90d40-a9c3-41d7-81a0-26899eb1f253"],
Cell[759629, 19477, 315, 7, 1464, "Input", "ExpressionUUID" -> \
"a1652d76-ff80-4e24-8edf-bdc6ccc48ed0"],
Cell[759947, 19486, 161, 3, 1464, "Text", "ExpressionUUID" -> \
"b91fb078-5a4f-48c3-a3ed-f987de8f1e07"],
Cell[760111, 19491, 360, 9, 1464, "Text", "ExpressionUUID" -> \
"d8ea1c06-3296-4441-a1bd-7322c426f2f8"],
Cell[760474, 19502, 528, 15, 1464, "Input", "ExpressionUUID" -> \
"2e5d62ec-4bf3-464f-8c28-90b8d198b609"],
Cell[761005, 19519, 658, 12, 1464, "Input", "ExpressionUUID" -> \
"de8bfe7a-d498-433b-8da6-898feba286e0"],
Cell[761666, 19533, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"1d0e10a5-af27-409b-8237-a6ace73f4bcf"],
Cell[761864, 19539, 334, 8, 1464, "Input", "ExpressionUUID" -> \
"2d7ceeb6-a38c-4058-9ad4-51dfcab85e66"],
Cell[762201, 19549, 780, 18, 1464, "Input", "ExpressionUUID" -> \
"022853cf-fb7e-4a81-8782-990fdc21e324"],
Cell[762984, 19569, 636, 12, 1464, "Input", "ExpressionUUID" -> \
"bbd7effa-666d-4cad-9f81-006f5370e3e4"],
Cell[763623, 19583, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"4e473a9b-e45e-4939-9e03-c6b6473c195c"],
Cell[763821, 19589, 169, 4, 1464, "Input", "ExpressionUUID" -> \
"aa21de01-9283-49a5-ba8e-f93226947aab"],
Cell[763993, 19595, 365, 8, 1464, "Input", "ExpressionUUID" -> \
"c45430bf-f2b6-400a-9433-34a704d4fb42"],
Cell[764361, 19605, 215, 5, 1464, "Input", "ExpressionUUID" -> \
"a2c567bf-3430-4489-bbc7-5b31bbb9d706"],
Cell[764579, 19612, 363, 9, 1464, "Input", "ExpressionUUID" -> \
"344f9d64-afa1-45c2-80e0-38eab00def0a"],
Cell[764945, 19623, 161, 3, 1464, "Text", "ExpressionUUID" -> \
"18f3201e-84a9-4dd4-8cdb-4a90ecc462df"],
Cell[765109, 19628, 417, 10, 1464, "Text", "ExpressionUUID" -> \
"a14765e9-5d91-4394-9caf-8295fce71bfd"],
Cell[765529, 19640, 179, 3, 1464, "Text", "ExpressionUUID" -> \
"3e8396f1-5611-4101-85bc-07680004e6b8"],
Cell[765711, 19645, 177, 4, 1464, "Input", "ExpressionUUID" -> \
"23904cda-a359-462b-a902-618f4d63a011"],
Cell[765891, 19651, 384, 10, 1464, "Input", "ExpressionUUID" -> \
"32ec612d-452d-4542-870e-4342fd0dc1e0"],
Cell[766278, 19663, 177, 3, 1464, "Text", "ExpressionUUID" -> \
"9637ee4b-6e7f-4025-8b0a-688ea08ceae5"],
Cell[766458, 19668, 247, 6, 1464, "Input", "ExpressionUUID" -> \
"1575a84c-e769-49cf-8d30-6a9b49506448"],
Cell[766708, 19676, 275, 6, 1464, "Input", "ExpressionUUID" -> \
"0fb68b35-6cfd-4bfd-b6d4-e307e0454b31"],
Cell[766986, 19684, 519, 14, 1464, "Input", "ExpressionUUID" -> \
"16029fc6-a31b-4175-8991-7646d6f9b46f"],
Cell[767508, 19700, 208, 4, 1464, "Text", "ExpressionUUID" -> \
"8ef59396-55b4-406e-a660-6898643d63fa"],
Cell[767719, 19706, 292, 8, 1464, "Text", "ExpressionUUID" -> \
"87602684-a4b7-4977-b997-df31d25e46f2"],
Cell[768014, 19716, 533, 14, 1464, "Input", "ExpressionUUID" -> \
"ab6be232-23b4-444f-a076-adb3b62f63ab"],
Cell[768550, 19732, 188, 3, 1464, "Text", "ExpressionUUID" -> \
"ebe767d8-35e3-42d8-8835-c54fdd5b99c2"],
Cell[768741, 19737, 304, 7, 1464, "Input", "ExpressionUUID" -> \
"933765ec-6d5d-42bf-8307-7694040443ee"],
Cell[769048, 19746, 231, 5, 1464, "Input", "ExpressionUUID" -> \
"ff8c9421-0ebc-4a58-b833-a4e454b23fae"],
Cell[769282, 19753, 335, 9, 1464, "Input", "ExpressionUUID" -> \
"5917bf2c-6d8c-4b63-90c2-9f0ee0debe60"],
Cell[769620, 19764, 356, 7, 1464, "Text", "ExpressionUUID" -> \
"828fa8cf-687e-48b7-99b9-849edda09f0f"],
Cell[769979, 19773, 606, 16, 1464, "Input", "ExpressionUUID" -> \
"27085d63-7332-4e7b-b22b-6a4fb8f0f9fe"],
Cell[770588, 19791, 280, 6, 1464, "Input", "ExpressionUUID" -> \
"490fee5a-b90f-4aec-8382-2d1401d876e2"],
Cell[770871, 19799, 166, 3, 1503, "Text", "ExpressionUUID" -> \
"7e3aef51-30f9-4a74-a7ef-e723957b1dd5"],
Cell[771040, 19804, 314, 9, 1464, "Text", "ExpressionUUID" -> \
"3a6702d2-c7af-4c38-ac5a-11a390cefb71"],
Cell[771357, 19815, 249, 5, 1464, "Text", "ExpressionUUID" -> \
"5600ae40-a498-4905-9269-4e636686bbf1"],
Cell[771609, 19822, 209, 5, 1464, "Input", "ExpressionUUID" -> \
"fd2bbdd1-3925-4df8-951f-1c610e96f4f4"],
Cell[771821, 19829, 542, 15, 1464, "Input", "ExpressionUUID" -> \
"1da3a31d-ba67-4c06-9ee2-1ece512760ed"],
Cell[772366, 19846, 300, 7, 1464, "Input", "ExpressionUUID" -> \
"453f72ee-267a-4c0b-9b43-d419c7f7c866"],
Cell[772669, 19855, 376, 8, 1464, "Input", "ExpressionUUID" -> \
"aa6ecf72-21c8-4181-87c0-5ed6f72f02b8"],
Cell[773048, 19865, 390, 11, 1464, "Input", "ExpressionUUID" -> \
"e0ba50b7-74b0-4837-bc1d-f6d310161dd0"],
Cell[773441, 19878, 326, 7, 1464, "Input", "ExpressionUUID" -> \
"ae40389f-a411-4815-9650-61afd2e79d07"],
Cell[773770, 19887, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"234047b7-b570-4444-aca9-a8ea8d608623"],
Cell[773968, 19893, 228, 6, 1464, "Input", "ExpressionUUID" -> \
"287f82ef-2509-4878-915c-928933657817"],
Cell[774199, 19901, 305, 8, 1464, "Input", "ExpressionUUID" -> \
"8e8e0bca-4949-49b5-828a-366b14302d34"],
Cell[774507, 19911, 437, 13, 1464, "Input", "ExpressionUUID" -> \
"b8bc3bc9-7b71-47b2-858a-815b690f3a1e"],
Cell[774947, 19926, 326, 7, 1464, "Input", "ExpressionUUID" -> \
"4c0fe9c0-b3b8-424a-aafd-d03d8329d78f"],
Cell[775276, 19935, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"e674ecaa-0c9d-4804-a3aa-d19be768c511"],
Cell[775474, 19941, 289, 7, 1464, "Input", "ExpressionUUID" -> \
"52014414-976d-4ad1-aed9-676ed31ba785"],
Cell[775766, 19950, 367, 8, 1464, "Input", "ExpressionUUID" -> \
"345e4b9a-be36-4992-9336-dd0b3a3574f5"],
Cell[776136, 19960, 428, 9, 1464, "Input", "ExpressionUUID" -> \
"df54c1c3-5819-498a-a1d1-9bfb7bebf253"],
Cell[776567, 19971, 435, 13, 1464, "Input", "ExpressionUUID" -> \
"6353cb41-55e7-47a0-b060-e1b75e9acc4c"],
Cell[777005, 19986, 326, 7, 1464, "Input", "ExpressionUUID" -> \
"026d2654-7a72-43fe-b9c7-cdd673535b2c"],
Cell[777334, 19995, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"4b0c8601-78cc-4c55-81c4-2de877ad24db"],
Cell[777532, 20001, 297, 7, 1464, "Input", "ExpressionUUID" -> \
"a2dd2808-d0f7-4e4b-b30f-954b7a0ccbd2"],
Cell[777832, 20010, 303, 8, 1464, "Input", "ExpressionUUID" -> \
"05cd66f9-40dc-48cb-87a5-d2f3bf6ae4b5"],
Cell[778138, 20020, 154, 3, 1464, "Input", "ExpressionUUID" -> \
"bbfb15fc-730a-432b-81ee-980e466181d3"],
Cell[778295, 20025, 437, 13, 1464, "Input", "ExpressionUUID" -> \
"8c9c8c5b-adf2-4af0-871a-1df53fd1bcc1"],
Cell[778735, 20040, 326, 7, 1464, "Input", "ExpressionUUID" -> \
"0f38f281-dccc-4a24-b719-69ee3b8d6b34"],
Cell[779064, 20049, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"64a354b5-2e0b-4cf6-bf3d-a25dfd8aa8cf"],
Cell[779262, 20055, 284, 7, 1464, "Input", "ExpressionUUID" -> \
"a75e9c11-b646-4ffd-9bfc-7e758a6d4763"],
Cell[779549, 20064, 464, 13, 1464, "Input", "ExpressionUUID" -> \
"3bc66be9-d3cc-4703-acb2-66dfa67b7032"],
Cell[780016, 20079, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"e614b9ec-dddc-4dfe-b7a3-052024eaf176"],
Cell[780214, 20085, 404, 9, 1464, "Input", "ExpressionUUID" -> \
"41f1745d-4927-4fef-8a0a-c8226d0f5e5a"],
Cell[780621, 20096, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"3a18b5c4-fb9f-43e4-96d8-3178fad3312c"],
Cell[780819, 20102, 388, 9, 1464, "Input", "ExpressionUUID" -> \
"90833d9a-efe5-46d2-9d09-0c6f871ec06f"],
Cell[781210, 20113, 251, 5, 1464, "Input", "ExpressionUUID" -> \
"dbb6ea42-dbe5-4689-8412-efcf3c66a45b"],
Cell[781464, 20120, 275, 7, 1464, "Input", "ExpressionUUID" -> \
"9aa145c7-bb6a-473b-acbb-3a6a227edc0a"],
Cell[781742, 20129, 161, 3, 1464, "Text", "ExpressionUUID" -> \
"6be41199-10ca-4762-aede-0b93d11551ea"]
}, Closed]],
Cell[CellGroupData[{
Cell[781940, 20137, 1215, 18, 79, "Subsubsection", "ExpressionUUID" -> \
"1d2549d3-f1be-40fb-8d4f-c436c5833611"],
Cell[783158, 20157, 250, 4, 1464, "Text", "ExpressionUUID" -> \
"e90e820a-17b2-49fa-95cd-5a75eea52836"],
Cell[783411, 20163, 314, 9, 1464, "Text", "ExpressionUUID" -> \
"13115d90-2faa-4679-90e0-dd26c16b07fb"],
Cell[783728, 20174, 857, 19, 1464, "Input", "ExpressionUUID" -> \
"98fd457e-6f22-418d-af7b-0d0bebb1cf86"],
Cell[784588, 20195, 382, 11, 1464, "Input", "ExpressionUUID" -> \
"517d4485-696c-48a7-8453-08afe4bfb347"],
Cell[784973, 20208, 309, 8, 1464, "Input", "ExpressionUUID" -> \
"6c48ddb6-c9b2-4769-9243-4a2ad90c750b"],
Cell[785285, 20218, 467, 9, 1464, "Input", "ExpressionUUID" -> \
"31e88a8a-746f-4802-85fc-edc3671dd6f8"],
Cell[785755, 20229, 303, 7, 1464, "Input", "ExpressionUUID" -> \
"690d7c18-2057-4c93-849f-9fcd4551aeaf"],
Cell[786061, 20238, 392, 9, 1464, "Input", "ExpressionUUID" -> \
"3b290bc0-be84-4489-8926-b07a2eda3a05"],
Cell[786456, 20249, 167, 4, 1464, "Input", "ExpressionUUID" -> \
"0f59b6e5-f995-4929-8be7-d647c30a170b"],
Cell[786626, 20255, 71, 0, 1464, "Text", "ExpressionUUID" -> \
"693241a7-be51-4591-8ef4-e7b79b9c255e"],
Cell[786700, 20257, 552, 15, 1464, "Input", "ExpressionUUID" -> \
"6476b383-7376-43c7-922a-51e14e1126f4"],
Cell[787255, 20274, 369, 10, 1464, "Input", "ExpressionUUID" -> \
"d1d59da6-856f-4600-9ae9-41d91c5b04ca"],
Cell[787627, 20286, 345, 7, 1464, "Input", "ExpressionUUID" -> \
"61d7246b-6932-4437-83e4-eb37384f88b8"],
Cell[787975, 20295, 120, 1, 1464, "Input", "ExpressionUUID" -> \
"f3b0961a-dae7-4e03-8941-3621d9309620"],
Cell[788098, 20298, 270, 7, 1464, "Input", "ExpressionUUID" -> \
"bc3f4f42-d764-4931-b745-c34101d561b9"],
Cell[788371, 20307, 475, 14, 1464, "Input", "ExpressionUUID" -> \
"73ab22cd-9198-46e2-8434-dcf1a1698eb5"],
Cell[788849, 20323, 346, 10, 1464, "Input", "ExpressionUUID" -> \
"99fa35c4-1b2a-472e-b95e-79af4c781f1f"],
Cell[789198, 20335, 191, 4, 1464, "Input", "ExpressionUUID" -> \
"41620aeb-013b-452c-b811-bd834e188f1e"],
Cell[789392, 20341, 216, 6, 1464, "Input", "ExpressionUUID" -> \
"a6682abf-786e-4428-a8ed-b6bcefe14a63"],
Cell[789611, 20349, 415, 9, 1464, "Input", "ExpressionUUID" -> \
"888b770e-74db-4b1e-9150-c93a5c68e4ee"],
Cell[790029, 20360, 161, 3, 1464, "Text", "ExpressionUUID" -> \
"fa2987f5-de11-4f6a-8ba6-8ffaad41fcc0"],
Cell[790193, 20365, 362, 9, 1464, "Text", "ExpressionUUID" -> \
"8052dd80-4d4a-4e0d-a15a-975f003229e2"],
Cell[790558, 20376, 528, 15, 1464, "Input", "ExpressionUUID" -> \
"76258884-9e79-4b47-b1eb-650dc65a12d0"],
Cell[791089, 20393, 301, 8, 1464, "Input", "ExpressionUUID" -> \
"4c0f3e25-603e-4b69-84fd-a491c946ef6a"],
Cell[791393, 20403, 701, 13, 1464, "Input", "ExpressionUUID" -> \
"8c61142f-3299-4154-b31b-071ccb679bb5"],
Cell[792097, 20418, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"bc18aba4-e3dc-45d4-a2a0-84d5097c2296"],
Cell[792295, 20424, 370, 8, 1464, "Input", "ExpressionUUID" -> \
"bd5d4fa1-45aa-4376-bd89-0c265d32799c"],
Cell[792668, 20434, 120, 1, 1464, "Input", "ExpressionUUID" -> \
"342168ec-6db4-435f-9a9a-c6ad56e1f938"],
Cell[792791, 20437, 780, 18, 1464, "Input", "ExpressionUUID" -> \
"5c7a872b-7fc1-4e42-abef-7879b9def95e"],
Cell[793574, 20457, 348, 10, 1464, "Input", "ExpressionUUID" -> \
"607091f6-d0ba-45c7-acae-6f2c577c0d14"],
Cell[793925, 20469, 677, 12, 1464, "Input", "ExpressionUUID" -> \
"e9a7aa30-980c-4c82-8f2b-8fb4ccfc5fd0"],
Cell[794605, 20483, 283, 8, 1464, "Input", "ExpressionUUID" -> \
"4bd988fe-a795-458d-9a01-6a7cfb8a93b9"],
Cell[794891, 20493, 412, 9, 1464, "Input", "ExpressionUUID" -> \
"4288c821-7fd6-4dcf-8706-021cb8afda05"],
Cell[795306, 20504, 215, 5, 1464, "Input", "ExpressionUUID" -> \
"1612d32f-eb01-4eb6-9c4a-040a322692cd"],
Cell[795524, 20511, 404, 9, 1464, "Input", "ExpressionUUID" -> \
"dc476540-2b6f-41b7-96f7-2cf18b1acf63"],
Cell[795931, 20522, 161, 3, 1464, "Text", "ExpressionUUID" -> \
"a26248b7-52d5-49d8-b065-d64675e58ce7"],
Cell[796095, 20527, 312, 9, 1464, "Text", "ExpressionUUID" -> \
"7f0ff1be-7d08-41a9-8a74-e5a1d5480947"],
Cell[796410, 20538, 363, 8, 1464, "Text", "ExpressionUUID" -> \
"b5c75fcd-35f7-46e3-ae41-25682a319ee7"],
Cell[796776, 20548, 209, 5, 1464, "Input", "ExpressionUUID" -> \
"57eb84aa-c473-4766-88bf-ebaa4e0c762f"],
Cell[796988, 20555, 542, 15, 1464, "Input", "ExpressionUUID" -> \
"905dfc54-def5-442e-9568-487c29d4f7d0"],
Cell[797533, 20572, 445, 13, 1464, "Input", "ExpressionUUID" -> \
"8e365d66-14dd-41f5-b36d-6514cb23c2b0"],
Cell[797981, 20587, 334, 9, 1464, "Input", "ExpressionUUID" -> \
"35e38cbf-8c41-4285-b6a7-1c453af7a80e"],
Cell[798318, 20598, 317, 7, 1464, "Input", "ExpressionUUID" -> \
"0add7885-ee88-4c73-968c-c30984ce9eee"],
Cell[798638, 20607, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"72a4df66-1284-46e9-b82a-357291662264"],
Cell[798833, 20613, 395, 9, 1464, "Input", "ExpressionUUID" -> \
"a69a00f3-8528-4bc4-8b4a-8b13358b024b"],
Cell[799231, 20624, 120, 1, 1464, "Input", "ExpressionUUID" -> \
"aa33f034-c620-4b2b-b674-324a082abc60"],
Cell[799354, 20627, 390, 11, 1464, "Input", "ExpressionUUID" -> \
"4a8723f3-70c4-4ccc-a287-feb011886848"],
Cell[799747, 20640, 346, 10, 1464, "Input", "ExpressionUUID" -> \
"0bebd4a7-1a13-446e-936e-68a7dbe62ca2"],
Cell[800096, 20652, 342, 7, 1464, "Input", "ExpressionUUID" -> \
"86cdcd13-7b87-41ed-92f2-4e7348266042"],
Cell[800441, 20661, 216, 4, 1464, "Input", "ExpressionUUID" -> \
"6652969f-902f-4695-987e-00451decda20"],
Cell[800660, 20667, 319, 8, 1464, "Input", "ExpressionUUID" -> \
"f791cbb0-b339-4e42-aa13-e36b94fd0281"],
Cell[800982, 20677, 120, 1, 1464, "Input", "ExpressionUUID" -> \
"8c23e298-d34b-49c3-a282-1151fdd7d686"],
Cell[801105, 20680, 466, 13, 1464, "Input", "ExpressionUUID" -> \
"98dbef38-fb6c-415f-873b-d69aa08116e5"],
Cell[801574, 20695, 301, 8, 1464, "Input", "ExpressionUUID" -> \
"4a63f35f-34d8-42ef-9aa7-4907838210c8"],
Cell[801878, 20705, 341, 7, 1464, "Input", "ExpressionUUID" -> \
"f208476c-8948-4a01-9d13-9ed5e19bcfd8"],
Cell[802222, 20714, 380, 8, 1464, "Input", "ExpressionUUID" -> \
"e022146e-d31e-463b-a056-941a189c75cc"],
Cell[802605, 20724, 443, 9, 1464, "Input", "ExpressionUUID" -> \
"b6322536-035e-41f0-bd39-1d4a504df64a"],
Cell[803051, 20735, 120, 1, 1464, "Input", "ExpressionUUID" -> \
"b7aebcdf-8252-4e4d-83a2-bc25c5337741"],
Cell[803174, 20738, 464, 13, 1464, "Input", "ExpressionUUID" -> \
"47fe2e6d-fe8b-4d57-bd4a-e8fdc0bc0ce7"],
Cell[803641, 20753, 346, 10, 1464, "Input", "ExpressionUUID" -> \
"9a26241e-d5d5-4d8c-bed8-271c9690216f"],
Cell[803990, 20765, 343, 7, 1464, "Input", "ExpressionUUID" -> \
"1fc6ffff-c49a-4587-a733-01daab4ac928"],
Cell[804336, 20774, 216, 4, 1464, "Input", "ExpressionUUID" -> \
"b04df6c9-caa8-4f1f-8e29-8c5636c1d677"],
Cell[804555, 20780, 316, 8, 1464, "Input", "ExpressionUUID" -> \
"d281fce9-71b4-47e8-ba96-1d1a1d78beeb"],
Cell[804874, 20790, 120, 1, 1464, "Input", "ExpressionUUID" -> \
"c9b9e957-5573-4763-8f76-e390b5551f47"],
Cell[804997, 20793, 466, 13, 1464, "Input", "ExpressionUUID" -> \
"4e5b92e2-30bc-4ea5-9a1b-83aa7167772a"],
Cell[805466, 20808, 343, 9, 1464, "Input", "ExpressionUUID" -> \
"b29da9fd-5849-4612-9589-32228fe9d630"],
Cell[805812, 20819, 340, 7, 1464, "Input", "ExpressionUUID" -> \
"f8a9a846-41ff-4856-9151-57e7b4bc1392"],
Cell[806155, 20828, 216, 4, 1464, "Input", "ExpressionUUID" -> \
"47123094-2d24-4d90-a785-09be1fe4a4f7"],
Cell[806374, 20834, 346, 8, 1464, "Input", "ExpressionUUID" -> \
"4722396c-de9e-438d-8540-dabc81b8fc3f"],
Cell[806723, 20844, 464, 13, 1464, "Input", "ExpressionUUID" -> \
"c94f88cd-002b-4f04-aed7-8cb459345268"],
Cell[807190, 20859, 346, 10, 1464, "Input", "ExpressionUUID" -> \
"b8ef322e-5f6f-4cfd-acec-abdc60bd189a"],
Cell[807539, 20871, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"7e802a2b-3c6c-4fea-b3a6-6802566fbd28"],
Cell[807737, 20877, 415, 8, 1464, "Input", "ExpressionUUID" -> \
"d1b51cb5-132f-4a09-b8e3-bdcf257be4f0"],
Cell[808155, 20887, 161, 3, 1464, "Text", "ExpressionUUID" -> \
"df29ea15-74ec-479f-ab54-750bea604179"]
}, Closed]],
Cell[CellGroupData[{
Cell[808353, 20895, 1248, 17, 42, "Subsubsection", "ExpressionUUID" -> \
"32fca9e3-fbff-4903-9bbb-10c2d8e524b1"],
Cell[809604, 20914, 349, 7, 2431, "Text", "ExpressionUUID" -> \
"a2de82e4-4cb5-48bb-9f15-5c2be9e4c8ce"],
Cell[809956, 20923, 1240, 32, 2460, "Input", "ExpressionUUID" -> \
"86b285d3-2ba0-4ce0-b546-dfc8701f1191"],
Cell[811199, 20957, 285, 7, 1464, "Input", "ExpressionUUID" -> \
"d834e9d3-6a31-4938-85fa-e7e9d73e7d98"],
Cell[811487, 20966, 724, 16, 1464, "Input", "ExpressionUUID" -> \
"2b3749ee-29f8-4a8f-aba0-9fb2b220c00f"],
Cell[812214, 20984, 717, 16, 1464, "Input", "ExpressionUUID" -> \
"fd12fb76-2451-4840-abb3-871c755d56d6"],
Cell[812934, 21002, 1043, 23, 1464, "Input", "ExpressionUUID" -> \
"c6aff3ae-cf9c-4e82-bbc6-8b0347a56396"],
Cell[813980, 21027, 477, 11, 3378, "Text", "ExpressionUUID" -> \
"89d32340-62a3-4aa3-8c8a-79be568946b8"],
Cell[814460, 21040, 252, 5, 1464, "Text", "ExpressionUUID" -> \
"af63d19b-4368-4451-93bd-aec8cf81820a"],
Cell[814715, 21047, 393, 12, 1464, "Input", "ExpressionUUID" -> \
"be5c56a2-aab1-4f46-b8ad-3709b5455b64"],
Cell[815111, 21061, 395, 12, 1464, "Input", "ExpressionUUID" -> \
"7109e570-4477-4d00-9c83-1f60cc093034"],
Cell[815509, 21075, 395, 12, 1464, "Input", "ExpressionUUID" -> \
"87b33d26-a93f-4f9e-a19f-bec43d829594"],
Cell[815907, 21089, 183, 4, 1464, "Input", "ExpressionUUID" -> \
"f1377bed-4636-41c8-93d5-351f3be1c8f3"],
Cell[816093, 21095, 298, 7, 1464, "Input", "ExpressionUUID" -> \
"44cc101b-ac5f-4bfd-8835-3b2b3be9a587"],
Cell[816394, 21104, 334, 10, 1464, "Input", "ExpressionUUID" -> \
"b21bebee-04a2-47c4-8058-609aadffa340"],
Cell[816731, 21116, 665, 17, 1464, "Input", "ExpressionUUID" -> \
"a58ae349-f9c4-4d53-bfe1-f4ad08885e76"],
Cell[817399, 21135, 754, 17, 1464, "Input", "ExpressionUUID" -> \
"fa6ad360-2266-4d53-9fd9-b24455df9738"],
Cell[818156, 21154, 297, 7, 2460, "Text", "ExpressionUUID" -> \
"1d49fa75-e2c0-4e4c-bc9a-79ce585ffdd1"],
Cell[818456, 21163, 428, 10, 1464, "Input", "ExpressionUUID" -> \
"96d85545-4303-49d8-9a6f-7f1c0bdae72a"],
Cell[818887, 21175, 326, 9, 1464, "Input", "ExpressionUUID" -> \
"d8b58f58-4bb8-47e9-b049-43d4652a2b5d"],
Cell[819216, 21186, 195, 3, 1464, "Text", "ExpressionUUID" -> \
"f56458c1-540e-4c7d-9580-afd8c2cf46d4"],
Cell[819414, 21191, 465, 11, 1464, "Input", "ExpressionUUID" -> \
"9ead324d-abbe-4b97-bbab-5f79459ea064"],
Cell[819882, 21204, 235, 5, 1464, "Input", "ExpressionUUID" -> \
"8ae4f4ed-c973-4a75-9afb-ea95b0695d56"],
Cell[820120, 21211, 278, 6, 1464, "Input", "ExpressionUUID" -> \
"74f9e614-fccb-465b-a08c-632188409afc"],
Cell[820401, 21219, 178, 3, 1464, "Text", "ExpressionUUID" -> \
"19daf000-c7d2-45f9-b786-ecca967cdd5a"],
Cell[820582, 21224, 372, 10, 1464, "Input", "ExpressionUUID" -> \
"bf12110e-e521-44c5-981e-06254eb132f7"],
Cell[820957, 21236, 923, 22, 1464, "Input", "ExpressionUUID" -> \
"d4c63903-6b66-4325-a451-6791d09a9082"],
Cell[821883, 21260, 190, 4, 1464, "Input", "ExpressionUUID" -> \
"eb7bd5bb-221f-4d8c-8385-39b7cbc8f7ed"],
Cell[822076, 21266, 264, 6, 1464, "Input", "ExpressionUUID" -> \
"7ba31c98-32b8-4a05-882d-f713374ad262"],
Cell[822343, 21274, 192, 3, 1464, "Text", "ExpressionUUID" -> \
"0c4185d3-10d4-407c-81b2-061ba6d4507e"],
Cell[822538, 21279, 608, 17, 1464, "Input", "ExpressionUUID" -> \
"574cb52e-c7ca-4333-ae3d-f3e6108132d6"],
Cell[823149, 21298, 264, 6, 1464, "Input", "ExpressionUUID" -> \
"ea495e04-907f-4d49-887c-b4c657e4bac3"],
Cell[823416, 21306, 428, 11, 1464, "Input", "ExpressionUUID" -> \
"ec297c91-0327-431f-b6a3-669927617ed4"],
Cell[823847, 21319, 450, 11, 1464, "Input", "ExpressionUUID" -> \
"d6bae59a-a2c3-478d-874f-2c175ae9bd7a"],
Cell[824300, 21332, 213, 5, 1464, "Input", "ExpressionUUID" -> \
"1e5fd443-9dd3-40d6-bb1e-937be15d529d"],
Cell[824516, 21339, 252, 5, 1464, "Input", "ExpressionUUID" -> \
"7b4787f8-2da3-49dd-838e-9001f75e92f5"],
Cell[824771, 21346, 217, 5, 1464, "Input", "ExpressionUUID" -> \
"1cc26778-a47b-49db-859a-f5006fb4e7a0"],
Cell[824991, 21353, 569, 11, 3349, "Text", "ExpressionUUID" -> \
"345b0029-7b09-4c87-9230-d2f6f697d5c2"],
Cell[825563, 21366, 438, 14, 1464, "Input", "ExpressionUUID" -> \
"c3ea2959-927c-4423-bf9f-e61a926b9346",
 Evaluatable->False],
Cell[826004, 21382, 205, 4, 1464, "Text", "ExpressionUUID" -> \
"d76a08fc-3db3-42ed-a6dc-9d35ecee869f"],
Cell[826212, 21388, 1752, 49, 4365, "Input", "ExpressionUUID" -> \
"09658b9c-d464-4a0d-93d3-ca6846184ee7",
 Evaluatable->False],
Cell[827967, 21439, 263, 6, 1464, "Text", "ExpressionUUID" -> \
"00590099-b738-4c73-8c0d-6ad3c7660878"],
Cell[828233, 21447, 176, 4, 1464, "Input", "ExpressionUUID" -> \
"2a4b6c2f-ae39-4ee2-b847-74a6bf9c1b76"],
Cell[828412, 21453, 162, 4, 1464, "Input", "ExpressionUUID" -> \
"35fccf6c-3b94-4920-9f00-0a0c0030747b"],
Cell[828577, 21459, 1918, 43, 4365, "Input", "ExpressionUUID" -> \
"a25607f3-a1a2-40ac-ab5b-ed62da19a1f3",
 InitializationCell->True],
Cell[830498, 21504, 680, 16, 4306, "Text", "ExpressionUUID" -> \
"0faf638d-e4cd-4732-9ba8-d5789a1892cb"],
Cell[831181, 21522, 440, 12, 1464, "Input", "ExpressionUUID" -> \
"317ee2b2-dcfa-4c7f-86dc-c3f6b7dc2285",
 InitializationCell->True],
Cell[831624, 21536, 926, 24, 2635, "Input", "ExpressionUUID" -> \
"1b0386c9-96fa-4151-8c01-821b78aa0ffd",
 Evaluatable->False],
Cell[832553, 21562, 171, 3, 1464, "Text", "ExpressionUUID" -> \
"eb7d096f-97e9-4241-b99a-594c5eacff41"],
Cell[832727, 21567, 1397, 42, 2196, "Input", "ExpressionUUID" -> \
"da54320f-fc41-4f9d-9697-e841c99b68fb",
 Evaluatable->False],
Cell[834127, 21611, 177, 3, 1464, "Text", "ExpressionUUID" -> \
"70af1058-9bc3-4bd4-bb84-4bd003b51eae"],
Cell[834307, 21616, 343, 9, 1464, "Input", "ExpressionUUID" -> \
"5fddd404-7397-4db3-be39-da2010d38351"],
Cell[834653, 21627, 393, 10, 1464, "Input", "ExpressionUUID" -> \
"bd586231-0b45-4a1e-a9a1-cb2d3ca83b61"],
Cell[835049, 21639, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"1753ea7e-c910-469d-a103-174d40a1fe84"],
Cell[835244, 21645, 239, 7, 1464, "Input", "ExpressionUUID" -> \
"dc42c7bb-377d-42a2-aad2-f14ccb9c605a"],
Cell[835486, 21654, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"c587ddc4-d249-4611-9bd0-d228eef7f3b9"],
Cell[835681, 21660, 342, 9, 1464, "Input", "ExpressionUUID" -> \
"73784e7d-438d-4b05-96af-933200a4673a"],
Cell[836026, 21671, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"981ae393-5f6c-4801-bf48-ebb347ecb274"],
Cell[836221, 21677, 361, 9, 1464, "Input", "ExpressionUUID" -> \
"b2d427b7-421d-4191-9a6c-9bacd9dea426"],
Cell[836585, 21688, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"b7a1af78-925b-4b68-bae6-9035b89bc8e6"],
Cell[836780, 21694, 398, 9, 1464, "Input", "ExpressionUUID" -> \
"50009595-4337-4b82-a1dc-5df86913fb4a"],
Cell[837181, 21705, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"82925196-b0cd-4420-86d2-d5e63e53d30d"],
Cell[837376, 21711, 528, 12, 1464, "Input", "ExpressionUUID" -> \
"46d58016-0f05-478c-9079-00c90bd3405b"],
Cell[837907, 21725, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"e812efd4-5cc4-4f13-b364-e3b051f285a3"],
Cell[838102, 21731, 302, 8, 1464, "Input", "ExpressionUUID" -> \
"8bc1b709-3de7-4fc1-a6fe-106d49178820"],
Cell[838407, 21741, 295, 4, 1464, "Text", "ExpressionUUID" -> \
"03443b96-984f-4652-898a-35cc950b3fe6"],
Cell[838705, 21747, 226, 5, 1464, "Input", "ExpressionUUID" -> \
"d3c847a4-9b25-4ab8-8d72-c764583865e2"],
Cell[838934, 21754, 317, 10, 1464, "Text", "ExpressionUUID" -> \
"b227e1ba-34e8-4f88-ac8e-57a7515f0d3b"],
Cell[839254, 21766, 1257, 32, 4365, "Input", "ExpressionUUID" -> \
"50db4f01-08c9-43d4-829c-65a207c705e6"],
Cell[840514, 21800, 239, 4, 1464, "Text", "ExpressionUUID" -> \
"87765663-f8c4-447e-af94-cfcb26c12557"],
Cell[840756, 21806, 370, 8, 1464, "Input", "ExpressionUUID" -> \
"8b77e916-39a3-4ef2-a345-e7d3192e60e4"],
Cell[841129, 21816, 198, 3, 1464, "Text", "ExpressionUUID" -> \
"1044378f-a2e0-4a78-b7ac-bff1e92642d7"],
Cell[841330, 21821, 353, 8, 1464, "Input", "ExpressionUUID" -> \
"34eef3fa-c9f5-48ae-84c3-18df6f587b15"],
Cell[841686, 21831, 183, 4, 1464, "Input", "ExpressionUUID" -> \
"a7b20c44-184b-4a20-885f-82a75a738cba"],
Cell[841872, 21837, 175, 3, 1464, "Text", "ExpressionUUID" -> \
"c0d88b46-7eae-41cf-a70f-2244db848338"],
Cell[842050, 21842, 287, 8, 1464, "Input", "ExpressionUUID" -> \
"6dc50195-5dfb-4bed-b13e-915a134fe3a5"],
Cell[842340, 21852, 225, 3, 1464, "Text", "ExpressionUUID" -> \
"1774121f-2b48-4efd-9141-39103f0c6030"],
Cell[842568, 21857, 487, 13, 1464, "Input", "ExpressionUUID" -> \
"ca73941e-5bb2-43d8-9e80-aa6cef8a49f2"],
Cell[843058, 21872, 245, 8, 1464, "Text", "ExpressionUUID" -> \
"0e70e184-863b-4124-9d96-996f36891dde"],
Cell[843306, 21882, 1186, 32, 3427, "Input", "ExpressionUUID" -> \
"b3044aff-fbb6-4f1a-898b-538a8934b76c"],
Cell[844495, 21916, 194, 3, 1464, "Text", "ExpressionUUID" -> \
"6f346957-e2e6-45f7-b59c-395ea4930d9f"],
Cell[844692, 21921, 324, 9, 1464, "Input", "ExpressionUUID" -> \
"62f35ea9-d995-4185-8352-c931126dc62d"],
Cell[845019, 21932, 326, 9, 1464, "Input", "ExpressionUUID" -> \
"779f50e1-cf40-4151-b46e-5837f98a087d"],
Cell[845348, 21943, 408, 12, 1464, "Input", "ExpressionUUID" -> \
"2f2166ff-e748-4d71-955d-6a8d0346a82b"],
Cell[845759, 21957, 244, 4, 1503, "Text", "ExpressionUUID" -> \
"acd34583-cb20-414c-9be3-16a1db683bab"],
Cell[846006, 21963, 342, 8, 3349, "Text", "ExpressionUUID" -> \
"827dde3d-a3c7-438e-b879-8dffb1f07928"],
Cell[846351, 21973, 1276, 32, 3427, "Input", "ExpressionUUID" -> \
"291683ad-76d8-407f-a465-867ccb3fe9cb"],
Cell[847630, 22007, 493, 12, 1464, "Input", "ExpressionUUID" -> \
"9947f64a-8280-4859-af49-fe7ec12e6f6e"],
Cell[848126, 22021, 191, 4, 1464, "Input", "ExpressionUUID" -> \
"b6edb116-fb0b-400a-ba78-24be80feb5a1"],
Cell[848320, 22027, 266, 6, 1464, "Input", "ExpressionUUID" -> \
"1ae2e770-6157-41ef-831d-37acd3e8c4ff"],
Cell[848589, 22035, 281, 6, 1464, "Input", "ExpressionUUID" -> \
"dd1fddc3-dec5-4272-81e5-c67542b2810e"],
Cell[848873, 22043, 217, 6, 1464, "Input", "ExpressionUUID" -> \
"c236f64c-dd7b-49a3-a32f-02455d739c40"],
Cell[849093, 22051, 193, 4, 1464, "Input", "ExpressionUUID" -> \
"46e1ec39-03b0-403b-ab51-04f4c70525e6"],
Cell[849289, 22057, 396, 10, 1464, "Input", "ExpressionUUID" -> \
"f8456c78-8f77-4f82-ba3f-8c8c82750c6b"],
Cell[849688, 22069, 280, 6, 1464, "Input", "ExpressionUUID" -> \
"c60e9b41-8e00-494b-b654-1e46da911c5a"],
Cell[849971, 22077, 219, 6, 1464, "Input", "ExpressionUUID" -> \
"e1c4a0f6-002f-4805-9a83-d0f00e02e809"],
Cell[850193, 22085, 193, 4, 1464, "Input", "ExpressionUUID" -> \
"6e86706b-aaab-4b60-8004-a72fc497f2ee"],
Cell[850389, 22091, 418, 11, 1464, "Input", "ExpressionUUID" -> \
"68cb480a-5592-47e0-8295-bdf5f7a10a4d"],
Cell[850810, 22104, 228, 5, 1464, "Input", "ExpressionUUID" -> \
"3c30e460-2586-4211-a23a-c5ac6521d722"],
Cell[851041, 22111, 341, 8, 1464, "Input", "ExpressionUUID" -> \
"8a7e862c-a14d-4902-bcb6-aa730cd1af99"],
Cell[851385, 22121, 381, 10, 1464, "Input", "ExpressionUUID" -> \
"e4375c8f-c20b-4992-94b4-1aa3140bc9a4"],
Cell[851769, 22133, 279, 6, 1464, "Input", "ExpressionUUID" -> \
"86f82528-fe6f-4d21-b4d4-3f1b0210a5ce"],
Cell[852051, 22141, 440, 11, 1464, "Input", "ExpressionUUID" -> \
"45e1270d-b4ba-4463-b8fe-e5742033e630"],
Cell[852494, 22154, 464, 12, 2431, "Text", "ExpressionUUID" -> \
"cfed021e-a68a-4b3b-90be-94a91fb9b971"],
Cell[852961, 22168, 593, 18, 1464, "Input", "ExpressionUUID" -> \
"040c31d0-3153-4e38-b702-5ea98c7ff0e6",
 Evaluatable->False],
Cell[853557, 22188, 241, 6, 1503, "Text", "ExpressionUUID" -> \
"08069e32-5752-4c1d-b69f-ce34dda0d0bd"],
Cell[853801, 22196, 215, 3, 1464, "Text", "ExpressionUUID" -> \
"b3354da2-9216-46e7-ba34-36b0dcec5da1"],
Cell[854019, 22201, 258, 5, 1464, "Text", "ExpressionUUID" -> \
"5d1e7379-e249-4451-90e6-f687b3094843"],
Cell[854280, 22208, 1655, 42, 3427, "Input", "ExpressionUUID" -> \
"529d6ff9-ce6c-4432-963b-33f68f3a7d08"],
Cell[855938, 22252, 266, 6, 1464, "Input", "ExpressionUUID" -> \
"7ef4fc5d-12db-4b3e-a461-e5d3bd7dd87b"],
Cell[856207, 22260, 260, 6, 1464, "Input", "ExpressionUUID" -> \
"5c11e937-1b4a-4c8d-95bc-6c604ad0d036"],
Cell[856470, 22268, 394, 10, 1464, "Input", "ExpressionUUID" -> \
"aeb500f0-3dad-48f6-a434-1cf540c24b71"],
Cell[856867, 22280, 396, 10, 1464, "Input", "ExpressionUUID" -> \
"e420b803-c2e4-41de-b00a-f359d9f2996d"],
Cell[857266, 22292, 349, 9, 1464, "Input", "ExpressionUUID" -> \
"fe124cca-bab3-41ea-846b-4c44df12732e"],
Cell[857618, 22303, 293, 9, 2460, "Text", "ExpressionUUID" -> \
"d2f5b448-8bd6-453a-8c50-6d65a8e2f0be"],
Cell[857914, 22314, 2045, 54, 5331, "Input", "ExpressionUUID" -> \
"659619fa-3e40-4592-b875-3c1b0cfddd81"],
Cell[859962, 22370, 268, 6, 1464, "Input", "ExpressionUUID" -> \
"278405f1-ec3e-407f-a722-e853a702bcf3"],
Cell[860233, 22378, 394, 10, 1464, "Input", "ExpressionUUID" -> \
"c4ce72f7-c3ed-4d20-89a9-51a79a814f90"],
Cell[860630, 22390, 396, 10, 1464, "Input", "ExpressionUUID" -> \
"0a15975f-c160-4e31-a9f1-0b35eaf11f66"],
Cell[861029, 22402, 378, 10, 1464, "Input", "ExpressionUUID" -> \
"1b35886b-4569-4cb2-8e9e-261f0519636a"],
Cell[861410, 22414, 163, 3, 1464, "Text", "ExpressionUUID" -> \
"d3135330-7558-459d-b291-50bb160d267c"],
Cell[861576, 22419, 192, 3, 1464, "Text", "ExpressionUUID" -> \
"3b7e903b-0139-4f0a-95d3-8e443d685a1a"],
Cell[861771, 22424, 349, 9, 1464, "Input", "ExpressionUUID" -> \
"19dfa0a7-f4f0-401f-a50f-4eb43eff9b3f"],
Cell[862123, 22435, 149, 3, 1464, "Text", "ExpressionUUID" -> \
"30640bf1-8506-4c08-8053-eadf8344afad"],
Cell[862275, 22440, 730, 21, 1464, "Input", "ExpressionUUID" -> \
"12e6a77f-e240-4836-a32b-fb9a3d37e1f2"],
Cell[863008, 22463, 175, 3, 1464, "Text", "ExpressionUUID" -> \
"9e16d69a-630a-4af6-b6ea-cedc056b436c"],
Cell[863186, 22468, 340, 8, 1464, "Text", "ExpressionUUID" -> \
"4a30d164-ebcc-405c-b8d1-987d33ff47f2"],
Cell[863529, 22478, 659, 19, 1464, "Input", "ExpressionUUID" -> \
"751ba374-1bf5-49f1-82f2-1fd3e18fec95",
 Evaluatable->False],
Cell[864191, 22499, 227, 4, 1503, "Text", "ExpressionUUID" -> \
"f8b4bf50-ee91-41f4-8b6a-e04fd80fb218"],
Cell[864421, 22505, 449, 11, 1464, "Input", "ExpressionUUID" -> \
"4834f760-bc11-4470-ae85-d3e2c381d98d"]
}, Closed]],
Cell[CellGroupData[{
Cell[864907, 22521, 444, 8, 79, "Subsubsection", "ExpressionUUID" -> \
"cef993ed-997a-4adf-aad6-7def2cae63df"],
Cell[865354, 22531, 384, 8, 2499, "Text", "ExpressionUUID" -> \
"3b2fa659-5c33-4c68-a5cb-dbe2c277b4f3"],
Cell[865741, 22541, 853, 24, 1464, "Input", "ExpressionUUID" -> \
"0f27bcfc-6291-4f36-9153-4cf098ded909",
 Evaluatable->False],
Cell[866597, 22567, 449, 10, 3349, "Text", "ExpressionUUID" -> \
"582096e3-a0c5-4c21-90b6-c1994edac61f"],
Cell[867049, 22579, 841, 19, 1464, "Input", "ExpressionUUID" -> \
"5bc4f8c7-a5a5-4e0d-a609-38b47da96d9b"],
Cell[867893, 22600, 170, 3, 1464, "Text", "ExpressionUUID" -> \
"947ed7c7-b718-4bb3-b9db-7252523dd174"],
Cell[868066, 22605, 264, 6, 1464, "Input", "ExpressionUUID" -> \
"0ab56e59-7641-411c-b23d-e331bd681619"],
Cell[868333, 22613, 165, 3, 1464, "Text", "ExpressionUUID" -> \
"1c2df296-3edf-4ef8-91cc-1e74fcf4f60b"],
Cell[868501, 22618, 679, 18, 2460, "Input", "ExpressionUUID" -> \
"a23baf47-df39-41ad-ad61-fc6f8bc1585c"],
Cell[869183, 22638, 235, 4, 1464, "Text", "ExpressionUUID" -> \
"53e285e0-be3e-4d98-97f6-ad374beb0764"],
Cell[869421, 22644, 1037, 28, 2460, "Input", "ExpressionUUID" -> \
"7b39b87f-fc75-48bd-8b0c-d311d2637ad9"],
Cell[870461, 22674, 179, 3, 1464, "Text", "ExpressionUUID" -> \
"745e51d9-4b29-4f4c-b530-bcee2a2a7f39"],
Cell[870643, 22679, 1311, 34, 4365, "Input", "ExpressionUUID" -> \
"8c02c36c-c1f0-4f81-8969-c7762c03c361"],
Cell[871957, 22715, 1362, 35, 4365, "Input", "ExpressionUUID" -> \
"c68f3506-abb9-44b3-9b86-8b3ee7a4d602"],
Cell[873322, 22752, 888, 23, 2460, "Input", "ExpressionUUID" -> \
"c27da298-5b1f-47b4-a490-ad3ccbbddc95"],
Cell[874213, 22777, 168, 3, 1464, "Text", "ExpressionUUID" -> \
"81e92da2-3a0d-4b49-8c46-302d5b316754"],
Cell[874384, 22782, 480, 9, 1464, "Input", "ExpressionUUID" -> \
"49eb3f0b-22b9-4b43-87de-d9b1053c456d"],
Cell[874867, 22793, 159, 3, 1464, "Text", "ExpressionUUID" -> \
"a8f0a34f-4c88-433c-adcc-47a47ccab5db"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[875087, 22803, 686, 11, 150, "Section", "ExpressionUUID" -> \
"8444e20e-e68a-498b-a0e7-13d1e2769ffb"],
Cell[CellGroupData[{
Cell[875798, 22818, 405, 6, 67, "Subsection", "ExpressionUUID" -> \
"bc7ff31c-8f62-4879-80e2-43b2a47f2ec1"],
Cell[876206, 22826, 192, 3, 292, "Text", "ExpressionUUID" -> \
"73eabd3d-5689-4011-ba6f-5b31c7c627e8"],
Cell[876401, 22831, 200, 4, 295, "Input", "ExpressionUUID" -> \
"2a7609dd-edf6-45a8-8c72-f2eac5f654c9"],
Cell[876604, 22837, 335, 9, 295, "Input", "ExpressionUUID" -> \
"6061f375-6524-4b11-ba09-f428b405f0bf"],
Cell[876942, 22848, 187, 5, 295, "Input", "ExpressionUUID" -> \
"eb2b25f6-f85c-43c9-aafc-9e84b2238dcf"],
Cell[877132, 22855, 241, 6, 295, "Input", "ExpressionUUID" -> \
"04ec3f93-92a8-4ee2-985c-1b5b2351e12e"],
Cell[877376, 22863, 348, 5, 1464, "Text", "ExpressionUUID" -> \
"c04c6dcf-fdc2-4a3b-8f74-808ce900a9de"],
Cell[877727, 22870, 311, 6, 1464, "Input", "ExpressionUUID" -> \
"530dc929-b5c2-4559-8e53-c0a37fca1594"],
Cell[878041, 22878, 263, 4, 1464, "Text", "ExpressionUUID" -> \
"a4eacb80-bb81-4c4e-8d60-41cbcf902ee1"],
Cell[878307, 22884, 331, 6, 1464, "Input", "ExpressionUUID" -> \
"9a9ff88a-c924-4efa-b212-7d840cde2ba6"],
Cell[878641, 22892, 215, 5, 1464, "Text", "ExpressionUUID" -> \
"64d4a194-0a1c-47c3-9b91-f0b308c80995"],
Cell[878859, 22899, 168, 3, 1464, "Input", "ExpressionUUID" -> \
"3a8bf759-e8b6-4624-b8e8-66e7b7f3a0d1"],
Cell[879030, 22904, 616, 11, 1464, "Input", "ExpressionUUID" -> \
"2dd8aa5e-b9cb-41b5-9e4d-0f480e79de94"],
Cell[879649, 22917, 658, 11, 1464, "Input", "ExpressionUUID" -> \
"4e3609c0-915f-41b0-90d3-144af4446e29"],
Cell[880310, 22930, 220, 5, 1464, "Input", "ExpressionUUID" -> \
"9eee865f-1630-4ac3-a4c1-ba299c03df64"],
Cell[880533, 22937, 248, 5, 1464, "Text", "ExpressionUUID" -> \
"56ddf38c-de52-43ab-b7fe-439fc8f6475c"],
Cell[880784, 22944, 1885, 40, 4365, "Input", "ExpressionUUID" -> \
"497ecb87-672f-4ca2-9ea4-3207458dfbf6"],
Cell[882672, 22986, 216, 6, 1464, "Input", "ExpressionUUID" -> \
"4f7be7d0-66a1-46f0-a1a2-f10663edb01a"],
Cell[882891, 22994, 170, 3, 1464, "Text", "ExpressionUUID" -> \
"8671a1e6-6fed-4162-b328-becb3026457e"],
Cell[883064, 22999, 552, 14, 1464, "Input", "ExpressionUUID" -> \
"ee9e8729-2de0-46b9-85ef-46a33a8765a1"],
Cell[883619, 23015, 213, 5, 1464, "Input", "ExpressionUUID" -> \
"ea08e82f-a59e-428b-aab6-f5f0b42b32ae"],
Cell[883835, 23022, 292, 6, 2431, "Text", "ExpressionUUID" -> \
"69d58b0d-51e7-434b-a6dc-534d2423982b"],
Cell[884130, 23030, 268, 6, 1464, "Input", "ExpressionUUID" -> \
"668e4791-f976-4ed2-ab88-ce68a36aba9e"],
Cell[884401, 23038, 160, 3, 1464, "Text", "ExpressionUUID" -> \
"66f97417-f507-4140-8e28-d3bc5c739fa1"],
Cell[884564, 23043, 254, 6, 1464, "Input", "ExpressionUUID" -> \
"8da6bf85-7caf-4189-ada3-92bfed2afffa"],
Cell[884821, 23051, 1256, 21, 4432, "Text", "ExpressionUUID" -> \
"a69b7a2e-5efc-4cfc-840b-20aeaab58b4b"],
Cell[886080, 23074, 245, 4, 1464, "Text", "ExpressionUUID" -> \
"a4d5266c-3a56-4982-b8e3-0f9adeefe7de"],
Cell[886328, 23080, 516, 15, 1708, "Input", "ExpressionUUID" -> \
"9f2f1a14-b478-40dd-9151-82c8152887d1",
 Evaluatable->False],
Cell[886847, 23097, 1189, 28, 3573, "Text", "ExpressionUUID" -> \
"1bf8bf7a-f336-4c5f-9e73-1b6c406e77cf"],
Cell[888039, 23127, 1650, 48, 1708, "Input", "ExpressionUUID" -> \
"e1383cf7-8eb3-4d9a-aaf6-a0d4f97bb190",
 Evaluatable->False],
Cell[889692, 23177, 1202, 29, 2724, "Text", "ExpressionUUID" -> \
"f82626b6-4732-4d05-88f7-d56e3dc13799"],
Cell[890897, 23208, 1439, 40, 1708, "Input", "ExpressionUUID" -> \
"bf9af45b-2257-42c2-8d47-fb20c388f9ab",
 Evaluatable->False],
Cell[892339, 23250, 2586, 54, 7567, "Text", "ExpressionUUID" -> \
"3d07847c-dc3b-4608-b53b-7d0726cb9ade"]
}, Closed]],
Cell[CellGroupData[{
Cell[894962, 23309, 520, 10, 53, "Subsection", "ExpressionUUID" -> \
"7c0f4aa8-e57e-4c5f-8ab4-52ebc8991276"],
Cell[CellGroupData[{
Cell[895507, 23323, 432, 7, 53, "Subsubsection", "ExpressionUUID" -> \
"2053ffc7-e32b-4c2b-8df7-d11e8a53aa46"],
Cell[895942, 23332, 2057, 34, 4306, "Text", "ExpressionUUID" -> \
"bdcdaf63-ae37-409a-83b8-d6072910cc34"],
Cell[898002, 23368, 686, 23, 1533, "Input", "ExpressionUUID" -> \
"3ed4db79-b202-468d-adca-bbc6ece279f8",
 Evaluatable->False],
Cell[898691, 23393, 2885, 49, 5283, "Text", "ExpressionUUID" -> \
"bc8b9592-1504-4276-8f9e-d04a450a3b84"],
Cell[901579, 23444, 1586, 42, 1533, "Input", "ExpressionUUID" -> \
"66a6e240-8ddb-45f8-befb-3f446977112d",
 Evaluatable->False],
Cell[903168, 23488, 1070, 21, 1503, "Text", "ExpressionUUID" -> \
"27a8db33-ed3a-48d2-a695-26b32b5ee7d4"],
Cell[904241, 23511, 542, 14, 1464, "Input", "ExpressionUUID" -> \
"b753a09c-59fc-4d69-8aab-f575727ebe45",
 Evaluatable->False],
Cell[904786, 23527, 539, 9, 1503, "Text", "ExpressionUUID" -> \
"75482fa3-e411-471f-adc7-3f665112a237"],
Cell[905328, 23538, 870, 31, 2031, "Input", "ExpressionUUID" -> \
"00f0dd21-e3b1-499a-bb21-8966d229b5a8",
 Evaluatable->False],
Cell[906201, 23571, 2329, 46, 4540, "Text", "ExpressionUUID" -> \
"1e41cfa1-2e38-49f9-b623-8bdbc0362d4a"],
Cell[908533, 23619, 1989, 37, 5224, "Text", "ExpressionUUID" -> \
"2ae368db-f4e2-4ec4-988f-9b8623c6b299"]
}, Closed]],
Cell[CellGroupData[{
Cell[910559, 23661, 532, 8, 42, "Subsubsection", "ExpressionUUID" -> \
"08f5f113-7e76-44fe-a6af-d61e398ba9ab"],
Cell[911094, 23671, 1204, 26, 3436, "Text", "ExpressionUUID" -> \
"10bbc49e-1fca-4ea5-84c3-62a58b2979c2"],
Cell[912301, 23699, 983, 22, 1708, "Input", "ExpressionUUID" -> \
"cc4da17e-5602-4efc-b588-feb3cda05a79",
 Evaluatable->False],
Cell[913287, 23723, 986, 16, 3349, "Text", "ExpressionUUID" -> \
"4c0d7bf1-4683-4d20-b55b-2cc422306ec4"],
Cell[914276, 23741, 1871, 47, 1708, "Input", "ExpressionUUID" -> \
"05fa5980-4986-4aa0-8d01-08614c4d8aba",
 Evaluatable->False],
Cell[916150, 23790, 3417, 62, 1581, "Text", "ExpressionUUID" -> \
"d9b9b4f5-8a1b-4e98-a221-5ad20c334423"],
Cell[919570, 23854, 721, 22, 1708, "Input", "ExpressionUUID" -> \
"bd9e17e3-e6ab-4fa3-befd-0332754fdb27",
 Evaluatable->False],
Cell[920294, 23878, 1306, 20, 4306, "Text", "ExpressionUUID" -> \
"59ed459b-d27b-40fa-8193-f992fe752df9"],
Cell[921603, 23900, 1052, 25, 1591, "Input", "ExpressionUUID" -> \
"7fd6185d-510a-40a6-99a3-aaa1b48f15fa",
 Evaluatable->False],
Cell[922658, 23927, 2139, 44, 2724, "Text", "ExpressionUUID" -> \
"399db336-f712-4173-9e75-699911398bd4"],
Cell[924800, 23973, 520, 15, 1669, "Input", "ExpressionUUID" -> \
"022ee80c-fd16-4cbe-ad36-afcf83870e48",
 Evaluatable->False],
Cell[925323, 23990, 1331, 21, 1464, "Text", "ExpressionUUID" -> \
"6731714d-ac7c-4063-be83-d765cebd2fba"],
Cell[926657, 24013, 1012, 32, 1533, "Input", "ExpressionUUID" -> \
"76240294-4c4d-41bc-abbc-36895f23ad70",
 Evaluatable->False],
Cell[927672, 24047, 1676, 26, 5361, "Text", "ExpressionUUID" -> \
"ee8677d8-db5a-4785-a8ef-5e8c9c8450b7"],
Cell[929351, 24075, 1013, 26, 1767, "Input", "ExpressionUUID" -> \
"d57f0c42-fafb-4631-9824-2935f64941a9",
 Evaluatable->False],
Cell[930367, 24103, 1974, 45, 2635, "Text", "ExpressionUUID" -> \
"c9891e55-67af-494a-b505-38854f12cc4d"],
Cell[932344, 24150, 1017, 27, 1767, "Input", "ExpressionUUID" -> \
"44622928-49dd-476d-a8a4-4e7796d13fe8",
 Evaluatable->False],
Cell[933364, 24179, 1260, 24, 2694, "Text", "ExpressionUUID" -> \
"b4e10f36-061f-4c0e-884f-53c9c8332a82"],
Cell[934627, 24205, 1696, 48, 1854, "Input", "ExpressionUUID" -> \
"e81a175e-31dd-43af-80d1-3d7259f62d3c",
 Evaluatable->False],
Cell[936326, 24255, 252, 4, 1464, "Text", "ExpressionUUID" -> \
"a0804bed-7332-4eeb-aea0-bf535c002abe"],
Cell[936581, 24261, 1119, 31, 1767, "Input", "ExpressionUUID" -> \
"080375cc-8425-494c-a78d-b132b6ff3522",
 Evaluatable->False],
Cell[937703, 24294, 184, 3, 1464, "Text", "ExpressionUUID" -> \
"ab973c84-8d35-4a3e-86ba-c498c0e0113c"],
Cell[937890, 24299, 1583, 44, 2851, "Input", "ExpressionUUID" -> \
"fa664a8b-97a2-429f-98ee-c53dcbd9d69a",
 Evaluatable->False],
Cell[939476, 24345, 348, 7, 1503, "Text", "ExpressionUUID" -> \
"3c8e3d10-11a6-4ab3-bdb2-a562136f4fbd"],
Cell[939827, 24354, 1729, 45, 1854, "Input", "ExpressionUUID" -> \
"cfd192c6-31a1-4379-970d-191a0fde0869",
 Evaluatable->False],
Cell[941559, 24401, 275, 6, 1464, "Text", "ExpressionUUID" -> \
"00287024-e101-49a5-b8b9-305bed30476c"],
Cell[941837, 24409, 1491, 38, 1854, "Input", "ExpressionUUID" -> \
"149285cb-f07d-4c1d-898b-7d87a5f51db1",
 Evaluatable->False],
Cell[943331, 24449, 1854, 38, 1767, "Text", "ExpressionUUID" -> \
"c83175d7-b4db-411d-a380-ec881454f0af"],
Cell[945188, 24489, 1268, 36, 1854, "Input", "ExpressionUUID" -> \
"7a39b636-4cdc-4cfb-9832-e190bbd1a7a6",
 Evaluatable->False],
Cell[946459, 24527, 1212, 29, 1767, "Text", "ExpressionUUID" -> \
"997e9733-cc78-48c0-b988-b721cb5755ff"],
Cell[947674, 24558, 1004, 28, 1708, "Input", "ExpressionUUID" -> \
"e07ac9cd-b98b-497b-b1be-1787c5976d85",
 Evaluatable->False],
Cell[948681, 24588, 255, 6, 1503, "Text", "ExpressionUUID" -> \
"ea68ca00-82c0-4bfa-9ebb-4068e93a0c23"],
Cell[948939, 24596, 1339, 36, 1854, "Input", "ExpressionUUID" -> \
"7fdd72b2-ef1e-435d-864c-57e4624837da",
 Evaluatable->False],
Cell[950281, 24634, 1855, 42, 2821, "Text", "ExpressionUUID" -> \
"84a7b250-057a-467f-b70c-24dee335391c"],
Cell[952139, 24678, 2365, 55, 4608, "Text", "ExpressionUUID" -> \
"7e4c49a9-af77-4b75-be3f-aaa5ac0665d0"],
Cell[954507, 24735, 620, 12, 2499, "Text", "ExpressionUUID" -> \
"3071c131-a739-426b-82d5-6b3a770e4ddb"],
Cell[955130, 24749, 418, 9, 1503, "Text", "ExpressionUUID" -> \
"eed96b68-31c3-4fc3-a32b-153bae128be5"],
Cell[955551, 24760, 942, 20, 1845, "Input", "ExpressionUUID" -> \
"5f36ba5e-475e-4207-ba38-b9c236258dd0",
 Evaluatable->False],
Cell[956496, 24782, 824, 20, 2499, "Text", "ExpressionUUID" -> \
"190d1bfc-f4fa-4224-8b06-2cf0c618f2c0"],
Cell[957323, 24804, 469, 10, 1464, "Text", "ExpressionUUID" -> \
"31700f5f-3444-4838-9582-90667c1358a3"],
Cell[957795, 24816, 783, 22, 1854, "Input", "ExpressionUUID" -> \
"092a7833-f092-493a-99ac-629cdfdeb678",
 Evaluatable->False],
Cell[958581, 24840, 321, 6, 1464, "Text", "ExpressionUUID" -> \
"8aea8363-0b4a-4de3-9c0e-ae0ea73339dc"],
Cell[958905, 24848, 743, 21, 1708, "Input", "ExpressionUUID" -> \
"130de56b-d651-4258-a910-d65e3b65402b",
 Evaluatable->False],
Cell[959651, 24871, 147, 3, 1464, "Text", "ExpressionUUID" -> \
"a3f56db3-fe09-4bfd-8c7e-9713543acc95"],
Cell[959801, 24876, 927, 26, 1708, "Input", "ExpressionUUID" -> \
"58e00696-96c6-410e-b6b0-bc82cfadd921",
 Evaluatable->False],
Cell[960731, 24904, 322, 8, 1464, "Text", "ExpressionUUID" -> \
"d43ccc33-05b0-427b-b04b-803447ff3a1b"],
Cell[961056, 24914, 1375, 38, 3427, "Input", "ExpressionUUID" -> \
"72536174-ab52-4a5b-ba11-83f032a3cec1",
 Evaluatable->False,
 InitializationCell->True],
Cell[962434, 24954, 322, 6, 1464, "Input", "ExpressionUUID" -> \
"b8ab1559-3846-4092-a52b-630bc8738161"],
Cell[962759, 24962, 253, 5, 1464, "Text", "ExpressionUUID" -> \
"2b52293b-a902-4df1-8b08-1ec6c0d32af2"],
Cell[963015, 24969, 315, 8, 1464, "Input", "ExpressionUUID" -> \
"67671b47-1e6e-4426-910e-77ddf0448edb"],
Cell[963333, 24979, 192, 4, 1464, "Input", "ExpressionUUID" -> \
"fe834f22-d396-4e5d-b31c-7d000b15457f"],
Cell[963528, 24985, 195, 4, 1464, "Input", "ExpressionUUID" -> \
"8a73a7e3-3ba6-4222-ab37-f64d11262636"],
Cell[963726, 24991, 263, 6, 1464, "Text", "ExpressionUUID" -> \
"96309d5c-6969-4ce5-a786-0183a86b23ab"],
Cell[963992, 24999, 410, 8, 3378, "Text", "ExpressionUUID" -> \
"caeeac11-398f-4d26-a7d8-255aa4887b57"],
Cell[964405, 25009, 1420, 38, 2460, "Input", "ExpressionUUID" -> \
"41657add-6cd5-43e5-90a2-7bc0186b9a77",
 Evaluatable->False,
 InitializationCell->True],
Cell[965828, 25049, 173, 4, 1464, "Input", "ExpressionUUID" -> \
"c009698c-18c2-467c-8b3c-7c103707477e"],
Cell[966004, 25055, 104, 1, 1464, "Input", "ExpressionUUID" -> \
"ecf158f5-3069-4035-93eb-93d89128a49a"],
Cell[966111, 25058, 190, 4, 1464, "Input", "ExpressionUUID" -> \
"bd4bcfd8-a998-4272-8c65-8c089230db30"],
Cell[966304, 25064, 641, 14, 2499, "Text", "ExpressionUUID" -> \
"4e139ab4-df47-44ac-836b-2dd9efaa9ecd"],
Cell[966948, 25080, 1007, 20, 2499, "Text", "ExpressionUUID" -> \
"28c46e1e-0f6c-4df0-9b41-3dbc3656f944"],
Cell[967958, 25102, 469, 11, 1464, "Input", "ExpressionUUID" -> \
"4abb2a9f-eb98-4c67-a5a9-8f640e879caf",
 Evaluatable->False],
Cell[968430, 25115, 333, 6, 1464, "Input", "ExpressionUUID" -> \
"7b631b08-cdac-484f-b0ca-51823aa99b54"],
Cell[968766, 25123, 286, 7, 1464, "Input", "ExpressionUUID" -> \
"d2cec096-d8aa-41c0-ac36-1ab31314c28e"],
Cell[969055, 25132, 1500, 31, 2499, "Text", "ExpressionUUID" -> \
"a6cf954f-1210-4888-b6b8-88876c3d3b0e"],
Cell[970558, 25165, 459, 14, 1533, "Input", "ExpressionUUID" -> \
"8ac76c4f-c0f6-453e-a5ce-cd5ac8418007",
 Evaluatable->False],
Cell[971020, 25181, 355, 7, 2431, "Text", "ExpressionUUID" -> \
"b30ba1be-f0fc-481e-96fc-46636bdec8a3"],
Cell[971378, 25190, 242, 6, 1464, "Input", "ExpressionUUID" -> \
"1c0060f1-13fb-4cb0-8119-ead231561fb4"],
Cell[971623, 25198, 226, 5, 1464, "Input", "ExpressionUUID" -> \
"f7a5a2b4-f0e4-4554-a5b6-dce5c0107982"],
Cell[971852, 25205, 612, 15, 1464, "Input", "ExpressionUUID" -> \
"bf6c0cf5-f3f6-4592-a590-0c79077ea14c"],
Cell[972467, 25222, 529, 12, 1464, "Input", "ExpressionUUID" -> \
"91157414-e6f1-4135-b49e-bb610d5423f1"],
Cell[972999, 25236, 220, 5, 1464, "Input", "ExpressionUUID" -> \
"eaa27ba4-804d-44a1-9ec0-b13b4ebac838"],
Cell[973222, 25243, 325, 8, 1503, "Text", "ExpressionUUID" -> \
"4085e9aa-31f7-4c0c-a4d1-152fa7ded096"],
Cell[973550, 25253, 340, 8, 1503, "Text", "ExpressionUUID" -> \
"ca23dd82-e654-4a45-abca-fc4bde89b21c"],
Cell[973893, 25263, 207, 5, 1464, "Input", "ExpressionUUID" -> \
"9f273585-0100-4389-b631-f20505fd8178"],
Cell[974103, 25270, 197, 3, 1464, "Text", "ExpressionUUID" -> \
"61134587-8c4d-4fbb-a468-0cf42e91238d"],
Cell[974303, 25275, 191, 4, 1464, "Input", "ExpressionUUID" -> \
"4692dc75-3aae-4d41-b2c5-861011a3ca1d"],
Cell[974497, 25281, 379, 9, 1464, "Input", "ExpressionUUID" -> \
"0df89fe7-1bdb-477e-bc8e-7fd5d2e89d3c"],
Cell[974879, 25292, 613, 14, 1464, "Input", "ExpressionUUID" -> \
"be92a03b-2804-4cf8-b93b-9f80d70b42aa"],
Cell[975495, 25308, 211, 5, 1464, "Input", "ExpressionUUID" -> \
"ff4f20aa-8b0f-48a7-82dc-bfb903bb429c"],
Cell[975709, 25315, 440, 13, 1464, "Input", "ExpressionUUID" -> \
"e9f4c5fa-de14-4ae2-8c85-ad35c717838b"],
Cell[976152, 25330, 209, 5, 1464, "Input", "ExpressionUUID" -> \
"c97138df-378e-493a-acb8-cce6c64f972c"],
Cell[976364, 25337, 535, 13, 1464, "Input", "ExpressionUUID" -> \
"fabea072-344c-4cfc-b4f0-731d2b009550"],
Cell[976902, 25352, 219, 5, 1464, "Input", "ExpressionUUID" -> \
"bf8c33d2-2441-4155-8c79-9554203dbd6d"],
Cell[977124, 25359, 630, 17, 1464, "Input", "ExpressionUUID" -> \
"83d9afcc-ee3c-4c7b-af21-739f691f2654"],
Cell[977757, 25378, 214, 5, 1464, "Input", "ExpressionUUID" -> \
"5316be20-20e3-49ac-a47d-a03a3b7b78e9"],
Cell[977974, 25385, 884, 24, 1464, "Input", "ExpressionUUID" -> \
"0d8cb6b7-c821-4052-aa09-613dff1203ab"],
Cell[978861, 25411, 212, 5, 1464, "Input", "ExpressionUUID" -> \
"e83d2f26-d15b-4bff-b274-de7dcd5bbbcf"],
Cell[979076, 25418, 467, 10, 1464, "Input", "ExpressionUUID" -> \
"87a901cf-b840-4a6f-b921-2bcf215868f1"],
Cell[979546, 25430, 220, 4, 1464, "Input", "ExpressionUUID" -> \
"da44ab6a-a55f-448c-bead-eeb3e6b2f38c"],
Cell[979769, 25436, 651, 18, 1854, "Input", "ExpressionUUID" -> \
"9b11da8a-ecb9-4280-9abc-c1726ea420a9",
 Evaluatable->False],
Cell[980423, 25456, 1458, 40, 2460, "Input", "ExpressionUUID" -> \
"42c40674-6935-47d5-8057-2332ba3b57f1"],
Cell[981884, 25498, 1648, 44, 2460, "Input", "ExpressionUUID" -> \
"c57d3842-8295-4b21-81d0-aad896fece73"],
Cell[983535, 25544, 1621, 43, 2460, "Input", "ExpressionUUID" -> \
"4239b410-c4f9-4703-a384-a02990eea94e"],
Cell[985159, 25589, 1940, 47, 2460, "Input", "ExpressionUUID" -> \
"20733ffc-fc21-43b0-8bf0-d3d478a1c01b"],
Cell[987102, 25638, 1587, 42, 2460, "Input", "ExpressionUUID" -> \
"c3bd6aae-1d8a-4425-9ead-8398c77cdb67"],
Cell[988692, 25682, 1675, 44, 2460, "Input", "ExpressionUUID" -> \
"f94901c3-2955-4562-be01-bce15423725a"],
Cell[990370, 25728, 1733, 45, 2460, "Input", "ExpressionUUID" -> \
"48f290e1-f9e9-4bcd-86ff-036e2b26e638"],
Cell[992106, 25775, 1921, 47, 2460, "Input", "ExpressionUUID" -> \
"13fce8bb-490c-4e58-ac77-e80dabf73150"],
Cell[994030, 25824, 571, 10, 2499, "Text", "ExpressionUUID" -> \
"2699d75f-fe2b-4dbf-a150-660f961d4d0f"],
Cell[994604, 25836, 1020, 28, 2460, "Input", "ExpressionUUID" -> \
"7b415449-55f1-4e11-bf51-665957c59840"],
Cell[995627, 25866, 319, 6, 1464, "Text", "ExpressionUUID" -> \
"388174cc-9e63-4c03-983a-f0897fc9d053"],
Cell[995949, 25874, 256, 5, 1464, "Input", "ExpressionUUID" -> \
"ab0408c4-4c61-408c-b878-c4d23131e1ad"],
Cell[996208, 25881, 416, 9, 1464, "Input", "ExpressionUUID" -> \
"7c3cc434-77b2-471f-b9dd-4b281b1a4207"],
Cell[996627, 25892, 997, 25, 2460, "Input", "ExpressionUUID" -> \
"4f21547f-82f3-46c3-a277-222f3d0b082e"],
Cell[997627, 25919, 138, 3, 1464, "Text", "ExpressionUUID" -> \
"023427e8-9cb6-4e72-ba6d-2537157b182b"],
Cell[997768, 25924, 454, 7, 1503, "Text", "ExpressionUUID" -> \
"0fd1dc46-c146-4d71-93d5-51f40d3c0d62"],
Cell[998225, 25933, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"c92663d9-fb00-454e-b6f3-49f1fcb7905e"],
Cell[998403, 25939, 229, 5, 2460, "Input", "ExpressionUUID" -> \
"fb3fb0dd-e6e7-4086-b34a-50800dd3bb81"],
Cell[998635, 25946, 512, 13, 1464, "Input", "ExpressionUUID" -> \
"ece9c4af-b548-49e1-8039-a12c0fc365bf"],
Cell[999150, 25961, 730, 17, 1464, "Input", "ExpressionUUID" -> \
"e4688d89-3a00-4a3a-94cf-5bfc3828bc80"],
Cell[999883, 25980, 326, 5, 1464, "Text", "ExpressionUUID" -> \
"57b334b3-a9cc-451e-ab4f-623ec5a98479"],
Cell[1000212, 25987, 177, 4, 1464, "Input", "ExpressionUUID" -> \
"95671b88-6d21-40a5-b830-3ba03b840b6e"],
Cell[1000392, 25993, 395, 8, 1464, "Input", "ExpressionUUID" -> \
"ee8daf7f-84ba-493e-8a35-d71fdfbcb4e9"],
Cell[1000790, 26003, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"81bc86b3-ce8f-4c28-9587-cdbdd5f0cdfb"],
Cell[1000980, 26009, 146, 3, 1464, "Input", "ExpressionUUID" -> \
"1589ecd1-93e0-4212-ae69-7baf2715abba",
 Evaluatable->False],
Cell[1001129, 26014, 264, 5, 1503, "Text", "ExpressionUUID" -> \
"f2c3b787-1c3f-47df-99c8-a1e5531929ba"],
Cell[1001396, 26021, 282, 8, 1464, "Text", "ExpressionUUID" -> \
"f9136cab-f083-494a-b8a9-6c8bea74555c"],
Cell[1001681, 26031, 402, 9, 2431, "Text", "ExpressionUUID" -> \
"80ad1153-59a7-4a59-9cc2-59e09f178d12"],
Cell[1002086, 26042, 304, 8, 1464, "Input", "ExpressionUUID" -> \
"f094e4fb-fd64-4882-9f65-f08303ab991a"],
Cell[1002393, 26052, 321, 9, 1464, "Input", "ExpressionUUID" -> \
"8b68f03f-611c-4383-97f2-0813ee2bc91b"],
Cell[1002717, 26063, 614, 11, 3349, "Text", "ExpressionUUID" -> \
"e95e79e5-35e4-4700-b5ce-55fe80c1d8a5"],
Cell[1003334, 26076, 280, 6, 2431, "Text", "ExpressionUUID" -> \
"22b5139d-726d-4367-9489-904d89309cf4"],
Cell[1003617, 26084, 528, 13, 1464, "Input", "ExpressionUUID" -> \
"ffee442f-beaf-4631-9ec0-87853fdc6253"],
Cell[1004148, 26099, 750, 17, 3514, "Text", "ExpressionUUID" -> \
"6810e564-e63d-4646-b601-b19889f66b0f"],
Cell[1004901, 26118, 436, 9, 1464, "Input", "ExpressionUUID" -> \
"1c94ff7d-0a53-42fd-be83-20598ad0e137"],
Cell[1005340, 26129, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"7e306c15-0da7-432a-ac81-cada6d3c956e"],
Cell[1005528, 26135, 478, 13, 1464, "Input", "ExpressionUUID" -> \
"84721f9a-043b-4c6f-b094-e939756ed923"],
Cell[1006009, 26150, 294, 7, 3378, "Text", "ExpressionUUID" -> \
"9b75b06c-0fc5-4373-8b11-7c92c7c3e414"],
Cell[1006306, 26159, 403, 12, 1464, "Input", "ExpressionUUID" -> \
"2233aafd-877c-43a3-ad75-aeae19f7a7e8"],
Cell[1006712, 26173, 457, 12, 1464, "Input", "ExpressionUUID" -> \
"e25299c1-18c9-42d8-8af1-82c3005eb52a"],
Cell[1007172, 26187, 414, 8, 3349, "Text", "ExpressionUUID" -> \
"37eba8c3-0483-4678-bf0b-f8f2973e4f07"],
Cell[1007589, 26197, 690, 19, 1464, "Input", "ExpressionUUID" -> \
"e19af47b-4875-4c0e-8a71-ed4d3ab15a3c"],
Cell[1008282, 26218, 178, 3, 1464, "Text", "ExpressionUUID" -> \
"4e8d1b7f-fd0c-4543-b6a9-b436765d9d40"],
Cell[1008463, 26223, 454, 12, 1464, "Input", "ExpressionUUID" -> \
"83e188b1-6d1c-41fd-b780-da2fc7a1fa26"],
Cell[1008920, 26237, 159, 3, 1464, "Text", "ExpressionUUID" -> \
"a7dfabc7-c0fa-4ef9-bc4e-7231bd351c0b"],
Cell[1009082, 26242, 71, 0, 1464, "Text", "ExpressionUUID" -> \
"5f465b5c-66df-4b2b-ad8c-e174afab3cbb"]
}, Closed]],
Cell[CellGroupData[{
Cell[1009190, 26247, 913, 14, 79, "Subsubsection", "ExpressionUUID" -> \
"53a6ac1f-4f4e-4dc2-aa22-b6184f4402fe"],
Cell[1010106, 26263, 1064, 20, 2431, "Text", "ExpressionUUID" -> \
"33c28844-756c-4515-b8a7-b46b55c461f8"],
Cell[1011173, 26285, 562, 14, 1591, "Input", "ExpressionUUID" -> \
"f916e2c2-a4c1-4e13-a9dd-f90c0e89e286",
 Evaluatable->False],
Cell[1011738, 26301, 238, 6, 1464, "Input", "ExpressionUUID" -> \
"f23fbeb5-8042-4fc5-a8be-d7c07c0a32c9"],
Cell[1011979, 26309, 184, 4, 1464, "Input", "ExpressionUUID" -> \
"a3ae5515-b7fa-4e1f-9415-fdc2ac454466"],
Cell[1012166, 26315, 355, 5, 1464, "Text", "ExpressionUUID" -> \
"2c330baa-afcf-49a0-a330-db6a5a0eb25a"],
Cell[1012524, 26322, 1455, 36, 3837, "Input", "ExpressionUUID" -> \
"934eceb0-0287-4178-8b43-85613c87c2ef",
 Evaluatable->False,
 CellTags->"gammatr"],
Cell[1013982, 26360, 200, 4, 1464, "Input", "ExpressionUUID" -> \
"ab64016a-26d0-4588-b2e1-cf619ae32a50"],
Cell[1014185, 26366, 788, 15, 1464, "Input", "ExpressionUUID" -> \
"5a19c509-932c-4e98-b455-e53c344294bc"],
Cell[1014976, 26383, 825, 19, 1464, "Input", "ExpressionUUID" -> \
"a5953371-67b2-4c7e-aabd-b7bea8f7012b",
 Evaluatable->False],
Cell[1015804, 26404, 229, 5, 1464, "Input", "ExpressionUUID" -> \
"41fbadf1-86c5-4766-8d06-d5f806e74ce6"],
Cell[1016036, 26411, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"087e3758-c892-4640-b13e-c36ca6972052"],
Cell[1016213, 26417, 155, 3, 1464, "Input", "ExpressionUUID" -> \
"6ad26a66-8177-47d3-ac21-c356384f1de3"],
Cell[1016371, 26422, 174, 4, 1464, "Input", "ExpressionUUID" -> \
"45be3b67-c890-4796-82e3-c08f452dcbe2"],
Cell[1016548, 26428, 986, 16, 3436, "Text", "ExpressionUUID" -> \
"11e72b91-ba17-469b-9dda-f3f81f2587da"],
Cell[1017537, 26446, 2392, 46, 4404, "Text", "ExpressionUUID" -> \
"6058aaa5-4206-4f7e-95b5-1ae1f177ff3c"],
Cell[1019932, 26494, 1582, 41, 1533, "Input", "ExpressionUUID" -> \
"8f47c1aa-d0aa-40b9-ab0a-5f7c0987ff06",
 Evaluatable->False],
Cell[1021517, 26537, 753, 11, 1464, "Text", "ExpressionUUID" -> \
"864df167-993d-4cce-9596-390abdf3cc8c"],
Cell[1022273, 26550, 841, 12, 1464, "Text", "ExpressionUUID" -> \
"c39c23d2-c0ca-4726-8bc1-676a9630b47f"],
Cell[1023117, 26564, 816, 14, 2431, "Text", "ExpressionUUID" -> \
"a527f0a3-0e32-4d2a-bc7e-3556d2fe24fd"],
Cell[1023936, 26580, 627, 14, 1464, "Input", "ExpressionUUID" -> \
"27a2cab6-a298-4c05-af95-35ec07b9231a",
 Evaluatable->False],
Cell[1024566, 26596, 610, 13, 2499, "Text", "ExpressionUUID" -> \
"bb552de7-1852-4e0f-b42b-2d1443e83bf5"],
Cell[1025179, 26611, 798, 21, 1533, "Input", "ExpressionUUID" -> \
"d565a965-0431-4902-8df8-05319e463131",
 Evaluatable->False],
Cell[1025980, 26634, 1301, 27, 2499, "Text", "ExpressionUUID" -> \
"5322cad6-207b-41eb-ab94-dad7457bdf0b"],
Cell[1027284, 26663, 1466, 41, 4432, "Input", "ExpressionUUID" -> \
"d4dc10e4-451c-46c4-8fd9-61038a01e372",
 Evaluatable->False],
Cell[1028753, 26706, 413, 7, 1464, "Text", "ExpressionUUID" -> \
"5a776ee5-f32d-4837-8d16-26a831c2436f"],
Cell[1029169, 26715, 528, 12, 1728, "Input", "ExpressionUUID" -> \
"7ab197d7-07fa-4128-8616-da835b51ca14",
 Evaluatable->False],
Cell[1029700, 26729, 308, 6, 1464, "Text", "ExpressionUUID" -> \
"fdcac56c-3635-42ac-aef9-7e42bdfaf5be"],
Cell[1030011, 26737, 872, 29, 1464, "Input", "ExpressionUUID" -> \
"a6a01e57-0b47-43f0-9816-1cfbfa46b808",
 Evaluatable->False],
Cell[1030886, 26768, 1805, 38, 4569, "Text", "ExpressionUUID" -> \
"b90ac6ab-428b-4007-8cf4-1d0e660e4a7d"],
Cell[1032694, 26808, 762, 25, 1669, "Input", "ExpressionUUID" -> \
"397b7350-b0ee-4de7-886b-d031bc559fb1",
 Evaluatable->False],
Cell[1033459, 26835, 3041, 56, 7206, "Text", "ExpressionUUID" -> \
"a2aa218c-a659-4462-ac24-3101dd4f163c"],
Cell[1036503, 26893, 172, 4, 1464, "Input", "ExpressionUUID" -> \
"c5565668-27c9-4e68-a019-6bc0e39bff1d"],
Cell[1036678, 26899, 468, 10, 1464, "Input", "ExpressionUUID" -> \
"08c0d59d-f938-4acf-9f70-217c2b855b0f"],
Cell[1037149, 26911, 1460, 20, 1464, "Text", "ExpressionUUID" -> \
"919d8e58-2c12-44ae-8ea4-d9bb093c50a0"],
Cell[1038612, 26933, 177, 4, 1464, "Input", "ExpressionUUID" -> \
"934182ea-0430-44dd-8d1b-d0c9312a87e1"],
Cell[1038792, 26939, 954, 19, 1464, "Input", "ExpressionUUID" -> \
"65018730-3f63-4828-a49d-3c236ce264ac",
 Evaluatable->False,
 InitializationCell->True],
Cell[1039749, 26960, 471, 10, 1464, "Input", "ExpressionUUID" -> \
"012e8d39-d447-458d-b74c-fb7458a0d6c7"],
Cell[1040223, 26972, 187, 4, 1464, "Input", "ExpressionUUID" -> \
"fa253dc1-134c-4d2c-badf-1ed7d9804652"],
Cell[1040413, 26978, 534, 13, 1464, "Input", "ExpressionUUID" -> \
"d4d26d46-2553-49f6-8dd7-93858da51ba9"],
Cell[1040950, 26993, 375, 9, 1464, "Input", "ExpressionUUID" -> \
"939ac70e-3c92-41cf-9413-48ccaf1661a3"],
Cell[1041328, 27004, 185, 4, 1464, "Input", "ExpressionUUID" -> \
"804c24f2-02ce-4294-9905-6a4426993ccf"],
Cell[1041516, 27010, 417, 8, 2431, "Text", "ExpressionUUID" -> \
"1aef142b-eb8a-4b0e-9942-01e515bc9d41"],
Cell[1041936, 27020, 563, 14, 1464, "Input", "ExpressionUUID" -> \
"24838d19-4dac-4d42-8c8a-01831a9ff29b"],
Cell[1042502, 27036, 326, 8, 1464, "Input", "ExpressionUUID" -> \
"1c120c38-120d-4a14-8c61-6a893c8a4bc4"],
Cell[1042831, 27046, 195, 3, 1503, "Text", "ExpressionUUID" -> \
"a47eec1f-fd8a-47fd-a4ce-79225a1ee2cc"],
Cell[1043029, 27051, 257, 7, 1464, "Input", "ExpressionUUID" -> \
"f4c4f731-e4a5-44d5-9648-2cc0d05acf0e"],
Cell[1043289, 27060, 486, 14, 1464, "Input", "ExpressionUUID" -> \
"f6a6fd25-b012-4301-861c-b66f4453acd4"],
Cell[1043778, 27076, 496, 14, 1464, "Input", "ExpressionUUID" -> \
"19dd409d-40f4-4427-a1f9-a2a6b5f89b7c"],
Cell[1044277, 27092, 192, 3, 1464, "Text", "ExpressionUUID" -> \
"f9899ad1-844f-4d51-a1bc-1c636fa34bab"],
Cell[1044472, 27097, 502, 14, 1464, "Input", "ExpressionUUID" -> \
"3a834201-ddec-49f9-adb5-45a909daa5cc"],
Cell[1044977, 27113, 309, 7, 2431, "Text", "ExpressionUUID" -> \
"0c736222-df65-4e03-b583-8eade4903d0d"],
Cell[1045289, 27122, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"709c9708-409c-4bf2-be4d-4afd79f7480c"],
Cell[1045467, 27128, 229, 4, 1464, "Text", "ExpressionUUID" -> \
"1a2f6c49-0372-4122-91f2-9ba2458bb9d6"],
Cell[1045699, 27134, 610, 16, 1464, "Input", "ExpressionUUID" -> \
"4c112bf9-c909-4411-af87-aaeab9598b5d"],
Cell[1046312, 27152, 152, 3, 1503, "Text", "ExpressionUUID" -> \
"c1fda347-73a0-41dc-b324-8190f1bf3486"],
Cell[1046467, 27157, 759, 19, 2460, "Input", "ExpressionUUID" -> \
"33806a95-f381-46e6-b20e-484be37ba6cf"],
Cell[1047229, 27178, 551, 10, 3436, "Text", "ExpressionUUID" -> \
"6323726a-111e-423e-ba08-567e78fee6e8"],
Cell[1047783, 27190, 253, 6, 1464, "Input", "ExpressionUUID" -> \
"bfb69729-4812-43d9-a165-9c043390885d"],
Cell[1048039, 27198, 367, 8, 2431, "Text", "ExpressionUUID" -> \
"c1df7b53-95fd-40fe-97b2-65f78a3d2024"],
Cell[1048409, 27208, 825, 22, 2460, "Input", "ExpressionUUID" -> \
"8bbb9630-e752-4a42-b627-6c26d1e50a04"],
Cell[1049237, 27232, 1466, 30, 3436, "Text", "ExpressionUUID" -> \
"4dabdeee-62d2-4b8a-92a6-2ad7b92ec267"],
Cell[1050706, 27264, 1096, 26, 1591, "Input", "ExpressionUUID" -> \
"59dbd65a-1c4f-4fe6-b9c6-b512d8220529",
 Evaluatable->False],
Cell[1051805, 27292, 665, 13, 3349, "Text", "ExpressionUUID" -> \
"8a54859d-1ad2-4e6a-82ab-5beef36ed253"],
Cell[1052473, 27307, 209, 5, 1464, "Input", "ExpressionUUID" -> \
"cb550792-17cb-434d-bceb-1479bfe17e3e"],
Cell[1052685, 27314, 682, 14, 1464, "Input", "ExpressionUUID" -> \
"540fa58b-db8f-4e1f-b358-39ef67d88bc8"],
Cell[1053370, 27330, 175, 4, 1464, "Input", "ExpressionUUID" -> \
"086c54cf-c073-4c43-be2b-f8d24e9053a4"],
Cell[1053548, 27336, 2187, 54, 5331, "Input", "ExpressionUUID" -> \
"c62f3dff-27bd-47c3-ba52-7a36622aa450",
 Evaluatable->False,
 InitializationCell->True],
Cell[1055738, 27392, 356, 8, 1464, "Input", "ExpressionUUID" -> \
"dc433e04-c252-4d80-abd5-68a7f48a853f"],
Cell[1056097, 27402, 253, 7, 1464, "Input", "ExpressionUUID" -> \
"ce2efe45-fe46-4899-8711-f07f76c9a690"],
Cell[1056353, 27411, 954, 19, 5361, "Text", "ExpressionUUID" -> \
"c8bdc31e-559c-42a4-bf5e-c9e338598b47"],
Cell[1057310, 27432, 737, 21, 1464, "Input", "ExpressionUUID" -> \
"cf982ebb-ecb2-4de7-a21f-74c82bdf3440",
 Evaluatable->False],
Cell[1058050, 27455, 446, 10, 1464, "Input", "ExpressionUUID" -> \
"c087bf19-ecb0-4e9f-a14e-55ac99ea39ca"],
Cell[1058499, 27467, 236, 5, 1464, "Input", "ExpressionUUID" -> \
"73c79f09-e9d8-42cb-8491-5fe15ec21b81"],
Cell[1058738, 27474, 356, 8, 1464, "Input", "ExpressionUUID" -> \
"7f728713-496e-49ad-a810-fe9757d1aae2"],
Cell[1059097, 27484, 274, 7, 1464, "Input", "ExpressionUUID" -> \
"8899c57c-f6df-4842-ac73-d6596d9bb3bf"],
Cell[1059374, 27493, 438, 10, 1464, "Input", "ExpressionUUID" -> \
"5913e5d5-8084-459d-aaf9-db701ac6ceac"],
Cell[1059815, 27505, 489, 11, 1464, "Input", "ExpressionUUID" -> \
"7e2ae6e0-a241-4311-be7d-358972894888"],
Cell[1060307, 27518, 253, 7, 1464, "Input", "ExpressionUUID" -> \
"e4e44838-c7cd-470d-ba58-7a56b4208716"],
Cell[1060563, 27527, 1161, 28, 3427, "Input", "ExpressionUUID" -> \
"03738c90-23be-44b0-9624-21f7c2427b5f"],
Cell[1061727, 27557, 236, 5, 1464, "Text", "ExpressionUUID" -> \
"f10a4007-4dac-4be9-a7bd-9e6a9eef2644"],
Cell[1061966, 27564, 353, 10, 1464, "Input", "ExpressionUUID" -> \
"9cb91603-6dbd-46df-8f15-016e7ef4a584",
 Evaluatable->False],
Cell[1062322, 27576, 302, 6, 1464, "Text", "ExpressionUUID" -> \
"7acb1531-f527-48b3-8fb1-7f0e3670f805"],
Cell[1062627, 27584, 246, 6, 1464, "Input", "ExpressionUUID" -> \
"dcdf1fd3-2c79-4ccd-8f34-3af86aeee0e3"],
Cell[1062876, 27592, 370, 9, 1464, "Input", "ExpressionUUID" -> \
"c63a05e5-962f-4f0b-bcab-a5574721de13"],
Cell[1063249, 27603, 299, 8, 1464, "Input", "ExpressionUUID" -> \
"16c08e41-fdcc-4f45-ba48-9dbbc1cab950"],
Cell[1063551, 27613, 303, 7, 1464, "Input", "ExpressionUUID" -> \
"9d34e733-0ed3-46b2-89ac-706250c8c06a"],
Cell[1063857, 27622, 298, 8, 1464, "Input", "ExpressionUUID" -> \
"bf1d932b-f866-4070-bcdd-c4a8fb5bef8b"],
Cell[1064158, 27632, 1014, 26, 4365, "Input", "ExpressionUUID" -> \
"60eda08f-094e-4afa-ac6a-fdf565db830a"],
Cell[1065175, 27660, 211, 5, 1464, "Input", "ExpressionUUID" -> \
"df5d395b-51b0-4f6b-8c58-707dc15c24ef"],
Cell[1065389, 27667, 277, 4, 1464, "Text", "ExpressionUUID" -> \
"e94c7ad4-ceac-4b12-9ccb-5003f750e2df"],
Cell[1065669, 27673, 704, 19, 1464, "Input", "ExpressionUUID" -> \
"74abf3c3-ba1e-4b4a-9ff0-be850bbfc8cf",
 Evaluatable->False],
Cell[1066376, 27694, 732, 13, 4432, "Text", "ExpressionUUID" -> \
"a3a0675e-138d-4cb5-92e0-b8ef6cf3c9aa"],
Cell[1067111, 27709, 418, 8, 2431, "Text", "ExpressionUUID" -> \
"b4452612-4152-410e-851a-38cbe2a19456"],
Cell[1067532, 27719, 159, 3, 1464, "Input", "ExpressionUUID" -> \
"1034435a-b973-441d-ab93-4f8745661be7"],
Cell[1067694, 27724, 528, 12, 1464, "Input", "ExpressionUUID" -> \
"716debf5-18ad-4cf1-899d-d242ce1e2e85"],
Cell[1068225, 27738, 210, 5, 1464, "Input", "ExpressionUUID" -> \
"46d52936-462b-405c-8e7f-4aff2f173ccf"],
Cell[1068438, 27745, 312, 7, 1464, "Text", "ExpressionUUID" -> \
"c1f35e68-8467-4d22-a4cc-f4bae4ad6235"],
Cell[1068753, 27754, 768, 17, 1708, "Input", "ExpressionUUID" -> \
"adbfa6cb-0d61-4f2a-9635-6a39d6e1d91e",
 Evaluatable->False],
Cell[1069524, 27773, 434, 9, 1464, "Text", "ExpressionUUID" -> \
"c928ba60-bd95-40a6-8c5a-f9f399191a21"],
Cell[1069961, 27784, 1781, 42, 3817, "Input", "ExpressionUUID" -> \
"5dad3c1f-bbb9-4fc8-83f2-37d31b814b74",
 Evaluatable->False],
Cell[1071745, 27828, 726, 12, 1464, "Text", "ExpressionUUID" -> \
"eff78d44-7400-410b-90f5-c2e93d84673e"],
Cell[1072474, 27842, 1961, 39, 1581, "Text", "ExpressionUUID" -> \
"8b53e040-c0c5-44a2-ac16-b337743d521a"],
Cell[1074438, 27883, 1658, 46, 1708, "Input", "ExpressionUUID" -> \
"8496916b-82cb-4684-9845-fccd7fee13c2",
 Evaluatable->False],
Cell[1076099, 27931, 402, 8, 2431, "Text", "ExpressionUUID" -> \
"fb4c4692-f636-4673-a455-b33c228d4284"],
Cell[1076504, 27941, 1649, 44, 1767, "Input", "ExpressionUUID" -> \
"bba67c90-c4f1-4c47-866d-9c6deb5a1a42",
 Evaluatable->False],
Cell[1078156, 27987, 197, 4, 1464, "Text", "ExpressionUUID" -> \
"5d6e120a-1921-4196-b2cf-3eb7cdefb8e4"],
Cell[1078356, 27993, 896, 26, 1708, "Input", "ExpressionUUID" -> \
"829a1ac2-e4c3-4936-a0c4-37fd07aa854e",
 Evaluatable->False],
Cell[1079255, 28021, 1917, 36, 6454, "Text", "ExpressionUUID" -> \
"c8e5a626-6e06-43ad-9b02-8d7afec227eb"],
Cell[1081175, 28059, 1129, 25, 1639, "Input", "ExpressionUUID" -> \
"ee156436-1c5e-4af9-9b99-4c5c1d56ca24",
 Evaluatable->False],
Cell[1082307, 28086, 2787, 53, 2557, "Text", "ExpressionUUID" -> \
"262baec1-e56f-448d-9a11-8ad87faffadf"],
Cell[1085097, 28141, 198, 2, 1464, "Text", "ExpressionUUID" -> \
"65fb4404-3f9a-44a1-8616-bb9f4fd3cffe"]
}, Closed]],
Cell[CellGroupData[{
Cell[1085332, 28148, 727, 12, 42, "Subsubsection", "ExpressionUUID" -> \
"ccea431e-814b-4fd9-8b8e-175ef1fecc27"],
Cell[1086062, 28162, 283, 7, 27591541, "Text", "ExpressionUUID" -> \
"ce75c213-a9ba-4a1e-9643-084fbd86b056"],
Cell[1086348, 28171, 2062, 58, 155470438, "Input", "ExpressionUUID" -> \
"3a412b48-3824-4374-ba56-0bec986544c8",
 Evaluatable->False],
Cell[1088413, 28231, 370, 7, 40061200, "Input", "ExpressionUUID" -> \
"2dcc55b5-b1ce-4db9-b9e2-a87ef0081005",
 InitializationCell->True],
Cell[1088786, 28240, 549, 11, 26707470, "Input", "ExpressionUUID" -> \
"3f78e6cc-3515-4c3b-8ffa-96856daa7ccf"],
Cell[1089338, 28253, 499, 8, 26707470, "Input", "ExpressionUUID" -> \
"8c8f0d2d-4e7b-4dd6-ad51-67829fbcf51d"],
Cell[1089840, 28263, 532, 10, 26707470, "Input", "ExpressionUUID" -> \
"ae63c8d8-6cb2-4362-bc6e-826c50255f9a"],
Cell[1090375, 28275, 1423, 38, 64171756, "Input", "ExpressionUUID" -> \
"e578d9b3-3e52-49b8-a059-598399c90b1d"],
Cell[1091801, 28315, 594, 13, 26707470, "Input", "ExpressionUUID" -> \
"1a360956-0385-419f-bba1-3bd7dbca109b"],
Cell[1092398, 28330, 176, 3, 27591541, "Text", "ExpressionUUID" -> \
"24277c02-25f5-4d9f-acc4-d2559629625d"],
Cell[1092577, 28335, 585, 14, 28657704, "Input", "ExpressionUUID" -> \
"ab8995c2-d2c9-4693-9779-9d704a6da185",
 Evaluatable->False],
Cell[1093165, 28351, 309, 8, 26707470, "Input", "ExpressionUUID" -> \
"57339c28-fa15-4c3c-8ab6-f5ed39ebe126"],
Cell[1093477, 28361, 858, 18, 26707470, "Input", "ExpressionUUID" -> \
"62fade79-c796-4c37-b6f1-ef2e70bb67ee"],
Cell[1094338, 28381, 247, 6, 45774022, "Text", "ExpressionUUID" -> \
"55f1fdea-0df5-4b95-b54c-b338b084af33"],
Cell[1094588, 28389, 825, 23, 29618788, "Input", "ExpressionUUID" -> \
"46520824-e62e-4ffc-b7c6-622bb909ee9b",
 Evaluatable->False],
Cell[1095416, 28414, 736, 21, 29618788, "Input", "ExpressionUUID" -> \
"33b01767-18c9-4406-afa4-11c9e19170a8",
 Evaluatable->False],
Cell[1096155, 28437, 270, 6, 45774022, "Text", "ExpressionUUID" -> \
"0b370000-fef4-4368-857f-b43d8e25fda3"],
Cell[1096428, 28445, 625, 18, 31172752, "Input", "ExpressionUUID" -> \
"e043ced8-7ca0-4720-95bc-f8aa2aec8448",
 Evaluatable->False],
Cell[1097056, 28465, 853, 17, 65734579, "Text", "ExpressionUUID" -> \
"779f7c7c-15eb-43e9-a21e-da75cd1c2f1c"],
Cell[1097912, 28484, 1735, 46, 50261748, "Input", "ExpressionUUID" -> \
"2043de11-dd63-43bd-aae0-76260e9b16d3",
 Evaluatable->False],
Cell[1099650, 28532, 1886, 34, 65734579, "Text", "ExpressionUUID" -> \
"926a1097-fa2e-438b-873c-638470094b9f"],
Cell[1101539, 28568, 1573, 42, 30966952, "Input", "ExpressionUUID" -> \
"93d78259-2c1d-4418-9791-fcb303d68626",
 Evaluatable->False],
Cell[1103115, 28612, 428, 8, 27591541, "Text", "ExpressionUUID" -> \
"232a30b0-6e0c-4a74-a0eb-4bea6de2e490"],
Cell[1103546, 28622, 2930, 80, 71910039, "Input", "ExpressionUUID" -> \
"6217052a-65e5-4b49-b594-3ed58464e7eb",
 Evaluatable->False],
Cell[1106479, 28704, 2210, 46, 48920545, "Text", "ExpressionUUID" -> \
"b1711eb8-44c5-4fc9-80df-e68840e7d203"],
Cell[1108692, 28752, 171, 3, 27591541, "Text", "ExpressionUUID" -> \
"8ec42df4-7d4f-4e11-aacb-dc71c90fe48e"],
Cell[1108866, 28757, 3450, 99, 166315575, "Input", "ExpressionUUID" -> \
"15326de1-eca2-4d30-8bc4-791706042f15",
 Evaluatable->False],
Cell[1112319, 28858, 984, 14, 27591541, "Text", "ExpressionUUID" -> \
"77d5d27d-d326-443d-aae5-7e9654af1f75"],
Cell[1113306, 28874, 3876, 108, 166315575, "Input", "ExpressionUUID" -> \
"42677f9d-5329-4b6d-b58e-f2cc4c69d7b3",
 Evaluatable->False],
Cell[1117185, 28984, 370, 7, 45774022, "Text", "ExpressionUUID" -> \
"35afb994-3d31-4bdd-bbd9-0b55bff41554"],
Cell[1117558, 28993, 1547, 49, 55649149, "Input", "ExpressionUUID" -> \
"8e65af67-3d81-48f8-a7c8-76c4d1c52d3f",
 Evaluatable->False],
Cell[1119108, 29044, 880, 19, 45774022, "Text", "ExpressionUUID" -> \
"5e82926a-444b-4cfc-a81d-95ace259d95a"],
Cell[1119991, 29065, 1222, 36, 30966952, "Input", "ExpressionUUID" -> \
"e9f9fdde-8161-4220-963f-91164e44f143",
 Evaluatable->False],
Cell[1121216, 29103, 164, 3, 27591541, "Text", "ExpressionUUID" -> \
"a57339d0-9789-48fc-b66f-728d2da73e0b"],
Cell[1121383, 29108, 805, 27, 30966952, "Input", "ExpressionUUID" -> \
"ee447a4d-a1c1-4665-bf11-f6a1ec8f7fa0",
 Evaluatable->False],
Cell[1122191, 29137, 245, 6, 45774022, "Text", "ExpressionUUID" -> \
"1ac13719-c01c-4afa-b38b-3e41ab5c486c"],
Cell[1122439, 29145, 837, 27, 34175448, "Input", "ExpressionUUID" -> \
"6e83f775-6e39-4d79-8633-fdb2eb79ef77",
 Evaluatable->False],
Cell[1123279, 29174, 312, 7, 45774022, "Text", "ExpressionUUID" -> \
"0cdf68d9-967a-4a9a-86a0-e8c146da0d58"],
Cell[1123594, 29183, 2535, 82, 119941230, "Input", "ExpressionUUID" -> \
"3288d436-d27f-4921-b254-71aea81db48b",
 Evaluatable->False],
Cell[1126132, 29267, 2731, 43, 171667587, "Text", "ExpressionUUID" -> \
"2c64f5da-6aad-47a6-8fd8-4062f7af49f1"],
Cell[1128866, 29312, 1921, 34, 135488847, "Text", "ExpressionUUID" -> \
"90565454-becf-4bbe-9ac1-52ad0ada41a7"]
}, Closed]],
Cell[CellGroupData[{
Cell[1130824, 29351, 838, 13, 42, "Subsubsection", "ExpressionUUID" -> \
"cee23004-312f-4c52-a619-5e59b950abe5"],
Cell[1131665, 29366, 404, 8, 30516015, "Text", "ExpressionUUID" -> \
"e258f474-de26-439b-a721-1452abb9c6dc"],
Cell[1132072, 29376, 623, 10, 17804979, "Input", "ExpressionUUID" -> \
"c6a2fcc3-1430-4723-939b-3656f5d32c27"],
Cell[1132698, 29388, 9169, 228, 311280000, "Input", "ExpressionUUID" -> \
"e4b41b89-89b4-4e6f-af54-63635837e8c6",
 Evaluatable->False,
 InitializationCell->True],
Cell[1141870, 29618, 543, 9, 17804979, "Input", "ExpressionUUID" -> \
"a07f51b1-261c-4efd-9bca-c7459a44df95"],
Cell[1142416, 29629, 3053, 79, 103646962, "Input", "ExpressionUUID" -> \
"744efc89-5a46-4f3c-92b5-5842cee5fea5",
 Evaluatable->False,
 InitializationCell->True],
Cell[1145472, 29710, 433, 10, 18394365, "Text", "ExpressionUUID" -> \
"303eebbb-5fdc-42f3-b316-b2bd6c8cf6b8"],
Cell[1145908, 29722, 654, 14, 17804979, "Input", "ExpressionUUID" -> \
"f216830f-ef0e-40b8-bb58-ccf2afacaf50"],
Cell[1146565, 29738, 236, 5, 17804979, "Input", "ExpressionUUID" -> \
"eb1223b1-f406-43b2-a374-acc7f59e5eb7"],
Cell[1146804, 29745, 383, 10, 17804979, "Input", "ExpressionUUID" -> \
"a8e6988f-16e4-4df1-9ae5-a5dfa632b95c"],
Cell[1147190, 29757, 269, 6, 17804979, "Input", "ExpressionUUID" -> \
"de650108-e0db-4100-b12e-9fa72f5e9873"],
Cell[1147462, 29765, 721, 15, 17804979, "Input", "ExpressionUUID" -> \
"188facb9-72a5-471e-9922-49eae190c266"],
Cell[1148186, 29782, 182, 4, 17804979, "Input", "ExpressionUUID" -> \
"b0b98d1f-9498-4755-8c37-4e760f0dbd69"],
Cell[1148371, 29788, 310, 7, 30516015, "Text", "ExpressionUUID" -> \
"b630c1c6-400f-4e71-a7af-9be3e3382105"],
Cell[1148684, 29797, 184, 4, 17804979, "Input", "ExpressionUUID" -> \
"7465a973-98a1-45a5-8bdf-032a3bd00c97"],
Cell[1148871, 29803, 246, 7, 17804979, "Input", "ExpressionUUID" -> \
"dbcc1da9-84c7-4da2-bbaf-3dbd253c1f67"],
Cell[1149120, 29812, 313, 7, 18394365, "Text", "ExpressionUUID" -> \
"43398a01-72d1-48de-b6d8-255940202fe6"],
Cell[1149436, 29821, 444, 9, 17804979, "Input", "ExpressionUUID" -> \
"001ba2e8-d18f-464e-a09a-f1ed930cfc9b"],
Cell[1149883, 29832, 180, 3, 18394365, "Text", "ExpressionUUID" -> \
"1f0c07df-732f-4851-a3fb-9668e23d90ac"],
Cell[1150066, 29837, 393, 11, 17804979, "Input", "ExpressionUUID" -> \
"b7626b69-315b-46a5-9d17-3d50418860ab"],
Cell[1150462, 29850, 222, 5, 17804979, "Input", "ExpressionUUID" -> \
"74669ee7-44bd-4bc2-98d0-ecc82f9855b0"],
Cell[1150687, 29857, 669, 17, 17804979, "Input", "ExpressionUUID" -> \
"ccb98d5c-65c3-48a7-bb82-6fbdef235fbb"],
Cell[1151359, 29876, 486, 13, 17804979, "Input", "ExpressionUUID" -> \
"acb25c72-74f2-4f5f-a3c1-c12158f783a7"],
Cell[1151848, 29891, 159, 3, 18394365, "Text", "ExpressionUUID" -> \
"99018c2d-47c8-45c4-a79e-549a6cdb45b7"],
Cell[1152010, 29896, 505, 11, 18394365, "Text", "ExpressionUUID" -> \
"b9130476-82b0-4b2f-8a73-5c2bd31dc984"],
Cell[1152518, 29909, 569, 13, 17804979, "Input", "ExpressionUUID" -> \
"e49814f5-bfcf-43a5-aaf9-5e6bb982ed5b"],
Cell[1153090, 29924, 269, 6, 17804979, "Input", "ExpressionUUID" -> \
"497de2c2-e067-4e51-a53b-5efaa6fb1809"],
Cell[1153362, 29932, 671, 14, 17804979, "Input", "ExpressionUUID" -> \
"38970397-b044-4fd9-a95c-d623cecfe97d"],
Cell[1154036, 29948, 310, 7, 30516015, "Text", "ExpressionUUID" -> \
"507fc7d4-732c-43d7-812e-8d99ad7e2e16"],
Cell[1154349, 29957, 184, 4, 17804979, "Input", "ExpressionUUID" -> \
"7bf3d7da-8eee-432e-8545-9624269836f1"],
Cell[1154536, 29963, 335, 7, 18394365, "Text", "ExpressionUUID" -> \
"a683cd60-6d49-441a-a8b6-d0c3ef4cca00"],
Cell[1154874, 29972, 398, 9, 17804979, "Input", "ExpressionUUID" -> \
"de8ec17d-cccb-4ec1-81da-20d5bbc46fec"],
Cell[1155275, 29983, 203, 3, 18394365, "Text", "ExpressionUUID" -> \
"13b781b2-7237-4828-ae8d-37a3bb3ce1e5"],
Cell[1155481, 29988, 445, 12, 17804979, "Input", "ExpressionUUID" -> \
"529bb0da-38a7-4ccd-8070-c248f86e21cf"],
Cell[1155929, 30002, 718, 18, 17804979, "Input", "ExpressionUUID" -> \
"630407b7-22ab-4bd2-81e6-d7c70732a92a"],
Cell[1156650, 30022, 540, 14, 17804979, "Input", "ExpressionUUID" -> \
"d1b61daa-4536-4c53-ad65-7ce2a8f938ce"],
Cell[1157193, 30038, 553, 10, 54635244, "Text", "ExpressionUUID" -> \
"2ad2f582-a6c1-4644-a88a-ae6c704a2c31"],
Cell[1157749, 30050, 513, 15, 17804979, "Input", "ExpressionUUID" -> \
"c07f5be1-ed3f-47f9-9a5d-5057bd5892eb"],
Cell[1158265, 30067, 271, 6, 18394365, "Text", "ExpressionUUID" -> \
"f5964b1f-20bc-4cc1-a1b2-769e3691f0c8"],
Cell[1158539, 30075, 504, 11, 17804979, "Input", "ExpressionUUID" -> \
"0a2ce3c5-e16e-44f6-96f7-a93f562777cb"],
Cell[1159046, 30088, 231, 6, 18394365, "Text", "ExpressionUUID" -> \
"4a59e054-cba9-4f80-ba23-6cc74257d3f3"],
Cell[1159280, 30096, 730, 20, 17804979, "Input", "ExpressionUUID" -> \
"e85c8dd7-4573-4f23-95e8-3827afed1e8f"],
Cell[1160013, 30118, 244, 6, 30516015, "Text", "ExpressionUUID" -> \
"cd5a733f-ffd1-43fc-a813-6f04bd3e0326"],
Cell[1160260, 30126, 459, 13, 17804979, "Input", "ExpressionUUID" -> \
"1c0a6998-c07f-4281-8088-eeb721e17082"],
Cell[1160722, 30141, 331, 8, 42352177, "Text", "ExpressionUUID" -> \
"ee7cfc4f-8d35-4444-acf3-5a15cbc282ef"]
}, Closed]],
Cell[CellGroupData[{
Cell[1161090, 30154, 686, 10, 42, "Subsubsection", "ExpressionUUID" -> \
"ec685c16-5a1b-4342-9b56-7d036647d570"],
Cell[1161779, 30166, 483, 11, 153, "Text", "ExpressionUUID" -> \
"52eb8786-5706-4cde-86db-8f62889de769"],
Cell[1162265, 30179, 623, 17, 112, "Input", "ExpressionUUID" -> \
"82c332cc-4056-4125-967d-27feeab83bf8"],
Cell[1162891, 30198, 509, 12, 67, "Input", "ExpressionUUID" -> \
"ab2ebfc7-ec19-43bd-8977-4a108e38c768"],
Cell[1163403, 30212, 213, 5, 67, "Input", "ExpressionUUID" -> \
"3704354b-89b3-4867-8b7c-4ae4823197ab"],
Cell[1163619, 30219, 185, 4, 67, "Input", "ExpressionUUID" -> \
"a2495668-2782-4b33-84cc-cdc96c0da667"],
Cell[1163807, 30225, 435, 10, 67, "Input", "ExpressionUUID" -> \
"9b837fc4-e532-4e90-9e89-5ea0e65a5b15"],
Cell[1164245, 30237, 181, 3, 67, "Input", "ExpressionUUID" -> \
"ab3441e9-1adf-4dd1-8cfc-01944b4186a1"],
Cell[1164429, 30242, 447, 10, 67, "Input", "ExpressionUUID" -> \
"e5fce69b-4123-41d4-8181-5e11dda70367"],
Cell[1164879, 30254, 474, 11, 67, "Input", "ExpressionUUID" -> \
"6b4ed6e1-3b65-41b3-9e56-5e2fe418850a"],
Cell[1165356, 30267, 450, 10, 67, "Input", "ExpressionUUID" -> \
"00aca6a1-821c-4e3b-99de-c532ca310ea9"],
Cell[1165809, 30279, 279, 7, 67, "Input", "ExpressionUUID" -> \
"067d72f7-f0c8-414c-bc3b-9404bb113d19"],
Cell[1166091, 30288, 440, 10, 67, "Input", "ExpressionUUID" -> \
"9d54fc56-5590-4648-b152-9638ba20bdb7"],
Cell[1166534, 30300, 235, 6, 67, "Input", "ExpressionUUID" -> \
"8ababded-ea0b-4118-b4f3-8d285446fdf0"],
Cell[1166772, 30308, 309, 7, 111, "Text", "ExpressionUUID" -> \
"171c1a6e-91ba-4f47-91ec-ca98f1e5f80e"],
Cell[1167084, 30317, 996, 27, 156, "Input", "ExpressionUUID" -> \
"d18ab5c9-0ccd-405d-af48-8ed3cb0e0224"],
Cell[1168083, 30346, 312, 8, 112, "Text", "ExpressionUUID" -> \
"5673aa98-7ec7-4eff-8e4d-7e10ebecb3fd"],
Cell[1168398, 30356, 388, 9, 67, "Input", "ExpressionUUID" -> \
"c907a563-ae9c-47ea-b692-e66c31c3b637"],
Cell[1168789, 30367, 484, 13, 67, "Input", "ExpressionUUID" -> \
"17e3e144-5282-4608-a694-72fa3c4638b1"],
Cell[1169276, 30382, 355, 9, 67, "Input", "ExpressionUUID" -> \
"527d97c9-9469-4bd1-84cc-f7db43476a12"],
Cell[1169634, 30393, 597, 14, 156, "Text", "ExpressionUUID" -> \
"abeb41fd-cfc3-49af-a081-64d01315cd0b"],
Cell[1170234, 30409, 472, 12, 67, "Input", "ExpressionUUID" -> \
"f4767e8a-3289-4804-af9e-640796466189"],
Cell[1170709, 30423, 547, 14, 67, "Input", "ExpressionUUID" -> \
"c2506a6f-d46e-4472-a679-a1673a3e0664"],
Cell[1171259, 30439, 212, 5, 67, "Input", "ExpressionUUID" -> \
"3996e12b-cb1c-4f85-a783-60aaa8233f73"],
Cell[1171474, 30446, 540, 13, 67, "Input", "ExpressionUUID" -> \
"0e06c69a-e065-4b24-8ab9-e6e5ea9a1741"],
Cell[1172017, 30461, 678, 16, 67, "Input", "ExpressionUUID" -> \
"917779d6-e0b3-4af0-bac9-fef06e4b4dfb"],
Cell[1172698, 30479, 376, 10, 67, "Input", "ExpressionUUID" -> \
"07d40dc0-bc34-4157-8c67-029dab9248ab"],
Cell[1173077, 30491, 185, 3, 69, "Text", "ExpressionUUID" -> \
"bea88556-ae26-4ebb-be1f-a60f22a29246"],
Cell[1173265, 30496, 310, 10, 67, "Text", "ExpressionUUID" -> \
"082d653b-b5fd-4c15-8b27-17afbc528e4e"],
Cell[1173578, 30508, 436, 10, 67, "Input", "ExpressionUUID" -> \
"ec57f9ea-6fcc-458d-aed3-7eddd693c944"],
Cell[1174017, 30520, 948, 25, 156, "Input", "ExpressionUUID" -> \
"cfa086bf-7f66-4297-96ea-31aed62554f0"],
Cell[1174968, 30547, 493, 11, 67, "Input", "ExpressionUUID" -> \
"9627c997-b717-4a32-b532-965125f9d21d"],
Cell[1175464, 30560, 584, 15, 67, "Input", "ExpressionUUID" -> \
"d992c87c-ff15-4947-ba98-a54117713ef1"],
Cell[1176051, 30577, 546, 15, 67, "Input", "ExpressionUUID" -> \
"b67575bc-d2f1-44c5-974f-ce8d9f1ede7e"],
Cell[1176600, 30594, 396, 9, 67, "Input", "ExpressionUUID" -> \
"4faea4e2-2c0b-460f-b463-1a49864e26a1"],
Cell[1176999, 30605, 476, 10, 69, "Text", "ExpressionUUID" -> \
"0ed10460-2993-4fa1-9859-ca1e58db856d"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1177524, 30621, 271, 4, 67, "Subsection", "ExpressionUUID" -> \
"3e6ccc2e-8ca9-49f0-bfe2-d2cf2ff22bc3"],
Cell[CellGroupData[{
Cell[1177820, 30629, 540, 8, 53, "Subsubsection", "ExpressionUUID" -> \
"74e42549-b4c9-4225-b16d-5bc5a8e88294"],
Cell[1178363, 30639, 1059, 22, 42352177, "Text", "ExpressionUUID" -> \
"37cfc1e2-cb3d-49bc-a625-7ee7b2c0d4cf"],
Cell[1179425, 30663, 415, 7, 17804979, "Input", "ExpressionUUID" -> \
"74da2d45-dd1a-4aa7-b7f1-0c773fbd130f"],
Cell[1179843, 30672, 4790, 122, 164825302, "Input", "ExpressionUUID" -> \
"802dc61a-e959-408f-9c29-dd66c33c90a7",
 Evaluatable->False,
 InitializationCell->True],
Cell[1184636, 30796, 248, 4, 18394365, "Text", "ExpressionUUID" -> \
"38efc0a2-63fa-449f-a574-5124b5f4b155"],
Cell[1184887, 30802, 449, 8, 17804979, "Input", "ExpressionUUID" -> \
"e4653645-4c52-46e9-9ff2-e0b75f93c947"],
Cell[1185339, 30812, 3189, 80, 140460829, "Input", "ExpressionUUID" -> \
"81125c3a-e193-46e7-8864-225853cca2aa",
 Evaluatable->False,
 InitializationCell->True],
Cell[1188531, 30894, 224, 7, 18394365, "Text", "ExpressionUUID" -> \
"f1c3f4bc-d5e0-4e0e-9180-157a66b42523"],
Cell[1188758, 30903, 525, 15, 17804979, "Input", "ExpressionUUID" -> \
"17dd5ccf-5916-4b37-b025-6d90c2e7d791"],
Cell[1189286, 30920, 306, 7, 17804979, "Input", "ExpressionUUID" -> \
"91f974cf-767c-4835-a962-962c902ce789"],
Cell[1189595, 30929, 177, 4, 17804979, "Input", "ExpressionUUID" -> \
"8dcdb7d2-02db-45d0-8ff3-397c8a50eae3"],
Cell[1189775, 30935, 378, 9, 17804979, "Input", "ExpressionUUID" -> \
"d16158a2-5558-472a-9ecc-1745760303c3"],
Cell[1190156, 30946, 329, 7, 18394365, "Text", "ExpressionUUID" -> \
"aa8dbc96-69cf-47ce-9d41-47c0ae70faa0"],
Cell[1190488, 30955, 414, 9, 17804979, "Input", "ExpressionUUID" -> \
"6f5e5b40-3272-4a0c-83a2-5749b5d7abc0"],
Cell[1190905, 30966, 357, 10, 17804979, "Input", "ExpressionUUID" -> \
"42ff2084-7d7c-47d7-8601-c444a6584331"],
Cell[1191265, 30978, 175, 4, 17804979, "Input", "ExpressionUUID" -> \
"0ad306cd-cec1-4111-8686-b08d279c6723"],
Cell[1191443, 30984, 252, 8, 30516015, "Text", "ExpressionUUID" -> \
"9fecbaa0-bc40-4ec3-9de0-57da8658e3bb"],
Cell[1191698, 30994, 170, 4, 17804979, "Input", "ExpressionUUID" -> \
"8fb9a62d-fbcf-47ce-9fb2-82ba028161e1"],
Cell[1191871, 31000, 661, 15, 17804979, "Input", "ExpressionUUID" -> \
"6cc0960d-bafa-4f8d-9fd0-adef6700d8f7"],
Cell[1192535, 31017, 310, 7, 17804979, "Input", "ExpressionUUID" -> \
"14e678a8-19ab-40e0-b4f7-a4759658ff33"],
Cell[1192848, 31026, 253, 7, 17804979, "Input", "ExpressionUUID" -> \
"925920df-52e3-46d7-9665-656398634354"],
Cell[1193104, 31035, 322, 5, 18394365, "Text", "ExpressionUUID" -> \
"2e907707-a259-4b32-9ee6-ccd5aa88ea56"],
Cell[1193429, 31042, 363, 9, 17804979, "Input", "ExpressionUUID" -> \
"3500fb91-ce24-40d1-8d2e-e5566e7efbb2"],
Cell[1193795, 31053, 347, 9, 17804979, "Input", "ExpressionUUID" -> \
"f4623480-dedf-4b74-b9b6-c81aaef3d183"],
Cell[1194145, 31064, 449, 8, 30516015, "Text", "ExpressionUUID" -> \
"9c8782e1-ca34-4a80-a1b4-dc2b20bc6159"],
Cell[1194597, 31074, 406, 10, 30516015, "Text", "ExpressionUUID" -> \
"74fcfa85-5d9c-4429-ab4d-d0249ade6e26"],
Cell[1195006, 31086, 711, 16, 17804979, "Input", "ExpressionUUID" -> \
"79916365-90ca-4df3-948f-387df29e7681"],
Cell[1195720, 31104, 312, 7, 17804979, "Input", "ExpressionUUID" -> \
"03a5f5ed-f4d0-430b-af34-6f17acbb0d88"],
Cell[1196035, 31113, 324, 9, 17804979, "Input", "ExpressionUUID" -> \
"aff62a19-0a34-4ad6-8d93-03fd69aa5429"],
Cell[1196362, 31124, 359, 8, 18394365, "Text", "ExpressionUUID" -> \
"e51e4a9c-5f92-4033-862e-f3776fa3ddc3"],
Cell[1196724, 31134, 413, 9, 18394365, "Text", "ExpressionUUID" -> \
"f104ee0a-a905-4abd-8acd-21a00131ee98"],
Cell[1197140, 31145, 566, 15, 17804979, "Input", "ExpressionUUID" -> \
"5674bd26-8562-4123-b38a-4a0b78040a44"],
Cell[1197709, 31162, 174, 4, 17804979, "Input", "ExpressionUUID" -> \
"d7f1f285-a6d8-442e-8860-247cb0cf4bb7"],
Cell[1197886, 31168, 506, 13, 17804979, "Input", "ExpressionUUID" -> \
"02056ba7-370d-44ec-9b36-e6043a3ad13b"],
Cell[1198395, 31183, 341, 8, 17804979, "Input", "ExpressionUUID" -> \
"90916214-caf5-4e4c-8dec-d4bcd78942e7"],
Cell[1198739, 31193, 316, 9, 17804979, "Input", "ExpressionUUID" -> \
"3262e4e2-f222-4c19-adbc-a88c4dcd66a0"],
Cell[1199058, 31204, 465, 10, 18394365, "Text", "ExpressionUUID" -> \
"65d2997c-8b12-4d73-853d-b008b6d28361"],
Cell[1199526, 31216, 459, 13, 17804979, "Input", "ExpressionUUID" -> \
"cc525ae5-15b8-4483-89d8-caee58cad504"],
Cell[1199988, 31231, 681, 16, 42781171, "Input", "ExpressionUUID" -> \
"8098f8c6-bf39-4b69-9d45-d0dcc410cfca"],
Cell[1200672, 31249, 504, 13, 17804979, "Input", "ExpressionUUID" -> \
"ceb480f8-e79d-4f76-ad5c-b55ecfb4d0af"],
Cell[1201179, 31264, 458, 12, 17804979, "Input", "ExpressionUUID" -> \
"f4757d7d-779c-4170-87c4-82f746a3e0d4"],
Cell[1201640, 31278, 310, 7, 30516015, "Text", "ExpressionUUID" -> \
"df8673ac-2093-4c73-ac25-31e956e3a97d"],
Cell[1201953, 31287, 359, 9, 30516015, "Input", "ExpressionUUID" -> \
"30f70875-3d97-4a88-a0d9-92e9ea32b6ac"],
Cell[1202315, 31298, 171, 3, 18394365, "Text", "ExpressionUUID" -> \
"3a7f44d8-dd3e-42d3-8fdd-4b3036cc52d0"],
Cell[1202489, 31303, 225, 5, 17804979, "Input", "ExpressionUUID" -> \
"52202446-2f0f-402f-ab76-d968c623d1a5"],
Cell[1202717, 31310, 164, 3, 18394365, "Text", "ExpressionUUID" -> \
"86db9880-cffe-47ba-a760-28089e3e0fc4"],
Cell[1202884, 31315, 250, 6, 17804979, "Input", "ExpressionUUID" -> \
"912faccd-29c0-4da6-8cc2-f7a3ede84d02"],
Cell[1203137, 31323, 168, 3, 18394365, "Text", "ExpressionUUID" -> \
"7c9328b6-e5de-4e69-bcf9-93759b0bbfa2"],
Cell[1203308, 31328, 228, 5, 17804979, "Input", "ExpressionUUID" -> \
"bdace28f-8771-4fb3-b87b-a66c7413724a"],
Cell[1203539, 31335, 423, 11, 18394365, "Text", "ExpressionUUID" -> \
"bab8e2ad-7b9a-45ab-92e2-be2371bed55c"],
Cell[1203965, 31348, 574, 14, 17804979, "Input", "ExpressionUUID" -> \
"113f3611-9b10-4319-9ebe-4ccd74044ddf"],
Cell[1204542, 31364, 159, 3, 18394365, "Text", "ExpressionUUID" -> \
"541c8bce-142d-4fb4-8f83-9e1599a1aaa6"],
Cell[1204704, 31369, 222, 5, 17804979, "Input", "ExpressionUUID" -> \
"3ec82f34-20d7-41ec-b52b-4adabe23d29d"]
}, Closed]],
Cell[CellGroupData[{
Cell[1204963, 31379, 546, 8, 42, "Subsubsection", "ExpressionUUID" -> \
"d32f9290-2228-44fc-a786-d7f7798ad5a5"],
Cell[1205512, 31389, 862, 16, 153, "Text", "ExpressionUUID" -> \
"e2f624b9-9064-4701-9ce1-b96d480ff710"],
Cell[1206377, 31407, 453, 12, 67, "Input", "ExpressionUUID" -> \
"7ef882da-6136-4f14-8145-4d00f3f7f732",
 Evaluatable->False],
Cell[1206833, 31421, 546, 10, 201, "Text", "ExpressionUUID" -> \
"d4cb35c1-f1a7-4aee-b722-6caba800b29d"],
Cell[1207382, 31433, 714, 21, 67, "Input", "ExpressionUUID" -> \
"4e9d961f-50b8-44f1-a506-bb654ecc2739",
 Evaluatable->False],
Cell[1208099, 31456, 664, 12, 243, "Text", "ExpressionUUID" -> \
"92dca7db-d277-46e3-9f18-4a9eecb6d076"],
Cell[1208766, 31470, 286, 7, 79, "Input", "ExpressionUUID" -> \
"2a717274-1fa6-4542-bc8a-72e1ed3d3d65",
 Evaluatable->False],
Cell[1209055, 31479, 239, 4, 67, "Text", "ExpressionUUID" -> \
"6c37a75e-b752-498d-ae96-eb06704b2952"],
Cell[1209297, 31485, 469, 14, 67, "Input", "ExpressionUUID" -> \
"fe176c96-af79-4e31-90df-633a14a47553",
 Evaluatable->False],
Cell[1209769, 31501, 421, 8, 111, "Text", "ExpressionUUID" -> \
"05db0c7c-9ff1-4e9d-a3b7-d3a9c334bdb4"],
Cell[1210193, 31511, 344, 7, 111, "Text", "ExpressionUUID" -> \
"7c9c85b6-47cf-4c2c-abf3-00a2f7c1cc26"],
Cell[1210540, 31520, 560, 15, 78, "Input", "ExpressionUUID" -> \
"6bd33721-2105-4082-a5e1-4b65efc9e71e",
 Evaluatable->False],
Cell[1211103, 31537, 358, 8, 114, "Text", "ExpressionUUID" -> \
"9533c0f1-ac64-4ba7-bae2-7f0707201299"],
Cell[1211464, 31547, 370, 9, 111, "Text", "ExpressionUUID" -> \
"96fa8f29-28a2-460d-ab08-4a170fb265fe"],
Cell[1211837, 31558, 1006, 31, 156, "Input", "ExpressionUUID" -> \
"f65d1300-6c6b-4092-b4b4-b613b00548f2"],
Cell[1212846, 31591, 913, 22, 112, "Input", "ExpressionUUID" -> \
"31e83532-019a-4357-94ac-6fb94f7f032c"],
Cell[1213762, 31615, 840, 19, 112, "Input", "ExpressionUUID" -> \
"617f4ae1-5ef0-4d7b-aa6c-278c5ec23d47"],
Cell[1214605, 31636, 417, 8, 111, "Text", "ExpressionUUID" -> \
"92f4623b-ef17-4b43-8818-1c54ac505d5f"],
Cell[1215025, 31646, 444, 13, 67, "Input", "ExpressionUUID" -> \
"05c2f64e-54a1-43a3-9c25-4a70e0f114ae"],
Cell[1215472, 31661, 361, 8, 67, "Input", "ExpressionUUID" -> \
"337f7fcf-414e-449d-b799-e50077b014c9"],
Cell[1215836, 31671, 343, 10, 67, "Input", "ExpressionUUID" -> \
"3ac169d9-07bb-42cf-813e-44562c902579"],
Cell[1216182, 31683, 189, 2, 67, "Text", "ExpressionUUID" -> \
"7c1f5ac1-121f-4492-ae2f-0aef6c5a0b24"],
Cell[1216374, 31687, 487, 14, 67, "Input", "ExpressionUUID" -> \
"22833cf1-de06-4d09-b1e8-c3a137cd0ea0"],
Cell[1216864, 31703, 158, 3, 67, "Text", "ExpressionUUID" -> \
"7116973f-7fe7-4aa9-ab4e-61cc7c5f765a"],
Cell[1217025, 31708, 720, 17, 67, "Input", "ExpressionUUID" -> \
"112654e3-9c07-4831-ace0-4eb2b7afe123"],
Cell[1217748, 31727, 163, 3, 67, "Text", "ExpressionUUID" -> \
"6c514204-2873-4c5f-925d-9ce7dce34050"],
Cell[1217914, 31732, 279, 7, 67, "Input", "ExpressionUUID" -> \
"5c59df49-f438-49dc-97bb-ec386e31adff"],
Cell[1218196, 31741, 571, 10, 153, "Text", "ExpressionUUID" -> \
"0b938766-0152-4a6d-98ee-7bdd12231f19"],
Cell[1218770, 31753, 1886, 51, 199, "Input", "ExpressionUUID" -> \
"ddbf5bf5-bf27-49b5-8421-09ae26935722"],
Cell[1220659, 31806, 526, 14, 67, "Input", "ExpressionUUID" -> \
"a0f79c7e-ed4c-42b7-9da7-477dcf1f8215"],
Cell[1221188, 31822, 243, 5, 67, "Text", "ExpressionUUID" -> \
"4b1c83ca-d27d-4c52-9d3d-b3aa58a53f8a"],
Cell[1221434, 31829, 1962, 52, 199, "Input", "ExpressionUUID" -> \
"ec4128ed-e3d9-45d1-a610-39c8fdfaf36f"],
Cell[1223399, 31883, 526, 14, 67, "Input", "ExpressionUUID" -> \
"e7d30bd1-4add-440b-ad8d-6a79df6c36f1"],
Cell[1223928, 31899, 2037, 53, 199, "Input", "ExpressionUUID" -> \
"f97c2e43-f0ba-43f3-acc0-1a38d5c2a9ba"],
Cell[1225968, 31954, 526, 14, 67, "Input", "ExpressionUUID" -> \
"0df311ab-b3fb-47b9-9ce5-d951b28dd8f7"],
Cell[1226497, 31970, 168, 3, 67, "Text", "ExpressionUUID" -> \
"ecce5bb0-01e9-4289-bb98-b701efaf3f6c"],
Cell[1226668, 31975, 2089, 54, 199, "Input", "ExpressionUUID" -> \
"2bc1f3c9-4003-422d-86c5-445a931e9050"],
Cell[1228760, 32031, 525, 14, 67, "Input", "ExpressionUUID" -> \
"f5107067-3c48-43c5-b92f-089fede12b33"],
Cell[1229288, 32047, 254, 6, 69, "Text", "ExpressionUUID" -> \
"2716b97f-3919-4a4e-8ca4-086f92fce384"],
Cell[1229545, 32055, 2184, 56, 243, "Input", "ExpressionUUID" -> \
"853db18d-5910-431f-8c2b-672af0fa8a1c"],
Cell[1231732, 32113, 613, 16, 67, "Input", "ExpressionUUID" -> \
"cb7f4ff9-d5c5-434a-afce-3b253a0523f5"],
Cell[1232348, 32131, 226, 3, 69, "Text", "ExpressionUUID" -> \
"c1dfe928-5054-4ce3-ab88-00d6526982f0"],
Cell[1232577, 32136, 418, 10, 67, "Input", "ExpressionUUID" -> \
"5590544c-1896-49d7-bafa-1445e0e6cb3c"],
Cell[1232998, 32148, 507, 9, 112, "Text", "ExpressionUUID" -> \
"8373f3ba-708a-42b2-b857-6700522dc63f"],
Cell[1233508, 32159, 2057, 53, 199, "Input", "ExpressionUUID" -> \
"2097513a-db25-459a-ae46-36a1e9ca87ce"],
Cell[1235568, 32214, 565, 14, 67, "Input", "ExpressionUUID" -> \
"07fd15d0-5d52-4424-b77f-4be3b03f7c77"],
Cell[1236136, 32230, 469, 11, 67, "Input", "ExpressionUUID" -> \
"41bc771a-ac88-4773-9c57-999f5516b4d6"],
Cell[1236608, 32243, 367, 7, 114, "Text", "ExpressionUUID" -> \
"933df079-4a70-4642-a284-698afe15a73b"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1237024, 32256, 290, 5, 67, "Subsection", "ExpressionUUID" -> \
"f97efea7-fac4-413c-b55d-1c5eb2f12604"],
Cell[CellGroupData[{
Cell[1237339, 32265, 968, 13, 53, "Subsubsection", "ExpressionUUID" -> \
"8c27d539-2c29-4e5b-8cef-cfe163df29b4"],
Cell[1238310, 32280, 961, 21, 1678, "Text", "ExpressionUUID" -> \
"6c2848c4-a337-47b7-8241-34721c357eab"],
Cell[1239274, 32303, 576, 10, 1210, "Text", "ExpressionUUID" -> \
"f5b15f5c-e173-4a3f-beb6-a75f158319eb"],
Cell[1239853, 32315, 876, 17, 712, "Input", "ExpressionUUID" -> \
"989d756d-ea2c-438c-9f71-0acdde76efd5"],
Cell[1240732, 32334, 211, 5, 712, "Input", "ExpressionUUID" -> \
"07e05d51-d8ca-485d-8719-ae0919e3ba36"],
Cell[1240946, 32341, 927, 18, 712, "Input", "ExpressionUUID" -> \
"55813b9b-85cc-4d60-b1e6-a5b776aa39e2"],
Cell[1241876, 32361, 488, 11, 1210, "Text", "ExpressionUUID" -> \
"4468828d-9089-4f15-844d-ea646c2632d9"],
Cell[1242367, 32374, 540, 11, 712, "Input", "ExpressionUUID" -> \
"1f4ca29d-fcce-40ee-af0a-dee15afef468",
 Evaluatable->False],
Cell[1242910, 32387, 780, 16, 2167, "Text", "ExpressionUUID" -> \
"97c1e5c8-2b33-4e78-945b-652fe06a77cd"],
Cell[1243693, 32405, 197, 4, 712, "Input", "ExpressionUUID" -> \
"52e097c0-0cc3-4e2d-8be0-448372ecfe8c"],
Cell[1243893, 32411, 749, 19, 712, "Input", "ExpressionUUID" -> \
"8c6035a6-8ede-40fb-9294-3581a4a3925c"],
Cell[1244645, 32432, 180, 3, 732, "Text", "ExpressionUUID" -> \
"a62ac8bc-113c-4469-a052-f6c997fbb82a"],
Cell[1244828, 32437, 324, 9, 712, "Input", "ExpressionUUID" -> \
"0fa093e9-bbb4-45b5-87d6-b7aaf84409f4"],
Cell[1245155, 32448, 368, 7, 712, "Input", "ExpressionUUID" -> \
"ddcfb97f-651f-4705-a1f8-f4489a589378"],
Cell[1245526, 32457, 664, 17, 712, "Input", "ExpressionUUID" -> \
"32436526-9f94-425b-ab78-d2afdcf98459"],
Cell[1246193, 32476, 388, 10, 712, "Input", "ExpressionUUID" -> \
"d2e11b27-b057-426e-86e8-820ad8ed1fc7"],
Cell[1246584, 32488, 375, 10, 712, "Input", "ExpressionUUID" -> \
"24dcff7d-0808-42e5-8aba-be48cdf1f29a"],
Cell[1246962, 32500, 341, 9, 712, "Input", "ExpressionUUID" -> \
"8405d5c0-4419-4c25-bc4f-7737573b5d05"],
Cell[1247306, 32511, 575, 17, 712, "Input", "ExpressionUUID" -> \
"77d2f613-270e-4239-8397-f320e4e8d3dd"],
Cell[1247884, 32530, 340, 7, 1210, "Text", "ExpressionUUID" -> \
"2a58fdb9-1090-4a67-aea9-cc4819d36b5b"],
Cell[1248227, 32539, 271, 7, 712, "Input", "ExpressionUUID" -> \
"99c239b2-70b8-4933-a95f-e14fb984ef7e"],
Cell[1248501, 32548, 357, 10, 712, "Input", "ExpressionUUID" -> \
"182e8740-aa3c-4fc8-8fd4-6c895f67cf53"],
Cell[1248861, 32560, 627, 12, 2167, "Text", "ExpressionUUID" -> \
"a3206238-8b95-4a4b-bc75-7e9239bc5e63"],
Cell[1249491, 32574, 437, 9, 1678, "Text", "ExpressionUUID" -> \
"79527412-39e2-4427-9112-a04eb67fb7f0"],
Cell[1249931, 32585, 1177, 26, 1210, "Input", "ExpressionUUID" -> \
"cd80d172-3f5a-4dff-b7e6-9ed5d39bb07a"],
Cell[1251111, 32613, 832, 18, 1678, "Text", "ExpressionUUID" -> \
"aae788e6-72a7-4122-9a74-accaae21d55a"],
Cell[1251946, 32633, 1446, 33, 1239, "Input", "ExpressionUUID" -> \
"af6680c9-940b-44ac-ab48-e654e344d65e",
 Evaluatable->False],
Cell[1253395, 32668, 847, 18, 1210, "Text", "ExpressionUUID" -> \
"6268b5cc-f6b0-4f7f-8a67-e876be63bd2d"],
Cell[1254245, 32688, 1240, 28, 907, "Input", "ExpressionUUID" -> \
"0591743e-82b9-445f-bd80-b46a1a0ae16b",
 Evaluatable->False],
Cell[1255488, 32718, 1615, 37, 1591, "Input", "ExpressionUUID" -> \
"c3f7b561-bdd9-459c-8c22-d798c6b051cc",
 Evaluatable->False],
Cell[1257106, 32757, 493, 9, 732, "Text", "ExpressionUUID" -> \
"c8fc2628-4f7f-4206-8523-9ce131be871e"],
Cell[1257602, 32768, 1004, 24, 907, "Input", "ExpressionUUID" -> \
"dab90942-56e8-44b3-a851-fedd90a9f1fc",
 Evaluatable->False],
Cell[1258609, 32794, 240, 4, 732, "Text", "ExpressionUUID" -> \
"0dfb44af-73e8-4417-87de-560dfbe65cf1"],
Cell[1258852, 32800, 1775, 40, 1239, "Input", "ExpressionUUID" -> \
"065bbdba-75c5-42c0-b0af-d652daa78f18",
 Evaluatable->False],
Cell[1260630, 32842, 796, 16, 732, "Text", "ExpressionUUID" -> \
"97e274f3-2e3b-4e09-a941-aa799ee1a088"],
Cell[1261429, 32860, 1734, 41, 1269, "Input", "ExpressionUUID" -> \
"f0b6a4ea-5696-49f9-9d25-ac5cd5c1072b",
 Evaluatable->False],
Cell[1263166, 32903, 942, 19, 1210, "Text", "ExpressionUUID" -> \
"d64f0830-b565-49c6-b0cc-729f7bb219c6"],
Cell[1264111, 32924, 2636, 59, 3670, "Input", "ExpressionUUID" -> \
"f3dc24d1-0455-41c3-b8c3-3ab277cdc21c",
 Evaluatable->False],
Cell[1266750, 32985, 702, 14, 1533, "Text", "ExpressionUUID" -> \
"a89a289d-3039-4de5-bd14-d0772c1828b2",
 CellTags->"Rtens"],
Cell[1267455, 33001, 371, 7, 712, "Input", "ExpressionUUID" -> \
"1b7c1ea7-78ba-4f61-91f3-6fafb959dd14"],
Cell[1267829, 33010, 2353, 61, 2176, "Input", "ExpressionUUID" -> \
"3e6c329e-7896-4aa2-bd6f-9e0c77fe2277",
 Evaluatable->False,
 InitializationCell->True],
Cell[1270185, 33073, 836, 18, 1210, "Text", "ExpressionUUID" -> \
"992f270a-cb80-4587-9755-54e42a70ad87"],
Cell[1271024, 33093, 181, 3, 732, "Text", "ExpressionUUID" -> \
"abb95b64-1e7f-47f0-a991-044ff99005ce"],
Cell[1271208, 33098, 228, 5, 712, "Input", "ExpressionUUID" -> \
"35651188-95c6-4887-9071-dbce2fa19d99"],
Cell[1271439, 33105, 1128, 33, 1698, "Input", "ExpressionUUID" -> \
"76792c45-d722-4fef-a664-d521deaea38d",
 Evaluatable->False],
Cell[1272570, 33140, 629, 10, 732, "Text", "ExpressionUUID" -> \
"bc0091e9-d2ff-4419-9e46-ddeb23111be9"],
Cell[1273202, 33152, 1279, 28, 1825, "Text", "ExpressionUUID" -> \
"9f3791f0-99ef-4673-b105-29878ac9dc31"],
Cell[1274484, 33182, 970, 25, 1854, "Input", "ExpressionUUID" -> \
"e7b7c53a-06c5-47fc-b7d7-3b15dc8f9c43",
 Evaluatable->False,
 InitializationCell->True],
Cell[1275457, 33209, 606, 11, 2167, "Text", "ExpressionUUID" -> \
"f563c600-3a3c-4735-b5d4-6abeaadb2f49"],
Cell[1276066, 33222, 245, 7, 732, "Text", "ExpressionUUID" -> \
"a51a3582-ef88-4192-bf98-2eb214048a88"],
Cell[1276314, 33231, 184, 4, 712, "Input", "ExpressionUUID" -> \
"7fb766d3-8360-4c13-900b-98eeb017fd23"],
Cell[1276501, 33237, 286, 7, 712, "Input", "ExpressionUUID" -> \
"91581f02-b9bf-4f47-9509-47e4c2a7b54e"],
Cell[1276790, 33246, 173, 4, 712, "Input", "ExpressionUUID" -> \
"25a67263-89d0-4443-9f85-55b00b09d552"],
Cell[1276966, 33252, 575, 13, 712, "Input", "ExpressionUUID" -> \
"302ba1cb-a812-41ee-961b-f69785b56c18"],
Cell[1277544, 33267, 285, 6, 712, "Input", "ExpressionUUID" -> \
"dd8448d4-8a3c-48ac-a2b6-e497dcbb5d68"],
Cell[1277832, 33275, 281, 7, 712, "Input", "ExpressionUUID" -> \
"eabe7d56-8be0-4eec-83c9-dfb3db8f65cb"],
Cell[1278116, 33284, 331, 9, 1210, "Text", "ExpressionUUID" -> \
"b653c3a7-52b1-4388-aa58-f3cfc54c3330"],
Cell[1278450, 33295, 476, 11, 712, "Input", "ExpressionUUID" -> \
"eb79c592-8a6d-443e-9577-f15c915a4112"],
Cell[1278929, 33308, 833, 18, 712, "Input", "ExpressionUUID" -> \
"c6746995-a3aa-490f-ba6d-dfd999e36c1a"],
Cell[1279765, 33328, 458, 13, 712, "Input", "ExpressionUUID" -> \
"02cb4797-b5bf-4097-bd25-90f12fa3b3f7"],
Cell[1280226, 33343, 399, 10, 1210, "Input", "ExpressionUUID" -> \
"ccf55731-82cd-4f6f-9723-c6310b351e05"],
Cell[1280628, 33355, 204, 5, 712, "Input", "ExpressionUUID" -> \
"a6a2c12b-ef2a-4dc6-9041-632e1cdc3fde"],
Cell[1280835, 33362, 153, 2, 732, "Text", "ExpressionUUID" -> \
"34e4c7a1-77a6-4445-8ae6-40c20a53c44d"]
}, Closed]],
Cell[CellGroupData[{
Cell[1281025, 33369, 1242, 17, 42, "Subsubsection", "ExpressionUUID" -> \
"50e25c95-63c6-4fd0-b1f0-46a071506155"],
Cell[1282270, 33388, 1380, 21, 3114, "Text", "ExpressionUUID" -> \
"d1a68a8f-4b4e-49ae-bd3d-9454f4f7e9cb"],
Cell[1283653, 33411, 950, 20, 712, "Input", "ExpressionUUID" -> \
"a27e57fe-e1d4-46e4-9923-06dc6965bb0a"],
Cell[1284606, 33433, 304, 8, 712, "Input", "ExpressionUUID" -> \
"da20e81c-bc6e-4d7c-97fc-44161d6b737f"],
Cell[1284913, 33443, 329, 7, 732, "Text", "ExpressionUUID" -> \
"8454562a-5d06-4f2e-ae7c-7cc9e5e541f5"],
Cell[1285245, 33452, 574, 14, 712, "Input", "ExpressionUUID" -> \
"d7469a0c-1511-4c27-b8c3-5c58c47c25f6"],
Cell[1285822, 33468, 308, 9, 712, "Input", "ExpressionUUID" -> \
"7b061eef-ef0c-44cb-a241-9fc42d10ae16"],
Cell[1286133, 33479, 329, 7, 732, "Text", "ExpressionUUID" -> \
"6cc4409d-ba64-4bce-b7f5-2d49b74bb998"],
Cell[1286465, 33488, 229, 5, 712, "Input", "ExpressionUUID" -> \
"2dfab3c6-4b13-4f16-bca2-8eda94dc9875"],
Cell[1286697, 33495, 346, 8, 712, "Input", "ExpressionUUID" -> \
"740ab6a5-3b9e-4265-a2d0-c15977c74702"],
Cell[1287046, 33505, 202, 3, 732, "Text", "ExpressionUUID" -> \
"94c053fe-6605-496d-b2c4-79dc75466a2b"],
Cell[1287251, 33510, 311, 8, 712, "Input", "ExpressionUUID" -> \
"6192f700-f09f-47ed-ba13-1e8570fb6316"],
Cell[1287565, 33520, 455, 9, 1210, "Text", "ExpressionUUID" -> \
"88fec714-593f-471f-9e2c-2ca3e0353bff"],
Cell[1288023, 33531, 775, 19, 1210, "Input", "ExpressionUUID" -> \
"f081e364-84f3-430b-b627-3b6676736f6f"],
Cell[1288801, 33552, 386, 9, 712, "Input", "ExpressionUUID" -> \
"cb9b0395-2972-465a-a601-a8dacb97ac99"],
Cell[1289190, 33563, 662, 13, 1210, "Text", "ExpressionUUID" -> \
"8b159e08-9dab-4100-b9d8-14ca4182fd23"],
Cell[1289855, 33578, 569, 9, 712, "Input", "ExpressionUUID" -> \
"4f5be41b-f42f-4dcf-ae7b-77410dc24101"],
Cell[1290427, 33589, 174, 4, 712, "Input", "ExpressionUUID" -> \
"8decd58e-b064-4272-b567-39a146d116f6"],
Cell[1290604, 33595, 445, 6, 732, "Text", "ExpressionUUID" -> \
"dcf6dbc9-03c6-40dd-aed0-d8a2c818ee74"],
Cell[1291052, 33603, 596, 15, 712, "Input", "ExpressionUUID" -> \
"6134dbfd-407c-42f0-8168-505dc57d16b1"],
Cell[1291651, 33620, 331, 9, 712, "Input", "ExpressionUUID" -> \
"f3bb1777-ff83-4917-8bdf-6af78b3d6a3d"],
Cell[1291985, 33631, 399, 7, 732, "Text", "ExpressionUUID" -> \
"75c2d370-0f22-4bc8-9a24-6585c960e664"],
Cell[1292387, 33640, 551, 12, 712, "Input", "ExpressionUUID" -> \
"6f936f12-8d76-4659-a400-d62f685e2b5f"],
Cell[1292941, 33654, 344, 10, 712, "Input", "ExpressionUUID" -> \
"e8898910-caeb-4566-b12b-be6495edde72"],
Cell[1293288, 33666, 248, 7, 712, "Input", "ExpressionUUID" -> \
"c8048a28-7ce2-436b-90c1-6f5a488f1541"],
Cell[1293539, 33675, 334, 7, 712, "Input", "ExpressionUUID" -> \
"7dd39255-0c1f-47b0-a709-4dd984dc23f1"],
Cell[1293876, 33684, 632, 13, 3114, "Text", "ExpressionUUID" -> \
"81b481dd-089e-4f3e-9c97-5855e1957754"],
Cell[1294511, 33699, 227, 5, 712, "Input", "ExpressionUUID" -> \
"fd8c960e-4591-46f5-a8fc-14da236ec158"],
Cell[1294741, 33706, 8195, 213, 12811, "Input", "ExpressionUUID" -> \
"fa3da4c2-93f5-4b34-af68-481760184c2a",
 Evaluatable->False,
 InitializationCell->True],
Cell[1302939, 33921, 224, 4, 732, "Text", "ExpressionUUID" -> \
"14f303dc-8379-4997-836c-1f1659b3526f"],
Cell[1303166, 33927, 2998, 72, 4110, "Input", "ExpressionUUID" -> \
"20c30fa4-df77-479f-a479-5ec2b7c1abfc",
 Evaluatable->False,
 InitializationCell->True],
Cell[1306167, 34001, 385, 8, 1678, "Text", "ExpressionUUID" -> \
"ac7578cf-ae22-4071-b909-dfc9d9c6b625"],
Cell[1306555, 34011, 282, 5, 732, "Text", "ExpressionUUID" -> \
"18141161-2c79-4898-bce8-b39fb5ce5fe9"],
Cell[1306840, 34018, 1381, 31, 3144, "Input", "ExpressionUUID" -> \
"65b990a2-3e12-4a63-bbaa-9dd6dee88265"],
Cell[1308224, 34051, 1180, 22, 712, "Input", "ExpressionUUID" -> \
"74c29bfb-32a4-4948-b3f6-abb89ee84b68"],
Cell[1309407, 34075, 434, 12, 712, "Input", "ExpressionUUID" -> \
"61f64db1-0120-4722-bea6-5230b6254ed0"],
Cell[1309844, 34089, 218, 5, 712, "Input", "ExpressionUUID" -> \
"e0f064db-52c0-4561-9f6d-3ef7ecf0263b"],
Cell[1310065, 34096, 256, 4, 732, "Text", "ExpressionUUID" -> \
"5d3a56d4-876f-4084-abd5-be2ce2c311ec"],
Cell[1310324, 34102, 280, 8, 732, "Text", "ExpressionUUID" -> \
"ac82b029-c96a-490b-a6fb-af7f5f7f3f4f"],
Cell[1310607, 34112, 1390, 28, 2176, "Input", "ExpressionUUID" -> \
"97aeac8c-d45d-4f8e-9256-13f1773c7461"],
Cell[1312000, 34142, 356, 10, 712, "Input", "ExpressionUUID" -> \
"e62cf7be-3122-4a94-833f-8cea73bbe357"],
Cell[1312359, 34154, 507, 11, 712, "Input", "ExpressionUUID" -> \
"5292dcb7-ef16-42ee-99c7-6206265d6216"],
Cell[1312869, 34167, 286, 7, 1678, "Text", "ExpressionUUID" -> \
"e33ee919-812c-475a-85cc-434c1a8d2b78"],
Cell[1313158, 34176, 200, 5, 712, "Input", "ExpressionUUID" -> \
"39a8e65c-cc6d-4a47-805f-fa077092e747"],
Cell[1313361, 34183, 771, 16, 1210, "Text", "ExpressionUUID" -> \
"66f2f1b7-e18b-4ba8-b5ab-6f99cf73cdf3"],
Cell[1314135, 34201, 1121, 22, 2226, "Text", "ExpressionUUID" -> \
"b9c83474-dd16-459a-bec4-73cd7e9f92e2"],
Cell[1315259, 34225, 870, 20, 790, "Text", "ExpressionUUID" -> \
"b01c7c0b-0346-4ed6-967b-2a647467c560"],
Cell[1316132, 34247, 468, 8, 732, "Text", "ExpressionUUID" -> \
"687865e3-4af9-4503-a6f4-4f962d790242"],
Cell[1316603, 34257, 393, 9, 732, "Text", "ExpressionUUID" -> \
"80f09319-fa97-4e75-93e5-db5ec6e0d915"],
Cell[1316999, 34268, 448, 11, 712, "Input", "ExpressionUUID" -> \
"2b6116aa-11b5-4ac8-8f67-3f5098319694",
 Evaluatable->False],
Cell[1317450, 34281, 448, 8, 1678, "Text", "ExpressionUUID" -> \
"b971e5f2-d12e-48f5-9875-57f00da220b9"],
Cell[1317901, 34291, 1222, 32, 1698, "Input", "ExpressionUUID" -> \
"87c05fa4-d719-40f5-b03a-80abe28dfe91",
 Evaluatable->False],
Cell[1319126, 34325, 1703, 33, 4540, "Text", "ExpressionUUID" -> \
"516abf66-6ba8-4594-a331-ed36b1ecc95c"]
}, Closed]],
Cell[CellGroupData[{
Cell[1320866, 34363, 1222, 17, 42, "Subsubsection", "ExpressionUUID" -> \
"a86f9de3-5343-444f-b7c7-c1db1510a921"],
Cell[1322091, 34382, 1566, 34, 2635, "Text", "ExpressionUUID" -> \
"feef1412-f468-4dc9-b0fd-9f59d94ab034"],
Cell[1323660, 34418, 579, 13, 712, "Input", "ExpressionUUID" -> \
"ac6c2b74-d5ba-4aaf-b5ff-55211f5090f6"],
Cell[1324242, 34433, 197, 5, 712, "Input", "ExpressionUUID" -> \
"7dc31d24-457c-459b-91a1-9b11eb39429a"],
Cell[1324442, 34440, 422, 11, 712, "Input", "ExpressionUUID" -> \
"7e9ab71e-8c4f-446e-885c-6dd7f26d975b"],
Cell[1324867, 34453, 536, 13, 712, "Input", "ExpressionUUID" -> \
"dbbb1e64-bf33-4935-a3e4-c788aaa3d1d9"],
Cell[1325406, 34468, 380, 8, 1210, "Text", "ExpressionUUID" -> \
"6adc52fd-b65c-4a1a-b783-283db0d7c2b7"],
Cell[1325789, 34478, 650, 18, 712, "Input", "ExpressionUUID" -> \
"33cf35cd-37f5-4dd8-82f1-b268cd1a9a21"],
Cell[1326442, 34498, 246, 4, 732, "Text", "ExpressionUUID" -> \
"f5540eec-c73c-457d-84f7-9a5d90a253df"],
Cell[1326691, 34504, 734, 21, 1386, "Input", "ExpressionUUID" -> \
"267cd09e-e728-44a7-9b0c-7846a6472e2c",
 Evaluatable->False],
Cell[1327428, 34527, 435, 9, 1210, "Text", "ExpressionUUID" -> \
"7032096e-4b63-4f27-96d4-d934b73035da"],
Cell[1327866, 34538, 906, 21, 712, "Input", "ExpressionUUID" -> \
"76f4dafb-e6dd-436b-93fc-41b0c0efc347"],
Cell[1328775, 34561, 1146, 24, 2665, "Input", "ExpressionUUID" -> \
"29288992-21ef-4edd-8176-79cf0975bfab"],
Cell[1329924, 34587, 1101, 23, 712, "Input", "ExpressionUUID" -> \
"0664b9da-8479-4862-884d-407bc7e204a9"],
Cell[1331028, 34612, 921, 21, 712, "Input", "ExpressionUUID" -> \
"03ab63ff-3ed7-4b29-a46b-bec1088c8b38"],
Cell[1331952, 34635, 372, 8, 712, "Input", "ExpressionUUID" -> \
"d8687bbc-2a44-466a-8687-0e20723ac97e"],
Cell[1332327, 34645, 360, 7, 1210, "Text", "ExpressionUUID" -> \
"ca538265-0b12-40f7-92f0-34ffd6b0d3a7"],
Cell[1332690, 34654, 1282, 20, 2635, "Text", "ExpressionUUID" -> \
"c55a33bc-d1b4-4923-b843-01df38a7e474"],
Cell[1333975, 34676, 1395, 35, 1210, "Input", "ExpressionUUID" -> \
"26e83810-b4f5-49c0-82dd-ebd760b27ec5",
 Evaluatable->False],
Cell[1335373, 34713, 1347, 23, 1210, "Text", "ExpressionUUID" -> \
"4174d0a1-24c5-4538-b2b4-75ec1a6f5a04"],
Cell[1336723, 34738, 781, 24, 771, "Input", "ExpressionUUID" -> \
"11665257-7942-4f00-9b16-6eed15f11de7",
 Evaluatable->False],
Cell[1337507, 34764, 448, 9, 732, "Text", "ExpressionUUID" -> \
"0ed09564-172a-45cc-88f2-5c5fe33efe34"],
Cell[1337958, 34775, 1621, 46, 1317, "Input", "ExpressionUUID" -> \
"d406fcc2-162d-4293-bb21-3ffe1137f603",
 Evaluatable->False],
Cell[1339582, 34823, 504, 13, 732, "Text", "ExpressionUUID" -> \
"37c7bd1a-db81-4c34-95f7-516dd16b4d3b"],
Cell[1340089, 34838, 930, 27, 771, "Input", "ExpressionUUID" -> \
"75c05c75-1559-4742-b82a-66197442b144",
 Evaluatable->False],
Cell[1341022, 34867, 1206, 23, 1210, "Text", "ExpressionUUID" -> \
"8bfe4204-004f-4e3e-a501-90080b1bf396"],
Cell[1342231, 34892, 758, 19, 712, "Input", "ExpressionUUID" -> \
"597fab26-6e8f-45a8-a6be-397dc9215bcf",
 Evaluatable->False],
Cell[1342992, 34913, 2743, 58, 6444, "Text", "ExpressionUUID" -> \
"e0690bd4-abf2-438e-96a7-4fd8376885e8"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1345784, 34977, 251, 4, 67, "Subsection", "ExpressionUUID" -> \
"149f77cd-e0e3-4247-945f-d91122eb30ad"],
Cell[CellGroupData[{
Cell[1346060, 34985, 346, 5, 53, "Subsubsection", "ExpressionUUID" -> \
"57e5b13d-6b0b-49bf-bb11-d56b7286abea"],
Cell[1346409, 34992, 774, 15, 1678, "Text", "ExpressionUUID" -> \
"90d55f84-8532-4f36-8bf0-2e51543f4471"],
Cell[1347186, 35009, 252, 5, 712, "Input", "ExpressionUUID" -> \
"9b4ec66e-fde7-40f9-8a8d-d10e28808823"],
Cell[1347441, 35016, 375, 9, 712, "Input", "ExpressionUUID" -> \
"e13ad7e0-ca04-42c2-9870-1575ae47e600"],
Cell[1347819, 35027, 220, 5, 732, "Text", "ExpressionUUID" -> \
"340bb2e2-9b06-42f1-a0cc-53aa8054afd0"],
Cell[1348042, 35034, 180, 4, 712, "Input", "ExpressionUUID" -> \
"269d1c5a-0494-4a67-8e0d-454cde31f1f7"],
Cell[1348225, 35040, 361, 7, 712, "Input", "ExpressionUUID" -> \
"d32d189e-627a-48c1-8eae-276760c207b0"],
Cell[1348589, 35049, 231, 7, 732, "Text", "ExpressionUUID" -> \
"7c599f82-c96a-4024-82b3-ee2f3c7343c9"],
Cell[1348823, 35058, 349, 9, 712, "Input", "ExpressionUUID" -> \
"8a427ea5-5e8a-4a16-bb14-7360108c30fd"],
Cell[1349175, 35069, 184, 4, 712, "Input", "ExpressionUUID" -> \
"988543ed-90da-4bf3-9ad9-0502fffe1e45"],
Cell[1349362, 35075, 232, 5, 712, "Input", "ExpressionUUID" -> \
"8ac913be-ed02-421f-ac36-284773d28381"],
Cell[1349597, 35082, 188, 4, 712, "Input", "ExpressionUUID" -> \
"90a44dfd-2b31-4cf8-8f3a-38fdf390af10"],
Cell[1349788, 35088, 178, 3, 732, "Text", "ExpressionUUID" -> \
"1d1c0fe6-fd0c-4f5c-a5b4-7be030adf55d"],
Cell[1349969, 35093, 373, 9, 712, "Input", "ExpressionUUID" -> \
"4ee86e00-7c00-45f3-996f-7c3c685757c3"],
Cell[1350345, 35104, 235, 5, 712, "Input", "ExpressionUUID" -> \
"594b32f0-b68a-4b4b-a2be-a20ec2b31953"],
Cell[1350583, 35111, 288, 8, 712, "Input", "ExpressionUUID" -> \
"925b54b7-0762-4deb-a458-5de483058a5f"],
Cell[1350874, 35121, 607, 16, 1210, "Text", "ExpressionUUID" -> \
"4efda785-76ab-4901-a06e-12c037f46518"],
Cell[1351484, 35139, 398, 10, 712, "Input", "ExpressionUUID" -> \
"0e4b650b-bb51-4fbc-af4a-e748f9f02aab"],
Cell[1351885, 35151, 330, 6, 712, "Input", "ExpressionUUID" -> \
"d3e4a387-ad24-4d49-9bec-add815ba99f6"],
Cell[1352218, 35159, 242, 5, 712, "Input", "ExpressionUUID" -> \
"ab98ed13-743b-4b8f-983e-1502a373f9ef"],
Cell[1352463, 35166, 183, 4, 712, "Input", "ExpressionUUID" -> \
"efe69fab-201a-4206-9d84-6458450c46e2"],
Cell[1352649, 35172, 272, 6, 712, "Input", "ExpressionUUID" -> \
"d66f27f9-cee6-445e-ad05-c96eb3513938"],
Cell[1352924, 35180, 184, 4, 712, "Input", "ExpressionUUID" -> \
"1b770c94-2f8b-4472-8f03-61b348d2b7b3"],
Cell[1353111, 35186, 793, 22, 1210, "Input", "ExpressionUUID" -> \
"b81edeeb-db5a-418d-813c-ac947033a44b"],
Cell[1353907, 35210, 359, 8, 1210, "Text", "ExpressionUUID" -> \
"f563e640-3f31-41a4-b6ab-5bc23f941a38"],
Cell[1354269, 35220, 470, 11, 712, "Input", "ExpressionUUID" -> \
"1a666fe3-8238-4b67-864a-2ac82922548f"],
Cell[1354742, 35233, 274, 6, 712, "Input", "ExpressionUUID" -> \
"6450f189-c289-4ba9-980a-0e9ad9f67368"],
Cell[1355019, 35241, 239, 5, 712, "Input", "ExpressionUUID" -> \
"16b21c5e-db5b-4bbf-81cb-e62b09e336d9"],
Cell[1355261, 35248, 198, 4, 712, "Input", "ExpressionUUID" -> \
"bff9010b-6396-4471-ae4c-3b6f4f855621"],
Cell[1355462, 35254, 227, 5, 732, "Text", "ExpressionUUID" -> \
"c91a98ce-da09-4c0c-852b-bdd7648bbad9"],
Cell[1355692, 35261, 744, 23, 1210, "Input", "ExpressionUUID" -> \
"ffe5ed06-44ce-4bf4-99fa-92115e10babe"],
Cell[1356439, 35286, 264, 6, 712, "Input", "ExpressionUUID" -> \
"6616315e-3db3-46db-9fec-1805543c35eb"],
Cell[1356706, 35294, 539, 12, 2167, "Text", "ExpressionUUID" -> \
"ccb9543f-6487-45e7-bcc4-208d28a20758"],
Cell[1357248, 35308, 480, 12, 790, "Input", "ExpressionUUID" -> \
"e49716c0-0f8b-42de-b2a0-b0f24710fd6a",
 Evaluatable->False],
Cell[1357731, 35322, 226, 3, 732, "Text", "ExpressionUUID" -> \
"79a2f3ba-9116-40aa-9926-bb8c3060b029"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1358006, 35331, 383, 6, 67, "Subsection", "ExpressionUUID" -> \
"481fb439-6833-470c-a98c-42b0037e5e06"],
Cell[CellGroupData[{
Cell[1358414, 35341, 496, 7, 53, "Subsubsection", "ExpressionUUID" -> \
"443b7d8f-1d2d-48cb-8d7a-d8eab76a0da8"],
Cell[1358913, 35350, 376, 5, 732, "Text", "ExpressionUUID" -> \
"d22c4538-c5ae-4780-a18a-550be2ab57b4"],
Cell[1359292, 35357, 492, 13, 712, "Input", "ExpressionUUID" -> \
"eb6ce697-e40b-4362-8cb4-21f801ee4fb6"],
Cell[1359787, 35372, 186, 3, 732, "Text", "ExpressionUUID" -> \
"a89d64de-9334-4fe9-894a-db029256968e"],
Cell[1359976, 35377, 201, 4, 712, "Input", "ExpressionUUID" -> \
"e237fc3a-b5bf-492f-9666-6252736f6dad"],
Cell[1360180, 35383, 592, 13, 712, "Input", "ExpressionUUID" -> \
"e08b5a84-acdd-4697-8da1-f10fe9ebb001"],
Cell[1360775, 35398, 239, 5, 732, "Text", "ExpressionUUID" -> \
"8023998f-061a-403f-8e70-f80085833bd6"],
Cell[1361017, 35405, 194, 4, 712, "Input", "ExpressionUUID" -> \
"0239468d-c99a-4afa-86bd-e3e5dc7d08b2"],
Cell[1361214, 35411, 221, 4, 732, "Text", "ExpressionUUID" -> \
"effcdb2a-e291-4039-a1f6-0c3037882d3a"],
Cell[1361438, 35417, 447, 11, 712, "Input", "ExpressionUUID" -> \
"ab24b8bd-df8a-48c1-b402-616030787757"],
Cell[1361888, 35430, 164, 3, 732, "Text", "ExpressionUUID" -> \
"ddbf48e3-8f2d-42c1-b7da-a0c11eb68e0b"],
Cell[1362055, 35435, 241, 5, 712, "Input", "ExpressionUUID" -> \
"fc5209ee-81f9-40d8-84c5-96278f837e97"],
Cell[1362299, 35442, 227, 6, 712, "Input", "ExpressionUUID" -> \
"4655ddd6-a582-4446-b7af-749d0f8f79b4"],
Cell[1362529, 35450, 265, 4, 732, "Text", "ExpressionUUID" -> \
"865c645f-ee64-4633-a71a-6f389494e8e6"],
Cell[1362797, 35456, 761, 21, 790, "Input", "ExpressionUUID" -> \
"4a02cf69-648d-4d21-b6bf-71c7a569747f",
 Evaluatable->False],
Cell[1363561, 35479, 301, 5, 732, "Text", "ExpressionUUID" -> \
"3e701042-5fb9-4d34-aaa8-794eaf7e16fa"],
Cell[1363865, 35486, 585, 14, 712, "Input", "ExpressionUUID" -> \
"5301a5f5-e590-4ba6-bd8d-0ff1dcb8ad91"],
Cell[1364453, 35502, 363, 8, 712, "Input", "ExpressionUUID" -> \
"ae82ea5d-a0ec-4b2c-9264-6dff7950cc5c"],
Cell[1364819, 35512, 297, 9, 712, "Input", "ExpressionUUID" -> \
"8bd86b72-56a3-4d88-949e-e22d35dd794b"],
Cell[1365119, 35523, 431, 9, 1210, "Text", "ExpressionUUID" -> \
"c3bb5b1a-ebb1-4105-b5e9-ec96146ed089"],
Cell[1365553, 35534, 759, 21, 790, "Input", "ExpressionUUID" -> \
"5bdbba99-61ff-4869-b064-43ab4c85655b",
 Evaluatable->False],
Cell[1366315, 35557, 393, 7, 732, "Text", "ExpressionUUID" -> \
"9b043ff3-0dee-4aca-9f96-3f359c699762"],
Cell[1366711, 35566, 1228, 33, 1317, "Input", "ExpressionUUID" -> \
"39a077c0-5b07-4351-ab5f-260adb776883",
 Evaluatable->False],
Cell[1367942, 35601, 605, 14, 790, "Text", "ExpressionUUID" -> \
"995671b6-2ba2-4f30-bf6a-666188771dc9"],
Cell[1368550, 35617, 383, 11, 712, "Input", "ExpressionUUID" -> \
"37eb81d2-33a8-4540-b529-401a6546cec3"],
Cell[1368936, 35630, 435, 10, 712, "Input", "ExpressionUUID" -> \
"d02cb2f9-db62-4367-b595-a5a650db96d3"],
Cell[1369374, 35642, 609, 19, 712, "Input", "ExpressionUUID" -> \
"404cda0f-6adf-4d54-acbd-23f1b297bbc9"],
Cell[1369986, 35663, 388, 11, 712, "Input", "ExpressionUUID" -> \
"0429d500-3ee6-43a7-a4e7-588d9ab12ba4"],
Cell[1370377, 35676, 380, 9, 712, "Input", "ExpressionUUID" -> \
"deb205c2-ac68-4ea1-9534-9d4962aa0475"],
Cell[1370760, 35687, 607, 15, 712, "Input", "ExpressionUUID" -> \
"74b85758-367f-4bb8-93b0-e06d2cda01a1"],
Cell[1371370, 35704, 402, 10, 712, "Input", "ExpressionUUID" -> \
"36681036-24fc-402f-bee3-82effc04a94b"],
Cell[1371775, 35716, 329, 8, 712, "Input", "ExpressionUUID" -> \
"811db6df-0dcd-4ac5-8eb9-c586e5082051"],
Cell[1372107, 35726, 322, 7, 1210, "Text", "ExpressionUUID" -> \
"555d7fac-98a4-4d28-a9c4-acda3a881377"],
Cell[1372432, 35735, 651, 18, 760, "Input", "ExpressionUUID" -> \
"801c749c-c20d-4237-87ac-557ad1a8c6d7",
 Evaluatable->False]
}, Closed]],
Cell[CellGroupData[{
Cell[1373120, 35758, 437, 6, 42, "Subsubsection", "ExpressionUUID" -> \
"bc3ce4c6-cf72-4c86-807d-e2371cc82ec9"],
Cell[1373560, 35766, 946, 18, 1678, "Text", "ExpressionUUID" -> \
"0df0a3d4-7bb3-4c8e-b507-56c46f87f354"],
Cell[1374509, 35786, 826, 22, 712, "Input", "ExpressionUUID" -> \
"e8c12f95-418d-47f7-bdd4-e7ddd29e3ce7",
 Evaluatable->False],
Cell[1375338, 35810, 946, 28, 771, "Input", "ExpressionUUID" -> \
"5102a98a-3494-4d71-9f47-996173284475",
 Evaluatable->False],
Cell[1376287, 35840, 1318, 22, 3583, "Text", "ExpressionUUID" -> \
"54a12b20-a3cd-4514-8cda-29755d97e19b"],
Cell[1377608, 35864, 724, 16, 712, "Input", "ExpressionUUID" -> \
"b49b5393-e39b-487e-9916-52f7944769b1"],
Cell[1378335, 35882, 645, 14, 712, "Input", "ExpressionUUID" -> \
"3e2145ae-2c80-46e9-a332-62f80c19ac34"],
Cell[1378983, 35898, 645, 14, 712, "Input", "ExpressionUUID" -> \
"9af131e2-cb5f-4ce1-aef2-f381b396dfc2"],
Cell[1379631, 35914, 260, 5, 712, "Input", "ExpressionUUID" -> \
"1600aece-f75c-40fd-9b23-4edb352ab193"],
Cell[1379894, 35921, 453, 7, 732, "Text", "ExpressionUUID" -> \
"b42d08ae-f99f-4118-a9ca-a66db54fb5c2"]
}, Closed]],
Cell[CellGroupData[{
Cell[1380384, 35933, 263, 4, 42, "Subsubsection", "ExpressionUUID" -> \
"5c26802e-4df9-4988-9b0c-4ae4c33dd392"],
Cell[1380650, 35939, 156, 3, 732, "Text", "ExpressionUUID" -> \
"b969d8f0-c960-4b29-99a6-8101bae588f2"],
Cell[1380809, 35944, 1286, 32, 2529, "Input", "ExpressionUUID" -> \
"551f87b6-2145-445c-812e-6186c58e143a",
 InitializationCell->True],
Cell[1382098, 35978, 154, 3, 732, "Text", "ExpressionUUID" -> \
"339be7fd-7c6f-4c41-9e93-c5c8a6338765"],
Cell[1382255, 35983, 175, 4, 712, "Input", "ExpressionUUID" -> \
"982d9a7f-fbe2-40ca-9f55-45b8886eba0a"],
Cell[1382433, 35989, 200, 3, 732, "Text", "ExpressionUUID" -> \
"dac68073-ab04-4071-8f5a-99666a6bc131"],
Cell[1382636, 35994, 341, 8, 712, "Input", "ExpressionUUID" -> \
"a834b3c0-85c4-4d24-9545-5cfa0e942b11"],
Cell[1382980, 36004, 291, 7, 712, "Input", "ExpressionUUID" -> \
"65781268-106d-48a3-a117-321fabf60b43"],
Cell[1383274, 36013, 191, 3, 732, "Text", "ExpressionUUID" -> \
"4568f9dc-a415-4c89-8335-5a09be0ef713"],
Cell[1383468, 36018, 1930, 50, 2529, "Input", "ExpressionUUID" -> \
"0ea215eb-eaf7-4bc4-978c-a9ee7d42fe30",
 InitializationCell->True],
Cell[1385401, 36070, 719, 16, 1063, "Input", "ExpressionUUID" -> \
"955b0e27-7670-497a-9868-ccb23670410e",
 InitializationCell->True],
Cell[1386123, 36088, 150, 3, 732, "Text", "ExpressionUUID" -> \
"d7734be4-0aaa-48c7-8943-a3b74c088e0a"],
Cell[1386276, 36093, 422, 9, 712, "Input", "ExpressionUUID" -> \
"0120b477-1a96-45be-a2d1-39d01911893c"],
Cell[1386701, 36104, 158, 3, 732, "Text", "ExpressionUUID" -> \
"8b65ce77-e8a8-481a-b777-a81052a01c9d"],
Cell[1386862, 36109, 925, 21, 1210, "Input", "ExpressionUUID" -> \
"0051ea99-700d-49a9-8289-75358ace5e8f"],
Cell[1387790, 36132, 882, 18, 1678, "Text", "ExpressionUUID" -> \
"6430cd1f-5e33-41c9-b0d5-2d065acfce91"],
Cell[1388675, 36152, 234, 5, 712, "Input", "ExpressionUUID" -> \
"b254e680-4837-4aff-a4b4-cee785b5ea12"],
Cell[1388912, 36159, 187, 5, 712, "Input", "ExpressionUUID" -> \
"97975377-fa90-4fdf-a23d-95c387f263b3"],
Cell[1389102, 36166, 714, 15, 712, "Input", "ExpressionUUID" -> \
"76edb91f-8026-4366-9a0c-c5d98b02ce93"],
Cell[1389819, 36183, 667, 14, 712, "Input", "ExpressionUUID" -> \
"c4383f21-8211-435e-b926-b3759b0e067e"],
Cell[1390489, 36199, 596, 13, 712, "Input", "ExpressionUUID" -> \
"586d6241-8695-4804-845f-ec423a090472"],
Cell[1391088, 36214, 1337, 32, 1698, "Input", "ExpressionUUID" -> \
"63717011-6059-4cfe-934e-8867e5e9adbf"],
Cell[1392428, 36248, 239, 6, 712, "Input", "ExpressionUUID" -> \
"c7bf32b8-32a3-4e11-9ec7-5114805cd65d"],
Cell[1392670, 36256, 246, 7, 712, "Input", "ExpressionUUID" -> \
"c0f68023-78b1-4153-b3cf-8601cb6c6ad3"],
Cell[1392919, 36265, 661, 15, 712, "Input", "ExpressionUUID" -> \
"04c81128-d2d0-4a4d-a0f5-0cae3f78e079"],
Cell[1393583, 36282, 186, 3, 732, "Text", "ExpressionUUID" -> \
"f8b39721-ae5e-4aef-9e75-2b1cee6c10f6"]
}, Closed]],
Cell[CellGroupData[{
Cell[1393806, 36290, 473, 7, 42, "Subsubsection", "ExpressionUUID" -> \
"b78fe128-339a-4d0e-a47e-dbc4b375560c"],
Cell[1394282, 36299, 244, 6, 712, "Input", "ExpressionUUID" -> \
"0482099b-7c8e-4899-9f4b-5e1072ed45ad"],
Cell[1394529, 36307, 220, 4, 732, "Text", "ExpressionUUID" -> \
"108f6fff-4c74-4619-84c3-f162ee060b83"],
Cell[1394752, 36313, 992, 29, 771, "Input", "ExpressionUUID" -> \
"56dd80e0-0fbf-4d16-89de-2181d479e792",
 Evaluatable->False],
Cell[1395747, 36344, 210, 3, 732, "Text", "ExpressionUUID" -> \
"620805f8-3731-425b-bf7c-9a481dbf48a6"],
Cell[1395960, 36349, 200, 4, 712, "Input", "ExpressionUUID" -> \
"98b7a4ec-a5d6-4bcb-beb8-fbc80b8de605"],
Cell[1396163, 36355, 256, 5, 712, "Input", "ExpressionUUID" -> \
"20c9433a-cdf2-4adf-9808-85670c3a7b99"],
Cell[1396422, 36362, 327, 7, 732, "Text", "ExpressionUUID" -> \
"82fa3612-9114-4c02-a79b-b890173f8715"],
Cell[1396752, 36371, 1513, 39, 2529, "Input", "ExpressionUUID" -> \
"850ec18f-3d90-459e-867a-1697c4b1ea60",
 InitializationCell->True],
Cell[1398268, 36412, 683, 18, 1561, "Input", "ExpressionUUID" -> \
"2d442add-9304-40df-bdc1-51cdb8bbd812",
 InitializationCell->True],
Cell[1398954, 36432, 548, 13, 712, "Input", "ExpressionUUID" -> \
"4fdd20e2-54de-4af3-b096-0cedc6dc2f69"],
Cell[1399505, 36447, 253, 7, 712, "Input", "ExpressionUUID" -> \
"03fd6c37-4436-485b-b0d9-a2e7c1793ffc"],
Cell[1399761, 36456, 306, 6, 1210, "Text", "ExpressionUUID" -> \
"d02a9271-b479-4321-bcef-d0f27f2c66c1"],
Cell[1400070, 36464, 667, 16, 712, "Input", "ExpressionUUID" -> \
"f6d8003c-2f83-40dd-918f-7b181e7a7fd9"],
Cell[1400740, 36482, 219, 3, 732, "Text", "ExpressionUUID" -> \
"ce4885f2-fcfe-4419-836f-df166e2b68ea"],
Cell[1400962, 36487, 980, 23, 1210, "Input", "ExpressionUUID" -> \
"dfa7ad1e-1185-4477-b53e-4fd6060e4f11"],
Cell[1401945, 36512, 376, 9, 712, "Input", "ExpressionUUID" -> \
"85f3ce9b-4d03-460b-9a88-c1d736997f98"],
Cell[1402324, 36523, 427, 8, 1210, "Text", "ExpressionUUID" -> \
"11a33e23-2969-4e5e-9a57-fc9585f36c04"],
Cell[1402754, 36533, 550, 13, 712, "Input", "ExpressionUUID" -> \
"74713db7-0934-4fea-8f83-db74cbaa51e4"],
Cell[1403307, 36548, 246, 7, 712, "Input", "ExpressionUUID" -> \
"d6636c39-33e2-48df-aa4d-4eca9c25d274"],
Cell[1403556, 36557, 963, 22, 1698, "Input", "ExpressionUUID" -> \
"1a308e58-0a5a-43db-afa1-fb016e249d46"],
Cell[1404522, 36581, 244, 4, 732, "Text", "ExpressionUUID" -> \
"733dca98-344f-45c6-9614-1d1d1f65b965"],
Cell[1404769, 36587, 1254, 32, 1698, "Input", "ExpressionUUID" -> \
"f6570945-d38f-403a-a7dd-44fb27717048"],
Cell[1406026, 36621, 397, 10, 712, "Input", "ExpressionUUID" -> \
"52790cdb-a49c-40ff-92ac-856a77ba8a8e"],
Cell[1406426, 36633, 395, 10, 712, "Input", "ExpressionUUID" -> \
"d997fdee-9e87-4034-a833-b3aa2f24867f"],
Cell[1406824, 36645, 796, 21, 712, "Input", "ExpressionUUID" -> \
"62830b2d-72d5-40d7-964f-4b2e5f6b532e"],
Cell[1407623, 36668, 843, 22, 712, "Input", "ExpressionUUID" -> \
"5d280c0f-5581-4a97-acef-76bfb1316430"],
Cell[1408469, 36692, 674, 20, 712, "Input", "ExpressionUUID" -> \
"8968fe1e-2d40-4480-a078-a6b321512593"],
Cell[1409146, 36714, 718, 21, 712, "Input", "ExpressionUUID" -> \
"97911a3c-d978-4623-9326-43b3e980f0d6"],
Cell[1409867, 36737, 486, 12, 712, "Input", "ExpressionUUID" -> \
"138ecd18-f5a0-4adc-a39b-e927ca29dac4"],
Cell[1410356, 36751, 704, 20, 712, "Input", "ExpressionUUID" -> \
"b8a552a1-fc5c-4ae6-a337-5f4c7515c59a"],
Cell[1411063, 36773, 1294, 34, 1327, "Input", "ExpressionUUID" -> \
"60bcbd94-9ebb-41ee-b7e8-e1d8e39e2334"],
Cell[1412360, 36809, 440, 8, 1210, "Text", "ExpressionUUID" -> \
"168858d5-6e8c-4f66-b3df-b32600797a91"],
Cell[1412803, 36819, 1093, 29, 771, "Input", "ExpressionUUID" -> \
"462496a3-7142-4873-9995-92baa10cace9",
 Evaluatable->False]
}, Closed]],
Cell[CellGroupData[{
Cell[1413933, 36853, 581, 8, 42, "Subsubsection", "ExpressionUUID" -> \
"6e3f7d8a-dda7-4fa6-b40d-0a3ef65b1321"],
Cell[1414517, 36863, 289, 6, 1210, "Text", "ExpressionUUID" -> \
"9bdc7e2a-1d85-471d-87b5-87c9f8075a13"],
Cell[1414809, 36871, 264, 6, 712, "Input", "ExpressionUUID" -> \
"52a17136-fc4b-4c02-a9ee-f320db894f66"],
Cell[1415076, 36879, 463, 11, 732, "Text", "ExpressionUUID" -> \
"6c68f853-1df0-45e2-9a06-215cdf023ee4"],
Cell[1415542, 36892, 578, 16, 712, "Input", "ExpressionUUID" -> \
"065cdce0-44d3-4eb2-b2b7-f89fb96277d6"],
Cell[1416123, 36910, 351, 7, 1210, "Text", "ExpressionUUID" -> \
"1783a8eb-c1b3-46c1-a521-f9dd6e2ced0a"],
Cell[1416477, 36919, 725, 19, 654, "Input", "ExpressionUUID" -> \
"1001e136-985c-4dde-8f7a-5f6af14ef9fa"],
Cell[1417205, 36940, 185, 3, 732, "Text", "ExpressionUUID" -> \
"c3b8c4c1-f7db-46f6-b52b-d788ddeba3a7"],
Cell[1417393, 36945, 230, 6, 712, "Input", "ExpressionUUID" -> \
"1512a414-5cd0-46a6-be63-3ba598df4349"],
Cell[1417626, 36953, 900, 20, 654, "Input", "ExpressionUUID" -> \
"d9402bd3-7e71-4171-88d7-f9f0477d7f21"],
Cell[1418529, 36975, 429, 9, 712, "Input", "ExpressionUUID" -> \
"5da481ec-175f-40cd-b377-d9da69ff448e"],
Cell[1418961, 36986, 171, 4, 712, "Input", "ExpressionUUID" -> \
"45aea6b1-1e87-47b1-8cea-3f0590e447f6"],
Cell[1419135, 36992, 369, 9, 712, "Input", "ExpressionUUID" -> \
"213fb1f7-37a0-48d6-803c-9d44338c7436"],
Cell[1419507, 37003, 299, 5, 732, "Text", "ExpressionUUID" -> \
"bc310361-a5d3-4076-981d-88c10ce3c81f"],
Cell[1419809, 37010, 992, 29, 771, "Input", "ExpressionUUID" -> \
"0a19aff3-6841-4389-82b0-77c9ebf03486",
 Evaluatable->False],
Cell[1420804, 37041, 210, 3, 732, "Text", "ExpressionUUID" -> \
"b013b0e5-d6c4-41d3-bb24-01ec7f23df5a"],
Cell[1421017, 37046, 610, 11, 1210, "Text", "ExpressionUUID" -> \
"9690a441-3af3-4ac0-b42b-40fc4d127ad9"],
Cell[1421630, 37059, 670, 17, 1698, "Input", "ExpressionUUID" -> \
"cc3e0535-b461-49eb-8422-753df91cc121"],
Cell[1422303, 37078, 504, 15, 712, "Input", "ExpressionUUID" -> \
"2aaf7fdc-1bd8-440b-ba48-bd8ccc646025"],
Cell[1422810, 37095, 559, 16, 712, "Input", "ExpressionUUID" -> \
"cb6ef28b-7426-456e-a756-15a862ea69ef"],
Cell[1423372, 37113, 183, 4, 712, "Input", "ExpressionUUID" -> \
"c887f94b-1606-446f-a144-1ae1e208aa9a"],
Cell[1423558, 37119, 619, 14, 712, "Input", "ExpressionUUID" -> \
"cae193cd-b575-4f69-9c7e-1356fb722ee7"],
Cell[1424180, 37135, 547, 15, 712, "Input", "ExpressionUUID" -> \
"95427180-f8f9-4b6d-af21-bee77beca8b1"],
Cell[1424730, 37152, 306, 6, 1210, "Text", "ExpressionUUID" -> \
"251cea54-603a-47f2-b923-f3f56d80b67a"],
Cell[1425039, 37160, 751, 17, 712, "Input", "ExpressionUUID" -> \
"ce8ab5b7-5702-40da-b3bc-7fab6578fd40"],
Cell[1425793, 37179, 219, 3, 732, "Text", "ExpressionUUID" -> \
"9911a75a-1edd-460d-9b42-1a6605d21e62"],
Cell[1426015, 37184, 1019, 24, 1210, "Input", "ExpressionUUID" -> \
"02d04763-0ee4-486b-a9b0-35629f962437"],
Cell[1427037, 37210, 376, 9, 712, "Input", "ExpressionUUID" -> \
"885bfc6a-daac-48ad-b12e-cfb6204bfbcf"],
Cell[1427416, 37221, 474, 8, 1210, "Text", "ExpressionUUID" -> \
"293b5f25-d903-4024-b1b4-829832de9b0d"],
Cell[1427893, 37231, 497, 12, 712, "Input", "ExpressionUUID" -> \
"9cc829e9-3ef2-4447-b8b9-0bcaa6f2563a"],
Cell[1428393, 37245, 905, 21, 1210, "Input", "ExpressionUUID" -> \
"336d038a-e183-4154-a129-b5504c1a34c8"],
Cell[1429301, 37268, 218, 3, 732, "Text", "ExpressionUUID" -> \
"77f4a822-9ecc-4209-b967-72ec654f71f1"],
Cell[1429522, 37273, 1200, 32, 1698, "Input", "ExpressionUUID" -> \
"7786d576-d0d6-4036-8516-29ec98f37e17"],
Cell[1430725, 37307, 395, 10, 712, "Input", "ExpressionUUID" -> \
"b4b7c65b-008e-456a-84fc-0bbaabbd4e8b"],
Cell[1431123, 37319, 393, 10, 712, "Input", "ExpressionUUID" -> \
"f046d043-9c88-453e-9b1d-336ee82512d9"],
Cell[1431519, 37331, 812, 21, 712, "Input", "ExpressionUUID" -> \
"0c58baba-7874-4a01-b97d-ff93a0e96a8d"],
Cell[1432334, 37354, 868, 22, 712, "Input", "ExpressionUUID" -> \
"646afc17-aa33-43f6-8deb-2ce9cf4426e4"],
Cell[1433205, 37378, 715, 21, 712, "Input", "ExpressionUUID" -> \
"e1f526f3-e4e9-48b5-bece-39dd0cf4c913"],
Cell[1433923, 37401, 761, 21, 712, "Input", "ExpressionUUID" -> \
"963827b4-cb06-40e5-a5f8-c16864d5e0b4"],
Cell[1434687, 37424, 437, 12, 712, "Input", "ExpressionUUID" -> \
"1beec8bc-ece5-4aec-839b-cf203105e7a9"],
Cell[1435127, 37438, 741, 20, 712, "Input", "ExpressionUUID" -> \
"b2d620cf-8acb-4065-bf41-86b104e3c39f"],
Cell[1435871, 37460, 334, 7, 1210, "Text", "ExpressionUUID" -> \
"5c08c79f-0b40-466a-9f6c-ba9e34418f57"],
Cell[1436208, 37469, 1254, 33, 771, "Input", "ExpressionUUID" -> \
"1f38dcad-338d-4bdf-972a-649c2e7357f0"],
Cell[1437465, 37504, 229, 6, 712, "Input", "ExpressionUUID" -> \
"eab5a534-184c-4bce-9173-1557b7cbf099"],
Cell[1437697, 37512, 262, 5, 732, "Text", "ExpressionUUID" -> \
"6b346247-db4e-4fb4-b505-bca0035b3dba"],
Cell[1437962, 37519, 795, 23, 712, "Input", "ExpressionUUID" -> \
"2a3ddfc1-9cef-49ce-bdc0-8a875c9aeb36"],
Cell[1438760, 37544, 156, 3, 732, "Text", "ExpressionUUID" -> \
"13021cc8-e50c-432c-a282-f673bf81d5c0"]
}, Closed]],
Cell[CellGroupData[{
Cell[1438953, 37552, 322, 5, 42, "Subsubsection", "ExpressionUUID" -> \
"8e7aab53-4f1e-4074-a6e9-3e68a95408e5"],
Cell[1439278, 37559, 295, 7, 1210, "Text", "ExpressionUUID" -> \
"f46e4d2d-19da-45a0-9513-1b4224166b7f"],
Cell[1439576, 37568, 449, 12, 712, "Input", "ExpressionUUID" -> \
"2a5ac0ed-608d-4836-9b13-16b00cbc0b72"],
Cell[1440028, 37582, 185, 4, 712, "Input", "ExpressionUUID" -> \
"e68bfa13-1685-4088-8274-bb6cce22e749"],
Cell[1440216, 37588, 225, 5, 732, "Text", "ExpressionUUID" -> \
"ee019aff-cb2d-40c9-8278-3db7d9e671b2"],
Cell[1440444, 37595, 1146, 34, 741, "Input", "ExpressionUUID" -> \
"c9c61513-f476-4b0e-bc2b-f24434fb8e4a",
 Evaluatable->False],
Cell[1441593, 37631, 623, 16, 1210, "Text", "ExpressionUUID" -> \
"ee414ecf-5ad9-4fe9-b13d-6b0ad3629766"],
Cell[1442219, 37649, 945, 27, 771, "Input", "ExpressionUUID" -> \
"bc5cc6cd-d6e7-41f1-b152-cedd21cbe444",
 Evaluatable->False],
Cell[1443167, 37678, 273, 6, 732, "Text", "ExpressionUUID" -> \
"6b08b7f5-2b2c-4463-b4c1-d24d12b57206"],
Cell[1443443, 37686, 599, 15, 1210, "Input", "ExpressionUUID" -> \
"0b55490f-9970-4708-a6da-b559f741d616"],
Cell[1444045, 37703, 618, 16, 1210, "Input", "ExpressionUUID" -> \
"1f32bced-7a87-4f71-ac57-2772e86faeb1"],
Cell[1444666, 37721, 361, 10, 712, "Input", "ExpressionUUID" -> \
"67fd6a89-05e4-40e0-887a-f5075efa16ef"],
Cell[1445030, 37733, 242, 4, 732, "Text", "ExpressionUUID" -> \
"34e56bcd-3eae-46b1-953c-0267fcaf7111"],
Cell[1445275, 37739, 512, 14, 712, "Input", "ExpressionUUID" -> \
"2d4149e4-345d-4568-a36c-b8d8295b4cd2"],
Cell[1445790, 37755, 181, 3, 732, "Text", "ExpressionUUID" -> \
"8842c0ff-a903-42b1-babd-a80612aa5252"],
Cell[1445974, 37760, 866, 22, 712, "Input", "ExpressionUUID" -> \
"afb0c7e2-d396-48f7-8b2d-4f4e540d1adb"],
Cell[1446843, 37784, 920, 23, 712, "Input", "ExpressionUUID" -> \
"ee850ca5-c176-4249-a725-17435ba01f4c"],
Cell[1447766, 37809, 917, 23, 712, "Input", "ExpressionUUID" -> \
"538566a8-09e3-4989-923e-044c8f28daf9"],
Cell[1448686, 37834, 943, 23, 712, "Input", "ExpressionUUID" -> \
"c8a02f95-d00e-465b-ad74-b710cfeb595f"],
Cell[1449632, 37859, 189, 3, 732, "Text", "ExpressionUUID" -> \
"da88a6bb-385d-4594-b174-3f5217d3096a"],
Cell[1449824, 37864, 456, 11, 654, "Input", "ExpressionUUID" -> \
"20371cf7-c4a2-46a0-b6bc-00e6b6e5a731"],
Cell[1450283, 37877, 241, 6, 732, "Text", "ExpressionUUID" -> \
"ae44ded9-1d2b-4453-8adf-9b2f31862542"],
Cell[1450527, 37885, 917, 23, 1210, "Input", "ExpressionUUID" -> \
"a524c4aa-c71f-4468-a065-53877e27ca1b"],
Cell[1451447, 37910, 376, 10, 712, "Input", "ExpressionUUID" -> \
"d31de32d-3439-4995-ad1e-307b44966b1c"],
Cell[1451826, 37922, 867, 16, 712, "Input", "ExpressionUUID" -> \
"fbfa7637-bf64-4bd9-97dd-0fdd6084ce99"],
Cell[1452696, 37940, 279, 4, 732, "Text", "ExpressionUUID" -> \
"7b54953a-d852-4b6c-9d35-a389e9c157c1"],
Cell[1452978, 37946, 919, 18, 712, "Input", "ExpressionUUID" -> \
"dedb2292-1299-4465-bc1b-c9f4372ff509"],
Cell[1453900, 37966, 309, 5, 624, "Text", "ExpressionUUID" -> \
"1aef4976-53e7-4177-8d48-ca6382519dda"],
Cell[1454212, 37973, 482, 9, 1678, "Text", "ExpressionUUID" -> \
"ffa3d755-ef25-47a2-8984-ee9c2fe354c7"]
}, Closed]],
Cell[CellGroupData[{
Cell[1454731, 37987, 271, 4, 42, "Subsubsection", "ExpressionUUID" -> \
"ce5ba630-4c77-431c-81c4-160c87c0dea8"],
Cell[1455005, 37993, 553, 10, 1678, "Text", "ExpressionUUID" -> \
"ccb08f6f-a252-4aa1-96e8-49a693e12006"],
Cell[1455561, 38005, 846, 24, 712, "Input", "ExpressionUUID" -> \
"e1a8982e-48a3-4da4-84c6-fffc5ab582fd",
 Evaluatable->False],
Cell[1456410, 38031, 268, 6, 732, "Text", "ExpressionUUID" -> \
"7e6e30dc-0634-4c80-96a3-5d12ec9fdf5b"],
Cell[1456681, 38039, 431, 13, 790, "Input", "ExpressionUUID" -> \
"80fd0fe0-6a09-46da-9780-10e30a43320b",
 Evaluatable->False],
Cell[1457115, 38054, 216, 5, 732, "Text", "ExpressionUUID" -> \
"063aef7e-9e24-447a-bcb6-14641f027a9e"],
Cell[1457334, 38061, 456, 13, 790, "Input", "ExpressionUUID" -> \
"6b181e7a-5110-4888-8e3a-96ce75f0bad2",
 Evaluatable->False],
Cell[1457793, 38076, 240, 8, 732, "Text", "ExpressionUUID" -> \
"93ea2530-14c4-4b0e-a36d-53527d3a43b2"],
Cell[1458036, 38086, 594, 15, 712, "Input", "ExpressionUUID" -> \
"0d9e1a2a-ff0c-469f-af9c-f633c1d135d6",
 Evaluatable->False],
Cell[1458633, 38103, 231, 5, 712, "Input", "ExpressionUUID" -> \
"ed2e84f6-962d-4b70-a09a-dc462c8aa5de"],
Cell[1458867, 38110, 294, 5, 712, "Input", "ExpressionUUID" -> \
"a57a2bd9-e8d2-4de7-9827-6cd67b6215fe"],
Cell[1459164, 38117, 345, 8, 712, "Input", "ExpressionUUID" -> \
"91703f5e-44b5-4461-9643-1b45702084e4"],
Cell[1459512, 38127, 176, 3, 732, "Text", "ExpressionUUID" -> \
"33aff587-586d-4984-a428-7c3856f6a2c8"],
Cell[1459691, 38132, 171, 4, 712, "Input", "ExpressionUUID" -> \
"78255c54-a95a-456a-a176-f8d2d43e11a1"],
Cell[1459865, 38138, 179, 4, 712, "Input", "ExpressionUUID" -> \
"b70c4350-18b8-4db3-a9cf-6d6bab86f6d6"],
Cell[1460047, 38144, 271, 5, 712, "Input", "ExpressionUUID" -> \
"a20d8b36-5fca-4c38-be5b-b05363cb3db6"],
Cell[1460321, 38151, 806, 21, 712, "Input", "ExpressionUUID" -> \
"89168fa7-796a-445f-8b90-72dda3bb7a02",
 Evaluatable->False],
Cell[1461130, 38174, 275, 8, 712, "Input", "ExpressionUUID" -> \
"7563ce59-d725-4289-a622-50024af7bf6c"],
Cell[1461408, 38184, 439, 10, 712, "Input", "ExpressionUUID" -> \
"691e3d77-8e7e-4782-8b5a-76d5329f300b"],
Cell[1461850, 38196, 763, 20, 1210, "Input", "ExpressionUUID" -> \
"b2b9dc29-3b4c-4f67-b4cc-d6fa15b48adc"],
Cell[1462616, 38218, 493, 9, 712, "Input", "ExpressionUUID" -> \
"a900a32a-a8e7-4b1a-80bb-2f4607cfe44f"],
Cell[1463112, 38229, 597, 14, 2635, "Text", "ExpressionUUID" -> \
"39a3dfbc-5dc7-4d38-aea6-15987c85afd0"],
Cell[1463712, 38245, 187, 4, 712, "Input", "ExpressionUUID" -> \
"83b3832a-6421-4987-bc71-e996dd94dde1"],
Cell[1463902, 38251, 173, 4, 712, "Input", "ExpressionUUID" -> \
"a4c221c0-ac4b-4a32-a83a-e33d9bd10fd5"],
Cell[1464078, 38257, 218, 3, 732, "Text", "ExpressionUUID" -> \
"f895b392-2fd9-4418-9d8c-40a9c6445825"],
Cell[1464299, 38262, 465, 10, 712, "Input", "ExpressionUUID" -> \
"210f651a-f908-4725-8ebe-948b992737f9"],
Cell[1464767, 38274, 194, 3, 732, "Text", "ExpressionUUID" -> \
"c876757a-9e89-4a00-9231-b2bd707b36d9"],
Cell[1464964, 38279, 314, 7, 712, "Input", "ExpressionUUID" -> \
"b2ed8aab-8f85-47f7-b802-029ecb497868"],
Cell[1465281, 38288, 149, 3, 732, "Text", "ExpressionUUID" -> \
"c8fba943-1010-441f-b7bf-6b6f89141bee"],
Cell[1465433, 38293, 198, 4, 712, "Input", "ExpressionUUID" -> \
"72cb038e-2197-41a0-8a3f-8bf9378f421c"],
Cell[1465634, 38299, 159, 3, 732, "Text", "ExpressionUUID" -> \
"b0883ef1-fa5a-4745-99f7-f89d54bae29a"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1465854, 38309, 273, 4, 97, "Section", "ExpressionUUID" -> \
"17943592-7608-4a77-8384-de67e5b47af4"],
Cell[CellGroupData[{
Cell[1466152, 38317, 356, 5, 67, "Subsection", "ExpressionUUID" -> \
"dd38dc69-f399-4245-8867-d88e7c690bc1"],
Cell[1466511, 38324, 1787, 39, 865, "Text", "ExpressionUUID" -> \
"7691c6c1-c61d-4433-b249-e7f5e545bc16"],
Cell[1468301, 38365, 788, 17, 670, "Text", "ExpressionUUID" -> \
"2a8affa5-48c2-4b7d-8c61-086687576fef"],
Cell[1469092, 38384, 507, 13, 663, "Text", "ExpressionUUID" -> \
"726ad21e-78f1-425b-bfc5-a0e94099f3e4"],
Cell[1469602, 38399, 481, 10, 481, "Text", "ExpressionUUID" -> \
"6217ab73-3b4d-480d-b0aa-95a731dc219e"],
Cell[1470086, 38411, 173, 4, 295, "Input", "ExpressionUUID" -> \
"59125225-32db-4394-84b3-122a455364d8"],
Cell[1470262, 38417, 557, 12, 295, "Input", "ExpressionUUID" -> \
"98964d3f-3303-472b-ade4-1316dbf1836e"],
Cell[1470822, 38431, 475, 9, 1210, "Text", "ExpressionUUID" -> \
"e4fa0fd7-c00d-4608-86a0-8e3f56b0a395"],
Cell[1471300, 38442, 171, 4, 712, "Input", "ExpressionUUID" -> \
"632b447e-8d45-4dc8-92ef-771312771121"],
Cell[1471474, 38448, 179, 3, 732, "Text", "ExpressionUUID" -> \
"7777274f-7da7-4007-97e6-5e02324d4f3d"],
Cell[1471656, 38453, 900, 19, 712, "Input", "ExpressionUUID" -> \
"43172ae8-bc68-479f-876d-bf7caee8deb6"],
Cell[1472559, 38474, 234, 6, 732, "Text", "ExpressionUUID" -> \
"989dbb75-f7c0-407d-9082-7c5925ce586e"],
Cell[1472796, 38482, 723, 19, 712, "Input", "ExpressionUUID" -> \
"94b10152-a51c-4497-a32f-5edc1464724c"],
Cell[1473522, 38503, 394, 8, 1210, "Text", "ExpressionUUID" -> \
"e8f88c78-f6d9-430c-9062-cf03c70d8535"],
Cell[1473919, 38513, 172, 4, 712, "Input", "ExpressionUUID" -> \
"998b75c0-1c1b-4fa4-b4ac-09702f35b2fd"],
Cell[1474094, 38519, 339, 8, 712, "Input", "ExpressionUUID" -> \
"28dcf2ea-f3e7-4973-9764-96bcc544f32c"],
Cell[1474436, 38529, 174, 4, 712, "Input", "ExpressionUUID" -> \
"d33ed4ef-fcc7-4058-a446-d94b0a0b922b"],
Cell[1474613, 38535, 395, 10, 712, "Input", "ExpressionUUID" -> \
"8b12f922-205d-4c01-82b5-507a42050d6d"],
Cell[1475011, 38547, 220, 5, 732, "Text", "ExpressionUUID" -> \
"96778aaa-9e33-471d-8c42-9fa907243a9f"],
Cell[1475234, 38554, 907, 26, 790, "Input", "ExpressionUUID" -> \
"81bab147-70e0-4741-a02c-d892851f2341",
 Evaluatable->False],
Cell[1476144, 38582, 144, 3, 732, "Text", "ExpressionUUID" -> \
"e44dbf17-99b7-4146-9f10-cfc176b02dac"],
Cell[1476291, 38587, 713, 22, 790, "Input", "ExpressionUUID" -> \
"1f5db553-28ba-4399-bb34-fdfe16883440",
 Evaluatable->False],
Cell[1477007, 38611, 367, 7, 732, "Text", "ExpressionUUID" -> \
"aae3f91f-6170-4182-811e-7fbac954926b"],
Cell[1477377, 38620, 764, 22, 790, "Input", "ExpressionUUID" -> \
"72f1e6eb-89e9-47bc-9b31-19c5fa585456",
 Evaluatable->False],
Cell[1478144, 38644, 740, 21, 790, "Input", "ExpressionUUID" -> \
"ae2479bd-2244-44e9-a28a-21e155edbbf0",
 Evaluatable->False],
Cell[1478887, 38667, 203, 3, 732, "Text", "ExpressionUUID" -> \
"91118914-431f-49f2-ae94-17210cf472d7"],
Cell[1479093, 38672, 446, 10, 790, "Input", "ExpressionUUID" -> \
"94d07adf-c65c-48b2-80ca-cade0c309797",
 Evaluatable->False],
Cell[1479542, 38684, 366, 7, 732, "Text", "ExpressionUUID" -> \
"376b9ac6-69e9-4f16-865b-c4baecd37e02"],
Cell[1479911, 38693, 883, 25, 760, "Input", "ExpressionUUID" -> \
"e08fd4ff-1121-4ec4-addf-f4aff4151d37",
 Evaluatable->False],
Cell[1480797, 38720, 903, 21, 1317, "Text", "ExpressionUUID" -> \
"08f41583-4ec7-4e2c-8bd4-5e897c81d73e"],
Cell[1481703, 38743, 820, 23, 858, "Input", "ExpressionUUID" -> \
"848c3341-7104-4e52-ad8b-72ca563409bf",
 Evaluatable->False],
Cell[1482526, 38768, 1173, 32, 888, "Input", "ExpressionUUID" -> \
"365f55ae-525f-4f18-b69c-fc507e379474",
 Evaluatable->False],
Cell[1483702, 38802, 997, 20, 3114, "Text", "ExpressionUUID" -> \
"dae376f1-356b-42ce-a92e-19ac19c780b5"],
Cell[1484702, 38824, 2525, 53, 6034, "Text", "ExpressionUUID" -> \
"eaa03464-85e4-4570-a8df-13ead35cb50c"],
Cell[1487230, 38879, 818, 20, 858, "Input", "ExpressionUUID" -> \
"9e1c6ea8-8dda-4eae-8f90-e712fcc17358",
 Evaluatable->False],
Cell[1488051, 38901, 620, 11, 1678, "Text", "ExpressionUUID" -> \
"bb6428dc-19f9-4264-81d0-fd354edaba07"],
Cell[1488674, 38914, 1000, 23, 1347, "Input", "ExpressionUUID" -> \
"f8e01acd-6867-43fa-9d3a-5a55d2e9b961",
 Evaluatable->False],
Cell[1489677, 38939, 1019, 16, 2635, "Text", "ExpressionUUID" -> \
"c63d2f15-73c8-4366-ac48-f7972341e6d1"],
Cell[1490699, 38957, 708, 19, 760, "Input", "ExpressionUUID" -> \
"0fa7ded4-5873-4d50-83d5-54f7cc4a06b6",
 Evaluatable->False],
Cell[1491410, 38978, 923, 19, 1239, "Text", "ExpressionUUID" -> \
"b6e921d7-cf85-415a-ba8a-c074c228ceee"],
Cell[1492336, 38999, 511, 11, 1210, "Text", "ExpressionUUID" -> \
"bd449bd0-9504-4cf1-beca-92da18ba1c07"],
Cell[1492850, 39012, 1156, 30, 1171, "Input", "ExpressionUUID" -> \
"a38f31b3-3bb3-48b7-984f-0bddc952ef90",
 Evaluatable->False],
Cell[1494009, 39044, 148, 3, 732, "Text", "ExpressionUUID" -> \
"e60cef49-33ed-4fa9-a5b1-2f1286d8735b"],
Cell[1494160, 39049, 392, 10, 712, "Input", "ExpressionUUID" -> \
"c11ca2fb-36f4-46ed-9c38-9ec4af0d568f"],
Cell[1494555, 39061, 238, 4, 732, "Text", "ExpressionUUID" -> \
"bdbe32fb-f0e8-4ce3-a79d-fcc3f378b0fc"],
Cell[1494796, 39067, 136, 3, 712, "Input", "ExpressionUUID" -> \
"1667737e-62ad-42e2-867e-79aa814e8e28"],
Cell[1494935, 39072, 163, 3, 732, "Text", "ExpressionUUID" -> \
"ec17c8e6-c256-4b59-852e-fc08af8208a0"],
Cell[1495101, 39077, 302, 6, 712, "Input", "ExpressionUUID" -> \
"6803a08e-0d6b-4218-8b0d-6d4d4df066ec"],
Cell[1495406, 39085, 385, 8, 1210, "Text", "ExpressionUUID" -> \
"fee48c3d-2d43-4e01-baa0-36a2108eb1b0"],
Cell[1495794, 39095, 395, 12, 712, "Input", "ExpressionUUID" -> \
"ef3733c8-65e6-4e04-afc8-76beed6bb262",
 Evaluatable->False],
Cell[1496192, 39109, 1041, 28, 1210, "Input", "ExpressionUUID" -> \
"0f0ffcdd-6833-43e3-a460-09d215ad1082"],
Cell[1497236, 39139, 666, 12, 1678, "Text", "ExpressionUUID" -> \
"7e0394ca-9ebb-49d3-b3a0-06b73b06c012"],
Cell[1497905, 39153, 1669, 36, 3114, "Text", "ExpressionUUID" -> \
"5d6939ad-d95d-4bb1-96b2-1a3d746b954a"],
Cell[1499577, 39191, 210, 4, 712, "Input", "ExpressionUUID" -> \
"5890b25b-2bdf-4634-9cd5-a4b144a50ea1"],
Cell[1499790, 39197, 2411, 68, 4110, "Input", "ExpressionUUID" -> \
"bbb40af1-bcb2-463a-bd62-7677f721a0e2",
 Evaluatable->False],
Cell[1502204, 39267, 199, 4, 712, "Input", "ExpressionUUID" -> \
"cfba3574-a772-4f1c-96f7-fb4a20db3051"],
Cell[1502406, 39273, 150, 3, 732, "Text", "ExpressionUUID" -> \
"622607bd-8c0d-44ee-8c5f-8976e8a613a4"],
Cell[1502559, 39278, 195, 4, 712, "Input", "ExpressionUUID" -> \
"cfad8cee-28b9-4a02-8923-223f95927566"],
Cell[1502757, 39284, 1267, 39, 1737, "Input", "ExpressionUUID" -> \
"eddbbb05-2c2f-4b59-b97c-57af282eba10",
 Evaluatable->False],
Cell[1504027, 39325, 594, 8, 732, "Text", "ExpressionUUID" -> \
"180b5732-5335-47e3-8f90-3210b23b84ce"],
Cell[1504624, 39335, 298, 8, 1210, "Text", "ExpressionUUID" -> \
"a53e651d-39b5-40f7-ac07-d1f7bf1433da"],
Cell[1504925, 39345, 220, 5, 712, "Input", "ExpressionUUID" -> \
"198905c3-68e8-4469-b115-bb79b12ed59c"],
Cell[1505148, 39352, 320, 9, 712, "Input", "ExpressionUUID" -> \
"43d5a27a-99dc-4ac8-929d-f7625b9a19e3"],
Cell[1505471, 39363, 242, 6, 712, "Input", "ExpressionUUID" -> \
"313866fa-dc0b-48a8-9347-063dc10e9c03"],
Cell[1505716, 39371, 708, 18, 712, "Input", "ExpressionUUID" -> \
"8d83da17-b4cf-4fcd-adf4-a1976665e86b"],
Cell[1506427, 39391, 164, 3, 732, "Text", "ExpressionUUID" -> \
"ac4e7aed-dbdc-46b3-b66e-2af54180a2df"],
Cell[1506594, 39396, 909, 15, 2635, "Text", "ExpressionUUID" -> \
"351cc2b2-1810-43a2-89bc-ecdde9640d6c"],
Cell[1507506, 39413, 661, 17, 1210, "Text", "ExpressionUUID" -> \
"8a1c7222-d857-468a-a1de-a729d36c5d65"],
Cell[1508170, 39432, 271, 5, 712, "Input", "ExpressionUUID" -> \
"714439da-152a-4bb7-8142-646bbb6bb969"],
Cell[1508444, 39439, 172, 4, 712, "Input", "ExpressionUUID" -> \
"633c6770-3381-4970-9de9-285673555b0f"],
Cell[1508619, 39445, 188, 4, 712, "Input", "ExpressionUUID" -> \
"aa682f77-40da-4a95-a9c9-2fe252d1944d"],
Cell[1508810, 39451, 383, 6, 732, "Text", "ExpressionUUID" -> \
"0011247f-39ba-4e87-b683-9e05d60d5e00"],
Cell[1509196, 39459, 315, 8, 712, "Input", "ExpressionUUID" -> \
"754aa680-71d6-470e-ba59-f0668ad27814"],
Cell[1509514, 39469, 176, 3, 732, "Text", "ExpressionUUID" -> \
"27aef91f-98a3-4333-b1c7-4eff01b6dc5a"],
Cell[1509693, 39474, 463, 9, 712, "Input", "ExpressionUUID" -> \
"9c0c68c5-6fed-4527-8887-fe706d1c9c5d"],
Cell[1510159, 39485, 645, 15, 712, "Input", "ExpressionUUID" -> \
"92da660f-d902-4871-9be6-a029c509918d"],
Cell[1510807, 39502, 220, 5, 712, "Input", "ExpressionUUID" -> \
"929fb7e3-a76c-4407-b667-4e7296588931"],
Cell[1511030, 39509, 341, 9, 712, "Input", "ExpressionUUID" -> \
"4679c75b-f3fe-46f2-9ba6-dda532d69f8d"],
Cell[1511374, 39520, 632, 14, 712, "Input", "ExpressionUUID" -> \
"fc59c945-ddab-448e-a030-c4c837c49d71"],
Cell[1512009, 39536, 303, 7, 712, "Input", "ExpressionUUID" -> \
"7f202afd-20ce-4d37-b802-fa6e262a11bd"],
Cell[1512315, 39545, 289, 4, 732, "Text", "ExpressionUUID" -> \
"53de238a-8cd1-4912-aba5-52083cd755d9"],
Cell[1512607, 39551, 186, 4, 712, "Input", "ExpressionUUID" -> \
"c33a9761-b55d-49d8-983c-5c3cbbf05192"],
Cell[1512796, 39557, 1387, 28, 1210, "Input", "ExpressionUUID" -> \
"b1a095ec-6844-4bc9-befc-663287384c45",
 Evaluatable->False],
Cell[1514186, 39587, 436, 10, 712, "Input", "ExpressionUUID" -> \
"a85e960f-22e8-4830-9af3-d087bf73a2f3"],
Cell[1514625, 39599, 285, 6, 732, "Text", "ExpressionUUID" -> \
"07678d2a-9eb0-4268-a228-401ab8cc8bef"],
Cell[1514913, 39607, 214, 6, 712, "Input", "ExpressionUUID" -> \
"75cc2d62-2a42-4a93-9383-8fd006161528"],
Cell[1515130, 39615, 175, 3, 732, "Text", "ExpressionUUID" -> \
"3254d41f-2ca3-4b13-b74b-cef0665c6678"],
Cell[1515308, 39620, 423, 11, 712, "Input", "ExpressionUUID" -> \
"4f676b5c-52f6-40e3-b7a5-a6562525c80c"],
Cell[1515734, 39633, 205, 5, 712, "Input", "ExpressionUUID" -> \
"9abc857d-93e3-4d9d-af0e-1e5209045831"],
Cell[1515942, 39640, 224, 5, 732, "Text", "ExpressionUUID" -> \
"4b319eff-d0d6-48e7-99a0-18c619bec183"],
Cell[1516169, 39647, 249, 6, 712, "Input", "ExpressionUUID" -> \
"88260fde-3d88-431c-9cdf-463cc1d0809f"],
Cell[1516421, 39655, 197, 4, 712, "Input", "ExpressionUUID" -> \
"07a3be50-629f-4431-8fb2-c41e71d160a2"],
Cell[1516621, 39661, 890, 18, 712, "Input", "ExpressionUUID" -> \
"faa836e9-8395-4e22-af30-9d36b442bab6"],
Cell[1517514, 39681, 357, 7, 712, "Input", "ExpressionUUID" -> \
"a7c584b1-66d8-4c8d-89c3-a5602d19319c"],
Cell[1517874, 39690, 400, 11, 712, "Input", "ExpressionUUID" -> \
"0212bc59-cd4e-4492-993b-149a41482505"],
Cell[1518277, 39703, 289, 8, 732, "Text", "ExpressionUUID" -> \
"113007e9-5f6a-4580-b7ed-3c872dda95ee"],
Cell[1518569, 39713, 178, 4, 712, "Input", "ExpressionUUID" -> \
"dd862fe5-5511-464b-a3f8-e7b25eb20cd8"],
Cell[1518750, 39719, 266, 7, 712, "Input", "ExpressionUUID" -> \
"305ca37a-52d4-4f4e-abe7-f85adb225358"],
Cell[1519019, 39728, 494, 14, 712, "Input", "ExpressionUUID" -> \
"245c42f7-5086-4d84-878f-85fa657c06b3"],
Cell[1519516, 39744, 350, 10, 712, "Input", "ExpressionUUID" -> \
"190db4de-ac4d-4824-9219-fbb51f544565"],
Cell[1519869, 39756, 423, 10, 712, "Input", "ExpressionUUID" -> \
"0b2e6821-1d6c-4db0-801d-a81bddbbd010"],
Cell[1520295, 39768, 340, 8, 732, "Text", "ExpressionUUID" -> \
"48e3ce41-71a7-4e1c-a6dc-23b7aedf9211"],
Cell[1520638, 39778, 248, 6, 712, "Input", "ExpressionUUID" -> \
"b0a6fa9a-0c8f-4e43-bbfb-098c840e0e91"],
Cell[1520889, 39786, 162, 3, 732, "Text", "ExpressionUUID" -> \
"77622bf5-dafd-4e4b-83d8-9f93f053f35d"],
Cell[1521054, 39791, 414, 9, 712, "Input", "ExpressionUUID" -> \
"76c663b7-1b2f-4a06-a350-abeba30b386f"],
Cell[1521471, 39802, 172, 3, 732, "Text", "ExpressionUUID" -> \
"ea117296-1ba2-4ca9-bdb1-d49b62d4197a"],
Cell[1521646, 39807, 346, 7, 712, "Input", "ExpressionUUID" -> \
"ece8ae50-6d13-495f-83aa-8433334384cb"],
Cell[1521995, 39816, 176, 3, 732, "Text", "ExpressionUUID" -> \
"ef106d2a-534b-4945-91f6-142321951114"],
Cell[1522174, 39821, 381, 10, 712, "Input", "ExpressionUUID" -> \
"35644a50-285b-4745-b4e9-235a7d07da7a"],
Cell[1522558, 39833, 186, 4, 712, "Input", "ExpressionUUID" -> \
"11cc9d9d-6ab3-498b-b356-e57bb466f4e7"],
Cell[1522747, 39839, 166, 3, 732, "Text", "ExpressionUUID" -> \
"36ebbad5-2dd7-462a-ae28-0bce6d88f0a9"],
Cell[1522916, 39844, 616, 15, 1210, "Input", "ExpressionUUID" -> \
"6259d59d-2ab5-4f72-870e-804e5d615c10"],
Cell[1523535, 39861, 214, 4, 712, "Input", "ExpressionUUID" -> \
"f60674af-0847-4c50-ab21-13de0e19799a"],
Cell[1523752, 39867, 340, 9, 712, "Input", "ExpressionUUID" -> \
"9de3350e-cd09-4b58-9c6c-b14a967101dd"],
Cell[1524095, 39878, 194, 3, 732, "Text", "ExpressionUUID" -> \
"efd416f3-de22-4c82-93f3-f08d87c262aa"],
Cell[1524292, 39883, 450, 9, 712, "Input", "ExpressionUUID" -> \
"d36feeef-c51b-4e3f-8acf-afa04492543b"],
Cell[1524745, 39894, 164, 3, 732, "Text", "ExpressionUUID" -> \
"a85b513e-e6a2-4b31-9df1-5d483027b7b7"],
Cell[1524912, 39899, 251, 7, 732, "Text", "ExpressionUUID" -> \
"1331139a-0881-449f-9df9-5e3d68437d2d"],
Cell[1525166, 39908, 168, 3, 712, "Input", "ExpressionUUID" -> \
"659773af-5cdf-4158-8f9d-9e251d3cbc14"],
Cell[1525337, 39913, 350, 8, 712, "Input", "ExpressionUUID" -> \
"f93b9a4b-88d7-497d-b67a-a1ee2dcb2050"],
Cell[1525690, 39923, 477, 12, 712, "Input", "ExpressionUUID" -> \
"dec74bf8-3e4b-4233-bf36-d84367842070"],
Cell[1526170, 39937, 213, 5, 712, "Input", "ExpressionUUID" -> \
"418e0011-9589-4c95-93f2-9d3e165e600d"],
Cell[1526386, 39944, 225, 5, 712, "Input", "ExpressionUUID" -> \
"61a269ab-a520-40d1-b2be-d1fdbe245e65"],
Cell[1526614, 39951, 591, 12, 712, "Input", "ExpressionUUID" -> \
"970eed8a-7c1b-448a-ab67-1781bb9b6e7a"],
Cell[1527208, 39965, 233, 5, 712, "Input", "ExpressionUUID" -> \
"fad48116-b7a1-43d9-8db5-76a2092e3228"],
Cell[1527444, 39972, 507, 12, 760, "Input", "ExpressionUUID" -> \
"be9a7f10-fc5a-4da8-9588-0767a74b7b7f"],
Cell[1527954, 39986, 687, 14, 1210, "Text", "ExpressionUUID" -> \
"e012fde6-a20c-42d7-b5f4-13be03a6ce3a"],
Cell[1528644, 40002, 164, 3, 732, "Text", "ExpressionUUID" -> \
"4948fc32-08c2-4231-b732-811fcd136130"]
}, Closed]],
Cell[CellGroupData[{
Cell[1528845, 40010, 246, 4, 53, "Subsection", "ExpressionUUID" -> \
"fdc14203-1f8d-4821-8649-13ec4fe915b9"],
Cell[CellGroupData[{
Cell[1529116, 40018, 388, 6, 53, "Subsubsection", "ExpressionUUID" -> \
"19fe0e55-b5a7-4717-8115-fcbe85092cbf"],
Cell[1529507, 40026, 234, 5, 732, "Text", "ExpressionUUID" -> \
"93a1a3c3-2366-4afd-8043-a0bd02f9645a"],
Cell[1529744, 40033, 280, 8, 760, "Input", "ExpressionUUID" -> \
"498e5183-9ccc-4b06-81a7-65f96244289b",
 Evaluatable->False],
Cell[1530027, 40043, 228, 4, 732, "Text", "ExpressionUUID" -> \
"7d9fd0f9-64fc-4cd5-9926-f37cbd6a6281"],
Cell[1530258, 40049, 230, 5, 712, "Input", "ExpressionUUID" -> \
"3bb76997-e431-4181-b74b-69e2414f5775"],
Cell[1530491, 40056, 179, 4, 712, "Input", "ExpressionUUID" -> \
"64b11e51-a6b7-4044-ada2-fe542726c96d"],
Cell[1530673, 40062, 161, 4, 712, "Input", "ExpressionUUID" -> \
"d7cb96a7-f87c-478f-bd89-a191a2e374ff"],
Cell[1530837, 40068, 464, 10, 712, "Input", "ExpressionUUID" -> \
"ccb6f454-1d34-4d0a-bacc-0b2807087395"],
Cell[1531304, 40080, 423, 10, 712, "Input", "ExpressionUUID" -> \
"abfe0f8d-13de-4c04-9726-01a8c49581bb"],
Cell[1531730, 40092, 171, 4, 712, "Input", "ExpressionUUID" -> \
"9d783c07-6a93-4fd2-aae3-01d28fa2379b"],
Cell[1531904, 40098, 471, 13, 712, "Input", "ExpressionUUID" -> \
"0fe63de1-b042-4cca-93e2-7c56ffc63744"],
Cell[1532378, 40113, 362, 11, 712, "Input", "ExpressionUUID" -> \
"d697fefb-c663-49f1-aa77-e4256d174eb1"],
Cell[1532743, 40126, 359, 9, 712, "Input", "ExpressionUUID" -> \
"05a28c49-8479-444d-ab7a-9de92af1d365"],
Cell[1533105, 40137, 225, 5, 712, "Input", "ExpressionUUID" -> \
"17759222-2396-43ce-95fe-15bafcffa5a6"],
Cell[1533333, 40144, 220, 3, 732, "Text", "ExpressionUUID" -> \
"2bccf7c7-13f1-43a7-a860-6dfa1a7ba3ef"],
Cell[1533556, 40149, 278, 5, 712, "Input", "ExpressionUUID" -> \
"17ce4c46-9231-4ace-8677-7a1863313090"],
Cell[1533837, 40156, 464, 10, 712, "Input", "ExpressionUUID" -> \
"41d76c66-f965-4c04-9a67-943d26bd3806"],
Cell[1534304, 40168, 790, 18, 712, "Input", "ExpressionUUID" -> \
"4ed68f9f-8db7-4aef-8bbb-e7bbab148027"],
Cell[1535097, 40188, 368, 7, 1210, "Text", "ExpressionUUID" -> \
"91c5daf3-b10b-4bf3-b743-da56a6f7bd19"],
Cell[1535468, 40197, 471, 13, 712, "Input", "ExpressionUUID" -> \
"0c1657f6-c380-4d5b-8973-839ee7814e40"],
Cell[1535942, 40212, 438, 12, 712, "Input", "ExpressionUUID" -> \
"20877207-d484-49ad-a05c-85b58e2238e9"],
Cell[1536383, 40226, 359, 9, 712, "Input", "ExpressionUUID" -> \
"899a2174-7208-4d30-b730-ad8f3a615ef7"],
Cell[1536745, 40237, 432, 10, 712, "Input", "ExpressionUUID" -> \
"3ab49ce7-f2f2-4751-9011-388f31d2ec9c"],
Cell[1537180, 40249, 279, 6, 732, "Text", "ExpressionUUID" -> \
"c6bbbd5b-d46f-4395-bc97-aad5abd52838"],
Cell[1537462, 40257, 201, 4, 732, "Text", "ExpressionUUID" -> \
"254255ca-12fb-4280-aff8-bd8768ac1b23"],
Cell[1537666, 40263, 179, 4, 712, "Input", "ExpressionUUID" -> \
"19527d26-459a-4fa4-b4f3-adb5747aefff"],
Cell[1537848, 40269, 471, 13, 712, "Input", "ExpressionUUID" -> \
"41e338df-e08d-4b6a-a1fb-2ccb80e735ac"],
Cell[1538322, 40284, 462, 12, 712, "Input", "ExpressionUUID" -> \
"195eec04-70bf-4302-bdf3-ec26fd456f29"],
Cell[1538787, 40298, 359, 9, 712, "Input", "ExpressionUUID" -> \
"66f88158-8523-451e-9e0c-15fa43afc4ad"],
Cell[1539149, 40309, 432, 10, 712, "Input", "ExpressionUUID" -> \
"8ffd4b24-577b-4b28-85b9-cc775febebae"],
Cell[1539584, 40321, 353, 7, 732, "Text", "ExpressionUUID" -> \
"150652dd-3a21-421d-8c55-14aeb4425d68"]
}, Closed]],
Cell[CellGroupData[{
Cell[1539974, 40333, 487, 7, 42, "Subsubsection", "ExpressionUUID" -> \
"664fdd2d-e31e-49f3-a455-18e1689f0283"],
Cell[1540464, 40342, 86, 0, 712, "Input", "ExpressionUUID" -> \
"d0c28cff-542f-4b4d-8bd4-709213066919"],
Cell[1540553, 40344, 1121, 20, 1678, "Text", "ExpressionUUID" -> \
"4bb49ba7-6965-404f-b13d-6a3d4c7aa2b1"],
Cell[1541677, 40366, 223, 5, 712, "Input", "ExpressionUUID" -> \
"a55699ca-d998-4ead-906e-3678e9409597"],
Cell[1541903, 40373, 491, 15, 712, "Input", "ExpressionUUID" -> \
"7ea2ec74-a4d8-4fce-bb66-3594ad9a9f5b",
 Evaluatable->False],
Cell[1542397, 40390, 239, 5, 712, "Input", "ExpressionUUID" -> \
"80c9e52a-90e6-4c18-9ff0-83e472b6e3f6"],
Cell[1542639, 40397, 171, 4, 712, "Input", "ExpressionUUID" -> \
"712dbdfa-0066-4041-adbe-24e028cf9881"],
Cell[1542813, 40403, 250, 6, 1210, "Text", "ExpressionUUID" -> \
"2b1fd965-b4d8-4fa9-a217-642bcbed38ba"],
Cell[1543066, 40411, 173, 4, 712, "Input", "ExpressionUUID" -> \
"0291efe2-5a2a-4c19-b905-ee8255501d40"],
Cell[1543242, 40417, 177, 3, 732, "Text", "ExpressionUUID" -> \
"a06fa3d4-ad66-4e74-9927-fe67a483e9e0"],
Cell[1543422, 40422, 577, 18, 1063, "Input", "ExpressionUUID" -> \
"b48661e8-ca05-4986-93e7-e480bf45fd2c",
 InitializationCell->True],
Cell[1544002, 40442, 3098, 87, 2646, "Input", "ExpressionUUID" -> \
"3c70b097-be37-4c80-85a8-039bc712b444",
 InitializationCell->True],
Cell[1547103, 40531, 278, 6, 1210, "Text", "ExpressionUUID" -> \
"5c47c694-2f70-4da2-ac97-4678ea3e2ac4"],
Cell[1547384, 40539, 172, 4, 712, "Input", "ExpressionUUID" -> \
"a9235f6c-e219-45af-b6af-ab86a1b3250d"],
Cell[1547559, 40545, 205, 5, 712, "Input", "ExpressionUUID" -> \
"73e1612a-fd2b-4c38-9629-5fcf1a189eed"],
Cell[1547767, 40552, 174, 4, 712, "Input", "ExpressionUUID" -> \
"860b54dd-153f-4db6-985a-5cf3f90aef3a"],
Cell[1547944, 40558, 268, 7, 712, "Input", "ExpressionUUID" -> \
"0fea26d3-5448-4985-af79-87f0ab3140d8"],
Cell[1548215, 40567, 197, 3, 732, "Text", "ExpressionUUID" -> \
"af428530-f87d-4d35-bd31-806645bde68a"],
Cell[1548415, 40572, 411, 11, 712, "Input", "ExpressionUUID" -> \
"df0c3fd4-5b37-4396-a33d-a8912a8d695b",
 Evaluatable->False],
Cell[1548829, 40585, 180, 3, 732, "Text", "ExpressionUUID" -> \
"7fbc0411-8c77-48bd-8450-cfaba0230f4a"],
Cell[1549012, 40590, 197, 3, 732, "Text", "ExpressionUUID" -> \
"95c4ea5d-21f3-43a6-b09c-607717179288"],
Cell[1549212, 40595, 172, 4, 712, "Input", "ExpressionUUID" -> \
"80c09b33-db97-44f4-be59-2d44366cf859"],
Cell[1549387, 40601, 324, 9, 712, "Input", "ExpressionUUID" -> \
"b684eee0-ad94-42d9-b080-920269b7333c"],
Cell[1549714, 40612, 254, 6, 712, "Input", "ExpressionUUID" -> \
"0287ca86-bc0c-4015-8b03-24e3804dcf26"],
Cell[1549971, 40620, 234, 6, 712, "Input", "ExpressionUUID" -> \
"7007e40d-d420-4df8-839a-70e58fa53a44"],
Cell[1550208, 40628, 435, 12, 712, "Input", "ExpressionUUID" -> \
"e6c20897-f9e7-4dec-8058-66fc01d32782"],
Cell[1550646, 40642, 233, 6, 732, "Text", "ExpressionUUID" -> \
"738a8e59-f42c-43ef-b259-cb689c66c54a"],
Cell[1550882, 40650, 265, 7, 712, "Input", "ExpressionUUID" -> \
"b6e186a5-c574-4607-9fa7-d59fecf3b75e"],
Cell[1551150, 40659, 462, 9, 1210, "Text", "ExpressionUUID" -> \
"f7c762c8-6b3a-4cc5-bc8b-0919c9e5067c"],
Cell[1551615, 40670, 243, 6, 1210, "Text", "ExpressionUUID" -> \
"1eb25575-af4d-4529-9b95-eafdde23b92a"],
Cell[1551861, 40678, 185, 4, 712, "Input", "ExpressionUUID" -> \
"769ff9ea-d72a-47d4-a774-142e07e7ba6e"],
Cell[1552049, 40684, 161, 3, 732, "Text", "ExpressionUUID" -> \
"7bdb8e70-7d89-430e-98da-70788c79fbfb"],
Cell[1552213, 40689, 171, 4, 712, "Input", "ExpressionUUID" -> \
"4ed4b3a7-4fd8-41f7-b6b2-7aea52985db3"],
Cell[1552387, 40695, 177, 4, 712, "Input", "ExpressionUUID" -> \
"7e6671c1-569e-447c-9285-8fa41c3053fd"],
Cell[1552567, 40701, 646, 13, 1678, "Text", "ExpressionUUID" -> \
"4384d8be-8078-4e2d-900b-da9fff326090"],
Cell[1553216, 40716, 260, 7, 732, "Text", "ExpressionUUID" -> \
"157f7feb-ac7b-4dfc-b574-c35340c66f0b"],
Cell[1553479, 40725, 204, 5, 712, "Input", "ExpressionUUID" -> \
"bcf24422-5d45-4271-b9ba-c17c43828d13"],
Cell[1553686, 40732, 212, 5, 712, "Input", "ExpressionUUID" -> \
"3b796942-307f-4738-8741-9eb856ffefa4"],
Cell[1553901, 40739, 181, 3, 732, "Text", "ExpressionUUID" -> \
"699d6d87-9cbb-4a87-9d13-99205abbea62"],
Cell[1554085, 40744, 173, 4, 712, "Input", "ExpressionUUID" -> \
"ca5a44f8-6b9d-4523-9bdf-30b17960b4fa"],
Cell[1554261, 40750, 244, 6, 712, "Input", "ExpressionUUID" -> \
"64eba363-1e97-40c0-abde-7ce8a4d4f5b7"],
Cell[1554508, 40758, 242, 6, 712, "Input", "ExpressionUUID" -> \
"ed082231-2f37-4e7a-a67b-bea7f5f1d2b5"],
Cell[1554753, 40766, 266, 7, 790, "Input", "ExpressionUUID" -> \
"3cafab1c-7b87-41ce-9662-98202fe9763e"],
Cell[1555022, 40775, 267, 7, 790, "Input", "ExpressionUUID" -> \
"2b821239-3666-480f-95f6-60ae95ae592c"],
Cell[1555292, 40784, 265, 6, 732, "Text", "ExpressionUUID" -> \
"fd387e9d-787f-4c40-87b2-343ea8163770"],
Cell[1555560, 40792, 331, 9, 712, "Input", "ExpressionUUID" -> \
"2b1af099-199c-43ff-870d-ad7cdc8a0044"],
Cell[1555894, 40803, 406, 11, 790, "Input", "ExpressionUUID" -> \
"b8b8521a-c18d-4d3f-bc20-aaeaf0b146e0"],
Cell[1556303, 40816, 357, 10, 790, "Input", "ExpressionUUID" -> \
"c4258a43-a4db-4fb2-b383-2aeb573d8138"],
Cell[1556663, 40828, 176, 4, 712, "Input", "ExpressionUUID" -> \
"94bb5383-2b88-4066-aedf-e027b625b1fe"],
Cell[1556842, 40834, 451, 13, 712, "Input", "ExpressionUUID" -> \
"ceb6f223-d202-4a57-bef2-10aab2b735db"],
Cell[1557296, 40849, 527, 17, 771, "Input", "ExpressionUUID" -> \
"58a657a5-7435-4934-baa5-76a5573d5d49"],
Cell[1557826, 40868, 178, 4, 712, "Input", "ExpressionUUID" -> \
"519a801c-44b7-4b81-ae91-0f6858dcce28"],
Cell[1558007, 40874, 156, 3, 732, "Text", "ExpressionUUID" -> \
"f7f2f569-8ebc-49d2-b067-ae6f93da5dc8"],
Cell[1558166, 40879, 499, 13, 712, "Input", "ExpressionUUID" -> \
"a15053b3-68f4-4b6a-b0ab-baafad673ad6"],
Cell[1558668, 40894, 166, 3, 732, "Text", "ExpressionUUID" -> \
"22ddc0bb-fa57-457c-bdf9-3ed05e2a1463"],
Cell[1558837, 40899, 526, 16, 712, "Input", "ExpressionUUID" -> \
"bc180588-65a0-4127-86b1-1bcad4756efa"],
Cell[1559366, 40917, 553, 17, 771, "Input", "ExpressionUUID" -> \
"1ffc0a04-9a74-4804-9161-eb02644a80f8"],
Cell[1559922, 40936, 659, 19, 771, "Input", "ExpressionUUID" -> \
"b56face1-d716-4f16-bdd5-caa2a51aa720"],
Cell[1560584, 40957, 704, 19, 790, "Input", "ExpressionUUID" -> \
"77ca85cf-7aae-4bb2-8103-778f3be8648f"],
Cell[1561291, 40978, 561, 10, 1678, "Text", "ExpressionUUID" -> \
"fd7922a9-d01e-4ace-b743-f7f5ee22517d"],
Cell[1561855, 40990, 261, 7, 732, "Text", "ExpressionUUID" -> \
"4da14bd7-3168-4b0a-9be1-c2c82026839f"],
Cell[1562119, 40999, 195, 5, 712, "Input", "ExpressionUUID" -> \
"f7c5b726-4860-48ea-8d4a-3eb1d552f69b"],
Cell[1562317, 41006, 310, 7, 1210, "Text", "ExpressionUUID" -> \
"9db65050-df54-4d1f-b3d1-6ce535986901"],
Cell[1562630, 41015, 174, 4, 712, "Input", "ExpressionUUID" -> \
"3b9c6669-c453-4e5a-837d-5b92d285c0b0"],
Cell[1562807, 41021, 185, 3, 732, "Text", "ExpressionUUID" -> \
"b2b84e6d-1b54-4dde-9e90-7b29afabe947"],
Cell[1562995, 41026, 171, 4, 712, "Input", "ExpressionUUID" -> \
"af00bdaf-7791-4862-aca7-1d784e4e7245"],
Cell[1563169, 41032, 285, 7, 712, "Input", "ExpressionUUID" -> \
"538be12c-17ab-43d6-923a-701b6107d18f"],
Cell[1563457, 41041, 194, 3, 732, "Text", "ExpressionUUID" -> \
"33b9d32b-8e0f-43ab-840c-dba093a6b702"],
Cell[1563654, 41046, 166, 4, 712, "Input", "ExpressionUUID" -> \
"c9128e94-bb4b-4c6b-a271-8447e3733d84"],
Cell[1563823, 41052, 225, 5, 732, "Text", "ExpressionUUID" -> \
"24240ce7-fc61-4ff2-be54-fdd61c364a82"],
Cell[1564051, 41059, 511, 15, 712, "Input", "ExpressionUUID" -> \
"57a7c119-3362-4392-9b14-2d533cf695f7"],
Cell[1564565, 41076, 819, 22, 1347, "Input", "ExpressionUUID" -> \
"13ded31e-d4a6-41f1-9e6b-72674d0ef793"],
Cell[1565387, 41100, 196, 5, 712, "Input", "ExpressionUUID" -> \
"3c7b1b2f-800b-498a-9721-b1c428038361"],
Cell[1565586, 41107, 217, 4, 732, "Text", "ExpressionUUID" -> \
"d2cdca3d-7757-448a-b4c9-e57cd3e70cda"],
Cell[1565806, 41113, 687, 18, 1210, "Input", "ExpressionUUID" -> \
"f6a9de95-73f9-440c-9f30-f32b9cb811cf"],
Cell[1566496, 41133, 423, 10, 712, "Input", "ExpressionUUID" -> \
"88074aed-26ee-4e05-b451-96f033c0e700"],
Cell[1566922, 41145, 247, 6, 732, "Text", "ExpressionUUID" -> \
"38fa4ba3-104d-49f3-8684-066e3da07c71"],
Cell[1567172, 41153, 492, 12, 712, "Input", "ExpressionUUID" -> \
"428f3ddc-1f2e-4a1f-a680-3d85a9e1c141"],
Cell[1567667, 41167, 238, 6, 732, "Text", "ExpressionUUID" -> \
"403b2399-e09c-4d9d-8001-22454287761b"],
Cell[1567908, 41175, 985, 24, 1210, "Input", "ExpressionUUID" -> \
"002934b8-22b4-43b5-98f5-a1b689ed965e"],
Cell[1568896, 41201, 197, 5, 712, "Input", "ExpressionUUID" -> \
"98fcc2da-8b46-47a2-8e45-5971ce58b508"],
Cell[1569096, 41208, 302, 8, 712, "Input", "ExpressionUUID" -> \
"e5bc0459-1ead-41f2-b28f-5c63cb66929b"],
Cell[1569401, 41218, 233, 4, 732, "Text", "ExpressionUUID" -> \
"162bd240-349d-4e2f-beb5-e6c6f472c5f5"],
Cell[1569637, 41224, 741, 20, 1210, "Input", "ExpressionUUID" -> \
"428a9fe6-05f4-4db1-99c8-d15f2e59a8a0"],
Cell[1570381, 41246, 552, 16, 790, "Input", "ExpressionUUID" -> \
"e41efcb7-987a-46fb-a4d1-71ef73e8f453"],
Cell[1570936, 41264, 285, 8, 790, "Input", "ExpressionUUID" -> \
"842e7693-fde4-4798-b281-7492ab3d8a26"],
Cell[1571224, 41274, 264, 7, 790, "Input", "ExpressionUUID" -> \
"483aa711-420e-4098-b281-88aa6d22131a"],
Cell[1571491, 41283, 296, 6, 732, "Text", "ExpressionUUID" -> \
"4b0e1e74-fc89-4800-8704-c744a787bff6"],
Cell[1571790, 41291, 1327, 31, 1347, "Input", "ExpressionUUID" -> \
"dd1be7ba-5dd9-4246-963c-9437153930a4"],
Cell[1573120, 41324, 349, 9, 712, "Input", "ExpressionUUID" -> \
"31a281a4-3ecb-4de1-b904-fa25fae68374"],
Cell[1573472, 41335, 351, 7, 1678, "Text", "ExpressionUUID" -> \
"3d68e5ec-c9a3-42f0-a128-6feadafba15e"],
Cell[1573826, 41344, 178, 4, 712, "Input", "ExpressionUUID" -> \
"2c5c7c56-08de-44cd-8ae7-637b078ad256"],
Cell[1574007, 41350, 865, 21, 1210, "Input", "ExpressionUUID" -> \
"2a3d196e-41c0-4a2c-bc78-9d2e2c68e3d8"],
Cell[1574875, 41373, 236, 6, 732, "Text", "ExpressionUUID" -> \
"c2096ea8-56c3-4ccc-b977-c403165f7293"],
Cell[1575114, 41381, 361, 8, 712, "Input", "ExpressionUUID" -> \
"340ef0e1-213c-4ce8-a8a4-86491125a6b7"],
Cell[1575478, 41391, 195, 3, 732, "Text", "ExpressionUUID" -> \
"0a4096ce-f7ee-4f18-9de3-7410a9c478e9"],
Cell[1575676, 41396, 436, 12, 712, "Input", "ExpressionUUID" -> \
"41842303-678a-41a7-9343-9a9c5edad15c"],
Cell[1576115, 41410, 527, 13, 712, "Input", "ExpressionUUID" -> \
"1681b9df-5a80-41d4-9af3-7c37e156fe6d"],
Cell[1576645, 41425, 287, 6, 1210, "Text", "ExpressionUUID" -> \
"f27ec42e-27f7-4259-bb81-a626b80410fa"],
Cell[1576935, 41433, 465, 13, 712, "Input", "ExpressionUUID" -> \
"e94cc71e-13d5-444d-a9c5-460f96bcddbe"],
Cell[1577403, 41448, 467, 9, 2167, "Text", "ExpressionUUID" -> \
"4d2425f2-d6a1-4b31-8778-4ead8891cf5f"],
Cell[1577873, 41459, 497, 15, 790, "Input", "ExpressionUUID" -> \
"96cda99f-39da-413b-aeea-3c60eb5068ac"],
Cell[1578373, 41476, 265, 7, 790, "Input", "ExpressionUUID" -> \
"d325dacc-03b1-462d-9207-16f869cf2488"],
Cell[1578641, 41485, 267, 7, 790, "Input", "ExpressionUUID" -> \
"b7f4e3f6-160c-419d-aba9-cdbfe734ba0e"],
Cell[1578911, 41494, 209, 5, 712, "Input", "ExpressionUUID" -> \
"ca219996-8915-411f-8309-a869809401ca"],
Cell[1579123, 41501, 246, 6, 1210, "Text", "ExpressionUUID" -> \
"9afda1e5-3556-4b24-beb6-efe3236f1353"],
Cell[1579372, 41509, 173, 4, 712, "Input", "ExpressionUUID" -> \
"d896164a-4dd1-49bd-a167-c09ba28e1b0a"],
Cell[1579548, 41515, 177, 4, 712, "Input", "ExpressionUUID" -> \
"72e498ff-aa3d-477a-8c73-3d3c187896de"],
Cell[1579728, 41521, 177, 3, 732, "Text", "ExpressionUUID" -> \
"d4087a73-4e12-4756-8f3a-5c7de33c49a9"],
Cell[1579908, 41526, 187, 4, 712, "Input", "ExpressionUUID" -> \
"b818966c-5e9b-44c8-9b18-c3acdc5937bb"],
Cell[1580098, 41532, 186, 3, 732, "Text", "ExpressionUUID" -> \
"014b0cca-c288-4655-9be8-6e077b23756d"],
Cell[1580287, 41537, 174, 4, 712, "Input", "ExpressionUUID" -> \
"d8ef1732-13fa-4927-8c36-474498b3ab5d"],
Cell[1580464, 41543, 677, 17, 712, "Input", "ExpressionUUID" -> \
"a042f272-545b-410a-9c16-72a973972bc1"],
Cell[1581144, 41562, 528, 14, 712, "Input", "ExpressionUUID" -> \
"94f33afc-85d9-4eed-ac4d-1b88b6c66319"],
Cell[1581675, 41578, 336, 8, 712, "Input", "ExpressionUUID" -> \
"4356102b-d56a-432a-a31b-ac8b38262e22"],
Cell[1582014, 41588, 183, 3, 732, "Text", "ExpressionUUID" -> \
"50ba99a6-630b-4ca5-bf40-c88db777be05"],
Cell[1582200, 41593, 240, 5, 712, "Input", "ExpressionUUID" -> \
"ec0f8cbf-21b5-46f7-b3be-8d1630264f6f"],
Cell[1582443, 41600, 277, 6, 1210, "Text", "ExpressionUUID" -> \
"d56b4d25-e269-4db9-97ed-2a963bc0253f"],
Cell[1582723, 41608, 472, 14, 712, "Input", "ExpressionUUID" -> \
"91acc6f6-fcfa-435a-91ad-7246b72cdab8"],
Cell[1583198, 41624, 234, 6, 771, "Input", "ExpressionUUID" -> \
"2e961375-5c74-49b3-9640-42f5da75cc53"],
Cell[1583435, 41632, 427, 12, 771, "Input", "ExpressionUUID" -> \
"6b21176e-7961-4fc8-9be3-849dcf5fe0e8"],
Cell[1583865, 41646, 557, 13, 790, "Input", "ExpressionUUID" -> \
"1359e4d6-98fb-4d4d-af49-81476d3fe523"],
Cell[1584425, 41661, 581, 15, 790, "Input", "ExpressionUUID" -> \
"0c580310-d945-4dc8-9626-c6ee855abd7b"],
Cell[1585009, 41678, 162, 3, 732, "Text", "ExpressionUUID" -> \
"903d314b-339b-4e15-b8e1-c3129544c8ef"],
Cell[1585174, 41683, 231, 5, 712, "Input", "ExpressionUUID" -> \
"05ca2c3c-80c8-41ee-91fe-adf00c3f6b38"],
Cell[1585408, 41690, 307, 7, 712, "Input", "ExpressionUUID" -> \
"1b5def44-4ed6-4583-b4dd-df067695d5ef"],
Cell[1585718, 41699, 161, 3, 732, "Text", "ExpressionUUID" -> \
"750bf6e8-dea2-4de6-9f68-9d00b86681bd"],
Cell[1585882, 41704, 268, 7, 712, "Input", "ExpressionUUID" -> \
"69af780e-7bf5-4270-9ac4-7f63e5635b5a"],
Cell[1586153, 41713, 218, 6, 1210, "Text", "ExpressionUUID" -> \
"b8b61af8-aae4-4b72-b5d8-378eced5584b"],
Cell[1586374, 41721, 233, 5, 771, "Input", "ExpressionUUID" -> \
"f96748ff-d8dc-4c50-976c-20696c54144e"],
Cell[1586610, 41728, 447, 10, 1678, "Text", "ExpressionUUID" -> \
"227ed8ae-fcfc-4dbe-9cfb-2cb371e4c35f"],
Cell[1587060, 41740, 991, 18, 3114, "Text", "ExpressionUUID" -> \
"0b1e30a5-2821-4533-8511-857c93f0d6c0"],
Cell[1588054, 41760, 172, 4, 712, "Input", "ExpressionUUID" -> \
"35ec399a-496a-4a63-991a-f96618cbf825"],
Cell[1588229, 41766, 180, 3, 732, "Text", "ExpressionUUID" -> \
"a5a4e0b7-478d-45e5-a919-82ab3af32627"],
Cell[1588412, 41771, 195, 5, 712, "Input", "ExpressionUUID" -> \
"8e9c825d-64c5-4e4e-bd41-2a7c6dbe890c"],
Cell[1588610, 41778, 266, 6, 712, "Input", "ExpressionUUID" -> \
"50f6f1de-ae96-467b-bb8b-42dd956aec09"],
Cell[1588879, 41786, 872, 20, 1239, "Input", "ExpressionUUID" -> \
"6aebe8b6-edab-455d-9c8b-0ae4b57faad0"],
Cell[1589754, 41808, 193, 4, 712, "Input", "ExpressionUUID" -> \
"4ce115f7-41eb-419f-9f78-0786bcfef7d2"],
Cell[1589950, 41814, 281, 7, 732, "Text", "ExpressionUUID" -> \
"05043410-5d79-4fa9-b263-fc5862c089a2"],
Cell[1590234, 41823, 200, 4, 712, "Input", "ExpressionUUID" -> \
"372bb2b6-869d-4d37-8f59-51cc1296efad"],
Cell[1590437, 41829, 210, 5, 712, "Input", "ExpressionUUID" -> \
"dbde3088-bec7-4263-a474-247b4175e13c"],
Cell[1590650, 41836, 443, 10, 712, "Input", "ExpressionUUID" -> \
"4ef8d81e-f1da-48ee-9fef-df7bb1efe4c1"],
Cell[1591096, 41848, 344, 7, 1210, "Text", "ExpressionUUID" -> \
"c073e965-a6ff-445e-bf9f-8d0d3c33fef5"],
Cell[1591443, 41857, 347, 10, 760, "Input", "ExpressionUUID" -> \
"a9be7d85-e9d3-433e-bd24-3183c4f6e0e5"],
Cell[1591793, 41869, 237, 4, 732, "Text", "ExpressionUUID" -> \
"5e4744ea-669c-47a9-9819-186922acb92f"],
Cell[1592033, 41875, 295, 6, 732, "Text", "ExpressionUUID" -> \
"af9e4a8e-e89a-4505-93da-a61297c33ca5"],
Cell[1592331, 41883, 656, 16, 760, "Input", "ExpressionUUID" -> \
"127c1f40-1cf8-443f-bf0e-119d12b4eb49"],
Cell[1592990, 41901, 593, 15, 760, "Input", "ExpressionUUID" -> \
"e137c6c2-ced4-4c30-8502-3f1bb89977d7"],
Cell[1593586, 41918, 376, 8, 1210, "Text", "ExpressionUUID" -> \
"5031ac18-4674-4615-9c02-8a218eaacd6a"],
Cell[1593965, 41928, 610, 17, 790, "Input", "ExpressionUUID" -> \
"4701bc36-1a36-490d-8ae0-4293b2a0d51d"],
Cell[1594578, 41947, 279, 4, 732, "Text", "ExpressionUUID" -> \
"d631035f-dc96-4edf-856c-5c1aeeadbaba"],
Cell[1594860, 41953, 685, 23, 1620, "Input", "ExpressionUUID" -> \
"965ea9ac-a27b-41f0-82e5-dcd27fd42a4c"],
Cell[1595548, 41978, 621, 13, 712, "Input", "ExpressionUUID" -> \
"4ec44543-94e0-48c8-9efb-6581d4c39588"],
Cell[1596172, 41993, 357, 9, 712, "Input", "ExpressionUUID" -> \
"7530aa09-a37f-41f0-a8cc-76671dd554a8"],
Cell[1596532, 42004, 180, 4, 712, "Input", "ExpressionUUID" -> \
"eb1a6d81-f31c-431b-a305-85cf2c53e89d"],
Cell[1596715, 42010, 872, 22, 1698, "Input", "ExpressionUUID" -> \
"834c0012-c50b-45d5-9422-deb15153b185"],
Cell[1597590, 42034, 218, 5, 712, "Input", "ExpressionUUID" -> \
"baa91dee-8c02-4d40-bff2-c163a1a0f7ab"],
Cell[1597811, 42041, 366, 7, 712, "Input", "ExpressionUUID" -> \
"ef1a3a77-ec22-4012-9cf0-759f200ff18f"],
Cell[1598180, 42050, 419, 8, 712, "Input", "ExpressionUUID" -> \
"efb1ed3a-b1b2-4634-9552-4b9e682fe8b9"],
Cell[1598602, 42060, 599, 13, 712, "Input", "ExpressionUUID" -> \
"6ce2acc4-da4a-427d-a31f-e5a69e531993"],
Cell[1599204, 42075, 761, 16, 1678, "Text", "ExpressionUUID" -> \
"29737a65-fad1-4721-a93a-5728539ce843"],
Cell[1599968, 42093, 263, 6, 1210, "Text", "ExpressionUUID" -> \
"7cfc6582-3848-4085-82b7-05b7a65e877f"],
Cell[1600234, 42101, 746, 20, 1698, "Input", "ExpressionUUID" -> \
"14b41f39-4b6a-4c6f-a498-78861343f813"],
Cell[1600983, 42123, 330, 7, 732, "Text", "ExpressionUUID" -> \
"ac4e918a-04ec-4966-95d5-573aaeb221d1"],
Cell[1601316, 42132, 605, 17, 1210, "Input", "ExpressionUUID" -> \
"55c5859b-6c59-49ef-9d26-9479edb20423"],
Cell[1601924, 42151, 302, 6, 732, "Text", "ExpressionUUID" -> \
"35bd2a02-2da1-4f7e-ab6a-cd5cd96718cd"],
Cell[1602229, 42159, 275, 6, 732, "Text", "ExpressionUUID" -> \
"4c2cce90-8abc-438c-9ecc-220cfc4b8a98"],
Cell[1602507, 42167, 171, 4, 712, "Input", "ExpressionUUID" -> \
"3523d1e8-b2fa-45bb-bfa5-1ecd43ed019e"],
Cell[1602681, 42173, 937, 25, 1698, "Input", "ExpressionUUID" -> \
"b83b8347-76e7-4c55-ae5f-59de20d8d7e1"],
Cell[1603621, 42200, 447, 8, 732, "Text", "ExpressionUUID" -> \
"28eb5a73-a084-4002-a715-58b482a4cca9"],
Cell[1604071, 42210, 246, 6, 712, "Input", "ExpressionUUID" -> \
"ce99d229-cda6-4d61-8531-f804d50f2ec6"],
Cell[1604320, 42218, 447, 8, 1210, "Text", "ExpressionUUID" -> \
"3612731b-2d7d-458d-b5e6-bcffa75dad9a"],
Cell[1604770, 42228, 295, 7, 712, "Input", "ExpressionUUID" -> \
"6e565048-1775-4bbe-adf4-28643be166ab"],
Cell[1605068, 42237, 615, 11, 2167, "Text", "ExpressionUUID" -> \
"80f435e9-a1fb-4a7e-8b99-b6d87a20e7ec"],
Cell[1605686, 42250, 286, 5, 732, "Text", "ExpressionUUID" -> \
"8a9586ec-6970-4cb4-b450-a245ebae946f"],
Cell[1605975, 42257, 173, 4, 712, "Input", "ExpressionUUID" -> \
"dbe52ea1-aa79-4d1f-8771-0f34854ee1cd"],
Cell[1606151, 42263, 177, 4, 712, "Input", "ExpressionUUID" -> \
"1253d1fa-7bd4-4d46-9c08-6167f97b457a"],
Cell[1606331, 42269, 297, 6, 1210, "Text", "ExpressionUUID" -> \
"105b22cf-3054-49b1-9702-9129bab5d5b6"],
Cell[1606631, 42277, 185, 4, 712, "Input", "ExpressionUUID" -> \
"93a5e33d-8eeb-4400-9730-434d28fc420f"],
Cell[1606819, 42283, 175, 3, 732, "Text", "ExpressionUUID" -> \
"006545c8-ec83-4a94-ba96-2660207935ea"],
Cell[1606997, 42288, 175, 4, 712, "Input", "ExpressionUUID" -> \
"42e6d407-6b20-4d25-8d07-8882d34d4b6d"],
Cell[1607175, 42294, 963, 21, 1210, "Input", "ExpressionUUID" -> \
"ec5a75dd-b21b-426a-85be-223ca964151c"],
Cell[1608141, 42317, 374, 9, 712, "Input", "ExpressionUUID" -> \
"2cc48b3a-ce4f-4979-b778-09fbb90a578c"],
Cell[1608518, 42328, 658, 15, 1210, "Input", "ExpressionUUID" -> \
"77bdb8fb-0553-4fb7-a82f-6e9c90cacbf5"],
Cell[1609179, 42345, 832, 21, 1210, "Input", "ExpressionUUID" -> \
"bd45df9e-6ecd-4414-ab8d-09ca2e6f023a"],
Cell[1610014, 42368, 882, 22, 1210, "Input", "ExpressionUUID" -> \
"81e07d7b-ff14-4f45-9ef6-7da7d9be2ca5"],
Cell[1610899, 42392, 884, 22, 1210, "Input", "ExpressionUUID" -> \
"39b1b43c-9f65-4fda-92bb-9ddda0b2deb8"],
Cell[1611786, 42416, 930, 22, 1210, "Input", "ExpressionUUID" -> \
"af1144ae-72bd-4624-9857-76da944d3820"],
Cell[1612719, 42440, 313, 7, 1210, "Text", "ExpressionUUID" -> \
"b67a68fb-8f6f-432e-94ae-2531ab3ed57c"],
Cell[1613035, 42449, 391, 8, 712, "Input", "ExpressionUUID" -> \
"7af72437-81a9-4b20-aa8f-0b9c7b72e514"],
Cell[1613429, 42459, 235, 5, 712, "Input", "ExpressionUUID" -> \
"0e162878-9706-4280-b244-ac8cecb052d3"],
Cell[1613667, 42466, 418, 10, 760, "Input", "ExpressionUUID" -> \
"aa1fb8ba-7a6c-4312-aa32-d9e6f5cbd602"],
Cell[1614088, 42478, 163, 3, 732, "Text", "ExpressionUUID" -> \
"56bd5a2f-4d5d-4526-9f47-5e165314a2a1"],
Cell[1614254, 42483, 182, 4, 712, "Input", "ExpressionUUID" -> \
"e93a93a1-0505-4c00-bb53-4342910740d6"],
Cell[1614439, 42489, 322, 7, 712, "Input", "ExpressionUUID" -> \
"b5c00c6d-038c-4c40-958c-46aa141e60ab"],
Cell[1614764, 42498, 230, 5, 712, "Input", "ExpressionUUID" -> \
"0c87ebed-df2e-4cb4-998f-87fa0b6db116"],
Cell[1614997, 42505, 193, 4, 712, "Input", "ExpressionUUID" -> \
"06069958-a0c7-42ae-9cca-31c3cf060e36"],
Cell[1615193, 42511, 523, 10, 1210, "Text", "ExpressionUUID" -> \
"8bc58364-57d6-4324-8bb2-25416153f900"],
Cell[1615719, 42523, 185, 4, 712, "Input", "ExpressionUUID" -> \
"de6486b7-9005-4e5a-8b6d-c56dddd6b0cd"],
Cell[1615907, 42529, 422, 9, 712, "Input", "ExpressionUUID" -> \
"33c3377c-b9fd-452e-8cba-0f44c1c633ef"],
Cell[1616332, 42540, 288, 8, 712, "Input", "ExpressionUUID" -> \
"76f87f4a-76fc-4f53-9a97-37d12f06cf50"],
Cell[1616623, 42550, 195, 4, 712, "Input", "ExpressionUUID" -> \
"cc1350a5-4ceb-4975-949b-e0e8cffb3052"],
Cell[1616821, 42556, 506, 12, 712, "Input", "ExpressionUUID" -> \
"44beb3bf-9d6d-457d-bcff-7b765e8054c8"],
Cell[1617330, 42570, 190, 3, 732, "Text", "ExpressionUUID" -> \
"a03f081b-5640-420d-b5fd-2c9340d834b2"],
Cell[1617523, 42575, 428, 9, 712, "Input", "ExpressionUUID" -> \
"d85c80b6-5269-4bf9-a30d-7c84776428ab"],
Cell[1617954, 42586, 233, 5, 712, "Input", "ExpressionUUID" -> \
"04619cfe-1312-4663-a75e-5c147db8d5e2"],
Cell[1618190, 42593, 160, 3, 732, "Text", "ExpressionUUID" -> \
"c5d9e997-6ea8-4141-96fe-a5ce7cfe795a"]
}, Closed]],
Cell[CellGroupData[{
Cell[1618387, 42601, 428, 6, 42, "Subsubsection", "ExpressionUUID" -> \
"52f7478a-ab6f-4799-bdc3-aa23b0d409c2"],
Cell[1618818, 42609, 262, 6, 292, "Text", "ExpressionUUID" -> \
"0d41a29e-330d-4ab4-9261-b60e634985ee"],
Cell[1619083, 42617, 304, 6, 295, "Input", "ExpressionUUID" -> \
"c96b5017-3329-47e1-9449-372c804334f4"],
Cell[1619390, 42625, 162, 3, 292, "Text", "ExpressionUUID" -> \
"b234e0e8-18df-4761-9652-907d96e93e9e"],
Cell[1619555, 42630, 230, 5, 295, "Input", "ExpressionUUID" -> \
"6083f9ab-303f-4776-80d3-6720404734c6"],
Cell[1619788, 42637, 318, 7, 295, "Input", "ExpressionUUID" -> \
"feed0615-81ea-49a6-8249-16f2c8b7c428"],
Cell[1620109, 42646, 192, 4, 295, "Input", "ExpressionUUID" -> \
"ad59759a-a3e7-4125-b12c-dff79be96772"],
Cell[1620304, 42652, 278, 6, 292, "Text", "ExpressionUUID" -> \
"04002039-4b95-4309-86e0-bdcfeca44feb"],
Cell[1620585, 42660, 177, 4, 295, "Input", "ExpressionUUID" -> \
"77931ced-5338-4fa1-94c3-8049d7b20f9b"],
Cell[1620765, 42666, 362, 8, 295, "Input", "ExpressionUUID" -> \
"324b0f66-4f28-4576-be3d-e30ad1160dea"],
Cell[1621130, 42676, 150, 3, 292, "Text", "ExpressionUUID" -> \
"b55bde4f-4759-493e-8eef-0555e9c5ebc0"],
Cell[1621283, 42681, 187, 4, 295, "Input", "ExpressionUUID" -> \
"e5ab71f1-8cd2-4005-b6a4-652f82e80c87"],
Cell[1621473, 42687, 197, 3, 292, "Text", "ExpressionUUID" -> \
"7333555b-08e7-402b-9391-39818646a440"],
Cell[1621673, 42692, 725, 19, 328, "Input", "ExpressionUUID" -> \
"33e5e57e-8942-4567-bd69-bbb850a12897",
 Evaluatable->False],
Cell[1622401, 42713, 241, 4, 292, "Text", "ExpressionUUID" -> \
"a4151cd7-0627-4e4b-b298-1c92c2c843cf"],
Cell[1622645, 42719, 327, 10, 328, "Input", "ExpressionUUID" -> \
"f0926389-940d-4baa-9320-266901780ab7",
 Evaluatable->False],
Cell[1622975, 42731, 465, 12, 295, "Input", "ExpressionUUID" -> \
"410fa450-a057-47dc-96a9-7e6abc4443d1"],
Cell[1623443, 42745, 316, 9, 481, "Text", "ExpressionUUID" -> \
"ea2d891b-ed3c-461a-a5bb-1c06bf83697b"],
Cell[1623762, 42756, 233, 5, 295, "Input", "ExpressionUUID" -> \
"fc5d51ad-69a8-495d-80bc-560e983c0c16"],
Cell[1623998, 42763, 154, 3, 292, "Text", "ExpressionUUID" -> \
"6cf9c327-d28f-40f5-9c8e-9887bae70bf9"],
Cell[1624155, 42768, 198, 4, 295, "Input", "ExpressionUUID" -> \
"a801cf83-c9ed-4406-b1be-f80bd9b07746"],
Cell[1624356, 42774, 595, 17, 490, "Input", "ExpressionUUID" -> \
"256c74b3-910b-4742-86df-9d71579acd04",
 Evaluatable->False],
Cell[1624954, 42793, 488, 12, 292, "Text", "ExpressionUUID" -> \
"cdaa0ea5-c7e1-481e-9905-26bcddf63ca9"],
Cell[1625445, 42807, 627, 20, 328, "Input", "ExpressionUUID" -> \
"2f9afa65-7e1b-479f-9584-e0ac8e6ae70c",
 Evaluatable->False],
Cell[1626075, 42829, 997, 23, 663, "Text", "ExpressionUUID" -> \
"c90f3545-786d-418a-9774-e046cca260ed"],
Cell[1627075, 42854, 748, 21, 328, "Input", "ExpressionUUID" -> \
"d122c82b-e83b-4f9b-82dc-78060c4d9907",
 Evaluatable->False],
Cell[1627826, 42877, 487, 12, 295, "Input", "ExpressionUUID" -> \
"2d83c6b9-4f61-4d9d-8b51-c85d7008201a",
 Evaluatable->False],
Cell[1628316, 42891, 761, 17, 487, "Text", "ExpressionUUID" -> \
"1a2db856-fc3a-48d6-aa42-099f43f7d556"],
Cell[1629080, 42910, 174, 4, 295, "Input", "ExpressionUUID" -> \
"60632459-bbc3-4d79-8d66-8860b123ea35"],
Cell[1629257, 42916, 265, 7, 295, "Input", "ExpressionUUID" -> \
"01d94126-b562-46cb-a681-c90bfac891cf"],
Cell[1629525, 42925, 461, 10, 295, "Input", "ExpressionUUID" -> \
"d45ce338-2b01-494e-a443-588e6a8be573"],
Cell[1629989, 42937, 174, 4, 295, "Input", "ExpressionUUID" -> \
"a3b8379d-df80-40bc-9622-c753b9d3969c"],
Cell[1630166, 42943, 394, 11, 295, "Input", "ExpressionUUID" -> \
"cdda1d4d-604f-472d-8389-72539beec0b0"],
Cell[1630563, 42956, 192, 4, 295, "Input", "ExpressionUUID" -> \
"760d3a0c-b5a6-4af8-ad59-e9aea235016d"],
Cell[1630758, 42962, 658, 12, 1035, "Text", "ExpressionUUID" -> \
"a657d169-d6a8-4a9b-b4c7-5f4da0289fbe"],
Cell[1631419, 42976, 471, 10, 295, "Input", "ExpressionUUID" -> \
"2a6fdf82-8686-4d27-a7b0-8be40b6fa078"],
Cell[1631893, 42988, 178, 3, 292, "Text", "ExpressionUUID" -> \
"a718a955-040e-4752-99e8-1b1fb54e59d8"],
Cell[1632074, 42993, 1102, 26, 490, "Input", "ExpressionUUID" -> \
"5ba2ebc7-8306-4653-be23-5c7c14276bab",
 Evaluatable->False],
Cell[1633179, 43021, 615, 14, 295, "Input", "ExpressionUUID" -> \
"703caf05-63a3-4c46-bb98-608fc41ba81d"],
Cell[1633797, 43037, 377, 8, 292, "Text", "ExpressionUUID" -> \
"e468ba03-d213-4c17-b75f-30fe5c6a8bd3"],
Cell[1634177, 43047, 500, 11, 295, "Input", "ExpressionUUID" -> \
"0f398b96-d319-41b6-864b-248b7d12290c"],
Cell[1634680, 43060, 465, 8, 292, "Text", "ExpressionUUID" -> \
"8d4c54e6-f3be-42a8-855a-b09d00312cbd"],
Cell[1635148, 43070, 378, 8, 295, "Input", "ExpressionUUID" -> \
"bee1d934-093a-459a-9205-5344016d905f"],
Cell[1635529, 43080, 241, 7, 481, "Text", "ExpressionUUID" -> \
"dd845c15-af31-44b4-bb4f-1b1969fd072b"],
Cell[1635773, 43089, 806, 15, 663, "Text", "ExpressionUUID" -> \
"d86b8d45-f10d-4183-8bdc-a2a3daaf5464"],
Cell[1636582, 43106, 309, 7, 292, "Text", "ExpressionUUID" -> \
"2f1e332a-6e6f-4825-b19c-8210fa96a75d"],
Cell[1636894, 43115, 907, 23, 679, "Input", "ExpressionUUID" -> \
"e17363d6-fdc9-4788-be45-39d619cfe243"],
Cell[1637804, 43140, 665, 18, 490, "Input", "ExpressionUUID" -> \
"91a8aeb1-b9fa-49fd-a0e9-3f42f501db40"],
Cell[1638472, 43160, 709, 18, 490, "Input", "ExpressionUUID" -> \
"dbe44363-0348-415f-a517-bce756fdb765"],
Cell[1639184, 43180, 1019, 26, 490, "Input", "ExpressionUUID" -> \
"1d218a0b-6ab1-412e-8eea-a3607fa9502b"],
Cell[1640206, 43208, 666, 14, 295, "Input", "ExpressionUUID" -> \
"105dd5be-9748-44bb-8a0a-89cf94822815"],
Cell[1640875, 43224, 253, 4, 292, "Text", "ExpressionUUID" -> \
"ba8bac2e-5f10-4669-ae41-7368062c593b"],
Cell[1641131, 43230, 332, 8, 295, "Input", "ExpressionUUID" -> \
"903249df-e36b-4d81-a176-0428ff1a2be7"],
Cell[1641466, 43240, 156, 3, 292, "Text", "ExpressionUUID" -> \
"21b354a1-d03e-4e49-ad4d-ce000a9f66f7"],
Cell[1641625, 43245, 905, 29, 490, "Input", "ExpressionUUID" -> \
"f65096ec-d67d-4e9f-8073-845f4b741269",
 Evaluatable->False],
Cell[1642533, 43276, 387, 8, 481, "Text", "ExpressionUUID" -> \
"99c13766-2916-48ba-b91a-7644b586ecab"],
Cell[1642923, 43286, 1172, 30, 868, "Input", "ExpressionUUID" -> \
"829b7f60-9393-4370-84cb-abd0893cb9d6"],
Cell[1644098, 43318, 342, 9, 679, "Input", "ExpressionUUID" -> \
"e1c8f6d5-06ec-4510-a8ab-647f4b774068",
 Evaluatable->False],
Cell[1644443, 43329, 471, 8, 493, "Text", "ExpressionUUID" -> \
"99368ce6-1b3b-4852-81cb-06c913a64177"],
Cell[1644917, 43339, 373, 9, 663, "Text", "ExpressionUUID" -> \
"76fd1dd9-4771-430e-b4f3-722d346a930d"],
Cell[1645293, 43350, 388, 11, 315, "Input", "ExpressionUUID" -> \
"6bf3e1ca-f182-4104-9eef-d4efeac830de",
 Evaluatable->False],
Cell[1645684, 43363, 386, 11, 315, "Input", "ExpressionUUID" -> \
"ddd9cea6-4fe3-42d1-87a0-985b75974919",
 Evaluatable->False],
Cell[1646073, 43376, 358, 10, 315, "Input", "ExpressionUUID" -> \
"8ed0ca5f-e44b-4d10-8384-4bba31308644",
 Evaluatable->False],
Cell[1646434, 43388, 494, 13, 315, "Input", "ExpressionUUID" -> \
"d282ba9d-dc73-46b1-8a58-84c68a5205d9",
 Evaluatable->False],
Cell[1646931, 43403, 797, 19, 493, "Text", "ExpressionUUID" -> \
"a752b111-1e98-441d-a22a-9d873a4ec4f4"],
Cell[1647731, 43424, 302, 7, 493, "Text", "ExpressionUUID" -> \
"018f8be0-fe13-4d73-90a4-fadddf7945d2"],
Cell[1648036, 43433, 407, 10, 295, "Input", "ExpressionUUID" -> \
"6203105c-407f-4320-88fa-bbfa54613cab"],
Cell[1648446, 43445, 1099, 28, 679, "Input", "ExpressionUUID" -> \
"041d6265-c1c2-49f0-926d-aa4e401d7fb3"],
Cell[1649548, 43475, 318, 6, 298, "Text", "ExpressionUUID" -> \
"43ad9b66-a0df-4a9e-a94d-d6a0ae3a707b"],
Cell[1649869, 43483, 651, 11, 1066, "Text", "ExpressionUUID" -> \
"fbba4795-2117-4761-b7a0-368082a79087"],
Cell[1650523, 43496, 748, 19, 295, "Input", "ExpressionUUID" -> \
"74f1f239-5b11-492b-a87b-f821f3661530",
 Evaluatable->False],
Cell[1651274, 43517, 197, 3, 292, "Text", "ExpressionUUID" -> \
"5ba84d14-750d-438c-a546-d8ccb4d0e6dd"],
Cell[1651474, 43522, 352, 10, 292, "Text", "ExpressionUUID" -> \
"2e7d692d-90b0-4379-982e-a49619643de0"],
Cell[1651829, 43534, 340, 5, 292, "Text", "ExpressionUUID" -> \
"93ac2b99-3b53-4cc2-9710-f66ab3b7b2ea"],
Cell[1652172, 43541, 202, 5, 295, "Input", "ExpressionUUID" -> \
"5fb1f786-3253-437b-b939-9a06a9873ec2"],
Cell[1652377, 43548, 234, 6, 295, "Input", "ExpressionUUID" -> \
"95104063-7219-4fbd-bf11-ff71ebb0242d"],
Cell[1652614, 43556, 165, 3, 292, "Text", "ExpressionUUID" -> \
"3aa00645-6b21-4ab2-84e1-93702f853878"],
Cell[1652782, 43561, 175, 4, 295, "Input", "ExpressionUUID" -> \
"a4012c21-d704-4e87-8b24-c8122180a9b1"],
Cell[1652960, 43567, 341, 8, 295, "Input", "ExpressionUUID" -> \
"bac076d2-aec1-46f5-b8c0-e7711e158951"],
Cell[1653304, 43577, 233, 5, 295, "Input", "ExpressionUUID" -> \
"6aac1c09-18a5-435b-9de6-17f867c199f6"],
Cell[1653540, 43584, 175, 3, 292, "Text", "ExpressionUUID" -> \
"1befac1a-5375-40e0-b568-bc9721f12e6a"],
Cell[1653718, 43589, 298, 7, 295, "Input", "ExpressionUUID" -> \
"ad220d01-b7f4-429f-a848-2e92cb1278e9"],
Cell[1654019, 43598, 316, 7, 295, "Input", "ExpressionUUID" -> \
"c2e1d7b9-4f95-45cc-9701-ed15ce68c5da"],
Cell[1654338, 43607, 713, 16, 487, "Text", "ExpressionUUID" -> \
"11cdd1e9-ec45-4b72-9fbf-a016957470f4"],
Cell[1655054, 43625, 1187, 27, 490, "Input", "ExpressionUUID" -> \
"1b120a17-7d1b-40d9-8aa3-19aa6502ba67"],
Cell[1656244, 43654, 182, 3, 292, "Text", "ExpressionUUID" -> \
"7e2a7759-1435-4c6b-9688-96fc19d760d8"],
Cell[1656429, 43659, 654, 17, 295, "Input", "ExpressionUUID" -> \
"3f54ad7a-ce3c-483b-8e94-f8047f53647b"],
Cell[1657086, 43678, 242, 4, 292, "Text", "ExpressionUUID" -> \
"ab078bd9-5b3b-4703-b021-5b19d7a7102b"],
Cell[1657331, 43684, 494, 15, 295, "Input", "ExpressionUUID" -> \
"f7a8e405-8f8d-429b-8aad-79d3743ac424"],
Cell[1657828, 43701, 602, 14, 318, "Text", "ExpressionUUID" -> \
"cb9246d1-ae8d-439f-8d27-004e8795df58"],
Cell[1658433, 43717, 1081, 28, 490, "Input", "ExpressionUUID" -> \
"3d4cfb4a-c4c4-4104-a944-e578ccc611ec"],
Cell[1659517, 43747, 599, 11, 865, "Text", "ExpressionUUID" -> \
"bed4d801-5154-4522-afbc-c2fc47c4c72c"],
Cell[1660119, 43760, 735, 14, 481, "Text", "ExpressionUUID" -> \
"88ddc8ab-00a2-4e0d-b564-cbb3acc78121"],
Cell[1660857, 43776, 267, 7, 295, "Input", "ExpressionUUID" -> \
"8e01afb4-209e-42af-ba83-ef0123d7ef65"],
Cell[1661127, 43785, 244, 6, 481, "Text", "ExpressionUUID" -> \
"2f88ec8f-0950-4077-9e36-d5c371ba494d"],
Cell[1661374, 43793, 327, 8, 295, "Input", "ExpressionUUID" -> \
"8029cc97-dadb-408f-b946-4cc6b86dc4fb"],
Cell[1661704, 43803, 230, 5, 295, "Input", "ExpressionUUID" -> \
"dfd96f57-4739-42fe-9424-0bc9a718eae2"],
Cell[1661937, 43810, 543, 9, 292, "Text", "ExpressionUUID" -> \
"c7cbec85-bd64-48f2-b62d-847ec26a767d"],
Cell[1662483, 43821, 174, 4, 295, "Input", "ExpressionUUID" -> \
"52b72a8f-a4fa-400a-b2a5-10883e01c44d"],
Cell[1662660, 43827, 562, 12, 295, "Input", "ExpressionUUID" -> \
"87dccf21-e1c3-4f4f-a17f-7ecf09e96e99"],
Cell[1663225, 43841, 263, 6, 292, "Text", "ExpressionUUID" -> \
"287dd801-650c-4632-bc61-3fa824f21166"],
Cell[1663491, 43849, 393, 9, 295, "Input", "ExpressionUUID" -> \
"06601980-8794-44f4-ae80-523c1d1699cf"],
Cell[1663887, 43860, 367, 8, 295, "Input", "ExpressionUUID" -> \
"914559e4-d602-4f7e-a1f9-46e0086ae1aa"],
Cell[1664257, 43870, 268, 6, 292, "Text", "ExpressionUUID" -> \
"4a88a6a7-ba35-455d-8be6-f07bd6499aa7"],
Cell[1664528, 43878, 932, 24, 490, "Input", "ExpressionUUID" -> \
"5ca54463-9137-4053-8f0a-e8acd8e07200"],
Cell[1665463, 43904, 539, 11, 676, "Text", "ExpressionUUID" -> \
"b0f0606b-e246-4b6c-b570-289f2b12ad30"]
}, Closed]],
Cell[CellGroupData[{
Cell[1666039, 43920, 470, 7, 42, "Subsubsection", "ExpressionUUID" -> \
"8b434415-766b-4fe8-ad80-d06aae5827a6"],
Cell[1666512, 43929, 3203, 84, 1503, "Text", "ExpressionUUID" -> \
"33f72e6c-057b-4418-8372-d4343191321a"],
Cell[1669718, 44015, 517, 17, 348, "Input", "ExpressionUUID" -> \
"adf68e03-f4ce-4cf4-9550-65ddc64fcf5f",
 Evaluatable->False],
Cell[1670238, 44034, 513, 12, 331, "Text", "ExpressionUUID" -> \
"c95d59ec-a280-4c51-96a2-6b150b2fe772"],
Cell[1670754, 44048, 1015, 29, 373, "Input", "ExpressionUUID" -> \
"10122f7e-9061-4a88-9183-bcb38727724f",
 Evaluatable->False],
Cell[1671772, 44079, 1070, 30, 373, "Input", "ExpressionUUID" -> \
"7e79ea62-6275-43a4-908d-c72f91f0b905",
 Evaluatable->False],
Cell[1672845, 44111, 162, 3, 292, "Text", "ExpressionUUID" -> \
"ba4a219d-4564-4b47-8264-fdc9242f8784"],
Cell[1673010, 44116, 1643, 44, 348, "Input", "ExpressionUUID" -> \
"17e3d223-3660-4cd1-8ba5-abedd4705616",
 Evaluatable->False],
Cell[1674656, 44162, 1659, 45, 348, "Input", "ExpressionUUID" -> \
"8e0226e1-904a-478d-a003-9c0f67aae883",
 Evaluatable->False],
Cell[1676318, 44209, 174, 3, 292, "Text", "ExpressionUUID" -> \
"23e2abe0-de0a-45b9-95bc-91f7a12501f9"],
Cell[1676495, 44214, 1846, 52, 549, "Input", "ExpressionUUID" -> \
"4e2d748b-2081-4ab0-a88e-6d634e2f6315",
 Evaluatable->False],
Cell[1678344, 44268, 4106, 83, 1450, "Text", "ExpressionUUID" -> \
"1f251c70-c0a7-40a8-b069-98abdfca6379"],
Cell[1682453, 44353, 356, 7, 481, "Text", "ExpressionUUID" -> \
"0dd57a04-4a88-404d-9590-f6b4bea71420"],
Cell[1682812, 44362, 556, 15, 295, "Input", "ExpressionUUID" -> \
"6bf58f21-b1b7-4d93-8bae-be10e7219ead",
 Evaluatable->False],
Cell[1683371, 44379, 2804, 65, 1894, "Text", "ExpressionUUID" -> \
"4c53e128-ad50-4606-a39d-7b8bcc110b5b"],
Cell[1686178, 44446, 1220, 33, 526, "Text", "ExpressionUUID" -> \
"3768a896-00b5-4f44-902f-55da59df0a0b"],
Cell[1687401, 44481, 1536, 39, 582, "Input", "ExpressionUUID" -> \
"7be41465-e682-45a0-83d1-721b67e9c07b",
 Evaluatable->False],
Cell[1688940, 44522, 526, 9, 663, "Text", "ExpressionUUID" -> \
"b5cc0087-8898-4eca-8500-2ece96fa4512"],
Cell[1689469, 44533, 1134, 33, 568, "Input", "ExpressionUUID" -> \
"fefea4e6-b9f5-485b-a9ac-6687ab8ce92c",
 Evaluatable->False],
Cell[1690606, 44568, 2025, 42, 1282, "Text", "ExpressionUUID" -> \
"a5c1d256-ee91-4d14-9bac-6e5e042ff645"],
Cell[1692634, 44612, 577, 17, 331, "Text", "ExpressionUUID" -> \
"1c49490b-051d-49e7-a66c-f7213a945055"],
Cell[1693214, 44631, 472, 13, 340, "Input", "ExpressionUUID" -> \
"71f8e3a8-6e26-4e3e-aa33-0e618903f574",
 Evaluatable->False],
Cell[1693689, 44646, 173, 3, 292, "Text", "ExpressionUUID" -> \
"4afd1f27-5f88-4f96-889f-261051da5ed6"],
Cell[1693865, 44651, 918, 26, 354, "Input", "ExpressionUUID" -> \
"08786d32-27ac-43e1-88db-cc15fc0c0d23",
 Evaluatable->False],
Cell[1694786, 44679, 656, 18, 559, "Text", "ExpressionUUID" -> \
"58a1869f-0193-4e3f-a53e-8e8b4be10626"],
Cell[1695445, 44699, 1303, 39, 334, "Input", "ExpressionUUID" -> \
"857c5caa-a371-43c6-af0c-44dbf9d8ba6a",
 Evaluatable->False],
Cell[1696751, 44740, 1396, 30, 552, "Text", "ExpressionUUID" -> \
"eb2511c8-ada9-426a-858d-7cff2a6d68c7"],
Cell[1698150, 44772, 1679, 42, 891, "Text", "ExpressionUUID" -> \
"427a589e-f7dc-4ce2-8e99-9450f7a8cf93"],
Cell[1699832, 44816, 985, 28, 340, "Input", "ExpressionUUID" -> \
"99ef9d1b-29ba-4c7c-83f0-c353aaf27086",
 Evaluatable->False],
Cell[1700820, 44846, 278, 6, 481, "Text", "ExpressionUUID" -> \
"80e370e9-4df3-4bed-85a4-893df1e74814"],
Cell[1701101, 44854, 961, 21, 709, "Text", "ExpressionUUID" -> \
"c4cc497f-c601-434d-8744-b8fa77349f07"],
Cell[1702065, 44877, 268, 6, 481, "Text", "ExpressionUUID" -> \
"fdc44b4e-844c-4d6f-b1d9-8f8ee16edba1"],
Cell[1702336, 44885, 524, 13, 295, "Input", "ExpressionUUID" -> \
"ec5a04a4-edd0-4a16-8842-02b0bbf61193",
 Evaluatable->False],
Cell[1702863, 44900, 287, 4, 292, "Text", "ExpressionUUID" -> \
"0e01e8f4-17f1-4036-92dc-fd7584dd7838"],
Cell[1703153, 44906, 1045, 21, 1066, "Text", "ExpressionUUID" -> \
"4440c5c4-ea80-4dca-ae3a-e2c75b68256d"],
Cell[1704201, 44929, 719, 18, 315, "Input", "ExpressionUUID" -> \
"e43d669f-4066-4a22-aece-bbfb785d72fb",
 Evaluatable->False],
Cell[1704923, 44949, 220, 5, 292, "Text", "ExpressionUUID" -> \
"92d1ac19-7cf8-4306-aae5-d55f12fa18fb"],
Cell[1705146, 44956, 1300, 44, 321, "Input", "ExpressionUUID" -> \
"c4ffb0fe-82ef-4621-a40f-21023db09e7b",
 Evaluatable->False],
Cell[1706449, 45002, 310, 8, 298, "Text", "ExpressionUUID" -> \
"3dc8a407-e2ac-48ac-b7c9-5309a38c0111"],
Cell[1706762, 45012, 1026, 29, 556, "Input", "ExpressionUUID" -> \
"63e44bc7-32c0-4db9-9b70-009f5565af28",
 Evaluatable->False],
Cell[1707791, 45043, 897, 24, 556, "Input", "ExpressionUUID" -> \
"c06a100c-358a-4666-8b32-8c3286260305",
 Evaluatable->False],
Cell[1708691, 45069, 1186, 29, 904, "Text", "ExpressionUUID" -> \
"07d7de2b-1725-4361-9ff1-21c97f9151da"],
Cell[1709880, 45100, 368, 8, 663, "Text", "ExpressionUUID" -> \
"0f49f8c4-733b-4193-84a9-7983a1baf93a"],
Cell[1710251, 45110, 178, 4, 295, "Input", "ExpressionUUID" -> \
"35043fb3-9259-4400-b067-67e775abf9b8"],
Cell[1710432, 45116, 353, 8, 295, "Input", "ExpressionUUID" -> \
"5dd66b6a-c627-44b6-a22c-2744578c57b2"],
Cell[1710788, 45126, 692, 14, 292, "Text", "ExpressionUUID" -> \
"572f6111-bfc2-44d3-98ea-13a2da4d3d2d"],
Cell[1711483, 45142, 231, 6, 295, "Input", "ExpressionUUID" -> \
"54e558e3-9995-4e88-a694-af48d8beb432"],
Cell[1711717, 45150, 177, 3, 292, "Text", "ExpressionUUID" -> \
"bfba0ceb-0113-4296-bfba-c53c47ed102e"],
Cell[1711897, 45155, 399, 10, 295, "Input", "ExpressionUUID" -> \
"f3bea879-53e8-47a7-863e-98fb45db9f73"],
Cell[1712299, 45167, 358, 7, 481, "Text", "ExpressionUUID" -> \
"ba17d4c6-514c-4db6-80a2-c75c66cb3b42"],
Cell[1712660, 45176, 282, 8, 295, "Input", "ExpressionUUID" -> \
"cb1dc3b7-95b5-4043-81ac-70752256deb5"],
Cell[1712945, 45186, 413, 10, 487, "Text", "ExpressionUUID" -> \
"ef70f698-f7ee-4a33-b11a-4cc1798c4df2"],
Cell[1713361, 45198, 196, 3, 292, "Text", "ExpressionUUID" -> \
"1e2dbbd1-f7d8-4fc6-b6ab-ad0d97a232b1"],
Cell[1713560, 45203, 2613, 62, 1057, "Input", "ExpressionUUID" -> \
"7d63b13c-572a-49d3-8b8c-846fb7782027",
 Evaluatable->False],
Cell[1716176, 45267, 555, 14, 490, "Input", "ExpressionUUID" -> \
"e158f5a3-909c-48c7-b6e2-01324b9de1ec",
 Evaluatable->False],
Cell[1716734, 45283, 174, 4, 295, "Input", "ExpressionUUID" -> \
"0fdae242-f41c-4475-847f-c2f861f28a65"],
Cell[1716911, 45289, 175, 4, 295, "Input", "ExpressionUUID" -> \
"ec31e670-9dc3-4fc7-867a-a5c64ec2ba7b"],
Cell[1717089, 45295, 320, 8, 295, "Input", "ExpressionUUID" -> \
"dec86dce-fd77-41a9-bcbf-533134da0bf9"],
Cell[1717412, 45305, 188, 4, 295, "Input", "ExpressionUUID" -> \
"aaee6dd0-aabb-4094-b0f9-08c95c391cee"],
Cell[1717603, 45311, 1376, 27, 490, "Input", "ExpressionUUID" -> \
"9d2e1865-1a37-49b3-982e-a23ea0d6c17b"],
Cell[1718982, 45340, 690, 19, 490, "Input", "ExpressionUUID" -> \
"6d54fc20-d053-4f35-8869-3bfab7e13874"],
Cell[1719675, 45361, 270, 6, 295, "Input", "ExpressionUUID" -> \
"bdb7d5c4-5735-4f61-bf93-0326121a3d05"],
Cell[1719948, 45369, 173, 4, 295, "Input", "ExpressionUUID" -> \
"657846f4-6900-440d-93d2-4014f7a60195"],
Cell[1720124, 45375, 564, 12, 295, "Input", "ExpressionUUID" -> \
"005769f8-e013-4629-80f4-7381b99d01db"],
Cell[1720691, 45389, 167, 3, 292, "Text", "ExpressionUUID" -> \
"711f4dde-6e86-4156-b396-c54cc8424e62"],
Cell[1720861, 45394, 173, 4, 295, "Input", "ExpressionUUID" -> \
"343adcad-b96d-490a-b058-29b6201ffa38"],
Cell[1721037, 45400, 653, 14, 295, "Input", "ExpressionUUID" -> \
"841372dd-4acf-43fd-baf0-1e5be9540041"],
Cell[1721693, 45416, 1051, 28, 490, "Input", "ExpressionUUID" -> \
"d83f5ec5-4e42-49bd-b80e-faa335d921d6"],
Cell[1722747, 45446, 159, 3, 292, "Text", "ExpressionUUID" -> \
"7d66be6c-ea1b-4191-baeb-b4fc4e1e70c7"],
Cell[1722909, 45451, 437, 8, 295, "Input", "ExpressionUUID" -> \
"85c89594-dbc9-4c35-9535-8b21a75f641b"],
Cell[1723349, 45461, 164, 3, 292, "Text", "ExpressionUUID" -> \
"a372bcdc-4410-46ae-8efc-fce3ce9fa551"],
Cell[1723516, 45466, 342, 7, 295, "Input", "ExpressionUUID" -> \
"272a2255-2e4e-442d-a9ab-db8ee7fb5077"],
Cell[1723861, 45475, 391, 8, 481, "Text", "ExpressionUUID" -> \
"a842cd84-881f-448a-a40f-e65d65d04d81"],
Cell[1724255, 45485, 1193, 32, 679, "Input", "ExpressionUUID" -> \
"0bbcf20c-65e7-4005-849f-591b4707ae48"],
Cell[1725451, 45519, 187, 4, 295, "Input", "ExpressionUUID" -> \
"e81b30a6-a11a-4d89-83d6-cc02c3394d55"],
Cell[1725641, 45525, 173, 3, 292, "Text", "ExpressionUUID" -> \
"350fbb15-58ff-47ba-9cf7-142fc7f53d35"],
Cell[1725817, 45530, 229, 5, 295, "Input", "ExpressionUUID" -> \
"6a7d3b75-9efc-4fa3-8a26-15951598ff76"],
Cell[1726049, 45537, 164, 3, 292, "Text", "ExpressionUUID" -> \
"4bd0758e-e911-420d-b3eb-aee1c2998ff8"],
Cell[1726216, 45542, 225, 5, 295, "Input", "ExpressionUUID" -> \
"3b967c8c-7c49-4384-b915-7bce0213806d"],
Cell[1726444, 45549, 1071, 31, 679, "Input", "ExpressionUUID" -> \
"312f0d9c-4e00-44cb-8c07-bac70966eb16"],
Cell[1727518, 45582, 191, 4, 295, "Input", "ExpressionUUID" -> \
"a8b7e6a1-9759-4841-ac28-9316b4fae40a"],
Cell[1727712, 45588, 364, 6, 292, "Text", "ExpressionUUID" -> \
"2f6c07ce-9556-4367-8835-260a97f075da"],
Cell[1728079, 45596, 272, 8, 292, "Text", "ExpressionUUID" -> \
"984da056-f391-468b-a48f-4800a85cb754"],
Cell[1728354, 45606, 174, 4, 295, "Input", "ExpressionUUID" -> \
"2e3e7954-9e69-46e1-b329-bc52ad906ead"],
Cell[1728531, 45612, 257, 7, 295, "Input", "ExpressionUUID" -> \
"f8c6291c-dd0b-4683-ae6a-ad378033baf1"],
Cell[1728791, 45621, 309, 7, 292, "Text", "ExpressionUUID" -> \
"2c53acae-1e9d-4252-8873-344d67397176"],
Cell[1729103, 45630, 202, 4, 295, "Input", "ExpressionUUID" -> \
"5cd15086-036a-45b2-8442-e975070bbe49"],
Cell[1729308, 45636, 14182, 321, 4833, "Input", "ExpressionUUID" -> \
"57503aed-2746-4720-9af9-c9f30c0f7834",
 Evaluatable->False],
Cell[1743493, 45959, 588, 11, 295, "Input", "ExpressionUUID" -> \
"feb0d7ca-0aa8-4120-b953-6b4738f572d1"],
Cell[1744084, 45972, 189, 4, 295, "Input", "ExpressionUUID" -> \
"93279211-b95a-45cf-b61c-c696fd45c80b"],
Cell[1744276, 45978, 609, 11, 295, "Input", "ExpressionUUID" -> \
"b4c3496a-9c00-4d16-a12e-98b5d16f928d"],
Cell[1744888, 45991, 189, 4, 295, "Input", "ExpressionUUID" -> \
"1a9bf318-63c2-4c50-bfb8-a4bcba77dfe9"],
Cell[1745080, 45997, 224, 6, 295, "Input", "ExpressionUUID" -> \
"7679782e-5b02-4d05-811a-09a103ab3e03"],
Cell[1745307, 46005, 189, 4, 295, "Input", "ExpressionUUID" -> \
"dc984ff6-12f3-4d8d-966b-1d24b30b7e45"],
Cell[1745499, 46011, 518, 10, 295, "Input", "ExpressionUUID" -> \
"83ea3e46-e65d-4cbe-af27-28f3a7372e7e"],
Cell[1746020, 46023, 264, 7, 292, "Text", "ExpressionUUID" -> \
"5d72d28e-0377-4515-9023-368339d218bf"],
Cell[1746287, 46032, 347, 9, 295, "Input", "ExpressionUUID" -> \
"8d4c122b-751a-48f7-8dee-c96d690c0115"],
Cell[1746637, 46043, 184, 4, 295, "Input", "ExpressionUUID" -> \
"595d1686-1ca1-47f1-900f-0a2d91efcf20"],
Cell[1746824, 46049, 279, 8, 295, "Input", "ExpressionUUID" -> \
"e82cd2ca-9b59-45fa-8b3e-e1a6fd85197a"],
Cell[1747106, 46059, 700, 13, 295, "Input", "ExpressionUUID" -> \
"3afbaf52-cf0d-4fea-b1c6-6593f6bfe8a0"],
Cell[1747809, 46074, 604, 10, 852, "Text", "ExpressionUUID" -> \
"b44fe447-48fb-4184-8c80-9a7249de9a84"],
Cell[1748416, 46086, 175, 4, 295, "Input", "ExpressionUUID" -> \
"c9cc5aa4-9174-4a3e-8163-0a1567701b20"],
Cell[1748594, 46092, 310, 6, 292, "Text", "ExpressionUUID" -> \
"48b16d7e-40ba-470e-9e85-b7dae55f8038"],
Cell[1748907, 46100, 219, 5, 295, "Input", "ExpressionUUID" -> \
"b4cc2081-033d-45c4-a6b7-dfe8ae8194ca"],
Cell[1749129, 46107, 297, 9, 487, "Text", "ExpressionUUID" -> \
"68bbe90c-96d0-4a96-8160-f8e9ecc1aa14"],
Cell[1749429, 46118, 614, 11, 295, "Input", "ExpressionUUID" -> \
"90e62ce0-e326-4c6a-9816-a0af471dddb5"],
Cell[1750046, 46131, 776, 14, 295, "Input", "ExpressionUUID" -> \
"7ce8d94f-115c-4b37-9c47-00816f74056d"],
Cell[1750825, 46147, 238, 5, 295, "Input", "ExpressionUUID" -> \
"d0e35929-7ed6-469f-9e00-22ab8d04f270"],
Cell[1751066, 46154, 811, 15, 295, "Input", "ExpressionUUID" -> \
"ec6e88db-ca21-44f9-aecd-2fb19dc5e13c"],
Cell[1751880, 46171, 187, 4, 295, "Input", "ExpressionUUID" -> \
"2b86c7da-b25d-425b-8c12-207135d9df7d"],
Cell[1752070, 46177, 559, 10, 682, "Text", "ExpressionUUID" -> \
"bc513a25-f037-4164-99d2-909e3809ec29"],
Cell[1752632, 46189, 896, 16, 295, "Input", "ExpressionUUID" -> \
"3709a327-4979-4a60-9f8b-f82f0c680bea"],
Cell[1753531, 46207, 186, 4, 295, "Input", "ExpressionUUID" -> \
"a9542336-b94f-4fa4-acba-e1ca858758d3"],
Cell[1753720, 46213, 157, 3, 12450, "Text", "ExpressionUUID" -> \
"f6792673-06bc-4a81-b4c9-b41fb60c3c35"],
Cell[1753880, 46218, 600, 13, 663, "Text", "ExpressionUUID" -> \
"9622ca59-aa05-4d4a-9094-ef3fec5fa98e"],
Cell[1754483, 46233, 404, 9, 295, "Input", "ExpressionUUID" -> \
"ecbbc950-2439-427a-8da3-e9d1baf45a1a"],
Cell[1754890, 46244, 302, 7, 295, "Input", "ExpressionUUID" -> \
"9a733735-aaef-4b10-a1fe-b3dee41ca4f5"],
Cell[1755195, 46253, 287, 7, 295, "Input", "ExpressionUUID" -> \
"7f5b6e4f-f5ce-479d-b63e-fa3b21d18fb0"],
Cell[1755485, 46262, 690, 16, 490, "Input", "ExpressionUUID" -> \
"8a87f69a-7c12-4d56-be75-0c306e100db8"],
Cell[1756178, 46280, 374, 7, 481, "Text", "ExpressionUUID" -> \
"148002a2-83ff-401b-8c25-4a69c05c557c"],
Cell[1756555, 46289, 590, 15, 295, "Input", "ExpressionUUID" -> \
"fb2f5f72-58b6-4c92-aa7e-b36afa85665a"],
Cell[1757148, 46306, 525, 11, 295, "Input", "ExpressionUUID" -> \
"e7bdc993-e980-4c67-b39f-ed7e734d7cda"],
Cell[1757676, 46319, 654, 14, 295, "Input", "ExpressionUUID" -> \
"ba75894b-46b2-44e8-b6a0-c4740d436368"],
Cell[1758333, 46335, 193, 4, 295, "Input", "ExpressionUUID" -> \
"7ebe3030-67f0-470e-864a-5a516f9559d1"],
Cell[1758529, 46341, 1580, 43, 1057, "Input", "ExpressionUUID" -> \
"70a84bc4-327f-4f7b-b17f-e1eb60e80753"],
Cell[1760112, 46386, 222, 5, 295, "Input", "ExpressionUUID" -> \
"f6d6c5dc-143b-40b5-99da-e3bc70368fdb"],
Cell[1760337, 46393, 802, 16, 295, "Input", "ExpressionUUID" -> \
"e3a70614-d9f8-40ed-bd5a-0aafbd465e87"],
Cell[1761142, 46411, 218, 4, 295, "Input", "ExpressionUUID" -> \
"b7b8e66f-b690-4ea8-8b5e-39ad9bce2f14"],
Cell[1761363, 46417, 166, 3, 292, "Text", "ExpressionUUID" -> \
"efd50e89-8a1d-4502-b0a8-fdb340728f59"],
Cell[1761532, 46422, 894, 18, 295, "Input", "ExpressionUUID" -> \
"ad0088be-609c-41db-a1f4-69bb66cf8934"],
Cell[1762429, 46442, 1688, 39, 679, "Input", "ExpressionUUID" -> \
"2837d718-cf78-4301-b231-7ed929b0e876"],
Cell[1764120, 46483, 257, 6, 481, "Text", "ExpressionUUID" -> \
"016e2faf-8fa7-4160-b204-258362fce79c"],
Cell[1764380, 46491, 186, 4, 295, "Input", "ExpressionUUID" -> \
"5efe970a-92ee-44ca-90cc-5d7596c5a3dc"],
Cell[1764569, 46497, 999, 27, 490, "Input", "ExpressionUUID" -> \
"ad8743fb-87f3-4bb8-b2ec-32aacc7f9d7e",
 Evaluatable->False],
Cell[1765571, 46526, 338, 9, 295, "Input", "ExpressionUUID" -> \
"d0c01528-e73f-4b86-a48f-0a7062d213cd"],
Cell[1765912, 46537, 638, 16, 490, "Input", "ExpressionUUID" -> \
"c6eaf78a-d1cd-475f-a743-21b0b6569795"],
Cell[1766553, 46555, 744, 17, 490, "Input", "ExpressionUUID" -> \
"44ae17bf-4ac5-4899-9f40-5b7658f71afe"],
Cell[1767300, 46574, 324, 8, 295, "Input", "ExpressionUUID" -> \
"bacd62d8-8939-46f1-8ef6-b3d4f6dafaee"]
}, Closed]],
Cell[CellGroupData[{
Cell[1767661, 46587, 394, 5, 42, "Subsubsection", "ExpressionUUID" -> \
"352a209a-1d66-4f4f-99c6-9684ae0182c9"],
Cell[1768058, 46594, 1505, 25, 1444, "Text", "ExpressionUUID" -> \
"64f33308-eac1-4816-b26e-c524bcd248d6"],
Cell[1769566, 46621, 1019, 29, 490, "Input", "ExpressionUUID" -> \
"33b77a6e-e711-4ca1-a8fe-1a820e7506ac",
 Evaluatable->False],
Cell[1770588, 46652, 2224, 32, 1594, "Text", "ExpressionUUID" -> \
"95c98e83-08fb-449e-998a-700dcae10705"],
Cell[1772815, 46686, 4578, 113, 2190, "Input", "ExpressionUUID" -> \
"0fc27a7a-9055-4fb1-a949-b3fc3cf94980",
 Evaluatable->False],
Cell[1777396, 46801, 359, 6, 295, "Input", "ExpressionUUID" -> \
"cbc5fa72-78f2-4336-ba4d-d59abf5de9db"],
Cell[1777758, 46809, 734, 17, 295, "Input", "ExpressionUUID" -> \
"9cb6fd3d-4b87-4d25-8f24-14ba9e44db2d"],
Cell[1778495, 46828, 709, 12, 493, "Text", "ExpressionUUID" -> \
"75b42b60-02d5-4de3-ae16-95283da0f982"],
Cell[1779207, 46842, 240, 7, 292, "Text", "ExpressionUUID" -> \
"e1b15781-5598-4963-9f8d-1ea9644d0432"],
Cell[1779450, 46851, 234, 6, 295, "Input", "ExpressionUUID" -> \
"7ff73e06-22d1-4363-b25b-303ed1217404"],
Cell[1779687, 46859, 174, 4, 295, "Input", "ExpressionUUID" -> \
"3c534d59-517f-4fd7-8612-6f8b5f486ba8"],
Cell[1779864, 46865, 603, 13, 295, "Input", "ExpressionUUID" -> \
"620b81af-c254-4bfd-9d69-cbc7519750c1"],
Cell[1780470, 46880, 367, 7, 481, "Text", "ExpressionUUID" -> \
"6c25b7b1-6fbe-4298-a499-d2fcee936228"],
Cell[1780840, 46889, 927, 23, 490, "Input", "ExpressionUUID" -> \
"103fe61c-3bfa-4fd4-bd17-e9d2571f5051"],
Cell[1781770, 46914, 333, 9, 295, "Input", "ExpressionUUID" -> \
"d5eae301-428a-4d22-b05a-fccb52f14024"],
Cell[1782106, 46925, 352, 9, 295, "Input", "ExpressionUUID" -> \
"ded6e5e0-89cc-42cf-a0a2-3ee0aa293b2f"],
Cell[1782461, 46936, 1014, 22, 679, "Input", "ExpressionUUID" -> \
"a7bd82b0-24ab-4d00-8508-5fe62acf5a87"],
Cell[1783478, 46960, 461, 9, 298, "Text", "ExpressionUUID" -> \
"b8b583c6-3b68-4ccc-9588-9bc5dec54137"],
Cell[1783942, 46971, 311, 7, 481, "Text", "ExpressionUUID" -> \
"a8b44049-d527-47f7-8f54-e5496aae908f"],
Cell[1784256, 46980, 256, 7, 292, "Text", "ExpressionUUID" -> \
"173478ab-16e8-49bd-9a31-80d800d9f965"],
Cell[1784515, 46989, 175, 4, 295, "Input", "ExpressionUUID" -> \
"54816db4-fe88-4c43-9d19-2046963ba708"],
Cell[1784693, 46995, 324, 8, 295, "Input", "ExpressionUUID" -> \
"62336ec0-ec3e-458f-8b77-c065d63e591c"],
Cell[1785020, 47005, 924, 22, 679, "Input", "ExpressionUUID" -> \
"b74799c8-8637-44e9-927f-6e82959d56be"],
Cell[1785947, 47029, 620, 9, 292, "Text", "ExpressionUUID" -> \
"afc044c8-0244-4430-b085-e616b47dd12b"],
Cell[1786570, 47040, 355, 8, 292, "Text", "ExpressionUUID" -> \
"e7b9241a-ce4a-4469-85f6-23a2dc66060c"],
Cell[1786928, 47050, 177, 4, 295, "Input", "ExpressionUUID" -> \
"e472c92b-424b-4581-aeb9-64fe48dddba3"],
Cell[1787108, 47056, 228, 5, 295, "Input", "ExpressionUUID" -> \
"70179bec-f4bf-44ca-9ec7-f890b33b03b4"],
Cell[1787339, 47063, 987, 24, 868, "Input", "ExpressionUUID" -> \
"7073bf72-22f4-4fa3-9286-fcadb9949025"],
Cell[1788329, 47089, 1037, 25, 868, "Input", "ExpressionUUID" -> \
"d84f03cc-8eb7-4d76-aaea-b297b93ee85e"],
Cell[1789369, 47116, 709, 13, 487, "Text", "ExpressionUUID" -> \
"095cb941-246b-4048-9d9f-15db743eea7f"],
Cell[1790081, 47131, 227, 5, 295, "Input", "ExpressionUUID" -> \
"701807d5-a7ed-4d74-bd57-d428a06ee44c"],
Cell[1790311, 47138, 1075, 25, 868, "Input", "ExpressionUUID" -> \
"ccaa69ce-2f65-4b1b-8029-56cbee3aa296"],
Cell[1791389, 47165, 1326, 29, 868, "Input", "ExpressionUUID" -> \
"588190bd-362f-4713-9e58-cd4bad0f2ae2"],
Cell[1792718, 47196, 580, 11, 298, "Text", "ExpressionUUID" -> \
"741edd22-e41c-4d11-a156-78f656d83327"],
Cell[1793301, 47209, 192, 3, 292, "Text", "ExpressionUUID" -> \
"25dfcf20-0012-4dbd-a02f-73154986577e"],
Cell[1793496, 47214, 182, 4, 295, "Input", "ExpressionUUID" -> \
"d3364065-5946-40e3-ad6a-6d97f4da0362"],
Cell[1793681, 47220, 192, 4, 295, "Input", "ExpressionUUID" -> \
"e13dd0b3-acc0-459e-865c-8e2bed8c56be"],
Cell[1793876, 47226, 1314, 29, 679, "Input", "ExpressionUUID" -> \
"65a94f07-41cb-401f-9132-91311475162c"],
Cell[1795193, 47257, 1342, 26, 982, "Text", "ExpressionUUID" -> \
"fa6ac2ed-9b6f-4d45-9b4c-45ab0852775a"],
Cell[1796538, 47285, 355, 11, 292, "Text", "ExpressionUUID" -> \
"b24e16fb-6079-4ef0-a286-c3f50ff69637"],
Cell[1796896, 47298, 1038, 28, 868, "Input", "ExpressionUUID" -> \
"45112a41-ef40-4d94-8414-a6c790c889f2"],
Cell[1797937, 47328, 352, 8, 295, "Input", "ExpressionUUID" -> \
"19cbc687-9fa6-42ee-8b5d-526737d78940"],
Cell[1798292, 47338, 266, 6, 292, "Text", "ExpressionUUID" -> \
"28512def-014b-4734-aa6d-397188b86fb9"],
Cell[1798561, 47346, 282, 7, 295, "Input", "ExpressionUUID" -> \
"3e198ea3-a26b-4ffc-ac8e-31a00091ada3"],
Cell[1798846, 47355, 300, 7, 295, "Input", "ExpressionUUID" -> \
"d1233411-bca0-4cb6-8033-d62581282886"],
Cell[1799149, 47364, 708, 17, 679, "Input", "ExpressionUUID" -> \
"22057347-347c-47b0-af00-df43f00f2b53"],
Cell[1799860, 47383, 350, 8, 295, "Input", "ExpressionUUID" -> \
"082171f8-bd6d-4538-a76d-4c3ec181ec73"],
Cell[1800213, 47393, 401, 9, 295, "Input", "ExpressionUUID" -> \
"f9bcbcf2-76d6-4fb5-9104-ac65fd8bcede"],
Cell[1800617, 47404, 607, 13, 682, "Text", "ExpressionUUID" -> \
"e7e83712-2803-4c3a-9f0d-7e262d4703be"],
Cell[1801227, 47419, 302, 8, 481, "Text", "ExpressionUUID" -> \
"73eec0a4-fc15-43f0-bee5-15bf5cb29919"],
Cell[1801532, 47429, 736, 18, 679, "Input", "ExpressionUUID" -> \
"e75bdb5f-a98c-4060-b3bc-913b6a668809"],
Cell[1802271, 47449, 401, 9, 481, "Text", "ExpressionUUID" -> \
"b764f670-e7c0-444c-bfe5-c5f7158a6ca8"],
Cell[1802675, 47460, 284, 7, 295, "Input", "ExpressionUUID" -> \
"b559afa1-497d-41da-aad8-782f3aaf31da"],
Cell[1802962, 47469, 170, 3, 295, "Input", "ExpressionUUID" -> \
"40157670-a3e1-4e96-a13f-975b77e97aa3"],
Cell[1803135, 47474, 486, 14, 295, "Input", "ExpressionUUID" -> \
"ad46037a-1d57-42ae-af12-e56a7d88ef88"],
Cell[1803624, 47490, 304, 8, 295, "Input", "ExpressionUUID" -> \
"4947bc45-49c7-403b-879f-23896213d7bc"],
Cell[1803931, 47500, 1090, 23, 490, "Input", "ExpressionUUID" -> \
"49a0efa8-ccac-478b-a4e7-5d946f05dfa5"],
Cell[1805024, 47525, 355, 8, 295, "Input", "ExpressionUUID" -> \
"64e30d49-70fa-41b0-a936-61cce4ec675c"],
Cell[1805382, 47535, 362, 8, 292, "Text", "ExpressionUUID" -> \
"bd231e8d-20ba-42dd-9420-49f56526e22c"],
Cell[1805747, 47545, 306, 7, 295, "Input", "ExpressionUUID" -> \
"4a391c56-39d4-4f83-a252-27cab9fc8140"],
Cell[1806056, 47554, 573, 14, 490, "Input", "ExpressionUUID" -> \
"3986b2c9-4796-41db-a570-d55f51d7477d"],
Cell[1806632, 47570, 926, 17, 1054, "Text", "ExpressionUUID" -> \
"a525efbb-c10d-4721-8eeb-cd1efaec1967"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1807619, 47594, 337, 8, 123, "Subtitle", "ExpressionUUID" -> \
"78de34fe-e325-486c-9d5d-8bf92c00f0d9"],
Cell[1807959, 47604, 3145, 60, 343, "Text", "ExpressionUUID" -> \
"a4354efa-c7c2-4293-a141-bee3036b4ef9"],
Cell[CellGroupData[{
Cell[1811129, 47668, 217, 4, 63, "Subsubtitle", "ExpressionUUID" -> \
"beacea25-3f6f-4bc1-84dc-383d09f44305"],
Cell[1811349, 47674, 2283, 45, 409, "Text", "ExpressionUUID" -> \
"8bb73728-9aab-4c65-9b93-badd74c80982"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1813669, 47724, 350, 5, 63, "Subsubtitle", "ExpressionUUID" -> \
"2393b25f-3c7f-4fac-b90c-01cce9c3a584"],
Cell[1814022, 47731, 2746, 50, 379, "Text", "ExpressionUUID" -> \
"ea0d6266-3ab4-4405-a82b-f7ac50e3c990"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1816805, 47786, 403, 7, 63, "Subsubtitle", "ExpressionUUID" -> \
"0679a443-6b10-4959-b6a4-0e03e76059de"],
Cell[1817211, 47795, 1639, 27, 420, "Text", "ExpressionUUID" -> \
"56bcdf2f-f257-42f8-a881-6b636ff1a158"],
Cell[1818853, 47824, 838, 12, 153, "Text", "ExpressionUUID" -> \
"e9efd352-0f45-434b-8785-b7691ed3c2d7"],
Cell[1819694, 47838, 281, 7, 112, "Text", "ExpressionUUID" -> \
"9d6d2c2a-8f99-4c5d-bd6e-0f6ba62976fa"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1820024, 47851, 369, 5, 76, "Subtitle", "ExpressionUUID" -> \
"082926a3-0911-46b8-bd79-e390d7d3fe79"],
Cell[1820396, 47858, 441, 9, 83, "Text", "ExpressionUUID" -> \
"b219b1b3-91a7-4877-8602-d309442b73be"],
Cell[1820840, 47869, 553, 10, 83, "Text", "ExpressionUUID" -> \
"9febeb6a-722c-46c8-a41f-083491e845e4"],
Cell[1821396, 47881, 776, 15, 72, "Text", "ExpressionUUID" -> \
"7faa1803-e9ac-4398-8bb1-84f3acca0425",
 CellTags->"IDG"],
Cell[1822175, 47898, 525, 9, 105, "Text", "ExpressionUUID" -> \
"21df92d0-0d9b-47dd-9f83-0bd9b35930f9",
 CellTags->"B-R"],
Cell[1822703, 47909, 408, 7, 105, "Text", "ExpressionUUID" -> \
"1f8500fc-8832-44fb-ae96-c5038fa453db",
 CellTags->"B-L"],
Cell[1823114, 47918, 684, 14, 72, "Text", "ExpressionUUID" -> \
"0667904c-1335-41aa-be8f-da12f32b1340",
 CellTags->"Christ"],
Cell[1823801, 47934, 556, 11, 83, "Text", "ExpressionUUID" -> \
"4ba80fa6-4efa-4c51-a683-312397d6b91d"],
Cell[1824360, 47947, 715, 14, 137, "Text", "ExpressionUUID" -> \
"cc8232d6-5c7f-4187-ace0-7b06ffe46740",
 CellTags->"Gray"],
Cell[1825078, 47963, 999, 18, 137, "Text", "ExpressionUUID" -> \
"898a7ded-4561-4d52-b930-9f4af91f2b54",
 CellTags->"Gray2"],
Cell[1826080, 47983, 690, 11, 137, "Text", "ExpressionUUID" -> \
"c6706746-5ca2-46ac-9e8d-809a3ea97382",
 CellTags->"Gray3"],
Cell[1826773, 47996, 458, 9, 105, "Text", "ExpressionUUID" -> \
"0758737c-9bc5-44ba-a44c-6f7e319bf127",
 CellTags->"H78"],
Cell[1827234, 48007, 448, 7, 105, "Text", "ExpressionUUID" -> \
"268e697c-a6d7-4db2-99be-89bde2586826",
 CellTags->"JMN"],
Cell[1827685, 48016, 413, 7, 105, "Text", "ExpressionUUID" -> \
"f4362455-c4bb-4a38-a039-80dd10285d0d",
 CellTags->"K-N1"],
Cell[1828101, 48025, 397, 7, 105, "Text", "ExpressionUUID" -> \
"84379d9b-123f-495a-9ac6-282ae8d762cd",
 CellTags->"Kr"],
Cell[1828501, 48034, 208, 4, 72, "Text", "ExpressionUUID" -> \
"e515a207-0109-4bdc-a391-6e6cc86c3d63",
 CellTags->"Mal"],
Cell[1828712, 48040, 523, 12, 105, "Text", "ExpressionUUID" -> \
"0033d828-935c-4125-a93a-682f3fe8b07f",
 CellTags->"xAct"],
Cell[1829238, 48054, 652, 11, 105, "Text", "ExpressionUUID" -> \
"3cdfef22-aea4-4a65-aa81-8bca3fd44206",
 CellTags->"OSI"],
Cell[1829893, 48067, 519, 8, 105, "Text", "ExpressionUUID" -> \
"4ae8ad1d-c3c6-4659-88df-49dee0826979",
 CellTags->"OSII"],
Cell[1830415, 48077, 584, 10, 105, "Text", "ExpressionUUID" -> \
"d8d078d5-e367-4e9b-af60-df057e345174",
 CellTags->"OSIII"],
Cell[1831002, 48089, 369, 6, 105, "Text", "ExpressionUUID" -> \
"7e0f735d-2d39-4c01-8b29-f97ba112e151",
 CellTags->"Sch54"],
Cell[1831374, 48097, 1036, 21, 137, "Text", "ExpressionUUID" -> \
"07049e92-b028-40ff-a664-c65723372cee",
 CellTags->"Soleng"],
Cell[1832413, 48120, 1270, 26, 137, "Text", "ExpressionUUID" -> \
"4cddb376-10e3-4ed2-a93e-c692609c788c",
 CellTags->"EDG"],
Cell[1833686, 48148, 289, 7, 72, "Text", "ExpressionUUID" -> \
"8f055b0c-6a33-4b5e-9a2f-1896ad783681",
 CellTags->"Tensors"],
Cell[1833978, 48157, 737, 14, 137, "Text", "ExpressionUUID" -> \
"bcec088b-3c67-4ac3-80fe-012dbf099f00",
 CellTags->"ccgrg"]
}, Closed]],
Cell[CellGroupData[{
Cell[1834752, 48176, 230, 5, 77, "Subtitle", "ExpressionUUID" -> \
"01187eef-dc97-40c3-b2ea-8daa6aa6539d"],
Cell[1834985, 48183, 301, 7, 83, "Text", "ExpressionUUID" -> \
"4a5970fe-41dd-4517-a508-eed629f1cefc"],
Cell[1835289, 48192, 427, 7, 72, "Text", "ExpressionUUID" -> \
"c7ecf6e1-c5e5-4aab-ae7b-974105bcc91c",
 CellTags->"Home"],
Cell[1835719, 48201, 614, 11, 115, "Text", "ExpressionUUID" -> \
"bc072768-af5b-4c61-bd00-88ab0e68f38e"],
Cell[1836336, 48214, 230, 4, 50, "Text", "ExpressionUUID" -> \
"70179e99-2791-4430-9ac8-557d515121f6"]
}, Open  ]]
}, Open  ]]
}
]
*)

